/*****************************************************************************
  $Id: compositions.tst,v 1.1.1.6 2005/01/26 13:42:25 nthiery Exp $
  File:		lib/COMBINAT/TEST/compositions.tst
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  		Florent Hivert <florent.hivert@univ-mlv.fr>
  Purpose:	Test file for the library 'combinat::compositions'
  Created:	24/02/2001
  License:	LGPL
*****************************************************************************/

prog::testinit("compositions", 20.5):

prog::testfunc(combinat::compositions::count):
prog::test(combinat::compositions::count(0),
           1):
prog::test(combinat::compositions::count(1),
           1):
prog::test(combinat::compositions::count(4),
           8):
prog::test(combinat::compositions::count(5, MaxPart = 2),
           8):
prog::test(combinat::compositions::count(10, MaxPart = 4),
           401):



prog::testfunc(combinat::compositions::list):

// The order in which the compositions are listed is not in the
// specifications of combinat::compositions::list, so we sort the
// result before comparing.

prog::test(sort(combinat::compositions::list(-1)),
           sort([])):
prog::test(sort(combinat::compositions::list(-1)),
           sort([])):
prog::test(sort(combinat::compositions::list(0)),
           sort([[]])):
prog::test(sort(combinat::compositions::list(1)),
           sort([[1]])):
prog::test(sort(combinat::compositions::list(2)),
           sort([[2],[1,1]])):
prog::test(sort(combinat::compositions::list(3)),
           sort([[3],[2,1],[1,2],[1,1,1]])):
prog::test(sort(combinat::compositions::list(4)),
           sort([[4],[3,1],[2,2],[2,1,1],[1,3],[1,2,1],[1,1,2],[1,1,1,1]])):


// Florent: Test du patch du 19/01/05.
prog::test(combinat::compositions::list(0,MinSlope=0), [[]]):
prog::test(combinat::compositions::list(0,Outer=[],MinSlope=0,Length=0), [[]]):


// Since list is based on generator, we only check generator.
for n from -2 to 6 do
    prog::test(combinat::compositions::count(n),
               combinat::generators::count
               (combinat::compositions::generator(n))):
end_for:


prog::testfunc(combinat::compositions::isFiner):
prog::test(combinat::compositions::isFiner([], []), TRUE): 
prog::test(combinat::compositions::isFiner([2], [1,1]), FALSE): 
prog::test(combinat::compositions::isFiner([1,1], [2]), TRUE): 
prog::test(combinat::compositions::isFiner([2,1,1,2,4], [2,2,6]), TRUE): 
prog::test(combinat::compositions::isFiner([0,2,1,1,0,2,4], [2,2,6]), TRUE): 
prog::test(combinat::compositions::isFiner([2,2,6], [2,1,1,2,4]), FALSE): 
// The error code is 1028 if the error was raised by the
// command error.
if prog::TestNoTestargs = FALSE then
prog::test(combinat::compositions::isFiner([2,1,4],[3,2]), TrapError = 1028):
end_if:

// Since list is based on generator, we only check list.
// Since the order is not specified and is probably about to changes
// we do a sort.
prog::testfunc(combinat::compositions::finer):
prog::test(sort(combinat::compositions::finer([])),    sort([[]])):
prog::test(sort(combinat::compositions::finer([1])),   sort([[1]])):
prog::test(sort(combinat::compositions::finer([2])),   sort([[2],[1,1]])):
prog::test(sort(combinat::compositions::finer([1,1])), sort([[1,1]])):
prog::test(sort(combinat::compositions::finer([2,4,1])),
	   sort([[2,4,1],[2,3,1,1],[2,2,2,1],[2,2,1,1,1],[2,1,3,1],
	    [2,1,2,1,1],[2,1,1,2,1],[2,1,1,1,1,1],[1,1,4,1],
	    [1,1,3,1,1],[1,1,2,2,1],[1,1,2,1,1,1],[1,1,1,3,1],
	    [1,1,1,2,1,1],[1,1,1,1,2,1],[1,1,1,1,1,1,1]])):


// Since list is based on generator, we only check list.
// Since the order is not specified and is probably about to changes
// we do a sort.
prog::testfunc(combinat::compositions::finerVectors):
prog::test(sort(combinat::compositions::finerVectors([], 0)),
	   sort([[]])):
prog::test(sort(combinat::compositions::finerVectors([], 6)),
	   sort([[0, 0, 0, 0, 0, 0]])):
prog::test(sort(combinat::compositions::finerVectors([1],3)),
	   sort([[1,0,0],[0,1,0],[0,0,1]])):
prog::test(sort(combinat::compositions::finerVectors([2],3)),
	   sort([[2,0,0],[1,1,0],[1,0,1],[0,2,0],[0,1,1],[0,0,2]])):
prog::test(sort(combinat::compositions::finerVectors([1,1],3)),
	   sort([[1,0,1],[0,1,1],[1,1,0]])):
prog::test(sort(combinat::compositions::finerVectors([2,1],3)),
	   sort([[2,0,1],[1,1,1],[0,2,1],[2,1,0]])):
prog::test(sort(combinat::compositions::finerVectors([1],6)),
	   sort([[1,0,0,0,0,0],[0,1,0,0,0,0],[0,0,1,0,0,0],
		 [0,0,0,1,0,0],[0,0,0,0,1,0],[0,0,0,0,0,1]])):
for n from 0 to 4 do // all composition upto sum 4 
    for c in combinat::compositions::list(n) do
	for len from 0 to 6 do // all length upto 6
prog::test(bool(sort(select(combinat::integerVectors::list(n,len), 
			    z->combinat::compositions::isFiner(z, c))) =
		sort(combinat::compositions::finerVectors(c,len)))):
	end_for;
    end_for:
end_for:


// Since list is based on generator, we only check list.
// Since the order is not specified and is probably about to changes
// we do a sort.
prog::testfunc(combinat::compositions::fatter):
prog::test(sort(combinat::compositions::fatter([])),    sort([[]])):
prog::test(sort(combinat::compositions::fatter([1])),   sort([[1]])):
prog::test(sort(combinat::compositions::fatter([2])),   sort([[2]])):
prog::test(sort(combinat::compositions::fatter([1,1])), sort([[2],[1,1]])):
prog::test(sort(combinat::compositions::fatter([2,4,1])),
	   sort([[7],[6,1],[2,5],[2,4,1]])):
prog::test(sort(combinat::compositions::fatter([1,2,4,1])),
	   sort([[8],[7,1],[3,5],[3,4,1],[1,7],[1,6,1],[1,2,5],[1,2,4,1]])):


prog::testfunc(combinat::compositions::conjugate):
prog::test(combinat::compositions::conjugate([]), []):
prog::test(combinat::compositions::conjugate([1]), [1]):
prog::test(combinat::compositions::conjugate([1,1]), [2]):
prog::test(combinat::compositions::conjugate([2]), [1,1]):
prog::test(combinat::compositions::conjugate([1,4,3,2]), [1,2,1,2,1,1,2]):
// testing for involutivity
for i in combinat::compositions::list(6) do
    prog::test(combinat::compositions::conjugate(
	combinat::compositions::conjugate(i)), i);
end_for:

prog::testfunc(combinat::compositions::majorIndex):
prog::test(combinat::compositions::majorIndex([]), 0):
prog::test(combinat::compositions::majorIndex([1]), 0):
prog::test(combinat::compositions::majorIndex([2]), 0):
prog::test(combinat::compositions::majorIndex([1,1]), 1):
prog::test(combinat::compositions::majorIndex([2,1,2]), 5):
prog::test(combinat::compositions::majorIndex([3,5,4,3,1]), 38):

prog::testfunc(combinat::compositions::descents):
prog::test(combinat::compositions::descents([]), []):
prog::test(combinat::compositions::descents([3]), []):
prog::test(combinat::compositions::descents([1,2]), [1]):
prog::test(combinat::compositions::descents([1,3,2]), [1,4]):
prog::test(combinat::compositions::descents([1,3,2,15]), [1,4,6]):
prog::test(combinat::compositions::descents([], FinalDescent), []):
prog::test(combinat::compositions::descents([3], FinalDescent), [3]):
prog::test(combinat::compositions::descents([1,2], FinalDescent), [1,3]):
prog::test(combinat::compositions::descents([1,3,2], FinalDescent), [1,4,6]):
prog::test(combinat::compositions::descents([1,3,2,15], FinalDescent), [1,4,6,21]):
prog::test(combinat::compositions::descents([0,1]), [0]):
prog::test(combinat::compositions::descents([0,1], FinalDescent), [0,1]):
prog::test(combinat::compositions::descents([0,1,0,0,1], FinalDescent), [0,1,1,1,2]):
prog::test(combinat::compositions::descents([0,1,0,0,1,0], FinalDescent), [0,1,1,1,2,2]):
prog::test(combinat::compositions::descents([0,1,3,2,15], FinalDescent), [0,1,4,6,21]):
prog::test(combinat::compositions::descents([1,3,0,2,15], FinalDescent), [1,4,4,6,21]):
prog::test(combinat::compositions::descents([0,1,3,0,0,2,15,0], FinalDescent), [0,1,4,4,4,6,21,21]):

prog::testfunc(combinat::compositions::descentsSet):
prog::test(combinat::compositions::descentsSet([]), {}):
prog::test(combinat::compositions::descentsSet([3]), {}):
prog::test(combinat::compositions::descentsSet([1,2]), {1}):
prog::test(combinat::compositions::descentsSet([1,3,2]), {1,4}):
prog::test(combinat::compositions::descentsSet([1,3,2,15]), {1,4,6}):
prog::test(combinat::compositions::descentsSet([], FinalDescent), {}):
prog::test(combinat::compositions::descentsSet([3], FinalDescent), {3}):
prog::test(combinat::compositions::descentsSet([1,2], FinalDescent), {1,3}):
prog::test(combinat::compositions::descentsSet([1,3,2], FinalDescent), {1,4,6}):
prog::test(combinat::compositions::descentsSet([1,3,2,15], FinalDescent), {1,4,6,21}):

prog::testfunc(combinat::compositions::fromDescents):
prog::test(combinat::compositions::fromDescents([]), []):
prog::test(combinat::compositions::fromDescents({}), []):
prog::test(combinat::compositions::fromDescents([], 6), [6]):
prog::test(combinat::compositions::fromDescents({}, 6), [6]):
prog::test(combinat::compositions::fromDescents([2]), [2]):
prog::test(combinat::compositions::fromDescents({2}), [2]):
prog::test(combinat::compositions::fromDescents([2], 6), [2,4]):
prog::test(combinat::compositions::fromDescents({2}, 6), [2,4]):
prog::test(combinat::compositions::fromDescents([2,3]), [2,1]):
prog::test(combinat::compositions::fromDescents({2,3}), [2,1]):
prog::test(combinat::compositions::fromDescents([2,3], 6), [2,1,3]):
prog::test(combinat::compositions::fromDescents({2,3}, 6), [2,1,3]):
prog::test(combinat::compositions::fromDescents([], 0), []):
prog::test(combinat::compositions::fromDescents({}, 0), []):
prog::test(combinat::compositions::fromDescents([], 1), [1]):
prog::test(combinat::compositions::fromDescents({}, 1), [1]):
if prog::TestNoTestargs = FALSE then
prog::test(combinat::compositions::fromDescents([1], 1),  TrapError = 1028):
prog::test(combinat::compositions::fromDescents({1}, 1),  TrapError = 1028):
prog::test(combinat::compositions::fromDescents([1,4], 4),TrapError = 1028):
prog::test(combinat::compositions::fromDescents({1,4}, 4),TrapError = 1028):
end_if:
prog::test(combinat::compositions::fromDescents([1,4], 5), [1,3,1]):
prog::test(combinat::compositions::fromDescents({1,4}, 5), [1,3,1]):

prog::testfunc(combinat::compositions::peaks):
prog::test(map(combinat::compositions::list(0), combinat::compositions::peaks),
           [[]]):
prog::test(map(combinat::compositions::list(1), combinat::compositions::peaks),
           [[]]):
prog::test(map(combinat::compositions::list(2), combinat::compositions::peaks),
           [[], []]):
prog::test(map(combinat::compositions::list(3), combinat::compositions::peaks),
           [[], [2], [], []]):
prog::test(map(combinat::compositions::list(4), combinat::compositions::peaks),
           [[], [3], [2], [2], [], [3], [], []]):
prog::test(map(combinat::compositions::list(5), combinat::compositions::peaks),
           [[], [4], [3], [3], [2], [2, 4], [2], [2], [], [4], [3], [3], [], [4], [], []]):
prog::test(map(combinat::compositions::list(6), combinat::compositions::peaks),
           [[], [5], [4], [4], [3], [3, 5], [3], [3], [2], [2, 5], [2, 4], 
	    [2, 4], [2], [2, 5], [2], [2], [], [5], [4], [4], [3], [3, 5], 
	    [3], [3], [], [5], [4], [4], [], [5], [], []]):

prog::testfunc(combinat::compositions::peaksComposition):
prog::test(map(combinat::compositions::list(0), combinat::compositions::peaksComposition),
           [[]]):
prog::test(map(combinat::compositions::list(1), combinat::compositions::peaksComposition),
           [[1]]):
prog::test(map(combinat::compositions::list(2), combinat::compositions::peaksComposition),
           [[2], [2]]):
prog::test(map(combinat::compositions::list(3), combinat::compositions::peaksComposition),
           [[3], [2, 1], [3], [3]]):
prog::test(map(combinat::compositions::list(4), combinat::compositions::peaksComposition),
           [[4], [3, 1], [2, 2], [2, 2], [4], [3, 1], [4], [4]]):
prog::test(map(combinat::compositions::list(5), combinat::compositions::peaksComposition),
           [[5], [4, 1], [3, 2], [3, 2], [2, 3], [2, 2, 1], [2, 3], [2, 3], [5], 
	    [4, 1], [3, 2], [3, 2], [5], [4, 1], [5], [5]]):
prog::test(map(combinat::compositions::list(6), combinat::compositions::peaksComposition),
           [[6], [5, 1], [4, 2], [4, 2], [3, 3], [3, 2, 1], [3, 3], [3, 3], [2, 4], 
	    [2, 3, 1], [2, 2, 2], [2, 2, 2], [2, 4], [2, 3, 1], [2, 4], [2, 4], [6], 
	    [5, 1], [4, 2], [4, 2], [3, 3], [3, 2, 1], [3, 3], [3, 3], [6], [5, 1], 
	    [4, 2], [4, 2], [6], [5, 1], [6], [6]]):

prog::testfunc(combinat::compositions::fromPeaks):
prog::test(combinat::compositions::fromPeaks([], 3),
           [[3], [1, 2], [1, 1, 1]]):
if prog::TestNoTestargs = FALSE then
prog::test(combinat::compositions::fromPeaks([1], 3), TrapError=1028):
prog::test(combinat::compositions::fromPeaks([3], 3), TrapError=1028):
end_if:
prog::test(combinat::compositions::fromPeaks([2], 3),
           [[2, 1]]):
prog::test(combinat::compositions::fromPeaks([2, 5, 7], 9),
           [[2, 3, 2, 2], [2, 3, 2, 1, 1], [2, 1, 2, 2, 2], [2, 1, 2, 2, 1, 1]]):
// test de coherence. Calcul par selection = calcul direct 
for n from 1 to 5 do
    peakComp := combinat::compositions::list(n, MinPart=2).
	      map(combinat::compositions::list(n-1, MinPart=2), _concat, [1]);
    for pComp in peakComp do
	peakSet := combinat::compositions::peaks(pComp);
	prog::test(bool(select(combinat::compositions::list(n), 
			       comp -> combinat::compositions::peaks(comp) = peakSet) 
		= combinat::compositions::fromPeaks(peakSet, n))):
	prog::test(bool(select(combinat::compositions::list(n), 
			       comp -> combinat::compositions::peaks(comp) = peakSet) 
		= combinat::compositions::fromPeaks(peakSet.[n]))):	
    end_for;
end_for:

//////////////////////////////////////////////////////////////////////////////
// Global checking of the package

// This is done at the end to ensure as much as possible of the
// package is initialized. A recursive prog::init(combinat::subsets)
// would be usefull here.

prog::testfunc(combinat::compositions):

prog::check(combinat::compositions, 3):

prog::testexit():

// Local Variables: 
// mode: mupad
// End:
