/*****************************************************************************
  $Id: dyckWords.tst,v 1.1.1.3 2004/10/04 17:25:17 nthiery Exp $
  File:		lib/COMBINAT/TEST/dyckWords.tst
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Purpose:	Test file for the library 'combinat::dyckWords'
  Created:	24/02/2001
  License:	LGPL
*****************************************************************************/

prog::testinit("dyckWords", 4.1):

alias(S=combinat::dyckWords):

//////////////////////////////////////////////////////////////////////////////
// In this test file we use 1 and -1 as opening and closing symbols
combinat::dyckWords:
sysassign(combinat::dyckWords::openSymbol,   1):
sysassign(combinat::dyckWords::closeSymbol, -1):

prog::testfunc(combinat::dyckWords::isA):
prog::test(combinat::dyckWords::isA([]),
           TRUE):
prog::test(combinat::dyckWords::isA([1]),
           FALSE):
prog::test(combinat::dyckWords::isA([-1]),
           FALSE):
prog::test(combinat::dyckWords::isA([1, -1]),
           TRUE):
prog::test(combinat::dyckWords::isA([1, -1], 1),
           TRUE):
prog::test(combinat::dyckWords::isA([1, -1], 2),
           FALSE):
prog::test(combinat::dyckWords::isA([1, 1, -1, -1], 2),
           TRUE):
prog::test(combinat::dyckWords::isA([1, -1, -1, 1], 2),
           FALSE):
prog::test(combinat::dyckWords::isA([1, -1, -1, 1], 2, 2),
           FALSE):
prog::test(combinat::dyckWords::isA([1, -1, 1, -1], 2, 2),
           TRUE):
prog::test(combinat::dyckWords::isA([1, -1, 1, -1, 1], 3, 2),
           TRUE):
prog::test(combinat::dyckWords::isA([1, -1, 1, 1, -1], 3, 2),
           TRUE):
prog::test(combinat::dyckWords::isA([1, -1, 1, 1], 3, 1),
           TRUE):

// combinat::dyckWords::count is the same as combinat::catalan. Not tested.

prog::testfunc(combinat::dyckWords::list):

// The order in which the dyckWords are listed is not in the
// specifications of combinat::dyckWords::list, so we sort the result
// before comparing.

prog::test(     S::list(0),	[[]] ):
prog::test(     S::list(1),	[[1,-1]] ):
prog::test(sort(S::list(2)),
           sort([[1,1,-1,-1], [1,-1,1,-1]])):
prog::test(sort(S::list(3)),
           sort([[1,1,1,-1,-1,-1], [1,1,-1,1,-1,-1], [1,1,-1,-1,1,-1],
                 [1,-1,1,1,-1,-1], [1,-1,1,-1,1,-1]])):

for n from 0 to 8 do
    // to Allows for reuse in the testsuite. 
    prog::test(nops((listDyck[n] := S::list(n))),S::count(n)):
end_for:

// We don't test combinat::dyckWords::generator, since it's build on
// top of list

prog::testfunc(combinat::dyckWords::height):
prog::test(combinat::dyckWords::height([]),
           0):
prog::test(combinat::dyckWords::height([1, -1]),
           1):
prog::test(combinat::dyckWords::height([1, 1, -1]),
           2):
prog::test(combinat::dyckWords::height([1, 1, -1, -1]),
           2):
prog::test(combinat::dyckWords::height([1, -1, 1, -1]),
           1):
prog::test(combinat::dyckWords::height([1, 1, -1, -1, 1, -1]),
           2):
prog::test(combinat::dyckWords::height([1, 1, -1, -1, 1, 1, 1, -1, -1, -1]),
           3):

prog::testfunc(combinat::dyckWords::associatedParenthesis):
prog::test(combinat::dyckWords::associatedParenthesis([1, -1], 1),
           2):
prog::test(combinat::dyckWords::associatedParenthesis([1, -1, 1, -1], 1),
           2):
prog::test(combinat::dyckWords::associatedParenthesis([1, -1, 1, -1], 2),
           1):
prog::test(combinat::dyckWords::associatedParenthesis([1, -1, 1, -1], 3),
           4):
prog::test(combinat::dyckWords::associatedParenthesis([1, -1, 1, -1], 4),
	   3):
prog::test(combinat::dyckWords::associatedParenthesis([1, 1, -1, -1], 2),
           3):
prog::test(combinat::dyckWords::associatedParenthesis([1, 1, -1], 2),
           3):
prog::test(combinat::dyckWords::associatedParenthesis([1, 1], 2),
           FAIL):
prog::test(combinat::dyckWords::associatedParenthesis([1, 1, 0], 1),
           FAIL):
prog::test(combinat::dyckWords::associatedParenthesis([1, 1, 0, -1], 1),
           FAIL):
prog::test(combinat::dyckWords::associatedParenthesis([1, 1, 0, -1], 2),
           4):
// check that 	   
for dyck in listDyck[4].listDyck[5] do
    for i from 1 to nops(dyck) do 
	prog::test(combinat::dyckWords::associatedParenthesis
		   (dyck,
		    combinat::dyckWords::associatedParenthesis(dyck, i)),
           i):
    end_for;
end_for:

prog::testfunc(combinat::dyckWords::toNonCrossingPartition):

prog::test(S::toNonCrossingPartition([1,-1]),		{[1]}):
prog::test(S::toNonCrossingPartition([1,1,-1,-1]),	{[1, 2]}):
prog::test(S::toNonCrossingPartition([1,1,1,-1,-1,-1]),	{[1, 2, 3]}):
prog::test(S::toNonCrossingPartition([1,-1,1,-1,1,-1]),	{[1], [2], [3]}):
prog::test(S::toNonCrossingPartition([1,1,-1,1,-1,-1]),	{[2], [1, 3]}):

prog::testfunc(combinat::dyckWords::fromNonCrossingPartition):

prog::test(S::fromNonCrossingPartition({[1,2]}),	[1, 1, -1, -1]):
prog::test(S::fromNonCrossingPartition({[1],[2]}),	[1, -1, 1, -1]):

// Check that from and toNonCrossingPartitions are indeed inverse
// of each others
for n from 0 to 7 do
    for w in listDyck[n] do 
	prog::test(S::fromNonCrossingPartition (S::toNonCrossingPartition(w)),
		   w):
    end_for;
end_for:

prog::testfunc(combinat::dyckWords::fromString):

prog::test(S::fromString("(())"),	[1, 1, -1, -1]):
prog::test(S::toString([1, -1, 1, -1]), "()()"):

//////////////////////////////////////////////////////////////////////////////
// Global checking of the package

prog::testfunc(combinat::dyckWords):
prog::testexit():

prog::check(S, 3):


// Local Variables: 
// mode: mupad
// End:
