/*****************************************************************************
  $Id: generators.tst,v 1.1.1.4 2005/01/26 13:42:24 nthiery Exp $
  File:		lib/COMBINAT/TEST/generators.tst
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
		Authors of the combinat package of MuPAD 2.0.0
  Purpose:	Test file for the library 'combinat::generators'
  Created:	24/02/2001
  License:	LGPL
*****************************************************************************/

prog::testinit("generators"):

alias(G=combinat::generators):
alias(list = combinat::generators::toList):

prog::testfunc(combinat::generators::empty):
prog::test(list(G::empty), []):

prog::testfunc(combinat::generators::fromList):
prog::test(list(G::fromList([a,b,c,d])), [a,b,c,d]):

prog::testfunc(combinat::generators::count):
prog::test(G::count(G::fromList([a,b,c,d])), 4):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc(combinat::generators::count):
//////////////////////////////////////////////////////////////////////////////
g := G::empty:
prog::test(G::accumulate(g, z, _mult, d),
           z):
prog::test(G::accumulate(g, z, _plus, d),
           z):
prog::test(G::accumulate(g),
           0):
g := G::fromList([a,b,c]):
prog::test(G::accumulate(g),
           a + b + c):
g := G::fromList([a,b,c]):
prog::test(G::accumulate(g, 0, _mult),
           0):
g := G::fromList([a,b,c]):
prog::test(G::accumulate(g, 1, _mult),
           a*b*c):
g := G::fromList([a,b,c]):
prog::test(G::accumulate(g, z, _mult, d),
           a*b*c*d^3*z):

prog::testfunc(combinat::generators::repeater):
prog::test(list(G::repeater(a,4)), [a,a,a,a]):
prog::test((g:=G::repeater(a): 
	    [g() $ i=1..5]), [a,a,a,a,a]):

prog::testfunc(combinat::generators::fromNext):
prog::test((g := G::fromNext(1, (x)->x+1):
	    [g() $ i=1..5]), [1,2,3,4,5]):
prog::test((g := G::fromNext(1, id):
	    [g() $ i=1..5]), [1,1,1,1,1]):

prog::testfunc(combinat::generators::fromUnrank):
prog::test((g := G::fromUnrank((x)->x, 5):
	    [g() $ i=1..5]), [1,2,3,4,5]):
prog::test((g := G::fromUnrank((x, z) -> if x<=z then x/z else FAIL end_if, 5):
	    [g() $ i=1..6]), [1/5, 2/5, 3/5, 4/5, 1, FAIL]):
prog::test((g := G::fromUnrank((x, n) -> if x <= n then [0$(x-1), 1, 0$(n-x)] 
					 else FAIL end_if, 4):
	    [g() $ i=1..5]), 
	   [[1,0,0,0], [0,1,0,0], [0,0,1,0], [0,0,0,1],FAIL]):

prog::testfunc(combinat::generators::fromRange):
prog::test(list(G::fromRange(1..4)), [1,2,3,4]):

prog::testfunc(combinat::generators::compose):
prog::test(list(G::compose(x->x^2, G::fromRange(1..4))), [1,4,9,16]):

prog::testfunc(combinat::generators::map):
prog::test(list(G::map(G::fromRange(1..4), x->x^2)), [1,4,9,16]):
// Test that map is a copyable generator.
if version() = [2,0,0] then
    warning("Skipping test: generators are not copyable in MuPAD 2.0.0");
else
prog::test((g  := G::map(G::fromRange(1..4), x->x^2):
	    g1 := G::copy(g): 
	    list(g)), [1,4,9,16]):
prog::test(list(g1),  [1,4,9,16]):
// Test that map is a copyable generator, even recursively
prog::test((g  := G::map(G::fromRange(1..4), x->x^2):
	    g1 := G::map(g, x -> x+1):
	    g2 := G::copy(g1):
	    list(g2)), [2,5,10,17]):
prog::test( list(g1),  [2,5,10,17]):
end_if:

prog::testfunc(combinat::generators::concat):
prog::test(list(G::concat(G::empty, G::repeater(a,2), G::fromList([1,2,3]))),
           [a,a,1,2,3]):
prog::test(list(G::concat()),
           []):
if version() = [2,0,0] then
    warning("Skipping test: generators are not copyable in MuPAD 2.0.0");
else
prog::test((g  := G::concat(G::empty, G::repeater(a,2), G::fromList([1,2,3])):
	    g1 := G::copy(g); // tests that concat is copyable. 
	    list(g1)), [a,a,1,2,3]):
prog::test( list(g),   [a,a,1,2,3]):
end_if:

prog::testfunc(combinat::generators::pipe):
prog::test(list(G::pipe(G::fromRange(1..4), i -> G::fromRange(1..i))),
           [1,1,2,1,2,3,1,2,3,4]):
if version() = [2,0,0] then
    warning("Skipping test: generators are not copyable in MuPAD 2.0.0");
else
prog::test((g  := G::pipe(G::fromRange(1..4), i -> G::fromRange(1..i)):
	    g1 := G::copy(g);
	    list(g1)), [1,1,2,1,2,3,1,2,3,4]):
prog::test( list(g),   [1,1,2,1,2,3,1,2,3,4]):
prog::test((g  := G::pipe(G::fromRange(1..4), i -> G::fromRange(1..i)):
	    g(), g(), g(), g(): g1 := G::copy(g); // discard some imput.
	    list(g1)), [2,3,1,2,3,4]):
prog::test( list(g),   [2,3,1,2,3,4]):
end_if:

prog::testfunc(combinat::generators::cartesian):
prog::test(list(G::cartesian(G::empty,G::empty)),
           []):
prog::test(list(G::cartesian(G::empty,G::fromList([3,4]))),
           []):
prog::test(list(G::cartesian(G::fromList([3,4]))),
           [[3],[4]]):
prog::test(list(G::cartesian(G::fromList([1,2]),G::fromList([3,4]))),
           [[1,3],[1,4],[2,3],[2,4]]):
prog::test(list(G::cartesian(G::fromList([1,2]),G::fromList([3,4]), G::fromList([a]))),
           [[1,3,a],[1,4,a],[2,3,a],[2,4,a]]):
// Fomerly : test for the non-recursive-copy bug of copyClosure
// Now : test that the recursive  copy for cartesian works.
gen1 := G::map(G::fromList([1,2,5]), z-> z+1):
gen2 := G::map(G::fromList([1,2]), z-> z+2):
gCar := G::cartesian(gen1,gen2):
if version() = [2,0,0] then
    warning("Skipping test: generators are not copyable in MuPAD 2.0.0");
else
gCC  := G::copy(gCar): // gCC = generatorCartesianCopy ;-).
prog::test(list(gCC),  [[2,3],[2,4],[3,3],[3,4],[6,3],[6,4]]):
end_if:
prog::test(list(gCar), [[2,3],[2,4],[3,3],[3,4],[6,3],[6,4]]):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc(combinat::generators::cartesian):
//////////////////////////////////////////////////////////////////////////////

prog::test(combinat::generators::toList(combinat::generators::cartesianPower(combinat::generators::fromList([1, 2, 3]), 0)),
           [[]]):
prog::test(combinat::generators::toList(combinat::generators::cartesianPower(combinat::generators::fromList([1, 2, 3]), 1)),
           [[1], [2], [3]]):
prog::test(combinat::generators::toList(combinat::generators::cartesianPower(combinat::generators::fromList([1, 2, 3]), 2)),
           [[1, 1], [1, 2], [1, 3], [2, 1], [2, 2], [2, 3], [3, 1], [3, 2], [3, 3]]):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc(combinat::generators::multiset):
//////////////////////////////////////////////////////////////////////////////

if version() = [2,0,0] then
    warning("Skipping test: generators are not copyable in MuPAD 2.0.0");
else

s := G::multiset(G::fromList([]), 0):
prog::test(combinat::generators::toList(s),
           [[]]):
s := G::multiset(G::fromList([]), 1):
prog::test(combinat::generators::toList(s),
           []):
s := G::multiset(G::fromList([]), 2):
prog::test(combinat::generators::toList(s),
           []):

s := G::multiset(G::fromList([1]), 0):
prog::test(combinat::generators::toList(s),
           [[]]):
s := G::multiset(G::fromList([1]), 1):
prog::test(combinat::generators::toList(s),
           [[1]]):
s := G::multiset(G::fromList([1]), 2):
prog::test(combinat::generators::toList(s),
           [[1, 1]]):

s := G::multiset(G::fromList([1,2]),0):
prog::test(combinat::generators::toList(s),
           [[]]):
s := G::multiset(G::fromList([1,2]),1):
prog::test(combinat::generators::toList(s),
           [[1], [2]]):
s := G::multiset(G::fromList([1,2]),2):
prog::test(combinat::generators::toList(s),
           [[1, 1], [1, 2], [2, 2]]):

s := G::multiset(G::fromList([a,b,c]),0):
s := G::multiset(G::fromList([a,b,c]),3):
prog::test(combinat::generators::toList(s),
           [[a, a, a], [a, a, b], [a, a, c],
	               [a, b, b], [a, b, c],
	                          [a, c, c],
	    [b, b, b], [b, b, c],
	               [b, c, c],
	    [c, c, c]]):

g := G::multiset(G::map(G::fromList([1, [a,b]]), op),2):
prog::test(G::toList(g),
           [[1, 1], [1, a, b], [a, b, a, b]]):
end_if:

//////////////////////////////////////////////////////////////////////////////
prog::testfunc(combinat::generators::powerset):
//////////////////////////////////////////////////////////////////////////////

if version() = [2,0,0] then
    warning("Skipping test: generators are not copyable in MuPAD 2.0.0");
else
s := G::powerset(G::fromList([]), 0):
prog::test(combinat::generators::toList(s),
           [[]]):
s := G::powerset(G::fromList([]), 1):
prog::test(combinat::generators::toList(s),
           []):
s := G::powerset(G::fromList([]), 2):
prog::test(combinat::generators::toList(s),
           []):
s := G::powerset(G::fromList([]), 3):
prog::test(combinat::generators::toList(s),
           []):
s := G::powerset(G::fromList([1]), 0):
prog::test(combinat::generators::toList(s),
           [[]]):
s := G::powerset(G::fromList([1]), 1):
prog::test(combinat::generators::toList(s),
           [[1]]):
s := G::powerset(G::fromList([1]), 2):
prog::test(combinat::generators::toList(s),
           []):
s := G::powerset(G::fromList([1]), 3):
prog::test(combinat::generators::toList(s),
           []):
s := G::powerset(G::fromList([1,2]), 0):
prog::test(combinat::generators::toList(s),
           [[]]):
s := G::powerset(G::fromList([1,2]), 1):
prog::test(combinat::generators::toList(s),
           [[1], [2]]):
s := G::powerset(G::fromList([1,2]), 2):
prog::test(combinat::generators::toList(s),
           [[1, 2]]):
s := G::powerset(G::fromList([1,2]), 3):
prog::test(combinat::generators::toList(s),
           []):
s := G::powerset(G::fromList([1,2,3]), 0):
prog::test(combinat::generators::toList(s),
           [[]]):
s := G::powerset(G::fromList([1,2,3]), 1):
prog::test(combinat::generators::toList(s),
           [[1], [2], [3]]):
s := G::powerset(G::fromList([1,2,3]), 2):
prog::test(combinat::generators::toList(s),
           [[1, 2], [1, 3], [2, 3]]):
s := G::powerset(G::fromList([1,2,3]), 3):
prog::test(combinat::generators::toList(s),
           [[1, 2, 3]]):
s := G::powerset(G::fromList([1,2,3]), 4):
prog::test(combinat::generators::toList(s),
           []):

s := G::powerset(G::fromList([1,2,3,4]), 0):
prog::test(combinat::generators::toList(s),
           [[]]):
s := G::powerset(G::fromList([1,2,3,4]), 1):
prog::test(combinat::generators::toList(s),
           [[1], [2], [3], [4]]):
s := G::powerset(G::fromList([1,2,3,4]), 3):
prog::test(combinat::generators::toList(s),
           [[1, 2, 3], [1, 2, 4], [1, 3, 4], [2, 3, 4]]):
s := G::powerset(G::fromList([1,2,3,4]), 2):
prog::test(combinat::generators::toList(s),
           [[1, 2], [1, 3], [1, 4], [2, 3], [2, 4], [3, 4]]):
s := G::powerset(G::fromList([1,2,3,4]), 3):
prog::test(combinat::generators::toList(s),
           [[1, 2, 3], [1, 2, 4], [1, 3, 4], [2, 3, 4]]):
s := G::powerset(G::fromList([1,2,3,4]), 4):
prog::test(combinat::generators::toList(s),
           [[1, 2, 3, 4]]):
prog::test(combinat::generators::toList(s),
           []):
s := G::powerset(G::fromList([1,2,3,4]), 6):
prog::test(combinat::generators::toList(s),
           []):
s := G::powerset(combinat::partitions::generator(3), 3):
prog::test(s(),[[3], [2, 1], [1, 1, 1]]):
s := G::powerset(combinat::partitions::generator(3), 2):
prog::test(s(),
           [[3], [2, 1]]):
prog::test(s(),
           [[3], [1, 1, 1]]):
prog::test(s(),
           [[2, 1], [1, 1, 1]]):
prog::test(s(),
           FAIL):
end_if:

//////////////////////////////////////////////////////////////////////////////
prog::testfunc(combinat::generators::select):
//////////////////////////////////////////////////////////////////////////////

prog::test(list(G::select(G::fromList([0,3,a,b,1,d]), testtype, DOM_INT)),
           [0,3,1]):
prog::test(list(G::select(G::fromList([a,b,d]), testtype, DOM_INT)),
           []):
if version() = [2,0,0] then
    warning("Skipping test: generators are not copyable in MuPAD 2.0.0");
else
prog::test((g  := G::select(G::fromList([0,3,a,b,1,d]), testtype, DOM_INT);
	    g1 := G::copy(g);
	    list(g1)), [0,3,1]): // test that select is a copyable generator
prog::test( list(g),   [0,3,1]):
end_if:

//////////////////////////////////////////////////////////////////////////////
prog::testfunc(combinat::generators::copy):
//////////////////////////////////////////////////////////////////////////////

makeGenerator :=
proc()
    option escape;
    local i;
begin
    i := -3;
    proc() begin i := i+3 end;
end:
g := makeGenerator(3):
prog::test([g() $ i = 1..3],
           [0, 3, 6]):
if version() = [2,0,0] then
    warning("Skipping test: generators are not copyable in MuPAD 2.0.0");
else
g1 := combinat::generators::copy(g):
prog::test([g() $ i = 1..3],
           [9, 12, 15]):
prog::test([g1() $ i = 1..2],
           [9, 12]):
prog::test([g() $ i = 1..3],
           [18, 21, 24]):
g2 := combinat::generators::copy(g1):
prog::test([g() $ i = 1..3],
           [27, 30, 33]):
prog::test([g1() $ i = 1..2],
           [15, 18]):
prog::test([g2() $ i = 1..1],
           [15]):
prog::test([g() $ i = 1..3],
           [36, 39, 42]):
prog::test([g1() $ i = 1..2],
           [21, 24]):
prog::test([g2() $ i = 1..1],
           [18]):

end_if:

//////////////////////////////////////////////////////////////////////////////
prog::testfunc(combinat::generators::generatorWithCopy):
//////////////////////////////////////////////////////////////////////////////

makeGenerator :=
proc()
    option escape;
    local g, n;
begin
    n := 0;
    g := G::fromNext(1, i->i+1);
    G::generatorWithCopy(proc()
			 begin
			     n := n+1;
			     [g(), n];
			 end_proc,
			 proc()
			 begin
			     g := G::copy(g);
			 end_proc);
end_proc:


g := makeGenerator(0):
prog::test([g() $ i = 1..3],
           [[1, 1], [2, 2], [3, 3]]):
if version() = [2,0,0] then
    warning("Skipping test: generators are not copyable in MuPAD 2.0.0");
else
g1 := g::copy(g):
prog::test([g() $ i = 1..3],
           [[4, 4], [5, 5], [6, 6]]):
prog::test([g1() $ i = 1..2],
           [[4, 4], [5, 5]]):
prog::test([g() $ i = 1..3],
           [[7, 7], [8, 8], [9, 9]]):
g2 := G::copy(g1):
prog::test([g() $ i = 1..3],
           [[10, 10], [11, 11], [12, 12]]):
prog::test([g1() $ i = 1..2],
           [[6, 6], [7, 7]]):
prog::test([g2() $ i = 1..1],
           [[6, 6]]):
prog::test([g() $ i = 1..3],
           [[13, 13], [14, 14], [15, 15]]):
prog::test([g1() $ i = 1..2],
           [[8, 8], [9, 9]]):
prog::test([g2() $ i = 1..1],
           [[7, 7]]):
end_if:

//////////////////////////////////////////////////////////////////////////////
// Global checking of the package

// This is done at the end to ensure as much as possible of the
// package is initialized. A recursive prog::init(combinat::generators)
// would be usefull here.

prog::testfunc(combinat::generators):
prog::testexit():

prog::check(combinat::generators, 3):


// Local Variables: 
// mode: mupad
// End:
