/*****************************************************************************
  $Id: graphs.tst,v 1.1.1.2 2004/10/16 15:22:40 nthiery Exp $
  File:		lib/COMBINAT/TEST/graphs.tst
  Authors:	Florent Hivert <florent.hivert@univ-mlv.fr>
		Nicolas Thiery <nthiery@users.sf.net>
  Purpose:	Test suite for the domain combinat::graphs
  Created:	Tue Apr 13 11:03:49 2004
  License:	LGPL
  Project:	MuPAD-Combinat, http://mupad-combinat.sf.net
*****************************************************************************/

export(prog, Alias, test):

prog::testinit("graphs"):

if module::which(hold(Nauty)) = FAIL then
    warning("module nauty not available; not checking the graphs library");
else
// Get around a kernel crash in MuPAD 2.0.0
// when the module stdmod is loaded before another module
%if version()=[2,0,0] then
    module::displace(stdmod):
end_if:

// Does not work with an alias because we are inside an if statement
// alias(graphs= combinat::graphs):
graphs := combinat::graphs:

//////////////////////////////////////////////////////////////////////////////
prog::testfunc(graphs):
//////////////////////////////////////////////////////////////////////////////

test(sort(expr(graphs::list(4))),
     sort([// 0 nodes
	   [[], [], [], []],
	   // 1 node
	   [[], [], [4], [3]],
	   // 2 nodes
	   [[2], [1], [4], [3]], [[], [4], [4], [2, 3]],
	   // 3 nodes
	   [[4], [4], [4], [1, 2, 3]], [[], [3, 4], [2, 4], [2, 3]],
	   [[3], [4], [1, 4], [2, 3]],
	   // 4 nodes
	   [[2, 3], [1, 4], [1, 4], [2, 3]], [[4], [3, 4], [2, 4], [1, 2, 3]],
	   // 5 nodes
	   [[3, 4], [3, 4], [1, 2, 4], [1, 2, 3]],
	   // 6 nodes
	   [[2, 3, 4], [1, 3, 4], [1, 2, 4], [1, 2, 3]]])):

//////////////////////////////////////////////////////////////////////////////
// Examples taken from geng.c:

// Trees on n nodes
test([graphs::count(n, Connected, MaxEdges = n - 1) $ n = 1..10],
     [1, 1, 1, 2, 3, 6, 11, 23, 47, 106]):
// C3-free graphs
test([graphs::count(n, C3Free) $ n = 1..8],
     [1, 2, 3, 7, 14, 38, 107, 410]):
// C4-free graphs
test([graphs::count(n, C4Free) $ n = 1..8],
     [1, 2, 4, 8, 18, 44, 117, 351]):
// Bipartite graphs
test([graphs::count(n, Bipartite) $ n = 1..9],
     [1, 2, 3, 7, 13, 35, 88, 303, 1119]):
// C4-free Bipartite graphs
test([graphs::count(n, Bipartite, C4Free) $ n = 1..10],
     [1, 2, 3, 6, 10, 21, 39, 86, 182, 440]):
// (C3,C4)-free Bipartite graphs
test([graphs::count(n, C3Free, C4Free) $ n = 1..10],
     [1, 2, 3, 6, 11, 23, 48, 114, 293, 869]):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("straighten"):
//////////////////////////////////////////////////////////////////////////////
test(graphs::straighten(graphs([])), graphs([])):
test(graphs::straighten(graphs([[]])), graphs([[]])):
test(graphs::straighten(graphs([[1],[2],[]])), graphs([[], [2], [3]])):
test(graphs::straighten(graphs([[4], [3], [2, 4], [3, 1, 5], [4]])),
     graphs([[5], [5], [4], [3, 5], [1, 2, 4]])):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("addEdge"):
//////////////////////////////////////////////////////////////////////////////
test(graphs::addEdge(graphs([[],[]]), [1,2]), graphs([[2],[1]])):
test(graphs::addEdge(graphs([[],[],[]]), [1,2]), graphs([[2],[1],[]])):
test(graphs::addEdge(graphs([[3],[3],[3]]), [1,2]), graphs([[2,3],[1,3],[3]])):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("removeEdge"):
//////////////////////////////////////////////////////////////////////////////
test(graphs::removeEdge(graphs([[2],[1]]), [1,2]), graphs([[],[]])):
test(graphs::removeEdge(graphs([[2],[1],[]]), [1,2]), graphs([[],[],[]])):
test(graphs::removeEdge(graphs([[2,3],[1,3],[3]]), [1,2]), graphs([[3],[3],[3]])):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("size"):
//////////////////////////////////////////////////////////////////////////////
test(graphs::size(graphs([])), 0):
test(graphs::size(graphs([[2],[1]])), 2):
test(graphs::size(graphs([[2],[1],[]])), 3):
test(graphs::size(graphs([[2,3],[1,3],[3]])), 3):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("getEdgesNumber"):
//////////////////////////////////////////////////////////////////////////////
test(graphs::getEdgeNumber(graphs([])), 0):
test(graphs::getEdgeNumber(graphs([[2],[1]])), 2):
test(graphs::getEdgeNumber(graphs([[2],[1],[]])), 2):
test(graphs::getEdgeNumber(graphs([[2,3],[1,3],[3]])), 5):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("complement"):
//////////////////////////////////////////////////////////////////////////////
test(graphs::complement(graphs([])), graphs([])):
test(graphs::complement(graphs([[2],[1]])), graphs([[],[]])):
test(graphs::complement(graphs([[2],[1],[]])), graphs([[3], [3], [1, 2]])):
test(graphs::complement(graphs([[2,3],[1,3],[3]])), graphs([[1], [2], [1, 2]])):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("revert"):
//////////////////////////////////////////////////////////////////////////////
test(graphs::revert(graphs([])), graphs([])):
test(graphs::revert(graphs([[2],[1]])), graphs([[2],[1]])):
test(graphs::revert(graphs([[2],[1],[]])), graphs([[2], [1], []])):
test(graphs::revert(graphs([[2,3],[1,3],[3]])), graphs([[2], [1], [1, 2, 3]])):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("relabel"):
//////////////////////////////////////////////////////////////////////////////
test(graphs::relabel(graphs([]), []), graphs([])):
test(graphs::relabel(graphs([[2],[1]]), [1,2]),
     graphs([[2],[1]])):
test(graphs::relabel(graphs([[2],[1]]), [2,1]),
     graphs([[2],[1]])):
test(graphs::relabel(graphs([[2],[1],[]]), [2,1,3]),
     graphs([[2],[1],[]])):
test(graphs::relabel(graphs([[2],[1],[]]), [3,2,1]),
     graphs([[],[3],[2]])):
test(graphs::relabel(graphs([[2],[1],[]]), [3,1,2]),
     graphs([[],[3],[2]])):
test(graphs::relabel(graphs([[2,3],[1,3],[3],[5],[3,6,4],[1,3,4]]),[3,2,6,4,5,1]),
     graphs([[1], [1, 6], [1, 4, 6], [5], [1, 3, 4], [1, 2]])):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("sublabel"):
//////////////////////////////////////////////////////////////////////////////
test(graphs::sublabel(graphs([[2,3],[1,3],[3],[5],[3,6,4],[1,3,4]]),[3,2,6,4]),
     graphs([[1], [1], [1, 4], []])):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("directSum"):
//////////////////////////////////////////////////////////////////////////////
test(expr(graphs::directSum(graphs([[2],[1]]),
				  graphs([[]]),
				  graphs([]),
				  graphs([[1]]))),
     [[2], [1], [], [4]]):
test(expr(graphs::directSum()), []):

//////////////////////////////////////////////////////////////////////////////
end_if:	// availability of Nauty
prog::testexit():
//////////////////////////////////////////////////////////////////////////////

prog::check(graphs, 3):

// Local Variables:
// mode: mupad
// End:
