/*****************************************************************************
  $Id: integerListsLexTools.tst,v 1.1.1.3 2005/01/26 13:42:25 nthiery Exp $
  File:		lib/COMBINAT/TEST/integerListsLexTools.tst
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
		Florent Hivert <Florent.Hivert@univ-mlv.fr>
  Purpose:	Test file for the library combinat::integerListsLexTools
  Created:	07/07/2001
  License:	LGPL
*****************************************************************************/

prog::testinit("integerListsLexTools"):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("infinity type checking"):
//////////////////////////////////////////////////////////////////////////////
prog::test(testtype( infinity, combinat::integerListsLexTools::typeINF )):
prog::test(testtype(-infinity, combinat::integerListsLexTools::typeNINF)):

// Optimization for MuPAD > 2.0.0: we use RD_INF which is faster than infinity
%if traperror(RD_INF+RD_INF) = 0 then
    alias(infinity  = RD_INF);
end_if:

prog::test(testtype( infinity, combinat::integerListsLexTools::typeINF )):
prog::test(testtype(-infinity, combinat::integerListsLexTools::typeNINF)):

//////////////////////////////////////////////////////////////////////////////
// Returns a function g such that g(i)=l[i]
f :=
proc(l)
    option escape;
begin
    proc(i: Type::PosInt)
    begin
        l[i]
    end_proc;
end_proc:

list:=
proc(n)
        local a, l;
begin
    l:=[];
    gen:=combinat::integerListsLexTools::generator(args());
    while (a:=gen()) <> FAIL do
        l:=append(l,a);
    end_while;
    return(l);
end_proc:

partitions     :=	0, infinity,	   ()->0, ()->infinity, -infinity, 0:
partitionsmin2 :=	0, infinity,	   ()->2, ()->infinity, -infinity, 0:

subpartitions  :=p ->	(0, infinity,	   ()->0, combinat::integerListsLexTools::comp2ceil(p,-infinity,0),-infinity, 0):
uppartitions   := p ->	(0, infinity,	   ()->0, combinat::integerListsLexTools::comp2floor(p,-infinity,0),-infinity, 0):

compositions   :=	0, infinity,	   ()->1, ()->infinity, -infinity, infinity:
integerVectors := l ->	(l, l,		   ()->0, ()->infinity, -infinity, infinity):
lowerMonomials := c ->	(nops(c), nops(c), ()->0, i->c[i],    -infinity, infinity):
upperMonomials := c ->	(nops(c), nops(c), ()->0, i->c[i],    -infinity, infinity):

//pseudoCompositions :=	0, infinity,	   ()->0, ()->infinity, -infinity, 0:

prog::testfunc(combinat::integerListsLexTools::lowerRegular):

prog::test(combinat::integerListsLexTools::lowerRegular([4,2,6],-1,1), [3,2,3]):
prog::test(combinat::integerListsLexTools::lowerRegular([4,2,6],-1,infinity), [3,2,6]):
prog::test(combinat::integerListsLexTools::lowerRegular([1,4,2],-1,1), [1,2,2]):
prog::test(combinat::integerListsLexTools::lowerRegular([4,2,6,3,7],-2,1), [4,2,3,3,4]):
prog::test(combinat::integerListsLexTools::lowerRegular([4,2,infinity,3,7],-2,1), [4,2,3,3,4]):
prog::test(combinat::integerListsLexTools::lowerRegular([1,infinity,2],-1,1), [1,2,2]):
prog::test(combinat::integerListsLexTools::lowerRegular([infinity,4,2],-1,1), [4,3,2]):

prog::testfunc(combinat::integerListsLexTools::upperRegular):
prog::test(combinat::integerListsLexTools::upperRegular([4,2,6],-1,1), [4,5,6]):
prog::test(combinat::integerListsLexTools::upperRegular([4,2,6],-2,1), [4,5,6]):
prog::test(combinat::integerListsLexTools::upperRegular([4,2,6,3,7],-2,1), [4,5,6,6,7]):
prog::test(combinat::integerListsLexTools::upperRegular([4,2,6,1],-2,1), [4,5,6,4]):

prog::testfunc(combinat::integerListsLexTools::positions):
prog::test(combinat::integerListsLexTools::positions(4,f([0,0,0,0]),f([4,4,4,4]),-1),
	   [[1, 1, 1, 1], [2, 1, 1, 2], [3, 1, 1, 3], [4, 1, 1, 4], [4, 2, 2, 4],
	   [4, 3, 3, 4], [4, 4, 4, 4]]):
prog::test(combinat::integerListsLexTools::positions(4,f([0,0,0,0]),f([4,4,4,4]),-2),
	[[1, 1, 1, 1], [1, 2, 1, 2], [2, 1, 1, 3], [2, 2, 1, 4], [3, 1, 2, 3],
	 [3, 2, 2, 4], [4, 1, 3, 3], [4, 2, 3, 4], [4, 3, 4, 3], [4, 4, 4, 4]]):

prog::testfunc(combinat::integerListsLexTools::first):
prog::test(combinat::integerListsLexTools::first(12, 4, 4, f([0,0,0,0]), f([4,4,4,4]), -1, +1),
	   [4, 3, 3, 2]):
prog::test(combinat::integerListsLexTools::first(36, 9, 9, f([3,3,3,2,1,1,0,0,0]), f([7,6,5,5,5,5,5,4,4]), -1, +1),
	   [7, 6, 5, 5, 4, 3, 3, 2, 1]):
prog::test(combinat::integerListsLexTools::first(25, 9, 9, f([3,3,3,2,1,1,0,0,0]), f([7,6,5,5,5,5,5,4,4]), -2, +1),
	   [7, 6, 5, 4, 2, 1, 0, 0, 0]):
prog::test(combinat::integerListsLexTools::first(36, 9, 9, f([3,3,3,2,1,4,2,0,0]), f([7,6,5,5,5,5,5,4,4]), -2, +1),
	   [7, 6, 5, 5, 5, 4, 3, 1, 0]):
// Florent Test du patch du 19/01/05.
prog::test(combinat::integerListsLexTools(0, 0, infinity, 1, infinity, 0, infinity), [[]]):
prog::test(combinat::integerListsLexTools(0, 0, infinity, 1, infinity, 0, 0), [[]]):
prog::test(combinat::integerListsLexTools(0, 0, 0, 1, infinity, 0, 0), [[]]):
prog::test(combinat::integerListsLexTools(0, 0, 0, 0, infinity, 0, 0), [[]]):
prog::test(combinat::integerListsLexTools(0, 0, infinity, 1, infinity, 0, infinity), [[]]):
prog::test(combinat::integerListsLexTools(1, 0, infinity, 1, infinity, 0, infinity), [[1]]):
prog::test(combinat::integerListsLexTools(0, 1, infinity, 1, infinity, 0, infinity), []):
prog::test(combinat::integerListsLexTools(0, 1, infinity, 0, infinity, 0, infinity), [[0]]):

prog::testfunc(combinat::integerListsLexTools::rightmostPivot):
prog::test(combinat::integerListsLexTools::rightmostPivot([7,6,5,5,4,3,3,2,1], 9, 9,
                              f([3,3,3,2,1,1,0,0,0]), f([7,6,5,5,5,5,5,4,4]),
                              -1, 0),
           [7,2]):
prog::test(combinat::integerListsLexTools::rightmostPivot([7,6,5,5,4,3,3,2,1], 9, 9,
                              f([3,3,3,2,1,1,0,0,0]), f([7,6,5,5,5,5,5,4,4]),
                              -2, 0),
           [7,1]):
prog::test(combinat::integerListsLexTools::rightmostPivot([7,6,5,5,4,3,3,2,1], 9, 9,
                              f([3,3,3,2,1,1,0,0,0]), f([7,6,5,5,5,5,5,4,4]),
                              -2, 4),
           [8,1]):
prog::test(combinat::integerListsLexTools::rightmostPivot([7,6,5,5,4,3,3,2,1], 9, 9,
                              f([3,3,3,2,1,1,0,0,0]), f([7,6,5,5,5,5,5,4,4]),
                              -2, 1),
           [8,1]):
prog::test(combinat::integerListsLexTools::rightmostPivot([7,6,5,5,5,5,5,4,4], 9, 9,
                              f([3,3,3,2,1,1,0,0,0]), f([7,6,5,5,5,5,5,4,4]),
                              -2, 1),
           FAIL):
prog::test(combinat::integerListsLexTools::rightmostPivot([3,3,3,2,1,1,0,0,0], 9, 9,
                              f([3,3,3,2,1,1,0,0,0]), f([7,6,5,5,5,5,5,4,4]),
                              -2, 1),
           FAIL):

prog::test(combinat::integerListsLexTools::rightmostPivot([1],1,1,()->0,()->2,-10,+10),
            FAIL):
prog::test(combinat::integerListsLexTools::rightmostPivot([1,2],2,2,()->0,()->2,-10,+10),
            FAIL):
prog::test(combinat::integerListsLexTools::rightmostPivot([1,2],2,2,()->1,()->2,-10,+10),
            FAIL):
prog::test(combinat::integerListsLexTools::rightmostPivot([1,2],2,3,()->1,()->2,-10,+10),
            [2, 1]):
prog::test(combinat::integerListsLexTools::rightmostPivot([2,2],2,3,()->2,()->2,-10,+10),
            FAIL):
prog::test(combinat::integerListsLexTools::rightmostPivot([2,3],2,3,()->2,()->2,-10,+10),
            FAIL):
prog::test(combinat::integerListsLexTools::rightmostPivot([3,2],2,3,()->2,()->2,-10,+10),
            FAIL):
prog::test(combinat::integerListsLexTools::rightmostPivot([3,3],2,3,()->2,()->2,-10,+10),
            [1, 2]):
prog::test(combinat::integerListsLexTools::rightmostPivot([6],1,3,()->0,()->6,-1,0),
            [1, 0]):
prog::test(combinat::integerListsLexTools::rightmostPivot([6],1,3,()->0,()->6,-2,0),
            [1, 0]):
prog::test(combinat::integerListsLexTools::rightmostPivot([7,9,8,7],1,5,()->0,()->10,-1,10),
            [2, 6]):
prog::test(combinat::integerListsLexTools::rightmostPivot([7,9,8,7],1,5,()->5,()->10,-10,10),
            [3, 5]):
prog::test(combinat::integerListsLexTools::rightmostPivot([7,9,8,7],1,5,()->5,()->10,-1,10),
            [2, 6]):
prog::test(combinat::integerListsLexTools::rightmostPivot([7,9,8,7],1,5,()->4,()->10,-2,10),
            [3, 7]):
prog::test(combinat::integerListsLexTools::rightmostPivot([9,8,7],1,4,()->4,()->10,-2,0),
            [1, 4]):

prog::test(combinat::integerListsLexTools::rightmostPivot([1,3],1,5,(i)->i,()->10,-10,10),
            FAIL):
prog::test(combinat::integerListsLexTools::rightmostPivot([1,4],1,5,(i)->i,()->10,-10,10),
            FAIL):
prog::test(combinat::integerListsLexTools::rightmostPivot([2,4],1,5,(i)->i,()->10,-10,10),
            [1, 1]):


prog::testfunc(combinat::integerListsLexTools::Next):

// Check generation of infinite list when 0 are allowed ?

constraints := 0, infinity, ()->1, ()->infinity, -1, 0:
prog::test(list(6, constraints),
            [[6], [3, 3], [3, 2, 1], [2, 2, 2], [2, 2, 1, 1],
             [2, 1, 1, 1, 1], [1, 1, 1, 1, 1, 1]]):

prog::test(list(1,partitionsmin2),
            []):
prog::test(list(2,partitionsmin2),
            [[2]]):
prog::test(list(3,partitionsmin2),
            [[3]]):
prog::test(list(4,partitionsmin2),
            [[4],
             [2, 2]]):
prog::test(list(5,partitionsmin2),
            [[5],
             [3, 2]]):
prog::test(list(6,partitionsmin2),
            [[6],
             [4, 2],
             [3, 3],
             [2, 2, 2]]):
prog::test(list(7,partitionsmin2),
            [[7],
             [5, 2],
             [4, 3],
             [3, 2, 2]
             ]):
prog::test(list(9,partitionsmin2),
            [[9],
             [7, 2],
             [6, 3],
             [5, 4],
             [5, 2, 2],
             [4, 3, 2],
             [3, 3, 3],
             [3, 2, 2, 2]]):
prog::test(list(10,partitionsmin2),
            [[10],
             [8, 2],
             [7, 3],
             [6, 4],
             [6, 2, 2],
             [5, 5],
             [5, 3, 2],
             [4, 4, 2],
             [4, 3, 3],
             [4, 2, 2, 2],
             [3, 3, 2, 2],
             [2, 2, 2, 2, 2]]):

prog::testexit():

// Local Variables: 
// mode: mupad
// End:
