/*****************************************************************************
  $Id: linearExtensions.tst,v 1.1.1.3 2004/10/04 17:25:17 nthiery Exp $
  File:         lib/COMBINAT/TEST/linearExtensions.tst
  Authors:      Florent Hivert <florent.hivert@univ-mlv.fr>
  Purpose:	Test file for the library 'combinat::linearExtensions'
  Created:      26/09/2002
  License:      LGPL
*****************************************************************************/

prog::testinit("linearExtensions", 8.9):


/***************************************************************************/
alias(LE=combinat::linearExtensions):

prog::testfunc(LE::isA):
gr := Graph([],[], Directed):
prog::test(LE::isA([], gr), TRUE):
gr := Graph([1],[], Directed):
prog::test(LE::isA([1], gr), TRUE):
gr := Graph([1],[], Directed):
prog::test(LE::isA([2], gr), FALSE):

gr := Graph([1,2],[[1,2]], Directed):
prog::test(LE::isA([1,2], gr), TRUE):
gr := Graph([1,2],[[2,1]], Directed):
prog::test(LE::isA([2,1], gr), TRUE):

gr := Graph([1,2,3],[[1,2],[1,3]], Directed):
prog::test(LE::isA([1,2,3], gr), TRUE):
gr := Graph([1,2,3],[[1,2],[1,3]], Directed):
prog::test(LE::isA([2,1,3], gr), FALSE):

gr := Graph([1,2,3],[[2,1],[2,3]], Directed):
prog::test(LE::isA([2,1,3], gr), TRUE):
gr := Graph([1,2,3],[[2,1],[2,3]], Directed):
prog::test(LE::isA([1,2,3], gr), FALSE):

gr := Graph([1,2,3,5,6,7],[[2,1],[2,3],[3,5],[1,7],[5,6],[7,5]], Directed):
prog::test(LE::isA([2,1,3,7,5,6], gr), TRUE):
gr := Graph([1,2,3,5,6,7],[[2,1],[2,3],[3,5],[1,7],[5,6],[7,5]], Directed):
prog::test(LE::isA([2,3,1,7,5,6], gr), TRUE):
gr := Graph([1,2,3,5,6,7],[[2,1],[2,3],[3,5],[1,7],[5,6],[7,5]], Directed):
prog::test(LE::isA([2,3,7,1,5,6], gr), FALSE):
gr := Graph([1,2,3,5,6,7],[[2,1],[2,3],[3,5],[1,7],[5,6],[7,6]], Directed):
prog::test(LE::isA([2,1,3,5,7,6], gr), TRUE):


/***************************************************************************/
prog::testfunc(LE::first):

gr := Graph([],[], Directed):
prog::test(LE::first(gr), []):
gr := Graph([1],[], Directed):
prog::test(LE::first(gr), [1]):
gr := Graph([1],[[1,1]], Directed):
prog::test(LE::first(gr), FAIL):

gr := Graph([1,2],[[1,2]], Directed):
prog::test(LE::first(gr), [1,2]):
gr := Graph([1,2],[[2,1]], Directed):
prog::test(LE::first(gr), [2,1]):
gr := Graph([1,2],[[1,2],[2,1]], Directed):
prog::test(LE::first(gr), FAIL):

gr := Graph([1,2,3],[[1,2],[1,3]], Directed):
prog::test(LE::first(gr), [1,2,3]):

gr := Graph([1,2,3],[[2,1],[2,3]], Directed):
prog::test(LE::first(gr), [2,1,3]):

gr := Graph([1,2,3,5,6,7],[[2,1],[2,3],[3,5],[1,7],[5,6],[7,5]], Directed):
prog::test(LE::first(gr), [2,1,3,7,5,6]):

gr := Graph([1,2,3,5,6,7],[[2,1],[2,3],[3,5],[1,7],[5,6],[7,6]], Directed):
prog::test(LE::first(gr), [2,1,3,5,7,6]):



/***************************************************************************/
// generator is not tested because it is called by list
prog::testfunc(LE::list):
gr := Graph([1,2,3,4],[[1,2]], Directed):
prog::test(combinat::linearExtensions::list(gr),
           [[1, 2, 3, 4], [1, 2, 4, 3], [1, 3, 2, 4], [1, 3, 4, 2], [1, 4, 2, 3], [1, 4, 3, 2], [3, 1, 2, 4], [3, 1, 4, 2], [3, 4, 1, 2], [4, 1, 2, 3], [4, 1, 3, 2], [4, 3, 1, 2]]):
gr := Graph([1,2,3,4],[[1,2],[3,2]], Directed):
prog::test(combinat::linearExtensions::list(gr),
           [[1, 3, 2, 4], [1, 3, 4, 2], [1, 4, 3, 2], [3, 1, 2, 4], [3, 1, 4, 2], [3, 4, 1, 2], [4, 1, 3, 2], [4, 3, 1, 2]]):


MaxTest := 7:

// The linear extensions of the empty graphs are all permutations
for i from 1 to MaxTest do
    gr := Graph([$1..i],[], Directed):
    prog::test(bool(combinat::linearExtensions::list(gr) = 
		    sort(combinat::permutations::list(i)))):
end_for:

// A cyclic graph has no linear extensions
for i from 1 to MaxTest do
    gr := Graph::createCircleGraph(i, Directed):
    prog::test(combinat::linearExtensions::list(gr), []);
end_for:


// Another way to enumerate the standard tableau of a given shape
listTab2 := 
    proc(sh)
	local n, fil, gr;
    begin
	n   := _plus(op(sh));
	fil := combinat::tableaux::fromShapeAndWord(sh,[$1..n]);
	gr  := combinat::tableaux::toPoset(fil);
	map(combinat::linearExtensions::list(gr), 
	    (l)->combinat::tableaux::fromShapeAndWord(sh,
		   combinat::permutations::inverse(l)));
    end_proc:

for n from 1 to MaxTest do
    for sh in combinat::partitions::list(n) do
	prog::test(bool(sort(listTab2(sh)) = sort(combinat::tableaux::list(sh))));
    end_for;
end_for:

nRandomTests := 20:
n := 6:
(gr := Graph::createRandomGraph(n, 4, Directed):
 prog::test(bool(sort(LE::list(gr)) = 
	    sort(select(combinat::permutations::list(n), 
			l -> LE::isA(l, gr)))))
) $nRandomTests:

/////////////////////////////////////////////////////////////////////////////
// Global checking of the package

// This is done at the end to ensure as much as possible of the
// package is initialized. A recursive prog::init(combinat::linearExtensions)
// would be usefull here.

prog::testfunc(linearExtensions):
prog::testexit():

prog::check(linearExtensions, 3):


// Local Variables:
// mode: mupad
// End:
