/*****************************************************************************
  $Id: permutations.tst,v 1.1.1.6 2005/01/26 13:42:25 nthiery Exp $
  File:		lib/COMBINAT/TEST/permutations.tst
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  		Florent Hivert <Florent.Hivert@univ-mlv.fr>
                Xavier Molinero <molinero@lsi.upc.es>
		Authors of the combinat package of MuPAD 2.0.0
  Purpose:	Test file for the library 'combinat::permutations'
  Created:	24/02/2001
  License:	LGPL
*****************************************************************************/

prog::testinit("permutations", 26.1):

alias(generators   = combinat::generators):
alias(permutations = combinat::permutations):
alias(compositions = combinat::compositions):
alias(words        = combinat::words):

prog::testfunc(permutations::isA):
prog::test( permutations::isA([]), TRUE):
prog::test( permutations::isA([], 0), TRUE):
prog::test( permutations::isA([1]), TRUE):
prog::test( permutations::isA([1], 1), TRUE):
prog::test( permutations::isA([1,2], 2), TRUE):
prog::test( permutations::isA([1,3], 2), FALSE):
prog::test( permutations::isA([1,3], 3), FALSE):
prog::test( permutations::isA([1,3,2], 3), TRUE):
prog::test( permutations::isA([2,3,2], 3), FALSE):
prog::test( permutations::isA([2,3,2], 2), FALSE):

prog::testfunc(permutations::testtype):
prog::test( testtype(3,       permutations::typeStandard), FALSE):
prog::test( testtype([a],     permutations::typeStandard), FALSE):
prog::test( testtype([],      permutations::typeStandard), TRUE):
prog::test( testtype([1],     permutations::typeStandard), TRUE):
prog::test( testtype([1,2],   permutations::typeStandard), TRUE):
prog::test( testtype([1,3],   permutations::typeStandard), FALSE):
prog::test( testtype([1,3,2], permutations::typeStandard), TRUE):
prog::test( testtype([2,3,2], permutations::typeStandard), FALSE):

prog::testfunc(permutations::isStandard):
prog::test( permutations::isStandard([]), TRUE):
prog::test( permutations::isStandard([1]), TRUE):
prog::test( permutations::isStandard([1,2]), TRUE):
prog::test( permutations::isStandard([1,3]), FALSE):
prog::test( permutations::isStandard([1,3,2]), TRUE):
prog::test( permutations::isStandard([2,3,2]), FALSE):

prog::testfunc(permutations::count):

prog::test( permutations::count(0), 1 ):
prog::test( permutations::count(1), 1 ):
prog::test( permutations::count(4), 4! ):
prog::test( permutations::count([]), 1 ):
prog::test( permutations::count([1,2,3,4]), 4! ):
prog::test( permutations::count([1,1,2,2,2]), 10 ):
prog::test( permutations::count([1,1,2,2,2], Duplicate), 120 ):

prog::testfunc(permutations::list):

prog::test(permutations::list(0),
	   [[]]):
prog::test(permutations::list(0, Duplicate),
           [[]]):
prog::test(permutations::list([a]),
           [[a]]):
prog::test(permutations::list([a], Duplicate),
           [[a]]):
prog::test(permutations::list([1,2]),
           [[1, 2], [2, 1]]):
prog::test(permutations::list([1,2], Duplicate),
	   [[1, 2], [2, 1]]):
prog::test(permutations::list([1,1]),
	   [[1, 1]]):
prog::test(permutations::list([1,1], Duplicate),
	   [[1, 1], [1, 1]]):
prog::test(permutations::list([a,b,b]),
	   [[a, b, b], [b, a, b], [b, b, a]]):

Pref::typeCheck(Always):
testargs(TRUE):

for l in [0, 1, 4, [], [1,2,3,4], [1,1,2,2,2], [d,b,a,c], [a,a,a,b,b,c]] do
    list1 := permutations::list(l):
    if testtype(l, Type::ListOf(Type::Real)) then
	list2 := generators::toList
		(generators::fromNext
		 (permutations::last(l),
		  permutations::previous)):
	prog::test(list1, revert(list2)):
    end_if:
    prog::test(list1, sort(list1)):
    prog::test(nops(list1), permutations::count(l)):
end_for:

// With duplicates, the permutations are not necessarily generated
// in lexicographic order
for l in
    [0, 1, 4, [], [1,2,3,4], [1,1,2,2,2], [a,b,a,c,d] ] do
    prog::test(generators::count(permutations::generator(l, Duplicate)),
	       permutations::count(l, Duplicate)):
end_for:

prog::testfunc(permutations::rank):
for n from 0 to 5 do
    prog::test(map(permutations(n), permutations::rank), [$1..n!]):
end_for:

prog::testfunc(permutations::unrank):
prog::test(permutations::unrank(3,2), FAIL):
for n from 0 to 5 do
    prog::test([permutations::unrank(i, n) $ i=1..n!], permutations(n)):
end_for:


prog::testfunc(permutations::random):
prog::test(permutations::random([]),[]):
prog::test(permutations::random([1]),[1]):
prog::test(sort(permutations::random([1,2,3])),
           sort([1,2,3])):
prog::test(sort(permutations::random([a,b,a,c,d,d,e,f])),
           sort([a,b,a,c,d,d,e,f])):

prog::testfunc(permutations::inverse):
prog::test(permutations::inverse([]), []):
prog::test(permutations::inverse([1,2,3]), [1,2,3]):
prog::test(permutations::inverse([2,1,3]), [2,1,3]):
prog::test(permutations::inverse([1,7,5,4,2,3,6,9,8]),
	       [1, 5, 6, 4, 3, 7, 2, 9, 8]):

prog::testfunc(permutations::toMatrix):
if traperror(matrix(0,0)) = 1028 then 
// matrix does not allows for 0x0 matrices.
    prog::test(permutations::toMatrix([]), TrapError=1028):
else
    prog::test(permutations::toMatrix([]), matrix(0,0)):
end_if:
prog::test(permutations::toMatrix([1]), matrix([[1]])):
prog::test(permutations::toMatrix([2,1]), matrix([[0,1],[1,0]])):

prog::test(permutations::toMatrix(permutations::inverse([5,7,6,2,4,3,1])), 
	       linalg::transpose(permutations::toMatrix([5,7,6,2,4,3,1]))):


prog::testfunc(permutations::_mult):
prog::test(permutations::_mult([], []),
	   []):
prog::test(permutations::_mult([1], [1]),
	   [1]):
prog::test(permutations::_mult([1,2], [2,1]),
	   [2,1]):
prog::test(permutations::_mult([2,1], [2,1]),
	   [1,2]):
prog::test(permutations::_mult([], [2,1,5,3,4]),
	   [2,1,5,3,4]):
prog::test(permutations::_mult([2, 1], [2, 5, 1, 3, 4]),
           [1, 5, 2, 3, 4]):
prog::test(permutations::_mult([2, 5, 1, 3, 4], [2, 1]),
           [5, 2, 1, 3, 4]):
prog::test(permutations::_mult([2,3,1,4,5], [5,2,3,4,1]),
	   [5, 3, 1, 4, 2]):
prog::test(permutations::_mult([a,c,c,e,3], [5,2,3,4,1]),
	   [3,c,c,e,a]):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc(permutations::multInverse):
//////////////////////////////////////////////////////////////////////////////

for p in [ [] ] . permutations::list(3) . permutations::list([a,b,c,d]) do
    for q in [ [] ] . permutations::list(3) do
	qInverse := permutations::inverse(q);
	prog::test(permutations::multInverse(p,q),
		   permutations::_mult(p, qInverse));
    end_for;
end_for:

delete(q):

prog::testfunc(permutations::inversions):
prog::test(permutations::inversions([]),[]):
prog::test(permutations::inversions([2,3,1]),[{1, 3}, {2, 3}]):
prog::test(permutations::inversions([3,2,1]),[{1, 2}, {1, 3}, {2, 3}]):

prog::testfunc(permutations::inversionsNumber):
prog::test(permutations::inversionsNumber([]),0):
prog::test(permutations::inversionsNumber([2,3,1]),2):
prog::test(permutations::inversionsNumber([3,2,1]),3):

prog::testfunc(permutations::isEven):
prog::test(permutations::isEven([])):
prog::test(not permutations::isEven([2,1])):

prog::testfunc(permutations::sign):
prog::test(permutations::sign([]),1):
prog::test(permutations::sign([1]),1):
prog::test(permutations::sign([2,1]),-1):
prog::test(permutations::sign([2,1,3]),-1):
prog::test(permutations::sign([2,3,1]),1):
prog::test(permutations::sign([3,2,1]),-1):
prog::test(permutations::sign([5,4,3,2,1]),1):
prog::test(permutations::sign([6,5,18,3,15,9,7,20,17,1,10,11,16,8,14,19,12,4,13,2]),
           -1):

prog::testfunc(permutations::code):
for n from 0 to 5 do
    g := permutations::generator(n):
    while (p:=g())<>FAIL do
	code := permutations::code(p):
	prog::test(_and(bool(code[i]<=n-i) $i=1..n)):
	prog::test(p, permutations::fromCode(code)):
	// Remove the trailing zeroes from code
	for i from nops(code) downto 1 do
	    if code[i]>0 then break: end_if:
	end_for:
	code := [code[j] $j=1..i]:
	prog::test(p, permutations::fromCode(code,n)):
    end_while:
end_for:

for w in [[], [5], [1,1], op(permutations([2,2,1])), [3,2,1,3,4,5,2,3,1]] do 
    prog::test(words::fromStandardAndEvaluation
	       (permutations::fromCode(permutations::code(w)),
		words::evaluation(w)),
	       w):
end_for:

//////////////////////////////////////////////////////////////////////////////
prog::testfunc(permutations::coCode):

prog::test(combinat::permutations::coCode([]), []):
prog::test(combinat::permutations::coCode([1, 2, 3]), [0, 0, 0]):
prog::test(combinat::permutations::coCode([1, 3, 2]), [0, 0, 1]):
prog::test(combinat::permutations::coCode([2, 1, 3]), [0, 1, 0]):
prog::test(combinat::permutations::coCode([2, 3, 1]), [0, 0, 2]):
prog::test(combinat::permutations::coCode([3, 1, 2]), [0, 1, 1]):
prog::test(combinat::permutations::coCode([3, 2, 1]), [0, 1, 2]):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc(permutations::fromReducedWord):
prog::test(permutations::fromReducedWord([]),
	   []):
prog::test(permutations::fromReducedWord([], 0),
           []):
prog::test(permutations::fromReducedWord([], 4),
           [1, 2, 3, 4]):
prog::test(permutations::fromReducedWord([1, 2], 4),
           [2, 3, 1, 4]):
prog::test(permutations::fromReducedWord([1, 2]),
           [2, 3, 1]):
prog::test(permutations::fromReducedWord([1, 2, 1], 4),
           [3, 2, 1, 4]):
prog::test(permutations::fromReducedWord([2, 1, 2], 4),
           [3, 2, 1, 4]):
prog::test(permutations::fromReducedWord([1,3], [a,b,c,d]),
	   [b,a,d,c]):
if prog::TestNoTestargs = FALSE then
prog::test(permutations::fromReducedWord([1], 1),
	   TrapError = 1028):
prog::test(permutations::fromReducedWord([1,3], 2),
	   TrapError = 1028):
prog::test(permutations::fromReducedWord([1,4,5,3,4,2,2,2,3], 5),
	   TrapError = 1028):
prog::test(permutations::fromReducedWord([1,3], [a,b,c]),
	   TrapError = 1028):
end_if:

prog::testfunc(permutations::reducedWord):
for n from 0 to 5 do
    g := permutations::generator(n):
    while (p:=g()) <> FAIL do
	prog::test(permutations::fromReducedWord(permutations::reducedWord      (p), n), p):
	prog::test(permutations::fromReducedWord(permutations::reducedWordLexMin(p), n), p);
    end_while:
end_for:

prog::testfunc(permutations::reducedWords):
for p in permutations::list(2).permutations::list(3).permutations::list(4).[[1,2,3,4,5],[5,4,3,2,1],[3,4,2,5,1]] do
    n := nops(p);
    wMin := permutations::reducedWordLexMin(p);
    for w in permutations::reducedWords(p) do
	prog::test(permutations::fromReducedWord(w ,n), p):
	prog::test(not words::lexLess(w, wMin));
    end_for;
end_for:

//////////////////////////////////////////////////////////////////////////////
// Cycle decomposition of standard permutations
//////////////////////////////////////////////////////////////////////////////

prog::testfunc(permutations::cycles):
prog::test(permutations::cycles([]),
	   []):
prog::test(permutations::cycles([1,2,3]),
	   [[1],[2],[3]]):
prog::test(permutations::cycles([2,3,1]),
	   [[1,2,3]]):
prog::test(permutations::cycles([1,6,9,5,8,7,4,2,3]),
	   [[1], [2,6,7,4,5,8], [3,9]]):
for n from 0 to 5 do
    g := permutations::generator(n):
    while (p:=g()) <> FAIL do
	prog::test(permutations::fromCycles(permutations::cycles(p)), p):
    end_while:
end_for:

prog::testfunc(permutations::cycleType):
prog::test(permutations::cycleType([1,6,9,5,8,7,4,2,3]),
	   [6,2,1]):

prog::testfunc(permutations::fixedPoints):
prog::test(permutations::fixedPoints([]),[]):
prog::test(permutations::fixedPoints([3,2,1]),[2]):
prog::test(permutations::fixedPoints([1,2,3]),[1,2,3]):
prog::testfunc(permutations::fixedPointsNumber):
prog::test(permutations::fixedPointsNumber([]),0):
prog::test(permutations::fixedPointsNumber([3,2,1]),1):
prog::test(permutations::fixedPointsNumber([1,2,3]),3):

//////////////////////////////////////////////////////////////////////////////
// Descents
//////////////////////////////////////////////////////////////////////////////

prog::testfunc(permutations::descents):
prog::test(permutations::descents([]),[]):
prog::test(permutations::descents([2,3,1]),[2]):
prog::test(permutations::descents([3,2,1]),[1,2]):
prog::test(permutations::descents([], FinalDescent),[]):
// the empty permutation has no descents at all
prog::test(permutations::descents([2,3,1], FinalDescent),[2,3]):
prog::test(permutations::descents([3,2,1], FinalDescent),[1,2,3]):

prog::testfunc(permutations::descentsComposition):
prog::test(permutations::descentsComposition([]),[]):
prog::test(permutations::descentsComposition([2,3,1]),[2,1]):
prog::test(permutations::descentsComposition([3,2,1]),[1,1,1]):

prog::testfunc(permutations::descentsCompositionToMin):
prog::test(permutations::descentsCompositionToMin([]),
           []):
prog::test(permutations::descentsCompositionToMin([2, 1]),
           [1, 3, 2]):
prog::test(permutations::descentsCompositionToMin([4]),
           [1, 2, 3, 4]):
prog::test(permutations::descentsCompositionToMin([4, 2, 1, 3]),
	   [1, 2, 3, 5, 4, 8, 7, 6, 9, 10]):
// check that the permutation belongs to the descent class   
for c in compositions::list(5) do
    prog::test(permutations::descentsComposition
	       (permutations::descentsCompositionToMin(c), 
		 FinalDescent), c):
end_for:

prog::testfunc(permutations::descentsCompositionToMax):
prog::test(permutations::descentsCompositionToMax([]),
           []):
prog::test(permutations::descentsCompositionToMax([2, 1]),
           [2, 3, 1]):
prog::test(permutations::descentsCompositionToMax([4]),
           [1, 2, 3, 4]):
prog::test(permutations::descentsCompositionToMax([4, 2, 1, 3]),
           [7, 8, 9, 10, 5, 6, 4, 1, 2, 3]):
// check that the permutation belongs to the descent class   
for c in compositions::list(5) do
    prog::test(permutations::descentsComposition
	       (permutations::descentsCompositionToMax(c), 
		 FinalDescent), c):
end_for:

prog::testfunc(permutations::fromDescentComposition):
prog::test(permutations::fromDescentsComposition([]),
           [[]]):
prog::test(permutations::fromDescentsComposition([1]),
           [[1]]):
prog::test(sort(permutations::fromDescentsComposition([1, 2])),
           sort([[2, 1, 3], [3, 1, 2]])):

for c in compositions::list(5) do
    prog::test(sort(permutations::fromDescentsComposition(c)),
	       sort(select(permutations(5), z->permutations::descentsComposition(z) = c))):
end_for:
prog::testfunc(permutations::fromDescents):
prog::test(permutations::fromDescents([],0),
           [[]]):
prog::test(permutations::fromDescents([], 3),
           [[1,2,3]]):
prog::test(sort(permutations::fromDescents([2], 3)),
           sort([[1,3,2], [2,3,1]])):

prog::testfunc(permutations::majorIndex):
prog::test(permutations::majorIndex([]),0):
prog::test(permutations::majorIndex([2,3,1]),2):
prog::test(permutations::majorIndex([3,2,1]),3):

for n from 1 to 5 do
    prog::test(_plus(hold(q)^(permutations::majorIndex(p))
		     $ p in permutations(n)),
	       expand(_mult(_plus(q^i $i=0..d-1) $d=1..n)));
end_for:

prog::testfunc(permutations::majorCode):
prog::test(permutations::majorCode([]),
           []):
prog::test(permutations::majorCode([1]),
           [0]):
prog::test(permutations::majorCode([2, 1]),
	   [1, 0]):
prog::test(permutations::majorCode([1, 2]),
           [0, 0]):
prog::test(permutations::majorCode([3, 5, 2, 7, 4, 6, 1]),
           [6, 3, 1, 0, 1, 1, 0]):

prog::testfunc(permutations::fromMajorCode):
prog::test(permutations::fromMajorCode([]),
           []):
prog::test(permutations::fromMajorCode([0]),
           [1]):
prog::test(permutations::fromMajorCode([1, 0]),
	   [2, 1]):
prog::test(permutations::fromMajorCode([0, 0]),
           [1, 2]):
prog::test(permutations::fromMajorCode([6, 3, 1, 0, 1, 1, 0]),
           [3, 5, 2, 7, 4, 6, 1]):
for n from 1 to 5 do
    for p in permutations::list(n) do
	prog::test(permutations::fromMajorCode(permutations::majorCode(p)), p):
    end_for;
end_for:

prog::testfunc(permutations::descentPolynomial):


prog::testfunc(permutations::peaks):
prog::test(permutations::peaks([]),[]):
prog::test(permutations::peaks([2,3,1]),[2]):
prog::test(permutations::peaks([3,2,1]),[]):
prog::test(permutations::peaks([3,4,2,1]),[2]):
prog::test(permutations::peaks([3,2,4,1]),[3]):
prog::test(permutations::peaks([3,2,4,5,1]),[4]):

prog::testfunc(permutations::saillances):
prog::test(permutations::saillances([]), []):
prog::test(permutations::saillances([4,3,2,1]), [1,2,3,4]):
prog::test(permutations::saillances([1,2,3,4]), [4]):
prog::test(permutations::saillances([4,1,2,3]), [1,4]):

prog::testfunc(permutations::saillancesNumber):
prog::test(permutations::saillancesNumber([]), 0):
prog::test(permutations::saillancesNumber([4,3,2,1]), 4):
prog::test(permutations::saillancesNumber([1,2,3,4]), 1):
prog::test(permutations::saillancesNumber([4,1,2,3]), 2):

//////////////////////////////////////////////////////////////////////////
// Patterns
//////////////////////////////////////////////////////////////////////////
 
prog::testfunc(permutations::patternPositions):
prog::test(permutations::patternPositions([1, 2], []),
           [[]]):
prog::test(permutations::patternPositions([1, 2], [1]),
           [[1], [2]]):
prog::test(permutations::patternPositions([1, 2], [1, 2]),
           [[1, 2]]):
prog::test(permutations::patternPositions([1, 2], [2, 1]),
           []):
prog::test(permutations::patternPositions([1, 2, 3], [2, 1]),
           []):
prog::test(permutations::patternPositions([1, 3, 2], [2, 1]),
           [[2, 3]]):
prog::test(permutations::patternPositions([1, 3, 3], [2, 1]),
           []):
prog::test(permutations::patternPositions([1, 3, 1], [2, 1]),
           [[2, 3]]):
prog::test(permutations::patternPositions([2, 4, 3, 6, 5, 1], [2, 3, 1]),
	   [[1, 2, 6], [1, 3, 6], [1, 4, 6], [1, 5, 6], 
	    [2, 4, 6], [2, 5, 6], [3, 4, 6], [3, 5, 6]]):
prog::testfunc(permutations::hasPattern):
prog::test(permutations::hasPattern([1, 2], [1]), TRUE):
prog::test(permutations::hasPattern([1, 2], [1, 2]), TRUE):
prog::test(permutations::hasPattern([1, 1], [1, 2]), TRUE):
prog::test(permutations::hasPattern([1, 1], [2, 1]), FALSE):
prog::test(permutations::hasPattern([2, 1], [2, 1]), TRUE):
prog::test(permutations::hasPattern([2, 4, 3, 6, 5, 1], [2, 3, 1]), TRUE):
// The permutation of n that avoids any 3-pattern are counted by the
// catalan numbers
n := 5:
l := permutations::list(n):
for p in permutations::list(3) do
    prog::test(nops(select(l, _not @ permutations::hasPattern, p)),
	       combinat::catalan(n)):
end_for:

prog::testfunc(permutations::has1324):
prog::test(permutations::has1324([1, 3, 2, 5]),
           TRUE):
prog::test(TRUE,
           TRUE):
prog::test(permutations::has1324([1, 3, 2, 5]),
           TRUE):
prog::test(permutations::has1324([1, 3, 2, 4]),
           TRUE):
prog::test(permutations::has1324([1, 4, 2, 3]),
           FALSE):
prog::test(permutations::has1324([1, 5, 4, 6]),
           TRUE):
prog::test(permutations::has1324([1, 5, 2, 7, 3, 4, 6]),
           TRUE):
prog::test(permutations::has1324([]),
           FALSE):
prog::test(permutations::has1324([1]),
           FALSE):
prog::test(permutations::has1324([1, 2]),
           FALSE):
prog::test(permutations::has1324([1, 4, 3, 2]),
           FALSE):


//////////////////////////////////////////////////////////////////////////////
// Connex permutations
//////////////////////////////////////////////////////////////////////////////
prog::testfunc(permutations::connectedShape):
prog::test(permutations::connectedShape([]),
           []):
prog::test(permutations::connectedShape([1, 2]),
           [1,1]):
prog::test(permutations::connectedShape([2, 1]),
           [2]):
prog::test(permutations::connectedShape([1, 3, 4, 2, 5]),
           [1, 3, 1]):
prog::test(permutations::connectedShape([2, 5, 4, 1, 3]),
           [5]):

prog::testfunc(permutations::isConnected):
prog::test(permutations::isConnected([]),
           FALSE):
prog::test(permutations::isConnected([1, 2]),
           FALSE):
prog::test(permutations::isConnected([2, 1]),
           TRUE):
prog::test(permutations::isConnected([1]),
           TRUE):
prog::test(permutations::isConnected([1, 3, 4, 2, 5]),
           FALSE):
prog::test(permutations::isConnected([2, 5, 4, 1, 3]),
           TRUE):

prog::testfunc(permutations::connected::isA):
prog::test(permutations::connected::isA([1, 2, 3]),
           FALSE):
prog::test(permutations::connected::isA([3, 2, 1]),
           TRUE):
prog::test(permutations::connected::isA([3, 2, 1], 4),
           FALSE):
prog::test(permutations::connected::isA([3, 2, 1], 3),
           TRUE):
prog::test(permutations::connected::isA([3, 1, 4, 2], 4),
           TRUE):
prog::test(permutations::connected::isA([3, 5, 1, 2, 4], 5),
           TRUE):
prog::test(permutations::connected::isA([3, 5, 1, 2, 4], 4),
           FALSE):

prog::testfunc(permutations::connected::list):
prog::test(permutations::connected::list(3),
           [[2, 3, 1], [3, 1, 2], [3, 2, 1]]):
prog::test(permutations::connected::list(0),
           []):
prog::test(permutations::connected::list(4),
           [[2, 3, 4, 1], [2, 4, 1, 3], [2, 4, 3, 1], [3, 1, 4, 2], [3, 2, 4,
1], [3, 4, 1, 2], [3, 4, 2, 1], [4, 1, 2, 3], [4, 1, 3, 2], [4, 2, 1, 3], [4,
2, 3, 1], [4, 3, 1, 2], [4, 3, 2, 1]]):
prog::test(bool(permutations::connected::list(5)=
     select(permutations::list(5),permutations::isConnected))):

prog::testfunc(permutations::connected::count):
for i from 0 to 5 do
    prog::test(bool(nops(permutations::connected::list(i))=
                    permutations::connected::count(i)));
end_for:

//////////////////////////////////////////////////////////////////////////////
// Bruhat order
//////////////////////////////////////////////////////////////////////////////
prog::testfunc(permutations::leequalBruhat):
prog::test(permutations::leequalBruhat([], []),
           TRUE):
prog::test(permutations::leequalBruhat([1], [1]),
           TRUE):
prog::test([(permutations::leequalBruhat(a, b) $ a in permutations(2)) 
	                                       $ b in permutations(2)],
           [TRUE, FALSE, TRUE, TRUE]):
prog::test([(permutations::leequalBruhat(a, b) $ a in permutations(3)) 
	                                       $ b in permutations(3)],
//         [[123], [132], [213], [231], [312], [321]]
	   [TRUE,  FALSE, FALSE, FALSE, FALSE, FALSE, 
	    TRUE,  TRUE,  FALSE, FALSE, FALSE, FALSE, 
	    TRUE,  FALSE, TRUE,  FALSE, FALSE, FALSE, 
	    TRUE,  TRUE,  TRUE,  TRUE,  FALSE, FALSE, 
	    TRUE,  TRUE,  TRUE,  FALSE, TRUE,  FALSE, 
	    TRUE,  TRUE,  TRUE,  TRUE,  TRUE,  TRUE ]):

prog::testfunc(permutations::bruhatInversions):
prog::test(permutations::bruhatInversions([1]),
           []):
prog::test(permutations::bruhatInversions([1, 2]),
           []):
prog::test(permutations::bruhatInversions([2, 1]),
           [{1, 2}]):
prog::test(permutations::bruhatInversions([2, 1, 3]),
           [{1, 2}]):
prog::test(permutations::bruhatInversions([3, 1, 2]),
           [{1, 2}, {1, 3}]):
prog::test(permutations::bruhatInversions([4, 3, 1, 2]),
           [{1, 2}, {2, 3}, {2, 4}]):

prog::testfunc(permutations::succBruhat):
prog::test(permutations::succBruhat([1, 2]),
           [[2, 1]]):
prog::test(permutations::succBruhat([4, 3, 1, 2]),
           [[4, 3, 2, 1]]):
prog::test(sort(permutations::succBruhat([1, 3, 2])),
           sort([[3, 1, 2], [2, 3, 1]])):

prog::testfunc(permutations::predBruhat):
prog::test(permutations::predBruhat([1, 2]),
           []):
prog::test(permutations::predBruhat([1, 3, 2]),
           [[1, 2, 3]]):
prog::test(sort(permutations::predBruhat([4, 3, 1, 2])),
	   sort([[3, 4, 1, 2], [4, 1, 3, 2], [4, 2, 1, 3]])):
// Coherence checking smallerBruhat <=> leequalBruhat
for perm in [op(permutations::list(i)) $ i = 0..5] do
    prog::test(bool({op(permutations::smallerBruhat(perm))}=
		    {op(select(permutations::list(nops(perm)), 
			       permutations::leequalBruhat, perm))
		    }), TRUE):
end_for:

//////////////////////////////////////////////////////////////////////////////
// Permutohedron
//////////////////////////////////////////////////////////////////////////////
prog::testfunc(permutations::leequalPermutohedron):
prog::test(permutations::leequalPermutohedron([], []),
           TRUE):
prog::test(permutations::leequalPermutohedron([1], [1]),
           TRUE):
prog::test([(permutations::leequalPermutohedron(a, b) $ a in permutations(2)) 
	                                       $ b in permutations(2)],
           [TRUE, FALSE, TRUE, TRUE]):
prog::test([(permutations::leequalPermutohedron(a, b) $ a in permutations(3)) 
	                                             $ b in permutations(3)],
//         [[123], [132], [213], [231], [312], [321]]
	   [TRUE,  FALSE, FALSE, FALSE, FALSE, FALSE, 
	    TRUE,  TRUE,  FALSE, FALSE, FALSE, FALSE, 
	    TRUE,  FALSE, TRUE,  FALSE, FALSE, FALSE, 
	    TRUE,  FALSE, TRUE,  TRUE,  FALSE, FALSE, 
	    TRUE,  TRUE,  FALSE, FALSE, TRUE,  FALSE, 
	    TRUE,  TRUE,  TRUE,  TRUE,  TRUE,  TRUE ]):
prog::test([(permutations::leequalPermutohedron(a, b, Left) 
	     $ a in permutations(3)) 
	    $ b in permutations(3)],
//         [[123], [132], [213], [231], [312], [321]]
	   [TRUE,  FALSE, FALSE, FALSE, FALSE, FALSE, 
	    TRUE,  TRUE,  FALSE, FALSE, FALSE, FALSE, 
	    TRUE,  FALSE, TRUE,  FALSE, FALSE, FALSE, 
	    TRUE,  TRUE,  FALSE, TRUE,  FALSE, FALSE, 
	    TRUE,  FALSE, TRUE,  FALSE, TRUE,  FALSE, 
	    TRUE,  TRUE,  TRUE,  TRUE,  TRUE,  TRUE ]):

// We do not test succPermutohedron because it is the base of greaterPermutohedron
prog::testfunc(permutations::greaterPermutohedron):
prog::test(sort(permutations::greaterPermutohedron([3, 1, 2], Left)),
           sort([[3, 1, 2], [3, 2, 1]])):
prog::test(sort(permutations::greaterPermutohedron([1, 3, 2], Right)),
           sort([[1, 3, 2], [3, 1, 2], [3, 2, 1]])):
prog::test(sort(permutations::greaterPermutohedron([1, 3, 2], Left)),
           sort([[1, 3, 2], [2, 3, 1], [3, 2, 1]])):

// We do not test predPermutohedron because it is the base of greaterPermutohedron
prog::testfunc(permutations::smallerPermutohedron):
prog::test(sort(permutations::smallerPermutohedron([1, 3, 2], Right)),
           sort([[1, 3, 2], [1, 2, 3]])):
prog::test(sort(permutations::smallerPermutohedron([3, 1, 2], Right)),
           sort([[3, 1, 2], [1, 3, 2], [1, 2, 3]])):
prog::test(sort(permutations::smallerPermutohedron([3, 1, 2], Left)),
	   sort([[3, 1, 2], [2, 1, 3], [1, 2, 3]])):

// Coherence checking: greater, smaller vs lequal
for p in _concat(permutations(i) $ i=0..4) . [[2,4,3,5,1], [3,4,1,2,5]] do
    prog::test(bool(sort(permutations::smallerPermutohedron(p, Right)) = 
		    sort(select(permutations(nops(p)), 
			   permutations::leequalPermutohedron, p, Right))));
    prog::test(bool(sort(permutations::smallerPermutohedron(p, Left)) = 
		    sort(select(permutations(nops(p)), 
			   permutations::leequalPermutohedron, p, Left ))));
    prog::test(bool(sort(permutations::greaterPermutohedron(p, Right)) = 
		    sort(select(permutations(nops(p)), 
			   z -> permutations::leequalPermutohedron(p, z, Right))
			)));
    prog::test(bool(sort(permutations::greaterPermutohedron(p, Left)) = 
		    sort(select(permutations(nops(p)), 
			   z -> permutations::leequalPermutohedron(p, z, Left))
			)));
end_for:
// Coherence checking: descent classes are interval for the left permutoedron
for compo in [op(compositions::list(i)) $ i = 0..5] do
    mi := permutations::descentsCompositionToMin(compo):
    ma := permutations::descentsCompositionToMax(compo):
    prog::test({op(permutations::greaterPermutohedron(mi, Left))} intersect
	       {op(permutations::smallerPermutohedron(ma, Left))},
	       {op(permutations::fromDescentsComposition(compo))}):
end_for:

//////////////////////////////////////////////////////////////////////////////
prog::testfunc(permutations::fromSetPartition):
//////////////////////////////////////////////////////////////////////////////
prog::test(testtype(a, permutations::fromSetPartition::typeStandardSetPartition),
           FALSE):
prog::test(testtype([[1, 2], [3, 4, 5]], permutations::fromSetPartition::typeStandardSetPartition),
           TRUE):

prog::test(testtype([{1, 2}, [3, 4, 5]], permutations::fromSetPartition::typeStandardSetPartition),
           TRUE):
prog::test(testtype([[1, a], [3, 4, 5]], permutations::fromSetPartition::typeStandardSetPartition),
           FALSE):
prog::test(testtype({[1, a], [3, 4, 5]}, permutations::fromSetPartition::typeStandardSetPartition),
           FALSE):
prog::test(testtype({[1, 2], [4, 5]}, permutations::fromSetPartition::typeStandardSetPartition),
           FALSE):

for setPartition in [ [], [[1]], [[1,2,3]], [[1,2],[3,4,5]],
		      [[1],[2],[3],[4]] ] do
    prog::test(generators::count
	       (permutations::fromSetPartition::generator(setPartition)),
	       permutations::fromSetPartition::count(setPartition));
end_for:

//////////////////////////////////////////////////////////////////////////////
prog::testfunc(youngSubGroup):
//////////////////////////////////////////////////////////////////////////////

prog::test(combinat::permutations::youngSubgroup::count([]),
           1):
prog::test(combinat::permutations::youngSubgroup::count([0]),
           1):
prog::test(combinat::permutations::youngSubgroup::count([1, 0, 1]),
           1):
prog::test(combinat::permutations::youngSubgroup::count([2, 3, 0, 1]),
           12):

prog::test(combinat::permutations::youngSubgroup::list([]),
           [[]]):
prog::test(combinat::permutations::youngSubgroup::list([0]),
           [[]]):
prog::test(combinat::permutations::youngSubgroup::list([0, 0]),
           [[]]):
prog::test(combinat::permutations::youngSubgroup::list([1]),
           [[1]]):
prog::test(combinat::permutations::youngSubgroup::list([0, 1, 0]),
           [[1]]):
prog::test(combinat::permutations::youngSubgroup::list([3]),
           [[1, 2, 3], [1, 3, 2], [2, 1, 3], [2, 3, 1], [3, 1, 2], [3, 2, 1]]):
prog::test(combinat::permutations::youngSubgroup::list([2, 2]),
           [[1, 2, 3, 4], [1, 2, 4, 3], [2, 1, 3, 4], [2, 1, 4, 3]]):
prog::test(combinat::permutations::youngSubgroup::list([2, 0, 2]),
           [[1, 2, 3, 4], [1, 2, 4, 3], [2, 1, 3, 4], [2, 1, 4, 3]]):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc(modYoungSubgroup):
//////////////////////////////////////////////////////////////////////////////
prog::test(permutations::modYoungSubgroup([4, 3, 2, 1], [1,1,1,1]),
           [4, 3, 2, 1]):
prog::test(permutations::modYoungSubgroup([4, 3, 2, 1], [2,1,1]),
           [3, 4, 2, 1]):
prog::test(permutations::modYoungSubgroup([4, 3, 2, 1], [3,1]),
           [2, 3, 4, 1]):
prog::test(permutations::modYoungSubgroup([4, 3, 2, 1], [4]),
           [1, 2, 3, 4]):
prog::test(permutations::modYoungSubgroup([4, 3, 2, 1], [1,3]),
           [4, 1, 2, 3]):
prog::test(permutations::modYoungSubgroup([4, 3, 2, 1], [1,2,1]),
           [4, 2, 3, 1]):
prog::test(permutations::modYoungSubgroup([], []),
           []):
prog::test(permutations::modYoungSubgroup([c, a, b], [1,2]),
           [c, a, b]):
prog::test(permutations::modYoungSubgroup([c, b, a], [1,2]),
           [c, a, b]):
prog::test(permutations::modYoungSubgroup([c, b, a, d], [1,3]),
           [c, a, b, d]):
prog::test(permutations::modYoungSubgroup([c, b, d, a], [1,3]),
           [c, a, b, d]):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc(rightCosetsBlah):
//////////////////////////////////////////////////////////////////////////////

prog::test(permutations::rightCosetsBlah([]),
           [[]]):
prog::test(permutations::rightCosetsBlah([1]),
           [[1]]):
prog::test(permutations::rightCosetsBlah([1, 5]),
           [[1, 2]]):
prog::test(permutations::rightCosetsBlah([5, 1]),
           [[1, 2], [2, 1]]):
prog::test(permutations::rightCosetsBlah([1, 5, 3, 4, 2, 2, 1]),
           // note: 5 is always before 6 here:
           [[1, 2, 3, 4, 5, 6, 7], [1, 2, 3, 4, 5, 7, 6],
	    [1, 2, 3, 4, 7, 5, 6], [1, 2, 3, 5, 4, 6, 7],
	    [1, 2, 3, 5, 4, 7, 6], [1, 2, 3, 5, 6, 4, 7],
	    [1, 2, 3, 5, 6, 7, 4], [1, 2, 3, 5, 7, 4, 6],
	    [1, 2, 3, 5, 7, 6, 4], [1, 2, 3, 7, 4, 5, 6],
	    [1, 2, 3, 7, 5, 4, 6], [1, 2, 3, 7, 5, 6, 4]]):

for p in _concat(combinat::partitions::list(i) $ i=1..5) do
    l := permutations::rightCosetsBlah(p);
    prog::test(permutations::rightCosetsBlah::count(p), nops(l));
    prog::test([permutations::_mult(p,q) $ q in l],
	       revert(permutations(p)));
end_for:



//////////////////////////////////////////////////////////////////////////////
// Global checking of the package

// This is done at the end to ensure as much as possible of the
// package is initialized. A recursive prog::init(permutations)
// would be usefull here.

prog::testfunc(permutations):
prog::testexit():

prog::check(permutations, 3):


// Local Variables: 
// mode: mupad
// End:
