/*****************************************************************************
  File:		lib/COMBINAT/TEST/rankers.mu
  Authors:	Florent Hivert <florent.hivert@univ-mlv.fr>
  Purpose:	Test file for the library 'combinat::rankers'
  Created:	03/02/2003
  License:	LGPL
*****************************************************************************/

prog::testinit("rankers"):

alias(R=combinat::rankers):

checkRanker :=
proc(rank, unrank)
    local i, obj;
begin
    i := 1;
    while ((obj := unrank(i)) <> FAIL) do
	prog::test(rank(obj), i);
	i := i+1;
    end_while;
end_proc:

//////////////////////////////////////////////////////////////////////////////
prog::testfunc(combinat::rankers::onFly):
[rank, unrank] :=
    combinat::rankers::onFly(DOM_IDENT):
prog::test(rank(a), 1):
prog::test(rank(b), 2):
prog::test(rank(c), 3):
prog::test(rank(a), 1):
prog::test(rank(d), 4):

// Test for the reference effect
rank2   := rank:
unrank2 := unrank:
prog::test(rank2(e), 5):
prog::test(rank(e),  5):

// Test for the reference effect for function parameters
proc(rank,unrank)
begin
    prog::test(rank(f),6):
    checkRanker(rank, unrank):
end_proc(rank2, unrank2):
prog::test(rank(f),6):

// Coherence checks
checkRanker(rank, unrank):
checkRanker(rank2, unrank2):
checkRanker(rank2, unrank):
checkRanker(rank, unrank2):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc(combinat::rankers::fromType):

[rank, unrank] :=
    combinat::rankers::fromType(Type::ListOf(Type::PosInt)):
prog::test(rank([]), 1):
prog::test(rank([1,2]), 2):
prog::test(rank([2,3]), 3):
prog::test(rank([1,2]), 2):
if prog::TestNoTestargs = FALSE then
prog::test(rank([-1,2]), TrapError=1202):
end_if:
prog::test(unrank(3), [2,3]):
prog::test(unrank(1), []):
prog::test(unrank(4), FAIL):
prog::test(([rank2, unrank2] :=
	    combinat::rankers::fromType(Type::ListOf(Type::PosInt))),
	   [rank, unrank]):

prog::test(unrank2(1), []):
prog::test(unrank2(2), [1,2]):
prog::test(unrank2(3), [2,3]):
prog::test(rank2([1,2,3]), 4):
prog::test(rank([1,2,3]), 4):
prog::test(rank([1,2,3,4]), 5):
prog::test(rank2([1,2,3,5]), 6):
prog::test(rank([1,2,3,4]), 5):
prog::test(rank([1,2,3,5]), 6):
prog::test(rank2([1,2,3,4]), 5):
prog::test(rank2([1,2,3,5]), 6):
checkRanker(rank, unrank):
checkRanker(rank2, unrank2):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc(combinat::rankers::rankFromList):

rank := combinat::rankers::rankFromList([a,b]):
prog::test(rank(a),1):
prog::test(rank(b),2):
prog::test(rank(c),TrapError=1028):
rank := combinat::rankers::rankFromList([a,b], FAIL):
prog::test(rank(a),1):
prog::test(rank(b),2):
prog::test(rank(c),FAIL):

//////////////////////////////////////////////////////////////////////////////
// Global checking of the library

prog::testfunc(combinat::rankers):
prog::testexit():

prog::check(combinat::rankers, 3):


// Local Variables:
// mode: mupad
// End:
