/*****************************************************************************
  $Id: subsets.tst,v 1.1.1.3 2005/01/26 13:42:25 nthiery Exp $
  File:		lib/COMBINAT/TEST/subsets.tst
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Purpose:	Test file for the library 'combinat::subsets'
  Created:	24/02/2001
  License:	LGPL
*****************************************************************************/

prog::testinit("subsets"):

prog::testfunc(combinat::subsets::isA):
prog::test( combinat::subsets::isA({})):
prog::test( combinat::subsets::isA({1})):
prog::test( combinat::subsets::isA({1,2})):
prog::test( combinat::subsets::isA([1,2]), FALSE):
prog::test( combinat::subsets::isA({}, {1,2,3})):
prog::test( combinat::subsets::isA({}, 3)):
prog::test( combinat::subsets::isA({}, 0)):
prog::test( combinat::subsets::isA({}, 1)):
prog::test( combinat::subsets::isA({1,2}, {1,2,3})):
prog::test( combinat::subsets::isA({1,2}, 3)):
prog::test( combinat::subsets::isA({1,2}, {}), FALSE):
prog::test( combinat::subsets::isA({1,2}, {1,2})):
prog::test( combinat::subsets::isA({1,2}, 2)):
prog::test( combinat::subsets::isA({1,2}, 0), FALSE):
prog::test( combinat::subsets::isA({1,2}, {1}), FALSE):
prog::test( combinat::subsets::isA({1,2}, 1), FALSE):
prog::test( combinat::subsets::isA({1,2}, {1,2}, 2)):
prog::test( combinat::subsets::isA({1,2}, 2, 2)):
prog::test( combinat::subsets::isA({1,2}, {1,2}, 1), FALSE):
prog::test( combinat::subsets::isA({1,2}, 2, 1), FALSE):
prog::test( combinat::subsets::isA({1,2}, {1,2,3}, 2)):
prog::test( combinat::subsets::isA({1,2}, 3, 2)):
prog::test( combinat::subsets::isA({}, {1,2,3}, 0)):
prog::test( combinat::subsets::isA({}, 3, 0)):

prog::testfunc(combinat::subsets::count):

prog::test( combinat::subsets::count(0,0), 1 ):
prog::test( combinat::subsets::count(1,0), 1 ):
prog::test( combinat::subsets::count(0,1), 0 ):
prog::test( combinat::subsets::count(5,2), 10):
prog::test( combinat::subsets::count({1,2,3,4},2),6):

prog::testfunc(combinat::subsets::list):

// The order in which the subsets are listed is not in the
// specifications of combinat::subsets::list. Moreover, there is no
// fixed internal order for sets, so we cannot even sort the result
// before comparing.

prog::test(     combinat::subsets::first(0,0),		{}		):
prog::test(     combinat::subsets::first(0,1),		FAIL		):
prog::test(     combinat::subsets::first(2,2),		{1,2}		):
prog::test(     combinat::subsets::first(7,3),		{op({$1..7}, 1..3)}		):
prog::test(     combinat::subsets::last(0,0),		{}		):
prog::test(     combinat::subsets::last(0,1),		FAIL		):
prog::test(     combinat::subsets::last(2,2),		{1,2}		):
prog::test(     combinat::subsets::last(7,3),		{op({$1..7}, 5..7)}):

prog::test(     combinat::subsets::list(0,0),       [{}]		):
prog::test(     combinat::subsets::list(0,1),       []		):
prog::test( {op(combinat::subsets::list(2,1))},     {{2},{1}}	):
for k from 0 to 11 do
    prog::test( nops(combinat::subsets::list(10,k)),  
	        combinat::subsets::count(10,k)):
end_for:

prog::test({op(combinat::subsets::list({}))},	{{}}):
prog::test({op(combinat::subsets::list(3))}, 
           {{}, {1}, {2}, {3}, {1, 2}, {1, 3}, {2, 3}, {1, 2, 3}}	):
prog::test({op(combinat::subsets::list({$1..3}))},
           {{}, {1}, {2}, {3}, {1, 2}, {1, 3}, {2, 3}, {1, 2, 3}}	):

for n from 0 to 6 do
    prog::test( nops(combinat::subsets::list(n)),		2^n):
    prog::test( combinat::subsets::count(n),			2^n):
    prog::test( nops(combinat::subsets::list({$1..n})),	2^n):
    prog::test( combinat::subsets::count({$1..n}),		2^n):
end_for:

prog::testfunc(combinat::subsets::unrank):
for n from 0 to 6 do
    prog::test(bool([combinat::subsets::unrank(i, n) $ i=1..2^n+2] 
		    = combinat::subsets::list(n).[FAIL, FAIL])):
end_for:

prog::test(bool([combinat::subsets::unrank(i, {}) $ i=1..2^0+2] 
		= combinat::subsets::list({}).[FAIL, FAIL])):
prog::test(bool([combinat::subsets::unrank(i, {a,b}) $ i=1..2^2+2] 
		= combinat::subsets::list({a,b}).[FAIL, FAIL])):
prog::test(bool([combinat::subsets::unrank(i, {a,b,c,d,e}) $ i=1..2^5+2] 
		= combinat::subsets::list({a,b,c,d,e}).[FAIL, FAIL])):
prog::test(bool([combinat::subsets::unrank(i, {a,b,c,d,e}, 3) $ i=1..binomial(5,3)+2] 
		= combinat::subsets::list({a,b,c,d,e}, 3).[FAIL, FAIL])):


prog::testfunc(combinat::subsets::random):
w := {1,3,5,6,8,10,12,13}: 
for n from 0 to 100 do
    prog::test(combinat::subsets::isA(combinat::subsets::random(w),
				       w));
    prog::test(combinat::subsets::isA(combinat::subsets::random(w, 5),
				       w, 5));
end_for:

//////////////////////////////////////////////////////////////////////////////
// Global checking of the package

// This is done at the end to ensure as much as possible of the
// package is initialized. A recursive prog::init(combinat::subsets)
// would be usefull here.

prog::testfunc(combinat::subsets):
prog::testexit():

prog::check(combinat::subsets, 3):


// Local Variables: 
// mode: mupad
// End:
