/*****************************************************************************
  $Id: trees.tst,v 1.1.1.3 2004/10/04 17:25:16 nthiery Exp $
  File:		lib/COMBINAT/TEST/binaryTrees.tst
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Purpose:	Test file for the library 'combinat::binaryTrees'
  Created:	27/06/2002
  License:	LGPL
*****************************************************************************/

prog::testinit("trees"):

alias(T=combinat::trees):

alias(d = T::dummyLabel):

prog::test(testtype(T::zero, T), TRUE):
prog::test(testtype(T::one, T), TRUE):

prog::testfunc(T::size):
prog::test(T::size(T::zero), 0):
prog::test(T::size(T::one), 1):
prog::test(T::size(T([d])), 1):
prog::test(T::size(T([d,[d]])), 2):
prog::test(T::size(T([d,[],[d]])), 2):
prog::test(T::size(T([d,[d],[d]])), 3):

prog::testfunc(T::op):
prog::test(op(T([d]), 0),
           d):
prog::test(op(T([d]), 1),
           FAIL):
prog::test(op(T([d])),
           null()):
prog::test(op(T([])),
	   null()):
prog::test(op(T([d, [d]])),
           T([d])):
prog::test(combinat::trees([d, [], [d]])[0],
	   d):
prog::test(combinat::trees([d, [], [d]])[1],
           T([])):
prog::test(combinat::trees([d, [], [d]])[2],
	   T([d])):

prog::testfunc(T::subsop):
prog::test(subsop(T([d, [], [d]]), 1 = T([d])),
	   T([d, [d], [d]])):
prog::test(subsop(T([d, [], [d]]), 0 = 2),
	   T([2, [], [d]])):
prog::test(subsop(T([d, [], [d]]), 3 = T([d])),
	   T([d, [], [d], [d]])):
prog::testfunc(T::set_index):
prog::test((t := T([d, [], [d]]): t[1] := T([d]): t),
	   T([d, [d], [d]])):
prog::test((t := T([d, [], [d]]): t[0] := 0: t),
	   T([0, [], [d]])):

prog::testfunc(T::shape):
for tree in [T([]), T([d]), T([d,[d]]), T([d,[],[d]]), 
	  T([d,[d,[],[d,[d],[d,[d,[d]]]],[d]],[d]])] do
    prog::test(T::shape(tree), tree);
end_for:

prog::testfunc(T::toWord):
prog::test(T::toWord(T([])),
           []):
prog::test(T::toWord(T([]), Prefix),
           []):
prog::test(T::toWord(T([]), Infix),
           []):
prog::test(T::toWord(T([]), Postfix),
           []):
prog::test(T::toWord(T([]), LeftPostfix),
           []):
prog::test(T::toWord(T([]), RightPostfix),
           []):
prog::test(T::toWord(T([1]), Prefix),
           [1]):
prog::test(T::toWord(T([1]), Postfix),
           [1]):
prog::test(T::toWord(T([1]), RightPostfix),
           [1]):
prog::test(T::toWord(T([1]), RightPrefix),
           [1]):
prog::test(T::toWord(T([1]), RightInfix),
           [1]):
prog::test(T::toWord(T([1, [2], [3]]), Prefix),
           [1, 2, 3]):
prog::test(T::toWord(T([1, [2], [3]]), Infix),
           [2, 1, 3]):
prog::test(T::toWord(T([1, [2], [3]]), Postfix),
           [2, 3, 1]):
prog::test(T::toWord(T([1, [2], [3]]), RightInfix),
           [3, 1, 2]):
prog::test(T::toWord(T([1, [2], [3]]), RightPrefix),
           [1, 3, 2]):
prog::test(T::toWord(T([1, [2], [3]]), RightPostfix),
	   [3, 2, 1]):
prog::test(T::toWord(T([1, [2], [3, [], [4]]])),
           [1,2,3,4]):
prog::test(T::toWord(T([1, [2], [3, [], [4]]]), Infix),
           [2, 1, 3, 4]):
prog::test(T::toWord(T([1, [2], [3], [4]]), RightPostfix),
           [4, 3, 2, 1]):
prog::test(T::toWord(T([1, [2], [3], [4]]), RightPrefix),
           [1, 4, 3, 2]):
prog::test(T::toWord(T([1, [2], [3], [4]]), RightInfix),
           [4, 3, 1, 2]):
prog::test(T::toWord(T([1, [2, [3, [4]]], [5], [6]])),
           [1, 2, 3, 4, 5, 6]):

for tree in [T([]), T([d]), T([d,[d]]), T([d,[],[d]]), 
	  T([d,[d,[],[d,[d],[d,[d,[d]]]],[d]],[d]])] do
    word := [$1..tree::dom::size(tree)];
    for o in {Prefix, Infix, Postfix, RightPrefix, RightInfix, RightPostfix} do
	prog::test(T::toWord(T::fromShapeAndWord(tree,word,o),o),word);
    end_for;
end_for:

prog::testfunc(T::count):
prog::test([T::count(i) $ i = 0..10],
           [0, 1, 1, 2, 5, 14, 42, 132, 429, 1430, 4862]):

prog::testfunc(T::list):
prog::test(T::list(0),
	   map([],T)):
prog::test(T::list(1),
	   map([[d]],T)):
prog::test(T::list(2),
           map([[d, [d]]],T)):
prog::test(T::list(3),
	   map([[d, [d], [d]], [d, [d, [d]]]],T)):
prog::test(T::list(4),
	   map([[d, [d], [d], [d]], [d, [d], [d, [d]]], [d, [d, [d]], [d]], [d, [d, [d], [d]]], [d, [d, [d, [d]]]]],T)):

prog::testfunc(T::random):
SEED:=1:
prog::test(expr(combinat::trees::random(12)),
           [d, [d, [d], [d], [d, [d], [d, [d], [d]]], [d, [d], [d]]]]):

prog::testfunc(T::unrank):
// Test bug #628090
prog::test(T::unrank(10,1), FAIL):
prog::test(T::random(0), FAIL):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc(T::revert):
//////////////////////////////////////////////////////////////////////////////

prog::test(expr(revert(T([]))),
           []):
prog::test(expr(revert(T([d]))),
           [d]):
prog::test(expr(revert(T([d,[],[]]))),
           [d, [], []]):
prog::test(expr(revert(T([d, [d, [d, [], []], []], [d, [d, [d, [], [d, [], []]], []], [d, [d, [], [d, [d, [d, [d, [], []], []], [d, [], []]], []]], [d, [], []]]]]))),
           [d, [d, [d, [d, [], []], [d, [d, [], [d, [d, [], []], [d, [], [d, [], []]]]], []]], [d, [], [d, [d, [], []], []]]], [d, [], [d, [], []]]]):


//////////////////////////////////////////////////////////////////////////////
prog::testfunc(T::toPoset):
//////////////////////////////////////////////////////////////////////////////
t := T([]): t := T::fromShapeAndWord(t, [$1..T::size(t)]):
prog::test(Graph::getEdges(combinat::trees::toPoset(t)),
           []):
t := T([d]): t := T::fromShapeAndWord(t, [$1..T::size(t)]):
prog::test(Graph::getEdges(combinat::trees::toPoset(t)),
           []):
t := T([d,[d],[d]]): t := T::fromShapeAndWord(t, [$1..T::size(t)]):
prog::test(Graph::getEdges(combinat::trees::toPoset(t)),
           [[1, 2], [1, 3]]):
t := T([2,[1],[3],[4]]):
prog::test(Graph::getEdges(combinat::trees::toPoset(t)),
           [[2, 1], [2, 3], [2, 4]]):
t := T([2,[1,[2]],[3],[4]]):
prog::test(Graph::getEdges(combinat::trees::toPoset(t)),
           TrapError=1028):
t := T([2,[1,[5]],[3,[4]],[4]]):
prog::test(Graph::getEdges(combinat::trees::toPoset(t)),
           TrapError=1028):
t := T([2,[1,[5,[8],[2]]],[3],[4]]):
prog::test(Graph::getEdges(combinat::trees::toPoset(t)),
           TrapError=1028):
t := T([2,[1,[5,[8],[6]]],[3,[7]],[4]]):
prog::test(Graph::getEdges(combinat::trees::toPoset(t)),
           [[1, 5], [2, 1], [2, 3], [2, 4], [3, 7], [5, 6], [5, 8]]):



//////////////////////////////////////////////////////////////////////////////
// Global checking of the package

prog::testfunc(T):
prog::testexit():

prog::check(T, 3):

// Local Variables: 
// mode: mupad
// End:
