/*****************************************************************************
  $Id: words.tst,v 1.1.1.5 2005/01/26 13:42:25 nthiery Exp $
  File:		lib/COMBINAT/TEST/words.tst
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  		Florent Hivert <florent.hivert@univ-mlv.fr>
  Purpose:	Test file for the library 'combinat::words'
  Created:	24/02/2001
  License:	LGPL
*****************************************************************************/

prog::testinit("words", 5.3):

export(combinat, Alias, words):

prog::testfunc(combinat::words::isA):
prog::test(combinat::words::isA([1, 2, 3]), TRUE):
prog::test(combinat::words::isA([a, b, a, b], {a, b, c}), TRUE):
prog::test(combinat::words::isA({a, b, a, b}, {a, b, c}), FALSE):
prog::test(combinat::words::isA([a, b, a, b], {a, c}), FALSE):
prog::test(combinat::words::isA([a, b, a, b], {a, b, c}, 4), TRUE):
prog::test(combinat::words::isA([a, b, a, b], {a, b, c}, 5), FALSE):
prog::test(combinat::words::isA([1, 3, 2, 3], 2), FALSE):
prog::test(combinat::words::isA([1, 3, 2, 3], 3), TRUE):
prog::test(combinat::words::isA([1, 3, 2, 3], 4), TRUE):
prog::test(combinat::words::isA([1, 3, 2, 3], 4, 3), FALSE):
prog::test(combinat::words::isA([1, 3, 2, 3], 4, 4), TRUE):
prog::test(combinat::words::isA([1, 3, 2, 3], 3, 4), TRUE):


prog::testfunc(combinat::words::count):

prog::test(combinat::words::count(0,0),		1):
prog::test(combinat::words::count(5,0),		1):
prog::test(combinat::words::count([a,b,c],0),	1):
prog::test(combinat::words::count(0,1),		0):
prog::test(combinat::words::count(5,1),		5):
prog::test(combinat::words::count([a,b,c],1),	3):
prog::test(combinat::words::count(7,13), 96889010407):
prog::test(combinat::words::count([a,b,c,d,e,f,g],13), 96889010407):

prog::testfunc(combinat::words::list):

// The order in which the words are listed is not in the
// specifications of combinat::words::list, so we sort the result
// before comparing.

prog::test(      combinat::words::list(0,0),       [[]] ):
prog::test(      combinat::words::list(10,0),      [[]] ):
prog::test( sort(combinat::words::list([a,b,c],0)),[[]] ):
prog::test(      combinat::words::list(0,1),       []   ):
prog::test( sort(combinat::words::list(5,1)),      [[1],[2],[3],[4],[5]] ):
prog::test( sort(combinat::words::list([a,b,c],1)),[[a],[b],[c]] ):
prog::test( sort(combinat::words::list([a,b],2)),  
            [[a,a],[a,b],[b,a],[b,b]] ):
prog::test( sort(combinat::words::list(3,3)),  
            sort([[1, 1, 1], [1, 1, 2], [1, 1, 3], [1, 2, 1], [1, 2, 2],
                  [1, 2, 3], [1, 3, 1], [1, 3, 2], [1, 3, 3], [2, 1, 1], 
                  [2, 1, 2], [2, 1, 3], [2, 2, 1], [2, 2, 2], [2, 2, 3],
                  [2, 3, 1], [2, 3, 2], [2, 3, 3], [3, 1, 1], [3, 1, 2],
                  [3, 1, 3], [3, 2, 1], [3, 2, 2], [3, 2, 3], [3, 3, 1],
                  [3, 3, 2], [3, 3, 3]])):

for n from 0 to 5 do
    for k from 0 to 4 do
        prog::test( nops(combinat::words::list(n,k)),
                   combinat::words::count(n,k)):
    end_for:
end_for:

// We don't test combinat::words::generator, since list is build
// on top of it.

prog::testfunc(combinat::words::evaluationTable):
prog::test(combinat::words::evaluationTable([b,a,d,b,c,d,b]),
           table(c = 1, d = 2, a = 1, b = 3)):
prog::test(combinat::words::evaluationTable([]),
           table()):

prog::test(combinat::words::evaluationSparse([]),
           []):
prog::test(combinat::words::evaluationSparse([4, 4, 2, 5, 2, 1, 4, 1]),
           [[1, 2], [2, 2], [4, 3], [5, 1]]):

prog::testfunc(combinat::words::evaluationPartition):
prog::test(combinat::words::evaluationPartition([b,a,d,b,c,d,b]),
           [3, 2, 1, 1]):
prog::test(combinat::words::evaluationPartition([]),
	   []):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc(combinat::words::evaluation):
//////////////////////////////////////////////////////////////////////////////
prog::test(combinat::words::evaluation([]),               []):
prog::test(combinat::words::evaluation([],4),             [0,0,0,0]):
prog::test(combinat::words::evaluation([],[a,b,c]),       [0,0,0]):
prog::test(combinat::words::evaluation([1,2,1]),          [2,1]):
prog::test(combinat::words::evaluation([1,3,1]),          [2,0,1]):
prog::test(combinat::words::evaluation([1,3,1], [1,3]),   [2,1]):
prog::test(combinat::words::evaluation([1,3,1], [1,2]),   [2,0]):
prog::test(combinat::words::evaluation([1,3,1], 4),       [2,0,1,0]):
prog::test(combinat::words::evaluation([1,3,1], 5),       [2,0,1,0,0]):
prog::test(combinat::words::evaluation([a,b,a], [a,b,c]), [2,1,0]):
prog::test(combinat::words::evaluation([a,b,a]), TrapError = 1028):
prog::test(combinat::words::evaluation([],{2}), [0]):

prog::testfunc(combinat::words::fromStandardAndEvaluation):
prog::test(combinat::words::fromStandardAndEvaluation([], []), 
	                                              []):
prog::test(combinat::words::fromStandardAndEvaluation([3,1,2], [1,2]), 
  	                                              [2,1,2]):
prog::test(combinat::words::fromStandardAndEvaluation([3,1,2], [1,0,2]), 
	                                              [3,1,3]):
for w in combinat::words::list(6,4) do
    prog::test(combinat::words::fromStandardAndEvaluation
	       (combinat::words::standard(w),
		combinat::words::evaluation(w,6)), 
	       w); 
end_for:
alpha := [a,b,s,c,d,e]:
for w in combinat::words::list(alpha,4) do
    prog::test(combinat::words::fromStandardAndEvaluation
	       (combinat::words::standard(w, alpha),
		combinat::words::evaluation(w, alpha),
		alpha), 
	       w); 
end_for:


prog::testfunc(combinat::words::alphabetToOrder):

order := combinat::words::alphabetToOrder([1,3,2,4]):
prog::test(bool(order(1,1)), FALSE):
prog::test(bool(order(1,2)), TRUE):
prog::test(bool(order(3,2)), TRUE):
prog::test(bool(order(2,3)), FALSE):
setTest := {a,b,c,d}:
orderSet := combinat::words::alphabetToOrder(setTest):
prog::test(orderSet(op(setTest,1),op(setTest,2)), 1<2): 
prog::test(orderSet(op(setTest,2),op(setTest,2)), 2<2):
prog::test(orderSet(op(setTest,3),op(setTest,2)), 3<2): 



prog::testfunc(combinat::words::standard):

order := combinat::words::alphabetToOrder([1,3,2,4]):
prog::test(combinat::words::standard([]), []):
prog::test(combinat::words::standard([3]), [1]):
prog::test(combinat::words::standard([1,1,3,2,3,1,2]), 
                                     [1,2,6,4,7,3,5]):
prog::test(combinat::words::standard([1,1,3,2,3,1,2], [1,2,3,4,5]),
                                     [1,2,6,4,7,3,5]):
prog::test(combinat::words::standard([1,1,3,2,3,1,2], [2,1,3,4,5]),
                                     [3,4,6,1,7,5,2]):
prog::test(combinat::words::standard([1,1,3,2,3,1,2], order),
                                     [1,2,4,6,5,3,7]):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("swap"):
prog::test(combinat::words::swap([a, b, c], 1), [b, a, c]):

prog::testfunc("swapIncrease"):
prog::test(combinat::words::swapIncrease([3, 1, 2], 1), [1, 3, 2]):
prog::test(combinat::words::swapIncrease([1, 3, 2], 1), [1, 3, 2]):

prog::testfunc("swapDecrease"):
prog::test(combinat::words::swapDecrease([3, 1, 2], 1), [3, 1, 2]):
prog::test(combinat::words::swapDecrease([1, 3, 2], 1), [3, 1, 2]):



//////////////////////////////////////////////////////////////////////////////
// shuffle, shuffling, split, splitting are tested in shuffle.tst
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
prog::testfunc(combinat::words::shuffleAugmented):
prog::test(combinat::words::shuffleAugmented([], []),
           [[]]):
prog::test(combinat::words::shuffleAugmented([], [1]),
           [[1]]):
prog::test(combinat::words::shuffleAugmented([1], []),
           [[1]]):
prog::test(combinat::words::shuffleAugmented([1], [1]),
           [[1, 1], [1, 1], [2]]):
prog::test(combinat::words::shuffleAugmented([a], [b]),
           [[b, a], [a, b], [a + b]]):
prog::test(combinat::words::shuffleAugmented([a], [1]),
           [[1, a], [a, 1], [a + 1]]):
prog::test(combinat::words::shuffleAugmented([a, b], [1, 2]),
	   [[1, 2, a, b], [1, a, 2, b], [a, 1, 2, b], [a + 1, 2, b],
	    [1, a + 2, b], [1, a, b, 2], [a, 1, b, 2], [a + 1, b, 2],
	    [a, b, 1, 2], [a, b + 1, 2], [1, a, b + 2], [a, 1, b + 2],
	    [a + 1, b + 2]]):
prog::test(combinat::words::shuffleAugmented([a, b, c], [1, 2]),
           [[1, 2, a, b, c], [1, a, 2, b, c], [a, 1, 2, b, c],
	    [a + 1, 2, b, c], [1, a + 2, b, c], [1, a, b, 2, c],
	    [a, 1, b, 2, c], [a + 1, b, 2, c], [a, b, 1, 2, c],
	    [a, b + 1, 2, c], [1, a, b + 2, c], [a, 1, b + 2, c],
	    [a + 1, b + 2, c], [1, a, b, c, 2], [a, 1, b, c, 2],
	    [a + 1, b, c, 2], [a, b, 1, c, 2], [a, b + 1, c, 2],
	    [a, b, c, 1, 2], [a, b, c + 1, 2], [1, a, b, c + 2],
	    [a, 1, b, c + 2], [a + 1, b, c + 2], [a, b, 1, c + 2],
	    [a, b + 1, c + 2]]):
prog::test(combinat::words::shuffleAugmented([a, b, c], [1, 2, 3]),
           [[1, 2, 3, a, b, c], [1, 2, a, 3, b, c], [1, a, 2, 3, b, c],
	    [a, 1, 2, 3, b, c], [a + 1, 2, 3, b, c], [1, a + 2, 3, b, c],
	    [1, 2, a + 3, b, c], [1, 2, a, b, 3, c], [1, a, 2, b, 3, c],
	    [a, 1, 2, b, 3, c], [a + 1, 2, b, 3, c], [1, a + 2, b, 3, c],
	    [1, a, b, 2, 3, c], [a, 1, b, 2, 3, c], [a + 1, b, 2, 3, c],
	    [a, b, 1, 2, 3, c], [a, b + 1, 2, 3, c], [1, a, b + 2, 3, c],
	    [a, 1, b + 2, 3, c], [a + 1, b + 2, 3, c], [1, 2, a, b + 3, c],
	    [1, a, 2, b + 3, c], [a, 1, 2, b + 3, c], [a + 1, 2, b + 3, c],
	    [1, a + 2, b + 3, c], [1, 2, a, b, c, 3], [1, a, 2, b, c, 3],
	    [a, 1, 2, b, c, 3], [a + 1, 2, b, c, 3], [1, a + 2, b, c, 3],
	    [1, a, b, 2, c, 3], [a, 1, b, 2, c, 3], [a + 1, b, 2, c, 3],
	    [a, b, 1, 2, c, 3], [a, b + 1, 2, c, 3], [1, a, b + 2, c, 3],
	    [a, 1, b + 2, c, 3], [a + 1, b + 2, c, 3], [1, a, b, c, 2, 3],
	    [a, 1, b, c, 2, 3], [a + 1, b, c, 2, 3], [a, b, 1, c, 2, 3],
	    [a, b + 1, c, 2, 3], [a, b, c, 1, 2, 3], [a, b, c + 1, 2, 3],
	    [1, a, b, c + 2, 3], [a, 1, b, c + 2, 3], [a + 1, b, c + 2, 3],
	    [a, b, 1, c + 2, 3], [a, b + 1, c + 2, 3], [1, 2, a, b, c + 3],
	    [1, a, 2, b, c + 3], [a, 1, 2, b, c + 3], [a + 1, 2, b, c + 3],
	    [1, a + 2, b, c + 3], [1, a, b, 2, c + 3], [a, 1, b, 2, c + 3],
	    [a + 1, b, 2, c + 3], [a, b, 1, 2, c + 3], [a, b + 1, 2, c + 3],
	    [1, a, b + 2, c + 3], [a, 1, b + 2, c + 3], [a + 1, b + 2, c + 3]]):
//////////////////////////////////////////////////////////////////////////////
prog::testfunc(combinat::words::fromStandard):

prog::test(sort(combinat::words::fromStandard([],3)), 
	   sort([[]])):
prog::test(sort(combinat::words::fromStandard([1],3)), 
	   sort([[1],[2],[3]])):
prog::test(sort(combinat::words::fromStandard([2,1],1)), 
	   sort([])):
prog::test(sort(combinat::words::fromStandard([3,2,1],2)), 
	   sort([])):
prog::test(sort(combinat::words::fromStandard([1,2],3)),
	   sort([[1,1],[1,2],[1,3],[2,2],[2,3],[3,3]])):
prog::test(sort(combinat::words::fromStandard([1,2],4)),
	   sort([[1,1],[1,2],[1,3],[2,2],[1,4],[2,3],[2,4],[3,3],[3,4],[4,4]])):
prog::test(sort(combinat::words::fromStandard([2,1],4)),
	   sort([[4, 1], [4, 2], [4, 3], [3, 1], [3, 2], [2, 1]])):
prog::test(sort(combinat::words::fromStandard([2,3,1],1)), 
	   sort([])):
prog::test(sort(combinat::words::fromStandard([2,3,1],2)), 
	   sort([[2,2,1]])):
prog::test(sort(combinat::words::fromStandard([2,3,1],3)), 
	   sort([[3, 3, 1], [3, 3, 2], [2, 2, 1], [2, 3, 1]])):
prog::test(sort(combinat::words::fromStandard([],[a])), 
	   sort([[]])):
prog::test(sort(combinat::words::fromStandard([2,3,1],[a])), 
	   sort([])):
prog::test(sort(combinat::words::fromStandard([2,3,1],[a,b])), 
	   sort([[b, b, a]])):
prog::test(sort(combinat::words::fromStandard([2,3,1],[a,b,c])), 
	   sort([[c, c, a], [c, c, b], [b, b, a], [b, c, a]])):
alpha := [a,b,c,d,e]:
for n from 0 to 4 do // for all permutations of size upto 4
    for p in combinat::permutations::list(n) do
	g := combinat::words::fromStandard_generator(p,alpha);
	while (w:=g()) <> FAIL do
	    prog::test(combinat::words::standard(w, alpha), p):
	end_while;
    end_for;
end_for:

//////////////////////////////////////////////////////////////////////////////
prog::testfunc(combinat::words::degInvLexLess):
//////////////////////////////////////////////////////////////////////////////

prog::test(combinat::words::degInvLexLess([], [1]),
           TRUE):
prog::test(combinat::words::degInvLexLess([], []),
           FALSE):
prog::test(combinat::words::degInvLexLess([1, 2], [1, 2]),
           FALSE):
prog::test(combinat::words::degInvLexLess([1, 2], [3, 2]),
           TRUE):
prog::test(combinat::words::degInvLexLess([1, 2], [4, 0]),
           TRUE):
prog::test(combinat::words::degInvLexLess([1, 2], [3, 0]),
           FALSE):
prog::test(combinat::words::degInvLexLess([1, 2, 0], [3, 2]),
           TRUE):
prog::test(combinat::words::degInvLexLess([1, 2, 0], [3]),
           FALSE):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc(words::hammingDistance):

prog::test(words::hammingDistance([],     []),        0):
prog::test(words::hammingDistance([1],    []),        1):
prog::test(words::hammingDistance([],     [1,2]),     2):
prog::test(words::hammingDistance([1, 2], []),        2):
prog::test(words::hammingDistance([1, 2], [1]),       1):
prog::test(words::hammingDistance([1, 2], [2]),       2):
prog::test(words::hammingDistance([1, 2], [2, 2, 3]), 2):

//////////////////////////////////////////////////////////////////////////////
// Global checking of the package

// This is done at the end to ensure as much as possible of the
// package is initialized. A recursive prog::init(combinat::words)
// would be usefull here.

prog::testfunc(combinat::words):
prog::testexit():

prog::check(combinat::words, 3):


// Local Variables: 
// mode: mupad
// End:
