/*****************************************************************************
  $Id: countingFunctions.mu,v 1.1.1.2 2005/01/26 13:42:24 nthiery Exp $
  File:		lib/COMBINAT/countingFunctions.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Purpose:	Source code for the domain combinat::countingFunctions
  Created:	Sat Jul  3 10:19:07 2004
  License:	LGPL
  Project:	MuPAD-Combinat, http://mupad-combinat.sf.net
*****************************************************************************/

alias(self = op(_act_proc_env(), 1)):

domain combinat::countingFunctions
    inherits Dom::BaseDomain;
    category Cat::Ring;
    axiom Ax::systemRep;

    info_str := "combinat::countingFunctions -- utilities for counting functions";
    interfaceAdd := {
		     hold(zero), hold(one),
		     hold(_plus), hold(_negate),
		     hold(_mult), hold(mult2), hold(_invert),
		     hold(D), hold(laplace), hold(invlaplace)
		    };

    testtype :=
    proc(f, T)
    begin
	if T=dom then
	    testtype(f, Type::AnyType);
	else
	    FAIL;
	end_if;
    end_proc;

    convert := id;
    iszero := iszero;
    expr := expr;

    zero := 0;

    one := n -> if n=0 then 1; else 0; end_if;

    _plus := _plus;

    _negate := _negate;

    // TODO: reimplement _mult using karatsuba
    mult2 :=
    proc(f1: dom, f2: dom) : dom
        option escape;
    begin
	proc(n: Type::NonNegInt)
	    option remember;
	    local i;
	begin
	    _plus ((f1(i) * f2(n-i)) $ i = 0..n);
	end_proc;
    end_proc;

    _mult := operators::makeAssociative(dom::mult2, Source=dom, ImageSet=dom, Zero = dom::one);

    _invert :=
    proc(f: dom) : dom
        option escape;
    begin
	if iszero(f(0)) then
	    return(FAIL);
	end_if;
	proc(n: Type::NonNegInt)
	    option remember;
	    local i;
	begin
	    if n=0 then
		1/f(0);
	    else
		- _plus (self(i) * f(n-i) $ i = 0..n-1) / f(0)
	    end_if;
	end_proc;
    end_proc;

    D :=
    proc(f: dom) : dom
        option escape;
    begin
	proc(n: Type::NonNegInt)
	begin
	    (n+1)*f(n+1);
	end_proc
    end_proc;

    int :=
    proc(f: dom) : dom
        option escape;
    begin
	proc(n: Type::NonNegInt)
	begin
	    if n=0 then
		0
	    else
		f(n-1)/n;
	    end_if;
	end_proc
    end_proc;

    laplace :=
    proc(f: dom) : dom
    begin
	fact*f;
    end_proc;

    invlaplace :=
    proc(f: dom) : dom
    begin
	f/fact;
    end_proc;

    // set(fA)
    // Input: fA is the counting function for a combinatorial class A
    // returns a counting function for the combinatorial class of sets of A
    
    // multiset, cycles, lyndon

end_domain:
