/*****************************************************************************
  $Id: decomposableObjects.mu,v 1.1.1.8 2005/01/26 13:42:24 nthiery Exp $
  File:		lib/COMBINAT/decomposableObjects.mu
  Authors:	Sbastien Cellier <snoteip@yahoo.fr>
		Nicolas M. Thiry <nthiery@users.sourceforge.net>
                Florent Hivert <florent.hivert@univ-mlv.fr>
  Created:	16/05/2002
  Adapted from:	CS package v1.5 for MuPAD 1.4.2
		http://dept-info.labri.u-bordeaux.fr/~dutour/CS
		Sylvie Corteel - LRI
		Alain Denise - LRI
		Isabelle Dutour - LaBRI
		Frdric Sarron - LRI
		Paul Zimmermann - INRIA
  License:	LGPL
*****************************************************************************/
/** specification is a set or list of productions of the form
    A = Epsilon
    A = Atom
    A = Primitive(size, ...)
    A = Union(B,C,...)
    A = Prod(B,C,...)
    A = MultX(B)
    A = DivX(B)
    A = Powerset(B)
    A = Multiset(B)
    A = Sequence(B)
    A = Cycle(B)
    A = Lyndon(B)
    A = Predefined(function)
    A = Domain(domain)

Old doc from CS:
compile(s [, type=labelled] [, target=C] [, mode=graphics] [, library=GMP] 
          [, file="B.c"] [, main=NT]) 
- type is either unlabelled (default) or labelled
- target is either MuPAD (default) or C or MODULE or JAVA
- mode is either text (default) or graphics or applet (in JAVA mode)
- library is either DPE (default) or GMP
- file (string) is the name of the output file (in C, MODULE or JAVA mode)
- main is a non-terminal (in C or JAVA mode)

        >> dom::standardForm({M=Union(Z,Prod(Z,M),Prod(Z,M,M))},unlabelled);
**/

export(combinat, Alias,
       generators):

domain combinat::decomposableObjects
    (Specification: Type::Union(Type::SetOf  (Type::Equation(DOM_IDENT)),
				Type::ListOf (Type::Equation(DOM_IDENT)),
				Type::TableOf(DOM_IDENT)))

    local Options;
    inherits Dom::BaseDomain;
    category Cat::GradedCombinatorialClass;
    axiom    Ax::systemRep;

    info_str := "Parametrized library 'combinat::decomposableObjects': utilities for decomposable combinatorial classes";

    interfaceAdd :=
	{hold(specification),
	 hold(standardSpecification), hold(standardSpecificationCount),
	 hold(unrank), hold(random),
	 hold(algebraicEquationSystem),
	 hold(algebraicEquation),
	 hold(recurrenceRelation),
	 hold(generateCode),
	 hold(listFromUnrankAndCount)
	 };

    initDomain :=
    proc()
    begin
        // Standard workaround for MuPAD 2.0.0's option remember bug
	%if version()=[2,0,0] then
	    dom::makeValuation;
	    dom::standardSpecification;
	    dom::standardSpecificationCount;
	end_if:
    end_proc;

    standardForm := dom::standardSpecification;
    standardForm_Count := dom::standardSpecificationCount;

    options := Options;

    /// The specification, as a table
    specification := Specification;

    // For each non terminal, we store a few information about the
    // corresponding generating series and elements
    count_functions  := table(); // function for counting
    unrank_functions := table(); // function for unranking
    random_functions := table(); // function for random
    valuation        := table(); // valuation of the generating series
    degree           := table(); // degree of the generating series
    computed         := table(); // approximative number of terms already computed

    notImplemented :=
    proc(operation: Type::Option, message: DOM_STRING)
    begin
	// Translates option to a string.  would be simpler with
	// stringlib::tolower, but that is not available in MuPAD 2.0.0
	operation :=
	table(Count               = "count",
	      Unrank              = "unrank",
	      Random              = "random",
	      Generator           = "generator",
	      GeneratorFromRank = "generatorFromRank") [ operation ];
	message := operation. " not implemented".message;
	slot(dom, operation."FailureReason") := message;
	dom::interface := dom::interface minus { text2expr(operation) }
			                 union { text2expr(operation."FailureReason") };
	error(message);
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // Utilities, ...
    //////////////////////////////////////////////////////////////////////////

    domtype := DOM_EXPR; // not quite perfect
    // type of elements generated by this domain
//    testtype  := (x) -> testtype(x, Type::AnyType);
    isA := (obj) -> UNKNOWN;

    // type for nonterminals
    typeNonTerminal := DOM_IDENT;
    // Generates fresh nonterminals named NonTerminal1, NonTerminal2, ...
    // One may optionaly specify another prefix to use than NonTerminal.
    // To avoid namespace polution, we recommend to only use prefixes like
    // NonTerminalProd, NonTerminalSequence, ...
    genident :=
    proc()
	local i;
	option escape;
    begin
	i:=0;
	proc(nonTerminal=hold(NonTerminal))
	begin
	    i := i+1;
	    nonTerminal := nonTerminal.i;
	    if nonTerminal <> eval(nonTerminal) then
		warning("Conflict: the identifier ".val(nonTerminal)." already has a value");
	    end_if;
	    eval(subsop(hold(protect)(NIL), 1=val(nonTerminal)));
	    nonTerminal;
	end_proc;
    end_proc();

    typeSpecification := Type::TableOf(dom::typeNonTerminal);

    //////////////////////////////////////////////////////////////////////////
    // types for standard specifications
    //////////////////////////////////////////////////////////////////////////

    /**
        A standard expression is an expression of the form:
        Primitive(size, <label>)// element of size size, printed out as label: z^size
        Primitive(size, <label>,// element of size size, printed out as label: z^size * weight
	     <weight>,
	     <exactWeight>)	// version of the weight to be used in precomputations
        Alias(B <, transform>)	// alias for B: B(z)
        Union(B,C,...)		// disjoint union: B(z)+C(z)+...
        Prod(B,C)		// product: B(z)*C(z)
        Theta(B)		// B with one pointed atom: z*d/dz B(z) or (B(z)-B(0))/z for labelled
        Int(B)			// inverse of Theta: 1/z*int(B(z)) or z*B(z) for labelled
        DivX(B)		        // B with one unlabelled atom: (B(z)-B(0))/z for labelled
        MultX(B)		// inverse of DivX: z*B(z) for labelled
	Delta(B,u)		// sequences of k>=1 identical elements of B;
				// each of those sequences is counted u(k) times
	Predefined(count, random, unrank)
	Domain(mydomain)	// a domain of category Cat::CombinatorialClass

	where B,C, ... are non terminals
    **/
    typeStandardExpression := Type::Predicate(dom::isStandardExpression);

    // Check length options for expressions like Sequence(B, Length=...)
    checkLengthOptions :=
    proc(expression)
	local i;
    begin
	if nops(expression) = 0 then
	    error(expr2text(op(expression,0))." requires one argument; got ".
		  expr2text(expression));
	end_if;
	for i from 2 to nops(expression) do
	    if not testtype(op(expression,i),
			    Type::Equation(Type::Predicate(opt -> contains({Length, MinLength, MaxLength, MinPart, MaxPart},opt)),
					   Type::NonNegInt)) then
		error("incorrect option ". expr2text(op(expression,i)) . " in expression ".
		      expr2text(expression));
	    end_if;
	end_for;
    end_proc;

    isStandardExpression :=
    proc(expression, returnUsed=FALSE, onError=proc() name doNothing; option hold; begin end)
	local used, A;
    begin
	used := {};
	if type(expression)<>"function" then
	    return(onError("Incorrect expression: ". expr2text(expression)));
	end_if;
	// Check the number of arguments
	case op(expression, 0)
	  of Alias do
	      if nops(expression) > 2 or nops(expression) = 0 then
		  return(onError("Alias expects one or two arguments: ". expr2text(expression)));
	      end;
	      used := {op(expression,1)};
	      break;
	  of Primitive do
	      if nops(expression) < 1 or nops(expression) > 4 then
		  return(onError(expr2text(op(expression,0))." expects one to four arguments: ". expr2text(expression)));
	      end_if;
	      break;
	  of Theta do
	  of Int do
	  of DivX do
	  of MultX do
	      if nops(expression) <> 1 then
		  return(onError(expr2text(op(expression,0))." expects one argument: ". expr2text(expression)));
	      end_if;
	      used := {op(expression,1)};
	      break;
	  of Multiset do
	  of Powerset do
	  of Cycle do
	  of Lyndon do
	      if dom::options[Labelled] then
		  return(onError(expr2text(op(expression,0))." is not allowed in a labelled standard expression: ". expr2text(expression)));
	      end_if;
	      if nops(expression) < 1 then
		  return(onError(expr2text(op(expression,0))." expects at least one argument: ". expr2text(expression)));
	      end_if;
	      used := {op(expression,1)};
	      break;
	  of Predefined do
	     if not contains({1,2,3}, nops(expression)) then
		 return(onError(expr2text(op(expression,0))." expects one to three arguments: ". expr2text(expression)));
	     end_if;
	     break;
	  of Domain do
	     if nops(expression) <> 1 then
		 return(onError(expr2text(op(expression,0))." expects one argument: ". expr2text(expression)));
	     end_if;
	     break;
	  of Delta do
	      if nops(expression) <> 2 then
		  return(onError(expr2text(op(expression,0))." expects two arguments: ". expr2text(expression)));
	      end_if;
	      used := {op(expression,1)};
	      break;
	  of Prod do
	      if nops(expression) <> 2 then
		  return(onError(expr2text(op(expression,0))." expects two arguments: ". expr2text(expression)));
	      end_if;
	      used := {op(expression)};
	      break;
	  of Union do
	      used := {op(expression)};
	      break;
	  otherwise
	      return(onError("Unknown constructor: ".expr2text(op(expression,0))));
	end_case;
	for A in used do
	    if not testtype(A, dom::typeNonTerminal) then
		return(onError("Not a non terminal: ".expr2text(A)));
	    end_if;
	end_for;
	if returnUsed then
	    used
	else
	    TRUE;
	end_if;
    end_proc;

    typeStandardSpecification :=
        Type::Predicate("StandardSpecification", dom::isStandardSpecification);
    isStandardSpecification :=
    proc(specification, onError = proc() option hold; begin FALSE end) : DOM_BOOL
	local used, u, defined, prod;
    begin
	if not testtype(specification, 
			Type::TableOfIndex(dom::typeNonTerminal)) then
	    return(onError("Expecting a specification of the form table(A = ..., B = ...); got ".
			   expr2text(specification)));
	end_if;
	defined := {};  // the non-terminals that are defined in the specif
	used := {};     // the non-terminals that are used in the specif
	for prod in specification do
	    defined := defined union({lhs(prod)});
	    u := dom::isStandardExpression(rhs(prod), TRUE, onError);
	    if u = FALSE then
		return(FALSE);
	    end_if;
	    used := used union u;
	end_for;
	// Finally, all the used non terminals should be defined
	if used minus defined <> {} then
	    return(onError("Undefined non-terminal(s)". expr2text(used minus defined)));
	end_if;
	return(TRUE);
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // types for standard context free specifications
    //////////////////////////////////////////////////////////////////////////

    /**
        A standard specification is context free if it only uses the
	productions Alias, Epsilon, Atom, Union, and Prod
    **/

    contextFreeConstructions := {Alias, Epsilon, Primitive, Union, Prod};

    typeContextFreeStandardSpecification :=
        Type::Predicate(dom::isContextFreeStandardSpecification);

    isContextFreeStandardSpecification :=
    proc(specification) : DOM_BOOL
	local prod;
    begin
	if not dom::isStandardSpecification(specification) then
	    return(FALSE);
	end_if;
	_lazy_and(contains(dom::contextFreeConstructions, op(prod,[2,0]))
		  $ prod in specification);
    end_proc;

    isContextFree :=
    proc()
	option remember;
    begin
	testtype(dom::standardSpecificationCount(),
		 dom::typeContextFreeStandardSpecification)
    end_proc;

    optimizeStandardSpecification :=
    proc(specification: dom::typeStandardSpecification)
	: dom::typeStandardSpecification
    /**
	Returns an equivalent standard specification which is hopefully
        smaller. Currently, this is achieved by factoring recursively
        common production rules through aliases (usual DAG optimisation).
    **/
	local aliases, newaliases, reverse, modified, eq;
    begin
        // Extract the aliases of the form A=Alias(B) from the specification
	specification := split(specification,
			       eq -> (op(eq,[2,0]) = Alias and nops(rhs(eq))=1));
	aliases       := map(specification[1], op, 1);
	// aliases now contain something of the form A=B, B=C
	specification := specification[2];

        // Simplify the aliases by transitivity, and apply them on the
        // rest of the specification
	while TRUE do
	    newaliases := subs(aliases, aliases);
	    if newaliases=aliases then break; end_if;
	    aliases := newaliases;
	end_while;
	// aliases now contain something of the form A=C, B=C
	specification := subs(specification, aliases);

        // Try to find a duplicate in the specification, such as
        // A1=Prod(B,C) and A2=Prod(B,C), and replace it by an alias
        // A2=A1. Repeat this process until nothing changes anymore
	repeat
            modified := FALSE;
            reverse := table();
	    for eq in specification do
		if contains(reverse, op(eq,2)) then
		    aliases[op(eq,1)] := reverse[op(eq,2)];
		    delete(specification[op(eq,1)]);
		    specification := subs(specification, 
					  op(eq,1)=reverse[op(eq,2)]);
		    aliases := subs(aliases, 
				    op(eq,1)=reverse[op(eq,2)]);
		    modified := TRUE;
		    break;
		end_if;
		reverse[op(eq,2)] := op(eq,1);
	    end_for;
	until not modified
	end_repeat;
	table(op(specification), op(map(aliases, Alias)));
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // Standard form of the specification
    //////////////////////////////////////////////////////////////////////////

    standardizeSpecification :=
    proc(specification: dom::typeSpecification, operation=Generator)
	: dom::typeStandardSpecification
    /**
       Returns a standard specification which is equivalent to 
       dom::specification
    **/
	local A, expression, production;
    begin
	// The following domain entries are shared with addProduction
	// and are used to store the created production rules A=expression

	dom::standardizeSpecificationInternal := table(); // will contain A=expression
	dom::standardizeSpecificationReverse  := table(); // will contain expression=A
	dom::standardizeSpecificationOperation:= operation;

	for production in specification do
	    A := lhs(production);
	    expression := rhs(production);
	    // Syntactic sugar:
	    //  - transform A = Atom    into A = Atom(A)
	    //              A = Epsilon into A = Epsilon(A) 
	    if contains({Atom, Epsilon}, expression) then
		expression := expression(A);
	    end_if;
	    dom::addProduction(expression, A);
	end_for;
	if testargs() then
	    dom::isStandardSpecification(dom::standardizeSpecificationInternal,
					 error);
	end_if;
	dom::optimizeStandardSpecification(dom::standardizeSpecificationInternal);
    end_proc;

    standardExpression :=
    // Take an expression, such as
    //     Z, Atom, Atom(X), combinat::partitions, Z, Prod(...), Sequence(...), ...
    // Returns an equivalent expression in standard form
    // If needed, some production rules are added to dom::standardizeSpecificationInternal
    proc(expression) : dom::typeStandardExpression
	local constructor;
    begin
	// syntactic sugar: rewrite anonymous Atom and Epsilon
	// as Atom(Atom) and Epsilon(Epsilon), non terminals
	// as Alias(nonTerminal), domains as Domain(domain)
	if contains({Atom, Epsilon}, expression) then
	    expression := expression(expression);
	elif testtype(expression, dom::typeNonTerminal) then
	    expression := Alias(expression);
	elif domtype(expression) = DOM_DOMAIN then
	    expression := Domain(expression);
	end_if;
	if testargs() and type(expression)<>"function" then
	    error("invalid production rule: ".expr2text(expression));
	end_if;

	constructor:=dom::standardizeSpecificationFunctions[op(expression,0)];
	if domtype(constructor)<>DOM_PROC then
	    error("invalid right-hand side of production: ".expr2text(expression));
	end_if;
	constructor(op(expression));
    end_proc;

    // returns a non terminal which is equivalent to the expression
    // in the existing list of standard production rules
    // if A is specified, it will be used as nonTerminal.
    addProduction :=
    proc(expression, A=FAIL)
    begin
	dom::addStandardProduction(dom::standardExpression(expression, FALSE), A);
    end_proc;

    addStandardProduction :=
    proc(expression: dom::typeStandardExpression, A=FAIL)
    begin
	if contains(dom::standardizeSpecificationReverse, expression) then
	    expression := Alias(dom::standardizeSpecificationReverse[expression]);
	end_if;
	if A = FAIL then
	    // Optimization: if the expression is a pure alias, and A
	    // as not been specified, we avoid creating a new non-terminal
	    if op(expression,0) = Alias and nops(expression)=1 then
		return(op(expression,1));
	    end_if;
	    A := dom::genident();
	end_if;
	dom::standardizeSpecificationInternal[A] := expression;
	dom::standardizeSpecificationReverse[expression] := A;
	A;
    end_proc;

    /** Specifications for standardSpecificationFunctions
     *
     *    Bla(B,C,D)
     * yields a call to
     *    standardSpecificationFunctions[Bla](B, C, D)
     * which should return an equivalent standard expression
    **/
    standardizeSpecificationFunctions :=
    map(table
    (
     Epsilon =
     proc()
     begin
	 if args(0)>3 then
	     error("invalid right-hand side of production: ".
		   expr2text(Epsilon(args())));
	 end_if;
	 dom::standardExpression(Primitive(0, args()));
     end_proc,

     // A=Atom(<label>, <weight>, <ExactWeight>)
     Atom =
     proc()
     begin
	 if args(0)>3 then
	     error("invalid right-hand side of production: ".
		   expr2text(Atom(args())));
	 end_if;
	 dom::standardExpression(Primitive(1, args()));
     end_proc,

     // A=Primitive(size, <label>, <weight>, <ExactWeight>)
     Primitive =
     proc(size)
     begin
	 if args(0)>4 then
	     error("invalid right-hand side of production: ".
		   expr2text(Primitive(args())));
	 end_if;
	 if dom::standardizeSpecificationOperation = Count then
	     if args(0) >= 3 then
		 Primitive(size, args(3..args(0)))
	     else
		 Primitive(size, dom::options[DefaultWeight])
	     end_if;
	 else
	     Primitive(args());
	 end_if;
     end_proc,

     Alias =
     proc(B, f=id)
	 local i;
     begin
	 if args(0)>2 then 
	     error("invalid right-hand side of production: ".
		   expr2text(Alias(args())));
	 end_if;
	 if not testtype(B, dom::typeNonTerminal) then
	 B := dom::standardExpression(B);
	 if op(B,0) = Alias then
	     // If B is also an Alias, compose them together, and
	     // do some simplifications
	     f := [ f, op(B, 2..nops(B)) ];
	     i := 1;
	     while i <= nops(f) do
		 if type(f[i]) = "_fconcat" then
		     f[i] := op(f[i]); next;
		 end_if;
		 if f[i] = id then
		     delete(f[i]); next;
		 end_if;
		 if f[i] = op and i<nops(f) and f[i+1] in {Prod, Sequence, Cycle, Multiset} then
		     delete(f[i+1]); delete(f[i]); next;
		 end_if;
		 i := i+1;
	     end_while;
	     f := _fconcat(op(f));
	     B := op(B,1);
	 else
	     B := dom::addStandardProduction(B);
	 end_if;
	 end_if;
	 if f = id or dom::standardizeSpecificationOperation = Count then
	     Alias(B)
	 else
	     Alias(B, f)
	 end_if;
     end_proc,

     Union =
     proc()
	local i, operands;
     begin
	 if args(0) = 1 then
	     // Optimize away the Union
	     dom::standardExpression(args(1));
	 else
	     operands := map([args()], dom::addProduction);
	     if dom::standardizeSpecificationOperation = Count then
		 operands := sort(operands);
	     end_if;
	     Union(op(operands));
	 end_if;
     end_proc,

     Prod =
     proc()
	local i, operands;
     begin
	 // Optimize away empty epsilons; this allow not to worry
	 // about them when writing specifications
	 operands := select([args()], not _in, {Epsilon(), Primitive(0)});
	 case nops(operands)
	   of 0 do // Optimize away the Prod
	       return(Epsilon(Prod()));
	   of 1 do // Optimize away the Prod
	       return(Alias(dom::addProduction(operands[1]), Prod));
	   of 2 do
	       operands := map(operands, dom::addProduction);
	       if dom::standardizeSpecificationOperation = Count then
		   operands := sort(operands);
	       end_if;
	       return(Prod(op(operands)));
	   otherwise
	       return(Prod(dom::addProduction(operands[1]),
			   dom::addProduction(Alias(Prod(operands[i] $ i=2..nops(operands)),
						    op))));
	 end_case;
     end_proc,

     Theta =
     proc(B)
     begin
	 if args(0)<>1 then 
	     error("invalid right-hand side of production: ".
		   expr2text(Theta(args())));
	 end_if;
	 return(Theta(dom::addProduction(B)));
     end_proc,

     Int =
     proc(B)
     begin
	 if args(0)<>1 then 
	     error("invalid right-hand side of production: ".
		   expr2text(Int(args())));
	 end_if;
	 return(Int(dom::addProduction(B)));
     end_proc,

     Delta =
     proc(B, u)
     begin
	 if args(0)<>2 then
	     error("invalid right-hand side of production: ".
		   expr2text(Delta(args())));
	 end_if;
	 return(Delta(dom::addProduction(B),u));
     end_proc,

     DivX =
     proc(B)
     begin
	 if args(0)<>1 then 
	     error("invalid right-hand side of production: ".
		   expr2text(DivX(args())));
	 end_if;
	 return(DivX(dom::addProduction(B)));
     end_proc,

     MultX =
     proc(B)
     begin
	 if args(0)<>1 then 
	     error("invalid right-hand side of production: ".
		   expr2text(MultX(args())));
	 end_if;
	 return(MultX(dom::addProduction(B)));
     end_proc,

     BoxedProd =
     proc(B, C)
     begin
	 if args(0)<>2 then
	     error("invalid right-hand side of production: ".
		   expr2text(BoxedProd(args())));
	 end_if;
	 if not dom::options[Labelled] then
	     error("Boxed products are only available for labelled structures");
	 end_if;
	 return(MultX(dom::addProduction(Prod(DivX(B), C))));
     end_proc,

     // standard form for A=SequenceGeneric(B, MinLength = , Length=, )
     SequenceGeneric =
     proc(B)
	 name standardExpressionSequenceGeneric;
	 local k, c, i, T, options,
	       FB, OB, FT, OT, minLength, maxLength,
	       result;
     begin
	 // Type checking and option parsing
	 if testargs() and args(0)=0 then
	    error("Invalid production rule: ". expr2text(SequenceGeneric(args())));
	 end_if;
	 if traperror((
	 options := combinat::getOptions
		  (table(MinLength = 0,
			 MaxLength = infinity,
			 Length = FAIL,
			 OperandsTrans = id,
			 Trans  = id,
			 FirstOperandTrans = FAIL,
			 FirstTrans = FAIL),
		   table(MinLength = Type::NonNegInt,
			 MaxLength = Type::Union(Type::NonNegInt,
						 Type::Singleton(infinity)),
			 Length    = Type::Union(Type::NonNegInt, DOM_FAIL),
			 FirstOperandTrans = Type::AnyType,
			 OperandsTrans = Type::AnyType,
			 FirstTrans = Type::AnyType,
			 Trans = Type::AnyType),
		   1,
		   Error))) <> 0 then
	     if testargs() then
		fprint(Unquoted, 0, "Invalid production rule: ". expr2text(SequenceGeneric(args())));
	     end_if;
	     lasterror();
	 end_if;
	 if options[Length] <> FAIL then
	     options[MinLength] := options[Length];
	     options[MaxLength] := options[Length];
	 end_if;
	 if options[MinLength] > options[MaxLength] then
	    error("Invalid production rule: ". expr2text(SequenceGeneric(args())));
	 end_if;
	 minLength := options[MinLength];
	 maxLength := options[MaxLength];

	 if options[FirstOperandTrans] = FAIL then
	     options[FirstOperandTrans] := options[OperandsTrans];
	 end_if;
	 if options[FirstTrans] = FAIL then
	     options[FirstTrans] := options[Trans];
	 end_if;
	 
	 if maxLength = 0 then
	     return(Primitive(0));
	 end_if;

	 B  := dom::addProduction(B);
	 FT := options[FirstTrans];
	 FB := dom::addProduction(options[FirstOperandTrans](B));
	 OT := options[Trans];
	 OB := dom::addProduction(options[OperandsTrans](B));
	 
	 assert(maxLength >= max(1, minLength));
	 // Build a production for sequences of length between
	 // 0 and maxlength-max(MinLength,1), with the last element being
	 // a B, and the others OB's
	 if maxLength = infinity then
	     T := dom::genident(hold(NonTerminalSequence));
	     dom::addProduction(Alias(Prod(OB, Union(Epsilon(), OT(T))), op),
				T);
	     if minLength <= 1 and OB = FB then
	     // This is a very often used case
	     // So, we treat it separately just to optimize it
		 return(dom::standardExpression(Union(if minLength = 0 then
							  Epsilon()
						      else
							  null()
						      end_if,
						      FT(T))));
	     end_if;
	     T := dom::addProduction(Union(Epsilon(), OT(T)));
	 else
	     T := Epsilon();
	     for i from maxLength downto max(minLength, 1) + 1 do
		 T := dom::addProduction(Union(Epsilon(),
					       OT(Alias(Prod(OB, T), op))));
	     end_for;
	 end_if;

         // Build a production for non empty sequences of length between
	 // max(1,MinLength)-1 and maxlength-1, with the last element being
	 // a B, and the others OB's
	 for i from max(minLength, 1) downto 2 do
	     // TODO: should define T as Prod( SequenceOfFixedLength(OB, n), T)
	     // with SequenceOfFixedLength using "a repeated squaring" strategy
	 	 T := dom::addProduction(OT(Alias(Prod(OB, T), op)));
	 end_for;

	 // Build an expression for sequences of length between max(1,minlength)
	 // and maxlength, with the first element being a FB, and the
	 // others OB's.
	 result := dom::standardExpression(FT(Alias(Prod(FB,T),op)));

	 dom::standardExpression
	 (Union
	  (if minLength  = 0 then Epsilon() else null() end_if,
	   result
	  ))
     end_proc,

     // returns a standard specification for the sequences of
     // B of length <length> (returned as MuPAD sequences B,B,B)

     // This uses the standard binary powering trick to produce
     // a standard specification of logarithmic size w.r.t. <length>.
     // We use the binary expansion of <length> to build this specification
     // from the specs of sequences of length 1, 2, 4, 8, ... This
     // increases the chances of reusing those specs for other sequences.
     SequenceOfLengthBinary =
     proc(B, length)
	 local i, binaryExpansion, BPow, result;
     begin
	 if length=0 then return(Primitive(0)); end_if;
	 binaryExpansion := numlib::g_adic(length,2);
	 // invariant: BPow is a specification for the sequences
	 // of length 2^i of B
	 BPow := dom::addProduction(B);
	 i := 1;
	 // note that at least one element of binaryExpansion is non-zero
	 while binaryExpansion[i] = 0 do
	     BPow := Alias(Prod(BPow,BPow), op);
	     i := i+1;
	 end_while;
	 result := BPow;
	 while i < nops(binaryExpansion) do
	     BPow := Alias(Prod(BPow,BPow), op);
	     i := i+1;
	     if binaryExpansion[i] <> 0 then
		 result := Alias(Prod(BPow, result), op);
	     end_if;
	 end_while;
	 dom::standardExpression(result);
     end,

     // returns a standard specification for the sequences of
     // B of length n (returned as a MuPAD sequence B,B,B))
     // Code duplication with the previous one!
     SequenceOfLengthAtMostBinary =
     proc(B, length)
	 local i, binaryExpansion, BPow, BPowAtMost, result;
     begin
	 if length=0 then return(Primitive(0)); end_if;
	 if length=infinity then
	     result := dom::genident(hold(NonTerminalSequence));
	     dom::addProduction(Union(Epsilon(), Alias(Prod(B, result), op)),
				result);
	     // Note: in most cases, the Alias will be automatically
	     // optimized away, and this makes for a standard expression,
	     // as required.
	     return(Alias(result));
	 end_if;
	 binaryExpansion := numlib::g_adic(length,2);
	 // invariant: BPow is a specification for the sequences
	 // of length 2^i of B
	 // BPowAtMost is a specification for the sequences
	 // of length at least 1 and at most 2^i of B
	 BPow := dom::addProduction(B);
	 BPowAtMost := BPow;
	 i := 1;
	 // note that at least one element of binaryExpansion is non-zero
	 while binaryExpansion[i] = 0 do
	     BPowAtMost := Union(Alias(Prod(BPow, BPowAtMost), op),
				 BPowAtMost);
	     BPow := Alias(Prod(BPow,BPow), op);
	     i := i+1;
	 end_while;
	 result := BPowAtMost;
	 while i < nops(binaryExpansion) do
	     BPowAtMost := Union(Alias(Prod(BPow, BPowAtMost), op),
				 BPowAtMost);
	     BPow := Alias(Prod(BPow,BPow), op);
	     i := i+1;
	     if binaryExpansion[i] <> 0 then
		 result := Union(Alias(Prod(BPow, result), op),
				 BPowAtMost);
	     end_if;
	 end_while;
	 dom::standardExpression(Union(Epsilon(), result));
     end,

     // standard form for A=Sequence(B <, Length =..., MinLength=..., MaxLength=...>)
     Sequence =
     proc(B)
	 local k, c, T1, T2,
	       options, minLength, maxLength, fixed, variable;
     begin
	 // Type checking and option parsing
	 dom::checkLengthOptions(Sequence(args()));
 	 if not Options[hold(RepeatedSquaring)] then
	     return(dom::standardExpression(Alias(SequenceGeneric(args()), Sequence)))
	 end_if;
	 // FIXME: lots of code duplication in this option parsing ...
	 if traperror((
	 options := combinat::getOptions
		  (table(MinLength = 0,
			 MaxLength = infinity,
			 Length = FAIL),
		   table(MinLength = Type::NonNegInt,
			 MaxLength = Type::Union(Type::NonNegInt,
						 Type::Singleton(infinity)),
			 Length    = Type::Union(Type::NonNegInt, DOM_FAIL)),
		   1,
		   Error))) <> 0 then
	     if testargs() then
		fprint(Unquoted, 0, "Invalid production rule: ". expr2text(Sequence(args())));
	     end_if;
	     lasterror();
	 end_if;
	 if options[Length] <> FAIL then
	     options[MinLength] := options[Length];
	     options[MaxLength] := options[Length];
	 end_if;
	 if options[MinLength] > options[MaxLength] then
	    error("Invalid production rule: ". expr2text(SequenceGeneric(args())));
	 end_if;
	 minLength := options[MinLength];
	 maxLength := options[MaxLength];
	 fixed    := SequenceOfLengthBinary(B, minLength);
	 if minLength=maxLength then
	     return(dom::standardExpression(Alias(fixed, Sequence)));
	 end_if;
	 variable := SequenceOfLengthAtMostBinary(B, maxLength-minLength);
	 if minLength=0 then
	     return(dom::standardExpression(Alias(variable, Sequence)));
	 end_if;
	 dom::standardExpression(Alias(Prod(fixed, variable), Sequence @ op));
     end_proc,

     // standard form for A=Cycle(B)
     Cycle =
     proc(B: Type::AnyType)
	 name standardExpressionCycle;
	 local T1;
     begin
	 dom::checkLengthOptions(Cycle(args()));
	 if dom::options[Labelled] then
	 // Could use instead Sequence and repeated squaring optimization
	 dom::standardExpression
	 (Alias(SequenceGeneric(B,
//				if dom::options[Labelled] then
				    FirstOperandTrans = DivX,
				    FirstTrans = MultX,
				    args(2..args(0))
/*				else
				    FirstOperandTrans = Theta,
				    FirstTrans = (e -> Int(Delta(e, numlib::phi)))
				end_if*/),
		Cycle));
	 elif contains({Count, Random}, dom::standardizeSpecificationOperation) and args(0) = 1 then
	     // Plain unlabelled cycles: for counting and random generation purposes
	     // we prefer to reduce to Int/Theta/Delta which are more often implemented
	     dom::standardExpression
	     (Union(Epsilon(Cycle()),
		    Int(Delta(Prod(Theta(B), Sequence(B)),numlib::phi))));
	 else
	     Cycle(dom::addProduction(B), args(2..args(0)))
	 end_if;
     end_proc,

     Lyndon =
     proc(B: Type::AnyType)
     begin
	 dom::checkLengthOptions(Lyndon(args()));
	 if dom::options[Labelled] then
	     Alias(dom::addProduction(Cycle(args())),
		   z -> Lyndon(op(z))); // Set@op is not fully evaluated when z = null();
	 else
	     Lyndon(dom::addProduction(B), args(2..args(0)))
	 end_if;
     end_proc,

     // standard form for A = Multiset(B <, options>)
     //                   A = Powerset(B <, options>)
     // The two are very similar, so we factorize the code into SetGeneric
     Multiset =
     proc(B)
     begin
	 dom::checkLengthOptions(Multiset(args()));
	 dom::standardizeSpecificationFunctions[SetGeneric](Multiset, 1, args());
     end_proc,

     Powerset =
     proc(B)
     begin
	 dom::checkLengthOptions(Powerset(args()));
	 dom::standardizeSpecificationFunctions[SetGeneric](Powerset, k -> if k mod 2 = 0 then -1 else 1 end, args());
     end_proc,

     SetGeneric =
     proc(constructor, f, B: Type::AnyType)
	 name standardExpressionMultiset;
	 local C, T1, T2;
     begin
	 if dom::options[Labelled] then
	     dom::standardExpression
	     (Alias(SequenceGeneric(B,
				    OperandsTrans = DivX,
				    Trans         = MultX,
				    args(4..args(0))),
		    constructor))
	 elif (args(0)=3 and contains({Random, Count}, dom::standardizeSpecificationOperation)) then
	     // plain unlabelled multi/power set, just for counting
	     dom::standardExpression
	     (Alias(SequenceGeneric(B,
				    OperandsTrans = (B -> Delta(Theta(B), f)),
				    Trans         = Int),
		    constructor));

	 else
	     // other unlabelled multiset
	     constructor(dom::addProduction(B), args(4..args(0)));
	 end_if;
     end_proc,

     // Aliases for backward compatibility and compatibility with combstruct
     hold(Set) =
     proc()
     begin
	 Alias(dom::addProduction(Multiset(args())),
	       z -> hold(Set)(op(z))); // Set@op is not fully evaluated where z = null();
     end_proc,

     hold(Bag) =
     proc()
     begin
	 Alias(dom::addProduction(Multiset(args())),
	       z -> hold(Bag)(op(z))); // Bag@op is not fully evaluated where z = null();
     end_proc,

     // Alias for compatibility with combstruct
     hold(PowerSet) =
     proc()
     begin
	 Alias(dom::addProduction(Powerset(args())),
	       z -> hold(PowerSet)(op(z))); // PowerSet@op is not fully evaluated when z = null();
     end_proc,


     // standard form for A = Predefined(...)
     // does nothing but argument checking
     // Allow for incomplete calls like Predefined(count) ?
     Predefined =
     proc(count, random, unrank)
     begin
	 if args(0) < 1 or args(0) > 3 then
	     error("invalid right-hand side of production: ".
		   expr2text(Predefined(args())));
	 end_if;
	 Predefined(args());
     end_proc,

     // standard form for A = Domain(...)
     // does nothing but argument checking
     Domain =
     proc(mydomain)
     begin
	 if args(0)<>1 or not testtype(mydomain, DOM_DOMAIN)
			  or not mydomain::hasProp(Cat::CombinatorialClass)
	     then 
	     error("invalid right-hand side of production: ".
		   expr2text(Domain(args())));
	 end_if;
	 Domain(mydomain);
     end_proc

    ), prog::bless, dom);

    //////////////////////////////////////////////////////////////////////////
    // The various standard specifications for counting and random generation
    //////////////////////////////////////////////////////////////////////////

    // main standard specification for generator, unrank
    standardSpecification :=
    proc() : dom::typeStandardSpecification
	local specification;
	option remember;
    begin
	specification := dom::specification;
	if testargs() and not testtype(specification, dom::typeSpecification) then
	    error("incorrect specification ".expr2text(specification));
	end_if;
	
	// Syntactic sugar:
        //  - if Z is used but not defined, define it by default as an Atom
	if has(specification,hold(Z)) and not contains(specification,hold(Z)) then
	    specification[hold(Z)] := Atom(hold(Z));
	end_if;
	
	dom::standardizeSpecification(specification);
    end_proc;

    // standard specification for random generation
    standardSpecificationRandom :=
    proc() : dom::typeStandardSpecification
	option remember;
    begin
	dom::standardizeSpecification(dom::standardSpecification(), Random);
    end_proc;

    // standard specification for counting
    /**
	Returns a simplified form of the specification in standard
        form which is equivalent for counting purposes.

	This is done by sorting the operands of Prod and Union (the
        operations + and * on generating series are commutative!),
	erasing the names of the primitive objects and assigning
	them with a default weight when it is not specified:

		Primitive(1, A,1.0,1) => Primitive(1, 1.0, 1)
		Primitive(1, A,q)     => Primitive(1, q)
		Primitive(1, B)       => Primitive(1, dom::options[DefaultWeight])

        Note: random, unrank, generator and others all depend on
	count. So, the standard specification for count needs to be a
	refinement of all the others: that is all non terminal
	introduced for random generation must also appear for count!
	For example, if one decides to standardize Multiset's into
	something else for random purposes, then this needs to be done
	simultaneously for count. That's why we use
	standardSpecificationRandom as starting point below.

    **/
    standardSpecificationCount :=
    proc() : dom::typeStandardSpecification
	option remember;
    begin
	dom::standardizeSpecification(dom::standardSpecificationRandom(), Count);
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // count, random, unrank, generator, ...
    //////////////////////////////////////////////////////////////////////////

    // FIXME: should be just dom::count <> FAIL

    // For this, we need to make a distinction between actual errors
    // (improper syntax, ...) that we should propagate, and "not implemented"
    // errors for which we should just set count to FAIL (on the top level,
    // one might as well want to warn the user).

    count :=
    (
     if traperror(dom::makeCount()) <> 0 then
	 if dom::countFailureReason <> FAIL then
	     FAIL;
	 else
	     dom::count := FAIL;
	     dom::interface := dom::interface minus { hold(count) };
	     lasterror();
	 end_if;
     else
     proc(n: Type::NonNegInt, terminal=dom::options[MainNonTerminal])
	 local i,f;
     begin
	 f := dom::count_functions[terminal];
	 if dom::computed[terminal] < n-1 then
	     // Note that dom::computed[terminal] can be infinity, so
	     // don't remove the test above!
	     // Compute all the previous unknown values
	     // This hack avoids deep recursion
	     for i from dom::computed[terminal]+1 to n-1 do
		 dom::count_functions[terminal](i);
	     end_for;
	 end_if;
	 // Well that's not really true if the following computation actually
	 // fails but this is not an issue.
	 dom::computed[terminal] := n;
	 dom::count_functions[terminal](n);
     end_proc;
     end_if;
    );

    getLabels :=
    proc(n)
	local labels;
    begin
	if testargs() and (not dom::options[Labelled])
			  and domtype(n) = DOM_LIST then
	    error("Cannot specify labels for unlabelled objects.");
	end_if;
	if dom::options[Labelled] then
	    labels := adt::Stack::newExtended([], i -> hold(Mark)(1-i));
	    if domtype(n) = DOM_LIST then
		labels := [n, labels];
		n := nops(n);
	    else
		labels := [[$1..n], labels];
	    end_if;
	else
	    labels := [n]
	end_if;
	[n, labels];
    end_proc;
    
    unrank :=
    (
     if traperror(dom::makeUnrank()) <> 0 then
	 if dom::unrankFailureReason <> FAIL then
	     FAIL;
	 else
	     dom::interface := dom::interface minus { hold(unrank) };
	     dom::unrank := FAIL;
	     lasterror();
	 end_if;
     else
     proc(r: Type::PosInt, 
	  n: Type::Union(Type::NonNegInt, DOM_LIST),
	  terminal=dom::options[MainNonTerminal])
	 local labels;
     begin
	 [n, labels] := dom::getLabels(n); labels := op(labels);
	 // Force the counting, to avoid deep recursion, and ensure that
	 // there exists an element with this size and rank
	 if r>dom::count(n, terminal) then
	     return(FAIL);
	 end_if;
	 dom::unrank_functions[terminal](r, labels) // lists of unmarked and marked labels
     end_proc;
     end_if;
    );

    random :=
    (
     if traperror(dom::makeRandom()) <> 0 then
	 if dom::randomFailureReason <> FAIL then
	     FAIL;
	 else
	     dom::interface := dom::interface minus { hold(random) };
	     dom::random := FAIL;
	     lasterror();
	 end_if;
     else
     proc(n: Type::Union(Type::NonNegInt, DOM_LIST),
	  terminal=dom::options[MainNonTerminal])
	 local labels;
     begin
	 [n, labels] := dom::getLabels(n); labels := op(labels);
	 // Force the counting, to avoid deep recursion, and ensure that
	 // there is at least one element of this size
	 if dom::count(n, terminal) = 0 then
	     return(FAIL);
	 end_if;
	 dom::random_functions[terminal](labels);
     end_proc;
     end_if;
     );

    generatorFromRank :=
    (
     if traperror(dom::makeGeneratorFromRank()) <> 0 then
	 if dom::generatorFromRankFailureReason <> FAIL then
	     FAIL;
	 else
	     dom::interface := dom::interface minus { hold(generatorFromRank) };
	     dom::generatorFromRank := FAIL;
	     lasterror();
	 end_if;
     else
     proc(n: Type::Union(Type::NonNegInt, DOM_LIST),
	  terminal=dom::options[MainNonTerminal])
	 local r, labels, mLabels, count;
	 option escape;
	 // The former version of the following generator called directly unrank
	 // I prefer to directy call unrank_functions at the cost of some
	 // code duplicata... Florent: Find a clean way to avoid this duplicata.
     begin
	 [n, labels] := dom::getLabels(n);
	 if dom::options[Labelled] then
	     mLabels := labels[2];
	 end_if;
	 labels := labels[1];
	 //  force the counting, to avoid deep recursion
	 count := dom::count(n, terminal);
	 if count = 0 then
	     return(combinat::generators::empty);
	 end_if;
	 r := 0;
	 prog::bless
	 (() -> (r:=r+1;
		 if r <= count then dom::unrank_functions[terminal]
		     (r, labels,
		      if dom::options[Labelled] then
			  mLabels::copy();
		      else
			  null()
		      end_if);
		 else FAIL end_if),
	  dom);
     end_proc
     end_if;
    );

    list :=
    if dom::generator <> FAIL then
	dom::listFromGenerator;
    else
	FAIL;
    end_if;

    generator :=
    (
     if traperror(dom::makeGenerator()) <> 0 then
	 if dom::generatorFailureReason = FAIL then
	    dom::interface := dom::interface minus { hold(generator) };
	     dom::generator := FAIL;
	     lasterror();
	 end_if;
	 FAIL;
     else
     proc(n: Type::Union(Type::NonNegInt, DOM_LIST),
	  terminal=dom::options[MainNonTerminal])
	 local labels;
     begin
	 [n, labels] := dom::getLabels(n); labels := op(labels);
	 /* Temporarily disabled since the counting is incorrect
	    Hmm, no, that's probably fine now; but we should check
	    whether dom::count is FAIL or not (2004/09/08)

	 // force the counting, to avoid deep recursion
	 count := dom::count(n, terminal);
	 if count = 0 then
	     return(combinat::generators::empty);
	 end_if;*/
	 dom::generator_functions[terminal](labels);
     end_proc
     end_if;
    );

    recurrenceRelation :=
    proc(NT=dom::options[MainNonTerminal],
	 u = hold(u) : DOM_IDENT,
	 n = hold(n) : DOM_IDENT) : Type::Union(Dom::ExpressionField(),
						DOM_FAIL)
	option remember;
	local spec;
    begin
	if not dom::isContextFree() then
	    error("Recurrence relations can only be computed for context free specifications");
	end_if;
	spec := dom::standardSpecificationCount();
	if not contains(spec, NT) then
	    error("not a non terminal of the specification");
	end_if;
	if op(spec[NT],0) = Alias then
	    NT := op(spec[NT],1);
	end_if;
	if op(spec[NT],0) = Primitive then
	    return(u(n));
	end_if;
	dom::speciftorec(dom::standardSpecificationCount(),NT,u(n));
    end_proc;

    generateCode :=
    proc(file: DOM_STRING)
    begin
	combinat::decomposableObjectsTools::gmp::write(dom, file);
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // Tools for the holonomic product
    //////////////////////////////////////////////////////////////////////////

    coeffRing := dom::options[CoeffRing];
    // The coefficient ring of the generating functions used internally

    z  := hold(z);
    Q  := Dom::Fraction(Dom::DistributedPolynomial([dom::z],dom::coeffRing));
    PQ :=
    proc(vars)
    begin
	polylib::Poly(if vars=[] then [hold(DummyVariable)] else vars end_if,
		      dom::Q);
    end_proc;
    MQ := Dom::DenseMatrix();


// useful functions for getting the list of indices or values of a table t 
listofindices:=
proc(t) begin map([op(t)], op, 1) end_proc;
listofvalues:=
proc(t) begin map([op(t)], op, 2) end_proc;


    //////////////////////////////////////////////////////////////////////////
    // makeValuation: computation of the valuations and degrees of the
    // generating series
    //////////////////////////////////////////////////////////////////////////

    /**
        Compute bounds on the valuation and degree of each non-terminal and
        store them in dom::valuation and dom::degree. That is, for each
        production rule A = ... we have:

	dom::valuation[A] <= size of any element of A <= dom::degree[A]

        When dom::valuation[A]=infinity, we should set dom::degree to -infinity.
    **/
    makeValuation := 
    proc() 
	option remember;
	local v, d, newv, newd, s, p, f, changed, A, expression, aliases, B;
    begin
	s := dom::standardSpecificationCount();
	v := table();  // v, d: temporary values for dom::valuation and dom::degree
	d := table();
	aliases := table(); // production rules which are aliases
	for p in s do
	    A := lhs(p);
	    case op(rhs(p), 0)
	      of Primitive do d[A] := v[A] := op(rhs(p),1); delete(s[A]); break;
	      of Alias   do aliases[A]:=s[A]; delete(s[A]);  break;
	      otherwise 	  v[A]:=infinity; d[A]:=infinity;
	    end_case;
	end_for;
	repeat
	    // Could be optimized by some djikstra style propagation
	    changed:=FALSE;
	    for p in s do
		A := lhs(p);
		expression := rhs(p);
		case op(expression,0)
		  of Union do
		      if nops(expression) = 0 then
			  newv := infinity;
			  newd := -infinity;
		      else
			  newv := min(v[B] $ B in op(expression));
			  newd := max(d[B] $ B in op(expression));
		      end_if;
		      break;
		  of Prod do
		      newv := v[op(expression,1)]+v[op(expression,2)];
		      newd := d[op(expression,1)]+d[op(expression,2)];
		      break;
		  of Theta do
		  of Int do  
			  newv := max(1,v[op(expression,1)]);
			  newd := d[op(expression,1)];
		      break;
		  of DivX do  
		      newv := max(1,v[op(expression,1)])-1;
		      newd := d[op(expression,1)]-1;
		      break;
		  of MultX do
		      newv := max(1,v[op(expression,1)]+1);
		      newd := d[op(expression,1)]+1;
		      break;
		  of Delta do
		      newv := v[op(expression,1)];
		      newd := infinity;
		      break;
		  of Predefined do
		      newv := 0;
		      f := op(expression,1);
		      while f(newv)=0 do newv := newv+1; end_while;
		      newd := infinity;
		      break;
		  of Domain do
		      f := (op(expression,1))::count;
		      if (op(expression,1))::hasProp(Cat::GradedCombinatorialClass) then
			  newv := 0;
			  while f(newv)=0 do newv := newv+1; end_while;
			  newd := infinity;
		      elif f() = 0 then // ungraded empty class
			  newv := infinity;
			  newd := -infinity;
		      else // ungraded non empty class
			  newv := 0;
			  newd := 0;
		      end_if;
		      break;
		  of Cycle do
		  of Lyndon do
		      newv := v[op(expression,1)];
		      newd := d[op(expression,1)];
		      if newd > 0 then newd := infinity; end_if;
		      break;
		  of Powerset do
		  of Multiset do
		      newv := 0;
		      newd := d[op(expression,1)];
		      if newd > 0 then
			  newd := infinity;
		      else
			  newd := 0;
		      end_if;
		      break;
		  otherwise
		      error("unknown constructor: ".expr2text(op(expression,0)))
		end_case;
		if newv<v[A] then v[A]:=newv; changed:=TRUE end_if;
		if newd<d[A] then d[A]:=newd; changed:=TRUE end_if;
	    end_for
	until not changed end_repeat;
	// set valutation and degree for aliases
	for p in aliases do
	    v[lhs(p)] := v[op(rhs(p),1)];
	    d[lhs(p)] := d[op(rhs(p),1)];
	end_for;
	dom::valuation:=v; dom::degree:=d;
	if has(v, infinity) then
	    userinfo(3, "Warning: some non terminal(s) do not derive any object (see <this>::valuation)\nMaybe the specification is incorrect.");
	end_if;
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // makeCount: construction of the counting functions
    //////////////////////////////////////////////////////////////////////////

    /** Specifications for the calls to the constructors:

	A=Union(B,C,D)

	yields calls to:

	makeCountFunctions[Union](B, C, D)
    **/

    makeCount :=
    proc()
	option remember, escape;
	local production, A, expression, constructor, f, i, r;
    begin

	dom::makeValuation();
	for production in dom::standardSpecificationCount() do
	    A := lhs(production);
	    if dom::valuation[A] = infinity then
		f := () -> 0;
		dom::computed[A] := infinity;
	    else
		expression := rhs(production);
	        // Get the constructor for this construction
		constructor := dom::makeCountFunctions[op(expression,0)];
		if domtype(constructor) <> DOM_PROC then
		    dom::notImplemented(Count, " for constructor ".op(expression,0));
		end_if;
		f := constructor(op(expression));
	        //
		for i from 0 to dom::valuation[A]-1 do
		    f(i):=0
		end_for;
		dom::computed[A] := dom::valuation[A]-1;
	    end_if;
	    // simple hack to rename the function
	    dom::count_functions[A] := subsop(f,6=hold(dom::count_functions)[A]);
        end_for;

	if dom::options[Products]=Auto then
	    if dom::isContextFree() and not dom::options[Labelled] then
		dom::options[Products]:=Holonomic
	    else
		dom::options[Products]:=Naive // replace this by Lazy soon!
	    end_if;
	end_if;

	if dom::options[Products]=Holonomic then
	    // ...
	    if testargs() then
		if not dom::isContextFree() then
		    error("Holonomic mode can only be used with context free specifications");
		end_if;
		if dom::options[Labelled] then
		    error("Holonomic and Labelled are currently mutually exclusive");
		end_if;
	    end_if;
	    for production in dom::standardSpecificationCount() do
		if op(production,[2,0]) = Prod then
		    A := lhs(production);
		    r := dom::speciftorec(dom::standardSpecificationCount(),A,hold(u)(hold(n)));
		    dom::count_functions[A] := 
		        subsop(dom::rectoproc(r,hold(u)(hold(n)),dom::count_functions[A]),
			       6=hold(dom::count_functions)[A])
		end_if;
	    end_for;
	elif dom::options[Products]=Lazy then
	    error("Lazy product temporarily desactivated!");
	end_if;    
    end_proc;

    makeCountFunctions :=
    map(table
    (
     Alias = 
     proc(B) // A=Alias(B)
	 option escape;
     begin
	 prog::bless(n -> dom::count_functions[B](n),
		     dom);
     end_proc,

     // A = Primitive(size, weight)
     Primitive =
     proc(size, weight)
	 option escape;
     begin
	 assert(args(0) in {2,3});
	 n -> if n=size then weight else 0 end_if;
     end_proc,

     Union =
     proc() // A = Union(B,C,D, ...)
	 option escape;
	 local operands, o;
     begin
	 operands := args();
	 prog::bless(n -> _plus(dom::count_functions[o](n) $ o in operands),
		     dom);
     end_proc,

     /**
	Build a counting procedure for A=Prod(B,C) 

        Note: it seems from the code of isprod1 and isprod2 that the
	holonomic product should not be used for rules of the form
	Prod(Atom, Epsilon) It seems that the lazy product should not
	be used for rules of the form Prod(B, Epsilon) or Prod(Atom, B)
     **/
     Prod =
    //if dom::options[Products]=Naive then
	/**
	    Using the naive product formulas of generating series:

            Unlabelled case:   a_n = \sum_k b_n c_{n-k}
            Labelled case:     a_n = \sum_k b_n c_{n-k} \binom{n,k}
        **/
     proc(B,C)
	 local vB, vC, tmp;
	 option escape;
     begin
	 vB := dom::valuation[B];
	 vC := dom::valuation[C];
	 if vC=dom::degree[C] then
		// Exchange B and C, to use the specialized version
		// when the generating series of B has just one term
		tmp := B;  B  := C;  C  := tmp;
		tmp := vB; vB := vC; vC := tmp;
	 end_if;
	 prog::bless(
	 if vB = dom::degree[B] then
	     // Specialized version when the generating series of B
	     // has just one term
	     if dom::options[Labelled] then
		 proc(n)
		     option remember;
		 begin
		     combinat::binomialRecursive(n,vB) *
		     dom::count_functions[B](vB  ) *
		     dom::count_functions[C](n-vB);
		 end_proc;
		else
		    proc(n)
			option remember;
		    begin
			dom::count_functions[B](vB  ) *
			dom::count_functions[C](n-vB);
		    end_proc;
		end_if;
	    else
		if dom::options[Labelled] then
		    proc(n)
			local b,k,s,k2,cB,cC; 
			option remember;
		    begin
			cB := dom::count_functions[B];
			cC := dom::count_functions[C];
			k2 := min(dom::degree[B],n-vC);
			b := combinat::binomialRecursive(n,vB); // Rewrite as a sum ?
			s := b*cB(vB)*cC(n-vB);
			for k from vB+1 to k2 do
			    b := b*(n-k+1)/k;
			    s := s+b*cB(k)*cC(n-k);
			end_for;
			s
		    end_proc;
		else
		    proc(n)
			local cB, cC, k;
			option remember;
		    begin
			cB := dom::count_functions[B];
			cC := dom::count_functions[C];
			_plus(cB(k) * cC(n-k)
			      $ k=vB..min(dom::degree[B],n-vC));
		    end_proc;
		end_if
	    end_if,
		     dom);
	end_proc,
/*    elif dom::options[Products]=Lazy then
	if dom::options[Labelled] then 
	    error("Labelled not yet implemented")
	else
	    error("Unlabelled not yet implemented")
	end_if

    elif dom::options[Products]=Holonomic then

    else error("Unknown product type");
    end_if;*/

     Theta =
     proc(B) // A=Theta(B)
	 option escape;
     begin
	 prog::bless(n -> n*dom::count_functions[B](n), dom);
     end_proc,

     Int =
     proc(B) // A=Int(B)
	 option escape;
     begin
	 prog::bless(n -> if n=0 then 0 else (dom::count_functions[B](n))/n; end_if, dom);
     end_proc,

     DivX =
     proc(B) // A=DivX(B)
	 option escape;
     begin
	 prog::bless(n -> dom::count_functions[B](n+1), dom);
     end_proc,

     MultX =
     proc(B) // A=MultX(B)
	 option escape;
     begin
	 prog::bless(n -> if n=0 then 0 else (dom::count_functions[B](n-1)); end_if, dom);
     end_proc,

     Delta =
     proc(B, u) // A = Delta(B,u)
	 local k;
	 option escape;
     begin
	 prog::bless(proc(n: Type::NonNegInt)
			 option remember;
			 local k;
		     begin
			 _plus(u(k)*dom::count_functions[B](n/k)
			       $ k in numlib::divisors(n));
		     end_proc,
		     dom);
     end_proc,

     Predefined =
     proc(count)
     begin
	 count
     end_proc,

     Domain =
     proc(mydomain)
	 option escape;
     begin
	 if mydomain::count = FAIL then
	     dom::notImplemented(Count, " for domain ".expr2text(mydomain));
	 end_if;
	 if mydomain::hasProp(Cat::GradedCombinatorialClass) then
	     // A count_function is supposed to be a normal function because
	     // it needs a remember table and to be renamed by the subsop hack. 
             // Or simply test whether mydomain::count is a DOM_PROC?
	     if type(mydomain::count) = "_fconcat" then
		 proc(n : Type::NonNegInt) // wrap f@g into a function
		 begin
		     mydomain::count(n);
		 end_proc;
	     else
		 mydomain::count
	     end_if;
	 else
	     proc(n : Type::NonNegInt)
	     begin
		 if n = 0 then mydomain::count()
		 else 0 // ungraded combinatorial class
		 end_if;
	     end_proc;
	 end_if;
     end_proc,

     // Currently not used: sets without length restrictions are
     // counted via Int/Delta/Theta, and the code below does not
     // take care properly of length restrictions ...
     SetGeneric =
     proc(B: dom::typeNonTerminal, u, X, nrEmptyStructs)
	 local rec;
	 option escape;
     begin
	 assert(not dom::options[Labelled]);
	 if X <> TRUE then
	     dom::notImplemented(Count, " for Multiset and Powerset with length restrictions");
	 end_if;
	 rec :=
	 prog::bless(
	 proc(n: Type::NonNegInt)
	     option remember;
	     local k, r;
	 begin
	     if n=0 then
		 if X(0) then
		     return(nrEmptyStructs);
		 else
		     return(0);
		 end_if;
	     end_if;
	     _plus(_plus(u(k) * dom::count_functions[B](r/k) * r/k
			 $ k in select(numlib::divisors(r), X))
		   * rec(n-r)
		   $ r=1..n) / n;
	 end_proc, dom);
     end_proc,

     SetGenericParseOptions =
     proc(B, u)
	 option escape;
	 local options, minLength, maxLength, X;
     begin
	 options :=
	 combinat::getOptions
	 ([MinLength=0, MaxLength=infinity, MinPart, MaxPart],
	  table(MinLength=Type::NonNegInt,
		MaxLength=Type::Union(Type::NonNegInt,
				      Type::Singleton(infinity))),
	  2,
	  Error);
	 minLength := options[MinLength];
	 maxLength := options[MaxLength];
	 if contains(options, MinPart) or contains(options, MaxPart) then
	     dom::nonImplemented(Count, " options MinPart or MaxPart");
	 end_if;
	 if minLength = 0 and maxLength = infinity then
	     X := TRUE;
	 else
	     X := n -> bool(minLength <= n and n <= maxLength);
	 end_if;
	 dom::makeCountFunctions[SetGeneric](B, u, X, 1);
     end_proc,

     Powerset = // Powerset(B, <MinLength=...>, <MaxLength=...>)
     proc(B)
     begin
	 assert(not dom::options[Labelled]);
	 dom::makeCountFunctions[SetGenericParseOptions]
	 (B, k -> if k mod 2 = 0 then -1 else 1 end, args(2..args(0)));
     end_proc,

     Multiset = // Multiset(B, <MinLength=...>, <MaxLength=...>)
     proc(B)
     begin
	 assert(not dom::options[Labelled]);
	 dom::makeCountFunctions[SetGenericParseOptions]
	 (B, 1, args(2..args(0)));
     end_proc
    ), prog::bless, dom);

    //////////////////////////////////////////////////////////////////////////
    // Utilities for Holonomic counting
    //////////////////////////////////////////////////////////////////////////

    rectoproc :=
    proc(r,uofn,initialValues)
	// r: recurrence relation
	// u: name of the function in the recurrence relation
	// initiaValues: function which returns the values to initiate the
	//               recurence
	local u,n,R,ORDER,lroots,i,result;
	option escape;
    begin
	u:=op(uofn,0);
	n:=op(uofn,1);
	R:=dom::formatrec(r,u,n);
	ORDER:=nops(R)-2;
	R:=subs(R,n=n-ORDER); 
	dom::LIM:=dom::LIM+ORDER;
	lroots:={};// Temporarily desactivated because of a bug in MuPAD3.0.0
//	lroots:=op(solve(R[nops(R)], [n], Domain=Z_),2);
	dom::LIM:=max(dom::LIM, ORDER, op(lroots));
	R := map(R, fp::unapply, n);
	result :=
        proc(n: Type::NonNegInt) 
	    local nmax,j;
	    option remember;
	begin
	    // Invariant: the indices in the remember table of result
	    // form an interval [0..nmax]
	    // Then, we can compute nmax in constant time instead of
	    // the linear time check:
	    // nmax := max(op(map( op(op(result,5)), op, 1)));
	    // This is not very robust!
	    // Typical bug source: result is called with -1 which gets inserted 
	    // in the remember table
	    nmax:=nops(op(result,5))-1;
	    for j from nmax+1 to n do
		result(j) := - _plus(R[1](j), R[i+1](j)*result(j-ORDER+i-1)$i=1..ORDER) / R[nops(R)](j);
	    end_for;
	end_proc;
	for n from 0 to dom::LIM do
	    result(n) := initialValues(n)
	end_for;
	n->result(n); // keeps result purely lexical without interaction with the big bad exterior
    end_proc;

    formatrec:=
    proc(r,u,n)
	local mi,ma,cr,i;
    begin
	mi:=dom::minindex(r,u,n);
	ma:=dom::maxindex(r,u,n);
	cr:=collect(r,[u(n+i)$i=mi..ma],normal);
	if mi<>0 then
	    cr:=subs(cr,n=n-mi);
	    ma:=ma-mi;
	    dom::LIM:=dom::LIM+mi;
	end_if;
	// I don't know if this test is really useful... 
	if has(map({op(r)},denom),n) then
	    cr:=collect(numer(normal(cr)),[u(n+i)$i=0..ma],normal)
	end_if;
	[subs(cr,[u(n+i)=0$i=0..ma]), coeff(cr,u(n+i),1)$i=0..ma];
    end_proc;
    
    speciftorec := 
    proc(s: dom::typeContextFreeStandardSpecification,
	 N: dom::typeNonTerminal,
	 uofn: DOM_EXPR // an unevaluated function call, like u(n)
	 )
	local algeq,deq;
    begin
	algeq:=dom::speciftoalgeq(s,N);
	deq:=dom::algeqtodiffeq(algeq,N(dom::z));
	dom::diffeqtorec(deq,N(dom::z),uofn)
    end_proc;
    
    algebraicEquation :=
    proc(NT=dom::options[MainNonTerminal]: dom::typeNonTerminal)
	local spec, constructor, size;
    begin
	spec := dom::standardSpecificationCount();
	if not contains(spec, NT) then
	    error("not a non terminal of the specification");
	end_if;
	constructor := op(spec[NT],0);
	if constructor = Primitive then
	    size := op(spec[NT],1);
	    NT - op(spec[NT],1)*(dom::z)^size;
	else
	    dom::speciftoalgeq(dom::standardSpecificationCount(), NT);
	end_if;
    end_proc;
    
    speciftoalgeq := 
    proc(s=dom::standardSpecificationCount(): dom::typeContextFreeStandardSpecification,
	 NT=dom::options[MainNonTerminal]: dom::typeNonTerminal)
	local g,vars,matA,V,nb_lines,vNT,pNT,pol,linear_depend,ens,i,T,tc,P;
    begin
        // computation of groebner basis
	([g,vars]) := dom::groebner_calculus(s);
        // successive normal forms up to linear dependencies 
	matA := [[1]];
	V := [1];  // involved terms 
	nb_lines := 1;
	// If NT is an alias NT=Alias(var) replace NT by the variable var
	if op(s[NT],0)=Alias then
	    vNT := op(s[NT],1);
	else
	    vNT := NT;
	end_if;
	assert(op(s[vNT],0) <> Primitive);
	P := dom::PQ(vars);
	pNT := P(vNT);
	pol := P(1);
	linear_depend := FALSE;
	
	while not linear_depend do
	    nb_lines := nb_lines + 1;
	    pol := groebner::normalf(pol*pNT,g,DegInvLexOrder);
	    userinfo(2,"polynomial is: ",expr(pol));
	    ens := {expr(nthterm(pol,i)) $ i=1..nterms(pol)};
	    if nops(ens union {op(V)}) < nb_lines then
		linear_depend := TRUE;
	    end_if;
	    ens := ens minus {op(V)};
	    for T in ens do
		V := append(V, T);
		matA := map (matA, append, 0);
	    end_for;
	    
	    userinfo(2,"involved terms are: ",V);
	    for T in V do
		tc[T] := 0;
	    end_for;
	    for i from 1 to nterms(pol) do
		tc[expr(nthterm(pol,i))] := nthcoeff(pol,i);
	    end_for;
	    
	    matA := append(matA, [ tc[T] $ T in V ]);
	end_while;
	
  // matrix building and gauss elimination
	for i from 1 to nops(matA) do
	    matA[i] := append(matA[i], NT^(i-1));
	end_for;
	matA:=dom::MQ(matA);
	matA:=linalg::gaussElim(matA);
	
  // take the algebraic equation (numer of the right coefficient)
	i := 1;
	while not has(matA[i,i], NT) do
	    i := i+1;
	end_while;
	numer(matA[i,i]);
    end_proc;

    // Compute the algebraic equations between the generating series
    // corresponding to the non terminals
    algebraicEquationSystem :=
    proc(s = dom::standardSpecificationCount(): dom::typeContextFreeStandardSpecification)
	option remember;
	local v, vars, eq, getWeight, P, z;
    begin
	// Precondition: s is a standard specification which has been
	// simplified by standardSpecificationCount(); hence the only productions
	// are of the form:
	//  A=Alias(B)
	//  A=Primitive(size, weight)
	//  A=Union(B, C, ...)
	//  A=Prod(B, C)
	//
	// Returns a reduced system of polynomial equations between
	// the generating series of the non trivial non terminals
	// (i.e. Union and Prod)

	// Get rid of the aliases
	s:=select(s, not has, Alias);

	// Get rid of the trivial productions A=Primitive()
	// by backsubstitution into the others
	s:=split(s, eq -> op(rhs(eq),0) = Primitive);
	s:=subs(s[2], op(s[1]));

	// Transform the rewriting rules into polynomials
	vars := [ lhs(eq) $ eq in s];
	userinfo(1,"vars are: ",vars);
	P := dom::PQ(vars);
	z := P(dom::z);

	// get the exact weight of a Primitive object
	getWeight :=
	proc(size: Type::NonNegInt)
	begin
	    if args(0) = 3 then P(args(3))
	    else P(args(2));
	    end_if * z^size;
	end_proc;

        // Transform into polynomial equations
	// We need to do it in two steps for safety; otherwise,
	// Union(Primitive(1, 1), Primitive(1, 1)) may be transformed into
	// 2*poly(z) which is not evaluated further.
	// First replace all the operands by elements of P
	s:=map(s, eval@subs,
	       [Primitive    = getWeight,
		v       = P(v) $ v in vars]):
	// And then do the arithmetic:
	s := map(s, eval@subs,
		 [Union   = _plus,
		  Prod    = _mult]);
	s := [rhs(eq)-P(lhs(eq)) $ eq in s];
	userinfo(1,"polynomials are: ",s);
	[s,vars];
    end_proc;

    // Compute a Grbner basis of the ideal generated by the relations
    // given by the specification between the generating series
    // corresponding to the non terminals
    groebner_calculus :=
    proc(s: dom::typeContextFreeStandardSpecification)
	option remember;
	local vars;
    begin
	[s, vars] := dom::algebraicEquationSystem(s);
        // Reduction of the system by Groebner basis
	if nops(s)>0 then
	    s := groebner::gbasis(s, DegInvLexOrder);
	end_if;
	userinfo(1,"groebner basis is: ",map(s,expr));
	[s,vars];
    end_proc;
    
    algeqtodiffeq := 
    proc(algeq, yofx: DOM_EXPR)
	local y,x,pol,g,u,v,deg,deq,Y,d,matA,V,T,i,inits,tc;
    begin
	y := op(yofx,0); 
	x := op(yofx,1);
	pol := poly(algeq, [y]);
	([g,u,v]) := [gcdex(diff(pol,y),pol)];
	if has(expr(g),y) then 
	    return (dom::algeqtodiffeq(expr(normal(pol/g)),y(x),inits)); 
	end_if;
	deg := degree(pol);
	if deg <= 1 then 
	    deq := subs(expr(pol),y=y(x));
	else
	    Y[1] := map(divide(-u * g * diff(pol,x), pol, Rem), normal);
	    for d from 2 to deg-1 do
		Y[d] := map(divide(diff(Y[d-1],x)+diff(Y[d-1],y)*Y[1], pol, Rem), normal);
	    end_for;
	    matA := [[1, 0 $ i=1..deg-1, 1], [0,1, 0 $ i=1..deg-2, y(x)]];
	    V := [y^i $ i=0..deg-1];  // involved terms 
	    for d from 1 to deg-1 do
		for T in V do tc[T] := 0; end_for;
		for i from 1 to nterms(pol) do
		    tc[expr(nthterm(Y[d],i))] := nthcoeff(Y[d],i);
		end_for;
		matA := append(matA, [ tc[T] $ T in V ]);
	    end_for;
    // matrix building and gauss elimination 
	    for i from 3 to nops(matA) do
		matA[i] := append(matA[i], (D@@(i-2))(y)(x));
	    end_for;
	    matA:=dom::MQ(matA);
	    matA:=linalg::gaussElim(matA);
    // take the differential equation (numer of the right coefficient) 
	    i := 1;
	    while not has(matA[i,i], y) do
		i := i+1;
	    end_while;
	    deq := numer(matA[i,i]);
	end_if;
	deq;
    end_proc;
    
    listofvalues:=
    proc(t) begin map([op(t)], op, 2) end_proc;
    
    diffeqtorec := 
    proc(deq, yofx: DOM_EXPR, uofn: DOM_EXPR)
    begin
	deq:=expand(deq);
   // minimal value of n for which the recurrence is valide 
	dom::LIM:=degree(select(deq,not has,op(yofx,0)),op(yofx,1))+1;
	case type(deq)
	  of DOM_INT do of DOM_IDENT do of "function" do
					of "_mult" do break
					of "_plus" do deq:=op(deq); break
					otherwise error("invalid argument")
	end_case;
	_plus(op(map([deq],dom::diffeqtorec2,op(yofx,0..1),op(uofn,0..1))))
    end_proc;
    
/* t is of the form c*x^j*(D@@k)(y)(x),
	   returns the coefficient of x^n in the g.f. of t in terms of u(n)=[x^n] y(x) */
    diffeqtorec2 := 
    proc(t, y, x, u, n) 
	local c,k,j,i;
    begin
	if type(t)="_mult" then ([t,c,k]):=split(t,has,{x,y}) else c:=1 end_if;
	j:=degree(t,[x]);
	t:=t/x^j;
    // now t = 1 or y(x) or D(y)(x) or (D@@k)(y)(x) 
	if not has(t,y) then 0
	else // t = y(x) or D(y)(x) or D(D(y))(x) or ... 
	    if type(t)<>"function" or [op(t)]<>[x] then
		error("unexpected term: ".expr2text(t))
	    end_if;
	    t:=op(t,0); // now y or D(y) or D(D(y)) or ... 
	    k:=0; while type(t)="D" do k:=k+1; t:=op(t) end_while;
	    if t<>y then error("unexpected term: ".expr2text(t)) end_if;
	    c*_mult(n+i-j $ i=1..k)*u(n+k-j)
	end_if
    end_proc;

    // returns the list of all i's such that u(n+i) appears in r
    allIndices :=
    proc(r, u: DOM_IDENT, n: DOM_IDENT)
	local l;
    begin
	l := [];
	eval(subs(r, n=0, u = (i -> (l := append(l,i);0))));
	l;
    end_proc;

    minindex := min @ op @ dom::allIndices;

// returns the largest i such that u(n+i) appears in a recurrence 
    maxindex := max @ op @ dom::allIndices;

///////////////////////// End of Holonomic part /////////////////////


    //////////////////////////////////////////////////////////////////////////
    // makeUnrank: construction of the unranking functions
    //////////////////////////////////////////////////////////////////////////

    // Returns the index of a random element of size n of A
    // Precondition: there should be at least one such element!
    randomIndex :=
    proc(A: dom::typeNonTerminal,
	 n: Type::NonNegInt) : Type::NonNegInt
    begin
	random(dom::count_functions[A](n))()+1;
    end_proc;

    rangeGenerator :=
    proc(range: "_range")
	option escape;
	local i, j, r, goingRight;
    begin
	if dom::options[Boustrophedonic] then
	    i := op(range,1);
	    j := op(range,2);
	    goingRight := TRUE;
	    proc()
	    begin
		// Invariant: all the elements in the range i..j are
		// still to be generated
		if i>j then
		    return(FAIL)
		elif goingRight then
		    goingRight := FALSE;
		    r := i;
		    i := i+1;
		    return(r)
		else
		    goingRight := TRUE;
		    r := j;
		    j := j-1;
		    return(r)
		end_if;
	    end_proc;
	else
	    combinat::generators::fromRange(range)
	end_if;
    end_proc;

    makeUnrank :=
    proc()
	option remember;
	local A, expression, production, constructor;
    begin
	dom::makeValuation();
	if dom::count = FAIL then
	    dom::notImplemented(Unrank, " because count is required");
	end_if;
	for production in dom::standardSpecification() do 
	    A := lhs(production);
	    expression := rhs(production);
	    // Get the constructor for this construction
	    constructor := dom::makeUnrankFunctions[op(expression,0)];
	    if domtype(constructor) <> DOM_PROC then
		dom::notImplemented(Unrank, " for constructor ".op(expression,0));
	    end_if;
	    dom::unrank_functions[A] := constructor(op(expression))
	end_for;
    end_proc;

    // Specifications for make unrank functions:
    //
    // A=Union(B,C,D) yields a call to makeUnrankFunctions[Union](B, C, D)
    //
    // Specifications for unrank functions:
    //
    // If this makes its code simpler or more efficient, an
    // unrank function is allowed to assume that there *does* exist
    // an object of rank r. Reciprocally, unrank should make sure
    // to call them with a valid value for r.

    makeUnrankFunctions :=
    map(table
    (
     Alias =
     proc(B, transform) // A=Alias(B), A=Alias(B,transform)
	 option escape;
     begin
	 prog::bless(if args(0)=1 then
			 () -> dom::unrank_functions[B](args());
		     else
			 () -> transform(dom::unrank_functions[B](args()));
		     end_if,
		     dom)
     end_proc,

     Primitive = // A = Primitive(size, <label>)
     proc(size: Type::NonNegInt, label=null(): Type::AnyType) : DOM_PROC
	 option escape;
     begin
	 if dom::options[Labelled] then
	     (r, labels) -> if nops(labels) = size and r=1 then label(op(labels)) else FAIL; end_if
	 else
	     (r,n) -> if n=size and r=1 then label else FAIL; end_if
	 end_if;
     end_proc,

     DivX =
     proc(B) : DOM_PROC // A = DivX(B)
	 option escape;
     begin
	 if not dom::options[Labelled] then
	     error("DivX does not make sense for unlabelled objects");
	 end_if;
	 prog::bless(proc(r, labels, mLabels)
			 name unrankDivXLabelled;
		     begin
		         // pop one label from the infinite stack mLabels
			 dom::unrank_functions[B](r, [mLabels::pop()].labels, mLabels);
		     end_proc,
		     dom);
     end_proc,

     MultX =
     proc(B) : DOM_PROC // A = MultX(B)
	 option escape;
     begin
	 if not dom::options[Labelled] then
	     error("MultX does not make sense for unlabelled objects");
	 end_if;
	 prog::bless(proc(r, labels, mLabels)
			 name unrankMultXLabelled;
			 local nmLabels, result;
		     begin
			 // There is no MultX object of size 0, so unrankMultXLabelled
			 // should never be called with an empty set of labels
			 assert(nops(labels) <> 0);

			 // Push the first label onto the stack
			 mLabels::push(labels[1]);
			 nmLabels := mLabels::depth();
			 delete(labels[1]);
			 // Invariant: mLabels can only decrease during the
			 // next call
			 result := dom::unrank_functions[B](r, labels, mLabels);
			 assert(mLabels::depth() <= nmLabels);
			 // If the label has not been consumed, and so is still
			 // on the top of the stack, pop it
			 if mLabels::depth() = nmLabels then
			     mLabels::pop();
			 end_if;
			 result;
		     end_proc,
		     dom);
     end_proc,

     Union =
     proc() // A = Union(B,C,D, ...)
	 option escape;
	 local operands;
     begin
	 operands := [args()];
	 prog::bless(proc(r: Type::PosInt)
			 name unrankUnion;
			 local o, r1, n;
		     begin
			 if dom::options[Labelled] then
			     n := nops(args(2));
			 else
			     n := args(2);
			 end_if;
			 for o in operands do
			     r1 := dom::count_functions[o](n);
			     if r <= r1 then
				 return(dom::unrank_functions[o](args()));
			     end_if;
			     r := r - r1;
			 end_for;
			 assert(FALSE);
		     end_proc,
		     dom);
     end_proc,

     Predefined =
     proc(count, random, unrank)
	 option escape;
     begin
	 if args(0)<=2 then
	     dom::notImplemented(Unrank, "for Predefined(count)");
	 end_if;
	 if dom::options[Labelled] then
	     (r, labels) -> unrank(r, labels);
	 else
	     unrank
	 end_if;
     end_proc,

     Domain =
     proc(thedom)
	 option escape;
     begin
	 if thedom::hasProp(Cat::GradedCombinatorialClass) then
	     if dom::options[Labelled] then
		 (r, labels) -> thedom::unrank(r, labels);
	     else
		 thedom::unrank
	     end_if;
	 else // ungraded Combinatorial Class
	     if dom::options[Labelled] then
		 (r, labels) -> (if labels = [] then thedom::unrank(r); else FAIL; end_if);
	     else
		 (r, size) -> (if size = 0 then thedom::unrank(r); else FAIL; end_if);
	     end_if;
	     
	 end_if;
     end_proc,

     Prod =
     proc(B,C)
	 option escape;
	 local vB,vC;
     begin
	 vB:=dom::valuation[B];
	 vC:=dom::valuation[C];
	 prog::bless(
	 if vB=dom::degree[B] then
	     // Florent : This is the case where B appear only in one degree
	     // The -1/+1 in "(r-1 mod binomial(n, vB)) + 1"
	     // is because unrank start from 1 instead of 0 
	     if dom::options[Labelled] then
		 proc(r: Type::PosInt, labels, mLabels)
		     name unrankProdLabelledValB;
		     local nC, rLabel, rNoLab, labelB, labelC, n;
		 begin
		     n  := nops(labels);
		     nC := dom::count_functions[C](n-vB);
                    // For each choice of B and C, there are
		    // binomial(n,vB) choices for relabelling
		    // We unrank them consecutively
		     rLabel := (r-1 mod combinat::binomialRecursive(n, vB)) + 1; // rank of the labelling.
		     rNoLab := (r-1 div combinat::binomialRecursive(n, vB)) + 1;
                     // We unrank the products lexicographically
		     // e.g.: Prod(A,A), Prod(A,B), Prod(B,A), Prod(B,B)
		     // Florent : Now we have to relabel the result according to rLabel
		     [labelB, labelC] := combinat::words::split::unrank(rLabel, labels, vB);
		     Prod(dom::unrank_functions[B]((rNoLab-1 div nC)+1, labelB, mLabels),
			  dom::unrank_functions[C]((rNoLab-1 mod nC)+1, labelC, mLabels));
		 end_proc;
	     else
		 proc(r: Type::PosInt, n: Type::NonNegInt)
		     name unrankProdValB;
		     local nC;
		 begin
		     nC := dom::count_functions[C](n-vB);
		     Prod(dom::unrank_functions[B]((r-1 div nC)+1, vB),
			  dom::unrank_functions[C]((r-1 mod nC)+1, n-vB));
		 end_proc
	     end_if;
	 elif vC=dom::degree[C] then 
	     if dom::options[Labelled] then
		 proc(r: Type::PosInt, labels, mLabels)
		     name unrankProdLabelledValC; 
		     local nC, rLabel, rNoLab, labelB, labelC, n;
		 begin
		     n  := nops(labels);
                    // For each choice of B and C, there are
		    // binomial(n,vB) choices for relabelling
		    // We unrank them consecutively
		     rLabel := (r-1 mod combinat::binomialRecursive(n, n-vC)) + 1; // rank of the labelling.
		     rNoLab := (r-1 div combinat::binomialRecursive(n, n-vC)) + 1; // rank of unlabelled object
                     // We unrank the products lexicographically
		     // e.g.: Prod(A,A), Prod(A,B), Prod(B,A), Prod(B,B)
		     nC := dom::count_functions[C](vC);
		     [labelB, labelC] := combinat::words::split::unrank(rLabel, labels, n-vC);
		     Prod(dom::unrank_functions[B]((rNoLab-1 div nC)+1, labelB, mLabels),
			  dom::unrank_functions[C]((rNoLab-1 mod nC)+1, labelC, mLabels))
		 end_proc;
	     else
		 proc(r: Type::PosInt, n: Type::NonNegInt)
		     name unrankProdValC;
		     local nC;
		 begin
		     nC := dom::count_functions[C](vC);
		     Prod(dom::unrank_functions[B]((r-1 div nC)+1, n-vC),
			  dom::unrank_functions[C]((r-1 mod nC)+1, vC));
		 end_proc
	     end_if;
	 elif dom::options[Labelled] then
	     proc(r: Type::PosInt, labels, mLabels)
		 name unrankProdLabelled;
		 local k, cB, cC, nC, r1, g, rLabel, rNoLab, labelB, labelC, n;
	     begin
		 n  := nops(labels);
		 cB := dom::count_functions[B];
		 cC := dom::count_functions[C];
		 g := dom::rangeGenerator(vB..n-vC);
		 while (k := g()) <> FAIL do
		     nC := cC(n-k);
		     r1 := cB(k)*nC*combinat::binomialRecursive(n, k);
		     if r <= r1 then
			 rLabel := (r-1 mod combinat::binomialRecursive(n, k)) + 1; // rank of the labelling.
			 rNoLab := (r-1 div combinat::binomialRecursive(n, k)) + 1; // rank of unlabelled object
			 [labelB, labelC] := combinat::words::split::unrank(rLabel, labels, k);
			 return(Prod(dom::unrank_functions[B]((rNoLab-1 div nC)+1, labelB, mLabels),
				     dom::unrank_functions[C]((rNoLab-1 mod nC)+1, labelC, mLabels)));
		     end_if;
		     r := r - r1;
		 end_while;
	     end_proc;
	 else
	     proc(r: Type::PosInt, n: Type::NonNegInt)
		 name unrankProd;
		 local k, cB, cC, nC, r1, g;
	     begin
		 cB := dom::count_functions[B];
		 cC := dom::count_functions[C];
		 g := dom::rangeGenerator(vB..n-vC);
		 while (k := g()) <> FAIL do
		     nC := cC(n-k);
		     r1 := cB(k)*nC;
		     if r <= r1 then
			 return(Prod(dom::unrank_functions[B]((r-1 div nC)+1, k),
				     dom::unrank_functions[C]((r-1 mod nC)+1, n-k)));
		     end_if;
		     r := r - r1;
		 end_while;
	     end_proc;
	 end_if,
		     dom);
     end_proc
    ), prog::bless, dom);

    //////////////////////////////////////////////////////////////////////////
    // make Random
    //////////////////////////////////////////////////////////////////////////

    makeRandom :=
    proc()
	option remember;
	local A, expression, production, constructor;
    begin
	if dom::count = FAIL then
	    dom::notImplemented(Random, " because count is required");
	end_if;
	for production in dom::standardSpecificationRandom() do 
	    A := lhs(production);
	    expression := rhs(production);
	    // Get the constructor for this construction
	    constructor := dom::makeRandomFunctions[op(expression,0)];
	    if domtype(constructor) <> DOM_PROC then
		dom::notImplemented(Random, " constructor ".op(expression,0));
	    end_if;
	    dom::random_functions[A] := constructor(A, op(expression))
	end_for;
    end_proc;

    // Specification:
    // A=Union(B,C,D) yields a call to makeRandomFunctions[Union](A, B, C, D)
    makeRandomFunctions :=
    map(table
    (
     Alias = // A=Alias(B), A=Alias(B,transform)
     proc(A: dom::typeNonTerminal, B: dom::typeNonTerminal, transform)
	 option escape;
     begin
	 prog::bless(if args(0)=2 then
			 // could be optimized to avoid the function call
			 () -> dom::random_functions[B](args());
		     else
			 () -> transform(dom::random_functions[B](args()));
		     end_if,
		     dom)
     end_proc,

     Primitive = // A = Primitive(size, <label>)
     proc(A: dom::typeNonTerminal,
	  size: Type::NonNegInt, label=null()) : DOM_PROC
	 option escape;
     begin
	 if dom::options[Labelled] then
	     labels -> if nops(labels) = size then label(op(labels)) else FAIL; end_if
	 else
	     n -> if n=size then label else FAIL; end_if
	 end_if;
     end_proc,

     Union =
     proc(A) : DOM_PROC // A = Union(B,C,D, ...)
	 option escape;
	 local operands;
     begin
	 operands := [args(2..args(0))];
	 prog::bless(proc()
			 name randomUnion;
			 local o, r1, r, n;
		     begin
			 if dom::options[Labelled] then
			     n := nops(args(1));
			 else
			     n := args(1);
			 end_if;
			 r := dom::randomIndex(A,n);
			 for o in operands do
			     r1 := dom::count_functions[o](n);
			     if r <= r1 then
				 return(dom::random_functions[o](args()))
			     end_if;
			     r := r - r1;
			 end_for;
		     end_proc,
		     dom);
     end_proc,

     DivX =
     proc(A, B) : DOM_PROC // A = DivX(B)
	 option escape;
     begin
	 if not dom::options[Labelled] then
	     error("DivX does not make sense for unlabelled objects");
	 end_if;
	 prog::bless(proc(labels, mLabels)
			 name randomDivXLabelled;
		     begin
		         // pop one label from the infinite stack mLabels
			 dom::random_functions[B]([mLabels::pop()].labels, mLabels);
		     end_proc,
		     dom);
     end_proc,

     MultX =
     proc(A, B) : DOM_PROC
	 option escape;
     begin
	 if not dom::options[Labelled] then
	     error("MultX does not make sense for unlabelled objects");
	 end_if;
	 prog::bless(proc(labels, mLabels)
			 name randomMultXLabelled;
			 local nmLabels, result;
		     begin
			 // MultX of 0-sized objects does not make sense
			 assert(nops(labels) > 0);
			 // Push the first label onto the stack
			 mLabels::push(labels[1]);
			 nmLabels := mLabels::depth();
			 delete(labels[1]);
			 // Invariant: mLabels can only decrease during the
			 // next call
			 result := dom::random_functions[B](labels, mLabels);
			 assert(mLabels::depth() <= nmLabels);
			 // If the label has not been consumed, and so is still
			 // on the top of the stack, pop it
			 if mLabels::depth() = nmLabels then
			     mLabels::pop();
			 end_if;
			 result;
		     end_proc,
		     dom);
     end_proc,

     Theta =
     proc(A,B) : DOM_PROC
	 option escape;
     begin
	 prog::bless(n -> dom::random_functions[B](n), dom);
     end_proc,

     Int =
     proc(A, B) : DOM_PROC
	 option escape;
     begin
	 prog::bless(n -> dom::random_functions[B](n), dom);
     end_proc,

     Delta =
     proc(A, B, u) : DOM_PROC
	 option escape;
     begin
	 prog::bless(proc(n: Type::NonNegInt)
			 name randomDelta;
			 local k, r, r1;
		     begin
			 r := dom::randomIndex(A,n);
			 for k in numlib::divisors(n) do
			     r1 := u(k)*dom::count_functions[B](n/k);
			     if r <= r1 then
				 return(dom::random_functions[B](n/k) $ k)
			     end_if;
			     r := r - r1;
			 end_for;
		     end_proc,
		     dom)
     end_proc,

     Predefined =
     proc(A, count, random, unrank)
	 option escape;
     begin
	 if args(0) <= 2 then
	     dom::notImplemented(Random, "for Predefined(count)");
	 end_if;
	 if dom::options[Labelled] then
	     labels -> random(labels);
	 else
	     random
	 end_if;
     end_proc,

     Domain =
     proc(A, thedom)
	 option escape;
     begin
	 if thedom::hasProp(Cat::GradedCombinatorialClass) then
	     if dom::options[Labelled] then
		 labels -> thedom::random(labels);
	     else
		 thedom::random
	     end_if;
	 else // ungraded combinatorial class
	     if dom::options[Labelled] then
		 (labels) -> (if labels = [] then thedom::random(); else FAIL; end_if);
	     else
		 (size) -> (if size = 0 then thedom::random(r); else FAIL; end_if);
	     end_if;
	     
	 end_if;
     end_proc,

     Prod =
     proc(A,B,C) : DOM_PROC
	 option escape;
	 local vB,vC;
     begin
	 vB:=dom::valuation[B];
	 vC:=dom::valuation[C];
	 prog::bless(
         if vB=dom::degree[B] then
	     if dom::options[Labelled] then
		 proc(labels, mLabels)
		     name randomProdLabelledValB;
		     local labelB, labelC;
		 begin
		     [labelB, labelC] := combinat::words::split::random(labels, vB);
		     Prod(dom::random_functions[B](labelB, mLabels),
			  dom::random_functions[C](labelC, mLabels));
		 end_proc;
	     else
		 proc(n: Type::NonNegInt)
		     name randomProdValB;
		 begin
		     Prod(dom::random_functions[B](vB),
			  dom::random_functions[C](n-vB));
		 end_proc
	     end_if;
	 elif vC=dom::degree[C] then
	     if dom::options[Labelled] then
		 proc(labels, mLabels)
		     name randomProdLabelledValC;
		     local labelB, labelC;
		 begin
		     [labelC, labelB] := combinat::words::split::random(labels, vC);
		     Prod(dom::random_functions[B](labelB, mLabels),
			  dom::random_functions[C](labelC, mLabels));
		 end_proc;
	     else
		 proc(n: Type::NonNegInt)
		     name randomProdValC;
		 begin
		     Prod(dom::random_functions[B](n-vC),
			  dom::random_functions[C](vC));
		 end_proc
	     end_if;
	 elif dom::options[Labelled] then
	     proc(labels, mLabels)
		 name randomProdLabelled;
		 local r, k, cB, cC, r1, g, labelB, labelC, n;
	     begin
		 n  := nops(labels);
		 r := dom::randomIndex(A,n);
		 cB := dom::count_functions[B];
		 cC := dom::count_functions[C];
		 g := dom::rangeGenerator(vB..n-vC);
		 while (k := g()) <> FAIL do
		     r1 := cB(k) * cC(n-k) * combinat::binomialRecursive(n,k);
		     if r <= r1 then
			 [labelB, labelC] := combinat::words::split::random(labels, k);
			 return(Prod(dom::random_functions[B](labelB, mLabels),
				     dom::random_functions[C](labelC, mLabels)));
		     end_if;
		     r := r - r1;
		 end_while;
	     end_proc;
	 else
	     proc(n: Type::NonNegInt)
		 name randomProd;
		 local r, k, cB, cC, r1, g;
	     begin
		 r := dom::randomIndex(A,n);
		 cB := dom::count_functions[B];
		 cC := dom::count_functions[C];
		 g := dom::rangeGenerator(vB..n-vC);
		 while (k := g()) <> FAIL do
		     r1 := cB(k) * cC(n-k);
		     if r <= r1 then
			 return(Prod(dom::random_functions[B](k),
				     dom::random_functions[C](n-k)));
		     end_if;
		     r := r - r1;
		 end_while;
	     end_proc;
	end_if,
		     dom)
     end_proc
    ), prog::bless, dom);

    //////////////////////////////////////////////////////////////////////////
    // make Generator
    //////////////////////////////////////////////////////////////////////////

    makeGenerator :=
    proc()
	option remember;
	local A, expression, production, constructor;
    begin
	dom::makeValuation();
	for production in dom::standardSpecification() do
	    A := lhs(production);
	    expression := rhs(production);
	    // Get the constructor for this construction
	    constructor := dom::makeGeneratorFunctions[op(expression,0)];
	    if domtype(constructor) <> DOM_PROC then
		dom::notImplemented(Generator, " for constructor ".op(expression,0));
	    end_if;
	    dom::generator_functions[A] := constructor(op(expression))
	end_for;
    end_proc;

    // Specification:
    // A=Union(B,C,D) yields a call to makeGeneratorFunctions[Union](B, C, D)
    makeGeneratorFunctions :=
    map(table
    (
     Alias =
     proc(B, transform) // A=Alias(B), A=Alias(B,transform)
	 option escape;
     begin
	 prog::bless(if args(0)=1 then
			 // could be optimized to avoid the function call
			 n -> dom::generator_functions[B](args());
		     else
			 n -> generators::map(dom::generator_functions[B](args()), transform);
		     end_if,
		     dom)
     end_proc,

     Primitive =
     proc(size: Type::NonNegInt, label=null(): Type::AnyType) : DOM_PROC
	 option escape;
     begin
	 if dom::options[Labelled] then
	     labels -> if nops(labels) = size then
			   generators::singleton(label(op(labels)))
		       else
			   FAIL;
		       end_if
	 else
	     n -> if n=size then
		      generators::singleton(label)
		  else
		      FAIL;
		  end_if
	 end_if;
     end_proc,

     Union =
     proc() : DOM_PROC // A = Union(B,C,D, ...)
	 option escape;
	 local operands;
     begin
	 operands := [args()];
	 prog::bless(proc()
			 name generatorUnionLabelled;
			 option escape;
			 local o, n, theContext;
		     begin
			 theContext := args();
			 if dom::options[Labelled] then
			     n := nops(args(1));
			 else
			     n := args(1);
			 end_if;
			 // Should we reset the stack at each step ?
			 generators::pipe
			 (generators::select
			  (generators::fromList(operands),
			   if dom::count <> FAIL then
			       prog::bless(o -> bool(dom::count_functions[o](n)>0), dom);
			   else
			       TRUE
			   end_if),
			  prog::bless(o -> dom::generator_functions[o](theContext), dom));
		     end_proc,
		     dom);
     end_proc,

     // Invariants:

     // At any given time, the generators are organized in a tree,
     // whose structure corresponds to the previously generated object.
     // The last generator in infix order is called active. It will be
     // the bottom of the
     //
     // The stack is shared by all the generators in the tree
     // Its state correspond to the active generator. Its state
     // is the same as the corresponding stack when unranking
     // the last piece of this object in infix order.

     DivX =
     proc(B) : DOM_PROC // A = DivX(B)
	 option escape;
     begin
	 if not dom::options[Labelled] then
	     error("DivX does not make sense for unlabelled objects");
	 end_if;
	 prog::bless(proc(labels, mLabels)
			 name generatorDivXLabelled;
			 local l, g;
			 option escape;
		     begin
			 l := mLabels::pop();
			 userinfo(0, "poping ".l);
		         // pop one label from the infinite stack mLabels
			 // and prepare to put it back after the last
			 // element has been generated
			 g := dom::generator_functions[B]([l].labels, mLabels);
			 proc()
			     local x;
			 begin
			     // Beware: the input may be an (empty) expression
			     // sequence!
			     if (x := g()) = FAIL then
				 userinfo(0, "pushing back ".l);
				 mLabels::push(l);
			     end_if;
			     x;
			 end_proc;
		     end_proc,
		     dom);
     end_proc,

     MultX =
     proc(B) : DOM_PROC // A = MultX(B)
	 option escape;
     begin
	 if not dom::options[Labelled] then
	     error("MultX does not make sense for unlabelled objects");
	 end_if;
	 prog::bless(proc(labels, mLabels)
			 name generatorMultXLabelled;
			 option escape;
			 local nmLabels, g;
		     begin
			 // There is no 0-sized MultX object
			 if nops(labels) = 0 then
			     return(generators::empty);
			 end_if;
			 // Push the first label onto the stack
			 mLabels::push(labels[1]);
			 userinfo(0, "pushing ".labels[1]);
			 nmLabels := mLabels::depth();
			 delete(labels[1]);
			 // Invariant: mLabels can only decrease during the
			 // lower level calls
			 g := dom::generator_functions[B](labels, mLabels);
			 proc()
			     local x;
			 begin
			     if (x := g()) = FAIL then
				 assert(mLabels::depth() <= nmLabels);
				 // If the label has not been consumed, and so is still
				 // on the top of the stack, pop it
				 if mLabels::depth() = nmLabels then
				     userinfo(0, "poping back ".mLabels::top());
				     mLabels::pop();
				 end_if;
			     end_if;
			     x;
			 end_proc;
		     end_proc,
		     dom);
     end_proc,

     Domain = // A = Domain(combinat::partitions)
     proc(thedom)
	 option escape;
     begin
	 if thedom::hasProp(Cat::GradedCombinatorialClass) then
	     if dom::options[Labelled] then
		 labels -> thedom::generator(labels);
	     else
		 thedom::generator
	     end_if;
	 else // ungraded combinatorial class
	     if dom::options[Labelled] then
		 (labels) -> (if labels = [] 
			      then thedom::generator(); 
			      else combinat::generators::empty; 
			      end_if);
	     else
		 (size) -> (if size = 0 
			    then thedom::generator(); 
			    else combinat::generators::empty; 
			    end_if);
	     end_if;
	     
	 end_if;
     end_proc,

     Prod = // A = Prod(B, C)
     proc(B, C) : DOM_PROC
	 option escape;
	 local vB, vC, DOM;
     begin
	 DOM := dom;
	 vB:=dom::valuation[B];
	 vC:=dom::valuation[C];
	 proc(labels)
	     name generatorProd;
	     option escape;
	     local cB, cC, n, theContext;
	 begin
	     theContext := args(2..args(0));
	     if DOM::options[Labelled] then
		 n := nops(labels);
	     else
		 n := labels;
	     end_if;
	     // We only use count to cut some useless branches
	     if DOM::count <> FAIL then
		 cB := DOM::count_functions[B];
		 cC := DOM::count_functions[C];
	     else
		 cB := 1;
		 cC := 1;
	     end_if;
	     generators::pipe
	     (DOM::rangeGenerator(vB..n-vC),
	      proc(k)
		  option escape;
	      begin
		  if cB(k) = 0 or cC(n-k) = 0 then
		      return(generators::empty);
		  end_if;
		  if DOM::options[Labelled] then
		      generators::pipe
		      (combinat::words::split::generator(labels, k),
		       proc(labelB, labelC)
			   option escape;
		       begin
			   generators::pipe
			   (DOM::generator_functions[B](labelB, theContext),
			    proc()
				option escape;
				local oB;
			    begin
				// Beware: the input may be an (empty) expression
				// sequence!
				oB := args();
				generators::map
				(DOM::generator_functions[C](labelC, theContext),
				 proc()
				 begin
				     Prod(oB, args());
				 end_proc)
			    end_proc);
		       end_proc @ op);
		  else
		      // Here, we are essentially doing the same
		      // as generators::cartesian(gB, gC)
		      // We do it by hand instead because:
		      //  - there is an easy way to rebuild the generators
		      //  - generators::cartesian also does not yet
		      //    deal well with (empty) expression sequences,
		      //    whereas we get a lot of them, say with Epsilon()'s
		      generators::pipe
		      (DOM::generator_functions[B](k, theContext),
		       proc()
			   option escape;
			   local oB;
		       begin
			   oB := args();
			   generators::map
			   (DOM::generator_functions[C](n-k, theContext),
			    proc()
			    begin
				Prod(oB, args());
			    end_proc)
		       end_proc);
		  end_if;
	      end_proc);
	 end_proc;
     end_proc,

     SetGeneric = // A = Multiset(B, <MinLength=...>, <MaxLength=>, <MinPart=...>, ....)
     proc(B: dom::typeNonTerminal, prodName: DOM_IDENT, skip: Type::NonNegInt)
	 option escape;
	 local vB, vC, DOM, partitionOptions;
     begin
	 partitionOptions := args(4..args(0));
	 assert(not dom::options[Labelled]);
	 DOM := dom;
	 proc(n: Type::NonNegInt)
	     name generatorMultisetUnlabelled;
	     option escape;
	 begin
	     // Build a generator for all the multisets of size n
	     generators::pipe
	     (combinat::partitions::generator(n,
					      MinPart=DOM::valuation[B],
					      MaxPart=DOM::degree[B],
					      partitionOptions),
	      proc(p: combinat::partitions)
		  option escape;
	      begin
		  // Build a generator for all the multisets with
		  // objects of respective size p[1], p[2], ...
		  generators::map
		  (// pretty print
		   generators::cartesian
		   (// Build a generator for all the multisets with
		    // nk[1] objects of size nk[2]
		    generators::multiset
		    (DOM::generator_functions[B](nk[1]),
		     nk[2], skip)
		    $ nk in combinat::words::evaluationSparse(p)),
		   list -> prodName(op(map(list, op))));
	      end_proc);
	 end_proc;
     end_proc,

     Multiset = // A = Multiset(B, <MinLength=...>, <MaxLength=>, <MinPart=...>, ....)
     proc(B)
     begin
	 dom::makeGeneratorFunctions[SetGeneric](B, Multiset, 0, args(2..args(0)));
     end_proc,

     Powerset = // A = Multiset(B, <MinLength=...>, <MaxLength=>, <MinPart=...>, ....)
     proc(B)
     begin
	 dom::makeGeneratorFunctions[SetGeneric](B, Powerset, 1, args(2..args(0)));
     end_proc,

     Lyndon =
     proc(B) : DOM_PROC
	 option escape;
	 local multisetGenerator;
     begin
	 multisetGenerator := dom::makeGeneratorFunctions[Multiset](args());
	 proc(n: Type::NonNegInt) : combinat::generators
	 begin
	     generators::map
	     (generators::pipe(multisetGenerator(n),
			       combinat::lyndonWords::fromContent::generator@DOM_LIST@op),
	      Lyndon@op);
	 end_proc;
     end_proc,

     Cycle =
     proc(B) : DOM_PROC
	 option escape;
	 local multisetGenerator;
     begin
	 multisetGenerator := dom::makeGeneratorFunctions[Multiset](args());
	 proc(n: Type::NonNegInt) : combinat::generators
	 begin
	     if n=0 then
		 return(generators::singleton(Cycle()));
	     end_if;
	     generators::map
	     (generators::pipe(multisetGenerator(n),
			       combinat::necklaces::fromContent::generator@DOM_LIST@op),
	      Cycle@op);
	 end_proc;
     end_proc

    ), prog::bless, dom);




    //////////////////////////////////////////////////////////////////////////
    // experiment with make generatorFromRank
    //////////////////////////////////////////////////////////////////////////

    makeGeneratorFromRank :=
    proc()
	option remember;
	local A, expression, production, constructor;
    begin
	dom::makeValuation();
	for production in dom::standardSpecification() do
	    A := lhs(production);
	    expression := rhs(production);
	    // Get the constructor for this construction
	    constructor := dom::makeGeneratorFromRankFunctions[op(expression,0)];
	    if domtype(constructor) <> DOM_PROC then
		dom::notImplemented(GeneratorFromRank, " for constructor ".op(expression,0));
	    end_if;
	    dom::generatorFromRank_functions[A] := constructor(op(expression))
	end_for;
    end_proc;

    // Specification:
    // A=Union(B,C,D) yields a call to makeGeneratorFunctions[Union](B, C, D)
    // each function is of the form (r, n, wantGenerator) -> ... or
    // each function is of the form (r, labels, mLabels, wantGenerator) -> ...
    makeGeneratorFromRankFunctions :=
    map(table
    (
     Alias =
     proc(B, transform) // A=Alias(B), A=Alias(B,transform)
	 option escape;
     begin
	 prog::bless(if args(0)=1 then
			 // could be optimized to avoid the function call
			 () -> dom::generatorFromRank_functions[B](args())
		     else
			 () -> generators::map(dom::generatorFromRank_functions[B](args()), transform);
		     end_if,
		     dom)
     end_proc,

     Primitive = // TODO: honor the wantGenerator argument
     proc(size: Type::NonNegInt, label=null(): Type::AnyType) : DOM_PROC
	 option escape;
     begin
	 if dom::options[Labelled] then
	     labels -> if nops(labels) = size then
			   generators::singleton(label(op(labels)))
		       else
			   FAIL;
		       end_if
	 else
	     n -> if n=size then
		      generators::singleton(label)
		  else
		      FAIL;
		  end_if
	 end_if;
     end_proc,

     // Everything needs to be done for all the other constructors

     Union =
     proc() : DOM_PROC // A = Union(B,C,D, ...)
	 option escape;
	 local operands;
     begin
	 operands := [args()];
	 prog::bless(proc()
			 name generatorUnionLabelled;
			 option escape;
			 local o, n, theContext;
		     begin
			 theContext := args();
			 if dom::options[Labelled] then
			     n := nops(args(1));
			 else
			     n := args(1);
			 end_if;
			 // Should we reset the stack at each step ?
			 generators::pipe
			 (generators::select
			  (generators::fromList(operands),
			   if dom::count <> FAIL then
			       prog::bless(o -> bool(dom::count_functions[o](n)>0), dom);
			   else
			       TRUE
			   end_if),
			  prog::bless(o -> dom::generator_functions[o](theContext), dom));
		     end_proc,
		     dom);
     end_proc,

     // Invariants:

     // At any given time, the generators are organized in a tree,
     // whose structure corresponds to the previously generated object.
     // The last generator in infix order is called active. It will be
     // the bottom of the stack.
     //
     // The stack is shared by all the generators in the tree
     // Its state correspond to the active generator. Its state
     // is the same as the corresponding stack when unranking
     // the last piece of this object in infix order.

     DivX =
     proc(B) : DOM_PROC // A = DivX(B)
	 option escape;
     begin
	 if not dom::options[Labelled] then
	     error("DivX does not make sense for unlabelled objects");
	 end_if;
	 prog::bless(proc(labels, mLabels)
			 name generatorDivXLabelled;
			 local l, g;
			 option escape;
		     begin
			 l := mLabels::pop();
			 userinfo(0, "poping ".l);
		         // pop one label from the infinite stack mLabels
			 // and prepare to put it back after the last
			 // element has been generated
			 g := dom::generator_functions[B]([l].labels, mLabels);
			 proc()
			     local x;
			 begin
			     // Beware: the input may be an (empty) expression
			     // sequence!
			     if (x := g()) = FAIL then
				 userinfo(0, "pushing back ".l);
				 mLabels::push(l);
			     end_if;
			     x;
			 end_proc;
		     end_proc,
		     dom);
     end_proc,

     MultX =
     proc(B) : DOM_PROC // A = MultX(B)
	 option escape;
     begin
	 if not dom::options[Labelled] then
	     error("MultX does not make sense for unlabelled objects");
	 end_if;
	 prog::bless(proc(labels, mLabels)
			 name generatorMultXLabelled;
			 option escape;
			 local nmLabels, g;
		     begin
			 // There is no 0-sized MultX object
			 if nops(labels) = 0 then
			     return(generators::empty);
			 end_if;
			 // Push the first label onto the stack
			 mLabels::push(labels[1]);
			 userinfo(0, "pushing ".labels[1]);
			 nmLabels := mLabels::depth();
			 delete(labels[1]);
			 // Invariant: mLabels can only decrease during the
			 // lower level calls
			 g := dom::generator_functions[B](labels, mLabels);
			 proc()
			     local x;
			 begin
			     if (x := g()) = FAIL then
				 assert(mLabels::depth() <= nmLabels);
				 // If the label has not been consumed, and so is still
				 // on the top of the stack, pop it
				 if mLabels::depth() = nmLabels then
				     userinfo(0, "poping back ".mLabels::top());
				     mLabels::pop();
				 end_if;
			     end_if;
			     x;
			 end_proc;
		     end_proc,
		     dom);
     end_proc,

     Domain = // A = Domain(combinat::partitions)
     proc(thedom)
	 option escape;
     begin
	 if thedom::hasProp(Cat::GradedCombinatorialClass) then
	     if dom::options[Labelled] then
		 labels -> thedom::generator(labels);
	     else
		 thedom::generator
	     end_if;
	 else // ungraded combinatorial class
	     if dom::options[Labelled] then
		 (labels) -> (if labels = [] 
			      then thedom::generator(); 
			      else combinat::generators::empty; 
			      end_if);
	     else
		 (size) -> (if size = 0 
			    then thedom::generator(); 
			    else combinat::generators::empty; 
			    end_if);
	     end_if;
	     
	 end_if;
     end_proc,

     Prod = // A = Prod(B, C)
     proc(B, C) : DOM_PROC
	 option escape;
	 local vB, vC, DOM;
     begin
	 DOM := dom;
	 vB:=dom::valuation[B];
	 vC:=dom::valuation[C];
	 proc(labels)
	     name generatorProd;
	     option escape;
	     local cB, cC, n, theContext;
	 begin
	     theContext := args(2..args(0));
	     if DOM::options[Labelled] then
		 n := nops(labels);
	     else
		 n := labels;
	     end_if;
	     // We only use count to cut some useless branches
	     if DOM::count <> FAIL then
		 cB := DOM::count_functions[B];
		 cC := DOM::count_functions[C];
	     else
		 cB := 1;
		 cC := 1;
	     end_if;
	     generators::pipe
	     (DOM::rangeGenerator(vB..n-vC),
	      proc(k)
		  option escape;
	      begin
		  if cB(k) = 0 or cC(n-k) = 0 then
		      return(generators::empty);
		  end_if;
		  if DOM::options[Labelled] then
		      generators::pipe
		      (combinat::words::split::generator(labels, k),
		       proc(labelB, labelC)
			   option escape;
		       begin
			   generators::pipe
			   (DOM::generator_functions[B](labelB, theContext),
			    proc()
				option escape;
				local oB;
			    begin
				// Beware: the input may be an (empty) expression
				// sequence!
				oB := args();
				generators::map
				(DOM::generator_functions[C](labelC, theContext),
				 proc()
				 begin
				     Prod(oB, args());
				 end_proc)
			    end_proc);
		       end_proc @ op);
		  else
		      // Here, we are essentially doing the same
		      // as generators::cartesian(gB, gC)
		      // We do it by hand instead because:
		      //  - in our case, there is an easy way to rebuild the generators
		      //    this avoids expanding the two generators into lists as does
		      //    generators::cartesian currently (as of 26/01/2004)
		      //  - generators::cartesian also does not yet
		      //    deal well with (empty) expression sequences,
		      //     whereas we get a lot of them, say with Epsilon()'s
		      generators::pipe
		      (DOM::generator_functions[B](k, theContext),
		       proc()
			   option escape;
			   local oB;
		       begin
			   oB := args();
			   generators::map
			   (DOM::generator_functions[C](n-k, theContext),
			    proc()
			    begin
				Prod(oB, args());
			    end_proc)
		       end_proc);
		  end_if;
	      end_proc);
	 end_proc;
     end_proc,

     SetGeneric = // A = Multiset(B, <MinLength=...>, <MaxLength=>, <MinPart=...>, ....)
     proc(B: dom::typeNonTerminal, prodName: DOM_IDENT, skip: Type::NonNegInt)
	 option escape;
	 local vB, vC, DOM, partitionOptions;
     begin
	 partitionOptions := args(4..args(0));
	 assert(not dom::options[Labelled]);
	 DOM := dom;
	 proc(n: Type::NonNegInt)
	     name generatorMultisetUnlabelled;
	     option escape;
	 begin
	     // Build a generator for all the multisets of size n
	     generators::pipe
	     (combinat::partitions::generator(n,
					      MinPart=DOM::valuation[B],
					      MaxPart=DOM::degree[B],
					      partitionOptions),
	      proc(p: combinat::partitions)
		  option escape;
	      begin
		  // Build a generator for all the multisets with
		  // objects of respective size p[1], p[2], ...
		  generators::map
		  (// pretty print
		   generators::cartesian
		   (// Build a generator for all the multisets with
		    // nk[1] objects of size nk[2]
		    generators::multiset
		    (DOM::generator_functions[B](nk[1]),
		     nk[2], skip)
		    $ nk in combinat::words::evaluationSparse(p)),
		   list -> prodName(op(map(list, op))));
	      end_proc);
	 end_proc;
     end_proc,

     Multiset = // A = Multiset(B, <MinLength=...>, <MaxLength=>, <MinPart=...>, ....)
     proc(B)
     begin
	 dom::makeGeneratorFunctions[SetGeneric](B, Multiset, 0, args(2..args(0)));
     end_proc,

     Powerset = // A = Multiset(B, <MinLength=...>, <MaxLength=>, <MinPart=...>, ....)
     proc(B)
     begin
	 dom::makeGeneratorFunctions[SetGeneric](B, Powerset, 1, args(2..args(0)));
     end_proc,

     Lyndon =
     proc(B)
	 option escape;
	 local multisetGenerator;
     begin
	 multisetGenerator := dom::makeGeneratorFunctions[Multiset](args());
	 proc(n)
	     option escape;
	 begin
	     generators::pipe(multisetGenerator(n),
			      combinat::lyndonWords::fromContent::generator@DOM_LIST@op);
	 end_proc;
     end_proc,

     Cycle =
     proc(B)
	 option escape;
	 local multisetGenerator;
     begin
	 multisetGenerator := dom::makeGeneratorFunctions[Multiset](args());
	 proc(n)
	     option escape;
	 begin
	     generators::pipe(multisetGenerator(n),
			      combinat::necklaces::fromContent::generator@DOM_LIST@op);
	 end_proc;
     end_proc

    ), prog::bless, dom);





//////////////////////////////////////////////////////////////////////////
// initialisations
//////////////////////////////////////////////////////////////////////////

begin
    if nops(Specification)  =0 then
	error("Empty specification!");
    end_if;
    // analysis of options, ...
    Options := combinat::getOptions
    (
     table(Labelled = FALSE,	// TRUE
	   Target       = MuPAD,	// (C,GMP), (C,DPE), Module?, 
			         	// (Java,Text), (Java, Applet)?,
				 	// (Java, Graphics)
	   Products     = Auto,		// Auto, Naive, Lazy, Holonomic
	   Float	= FALSE,
	   CoeffRing    = Dom::Rational,
	   DefaultWeight= 1,
	   File         = "cstemp",	//
	   MainNonTerminal = op(Specification,[1,1]),
	   MainTerminal = FAIL,
	   Boustrophedonic = TRUE,
	   RepeatedSquaring = TRUE
	  ),
     table(Labelled = DOM_BOOL,
	   Float    = DOM_BOOL,
	   CoeffRing = DOM_DOMAIN,
	   DefaultWeight = Type::AnyType,
	   Target   = Type::Union(DOM_IDENT, Type::SequenceOf(DOM_IDENT)),
	   Products = Type::Union(Type::Singleton(Auto),
				  Type::Singleton(Naive),
				  Type::Singleton(Holonomic),
				  Type::Singleton(Lazy)),
	   MainNonTerminal = DOM_IDENT,
	   MainTerminal = Type::Union(DOM_IDENT, DOM_FAIL),
           File     = DOM_STRING,
	   Boustrophedonic = DOM_BOOL,
	   RepeatedSquaring = DOM_BOOL
	  ),
     1,
     Error
    );
    if Options[MainTerminal] <> FAIL then
	// warnDeprecated?
	Options[MainNonTerminal] := Options[MainTerminal];
    end_if;

    if Options[Float] then
	Options[DefaultWeight] := 1.0, 1;
    end_if;

    // Make sure Specification is a table of production rules
    Specification := table( op(Specification) );
    if not contains(Specification, Options[MainNonTerminal]) then
	error("The specification does not contain the main terminal ".
	      expr2text(Options[MainNonTerminal]));
    end_if;
end_domain:

domain combinat::decomposableObjectsTools
begin
end_domain:

unalias(Print):
alias(Print=combinat::decomposableObjectsTools::gmp::printFd):

domain combinat::decomposableObjectsTools::gmp

    interface:= { hold(write) };

    write :=
    proc(lib, file: DOM_STRING)
    begin
	dom::fd:=fopen(Text, file, Write);
	dom::header();
	dom::initArithmeticLibrary();
	dom::initMacros();
	dom::makeDeclarations(lib);
	dom::makeMain(lib);
	dom::makeCount(lib);
	dom::makeRandom(lib);
	null();
    end_proc;

    fd := 0;

    printFd :=
    proc()
    begin
	if args(0)>0 then // MuPAD 2.0.0 does not accept fprint(Unquoted, fd)
	    fprint(Unquoted, dom::fd, args());
	end_if;
    end_proc;

    header :=
    proc()
    begin
	Print("/*
  This C source file was automatically generated by the library
  combinat::decomposableObjects of MuPAD-Combinat.
  See http://mupad-combinat.sourceforge.net/ for details

  Authors:
	Sbastien Cellier - LaPCS, Lyon I
	Sylvie Corteel - LRI
	Alain Denise - LRI
	Isabelle Dutour - LaBRI
	Nicolas Thiry - LaPCS, Lyon I
	Paul Zimmermann - INRIA
*/
");
    end_proc;

    initMacros :=
    proc()
    begin
	Print("/* Some macros */
/* if <> 0, then the next construction Prod(blahblah) should be printed out as blahblah */
int printConstructionFlag;
/* if <> 0, then the next non empty output should be prefixed by the separator ", " */
int printSeparatorFlag;

#ifndef WORDS
/* same as printf, but if the string is non empty, prefix it with the separator ", " */
void PRINT(char *s) {
    if (s[0]!='\\0' && printSeparatorFlag) printf(\", \");
    printSeparatorFlag=0;
    printf(s);
}
#define PRINT_OPEN(s) ppflag = printConstructionFlag; printConstructionFlag=1; if (ppflag) { PRINT(s); printf(\"(\"); }
#define PRINT_CLOSE() if (ppflag) printf(\")\");
#else
#define PRINT_OPEN(s) {}
#define PRINT_CLOSE() {}
#define PRINT printf
#endif

");
    end_proc;

    initArithmeticLibrary :=
    proc()
    begin
    Print("/* GMP initialization */
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <time.h>
#include <gmp.h>
#include <assert.h>

int ISINITIALIZED(mpz_t p) { return(mpz_cmp_si(p,-1)); }
int INIT(mpz_t p) { return(mpz_cmp_si(p,0)); }

unsigned long graine;
time_t t;
gmp_randstate_t state;

mpz_t tmp, s;

");
    end_proc;

    makeDeclarations :=
    proc(lib)
	local production, A;
    begin
	for production in lib::standardSpecificationCount() do
	    A:=lhs(production);
	    Print("/* Non terminal ".A." */");
	    Print("mpz_t *".A.";");
	    Print("void count".A."();");
	    Print("void random".A."();\n");
	end_for;
	/*if cs::HOLONOMIC and cs::testcontextfreespecif(sys) then
	    for A in select(sys,cs::isProd[HOLONOMIC],sys) do
		if type(cs::lequalcount[op(A,1)])="_index" then
		    Print("void initrec".op(A,1)."();");
		end_if;
	    end_for;
	end_if;*/
	Print("\n");
    end_proc;

    makeMain :=
    proc(lib)
	local A, production;
    begin
	Print("/* Main */
int main(int argc,char *argv[]) {
  int i,n,k;
  if (argc<=1) {
    printf(\"Usage: a.out n [k seed] to generate k objects of size n\\n\");
    exit(1);
  }
  n=atoi(argv[1]);
  k=(argc>=3) ? atoi(argv[2]) : 1;");
	Print("  printSeparatorFlag=0;");
	Print("  printConstructionFlag=1;");
	Print();
	Print("  /* Initialization of the random generator */");
	Print("  gmp_randinit_default(state);");
	Print("  time(&t);");
	Print("  gmp_randseed_ui (state,t);");
	Print();
	Print("  mpz_init(tmp);");
	Print("  mpz_init(s);");
	Print();
        Print("  /* Create an array A[0..n] for each non-terminal A which is not an alias */");
	lib::makeValuation();
	for production in lib::standardSpecificationCount() do
	    A:=lhs(production);
	    if op(rhs(production),0) <> Alias then
		Print("  ".A."=(mpz_t *)malloc((n+1)*sizeof(mpz_t));");
		if lib::valuation[A] = infinity then
		    Print("  for (i=0;i<=n;i++) mpz_init_set_si(".A."[i],0);");
		else
		    Print("  for (i=0;i<".lib::valuation[A]." && i<=n;i++) mpz_init_set_si(".A."[i],0);");
		    Print("  for (;i<=n;i++)  mpz_init_set_si(".A."[i],-1);");
		end_if;
	    end_if;
	end_for;
        Print("  /* Aliase (if A=Alias(B), A becomes a pointer to B) */");
	for production in lib::standardSpecificationCount() do
	    if op(rhs(production),0) = Alias then
		Print("   ".lhs(production)."=".op(rhs(production),1).";");
	    end_if;
	end_for;
	Print("");

/*	if lib::options[Product]=Holonomic then
	    /// ??????
	    for A in select(sys,cs::isProd[HOLONOMIC],sys) do
		if type(cs::lequalcount[op(A,1)])="_index" then
		    Print("   initrec".op(A,1)."(n);");
		end_if;
	    end_for;
	end_if;*/
	A:=lib::options[MainNonTerminal];
	Print("  count".A."(n);");
	Print("  if (k==0) {");
	Print("    mpz_out_str (stdout, 10, ".A."[n]);");
	Print("    puts(\"\");");
	Print("  } else {");
	Print("    for (i=1;i<=k;i++) {");
	Print("      if (mpz_sgn(".A."[n])==0) {");
	Print("        printf(\"FAIL\\n\");");
	Print("      } else {");
	Print("        random".A."(n); putchar('\\n');");
	Print("      }");
	Print("    }");
	Print("  }");
	Print("  mpz_clear(tmp);");
	Print("  mpz_clear(s);");
	Print("  exit(0);");
	Print("}");
    end_proc;

    makeCount :=
    proc(lib)
	local p, A, expression, constructor;
    begin
	if lib::options[Labelled] then 
	    error("Labelled not yet implemented") 
	end_if;
	lib::makeValuation();
	lib::makeCount();
	for p in lib::standardSpecificationCount() do
	    A   := lhs(p);
	    expression := rhs(p);
	    Print("/* ".expr2text(A=expression)." */");
	    Print("void count".A."(int n) {");
	    if lib::valuation[A] = infinity then
		Print("  return;");
	    else
		Print("  int k;");
		Print("  if (ISINITIALIZED(".A."[n])) return;");
		constructor := dom::makeCountFunctions[op(expression,0)];
		if domtype(constructor) <> DOM_PROC then
		    error("C code generation counting not implemented for ".op(expression,0));
		end_if;
		constructor(lib, A, op(expression));
	    end_if;
	    Print("}");
	end_for;
    end_proc;

    makeCountFunctions :=
    table
    (
     Alias = // A=Alias(B)
     proc(lib, A: DOM_IDENT, B: DOM_IDENT)
     begin
	 Print("    count".B."(n);");
     end_proc,

     Primitive = // Epsilon
     proc(lib, A: DOM_IDENT, size: Type::NonNegInt)
     begin
	 Print("  if (n==".size.")");
	 Print("    mpz_set_si(".A."[n],1);");
	 Print("  else");
	 Print("    mpz_set_si(".A."[n],0);");
     end_proc,

     Union =
     proc(lib: DOM_DOMAIN, A: DOM_IDENT) // A=Union(B,C,...)
	 local i,k;
     begin
	 Print("  count".args(3)."(n);");
	 Print("  mpz_set(".A."[n],".args(3)."[n]);");
	 for i from 4 to args(0) do
	     Print("  count".args(i)."(n);");
	     Print("  mpz_add(".A."[n],".A."[n],".args(i)."[n]);");
	 end_for
     end_proc,

     Prod = // A=Prod(B,C)
     proc(lib: DOM_DOMAIN, A: DOM_IDENT, B: DOM_IDENT, C: DOM_IDENT) 
	 local k0, s0;
     begin
	 if lib::valuation[C] = lib::degree[C] then
             // counting is symetrical, so we may exchange B and C
	     B; B:=C; C:=%2
	 end_if;
	 k0 := lib::valuation[B]; 
	 s0 := (if k0=0 then "" else "-".k0 end_if);
	 Print("  count".B."(".k0."); count".C."(n".s0.");");
	 Print("  mpz_mul(".A."[n],".B."[".k0."],".C."[n".s0."]);");

	 if k0 = lib::degree[B] then return(); end_if;

	 s0:=(if lib::valuation[C]=0 then "" else "-".lib::valuation[C] end_if);
	 Print("  for (k=".(k0+1).";k<=n".s0.";k++) {");
	 Print("    count".B."(".k."); count".C."(n-k);");
	 Print("    mpz_mul(tmp,".B."[k],".C."[n-k]);");
	 Print("    mpz_add(".A."[n],".A."[n],tmp);");
	 Print("  }");
     end_proc,

     Int = // B=Int(A) ==> B(n) = A(n)/n 
     proc(lib, A: DOM_IDENT, B: DOM_IDENT) 
     begin
	 Print("  count".B."(n);");
	 Print("  mpz_fdiv_q_ui(".A."[n],".B."[n],n);");
     end_proc,

     Theta = // B=Theta(A) ==> B(n) = A(n)*n
     proc(lib, A: DOM_IDENT, B: DOM_IDENT) 
     begin
	 Print("  count".B."(n);");
	 Print("  mpz_mul_ui(".A."[n],".B."[n],n);");
     end_proc,

     Delta = // A = Delta(B,u) 
     proc(lib, A: DOM_IDENT, B: DOM_IDENT, u) 
     begin
	 if u=1 then
	     Print("  count".B."(n);");
	     Print("  mpz_set(".A."[n],".B."[n]);");
	     Print("  for (k=2;k<=n;k++)");
	     Print("    if (n%k==0) {");
	     Print("      count".B."(n/k);");
	     Print("      mpz_add(".A."[n],".A."[n],".B."[n/k]);");
	     Print("    }");
	 else 
	     error("Not yet implemented for u(n)<>1 (it would be easy to do it for u(n) = (-1)^n, e.g. Powersets)") ;
	 end_if
     end_proc,

     Predefined = // A = Predefined(count, unrank, random)
     proc()
     begin
	 error("Predefined(...) not implemented in C");
     end_proc,

     Domain = // A = Predefined(count, unrank, random)
     proc()
     begin
	 error("Domain(...) not implemented in C");
     end_proc
     );

    makeRandom :=
    proc(lib)
	local p, A, expression, constructor;
    begin
	if lib::options[Labelled] then 
	    ("Labelled not yet implemented") 
	end_if;
	for p in lib::standardSpecificationRandom() do //???
	    A   := lhs(p);
	    expression := rhs(p);
	    Print("/* ".expr2text(A=expression)." */");
	    Print("void random".A."(int n) {");
	    Print("  int k, c, ppflag;");
	    constructor := dom::makeRandomFunctions[op(expression,0)];
	    assert(domtype(constructor) = DOM_PROC);
	    constructor(lib, A, op(expression));
	    Print("}");
	end_for;
    end_proc;

    // Assumptions:
    // - the function randomA(n) is never called when there is no
    //   element derived from A of size n

    makeRandomFunctions :=
    table
    (
     Alias = // A=Alias(B), A=Alias(B, Bla), A=Alias(B, op), A=Alias(B, Bla@op)
     proc(lib, A, B, transform)
	 local flag;
     begin
	 if args(0) > 3 then
	     flag := FALSE; // whether to disable the printing of the next construction
	     if transform=op then
		 flag := TRUE;
	     else
		 if type(transform)="_fconcat" and nops(transform)=2 and op(transform,2) = op then
		     transform := op(transform,1);
		     flag := TRUE;
		 end_if;
		 if testtype(transform, DOM_IDENT) then
		     Print("  PRINT_OPEN(\"".expr2text(transform)."\");");
		 else
		     error("Don't known how to generate C code for transformation ".expr2text(transform));
		 end_if;
	     end_if;
	     if flag then Print("  printConstructionFlag=0;"); end;
	 end_if;
	 Print("  random".B."(n);");
	 if args(0)>3 and testtype(transform, DOM_IDENT) then
	     Print("  PRINT_CLOSE();");
	 end_if;
     end_proc,

     Primitive = // A=Primitive(size, B)
     proc(lib, A, size, label=null())
     begin
	 Print("  assert(n==".size.");");
	 Print("  PRINT(\"".expr2text(label)."\");")
     end_proc,

     Union = // A=Union(...)
     proc(lib, A)
	 local i,k;
     begin
	 Print("  count".A."(n);");
	 Print("  mpz_urandomm(s,state,".A."[n]);");
	 for i from 3 to args(0) do
	     Print("  count".args(i)."(n);");
	     Print("  mpz_add(s,s,".args(i)."[n]);");
	     Print("  if (mpz_cmp(s,".A."[n])>=0) {"); 
	     Print("     random".args(i)."(n);");
	     Print("     return;");
	     Print("  }");
	 end_for;
     end_proc,

     Int = // A=Int(B)
     proc(lib, A, B) 
     begin
	 Print("  random".B."(n);");
     end_proc,

     Theta =
     proc(lib, A, B)
     begin
	 Print("  random".B."(n);");
     end_proc,

     Delta =
     proc(lib, A, B, u) 
     begin
	 if u<>1  then
	     error("not yet implemented for u(n)<>1") 
	 end_if;
	 Print("  count".A."(n);");
	 Print("  mpz_urandomm(s,state,".A."[n]);");
	 Print("  for (k=1;k<=n;k++)");
	 Print("    if (n%k==0) {");
	 Print("      count".B."(n/k);");
	 Print("      mpz_add(s,s,".B."[n/k]);");
	 Print("      if (mpz_cmp(s,".A."[n])>=0) {");
	 Print("        random".B."(n/k);");
	 Print("        if (k>1) printf(\"$%u\",k);");
	 Print("        return;");
	 Print("      }");
	 Print("    }");
	 Print("  printf(\"Error: you should not get there\\n\");");
	 Print("  exit(1);");
     end_proc,

     Prod =
     proc(lib, A, B, C) 
	 local k0,k1,s0;
     begin
	 k0:=lib::valuation[B];  k1:=lib::valuation[C];
	 if k0 = infinity or k1 = infinity then
	     Print("    printf(\"Error: you should not get there\\n\");");
	     Print("    exit(1);");
	     return();
	 end_if;
	 if k0=lib::degree[B] then
	     s0:=(if k0=0 then "" else "-".k0 end_if);
	     Print("   PRINT_OPEN(\"Prod\"); random".B."(".k0."); printSeparatorFlag=1; random".C."(n".s0."); PRINT_CLOSE();")
	 elif k1=lib::degree[C] then
	     s0:=(if k1=0 then "" else "-".k1 end_if);
	     Print("   PRINT_OPEN(\"Prod\"); random".B."(n".s0."); printSeparatorFlag=1; random".C."(".k1."); PRINT_CLOSE();")
	 else
	     Print("  count".A."(n);
   mpz_urandomm(s,state,".A."[n]);
   for (k=".lib::valuation[B].";;k++) {
      count".B."(".k."); count".C."(n-k);
      mpz_mul(tmp,".B."[".k."],".C."[n-k]);
      mpz_add(s,s,tmp);
      if (mpz_cmp(s,".A."[n])>=0)  break;");
      k0:=k0-lib::valuation[C];
      //boustrophedon
      s0:=(if k0=0 then "" elif k0>0 then "+".k0 else "-".(-k0) end_if);
      Print("      k=n".s0."-k;
      count".B."(".k."); count".C."(n-k);
      mpz_mul(tmp,".B."[".k."],".C."[n-k]);
      mpz_add(s,s,tmp);
      if (mpz_cmp(s,".A."[n])>=0)  break;
      k=n".s0."-k;
   }
   PRINT_OPEN(\"Prod\"); 
   random".B."(k); 
   printSeparatorFlag=1; 
   random".C."(n-k); 
   PRINT_CLOSE();");
   end_if
end_proc
     );

begin
end_domain:

