/*****************************************************************************
  $Id: graphs.mu,v 1.1.1.3 2005/01/26 13:42:24 nthiery Exp $
  File:		lib/COMBINAT/graphs.mu
  Authors:	Florent Hivert <florent.hivert@univ-mlv.fr>
                Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Purpose:	Source code for the domain combinat::graphs
  Created:	Tue Apr 13 11:03:49 2004
  License:	LGPL
  Project:	MuPAD-Combinat, http://mupad-combinat.sf.net
*****************************************************************************/

/** Changes:
    canonical => dom::straighten
    permute   => relabel
    extract   => sublabel
    valences  => inDegree
**/

alias(Nauty=combinat::Nauty): // Argl; can't be done inside the else ...

%if combinat::Nauty = FAIL then
    userinfo(1, "Nauty dynamic module not available; disabling combinat::graphs"):
    combinat::graphs := FAIL;
    combinat::interface := combinat::interface minus
       { hold(Nau).hold(ty), hold(graphs) };
else

domain combinat::graphs
    inherits Dom::BaseDomain;
    category Cat::GradedCombinatorialClass;

    info_str := "combinat::graphs -- the combinatorial class of unlabelled simple graphs";

    interfaceAdd :=
	       {hold(empty), hold(new), hold(expr), hold(size),
		hold(getVertexNumber), hold(getVertices),
		hold(complement), hold(revert),
		hold(getEdgeNumber), hold(getEdges),
		hold(addEdge), hold(removeEdge), hold(addEdges), hold(removeEdges),
		hold(straighten), hold(relabel), hold(sublabel), hold(LandoTilde),
		hold(outDegree), hold(outDegrees), hold(directSum),
		hold(removeIsolatedVertices)
//		, hold(contraction), hold(contractEdges), 
	       };

    new            :=
    proc(n) : Type::Union(dom, Type::ListOf(dom))
    begin
	if domtype(n)=DOM_INT then
	    dom::list(args());
	else
	    Nauty::new(args())
	end_if;
    end_proc;
    convert        := Nauty::new;
    expr           := Nauty::expr;
    size           := Nauty::size;
    straighten     := Nauty::canonise;
    getEdgeNumber  := Nauty::getEdgeNumber;
    complement     := Nauty::complement;
    revert         := Nauty::converse;
    addEdge        := Nauty::addEdge;
    removeEdge     := Nauty::removeEdge;
    relabel        := Nauty::relabel;
    sublabel       := Nauty::sublabel;
    directSum      := Nauty::directSum;
    outDegree      := Nauty::outDegree;
    LandoTilde     := Nauty::LandoTilde;

    // For compatibility with Graph
    getVertexNumber:= dom::size;

    typeEdge := Type::ListProduct(Type::NonNegInt, Type::NonNegInt);

    empty := dom::new([]);

    countSimple := // By Polya Enumeration
    proc(n: Type::NonNegInt, extParEdge = 1)
	local Sn, Gn, Zn;
    begin
	Sn := Dom::SymmetricGroup(n);
	Gn := Dom::PermutationGroupOnSets(Sn, 2);
	Zn := Gn::cycleIndicator();
	Zn([1,extParEdge]);
    end_proc;

    list :=
    funcenv(
    proc(n: Type::NonNegInt) : Type::ListOf(dom)
	local options, gengOptions, m, M;
    begin
	if n>32 then
	    error("Only graphs with at most 32 nodes");
	end_if;
	options := combinat::getOptions(dom::list, 1, Error);
	gengOptions := ["geng", expr2text(n), "-u"];
	if options[Connected] then
	    gengOptions := append(gengOptions, "-c");
	elif options[Biconnected] then
	    gengOptions := append(gengOptions, "-C");
	end_if;
	if options[C3Free] then
	    gengOptions := append(gengOptions, "-t");
	end_if;
	if options[C4Free] then
	    gengOptions := append(gengOptions, "-f");
	end_if;
	if options[Bipartite] then
	    gengOptions := append(gengOptions, "-b");
	end_if;
	if options[Quiet] then
	    gengOptions := append(gengOptions, "-q");
	end_if;
	if options[Straighten] then
	    gengOptions := append(gengOptions, "-l");
	end_if;
	if contains(options, MinDegree) then
	    gengOptions := append(gengOptions, "-d".expr2text(options[minDegree]));
	end_if;
	if contains(options, MaxDegree) then
	    gengOptions := append(gengOptions, "-D".expr2text(options[MaxDegree]));
	end_if;
	m := expr2text(options[MinEdges]);
	M := expr2text(options[MaxEdges]);
	if contains(options, MaxEdges) then
	    if options[MinEdges]>options[MaxEdges] then
		error("Incompatible MinEdges:MaxEdges bounds: ".m.":".M)
	    end_if;
	    gengOptions := append(gengOptions, m.":".M);
	elif options[MinEdges] > 0 then
	    gengOptions := append(gengOptions, m.":0");
	end_if;

	if n=0 then // Workaround a crash in geng
	    [dom::empty]
	else
	    userinfo(1, _concat("Calling geng with options",
				(" ",expr2text(opt)) $ opt in gengOptions));
//	    [op(Nauty::list(gengOptions))]; // Nauty returns a set
	    Nauty::list(gengOptions); // Nauty returns a list now
	end_if;
    end_proc,
        FAIL,
	table("options"     = [Connected    = FALSE,
			       Biconnected  = FALSE,
			       C3Free       = FALSE,
			       C4Free       = FALSE,
			       Bipartite    = FALSE,
			       Straighten   = TRUE,
			       Quiet        = TRUE,
			       MinEdges     = 0,
			       MaxEdges,
			       GengOptions  = ()
			      ],
	      "optionTypes" = [Connected    = DOM_BOOL,
			       Biconnected  = DOM_BOOL,
			       C3Free       = DOM_BOOL,
			       C4Free       = DOM_BOOL,
			       Bipartite    = DOM_BOOL,
			       Straighten   = DOM_BOOL,
			       Quiet        = DOM_BOOL,
			       MinEdges     = Type::NonNegInt,
			       MaxEdges     = Type::NonNegInt,
			       MinDegree    = Type::NonNegInt,
			       MaxDegree    = Type::NonNegInt,
			       GengOptions  = Type::SequenceOf(DOM_STRING)]));

    //generator :=

    getVertices :=
    proc(g: dom) : Type::ListOf(Type::PosInt)
    begin
	[$ 1..dom::size(g)];
    end_proc;

    addEdges :=
    proc(g: dom, edges: Type::ListOf(dom::typeEdge)) : dom
	local e;
    begin
	if edges = [] then return(g) end_if;
	for e in edges do g := dom::addEdge(g, e); end_for;
    end_proc;

    removeEdges :=
    proc(g: dom, edges: Type::ListOf(dom::typeEdge)): dom
	local e;
    begin
	if edges = [] then return(g) end_if;
	for e in edges do g := dom::removeEdge(g, e); end_for
    end_proc;

    getEdges :=
    proc(g: dom) : Type::ListOf(dom::typeEdge)
	local i, j;
    begin
	g := expr(g);
	[ [i,j] $ j in g[i] $ i=1..nops(g) ]
    end_proc;

    outDegrees :=
    proc(g: dom) : Type::ListOf(Type::NonNegInt)
	local i;
    begin
	[dom::outDegree(g, i) $ i=1..dom::size(g)];
    end_proc;

    
    contraction := 
    proc(g: dom, edge : dom::typeEdge): dom
	local edgs, a, n, res; 
    begin
	n    := dom::size(g);
	edge := sort(edge);
	edgs := dom::getEdges(g);
	edgs := map(edgs, subs, edge[2]=edge[1]);
	edgs := map(edgs, subs, {a=a-1 $ a=edge[2]..n});
	edgs := subs(edgs, [edge[1] $2] = null());
	res    := dom([[] $ n-1]);
	dom::addEdges(res, edgs);
    end_proc;

    contractEdges := 
    proc(g: dom, edges: Type::ListOf(dom::typeEdge)): dom
	local e, n, a;
    begin
	n    := dom::size(g);
	while(edges <> []) do
	    e := edges[1];
	    delete(edges[1]);
	    g := dom::contraction(g, e);
	    edges := subs(edges, {a=a-1 $ a=e[2]..n});
	    n := n-1;
	end_while;
	g
    end_proc;
    
    removeIsolatedVertices := 
    proc(gr: dom)
	local neighb;
    begin
	neighb := dom::expr(gr);
	dom::sublabel(gr, select([$1..dom::size(gr)], z -> (neighb[z] <> [])));
    end_proc;
    
    isSimplifiable := 
    proc(gr: dom)
	local adj, n, a, b, linkedToa, linkedTob, common, linkedTobButa, grEdges;
    begin
	if contains(dom::outDegrees(gr), 1) <> 0 then return(TRUE); end_if;
	n       := dom::size(gr);
	grEdges := dom::expr(gr);
	for a from 0 to n-1 do
	    adj := grEdges[a];
	    for b in adj do
		linkedToa     := {op(grEdges[a])} minus {b};
		linkedTob     := {op(grEdges[b])} minus {a};
		linkedTobButa := linkedTob minus linkedToa;
		common        := linkedTob intersect linkedToa;
		if nops(linkedTobButa) < nops(common) then
		    return(TRUE);
		end_if;
	    end_for;
	end_for;
	FALSE;
    end_proc;

end_domain:

end_if: // availability of combinat::Nauty
