/*****************************************************************************
  $Id: independentSetsOfVectorSpace.mu,v 1.1.1.1 2005/01/26 13:42:24 nthiery Exp $
  File:		lib/COMBINAT/independentSetsOfVectorSpace.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Purpose:	Source code for the domain combinat::independentSetsOfVectorSpace
  Created:	Thu Dec 16 17:45:06 2004
  License:	LGPL
  Project:	MuPAD-Combinat, http://mupad-combinat.sf.net
*****************************************************************************/

export(combinat, Alias, generators):

domain combinat::independentSetsOfVectorSpace
    category Cat::CombinatorialClass;
    inherits Dom::BaseDomain;
    axiom Ax::systemRep;

    info_str := "The combinatorial class of the independent sets of a vector space";

    // Returns a generator for all the linear combinations of the
    // column vectors of B, starting from 0. Note that if the column
    // are not linearly independent, there will be repetitions in the
    // result

    linearSpanGenerator :=
    proc(B // a matrix
	) : generators
	 local M, K, k;
	 option escape;
    begin
	M := B::dom;
	K := M::coeffRing;
	assert(K::hasProp(Dom::IntegerMod));
	// FIXME: only works for Dom::IntegerMod(...)
	K::generator := () -> generators::map
			     (generators::fromRange(0..K::size-1),
			      K);
	k := M::matdim(B)[2];
	if k=0 then
	    return(generators::singleton(M(M::matdim(B)[1],1)))
	end_if;
	generators::map
	(generators::cartesianPower(K::generator(), k),
	 vector -> B*M(vector)):
    end_proc;

    generator :=
    proc(K: Cat::Field, // Ground field
	 n: Type::NonNegInt, // Dimension of the vector space
	 k: Type::NonNegInt, // size of the independent sets
	 returnColumnEchelonForm = FALSE
	 )
	option escape;
	local M, i;
    begin
	M := Dom::Matrix(K):
	if k=0 then
	    return(
	    generators::singleton
	    (if returnColumnEchelonForm then
		 [M(n,0), M(n,0), {}]
	     else
		 M(n,0)
	     end_if))
	end_if;

	generators::pipe
	(dom::generator(K, n, k-1, TRUE),
	 prog::bless(
	 proc(II: Type::ListProduct(M, M, DOM_SET)) : generators
	     option escape;
	     local g, B, B2, charRows, nonCharRows, x;
	 begin
	     // Induction hypothesis:
	     // II is an independent set
	     // B is the basis of <II> in column echelon form (zeroes above the
	     // "diagonal")
	     // B union e_i, i not in charRows is a basis of K^n
	     [II, B, charRows] := II;
	     assert(nops(charRows)=k-1);
	     // Build a basis of a suplementary of <II>
	     B2 := M(n, n-(k-1));
	     // the sort is just there for coherence among MuPAD versions
	     nonCharRows := sort([op({$1..n} minus charRows)]);
	     for i from 1 to n-k+1 do
		 B2[nonCharRows[i], i] := 1;
	     end_for;

	     // A generator for the non zero elements of <B2>
	     g := dom::linearSpanGenerator(B2);
	     x := g();
	     assert(iszero(x));
	     generators::map
	     (generators::cartesian
	      (// A generator for the elements of <II>
	       dom::linearSpanGenerator(B),
	       g),
	      proc(l) // : [ element of <II>, element of <B2> ]
		  local v, newB;
	      begin
		  // Build the vector of coordinates given respectively
		  // by l[1] and l[2] on the charRows/non charRows indexes
		  v := l[1]+l[2];
		  if not returnColumnEchelonForm then
		      return(II.v);
		  end_if;
		  newB := M::gaussElim(B.v, ColumnElimination);
		  charRows := newB[4];
		  newB := newB[1];
		  return([II.v, newB, charRows]);
	      end_proc)
	 end_proc, dom));
    end_proc;

end_domain:
