/*****************************************************************************
  $Id: integerVectorsModPermutationGroup.mu,v 1.1.1.5 2005/01/26 13:42:24 nthiery Exp $
  File:		lib/COMBINAT/integerVectors.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Created:	8/09/2003
  License:	LGPL
*****************************************************************************/

domain combinat::integerVectorsModPermutationGroup(G)
    inherits combinat::integerVectors;
    category Cat::CombinatorialClass;
    axiom    Ax::systemRep;

    info_str := "Library 'combinat::integerVectorsMod': utilities for integer vectors";

    interface := combinat::integerVectors::interface
	       minus {
		      hold(shuffle), hold(shufflings), hold(split), hold(splittings),
		      hold(fromNonWeakDescentsComposition),
		      hold(fromStandard), hold(first), hold(Next)
		     }
	       union {
		      hold(straighten),
		      hold(isStraight),
		      hold(orbit),
		      hold(orbitSet),
		      hold(orbitSize),
		      hold(automorphismGroupSize)
		     };

    group := G;
    dimen := dom::group::card;

    // domtype is inherited

    isA :=
    proc(v : Type::AnyType, n=FAIL: Type::NonNegInt)
    begin
	_lazy_and(dom::isAnIntegerVector(v),
		  dom::isStraight(v));
    end_proc;

    isAnIntegerVector :=
    proc(v : Type::AnyType, n=FAIL: Type::NonNegInt)
    begin
	_lazy_and(testtype(v, Type::ListOf(Type::NonNegInt)),
		  nops(v) = dom::dimen,
		  if n<>FAIL then _plus(op(v))=n else TRUE; end);
    end_proc;

    typeIntegerVector := Type::Predicate(dom::isAnIntegerVector);

    // integer vector with decreasing parts
    typeShapeList := Type::Predicate(dom::isAnIntegerVector);

    isStraight := id@G::isCanonic;

    straighten := id@G::canonic;

    orbit := id@G::orbit;

    orbitSet := id@G::orbitSet;

    orbitSize := id@G::orbitSize;

    automorphismGroupSize := id@G::automorphismGroupSize;

    _less := combinat::words::lexLess;

    setRanker := combinat::rankers::fromList(dom::group::set);
    setRank := dom::setRanker[1];
    setUnrank := dom::setRanker[2];

    fromTable :=
    proc(t: Type::TableOf(Type::AnyType, Type::PosInt)) : dom
	local l, eq;
    begin
	l := [0 $ dom::dimen];
	for eq in t do
	    l[dom::setRank(op(eq,1))] := op(eq,2);
	end_for;
	l;
    end_proc;

    list :=
    proc(n: Type::Union(Type::NonNegInt,dom::typeShapeList))
    begin
	sort(map(if domtype(n)=DOM_INT then
		     G::canonicCompositions(n)
		 else
		     G::canonicListsByShape(n)
		 end_if,
		 G::canonic),
	     dom::_less
	    );
    end_proc;

    generatingSeries := dom::generatingSeries_Polya;

/*--
    generatingSeries_Polya(z:DOM_IDENT)

    Computes the generating series by a variation on Polya
    substitution in the cycle indicator polynomial. Here the
    weights are [1,z,z^2,...]. One avoids to deal with an infinite
    number of weights using the following equality:

    \sum_{k=1}^\infty (z^k)^i = 1/(1-z^i)

    Ref: B. Sturmfels, "Algorithms in Invariant Theory", p 72 ?.
	 A. Kerber, "Algebraic Combinatoric Via Finite Group Action" 2.2 p 71
--*/

    generatingSeries_Polya :=
    proc(z = hold(z) : DOM_IDENT)
    option remember;
    local p, eqns, var;
    begin
	userinfo(3, "generating Series: computation using Polya Substitution");
	eqns:=[];
	/* Hypothesis: The variables of P are of the form z[1], z[2], ... */
	p:=dom::group::cycleIndicatorPoly();
	for var in op(p,2) do
	    eqns:=eqns.[var = 1/(1-z^op(var,2))];
	end_for;
	subs(expr(p), eqns);
    end_proc;

    count :=
    proc(d: Type::Union(Type::NonNegInt, dom::typeShapeList))
	option remember;
    begin
	if domtype(d)=DOM_INT then
	    coeff(series(dom::generatingSeries(hold(z)),hold(z),d+1),d);
	else
	    dom::group::numberOfCanonicListsByShape(d);
	end_if
    end_proc;

end_domain:

