/*****************************************************************************
  $Id: integerVectorsWeighted.mu,v 1.1.1.6 2005/01/26 13:42:24 nthiery Exp $
  File:		lib/COMBINAT/integerVectorsWeighted.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Created:	16/02/2001
  License:	LGPL
*****************************************************************************/

domain combinat::integerVectorsWeighted
    inherits Dom::BaseDomain;
    category Cat::CombinatorialClass;
    axiom    Ax::systemRep;

    info_str := "Library 'combinat::integerVectorsWeighted': utilities for weighted integer vectors";

    interfaceAdd :=
        { hold(generatingSeries) };

    domtype := DOM_LIST;

    isA := 
    proc(obj      : Type::AnyType,
	 n = FAIL : Type::Union(Type::NonNegInt, DOM_FAIL),
	 w = FAIL : Type::Union(dom::typeWeightVector, DOM_FAIL))
	local i;
    begin
	if not testtype(obj, Type::ListOf(Type::NonNegInt)) then
	    return(FALSE);
	end_if;
	if args(0) = 1 then return(TRUE); end_if;
	if args(0) = 2 then error("Unable to compute the weight without the weight vector"); end_if;
	if nops(obj) <> nops(w) then return(FALSE); end_if;
	if _plus((obj[i]*w[i]) $ i = 1..nops(obj)) <> n then 
	    FALSE;
	else
	    TRUE;
	end_if;
    end_proc;

    typeWeightVector := Type::ListOf(Type::PosInt);

    //////////////////////////////////////////////////////////////////////////
    // count
    //////////////////////////////////////////////////////////////////////////

    // Algorithm: computation from the generating series.
    // Isn't that actually even worst than generating them ?

    count :=
    proc(n: Type::NonNegInt, w: dom::typeWeightVector) : Type::NonNegInt
    begin
        coeff(series(dom::generatingSeries(w,hold(z)), hold(z), n+1), n);
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // generatingSeries
    //////////////////////////////////////////////////////////////////////////

    generatingSeries :=
    proc(w: dom::typeWeightVector, z=hold(z): DOM_IDENT)
	local i;
    begin
        1/_mult((1-z^w[i]) $ i=1..nops(w));
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // list
    //////////////////////////////////////////////////////////////////////////

    list :=
    proc(n: Type::NonNegInt, l: dom::typeWeightVector)
        : Type::ListOf(dom)
        local recfun, perm;
    begin
	if nops(l)=0 then
	    if n=0 then 
		return([[]]);
	    else
		return([]);
	    end_if;
	end_if;

	// Heuristic: we sort the weights increasingly to limit
	// the backtracking, and reorder back the result afterward
	perm := combinat::words::standard(l);
	l := sort(l);

        recfun:=
        proc(n: Type::NonNegInt, l: dom::typeWeightVector)
            : Type::ListOf(dom)
            local w, d, result;
        begin
            result:=[];
            w:=op(l,nops(l));
            delete(l[nops(l)]);
            if l=[] then
                d:=n div w;
                if n mod w =0 then
                    return([[d]])
                else
                    return([]);		// bad branch ...
                end_if;
            end_if;
            for d from floor(n/w) downto 0 do
                result:=result. map(recfun(n-d*w, l), x->append(x,d))
            end_for;
            result;
        end_proc:

	map(recfun(n,l), combinat::permutations::_mult, perm);
    end_proc;

begin
end_domain:

