/*****************************************************************************
  $Id $
  File:		lib/COMBINAT/lyndonWords.mu
  Authors:	Xavier Molinero <molinero@lsi.upc.es>
            	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Purpose:	Source code for the domain combinat::lyndonWords
  Created:	Wed Jan 28 23:42:48 2004
  License:	LGPL
  Project:	MuPAD-Combinat, http://mupad-combinat.sf.net
*****************************************************************************/

domain combinat::lyndonWords
    inherits Dom::BaseDomain;
    category Cat::CombinatorialClass;
    axiom    Ax::systemRep;

    info_str := "combinat::lyndonWords -- the combinatorial class of lyndon words";

    interfaceAdd :=
        {hold(fromEvaluation),
	 hold(fromContent)
	};

    domtype := DOM_LIST;

end_domain:

domain combinat::lyndonWords::fromEvaluation
    inherits Dom::BaseDomain;
    category Cat::CombinatorialClass;
    axiom    Ax::systemRep;

    info_str := "combinat::lyndonWords -- the combinatorial class of integer lyndon Words with prescribed evaluation";

    domtype := DOM_LIST;

    //////////////////////////////////////////////////////////////////////////
    // isA
    //////////////////////////////////////////////////////////////////////////

    isA :=
    proc(object: Type::AnyType,
	 evaluation=FAIL: Type::Union(combinat::compositions, DOM_FAIL))
	: DOM_BOOL
    begin
	if not testtype(object, Type::ListOf(Type::PosInteger)) then
	    FALSE
	elif evaluation <> FAIL and combinat::words::evaluation(object) <> evaluation then
	    FALSE
	else
	    // Check that object is lexicographically smallest and aperiodic
	    UNKNOWN
	end_if;
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // count
    //////////////////////////////////////////////////////////////////////////

    count :=
    proc(evaluation: combinat::compositions)
      local n, i, j, ni;
    begin
      if nops(evaluation)=0 then return(0) end_if;
      n := _plus(i $ i in evaluation);
      _plus(numlib::moebius(j) * (n/j)! / (_mult( (ni/j)! $ ni in evaluation ))
	    $ j in numlib::divisors(gcd(op(evaluation)))) / n
    end_proc;
    
    //////////////////////////////////////////////////////////////////////////
    // generator
    //////////////////////////////////////////////////////////////////////////

    generator :=
    proc(evaluation : combinat::compositions)
    begin
	combinat::necklaces::fromEvaluation::generator(evaluation, TRUE);
    end_proc;

end_domain:

domain combinat::lyndonWords::fromContent
    inherits Dom::BaseDomain;
    category Cat::CombinatorialClass;
    axiom    Ax::systemRep;

    info_str := "combinat::lyndonWords -- the combinatorial class of integer lyndon Words with prescribed content";

    domtype := DOM_LIST;

    //////////////////////////////////////////////////////////////////////////
    // isA
    //////////////////////////////////////////////////////////////////////////

    isA :=
    proc(object: Type::AnyType,
	 content=FAIL: Type::Union(combinat::words, DOM_FAIL))
	: DOM_BOOL
    begin
	if not testtype(object, DOM_LIST) then
	    FALSE
	elif content <> FAIL and sort(object) <> sort(content) then
	    FALSE
	else
	    // Check that object is aperiodic
	    UNKNOWN
	end_if;
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // count
    //////////////////////////////////////////////////////////////////////////

    count :=
    proc(content: combinat::words)
    begin
	combinat::lyndonWords::fromEvaluation::count
	(combinat::words::evaluation(content, {op(content)}));
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // generator
    //////////////////////////////////////////////////////////////////////////

    generator :=
    proc(content: combinat::words)
	option escape;
	local alphabet, unrank;
    begin
	alphabet := sort([op({op(content)})]);
	unrank := combinat::rankers::fromList(alphabet)[2];
	combinat::generators::map
	(combinat::lyndonWords::fromEvaluation::generator
	 (combinat::words::evaluation(content, alphabet)),
	 map, unrank);
    end_proc;

end_domain:
