/*****************************************************************************
  $Id: nonCrossingPartitions.mu,v 1.1.1.6 2005/01/26 13:42:24 nthiery Exp $
  File:		lib/COMBINAT/nonCrossingPartitions.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Created:	16/02/2001
  License:	LGPL
*****************************************************************************/

domain combinat::nonCrossingPartitions
    inherits Dom::BaseDomain;
    category Cat::GradedCombinatorialClass;
    axiom    Ax::systemRep;

    info_str := "Library 'combinat::nonCrossingPartitions': utilities for non crossing set partitions";

    domtype := DOM_SET;

    isA := 
    proc(obj      : Type::AnyType,
	 c = FAIL : Type::Union(Type::NonNegInt, DOM_FAIL))
	local sizes, unionSet, part, iPart, part2, partInterv;
    begin
	if not testtype(obj, Type::SetOf(Type::ListOf(Type::PosInt))) then
	    return(FALSE);
	end_if;
	sizes := map(obj,nops);
        if contains(sizes,0) then return(FALSE); end_if;
	unionSet := _union(op(map(obj, DOM_SET@op)));
        if nops(unionSet)<>_plus(nops(part) $ part in obj) then return(FALSE); end_if;
	if unionSet <> {$1..nops(unionSet)} then return(FALSE); end_if;
	for part in obj do
	    part := sort(part);
	    for part2 in obj do
		if part2 <> part then
		    for iPart from 1 to nops(part)-1 do
			partInterv := select(part2, _in, {$part[iPart]..part[iPart+1]});
			if partInterv <> [] and partInterv <> part2 then
			    return(FALSE);
			end_if;
		    end_for;
		end_if;
	    end_for;
	end_for;
	if args(0) = 1 then return(TRUE); 
	elif nops(unionSet) = c then
	    TRUE
	else 
	    FALSE
	end_if;
    end_proc;

    size := 
    proc(z : dom)  
	local i;
    begin
	_plus(nops(i) $ i in z);
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // Methods
    //////////////////////////////////////////////////////////////////////////

    count := combinat::catalan;

    generator :=
    proc(n)
    begin
	combinat::generators::map(combinat::dyckWords::generator(n),
				  combinat::dyckWords::toNonCrossingPartition
				  );
    end_proc;

end_domain:

