/*****************************************************************************
  $Id: parkingFunctions.mu,v 1.1.1.1 2005/01/26 13:42:24 nthiery Exp $
  File:		lib/COMBINAT/parkingFunctions.mu
  Authors:	Jean-Christophe Novelli <novelli@univ-mlv.fr>
  Purpose:	Source code for the domain combinat::parkingFunctions
  Created:	Fri Nov  5 19:39:23 2004
  License:	LGPL
  Project:	MuPAD-Combinat, http://mupad-combinat.sf.net
*****************************************************************************/

domain combinat::parkingFunctions
    inherits combinat::words;
    category Cat::GradedCombinatorialClass;
    axiom    Ax::systemRep;

    info := "combinat::parkingFunctions -- The combinatorial class of Parking
             Functions";
    interfaceAdd :=
       { hold(nonDecreasing),       // les Catalan parking croissantes
         hold(word2Parking),        // parkisation
         hold(deparkisation),       // mots avec n trous dans l'alphabet
                                    // par rapport a leur park
         hold(prime),               // parking primitives 
         hold(breaks),              // coupures d'une parking (=de sa triee)
         hold(type),                // type = taille des blocs du deconcatenate
         hold(fromType),            // liste des parking de ce type
         hold(park2Connected),      // deconcatenation a droite
         hold(park2ConnectedLeft),  // deconcatenation a droite
         hold(shiftedConcatenation) // concatenation decalee

       };

    domtype := DOM_LIST;
    //typeAlphabet := Type::Union(DOM_LIST, DOM_SET, Type::NonNegInt);
    //typeOrder    := Type::Function;

    //////////////////////////////////////////////////////////////////////////
    // isA
    //////////////////////////////////////////////////////////////////////////

    isA :=
    proc(obj : Type::AnyType) : DOM_BOOL
// TODO : ajouter la taille en option et en test
    local ltmp, i;
    begin
        if not(testtype(obj,Type::ListOf(Type::PosInt))) then
           return(FALSE); end_if;
        ltmp:=sort(obj);
        for i from 1 to nops(ltmp) do
          if ltmp[i]>i then return(FALSE); end_if;
        end_for;

        return(TRUE);
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // size and count
    //////////////////////////////////////////////////////////////////////////

    size := proc(p : dom) begin nops(p); end_proc;

    count :=
    proc(n: Type::Union(Type::NonNegInt, DOM_LIST))
        local list;
    begin

        if testargs() then
            if args(0) > 2 then
                error("wrong number of arguments!");
            end_if;
            if args(0) = 2 and args(2) <> Duplicate then
                error("Unknown option '".expr2text(args(2))."'") ;
            end_if;
        end_if;

        if domtype(n)=DOM_LIST then list:=n;
        else return((n+1)^(n-1)); end_if;

        if not(dom::isA(list)) then return(0); end_if;

        if args(0) = 2 then return(nops(list)!);
        else return(combinat::permutations::count(list));
        end_if ;
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // Objects (list)
    //////////////////////////////////////////////////////////////////////////

    list := proc(n: Type::Union(Type::NonNegInt, DOM_LIST))
    local list;
    begin
      
      if domtype(n)=DOM_LIST then
            list:=n;
        else
            return(map(dom::nonDecreasing::list(n),
                   op @ combinat::permutations::list));
        end_if;

      if not(dom::isA(list)) then return([]); end_if;

      return(combinat::permutations(list));
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // straightening (parkisation) of a word
    //////////////////////////////////////////////////////////////////////////

    //word2Parking := dom::word2ParkingOrig;

    word2Parking := proc(l : Type::ListOf(Type::Integer))
    local mymin,ltrie,lbis,i;
    begin
      if l=[] then return([]); end_if;

      mymin := min(op(l));
      if mymin<=0 then return(dom::word2Parking(map(l,_plus,1-mymin))); end_if;

      lbis := l; i := 1;

      repeat
        ltrie := sort(lbis);
        while ((i<=nops(ltrie)) and (ltrie[i]<=i)) do i:=i+1; end_while;
        lbis := map(lbis, z-> (if z>i then z-(ltrie[i]-i); else z; end_if));
        //ltrie := map(ltrie, z-> (if z>i then z-(ltrie[i]-i); else z; end_if));
      until (i>=nops(lbis)) end_repeat;

      return(lbis);
    end_proc;

    word2ParkingNicolas :=
    proc(l : Type::ListOf(Type::Integer))
	local evaluation, shift, relabeling, e, k;
    begin
	evaluation := combinat::words::evaluationSparse(l);
	relabeling := ()->error("");
	k := 0;
	shift := -infinity;
	for e in evaluation do
	    // Invariants:
	    // - after relabelling the values < e[1] in l, we have
	    //   f^{-1}( { 1..k } ) >= k
	    // - all the values >= e[1] are shifted down by at least shift
	    shift := max(shift, e[1]-k-1);
	    relabeling(e[1]) := e[1]-shift;
	    k := k + e[2];
	end_for;
	map(l, relabeling);
    end_proc;

/*
   // Ceci est un essai rate pour aller plus vite... (49'' pour tous les mots
   // de longueur 6 a 6 lettres a comparer avec 18'' pour la version prec.)
   // (JCN)

    word2Parkingb := proc(l : Type::ListOf(Type::Integer))
    local mymin,ltmp,i,cpt,evpark,myev,per,tot;
    begin
      mymin := min(op(l));
      if mymin<=0 then return(dom::word2Parking(map(l,_plus,1-mymin))); end_if;

      myev := combinat::words::evaluation(l);
      evpark := []; tot :=0; cpt:=0;
      for i from 1 to nops(myev) do
        tot := tot+myev[i];
        if tot+myev[i]>=cpt+1 then evpark := append(evpark,myev[i]);
        cpt:=cpt+1;
        end_if;
      end_for;

      ltmp := [(i$evpark[i])$i=1..nops(evpark)];
      per  := combinat::words::standard(l);

      return([ltmp[per[i]]$i=1..nops(l)]);
    end_proc;
*/
  

    straighten := dom::word2Parking;

    //////////////////////////////////////////////////////////////////////////
    // list of words in a given alphabet having a given parkised word
    //////////////////////////////////////////////////////////////////////////

// /*
    deparkisation := proc(p : dom, n : Type::Integer)
    local ptmp,p1,i,res,n1,comp;
    begin
      ptmp := sort(p);
      p1 := select([$1..nops(ptmp)], z-> bool(ptmp[z]=z));

      res := [p];
      for n1 from 1 to n do
        for comp in combinat::integerVectors::list(n1,nops(p1)) do
          ptmp := p;
          for i from nops(p1) downto 1 do
            ptmp := map(ptmp, z->if (z>=p1[i]) then z+comp[i]; else z; end_if);
          end_for;
          res := append(res,ptmp);
        end_for;
      end_for;

      return(res);
    end_proc;
// */

    fastIntegerList := (n,l) -> combinat::generators::toList(
       combinat::integerListsLexTools::generator(n,l,l,0,RD_INF,RD_NINF,
       RD_INF)
    );

    deparkisation := proc(park : dom, n : Type::Integer)
    local parktmp,coupures,nbcou,coup,res,n1,comp,des,i,som;
    begin
      parktmp := sort(park);
      coupures := select([$1..nops(parktmp)], z-> bool(parktmp[z]=z));
      nbcou := nops(coupures);
      coup := [(i$coupures[i+1]-coupures[i])$i=1..nbcou-1,
               nbcou$(nops(parktmp)-coupures[nbcou]+1)];

      res := [park];
      for n1 from 1 to n do
        //for comp in combinat::integerVectors::list(n1,nbcou)
        for comp in dom::fastIntegerList(n1,nbcou)
        do
          som := 0;
          des := [(som := comp[i]+som) $ i=1..nops(comp)];
          res := append(res, map(park, z-> z+des[coup[z]]));
        end_for;
      end_for;

      return((res));
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // decomposition of a parking function in connected parts (Right/Left)
    //////////////////////////////////////////////////////////////////////////

    park2Connected := proc(park : dom)
    local res, ptmp, i, j, k, p1, p2;
    begin
      res := []; ptmp := park; i := 1; 

      for k from 1 to nops(park)-1 do
        p1 := [ptmp[j]$j=1..i];
        if (dom::isA(p1)) then
          p2 := dom::word2Parking([ptmp[j]$j=i+1..nops(ptmp)]);
          if (p1.map(p2,_plus,nops(p1))=ptmp) then
            res := append(res,p1);
            ptmp := p2; i:=1;
          else i:=i+1;
          end_if;
        else i:=i+1;
        end_if;
      end_for;

      return(append(res,ptmp));
    end_proc;

    park2ConnectedLeft := proc(park : dom)
    begin
      revert(map(dom::park2Connected(revert(park)),revert));
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // shifted concatenation
    //////////////////////////////////////////////////////////////////////////

    shiftedConcatenation := proc(lis : Type::ListOf(dom))
    local res, i;
    begin
      res := lis[1];
      for i from 2 to nops(lis) do
        res := res.map(lis[i],_plus,nops(res));
      end_for;
      return(res);
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // prime: with no break point except 'n'
    // break point: j is iff  #{park_i<=j} = j
    // type: composition in which its nondecreasing reordering is 
    //////////////////////////////////////////////////////////////////////////

    prime := proc(n : Type::NonNegInt)
    begin
      return(map(dom::nonDecreasing::prime(n),op@combinat::permutations));
    end_proc;

    breaks := proc(park : dom)
    begin
      return(dom::nonDecreasing::breaks(sort(park)));
    end_proc;

    type := proc(park : dom)
    begin
      return(dom::nonDecreasing::type(sort(park)));
    end_proc;

    fromType := proc(compo)
    begin
      return(map(dom::nonDecreasing::fromType(compo),
                 op@combinat::permutations));
    end_proc;

begin
end_domain:

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
///////////////// New domain inside parking functions  ///////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

domain combinat::parkingFunctions::nonDecreasing
    inherits Dom::BaseDomain;
    category Cat::GradedCombinatorialClass;
    axiom    Ax::systemRep;

    info := "combinat::parkingFunctions::nonDecreasing -- The combinatorial
             class of non-Decreasing Parking Functions";

    interfaceAdd :=
       {
         hold(prime),         // primitives parking croissantes
         hold(breaks),        // coupures d'une parking croissante
         hold(type),          // type = taille des blocs du deconcatenate
         hold(fromType),      // liste des park. croiss. avec ce type
         hold(park2Connected) // deconcatenate a droite
       };

    domtype := DOM_LIST;
    //typeAlphabet := Type::Union(DOM_LIST, DOM_SET, Type::NonNegInt);
    //typeOrder    := Type::Function;

    //////////////////////////////////////////////////////////////////////////
    // isA
    //////////////////////////////////////////////////////////////////////////

    isA :=
    proc(obj : Type::AnyType) : DOM_BOOL
// TODO : ajouter la taille en option et en test
    local i;
    begin
        if not(testtype(obj,Type::ListOf(Type::PosInt))) then
           return(FALSE); end_if;
        if obj<>sort(obj) then return(FALSE); end_if;
        for i from 1 to nops(obj) do
          if obj[i]>i then return(FALSE); end_if;
        end_for;

        return(TRUE);
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // size and count
    //////////////////////////////////////////////////////////////////////////

    size := proc(p : dom) begin nops(p); end_proc;

    count :=
    proc(n: Type::Union(Type::NonNegInt, DOM_LIST))
        local list;
    begin
        if testargs() then
            if args(0) > 2 then
                error("wrong number of arguments!");
            end_if;
            if args(0) = 2 and args(2) <> Duplicate then
                error("Unknown option '".expr2text(args(2))."'") ;
            end_if; 
        end_if;
    
        if domtype(n)=DOM_LIST then list:=n;
        else return(combinat::catalan(n)); end_if;

        if not(dom::isA(list)) then return(0); else return(1); end_if;
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // objects (list)
    //////////////////////////////////////////////////////////////////////////

    list := proc(n : Type::NonNegInt)
        local k;
    begin
       if n=0 then return([[]]); end_if;
       return([op(combinat::compositions::list(
              k,Outer=[$1..n],MinSlope=0,Length=n))$k=n..n*(n+1)/2]);
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // cut in connected parking functions
    //////////////////////////////////////////////////////////////////////////

    prime := proc(n : Type::NonNegInt)
    local tmp;
    begin
      tmp := dom::list(n-1);
      tmp := map(tmp, z-> _concat([1],z));
      return(tmp);
    end_proc;

    breaks := proc(park : dom)
    begin
      return(select([$1..nops(park)-1], z-> bool(park[z+1]>z)).[nops(park)]);
    end_proc;


    type := proc(park : dom)
    local coupures; 
    begin
      coupures := dom::breaks(park);
      return(combinat::compositions::fromDescents(coupures));
    end_proc;

    fromType := proc(compo) // : combinat::composition)
    local lis,i;
    begin
       lis := dom::prime(compo[1]);
       for i from 2 to nops(compo) do
         lis := combinat::cartesianProduct(lis,
                  map(dom::prime(compo[i]), map,z->z+nops(lis[1])));
         lis := map(lis,map,op);
       end_for;
       return(lis);
    end_proc;

    park2Connected := proc(park : dom)
    local coupures; 
    begin
      coupures := dom::breaks(park);
      coupures := _concat([0],coupures);
       return([ combinat::parkingFunctions::word2Parking(
                [park[j]$j=1+coupures[i-1]..coupures[i]])
               $i=2..nops(coupures)]);
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // objects (list)
    //////////////////////////////////////////////////////////////////////////

begin
end_domain:
