/*****************************************************************************
  $Id: rankers.mu,v 1.1.1.3 2005/01/26 13:42:24 nthiery Exp $
  File:		lib/COMBINAT/rankers.mu
  Authors:	Florent Hivert <florent.hivert@univ-mlv.fr>
  Created:	03/02/2003
  License:	LGPL
*****************************************************************************/


/*****************************************************************************
 A ranker is a device that allow for assigning a number (called the rank) 
 to a set of objects. This typically is very handy in combinatorics.
*****************************************************************************/

domain combinat::rankers

    info_str := "Library 'combinat::rankers': utilities for rankers";

    interface := {hold(onFly), hold(fromType), hold(fromList)};
  
    dataType := Type::Union(Type, DOM_DOMAIN);
    typeRanker := Type::AnyType;
    typeUnranker := Type::AnyType;
    
    /*************************************************************************
     * Returns a new on-the-fly ranker
    *************************************************************************/

    onFly :=
    proc(typeOfObjects = Type::AnyType: dom::dataType)
	option escape;
	local rank, unrank, nbr;
    begin
	nbr := 0;
	unrank := () -> FAIL;
	//proc() option remember;
	//	  begin
	//	      print("unranking Error", args());
	//	      FAIL;
	//	  end_proc;
	rank := proc(el)
		    option remember;
		begin
		    nbr := nbr + 1;
		    (unrank)(nbr) := el;
		    nbr;
		end_proc;
	[
         proc(el : typeOfObjects) : Type::PosInt
	 begin rank(el) end,
         proc(i : Type::PosInt) : Type::Union(typeOfObjects, DOM_FAIL)
         begin unrank(i) end
	];
    end_proc;

    /*************************************************************************
     * Returns an on-fly ranker for the objects of type typeOfObject
     * This ranker is uniq in a given MuPAD session
    *************************************************************************/
  fromType :=
    proc(typeOfObjects : dom::dataType)
	option remember;
    begin
	userinfo(1, "Creating a new ranker for ".expr2text(typeOfObjects));
	dom::onFly(typeOfObjects);
    end_proc;

  fromTypeOld := 
    proc(typeOfObjects : dom::dataType)
	option remember, escape;
	local list, listSize, allocated,
	      rank, unrank;
    begin
	allocated := 16; 
	list     := [FAIL $ allocated];
	listSize     := 0;
	rank := proc(el : typeOfObjects) : Type::PosInt
		    option remember; 
		begin
		    if (listSize := listSize + 1) = allocated then
			list := append(list, FAIL $ allocated);
			allocated := 2 * allocated;
		    end_if;
		    list[listSize] := el;
		    listSize;
		end_proc;
	
	unrank := proc(i : Type::PosInt) 
		      : Type::Union(typeOfObjects, DOM_FAIL)
		  begin 
		      if i <= listSize then list[i] 
		      else FAIL 
		      end_if;
		  end_proc;
	[rank, unrank];
    end_proc;

  fromList := 
    proc(list : DOM_LIST)
	option escape;
	local rank, unrank, n, i;
    begin
	n := nops(list);
	unrank := proc(j : Type::PosInt) 
		  begin 
		      if j <= n then list[j] 
		      else FAIL 
		      end_if;
		  end_proc;	
	rank := (obj) -> error("Unknown object: ".expr2text(obj));
	for i from 1 to n do 
	    rank(list[i]) := i; 
	end_for;
	[rank, unrank];
    end_proc;

    rankFromList := 
    proc(list : DOM_LIST, onUnknown = Error)
	local rank, i;
    begin
	if onUnknown = Error then
	    rank := obj -> error("Unknown object: ".expr2text(obj));
	else
	    rank := obj -> FAIL;
	end_if;
	for i from 1 to nops(list) do 
	    rank(list[i]) := i; 
	end_for;
	rank;
    end_proc;

    unrankFromList := 
    proc(list : DOM_LIST)
	option escape;
	local n;
    begin
	n := nops(list);
	proc(j : Type::PosInt) 
	begin 
	    if j <= n then list[j] else FAIL end_if;
	end_proc;	
    end_proc;

begin
end_domain:

%if version()=[2,0,0] then 
    // Force the construction of operators::rankers::fromType
    // Standard workaround for MuPAD 2.0.0's option remember bug
    combinat::rankers::fromType;
end_if:
