/*****************************************************************************
  $Id: setPartitionsOrdered.mu,v 1.1.1.4 2005/01/26 13:42:24 nthiery Exp $
  File:		lib/COMBINAT/setPartitionsOrdered
  Authors:	Florent Hivert <florent.hivert@univ-mlv.fr>
  Created:	26/09/2003
  License:	LGPL
*****************************************************************************/
protect(AllowsEmptyParts):

domain combinat::setPartitionsOrdered
    inherits Dom::BaseDomain;
    category Cat::GradedCombinatorialClass;
    axiom    Ax::systemRep;

    info_str := "Library 'combinat::setPartitionsOrdered': utilities for ordered set partitions";

    interfaceAdd :=
       {hold(intersectionsMatrix)};

    //////////////////////////////////////////////////////////////////////////
    // types
    //////////////////////////////////////////////////////////////////////////

    domtype := DOM_LIST;
    typeSet := Type::Union(DOM_SET, DOM_LIST, Type::NonNegInt);
    typeWithEmptyParts := Type::Predicate(expr2text(dom)."WithEmptyParts", 
					  dom::isAWithEmptyParts);

    //////////////////////////////////////////////////////////////////////////
    // isA
    //////////////////////////////////////////////////////////////////////////

    isA := 
    proc(obj          : Type::AnyType,
         set = FAIL   : Type::Union(DOM_FAIL, dom::typeSet),
         comp = FAIL  : Type::Union(DOM_FAIL, Type::NonNegInt, combinat::integerVectors)
	 ) : DOM_BOOL
    begin
	if not testtype(obj, Type::ListOf(DOM_SET)) then return(FALSE); end_if;
	_lazy_and(dom::isAWithEmptyParts(args()),
		  _lazy_or(domtype(comp) = DOM_LIST, contains(obj, {}) = 0));
    end_proc;

    isAWithEmptyParts := 
    proc(obj          : Type::AnyType,
         set = FAIL   : Type::Union(DOM_FAIL, dom::typeSet),
         comp = FAIL  : Type::Union(DOM_FAIL, Type::NonNegInt, combinat::integerVectors)
	 ) : DOM_BOOL
	local sizes;
    begin
	if not testtype(obj, Type::ListOf(DOM_SET)) then return(FALSE); end_if;
	sizes := map(obj, nops);
	if nops(_union(op(obj))) <> _plus(op(sizes)) then return(FALSE); end_if;
	if args(0) = 1 then return(TRUE); end_if;

	if testtype(set, Type::NonNegInt) then 
	    set := {$1..set};
	else
	    set := {op(set)};
	end_if;
	if _union(op(obj)) <> set then return(FALSE); end_if;
	if args(0) = 2 then return(TRUE); end_if;
	
	if domtype(comp) = DOM_INT then
	    if nops(sizes) <> comp then return(FALSE); end_if;
	elif sizes <> comp then return(FALSE); 
	end_if;
        TRUE;
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // count
    //////////////////////////////////////////////////////////////////////////

    count := 
    proc(set        : dom::typeSet, 
         comp = FAIL  : Type::Union(DOM_FAIL, Type::NonNegInt, combinat::integerVectors))
	local i, c;
    begin
	if domtype(set) <> DOM_INT then 
	    set := nops(set);
	end_if;
	if args(0) = 1 then
	    _plus(dom::count(set, c) 
		  $ c in combinat::compositions::list(set));
	elif domtype(comp) = DOM_INT then
	    _plus(dom::count(set, c) 
		  $ c in combinat::compositions::list(set, Length=comp));
	else
	    set! / _mult(i! $ i in comp);
	end_if;
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // list
    //////////////////////////////////////////////////////////////////////////
 
    list := 
    proc(set        : dom::typeSet, 
         comp = FAIL  : Type::Union(DOM_FAIL, Type::NonNegInt, combinat::integerVectors))
	local n, dcomp, l, j;
    begin
	if domtype(set) = DOM_INT then 
	    set := [$1..set];
	else
	    set := [op(set)];
	end_if;
	n := nops(set);
	if args(0) = 1 then
	    return(map(combinat::compositions::list(n),
		       prog::bless(z -> op(dom::list(set, z)), dom)));
	elif domtype(comp) = DOM_INT then
	    return(map(combinat::compositions::list(n, Length=comp),
		       prog::bless(z -> op(dom::list(set, z)), dom)));
	end_if;
	if testargs() and n <> _plus(op(comp)) then
	    error("Size mismatch: ".expr2text(set).", ".expr2text(comp));
	end_if;

	l := nops(comp);
	if n = 0 then return([[{}$ l]]); end_if;

	dcomp := [0].combinat::compositions::descents(comp, FinalDescent);
	
	map(combinat::permutations::list([(j$comp[j]) $j=1..l]),
	    proc(perm)
		local res, i;
	    begin
		res := combinat::permutations::_mult(set,
			   combinat::permutations::inverse(combinat::words::standard(perm)));
		[{op(res, dcomp[i]+1..dcomp[i+1])} $ i=1..l];
	    end_proc);
    end_proc;

    list2 := 
    proc(set        : dom::typeSet, 
         comp = FAIL  : Type::Union(DOM_FAIL, Type::NonNegInt, combinat::integerVectors))
	local n, dcomp, l;
    begin
	if domtype(set) = DOM_INT then 
	    set := [$1..set];
	else
	    set := [op(set)];
	end_if;
	n := nops(set);
	if args(0) = 1 then
	    return(map(combinat::compositions::list(n),
		       z -> op(dom::list(set, z))));
	elif domtype(comp) = DOM_INT then
	    return(map(combinat::compositions::list(n, Length=comp),
		       z -> op(dom::list(set, z))));	    
	end_if;
	l := nops(comp);
	if testargs() and n <> _plus(op(comp)) then
	    error("Size mismatch: ".expr2text(set).", ".expr2text(comp));
	end_if;
	if n = 0 then return([[{}$ l]]); end_if;

	dcomp := [0].combinat::compositions::descents(comp, FinalDescent);
	
	map(combinat::integerMatrices::list(comp, [1$n]), 
	    proc(mat)
		local res, i;
	    begin
		res := combinat::permutations::_mult(set,
		           combinat::integerMatrices::standard(mat));
		[{op(res, dcomp[i]+1..dcomp[i+1])} $ i=1..l];
	    end_proc);
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // generator 
    //////////////////////////////////////////////////////////////////////////

    generator := 
    proc(set        : dom::typeSet, 
         comp = FAIL  : Type::Union(DOM_FAIL, Type::NonNegInt, combinat::integerVectors))
	option escape;
	local n, dcomp, l, DOM, j;
    begin
	DOM := dom;
	if domtype(set) = DOM_INT then 
	    set := [$1..set];
	else
	    set := [op(set)];
	end_if;
	n := nops(set);
	if args(0) = 1 then
	    return(combinat::generators::pipe(combinat::compositions::generator(n),
					      z -> DOM::generator(set, z)));
	elif domtype(comp) = DOM_INT then
	    return(combinat::generators::pipe(combinat::compositions::generator(n, Length=comp),
					      z -> DOM::generator(set, z)));
	end_if;
	l := nops(comp);
	if testargs() and n <> _plus(op(comp)) then
	    error("Size mismatch".expr2text(set).", ".expr2text(comp));
	end_if;
	if n = 0 then return(combinat::generators::repeater([{}$ l], 1)); end_if;

	dcomp := [0].combinat::compositions::descents(comp, FinalDescent);

	combinat::generators::map(combinat::permutations::generator([(j$comp[j]) $j=1..l]),
	    proc(perm)
		local res, i;
	    begin
		res := combinat::permutations::_mult(set,
			   combinat::permutations::inverse(combinat::words::standard(perm)));
		[{op(res, dcomp[i]+1..dcomp[i+1])} $ i=1..l];
	    end_proc);
    end_proc;


    generator2 := 
    proc(set        : dom::typeSet, 
         comp = FAIL  : Type::Union(DOM_FAIL, Type::NonNegInt, combinat::integerVectors))
	option escape;
	local n, dcomp, l, DOM;
    begin
	DOM := dom;
	if domtype(set) = DOM_INT then 
	    set := [$1..set];
	else
	    set := [op(set)];
	end_if;
	n := nops(set);
	if args(0) = 1 then
	    return(combinat::generators::pipe(combinat::compositions::generator(n),
					      z -> DOM::generator(set, z)));
	elif domtype(comp) = DOM_INT then
	    return(combinat::generators::pipe(combinat::compositions::generator(n, Length=comp),
					      z -> DOM::generator(set, z)));
	end_if;
	l := nops(comp);
	if testargs() and n <> _plus(op(comp)) then
	    error("Size mismatch".expr2text(set).", ".expr2text(comp));
	end_if;
	if n = 0 then return(combinat::generators::repeater([{}$ l], 1)); end_if;

	dcomp := [0].combinat::compositions::descents(comp, FinalDescent);
	
	combinat::generators::map(combinat::integerMatrices::generator(comp, [1$n]), 
				  proc(mat)
				      local res, i;
				  begin
				      res := combinat::permutations::_mult(set,
					   combinat::integerMatrices::standard(mat));
				      [{op(res, dcomp[i]+1..dcomp[i+1])} $ i=1..l];
				  end_proc);
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // Methods
    //////////////////////////////////////////////////////////////////////////


    intersectionsMatrix := 
    proc(p1 : dom::typeWithEmptyParts, p2 : dom::typeWithEmptyParts)
	local i,j;
    begin
	if testargs() and _union(op(p1)) <> _union(op(p2)) then
	    error(expr2text(p1).", ".expr2text(p2)." must be partitions of the same set");
	end_if;	
	[[p1[i] intersect p2[j] $ i = 1..nops(p1)] $ j = 1..nops(p2)];
    end_proc;
begin
end_domain:
