/*****************************************************************************
  $Id: singleton.mu,v 1.1.1.2 2005/01/26 13:42:24 nthiery Exp $
  File:		lib/COMBINAT/singleton.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Purpose:	Source code for the domain combinat::singleton
  Created:	Sat Jul  3 13:26:22 2004
  License:	LGPL
  Project:	MuPAD-Combinat, http://mupad-combinat.sf.net
*****************************************************************************/

domain combinat::singleton(blah: Type::AnyType)
    local weight, zeroWeight, size;
    inherits Dom::BaseDomain;
    category Cat::GradedCombinatorialClass;	// Not quite true if size is not an integer


    typeSize := Type::AnyType;

    isA := x -> bool(x=object);

    info_str := "combinat::singleton -- Combinatorial classes with a single element";

    size := size;	// Might be safer as () -> size, possibly with typeChecking

    count  := n -> if n=size then Weight else ZeroWeight end_if;

    unrank :=
    proc(r: Type::NonNegInt, n) : Type::Union(dom, DOM_FAIL)
    begin
	if r=1 and n=size then
	    object
	else
	    FAIL
	end_if;
    end_proc;

    rank :=
    proc(x: dom) : Type::NonNegInt
    begin
	1
    end_proc;

    generator :=
    proc(n: dom::typeSize) : combinat::generators
    begin
	if n=size then
	    combinat::generators::singleton(object)
	else
	    combinat::generators::empty;
	end_if;
    end_proc;

    generatorFromRank :=
    proc(r: Type::NonNegInt, n)
    begin
	if n=size and r=1 then
	    combinat::generators::singleton(object)
	else
	    combinat::generators::empty;
	end_if;
    end_proc;

begin
    Options := combinat::getOptions
	     (1,
	      [Weight     = 1,
	       ZeroWeight = 0,
	       Size       = 1],
	      [Weight=     Type::AnyType, // should belong to some ring
	       ZeroWeight= Type::AnyType,
	       Size=   Type::AnyType]);

    weight     := Options[Weight];
    zeroWeight := Options[ZeroWeight];
    size       := Options[Size];
end_domain:
