/*****************************************************************************
  $Id: splitNK.mu,v 1.1.1.2 2005/01/26 13:42:24 nthiery Exp $
  File:		lib/COMBINAT/splitN.mu
  Authors:	Florent Hivert <florent.hivert@univ-mlv.fr>
                Xavier Molinero <molinero@lsi.upc.es>
  Created:	08/01/2004
  License:	LGPL
*****************************************************************************/

domain combinat::splitNK
    inherits Dom::BaseDomain;
    category Cat::CombinatorialClass;
    axiom    Ax::systemRep;

    info_str := "Library 'combinat::splitNK': utilities for spliting of the set {1..n}";

    isA  := 
    proc(obj : Type::AnyType, 
	 n = null()  : Type::Union(DOM_NULL, Type::NonNegInt), 
	 k = null()  : Type::Union(DOM_NULL, Type::NonNegInt))
	local un, maxi;
    begin
	if not testtype(obj, Type::ListProduct(Type::ListOf(Type::PosInt) $ 2))
	   then return(FALSE)
	end_if;
	un   := {op(obj[1])} union {op(obj[2])};
	maxi := max(0,op(un));
	if un <> {$1..maxi} then return(FALSE) end_if;
	if nops(obj[1]) + nops(obj[2]) <> maxi then return(FALSE) end_if;
	if args(0) > 1 and maxi <> n then return(FALSE) end_if;
	if args(0) > 2 and nops(obj[1]) <> k then return(FALSE) end_if;
	return(TRUE);
    end_proc;

    count := 
    proc(n: Type::NonNegInt, k: Type::NonNegInt)
    begin
	combinat::binomialRecursive(n, k);
    end_proc;

    unrank := 
    proc(r : Type::PosInt, n: Type::NonNegInt, k: Type::NonNegInt)
	local resA, resB, posResA, posResB, colPascal, linPascal, bin, i;
    begin
	if r > combinat::binomialRecursive(n,k) then return(FAIL); end_if;
	resA    := [0$k];
	resB    := [0$n-k];
	posResA := 0;
	posResB := 0;	
	r := r-1; // the rank start from 1...
	colPascal := k-1;
	linPascal := n-1;
	for i from 1 to n do
	    bin := combinat::binomialRecursive(linPascal, colPascal);
	    if r >= bin then 
		r         := r - bin;
		resB[(posResB := posResB+1)] := i;
	    else
		colPascal := colPascal - 1;
		resA[(posResA := posResA+1)] := i;
	    end_if;
	    linPascal := linPascal - 1;
	end_for;
	[resA, resB];
    end;

    // O(k + n*log(n)); it's based in Floyd's algorithm
    randomSlow :=
    proc(n : Type::NonNegInt, k : Type::NonNegInt)
	local l, i, rnd, lRes, r;
    begin
        l:=[$1..n];
        for i from n downto n-k+1 do
            rnd        := random(i);
            r          := rnd() + 1;
            lRes[n-i+1]:= l[r];
            if r < i then l[r]:= l[i] end_if
        end_for;
        [sort([lRes[i] $ i=1..k]), sort([l[i] $ i=1..n-k])];
    end_proc;
    // O(k*log(n) + k*log(k)); can probably be improved
    random :=
    proc(n : Type::NonNegInt,
	 k : Type::NonNegInt)
	local i, rem, res, choice;
    begin
	rem    := [$1..n];
	res    := [0$k];
	for i from 1 to k do
	    choice := random(nops(rem))()+1;
	    res[i]  := rem[choice];
	    delete(rem[choice]);
	end_for;
	[sort(res), rem];
    end;

end_domain:
