/*****************************************************************************
  $Id $
  File:		lib/COMBINAT/treesGeneric.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Purpose:	Source code for the domain combinat::treesGeneric
  Created:	Mon Jan 12 10:21:32 2004
  License:	LGPL
  Project:	MuPAD-Combinat, http://mupad-combinat.sf.net
*****************************************************************************/

// TODO: make the the bulk of this into a category, and just leave
// a basic constructor

category Cat::TreesClass
    (Specification = [] :
     Type::Union(Type::SetOf  (Type::Equation(DOM_IDENT)),
		 Type::ListOf (Type::Equation(DOM_IDENT)),
		 Type::TableOf(DOM_IDENT),
		 Type::Option))
    local Options, Skip;
    category Cat::DecomposableClass;

    info_str := "Cat::TreesClass -- the category of tree-like combinatorial classes";

    specification :=
    table(Tree = Union(if Options[Binary] then
			   LeafNil
		       else
			   Leaf
		       end_if,
		       Node
		       ),

	  Node = Alias(NodeRaw,
		       prog::bless(x -> dom([ op(x,1), op(op(x,2))]),
				   dom)
		      ),
	  NodeRaw = Prod(NodeLabel, ChildList),
	  NodeLabel = if Options[Labelled] then
			  Atom(id)
		      else
			  Atom(dom::dummyLabel);
		      end_if,
	  ChildList = if Options[Binary] or Options[CompleteBinary] then
			  if Options[Ordered] then
			      Prod(Tree, Tree);
			  else
			      Multiset(Tree, Length=2);
			  end_if;
		      else
			  // Internal nodes have at least one child
			  if Options[Ordered] then
			      Sequence(Tree, MinLength=1);
			  else
			      Multiset(Tree, MinLength=1);
			  end_if;
		      end_if,

	  Leaf = Alias(LeafLabel, dom @ DOM_LIST),
	  LeafLabel = if Options[Labelled] then
			  Atom(id);
		      else
			  Atom(dom::dummyLabel);
		      end_if,
	  LeafNil = Epsilon(dom([])),
	  if Skip=1 then
	      op(Specification);
	  else
	      null();
	  end_if);

    decomposableClassOptions := [MainNonTerminal = Tree,
				 Labelled = Options[Labelled],
				 Boustrophedonic = FALSE];


    shuffle :=
    if Options[Binary] or Options[CompleteBinary] then
	combinat::binaryTrees::shuffle;
    end_if;

begin
    // Check if a specification is provided, and do type checking
    if domtype(Specification) in {DOM_SET, DOM_LIST, DOM_TABLE} then
	Skip := 1;
    else
	Skip := 0;
    end_if;
    // Option analysis
    Options := combinat::getOptions
    (
     table(Labelled = FALSE,
	   Binary   = FALSE,
	   CompleteBinary = FALSE,
	   Ordered  = TRUE,
	   Rooted = TRUE
	  ),
     table(Labelled = DOM_BOOL,
	   Binary   = DOM_BOOL,
	   CompleteBinary = DOM_BOOL,
	   Ordered  = DOM_BOOL,
	   Rooted   = DOM_BOOL
	  ),
     Skip,
     Error
    );
end_category:

domain combinat::treesGeneric()
    local Options;
    category C;
    inherits if Options[Binary] or Options[CompleteBinary] then
		 combinat::binaryTrees;
	     else
		 combinat::trees;
	     end_if;
begin
    C := Cat::TreesClass(args());
    Options := extop(extop(C,2),6); // Kluge to get the options from the category
end_domain:
