/*****************************************************************************
  $Id: CombinatorialHopfAlgebra.mu,v 1.1.1.3 2005/01/26 13:42:22 nthiery Exp $
  File:		lib/DOMAINS/CATEGORY/HopfAlgebraWithBasis.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
                Florent Hivert <Florent.Hivert@univ-mlv.fr>
  Purpose:	Source code for the category Cat::HopfAlgebraWithBasis
  Created:	Wed Sep  3 13:58:38 2003
  License:	LGPL
  Project:	MuPAD-Combinat, http://mupad-combinat.sf.net
*****************************************************************************/

/**
    Cat::HopfAlgebraWithBasis(R)

    R: ring

    For now, R needs to be commutative.
**/

alias(declareConversion = operators::overloaded::declareConversion):
alias(declareSignature  = operators::overloaded::declareSignature);

category Cat::CombinatorialHopfAlgebra(R) // This is the former HopfAlgebraWithBasis
    category Cat::AlgebraWithBasis(R);

    info_str := "Cat::HopfAlgebraWithBasis -- the category of Hopf algebras over a distinguished basis";

    interfaceAdd :=
	{
	 hold(coproduct), hold(antipode)
	};

    //////////////////////////////////////////////////////////////////////////
    // Conversions
    //////////////////////////////////////////////////////////////////////////

    tensorAlgebra := experimental::TensorAlgebra(dom);

    // Default initialization using the overloaded and conversion
    // mechanism from operators::overloaded.

    // A domain in this category can set its own initialization by
    // overriding initDomainCombinatorialHopfAlgebra

    coproduct := if dom::coproductBasis <> FAIL then
		     operators::makeLinear
		     (dom::coproductBasis,
		      Source=dom, ImageSet=dom::tensorAlgebra);
		 else
		     proc(x : dom)
			 local res; 
		     begin 
			 res := operators::coproduct(x);
			 dom::tensorAlgebra::convert(res);
		     end_proc;
		 end_if;

    antipode   := operators::makeLinear
	       (dom::antipodeBasis,
		Source=dom, ImageSet=dom);

    antipodeBasis := // calcul automatique de l'antipode...
	proc(el : dom::basisIndices)
	local cursize, size, res, workList, elTerm, curRes, pair, DOM;
	begin
	    DOM := dom;
	    size   := dom::basisIndices::size(el); 
	    elTerm := dom::term(el);
	    if size = 0 then return(elTerm); end_if;
	    res := dom::_negate(elTerm);
	    if size > 1 then
		workList := dom::tensorAlgebra::poly2list(dom::coproductBasis(el) 
				    - dom::tensorAlgebra::tensorConstructor(dom::one, elTerm)
				    - dom::tensorAlgebra::tensorConstructor(elTerm, dom::one));
		workList := map(workList, // tens is of the form [coeff, [x1, x2]] 
	       		                  // returns [coeff * dom(x1), x2]
				(tens) ->[DOM::monomial(tens[1], tens[2][1]), tens[2][2]]);		
					 
		for cursize from 2 to size do
		    curRes := dom::plus(dom::mult(pair[1], dom::term(pair[2])) $ pair in workList);
		    if cursize mod 2 = 1 then
			curRes := dom::_negate(curRes);
		    end_if;
//		print(cursize, curRes);
		    res := res + curRes;
		    workList := map(workList,
		        proc(pair)		
			    local res, t1, t2, t2Term;
			begin
			    [t1, t2] := pair;
			    t2Term   := DOM::term(t2);
			    res := DOM::tensorAlgebra::poly2list(DOM::coproductBasis(pair[2])
					   - DOM::tensorAlgebra::tensorConstructor(DOM::one, t2Term)
					   - DOM::tensorAlgebra::tensorConstructor(t2Term, DOM::one));
			    op(map(res, // tens is of the form [coeff, [x1, x2]] 
				        // returns [coeff * dom(x1), x2]
				   (tens) -> [DOM::mult(t1, DOM::monomial(tens[1], tens[2][1])), tens[2][2]]));
			end_proc);
		end_for;	    
		assert(workList=[]);
	    end_if;
	    res;
	end_proc;


    initDomainAdd := dom::initDomainHopfAlgebraWithBasis;
    initDomainHopfAlgebraWithBasis :=
    proc()
	local p;
	option escape;
    begin
	// This part should probably be done by Cat::CombinatorialBialgebra
	if dom::coproductBasis <> FAIL then
	    declareSignature
	    (operators::coproduct, [dom], dom::coproduct);
	end_if;
	declareSignature // Linearization of the product
	(operators::mu, [dom::tensorAlgebra],
	 operators::makeLinear
	 (prog::bless(proc(t: dom::basisIndices) : dom::tensorAlgebra
		      begin
			  _mult(dom::term(p) $ p in t);
		      end_proc,
		      dom),
	  Source=dom::tensorAlgebra, ImageSet=dom));
	declareSignature // Antipode
	 (operators::antipode, [dom], dom::antipode)
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // Arithmetic operations
    //////////////////////////////////////////////////////////////////////////
end_category:
