/*****************************************************************************
  $Id: FiniteDimensionalAlgebraWithBasis.mu,v 1.1.1.3 2004/10/04 17:25:13 nthiery Exp $
  File:		lib/DOMAINS/CATEGORY/FiniteDimensionalAlgebraWithBasis.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Created:	03/09/2003
  License:	LGPL
*****************************************************************************/

/**
    Cat::FiniteDimensionalAlgebraWithBasis(R) -- R-Free module over some basis dom::basisIndices
**/

alias(SubalgebraFromRowReducedBasis= experimental::HeckeSymmetricGroupAlgebraTools::SubalgebraFromRowReducedBasis):

// That might as well be an axiom, since this is an information on
// the underlying vector space structure which is completely independent
// of how the further operations are defined.
// In the same maner there should axioms Connected and OfFiniteType for
// graded vector spaces, rings, ...

category Cat::FiniteDimensionalAlgebraWithBasis(R)
    category Cat::AlgebraWithBasis(R);

    interfaceAdd := {hold(dimension), hold(algebraGenerators),
		     hold(center), hold(semiSimple),
		     hold(rightMatrix), hold(leftMatrix)}; 
    
    dimension := dom::basisIndices::count();

    algebraGenerators := () -> UNKNOWN; // if a set of generator of dom as an algebra
                                        // is known put them here.
    
    // representation theoretical stuff.
    // TODO : tranfer from algTools to there. 
    center           := () -> experimental::algTools::CenterOfAlgebra(dom);
    radicalBasis     := () -> (dom::semiSimple())::RRBasis;
    semiSimple       := () -> experimental::algTools::SemiSimpleOfAlgebra(dom);
    isSemiSimple     := () -> bool(dom::semiSimple = dom);
    semiSimpleCenter := () -> experimental::algTools::CenterOfAlgebra(dom::semiSimple());

    action := proc(x:dom, y:dom) begin dom::mult2(y,x); end_proc;

    /**
     * Matrix of the action of x on the algebra by left multiplication
     **/
    leftMatrix :=
    proc(x: dom) : dom::sparseMatricesDomain
	local t;
    begin
	_concat(dom::sparseVector(dom::mult2(x, t)) $ t in dom::basis::list());  
    end_proc;

    /**
     * Matrix of the action of x on the algebra by right multiplication
     **/
    rightMatrix :=
    proc(x: dom) : dom::sparseMatricesDomain
	local t;
    begin
	_concat(dom::sparseVector(dom::mult2(t,x)) $ t in dom::basis::list());
    end_proc;

    /**
     * Computes the minimal idempotents of a commutative radical-free algebra
     * (i.e. a quotient of a polynomial ring by a radical ideal)
     * Assumption: the coefficient ring is a field of characteristic 0.
     **/
    minimalIdempotents :=
    proc() : Type::ListOf(dom)
	local basis, M, dimensions, eigenspace, eigenspaces, 
	      x, c, b, d, r, i, renormalizeIdempotent;
    begin
	renormalizeIdempotent :=
	prog::bless(proc(x: dom)
		    begin
			c := lcoeff(dom::mult2(x,x));
			if iszero(c) then
			    error("A is not radical-free!");
			end_if;
			dom::multcoeffs(x, 1/c);
		    end_proc,
		    dom);

	basis := dom::basisIndices();
	if nops(basis) = 1 then
	    return([renormalizeIdempotent(dom::term(basis[1]))]);
	end_if;
	userinfo(1, "Computing the minimal idempotents of an algebra `A' of dimension ".
		 dom::dimension());
	d := 3;
	while TRUE do
	    userinfo(2, "Computing the eigenvectors of a (hopefuly) generic element `gen' of `A'");
	    // Heuristic:
	    // At first we try with 1,2,3 coefficients to hopefully
	    // simplify the following computations, and then
	    // increase progressively the bound d on the coefficients
	    r := random(d);
	    x := dom::fromList([[R(r()+1), b] $ b in basis]);
	    
	    M := dom::leftMatrix(x);
	    eigenspaces := linalg::eigenvectors(M);
	    dimensions := map(eigenspaces, op, 2);
	    eigenspaces := map(eigenspaces, op, 3);
	    if _plus(op(dimensions)) < dom::dimension then
		error("There is an element of the algebra which is not diagonalized by the minimal idempotents. Is the algebra really commutative and radical-free?");
	    end_if;
	    if nops(eigenspaces) = 1 then
		userinfo(1, "Bad luck: `gen' did not split `A' at all (it must be a multiple of the identity. Trying again with another element");
		next;
	    end_if;
	    if {op(dimensions)} = {1} then
		userinfo(1, "`A' was completely splitted in ".dom::dimension." minimal idempotents by `gen'. Cool!");
	    else
		userinfo(1, "A splits into subalgebras of dimensions ".
			 expr2text(dimensions).".");
		userinfo(2, "Computing recursively their idempotents.");
	    end_if;
	    return
	    (map(eigenspaces,
		 prog::bless
		 (proc(eigenvectors: Type::ListOf(dom))
		  begin
		      eigenvectors := map(eigenvectors, dom::fromSparseVector);
		      if nops(eigenvectors) = 1 then
			  // This trivial case is the most common one;
			  // we treat it separately for efficiency
			  return(renormalizeIdempotent(eigenvectors[1]));
		      end_if;
		      eigenspace := SubalgebraFromRowReducedBasis(eigenvectors);
		      eigenspace::toA(i) $ i in eigenspace::minimalIdempotents()
		  end_proc,
		  dom)));
	end_while;
    end_proc;

    initDomainAdd :=
    proc()
    begin
	// WARNING: !!!!
	// Hack to try to ensure that the ranker is synchronized with list.
	dom::basis::list();
    end_proc;

begin
        userinfo(1, "Creating Cat::FiniteDimensionalAlgebraWithBasis(".expr2text(args()).")");
end_category:
