/*****************************************************************************
  $Id: GradedAlgebraWithBasis.mu,v 1.1.1.3 2005/01/26 13:42:22 nthiery Exp $
  File:		lib/DOMAINS/CATEGORY/GradedAlgebraWithBasis.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Purpose:	Source code for the category Cat::GradedAlgebraWithBasis
  Created:	Wed Sep  3 14:25:56 2003
  License:	LGPL
  Project:	MuPAD-Combinat, http://mupad-combinat.sf.net
*****************************************************************************/

/**
 * Cat::GradedAlgebraWithBasis -- the category of graded algebras over a distinguished basis
 *
 * A Cat::GradedAlgebraWithBasis is a 
 *
 *
 **/

category Cat::GradedAlgebraWithBasis(R)
    category Cat::AlgebraWithBasis(R), Cat::GradedModuleWithBasis(R);

    info_str := "Cat::GradedAlgebraWithBasis -- the category of graded algebras over a distinguished basis";

    interfaceAdd := { hold(expTruncated), hold(logTruncated) };

    // Truncated exponnential function for a graded algebra
    expTruncated :=
    proc(x: dom, order = ORDER: Type::NonNegInt) : dom
        local result, xp, k;
    begin
	if not iszero(ground(x)) then error("Need a zero constant term!"): end;
        result := dom::one;
        xp := dom::one;
        for k from 1 to order do
            xp := dom::truncate(xp*x/k, order);
            result := result + xp;
        end;
    end;

    // Truncated logarithmic function for a graded algebra
    logTruncated :=
    proc(x: dom, order = ORDER: Type::NonNegInt) : dom
	local result, xp, k, a;
    begin
	result := x::dom::zero;
	a := ground(x);
	if iszero(a) then error("Need a non zero constant term!"): end;
	x := x::dom::one - x/a;
	xp := -x::dom::one;
	for k from 1 to order do
	    xp := dom::truncate(xp*x, order);
	    result := result + xp/k;
	end;
    end;


end_category:
