/*****************************************************************************
  $Id: GradedHopfAlgebraWithBasis.mu,v 1.1.1.3 2005/01/26 13:42:22 nthiery Exp $
  File:		lib/DOMAINS/CATEGORY/GradedHopfAlgebraWithBasis.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Purpose:	Source code for the category Cat::GradedHopfAlgebraWithBasis
  Created:	Wed Sep  3 14:25:48 2003
  License:	LGPL
  Project:	MuPAD-Combinat, http://mupad-combinat.sf.net
*****************************************************************************/

/**
 * Cat::GradedHopfAlgebraWithBasis -- the category of graded Hopf algebras over a distinguished basis
 **/

category Cat::GradedHopfAlgebraWithBasis(R : DOM_DOMAIN)

    category Cat::GradedAlgebraWithBasis(R), 
             Cat::HopfAlgebraWithBasis(R);

    info_str := "Cat::GradedHopfAlgebraWithBasis -- the category of graded Hopf algebras over a distinguished basis";

    interfaceAdd := { hold(checkCompatProdCoprod) };

    // WARMING: this assumes that the Hopf algebra is connected (that
    // is that the homogeneous component of degree zero consists only
    // of scalars). All our graded Hopf algebras are graded for the
    // moment, but this will certainly change in the future!
    counit  := dom::ground;

    antipodeBasis := // calcul automatique de l'antipode
    proc(el : dom::basisIndices) 
	local deg, sqCart, sqConstr, sqPlus, domMOne, curDeg, res, curRes,
	      domMult, domCoprod, workList, pair;
    begin
	el  := dom::term(el); // do it by linarity because of the degree.
	deg := dom::degree(el);
	if deg = 0 then return(el); end_if;
	res := dom::_negate(el);
	if deg = 1 then return(res); end_if;
	sqConstr := dom::tensorConstructor;
	sqCart   := dom::toCartesianProduct;
	sqPlus   := dom::tensorSquare::plus;
	domMult  := dom::mult;
	domMOne  := dom::_negate(dom::one);
	domCoprod := dom::coproduct;
	workList := sqCart(sqPlus(domCoprod(el),
				  sqConstr(domMOne, el),
				  sqConstr(el, domMOne)));
	for curDeg from 2 to deg do
	    if workList=[] then break; end_if;
	    curRes := dom::plus(dom::mult(pair[1], pair[2]) $ pair in workList);
	    if curDeg mod 2 = 1 then
		curRes := dom::_negate(curRes);
	    end_if;
	    res := dom::plus(res, curRes);
	    workList := map(workList,
	        proc(pair)		
		    local res, t1, t2;
		begin
		    [t1, t2] := pair;
		    res := sqCart(sqPlus(domCoprod(t2),
					 sqConstr(domMOne, t2),
					 sqConstr(t2, domMOne)));
		    op(map(res, (tens) -> [domMult(t1, tens[1]), tens[2]]));
		end_proc);
	end_for;
	assert(workList=[]);
	res;
    end_proc;

    projectPrimitifBasis := // compute log^*(id)
    proc()
    begin
    end_proc;
	
    checkCompatProdCoprod :=
    proc(n,m)
// what if typeDegree is undeclared ???	check with the person who added this line...
//    proc(n: dom::typeDegree, m: dom::typeDegree) 
	local p1, p2;
    begin
      for p1 in dom::basis::list(n) do
        for p2 in dom::basis::list(m) do
          if not(iszero(dom::coproduct(dom::mult(p1,p2))-
             dom::tensorSquare::mult(dom::coproduct(p1),dom::coproduct(p2))))
          then return(p1,p2,FALSE);
          end_if;
        end_for;
      end_for;
      return(TRUE);
    end_proc;

end_category:
