/*****************************************************************************
  $Id: GradedModuleWithBasis.mu,v 1.1.1.2 2005/01/26 13:42:22 nthiery Exp $
  File:		lib/DOMAINS/CATEGORY/GradedModuleWithBasis.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Purpose:	Source code for the category Cat::GradedModuleWithBasis
  Created:	Tue Dec  9 21:05:11 2003
  License:	LGPL
  Project:	MuPAD-Combinat, http://mupad-combinat.sf.net
*****************************************************************************/

category Cat::GradedModuleWithBasis(R)
    category Cat::ModuleWithBasis(R);

    info_str := "Cat::GradedModuleWithBasis -- the category of graded modules with a distinguished basis";

    interfaceAdd :=
        { hold(degree), hold(degreeBasis), hold(isHomogeneous),
	  hold(homogeneousComponent), hold(homogeneousComponents),
	  hold(truncate)
	 };

    typeDegree := Type::AnyType;

    degreeBasis :=
    if dom::basisIndices::size <> FAIL then
	// the id@ prevents changing dom in dom::basisIndices::degree
	id@dom::basisIndices::size;
    elif dom::basisIndices::degree <> FAIL then
	// the id@ prevents changing dom in dom::basisIndices::degree
	id@dom::basisIndices::degree;
    else
	degree; // or nothing, or toBeDefined ???
    end_if;

    degree :=
    proc(x: dom) : dom::typeDegree
	local s;
    begin
	max(0,dom::degreeBasis(s, args(2..args(0))) $ s in dom::support(x));
    end_proc;

    isHomogeneous :=
    proc(p: dom) : DOM_BOOL
	local v;
    begin
	bool(nops({ dom::degreeBasis(v) $ v in dom::support(p)}) <= 1);
    end_proc;

    homogeneousComponents :=
    proc(p: dom) : dom::TableOf(dom::typeDegree, dom)
	local v, res, degMon;
    begin
	res := table();
	for v in dom::monomials(p) do
	    degMon := dom::degree(v);
	    if contains(res, degMon) then
		res[degMon] := dom::plus(res[degMon], v);
	    else
		res[degMon] := v;
	    end_if;
	end_for;
	res;
    end_proc;

    homogeneousComponent :=
    proc(p: dom, d: dom::typeDegree) : dom
    begin
	dom::selectSupport(p, (deg->bool(d=deg)) @ dom::degreeBasis);
    end_proc;

    truncate :=
    proc(p: dom, order=ORDER: dom::typeDegree) : dom
    begin
	dom::selectSupport(p, t->dom::degreeBasis(t)<order)
    end;

end_category:
