/*****************************************************************************
  $Id: HopfAlgebraWithBasis.mu,v 1.1.1.3 2005/01/26 13:42:22 nthiery Exp $
  File:		lib/DOMAINS/CATEGORY/HopfAlgebraWithBasis.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
                Florent Hivert <Florent.Hivert@univ-mlv.fr>
  Purpose:	Source code for the category Cat::HopfAlgebraWithBasis
  Created:	Wed Sep  3 13:58:38 2003
  License:	LGPL
  Project:	MuPAD-Combinat, http://mupad-combinat.sf.net
*****************************************************************************/

/**
    Cat::HopfAlgebraWithBasis(R)

    R: ring

    For now, R needs to be commutative.
**/
alias(declareConversion = operators::overloaded::declareConversion):
alias(declareSignature  = operators::overloaded::declareSignature);

category Cat::HopfAlgebraWithBasis(R)
    category Cat::AlgebraWithBasis(R);

    info_str := "Cat::HopfAlgebraWithBasis -- the category of Hopf algebras over a distinguished basis";

    interfaceAdd :=
	{
	 hold(coproduct), hold(antipode), hold(mu), hold(counit),
	 hold(tensorSquare)
	};

    //////////////////////////////////////////////////////////////////////////
    // Algebraic operations
    //////////////////////////////////////////////////////////////////////////

    tensorSquare       := Dom::TensorProductOfFreeModules([dom, dom]);
    tensorConstructor  := dom::tensorSquare::tensorConstructorFromSignature([dom, dom]) @ DOM_LIST;
    toCartesianProduct := id @ dom::tensorSquare::toCartesianProductFromSignature([dom, dom]);

    mu        := // linearisation of the product
    proc(el : dom::tensorSquare)
    begin
	dom::plus(op(map(dom::toCartesianProduct(el), dom::mult@op)));
    end_proc;
//     operators::makeLinear // the previous implementation is ~15% faster 
//     (prog::bless(proc(t: dom::basisIndices) : dom::tensorSquare
// 		    begin dom::mult(dom::term(p) $ p in t); end_proc,
// 		    dom), Source=dom::tensorSquare, ImageSet=dom);

    coproduct := if dom::coproductBasis <> FAIL then
		     operators::makeLinear
		     (dom::coproductBasis,
		      Source=dom, ImageSet=dom::tensorSquare);
		 else
		     dom::coproductByOverloading := TRUE;
		     proc(x : dom)
		     begin 
			 dom::tensorSquare::convert(operators::coproduct(x));
		     end_proc;
		 end_if;

    antipode := if dom::antipodeBasis <> FAIL then
		   operators::makeLinear
		   (dom::antipodeBasis,
		    Source=dom, ImageSet=dom);
		 else
		     dom::antipodeByOverloading := TRUE;
		     proc(x : dom)
		     begin 
			 dom::convert(operators::antipode(x));
		     end_proc;
		 end_if;


    // Default initialization using the overloaded and conversion
    // mechanism from operators::overloaded.

    // A domain in this category can set its own initialization by
    // overriding initDomainHopfAlgebraWithBasis

    initDomainAdd := dom::initDomainHopfAlgebraWithBasis;
    initDomainHopfAlgebraWithBasis :=
    proc()
	option escape;
    begin
	// This part should probably be done by Cat::CombinatorialBialgebra
	dom::coproduct; // force evaluation of dom::coproduct
	if dom::coproductByOverloading <> TRUE then
	    declareSignature // coproduct
	    (operators::coproduct, [dom], dom::coproduct);
	end_if;

	declareSignature // linearization of the product
	(operators::mu, [dom::tensorSquare], dom::mu);
	
	dom::antipode; // force evaluation of dom::antipode
	if dom::antipodeByOverloading <> TRUE then 
	    declareSignature // antipode
	    (operators::antipode, [dom], dom::antipode);
	end_if;
    end_proc;

end_category:
