/*****************************************************************************
  $Id: HopfAlgebraWithSeveralBases.mu,v 1.1.1.2 2005/01/26 13:42:22 nthiery Exp $
  File:		lib/DOMAINS/CATEGORY/HopfAlgebraWithSeveralBases.mu
  Authors:	Florent Hivert <florent.hivert@univ-mlv.fr>
  Purpose:	Source code for the category Cat::HopfAlgebraWithSeveralBases
  Created:	Thu Sep 23 15:02:58 2004
  License:	LGPL
  Project:	MuPAD-Combinat, http://mupad-combinat.sf.net
*****************************************************************************/

alias(declareConversion = operators::overloaded::declareConversion):

category Cat::HopfAlgebraWithSeveralBases(CoeffRing : DOM_DOMAIN)
    category Cat::AlgebraWithSeveralBases(CoeffRing);
          // TODO: add Cat::HopfAlgebra as soon as it exists
    
    info_str := "Cat::HopfAlgebraWithSeveralBases -- the category of Hopf algebras expanded on several bases";

    interfaceAdd :=
        {hold(antipode),
	 hold(mu),
	 hold(coproduct),
	 hold(tensor)
	};

    antipode       := operators::antipode;
    mu             := operators::mu;
    coproduct      := operators::coproduct;
    tensor         := operators::tensor;

    initDomainAdd := dom::initDomainHopfAlgebraWithSeveralBases;
    initDomainHopfAlgebraWithSeveralBases :=
    proc()
	local chngt, source, target, basisChange;
    begin
	// Extend the known basis changes A -> B to
	// basis changes A \otimes A -> B\otimes B
	// Caveats:
	//  - this could go in a category Cat::BigebraWithSeveralBases
	//  - basis changes declared a posteriori wont be extended
	//    we should overload declareBasisChange
	//  - this makes a lot of conversions
	//  - we don't get automatically conversions between mixed tensor
	//    products: A \otimes A' -> B \otimes A'
	for chngt in dom::basisChanges do
	    source   := (op(chngt, [1, 1]))::tensorSquare;
	    target   := (op(chngt, [1, 2]))::tensorSquare;
	    basisChange := op(chngt, 2);
	    declareConversion(source, target,
			      operators::tensorProductOfMaps
			      ([basisChange, basisChange],
			       source, target));
	end_for;
    end_proc;
end_category:
