/*****************************************************************************
  $Id $
  File:		lib/DOMAINS/CATEGORY/TEST/ModuleWithSeveralBases.tst
  Authors:	Florent Hivert <florent.hivert@univ-mlv.fr>
  Purpose:	Test suite for the category Cat::ModuleWithSeveralBases
  Created:	Fri Jan 30 18:03:00 2004
  License:	LGPL
  Project:	MuPAD-Combinat, http://mupad-combinat.sf.net
*****************************************************************************/

prog::testinit("ModuleWithSeveralBases", 2.2):

prog::testfunc(Cat::ModuleWithSeveralBases::initDomain):

//ring := Dom::UnivariatePolynomial(Unk, Dom::Rational):
ring := Dom::IntegerMod(7): Unk := ring(3):
domain subsetABC 
    category Cat::FreeModule(ring);
    inherits Dom::FreeModule(ring, 
			     combinat::subClass(combinat::subsets, Parameters = {A, B, C}));
end_domain:

// subsetInABC(Set) = \sum_{X \subset Set} X 
domain subsetInABC 
    category Cat::FreeModule(ring);
    inherits subsetABC;
end_domain:

// Dual Basis. 
// subsetOutABC(Set) = \sum_{Set\subset X} (-1)^(nops(X)-nops(Set)) X 
domain subsetOutABC
    category Cat::FreeModule(ring);
    inherits subsetABC;
end_domain:

// Do the creation under prog::test
//prog::test
((
domain test
    inherits Dom::BaseDomain;
    category Cat::ModuleWithSeveralBases(ring);

    autoDefineBasisChanges := FALSE; // For tests.
    shC := subsetABC; // To check that shortcuts work. 
    basisChangesBasis := table(
        (subsetInABC, dom::shC) = 
		// subsetInABC(Set) = \sum_{X \subset Set} X 
	    proc(set : subsetInABC::basisIndices) : dom::shC
		local xSet;
	    begin
		dom::shC::plus(dom::shC::term(xSet) $ 
				xSet in combinat::subsets::list(set));
	    end_proc);
    dual := dom;
    dualBasesPairs := {[subsetABC, subsetABC]};
end_domain:
domtype(test)),		   
DOM_DOMAIN):

// check that the bases are declared. 
prog::test(test::bases,
           {subsetInABC, subsetABC}):
// Test that the conversion works. 
prog::test(bool(subsetABC(subsetInABC({})) = subsetABC({})),
           TRUE):
prog::test(bool(subsetABC(subsetInABC({A})) = subsetABC({}) + subsetABC({A})),
           TRUE):
prog::test(bool(subsetABC(subsetInABC({A,B})) = subsetABC({}) + subsetABC({A})
					      + subsetABC({B}) + subsetABC({A,B})),
           TRUE):

//////////////////////////////////////////////////////////////////////////////
// A tentative to create an element of test raise an error. 
prog::testfunc(Cat::ModuleWithSeveralBases::new):
prog::test(test({a,b,c}), TrapError = 1028):


//////////////////////////////////////////////////////////////////////////////
prog::testfunc(Cat::ModuleWithSeveralBases::testtype):
prog::test(testtype({a, b, c}, test),
           FALSE):
prog::test(testtype(subsetABC({A, B}), test),
           TRUE):
prog::test(testtype(subsetInABC({A, B}), test),
           TRUE):


//////////////////////////////////////////////////////////////////////////////
prog::testfunc(Cat::ModuleWithSeveralBases::basisChangeMatrix):
prog::test(test::basisChangeMatrix(subsetInABC, subsetABC), 
           Dom::Matrix(ring)([[1, 1, 1, 1, 1, 1, 1, 1], 
				       [0, 1, 0, 0, 1, 1, 0, 1], 
				       [0, 0, 1, 0, 1, 0, 1, 1], 
				       [0, 0, 0, 1, 0, 1, 1, 1], 
				       [0, 0, 0, 0, 1, 0, 0, 1], 
				       [0, 0, 0, 0, 0, 1, 0, 1], 
				       [0, 0, 0, 0, 0, 0, 1, 1], 
				       [0, 0, 0, 0, 0, 0, 0, 1]])):
prog::test(test::basisChangeMatrix(subsetInABC, subsetABC, [{}, {A}]),
           Dom::Matrix(ring)([[1, 1], [0, 1]])):
prog::test(test::basisChangeMatrix(subsetInABC, subsetABC, [{}, {A, B}]),
	   TrapError = 1028):
prog::test(test::basisChangeMatrix(subsetInABC, subsetABC, [{}, {A}, {B}]),
           Dom::Matrix(ring)([[1, 1, 1], [0, 1, 0], [0, 0, 1]])):
prog::test(test::basisChangeMatrix(subsetInABC, subsetABC, [{}, {A}, {B}], [{B}, {A}, {}]),
           Dom::Matrix(ring)([[0, 0, 1], [0, 1, 0], [1, 1, 1]])):


//////////////////////////////////////////////////////////////////////////////
prog::testfunc(Cat::ModuleWithSeveralBases::invertBasisChange):
prog::test((f := test::invertBasisChange(subsetInABC, subsetABC); f({})),
	   subsetInABC({})):
prog::test(f({A, B}),
           subsetInABC::fromList([[ring(1), {A, B}], [ring(-1), {B}], [ring(-1), {A}], [ring(1), {}]])):
prog::test(f({A, B, C}),
           subsetInABC::fromList([[ring(1), {A, B, C}], [ring(-1), {B, C}], [ring(-1), {A, C}], 
				  [ring(-1), {A, B}], [ring(1), {C}], [ring(1), {B}], 
				  [ring(1), {A}], [ring(-1), {}]])):
for set in combinat::subsets({A,B,C}) do 
    prog::test(iszero(subsetABC(f(set)) - subsetABC(set))):
end_for:

//////////////////////////////////////////////////////////////////////////////
prog::testfunc(Cat::ModuleWithSeveralBases::declareBasisChangeBasis):
test::declareBasisChangeBasis(subsetABC, subsetInABC, 
		     test::invertBasisChange(subsetInABC, subsetABC)): 
prog::test(subsetInABC(subsetABC({A, B})),
           (subsetInABC)::fromList([[ring(-1), {B}], [ring(-1), {A}], [ring(1), {}], [ring(1), {A, B}]])):
prog::test(subsetInABC(subsetABC({})),
           (subsetInABC)::fromList([[ring(1), {}]])):
for set in combinat::subsets({A,B,C}) do 
    prog::test(iszero(subsetABC(subsetInABC(subsetABC(set)))   - subsetABC(set))):
    prog::test(iszero(subsetInABC(subsetABC(subsetInABC(set))) - subsetInABC(set))):
end_for:
prog::test(iszero(subsetABC(subsetInABC(ring(Unk) * subsetABC({A,B,C})))
		  - ring(Unk) * subsetABC({A,B,C}))):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc(Cat::ModuleWithSeveralBases::declareBasis):
prog::test(test::declareBasis(subsetOutABC),
           {subsetOutABC, subsetInABC, subsetABC}):
// Do it twice to check that there is no problem. 
prog::test(test::declareBasis(subsetOutABC),
           {subsetOutABC, subsetInABC, subsetABC}):
prog::test(testtype(subsetOutABC({A, B}), test),
           TRUE):


//////////////////////////////////////////////////////////////////////////////
prog::testfunc(Cat::ModuleWithSeveralBases::transposeBasisChange):
test::declareBasisChangeBasis(subsetABC, subsetOutABC, 
		     test::transposeBasisChange(subsetInABC, subsetABC, 
						subsetABC, subsetOutABC)):
prog::test(subsetOutABC(subsetABC({A, B, C})),
           (subsetOutABC)::fromList([[ring(1), {A, B, C}]])):
prog::test(subsetOutABC(subsetABC({A})),
           (subsetOutABC)::fromList([[ring(1), {A, C}], [ring(1), {A, B}], [ring(1), {A}], [ring(1), {A, B, C}]])):
prog::test(subsetOutABC(subsetABC({})),
           (subsetOutABC)::fromList([[ring(1), {B, C}], [ring(1), {A, C}], [ring(1), {A, B}], 
				     [ring(1), {C}], [ring(1), {B}], [ring(1), {A}], 
				     [ring(1), {}], [ring(1), {A, B, C}]])):
prog::test(subsetOutABC(subsetInABC({})),
           (subsetOutABC)::fromList([[ring(1), {B, C}], [ring(1), {A, C}], [ring(1), {A, B}], 
				     [ring(1), {C}], [ring(1), {B}], [ring(1), {A}], [ring(1), {}], 
				     [ring(1), {A, B, C}]])):
prog::test(subsetOutABC(subsetInABC({A})),
           (subsetOutABC)::fromList([[ring(1), {B, C}], [ring(2), {A, C}], [ring(2), {A, B}], 
				     [ring(1), {C}], [ring(1), {B}], [ring(2), {A}], 
				     [ring(1), {}], [ring(2), {A, B, C}]])):
prog::test(subsetOutABC(subsetInABC({A, B, C})),
           (subsetOutABC)::fromList([[ring(4), {B, C}], [ring(4), {A, C}], [ring(4), {A, B}], 
				     [ring(2), {C}], [ring(2), {B}], [ring(2), {A}], 
				     [ring(1), {}], [ring(8), {A, B, C}]])):


//////////////////////////////////////////////////////////////////////////////
prog::testfunc(Cat::ModuleWithSeveralBases::declareDualBases):
// test for automatic initialisation of the scalar product pair [subsetABC, subsetABC].
prog::test(test::scalar(subsetABC({A, B}), subsetABC({A, B})),
           ring(1)):
// Check that the system takes also care of change of bases. 
prog::test(test::scalar(subsetABC({A, B}), subsetInABC({A, B, C})),
           ring(1)):
// Check that for the moment we do not know how to convert a subsetOutABC into a subsetABC.
prog::test(test::scalar(subsetABC({A, B}), subsetOutABC({A, B, C})),
           TrapError = 1028): 
prog::test(test::scalar(subsetInABC({A, B}), subsetOutABC({A, B, C})),
           TrapError = 1028): 
prog::test((test::declareDualBases(subsetInABC, subsetOutABC); test::dualBasesPairs), 
	   {[subsetABC, subsetABC], [subsetInABC, subsetOutABC], [subsetOutABC, subsetInABC]}):
// Now we can compute the scalar products. 
prog::test(test::scalar(subsetInABC({A, B}), subsetOutABC({A, B})),
           ring(1)): 
prog::test(test::scalar(subsetInABC({A, B}), subsetOutABC({A, B, C})),
           ring(0)): 
prog::test(test::scalar(subsetInABC({A, B, C}), subsetOutABC({A, B})),
           ring(0)): 
prog::test(test::scalar(subsetInABC({A, B}), subsetOutABC({A, B, C})),
           ring(0)): 
prog::test(test::scalar(subsetABC({A, B}), subsetOutABC({A, B, C})),
           ring(0)): 
prog::test(test::scalar(subsetABC({A, B, C}), subsetOutABC({A, B})),
           ring(-1)): 
prog::test(test::scalar(ring(2*Unk)*subsetABC({A, B, C}), ring(Unk+2)*subsetOutABC({A, B})),
           ring(-2*Unk^2 - 4*Unk)): 
prog::test(test::scalar(subsetOutABC({A, B}), subsetABC({A, B, C})),
           ring(-1)): 
// But since the change subsetOutABC => subsetABC is still not declared,
// we still do not know how to compute the scalar product of two subsetOutABC.
prog::test(test::scalar(subsetOutABC({A, B}), subsetOutABC({A, B, C})),
           TrapError = 1028): 
// Correct this problem. 
test::declareBasisChangeBasis(subsetOutABC, subsetABC, 
		     proc(set : subsetOutABC::basisIndices) : subsetABC
			 local xSet;
		     begin
			 subsetABC::plus(subsetABC::monomial(ring((-1)^(nops(xSet)-nops(set))), xSet) 
					 $ xSet in map(combinat::subsets::list({A,B,C} minus set),
						       _union, set));
		     end_proc):
// We compute the scalar product in different ways and check the result
prog::test(test::scalar(subsetOutABC({A, B}), subsetOutABC({A, B, C})),
	   ring(-1)):
prog::test(test::scalar(subsetABC(subsetOutABC({A, B})), 
			subsetABC(subsetOutABC({A, B, C}))),
	   ring(-1)):
prog::test(test::scalar(subsetInABC(subsetOutABC({A, B})), 
			subsetOutABC({A, B, C})),
	   ring(-1)):
prog::test(test::scalar(subsetOutABC({A, B}), 
			subsetInABC(subsetOutABC({A, B, C}))),
	   ring(-1)):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc(Cat::ModuleWithSeveralBases::autoDefineBasisChanges):
// Do the same automagically :-)								  
ring := Dom::Rational:
domain subsetXs 
    category Cat::FreeModule(ring);
    inherits Dom::FreeModule(ring, combinat::subClass(combinat::subsets, Parameters = {x.i $ i=1..4}));
end_domain:
domain subsetInXs
    category Cat::FreeModule(ring); inherits subsetXs;
end_domain:
domain subsetOutXs
    category Cat::FreeModule(ring); inherits subsetXs;
end_domain:

//setuserinfo(All, 2);

prog::test((
domain test2
    inherits Dom::BaseDomain;
    category Cat::ModuleWithSeveralBases(ring);
///////////////////////////////////////
    autoDefineBasisChanges := TRUE;
///////////////////////////////////////
    shC := subsetXs; // To check that shortcuts work. 
    basisChangesBasis := table(
        (subsetInXs, dom::shC) = 
	    proc(set : subsetInXs::basisIndices) : dom::shC
		local xSet;
	    begin
		dom::shC::plus(dom::shC::term(xSet) $ 
				xSet in combinat::subsets::list(set));
	    end_proc);
    dual := dom;
    dualBasesPairs := [[subsetInXs, subsetOutXs], [subsetXs, subsetXs]];
end_domain:
domtype(test2)), DOM_DOMAIN):
prog::test(test2::bases, {subsetXs, subsetOutXs, subsetInXs}):
for bas1 in test2::bases do
    for bas2 in test2::bases do
	prog::test(iszero(bas1(bas2(bas1({x1, x3})))
			  - bas1({x1,x3})));
	prog::test(test2::scalar(bas1({x1}), bas2({x1, x2})) -
		   test2::scalar(test2::shC(bas1({x1})), test2::shC(bas2({x1, x2}))),
		   ring(0));
    end_for;
end_for:
//////////////////////////////////////////////////////////////////////////////
prog::testfunc(Cat::ModuleWithSeveralBases):
prog::testexit():
//////////////////////////////////////////////////////////////////////////////

prog::check(test, 3):
prog::check(test2, 3):


// Local Variables:
// mode: mupad
// End:
