/**
  * DomainConstructor::getEntryFromAllSuperDomains(A, "bla")
  *
  * Look for all the definitions of the entry "bla" in the constructor
  * of the domain A, and in all the constructors of its super domains;
  * evaluate them in the context of their respective domains, and, for
  * procedures, bless them as methods of A; finally, return them as a
  * list, starting from A.
  *
  * Largely inspired from DomainConstructor::getSuperDomainEntry.
  **/

DomainConstructor::getEntryFromAllSuperDomains :=
proc(DOM: DOM_DOMAIN, e: DOM_STRING) : DOM_LIST
    option noDebug;
    local SD, r, allDefinitions;
begin
    allDefinitions := [];
    for SD in [DOM, op(DOM::super_domains)] do
        if (r := DomainConstructor::getEntryFromSuperDomain(DOM, SD, e)) <> NIL then
	    allDefinitions := append(allDefinitions, r);
	end_if;
    end_for;
    allDefinitions;
end_proc:
