/*****************************************************************************
  $Id: AlgebraWithBasis_Table.mu,v 1.1.1.2 2005/01/26 13:42:22 nthiery Exp $
  File:		lib/DOMAINS/DOMAIN/AlgebraWithBasis_Table.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Created:	16/02/2001
  License:	LGPL
*****************************************************************************/

// TODO: fusion with experimental::AlgebraWithBasis, under the name
// Dom::AlgebraWithBasis.

unalias(REP,TYPE_ERR):
alias(REP(x)=extop(x,1),
      TYPE_ERR=error("wrong type of argument")
      ):
alias(mult2		= operators::mult2):
alias(declareDomain     = operators::overloaded::declareDomain):
alias(declareConversion = operators::overloaded::declareConversion):
alias(declareSignature  = operators::overloaded::declareSignature);

domain Dom::AlgebraWithBasis_Table(R: DOM_DOMAIN,
                                   S: DOM_DOMAIN
                                  )
    local Options;
    inherits Dom::FreeModuleTable(R,S);
    category
        Cat::AlgebraWithBasis(R),
        (if Options[hold(BasisMonoidMult)]<>FAIL and 
            domtype(S)=DOM_DOMAIN and S::hasProp<>FAIL and
	    (S::hasProp(Cat::OrderedMonoid) or
	     S::hasProp(Cat::OrderedAbelianMonoid))
						  then 
             Cat::FiniteMonoidRing(R,S);
         end_if);

    axiom
        (if Options[hold(BasisMonoidMult)]<>NIL then 
             Ax::monoidAlgebra;
         end_if);

    info_str := "Dom::AlgebraWithBasis_Table: domain for algebras with a basis, implemented with tables";

    //////////////////////////////////////////////////////////////////////////
    //
    //////////////////////////////////////////////////////////////////////////

    options := Options;

    oneBasis  := Options[hold(BasisOne)];
    //This ain't a good idea: assigning to dom::mult2Basis will change the
    //value of dom if it's used inside mult2Basis!
    //mult2Basis := Options[hold(BasisMult)];
    //basisMonoidMult := Options[hold(BasisMonoidMult)];

    one := dom::monomial(R::one, dom::oneBasis);

    mult2 :=
    if dom::hasProp(Ax::monoidAlgebra) then
        proc(x1: dom, x2: dom)
	    local t, s1, s2;
        begin
	    t := table();
            for s1 in REP(x1) do
                for s2 in REP(x2) do
                    t:=dom::addToRep(t,
                                     Options[hold(BasisMonoidMult)](op(s1,1), op(s2,1)),
                                     R::_mult(op(s1,2),op(s2,2)));
                end_for;
            end_for;
            return(new(dom, t));
        end_proc;
    else
	dom::makeBilinearProduct(Options[hold(BasisMult)]);
    end_if;

    // _invert
    // Following the specifications of Cat::Monoid, tries to compute
    // an inverse for x, and return FAIL else.
    // This can only be done for monomials of the form c x^s,
    // where c is invertible in R, s is "negatible" in S, and
    // s is invariant under the action group.

    _invert :=
    proc(x: dom)
	local s, c;
    begin
	x:=REP(x);
	if nops(x)=1 then
	    if (s:=S::_negate(op(x,[1,1]))) <> FAIL and
	       (c:=R::_invert(op(x,[1,2]))) <> FAIL and
	       S::orbitSize(s)=1
	       then
		return(dom::monomial(c,s));
	    end_if;
	end_if;
	FAIL;
    end_proc;

    divides :=
    proc(m1: S, m2: S)
	local i, m;
    begin
	for i from 1 to nops(m1) do
	    if m1[i]>m2[i] then
		return(FALSE);
	    end_if;
	end_for;
	m:=m2-m1;
	if dom::chainProduct and
	    S::shape(m2)<>zip(S::shape(m1),S::shape(m), _plus) then
	    return(FALSE);
	end_if;
	return(S::isCanonic(m));
    end_proc;

begin
    if (domtype(R)<>DOM_DOMAIN) or (R::hasProp=FAIL) then
        error("argument 1 must be a domain");
    end_if;

    Options:=table(hold(BasisOne) = toBeDefined, 
                   hold(BasisMult) = NIL,
		   hold(BasisMonoidMult) = NIL
                  );
    if domtype(S) = DOM_DOMAIN and S::hasProp<>FAIL then
        if S::hasProp(Cat::Monoid) then
            Options[hold(BasisOne)]:=S::one;
            Options[hold(BasisMonoidMult)]:=S::_mult;
        elif S::hasProp(Cat::AbelianMonoid) then
            Options[hold(BasisOne)] := S::zero;
            Options[hold(BasisMonoidMult)] := S::_plus;
        end_if;
    end_if;
    
    Options := combinat::getOptions(Options, 2, Error);

    if Options[hold(BasisOne)] = NIL then
        error("option BasisOne needed");
    end_if;;
    if Options[hold(BasisMult)] = NIL and Options[hold(BasisMonoidMult)] = NIL then
        error("option BasisMult or BasisMonoidMult needed");
    end_if;;
end_domain:

unalias(REP,TYPE_ERR):
