/*****************************************************************************
  $Id: MinPlusSemiRing.mu,v 1.1.1.2 2004/10/04 17:25:13 nthiery Exp $
  File:		lib/DOMAINS/DOMAIN/MinPlusSemiRing.mu
  Authors:	Eric Laugerotte <Eric.Laugerotte@univ-rouen.fr>
  Created:	2002/10/25
  License:	LGPL
*****************************************************************************/

/*++

Dom::MinPlusSemiRing --  the domain of the MinPlus semi-ring 

Domains: 
Dom::BaseDomain

Categories:
Cat::SemiRing

Methods:
zero         	    - returns the neutral element for +
one	    	    - returns the neutral element for *
convert_to(x,T)	    - converts x to an element of type T
_plus(x1,x2,...)    - returns the sum of x1+x2+...
_mult(x1,x2,...)    - returns the product of x1*x2*...
_power(x,i)   	    - returns x*x*...*x (i times) for a positive integer i
star(x)		    - returns the star of x

++*/

domain Dom::MinPlusSemiRing
    inherits Dom::BaseDomain;
    category Cat::SemiRing;
    axiom Ax::canonicalRep,Ax::normalRep;

    interfaceAdd := { hold(star) };

    /////////////////////////////////////////////////////////////////////////
    // Constructors and constants
    /////////////////////////////////////////////////////////////////////////

    new :=
    if args(0)=0 then
        proc(n : Type::Union(Type::Constant,stdlib::Infinity))
        begin
	    return(new(dom,n))
        end_proc
    end_if;

    convert :=
    if args(0)=0 then
        proc(n : Type::Union(Type::Constant,stdlib::Infinity))
        begin
	    return(new(dom,n))
        end_proc
    end_if;

    convert_to :=
    proc(x : dom,
	 T : DOM_DOMAIN)
    begin
	if x=dom::zero then
	   return(T::zero)
	elif x=dom::one then
	     return(T::one)
	end_if;	
	return(T(extop(x,1)))
    end_proc;

    print :=
    proc()
    begin
	return(extop(args(1),1))
    end_proc;

    expr := 
    proc(x : dom)
    begin
	return(extop(x,1))
    end_proc;

    zero := dom::new(infinity);
       
    one := dom::new(0);

    /////////////////////////////////////////////////////////////////////////
    // Arithmetic operations
    /////////////////////////////////////////////////////////////////////////

    _plus :=
    proc()
        local x,i;
    begin
	if args(0)=0 then
	    return(dom::zero)
	end_if;
	x:=infinity;
	for i from 1 to args(0) do 
	    x:=simplify(min(x,extop(args(i),1)))
	end_for;
	return(dom::new(x))
    end_proc;

    _mult :=
    proc()
	local x,i;
    begin
	if args(0)=0 then
	    return(dom::zero)
	end_if;
	x:=0;
	for i from 1 to args(0) do
	    x:=_plus(x,extop(args(i),1))
	end_for;
	return(dom::new(x))
    end_proc;

    _invert := 
    proc(x : dom)
    begin
    	if x=dom::one then
	    return(dom::one)
	end_if;
	return(FAIL)
    end_proc;

    _power := domains::repeatedSquaring(dom,"_mult");

    star :=
    proc(x : dom)
    begin
	if is(op(x,1),Type::Negative) then
	    return(dom::new(-infinity))
	end_if;
	return(dom::one)
    end_proc;

end_domain:






















