/*++ $Id: PermutationGroupDirectProduct.mu,v 1.1.1.3 2003/10/21 22:39:50 nthiery Exp $

Dom::PermutationGroupDirectProduct([G1,...,Gk]) --
	Given finite groups G1,...,Gk acting on finite sets
	S1,...,Sk, construct the action of the direct product G1,...,Gk
	on the cartesian product S1 x ... x Sk.

Author:	     Nicolas M. Thiry <nthiery@users.sourceforge.net>
Created:     June 2000
Last update: $Date: 2003/10/21 22:39:50 $

++*/

unalias(REP):
alias(REP(x)=extop(x,1)):

domain Dom::PermutationGroupDirectProduct(Groups: Type::ListOf(DOM_DOMAIN))
    local Opts, group;
    inherits Dom::PermutationGroup(Card, toBeDefined);
    category Cat::PermutationGroup;

    options := Opts;

    size := _mult((Groups[i])::size $ i=1..nops(Groups));

/*--
    set

    All the tuples on which the Group acts, returned as a sequence.
--*/

    set := [op(combinat::cartesianProduct::list((Groups[i])::set $ i=1..nops(Groups)))];

    compactName := 
    proc(e)
	option remember;
    begin
	_concat(op(zip(e,Groups, (x,f) -> f::compactName(x))));
    end_proc;

/*--
--*/

    induced :=
    proc(s: DOM_LIST)
	local pind;
    begin
	pind:=[ dom::invertSet[ zip(x,s, (y,f) -> f(y)) ]
		$ x in dom::set ];
	new(dom, pind);
    end_proc;

    allElements :=
    proc()
    begin
	map(
	combinat::cartesianProduct::list
                       ((Groups[i])::allElements() $ i=1..nops(Groups)),
	dom::induced);
    end_proc;

    generators := map(
	combinat::cartesianProduct::list
                      (append((Groups[i])::generators,(Groups[i])::one)
                       $ i=1..nops(Groups)),
	dom::induced);

    act :=
    proc(g:dom, e)
    begin
	if testargs() and not contains(dom::invertSet,e) then
		error("The 2nd argument must belong to the set on which the permutation acts");
	end_if;
	dom::set[op(extop(g,1),dom::invertSet[e])]
    end_proc;

    func_call := 
    proc(g:dom, element)
    begin
	if testargs() and args(0)<>2 then
		error("wrong no of arguments");
	end_if;
	element:=context(element);
	if testtype(element, DOM_INT) or testtype(element, DOM_LIST) then
	// BUG due to the horrors of dynamic variable scoping,
	// func_calls which automatically hold their variables,
	// and corresponding uses of context.
	// <element> is going to be evaluated by the following function,
	// in the context of this procedure, instead of the original
	// context. So if <element> contains some identifier which
	// conflicts with a variable of this procedure, the result will
	// be incorrect.
		dom::func_call_inherits(g,element);
	else
		dom::act(g,element);
	end_if;
    end_proc;

/*--
   cycleTypes():

   Count the elements of the group by cycle type.
   (Cf Cat::PermutationGroupModule).

   Let g be the permutation induced by the permutations h1,...,hk. The
   cycle type of g depends only on the cycle types of h1,...,hk. This
   yields a fast computation of the cycleTypes of this group from the
   cycleTypes of the underlying groups.
--*/

    cycleTypes :=
    proc()
	option remember;
	local recfun, card;
    begin
	userinfo(3, "cycleTypes: starting computation");
	card:=dom::card;
	recfun :=
	proc(g)
	    local i1, i2, cycleType1, cycleType2, cycleTypes2, ct1, ct2, a, l,t;
	begin
	    if args(0) = 1 then
		return(g::cycleTypes());
	    elif args(0) = 2 then
		cycleTypes2:=(args(2))::cycleTypes();
	    else
		cycleTypes2:=recfun(args(2..args(0)));
	    end_if;
	    t:=table();
	    for cycleType1 in g::cycleTypes() do
		ct1:=op(cycleType1,1);
		for cycleType2 in cycleTypes2 do
		    ct2:=op(cycleType2,1);
		    a:=[0 $ card];
		    for i1 from 1 to nops(ct1) do
			if ct1[i1]>0 then
			    for i2 from 1 to nops(ct2) do
				if ct2[i2]>0 then
				    l:=lcm(i1,i2);
				    a[l]:=a[l]+gcd(i1,i2)*ct1[i1]*ct2[i2];
				end_if
			    end_for;
			end_if;
		    end_for;
		    if contains(t,a) then
			t[a]:=t[a]+op(cycleType1,2)*op(cycleType2,2);
		    else
			t[a]:=op(cycleType1,2)*op(cycleType2,2);
		    end_if;
		end_for;
	    end_for;
	    return(t);
	end_proc;
	return(recfun(op(Groups)));
    end_proc;

begin
    if testargs() then
	for group in Groups do
	    if not group::hasProp(Cat::PermutationGroup) then
		error("Groups must be Cat::PermutationGroup's");
	    end_if;
	end_for;
    end_if;
    Opts:=table(args(2..args(0)));
    Card:=_mult(group::card $ group in Groups);
end_domain;

unalias(REP):

// end of file
