/*****************************************************************************
  $Id: PermutationGroup_GLIP.mu,v 1.1.1.2 2003/09/19 15:11:40 nthiery Exp $
  File:		lib/DOMAINS/DOMAIN/PermutationGroup.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Taken from:	PerMuVAR, 02/12/2001
  License:	LGPL
*****************************************************************************/

/**
Dom::PermutationGroup_GLIP - fast implementation of permutation groups using GLIP

Dom::PermutationGroup_GLIP provides an alternative implementation of
several expensive methods in Cat::PermutationGroup, using the external
C++ dynamic module GLIP.

Maybe this should be a category. But this does not work well with
inheritance.

Caveat: as soon as this domain is accessed, it's created and the
dynamic module GLIP is loaded !

**/

alias(GLIP=combinat::GLIP):
domain Dom::PermutationGroup_GLIP(n: Type::PosInt,
                             generators=FAIL: Type::Union(DOM_LIST,
                                                          DOM_FAIL,
                                                          DOM_IDENT
                                                          ))

    inherits Dom::BaseDomain;
  
    fastGroup	:= NIL;

    // GLIP::new cannot accept an empty list of generators

    CGroup	:= GLIP::new(if nops(dom::generators)=0 then
				 [[$1..dom::card]]
			     else
				 map(dom::generators, expr)
			     end_if);

    size	:= GLIP::size(dom::CGroup);
    
    allElements :=
    proc()
    begin
	(map(GLIP::allElements(dom::CGroup),dom));
    end_proc;
    
    isCanonic	:= 
    () ->GLIP::isCanonic(dom::CGroup, args());

    canonic := 
    () -> GLIP::canonic(dom::CGroup, args());

    orbit := 
    () -> GLIP::orbit(dom::CGroup, args());

    orbitSet := 
    () -> GLIP::orbitSet(dom::CGroup, args());

    orbitSize := 
    () -> GLIP::orbitSize(dom::CGroup, args());

    automorphismGroupSize :=
    () -> GLIP::automorphismGroupSize(dom::CGroup, args());

    canonicCompositions :=
    () -> GLIP::canonicCompositions(dom::CGroup, args());

    canonicBinaryLists :=
    proc() 
	local m; 
    begin
	[ map(m, x->if args(1) then 1 else 0; end_if)
	 $ m in GLIP::canonicBinaryLists(dom::CGroup, args()) ]
    end_proc;

    canonicListsByShape :=
    () -> GLIP::canonicListsByShape(dom::CGroup, args());

begin
    if GLIP=FAIL then
	warning("Dom::PermutationGroup_GLIP requires the dynamic module combinat::GLIP");
    end_if;
end_domain:
