/*****************************************************************************
  $Id: QuotientFreeModule.mu,v 1.1.1.2 2005/01/26 13:42:22 nthiery Exp $
  File:		lib/DOMAINS/DOMAIN/QuotientFreeModule.mu
  Authors:	Florent Hivert <florent.hivert@univ-mlv.fr>
  Purpose:	Source code for the domain Dom::QuotientFreeModule
  Created:	Sun Jun 13 18:19:07 2004
  License:	LGPL
  Project:	MuPAD-Combinat, http://mupad-combinat.sf.net
*****************************************************************************/

/**
 * Let M be a finite dimensional free module over a ring R
 * Let B be a sub free module of M
 * Let basis be a vector basis of B in row echelon form with
 * respect to the term order
 *
 * This implements the quotient module M / B
 **/
domain Dom::QuotientFreeModule(RRBasis : DOM_LIST)
    local M, BasisClass, LSupports;
    inherits Dom::FreeModule(M::coeffRing, BasisClass, 
			     RankerBasis = BasisClass::ranker);
    category Cat::ModuleWithBasis(M::coeffRing);

    info_str := "Dom::QuotientFreeModule -- domain for a quotient free module of a free module";

    supModule := M;
    RRBasis   := RRBasis;	     
    dimension := dom::basisIndices::count();

    print := M::print@dom::toSupModule;

    fromSupModuleBasis :=
    proc(x: M::basisIndices) : dom
	option remember;
	local termToRewrite;
    begin
	termToRewrite := contains(LSupports, x);
	if termToRewrite = 0 then
	    dom::term(x);
	else
	    termToRewrite := RRBasis[termToRewrite];
	    dom::fromList(M::toList(M::term(x) - termToRewrite));
	end_if;
    end_proc;

    fromSupModule :=
    operators::makeLinear(dom::fromSupModuleBasis,
			  Source=M, ImageSet=dom);

    toSupModule :=
    proc(s: dom) : M
	// may Be optimized by sharing of rankers (using SupModule::getRankerParameters()). 
	// but sparseVector suppose that rank goes from 1 to dimension.
    begin
	M::fromList(dom::toList(s));
    end_proc;

begin
    if nops(RRBasis)=0 then
	error("Taking quotient by an empty space, use the ambient space itself.");
    end_if;
    experimental::algTools::checkRRBasis(RRBasis);
    M := (RRBasis[1])::dom;
    LSupports := map(RRBasis, M::lsupport);
    BasisClass := select(M::basisIndices::list(), not _in, {op(LSupports)});
    BasisClass := combinat::finiteClass(BasisClass);
    BasisClass::domtype := M::basisIndices::domtype;
end_domain:
