/*****************************************************************************
  $Id: SteenrodAlgebraP.mu,v 1.1.1.4 2005/01/26 13:42:22 nthiery Exp $
  File:		lib/DOMAINS/DOMAIN/SteenrodAlgebraP.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Created:	26/06/2002
  License:	LGPL
*****************************************************************************/

alias(mult2		= operators::mult2):
alias(declareConversion = operators::overloaded::declareConversion):
alias(declareSignature  = operators::overloaded::declareSignature):

domain Dom::SteenrodAlgebraP(q=hold(q), R = Dom::ExpressionField())

    inherits Dom::FreeModule(R, combinat::partitions);

    category Cat::AlgebraWithBasis(R);

    info_str := "Domain for the q-Steenrod algebra on the powersum basis";

    one := dom::term([]);

    //////////////////////////////////////////////////////////////////////////
    // Conversions and overloading
    //////////////////////////////////////////////////////////////////////////

    exprTerm := b -> hold(P)[op(b)];

    fromIndexed := () -> dom::straightenBasis([args()]);

    // Express an element p of the steenrod algebra in the Weyl algebra W
    toWeyl :=
    proc(W: DOM_DOMAIN, p: dom)
	local d, m, x;
    begin
	_plus(
	      dom::coeff(p,m) *
	      _mult(W(_plus(x^d*(1+q*x*W::DX(x)) $ x in W::XVariables))
		    $ d in m)
	      $ m in dom::support(p));
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // Commutation rules and product
    //////////////////////////////////////////////////////////////////////////

    straightenBasis :=
    proc(l: combinat::compositions)
	local i;
    begin
	for i from 1 to nops(l)-1 do
	    if l[i] >= l[i+1] then 
		next;
	    end_if;
	    return(dom::straightenBasis([op(l,1..i-1),l[i+1],l[i],op(l,i+2..nops(l))]) +
		   dom::multcoeffs(dom::straightenBasis([op(l,1..i-1),l[i]+l[i+1],op(l,i+2..nops(l))]),
				   q*(l[i+1]-l[i]))
		  );
	end_for;
	dom::term(l);
    end_proc;

    straighten :=
    dom::makeLinearFunction(dom::straightenBasis);

    mult2Basis :=
    proc(l1: dom::basisIndices, l2: dom::basisIndices)
    begin
	dom::straightenBasis(l1.l2);
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // dual / antipode
    //////////////////////////////////////////////////////////////////////////

    antipodeBasis :=
    proc(l: dom::basisIndices)
    begin
	if testtype(nops(l), Type::Even) then
	    dom::straightenBasis(revert(l));
	else
	    -dom::straightenBasis(revert(l));
	end_if;
    end_proc;

    antipode :=
    dom::makeLinearFunction(dom::antipodeBasis);

begin
end_domain:
