/*****************************************************************************
  $Id: SubFreeModule.mu,v 1.1.1.2 2005/01/26 13:42:22 nthiery Exp $
  File:		lib/DOMAINS/DOMAIN/SubFreeModule.mu
  Authors:	Florent Hivert <florent.hivert@univ-mlv.fr>
  Purpose:	Source code for the domain Dom::SubFreeModule
  Created:	Sun Jun 13 15:40:30 2004
  License:	LGPL
  Project:	MuPAD-Combinat, http://mupad-combinat.sf.net
*****************************************************************************/

/**
 * Let M be a finite dimensional free module over a ring R
 * Let B be a sub free module of M
 * Let basis be a vector basis of RRBasis in row echelon form with
 * respect to the term order
 *
 * This implements the sub module spanned by RRBasis in M
 **/
domain Dom::SubFreeModule(RRBasis : DOM_LIST)
    local M, BasisClass, BasisClassSet, BasisClassList;
    inherits Dom::FreeModule(M::coeffRing, BasisClass, 
			     RankerBasis = BasisClass::ranker);
    category Cat::ModuleWithBasis(M::coeffRing);

    info_str := "Dom::SubFreeModule -- domain for a sub free module of a free module";

    supModule := M;
    RRBasis   := RRBasis;	     
    dimension := dom::basisIndices::count();

    print := M::print@dom::toSupModule;

    fromSupModule :=
    proc(x: M) : dom
        local b;
    begin
	if M::nterms(x) >= dom::dimension then
	    dom::fromList([[coeff(x, b), b] $ b in BasisClassList]);
	else 
	    dom::fromList(select(M::toList(x), z -> z[2] in BasisClassSet));
	end_if;
    end_proc;

    toSupModuleBasis :=
    proc(s: dom::basisIndices)
    begin
        RRBasis[dom::rankBasis(s)];
    end_proc;

    toSupModule := operators::makeLinear(dom::toSupModuleBasis,
					 Source=dom, ImageSet=M);

begin
    if nops(RRBasis)=0 then
        error("An empty basis can't span a submodule!")
    end_if;
    experimental::algTools::checkRRBasis(RRBasis);
    BasisClass := combinat::finiteClass
		(map(RRBasis, x -> x::dom::lsupport(x)));
    M := (RRBasis[1])::dom;
    BasisClass::domtype := M::basisIndices::domtype;
    BasisClassList := BasisClass::list();
    BasisClassSet := {op(BasisClassList)};
end_domain:
