/*****************************************************************************
  $Id: FreeModule.tst,v 1.1.1.5 2005/01/26 13:42:22 nthiery Exp $
  File:		lib/DOMAINS/DOMAIN/TEST/FreeModule.tst
  Authors:	Nicolas Thiry <nthiery@users.sf.net>
  Purpose:	Test file for the domains 'Dom::FreeModule*'
  Created:	01/07/2003
  License:	LGPL
*****************************************************************************/

bla := [0,1,2,3,4,5]:
basis := combinat::finiteClass(bla):

freeModules := [
		Dom::FreeModuleTable(Dom::Rational, basis),
		Dom::FreeModulePoly (Dom::Rational, basis),
		Dom::FreeModulePoly (Dom::IntegerMod(7), basis),
		Dom::FreeModulePoly (Dom::Rational, basis, Variables=1, RankerBasis=[id,id]),
		Dom::FreeModulePoly (Dom::Rational, basis, Variables=2, MultirankerBasis=[i->[i,0],l->l[1]]),
		Dom::FreeModuleList (Dom::Rational, basis)
	       ]:

prog::testinit("FreeModule"):

for F in freeModules do
    R := F::coeffRing;
    prog::testfunc(F):
    prog::test(F::basisIndices::count(), nops(bla)):
    prog::test(F::basisIndices::list(),  bla):
    prog::test(F::basis::count(),        nops(bla)):
    prog::test(F::basis::list(),         map(bla, F::term)):
    prog::test(iszero(F::zero)):
    x := F::term(1):
    y := F::monomial(R(2),3):
    y1 := F::term(3):
    prog::test(iszero(y-2*y1)):
    z := R(1/2)*x+y:
    t := R(3/4)*x;

    prog::testfunc(toList):
    prog::test(F::toList(F::zero), []):
    prog::test(F::toList(x), [[R(1),1]]):
    prog::test(F::toList(y),  [[R(2),3]]):
    prog::test(F::toList(y1), [[R(1),3]]):
    prog::test(sort(F::toList(z)), sort([[R(1/2),1], [R(2),3]])):
    
    prog::testfunc(fromList):
    prog::test(F::fromList([]), F::zero):
    prog::test(F::fromList([[R(1),1]]), x):
    prog::test(F::fromList([[R(2),3]]),  y):
    prog::test(F::fromList([[R(1),3]]), y1):
    prog::test(F::fromList([[R(1/2),1], [R(2),3]]), z):

    prog::testfunc(nterms):
    prog::test(F::nterms(F::zero), 0):
    prog::test(F::nterms(y), 1):
    prog::test(F::nterms(z), 2):

    prog::testfunc(lsupport):
    prog::test(F::lsupport(F::zero), FAIL):
    prog::test(F::lsupport(y), 3):
    prog::test(contains({1,3}, F::lsupport(z))):

    prog::testfunc(lcoeff):
    prog::test(F::lcoeff(F::zero), R(0)):
    prog::test(F::lcoeff(y), R(2)):
    prog::test(contains({R(1/2),R(2)}, F::lcoeff(z))):

    prog::testfunc(lterm):
    prog::test(lterm(F::zero), F::zero):
    prog::test(lterm(y), y1):
    prog::test(contains({x,y1}, F::lterm(z))):

    prog::testfunc(lmonomial):
    prog::test(lmonomial(F::zero), F::zero):
    prog::test(lmonomial(y), y):
    prog::test(contains({y,R(1/2)*x}, F::lmonomial(z))):

    prog::testfunc(tsupport):
    prog::test(F::tsupport(F::zero), FAIL):
    prog::test(F::tsupport(y), 3):
    prog::test(contains({1,3}, F::tsupport(z))):

    prog::testfunc(tcoeff):
    prog::test(F::tcoeff(F::zero), R(0)):
    prog::test(F::tcoeff(y), R(2)):
    prog::test(contains({R(1/2),R(2)}, F::tcoeff(z))):

    prog::testfunc(tterm):
    prog::test(lterm(F::zero), F::zero):
    prog::test(lterm(y), y1):
    prog::test(contains({x,y1}, F::tterm(z))):

    prog::testfunc(tmonomial):
    prog::test(lmonomial(F::zero), F::zero):
    prog::test(lmonomial(y), y):
    prog::test(contains({y,R(1/2)*x}, F::tmonomial(z))):

    prog::testfunc(coeff):
    prog::test(     [F::coeff(F::zero)], []):
    prog::test(     [F::coeff(y)],       [R(2)]):
    prog::test(sort([F::coeff(z)]),      sort([R(1/2),R(2)])):
    prog::test(coeff(t),                 R(3/4));
    prog::test(coeff(z,1),		 R(1/2)):
    prog::test(coeff(z,3),		 R(2)):
    prog::test(coeff(z,4),		 R(0)):

    prog::testfunc(coeffs):
    prog::test(     F::coeffs(F::zero), []):
    prog::test(     F::coeffs(y),       [R(2)]):
    prog::test(sort(F::coeffs(z)),      sort([R(1/2),R(2)])):

    prog::testfunc(support):
    prog::test(     F::support(F::zero), []):
    prog::test(     F::support(y),       [3]):
    prog::test(sort(F::support(z)),      sort([1,3])):

    // Elements of Dom::FreeModuleTable can't be compared and sorted with
    // sysorder. So, we sort using their expressions
    sort2 := l -> sort(l, (x,y) -> sysorder(expr(x), expr(y))):

    prog::testfunc(terms):
    prog::test(      F::terms(F::zero), []):
    prog::test(      F::terms(y),       [y1]):
    prog::test(sort2(F::terms(z)),      sort2([x,y1])):

    prog::testfunc(monomials):
    prog::test(      F::monomials(F::zero), []):
    prog::test(      F::monomials(y),       [y]):
    prog::test(sort2(F::monomials(z)),      sort2([R(1/2)*x,y])):

    prog::testfunc(nthmonomial):
    prog::test(F::nthmonomial(F::zero, 1), FAIL):
    prog::test(F::nthmonomial(z, 3), FAIL):
    prog::test(F::nthmonomial(z, 1), F::lmonomial(z)):
    prog::test(F::nthmonomial(z, 2), F::tmonomial(z)):

    prog::testfunc(nthsupport):
    prog::test(F::nthsupport(F::zero, 1), FAIL):
    prog::test(F::nthsupport(z, 3), FAIL):
    prog::test(F::nthsupport(z, 1), F::lsupport(z)):
    prog::test(F::nthsupport(z, 2), F::tsupport(z)):

    prog::testfunc(nthterm):
    prog::test(F::nthterm(F::zero, 1), FAIL):
    prog::test(F::nthterm(z, 3), FAIL):
    prog::test(F::nthterm(z, 1), F::lterm(z)):
    prog::test(F::nthterm(z, 2), F::tterm(z)):

    prog::testfunc(nthcoeff):
    prog::test(F::nthcoeff(F::zero, 1), FAIL):
    prog::test(F::nthcoeff(z, 3), FAIL):
    prog::test(F::nthcoeff(z, 1), F::lcoeff(z)):
    prog::test(F::nthcoeff(z, 2), F::tcoeff(z)):

    prog::testfunc(nthall):
    for s in [F::zero, x, y, y1, z] do
	prog::test(iszero(F::_plus(-s,
				   F::nthterm(s,i) * F::nthcoeff(s,i)
				   $ i=1..F::nterms(s)))):
	prog::test(iszero(F::_plus(-s,
				   F::nthmonomial(s,i)
				   $ i=1..F::nterms(s)))):
	prog::test(iszero(F::_plus(-s,
				   F::monomial(F::nthcoeff(s,i), F::nthsupport(s,i))
				   $ i=1..F::nterms(s)))):
    end_for:

    prog::testfunc("mapterms"):
    prog::test(F::mapterms(z, _plus, 1),    F::fromList([[R(1/2),2], [R(2),4]])):

    prog::testfunc("endomorphismMatrix"):
    prog::test(F::endomorphismMatrix(x -> 2*x, RowArgs=()),
	       Dom::Matrix(R)([[2, 0, 0, 0, 0, 0],
			       [0, 2, 0, 0, 0, 0],
			       [0, 0, 2, 0, 0, 0],
			       [0, 0, 0, 2, 0, 0],
			       [0, 0, 0, 0, 2, 0],
			       [0, 0, 0, 0, 0, 2]])):
    prog::test(F::endomorphismMatrix(x -> 2*x,
				     ColumnIndices = [1, 2],
				     RowIndices    = [1, 2]),
	       Dom::Matrix(R)([[2, 0],
			       [0, 2]])):
    prog::test(F::endomorphismMatrix(x -> 2*x,
				     ColumnIndices = [1, 2],
				     RowIndices    = [1, 3]),
	       Dom::Matrix(R)([[2, 0],
			       [0, 0]])):
    prog::test(F::endomorphismMatrix(x -> 2*x,
				     ColumnVectors = [F(2), F(1) - F(2)],
				     RowArgs = ()),
	       Dom::Matrix(R)([[0, 0],
			       [0, 2],
			       [2,-2],
			       [0, 0],
			       [0, 0],
			       [0, 0]])):
    prog::test(F::endomorphismMatrix(x -> 2*x,
				     RowRank = (i -> 6 - i)),
	       Dom::Matrix(R)([[0, 0, 0, 0, 0, 2],
			       [0, 0, 0, 0, 2, 0],
			       [0, 0, 0, 2, 0, 0],
			       [0, 0, 2, 0, 0, 0],
			       [0, 2, 0, 0, 0, 0],
			       [2, 0, 0, 0, 0, 0]])):
end_for:

//////////////////////////////////////////////////////////////////////////////
// tests with infinite dimensional free modules
for F in [Dom::FreeModuleTable(Dom::Rational, combinat::partitions),
	  Dom::FreeModulePoly (Dom::Rational, combinat::partitions),
	  Dom::FreeModuleList (Dom::Rational, combinat::partitions)] do

    //////////////////////////////////////////////////////////////////////////
    prog::testfunc("endomorphismMatrix"):

    // Those tests are fragile and context dependent.
    // They depend on the current on fly ranker feature
    prog::test(F::endomorphismMatrix(z -> 2*z, ColumnArgs = 0),
	       Dom::Matrix(Dom::Rational)([[2]])):
    prog::test(F::endomorphismMatrix(z -> 2*z, ColumnArgs = 1),
	       Dom::Matrix(Dom::Rational)([[0],
					   [2]])):
    prog::test(F::endomorphismMatrix(z -> 2*z, ColumnArgs = 2),
	       Dom::Matrix(Dom::Rational)([[0, 0],
					   [0, 0],
					   [2, 0],
					   [0, 2]])):
    prog::test(F::endomorphismMatrix(z -> 2*z, ColumnArgs = 3),
	       Dom::Matrix(Dom::Rational)([[0, 0, 0],
					   [0, 0, 0],
					   [0, 0, 0],
					   [0, 0, 0],
					   [2, 0, 0],
					   [0, 2, 0],
					   [0, 0, 2]])):
end_for:

//////////////////////////////////////////////////////////////////////////////
// Global checking of the package

prog::testfunc(Dom::FreeModule):
prog::testexit():

for F in freeModules do
    prog::check(F, 3);
end_for:


// Local Variables: 
// mode: mupad
// End:
