/*****************************************************************************
  $Id: PermutationGroup.tst,v 1.1.1.1 2003/09/15 16:37:44 nthiery Exp $
  File:		lib/DOMAINS/DOMAIN/TEST/PermutationGroup.tst
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Purpose:	Test file for the domain 'Dom::PermutationGroup'
  Taken From:	PerMuVAR, 02/12/2001
  Created:	24/02/2001
  License:	LGPL
*****************************************************************************/

prog::testinit("PermutationGroup"):

prog::testfunc(Dom::PermutationGroup):
C4:=Dom::PermutationGroup(4,  [[[1,2,3,4]]]):		// The cyclic group C4
G4:=Dom::PermutationGroup(6,  [[1,3,2,5,4,6],[3,5,6,1,2,4]]):
G5:=Dom::PermutationGroup(10, [[1,3,2,5,4,6,8,7,9,10],[3,5,6,8,9,10,1,2,4,7]]):
G6:=Dom::PermutationGroup(15, [[1,3,2,5,4,6,8,7,9,10,12,11,13,14,15],
                               [3,5,6,8,9,10,12,13,14,15,1,2,4,7,11]]):

prog::test(C4::set,
	   [1,2,3,4]):
prog::test(C4::card,4):
prog::test(C4::size,4):

prog::test({op(C4::allElements())},
	   map({[1, 2, 3, 4], [4, 1, 2, 3], [3, 4, 1, 2], [2, 3, 4, 1]}, C4)):

Id5:=Dom::PermutationGroup(5,[]):
prog::test(Id5::allElements(),[Id5([1,2,3,4,5])]):

// Test the action of the group

g:=C4([3, 2, 4, 1]):
prog::test(g(1),3):
prog::test(g([a,b,c,d]),[d,b,a,c]):
prog::test(expr(C4([1,4,2,3])), [1,4,2,3]):
prog::test(C4([2,3,4,1])^4,
	   C4::one):
prog::test(C4([2,1,3,4])*C4([1,2,4,3]),
	   C4([2,1,4,3])):

prog::test(expr(C4([2,1,3,4])), [2,1,3,4]):
	// Should actually complain since this permutation is not in C4

prog::test(C4::canonic([1,2,3,4]),C4::canonic([2,3,4,1])):
prog::test(bool(C4::canonic([4,3,2,1])<>C4::canonic([1,2,3,4]))):

// C4::canonicBinaryLists

prog::test(sort(C4::canonicBinaryLists(2)),
           sort([[1, 0, 1, 0], [1, 1, 0, 0]])):
prog::test(_plus(nops(C4::canonicBinaryLists(d))*t^d $d=0..C4::card),
           expand(C4::weightEnum([1,t]))):

// Should test C4::canonicCompositions

// Test Polya Enumeration

prog::test(C4::charPolynomial(C4([2,3,1,4]),z),
	   (1-z^3)*(1-z)):
prog::test(C4::cycleTypes(),
	   table(
		 [0, 0, 0, 1] = 2,
		 [0, 2, 0, 0] = 1,
		 [4, 0, 0, 0] = 1
		 )):
	// Should also test the version of cycleTypes for subsets of size <> 2
prog::test(C4::cycleIndicatorPoly(hold(z)),
	   poly(1/4*z[1]^4 + 1/4*z[2]^2 + 1/2*z[4], [z[1], z[2], z[3], z[4]], Dom::Rational)):
	// cycle indicator polynom
prog::test(expand(C4::weightEnum([1,1])),
	   6):
	// Number of non isomorphic binary lists
prog::test(expand(C4::weightEnum([x,y,z])),
	   x^4 + y^4 + z^4 + x*y^3 + x^3*y + x*z^3 + x^3*z + y*z^3 + y^3*z + x*y*z^2*3 + x*y^2*z*3 + x^2*y*z*3 + x^2*y^2*2 + x^2*z^2*2 + y^2*z^2*2):
	// Number of non isomorphic lists by shape

prog::testfunc(Cat::PermutationGroup::numberOfSetPartitionsOrbits):
prog::test([G6::numberOfSetPartitionsOrbits([15-k,k]) $k=0..15],
	   [1,1,2,5,9,15,21,24,24,21,15,9,5,2,1,1]):

prog::testfunc(Cat::PermutationGroup::numberOfCanonicListsByShape):

prog::test(G5::numberOfCanonicListsByShape([2,1,1,0 $7]),	6):
prog::test(G6::numberOfCanonicListsByShape([2,1,1,0 $12]),	7):
prog::test(G4::canonicListsByShape([1,0,0,0,0,0]), [[1,0,0,0,0,0]]):

// Test

prog::testfunc(Dom::PermutationGroup::isCyclic):

prog::test(Id5::isCyclic(), TRUE):
prog::test(C4::isCyclic(), TRUE):
prog::test(G4::isCyclic(), FALSE):
prog::test(G5::isCyclic(), FALSE):
prog::test(G6::isCyclic(), FALSE):
prog::test((Dom::PermutationGroup(6,[[[1,2],[3,4]]]))::isCyclic(), TRUE):
prog::test((Dom::PermutationGroup(6,[[[1,3,5],[2,4,6]],[[1,4],[2,5],[3,6]]]))::isCyclic(), TRUE):

// Test exportation to other CAS

prog::testfunc(Cat::PermutationGroup::Invar):
prog::test(C4::Invar(),
	   "permgroup(4, {[[1, 2, 3, 4]]})"):
prog::testfunc(Cat::PermutationGroup::Invar2):
prog::test(C4::Invar2(),
	   "pgroup({[[1, 2, 3, 4]]}, 0, 0, 4)"):

//////////////////////////////////////////////////////////////////////////////
// Global checking of the package

// This this done at the end to ensure as much as possible of the
// package is initialized. A recursive prog::init(combinat::words)
// would be usefull here.

prog::testfunc(Dom::PermutationGroup):

prog::check(C4, 3):

prog::testexit():

// Local Variables: 
// mode: mupad
// End:
