/*****************************************************************************
  $Id: PermutationGroupInvariantRing.tst,v 1.1.1.3 2003/09/22 18:05:07 nthiery Exp $
  File:		lib/DOMAINS/DOMAIN/TEST/PermutationGroupInvariantRing.tst
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Purpose:	Test suite for the domain Dom::PermutationGroupInvariantRing
  Created:	Mon Sep  8 14:50:04 2003
  License:	LGPL
  Project:	MuPAD-Combinat, http://mupad-combinat.sf.net
*****************************************************************************/

prog::testinit("PermutationGroupInvariantRing"):

prog::testfunc(Dom::PermutationGroupInvariantRing):


//////////////////////////////////////////////////////////////////////////////
// Short hands
//////////////////////////////////////////////////////////////////////////////

Tn:=proc(n) begin Dom::PermutationGroup(n, [[$1..n]]): end_proc:
Sn:=proc(n) begin Dom::SymmetricGroup(n): end_proc:
An:=proc(n) begin Dom::AlternatingGroup(n): end_proc:
Cn:=proc(n) begin Dom::PermutationGroup(n, [[[$1..n]]]); end_proc:
Gn:=proc(n) begin Dom::PermutationGroupOnSets(Sn(n),2): end_proc:
SnVm:=proc(n,m)
            begin Dom::PermutationGroupDirectProduct([Sn(n),Tn(m)]): end_proc:

//IC1   := Dom::PermutationGroupInvariantRing(Dom::Rational, Cn(1)):
IC2   := Dom::PermutationGroupInvariantRing(Dom::Rational, Cn(2)):
IC5   := Dom::PermutationGroupInvariantRing(Dom::Rational, Cn(5)):
IC3Z3 := Dom::PermutationGroupInvariantRing(Dom::IntegerMod(3), Cn(3)):
IC3Z5 := Dom::PermutationGroupInvariantRing(Dom::IntegerMod(5), Cn(3)):
IC4C  := Dom::PermutationGroupInvariantRing(Dom::Rational, Cn(4),
					    hold(ChainProduct)=TRUE):
IA3   := Dom::PermutationGroupInvariantRing(Dom::Rational, An(3)):
IS4   := Dom::PermutationGroupInvariantRing(Dom::Rational, Sn(4)):
IG4   := Dom::PermutationGroupInvariantRing(Dom::Rational, Gn(4)):
IG4::setPrimaryInvariantsDegrees([1,2,3,4,2,3]):
IG4C  := Dom::PermutationGroupInvariantRing(Dom::Real, Gn(4),
					 hold(ChainProduct)=TRUE):
IG5   := Dom::PermutationGroupInvariantRing(Dom::Rational, Gn(5)):
IS2V2 := Dom::PermutationGroupInvariantRing(Dom::Rational, SnVm(2,2)):

// Basic tests

prog::check(IA3,3):
prog::test(IA3::dimen,3): // Test Krull dimension
prog::test(IC3Z3::isModular, TRUE):
prog::test(IC3Z3::isCohenMacaulay, UNKNOWN):
prog::test(IC3Z5::isModular, FALSE):
prog::test(IA3::isModular, FALSE):
//prog::test(IC1::degreeBound(), 1):
prog::test(IC2::degreeBound(), 2):
prog::test(IC5::degreeBound(), 5):
prog::test(IS4::degreeBound(), 4):
prog::test(IG4::degreeBound(), 6):
prog::test(IG4C::degreeBound(), 15):
prog::test(IS4::secondaryInvariantsMaxDegree(), 0):

// Algebraic operations

prog::testfunc(_mult):
p1:=IA3([1,0,0]):
p2:=IA3([1,1,0]):
prog::test(sort(IA3::toList(p1+2*p2)),		// vector space operations
	   sort([[2, [1, 1, 0]],[1,[1, 0, 0]]])):
prog::test(p1*p2,
	   IA3([2, 0, 1]) + IA3([2, 1, 0]) + 3*IA3([1, 1, 1])):
prog::test(p1*p1,
	   IA3([2, 0, 0]) + 2*IA3([1, 1, 0])):
// chain product
prog::test(IG4C([1,1,0,0,0,1])*IG4C([1,0,0,0,0,0]),
	   IG4C([2,1,0,0,0,1])+IG4C([2,1,0,0,1,0])):

//////////////////////////////////////////////////////////////////////////////
// elementary and powersums
prog::testfunc(symmetric):
//////////////////////////////////////////////////////////////////////////////

prog::test(IA3::elementaries(),[IA3([1,0,0]),IA3([1,1,0]), IA3([1,1,1])]):
prog::test(IS2V2::powersums(), [IS2V2([1, 0, 0, 0]) + IS2V2([0, 1, 0, 0]),
                               IS2V2([2, 0, 0, 0]) + IS2V2([0, 2, 0, 0]),
                               IS2V2([3, 0, 0, 0]) + IS2V2([0, 3, 0, 0]),
                               IS2V2([4, 0, 0, 0]) + IS2V2([0, 4, 0, 0])]):

// misc

delete(x):
prog::testfunc(variables):
// add test for compact variables
prog::test(IS4::variables, [x1,x2,x3,x4]):
prog::testfunc(poly):
prog::test(IS2V2::poly(IS2V2::powersum(2)), 
           poly(x11^2 + x12^2 + x21^2 + x22^2, [x11, x12, x21, x22], Dom::Rational)):
// prog::test(IS2V2::poly(IS2V2::powersum(2)), 
//            poly(x[[1, 1]]^2 + x[[2, 1]]^2 + x[[1, 2]]^2 + x[[2, 2]]^2,
//                 [x[[1, 1]], x[[2, 1]], x[[1, 2]], x[[2, 2]]])):

// Operations on the Initial poset

m1:=[1,0,0]:
m2:=[2,0,0]:
m3:=[2,1,0]:
prog::testfunc(divides):
prog::test(IA3::divides(m1,m2), TRUE):
prog::test(IA3::divides(m1,m3), TRUE):
prog::test(IA3::divides(m1,m2), TRUE):
prog::test(IA3::divides(m2,m1), FALSE):
prog::test(IA3::divides(m2,m2), TRUE):
prog::test(IA3::divides(m2,m3), FALSE):
prog::testfunc(isPrime):
prog::test(IA3::isPrime([3,0,2]), TRUE):
prog::test(IA3::isPrime([2,0,1]), TRUE):
prog::test(IA3::isPrime([2,1,0]), FALSE):
prog::test(IA3::primes(3),			[[1, 1, 1], [2, 0, 1]]):
prog::test(IA3::divisors([3,1,2]),		[[1, 1, 1], [2, 0, 1]]):
prog::test(IA3::aDivisor([3,1,2]),		[1, 1, 1]):
prog::test(IA3::smallPrimeDivisors([3,1,2]), [[1, 1, 1]]):
prog::test(IA3::constructInitialPoset_LCMS(6),	null()):
prog::test(IA3::aDivisor([3,1,2]),		[1, 1, 1]):
prog::test(IA3::smallPrimeDivisors([3,1,2]), [[1, 1, 1]]):

m1:=[1,0,0,0,0,1]:
m2:=[2,0,0,0,0,2]:
m3:=[2,1,0,0,1,2]:
prog::testfunc(divides):
prog::test(IG4::divides(m1,m2), TRUE):
prog::test(IG4::divides(m1,m3), TRUE):
prog::test(IG4::divides(m1,m2), TRUE):
prog::test(IG4::divides(m2,m1), FALSE):
prog::test(IG4::divides(m2,m2), TRUE):
prog::test(IG4::divides(m2,m3), FALSE):
prog::test(IG4::smallPrimeDivisors([4,1,0,1,0,0]),
	   [[1,0,0,0,0,0], [1,1,0,1,0,0]]):
prog::test(IG4::constructInitialPoset_Divisors(6), null()):
prog::test(IG4::smallPrimeDivisors([4,1,0,1,0,0]),
	   [[1,0,0,0,0,0], [1,1,0,1,0,0]]):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc(sparseVector):
//////////////////////////////////////////////////////////////////////////////
prog::test(IG4::sparseVector(IG4::elementary(3)),[[3,1],[4,1],[5,1]],
	   Status=Analyzed):

// Gorenstein property

prog::testfunc(isGorenstein):
prog::test(IA3::isGorenstein, TRUE):
prog::test(IG4::isGorenstein, TRUE):
// This test can fail due to an internal error when mupad is in debug
// mode. See comment in SymGrp.tst
prog::test(IG5::isGorenstein, FALSE):
prog::test(IA3::secondaryInvariantsMaxDegree(),3):
prog::test(IG4::secondaryInvariantsMaxDegree(),9):

// Hilbert series

hsr  := factor(1/(3*(1-z)^3) + 2/(3*(1-z^3))):
hs   := factor(IA3::HilbertSeries(z)):
hsp  := factor(IA3::HilbertSeries_Polya(z)):
hsm  := factor(IA3::HilbertSeries_Molien(z)):
hsfs := factor(IA3::HilbertSeries_FromSecondary(z)):

prog::testfunc(HilbertSeries):
prog::test(expr(series(hsr,z)), 1 + z + 2*z^2 + 4*z^3 + 5*z^4 + 7*z^5):
prog::test(hs,   hsr):
prog::test(hsp,  hsr):
prog::test(hsm,  hsr):
prog::test(hsfs-hsr, 0):

// Counting secondary Invariants

prog::testfunc(secondaryInvariantsSeries):
sis   := IA3::secondaryInvariantsSeries(z):
sisfct:= IA3::secondaryInvariantsSeries_FromCycleTypes(z):
sisfh := IA3::secondaryInvariantsSeries_FromHilbert(z):
prog::test(sis, poly(z^3  + 1, [z], Dom::Integer)):
prog::test(sis, sisfct):
prog::test(sis, sisfh):

prog::testfunc(numberOfSecondaryInvariants):
prog::test(IA3::numberOfSecondaryInvariants(),2):
prog::test(IA3::numberOfSecondaryInvariants(3),1):
prog::test(IG4C::numberOfSecondaryInvariants(),30):
prog::test(table(c=IG4C::numberOfSecondaryInvariants(c)
		 $ c in combinat::subsets::list({$1..5})),
	   table({} = 1,
		 {1} = 0, {3} = 2, {5} = 0, {4} = 1, {2} = 1,
		 {1, 3} = 1, {1, 5} = 1, {1, 4} = 2, {1, 2} = 0,
		 {3, 5} = 1, {3, 4} = 0, {4, 5} = 0, {2, 3} = 0,
		 {2, 5} = 2, {2, 4} = 3,
		 {1, 3, 5} = 3, {1, 3, 4} = 2, {1, 4, 5} = 0,
		 {1, 2, 3} = 0, {1, 2, 5} = 0, {1, 2, 4} = 1,
		 {3, 4, 5} = 0, {2, 3, 5} = 2, {2, 3, 4} = 1,
		 {2, 4, 5} = 1,
		 {1, 3, 4, 5} = 1, {1, 2, 3, 5} = 1, {1, 2, 3, 4} = 0,
		 {1, 2, 4, 5} = 2, {2, 3, 4, 5} = 0,
		 {1, 2, 3, 4, 5} = 1 )):
s:=0:
for c in combinat::subsets::list({$1..IG4C::dimen-1}) do
    s:=s+IG4C::numberOfSecondaryInvariants(c)*hold(z)^_plus(op(c));
end_for:
s:=poly(s,[hold(z)], Dom::Integer):
prog::test(s, IG4C::secondaryInvariantsSeries(z)):

// fine series (TODO!)

// Tests for (irreducible) secondary invariants

//////////////////////////////////////////////////////////////////////////////
prog::testfunc(secondaryInvariants):
//////////////////////////////////////////////////////////////////////////////
IA3::secondaryInvariants():
prog::test([nops(IA3::secondaryInvariants(i)) $ i=0..3],
	   [1,0,0,1]):
prog::test([nops(IA3::irreducibleSecondaryInvariants(i)) $ i=0..3],
	   [0,0,0,1]):
prog::test(IA3::degreeBound(), 3):
prog::test(IA3::degreeLowerBound(), 3):
prog::test(IA3::degreeBoundIsOptimal(), TRUE):
// Note: the order of the elements in the result is not guaranted
prog::test(map(IC4C::secondaryInvariants(),
	       map,
               sort, (p1,p2)->bool(IC4C::basisIndices::_less(IC4C::lsupport(p1)<IC4C::lsupport(p2)))),
	   table(0=table({}    =[IC4C([0, 0, 0, 0])]),
		 2=table({2}   =[IC4C([1, 0, 1, 0])]),
		 3=table({1, 2}=[IC4C([2, 0, 0, 1])]),
		 4=table({1, 3}=[IC4C([2, 0, 1, 1]), IC4C([2, 1, 0, 1])]),
		 5=table({2, 3}=[IC4C([2, 2, 0, 1])])),
	   Status=Registered):
prog::test(map(IC4C::irreducibleSecondaryInvariants(),
	       map,
               sort, (p1,p2)->bool(IC4C::basisIndices::_less(IC4C::lsupport(p1)<IC4C::lsupport(p2)))),
	   table(2=table({2}   =[IC4C([1, 0, 1, 0])]),
		 3=table({1, 2}=[IC4C([2, 0, 0, 1])]),
		 4=table({1, 3}=[IC4C([2, 0, 1, 1]), IC4C([2, 1, 0, 1])]),
		 5=table({2, 3}=[IC4C([2, 2, 0, 1])])),
	   Status=Registered):
prog::test(IC4C::degreeBound(),5):
prog::test(IC4C::degreeBoundIsOptimal(), TRUE, Status=Registered):

// Tests for SAGBI basis

prog::testfunc(SAGBI):
prog::test([nops(IA3::SAGBI(i)) $ i=0..10],
	   [1, 1, 1, 2, 0, 1, 0, 1, 0, 1, 0]):
prog::test([nops(IG4::SAGBI(i)) $ i=0..10],
	   [1, 1, 2, 3, 2, 2, 2, 2, 2, 3, 3]):

// Tests for minimal generating sets

prog::testfunc(minimalGeneratingSets):
IA3::minimalGeneratingSetInit():
prog::test([nops(IA3::minimalGeneratingSet(i)) $ i=0..3],
	   [0, 1, 1, 2]):
prog::test(IA3::degreeBoundIsOptimal(), TRUE):
prog::test(IA3::isGeneratingSet(IA3::minimalGeneratingSet())):

prog::test(IG4::degreeBound(), 6):
prog::test(IG4::degreeLowerBound(), 1):
IG4::minimalGeneratingSet(hold(maxDegree=3)):
prog::test(IG4::degreeBound(), 6):
prog::test(IG4::degreeLowerBound(), 3):
prog::test(IG4::degreeBoundIsOptimal(), UNKNOWN):
IG4::minimalGeneratingSet():
prog::test([nops(IG4::minimalGeneratingSet(i)) $ i=0..6],
	   [0, 1, 2, 3, 2, 1, 0]):
prog::test(IG4::degreeBound(), 5):
prog::test(IG4::degreeBoundIsOptimal(), TRUE):
prog::test(IG4::isGeneratingSet(IG4::minimalGeneratingSet())):

prog::testexit():
null():

// Local Variables: 
// mode: mupad
// End:
