/*****************************************************************************
  $Id: TensorProductOfFreeModules.tst,v 1.1.1.2 2004/10/16 15:15:21 nthiery Exp $
  File:		lib/DOMAINS/DOMAIN/TEST/TensorProductOfFreeModules.tst
  Authors:	Florent Hivert <florent.hivert@univ-mlv.fr>
  Purpose:	Test suite for the domain Dom::TensorProductOfFreeModules
  Created:	Thu Feb  5 20:04:58 2004
  License:	LGPL
  Project:	MuPAD-Combinat, http://mupad-combinat.sf.net
*****************************************************************************/

alias(TPFM = Dom::TensorProductOfFreeModules):
prog::testinit("TensorProductOfFreeModules", 2.3):

//////////////////////////////////////////////////////////////////////////////
// Let's build some free modules with different ranking methods
SGA := (N, R) -> Dom::FreeModulePoly(R, combinat::subClass(combinat::permutations, 
							   Parameters = N),
				     Variables = N,
				     MultirankerBasis = [id, id]):


///////////////////////////////////////////////////////////////////////
// The polynomial optimisation is possible => test over different rings
for ring in [Dom::IntegerMod(7), 
	     Dom::UnivariatePolynomial(X, Dom::Rational), 
	     Dom::ExpressionField()] do

SGAN := Dom::FreeModule(ring, combinat::permutations):
SGA2 := SGA(2, ring):
SGA3 := SGA(3, ring):
///////////////////////////////////////////////////////////////////////
    
    
//////////////////////////////////////////////////////////////////////////////
prog::testfunc("Dom::TensorProductOfFreeModules"):
// Try to construct some tensor product and test for the associativity. 
prog::test(domtype((SGA22 := TPFM([SGA2,SGA2]))), DOM_DOMAIN):
prog::test(domtype((SGA23 := TPFM([SGA2,SGA3]))), DOM_DOMAIN):
prog::test(domtype((SGANN := TPFM([SGAN,SGAN]))), DOM_DOMAIN):
prog::test(bool((SGA222 := TPFM([SGA22,SGA2])) = TPFM([SGA2,SGA22]))):
prog::test(bool(SGA222 = TPFM([SGA2,SGA2,SGA2]))):
prog::test(bool((SGA23N := TPFM([SGA23,SGAN])) = TPFM([SGA2,SGA3,SGAN]))):
// Tensor product on different rings raise an error
prog::test(TPFM([SGA2,SGA(3, Dom::Rational)]), TrapError = 1028):
prog::test(TPFM([SGA(3, Dom::Polynomial([X,Y], Dom::Rational)),
		 SGA(3, Dom::Rational)]), 
	   TrapError = 1028):


//////////////////////////////////////////////////////////////////////////////
prog::testfunc("tensorConstructorFromSignature"):
prog::test((Tsga22 := SGA22::tensorConstructorFromSignature([SGA2, SGA2]);
	    iszero(Tsga22([SGA2([2,1]), SGA2([1,2])]) - 
		   SGA22([[2,1],[1,2]])))):
prog::test(Tsga22([SGA2::zero, SGA2([1,2])]),
	   SGA22::zero):
prog::test(Tsga22([SGA2([1,2]), SGA2::zero]),
	   SGA22::zero):
prog::test(iszero(Tsga22([SGA2([2,1]) + SGA2([1,2]), SGA2([1,2])]) - 
		  (SGA22([[2,1],[1,2]]) + SGA22([[1,2],[1,2]])) )):
prog::test(iszero(Tsga22([SGA2([2,1]) + SGA2([1,2]), SGA2([1,2]) + ring(2)*SGA2([2,1])]) - 
		  (SGA22([[2,1],[1,2]]) + SGA22([[1,2],[1,2]]) 
		   + ring(2)*SGA22([[2,1],[2,1]]) + ring(2)*SGA22([[1,2],[2,1]])) )):
if prog::TestNoTestargs = FALSE then
prog::test(SGA22::tensorConstructorFromSignature([SGA2, SGA2, SGA2]),
	   TrapError = 1028): // wrong tensor product domain
end_if:
prog::test((Tsga2_2_2 := SGA222::tensorConstructorFromSignature([SGA2, SGA2, SGA2]);
	    iszero(Tsga2_2_2([SGA2([2,1]), SGA2([1,2]), SGA2([1,2])]) - 
		   SGA222([[2,1], [1,2], [1,2]])))):
prog::test(Tsga2_2_2([SGA2([2,1]), SGA2::zero, SGA2([1,2])]),
	   SGA222::zero):
if prog::TestNoTestargs = FALSE then
prog::test(Tsga2_2_2([SGA2([2,1]), SGA22([[1,2], [1,2]])]),
	   TrapError = 1202); // Called a constructor with wrong elements
end_if:
prog::test((Tsga22_2 := SGA222::tensorConstructorFromSignature([SGA22, SGA2]);
	    iszero(Tsga22_2([SGA22([[2,1], [1,2]]), SGA2([1,2])]) - 
		   SGA222([[2,1], [1,2], [1,2]])))):
prog::test((Tsga2_22 := SGA222::tensorConstructorFromSignature([SGA2, SGA22]);
	    iszero(Tsga2_22([SGA2([2,1]), SGA22([[1,2], [1,2]])]) - 
		   SGA222([[2,1], [1,2], [1,2]])))):
prog::test((Tsga23N := SGA23N::tensorConstructorFromSignature([SGA2, SGA3, SGAN]);
	    iszero(Tsga23N([SGA2([2,1]), SGA3([1,3,2]), SGAN([1,2,4,3])]) - 
		   SGA23N([[2,1], [1,3,2], [1,2,4,3]])))):
prog::test((Tsga23_N := SGA23N::tensorConstructorFromSignature([SGA23, SGAN]);
	    iszero(Tsga23_N([SGA23([[2,1], [1,3,2]]), SGAN([1,2,4,3])]) - 
		   SGA23N([[2,1], [1,3,2], [1,2,4,3]])))):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("toCartesianProduct"):
prog::test(SGA222::toCartesianProduct(SGA222::zero),
           []):
x := SGA2([1,2]): y1 := SGA3([1,3,2]): y2 := ring(3)*SGA3([3,1,2]): z3 := SGAN([3]):
prog::test(sort(SGA23N::toCartesianProduct(Tsga23N([x, y1, z3]) + Tsga23N([x, y2, z3]))),
	   sort([[(SGA2)::fromList([[ring(3), [1,2]]]), 
		  (SGA3)::fromList([[ring(1), [3,1,2]]]), 
		  (SGAN)::fromList([[ring(1), [3]]])], 
		 [(SGA2)::fromList([[ring(1), [1,2]]]), 
		  (SGA3)::fromList([[ring(1), [1,3,2]]]), 
		  (SGAN)::fromList([[ring(1), [3]]])]])):
prog::test(sort(SGA23N::toCartesianProductFromSignature
		([SGA23, SGAN])
		(Tsga23N([x, y1, z3]) + Tsga23N([x, y2, z3]))),
	   sort([[(SGA23)::fromList([[ring(3), [[1,2], [3,1,2]]]]), 
		   (SGAN)::fromList([[ring(1), [3]]])], 
		 [(SGA23)::fromList([[ring(1), [[1,2],[1,3,2]]]]), 
		   (SGAN)::fromList([[ring(1), [3]]])]])):
if prog::TestNoTestargs = FALSE then
prog::test(SGA23N::toCartesianProductFromSignature([SGA2, SGA2, SGA2]),
	   TrapError = 1028):
end_if:

///////////////////////////////////////////////////////////////////////
end_for: // for ring in [...] do 
///////////////////////////////////////////////////////////////////////




///////////////////////////////////////////////////////////////////////
// The polynomial optimisation is impossible
SGAL   := Dom::FreeModuleTable(Dom::Rational, combinat::permutations):
SGALL  := Dom::TensorProductOfFreeModules([SGAL, SGAL]):
SGALLL := Dom::TensorProductOfFreeModules([SGAL, SGAL, SGAL]):
//////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////
prog::testfunc("tensorConstructorFromSignature"):

prog::test((TsgaLL := SGALL::tensorConstructorFromSignature([SGAL, SGAL]);
	    iszero(TsgaLL([SGAL([2,1]), SGAL([1,2])]) - 
		   SGALL([[2,1],[1,2]])))):
prog::test(TsgaLL([SGAL::zero, SGAL([1,2])]), 
	   SGALL::zero):
prog::test(TsgaLL([SGAL([2,1]), SGAL::zero]), 
	   SGALL::zero):
prog::test(iszero(TsgaLL([SGAL([2,1]) + 2*SGAL([1]), SGAL([1,2])]) - 
		  (SGALL([[2,1],[1,2]]) + 2*SGALL([[1],[1,2]])))):
prog::test((TsgaLLL := SGALLL::tensorConstructorFromSignature([SGAL, SGAL, SGAL]);
	    iszero(TsgaLLL([SGAL([2,1]), SGAL([1,2]), SGAL([1,2,3])]) - 
		   SGALLL([[2,1],[1,2],[1,2,3]])))):
prog::test((TsgaLLL := SGALLL::tensorConstructorFromSignature([SGAL, SGAL, SGAL]);
	    iszero(TsgaLLL([SGAL([2,1]), SGAL([1,2]), SGAL([1,2,3])]) -
		   SGALLL([[2,1],[1,2],[1,2,3]])))):
prog::test((TsgaLL_L := SGALLL::tensorConstructorFromSignature([SGALL, SGAL]);
	    iszero(TsgaLL_L([SGALL([[2,1], [1,2]]), SGAL([1,2,3])]) -
		   SGALLL([[2,1],[1,2],[1,2,3]])))):
prog::test((TsgaL_LL := SGALLL::tensorConstructorFromSignature([SGAL, SGALL]);
	    iszero(TsgaL_LL([SGAL([2,1]), SGALL([[1,2],[1,2,3]])]) -
		   SGALLL([[2,1],[1,2],[1,2,3]])))):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("toCartesianProduct"):
prog::test(SGALLL::toCartesianProduct(SGALLL::zero),
           []):
x := SGAL([1]): y := SGAL([2]): z3 := SGAL([3]):
prog::test(sort(SGALLL::toCartesianProduct(TsgaLLL([x, y, z3]) + 3*TsgaLLL([y, x, z3]))),
           sort([[SGAL::fromList([[3, [2]]]), SGAL::fromList([[1, [1]]]), SGAL::fromList([[1, [3]]])], 
		 [SGAL::fromList([[1, [1]]]), SGAL::fromList([[1, [2]]]), SGAL::fromList([[1, [3]]])]])):



//////////////////////////////////////////////////////////////////////////////
prog::testfunc("basisIndices::testtype"):
// !!! le dernier argument ne teste pas si la permutations est standard...
prog::test(testtype([[1,2],[2,1]], SGA22::basisIndices)):
prog::test(testtype([[1,2],[2,1,3]], SGA22::basisIndices), FALSE):
prog::test(testtype([[1,2],[2,1,3]], SGA23::basisIndices)):
prog::test(testtype([[1,2],[2,1,3], []], SGA23N::basisIndices)):
prog::test(testtype([[1,2],[2,1,3], [1]], SGA23N::basisIndices)):
prog::test(testtype([[1,2],[2,1,3], [2,1]], SGA23N::basisIndices)):
prog::test(testtype([[1,1],[2,1,3], [2,1]], SGA23N::basisIndices), FALSE):
prog::test(testtype([[1,2],[2,1,2], [2,1]], SGA23N::basisIndices), FALSE):
prog::test(testtype([[1,2],[2,1], [2,1]], SGA23N::basisIndices), FALSE):
prog::test(testtype([[1],[2,1,3], [2,1]], SGA23N::basisIndices), FALSE):


//////////////////////////////////////////////////////////////////////////////
prog::testfunc("basisIndices::list"):
prog::test(SGA22::basisIndices::list(), 
	   [[[1, 2], [1, 2]], [[1, 2], [2, 1]], [[2, 1], [1, 2]], [[2, 1], [2, 1]]]):
prog::test(SGA23::basisIndices::list(), 
	   [[[1, 2], [1, 2, 3]], [[1, 2], [1, 3, 2]], [[1, 2], [2, 1, 3]], 
	    [[1, 2], [2, 3, 1]], [[1, 2], [3, 1, 2]], [[1, 2], [3, 2, 1]], 
	    [[2, 1], [1, 2, 3]], [[2, 1], [1, 3, 2]], [[2, 1], [2, 1, 3]], 
	    [[2, 1], [2, 3, 1]], [[2, 1], [3, 1, 2]], [[2, 1], [3, 2, 1]]]):
prog::test(bool(SGA23N::basisIndices::list(i) =
		combinat::cartesianProduct(combinat::permutations::list(2),
					   combinat::permutations::list(3),
					   combinat::permutations::list(i)))) $ i=0..4:


//////////////////////////////////////////////////////////////////////////////
prog::testfunc("basisIndices::size"):
prog::test(SGA22::basisIndices::size([[1,2],[1,2]]), 0):
prog::test(SGA23::basisIndices::size([[1,2],[1,2,3]]), 0):
prog::test(SGA23N::basisIndices::size([[1,2],[1,2,3],[]]), 0):
prog::test(SGA23N::basisIndices::size([[1,2],[1,2,3],[1]]), 1):
prog::test(SGA23N::basisIndices::size([[1,2],[1,2,3],[3,1,2]]), 3):
prog::test(SGANN::basisIndices::size([[],[]]), 0):
prog::test(SGANN::basisIndices::size([[1],[]]), 1):
prog::test(SGANN::basisIndices::size([[1,2],[1,2,3]]), 5):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc(Dom::TensorProductOfFreeModules):
prog::testexit():
//////////////////////////////////////////////////////////////////////////////

prog::check(SGA23N, 3): // Only this one is checked for efficiency.
prog::check(SGALLL, 3):


// Local Variables:
// mode: mupad
// End:
