/*****************************************************************************
  $Id: TensorProductOfFreeModules.mu,v 1.1.1.2 2005/01/26 13:42:22 nthiery Exp $
  File:		lib/DOMAINS/DOMAIN/TensorProductOfFreeModules.mu
  Authors:	Florent Hivert <florent.hivert@univ-mlv.fr>
  Purpose:	Source code for the domain Dom::TensorProductOfFreeModules
  Created:	Thu Feb  5 20:04:58 2004
  License:	LGPL
  Project:	MuPAD-Combinat, http://mupad-combinat.sf.net
*****************************************************************************/

alias(TPFM      = Dom::TensorProductOfFreeModules):
alias(TPFMTools = Dom::TensorProductOfFreeModulesTools):
alias(declareConversion = operators::overloaded::declareConversion):
alias(declareSignature  = operators::overloaded::declareSignature):
alias(assocTP   = TPFMTools::assocTensorProduct):
alias(REP(x)    = extop(x,1)):

//////////////////////////////////////////////////////////////////////////////
domain TPFMTools
    assocTensorProduct := 
// Given a list of module, replace tensor product of modules by the modules. 
// This can't be placed into to TPFM itself because it is called by the constructor.
    proc(signature : Type::ListOf(DOM_DOMAIN))
    begin
	map(signature, (modu) -> if modu::hasProp(TPFM) then op(modu::modules);
				  else                        modu;
				  end_if);
    end_proc;
    
    defaultPolynomialOptimization := TRUE;
end_domain:
//////////////////////////////////////////////////////////////////////////////
domain TPFMTools::cartesianProduct(classes: Type::ListOf(DOM_DOMAIN))
    local isGraded; // is all the classes are ungraded, then dom is ungraded.
    category if isGraded then 
		 Cat::GradedCombinatorialClass; 
	     else
		 Cat::CombinatorialClass; 
	     end_if;
    inherits if isGraded then 
		 combinat::decomposableObjects
		     ([hold(CartProd) = Alias(Prod(op(map(classes, Domain))), DOM_LIST@op)]);
	     else
		 combinat::subClass(combinat::decomposableObjects
		     ([hold(CartProd) = Alias(Prod(op(map(classes, Domain))), DOM_LIST@op)]),
		     Parameters = 0);
	     end_if;
    axiom Ax::systemRep;
    domtype  := DOM_LIST;
    testtype := z -> testtype(z, Type::ListProduct(op(classes)));
    sizeClasses := map(classes, 
		       proc(cl) 
		       begin 
			   if cl::hasProp(Cat::GradedCombinatorialClass) then 
			       cl::size
			   else
			       0
			   end_if;
		       end_proc);
    size     := if contains(dom::sizeClasses, FAIL) = 0 then
		    proc(x : dom)
		    begin
			_plus(op(zip(dom::sizeClasses, x, fp::apply)));
		    end_proc:
		end_if;
begin
    isGraded := _or(op(map(classes, 
			   cl -> cl::hasProp(Cat::GradedCombinatorialClass))));
end_domain:



//////////////////////////////////////////////////////////////////////////////
domain TPFM(modules : Type::ListOf(DOM_DOMAIN))
//	    polynomialOptimization = TPFMTools::defaultPolynomialOptimization : Type::Boolean)
//////////////////////////////////////////////////////////////////////////////
//    Take a list of modules. If there are tensor product among them, the
// initialisation of the domain rewrite it according to the associativity
// of the tensor product. 
//////////////////////////////////////////////////////////////////////////////
//    When the free modules are implemented by polynomial
// (Dom::FreeModulePoly), we use the polynomial optimization, i.e.: 
// if the modules mod_i are represented by polynomials in n_i variables,
// the tensor product of mod_i is represented by polynomials in 
// _plus(n_i) variables. The variables _X1..._Xn_1 is used to represent
// the first component of the tensor product, _Xn_1+1..._Xn_2, the
// second component and so on.     
//    This allows to compute the tensor product by simple product of 
// polynomials after a suitable renaming of the variables. Another 
// advantage is that we can use the same rankers/unrankers.     
/////
//    If its not possible, we represent tensor product by default 
// FreeModule implmentation and compute the tensor product by 
// associativity and bilinearity, wich is rather slow. 
//////////////////////////////////////////////////////////////////////////////
    
    local indicesFM,   // The list of the basisIndices of the free modules
          basisIndices, R, i, 
    // In case of polynomial optimization, we need also the following
    // informations for each free module. 
          NFM,           // The list of numbers of variables
	  rankersFM,     // The list of the rankers
          unrankersFM,   // The list of the unrankers
	  variablesFM,   // The list of the list variables
	  PolyVariables, // Variables for internal polynomials.
          polynomialOptimization;

    category (if _lazy_and(i::hasProp(Cat::AlgebraWithBasis(R))
			   $ i in modules)then
		   Cat::AlgebraWithBasis(R)
	      else Cat::ModuleWithBasis(R)
	      end_if);
    inherits (if polynomialOptimization then 
		 Dom::FreeModulePoly // polynomial optimization
		       (R, basisIndices,
		        hold(Variables) = PolyVariables,
			hold(MultirankerBasis) = 
			[proc(elts : basisIndices)
			     name multiPolyRanker;
			 begin
			     zip(rankersFM, elts, ((f,z) -> op(f(z))));
			 end_proc,
			 proc(rnk)
			     name multiPolyUnranker;			     
			     local i, j, res;
			 begin
			     i := 0;
			     res := [if j=1 then i := i + 1; rnk[i]
				     else [op(rnk, i+1..(i := i+j))];
				     end_if $ j in NFM];
			     zip(unrankersFM, res, ((f,z) -> (f(z))));
			 end_proc]);
	     else // no polynomial optimization possible
		 Dom::FreeModule(R, basisIndices);
	     end_if);

    info_str := "Dom::TensorProductOfFreeModules -- The domain of Tensor Product of Free Module";

//////////////////////////////////////////////////////////////////////////////
// Basic informations. These are needed outside the domain. 
    basisIndices := basisIndices;
    coeffRing    := R;
    modules      := modules;
//////////////////////////////////////////////////////////////////////////////

// Is it specified that the pritting of a free module element is
// done by printTerm ???
    printTerm := 
    proc(z) 
	begin 
	    hold(slot)(operators, "tensorSymbolic")(op(zip(z, modules, 
					            (el, modu)-> modu::printTerm(el))));
	end_proc;


    tensorConstructorFromSignature := 
    proc(signature : Type::ListOf(DOM_DOMAIN))
	option escape, remember;
	local sig, subsvars, var, i, PolyR, tensRec, tensRec2, sigRec, l1, l2;
    begin
	if testargs() and assocTP(signature) <> modules
	    then
	    error("The tensor product of ".expr2text(op(signature)).
		  " does not live in ".expr2text(dom));
	end_if;
	if polynomialOptimization then // polynomial optimization 
	    i := 0;
            // We build the suitable lists of substitutions of variables, for
	    // each free module. 
	    subsvars := [[var = (hold(_X).(i := i+1)) $ var in op(sig::getRankerParameters(), 4)]
		         $ sig in signature];
	    PolyR := (dom::getPolyRingParameters())[1]; 
	                             // This should be a global variable of this domain
 	                             // but I don't now how to get it from the super domain. 
	    prog::bless(proc(elts : Type::ListProduct(op(signature)))
		name tensorConstructor;
		local res;
	    begin
		res := zip(map(elts, extop), subsvars, subs); // substitute the variables
		res := map(res, poly, PolyVariables, PolyR);  // extend the variables
		new(dom, _mult(op(res)));                     // And compute the product.
	    end_proc, dom);
	else  // no polynomial optimization possible
	    sig := nops(signature);
	    if sig > 2 then 
		sigRec   := [op(signature, 1..sig-1)]; 
		tensRec  := TPFM(sigRec); 
		tensRec2 := dom::tensorConstructorFromSignature([tensRec, signature[sig]]);
		tensRec  := tensRec::tensorConstructorFromSignature(sigRec);
		proc(elts : Type::ListProduct(op(signature)))		    
		begin
		    tensRec2([tensRec([op(elts, 1..sig-1)]), elts[sig]]);
		end_proc;	    
	    else
		if (signature[1])::hasProp(TPFM) then l1 := id else l1 := DOM_LIST end_if;
		if (signature[2])::hasProp(TPFM) then l2 := id else l2 := DOM_LIST end_if;
		operators::makeBilinear
		(prog::bless(proc(t1: (signature[1])::basisIndices, 
				  t2: (signature[2])::basisIndices)
                                    begin
                                        dom::term(l1(t1).l2(t2));
                                    end_proc,
                                    dom),
		 Source1=signature[1],
		 Source2=signature[2],
		 ImageSet=dom)@op;
	    end_if;
	end_if;
    end_proc;

    tensorConstructor := dom::tensorConstructorFromSignature(modules);


    toCartesianProductFromSignature := 
    proc(signature : Type::ListOf(DOM_DOMAIN))
	option escape;
	local varFM, nFM, cutRank, PolyR, PolyOne, DOM;
    begin
	DOM := dom;
	if testargs() and assocTP(signature) <> modules
	    then
	    error("The tensor product of ".expr2text(op(signature)).
		  " does not live in ".expr2text(dom));
	end_if;
// If x = _plus(a_i \tensor b_i \tensor ...) returns the lists of tuples
//           [[a_1,b_1,c_1], [a_2,b_2,c_2], ... ] 
// The coefficients are set on the first part. 
// when possible, avoid dom::poly2list and (un)rankings and use rather 
// poly2list on polynomials. 
	if polynomialOptimization then
	    varFM := map(signature, fp::apply@slot, "getRankerParameters");
	    nFM   := map(varFM, op, 1);	
	    varFM := map(varFM, op, 4);
	    PolyR   := dom::getPolyRingParameters(); //[PolyR, R2polyCoerce, Rcoerce];
	    PolyOne := (PolyR[2])(R::one);
	    PolyR   := PolyR[1];
	    cutRank := 
	    proc(ll)
		name toCartesianProduct::cutPoly2List;
	        local co, i, k;
	    begin
		co := ll[1]; ll := ll[2];
		i := nFM[1];
		[ new(signature[1], poly([[co,      [op(ll, 1..i)]]], 
					 varFM[1], PolyR)),
		 (new(signature[k], poly([[PolyOne, [op(ll, i+1..(i := i+nFM[k]))]]], 
					 varFM[k], PolyR))) $ k=2..nops(nFM)];				 
	    end_proc;
	    proc(el : DOM)
	    begin
		map(poly2list(REP(el)), cutRank);
	    end_proc;
	else
	    proc(el : DOM)
            begin
		map(DOM::poly2list(el),
		    proc(ll)
			name toCartesianProduct::cutPoly2List;
	                local co, k;
		    begin
			co := ll[1]; ll := ll[2];
			[ (signature[1])::monomial(co, ll[1]),
			 ((signature[k])::term(ll[k])) $ k = 2..nops(signature)];
		    end_proc);
	    end_proc;
	end_if;
    end_proc;

    toCartesianProduct := dom::toCartesianProductFromSignature(modules);

    one := if dom::hasProp(Cat::AlgebraWithBasis(R)) then
	       dom::tensorConstructor([i::one $ i in modules]);
	   end_if;

    mult2Basis := 
    if contains([(modules[i])::mult2Basis $ i=1..nops(modules)], FAIL) =0 then
	proc(t1: dom::basisIndices, t2: dom::basisIndices)
	    local i;
	begin
	    dom::tensorConstructor([(modules[i])::mult2Basis(t1[i], t2[i])
				    $ i=1..nops(modules)]);
	end_proc;
    else
	proc(t1: dom::basisIndices, t2: dom::basisIndices)
	    local i;
	begin
	    dom::tensorConstructor([(modules[i])::mult2((modules[i])::term(t1[i]), 
							(modules[i])::term(t2[i]))
				    $ i=1..nops(modules)]);
	end_proc;	
    end_if;


//////////////////////////////////////////////////////////////////////////////
// Initialization, type checking and associativity rewriting. 
begin
    if TRUE then
	// testargs() is never called at the top level and the type checking is never done.
       for i in modules do
	   if not _lazy_and(domtype(i) = DOM_DOMAIN,
			    i::hasProp <> FAIL,
			    i::hasProp(Cat::ModuleWithBasis)
			    $ i in modules) then
	       error("Incorrect arg: " . expr2text(i) .
		     "Can only compute tensor products of Cat::ModuleWithBasis,");
	   end_if;
       end_for;
    end_if;
    R := map({op(modules)}, slot, "coeffRing");
    if // testargs() and 
       nops(R) <> 1 then
	error("Cannot compute tensor product of modules over different rings:\n".expr2text(op(modules)));
    end_if;
    R := op(R,1);
    
// Replace each tensor product of free modules by the free modules themselves. 
// This is done before the creation of the domain and thus this affects the "key" slot
// of the resulting domain.     
    modules := assocTP(modules);
    
// build the domain for the indices. 
    indicesFM    := map(modules, slot, "basisIndices");
    basisIndices := TPFMTools::cartesianProduct(indicesFM);
    
// build the ranking system 
    variablesFM  := map(modules, fp::apply@slot, "getRankerParameters");
// rankerParameters returns [N, Rank, Unrank, PolyVariables];
    polynomialOptimization := TPFMTools::defaultPolynomialOptimization;
    if not polynomialOptimization or contains(variablesFM, FAIL) <> 0 then
	polynomialOptimization := FALSE;
    else
	NFM         := map(variablesFM, op, 1);	
	rankersFM   := map(variablesFM, op, 2);
	unrankersFM := map(variablesFM, op, 3);
	variableFM  := map(variablesFM, op, 4);
	PolyVariables := [hold(_X).i $ i=1.._plus(op(NFM))];
    end_if;
end_domain:

