/*****************************************************************************
  $Id: WeylAlgebra_Table.mu,v 1.1.1.5 2005/01/26 13:42:22 nthiery Exp $
  File:		lib/DOMAINS/DOMAIN/WeylAlgebra_Table.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Created:	25/06/2002
  License:	LGPL
*****************************************************************************/

alias(declareConversion = operators::overloaded::declareConversion):

alias(WAT= Dom::WeylAlgebraTools):

//////////////////////////////////////////////////////////////////////////////
domain WAT
    info_str := "Helper tools for the domain Dom::WeylAlgebraTools";
    
end_domain:

domain WAT::integerVectorsOfLength(N: Type::NonNegInt)
    inherits Dom::BaseDomain;
    category Cat::IntegerListsLexClass, Cat::GradedCombinatorialClass;
    axiom Ax::systemRep;

    parseOptions :=
    proc()
    begin
        dom::parseOptionsIntegerListsLexClass
        (table(Length	= N,
               MinPart	= 0,
               MaxPart	= infinity,
               Inner	= NIL,
               Outer	= NIL,
	       MinSlope	= -infinity,
	       MaxSlope	=  infinity
              ),
         args(1..args(0))
         ):
    end_proc;

end_domain:

//////////////////////////////////////////////////////////////////////////////
domain Dom::WeylAlgebra_Table(Vars: Type::ListOf(DOM_IDENT), R = Dom::Rational)

    local N; // nops(Vars);

    inherits Dom::FreeModule(R, WAT::integerVectorsOfLength(2*N));
    category Cat::AlgebraWithBasis(R);

    info_str := "Domain for the Weyl Algebra, implemented via tables";

    dimen := N;
    
    one := dom::term([ 0 $ 2*N]);
    
    XVariables := Vars;

    DX := x->D(x);
    euler := x->dom(x)*dom(dom::DX(x));
    DXVariables := map(Vars, dom::DX);

    variables := dom::XVariables. dom::DXVariables;

    degreeBasis :=
    proc(t: dom::basisIndices)
    begin
	_plus(op(t, 1..N)) - _plus(op(t, N+1..2*N));
    end_proc;

    orderBasis :=
    proc(t: dom::basisIndices)
    begin
	_plus(op(t, N+1..2*N))
    end_proc;

    order :=
    proc(x: dom)
	local s;
    begin
	max(0,dom::orderBasis(s, args(2..args(0))) $ s in dom::support(x));
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // Conversions and overloading
    //////////////////////////////////////////////////////////////////////////

    initDomainAdd :=
    proc()
    begin
	declareConversion(DOM_EXPR,  dom, dom::fromExpr);
	declareConversion(DOM_IDENT, dom, dom::fromExpr);
    end_proc;

    poly :=
    proc(p: dom) : DOM_POLY
    begin
	poly(dom::toList(p), dom::variables, dom::coeffRing)
    end_proc;

    expr :=
    proc(p: dom)
    begin
	expr(dom::poly(p));
    end_proc;

    print := expr;

    fromPoly :=
    proc(p: DOM_POLY)
    begin
	dom::fromList(poly2list(p));
    end_proc;

    // BUG: the expression dx1*x1 is implicitely normalized into x1 * dx1
    // which is not quite correct.
    fromExpr :=
    proc(e)
    begin
	dom::fromPoly(poly(e, dom::variables, R));
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // Commutation rules and product
    //////////////////////////////////////////////////////////////////////////

    straightenOneVariable :=
    proc(t1: Type::ListProduct(Type::NonNegInt $ 2),
	 t2: Type::ListProduct(Type::NonNegInt $ 2),
	 c = dom::coeffRing::one : dom::coeffRing)
    // computes the normal form of c x^t1[1] d^t1[2] x^t2[1] d^t2[2]
    // as a sum of monomials c x^t[1] d^t[1] + ..., which is as a list
    // [ t = c, ... ]

    // We could use the general formula (see Sturmfels, Grbner Deformations
    // of Hypergeometric Differention Equations, p.3)
    // d^i x^j = \sum_k=0 ^{min(i,j)} \frac{i(i-1)...(i-k+1) j(j-1)...(j-k+1)}{k!} x^{j-k} d^{i-k}
    begin
	if t1[2]=0 or t2[1]=0 then
	    [[t1[1]+t2[1], t1[2]+t2[2]] = c]
	else
	    dom::straightenOneVariable([t1[1], t1[2]-1], [t2[1],  t2[2]+1], c      ) .
	    dom::straightenOneVariable([t1[1], t1[2]-1], [t2[1]-1,t2[2]  ], c*t2[1]);
	end_if;
    end_proc;

    straightenMultiVariable :=
    proc(t1,t2)
	local i,l;
    begin
	_plus(dom::monomial(_mult(op(map(l,op,2))),
			    map(l,op,[1,1]).map(l,op,[1,2]))
	      $ l in combinat::cartesianProduct
		     (dom::straightenOneVariable([t1[i],t1[N+i]],
						 [t2[i],t2[N+i]]) $i=1..N));
    end_proc;

    mult2Basis := dom::straightenMultiVariable;

    //////////////////////////////////////////////////////////////////////////
    // dual / antipode
    //////////////////////////////////////////////////////////////////////////

    // FIXME: check if this is an antipode, and if yes, call it that way!

    adjoint :=
    proc(p: dom)
    begin
	dom::mapterms(p, l -> [op(l, N+1..2*N), op(l,1..N)]);
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // Action on polynomials
    //////////////////////////////////////////////////////////////////////////

    onPoly :=
    proc(w: dom, p)
	local i, m;
    begin
 	_plus(coeff(w,m) *
 	      _mult(dom::XVariables[i]^m[i] $i=1..N) * 
 	      diff(p,(dom::XVariables[i] $ m[N+i]) $i=1..N)
 	      $ m in dom::support(w));
    end_proc;

    func_call :=
    proc(w: dom, p)
    begin
	dom::onPoly(w, context(p));
    end_proc;

begin
    N := nops(Vars);
end_domain:
