/*****************************************************************************
  $Id: CyclesHopfAlgebra.mu,v 1.1.1.2 2005/01/26 13:42:22 nthiery Exp $
  File:		lib/EXAMPLES/CycleHopfAlgebra.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
                Florent Hivert <florent.hivert@univ-mlv.fr>
  Created:	27/03/2003 (first created 25/11/2002 in experimental)
  License:	LGPL
*****************************************************************************/

alias(CHA=examples::CyclesHopfAlgebra):
alias(CHAT=examples::CyclesHopfAlgebraTools):
alias(declareConversion = operators::overloaded::declareConversion):
alias(declareSignature  = operators::overloaded::declareSignature):
alias(Perm=combinat::permutations):


//////////////////////////////////////////////////////////////////////////////
domain CHAT
    info_str := "Helper tools for the domain examples::CyclesHopfAlgebra";
end_domain:

domain CHAT::CyclesDecompositions
    category Cat::DecomposableClass;
    inherits Dom::BaseDomain;
    axiom    Ax::systemRep, Ax::normalRep, Ax::canonicalRep;

    specification := [SetOfCycles = Alias(Set(Alias
					      (Cycle(Alias(Z, op)), 
					       DOM_LIST@op)), 
					  DOM_LIST@op)];

    decomposableClassOptions := [MainNonTerminal = SetOfCycles, 
				 Labelled     = TRUE];

    domtype := DOM_LIST;

    size    := 
    proc(cy : dom)
    begin
	_plus(op(map(cy, nops)));
    end_proc;

    isA     := 
    proc(obj : Type::AnyType) 
    begin 
	testtype(obj, Type::ListOf(Type::ListOf(Type::NonNegInt)));
    end_proc; 
end_domain:

domain CHA(R = Dom::ExpressionField())
    category Cat::AlgebraWithBasis(R);
    inherits Dom::FreeModule(R, CHAT::CyclesDecompositions);

    basisName := G;

    one := dom::term([]);

    printTerm :=
    proc(p)
    begin
        _concat("G|", (_concat("",op(i), "|") $ i in p))
    end_proc;

    melangeAlphMots := 
    proc(cy1: Type::ListOf(Type::NonNegInt), 
	 cy2: Type::ListOf(Type::NonNegInt))
	local alph, len1, len2, std1, std2;
    begin
	len1 := nops(cy1); std1 := Perm::standard(cy1);
	len2 := nops(cy2); std2 := Perm::standard(cy2);
	alph := sort(cy1.cy2);
	map(combinat::splitNK(len1+len2, len1),
	    proc(spl)
		local p1, p2, i;
	    begin
		[p1,p2] := spl;
		p1 := Perm::_mult([alph[i] $ i in p1], std1);
		p2 := Perm::_mult([alph[i] $ i in p2], std2);
		p1.p2;
	    end_proc);
    end_proc;

    shuffleCyclesRec := 
    proc(cySet1: dom::basisIndices, 
	 cySet2: dom::basisIndices)
	local cy1, cy2, res, choice, deb;
    begin
	if   nops(cySet1) = 0 then return([cySet2]);
	elif nops(cySet2) = 0 then return([cySet1]);
	end_if;
	cy1 := cySet1[1];
	res := map(dom::shuffleCyclesRec(cySet1[2..nops(cySet1)], cySet2),
		   z -> [cy1].z);
	for choice from 1 to nops(cySet2) do
	    cy2 := cySet2[choice];
	    deb := map(_concat(dom::melangeAlphMots(cy1[2..nops(cy1)], 
						    [op(cy2, k+1..nops(cy2)), op(cy2, 1..k)])
			       $ k=1..nops(cy2)),
		       z-> [cy1[1], op(z)]);
	    res := res . map(combinat::cartesianProduct
			     (deb,
			      dom::shuffleCyclesRec(cySet1[2..nops(cySet1)], 
						    subsop(cySet2, choice=null()))),
			     lst -> [lst[1],op(lst[2])]);
	end_for;
	res;
    end_proc;

    mult2Basis :=
    proc(cySet1: dom::basisIndices, cySet2: dom::basisIndices)
        local l, cySet2dec, cySet;
    begin
	l         := dom::basisIndices::size(cySet1);
	cySet2dec := map(cySet2, map, _plus, l);
	dom::plus(dom::term(cySet) $ cySet in dom::shuffleCyclesRec(cySet1, cySet2dec));
    end_proc;
    
end_domain:
