/*****************************************************************************
  $Id: FreeQuasiSymmetricFunctionsZero.mu
  File:		lib/EXAMPLES/FreeQuasiSymmetricFunctions.mu
  Authors:	Jean-Christophe Novelli <novelli at univ-mlv.fr>
  Created:	16/09/2004
  License:	LGPL
*****************************************************************************/

alias(FQSym0d=examples::FQSymZd):
alias(FQSym0dT=examples::FQSymZdTools):
alias(Perm=combinat::permutations);
alias(Comp= combinat::compositions);

domain FQSym0dT
    info_str := "Helper tools for the domain
             examples::FQSymZd";
end_domain:

// The combinatorial class of standard permutations
// (just a subset of combinat permutations)
domain FQSym0dT::standardPermutations
    inherits combinat::permutations;
    category Cat::GradedCombinatorialClass;
    axiom    Ax::systemRep;

    isA := dom::isAStandard;
end_domain:

//////////////////////////////////////////////////////////////////////////////
domain FQSym0dT::common(R: DOM_DOMAIN)
    inherits Dom::FreeModule(R, FQSym0dT::standardPermutations);

    info_str := "Domain for dual free quasi-symmetric functions";

    basisName := toBeDefined;
    exprTerm := dom::exprTermIndex;
    oneBasis := [];
    degreeBasis   := _plus@op;
end_domain:

//////////////////////////////////////////////////////////////////////////////
domain FQSym0dT::complete(R: DOM_DOMAIN)
    inherits FQSym0dT::common(R);
    category Cat::GradedHopfAlgebraWithBasis(R);

    info_str := "Domain for co-commutative free quasi-symmetric functions
             expanded on the S basis";

    basisName := hold(S);

    mult2Basis    :=
    proc(perm1: dom::basisIndices,
         perm2: dom::basisIndices)
        local n, m, nu, perm;
    begin
        dom::term(perm1.map(perm2,_plus,nops(perm1)));
    end_proc;

    coproductBasis :=
    proc(perm: dom::basisIndices)
        local i, n, cuts, TS;
    begin
        TS := dom::tensorSquare;
        res := TS::zero;

        cyc := {op(Perm::cycles(perm))};
        for ssenscyc in combinat::subsets(cyc) do
          ssensnb := map(ssenscyc,op);
          res := TS::plus(res, TS::term([
              Perm::standard(select(perm,z->contains(ssensnb,z))),
              Perm::standard(select(perm,z-> not contains(ssensnb,z)))]));
        end_for;
        return(res);
    end_proc;
end_domain:
// Valide JCN sur 321, 1324, 2134, 4321.

//////////////////////////////////////////////////////////////////////////////
domain FQSym0dT::gamma(R: DOM_DOMAIN)
    inherits FQSym0dT::common(R);
    category Cat::GradedHopfAlgebraWithBasis(R);

    info_str := "Domain for free quasi-symmetric functions expanded on the G
basis";

    basisName := hold(gamma);

    melangeAlphMots :=
    proc(cy1: Type::ListOf(Type::NonNegInt),
         cy2: Type::ListOf(Type::NonNegInt))
        local alph, len1, len2, std1, std2;
    begin
        len1 := nops(cy1); std1 := Perm::standard(cy1);
        len2 := nops(cy2); std2 := Perm::standard(cy2);
        alph := sort(cy1.cy2);
        map(combinat::splitNK(len1+len2, len1),
            proc(spl)
                local p1, p2, i;
            begin
                [p1,p2] := spl;
                p1 := Perm::_mult([alph[i] $ i in p1], std1);
                p2 := Perm::_mult([alph[i] $ i in p2], std2);
                p1.p2;
            end_proc);
    end_proc;

    shuffleCyclesRec :=
    proc(cySet1,
         cySet2)
        local cy1, cy2, res, choice, deb;
    begin
        if   nops(cySet1) = 0 then return([cySet2]);
        elif nops(cySet2) = 0 then return([cySet1]);
        end_if;
        cy1 := cySet1[1];
        res := map(dom::shuffleCyclesRec([op(cySet1,2..nops(cySet1))],
                                         cySet2),
                   z -> [cy1].z);
        for choice from 1 to nops(cySet2) do
            cy2 := cySet2[choice];
            deb := map(_concat(dom::melangeAlphMots([op(cy1,2..nops(cy1))],
                                    [op(cy2, k+1..nops(cy2)), op(cy2, 1..k)])
                               $ k=1..nops(cy2)),
                       z-> [cy1[1], op(z)]);
            res := res . map(combinat::cartesianProduct
                             (deb,
                              dom::shuffleCyclesRec(
                                [op(cySet1,2..nops(cySet1))],
                                                    subsop(cySet2,
                                                    choice=null()))),
                             lst -> [lst[1],op(lst[2])]);
        end_for;
        res;
    end_proc;

    mult2Basis :=
    proc(perm1: dom::basisIndices, perm2: dom::basisIndices)
        local l, cySet1, cySet2, cySet2dec, cySet;
    begin
        cySet1 := Perm::cycles(perm1);
        cySet2 := Perm::cycles(perm2);

        l         := nops(perm1);
        cySet2dec := map(cySet2, map, _plus, l);
        res := dom::plus(dom::term(Perm::fromCycles(cySet)) $ cySet in
                         dom::shuffleCyclesRec(cySet1, cySet2dec));
return(res);
    end_proc;

    coproductBasis := 
    proc(perm: dom::basisIndices)
        local i, n, cuts, TS;
    begin
        TS := dom::tensorSquare;
        res := TS::zero;
    
        cyc := {op(Perm::cycles(perm))};
        for ssenscyc in combinat::subsets(cyc) do
          ssensnb := map(ssenscyc,op);
          res := TS::plus(res, TS::term([
              Perm::standard(select(perm,z->contains(ssensnb,z))),
              Perm::standard(select(perm,z-> not contains(ssensnb,z)))]));
        end_for;
        return(res);
    end_proc;

end_domain:

//////////////////////////////////////////////////////////////////////////////
domain FQSym0dT::sz(R: DOM_DOMAIN)
    inherits FQSym0dT::common(R);
    category Cat::GradedHopfAlgebraWithBasis(R);

    info_str := "Domain for co-commutative free quasi-symmetric functions
             expanded on the gamma basis";

    basisName := hold(sz);

    coproduct :=
(FQSym0d(R))::gammat2szt@(FQSym0d(R))::gamma::coproduct@(FQSym0d(R))::gamma;
end_domain:

//////////////////////////////////////////////////////////////////////////////
domain FQSym0d(R=Dom::ExpressionField(): DOM_DOMAIN)
    inherits Dom::BaseDomain;
    category Cat::AlgebraWithSeveralBases(R);

    info_str := "Library for free quasy-symmetric functions";

    interfaceAdd := { hold(complete), hold(S),
                      hold(gamma),
                      hold(sz)};

    complete := FQSym0dT::complete(R); S := dom::complete;
    gamma := FQSym0dT::gamma(R);
    sz := FQSym0dT::sz(R);

    //dualBasesPairs := {[dom::M, dom::S]};

    autoDefineBasisChanges := TRUE;
    
    basisChangesBasis :=
    table(
      (dom::sz, dom::gamma) =
         proc(SZPerm : dom::sz::basisIndices)
         local cuts,i;
         begin
           cuts := [0].Comp::descents(Perm::connectedShape(SZPerm), FinalDescent);
           dom::gamma::mult(dom::gamma::term(Perm::standard(
                [op(SZPerm,cuts[i]+1..cuts[i+1])])) $i =1..nops(cuts)-1);
         end_proc
         );

gammat2szt := operators::tensorProductOfMaps([dom::sz,dom::sz],
[dom::gamma,dom::gamma],dom::sz::tensorSquare);
szt2gammat := operators::tensorProductOfMaps([dom::gammat,dom::gamma],
[dom::sz,dom::sz],dom::gamma::tensorSquare);

initDomainAdd := proc()
begin
  operators::overloaded::declareConversion(dom::sz::tensorSquare,
  dom::gamma::tensorSquare, dom::szt2gammat);
  operators::overloaded::declareConversion(dom::gamma::tensorSquare,
  dom::sz::tensorSquare, dom::gammat2szt);
end_proc;

end_domain:


domain FQSym0dT::Y(R=Dom::ExpressionField(): DOM_DOMAIN)
    category Cat::HopfAlgebraWithBasis(R);
    inherits Dom::FreeModule(R, combinat::partitions);

    info_str := "Library for free quasy-symmetric functions";

    basisName := hold(Y);
    one  := dom::term([]);
    exprTerm := dom::exprTermIndex;

    gamma     := (FQSym0d(R))::gamma;
    fromGamma := operators::makeLinear(dom::term@Perm::cycleType,
				       Source = dom::gamma, ImageSet = dom);
    toGamma   :=
    proc(l : dom::basisIndices)
	local permL, i, j, k, res;
    begin
	i := 0;
	permL := [[((i := i+1);i) $ k=1..j] $ j in l];
	dom::gamma::term(Perm::fromCycles(permL));
    end_proc;
	     
    mult2Basis :=
    proc(l1 : dom::basisIndices, l2 : dom::basisIndices)
	local res;
    begin
	res := dom::gamma::mult2(dom::toGamma(l1),
				 dom::toGamma(l2));
	dom::fromGamma(res);
    end_proc;

    szt2domt := operators::tensorProductOfMaps([dom::fromGamma,dom::fromGamma],
					       dom::gamma::tensorSquare, dom::tensorSquare);
	     
    coproductBasis := dom::szt2domt @ dom::gamma::coproduct @ dom::toGamma;
end_domain:

domain FQSym0dT::m(R=Dom::ExpressionField(): DOM_DOMAIN)
    category Cat::HopfAlgebraWithBasis(R);
    inherits Dom::FreeModule(R, combinat::partitions);

    basisName := hold(m);
    one  := dom::term([]);
    exprTerm := dom::exprTermIndex;

    coproduct := (sym(R))::Yt2mt @ (sym(R))::Y::coproduct @ (sym(R))::Y;
end_domain:


//////////////////////////////////////////////////////////////////////////////
domain sym(R=Dom::ExpressionField(): DOM_DOMAIN)
    inherits Dom::BaseDomain;
    category Cat::AlgebraWithSeveralBases(R);

    info_str := "Library for free quasy-symmetric functions";

    interfaceAdd := { hold(m), hold(Y)};

    m := FQSym0dT::m(R);
    Y := FQSym0dT::Y(R);

    autoDefineBasisChanges := TRUE;
    
    basisChangesBasis :=
    table(
      (dom::Y, dom::m) =
	  proc(YPart : dom::Y::basisIndices)
	      local lambdaM, p;
	  begin
	      lambdaM := combinat::partitions::toExp(YPart);
	      dom::m::monomial(_mult(p! $ p in lambdaM) / _mult((p-1)! $ p in YPart), YPart);
	  end_proc
         );

    Yt2mt := operators::tensorProductOfMaps([dom::m,dom::m],
					    dom::Y::tensorSquare, dom::m::tensorSquare);
    mt2Yt := operators::tensorProductOfMaps([dom::Y,dom::Y],
					    dom::m::tensorSquare, dom::Y::tensorSquare);

    initDomainAdd :=
    proc()
    begin
	operators::overloaded::declareConversion(dom::Y::tensorSquare,
						 dom::m::tensorSquare, dom::Yt2mt);
	operators::overloaded::declareConversion(dom::m::tensorSquare,
						 dom::Y::tensorSquare, dom::mt2Yt);
    end_proc;
    
end_domain:
