/*****************************************************************************
  $Id: FreeQuasiSymmetricFunctions.mu,v 1.1.1.5 2005/01/26 13:42:22 nthiery Exp $
  File:		lib/EXAMPLES/FreeQuasiSymmetricFunctions.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Created:	27/03/2003 (first created 25/11/2002 in experimental)
  License:	LGPL
*****************************************************************************/

alias(FQSym=examples::FreeQuasiSymmetricFunctions):
alias(FQSymT=examples::FreeQuasiSymmetricFunctionsTools):
alias(Perm=combinat::permutations);

domain FQSymT
    info_str := "Helper tools for the domain examples::FreeQuasiSymmetricFunctions";
end_domain:

// The combinatorial class of standard permutations
// (just a subset of combinat permutations)
domain FQSymT::standardPermutations
    inherits combinat::permutations;
    category Cat::GradedCombinatorialClass;
    axiom    Ax::systemRep;

    isA := dom::isAStandard;
end_domain:

//////////////////////////////////////////////////////////////////////////////
domain FQSymT::common(R: DOM_DOMAIN)
    inherits Dom::FreeModule(R, FQSymT::standardPermutations);

    info_str := "Domain for free quasi-symmetric functions";

    basisName := toBeDefined;
    exprTerm := dom::exprTermIndex;
    oneBasis := [];
    degreeBasis   := nops;
end_domain:

//////////////////////////////////////////////////////////////////////////////
domain FQSymT::F(R: DOM_DOMAIN)
    inherits FQSymT::common(R);
    category Cat::GradedHopfAlgebraWithBasis(R);

    info_str := "Domain for free quasi-symmetric functions expanded on the F basis";

    basisName := hold(F);
    mult2Basis :=
    proc(l1: dom::basisIndices, l2: dom::basisIndices)
	local n1, x;
    begin
	n1 := nops(l1);
	_plus(dom::term(x) $ x in combinat::words::shuffle(l1, map(l2, z->z+n1)));
    end_proc;

    coproductBasis :=
    proc(sigma: dom::basisIndices)
        local i;
    begin
        dom::tensorSquare::plus(dom::tensorSquare::term([
             Perm::standard([op(sigma,1..i)]),
             Perm::standard([op(sigma,i+1..nops(sigma))])])
                                 $ i = 0 .. nops(sigma));
    end_proc;

end_domain:

//////////////////////////////////////////////////////////////////////////////
domain FQSymT::G(R: DOM_DOMAIN)
    inherits FQSymT::common(R);
    category Cat::GradedHopfAlgebraWithBasis(R);

    info_str := "Domain for free quasi-symmetric functions expanded on the G basis";

    basisName := hold(G);
    mult2Basis :=
    proc(l1: dom::basisIndices, l2: dom::basisIndices) : dom
	local n1, x;
    begin
	n1 := nops(l1);
	_plus(dom::term(combinat::permutations::inverse(x))
	      $ x in combinat::words::shuffle(combinat::permutations::inverse(l1), map(combinat::permutations::inverse(l2), z->z+n1)));
    end_proc;

    coproductBasis :=
    proc(sigma: dom::basisIndices)
        local i;
    begin
        dom::tensorSquare::plus(dom::tensorSquare::term([
             select(sigma, _leequal, i),
             Perm::standard(select(sigma, not _leequal, i))])
                                 $ i = 0 .. nops(sigma));
    end_proc;

end_domain:

//////////////////////////////////////////////////////////////////////////////
domain FQSymT::complete(R: DOM_DOMAIN)
    inherits FQSymT::common(R);
    category Cat::GradedHopfAlgebraWithBasis(R);

    info_str := "Domain for free quasi-symmetric functions expanded on the G
basis";

    basisName := hold(S);

    mult2Basis := proc(l1: dom::basisIndices, l2: dom::basisIndices) : dom
    begin
        dom::term(map(l1,_plus,nops(l2)).l2);
    end_proc;

    // NT 2004/11/03: letting the overloading mechanism do the job
    // should be more flexible and essentially as efficient
    coproduct := (FQSym(R))::Gt2S@(FQSym(R))::G::coproduct@(FQSym(R))::G;
end_domain:

//////////////////////////////////////////////////////////////////////////////
domain FQSymT::elementary(R: DOM_DOMAIN)
    inherits FQSymT::common(R);
    category Cat::GradedHopfAlgebraWithBasis(R);
    
    info_str := "Domain for free quasi-symmetric functions expanded on the G
basis";

    basisName := hold(E);
    
    mult2Basis := proc(l1: dom::basisIndices, l2: dom::basisIndices) : dom
    begin
        dom::term(l1.map(l2,_plus,nops(l1)));
    end_proc;

    // NT 2004/11/03: letting the overloading mechanism do the job
    // should be more flexible and essentially as efficient
    coproduct := (FQSym(R))::Gt2E@(FQSym(R))::G::coproduct@(FQSym(R))::G;
end_domain:

//////////////////////////////////////////////////////////////////////////////
domain FQSymT::monomial(R: DOM_DOMAIN)
    inherits FQSymT::common(R);
    category Cat::GradedHopfAlgebraWithBasis(R);

    info_str := "Domain for free quasi-symmetric functions expanded on the G
basis";

    basisName := hold(m);

end_domain:

//////////////////////////////////////////////////////////////////////////////
domain FQSym(R=Dom::ExpressionField(): DOM_DOMAIN)
    inherits Dom::BaseDomain;
    category Cat::AlgebraWithSeveralBases(R);

    info_str := "Library for free quasy-symmetric functions";

    interfaceAdd := { hold(F), hold(G),
                      hold(complete), hold(S),
                      hold(elementary), hold(E),
                      hold(monomial), hold(m) };

    F := FQSymT::F(R);
    G := FQSymT::G(R);
    complete := FQSymT::complete(R); S := dom::complete;
    // Note: the E below breaks MuPAD 2.0.0
    elementary := FQSymT::elementary(R); E := dom::elementary;
    monomial := FQSymT::monomial(R); m := dom::monomial;

    dual := dom;
    dualBasesPairs := {[dom::F, dom::G], [dom::S, dom::m]};

    basisChangesBasis :=
    table(
    (dom::F, dom::G) = 
          proc(FPerm : dom::F::basisIndices)
          begin
            dom::G::term(combinat::permutations::inverse(FPerm));
          end_proc,
    (dom::G, dom::F) = 
          proc(GPerm : dom::G::basisIndices)
          begin
            dom::F::term(combinat::permutations::inverse(GPerm));
          end_proc,
    (dom::S, dom::G) = 
          proc(SPerm : dom::S::basisIndices)
          begin
            dom::G::plus(dom::G::term(i) $ i in
                 Perm::smallerPermutohedron(SPerm, Left));
          end_proc,
    (dom::E, dom::G) = 
          proc(EPerm : dom::E::basisIndices)
          begin
            dom::G::plus(dom::G::term(i) $ i in
                 Perm::greaterPermutohedron(EPerm, Left));
          end_proc
         );


Gt2S := operators::tensorProductOfMaps([dom::S,dom::S],
[dom::G,dom::G],dom::S::tensorSquare);
St2G := operators::tensorProductOfMaps([dom::G,dom::G],
[dom::S,dom::S],dom::G::tensorSquare);

Gt2E := operators::tensorProductOfMaps([dom::E,dom::E],
[dom::G,dom::G],dom::E::tensorSquare);
Et2G := operators::tensorProductOfMaps([dom::G,dom::G],
[dom::E,dom::E],dom::G::tensorSquare);

end_domain:
