/*****************************************************************************
  $Id: FreeSymmetricFunctions.mu,v 1.1.1.2 2004/10/04 17:25:13 nthiery Exp $
  File:         lib/EXAMPLES/FreeSymmetricFunctions.mu
  Authors:      Florent Hivert <florent.hivert@univ-mlv.fr>
  Created:      20/03/2003
  License:      LGPL
*****************************************************************************/
// Define ShortCuts
alias(Tab=combinat::tableaux):
alias(Word=combinat::words):
alias(Perm=combinat::permutations):
  
  //plactic class of a standard tableaux
placticClassTab := 
proc(tab : Tab)
begin
    map(Tab::list(Tab::shape(tab)), z -> Tab::invSchensted(tab, z));
end_proc:

isWordStandardTableaux := 
proc(w : Word)
begin
    testtype(Tab::fromWord(w), Tab::typeStandard);
end_proc:

// Free Littlewood-Richardson rule :
//**********************************
// The product of two tableaux t1 t2 is defined as the set of
// tableaux in the shuffle of t1 with the plactic class of t2
FreeLR := 
proc (t1 : Tab::typeStandard, t2 : Tab::typeStandard)
    local m1, t2shifted, k, res;
begin
    m1        := Tab::toWord(t1); k := nops(m1);
    t2shifted := map(t2, map, z -> z+k); // shift of t2
    res       := [];
    for i in placticClassTab(t2shifted) do
	res := append(res, 
		      op(select(Word::shuffle::list(m1, i),
				isWordStandardTableaux)));
    end_for;
    map(res, z -> Tab::insertWord([], z));
end_proc:


domain FreeSymm(R = Dom::ExpressionField())
  inherits experimental::AlgebraWithBasis(R, Tab);
  category Cat::AlgebraWithBasis(R);

  printTerm := z -> Tab::printPretty(z);
  
  one := dom::term([]);

  mult2Basis := (t1,t2) -> dom::plus(dom::term(t) $ t in FreeLR(t1,t2));
begin
end_domain:


predPlax := 
proc(tab)
    local res;
begin
    res := {op(placticClassTab(tab))};
    res := map(res, op @ Perm::predPermutohedron) minus res;
    map(res, z -> Tab::insertWord([], z));
end_proc:

succPlax := 
proc(tab)
    local res;
begin
    res := {op(placticClassTab(tab))};
    res := map(res, op @ Perm::succPermutohedron) minus res;
    map(res, z -> Tab::insertWord([], z));
end_proc:

FreeLRconj := 
proc (t1 : Tab::typeStandard, t2 : Tab::typeStandard)
    local k, t2shifted, mint, maxt, res;
begin
    k := _plus(op(map(t1, nops)));
    t2shifted := map(t2, map, z -> z+k); // shift of t2
    mint  := Tab::insertWord(t1, Tab::toWord(t2shifted));
    maxt  := Tab::insertWord(t2shifted, Tab::toWord(t1));
    res := {op(combinat::transitiveIdeal(predPlax, maxt))} intersect 
	 {op(combinat::transitiveIdeal(succPlax, mint))};
    [op(res)];
end:

listTab := n -> map(combinat::partitions::list(n), op@Tab::list);

BuildGraph := 
proc(n)
    local res, lTab, t, t1;
begin
    lTab := listTab(n);
    res  := Graph::new(lTab, [], Directed);
    for t in lTab do
	for t1 in predPlax(t) do
	    res := Graph::addEdges(res, [[t, t1]]);
	end_for;
    end_for;
    res;
end_proc:
    
null():
