/*****************************************************************************
  $Id: GraphQuasiSymmetricFunctions.mu,v 1.1.1.3 2005/01/26 13:42:22 nthiery Exp $
  File:		lib/EXAMPLES/GraphQuasiSymmetricFunctions.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Purpose:	Source code for the domain examples::GraphQuasiSymmetricFunctions
  Created:	Tue Mar 23 08:53:54 2004
  License:	LGPL
  Project:	MuPAD-Combinat, http://mupad-combinat.sf.net
*****************************************************************************/

export(combinat, Alias,
       generators, compositions, integerMatrices, integerVectors):
export(operators::overloaded, Alias,
       declareConversion, declareSignature):
alias(GQSym=examples::GraphQuasiSymmetricFunctions):
alias(GQSymT=examples::GraphQuasiSymmetricFunctionsTools):

domain GQSymT
    info_str := "Helper tools for the domain examples::GraphQuasiSymmetricFunctions";
    
    interface := { hold(packedMatrices) };
end_domain:

domain GQSymT::packedMatrices
    category Cat::CombinatorialClass;
    inherits Dom::BaseDomain;
    axiom Ax::systemRep;

    isA :=
    proc(m) : DOM_BOOL
    begin
	_lazy_and(integerMatrices::isA(m),
		  integerMatrices::isSquare(m),
		  dom::isPacked(m));
    end_proc;
    
    isPacked :=
    proc(m: integerMatrices) : DOM_BOOL
	local columnSums, i, rowSums;
    begin
	rowSums := integerMatrices::rowSums(m);
	columnSums  := integerMatrices::columnSums(m);
	if nops(columnSums) <> nops(rowSums) then
	    error("Not a square matrix ".expr2text(m));
	end_if;
	for i from 1 to nops(columnSums) do
	    if columnSums[i]=0 and rowSums[i]=0 then
		return(FALSE);
	    end_if;
	end_for;
	TRUE;
    end_proc;
    
    generator :=
    proc(n)
	option escape;
    begin
	generators::pipe
	(// Generators for indegree/outdegree lists for packed integer matrices
	 generators::pipe
	 (// Generator for integer vectors of sum n and length in 0..2n
	  generators::pipe(generators::fromRange(0..2*n),
			   k->integerVectors::generator(n,k)),
	  proc(rowSums)
	      option escape;
	      local part;
	  begin
	      generators::map
	      (integerVectors::generator(n, nops(rowSums),
					 Inner=[if part>0 then 0 else 1 end_if
						$ part in rowSums]),
	       columnSums -> (columnSums, rowSums));
	  end_proc),
	 integerMatrices::generator);
    end_proc;
end_domain:


domain examples::GraphQuasiSymmetricFunctions

    info_str := "examples::GraphQuasiSymmetricFunctions -- The domain of graph quasi symmetric functions";

end_domain:
