/*****************************************************************************
  $Id: HeckeAlgebra.mu,v 1.1.1.2 2005/01/26 13:42:22 nthiery Exp $
  File:         lib/EXAMPLES/HeckeAlgebra.mu
  Authors:      Florent Hivert <florent.hivert@univ-mlv.fr>
  Created:      20/01/2003
  License:      LGPL
*****************************************************************************/

alias(Perm = combinat::permutations):
alias(makeLinear = operators::makeLinear):
alias(HeckeAlgebra = examples::HeckeAlgebra):

/****
 generic Hecke Algebra
**********************************************/
domain HeckeAlgebra(N : Type::NonNegInt,
		    q = hold(q),
		    R = Dom::ExpressionField())

    local q1, q2;
    category Cat::FiniteDimensionalAlgebraWithBasis(R);
    inherits Dom::FreeModule(R, combinat::subClass(Perm, Parameters=N));

    type1N1  := Type::Interval([1, N-1], Type::Integer);

    printTerm := l -> _concat(hold(T),op(l)); 
  
    oneBasis := [$1..N];

    Ti := (i) -> dom::term(subsop([$1..N], i=i+1, i+1=i));

    q  := q1;
    q1 := q1;
    q2 := q2;	     

    algebraGenerators := () -> [dom::Ti(i) $ i=1..N-1];	     
	     
    tiActBasis :=
    proc(perm: dom::basisIndices,
	 i:    dom::type1N1)
	local permI; 
    begin
	permI := subsop(perm, i=perm[i+1], (i+1)=perm[i]);
	if perm[i] < perm[i+1] then
	    dom::term(permI);
	else // Ti^2 = (q1+q2)*Ti -q1*q2
	    dom::plus2(dom::monomial(-q1*q2, permI), 
		       dom::monomial(q1+q2, perm));
	end_if;
    end_proc;
	     
    tiAct := makeLinear(dom::tiActBasis,
			Source=dom, ImageSet=dom);
	     
    mult2Basis := 
    proc(perm1 : dom::basisIndices, perm2 : dom::basisIndices)
	local res, sgn, resPerm; 
    begin
	res := dom::term(perm1);
	for i in Perm::reducedWord(perm2) do
	    res := dom::tiAct(res, i);
	end_for;
	res;
    end_proc;
    
begin
    if domtype(q) = DOM_LIST then
	[q1,q2] := q;
    else
	q1 := q; q2 := q::dom(-1);
    end_if;
end_domain:


