/*****************************************************************************
  $Id: LodayRoncoAlgebra.mu,v 1.1.1.4 2005/01/26 13:42:22 nthiery Exp $
  File:		lib/EXAMPLES/LodayRoncoAlgebra.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
                Florent Hivert <florent.hivert@univ-mlv.fr>
  Created:	20/09/2002
  License:	LGPL
*****************************************************************************/

// TODO:
// - check if the o basis is equivalent to the e or h basis
// - double check all basis changes defined explicitely by transposition
//   and inversions
// - maybe use autoDefineBasisChanges?


alias(BT=combinat::binaryTrees);
alias(LBT=combinat::labelledBinaryTrees):
alias(DT=combinat::binaryTrees::decreasingTree):
alias(words=combinat::words):
alias(Perm=combinat::permutations):
alias(PBT=experimental::PBT):
alias(LRA=examples::LodayRoncoAlgebra):
alias(LRAT=examples::LodayRoncoAlgebraTools):
alias(declareConversion = operators::overloaded::declareConversion):
alias(declareSignature  = operators::overloaded::declareSignature):
alias(BTInvolution=combinat::binaryTrees::recursiveRevertRightBranch):

domain LRAT
    info_str := "Helper tools for the domain examples::LodayRoncoAlgebra";
end_domain:

LRAT::basisChangeMatrix := 
proc(domSource : DOM_DOMAIN,
     domTarget : DOM_DOMAIN,
     size      : Type::AnyType,
     lBas      = null()) // list of the basis elements if known
    local mat, dim, elem, co, i, rank;
begin
    if args(0) = 3 then 
	lBas   := domSource::basisIndices::list(size);
    end_if;
    rank := combinat::rankers::rankFromList(lBas);
    dim  := nops(lBas);
    mat  := Dom::Matrix(domSource::coeffRing)(dim, dim);
    for i from 1 to dim do
	elem := domTarget::poly2list(domTarget(domSource(lBas[i])));
	for co in elem do
	    mat[i, rank(co[2])] := co[1]; // concatenation de vecteurs creux ???
	end_for;
    end_for;
    [mat, lBas, rank];
end_proc:

LRAT::basisChangeInvertMatrix := 
proc(domSource : DOM_DOMAIN,
     domTarget : DOM_DOMAIN,
     size      : Type::AnyType,
     lBas      = null()) // list of the basis elements if known
    option remember;
    local res;
begin
    res := LRAT::basisChangeMatrix(args());
    subsop(res , 1 = _invert(res[1]));
end_proc:

LRAT::basisChangeByMatrixInversion :=
proc(domSource : DOM_DOMAIN,
     domTarget : DOM_DOMAIN)
    option escape;
begin
    proc(el : domSource::basisIndices)
	local i, mat, lBas, rank;
    begin
	[mat, lBas, rank] := LRAT::basisChangeInvertMatrix(domTarget, domSource, 
						     domSource::basisIndices::size(el));
	domTarget::plus(domTarget::multcoeffs(domTarget::term(lBas[i]), mat[rank(el), i])
					     $ i = 1..nops(lBas));
    end_proc;
end_proc:

LRAT::basisChangeTransposeMatrix := 
proc(domSource : DOM_DOMAIN,
     domTarget : DOM_DOMAIN,
     size      : Type::AnyType,
     lBas      = null()) // list of the basis elements if known
    option remember;
    local res;
begin
    res := LRAT::basisChangeMatrix(args());
    subsop(res , 1 = linalg::transpose(res[1]));
end_proc:

LRAT::basisChangeByMatrixTranspositions :=
proc(domSource : DOM_DOMAIN,
     domTarget : DOM_DOMAIN,
     domSourceDual : DOM_DOMAIN,
     domTargetDual : DOM_DOMAIN)
    option escape;
begin
    proc(el : domSource::basisIndices)
	local i, mat, lBas, rank;
    begin
	[mat, lBas, rank] := LRAT::basisChangeTransposeMatrix(domTargetDual, domSourceDual, 
							domSource::basisIndices::size(el));
	domTarget::plus(domTarget::multcoeffs(domTarget::term(lBas[i]), mat[rank(el), i])
					     $ i = 1..nops(lBas));
    end_proc;
end_proc:


domain LRAT::defaultBasis(R)
    // default implementations for all bases
    category Cat::ModuleWithBasis(R);
    inherits Dom::FreeModule(R, BT);

    oneBasis  := dom::basisIndices::zero;
    printTerm := z -> dom::basisName(z);
    exprTerm  := dom::basisName@expr;
    TeXTerm   := z -> expr2text(dom::basisName)."_{".BT::TeX(z)."}";
    basisName := toBedefined;

//     nearestBasis := LRAT::p(R);

//     mult2Basis :=
//     proc(t1: dom::basisIndices, t2: dom::basisIndices)
//     begin
// 	dom(dom::nearestBasis::mult(dom::nearestBasis(dom::term(t1)),
// 			       dom::nearestBasis(dom::term(t2))));
//     end_proc;

//     coproductBasis :=     
//     proc(t: dom::basisIndices)
//     begin
// 	dom::tensorSquare(dom::nearestBasis::coproduct(dom::nearestBasis(dom::term(t))));
//     end_proc;
end_domain:


/////////////////////////////////////////////////////////////////////////////////    
domain LRAT::p(R)
/////////////////////////////////////////////////////////////////////////////////    
    category Cat::GradedHopfAlgebraWithBasis(R);
    inherits LRAT::defaultBasis(R);

    info_str := "Domain for the Loday-Ronco algebra on the fundamental basis";

    basisName := hold(p);

    mult2Basis :=
    proc(t1: dom::basisIndices, t2: dom::basisIndices)
	local t;
    begin
	dom::plus(dom::term(t) $ t in BT::shuffle(t1,t2));
    end_proc;

    coproductBasis :=
    proc(t: dom::basisIndices)
	local class, i, tc, n, tc1, tc2;
    begin
	n := BT::size(t);
	class := PBT::class(t);
	dom::tensorSquare::_plus(
             (tc1 := [op(tc, 1..i)]; tc2 := [op(tc, i+1..n)];
	      if PBT::isCanonical(tc1) and PBT::isCanonical(tc2) then
		  dom::tensorSquare::term([PBT::UP(tc1), PBT::UP(tc2)])
	      else
		  null();
	      end_if) $ tc in class $ i = 0..n);
    end_proc;
end_domain:

/////////////////////////////////////////////////////////////////////////////////    
domain LRAT::complete(R)
/////////////////////////////////////////////////////////////////////////////////    
    category Cat::GradedHopfAlgebraWithBasis(R);
    inherits LRAT::defaultBasis(R);

    info_str := "Domain for the Loday-Ronco algebra on the complete basis";

    basisName := hold(h);

    mult2Basis :=
    proc(t1: dom::basisIndices, t2: dom::basisIndices)
    begin
	dom::term(BT::under(t1,t2));	
    end_proc;
end_domain:

/////////////////////////////////////////////////////////////////////////////////    
domain LRAT::elementary(R)
/////////////////////////////////////////////////////////////////////////////////    
    category Cat::GradedHopfAlgebraWithBasis(R);
    inherits LRAT::defaultBasis(R);

    info_str := "Domain for the Loday-Ronco algebra on the elementary basis";

    basisName := hold(e);

    mult2Basis :=
    proc(t1: dom::basisIndices, t2: dom::basisIndices)
    begin
	dom::term(BT::over(t1,t2));	
    end_proc;
end_domain:

/////////////////////////////////////////////////////////////////////////////////    
domain LRAT::monomial(R)
/////////////////////////////////////////////////////////////////////////////////    
    category Cat::GradedHopfAlgebraWithBasis(R);
    inherits LRAT::defaultBasis(R);

    info_str := "Domain for the Loday-Ronco algebra on the monomial basis";

    basisName    := hold(m);
//    nearestBasis := LRAT::q(R);
end_domain:

/////////////////////////////////////////////////////////////////////////////////    
domain LRAT::forgotten(R)
/////////////////////////////////////////////////////////////////////////////////    
    category Cat::GradedHopfAlgebraWithBasis(R);
    inherits LRAT::defaultBasis(R);

    info_str := "Domain for the Loday-Ronco algebra on the forgotten basis";

    basisName    := hold(v);
//    nearestBasis := LRAT::q(R);
end_domain:

/////////////////////////////////////////////////////////////////////////////////    
domain LRAT::q(R)
/////////////////////////////////////////////////////////////////////////////////    
    category Cat::GradedHopfAlgebraWithBasis(R);
    inherits LRAT::defaultBasis(R);

    info_str := "Domain for the Loday-Ronco algebra on the fundamental dual basis";

    basisName := hold(q);

    mult2Basis :=
    proc(t1: dom::basisIndices, t2: dom::basisIndices)
	local p1, p2, convol;
    begin
	p1 := PBT::canonicalWord(t1);
	p2 := PBT::canonicalWord(t2);
	convol := map(words::shuffle(Perm::inverse(p1), 
				     map(Perm::inverse(p2), _plus, nops(p1))), 
		      Perm::inverse);
	dom::plus(op(map(convol, dom::term@PBT::UP)));
    end_proc;

    coproductBasis :=
    proc(t: dom::basisIndices)
	local canon, n, i, tc1, tc2;
    begin
	canon := PBT::canonicalWord(t);
	n := nops(canon);
	dom::tensorSquare::_plus(
             (tc1 := PBT::UP(select(canon, _leequal, i));
	      tc2 := PBT::UP(select(canon, (z,i) ->(z > i), i));
	      dom::tensorSquare::term([tc1, tc2]))
		   $ i=0..n);
    end_proc;
end_domain:

/////////////////////////////////////////////////////////////////////////////////    
domain LRAT::o(R)
// FIXME: merge? with elementary?
/////////////////////////////////////////////////////////////////////////////////    
    category Cat::GradedHopfAlgebraWithBasis(R);
    inherits LRAT::defaultBasis(R);

    info_str := "Domain for the Loday-Ronco with the over product";

    basisName := hold(o);

    mult2Basis :=
    proc(t1: dom::basisIndices, t2: dom::basisIndices)
    begin
	dom(BT::over(t1,t2));
    end_proc;

    coproductBasis :=
    proc(DOM)
	local Delta, delta, V, VTensorId;
	option escape;
    begin
	// Computing the coproduct requires several subprocedures
	// which call themselves recursively, so we embed everything
	// in a procedure environment.

	V := // Transform t into a right tree by adding a root node
	proc(t: BT) : BT
	begin
	    subsop(BT::one,2=t);
	end_proc;
	VTensorId :=
	operators::makeLinear
	(proc(t: DOM::tensorSquare::basisIndices) : DOM::tensorSquare
	 begin
	     DOM::tensorSquare::term([V(op(t,1)), op(t,2)]);
	 end_proc,
	 Source=(DOM::tensorSquare), ImageSet=(DOM::tensorSquare));
	delta :=
	proc(t: BT) : DOM::tensorSquare
	begin
	    if t=BT::zero then
		DOM::tensorSquare::term([BT::zero, BT::zero]);
	    elif t[1]=BT::zero then
		VTensorId(delta(t[2]))
	    else
		Delta(t[1]) * delta(V(t[2]));
	    end_if;
	end_proc;
	Delta :=
	proc(t: BT) : DOM::tensorSquare
	begin
	    if t=BT::zero then
		DOM::tensorSquare::term([BT::zero, BT::zero]);
	    elif t[1]=BT::zero then
		DOM::tensorSquare::term([BT::zero, t]) + delta(t)
	    else
		Delta(t[1]) * Delta(V(t[2]));
	    end_if;
	end_proc;
    end_proc(dom);
end_domain:


/////////////////////////////////////////////////////////////////////////////////    
//                                                                             //
//                         The Algebra Itself                                  //
//                                                                             //
/////////////////////////////////////////////////////////////////////////////////    
domain LRA(R=Dom::ExpressionField())
    inherits Dom::BaseDomain;
    category Cat::HopfAlgebraWithSeveralBases(R);

    interfaceAdd  := {hold(o), hold(p), hold(q), hold(h), hold(e), 
		   hold(complete), hold(elementary), 
		   hold(monomial), hold(forgotten), hold(m), hold(v) };

    // the following is needed by the category Cat::AlgebraWithSeveralBases(R).
    bases      := {dom::p, dom::q,
		   dom::complete,   dom::monomial,
		   dom::elementary, dom::forgotten,
		   dom::o
		  };

    db         := LRAT::defaultBasis(R);
    p          := LRAT::p(R);
    q          := LRAT::q(R);
    complete   := LRAT::complete(R);
    elementary := LRAT::elementary(R);
    monomial   := LRAT::monomial(R);
    forgotten  := LRAT::forgotten(R);
    o          := LRAT::o(R);
    h          := dom::complete;
    e          := dom::elementary;
    m          := dom::monomial;
    v          := dom::forgotten;

    dual       := dom;
    dualBasesPairs :=
       {[dom::q,          dom::p],
	[dom::complete,   dom::monomial],
	[dom::elementary, dom::forgotten]
       };

// Changes of Basis
/////////////////////////////////////////////////////////////////////////////////
    basisChangesBasis :=
    table(
    (dom::o, dom::p) =
	/**
           The isomorphism between the Loday-Ronco Algebra for the
           over and for the shuffle product:
	   Split the tree in right branches, and take their shuffle product
           Florent ==== This is not the only isomorphism, the h basis does the same.
        **/
    proc(t: BT)
	local branch;
    begin
	_mult(dom::p(branch)
	      $ branch in revert(t::dom::branchSlice(t, Left)))
    end_proc,
    (dom::p, dom::o) = dom::invertBasisChange(dom::o, dom::p),
    
    (dom::p, dom::q) =
    proc(t: BT)
    begin
	dom::q::plus(op(map(PBT::class(t), 
			    dom::q::term@PBT::UP@Perm::inverse)));
    end_proc,
    (dom::q, dom::p) = dom::invertBasisChange(dom::p, dom::q),

    (dom::complete, dom::p) =
    proc(t: BT)
    begin
	dom::p::plus(op(map(select(BT::list(BT::size(t)), PBT::order, t),
			    dom::p::term)));
    end_proc,
    (dom::p, dom::complete) =
	  dom::invertBasisChange(dom::complete, dom::p),
    
    (dom::elementary, dom::p) =
    proc(t: BT)
    begin
	dom::p::plus(op(map(select(BT::list(BT::size(t)), (t1,t2) -> PBT::order(t2,t1), t),
			    dom::p::term)));
    end_proc,
    (dom::p, dom::elementary) =
	  dom::invertBasisChange(dom::elementary, dom::p),

    (dom::q, dom::monomial) =
	  dom::transposeBasisChange
	  (dom::complete, dom::p, dom::q, dom::monomial),
    (dom::monomial, dom::q) =
	  dom::invertBasisChange(dom::q, dom::monomial),

    (dom::q, dom::forgotten) =
	  dom::transposeBasisChange
	  (dom::p, dom::elementary, dom::forgotten, dom::q),
    (dom::forgotten, dom::q) =
	  dom::invertBasisChange(dom::q, dom::forgotten)
    );

end_domain:

// some example of usage.
/////////////////////////
%if FALSE then
// initialisation for the use of the following lines.    
reset():TEXTWIDTH:=80:lra := examples::LodayRoncoAlgebra():
alias(BT=combinat::binaryTrees, PBT=experimental::PBT, Perm=combinat::permutations):
operators::setTensorSymbol("#"):    
    
print(compo, (t:=PBT::UP(Perm::descentsCompositionToMax(compo))), 
      map(lra::p::support(lra::p(lra::h(combinat::binaryTrees::recursiveRevertRightBranch(t)))), op@PBT::class))
$ compo in combinat::compositions::list(4);

// check the realization of the noncommutative complete in PBT.
print(compo, 
      (t:=combinat::binaryTrees::recursiveRevertRightBranch(PBT::UP(Perm::descentsCompositionToMax(compo)))), 
      bool({op(map(lra::p::support(lra::p(lra::h(t))), 
		   op@PBT::class))}
       = {op(map(Perm::fromDescentsComposition(c1), Perm::inverse)) $ c1 in combinat::compositions::fatter(compo)})
     ) $ compo in combinat::compositions::list(4);
    
// puissance de 2 for jc.
print(es, (lra::h(lra::e(es)))) $ es in BT(3);    
print(EE(es),ETOH, (lra::h(lra::e(es)))) $ es in BT(3);    

r := lra::e(BT([1,[],BT::random(5)]));
lra::h(r); lra::h::nterms(%);



print(AntipE(es),ETOH, (lra::h(operators::antipode(lra::p(lra::e(es)))))) $ es in BT(3);    
print(AntipP(es),PTOP, ((operators::antipode(lra::p(es))))) $ es in BT(3);    

print(Delta(lra::h(t)) = operators::coproduct(lra::h(BT(t)))) $ t in BT(4);    
print(Delta(lra::h(t)) = lra::e::tensorSquare(operators::coproduct(lra::h(BT(t))))) $ t in BT(4);
print(Alpha(lra::h(t)) = lra::e(operators::antipode(lra::h(BT(t))))) $ t in BT(4);

lra::p::printTerm := (arbre) -> dom::basisName(arbre);
lra::p::printTerm := (arbre) -> p[op(PBT::canonicalWord(arbre))];
lra::p::exprTerm  := (arbre) -> p[op(PBT::canonicalWord(arbre))];
lra::q::exprTerm  := (arbre) -> q[op(PBT::canonicalWord(arbre))];
lra::p::tensorSquare::TeXTerm := z -> _concat( generate::tex(z[1]), 
					       ("\\otimes", generate::tex(z[i])) $ i = 2..nops(z));    
    
fprint(Unquoted, Text, "dessinTree4", _concat(generate::TeX(_concat(op(map(PBT::canonicalWord(es), expr2text)))=es)."\n" $ es in BT(4)));
    
(lt := map(sort(map(BT::list(3), PBT::canonicalWord), combinat::integerVectors::_less), PBT::UP);
 [b1,b2] := map(bs, z-> slot(lra, expr2text(z)));
 ex := linalg::transpose(op(LRAT::basisChangeMatrix(b1, b2, 3, lt), 1));
 print(NoNL, bs, "\n",generate::TeX(ex),"\n\n\n")) $ bs in [[e,p], [h,p], [e, h],[p, q],[m, q], [v, q]];
    
%end_if;
