/*****************************************************************************
  $Id: MyMultivariatePolynomials.mu,v 1.1.1.3 2005/01/26 13:42:22 nthiery Exp $
  File:		lib/EXAMPLES/MyMultivariatePolynomials.mu
  Authors:	
  Created:	
  License:	LGPL
*****************************************************************************/

alias(MMP  = examples::MultivariatePolynomials):
alias(MMPT = examples::MultivariatePolynomialsTools):

domain MMPT end_domain;
domain MMPT::IntegerVectors(N : Type::NonNegInt)
    category Cat::GradedCombinatorialClass; 
    inherits combinat::subClass(combinat::integerVectors,
				InheritsWithParameters    = ["count", "list", "generator", ["unrank", 1]],
				Parse = (z -> (z, N, args(2..args(0)))), 
				Size  = (z -> _plus(op(z))));
    axiom Ax::systemRep;

    isA := 
    proc(obj          : Type::AnyType, 
	 deg = null() : Type::Union(DOM_NULL, Type::NonNegInt))
    begin
	if args(0) = 1 then 
	    _lazy_and(combinat::integerVectors::isA(obj), nops(obj) = N);
	else combinat::integerVectors::isA(obj, deg, N);
	end_if;
    end_proc;
end_domain:

//////////////////////////////////////////////////////////////////////////////
domain MMP(L : Type::ListOf(DOM_IDENT), 
	   R = Dom::ExpressionField()) 
    category Cat::GradedHopfAlgebraWithBasis(R), 
             Cat::CommutativeRing; // this is necessary to define _negate... Why ???

    inherits Dom::FreeModulePoly(R, MMPT::IntegerVectors(nops(L)),
				 Variables=L,
				 MultirankerBasis=[id, id]);
    info_str := "Domain for multivariate polynomials";

    variables := L;
    exprTerm := z -> _mult(op(zip(L, z, (a,b) -> a^b)));
    one := dom::term([0 $ nops(L)]);

    // Used at least in experimental::Steenrod::PolynomialsWithSkewCoefficientsRight
    mult2Basis := dom::term@((l1, l2) -> zip(l1, l2, _plus));
    mult2 := // rely on the stucture of freemodules.
    proc(l1, l2)
    begin
	new(dom, extop(l1)*extop(l2));
    end_proc;

    mult := // rely on the stucture of freemodules. 
    proc(l1)
	local l;
    begin
	if args(0) = 0 then 
	    dom::zero
	else
	    new(dom, _mult(extop(l) $ l in [args()]));
	end_if;
    end_proc;

    fromPoly :=
    proc(p: DOM_POLY)
    begin
	dom::fromList(poly2list(p));
    end_proc;

    fromExpr :=
    proc(e)
    begin
	dom::fromPoly(poly(e, dom::variables, R));
    end_proc;

    D := 
    proc(vars, pol)
    begin
	new(dom, D(vars,extop(pol)));
    end_proc;

    coproductBasis := 
    proc(monom : dom::basisIndices)
	local res, g, i, dSqTerm, dSqPlus;
    begin
	dSqTerm := dom::tensorSquare::term;
	dSqPlus := dom::tensorSquare::plus2;
	g := stdlib::_orderedMultisetPartitionsGenerator(monom, 2);
	res := dom::tensorSquare::zero;
	while (i := g()) <> FAIL do
	    res := dSqPlus(res, dSqTerm(i));
	end_while;
	res;
    end_proc;
				 
end_domain:

%if FALSE then
reset():TEXTWIDTH:=118:operators::setTensorSymbol("#"):
mmp := examples::MultivariatePolynomials([a,b,c]):
es := mmp::mu @ operators::tensorProductOfMaps([id, mmp::antipode], 
                   mmp::tensorSquare, mmp::tensorSquare) @ mmp::coproduct:
end_if: 
