/*****************************************************************************
  $Id: NonCommutativePolynomials.mu,v 1.1.1.2 2005/01/26 13:42:22 nthiery Exp $
  File:		lib/EXAMPLES/NonCommutativePolynomials.mu
  Authors:	Florent Hivert <florent.hivert@univ-mlv.fr>
  Created:	24/01/2004
  License:	LGPL
*****************************************************************************/

alias(NCPol=examples::NonCommutativePolynomials):
alias(NCPolT=examples::NonCommutativePolynomialsHelperTools):
alias(declareConversion = operators::overloaded::declareConversion):
alias(declareSignature  = operators::overloaded::declareSignature):

domain NCPolT
    info_str := "Helper tools for the domain examples::NonCommutativePolynomials";
end_domain:

//////////////////////////////////////////////////////////////////////////////
domain NCPol(n : combinat::words::typeAlphabet, 
	     R = Dom::ExpressionField() : DOM_DOMAIN)
    // Why not Dom::FreeModule ??????
    inherits experimental::AlgebraWithBasis(R, combinat::subClass(combinat::words, Parameters=n));
    category Cat::AlgebraWithBasis(R);

    interfaceAdd := {hold(shuffle), hold(hook)};

    info_str := "Domain for non commutative polynomials";

    exprTerm := b->(b);

    one := dom::term([]);

    mult2Basis :=
    proc(l1: dom::basisIndices, l2: dom::basisIndices)
	local n1, x;
    begin
	dom::term(l1.l2);
    end_proc;

    shuffle2Basis := 
    proc(l1: dom::basisIndices, l2: dom::basisIndices)
    begin
	dom::plus(op(map(combinat::words::shuffle(l1,l2), dom::term)));
    end_proc;
    shuffle := operators::makeBilinear(dom::shuffle2Basis, Source=dom, ImageSet=dom);

    hook2Basis := 
    proc(l1: dom::basisIndices, l2: dom::basisIndices)
    begin
	dom::plus(dom::term(l1.l2), dom::monomial(-1, l2.l1));
    end_proc;
    hook := operators::makeBilinear(dom::hook2Basis, Source=dom, ImageSet=dom);

    hadamar := 
    proc(a : dom, b : dom)
	local x, supp;
    begin
	supp := {op(dom::support(a))} intersect {op(dom::support(b))};
	dom::plus(dom::monomial(dom::coeff(a,x) * dom::coeff(b,x),x)
		  $ x in supp);
    end_proc;

    circle := 
    proc(w : dom::basisIndices)
	local n, i, k, clCirc;
    begin
	n := nops(w);
	clCirc := {([w[i] $ i=k+1..n].[w[i] $ i=1..k]) $ k=1..n};
	dom::plus(dom::term(i) $ i in clCirc);
    end_proc;

end_domain:
