/*****************************************************************************
  $Id: NonCommutativeSymmetricFunctions.mu,v 1.1.1.4 2005/01/26 13:42:22 nthiery Exp $
  File:		lib/EXAMPLES/NonCommutativeSymmetricFunctions.mu
  Authors:	Florent Hivert <florent.hivert@univ-mlv.fr>
  Created:	21/05/2003
  License:	LGPL
*****************************************************************************/

export(operators::overloaded, Alias,
       declareConversion, declareSignature):
export(combinat, Alias, compositions):

alias(NSym=examples::NonCommutativeSymmetricFunctions):
alias(NSymT=examples::NonCommutativeSymmetricFunctionsTools):
alias(makeLinearConversion(proced, source, target) =
      declareConversion(source, target,
	 operators::makeLinear(proced, Source=source, ImageSet=target))):

//////////////////////////////////////////////////////////////////////////////
domain NSymT
    info_str := "Helper tools for the domain examples::NonCommutativeSymmetricFunctions";
    
    onGenerator := 
    proc(sourceDom, targetDom, procAlg)
	option escape; 
	local procRes;
    begin
	procRes := 
	proc(compo : compositions)
	begin
	    if nops(compo)=0 then targetDom::one;
	    else targetDom::mult(op(map(compo, procAlg)));
		 //_mult(op(map(compo, procAlg)));
	    end_if;
	end_proc;
	operators::makeLinear(procRes, Source=sourceDom, ImageSet=targetDom);
    end_proc;
    
begin
end_domain:

//////////////////////////////////////////////////////////////////////////////
domain NSymT::common(Rng: DOM_DOMAIN)
    inherits Dom::FreeModule(Rng, compositions);
    info_str := "Common stuff for all bases of the noncommutative symmetric functions";

    basisName     := toBeDefined;
    exprTerm      := dom::exprTermIndex;
    oneBasis      := [];
    degreeBasis   := _plus@op;
    mult2Basis := 
    proc(c1: dom::basisIndices, c2: dom::basisIndices)
    begin // most of the basis are multiplicative basis.
	  // This can be overriden if needed. 
	dom::term(c1.c2); 
    end_proc;
    // Question: what's the best way to refer to the other basis ?
    func_call     := (x, alphabet) -> ((NSym(Rng))::L(x))(context(alphabet));
    antipode       := operators::antipode;
end_domain:


//////////////////////////////////////////////////////////////////////////////
domain NSymT::complete(Rng: DOM_DOMAIN)
    inherits NSymT::common(Rng);
    category Cat::GradedHopfAlgebraWithBasis(Rng);
    info_str := "Domain for noncommutative symmetric functions expanded on the complete basis";

    basisName := hold(S);

    coproductBasis := 
    proc(compo : dom::basisIndices) : dom::tensorSquare
	local i, c;
    begin
	if nops(compo)=0 then dom::tensorSquare::term([[], []]);
	else _mult(_plus(dom::tensorSquare::term([[],   [c]]),
			 dom::tensorSquare::term([[i], [c-i]]) $ i=1..c-1,
			 dom::tensorSquare::term([[c], []]))
		  $ c in compo);
	end_if;
    end_proc;
	
    func_call :=
    proc(x: dom, alphabet)
    begin
	alphabet := context(alphabet);
	case type(alphabet)
	  of DOM_LIST do break;
	  of "_plus"  do alphabet := [op(alphabet)]; break;
	  otherwise      alphabet := [args()];
	end_case;
	error("func_call not implemented yet");
    end_proc;
end_domain:

//////////////////////////////////////////////////////////////////////////////
domain NSymT::elementary(Rng: DOM_DOMAIN)
    inherits NSymT::complete(Rng); // reuse 
    
    category Cat::GradedHopfAlgebraWithBasis(Rng);
    info_str := "Domain for noncommutative symmetric functions expanded on the elementary basis";

    basisName := hold(L);

    func_call :=
    proc(x: dom, alphabet)
    begin
	alphabet := context(alphabet);
	case type(alphabet)
	  of DOM_LIST do break;
	  of "_plus"  do alphabet := [op(alphabet)]; break;
	  otherwise      alphabet := [args()];
	end_case;
	error("not implemented");
    end_proc;
end_domain:

//////////////////////////////////////////////////////////////////////////////
domain NSymT::ribbon(Rng: DOM_DOMAIN)
    inherits NSymT::common(Rng);
    
    category Cat::GradedHopfAlgebraWithBasis(Rng);
    info_str := "Domain for noncommutative symmetric functions expanded on the ribbon basis";

    basisName := hold(R);

    mult2Basis := 
    proc(c1: dom::basisIndices, c2: dom::basisIndices)
	local lc1, lc2, pc1, pc2;
    begin 
	lc1 := nops(c1); lc2 := nops(c2);
	if   lc1 = 0 then dom::term(c2);
	elif lc2 = 0 then dom::term(c1);
	else
	    pc1 := [op(c1,1..lc1-1)];
	    pc2 := [op(c2,  2..lc2)];
	    dom::term(c1.c2) + dom::term(pc1.[c1[lc1]+ c2[1]].pc2); 
	end_if;
    end_proc;
end_domain:

//////////////////////////////////////////////////////////////////////////////
domain NSymT::powerSum1(Rng: DOM_DOMAIN)
    inherits NSymT::common(Rng);
    category Cat::GradedHopfAlgebraWithBasis(Rng);
    info_str := "Domain for noncommutative symmetric functions expanded on the".
	  "power-sum of the first kind basis";
    basisName := hold(Psi);
    coproductGenerator := c -> dom::tensorSquare::plus2(dom::tensorSquare::term([[],   [c]]),
							 dom::tensorSquare::term([[c], []]) );
    coproduct := NSymT::onGenerator
		    (dom, dom::tensorSquare, dom::coproductGenerator);
end_domain:

//////////////////////////////////////////////////////////////////////////////
domain NSymT::powerSum2(Rng: DOM_DOMAIN)
    inherits NSymT::powerSum1(Rng);
    category Cat::GradedHopfAlgebraWithBasis(Rng);
    info_str := "Domain for noncommutative symmetric functions expanded on the".
	  "power-sum of the second kind basis";
    basisName := hold(Phi);
end_domain:

//////////////////////////////////////////////////////////////////////////////
domain NSym(Rng=Dom::ExpressionField(): DOM_DOMAIN)
    inherits Dom::BaseDomain;
    category Cat::HopfAlgebraWithSeveralBases(Rng);

    info_str := "Library for the Noncommutative Symmetric Functions";

    interfaceAdd := {
		hold(S), hold(complete),
		hold(L), hold(elementary),
		hold(R), hold(ribbon),
		hold(Psi), hold(powerSum1),
		hold(Phi), hold(powerSum2)};

    Sym            := examples::SymmetricFunctions(Rng);
    dual           := examples::QuasiSymmetricFunctions(Rng);

    complete       := NSymT::complete(Rng);
    S              := dom::complete;
    elementary     := NSymT::elementary(Rng);
    L              := dom::elementary;
    ribbon         := NSymT::ribbon(Rng);
    R              := dom::ribbon;
    powerSum1      := NSymT::powerSum1(Rng);
    Psi            := dom::powerSum1;
    powerSum2      := NSymT::powerSum2(Rng);
    Phi            := dom::powerSum2;

    bases := {dom::complete, dom::elementary,
	      dom::ribbon,
	      dom::powerSum1, dom::powerSum2};

    basisChangesBasis :=
    table(
    (dom::complete, dom::elementary) =
	  proc(comp: compositions) : dom::elementary local n, c;
	  begin
	      n := _plus(op(comp));
	      _plus(dom::elementary::monomial((-1)^(n-nops(c)), c) 
		    $ c in compositions::finer(comp));
	  end_proc,
    (dom::elementary, dom::complete) =
	  proc(comp: compositions) : dom::complete local n, c;
	  begin
	      n := _plus(op(comp));
	      _plus(dom::complete::monomial((-1)^(n-nops(c)), c) 
		    $ c in compositions::finer(comp));
	  end_proc,

    (dom::complete, dom::ribbon) =
          proc(comp: compositions) : dom::ribbon local n, c;
	  begin
	      n := nops(comp);
	      _plus(dom::R::term(c) 
		    $ c in compositions::fatter(comp));
	  end_proc,
    (dom::ribbon, dom::complete) =
	  proc(comp: compositions) : dom::complete local n, c;
	  begin
	      n := nops(comp);
	      _plus(dom::complete::monomial((-1)^(nops(c)-n), c) 
		    $ c in compositions::fatter(comp));
	  end_proc,

    (dom::elementary, dom::ribbon) =
	  proc(comp: compositions) : dom::ribbon local conj, c;
	  begin
	      conj := revert(compositions::conjugate(comp));
	      _plus(dom::R::term(c)
		    $ c in compositions::finer(conj));
	  end_proc,
    (dom::ribbon, dom::elementary) =
	  proc(comp: compositions) : dom::elementary local conj, n, c;
	  begin
	      conj := revert(compositions::conjugate(comp)); 
	      n := nops(conj);
	      _plus(dom::L::monomial((-1)^(nops(c)-n), c)
		    $ c in compositions::fatter(conj));
	  end_proc);

    // TODO: conversions to and from powersums

    initDomainAdd :=
    proc()
        option remember;
        option escape;
        local DOM;
    begin
	DOM:=dom;

	// commutative image.
	makeLinearConversion(comp -> DOM::Sym::e(revert(sort(comp))),
			     dom::L, dom::Sym::e);
	makeLinearConversion(comp -> DOM::Sym::h(revert(sort(comp))),
			     dom::S, dom::Sym::h);
    end_proc;

end_domain:
