/*****************************************************************************
  $Id: QuasiSymmetricFunctions.mu,v 1.1.1.3 2005/01/26 13:42:22 nthiery Exp $
  File:		lib/EXAMPLES/QuasiSymmetricFunctions.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
                Florent Hivert <florent.hivert@univ-mlv.fr>
  Created:	27/03/2003 (first created 25/11/2002 in experimental)
  License:	LGPL
*****************************************************************************/

alias(Comp=combinat::compositions);
alias(QSym=examples::QuasiSymmetricFunctions):
alias(QSymT=examples::QuasiSymmetricFunctionsTools):
alias(declareConversion = operators::overloaded::declareConversion):
alias(declareSignature  = operators::overloaded::declareSignature):

//////////////////////////////////////////////////////////////////////////////
domain QSymT
    info_str := "Helper tools for the domain examples::QuasiSymmetricFunctions";
end_domain:

//////////////////////////////////////////////////////////////////////////////
domain QSymT::common(R: DOM_DOMAIN)
    inherits Dom::FreeModule(R, combinat::compositions);
    info_str := "Common stuff for all bases of the quasi-symmetric functions";

    basisName     := toBeDefined;
    exprTerm      := dom::exprTermIndex;
    oneBasis      := [];
    degreeBasis   := _plus@op;
    // Question: what's the best way to refer to the other basis ?
    func_call     := (x, alphabet) -> ((QSym(R))::m(x))(context(alphabet));
    antipode       := operators::antipode;
end_domain:

//////////////////////////////////////////////////////////////////////////////
domain QSymT::monomial(R: DOM_DOMAIN) // dual of NCSF::complete
    inherits QSymT::common(R);
    category Cat::GradedHopfAlgebraWithBasis(R), Cat::CommutativeRing;
    info_str := "Domain for quasi-symmetric functions expanded on the monomial basis";

    basisName := hold(m);

    mult2Basis :=
    proc(c1: dom::basisIndices, c2: dom::basisIndices)
	local c;
    begin
	dom::plus(dom::term(c) $ c in combinat::words::shuffleAugmented(c1,c2));
    end_proc;

    coproductBasis := // coconcatenation
    proc(c: dom::basisIndices)
	local i;
    begin
	dom::tensorSquare::plus(dom::tensorSquare::term([[op(c,1..i)], [op(c,i+1..nops(c))]])
				 $ i = 0 .. nops(c));
    end_proc;

    func_call :=
    proc(x: dom, alphabet)
	local f, m;
    begin
	alphabet := context(alphabet);
	case type(alphabet)
	  of DOM_LIST do break;
	  of "_plus"  do alphabet := [op(alphabet)]; break;
	  otherwise      alphabet := [args()];
	end_case;
	f :=
	proc(m: dom::basisIndices)
	    local degrees;
	begin
	    if nops(m)>nops(alphabet) then
		// This should return the zero of the domain of the alphabet
		return(0)
	    end_if;
	    _plus(_mult(op(zip(alphabet, degrees, _power)))
		  $ degrees in combinat::words::shuffle
			       (m, [0$nops(alphabet)-nops(m)]));
	end_proc;
	_plus(op(m,1)*f(op(m,2)) $ m in poly2list(x));
    end_proc;
end_domain:


//////////////////////////////////////////////////////////////////////////////
domain QSymT::quasiRibbon(R: DOM_DOMAIN) // Dual of NCSF::ribbon
    inherits QSymT::common(R);
    category Cat::GradedHopfAlgebraWithBasis(R), Cat::CommutativeRing;
    info_str := "Domain for quasi-symmetric functions expanded on the quasi-ribbon basis";

    basisName := hold(F);

    mult2Basis :=
    proc(c1: dom::basisIndices, c2: dom::basisIndices) : dom
	local c, p1, p2, k;
    begin
	p1 := combinat::permutations::descentsCompositionToMin(c1);
	p2 := combinat::permutations::descentsCompositionToMin(c2);
	k  := nops(p1);
	p2 := map(p2, _plus, k);
	dom::plus(dom::term(combinat::permutations::descentsComposition(c)) $ 
		  c in combinat::words::shuffle(p1, p2));
    end_proc;

    coproductBasis :=
    proc(c: dom::basisIndices) : dom::tensorSquare
	local i, j;
    begin
	// Split a ribbon with n cells into two ribbons made
	// respectively of the k first and n-k last cells,
	// for k from 0 to n
	dom::tensorSquare::plus
	(dom::tensorSquare::term([[],c]),
	 (dom::tensorSquare::term([[op(c,1..i-1),j], [c[i]-j,op(c,i+1..nops(c))]])
	  $ j = 1 .. c[i]-1,
	  dom::tensorSquare::term([[op(c,1..i)], [op(c,i+1..nops(c))]]))
	 $ i = 1 .. nops(c));
    end_proc;

end_domain:

//////////////////////////////////////////////////////////////////////////////
domain QSymT::psi(R: DOM_DOMAIN) // dual of NCSF::Psi
    inherits QSymT::common(R);
    category Cat::GradedHopfAlgebraWithBasis(R), Cat::CommutativeRing;
    info_str := "Domain for quasi-symmetric functions expanded on the psi basis";

    basisName := hold(psi);

    mult2Basis :=
    proc(c1: dom::basisIndices, c2: dom::basisIndices)
	local c;
    begin
	dom::plus(dom::term(c) $ c in combinat::words::shuffle(c1,c2));
    end_proc;

    coproductBasis := // coconcatenation
    proc(c: dom::basisIndices)
	local i;
    begin
	dom::tensorSquare::plus(dom::tensorSquare::term([[op(c,1..i)], [op(c,i+1..nops(c))]])
			       $ i = 0 .. nops(c));
    end_proc;
end_domain:

//////////////////////////////////////////////////////////////////////////////
domain QSymT::phi(R: DOM_DOMAIN) // dual of NCSF::Phi
    inherits QSymT::common(R);
    category Cat::GradedHopfAlgebraWithBasis(R), Cat::CommutativeRing;
    info_str := "Domain for quasi-symmetric functions expanded on the phi basis";

    basisName := hold(phi);

    mult2Basis :=
    proc(c1: dom::basisIndices, c2: dom::basisIndices)
	local c;
    begin
	dom::plus(dom::term(c) $ c in combinat::words::shuffle(c1,c2));
    end_proc;

    coproductBasis := // coconcatenation
    proc(c: dom::basisIndices)
	local i;
    begin
	dom::tensorSquare::plus(dom::tensorSquare::term([[op(c,1..i)], [op(c,i+1..nops(c))]])
				 $ i = 0 .. nops(c));
    end_proc;
end_domain:

//////////////////////////////////////////////////////////////////////////////
domain QSym(R=Dom::ExpressionField(): DOM_DOMAIN)
    inherits Dom::BaseDomain;
    category Cat::HopfAlgebraWithSeveralBases(R);
    axiom    Ax::systemRep;
	
    info_str := "The domain of quasi symmetric functions";

    interfaceAdd:={hold(psi), hold(phi), hold(tensor),
                hold(m), hold(monomial), hold(F), hold(quasiRibbon)};


    monomial       := QSymT::monomial(R);
    m              := dom::monomial;
    psi            := QSymT::psi(R);
    phi            := QSymT::phi(R);
    quasiRibbon    := (QSymT::quasiRibbon(R));
    QuasiRibbon    := dom::quasiRibbon; // For backward compatibility
    F              := dom::quasiRibbon;
    
    convert :=
    proc(el)
        local res;
    begin
        res := eval(subs(el,
                        [hold(m)  = dom::m::domainWrapper,
                         hold(F)  = dom::F::domainWrapper]));
    end;

    autoDefineBasisChanges := TRUE;
    basisChangesBasis :=
    table(
    (dom::m, dom::F) = 
	  proc(mComp : dom::m::basisIndices)
	      local c;
	  begin
	      dom::F::plus(dom::F::monomial((-1)^(nops(c)-nops(mComp)), c)
			   $ c in Comp::finer(mComp));
	  end_proc,
    (dom::F, dom::m) =
	  proc(FComp : dom::F::basisIndices)
	      local c;
	  begin
	      dom::m::plus(dom::m::term(c)
			   $ c in Comp::finer(FComp));
	  end_proc,
/*    (dom::m, dom::psi) =
	  proc(mComp : dom::m::basisIndices)
	  begin
	      toBeDefined
	  end_proc,
    (dom::psi, dom::m) =
	  proc(mComp : dom::psi::basisIndices)
	  begin
	      toBeDefined
	  end_proc,
    (dom::m, dom::phi) =
	  proc(mComp : dom::m::basisIndices)
	  begin
	      toBeDefined
	  end_proc,
*/
    (dom::phi, dom::m) =
	  proc(phiComp : dom::phi::basisIndices)
	      local c, cut, p;
	  begin
	      dom::m::plus((cuts := [0].Comp::descents(c, FinalDescent);
	                    dom::m::monomial(1/_mult(p! $ p in c),
			                     [_plus(op(phiComp, cuts[i]+1..cuts[i+1]))
					     $ i=1..nops(cuts)-1]))
			   $ c in Comp::list(nops(phiComp)));
	  end_proc
	 );

		      
end_domain:
