/*****************************************************************************
  $Id $
  File:		lib/EXAMPLES/SubsetsSpace.mu
  Authors:	Florent Hivert <florent.hivert@univ-mlv.fr>
  Purpose:	Source code for the domain examples::SubsetsSpace
  Created:	Sun Feb  1 17:38:19 2004
  License:	LGPL
  Project:	MuPAD-Combinat, http://mupad-combinat.sf.net
*****************************************************************************/
//////////////////////////////////////////////////////////////////////////
// This file demonstrate how to build a free module with several bases. //
//////////////////////////////////////////////////////////////////////////

// First we define some shortcut : 
alias(SubSp  = examples::SubsetsSpace);      // The name of the space itself.
alias(SubSpT = examples::SubsetsSpaceTools); // The name of the tools placeholder.

// The domain SubSpt=examples::SubsetsSpaceTools is just a placeholder
// for containing the different bases. These bases must be created *before*
// the module SubsetsSpace itself so they cannot simply be placed in it. 
// It can be also used to contain the combinatorial class of the indices, 
// if it cannot be directly build (this is not the case here). 
domain SubSpT
    info_str := "Helper tools for the domain 'examples::SubsetsSpace'";
end_domain:

// We define now the module expanded if the different bases. 
// The fundamental basis. It will be declared as Self Dual.
domain SubSpT::Fundamental(TheSet, Ring)
    category Cat::AlgebraWithBasis(Ring);
    inherits Dom::FreeModule(Ring, 
			     combinat::subClass(combinat::subsets, Parameters = TheSet));
// Place here the methods particular to this basis. 
    info_str := "Domain for the subset space on the fundamental basis";
    basisName := hold(F);

    oneBasis := TheSet;	     
	     
    mult2Basis := 
    proc(s1: dom::basisIndices, s2: dom::basisIndices)
    begin
	dom::term(s1 intersect s2);
    end_proc;
end_domain:


// Another basis. We will furter declare the following basis change :
//   In_Set = \sum_{X \subset Set} F_X 
domain SubSpT::In(TheSet, Ring)
    category Cat::AlgebraWithBasis(Ring);
    inherits Dom::FreeModule(Ring, 
			     combinat::subClass(combinat::subsets, Parameters = TheSet));
    info_str := "Domain for the subset space on the 'In' basis";
    basisName := hold(In);
//  The product is computed by overloading so there is nothing to do.
end_domain:


// The dual basis of In. The basis change is
//    Out_Set = \sum_{Set\subset X} (-1)^(nops(X)-nops(Set)) F_X 
// It will be computed automatically. 
domain SubSpT::Out(TheSet, Ring)
    category Cat::AlgebraWithBasis(Ring);
    inherits Dom::FreeModule(Ring, 
			     combinat::subClass(combinat::subsets, Parameters = TheSet));
    info_str := "Domain for the subset space on the 'Out' basis";
    basisName := hold(Out);
//  The product is computed by overloading so there is nothing to do.
end_domain:


// The definition of the module itself. 
///////////////////////////////////////
// We now place all these stuff together, and declare the basis changes. 
domain SubSp(TheSet : DOM_SET, Ring = Dom::Rational : DOM_DOMAIN)
    category Cat::ModuleWithSeveralBases(Ring);
    inherits Dom::BaseDomain;
    
    info_str := "examples::SubsetsSpace -- Example d'utilisation de 'AlgebraWithSeveralBases'";

// Place here the bases. 
    Fundamental := SubSpT::Fundamental(TheSet, Ring);
    In          := SubSpT::In(TheSet, Ring);
    Out         := SubSpT::Out(TheSet, Ring);
// some useful shortcuts for bases.  
    F           := dom::Fundamental;

// We declare known the basis changes.
    autoDefineBasisChanges := TRUE; // The undefined basis change are computed by 
        // tansposition or inversion of matrices. For experimental purposes. 

    basisChangesBasis := table(
        (dom::In, dom::F) = // subsetInABC(Set) = \sum_{X \subset Set} X 
	    proc(set : dom::In::basisIndices) : dom::F
		local xSet;
	    begin 
		dom::F::plus(dom::F::term(xSet) $ 
				xSet in combinat::subsets::list(set));
	    end_proc);

// We deal now with the duality scalar product.
    dual := dom; // The dual of this space. 
// The pairs of dual bases. 
// The pair [dom::Out, dom::In] will be automatically declared.
    dualBasesPairs := {[dom::F, dom::F], [dom::In, dom::Out]};

end_domain:

/////////////////////////
// Thats all folks !!! //
/////////////////////////
