/*****************************************************************************
  $Id: SymmetricFunctions.mu,v 1.1.1.5 2005/01/26 16:56:04 nthiery Exp $
  File:		lib/EXAMPLES/SymmetricFunctions.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
		Florent Hivert <florent.hivert@univ-mlv.fr>
  Created:	??/11/2002
  License:	LGPL
*****************************************************************************/

/*
TODO:
 - Implement s2m and s2forgotten using lrcalc; those are the dual
   conversions from h2s and e2s.
   s_lambda = \sum_mu C^tau(mu)_{lambda, sigma(mu)} m_mu,
   with tau and sigma being the inner and outer shape of the ribbon
   mu_1,\dots,m_k.
 - Implement a pure MuPAD LR-calculator
 - Implement schur::multBasis using lrcalc::skew
 - Implement a pure MuPAD product of m
   => conversions p->m, h->m, and e->m
 - Implement a pure MuPAD s->e or s->h, the other being computed
   by antipode. Use the determinental formula?

Comment: The conversions s2e from Symmetrica is so slow that it's even
worth doing the stupid linear algebra...

for N from 1 to 12 do
L:=combinat::partitions::list(N):
print(N,time(_concat(S::m::sparseVector(S::m(S::e(p)), L) $ p in L)^-1))
end
                                     1, 10

                                      2, 0

                                     3, 10

                                     4, 40

                                     5, 90

                                     6, 300

                                     7, 800

                                    8, 2730

                                    9, 8760

                                   10, 32160

                                   11, 117470

                                   12, 475750
*/


export(combinat, Alias, permutations):
alias(Sym=examples::SymmetricFunctions):
alias(SymT=examples::SymmetricFunctionsTools):
alias(partitions = combinat::partitions):
alias(declareConversion = operators::overloaded::declareConversion):
alias(declareSignature  = operators::overloaded::declareSignature):

//////////////////////////////////////////////////////////////////////////////
domain SymT
    info_str := "Helper tools for the domain examples::SymmetricFunctions";
end_domain:

//////////////////////////////////////////////////////////////////////////////
domain SymT::common(R: DOM_DOMAIN)
    inherits Dom::FreeModule(R, partitions);
    info_str := "Common stuff for all bases of the symmetric functions";

    basisName    := toBeDefined;
    printTerm    := dom::exprTermIndex;
    exprTerm     := dom::exprTermIndex;
    oneBasis     := [];
    degreeBasis  := _plus@op;
    // Question: what's the best way to refer to the other basis ?
    func_call    := (x, alphabet) -> ((Sym(R))::p(x))(context(alphabet));
    antipode      := operators::antipode;
    omega        := operators::omega;
end_domain:

//////////////////////////////////////////////////////////////////////////////
domain SymT::powersum(R: DOM_DOMAIN)
    inherits SymT::common(R);
    category Cat::GradedHopfAlgebraWithBasis(R), Cat::CommutativeRing;
    info_str := "Domain for symmetric functions expanded on the powersum multiplicative basis";

    basisName := hold(p);

    straightenBasis :=
    proc(l: Type::ListOf(Type::PosInt)) // p[0] does not exists!
    begin
	dom::term(revert(sort(l)));
    end_proc;

    mult2Basis := dom::term@revert@sort@_concat;

    // Coproduct defined by multiplicativity from the primitive elements
    coproductBasis :=
    proc(partition: dom::basisIndices) : dom::tensorSquare
	local k;
    begin
	if nops(partition)=0 then
	    dom::tensorSquare::term([[], []]);
	else
	    dom::tensorSquare::mult(dom::tensorSquare::term([[k], []]) +
				    dom::tensorSquare::term([[], [k]])
				    $ k in partition);
	end_if;
    end_proc;

	
    ////////////////////////////////////////////////////	
    // TODO? define func_call via an overloaded operator
    func_call :=
    proc(x: dom, alphabet)
	local a, m, i;
    begin
	alphabet := context(alphabet);
	case type(alphabet)
	  of DOM_LIST do break;
	  of "_plus" do
	    alphabet := [op(alphabet)]; break;
	  of dom do
	     return(dom::plethysm(x, alphabet));
	  otherwise
	      alphabet := [args()];
	end_case;
	// This should return the zero of the domain of the alphabet
	// when x is empty
	_plus(op(m,1)*_mult(_plus(a^i $ a in alphabet) $ i in op(m,2))
	      $ m in poly2list(x));
    end_proc;

    lambdaBasis := // This should replace func_call in the future. 
    proc(x         : dom::basisIndices, 
	 alphabet, 
	 variables = null() : Type::Union(DOM_NULL, Type::ListOf(DOM_IDENT), Type::SetOf(DOM_IDENT)))
	local alph, i;
    begin
	if args(0) = 2 then 
	    variables := indets(alphabet);
	end_if;
// 	if testargs() and 
// 	   _lazy_or(not testtype(alphabet, Type::RatExpr(var)) $ var in variables) then
// 		    error(expr2text(alphabet)." has to be a rational expression in ".expr2text(variables));
// 	end_if;
	if not has(alphabet, variables) then 
	    alphabet^nops(x)
	elif nops(x) > 1 then 
	    _mult(dom::lambdaBasis([i], alphabet, variables) $ i in x);
	else
	    case type(alphabet)
		of "_plus" do
		    _plus(dom::lambdaBasis(x, alph, variables) $ alph in alphabet); break;
		of "_mult" do
		    if op(alphabet,1) = -1 then // WARNING !!! A-B <> A+(-1)*B
			-_mult(dom::lambdaBasis(x, alph, variables) $ alph in alphabet); break;
		    else
			_mult(dom::lambdaBasis(x, alph, variables) $ alph in alphabet); break;
		    end_if;
		of "_power" do
		    _power(dom::lambdaBasis(x, op(alphabet,1), variables), op(alphabet,2)); break;
		of DOM_IDENT do
		    alphabet^x[1]; break; // x is suppose to have only one part. 
		otherwise
		    error("unknown type of alphabet :".expr2text(type(alphabet)));
	    end_case;
	end_if;
    end_proc;

    lambda := operators::makeLinear(dom::lambdaBasis, Source=dom, Multcoeffs=_mult);

    plethysmBasis := 
    proc(x : dom::basisIndices, y : dom)
	local i,z;
    begin
	dom::mult(dom::plus(dom::monomial(z[1], map(z[2], _mult, i))
			    $ z in dom::poly2list(y))
		  $ i in x);
    end_proc;

    plethysm := operators::makeLinear(dom::plethysmBasis, Source=dom, ImageSet=dom);

end_domain:

//////////////////////////////////////////////////////////////////////////////
domain SymT::complete(R: DOM_DOMAIN)
    inherits SymT::common(R);
    category Cat::GradedHopfAlgebraWithBasis(R), Cat::CommutativeRing;
    info_str := "Domain for symmetric functions expanded on the complete multiplicative basis";

    basisName := hold(h);

    straightenBasis :=
    proc(l: Type::ListOf(Type::NonNegInt))
    begin
	dom::term(revert(sort(select(l, not iszero))));
    end_proc;

    mult2Basis := dom::term@revert@sort@_concat;
	
    // Coproduct defined by multiplicativity from the primitive elements
    coproductBasis :=
    proc(partition: dom::basisIndices) : dom::tensorSquare
	local k, i;
    begin
	if nops(partition)=0 then
	    dom::tensorSquare::term([[], []]);
	else
	    dom::tensorSquare::mult(
               dom::tensorSquare::plus(
	           dom::tensorSquare::term([[k], []]),
	           dom::tensorSquare::term([[], [k]]),
	           dom::tensorSquare::term([[i], [k-i]])
				       $ i=1..k-1)
				    $ k in partition);
	end_if;
    end_proc;
	
end_domain:

//////////////////////////////////////////////////////////////////////////////
domain SymT::elementary(R: DOM_DOMAIN)
    inherits SymT::complete(R);
    category Cat::GradedHopfAlgebraWithBasis(R), Cat::CommutativeRing;
    info_str := "Domain for symmetric functions expanded on the elementary multiplicative basis";

    basisName := hold(e);

end_domain:

//////////////////////////////////////////////////////////////////////////////
domain SymT::monomial(R: DOM_DOMAIN)
    inherits SymT::common(R);
    category Cat::GradedHopfAlgebraWithBasis(R), Cat::CommutativeRing;
    info_str := "Domain for symmetric functions expanded on the monomial basis";

    basisName := hold(m);

    straightenBasis :=
    proc(l: Type::ListOf(Type::NonNegInt))
    begin
	dom::term(revert(sort(select(l, not iszero))));
    end_proc;

    // Disabled by Nicolas; doesn't the overloading mechanism take care of it?
    //mult2Basis := (x,y) -> dom(_mult(SymT::Schur(R)(dom::term(x)), SymT::Schur(R)(dom::term(y))));

end_domain:

//////////////////////////////////////////////////////////////////////////////
domain SymT::Schur(R: DOM_DOMAIN, Options: Type::OptionsTable)
    inherits SymT::common(R);
    category Cat::GradedHopfAlgebraWithBasis(R), Cat::CommutativeRing;
    info_str := "Domain for symmetric functions expanded on the Schur basis";

    basisName := hold(s);

    straightenBasis :=
    proc(l: Type::Union(combinat::integerVectors, combinat::skewPartitions)) : dom
	local i, rho;
    begin
	if nops(l)=2 and domtype(l[1])=DOM_LIST then // input is a skew partition then
	    if combinat::lrcalc = FAIL then
		error("lrcal dynamic module required to straighten skew schur functions");
	    end_if;
	    dom::fromList(combinat::lrcalc::skew(op(l)));
	else
	    if not combinat::partitions::isA(l) then
		// Straightening following Jacobi's rule
	        // TODO: optimize!
		l := revert(l):
		rho := [i $ i = 0..nops(l)-1];
		l := zip(l, rho, _plus);
		if nops({op(l)}) <> nops(l) then
		    dom::zero;
		else
		    dom::monomial(permutations::signFromInversionsNumber(l),
				  revert(select(zip(sort(l), rho, _subtract), not iszero)));
		end_if;
	    else
		dom::term(l);
	    end_if;
	end_if;
    end_proc;

    mult2Basis :=
    if Options[Lrcalc] then
	dom::fromList @ combinat::lrcalc::mult
    elif Options[hold(Symmetrica)] then
	(x,y) -> eval(subs(Symmetrica::LRrSf(x,y),
			   hold(s)=dom::domainWrapper));
    end_if;

    coproductBasis :=
    if Options[Lrcalc] then
    proc(partition: dom::basisIndices) : dom::tensorSquare
    begin
	dom::tensorSquare::fromList(combinat::lrcalc::coprod(partition));
    end_proc;
    end_if;

end_domain:



//////////////////////////////////////////////////////////////////////////////
/// Symmetric functions on the HallLittlewood P basis
/// P_lambda(X, 1) = m_lambda(X)
/// P_lambda(X, 0) = s_lambda(X)
/// Computations are currently done in a very slow manner.
domain SymT::HallLittlewood(R: DOM_DOMAIN)
    inherits SymT::common(R);
    category Cat::GradedHopfAlgebraWithBasis(R), Cat::CommutativeRing;
    // Bug: S::P([1,2,3]) does not complain nor straighten the partition

    info_str := "Domain for symmetric functions expanded on the HallLittlewood P basis";

    basisName := hold(P);
end_domain:

//////////////////////////////////////////////////////////////////////////////
/// Symmetric functions on the Hall-Littlewood Q' basis
domain SymT::QP(R: DOM_DOMAIN)
    inherits SymT::common(R);
    category Cat::GradedHopfAlgebraWithBasis(R), Cat::CommutativeRing;
    info_str := "Domain for symmetric functions expanded on the HallLittlewood Q' basis basis";
/// This is the dual of the P basis for the usual scalar product.
/// Q'_lambda(X, 1) = h_lambda(X)
/// Q'_lambda(X, 0) = s_lambda(X)
/// Computations are currently done in a very slow manner.

    // Bug: S::QP([1,2,3]) does not complain nor straighten the partiiton

    basisName := hold(QP);
	
    straightenBasis :=
    proc(l: Type::Union(combinat::partitions, combinat::skewPartitions)) : dom
      local i, nu, weight;
    begin
      if nops(l)=2 and domtype(l[1])=DOM_LIST  // The partition is a skew partition
	then
	weight := combinat::skewPartitions::rowLenghts(l);
	_plus( dom::scalar(dom(l[1]),  (Sym(R))::P(l[2])*(Sym(R))::P(nu)  )*dom(nu) $ nu in combinat::partitions::list(_plus(weight[i] $ i=1..nops(weight)) ) );
      else
	dom::term(l);
      end_if;
    end_proc;
	
end_domain:

//////////////////////////////////////////////////////////////////////////////
// Symmetric functions on functions G basis
// For each level these functions are indexed by partitions
// Function built with spin statistic
domain SymT::HSpin(SymR, k: Type::NonNegInt)
    local R;    
    inherits SymT::common(SymR::coeffRing);
    category Cat::GradedHopfAlgebraWithBasis(SymR::coeffRing), Cat::CommutativeRing;
	info := "Domain for symmetric functions expanded on H function basis built with spin";
	basisName := hold(Hs);
end_domain:
  
//Function built with cospin statistic
 domain SymT::HCospin(SymR, k: Type::NonNegInt)
    local R;    
    inherits SymT::common(R);
    category Cat::GradedHopfAlgebraWithBasis(R), Cat::CommutativeRing;
    info := "Domain for symmetric functions expanded on H function basis built with spin";

    basisName := hold(Hcs);
end_domain:
 
//////////////////////////////////////////////////////////////////////////////
//                                                                          //
//                                                                          //
// The algebra of symmetric functions itself                                //
//                                                                          //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

domain Sym(R=Dom::ExpressionField(): DOM_DOMAIN)
    local Options;
    inherits Dom::BaseDomain;
    category Cat::HopfAlgebraWithSeveralBases(R), Cat::CommutativeRing;
	
	info_str := "Library for symmetric functions";
	
	interfaceAdd:={hold(antipode), hold(omega), hold(fromPoly),
		       hold(p), hold(e), hold(s), hold(h), hold(m), hold(P), hold(QP), 
		       hold(powersum), hold(elementary), hold(Schur), hold(complete), hold(monomial), 
		       hold(HallLittlewood), hold(HSpinBase),hold(HCospinBase)
		      };
	
	options := Options;
	
	varHallLittlewood := hold(t);
	
	powersum         := SymT::powersum(R);
	p                := dom ::powersum;
	elementary       := SymT::elementary(R);
	e                := dom ::elementary;
	Schur            := SymT::Schur(R, Options);
	s                := dom ::Schur;
	complete         := SymT::complete(R);
	homogeneous      := SymT::complete(R); // backward compatibility
	h                := dom ::homogeneous;
	monomial         := SymT::monomial(R);
	m                := dom ::monomial;
	HallLittlewood   := SymT::HallLittlewood(R);
	P                := dom ::HallLittlewood;
	QP               := SymT::QP(R);  
	declareHSpin     := k->SymT::HSpin(dom,k);
	declareHCospin   := k->SymT::HCospin(dom,k);
	
	bases := {dom::p, dom::e, dom::s, dom::h, dom::m, dom::P, dom::QP};

	dual             := dom;
	dualBasesPairs   := {[dom::s,        dom::s],
			     [dom::h,        dom::m],
			     [dom::QP,       dom::P]};
	                          
	
	basisChangesBasis :=
	table(
	      (dom::s, dom::p) = proc(part: combinat::partitions) 
				 begin
				   if part=[] then
				     dom::p::one;
				   else
				     dom::p(eval(subs(Symmetrica::s2p(part),
						      hold(p)=dom::p::domainWrapper)))
				   end_if;
				 end_proc,
	      (dom::p, dom::s) = proc(part: combinat::partitions) 
				 begin
				   if part=[] then
				     dom::s::one;
				   else
				     dom::s(eval(subs(Symmetrica::p2s(part),
						      hold(s)=dom::s::domainWrapper)))
				   end_if;
				 end_proc,
	      
	      (dom::s, dom::e) = proc(part: combinat::partitions)
				   option remember;
				 begin
				   if part=[] then
				     dom::e::one;
				   else
				     dom::e(eval(subs(Symmetrica::s2e(part),
						      hold(e)=dom::e::domainWrapper)))
				   end_if;
				 end_proc,
	      (dom::e, dom::s) = proc(part: combinat::partitions)
				   option remember;
				 begin
				   if part=[] then
				     dom::s::one;
				   else
				     dom::s(eval(subs(Symmetrica::e2s(part),
						      hold(s)=dom::s::domainWrapper)))
				   end_if;
				 end_proc,
	      
	      (dom::s, dom::h) = proc(part : combinat::partitions)
				 begin
				   if part=[] then
				     dom::h::one;
				   else
				     dom::h(eval(subs(Symmetrica::s2h(part),
						      hold(h)=dom::h::domainWrapper)));
				   end_if;
				 end_proc,
	      (dom::h, dom::s) = proc(part: combinat::partitions) : dom::s
				   local i;
				 begin 
				   if Options[Lrcalc] then
	                                 // Build the skew partition with disconnected rows of boxes
	                                 // of size part[1], part[2], ...
	                                 // Extract this as a function combinat::skewPartition::???
				     for i from nops(part)-1 downto 1 do
				       part[i] := part[i+1] + part[i];
				     end_for;
				     dom::s::fromList(combinat::lrcalc::skew(part, [op(part,2..nops(part))]));
				   elif Options[hold(Symmetrica)] then
				     if part=[] then
				       dom::s::one;
				     else
				       dom::s(eval(subs(Symmetrica::h2s(part),
							hold(s)=dom::s::domainWrapper)))
				     end_if;
				   else
				     () -> error("h2s not implemented in pure MuPAD")
				   end_if;
				 end_proc,
	      
	      (dom::s, dom::m) = proc(part : combinat::partitions)
				 begin
				   if part=[] then
				     dom::m::one;
				   else
				     dom::m(eval(subs(Symmetrica::s2m(part),
						      hold(m)=dom::m::domainWrapper)));
				   end_if;
				 end_proc,
	      (dom::m, dom::s) =  proc(part : combinat::partitions)
				  begin
				    if part=[] then
				      dom::s::one;
				    else
				      dom::s(eval(subs(Symmetrica::m2s(part),
						       hold(s)=dom::s::domainWrapper)));
				    end_if;
				  end_proc,
	
	      (dom::QP, dom::s) = proc(part) 
				    local mu;
				  begin
				    _plus(combinat::tableaux::kostkaPol(mu, part, dom::varHallLittlewood)
					  * dom::s(mu)
					  $ mu in partitions::list(_plus(op(part))));
				  end_proc,
	      (dom::s, dom::QP) = dom::invertBasisChange(dom::QP, dom::s),
	      
	      (dom::s, dom::P)  = dom::transposeBasisChange(dom::QP, dom::s, dom::s, dom::P),	  
	      (dom::P, dom::s)  = dom::invertBasisChange(dom::s, dom::P)
				 
	     );
	hChangesBasis := (l , f) -> proc(part: combinat::partitions, level=l, function = f ) 
				    local tmp, sup, list,i;
				  begin
				    tmp := part;
				    if part=[]
				      then dom::s::one;
				    else
				      part    := [part[i]*level $i=1..nops(part)];
				      sup     := _plus(op(part)) / level;
				      list    := select(combinat::partitions::list(sup),
							x->combinat::partitions::dominate(x,tmp)=TRUE);
				      tmp     := dom::s(_plus( function(part, list[i], level) *dom::m(list[i]) $i=1..nops(list)));
				      return(_plus(coeff(tmp,list[i])*dom::s(list[i]) $i=1..nops(list) ));
				    end_if;
				  end_proc;
	
	//declaring Hs and Hcs as procedure which declare a new basis as described in ModuleWithSeveralBasis
	HSpinBase   := proc(k: Type::NonNegInt)
			 local HFunctionDom;
		       begin
		   // building the domain
			 HFunctionDom := dom::declareHSpin(k);
		   // adding this domain into the bases lists
			 dom::declareBasis(HFunctionDom);
		   // adding new changes of basis
			
			 dom::declareBasisChangeBasis(HFunctionDom, dom::s, dom::hChangesBasis(k, combinat::ribbonsTableaux::spinPolynom));
			 dom::declareBasisChangeBasis(dom::s, HFunctionDom, dom::invertBasisChange(HFunctionDom, dom::s));
			 return(HFunctionDom);
		       end_proc;
	HCospinBase   := proc(k: Type::NonNegInt)
			 local HFunctionDom;
		       begin
		   // building the domain
			 HFunctionDom := dom::declareHCospin(k);
		   // adding this domain into the bases lists
			 dom::declareBasis(HFunctionDom);
		   // adding new changes of basis
			
			 dom::declareBasisChangeBasis(HFunctionDom, dom::s, dom::hChangesBasis(k, combinat::ribbonsTableaux::cospinPolynom));
			 return(HFunctionDom);
		       end_proc;
	
	
	zero  := dom::e::zero;
	one   := dom::e::one;
	_plus :=
	proc()
	// try to make dom::e::zero the zero of all the bases
	  local ar;
	begin
	  ar := select([args()], not _equal, dom::zero);
	  if nops(ar) = 0 then dom::zero
	  else operators::_plus(op(ar));
	  end_if;
	end_proc;
	_mult :=
	proc()
	// try to make dom::e::one the one of all the bases
	  local ar;
	begin
	  ar := select([args()], not _equal, dom::one);
	  if nops(ar) = 0 then dom::one
	  else operators::_mult(op(ar));
	  end_if;
	end_proc;
	_invert := _invert;
	_negate := _negate;
	expr    := expr;
	iszero  := iszero;
	print   := print;	
	
	convert := 
	proc(el)
	  local res;
	begin
	  res := eval(subs(el, 
			   [hold(p)   = dom::p::domainWrapper,
			    hold(e)   = dom::e::domainWrapper,
			    hold(s)   = dom::s::domainWrapper,
			    hold(h)   = dom::h::domainWrapper,
			    hold(m)   = dom::m::domainWrapper,
			    hold(P)   = dom::P::domainWrapper,
			    hold(QP)  = dom::QP::domainWrapper //,
			    // Those can't possibly work! (NT 2005-01-21)
			    //hold(Hs)  = dom::Hs::domainWrapper,
			    //hold(Hcs) = dom::Hcs::domainWrapper
				       ]));
	end;
	
	
	scalar := operators::scalar;
	antipode := operators::antipode;
	
	omega :=	// algebra homomorphism
	       operators::overloaded
	       (()->FAIL,
		table([dom::e]=operators::makeLinear
			       (dom::h::term,
				Source=dom::e, ImageSet=dom::h),
		      [dom::h]=operators::makeLinear
			       (dom::e::term,
				Source=dom::h, ImageSet=dom::e),
		      [dom::p]=operators::makeLinear
			       (dom::p::monomial@(part->((-1)^(_plus(op(part))-nops(part)),part)),
				Source=dom::p, ImageSet=dom::p),
		      [dom::s]=operators::makeLinear
			       (dom::s::term@partitions::conjugate,
				Source=dom::s, ImageSet=dom::s)));

    // TODO: make plethysm a standard operator in operators::overloaded
    // as soon as it is used in some other algebra of ours, and then
    // do the overloading declaration in dom::p::initDomainAdd
    // DESIGN ME: name it as "compose" instead? overload @ with _fconcat?
    // TODO: call the gordan dynamic module for elementary/complete functions

    // TODO
	plethysm :=
	operators::overloaded
	(()->FAIL,
	 table([dom::p, dom::p] = dom::p::plethysm));
	
	fromPoly :=
	proc(p: DOM_POLY) : dom
	begin
	  dom::m::fromList
	  (map(select(poly2list(p),
		      t -> partitions::isA(t[2], MinPart=0)),
	       t -> [t[1], select(t[2], not _equal, 0)]));
	end_proc;
	
    ////////////////////////////////////////////////////////////////////////////////////
    // Changes of basis as automaticaly defined in the ModuleWithSeveralBases         //
    ///////////////////////////////////////////////////////////////////////////////////

	initDomainAdd := proc()
			option remember;
			option escape;
		      begin
		
	////////////////////////////////////////////////////////////////////////
	// antipode
	
			declareSignature
			(operators::antipode, [dom::p],
			 operators::makeLinear
			 (dom::p::monomial@(part->((-1)^nops(part),part)),
			  Source=dom::p, ImageSet=dom::p));
			declareSignature
			(operators::antipode, [dom::s],
			 operators::makeLinear
			 (dom::s::monomial@(part->((-1)^_plus(op(part)),partitions::conjugate(part))),
			  Source=dom::s, ImageSet=dom::s));
			declareSignature
			(operators::antipode, [dom::h],
			 operators::makeLinear
			 (dom::e::monomial@(part->((-1)^_plus(op(part)),part)),
			  Source=dom::h, ImageSet=dom::e));
			declareSignature
			(operators::antipode, [dom::e],
			 operators::makeLinear
			 (dom::h::monomial@(part->((-1)^_plus(op(part)),part)),
			  Source=dom::e, ImageSet=dom::h));

	////////////////////////////////////////////////////////////////////////
	// plethysm

	// TODO: define an overloaded operator operators::plethysm
	// call gordan when appropriate


	////////////////////////////////////////////////////////////////////////
	// scalar product
		
			declareSignature
			(operators::scalar, [dom::s,dom::s],
			 proc(f,g) : R  // should be generalized into ModuleWithBasis as standardScalarProduct
			   local t;
			 begin
			   _plus(conjugate(coeff(f,op(t,2))) *op(t,1) $ t in poly2list(g));
			 end_proc);
			
			//dom::initDomainModuleWithSeveralBases();
		      end_proc;
    begin
      Options := combinat::getOptions([hold(Symmetrica) = bool(Symmetrica<>FAIL),
				       Lrcalc           = bool(combinat::lrcalc<>FAIL)],
				      [hold(Symmetrica) = DOM_BOOL,
				       Lrcalc           = DOM_BOOL],
				      1);
end_domain:












