/*****************************************************************************
  $Id: SymmetricGroupAlgebra.mu,v 1.1.1.5 2005/01/26 13:42:22 nthiery Exp $
  File:         lib/EXAMPLES/SymmetricGroupAlgebra.mu
  Authors:      Florent Hivert <florent.hivert@univ-mlv.fr>
  Created:      20/01/2003
  License:      LGPL
*****************************************************************************/

//     reset(); read ("SymmetricGroupAlgebra.mu"); 
//  // Algebra of SG_3
//     alias(SGA3 = SymmetricGroupAlgebra(3));
//  // Algebra of SG_infinity
//     alias(SGA = SymmetricGroupAlgebra());

alias(permutations = combinat::permutations):
alias(tableaux = combinat::tableaux):
alias(SGA=examples::SymmetricGroupAlgebra):
alias(SGAT=examples::SymmetricGroupAlgebraTools):

domain SGAT
    info_str := "Helper tools for the domain examples::SymmetricGroupAlgebra";
end;

domain SGAT::standardPermutations
    inherits permutations;
    category Cat::CombinatorialClass;
    axiom    Ax::systemRep;

    isA := isAStandard;
end_domain:

domain SGA(N = infinity : Type::Union(Type::NonNegInt, Type::Singleton(infinity)),
			     R = Dom::ExpressionField())

    category if N=infinity then
		 Cat::AlgebraWithBasis(R);
	     else
		 Cat::FiniteDimensionalAlgebraWithBasis(R);
	     end_if;
    inherits if N=infinity then
		 Dom::FreeModule(R, SGAT::standardPermutations)
	     else
		 Dom::FreeModulePoly(R, combinat::subClass(permutations, Parameters = N),
				      Variables = N,
				      MultirankerBasis = [id, id]);
	     end_if;

    interfaceAdd := {
		  hold(fromPermutation), hold(fromReducedWord),
		  hold(onPermutationLeftMatrix), hold(onPermutationRightMatrix),
		  hold(YoungSymmetrizer)
		 };

    basisName := hold(A);

    N := N;

    printTerm :=
    if N<10 then
	perm -> _concat(hold(p),op(perm));
    end_if;

    exprTerm := perm -> hold(_index)(dom::basisName, op(perm));

    indexed := combinat::indexedProcedure(dom::term@DOM_LIST);

    one := dom::term([$1..N]);

    mult2Basis := dom::term @ permutations::_mult;

    onPolynomial := toBeImplemented;

    fromPermutation := dom::term;
    fromReducedWord := dom::term @ permutations::fromReducedWord @ (z->(z,N));

    onPermutationLeftBasis :=
    proc(x: dom::Basis, p: permutations) : permutations
    begin
	permutations::_mult(x,p):
    end_proc;

    onPermutationLeftMatrix :=
	experimental::makeMatrixFromPermutationAction
	(dom, dom::basisIndices(), dom::onPermutationLeftBasis);

    onPermutationRightBasis :=
    proc(x: dom::Basis, p: words) : words
    begin
	permutations::_mult(p,permutations::inverse(x)):
    end_proc;

    onPermutationRightMatrix :=
	experimental::makeMatrixFromPermutationAction
	(dom, dom::basisIndices(), dom::onPermutationRightBasis);

    YoungSymmetrizer :=
    proc(t: tableaux::typeStandardFilling, direction=Right) : dom
	local p, x;
    begin
	if testargs() and _plus(nops(x) $ x in t) <> N then
	    error("tableau of size ".N." expected; got ". expr2text(t));
	end_if;
	if direction=Right then
	    _plus(dom::term(p)
		  $ p in tableaux::rowStabilizer(t)) *
	    _plus(dom::term(p) * permutations::sign(p)
		  $ p in tableaux::columnStabilizer(t))
	else
	    _plus(dom::term(p) * permutations::sign(p)
		  $ p in tableaux::columnStabilizer(t)) *
	    _plus(dom::term(p)
		  $ p in tableaux::rowStabilizer(t))
	end_if;
    end_proc;
end_domain:
