/*****************************************************************************
  $Id: SymmetricFunctions.tst,v 1.1.1.5 2005/01/26 13:42:22 nthiery Exp $
  File:		lib/EXAMPLES/TEST/SymmetricFunctions.tst
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Purpose:	Test file for the library 'examples::SymmetricFunctions'
  Created:	24/02/2001
  License:	LGPL
*****************************************************************************/

prog::testinit("SymmetricFunctions"):
S := examples::SymmetricFunctions():

prog::testfunc(S::e):
prog::test(expr(S::e([0,3,2,5])), e[5,3,2]):

prog::testfunc(S::h):
prog::test(expr(S::h([0,3,2,5])), h[5,3,2]):

prog::testfunc(S::m):
prog::test(expr(S::m([0,3,2,5])), m[5,3,2]):

prog::testfunc(S::s):
prog::test(expr(S::s([3,2,0,0])), s[3,2]):

prog::test(S::s([]),
           S::s::fromList([[1, []]])):
prog::test(S::s([1]),
           S::s::fromList([[1, [1]]])):
prog::test(S::s([2, 1]),
           S::s::fromList([[1, [2, 1]]])):
prog::test(S::s([1, 2]),
           S::s::fromList([])):
prog::test(S::s([1, 3]),
           S::s::fromList([[-1, [2, 2]]])):
prog::test(S::s([7, 0, 0]),
           S::s::fromList([[1, [7]]])):
prog::test(S::s([0, 0, 7]),
           S::s::fromList([[1, [5, 1, 1]]])):
prog::test(S::s([7, 3, 0, 9, 4]),
           S::s::fromList([[-1, [7, 7, 4, 3, 2]]])):
prog::test(S::s([0,3,2,5]), S::s::fromList([])):

prog::testfunc(S::p):
prog::test(expr(S::p([3,2,5])), p[5,3,2]):
if prog::TestNoTestargs = FALSE then
prog::test(expr(S::p([0])), TrapError=1202):
end_if:
prog::test(expr(S::p([])), p[]):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("conversions"):

//Basis can be parameted domains as H functions
//We do test on a fixed parameted domain
//declaration of a new base 
S::HSpinBase(3):

for s1 in S::bases do
  for s2 in S::bases do
    prog::test( s1(s2([])),s1::one):
  end_for:
end_for:
// This one characterized a bug in lrcalc!
for x in [S::s([6,5,1])] do
    prog::test(S::s(S::h(x)),x):
end_for:

// Conversion from expression
prog::test(S(p[1]), S::p([1])):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("_substract"):

// Make sure that _substract is implemented properly, and does the
// appropriate conversions
prog::test(not iszero(S::h([2]) - S::e([2]))):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("scalar"):

for n from 0 to 5 do
    for p1 in combinat::partitions::list(n) do
	for p2 in combinat::partitions::list(n) do
            // The Schur functions form an orthonormal basis
	    prog::test(S::scalar(S::s(p1), S::s(p2)),
		       if p1=p2 then 1 else 0 end);
	    // The powersum functions form an orthogonal basis
	    prog::test(S::scalar(S::p(p1), S::p(p2)),
		       if p1=p2 then
			   combinat::partitions::centralizerSize(p1)
		       else
			   0
		       end);
	end_for;
    end_for;
end_for:

//////////////////////////////////////////////////////////////////////////////
prog::testfunc(S::QP):

el := S::QP([3, 2, 1, 1]):
prog::test(mapcoeffs(S::s(el), subs, t = 0),
	   S::s([3, 2, 1, 1])):
prog::test(mapcoeffs(S::h(el), subs, t = 1),
	   S::h([3, 2, 1, 1])):

// The computations below are really to slow in debug mode ...
oldTypeCheck := Pref::typeCheck(Interactive):

prog::test(S::P(S::s(S::P([2, 1, 1]))),
           S::P([2, 1, 1])):
prog::test(S::P(S::s(S::P([2, 1, 1]) + S::P([2, 1]))),
           S::P([2, 1]) + S::P([2, 1, 1])):
prog::test(mapcoeffs(S::P(t*S::s(S::P([2,1]))+ (t^2+1)*S::p(S::P([2,1]))), expand), 
	   (t^2+t+1)*S::P([2,1])):

// Takes to much time...
//prog::test(expand(S::P([2, 1, 1])([x, y])),
//           0):
//prog::test(expand(S::P([3, 2, 1, 1])([x, y, t])),
//           0):



prog::test(mapcoeffs(S::QP(S::s(S::QP([2, 1, 1]))),expand),
           S::QP([2, 1, 1])):
prog::test(mapcoeffs(S::QP(S::s(S::QP([2, 1, 1]) + S::QP([2, 1]))),expand),
           S::QP([2, 1]) + S::QP([2, 1, 1])):
prog::test(mapcoeffs(S::QP(t*S::s(S::QP([2,1]))+ (t^2+1)*S::p(S::QP([2,1]))),expand), 
	   (t^2+t+1)*S::QP([2,1])):


//// expand on monomial (very big)
//muEC::SYMF::SfEval(expr(SF::p(el)), x+y+z);

/*

Identity between the Hall-Littlewood polynomials of Frdric Jouhet:
*/

delete(x):
n:=3:
k:=2:

formula :=
proc(n: Type::NonNegInt, k: Type::NonNegInt)
    local i, j, l, ksi, res;
begin
    res := _plus(_mult(x.i^(k*(1-ksi[i])/2) /(1-x.i^ksi[i]) $ i=1..n) *
		 _mult((1-t*x.j^ksi[j] * x.l^ksi[l]) / (1-x.j^ksi[j] * x.l^ksi[l])
		       $ l = j+1..n $j=1..n-1)
		 $ ksi in combinat::cartesianProduct([1,-1] $ n));
    simplify(res);
end_proc:

prog::test(normal( (_plus(S::P(lambda) 
			  $ lambda in [(op(combinat::partitions::list
					   (s, MaxPart=k, MaxLength=n)) 
					$ s = 0..n*k)]
			 ))([x.i$i=1..n]) -
		  formula(n,k)),
	   0):

Pref::typeCheck(oldTypeCheck):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("antipode"):

for p in [[1],[2],[5],[5,4],[3,2,1],[1,1,1,1]] do
    omega   := S::omega  (S::e(p));
    antipode := S::antipode(S::e(p));
    prog::test(omega,                     S::h(p));
    prog::test(antipode, (-1)^_plus(op(p))*S::h(p));
    for base in [S::p, S::s, S::h] do
	prog::test(testtype((x := base(S::e(p))),base));
	prog::test(S::h(S::omega  (x)), omega  );
	prog::test(S::h(S::antipode(x)), antipode);
    end_for;
end_for:

//////////////////////////////////////////////////////////////////////////////
prog::testfunc(S::m::fromPoly):
//////////////////////////////////////////////////////////////////////////////
for p in [S::m([]), S::m([1]), S::m([1])^4, S::m([4,3,2,1])] do
    prog::test(S::fromPoly(poly(p([x1, x2, x3, x4]), [x1, x2, x3, x4])),
	       S::m(p));
end_for:
//////////////////////////////////////////////////////////////////////////////
// Global checking of the library

prog::testfunc(S):

prog::check(S, 3):

prog::testexit():

// Local Variables: 
// mode: mupad
// End:
