/*****************************************************************************
  $Id: AlgTools.mu,v 1.1.1.3 2005/01/26 13:42:23 nthiery Exp $
  File:         experimental/AlgTools.mu
  Authors:      Florent Hivert <florent.hivert@univ-mlv.fr>
  Created:      2004-06-12
  Purpose:      Research toolbox in representations theory of 
                finite dimensional algebras.
  License:      LGPL
*****************************************************************************/


domain experimental::algTools
    
    info_str :="Research toolbox in representation of finite dimensional algebras";

    interface := {hold(RRBasis), hold(solve2RRBasis),
		  hold(traceABt),
		  
		  hold(checkAssociativeAlgebra),

		  hold(radicalAlgebraRRBasis), 
		  hold(centerAlgebraRRBasis),
		  
		  hold(leftIdealGenerated), 
		  hold(decomposeSemiSimpleCommutativeModule),
		  hold(CenterOfAlgebra),
		  hold(SemiSimpleOfAlgebra),
		  hold(Desosse)};

/*************************************************************
                  Free-module Basic Tools
*************************************************************/

/* RRBasis a basis of a sub-free-module 
 ****
 A row reduced basis is a List of vectors lVect,
 where coeff(lVect[i], lcoeff(lVect([j]))) = 1 if i=j an 0 else.
 RRBasis compute the reduced basis of the submodule spanned by basList
 in the freeModul modul. This is donne essentially by linalg::gaussJordan
*************************************************************************/
    RRBasis := 
    proc(basList : DOM_LIST)
	local modul, res, rank, i, charCols, symMat;
    begin
	modul := (basList[1])::dom;
	symMat := mat -> extsubsop(mat, 3=revert(extop(mat, 3)));
	res   := symMat(linalg::transpose(_concat(op(map(basList, 
							 modul::sparseVector)))));
	[res, rank, i, charCols] := linalg::gaussJordan(res, All); // i is not used
	if rank = 0 then return([]); end_if; // linalg::row( ..., 1..0) is illegal;
	res := linalg::transpose(symMat(res));
	res := linalg::col(res, 1..rank);
	map(res, modul::fromSparseVector);
    end_proc;


/* Check a RRBasis a basis of a sub-free-module 
 ****
 Check that a list is a RRBasis.
*************************************************************************/
    checkRRBasis := 
    proc(basl : DOM_LIST)
	local domm, bas, basEl, lMult;
    begin
	if not testargs() then return(); end_if;
	domm  := (basl[1])::dom;
	bas   := map(basl, domm::lsupport);
	lMult := [map(basl, coeff, domm::lsupport(basEl)) $ basEl in basl];
	lMult := [nops(select(map(basl, coeff, domm::lsupport(basEl)), not iszero)) 
		  $ basEl in basl];
	if nops(bas) <> nops({op(bas)}) or lMult <> [1 $ nops(basl)] then
	    print(basl);
	    print(map(basl, extop));
	    error("Not in row Echelon form ");
	end_if;
    end_proc;

/* tranform the result of a 'solve' to a row reduced basis
 ****
 sol     : a set of linear equations indt = ...
 alg     : the module module
 indts   : the list of the indeterminates of the solve
 lBasInd : the list of the basisIndices of the free module
 ****
 compute a row reduced basis of the space spanned by the generic element 
    gen := _plus(indts[i]*modul::term(lBasInd[i]))
 where the indeterminate indts are given by sol. Such a system of equation 
 is typically obtained by a solve({coeff(...)}, indts)
**************************************************************************/
    solve2RRBasis :=
    proc(sol, modul, indts, lBasInd)
	local params, par0, par;
    begin
	params  := subs(indts, op(par,1) = null() $ par in sol);
	par0    := map(params, _equal, 0);
	sol     := [modul::fromList(zip(subs(indts, sol, par=1, par0) ,
					lBasInd,
					DOM_LIST)) $ par in params];
	sol;
    end_proc;


/*************************************************************
                       Advanced Tools
*************************************************************/

/* Check for the associativity of the product.
 *********************************************/
    checkAssociativeAlgebra := 
    proc(alg : DOM_DOMAIN)
	local a,b,c, aErr, bErr, cErr, res, lb;
//    global aErr, bErr, cErr;
    begin
	lb := alg::basis::list();
	lb := revert(lb);
	print(NoNL, "Checking the associative algebra ".
	      expr2text(alg)." of dimension ".expr2text(alg::dimension));
	((if not iszero(alg::one*c-c) or
	     not iszero(c*alg::one-c) then print(c); 
	      cErr := c;
	      error("No Unit"); 
	  else print(NoNL, "."); null(); 
	  end_if)) $ c in lb;
	(print (c); (res := (a*b)*c-a*(b*c);
		     res := mapcoeffs(res, simplify);
		     if not iszero(res) then print(a,b,c, res); 
			 aErr := a; bErr := b; cErr := c;
			 error("Non associative product"); 
		     else print(NoNL, "."); null(); 
		     end_if)
	 $ a in lb $ b in lb ) $ c in lb;
	print(NoNL, "\nOk !!!!\n");
    end_proc;


/* fast computation of the trace of A*B
 ****
 should perhaps go in linalg / needed by radicalAlgebraRRBasis.
***************************************************************/
    traceAB := 
    proc(a, b)
	local i;
    begin
	_plus((linalg::row(a, i) * linalg::col(b, i))[1,1]
	      $ i=1..linalg::nrows(a));
    end_proc;
    traceABt := 
    proc(a, b)
	local i, res, cola, colb, mon;
    begin
 	res := 0;
 	a := extop(a, 3);
	b := extop(b, 3);
 	for i from 1 to nops(a) do
 	    cola := poly2list(a[i]);
 	    colb := b[i];
 	    res := _plus(res, mon[1]*coeff(colb, mon[2]) $ mon in cola);
 	end_for;
 	res;
    end_proc;


/* radical of a finite dimensional algebra
 ****
 compute a row-reduced basis of the radical of alg.
 ****
 Algorithm: Dickson's methods
    f is in rad(alg) <=> trace(x) = 0 for all x in LeftIdeal(f)
 We compute once for all the matrices of the element of a basis 
 of alg. Build the matrice of a generic element and solve the linear
 system given by the traces. And finally put the solution is a row
 reduced basis form. 
********************************************************************/
    radicalAlgebraRRBasisExpr := // This version only works when
			       // alg::coeffRing = Dom::ExpressionField().
    proc(alg : DOM_DOMAIN)
	local lBasInd, lBasMat, 
          genMat, // transpose of the generic matrice of an element of alg.
          i, dim, sys, res, 
          indts;
	  save co;
    begin
	delete(co);
	dim     := alg::dimension;
	lBasInd := alg::basisIndices::list();
	userinfo(NoNL, 1, "Computing the matrices (dim = ".expr2text(dim).") ");
	lBasMat := [(userinfo(NoNL, 3, ".");
	             alg::leftMatrix(alg::term(i))) $ i in lBasInd];
	genMat  := linalg::transpose // transpose once for all because its faster to
		 (_plus(co.i*lBasMat[i] $ i=1..dim)); // compute trace(a tranpose(b)).
	userinfo(NoNL, 1, "\nComputing the traces ");
	sys     := {(userinfo(NoNL, 3, ".");
	             dom::traceABt(lBasMat[i], genMat)) $ i=1..dim};
	userinfo(NoNL, 1, "\nSolving..."):
	indts   := [co.i $ i=1..dim];
        res     := numeric::linsolve(sys, indts, Symbolic);
	userinfo(NoNL, 1, "Ok (nEqu=".expr2text(nops(res)).", dim=".expr2text(dim-nops(res)).")\n");
	dom::solve2RRBasis(res, alg, indts, lBasInd);
    end_proc;

    radicalAlgebraRRBasis := 
    proc(alg : DOM_DOMAIN)
	local lBasInd, lBasMat, 
          tMat, i, j, dim, sys, res;
    begin
	dim     := alg::dimension;
	lBasInd := alg::basisIndices::list();
	userinfo(NoNL, 1, "Computing the matrices (dim = ".expr2text(dim).") ");
	lBasMat := [(userinfo(NoNL, 3, ".");
	             alg::leftMatrix(alg::term(i))) $ i in lBasInd];
	userinfo(NoNL, 1, "\nComputing the traces ");
	sys     := [(userinfo(NoNL, 3, ".");
		     tMat := linalg::transpose(lBasMat[j]);
		     [dom::traceABt(lBasMat[i], tMat) $i=1..dim])
		    $ j=1..dim];
	sys     := (Dom::Matrix(alg::coeffRing))(sys);
	userinfo(NoNL, 1, "\nSolving..."):
        res     := linalg::nullspace(sys);
	userinfo(NoNL, 1, "Ok (nEqu=".expr2text(nops(res)).", dim=".expr2text(dim-nops(res)).")\n");
	map(res, alg::fromSparseVector);
    end_proc;

/* center of a finite dimensional algebra
 ****
 compute a row-reduced basis of the center of alg.
 if the algebra is commutative return FAIL
 If no algebraic generating system of alg is known check
 the commutation against the full linear basis.
************************************************************************/
    centerAlgebraRRBasisExpr := // This version only works when
			       // alg::coeffRing = Dom::ExpressionField().
    proc(alg: DOM_DOMAIN, testCommutative = FALSE : DOM_BOOL)
	local dim, lBasInd, gener, sol, indts, b, i, sys, gens;
	save co;
    begin
	delete(co);
	dim     := alg::dimension;
	lBasInd := alg::basisIndices::list();
	gener   := alg::fromList([[co.i, lBasInd[i]] $ i=1..dim]);
	indts   := [co.i $ i=1..dim];
	gens := alg::algebraGenerators();
	if gens = UNKNOWN then
	    userinfo(NoNL, 3, "(using the full basis)");
	    gens := alg::basis::list();
	else
	    userinfo(NoNL, 3, "(using ".nops(gens)." algebraic generators)");
	end_if;
	sys     := {(userinfo(NoNL, 1, "."); 
		     coeff(alg::plus(alg::mult2(gener,b),
				     alg::_negate(alg::mult2(b,gener))))
		    ) $ b in gens};
	if sys = {} and testCommutative then return(hold(IsCommutative)) end_if;
	sol     := linsolve(sys , indts);
	dom::solve2RRBasis(sol, alg, indts, lBasInd);
    end_proc;

    centerAlgebraRRBasis :=
    proc(alg: DOM_DOMAIN, testCommutative = FALSE : DOM_BOOL)
	local dim, lBas, gens, b, sys, res, sparseCom;
    begin
	sparseCom := proc(g,b) begin alg::sparseVector(b*g-g*b); end;
	dim     := alg::dimension;
	lBas := alg::basis::list();
	gens := alg::algebraGenerators();
	if gens = UNKNOWN then
	    userinfo(NoNL, 3, "(using the full basis)");
	    gens := alg::basis::list();
	else
	    userinfo(NoNL, 3, "(using ".nops(gens)." algebraic generators)");
	end_if;
	sys := _concat((userinfo(NoNL, 1, ".");
			linalg::stackMatrix(op(  map(gens, sparseCom, b)  ))
		       ) $ b in lBas);
	userinfo(NoNL, 1, "\nSolving..."):
        res     := linalg::nullspace(sys);
	userinfo(NoNL, 1, "Ok (nEqu=".expr2text(nops(res)).", dim=".expr2text(dim-nops(res)).")\n");
	map(res, alg::fromSparseVector);
    end_proc;

    
/* radical of a Left-A module M
 ****
 compute a row-reduced basis of the radical of M.
 ****
 Algorithm: Rad(M) = Rad(A)*M
 Need the radical of A.  
********************************************************************/
    radicalLeftAModuleRRBasis := 
    proc(modul : DOM_DOMAIN)
	local alg, algRad, b, bA;
    begin
	alg     := modul::A;
	algRad  := A::radical();
	RRBasis([modul::mult2(bA, b) $ b in modul::basis::list()
					    $ bA in alg::basis::list()]);
    end_proc;

    
/* left ideal generated in an algebra
 ****
 compute a row-reduced basis of the left ideal generated by some elements
 ************************************************************************/
    leftIdealGenerated := 
    proc(g : Type::AnyType)
	local gen, alg, v;
    begin
	alg := g::dom;
	dom::RRBasis([alg::mult2(v, gen) $ gen in [args()] 
						 $ v in alg::basis::list()]);
    end_proc;
    rightIdealGenerated := 
    proc(g : Type::AnyType)
	local gen, alg, v;
    begin
	alg := g::dom;
	dom::RRBasis([alg::mult2(gen, v) $ gen in [args()] 
						 $ v in alg::basis::list()]);
    end_proc;
    idealGenerated := dom::leftIdealGenerated@op@rightIdealGenerated;

    
/* decomposition of a semisimple module over a finite-dim commutative algebra
 ****
 Decompose a semisimple module over a commutative algebra
 into simple modules of dimension 1.
 ****
 Algorithm: diagonalize consecutively the element of a generating system 
 of the algebra. 
************************************************************************/
    decomposeSemiSimpleCommutativeModule :=
    proc(modul             : DOM_DOMAIN,
	 listGen  = null() : Type::Union(DOM_LIST, DOM_NULL),
	 topLevel = TRUE)
	local aMat, res, iGen, curGen;
    begin
	if listGen = null() then
	    listGen := modul::algebraGenerators();
	    if domtype(listGen) = DOM_LIST then
		userinfo(NoNL, 3, "(using ".nops(listGen)." algebraic generators) ");
	    else
		userinfo(NoNL, 3, "(using the full basis) ");
		listGen := subs(modul::basis::list(), modul::one = null());
	    end_if;
	end_if;
	userinfo(NoNL, 3, expr2text(modul::dimension)." ");
	if modul::dimension = 1 then
	    if topLevel then 
		return(modul::basis::list());
	    else
		return(map(modul::basis::list(), modul::toSupModule));
	    end_if;
	end_if;
	
	if listGen = [] then
	    error("Unable to fully decompose ".
		  expr2text(modul, modul::dimension).
		  "\nPlease check that\n". 
		  " - the algebra is commutative\n".
		  " - the module is semi-simple\n".
		  " - the generators actually generate the algebra.");
	end_if;
	
	iGen   := random(1..nops(listGen))(); // choosing a generator at random.
//	iGen   := 1;
	curGen := listGen[iGen];
	delete(listGen[iGen]);
	
	aMat := modul::leftMatrix(curGen);
	res  := linalg::eigenvectors(aMat);
	if nops(res) = 1 then // bad luck aMat is a scalar matrix
	    userinfo(NoNL, 3, ".");
	    return(dom::decomposeSemiSimpleCommutativeModule
		       (modul, listGen, topLevel));
	end_if;
	res  := map(res, op, 3);
	res  := map(res, map, modul::fromSparseVector);
	res  := map(res, z -> experimental::SubLeftAModule(curGen::dom, z));
	res  := map(res,
		    op@dom::decomposeSemiSimpleCommutativeModule,
		    listGen, FALSE);
	if topLevel then 
	    res
	else
	    map(res, modul::toSupModule);
	end_if;
    end_proc;
    

/* minimal idempotent
 ****
 compute a minimal idempotent of a simple algebra
 ****
 Algorithm:
 - take a random element a of A, compute its characteristic polynomial P(X)
 and try to factor it first by a square-free factorization and if it fails by
 solving the polynomial using alegbraic extensions.
 - then choose the minimal degree of the factors and using Bezout algorithm
 compute the idempotent R(X) associated to this factor in K[X]/P(X). We know
 have an idempotent R(a). 
 - If if the image of R(a) in A is one dimensional we are done. If not 
 repeat the process with R(a).A.R(a) until the image of the idempotent is
 one dimensional.
*****************************************************************************/
    minimalIdempotentOfSimpleAlgebra :=
    proc(alg               : DOM_DOMAIN,
	 listGen  = null() : Type::Union(DOM_LIST, DOM_NULL))
	local aMat, res, iGen, curGen, charPol, facto,
	      eiv, i, degr, chosen, pol1, pol2, bezout;
    begin
	userinfo(NoNL, 3, "dim=".expr2text(alg::dimension)."\n");
	if alg::dimension = 1 then
	    userinfo(NoNL, 3, "Minimal idempotent done\n");
	    return(alg::one);
	end_if;
	userinfo(NoNL, 3, "Looking for an eigenprojector ");
	if listGen = null() then
	    listGen := alg::algebraGenerators();
	    if domtype(listGen) = DOM_LIST then
		userinfo(NoNL, 3, "(using ".nops(listGen)." algebraic generators)...\n");
	    else
		userinfo(NoNL, 3, "(using the full basis)...\n");
		listGen := subs(alg::basis::list(), alg::one = null());
	    end_if;
	end_if;
	repeat // look for a non trivial factorization of an element of alg
	    iGen   := random(1..nops(listGen))(); // choosing a generator at random.
//	    iGen   := 1;
	    curGen := listGen[iGen];
	    delete(listGen[iGen]);
	
	    userinfo(NoNL, 3, "matrix...");
	    aMat    := alg::leftMatrix(curGen);
	    charPol := linalg::charpoly(aMat, X);
	    userinfo(NoNL, 3, "charpoly...");
	    assert(iszero(subs(expr(charPol), X=curGen)));
	    userinfo(NoNL, 3, "done. Trying SqFree factor...");
	    facto   := polylib::sqrfree(charPol);
	    if nops(facto) = 3 then // no factorization: use solve instead
		userinfo(NoNL, 3, "no factorization ! Using Solve ");
		eiv   := solve(charPol, X, Multiple,  MaxDegree = 4);
		eiv   := map([op(eiv)], z-> ((X-z[1]), z[2]));
		facto := Factored([1,op(eiv)]);
	    end_if;
	    if nops(facto) = 3 then
		userinfo(NoNL, 3, " Bad luck ! Need another element \n");
	    end_if;
	until nops(facto) > 3 end_repeat;
	userinfo(NoNL, 3, " Ok...\nFactorization: ".expr2text(facto)."\n");
	facto := [op(facto)]; delete(facto[1]);

        // find the factor of minimal degree
	degr := infinity;
	for i from 1 to nops(facto) step 2 do 
	    if facto[i+1] < degr then
		degr   := facto[i+1];
		chosen := facto[i];
	    end_if;
	end_for;
	pol1 := (chosen)^degr;
	pol2 := divide(charPol, pol1, Quo);
	bezout := [gcdex(expr(pol1), expr(pol2), X)];
	assert(bezout[1] = 1);
	userinfo(NoNL, 3, "Computing the corresponding algebraic element\n");
	res := mapcoeffs(subs(expr(pol2*bezout[3]), X=curGen), simplify);
	assert(iszero(res^2-res));
	userinfo(NoNL, 3, " Ok...\n");
        res := dom::minimalIdempotentOfSimpleAlgebra(dom::BlockAlgebra(res));
	res::dom::toSupModule(res);
    end_proc;
    

/* Lift a idempotent from the quotient by the radical
 ****
 Hypothese: idemp in an element of a algebra of minimal polynomial 
        mimpoly(idemp)  =  (X-1)^a * X^b
 Compute in the algebra an idempotent whose image is the characteristic 
 space of idemp associated to the eigenvalue 1. 
 ****
 Algorithm: We dont need to know a and b: the min poly of 
              (1-(1-idemp^2)^2) 
 is 
              (X-1)^(ceil(a/2)) * X^(ceil(b/2))
 So we iterate the formula until the sequence stabilise (sometimes, 
 knowing the dimension of alg allows to stop earlier but by this way, we
 ensure that the result is actually an idempotent). 
**************************************************************************/
    liftIdempotent := 
    proc(idemp)
	local idempOld;
    begin
	idempOld := NIL;
	while(idemp <> idempOld) do
	    userinfo(NoNL, 3, ".");
	    idempOld := idemp;
	    idemp    := (1-(1-idemp^2)^2);
	end_while;
	userinfo(NoNL, 3, "Ok ");
	idemp;
    end_proc;

/* Normalise and check a pseudo idempotent
******************************************/
    normalisePseudoIdempotent := 
    proc(idemp)
	local const, supp, bas, idemp2;
    begin
	idemp2 := idemp^2;
	supp := {op(polylib::support(idemp))};
	const := { (coeff(idemp2, bas) / coeff(idemp, bas)) $ bas in supp};
	if {op(polylib::support(idemp2))} <> supp or nops(const) <> 1 then
	    print(idemp, idemp^2);
	    error("Unable to normalise non pseudo-idempotent");
	end_if;
	multcoeffs(idemp, 1/const[1]);
    end_proc;


/* Image set of an idempotent in a module
 ****
   Given a idempotent idemp of A, compute the image of idemp in a module
************************************************************************/
    idempotentImageSet := 
    proc(modul : DOM_DOMAIN, idemp)
	local g, v;
    begin
	dom::RRBasis([modul::mult2(idemp, v) $ v in modul::basis::list()]);
    end_proc;



/* compute map(line, fun) and print with userinfo the progress of
   nops(fun(line[i]))/iDiv[i])
************************************************************************/
    mapDimInfo :=
    proc(line: DOM_LIST, funct:Type::AnyType, iDiv)
	local iLine, iRes, iNops, res, resNops;
    begin
	if args(0) = 2 then iDiv := [1$nops(line)]; end_if;
	userinfo(NoNL, 1, "[");
	iRes  := funct(line[1]);
	iNops := nops(iRes)/iDiv[1];
	userinfo(NoNL, 1, expr2text(iNops));
	res := [iRes]; resNops := [iNops]; 
	for iLine from 2 to nops(line) do
	    iRes  := funct(line[iLine]);
	    iNops := nops(iRes)/iDiv[iLine];
	    userinfo(NoNL, 1, ", ".expr2text(iNops));
	    res     := append(res, iRes);
	    resNops := append(resNops, iNops);
	end_for;
	userinfo(NoNL, 1, "]");
	[res, resNops];
    end_proc;
		      
/************************************************************************/
/************************************************************************/
/************************************************************************/
    Desosse := 
    proc(alg: DOM_DOMAIN)
     local semiSimple, isSemiSimple, 
	   center, isSemiSimpleCommutative,
	   idempCenter,
	   rankCentralIdemp, dimSimples, 
	   blockIdemps,
	   projs, dimProjs,
	   CartanMatrix, CartanLine,
	   i, timeBefore;
    begin
	userinfo(NoNL, 1, "Representation theory of ".expr2text(alg).
		 " of dimension ".expr2text(alg::dimension)."\n");
	timeBefore := time();
	alg::repTheo := table();
	alg::interface := alg::interface union {hold(repTheo)};
	userinfo(NoNL, 1, "Preliminary temporary checks..."):
	if alg::hasProp(Dom::FreeModulePoly) 
	   and alg::getRankerParameters()[1] = 1 
	   and map(alg::basis::list(), ldegree@extop) <> [$1..alg::dimension] then
	    userinfo(NoNL, 1, "\n"):
	    error("The order of the basis is not coherent with the ranker\n".
		  "Desosse does not work. Try option RankerBasis...");
	end_if;
	userinfo(NoNL, 1, "done\n"):
	
	
	if args(0) <> 2 or args(2) <> hold(AssumeSemiSimple) then
	    semiSimple := dom::SemiSimpleOfAlgebra(alg);    
	    isSemiSimple := bool(semiSimple = alg);
	    if isSemiSimple then
		userinfo(NoNL, 1, "Semi-simple algebra !\n");
	    else
		userinfo(NoNL, 1, "Dimension of the semi-simple quotient: ".
		      expr2text(semiSimple::dimension)."\n"):
	    end_if;
	else 
	    isSemiSimple := TRUE;
	    userinfo(NoNL, 1, "Assuming semi-simple algebra !\n");
	    semiSimple := alg;
	end_if;
	
        alg::repTheo[hold(isSemiSimpleAlgebra)] := isSemiSimple;
        alg::repTheo[hold(semiSimpleQuotient)]  := semiSimple;

	userinfo(NoNL, 1, "Computing the semi-simple center..."):
	center := dom::CenterOfAlgebra(semiSimple);
	userinfo(NoNL, 1, "done\n");
	
	isSemiSimpleCommutative := bool(center=semiSimple);
	if isSemiSimpleCommutative then
	    userinfo(NoNL, 1, "Commutative semi-simple quotient\n");
	else
	    userinfo(NoNL, 1, "Dimension of the center: ".
		  expr2text(center::dimension)."\n"):
	end_if;
	
	alg::repTheo[hold(isSemiSimpleQuotientCommutative)] := isSemiSimpleCommutative;
	alg::repTheo[hold(centerOfSemiSimpleQuotient)] := center;
	
	userinfo(NoNL, 1, "Decomposing the semi-simple center..."):
	idempCenter := dom::decomposeSemiSimpleCommutativeModule(center);
	userinfo(NoNL, 1, "done\n"):

	userinfo(NoNL, 1, "Normalizing into idempotents (and checking) "):
	idempCenter := [(userinfo(NoNL, 3, "."); 
			 dom::normalisePseudoIdempotent(i)) $ i in idempCenter];

	alg::repTheo[hold(semiSimpleCentralIdempotents)] := idempCenter;

	if not isSemiSimpleCommutative then
	    idempCenter := map(idempCenter, center::toSupModule);
	end_if;
	userinfo(NoNL, 1, "done\n"):

	userinfo(NoNL, 1, "Dimensions of simples:\n ");
        rankCentralIdemp := map(idempCenter, nops@dom::leftIdealGenerated);
        dimSimples       := map(rankCentralIdemp, isqrt);
        if map(dimSimples, _power, 2) <> rankCentralIdemp then
            print(idempCenter, map(idempCenter, domtype));
            error("Rank of central indempotents must be square".expr2text(rankCentralIdemp));
        end_if;
        userinfo(NoNL, 1, expr2text(dimSimples)."\n");
	
	if not isSemiSimple then
	    userinfo(NoNL, 1, "Lifting idempotents"):
	    blockIdemps := map(idempCenter, dom::liftIdempotent@(idmp -> idmp::dom::toSupModule(idmp)));
	    userinfo(NoNL, 1, "done\n"):
	
	    userinfo(NoNL, 1, "Dimensions of projectives:\n ");
	    [projs, dimProjs] := dom::mapDimInfo(blockIdemps, dom::leftIdealGenerated, dimSimples);
	    userinfo(NoNL, 1, "\n");
	    if _plus(op(zip(dimProjs, dimSimples, _mult))) <> alg::dimension then
		error("Sum of dimension of minimal ideals is not the dimensiom of the algebra !!!");
	    end_if;

	    userinfo(NoNL, 1, "Cartan Matrix:\n ");
	    projs     := map(projs, z -> experimental::SubLeftAModule(alg, z));
	    CartanMatrix := [];

	    (CartanLine := dom::mapDimInfo(blockIdemps,
					   z -> dom::idempotentImageSet(projs[i], z),
					   map(dimSimples, _mult, dimSimples[i])); 
	     CartanMatrix := append(CartanMatrix, CartanLine[2]);
	     userinfo(NoNL, 1, ", ".expr2text(dimSimples[i])."\n ")
	    ) $ i = 1..nops(blockIdemps);
	    
	    alg::repTheo[hold(blockIdempotents)] := blockIdemps;
	    alg::repTheo[hold(blockProjectivesModules)] := projs;
	    alg::repTheo[hold(cartanInvariantsMatrix)] := matrix(CartanMatrix);

	end_if;
	userinfo(NoNL, 1, "\nDesosse finished. Elapsed time: ".expr2text(time()-timeBefore)." ms\n");
	alg::repTheo;
    end_proc;

end_domain:


/* Center of a finite dimensional algebra
************************************************************************/
domain experimental::algTools::CenterOfAlgebra(alg)
    local RRBasis;
    category Cat::FiniteDimensionalAlgebraWithBasis(alg::coeffRing);
    inherits experimental::SubAlgebra(RRBasis);
begin
    RRBasis := alg;
    alg := (RRBasis[1])::dom;
end_domain:
experimental::algTools::CenterOfAlgebraSave := experimental::algTools::CenterOfAlgebra;
experimental::algTools::CenterOfAlgebra :=
proc(alg: DOM_DOMAIN, testCommutative = TRUE : DOM_BOOL)
    option remember;
    local cRRBas;
begin
    cRRBas := dom::centerAlgebraRRBasis(alg, testCommutative);
    if testCommutative and cRRBas = hold(IsCommutative) then
	return(alg);
    end_if;
    experimental::algTools::CenterOfAlgebraSave(cRRBas);
end_proc:


/* SemiSimple of a finite dimensional algebra
************************************************************************/
domain experimental::algTools::SemiSimpleOfAlgebra(alg)
    local RRBasis;
    category Cat::FiniteDimensionalAlgebraWithBasis(alg::coeffRing);
    inherits experimental::QuotientAlgebra(RRBasis);
begin
    RRBasis := alg;
    alg := (RRBasis[1])::dom;
end_domain:
experimental::algTools::SemiSimpleOfAlgebraSave := experimental::algTools::SemiSimpleOfAlgebra;
experimental::algTools::SemiSimpleOfAlgebra :=
proc(alg: DOM_DOMAIN)
    option remember;
    local cRRBas;
begin
    userinfo(NoNL, 1, "Computing the radical...\n"):
    cRRBas := dom::radicalAlgebraRRBasis(alg);
    userinfo(NoNL, 1, "radical done\n"):
    if cRRBas = [] then
	alg;
    else
	experimental::algTools::SemiSimpleOfAlgebraSave(cRRBas);
    end_if;
end_proc:


/* Block algebra of an idempotent
************************************************************************/
// The idempotent is passed as the first argument
domain experimental::algTools::BlockAlgebra(idemp)
    local A, basisOfB;
    inherits experimental::SubAlgebra(basisOfB);
    category Cat::FiniteDimensionalAlgebraWithBasis(A::coeffRing);

    one := dom::fromSupModule(idemp);

begin
    basisOfB := idemp;
    idemp    := basisOfB[1];
    A        := idemp::dom;
    delete(basisOfB[1]);
end_domain:
experimental::algTools::BlockAlgebraSave := experimental::algTools::BlockAlgebra;
experimental::algTools::BlockAlgebra :=
proc(idemp)
    option remember;
    local idRRBas, A, x;
begin
    A := idemp::dom;
    userinfo(NoNL, 1, "Computing idempotent block (dim=".expr2text(A::dimension).")..."):
    idRRBas := [(userinfo(NoNL, 1, ".");
		 idemp*x*idemp)  $ x in A::basis::list()];
    idRRBas := experimental::algTools::RRBasis(idRRBas);
    userinfo(NoNL, 1, "done\n"):
    experimental::algTools::BlockAlgebraSave([idemp].idRRBas);
end_proc:


/* SemiSimple of a Left A-Module
************************************************************************/
domain experimental::algTools::semiSimpleOfLeftAModule(modul)
    local RRBasis;
    category Cat::LeftAModuleWithBasis(modul::A::coeffRing);
    inherits experimental::QuotientLeftAModule(RRBasis);
begin
    RRBasis := modul;
    modul := (RRBasis[1])::dom;
end_domain:
experimental::algTools::semiSimpleOfLeftAModuleSave := experimental::algTools::semiSimpleOfLeftAModule;
experimental::algTools::semiSimpleOfLeftAModule :=
proc(modul: DOM_DOMAIN)
    option remember;
    local cRRBas;
begin
    userinfo(NoNL, 1, "Computing the radical...\n"):
    cRRBas := dom::radicalLeftAModuleRRBasis(modul);
    userinfo(NoNL, 1, "radical done\n"):
    if cRRBas = [] then
	modul;
    else
	experimental::algTools::semiSimpleOfLeftAModuleSave(cRRBas);
    end_if;
end_proc:
