/*****************************************************************************
  $Id: ArikiKoike.mu,v 1.1.1.2 2005/01/26 13:42:23 nthiery Exp $
  File:         lib/EXPERIMENTAL/ArikiKoike/ArikiKoike.mu
  Authors:      Florent Hivert <florent.hivert@univ-mlv.fr>
  Created:      2004-06-11
  License:      LGPL
*****************************************************************************/

alias(declareConversion = operators::overloaded::declareConversion);
alias(Perm = combinat::permutations):
alias(AK = experimental::ArikiKoikeAlgebra);
alias(AKTools = experimental::ArikiKoikeAlgebraTools);
alias(HeckeAlg = examples::HeckeAlgebra);

domain AKTools
    info_str := "Helper tools for the domain experimental::ArikiKoikeAlgebra";
end_domain:
    
domain AKTools::Polynomials(N    : Type::PosInt,
			    Cols : DOM_LIST,
			    R    : DOM_DOMAIN)
    local nCols;
    category Cat::FiniteDimensionalAlgebraWithBasis(R);
    inherits Dom::TensorProductOfFreeModules([experimental::FiniteQuotientOfPolynomials(x.i, Cols, R)
					      $ i=1..N]);
    
    info_str := "Quotient of the polynomials in [x1..xN] by the relations Prod_j (xi-Cols[j]) = 0";

    type1N     := Type::Interval([1, N], Type::Integer);
    type1N1    := Type::Interval([1, N-1], Type::Integer);
    type1nCols := Type::Interval([1, nCols], Type::Integer);

    vars      := [x.i $ i=1..N];
    print      := proc(z: dom) begin extop(subs(extop(z), _X.i=x.i $ i=1..N), 1); end_proc;
    expr       := dom::print;
    xi         := proc(i: dom::type1N) begin dom::term(subsop([0$N], i=1)); end_proc;

    VandermondeDet       := _mult((Cols[i]-Cols[j]) $ j=1..i-1 $ i=1..nCols);
    VandermondeMatrix    := linalg::vandermonde(Cols);
    VandermondeMatrixInv := linalg::invvandermonde(Cols);
    BMatrix := multcoeffs(dom::VandermondeMatrixInv, dom::VandermondeDet);

    FkPol   := // Polynomial F_k(xi) (cf Shoji)
    proc(k:  dom::type1nCols, 
	 xi: dom::type1N): dom
        local i;
    begin
	xi := dom::xi(xi);
	dom::plus(dom::multcoeffs(dom::_power(xi, i), dom::BMatrix[i+1, k]) $ i=0..nCols-1);
    end_proc;

    Pj      := 
    proc(j: dom::type1N): dom
	option remember;
        local c1, c2;
    begin
	multcoeffs(_plus(multcoeffs(dom::FkPol(c1, j)*dom::FkPol(c2, j+1), 
				    (Cols[c2]-Cols[c1]))
			 $ c1 = 1..c2 $ c2 = 2..nCols),
		   dom::VandermondeDet^(-2));
    end_proc;

    Lagrange := 
    proc(k:  dom::type1nCols,
	 xi: dom::type1N): dom
        local j;
    begin
	xi := dom::xi(xi);
	dom::mult(((xi - Cols[j])/(Cols[k] - Cols[j])) $ j = 1..k-1,
		  ((xi - Cols[j])/(Cols[k] - Cols[j])) $ j = k+1..nCols);
    end_proc;
	     
    LagrangeComp :=
    proc(lcol: Type::ListProduct(dom::type1nCols $ N))
	local icol;
    begin
	dom::mapcoeffs(dom::mult(dom::Lagrange(lcol[icol], icol) $ icol=1..N),
		       simplify);
    end_proc;
	     
    splitPol := 
    proc(j: dom::type1N1)
	option remember;
	local i, before, after, doms;
    begin
	doms := [];
	if j > 1 then
	    doms := [Dom::TensorProductOfFreeModules([experimental::FiniteQuotientOfPolynomials(x.i, Cols)
						     $ i=1..j-1])];
	end_if;
	doms := append(doms, 
		       experimental::FiniteQuotientOfPolynomials(x.j, Cols),
		       experimental::FiniteQuotientOfPolynomials(x.(j+1), Cols));
	if j < N-1 then
	    doms := append(doms, 
			   Dom::TensorProductOfFreeModules([experimental::FiniteQuotientOfPolynomials(x.i, Cols)
							    $ i=j+2..N]));
	end_if;
	[dom::toCartesianProductFromSignature(doms),
	 dom::tensorConstructorFromSignature(doms)];
    end_proc;

    deltaiOnPol :=
    proc(pol: dom, i: dom::type1N1): dom
    begin
	dom::fromList(map(poly2list(extop(pol)),
			  proc(mon)
			      local co, l, a, b, k;
			  begin
			      [co, l] := mon;
			      a := l[i];
			      b := l[i+1];
			      if   a > b then
				  [co,  subsop(l, i=a-k, i+1=b-1+k)] $ k=1..a-b;
			      elif a < b then
				  [-co, subsop(l, i=b-k, i+1=a-1+k)] $ k=1..b-a;
			      else 
				  null();
			      end_if;
			  end_proc));	
    end_proc;

    siOnPol := // rely on the structure of TensorProductOfFreeModules. 
    proc(pol: dom, i: dom::type1N1): dom
    begin
	dom::fromList(map(poly2list(extop(pol)),
			  proc(mon)
			      local co, l;
			  begin
			      [co, l] := mon;
			      [co, subsop(l, i=l[i+1], i+1=l[i])];
			  end_proc));
    end_proc;

    tiOnDom :=
    proc(pol: dom, i: dom::type1N1): dom
    begin
	dom::mult2(dom::deltaiOnPol(pol, i), dom::Pj(i))
    end_proc;
	     
begin
    nCols := nops(Cols);
end_domain:


domain AK(N    : Type::PosInt,
	  Cols : DOM_LIST,
	  q    = 0,
	  R    = Dom::ExpressionField() : DOM_DOMAIN)
    local nCols, Pol, Hecke;
    category Cat::FiniteDimensionalAlgebraWithBasis(R);
    inherits Dom::TensorProductOfFreeModules([Pol, Hecke]);

    info_str := "domain experimental::ArikiKoikeAlgebra";

    type1N     := Type::Interval([1, N], Type::Integer);
    type1N1    := Type::Interval([1, N-1], Type::Integer);
    type1nCols := Type::Interval([1, nCols], Type::Integer);

    pol   := Pol;
    Hecke := Hecke;
    splitPolHecke := dom::toCartesianProductFromSignature([Pol, Hecke]);
    constPolHecke := dom::tensorConstructorFromSignature([Pol, Hecke]);

    printMonomial := z -> hold(_mult)(op(op(dom::splitPolHecke(z))));
	     
// creation d'elements.
	     
    ti :=
    proc(i : dom::type1N1)
    begin
	dom(Hecke::term(subsop([$1..N], i=i+1, i+1=i)));
    end_proc;

    xi :=
    proc(i : dom::type1N)
    begin
	dom(Pol::xi(i));
    end_proc;

    algebraGenerators :=
    proc() local i;
    begin [dom::ti(i) $ i=1..N-1, dom::xi(i) $ i=1..N]; end_proc;
	     
	     
    tiOnDom := 
    proc(vct: dom, i: dom::type1N1): dom
	local pol, hecke;
    begin
	dom::plus(op(map(dom::splitPolHecke(vct),
	    proc(v)
		local pl, hck;
	    begin
		[pl, hck] := v;
		dom::plus(dom::constPolHecke(
		               [Pol::siOnPol(pl, i), 
				Hecke::mult2(Hecke::Ti(i), hck)]),
			  dom::multcoeffs(
			        dom::constPolHecke(
				    [Pol::tiOnDom(pl, i), hck]),
			        -(Hecke::q1+Hecke::q2))
			  );
	    end_proc)));
    end_proc;

    multPolDom := 
    proc(vct: dom, pol: Pol): dom
    begin
	dom::plus(op(map(dom::splitPolHecke(vct), 
			 (z) -> dom::constPolHecke([Pol::mult2(pol, z[1]), z[2]]))));
    end_proc;

    mult2Slow := 
    proc(t1: dom, t2: dom): dom
    begin
	dom::plus(op(map(dom::splitPolHecke(t1),
			 proc(v)
			     local pl, hck, res, i;
			 begin
			     [pl, hck] := v;
			     hck := Hecke::lsupport(hck);
			     res := t2;
			     for i in revert(Perm::reducedWord(hck)) do
				 res := dom::tiOnDom(res, i);
			     end_for;
			     dom::multPolDom(res, pl);
			 end_proc)));
    end_proc;

    mult2Basis := 
    proc(x1,x2): dom
	option remember;
    begin
	dom::mult2Slow(dom::term(x1), dom::term(x2));
    end_proc;

    pol2Dom   := proc(pol: Pol): dom   begin dom::constPolHecke([pol, Hecke::one]); end_proc;
    Hecke2Dom := proc(hck: Hecke): dom begin dom::constPolHecke([Pol::one, hck]);   end_proc;

    initDomainAdd :=
    proc()
    begin
        declareConversion(Pol,   dom, dom::pol2Dom):
        declareConversion(Hecke, dom, dom::Hecke2Dom):
    end_proc;

begin
    nCols    := nops(Cols);
    Pol      := AKTools::Polynomials(N, Cols, R);
    Hecke    := HeckeAlg(N, q, R);
end_domain:


/* Exemple D'utilisation :
   ak := experimental::ArikiKoikeAlgebra(3, [-1,0,1]);
***Algebre de Hecke sous jacente:   
>> ak::Hecke ;
             experimental::HeckeAlgebra(3, Dom::ExpressionField())
***Creation d'elements
>> ak::Hecke::Ti(1);
                                      P213
>> ak::Hecke::Ti(1)+ak::Hecke::Ti(2);;
                                  P213 + P132
>> ak::Hecke([1,3,2]);
                                      P132
*** conversion Hecke -> Ariki
>> ak(ak::Hecke::Ti(1)+ak::Hecke::Ti(2));
                                1 P213 + 1 P132
*** creation directe dans Ariki
>> ak::ti(2);
                                     1 P132
*** idem pour les variables:
>> ak::pol::xi(3);
                                       x3
>> %^4;
                                        2
                                      x3
*** conversion, creation dans Ariki
>> ak(ak::pol::xi(3)), ak::xi(3);
                                x3 P123, x3 P123
*** les polynomes F_k : F_i(x^2)
>> ak::pol::FkPol(1, 2);
                                            2
                                   - x2 + x2
*** et la grosse formule merdique delta^{-2} \sum (...)
*** Je les appelle P_j
>> ak::pol::Pj(2);
                                  2   2             2     2
                      x2   x3   x2  x3    x2 x3   x2    x3
                    - -- + -- - ------- - ----- + --- + ---
                       2    2      2        2      2     2
*** Tu peux mlanger tout a 
>> %*ak::ti(2);
/     2   2 \        /   2 \
|   x2  x3  |        | x2  |        /   x2 x3 \        /   x2 \
| - ------- | P132 + | --- | P132 + | - ----- | P132 + | - -- | P132 +
\      2    /        \  2  /        \     2   /        \    2 /
   /   2 \
   | x3  |        / x3 \
   | --- | P132 + | -- | P132
   \  2  /        \  2 /
*** Pour utiliser le dsosseur:
aller dans le fichier
ligne 3825:
       smallmatrix:= bool(r*c <= 2*10^4) or //heuristics
                     bool(r = 1) or         //tested in
                     bool(c = 1);           //MuPAD 3.0
       if smallmatrix then
            s:= [[[Rzero,0] $ c] $ r];
rajouter la ligne:
       smallmatrix:= bool(r*c <= 2*10^4) or //heuristics
                     bool(r = 1) or         //tested in
                     bool(c = 1);           //MuPAD 3.0
       smallmatrix:=TRUE;		     
       if smallmatrix then
            s:= [[[Rzero,0] $ c] $ r];
ensuite faire:
>> setuserinfo(experimental::algTools, 3);
puis
>> os := experimental::algTools::Desosse(ak);
Representation theory of experimental::ArikiKoikeAlgebra(3, [-1, 0], Dom::Expre\
ssionField()) of dimension 48
Preliminary temporary checks...done
Computing the radical...
Computing the matrices (dim = 48) .............................................\
...
Computing the traces ................................................
Solving...Ok (nEqu=18, dim=30)
radical done
Dimension of the semi-simple quotient: 18
Computing the semi-simple center.....................done
Commutative Semi-simple quotient
Decomposing the semi-simple center...18 9 4 .4 1 3 .3 .3 .3 1 2 .2 .2 .2 1 1 5 \
.5 2 .2 .2 .2 .2 .2 .2 .2 1 1 3 .3 .3 .3 2 .2 .2 .2 1 1 1 9 5 .5 3 .3 .3 .3 1 2\
 .2 .2 .2 1 1 2 .2 .2 .2 .2 .2 .2 .2 1 1 4 .4 1 3 .3 .3 .3 2 .2 .2 .2 1 1 1 don\
e
Normalizing into idempotents (and checking) ..................done
Dimensions of simples:
 [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1]
Lifting idempotents..Ok ...Ok ..Ok ...Ok ..Ok ..Ok ..Ok ..Ok ..Ok ..Ok ..Ok ..O\
k ..Ok ..Ok ..Ok ...Ok ..Ok ...Ok done
Dimensions of projectives:
 [1, 3, 3, 1, 2, 3, 3, 6, 2, 2, 6, 3, 3, 2, 1, 1, 3, 3]
Cartan Matrix:
 [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], 1
 [0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0], 1
 [0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0], 1
 [0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], 1
 [0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0], 1
 [0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0], 1
 [0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0], 1
 [0, 0, 1, 0, 0, 0, 0, 2, 0, 0, 0, 1, 1, 0, 0, 0, 0, 1], 1
 [0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0], 1
 [0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0], 1
 [0, 1, 0, 0, 0, 1, 1, 0, 0, 0, 2, 0, 0, 0, 0, 0, 1, 0], 1
 [0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0], 1
 [0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1], 1
 [0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0], 1
 [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0], 1
 [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0], 1
 [0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0], 1
 [0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1], 1
*** Le resultat est une table que je ne reproduis pas ici qui
contient toutes les info utiles:
>> os[cartanInvariantsMatrix];

           +-                                                      -+
           |  1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0  |
           |                                                        |
           |  0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0  |
           |                                                        |
           |  0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0  |
           |                                                        |
           |  0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0  |
           |                                                        |
           |  0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0  |
           |                                                        |
           |  0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0  |
           |                                                        |
           |  0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0  |
           |                                                        |
           |  0, 0, 1, 0, 0, 0, 0, 2, 0, 0, 0, 1, 1, 0, 0, 0, 0, 1  |
           |                                                        |
           |  0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0  |
           |                                                        |
           |  0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0  |
           |                                                        |
           |  0, 1, 0, 0, 0, 1, 1, 0, 0, 0, 2, 0, 0, 0, 0, 0, 1, 0  |
           |                                                        |
           |  0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0  |
           |                                                        |
           |  0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1  |
           |                                                        |
           |  0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0  |
           |                                                        |
           |  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0  |
           |                                                        |
           |  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0  |
           |                                                        |
           |  0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0  |
           |                                                        |
           |  0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1  |
           +-                                                      -+
 */
