/*****************************************************************************
  $Id: ArikiKoikeLagrange.mu,v 1.1.1.2 2005/01/26 13:42:23 nthiery Exp $
  File:         lib/EXPERIMENTAL/ArikiKoike/ArikiKoike.mu
  Authors:      Florent Hivert <florent.hivert@univ-mlv.fr>
  Created:      2004-06-11
  License:      LGPL
*****************************************************************************/

alias(declareConversion = operators::overloaded::declareConversion);
alias(Perm = combinat::permutations):
alias(AKL = experimental::ArikiKoikeLagrangeAlgebra);
alias(AKTools = experimental::ArikiKoikeAlgebraTools);
alias(HeckeAlg = experimental::HeckeAlgebra);

experimental::ArikiKoikeAlgebra: // Force the loading ok AKTools...

domain AKTools::vctBas(N : Type::NonNegInt, C : Type::NonNegInt)
    category Cat::CombinatorialClass;
    inherits combinat::finiteClass
	     (combinat::cartesianProduct([$0..C-1]
//	      (combinat::cartesianProduct([$1..i+1], [$0..C-1])
	       $ i=0..N-1));
    axiom Ax::systemRep;
    domtype := DOM_LIST;
end_domain:

domain AKTools::Lagrange(N       : Type::PosInt,
			 nColors : Type::PosInt,
			 R       : DOM_DOMAIN)
    category Cat::FiniteDimensionalAlgebraWithBasis(R);
    inherits Dom::FreeModule(Dom::ExpressionField(),
                      AKTools::vctBas(N, nColors),
                      RankerBasis = (AKTools::vctBas(N, nColors))::ranker );
    info_str := "Quotient of the polynomials in [x1..xN]".
	  " by the relations Prod_j (xi-Cols[j]) = 0".
	  ", on the Lagrange interpolator basis";

    type1N     := Type::Interval([1, N], Type::Integer);
    type1N1    := Type::Interval([1, N-1], Type::Integer);
    type1nCols := Type::Interval([1, nColors], Type::Integer);

    printTerm  := proc(l: dom::basisIndices)
		  begin
		      _concat(Lag, op(l));
		  end_proc;

    one := dom::plus(op(dom::basis::list()));	     
	     
    siOnDomBasis := 
    proc(pol: dom::basisIndices, i: dom::type1N1): dom
    begin
	dom::term(subsop(pol, i=pol[i+1], i+1=pol[i]));
    end_proc;
    siOnDom := operators::makeLinear(dom::siOnDomBasis,
				     Source=dom, ImageSet=dom);


    tiOnDomBasis := 
    proc(pol: dom::basisIndices, i: dom::type1N1): dom
    begin
	if   pol[i] = pol[i+1] then dom::zero;
	elif pol[i] < pol[i+1] then -dom::term(pol);
	else
	    dom::term(subsop(pol, i=pol[i+1], i+1=pol[i]));
	end_if;
    end_proc;
    tiOnDom := operators::makeLinear(dom::tiOnDomBasis,
				     Source=dom, ImageSet=dom);

    mult2Basis := 	     
    proc(lag1: dom::basisIndices, lag2: dom::basisIndices)
    begin
	if lag1=lag2 then dom::term(lag1)
	else dom::zero;
	end_if;
    end_proc;
	     
begin
end_domain:


domain AKL(N       : Type::PosInt,
	   nColors : Type::PosInt,
	   R       = Dom::ExpressionField() : DOM_DOMAIN)
    local Pol, Hecke;
    category Cat::FiniteDimensionalAlgebraWithBasis(R);
    inherits Dom::TensorProductOfFreeModules([Pol, Hecke]);

    info_str := "domain experimental::ArikiKoikeAlgebra";

    type1N     := Type::Interval([1, N], Type::Integer);
    type1N1    := Type::Interval([1, N-1], Type::Integer);
    type1nCols := Type::Interval([1, nColors], Type::Integer);

    pol   := Pol;
    Hecke := Hecke;
    splitPolHecke := dom::toCartesianProductFromSignature([Pol, Hecke]);
    constPolHecke := dom::tensorConstructorFromSignature([Pol, Hecke]);

    printMonomial := z -> hold(_mult)(op(op(dom::splitPolHecke(z))));
	     
// creation d'elements.
	     
    ti :=
    proc(i : dom::type1N1)
    begin
	dom(Hecke::term(subsop([$1..N], i=i+1, i+1=i)));
    end_proc;
	     
    tiOnDom := 
    proc(vct: dom, i: dom::type1N1): dom
	local pol, hecke;
    begin
	dom::plus(op(map(dom::splitPolHecke(vct),
			 proc(v)
			     local pl, hck;
			 begin
			     [pl, hck] := v;
			     dom::plus(dom::constPolHecke([Pol::siOnDom(pl, i), 
							   Hecke::mult2(Hecke::Ti(i), hck)]),
				       dom::constPolHecke([Pol::tiOnDom(pl, i), hck]));
			 end_proc)));
    end_proc;

    multPolDom := 
    proc(vct: dom, pol: Pol): dom
    begin
	dom::plus(op(map(dom::splitPolHecke(vct), 
			 (z) -> dom::constPolHecke([Pol::mult2(pol, z[1]), z[2]]))));
    end_proc;

    mult2Slow := 
    proc(t1: dom, t2: dom): dom
    begin
	dom::plus(op(map(dom::splitPolHecke(t1),
			 proc(v)
			     local pl, hck, res, i;
			 begin
			     [pl, hck] := v;
			     hck := Hecke::lsupport(hck);
			     res := t2;
			     for i in revert(Perm::reducedWord(hck)) do
				 res := dom::tiOnDom(res, i);
			     end_for;
			     dom::multPolDom(res, pl);
			 end_proc)));
    end_proc;

    mult2Basis := 
    proc(x1,x2): dom
	option remember;
    begin
	dom::mult2Slow(dom::term(x1), dom::term(x2));
    end_proc;

    pol2Dom   := proc(pol: Pol): dom   begin dom::constPolHecke([pol, Hecke::one]); end_proc;
    Hecke2Dom := proc(hck: Hecke): dom begin dom::constPolHecke([Pol::one, hck]);   end_proc;

    initDomainAdd :=
    proc()
    begin
        declareConversion(Pol,   dom, dom::pol2Dom):
        declareConversion(Hecke, dom, dom::Hecke2Dom):
    end_proc;

begin
    Pol      := AKTools::Lagrange(N, nColors, R);
    Hecke    := HeckeAlg(N, R);
end_domain:


