export(combinat):
/**********************************************
La base est celle de Kang-Lee-Lee-Oh 
on code par 
[[ai, ki] $ 0=1..N-1] avec 1<=ai<i+1 et 0<=ki<=r-1 le mot de
la formule (2.4) page 7
une telle suite code un produit
  \Prod_{i=0..N-1}  T_{0,a_i}^{ki}
***********************************************/


domain vctBas(N : Type::NonNegInt, C : Type::NonNegInt) 
    category Cat::CombinatorialClass;
    inherits finiteClass(cartesianProduct(cartesianProduct([$1..i+1], [$0..C-1])
					  $ i=0..N-1));
    axiom Ax::systemRep;
    domtype := DOM_LIST;
end_domain:


domain AKn(N : Type::NonNegInt, Pol : Type::PolyExpr(X))
    local nColor, cstPol;
    category Cat::FiniteDimensionalAlgebraWithBasis(Dom::ExpressionField());
    inherits Dom::FreeModule(Dom::ExpressionField(), vctBas(N, nColor),
			     RankerBasis = (vctBas(N, nColor))::ranker );
    
    basisName := hold(AK);			     
    oneBasis := [[j+1, 0]$ j = 0..N-1];

    printMonomialOnGenT := 
    proc(mon)
	local co, v, pow, i, j, k, l, res;
    begin
	pow := (a, b) -> if b <> 0 then a, b else null() end_if;
	[co, v] := op(poly2list(mon));
	Factored([co, 
		  ([j, k] := op(v, i+1);
		   if k = 0 then
		       op(map(revert([hold(T)[l] $ l=j..i]), z -> (z, 1)))
		   else
		       op(map(revert([hold(T)[l] $ l=1..i]), z -> (z, 1))),
		       T[0], k,
		       (hold(T)[l], 1) $ l=1..j-1
		   end_if
		  ) $ i=0..N-1 ]);
    end_proc;

    printMonomialOnGen := 
    proc(mon)
	local co, v, pow, i, j, k, l, res;
    begin
	pow := (a, b) -> if b <> 0 then a, b else null() end_if;
	[co, v] := op(poly2list(mon));
	Factored([co, 
		  ([j, k] := op(v, i+1);
		   if k = 0 then
		       op(map(revert([hold(T).l $ l=j..i]), z -> (z, 1)))
		   else
		       op(map(revert([hold(T).l $ l=1..i]), z -> (z, 1))),
		       hold(x), k,
		       (hold(T).l, 1) $ l=1..j-1
		   end_if
		  ) $ i=0..N-1 ]);
    end_proc;

    printMonomialOnTijk :=
    proc(mon)
	local i, co, v;
    begin
	[co, v] := op(poly2list(mon));
	Factored([co, 
		  (hold(T)[i, v[i+1][1]], 
		   output::asciiArt(["(".expr2text(v[i+1][2]).")"])
		  ) $ i=0..N-1 ]);
    end_proc;

    printMonomial := dom::printMonomialOnGen;


    x  := dom::term([[1,1], [j+1, 0]$ j = 1..N-1]);
    t  := (i) -> dom::term(subsop([[j+1, 0] $ j = 0..N-1], (i+1) = [i,0]));

    xActBasis := 
/***********************************
  Redresse sur la base le produit
  \Prod_{i=0..r}  T_{0,a_i}^{ki} x
  Le reste du produit est inchang.
    0<=r<=N-1
**********************************/
    proc(v : dom::basisIndices, r = N-1 : Type::NonNegInt)
	option remember;
        local left, right, curJ, curK, j, tmp;
    begin
	left  := op(v, 1..r);
	[curJ, curK] := op(v, r+1);
	right := op(v, r+2..N);

	if r = 0 then
	    if curK+1 = nColor then
		return(dom::plus(dom::monomial(-coeff(Pol, X, j), [[curJ, j], right])
				 $ j = 0..nColor-1));
	    else
		return(dom::term([[curJ, curK+1], right]));
	    end_if;
	end_if;
	
	if curK = 0 then
	    if curJ = 1 then
		dom::term([left, [1,1], right]);
	    else // curK = 0 and curJ > 1 
		dom::xActBasis(v, r-1);
	    end_if;
	else /* curK <> 0 */
	    if curJ = 1 then
		if curK < nColor-1 then
		    dom::term([left, [curJ, curK+1], right]);
		else /* curK+1 = nColor, on applique le Polynome */
		    dom::plus(dom::monomial(-coeff(Pol, X, j), [left, [curJ, j], right])
			      $ j = 0..nColor-1);
		end_if;
	    else // curK <> 0 and curJ > 1 then
		dom::plus(dom::xActBasis(v, r-1),
			  (tmp := dom::xActBasis([left, [1, curK], right], r-1);
			   for j from 1 to curJ-2 do
			       tmp := dom::tAct(tmp, j, r-1);
			   end_for;
			   tmp),
			  (tmp := dom::term([left, [1, 1], right]);
			   for j from 1 to curK do
			       tmp := dom::xAct(tmp, r-1);
			   end_for;
			   for j from 1 to curJ-2 do
			       tmp := dom::tAct(tmp, j, r-1);
			   end_for;
			   -tmp)			  
			 ); 
	    end_if;
	end_if;
    end_proc;

    xAct := operators::makeLinear(dom::xActBasis, Source=dom, ImageSet=dom);

    tActBasis := // action a droite !!!
/***********************************
  Redresse sur la base le produit
  \Prod_{i=0..r}  T_{0,a_i}^{ki} T_l
  Le reste du produit est inchang.
    1 <= l <= r <= N-1
**********************************/
    proc(v : dom::basisIndices, 
	 l : Type::NonNegInt, 
	 r = N-1 : Type::NonNegInt)
	option remember;
        local left, right, curJ, curK, j, tmp;
    begin
	left  := op(v, 1..r);
	[curJ, curK] := op(v, r+1);
	right := op(v, r+2..N);

	if r = l and curJ = r + 1 and curK = 0 then 
	    return(dom::term([left, [r, 0], right]));
	end_if;
	
	if curK = 0 then
	    if l < curJ-1 then
		dom::tActBasis(v, l, r-1);
	    elif l = curJ-1 then
		dom::term([left, [curJ-1, curK], right]);
	    elif l = curJ then
		dom::monomial(-1, v);
	    else // l > curJ
		dom::tActBasis(v, l-1, r-1);		
	    end_if;
	else // curK <> 0
	    if l < curJ-1 then
		dom::tActBasis(v, l, r-1);
	    elif l = curJ-1 then
		dom::monomial(-1, v);
	    elif l = curJ then
		dom::term([left, [curJ+1, curK], right]);
	    else // l > curJ
		dom::tActBasis(v, l-1, r-1);		
	    end_if;	    
	end_if;
    end_proc;

    tAct := operators::makeLinear(dom::tActBasis, Source=dom, ImageSet=dom);

    mult2Basis := 
/***********************************
  Multiplication de deux produits
  \Prod_{i=0..N-1}  T_{0,a_i}^{ki}
**********************************/
    proc(v1 : dom::basisIndices, v2 : dom::basisIndices) : dom
	option remember;
	local res, l, curJ, curK, curI;
    begin
	res := dom::term(v1);
	for curI from 0 to N-1 do
	    [curJ, curK] := op(v2, curI+1); // action de T_{curI, curJ}^(curK)
	    if curK = 0 then // action de T_{i,j}
		for l from curI downto curJ do
		    res := dom::tAct(res, l);
		end_for;
	    else // action de T_{curI,1} x^curK T^{1,curJ-1}
		for l from curI downto 1 do
		    res := dom::tAct(res, l);
		end_for;		
		for l from 1 to curK do
		    res := dom::xAct(res);
		end_for;		
		for l from 1 to curJ-1 do
		    res := dom::tAct(res, l);
		end_for;				
	    end_if;
	end_for;
	res;
    end_proc;

begin
    Pol    := expand(Pol);
    cstPol := subs(Pol, X=0);    
    nColor := degree(Pol);
end_domain:

/* usage
reset():TEXTWIDTH:=118:read("H0n.mu");myTools::devMode(FALSE):
AK := AKnc(3, X^2-1):x := AK::x:T := AK::t:
*/

/* Check for the associativity of the product.
 *********************************************
bs := AK::basis():
(print (cc); (ress := (aa*bb)*cc-aa*(bb*cc); 
             if not iszero(ress) then print(aa,bb,cc, ress); error("Produit non Associatif"); 
             else print(NoNL, "."); null(); 
             end_if) $ aa in bs $ bb in bs) $ cc in bs;

bs := AK::basis(): aleat := random(1..nops(bs)):
(aa := bs[aleat()]; bb := bs[aleat()]; cc := bs[aleat()]; 
 ress := (aa*bb)*cc-aa*(bb*cc); 
 if not iszero(ress) then print(aa,bb,cc, ress); error("Produit non Associatif"); 
 else print(NoNL, "."); null(); 
 end_if) $ cpt = 1..10000
*/

/*
gen := AK::genericElement(z -> c[op(z)]);
g0 := mapcoeffs(gen, subs, op(solve({coeff((t)*gen)})));
g1 := mapcoeffs(gen, subs, op(solve({coeff((t+1)*gen)})));
g00 := mapcoeffs(gen, subs, op(solve({coeff((t)*gen*(t))})));
g10 := mapcoeffs(gen, subs, op(solve({coeff((t+1)*gen*(t))})));
g01 := mapcoeffs(gen, subs, op(solve({coeff((t)*gen*(t+1))})));
g11 := mapcoeffs(gen, subs, op(solve({coeff((t+1)*gen*(t+1))})));

genSym := _plus((a[op(p)]*Sym::h(p)([x,y])) $ p in partitions::list(i, MaxLength=2, MaxPart=3) $ i=0..4);
*/

/* Calcul du radical
gen := AK::genericElement(z->c[op(z)]):
print("ideal..."):
ideal := [(print(NoNL, "."); b*gen) $ b in AK::basis()]:
traceAk := 
proc(el)
begin
    _plus((print(NoNL, "."); coeff(el*AK::term(b), b)) $ b in AK::basisIndices());
end_proc:
print("traces..."):i:=0:
traces   := [ (print(NoNL, "Ok",(i:=i+1),"\n"); traceAk(q)) $ q in ideal ]:
print("solving..."):
linsolve(traces);
*/

/* recherche des idempotents de rang 1
gen := AK::genericElement(z->c[op(z)]);

[vpx, vpt] := [2, -1]: 
res := mapcoeffs(gen, subs, linsolve({coeff(x*gen-vpx*gen), coeff(t*gen-vpt*gen)}));
resset := solve({coeff(res^2-res)});
idemp[vpx, vpt] := op(select(map(resset, z -> mapcoeffs(res, subs, z)), not iszero));

map(linalg::basis(map([v*iR $ v in AK::basis()], AK::sparseVector)), AK::fromSparseVector);
*/

