// Action of cristal operators on words.
////////////////////////////////////////////
//export(combinat);
//

// Warning : Matrix are stored in cartesian coordinates !!!

matrix2BiWordRows := 
// read the matrix rows by rows and returns the biword
// [lig, column]
proc(m : integerMatrices)
    local i, j, n, nlig1, nloc1;
begin
    nlig1 := nops(m)+1;
    ncol1 := nops(m[1])+1;
    [(([nlig1 - i, j] $ m[i][j])
      $ j = 1..nops(m[1])) $ i = 1..nops(m)];
end_proc:

matrix2BiWordColumns := 
// read the matric columns by columns and returns the biword
// [lig, column]
proc(m : integerMatrices)
    local i, j, n, nlig1;
begin
    nlig1 := nops(m)+1;
    ncol1 := nops(m[1])+1;
    [(([i, ncol1 - j] $ m[nlig1 - i][ncol1 - j])
      $ i = 1..nops(m)) $ j = 1..nops(m[1])];
end_proc:

rskPQschutz := 
proc(m : integerMatrices)
begin
    [tableaux::insertWord([], map(matrix2BiWordRows(m),    op, 2)),
     tableaux::insertWord([], map(matrix2BiWordColumns(m), op, 1))]; 
end_proc:

schutzTab :=
proc(t: tableaux, n = null(): Type::Union(DOM_NULL, Type::Integer))
begin
    if args(0) = 1 then
	n :=  max(op(map(t, max@op)));
    end_if;
    tableaux::insertWord([],
        map(revert(tableaux::toWord(t)), z-> n+1-z)); 
end_proc:

word2matrix :=
proc(w: words, n = null(): Type::Union(DOM_NULL, Type::Integer))
    local i;
begin
    if args(0) = 1 then
	n :=  max(op(w));
    end_if;
    n := [0$n];
    [subsop(n, i=1) $ i in w];
end_proc:

rskInvPQschutz :=
proc(t1: tableaux, t2: tableaux, width, height)
    local res, mults, i, ml, sml, mli;
begin
    if args(0) < 3 then
	width  := max(op(map(t1, max@op)));
    end_if;
    if args(0) < 4 then
	height := max(op(map(t2, max@op)));
    end_if;
    mults := integerMatrices::transpose(revert
					(tableaux2RowMatrix(t1, width, width)));
    mults := [[op(mults[i], 1..min(i-1, nops(mults[i])))]
	      $ i = 1..nops(mults)];
    mults := revert(select(mults, not _equal, []));
    res   := tableaux2ColumnMatrix(t2, height, width);
    for ml in mults do
	sml := 0;
	for i from 1 to nops(ml) do
	    sml := sml + ml[i];
	    res := rightMatrix(res, i, sml);
	end_for;
    end_for;
    assert(rskPQschutz(res) = [t1, t2]);
    res;
end_proc:

tableaux2RowMatrix := 
proc(t1: tableaux, width, height)
    local l, rowT, rowRes, res;
begin
    if args(0) = 1 then
	width := max(op(map(t1, max@op)));
    end_if;
    if args(0) = 3 then
	res := [[0 $ width] $ height-nops(t1)];
    else
	res  := [];
    end_if;
    t1   := t1;
    for rowT in t1 do
	rowRes := [0 $ width];
	for l in rowT do
	    rowRes[l] := rowRes[l] + 1;
	end_for;
	res := append(res, rowRes);
    end_for;
    res;
end_proc:

tableaux2ColumnMatrix := 
proc(t1: tableaux, height, width)
    local res;
begin
    res := revert(integerMatrices::transpose
		  (revert(tableaux2RowMatrix(args()))));
end_proc:

schutzMatrix :=
proc(m : integerMatrices)
    local tP, tQ;
begin
    [tP, tQ] := rskPQschutz(m);
    tQ := schutzTab(tQ, nops(m));
    rskInvPQschutz(tP, tQ, nops(m[1]), nops(m));
end_proc:

stepSchOld :=
proc(a : integerMatrices, b : integerMatrices)
    local ha, hb, res;
begin
    ha := nops(a);
    hb := nops(b);
    assert(a=schutzMatrix(schutzMatrix(a)));
    assert(b=schutzMatrix(schutzMatrix(b)));
    a := schutzMatrix(a);
    b := schutzMatrix(b);
    assert(a.b=schutzMatrix(schutzMatrix(a.b)));
    res := schutzMatrix(a.b);
    [res[1..hb], res[hb+1..ha+hb]]; 
end_proc:

stepSch :=
proc(a : integerMatrices, b : integerMatrices)
    local ha, hb, res;
begin
    ha := nops(a);
    hb := nops(b);
    res := schutzMatrix(a.b);
    [res[1..hb], res[hb+1..ha+hb]]; 
end_proc:

checkRMatrix :=
proc(c, b, a)
    local cba, bca, cab, acb, bac, abc, abc2;
begin
    cba := [c,b,a];

    cab := [cba[1]].stepSch(cba[2], cba[3]); 
    acb := stepSch(cab[1], cab[2]).[cab[3]]; 
    abc := [acb[1]].stepSch(acb[2], acb[3]); 
    
    bca := stepSch(cba[1], cba[2]).[cba[3]]; 
    bac := [bca[1]].stepSch(bca[2], bca[3]); 
    abc2:= stepSch(bac[1], bac[2]).[bac[3]];

    [abc, abc2];
end_proc;



downMatrix := 
// compute the action of down_i^power on the matrix m
proc(m : integerMatrices, i : Type::PosInt, power = 1)
    local w, unpairI1, unpairI2, j, lig, col, nlig1;
begin
    nlig1 := nops(m)+1;
    w := matrix2BiWordColumns(m);
    [unpairI1, unpairI2] := words::unpairedLetters(map(w, op, 1), i);
    for j from 1 to min(nops(unpairI2), power) do
	// first unpaired i+1 
	[lig, col] := w[unpairI2[j]];
	lig := nlig1 - lig;
	m[lig][col]   := m[lig][col] - 1;
	m[lig+1][col] := m[lig+1][col] + 1;
    end_for;
    m;
end_proc:

upMatrix := 
// compute the action of up_i^power on the matrix m
proc(m : integerMatrices, i : Type::PosInt, power = 1)
    local w, unpairI1, unpairI2, j, lig, col, nlig1;
begin
    nlig1 := nops(m)+1;
    w := matrix2BiWordColumns(m);
    [unpairI1, unpairI2] := words::unpairedLetters(map(w, op, 1), i);
    for j from 1 to min(nops(unpairI1), power) do
	// last unpaired i
	[lig, col] := w[unpairI1[nops(unpairI1)+1-j]];
	lig := nlig1 - lig;
	m[lig][col]   := m[lig][col] - 1;
	m[lig-1][col] := m[lig-1][col] + 1;
    end_for;
    m;
end_proc:


leftMatrix := 
// compute the action of down_i^power on the matrix m
proc(m : integerMatrices, i : Type::PosInt, power = 1)
    local w, unpairI1, unpairI2, j, lig, col, nlig1;
begin
    nlig1 := nops(m)+1;
    w := matrix2BiWordRows(m);
    [unpairI1, unpairI2] := words::unpairedLetters(map(w, op, 2), i);
    for j from 1 to min(nops(unpairI2), power) do
	// first unpaired i+1 
	[lig, col] := w[unpairI2[j]];
	lig := nlig1 - lig;
	m[lig][col]   := m[lig][col] - 1;
	m[lig][col-1] := m[lig][col-1] + 1;
    end_for;
    m;
end_proc:

rightMatrix := 
// compute the action of up_i^power on the matrix m
proc(m : integerMatrices, i : Type::PosInt, power = 1)
    local w, unpairI1, unpairI2, j, lig, col, nlig1;
begin
    nlig1 := nops(m)+1;
    w := matrix2BiWordRows(m);
    [unpairI1, unpairI2] := words::unpairedLetters(map(w, op, 2), i);
    for j from 1 to min(nops(unpairI1), power) do
	// last unpaired i
	[lig, col] := w[unpairI1[nops(unpairI1)+1-j]];
	lig := nlig1 - lig;
	m[lig][col]   := m[lig][col] - 1;
	m[lig][col+1] := m[lig][col+1] + 1;
    end_for;
    m;
end_proc:

m := [[2, 1],[1,1],[0,2]];

null():

/*
n := 6;
([t1, t2] := rskPQschutz(m);
 m2 := rskInvPQschutz(t1,t2);
 print([integerMatrices::printCompact(m), 
        tableaux::printCompact(t1), tableaux::printCompact(t2), 
       integerMatrices::printCompact(m2)]);
  if m <> m2 then error("..."); else null() end_if) $
  m in integerMatrices(i,j) $ i in compositions(n) $ j in compositions(n);
*/

/*
 (if tableaux::schensted(w) <> ([t1,t2] := rskPQschutz(word2matrix(w));
                               [t1, schutzTab(t2)])
  then error(w); else null() end_if
 ) $ w in words::list(3,5);
 */

listMatDim :=
proc(n, w, h)
    local rs, cs;
begin
    [op(integerMatrices(rs, cs))
	$ rs in integerVectors::list(n, h)
		$ cs in integerVectors::list(n, w)]
end_proc:
/*
 (res := checkRMatrix(a,b,c);
  print(map([a,b,c], integerMatrices::printCompact)),
       if not _equal(op(res)) then print(a,b,c); error();
       else null(); end_if
 ) $ a in listMatDim(5, 1, 4) $ b in listMatDim(6, 3, 4) $ c in listMatDim(7, 2, 4) ;   
*/
                     
mats := [[0, 4, 1]], [[3, 2, 0], [1, 0, 0]], [[7, 0, 0]];

listMatRowSum :=
proc(rs, height, width)
begin
    [op(integerMatrices(cs, [rs $ height]))
     $ cs in integerVectors::list(rs*height, width)]
end_proc:

/* TODO: compute the percentage of matrices where checkRMatrix is wrong.
 ***********************************************************************  
 (res := checkRMatrix(a,b,c);
  print(map([a,b,c], integerMatrices::printCompact)),
       if not _equal(op(res)) then print(a,b,c); error();
       else null(); end_if
 ) $ a in listMatRowSum(3, 2, 3) $ b in listMatRowSum(3, 2, 3) 
$ c in listMatRowSum(3, 2, 3);
*/
