/*****************************************************************************
  $Id: DoubleHecke.mu,v 1.1.1.2 2005/01/26 13:42:23 nthiery Exp $
  File:         lib/EXPERIMENTAL/HeckeClifford/Hecke0.mu
  Authors:      Florent Hivert <florent.hivert@univ-mlv.fr>
  Created:      20/01/2003
  License:      LGPL
*****************************************************************************/

alias(Perm = combinat::permutations):

alias(declaredeclareDomain = operators::overloaded::declaredeclareDomain):
alias(declareSignature = operators::overloaded::declareSignature):
alias(declareConversion = operators::overloaded::declareConversion):
alias(plus2 = operators::plus2):
alias(mult2 = operators::mult2):
alias(makeLinear = operators::makeLinear):

// alias(DoubleHeckeAlgebra = experimental::DoubleHeckeAlgebra):

alias(HAT=DoubleHeckeAlgebraTools):

//////////////////////////////////////////////////////////////////////////////
domain HAT
    info_str := "Helper tools for the domain experimental::HeckeAlgebra";
end_domain:
domain HAT::permutationsOf(n: Type::NonNegInt)
    inherits combinat::permutations;
    category Cat::CombinatorialClass;
    axiom    Ax::systemRep;

    type := Type::Predicate("Permutation(".expr2text(n).")",
	    (p) -> _lazy_and(
		testtype(p, Type::ListOf(Type::PosInt)),
		nops(p)=n,
                sort(p)=[$1..n]));
end_domain:
//////////////////////////////////////////////////////////////////////////////
domain HAT::pairsOfPermutationsOf(n: Type::NonNegInt)
    inherits Dom::BaseDomain;
    category Cat::CombinatorialClass;
    axiom    Ax::systemRep;
  
  domtype := DOM_LIST;
  type := Type::ListProduct((HAT::permutationsOf(n)) $ 2);
	 
end_domain:

/****
 0-Hecke Algebra
**********************************************/
domain HeckeAlgebra(n : Type::NonNegInt,
		    R = Dom::ExpressionField())

  category Cat::AlgebraWithBasis(R);
  inherits experimental::AlgebraWithBasis(R, HAT::permutationsOf(n));

  basisType := HAT::permutationsOf(n);

  exprTerm := l -> _concat(hold(P),op(l)); 
  
  one := dom::term([$1..n]);
  N   := n;
  R   := R;
  
  mult2perm := 
    proc(perm1, perm2)
	local res, sgn; 
    begin
	res := perm1;
	sgn := 1;
	for i in Perm::reducedWord(perm2) do
	    if res[i] < res[i+1] then 
	    res := subsop(res, i=res[i+1], (i+1)=res[i]);
	    else 
		sgn := -sgn;
	    end_if;
	end_for;
	[R(sgn), res];
    end_proc;
  
  mult2basis := 
    proc(pair1, pair2)
	local p1, p2, sgn1, sgn2;
    begin
	dom::monomial(op(mult2perm(args())));
    end_proc;
  
begin
end_domain:


domain DoubleHeckeAlgebra(n : Type::NonNegInt,
		    R = Dom::ExpressionField())

  category Cat::AlgebraWithBasis(R);
  inherits experimental::AlgebraWithBasis(R, HAT::pairsOfPermutationsOf(n));

  basisType := HAT::pairsOfPermutationsOf(n);

  exprTerm := l -> _concat(hold(T),op(l[1]),hold("T"),op(l[2])); 
  
  one := dom::term([[$1..n]$2]);
  N   := n;
  R   := R;
  
  mult2Perm := 
    proc(perm1, perm2)
	local res, sgn; 
    begin
	res := perm1;
	sgn := 1;
	for i in Perm::reducedWord(perm2) do
	    if res[i] < res[i+1] then 
	    res := subsop(res, i=res[i+1], (i+1)=res[i]);
	    else 
		sgn := -sgn;
	    end_if;
	end_for;
	[R(sgn), res];
    end_proc;
  
  mult2Basis := 
    proc(pair1, pair2)
	local p1, p2, sgn1, sgn2;
    begin
	[sgn1, p1] := dom::mult2Perm(pair1[1], pair2[1]);
	[sgn2, p2] := dom::mult2Perm(pair1[2], pair2[2]);
	dom::monomial(sgn1*sgn2,[p1, p2])
    end_proc;
  
begin
end_domain:
