/*****************************************************************************
  $Id: DoubleSymmetricGroupAlgebra.mu,v 1.1.1.5 2005/01/26 13:42:23 nthiery Exp $
  File:         lib/EXPERIMENTAL/HeckeClifford/Hecke0.mu
  Authors:      Nicolas M. Thiry <nthiery@users.sf.net>
  Created:      05/06/2003
  License:      LGPL
*****************************************************************************/

/*

Let V be the vector space spanned by permutations by transposition of
the positions. Consider the elementary transpositions \sigma_i seen as
operators in GL(V) by acting on the positions. Similarwise, consider
the operators $\pi_i$ of the Hecke algebra that act on V by sorting
between positions i and i+1.

This file implements the algebra A generated by the \sigma_i's and
\pi_i's together. Here is a description of the relations between the
\sigma_i's and \pi_i's

\pi1 \sigma1 = \pi1

(sigma1 + id) pi1 = sigma1 + id

Note: simpler relation $(1+s_i)\pi_i=\pi_i(1+s_i)=2\pi_i$.

sigma1 pi1 = sigma1 - pi1 + id
sigma1 pi2 = pi_231 + pi_312 sigma 312 - pi_321
sigma1 pi2 = pi_12 + pi_21 sigma 21 - pi_212
sigma2 pi1 = pi_312 + pi_231 sigma 231 - pi_321

From those relations, this algebra is spanned by the products \pi_u
\sigma_v, where u and v are permutations such that the descents of u
are disjoint from the reculs of v. Those products actually form a
basis of A.

*/

alias(declareDomain = operators::overloaded::declareDomain):
alias(declareSignature = operators::overloaded::declareSignature):
alias(declareConversion = operators::overloaded::declareConversion):
alias(plus2 = operators::plus2):
alias(mult2 = operators::mult2):
alias(makeLinear = operators::makeLinear):
alias(generators = combinat::generators):
alias(permutations = combinat::permutations):
alias(subClass = combinat::subClass):
alias(cartesianProduct = combinat::cartesianProduct):

alias(DSGA=experimental::DoubleSymmetricGroupAlgebra):

alias(DSGAT=experimental::DoubleSymmetricGroupAlgebraTools):


//////////////////////////////////////////////////////////////////////////////
domain DSGAT
    info_str := "Helper tools for the domain experimental::DoubleSymmetricGroupAlgebra";

end_domain:

domain DSGA(N : Type::NonNegInt,
	    R = Dom::ExpressionField())

    category Cat::AlgebraWithBasis(R);
    inherits Dom::FreeModule(R, subClass(cartesianProduct,
					 permutations::list(N) $ 2));

    interfaceAdd := {
		   hold(toMatrix)
		 };

    N := N;

    printTerm :=
    proc(l: dom::basisIndices)
    begin
	_concat(hold(s), op(l[1]), hold(s), op(l[2]));
    end_proc;

    one := dom::term([ [$1..N] $ 2 ]);

    mult2Basis :=
    proc(l1: dom::basisIndices, l2: dom::basisIndices) : dom
    begin
	dom::term(zip(l1, l2, permutations::_mult));
    end_proc;

    onListBasis :=
    proc(x: dom::basisIndices, l: permutations) : permutations
	local i;
    begin
	if testargs() and nops(l) <> N then
	    error("Incoherent list size");
	end_if;
	permutations::_mult(x[1],
			    permutations::_mult(l, permutations::inverse(x[2])));
    end_proc;

    toMatrixBasis :=
    proc(x: dom::basisIndices) //: Dom::Matrix(R)
        local i, rank, unrank;
    begin
	[rank, unrank] := combinat::rankers::fromList(combinat::permutations(N));
	Dom::Matrix(R)(N!, N!, 
			     [(rank(dom::onListBasis(x, unrank(i))), i) = 1
                              $ i=1..N!]):
    end_proc;

    toMatrix := operators::makeLinear
	      (dom::toMatrixBasis,
	       Source=dom, ImageSet=Dom::Matrix(R),
	       %if traperror(multcoeffs(matrix(2,2),2))<>0 then
	           // temporary workaround for MuPAD 3.0.0's bug with multcoeff
		   Multcoeffs = ((m,c) -> m*c)
	       else
		   null();
	       end_if);

    // Matrix pretty print of the coefficients of an element of h
    printMatrix :=
    proc(x: dom)
	local P;
    begin
	P:=permutations(N):
	Dom::Matrix(R)(N,N,[(i,j)=coeff(x,[P[i],P[j]]) $i=1..N! $ j=1..N!]);
    end_proc;

    // Matrix pretty print of the coefficients of an element of h
    printMatrixAnnotated :=
    proc(x: dom)
	local P;
    begin
	P := map(permutations::list(N), l->_concat(hold(p),op(l)));
	DSGAT::annotate(dom::printMatrix(x), P, P, `p \ s`);
    end_proc;

end_domain:

