/*****************************************************************************
  $Id: FreeOperad.mu,v 1.1.1.2 2005/01/26 13:42:23 nthiery Exp $
  File:		lib/EXPERIMENTAL/FreeOperad.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
		Frdric Chapoton <chapoton at igd.univ-lyon1.fr>
  Purpose:	Source code for the domain experimental::FreeOperad
  Created:	Tue Dec  9 21:59:27 2003
  License:	LGPL
  Project:	MuPAD-Combinat, http://mupad-combinat.sf.net
*****************************************************************************/

//////////////////////////////////////////////////////////////////////////////
domain experimental::FreeOperadTools
    info_str := "Helper tools for the domain experimental::FreeOperad";
end_domain:

//////////////////////////////////////////////////////////////////////////////
alias(Trees = experimental::FreeOperadTools::orderedCompleteBinaryTreesLabelledOnLeaves):

domain Trees(Operators=[combinat::trees::dummyLabel])
    inherits combinat::trees; // Not binaryTrees for the moment!
    category Cat::TreesClass([NodeLabel=Union(Epsilon(x) $ x in Operators)],
			     Labelled, CompleteBinary);
    axiom    Ax::normalRep, Ax::canonicalRep;

    info_str := "A domain for ordered complete binary trees with labels on leaves";

    one := dom([1]);

    size :=	// Could be implemented in Cat::TreesClass
    proc(x: dom)
	local child;
    begin
	if nops(x) = 0 then
	    1
	else
	    _plus(dom::size(child) $ child in op(x));
	end_if;
    end_proc;

    mapLeaves := // can't use mapLabels, because we use labels on
	         // internal nodes to store which operator is used
    proc(y: dom, f)
    begin
	if nops(y)=0 then
	    subsop(y, 0=f(op(y,0), args(3..args(0))));
	else
	    map(y, dom::mapLeaves, args(2..args(0)));
	end_if;
    end_proc;

    graft :=
    proc(x: dom, y: dom, i : Type::PosInt)
	local m, n, f;
    begin
	m := dom::size(x);
	n := dom::size(y);
	if i>m then
	    error("wrong composition index")
	end_if;
	
	f :=
	prog::bless(
	proc(x: dom) : dom
	    local label;
	begin
	    if nops(x) = 0 then
		label := op(x,0);
		if label = i then
		    dom::mapLeaves(y, _plus, i-1);
		elif label < i then
		    x
		else
		    new(dom, label + n - 1);
		end_if;
	    else
		map(x, f);
	    end_if;
	end_proc, dom);
	f(x);
    end_proc;
end_domain:

//////////////////////////////////////////////////////////////////////////////
domain experimental::FreeOperad(R = Dom::ExpressionField(): DOM_DOMAIN,
				Operators = null())
    inherits Dom::FreeModule(R, Trees(Operators));
    category Cat::OperadWithBasis(R);

    info_str := "experimental::FreeOperad -- domain for the free operad on one binary generator";

    oneBasis := dom::basisIndices::one;

    printTerm := id; // probably just a temporary hack

    compositionBasis := dom::term @ dom::basisIndices::graft;

end_domain:
