/*****************************************************************************
  $Id: HeckeClifford.mu,v 1.1.1.4 2005/01/26 13:42:23 nthiery Exp $
  File:         lib/EXAMPLES/SymmetricGroupAlgebra.mu
  Authors:      Florent Hivert <florent.hivert@univ-mlv.fr>
  Created:      20/01/2003
  License:      LGPL
*****************************************************************************/

alias(Perm = combinat::permutations):
alias(IntV = combinat::integerVectors):

alias(declaredeclareDomain = operators::overloaded::declaredeclareDomain);
alias(declareSignature = operators::overloaded::declareSignature);
alias(declareConversion = operators::overloaded::declareConversion);
alias(plus2 = operators::plus2);
alias(mult2 = operators::mult2);
alias(makeLinear = operators::makeLinear);

alias(CliffordAlgebra = experimental::CliffordAlgebra);
alias(HeckeCliffordAlgebra = experimental::HeckeCliffordAlgebra);
alias(HeckeCliffordModuleMI = experimental::HeckeCliffordModuleMI);

alias(HCAT=experimental::HeckeCliffordAlgebraTools):

//////////////////////////////////////////////////////////////////////////////
domain HCAT
    info_str := "Helper tools for the domain examples::MyMultivariatePolynomials";
end_domain:

//////////////////////////////////////////////////////////////////////////////
domain HCAT::zeroOneVectorsOfLength(N: Type::NonNegInt)
    inherits combinat::integerVectors;
    category Cat::CombinatorialClass;
    axiom Ax::systemRep;

    type := Type::ListProduct(Type::Union(Type::Singleton(0),
					  Type::Singleton(1)) $ N);

    list  := 
    proc() 
	local S; 
    begin 
	[op(IntV::list(S, N, MaxPart=1)) $ S=0..N]; 
    end_proc;
    count := countFromGenerator;
    generator := dom::generatorFromList;
end_domain:

//////////////////////////////////////////////////////////////////////////////
domain HCAT::integerVectorsOfLength(N: Type::NonNegInt)
    inherits combinat::integerVectors;
    category Cat::CombinatorialClass;
    axiom Ax::systemRep;

    type := Type::ListProduct(Type::NonNegInt $ N);
end_domain:

//////////////////////////////////////////////////////////////////////////////
domain HCAT::permutationsOf(n: Type::NonNegInt)
    inherits combinat::subClass(combinat::permutations, Parameters=n);
    category Cat::CombinatorialClass;
    axiom    Ax::systemRep;

    type := Type::Predicate("Permutation(".expr2text(n).")",
	    (p) -> _lazy_and(
		testtype(p, Type::ListOf(Type::PosInt)),
		nops(p)=n,
                sort(p)=[$1..n]));
end_domain:

//////////////////////////////////////////////////////////////////////////////
domain HCAT::pairsOfZeroOneVectorAndPermutation(n: Type::NonNegInt)
    inherits (Dom::TensorProductOfFreeModules; // To force the loading.
	      Dom::TensorProductOfFreeModulesTools::cartesianProduct(
                 [HCAT::zeroOneVectorsOfLength(n),
		  HCAT::permutationsOf(n)]));
    category Cat::CombinatorialClass;
    axiom Ax::systemRep;

    domtype:= DOM_LIST;
    type := Type::ListProduct(HCAT::zeroOneVectorsOfLength(n),
			      HCAT::permutationsOf(n)
			     );
end_domain:

//////////////////////////////////////////////////////////////////////////////
domain HCAT::pairsOfZeroOneVectorAndIntegerVector(n: Type::NonNegInt)
    inherits Dom::BaseDomain;
    category Cat::CombinatorialClass;
    axiom Ax::systemRep;

    domtype:= DOM_LIST;
    type := Type::ListProduct(HCAT::zeroOneVectorsOfLength(n),
			      HCAT::integerVectorsOfLength(n)
			     );
end_domain:

/****
 Clifford Algebra
***************************************
generators: C[1]..C[n]
relations:  C[i]^2 = -1
             C[i]C[j] + C[j]C[i] = 0
internal rep: [i_1,...,i_n] in {0,1}^n  
***************************************/
domain CliffordAlgebra(n : Type::NonNegInt,
		       R = Dom::ExpressionField())

  category Cat::FiniteDimensionalAlgebraWithBasis(R);
  inherits experimental::AlgebraWithBasis(R, HCAT::zeroOneVectorsOfLength(n));

  dimension := 2^n;

  N := n;
  basisName := C;
  R := R;

//  exprTerm := l -> hold(_index)(dom::basisName, op(l)); 
  exprTerm := l -> _mult( hold(C).i^l[i] $ i = 1..n);
  
  one := dom::term([0 $ n]);

  mult2Basis := 
    proc(l1, l2)
	local sgn, res, i;
    begin
	res := l2;
	sgn := R::one;
	for i from n downto 1 do 
	    if l1[i] = 1 then
		if testtype(nops(select([op(l2, 1..i)], _equal, 1)),
			    Type::Odd) 
		    then sgn := -sgn;
		end_if;
		res[i] := 1-res[i];
	    end_if;
	end_for;
	dom::monomial(sgn, res)
    end_proc;
  
  begin
  end_domain:


/****
 Hecke-Clifford Algebra
**********************************************
Generators: C[1]..C[n], T[1]..T[n-1]
relations: Hecke + Clifford + 
           T_i C_i = C_{i+1} T_i
           (T_i + 1) C_{i+1} = C_i (T_i + 1) 
internal rep: [{0,1}^n, permutation(n)]
**********************************************/
domain HeckeCliffordAlgebra(n : Type::NonNegInt,
			    R = Dom::ExpressionField())

  category Cat::FiniteDimensionalAlgebraWithBasis(R);
  inherits Dom::FreeModule(R, HCAT::pairsOfZeroOneVectorAndPermutation(n));
  ZeroOneVectors := HCAT::zeroOneVectorsOfLength(n);

  exprTerm := l ->  _mult( hold(C).i^l[1][i] $ i = 1..n, 
			   _concat(hold(P),op(l[2]))); 

  dimension := 2^n*n!;
  one := dom::term([[0$n], [$1..n]]);
  N   := n;
  R   := R;
  
  Clifford := CliffordAlgebra(n, R);
  
  cliffMultList := // Take from CliffordAlgebra
    proc(l1 : dom::ZeroOneVectors, l2 : dom::ZeroOneVectors)
	local sgn, res, i;
    begin
	res := l2;
	sgn := R::one;
	for i from n downto 1 do 
	    if l1[i] = 1 then
		if testtype(nops(select([op(l2, 1..i)], _equal, 1)),
			    Type::Odd) 
		    then sgn := -sgn;
		end_if;
		res[i] := 1-res[i];
	    end_if;
	end_for;
	[sgn, res];
    end_proc;
  
  cliffMultBasis := 
    proc(p : DOM_LIST, l : dom::ZeroOneVectors)
	local cli, per, sgn, res;
    begin
	[cli, per] := p;
	[sgn, res] := dom::cliffMultList(l, cli);
	dom::monomial(sgn, [res, per]);  
    end_proc;
  cliffMult := makeLinear(dom::cliffMultBasis, Source=dom, ImageSet=dom);
  
  // relations:
  // T_i C_i = C_{i+1} T_i
  // (T_i + 1) C_{i+1} = C_i (T_i + 1) 
  tiMultBasis := // ti * 
    proc(p, i)
	local tiPerm, l2, perm, sgn;
    begin
	[l2, perm] := p;
	sgn := R::one;
	if contains(perm, i) < contains(perm, i+1) then 
	    tiPerm := subs(perm, [i=(i+1), (i+1)=i]);
	else 
	    tiPerm := perm;
	    sgn := -sgn;
	end_if;
	case [l2[i], l2[i+1]]
	      of [0,0] do (sgn*dom::term([l2, tiPerm]) ); break;
              // T_i C_i = C_{i+1} T_i
	      of [1,0] do (sgn*dom::term([subsop(l2, i=0, i+1=1), tiPerm])); break;
              // T_i C_{i+1} = C_i T_i + C_i - C_{i+1}
	      of [0,1] do (sgn*dom::term([subsop(l2, i=1, i+1=0), tiPerm])  
			   + dom::term([subsop(l2, i=1, i+1=0), perm]) 
			   - dom::term([subsop(l2, i=0, i+1=1), perm])); break;
              // T_i C_i C_{i+1} = -C_i C_{i+1} T_i - C_i C_{i+1} + 1
	      of [1,1] do ((-sgn)*dom::term([subsop(l2, i=1, i+1=1), tiPerm])  
			   - dom::term([subsop(l2, i=1, i+1=1), perm]) 
			   + dom::term([subsop(l2, i=0, i+1=0), perm])); break;
	  end_case;
    end_proc;
  
  tiMult := makeLinear(dom::tiMultBasis, Source=dom, ImageSet=dom );
		
  mult2Basis := 
    proc(el1, el2)
	local c1, p1, c2, p2, res; 
    begin
	[c1, p1] := el1;
	res := dom::term(el2);
	for i in revert(Perm::reducedWord(p1)) do
	    res := dom::tiMult(res, i);
	end_for;
	dom::cliffMult(res, c1);
    end_proc;
  
  initDomainAdd := 
    proc()
	option escape;
    begin
	dom::initAlgebraWithBasis();
	dom::CliffordToDomBasis := 
        proc(c) 
	    name HeckeCliffordAlgebra::CliffordToDomBasis; // Workaround of the bug of the profiler
	begin
	    dom::term([c, [$1..n]]):
	end_proc:
	dom::CliffordToDom := makeLinear(dom::CliffordToDomBasis, 
					 Source=dom::Clifford, ImageSet=dom):
	declareConversion(dom::Clifford, dom, dom::CliffordToDom):
    end_proc;
		       
// Modules: these declaration should be done
// by initDomain of the module itself, rather
// that by the algebra.
//*******************************************
//   MModule := // Induced by simples of Hecke
//     proc(compo)
// 	local res;
//     begin
// 	res := HeckeCliffordModuleMI(dom, compo);
// 	declareDomain(res);
// 	declareSignature(mult2, [dom, res], res::action);
// 	res;
//     end_proc;
 
//   makeWordModule := // Module on words
//     proc()
// 	local res;
//     begin
// 	res := experimental::HeckeCliffordWordModule(dom);
// 	declareDomain(res);
// 	declareSignature(mult2, [dom, res], res::action);
// 	res;
//     end_proc;
//   wordModule := dom::makeWordModule();
  
begin
end_domain:


domain experimental::HeckeCliffordWordModule(HCl : DOM_DOMAIN)
  local n;
  
  category Cat::LeftModule(HCl), Cat::ModuleWithBasis(HCl::R);
  inherits Dom::FreeModule(HCl::R, HCAT::pairsOfZeroOneVectorAndIntegerVector(HCl::N));

  N := HCl::N;
  R := HCl::R;

  wordType := HCAT::integerVectorsOfLength(dom::N);
  
  exprTerm := l -> _mult(hold(C).i^l[1][i] $ i = 1..dom::N, 
			 _concat(hold(W),op(l[2])));
  
  actionHeckeCliffBasisOnWord := 
    proc(el : HCl::basisIndices, word : dom::wordType)
	local zeroOne, perm, red, i, sgn;
    begin
	[zeroOne, perm] := el;
	red := combinat::permutations::reducedWord(perm);
	sgn := 1;
	for i in revert(red) do
	    if   word[i] = word[i+1] then return(dom::zero);
	    elif word[i] > word[i+1] then sgn := -sgn;
	    else word := subsop(word, i=word[i+1], i+1=word[i]);
	    end_if;
	end_for;
	dom::monomial(sgn, [zeroOne, word]);
    end_proc;
  
  // bilinearity on a non-commutative ring !!!
  
  actionHeckeCliffOnWord := 
  makeLinear(dom::actionHeckeCliffBasisOnWord, 
			Source=HCl, ImageSet=dom);

  actionHeckeCliffOnBasis := 
    proc(basisEl : dom::basisIndices, el: HCl) 
    begin
	dom::actionHeckeCliffOnWord(el*HCl::Clifford(basisEl[1]), basisEl[2]) 
    end_proc;
    
  actionHeckeCliffOnDom := 
  makeLinear(dom::actionHeckeCliffOnBasis,
			Source=dom, ImageSet=dom);

  action := (el, v) -> dom::actionHeckeCliffOnDom(v, el);
  
  initDomainAdd := 
    proc()
	option escape;
    begin
        declareDomain  (dom);
        declareSignature(plus2, [dom, dom],   dom::plus2);
 	declareSignature(mult2, [HCl, dom], dom::action);
        declareSignature(mult2, [dom::R, dom],   (r,x)->dom::multcoeffs(x,r));
    end_proc;
  
begin
    n := HCl::N;
end_domain:



domain HeckeCliffordModuleMI(HCl : DOM_DOMAIN, Compo: combinat::compositions)
  category Cat::LeftModule(HCl), Cat::ModuleWithBasis(HCl::R);
  
  inherits HCl::Clifford;
  N := HCl::N;
  R := HCl::R;
  
  exprTerm := l -> _mult(hold(C).i^l[i] $ i = 1..dom::N, 
			 hold(_index)(hold(S), op(Compo)));
  
  printTerm := l -> _mult( hold(C).i^l[i] $ i = 1..dom::N);

  print := 
    proc(x: dom)
	local res;
    begin
	case dom::nterms(x)
	    of 0 do return(0);
	    of 1 do res := dom::printMonomial(dom::lmonomial(x));
		break;
	    otherwise
		res := 
		hold(_plus)(dom::printMonomial(x) $ x in dom::monomials(x));
	    end_case;
	_mult(res, hold(_index)(hold(S), op(Compo)));
    end_proc;
  
  descentsSet := {op(combinat::compositions::descents(Compo))}; 
  
  actionHeckeCliffBasisOnEI := // action on the generator Ei
    proc(el : HCl::basisIndices)
	local zeroOne, perm, red, tmp, tmp2;
    begin
	[zeroOne, perm] := el;
	red := combinat::permutations::reducedWord(perm);
	[red, tmp, tmp2] := split(red, _in, dom::descentsSet);
	if nops(tmp) <> 0 then 
	    dom::zero;
	else
	    dom::multcoeffs(dom::term(zeroOne), (-1)^nops(red));
	end_if;
    end_proc;
  
  actionHeckeCliffOnEI := makeLinear(dom::actionHeckeCliffBasisOnEI, 
				     Source=HCl, ImageSet=dom);
  toClifford   := x-> new(HCl::Clifford, extop(x)); 
  fromClifford := x-> new(dom, extop(x));
  
  action :=
    proc(el : HCl, v : dom)
    begin
	dom::actionHeckeCliffOnEI(el * dom::toClifford(v));
    end_proc;
  
  initDomainAdd := 
    proc()
    begin
 	declareDomain(dom);
 	declareSignature(mult2, [HCl, dom], dom::action);
    end_proc;
  
  CMorphism := 
    proc (el : HCl::Clifford)
	option escape;
	local DOM;
    begin
	DOM := dom;
        proc(v : DOM)
	begin
	    DOM::fromClifford(DOM::toClifford(v) * el);
	end_proc:
    end_proc;
  
begin
    if _plus(op(Compo)) <> HCl::N then 
	error(expr2text(Compo)." in not a composition of ".expr2text(HCl::N));
    end_if;
end_domain:
