/*****************************************************************************
  $Id: LandoHopfAlgebra.mu,v 1.1.1.3 2005/01/26 13:42:23 nthiery Exp $
  File:		lib/EXPERIMENTAL/LandoHopfAlgebra.mu
  Authors:	Florent Hivert <florent.hivert@univ-mlv.fr>
  Purpose:	Source code for the domain experimental::LandoHopfAlgebra
  Created:	Tue Apr 13 12:06:02 2004
  License:	LGPL
  Project:	MuPAD-Combinat, http://mupad-combinat.sf.net
*****************************************************************************/

alias(GR = combinat::graphs):
%if combinat::graphs = FAIL then
    userinfo(1, "combinat::graphs not available (missing Nauty dynamic module?); disabling experimental::LandoHopfAlgebra"):
    experimental::LandoHopfAlgebra := FAIL;
    experimental::interface := experimental::interface minus
       { hold(LandoHopfAlgebra) };
else

domain experimental::LandoHopfAlgebra(R=Dom::ExpressionField())
    inherits Dom::FreeModule(R, GR);
    category Cat::GradedHopfAlgebraWithBasis(R);

    info_str := "experimental::LandoHopfAlgebra -- S.~Lando's Hopf Algebra";

    basisName := hold(G);

    straightenBasis := dom::term@GR::straighten;
    create          := dom::straightenBasis;

    oneBasis := GR([]);

    mult2Basis := dom::term@GR::straighten@GR::directSum;

    coproductBasis := 
    proc(gr : dom::basisIndices)
	local n, spl, k;
    begin
	n := GR::size(gr);
	dom::tensorSquare::plus
	(dom::tensorConstructor(dom::straightenBasis(GR::sublabel(gr, spl[1])),
				dom::straightenBasis(GR::sublabel(gr, spl[2])))
		  $ spl in combinat::splitNK(n, k)
			   $ k=0..n);
    end_proc;

//     coproductBasis := 
//     proc(gr : dom::basisIndices)
// 	local n, edges, nEdges, chosen, create;
//     begin
// 	n := GR::size(gr);
// 	create := dom::straightenBasis@GR::removeIsolatedVertices;
// 	edges := select(GR::getEdges(gr), ed -> (ed[1]<ed[2]));
// 	dom::tensorSquare::plus
// 	((print(gr, chosen);
// 	   dom::tensorConstructor(
// 	       create(GR::sublabel(GR::addEdges(GR([[] $ n]), chosen),
// 				    [op({op(map(chosen, op))})])),
// 	       create(GR::contractEdges(gr, chosen))))
// 	          $ chosen in combinat::subwords(edges));
//     end_proc;

    projectPrimitifsBasis := 
    proc(gr : dom::basisIndices)
	local n, part, p;
    begin
	n := GR::size(gr);
	dom::plus((-1)^(nops(part)-1) * (nops(part)-1)! * 
		  dom::mult(dom::straightenBasis(GR::sublabel(gr, [op(p)])) $ p in part)
		  $ part in combinat::setPartitions::list([$1..n]));
    end_proc;

    projectPrimitifs := operators::makeLinear(dom::projectPrimitifsBasis, 
					      Source = dom, ImageSet = dom);

    scalarBasis := 
    proc(b1 : dom::basisIndices, b2 : dom::basisIndices)
    begin
	if b1=b2 then 1 else 0; end_if;
    end_proc;

    scalarOld := operators::makeBilinear(dom::scalarBasis);

    scalar := 
    proc(v1 : dom, v2 : dom)
	local sv1, g;
    begin
	sv1 := dom::support(v1);
	_plus(dom::coeff(v1, g) * dom::coeff(v2, g) $ g in sv1);
    end_proc;

    relations := 
    proc(n : Type::NonNegInt)
	local igr, lgr, gr, grTilde, grEdges, edg, res, nres;
    begin
	res := [];
	lgr := GR::list(n);
	for igr from 1 to nops(lgr) do
	    print(NoNL, ".",igr);
	    gr      := lgr[igr];
	    grEdges := GR::getEdges(gr);
	    for edg in grEdges do
		grTilde := GR::LandoTilde(gr, edg);
		nres :=  dom::create(gr) 
			      - dom::create(GR::removeEdge(gr, edg))
			      - dom::create(grTilde) 
			      + dom::create(GR::removeEdge(grTilde, edg));
		res := append(res, nres);
	    end_for;
	end_for;
	select(res, z->not iszero(z));
    end_proc;

    relationsBasis := 
    proc(n)
	option remember;
        local mat, rels;
    begin
	rels := dom::relations(n);
	if rels = [] then return([]) end_if;
	mat := _concat(op(map(rels, dom::sparseVector)));
	map(linalg::basis(linalg::col(mat, 1..linalg::ncols(mat))),
	    dom::fromSparseVector);
    end_proc;
			  
    relationsDim := 
    proc(n)
	option remember;
        local mat, rels;
    begin
	rels := dom::relations(n);
	if rels = [] then return([]) end_if;
	mat := _concat(op(map(rels, dom::sparseVector)));
	linalg::rank(mat);
    end_proc;

    linboxrel := 
    // smithform_gap essaimat.txt 10 10 -3 6 prime 1 1
    // smithform_gap relat5.mat 10 10 -4 4 17 2 1000
    proc(n)
	local nrel, rel, trm, file, igr, lgr, gr, filename, grEdges, nEdges,
	      edg, grTilde;
    begin
	filename := "relat".n.".mat";
	lgr := GR::list(n);
	file := fopen(filename, Write, Text);
	// +1 the place of the empty graphs 
	fprint(Unquoted, file, "".(n*(n-1)/2*nops(lgr))." ".(nops(lgr)+1)." M"); 
	nrel := 0;
	for igr from 1 to nops(lgr) do
	    print(NoNL, ".",igr);
	    gr := lgr[igr];
	    grEdges := GR::getEdges(gr);
	    nEdges  := GR::getEdgeNumber(gr);
	    for edg in grEdges do
		grTilde := GR::LandoTilde(gr, edg);
		if GR::getEdgeNumber(grTilde) > nEdges then next; end_if;
		rel :=  dom::create(gr) 
			      - dom::create(GR::removeEdge(gr, edg))
			      - dom::create(grTilde) 
			      + dom::create(GR::removeEdge(grTilde, edg));
		if not iszero(rel) then 
		    nrel := nrel + 1;
		    (fprint(Unquoted, file, 
			   _outputSequence((nrel," ",op(trm[2])," ",expr2text(trm[1])))))
		     $ trm in poly2list(extop(rel));
		end_if;
	    end_for;
	end_for;
	fprint(Unquoted, file, "0 0 0");
	fclose(file);
    end_proc;

    genQuotient := 
    proc(n)
	local relat, res, rel, size, scal, sol;
    begin
	print("Generic Element");
	res   := dom::genericElement(z -> hold(a)[op(z)], n);
	print("Relations");	
	relat := dom::relations(n);
	size  := dom::basisIndices::count(n);
	print("C'est parti...");
	for rel in relat do
	    if not iszero((scal := dom::scalar(rel, res))) then
		size := size-1;
		print(degree(extop(rel)), rel);
		sol := op(linsolve({scal}));
		res := mapcoeffs(res, subs, sol);
	    end_if;
	end_for;
	size, res;
    end_proc;


    genQuotient2 := proc(n) // Time: 240970 msec pour n=7
	local ntime, otime, stime, res, lgr, gsize, size, igr, scal, sol, grEdges, edj, inci, gr, grTilde, inc;
    begin
	print("Generic Element");
	inci  := 0;
	res   := dom::genericElement(z -> inc.(inci := inci+1), n);
	lgr   := dom::basisIndices::list(n);
	size  := nops(lgr);
	gsize := size;
	stime := time();
	ntime := stime();	       
	for igr from 1 to nops(lgr) do
	    print(NoNL, "\nGr[".igr."/".gsize."]");
	    gr := lgr[igr];
	    grEdges := GR::getEdges(gr);	    
	    for edj in grEdges do
		grTilde := GR::LandoTilde(gr, edj);
		scal := dom::coeff(res, gr) 
		      - dom::coeff(res, GR::straighten(GR::removeEdge(gr, edj)))
		      - dom::coeff(res, GR::straighten(grTilde)) 
		      + dom::coeff(res, GR::straighten(GR::removeEdge(grTilde, edj)));
		if not iszero(scal) then 
		    otime := ntime; ntime := time();
		    size := size-1;
		    print(NoNL, "--Rel (t=".(ntime-otime)." m=".
			  round((ntime-stime)/(gsize-size)).") dim=".size);
		    sol := op(linsolve({scal}));
		    res := mapcoeffs(res, subs, sol);
		end_if;
	    end_for;
	end_for;
	size, res;
    end_proc;

    genQuotient3 := proc(n) // 240970 msec pour n=7, maxrelat = 1
                            // 175240 msec pour n=7, maxrelat = 20
			    // 143460 msec pour n=7, maxrelat = 50
			    // 141280 msec pour n=7, maxrelat = 200
	local ntime, otime, stime, res, lgr, gsize, size, igr, a, b, gtilde, scal, sol, relat, maxrelat, grEdges, gr, i, inc;
    begin
	maxrelat := 200;
	print("Graphs List...");
	lgr   := dom::basisIndices::list(n);
	size  := nops(lgr);
	print("Generic Element");
	res := dom::plus(dom::monomial(inc.i, lgr[i]) $ i = 1..size);
//	res   := dom::_plus(inc.i*dom::term(lgr[i]) $ i = 1..size);
	print("Let's go");
	size  := nops(lgr);
	gsize := size;
	stime := time();
	ntime := stime();	       
	relat := [];
	for igr from 1 to nops(lgr) do
	    print(NoNL, "[".igr."/".gsize."]");
	    gr      := lgr[igr];
	    grEdges := GR::getEdges(gr);
	    for a from 0 to n-1 do
		for b in grEdges[a+1] do
		    gtilde := GR::tilde(gr, a, b);
		    scal := dom::coeff(res, gr) 
			  - dom::coeff(res, GR::straighten(GR::removeEdge(gr, a, b)))
			  - dom::coeff(res, GR::straighten(gtilde)) 
			  + dom::coeff(res, GR::straighten(GR::removeEdge(gtilde, a, b)));
		    if not iszero(scal) then 
			otime := ntime; ntime := time();
			print(NoNL,"-");
			relat := append(relat, scal);
			if nops(relat) = maxrelat then 
			    print(NoNL, "-Solving...");
			    sol   := linsolve(relat);
			    size  := size - nops(sol);
			    res   := mapcoeffs(res, subs, sol);
			    print(NoNL, "dim=".size."\n");
			    relat := [];
			end_if;
		    end_if;
		end_for;
	    end_for;
	end_for;
	print(NoNL, "-Solving...");
	sol   := linsolve(relat);
	size  := size - nops(sol);
	res   := mapcoeffs(res, subs, sol);
	print(NoNL, "dim=".size."\n");
	size, res;
    end_proc;

begin
end_domain:

end_if: // availability of combinat::graphs

//myTools::devMode(FALSE):
//n := 7;
//FHopf := experimental::LandoHopfAlgebra();
//time(FHopf::genQuotient2(n));
//time((bla := FHopf::genQuotient3(n)));
//bla[1];
