// Calcul du produit interne de MQSym
/////////////////////////////////////
export(combinat):

domain partitionModule(R = Dom::ExpressionField())
    // Module of standard Multiwords stored as ordered set partitions. 
    category Cat::FreeModule(R);
    inherits Dom::FreeModule(R, setPartitionsOrdered);

    exprTerm := dom::exprTermIndex;

    matrixOnSetPartitions :=
//  action of a matrix on multiwords. 
    proc(part : setPartitionsOrdered, mat : integerMatrices)
	local i, rowS, colS, comp, res, row;
    begin
	rowS := integerMatrices::rowSums(mat);
	colS := integerMatrices::columnSums(mat);
	comp := map(part, nops);
	if rowS <> comp then
	    return(dom::zero);
	end_if;
	res := [[{} $ nops(colS)]];
	for row from 1 to nops(rowS) do
	    res := map(setPartitionsOrdered::list(part[row], mat[row]),
		       (rowP) -> op(map(res, zip, rowP, _union)));
	end_for;
        _plus(op(map(res, dom::term)));
    end_proc;

    matrixAct := operators::makeLinear(dom::matrixOnSetPartitions,
				       Source=dom, ImageSet=dom);

    standard := 
    // compute the hyper-stardard multiword of multidegree comp
    proc(comp : compositions)
	local dcomp, i;
    begin
	dcomp := [0].combinat::compositions::descents(comp, FinalDescent);
	dom::term([{$dcomp[i]+1..dcomp[i+1]} $ i=1..nops(dcomp)-1]);
    end_proc;
begin
end_domain:


domain MQSym(//N : Type::NonNegInt, // not used for the moment
	     R = Dom::ExpressionField())
    category Cat::AlgebraWithBasis(R);
    inherits Dom::FreeModule(R, integerMatrices); //subclass(integerMatrices, N));

    basisName := hold(F);

    printTerm := m -> [dom::basisIndices::printCompact(m)];

    partitionModule := partitionModule(R);

    multBasis := 
// compute the product of to element of MQSym for the internal product 
// using the definition. It is sufficient to know the image of the hyper standard 
// multi words to know entirely an element of MQSym (cf F.H. Thesis p. 166).
    proc(mat1 : dom::basisIndices, mat2 : dom::basisIndices)
	local i, n, lmat, mat, genRes, genInc, vect, res;
    begin
	n    := _plus(op(map(mat1, _plus@op)));
	if integerMatrices::rowSums(mat1) <> integerMatrices::columnSums(mat2) then
	    return(dom::zero);
	end_if;
	lmat := integerMatrices::list(integerMatrices::columnSums(mat1),
				      integerMatrices::rowSums(mat2));
	genRes := v -> _plus(a[mat] * dom::partitionModule::matrixAct(v, mat) $ mat in lmat);
	genInc := [a[mat] $ mat in lmat];
	vect   := dom::partitionModule::standard(integerMatrices::rowSums(mat2));
	res    := dom::partitionModule::matrixAct(dom::partitionModule::matrixAct(vect, mat2), mat1);
	res    := [coeff(res - genRes(vect))];
	res    := linsolve(res, genInc); // identify mat1(mat2(vect)) with genRes(vect)
	res    := subs(genInc, res);
	_plus(res[i] * dom::term(lmat[i]) $ i = 1..nops(lmat)); 
    end_proc;

begin
end_domain:

// some useful functions missing in integerMatrices
listMat := proc(n) local a,b; 
begin 
    [(op(integerMatrices(a,b)) $ a in compositions(n) $ b in compositions(n))];
end_proc:

listMatRow := proc(row) local a,b; 
begin 
    [op(integerMatrices(a,row)) $ a in compositions(_plus(op(row)))];
end_proc:

listMatCol := proc(col) local a,b; 
begin 
    [op(integerMatrices(col,b)) $ b in compositions(_plus(op(col)))];
end_proc:

// to Check for the associativity. 
//n := 2; if not iszero((MQ(m1)*MQ(m2))*MQ(m3) - MQ(m1)*(MQ(m2)*MQ(m3))) then print(m1,m2,m3); error("...") end_if$ m1 in listMat(n) $ m2 in listMat(n) $ m3 in listMat(n);

// Example of usage :
// read ("InternalProduct.mu");
// MQ := MQSym(); 
// MQ([[1,1]])*MQ([[1],[1]]);
// MQ([[1,1],[1,1]])^2;

// To get the list of all non-trivial products. 
// print(hold(_mult)(MQ(m1), MQ(m2)) = MQ(m1)*MQ(m2)) $ m1 in listMatRow(comp) $ m2 in listMatCol(comp) $ comp in compositions(2)
