/*****************************************************************************
  $Id: PBT.mu,v 1.1.1.4 2005/01/26 13:42:23 nthiery Exp $
  File:		experimental/PBT.mu
  Authors:	Florent Hivert <florent.hivert@univ-mlv.fr>
                Jean-Christophe Novelli <novelli@univ-mlv.fr>
		Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Created:	14/01/2003
  Purpose:	Research tools around the Planar Binary Trees algebra
  License:	LGPL
*****************************************************************************/

// TODO: free Algebra dans examples::FreeAlgebra; base W
//       vrifier si l'on peut rcuprer des trucs dans
//	 saillances.mu cartanMatrix.mu FQSymTree3.mu TreeSym.mu (Florent)
//	 calcul du carquois et de la matrice de cartan
//
//	 co sylvester class: coclass, coCanonical, ... (JC)
//	 standardize row/column/left/right (JC)
//
//   	 use examples::SymmetricGroupAlgebra instead of SG (Florent)
//
//       Better names for UP and UQ

alias(BT=combinat::binaryTrees):
alias(LBT=combinat::labelledBinaryTrees):
alias(DT=combinat::binaryTrees::decreasingTree):
alias(BTInvolution=combinat::binaryTrees::recursiveRevertRightBranch):
alias(declareSignature  = operators::overloaded::declareSignature);
alias(PBT=experimental::PBT);
alias(words=combinat::words);
alias(permutations=combinat::permutations):

domain PBT

    //////////////////////////////////////////////////////////////////////////
    // Labelled P-symbol of a word
    // It can be reconstructed from the unlabelled P-symbol, so if the
    // argument is an unlabelled tree, this function reconstructs the
    // standard labelled P-symbol
    //////////////////////////////////////////////////////////////////////////
    pSymbol :=
    proc(word: Type::Union(BT, words)) : LBT
    begin
	if domtype(word)=BT then
	    // Reconstruct the labelled P symbol from its shape
	    LBT::fromShapeAndWord(word, [$1..BT::size(word)], Infix);
	else
	    LBT::searchTreeInsertWord(LBT::zero, revert(word));
	end_if;
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // Unlabelled P symbol of a word
    //////////////////////////////////////////////////////////////////////////
    UP:=
    proc(p: words) : BT
    begin
	LBT::toBinaryTree(dom::pSymbol(p))
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // Q symbol of a word
    //////////////////////////////////////////////////////////////////////////
    qSymbol :=
    proc(p: words) : LBT
    begin
	LBT::decreasingTree(permutations::inverse(words::standard(p)));
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // Unlabelled Q symbol of a word
    //////////////////////////////////////////////////////////////////////////
    UQ :=
    proc(p: words) : BT
    begin
	LBT::toBinaryTree(dom::qSymbol(p));
    end_proc;

/*
>> w:=[1,3,5,4,2]: [dom::UP(w), dom::UQ(w)];
                            --  o   ,  o    --
                            |  /  \   /  \   |
                            |    / \    / \  |
                            --              --
*/

    //////////////////////////////////////////////////////////////////////////
    // Canonical word (Lex Max) in a sylvester class defined by a word
    // or a labelled P-symbol
    //////////////////////////////////////////////////////////////////////////
    canonicalWord :=
    proc(tree: Type::Union(LBT, BT, words)) : words
    begin
	if domtype(tree)<>LBT then
	    tree := dom::pSymbol(tree);
	end_if;
	LBT::toWord(tree, RightPostfix);
    end_proc;

    isCanonical := 
    proc(w : words)
    begin
	not (combinat::permutations::hasPattern(w, [1,3,2]));
    end_proc;
		       
    //////////////////////////////////////////////////////////////////////////
    // Lex min word in a sylvester class defined by a word
    // or a labelled P-symbol
    //////////////////////////////////////////////////////////////////////////
    canonicalMinWord :=
    proc(tree : Type::Union(LBT, BT, words)) : words
    begin
	if domtype(tree)<>LBT then
	    tree := dom::pSymbol(tree);
	end_if;
	LBT::toWord(tree, LeftPostfix);
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // Sylvester class from its labelled P-symbol
    //////////////////////////////////////////////////////////////////////////
    class :=
    proc(tree : Type::Union(LBT, BT, permutations)) : Type::ListOf(words)
	local wrd, std, res, subst, i;
    begin
	if domtype(tree)<>LBT then
	    if domtype(tree) = DOM_LIST and nops({op(tree)}) <> nops(tree) then
		// there are some repetition so you have to standardize the word.
		wrd  := tree;
		std := combinat::words::standard(wrd);
		tree := dom::pSymbol(std);
	    else
		wrd := FAIL;
		tree := dom::pSymbol(tree);
	    end_if;
	end_if;
	res := map(combinat::linearExtensions::list(LBT::toPoset(tree)), revert);
	if wrd <> FAIL then
	    subst := [std[i] = wrd[i] $ i = 1..nops(std)];
	    res := listlib::removeDuplicates(map(res, map, subs, subst));
	end_if;
	res;
    end_proc;

    order := 
    proc(t1 : BT, t2 : BT)
    begin
	combinat::permutations::leequalPermutohedron(dom::canonicalWord(t1), dom::canonicalWord(t2))
    end_proc;

    succOrder := 
    proc(t)
	local res;
    begin
	res := {op(combinat::permutations::succPermutohedron(dom::canonicalWord(t)))} ;
	res := res minus {dom::canonicalWord(t)};
	map(res, dom::UP);
    end_proc;
    
    //////////////////////////////////////////////////////////////////////////
    // elementary q-sylvester bubble antisorting operators on SG
    //
    // If p is a permutation, then
    // PBT::qBubbleSortInSylvesterClass(i,q)(SG(p)) returns:
    // If there is a descent at position i in p: p
    // Otherwise:
    //   if p o (i,i+1) is in the same sylvester class, then: p o (i,i+1)
    //   otherwise: q*p
    //////////////////////////////////////////////////////////////////////////
    qBubbleSortInSylvesterClass :=
    proc(i: Type::PosInt, q=hold(q))
	option escape;
    begin
	operators::makeLinear(proc(p: permutations)
				  local p2;
			      begin
				  if p[i] < p[i+1] then
				      p2:=[p[j]$j=1..i-1,p[i+1],p[i],p[j]$j=i+2..nops(p)];
				      if dom::UP(p)=dom::UP(p2) then
					  return(SG::term(p2));
				      else
					  return(q*SG::term(p));
				      end_if;
				  end_if;
				  SG::term(p);
			      end_proc,
			      Source=SG, ImageSet=SG);
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // Shuffle dcall de deux permutations standard
    //////////////////////////////////////////////////////////////////////////
    shuffleDec := 
    proc(p,q)
    begin
	combinat::words::shuffle(p, map(q, _plus, nops(p)));
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // sylvester length of a maximal permutation
    // TODO: rename to lengthToCanonical
    //////////////////////////////////////////////////////////////////////////
    sylvesterLength :=
    proc(p: permutations::typeStandard)
    begin
	permutations::inversionsNumber(dom::canonicalWord(dom::pSymbol(p))) -
	permutations::inversionsNumber(p);
    end_proc;

    Agn := n -> dom::An(Dom::ExpressionField(), n, c);

    //////////////////////////////////////////////////////////////////////////
    // Induction product A_n x A_m -> A_{n+m} of JC
    //////////////////////////////////////////////////////////////////////////

    // induction of two basis elements
    inductionBasis :=
    proc(p: permutations::typeStandard, 
	 q: permutations::typeStandard)
	local lp, lq;
    begin
	lp := sylvesterLength(p);
	lq := sylvesterLength(q);
	select(shuffleDec(p,q), r -> bool(sylvesterLength(r) = lp+lq))
    end_proc;

    induction2 := operators::overloaded((x,y)->error("not implemented"));

    induction := operators::makeAssociative(dom::induction2, Zero=1);

    declareInduction :=
    proc(n, m)
	local An, Am, Anm;
	option escape;
    begin
	An:=Agn(n):
	Am:=Agn(m):
	Anm:=Agn(n+m):
	operators::overloaded::declareSignature
	(dom::induction2, [ An, Am ],
	 operators::makeBilinear((p,q) -> (_plus(Anm(r) $ r in dom::inductionBasis(p,q))),
				 Source1=An, Source2=Am, ImageSet=Anm));
    end_proc;

end_domain:

/*****************************************************************************

A domain for A_n defined by the Loday Ronco sandwiches.

Basis: permutations of [1..n]

 - Each permutation p lies in the sandwich w1 A_n w2, where w1 and w2 are
   the idempotents indexed respectively by the trees dom::pSymbol(p) and
   BTInvolution(PBT::UP(p^-1)).

Product:

   Assume that p and q are respectively in the sandwiches w1 A_n w2 and w3 A_n w4.

   If w_2\ne w_3 then pq=0.	// the w_i's are orthogonal: w_2 w_3=0
   If p=w_1 then pq=q.		// w_1 is idempotent : w_1^2=w_1
   If q=w_2 then pq=p.          // w_2 is idempotent : w_2^2=w_2

   Otherwise, pq is a linear combination of the permutations r
   in the sandwich w1 A_n w4, with coefficients sandwichCoeff(p,q,r)

Note:

Setting sandwichCoeff to the constant function 1 (the default)
basically amounts to saying that all arrows compose.

*****************************************************************************/

domain experimental::PBT::An(R = Dom::ExpressionField(), n: Type::NonNegInt, sandwichCoeff=1)

    inherits experimental::AlgebraWithBasis(R, permutations);
    category Cat::AlgebraWithBasis(R);

    info_str := "Domain for the carquois Algebra of A_n";

    one := _plus(dom(PBT::canonicalWord(dom::pSymbol(p))) $ p in BT::list(n));

    // Returns a generic element in this domain
    generic :=
    proc(a: DOM_IDENT)
    begin
	_plus(a.op(p) * dom::term(p) $ p in permutations(dom::n));
    end_proc;

    printTerm := id;

    n := n;

    left  := p -> PBT::UP(p);
    right := p -> BTInvolution(PBT::UP(permutations::inverse(p)));

    // sandwich(t1, t2)
    // Returns the list of the permutations which lie in the sandwich(t1, t2)
    sandwich := dom::makeSandwich();

    makeSandwich :=
    proc()
	local f, p, bounds;
    begin
	f :=
	proc(t1: BT, t2: BT)
	begin
	    [];
	end_proc:
	for p in permutations(n) do
	    bounds    := dom::left(p), dom::right(p);
	    f(bounds) := append(f(bounds), p);
	end_for;
	f;
    end_proc;

    mult2Basis :=
    proc(p1: permutations, p2: permutations)
    begin
	if dom::right(p1) <> dom::left(p2) then // Incompatible sandwiches
	    return(dom::zero);
	end_if;
	if dom::left(p1)=dom::right(p1) then // p1 is an idempotent
	    return(dom::term(p2));
	end_if;
	if dom::left(p2)=dom::right(p2) then // p2 is an idempotent
	    return(dom::term(p1));
	end_if;
	sandwich := dom::sandwich(dom::left(p1), dom::right(p2));
	if nops(sandwich) = 0 then
	    return(dom::zero);
	end_if;
	_plus(dom::monomial(sandwichCoeff(p1,p2,p),p) $ p in sandwich);
    end_proc;

end_domain:

/**

Example:

 try to find an internal multiplication that is compatible with JC's induction

export(experimental, PBT);

(A.n :=PBT::Agn(n) ) $n =1..6:

declareInduction(n,nm-n) $ n=1..nm-1 $nm=1..6:

{coeff(induction(A2(p), A3(q)*A3(r)) - induction(A2(p),A3(q))*induction(A2(p),A3(r))) $ p in permutations(2) $ q in permutations(3) $ r in permutations(3)}

// Darn, it seems that this does not work for A3 x A3 -> A3 !
{coeff(induction(A3(p), A3(q)*A3(r)) - induction(A3(p),A3(q))*induction(A3(p),A3(r))) $ p in permutations(3) $ q in permutations(3) $ r in permutations(3)}

**/
