/*****************************************************************************
  $Id: QuotientAlgebra.mu,v 1.1.1.1 2004/10/04 17:25:13 nthiery Exp $
  File:         lib/EXPERIMENTAL/QuotientAlgebra.mu
  Authors:      Florent Hivert <florent.hivert@univ-mlv.fr>
                Nicolas M. Thiry <nthiery@users.sf.net>
  Created:      05/06/2003
  License:      LGPL
*****************************************************************************/

/**
 * Let A be a finite dimensional combinatorial algebra
 * Let B be an ideal of A
 * Let basis be a vector space basis of B in row echelon form with
 * respect to the term order, 
 *
 * This implements the quotient of A by B
 **/
domain experimental::QuotientAlgebra(basisOfB: DOM_LIST)
    local A;
    inherits Dom::QuotientFreeModule(basisOfB);
    category Cat::FiniteDimensionalAlgebraWithBasis(A::coeffRing);

    oneBasis := if contains(dom::basisIndices::list(), A::oneBasis) <> 0 then
		    A::oneBasis;
		end_if;

    one := if dom::oneBasis = FAIL then
	       dom::fromSupModule(A::one);
	   end_if;

    algebraGenerators :=
    proc()
	option remember;
	local AGens;
    begin
	AGens := A::algebraGenerators();
	if AGens <> UNKNOWN then
	    map(AGens, dom::fromSupModule);
	else
	    UNKNOWN;
	end_if;
    end_proc;
    
    multBasis :=
    proc() : dom
	option remember; // The computations in the big algebra will typically be expensive
	local i;
    begin
	dom::fromSupModule(A::mult(dom::toSupModule(dom::term(args(i))) $ i=1..args(0)));
    end_proc;

begin
    A     := (basisOfB[1])::dom;
end_domain:
