/*****************************************************************************
  $Id: RQuasiSymmetricFunctions.mu,v 1.1.1.2 2005/01/26 13:42:23 nthiery Exp $
  File:		lib/EXPERIMENTAL/RQuasiSymmetricFunctions.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Purpose:	Source code for the domain experimental::RQuasiSymmetricFunctions
  Created:	Mon Apr 19 18:12:16 2004
  License:	LGPL
  Project:	MuPAD-Combinat, http://mupad-combinat.sf.net
*****************************************************************************/

export(combinat, Alias,
       partitions, multinomial, generators, integerVectors, words):

// TODO:
// - Find a proper name, and move to combinat
// - Implement evaluationSparse to make it more efficient
//   while accepting any values
domain combinat::submultisets
    inherits Dom::BaseDomain;
    category Cat::CombinatorialClass;
    axiom Ax::systemRep;
    
    isA :=
    proc(m: Type::AnyType)
    begin
	if not testtype(m, combinat::words) then return(FALSE) end_if;
	UNKNOWN;
    end_proc;

    evaluationSparse := 
    proc(m: dom)
	local i, pair, res;
    begin
	if nops(m)=0 then return([]); end_if;
	pair := [m[1],1];
	res  := [];
	for i from 2 to nops(m) do
	    if m[i]=pair[1] then
		pair[2] := pair[2] + 1;
	    else
		res := append(res, pair);
		pair := [m[i], 1];
	    end_if;
	end_for;
	append(res, pair);
    end_proc;

    fromEvaluationSparse :=
    proc(sparse: Type::ListProduct(DOM_LIST, Type::ListOf(Type::PosInt)))
    begin
	zip(op(sparse), _seqgen);
    end_proc;

    generator :=
    proc(m: dom, d: Type::NonNegInt)
	option escape;
	local values, counts;
    begin
	values := dom::evaluationSparse(m);
	counts := map(values, op, 2);
	values := map(values, op, 1);
	generators::map(integerVectors::generator(d, nops(counts), Outer=counts),
			subcount -> zip(values, subcount, _seqgen));
    end_proc;

    // Submultisets, where you are allowed to remove at most one occurence of
    // each value

    generatorOne :=
    proc(m: dom, d: Type::NonNegInt)
	option escape;
	local values, counts;
    begin
	values := dom::evaluationSparse(m);
	counts := map(values, op, 2);
	values := map(values, op, 1);
	generators::map(integerVectors::generator(d, nops(counts), Outer=counts, Inner=map(counts, _subtract, 1)),
			subcounts -> zip(values, subcounts, _seqgen));
    end_proc;

end_domain:

domain experimental::RQuasiSymmetricFunctions(R : Type::NonNegInt,
					      N : Type::NonNegInt,
					      K=Dom::ExpressionField(): DOM_DOMAIN)

    info_str := "experimental::RQuasiSymmetricFunctions -- The domain of R-quasi symmetric functions";

    dimension :=
    proc(lambda: Type::Union(Type::NonNegInt, partitions)) : Type::NonNegInt
	option remember;
	local i, l;
    begin
	if domtype(lambda) = DOM_INT then
	    return(_plus(dom::dimension(l) $ l in partitions::list(lambda, MaxPart=N)));
	end_if;
	if R=0 then
	    lambda := [N] . lambda;
	else
	    lambda := [ op(lambda, R..nops(lambda)) ];
	end_if;
	if lambda = [] then return(1); end_if;
	multinomial([lambda[i]-lambda[i+1] $ i=1..nops(lambda)-1, lambda[nops(lambda)]]);
    end_proc;

    SymDimension :=
    proc(lambda: Type::Union(Type::NonNegInt, partitions))
	option remember;
	local d, dim, sgn;
    begin
        // TODO: compute all at once for R<=1, by applying 1 + M + M^2/2 + M^3/6 + ...
	// where M is the incidence matrix
	// Possible to extend for R>1?
	if domtype(lambda) = DOM_INT then
	    return(generators::accumulate
		   (generators::map
		    (partitions::generator(lambda, MaxPart=N),
		     dom::SymDimension)))
	end_if;
	sgn := 1;
	dim := dom::dimension(lambda);
	for d from nops(lambda)-1 downto 0 do
	    sgn := -sgn;
	    dim := dim + sgn *
		 generators::accumulate
		 (generators::map
		  (combinat::submultisets::generatorOne(lambda, d),
		   dom::dimension));
	end_for;
	dim;
    end_proc;

    checkSymDimension :=
    proc(lambda)
    begin
	generators::accumulate
	(generators::map
	 (generators::pipe
	  (generators::fromRange(0..nops(lambda)),
	   d->combinat::submultisets::generator(lambda, d)),
	  dom::SymDimension));
    end_proc;
end_domain:
