/*****************************************************************************
  $Id: HeckeSymmetricGroupAlgebra.tst,v 1.1.1.2 2004/10/04 17:25:13 nthiery Exp $
  File:		experimental/Steenrod.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Purpose:	Test file for the library 'experimental::HeckeSymmetricGroupAlgebra'
  Created:	05/06/2003
  License:	LGPL
*****************************************************************************/

prog::testinit("HeckeSymmetricGroupAlgebra"):

alias(HSGAT=experimental::HeckeSymmetricGroupAlgebraTools):

prog::testfunc(HeckeSymmetricGroupAlgebraAlgebra):

p:=()->H([combinat::permutations::fromReducedWord([args()], H::N), [$1..H::N]]):
s:=()->H([[$1..H::N], combinat::permutations::fromReducedWord([args()], H::N)]):

n:=3:
H:=experimental::HeckeSymmetricGroupAlgebra(n):
prog::test(bool({op(map(combinat::cartesianProduct(combinat::permutations(n) $2),
			H::basisIndices::straighten))} =
		{op(H::basisIndices())})):

prog::test(H([[2, 1, 3], [1, 2, 3]])*H([[2, 1, 3], [1, 2, 3]]),
	   H::fromList([[1, [[2, 1, 3], [1, 2, 3]]]])):
prog::test(H([[1, 2, 3], [1, 2, 3]])*H([[1, 2, 3], [1, 2, 3]]),
           H::fromList([[1, [[1, 2, 3], [1, 2, 3]]]])):
prog::test(H([[1, 2, 3], [2, 1, 3]])*H([[2, 1, 3], [1, 2, 3]]),
           H::fromList([[-1, [[2, 1, 3], [1, 2, 3]]], [1, [[1, 2, 3], [2, 1, 3]]], [1, [[1, 2, 3], [1, 2, 3]]]])):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("to/fromMatrix"):
//////////////////////////////////////////////////////////////////////////////
H:=experimental::HeckeSymmetricGroupAlgebra(4):
// Check that H::toMatrix is an injection with an inverse
for t in H::basisIndices() do
    t := H::term(t);
    prog::test(H::fromMatrix(H::toMatrix(t)), t);
end:
prog::test(H::fromMatrix(H::toMatrix(p(3, 2, 1)*s(2, 3) + 3*p(1, 3)*s(2) + 5)),
           (experimental::HeckeSymmetricGroupAlgebra(4, Dom::ExpressionField()))::fromList([[1, [[4, 1, 2, 3], [1, 3, 4, 2]]], [3, [[2, 1, 4, 3], [1, 3, 2, 4]]], [5, [[1, 2, 3, 4], [1, 2, 3, 4]]]])):

// Check that H::toMatrix is a ring morphism
for n from 2 to 3 do
    H:=experimental::HeckeSymmetricGroupAlgebra(3):
    f:=H::genericElement(a):
    prog::test(iszero(map(H::toMatrix(f^2) - H::toMatrix(f)^2, normal)),
	       TRUE):
end_for:

/*
for f in map(H::basisIndices(), H::term) do
    for g in map(H::basisIndices(), H::term) do
         print(f,g,H::toMatrix(f*g)- H::toMatrix(f)*H::toMatrix(g));
    end_for
end_for
*/

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("poly2powersums"):
//////////////////////////////////////////////////////////////////////////////

// Timings Athlon 1GHz
// N               1	10  12	13   50	   100   200  400 500   600  800
// Symbolic roots  0s	.9s 8s out of mem
// Integer  roots  0s           30ms 0.04s 0.14s 0.7s 6.5 14.5s 29s  90s
// IntegerMod roots                        0.490 1.9  7.7       17s  30s
// Seems to be essentially n^4; should be n^2 but that's probably
// killed by the coefficients growth
// Well, actually, not that much!
for p in [poly(1, [x]), _mult(poly(x-r, [x]) $r in [a,b,c])] do
    prog::test(HSGAT::powersums2poly(HSGAT::poly2powersums(p)), p):
end_for:

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("changeRootsSquareDifferences"):
//////////////////////////////////////////////////////////////////////////////

S:=examples::SymmetricFunctions():
for alphabet in [[], [a], [a,b], [a,b,c]] do
    Alphabet := [(x[1]-x[2])^2 $ x in combinat::subwords(alphabet,2)];
    for d from 1 to 3 do
	prog::test(iszero(expand(HSGAT::changeRootsSquareDifferencesPk2m
				 (S, d, nops(alphabet))(alphabet) -
				 S::p([d])(Alphabet)))):
	prog::test(iszero(expand(HSGAT::changeRootsSquareDifferencesE2e
				 (S, S::e([d]), nops(alphabet))(alphabet)) -
			  expand(S::e([d])(Alphabet))))
    end_for;

    prog::test(iszero(mapcoeffs(HSGAT::changeRootsSquareDifferences
				(poly(_mult((X-x) $ x in alphabet), [X])) -
				 poly(_mult((X-x) $ x in Alphabet), [X]),
				normal))):
end_for:

//////////////////////////////////////////////////////////////////////////////
prog::testfunc(isHecke):
//////////////////////////////////////////////////////////////////////////////

export(combinat, Alias, permutations):
n:=3:
H:= experimental::HeckeSymmetricGroupAlgebra(n):
M := H::matrixDomain::genericElement(()->z[args()]):

prog::test(n!^2 - nops(linsolve(H::isHeckeEquations(M))), n!):
for p in permutations(n) do
    prog::test(H::isHecke(H::toMatrix(H::term([p,[$1..n]])))):
end_for:

prog::test(n!^2 - nops(linsolve(H::isSGEquations(M))), n!):
for p in permutations(n) do
    prog::test(H::isSG(H::toMatrix(H::fromPermutation(p))));
end_for:

prog::test(n!^2 - nops(linsolve(H::isHeckeSGEquations(M))), H::basis::count()):
for h in H::basis() do
    prog::test(H::isHeckeSG(H::toMatrix(h))):
end_for:



//////////////////////////////////////////////////////////////////////////////
// Global checking of the package

prog::testfunc(experimental::HeckeSymmetricGroupAlgebra):

prog::check(H, 3):

prog::testexit():

// Local Variables: 
// mode: mupad
// End:
