/*****************************************************************************
  $Id: Steenrod.tst,v 1.1.1.2 2004/10/04 17:25:13 nthiery Exp $
  File:		experimental/Steenrod.mu
  Authors:	Florent Hivert <florent.hivert@univ-mlv.fr>
		Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Purpose:	Test file for the library 'experimental::Steenrod'
  Created:	14/02/2003
  License:	LGPL
*****************************************************************************/

prog::testinit("Steenrod"):

prog::testfunc(SteenrodAlgebra):

alias(S = experimental::SteenrodAlgebra()):
alias(P = S::p::domainWrapper):
alias(TP = S::Tp::domainWrapper):
alias(antipode = operators::antipode):
alias(coproduct = operators::coproduct):
alias(scalar = operators::scalar):
alias(Sym = examples::SymmetricFunctions()):

prog::testfunc(SteenrodAlgebra::p):

for p in [[], [1], [2,1], [7,4,2,1]] do
    prog::test(expr(S::p(p)), hold(p)[op(p)]);
end_for:
prog::test(expr(S::p::one), hold(p)[]):
prog::test(P[1,2],
	   q*P[3] + P[2, 1]):
prog::test(P[1,3],
	   (2*q)*P[4]+P[3,1]):
prog::test(mapcoeffs(P[1,2,3], normal),
	   q*P[3, 3] + (2*q)*P[4, 2] + (4*q^2)*P[6] + q*P[5, 1] + P[3, 2, 1]
	  ):
p1:=P[1]:
p2:=P[2]:
prog::test(p1*p2-p2*p1, q*P[3]):
prog::test(P[2,1]^2, // correct ?
	   P[2, 2, 1, 1] + q^2*P[5, 1] + q*P[3, 2, 1]):
prog::test(P[1,3] + 2*P[1,2],
           2*q*P[3] + 2*P[2, 1] + 2*q*P[4] + P[3, 1]):
prog::test(P[3]*P[1], P[3, 1]):


//////////////////////////////////////////////////////////////////////////////
prog::testfunc(antipode):
//////////////////////////////////////////////////////////////////////////////
prog::test(antipode(P[]),
	   P[]):
prog::test(antipode(P[1]),
	   -1*P[1]):
prog::test(antipode(P[3]),
	   -1*P[3]):
prog::test(antipode(P[1,1]),
	   P[1, 1]):
prog::test(antipode(P[2,1]),
	   q*P[3] + P[2, 1]):

for p in combinat::partitions::list(5) do
    prog::test(simplify(antipode(antipode(S::p(p)))),
	       S::p(p)):
end_for:

//////////////////////////////////////////////////////////////////////////////
prog::testfunc(coproduct):
//////////////////////////////////////////////////////////////////////////////
prog::test(operators::coproduct(P[]), TP([[], []])):  // One
prog::test(coproduct(P[4]), TP[[4],[]] + TP[[],[4]]): // Primitive element
prog::test(operators::coproduct(P[1, 1]),
           TP([[1, 1], []]) + TP([[], [1, 1]]) + 2*TP([[1], [1]])):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc(scalar):
//////////////////////////////////////////////////////////////////////////////
prog::test(scalar(S::p::dual([]),    S::p([])),  1):
prog::test(scalar(S::p::dual([]),    S::p([1])), 0):
prog::test(scalar(S::p::dual([4,3]), S::p([1])), 0):
// If we specialize to 1 the parameters
//  f(i):=scalar(S::p::dual([i]),S::p([i])),
// we should get the scalar product of Sym.
for n from 1 to 5 do
    for p in combinat::partitions::list(n) do
	prog::test(eval(subs(scalar(S::p::dual(p), S::p(p)), f=id)),
		   scalar(Sym::p(p), Sym::p(p)));
    end_for;
end_for:

//////////////////////////////////////////////////////////////////////////////
// Global checking of the domain

prog::testfunc(experimental::SteenrodAlgebra):

prog::check(S, 3):

//////////////////////////////////////////////////////////////////////////////
export(experimental::Steenrod):
//////////////////////////////////////////////////////////////////////////////

prog::testfunc(OREtx):
A:=OREtx([x1,x2,x3]):
prog::test(A(tx1)*A(x1^4) - A(x1^4)*A(tx1), A(4*x1^4)):
(hold(p).k:=qPk(A,k)) $k=1..3:
prog::test(normal(p1*p2-p2*p1), normal(q*p3)):
(hold(p).k:=qPPk(A,k)) $k=1..3:
prog::test(normal(p1*p2-p2*p1), normal(q*p3)):
prog::test(degree(p1),1):
prog::test(degree(p2),2):
prog::test(degree(p3),3):
prog::test(degree(A(tx1)),0):

prog::testfunc(ORExs):
A:=ORExs([x1,x2,x3]):
prog::test(expr(A(x1)^3*A(sx1)),
           sx1*x1^3):
prog::test(expr(A(sx1)*A(x1)^3),
           3*q*x1^3 + sx1*x1^3):
prog::test(expr(A(sx1)*A(x1)),
           q*x1 + sx1*x1):
(hold(p).k:=qPk(A,k)) $k=1..3:
prog::test(normal(p1*p2-p2*p1), normal(q*p3)):
(hold(p).k:=qPPk(A,k)) $k=1..3:
prog::test(normal(p1*p2-p2*p1), normal(q*p3)):
prog::test(degree(p1),1):
prog::test(degree(p2),2):
prog::test(degree(p3),3):
prog::test(degree(A(sx1)),0):

Otx:=OREtx([x]):
Oxs:=ORExs([x]):

prog::test(expr(Otx(Oxs(x*sx))),
           q*x*tx + x*(- q + 1)):
prog::test(expr(Otx(Oxs(sx))),
	   q*tx + 1):
prog::test(expr(Otx(Oxs(x))),
           x):

W:=ORExt([x,y]):
generic:=experimental::Steenrod::genericWeylElementWithSymmetries:
prog::test(iszero(generic(W, 1, 2, [2], f)-
		  (
           x*tx*ty*f([[1, 0], [1, 1]]) + y*tx*ty*f([[1, 0], [1, 1]]) +
	   x*tx^2*f([[1, 0], [2, 0]])  + x*ty^2*f([[1, 0], [0, 2]])  +
	   y*tx^2*f([[1, 0], [0, 2]])  + y*ty^2*f([[1, 0], [2, 0]])))):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc(PolynomialsWithSkewThetaCoefficients):
//////////////////////////////////////////////////////////////////////////////

vars := [X,Y,Z,T]:
Q:=experimental::Steenrod::PolynomialsWithSkewThetaCoefficients(vars, Dom::ExpressionField()):
[x,y,z,t] := [Q([0$i-1,1,0$nops(vars)-i]) $i=1..nops(vars)]:

prog::test(tX*x, x*(tX+1)):
prog::test((tX*x)^2, x^2*(tX+1)*(tX+2)):

p:=Q([4,3,2,1]):

prog::test(Q::symmetricGroupAction([1, 2, 3, 4], p), p):
prog::test(Q::symmetricGroupAction([1, 2, 3, 4], p*tX), p*tX):

prog::test(Q::symmetricGroupAction([2, 3, 4, 1], p),
           (experimental::Steenrod::PolynomialsWithSkewThetaCoefficients([X, Y, Z, T], Dom::ExpressionField()))::fromList([[1, [1, 4, 3, 2]]])):
prog::test(Q::symmetricGroupAction([2, 3, 4, 1], p*tX),
           (experimental::Steenrod::PolynomialsWithSkewThetaCoefficients([X, Y, Z, T], Dom::ExpressionField()))::fromList([[tY, [1, 4, 3, 2]]])):
prog::test(Q::symmetricGroupAction([2, 3, 4, 1], p*tX + p*tY),
           (experimental::Steenrod::PolynomialsWithSkewThetaCoefficients([X, Y, Z, T], Dom::ExpressionField()))::fromList([[tY + tZ, [1, 4, 3, 2]]])):

/*
vars := [x,y,z]:
P:=experimental::Steenrod::Polynomials(vars, Dom::ExpressionField()):
debugOf():
operators::overloaded::declareConversion(DOM_EXPR, P, P::fromExpr, Explicit):
operators::overloaded::declareConversion(DOM_IDENT, P, P::fromExpr, Explicit):
S:=examples::SymmetricFunctions():
setuserinfo(Any,1):
P::groebner([P(x), P(x*y)]);
P::groebner([P((x+1)), P((x-1))]);
P::groebner([P((x+1)*(x+2)^3), P((x-1)*(x-2)^3)]);
P::groebner([P((x+1)*(x+2)), P((x+1)*(x-2))]);
P::groebner(revert([ P(S::e([i])(vars)) $i=1..nops(vars)]));
P::groebner([ P(S::e([i])(vars)) $i=1..nops(vars)]);

                                          [[1, 1, x]]
                                                                                  Time: 260 msec
++*
                                        [[2, 1*-1/2, 1]]
                                                                                   Time: 40 msec
++*
                                           3
                                     [[2, x *53/13824, 1]]
                                                                                   Time: 30 msec
++*
                                      [[2, 1*-1/4, x + 1]]
                                                                                   Time: 10 msec
++++*+!
                                                2          2        2   3
                    [[3, 1, x + y + z], [3, y, y  + y z + z ], [3, z , z ]]
                                                                                  Time: 390 msec
+++
                                                  2          2           3
                   [[1, 1, x + y + z], [2, 1*-1, y  + y z + z ], [3, 1, z ]]
                                                                                   Time: 30 msec
>> 
*/

//////////////////////////////////////////////////////////////////////////////
// Global checking of the package

prog::testfunc(experimental::Steenrod):

prog::check(experimental::Steenrod, 3):

prog::testexit():

// Local Variables: 
// mode: mupad
// End:
