/*****************************************************************************
  $Id: YangBaxterGraph.tst,v 1.1.1.2 2004/10/16 15:16:52 nthiery Exp $
  File:		lib/EXPERIMENTAL/TEST/YangBaxter.tst
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Purpose:	Test suite for the domain experimental::YangBaxter
  Created:	Tue Jul  6 10:38:35 2004
  License:	LGPL
  Project:	MuPAD-Combinat, http://mupad-combinat.sf.net
*****************************************************************************/

prog::testinit("YangBaxterGraph"):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc(experimental::YangBaxterGraph):


export(combinat, Alias, permutations, words):
K := Dom::ExpressionField(normal):
operators::overloaded::declareConversion(Dom::ExpressionField(), K, K::new):

makeTranspositionOperator :=
proc(i: Type::PosInt) : DOM_PROC
    option escape;
begin
    proc(x, w=0 : Type::AnyType)
    begin
	x::dom::mapsupport(x, words::swap, i) + x*w;
    end_proc;
end_proc:

N := 3:
SG:=examples::SymmetricGroupAlgebra(N, K):

// All permutations, as elements of SG
YB := experimental::YangBaxterGraph(SG::one,
				    [makeTranspositionOperator(i) $i=1..N-1]):
prog::test({op(YB::basis())}, {op(SG::basis())}):

YB := experimental::YangBaxterGraph(SG::one,
				    [makeTranspositionOperator(i) $i=1..N-1],
				    SpectralParameters = [hold(x).i $ i=1..N]):
prog::test(bool(map(YB::basisIndices(), YB::termIterative) = YB::basis())):


N:=5:
P:=examples::MultivariatePolynomials([x.i $i=1..N]):
top      := [0,0,0,3,3]:
values   := [0,1,2,0,1]:
spectral := map([0,1,2,-1,0], _negate):
//P::printTerm:=x->output::asciiArt([_concat(op(map(x, expr2text)))]):

YB := experimental::YangBaxterGraph(P::term(top),
				    [makeTranspositionOperator(i) $i=1..N-1],
				    Values=values):

prog::test(YB::basisIndices(),
           [[], [3], [3, 2], [3, 4], [3, 2, 4]]):
prog::test(map(YB::basis(), expr),
           [x4^3*x5^3, x3^3*x5^3, x2^3*x5^3, x3^3*x4^3, x2^3*x4^3]):
prog::test(map(YB::basis(), x -> x::dom::lsupport(x)),
           [[0, 0, 0, 3, 3], [0, 0, 3, 0, 3], [0, 3, 0, 0, 3], [0, 0, 3, 3, 0], [0, 3, 0, 3, 0]]):

YB := experimental::YangBaxterGraph(P::term(top),
				    [makeTranspositionOperator(i) $i=1..N-1],
				    Values=values,
				    SpectralParameters = spectral):

prog::test(YB::basis(),
           [P::fromList([[1, [0, 0, 0, 3, 3]]]),
	    P::fromList([[1, [0, 0, 3, 0, 3]], [1/3, [0, 0, 0, 3, 3]]]),
	    P::fromList([[1, [0, 3, 0, 0, 3]], [1/2, [0, 0, 3, 0, 3]], [1/2, [0, 0, 0, 3, 3]]]),
	    P::fromList([[1, [0, 0, 3, 3, 0]], [1/2, [0, 0, 3, 0, 3]], [1/2, [0, 0, 0, 3, 3]]]),
	    P::fromList([[1, [0, 3, 0, 3, 0]], [1/2, [0, 3, 0, 0, 3]], [1/2, [0, 0, 3, 3, 0]], [1/4, [0, 0, 3, 0, 3]], [3/4, [0, 0, 0, 3, 3]]])]):

scalarProduct :=
operators::makeBilinear(
 proc(m1: P::basisIndices, m2: P::basisIndices) : P::coeffRing
 begin
     p := map(zip(m1,m2,_plus), _plus, 1);
     if permutations::isStandard(p) then
	 P::coeffRing(permutations::sign(p));
     else
	 P::coeffRing::zero
     end_if;
 end_proc, Source=P, ImageSet=P::coeffRing):

prog::test(scalarProduct(P([0, 0, 0, 0, 0]), P([4, 3, 2, 1, 0])),  1):
prog::test(scalarProduct(P([0, 0, 0, 0, 0]), P([3, 4, 2, 1, 0])), -1):
prog::test(scalarProduct(P([0, 0, 0, 0, 0]), P([3, 4, 0, 1, 0])),  0):
prog::test(scalarProduct(P([3, 2, 1, 0, 0]), P([1, 1, 1, 1, 0])),  1):
prog::test(scalarProduct(P([3, 2, 1, 0, 0]), P([1, 1, 1, 0, 1])), -1):

// Taken from Lascoux notes p. 44
topDual      := [0,0,1,1,2]:
valuesDual   := [0,1,0,1,0]:
spectralDual := map([0,1,-1,0,-2], _negate):

YBdual := experimental::YangBaxterGraph(P::term(topDual),
				    [makeTranspositionOperator(i) $i=1..N-1],
				    Values=valuesDual,
				    SpectralParameters = spectralDual):

prog::test(map(YBdual::basis(), expr),
           [x3*x4*x5^2,
	    x2*x4*x5^2 + 1/2*x3*x4*x5^2,
	    x3*x4^2*x5 + 1/2*x3*x4*x5^2,
	    1/2*x2*x4*x5^2 + x2*x4^2*x5 + 1/4*x3*x4*x5^2 + 1/2*x3*x4^2*x5,
	    1/2*x3^2*x4*x5 + x2*x3^2*x5 + 1/6*x3*x4^2*x5 + 1/3*x3*x4*x5^2 + 1/2*x2*x3*x5^2 + 1/3*x2*x4^2*x5 + 1/6*x2*x4*x5^2]):

// The two basis are adjoint to each other (up to a sign)
prog::test([(scalarProduct(p, q) $ p in YB::basis()) $ q in YBdual::basis()],
           [0, 0, 0, 0, -1,
	    0, 0, 0, 1, 0,
	    0, 0, 1, 0, 0,
	    0,-1, 0, 0, 0,
	    1, 0, 0, 0, 0]):

//////////////////////////////////////////////////////////////////////////////
prog::testexit():
prog::check(test, 3):


// Local Variables:
// mode: mupad
// End:
