/*****************************************************************************
  $Id: SGAExp.mu,v 1.1.1.1 2003/09/15 16:37:47 nthiery Exp $
  File:         lib/DOMAINS/DOMAIN/AlgebraWithBasis_Table.mu
  Authors:      Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Created:      16/02/2001
  License:      LGPL
*****************************************************************************/

export(combinat);
domain SGA
    category Cat::AlgebraWithBasis(Dom::Rational);
    axiom Ax::canonicalRep, Ax::normalRep;

    inherits experimental::AlgebraWithBasis(Dom::Rational, 
					    permutations);

//    exprTerm := perm->hold(SGA)[op(perm)];
    exprTerm := perm->_concat(op(map(perm, expr2text)));

    indexed := indexedProcedure(dom::term@DOM_LIST);

    one := dom::term([]);

    mult2Basis := dom::term @ permutations::_mult;

    toVect := 
    proc(el, lp)
	local n;
    begin
	[coeff(el, p) $ p in lp];
    end_proc;

begin
end_domain:

    TemperleyLiebPerm321 := 
    proc(p : permutations)
	option remember;
	local i, j, k, n, res;
    begin
	n := nops(p);
	for i from 1 to n-2 do
	    for j from i+1 to n-1 do
		if p[i] > p[j] then 
		    for k from j+1 to n do
			if p[j] > p[k] then
// 			    res := [ SGA::term(subsop(p, i=p[j], j=p[i]     )),
// 				     SGA::term(subsop(p,         j=p[k], k=p[j])),
// 				     SGA::term(subsop(p, i=p[k],      k=p[i])),
// 				    -SGA::term(subsop(p, i=p[j], j=p[k], k=p[i])),
// 				    -SGA::term(subsop(p, i=p[k], j=p[i], k=p[j])) ];
// 			    return(_plus(op(map(res, TemperleyLieb321))));
			    res := TemperleyLieb321(
				     SGA::term(subsop(p, i=p[j], j=p[i]     )),
				    +SGA::term(subsop(p,         j=p[k], k=p[j])),
				    +SGA::term(subsop(p, i=p[k],      k=p[i])),
				    -SGA::term(subsop(p, i=p[j], j=p[k], k=p[i])),
				    -SGA::term(subsop(p, i=p[k], j=p[i], k=p[j])));

			end_if;
		    end_for;
		end_if;
	    end_for;
	end_for;
	SGA::term(p);
    end_proc;
	    
    TemperleyLieb321 := 
    operators::makeLinear(TemperleyLiebPerm321,
			  Source=SGA, ImageSet=SGA);

    TemperleyLiebPerm132 := 
    proc(p : permutations)
	option remember;
	local i, j, k, n, res;
    begin
	n := nops(p);
	for k from n downto 3 do
	    for j from k-1 downto 2 do
		if p[j] > p[k] then 
		    for i from j-1 downto 1 do
			if p[i] < p[k] then
			    res := [ SGA::term(subsop(p, i=p[j], j=p[i]     )),
				     SGA::term(subsop(p,         j=p[k], k=p[j])),
				     SGA::term(subsop(p, i=p[k],      k=p[i])),
				    -SGA::term(subsop(p, i=p[j], j=p[k], k=p[i])),
				    -SGA::term(subsop(p, i=p[k], j=p[i], k=p[j])) ];
			    return(_plus(op(map(res, TemperleyLieb132))));
			end_if;
		    end_for;
		end_if;
	    end_for;
	end_for;
	SGA::term(p);
    end_proc;
	    
    TemperleyLieb132 := 
    operators::makeLinear(TemperleyLiebPerm132,
			  Source=SGA, ImageSet=SGA);

    TemperleyLiebInductPerm := 
    proc(l1 : permutations, l2 : permutations)
	local n1, x;
    begin
	n1 := nops(l1);
	TemperleyLieb132(
        _plus(SGA::term(permutations::inverse(x))
	      $ x in words::shuffle(permutations::inverse(l1), 
				    map(permutations::inverse(l2),
					z->z+n1))));
    end_proc;

    TemperleyLiebInduct := 
    operators::makeAssociative(
	operators::makeBilinear(TemperleyLiebInductPerm, 
	    Source=SGA, ImageSet=SGA));

    
read("../experimental/shortcuts.mu"):   
    
// map(permutations(3), TemperleyLiebPerm321);
    
// bla := TemperleyLiebPerm321([3,2,4,1]);
    
// SGA::toVect(bla, permutations::list(4));
    
canonical :=
proc(tree : BT)  
begin
    LBT::toWord(LBT::fromShapeAndWord(tree, [$1..BT::size(tree)], Infix),
		RightPostfix);
end_proc:
    
sylvesterClass :=
proc(tree : BT) : Type::ListOf(words)
begin
    tree := LBT::fromShapeAndWord(tree, [$1..BT::size(tree)], Infix);
    map(combinat::linearExtensions::list(LBT::toPoset(tree)), revert);
end_proc:

  
matPermToCan := 
proc(n)
begin
    lCan  := map(revert(binaryTrees::list(n)), canonical):    
//    print(toto);    
    lElem := map(permutations(n), TemperleyLiebPerm132):
//    print(toto);    
    lVect := map(lElem, SGA::toVect, lCan):
//   print(lVect);    
    matr  := sparsematrix(n!, catalan(n), lVect):
end_proc:
    
n := 3;    
matSylvToTL := 
proc(n)
begin
    lt    := revert(binaryTrees::list(n));
    
    lCan  := map(lt, canonical):
//    lCan := select(permutations(n), _not @ permutations::hasPattern, [3,2,1]);
    print(lCan);    
    lElem := map(lt, sylvesterClass);
    lElem := map(lElem, map, TemperleyLiebPerm132);
//    lElem := map(lElem, map, TemperleyLiebPerm321);
    print(lElem);
    lElem := map(lElem, SGA::_plus @ op);
    print(lElem);    
    lVect := map(lElem, SGA::toVect, lCan):
    print(lVect);    
    matr  := sparsematrix(catalan(n), catalan(n), lVect):
    print("ok...");
end_proc:
    
//matSylvToTL(4);
//linalg::rank(matr);
//b := matrix(catalan(4), 1, 0):
//linalg::matlinsolve(linalg::transpose(matr), b);
// La rponse est    
//   [-1, 1, -1, 1, 1, 1, -1, 1, -1, -1, -1, 1, -1, 1]
// Ce qui veux dire que la somme de ces arbres moins la somme des autres 
// (leur symmtrique, pourquoi ?) fait 0 !!! But Why ???   
//    --    o,    o,    o,   o ,  o  , o   , o    --
//    |    /   /     /      / \  /  \    \    \    |
//    |  /       \    \    /       /    / \    \   |
//    |   \     /      \                        \  |
//    --                                          --
// Les sommes des classes sylvestres sont lies !!! DOMMAGE !!!
    
CheckLibre := 
proc(n)
//    local lc, lProd, lProdCan, lELem;
begin
    lCan     := map(revert(binaryTrees::list(n)), canonical):    
    print(lCan);
    lc       := compositions(n, MinLength = 2);
    lProd    := map(lc, op @ cartesianProduct::list @ op @ map, binaryTrees::list);
    print(lProd);
    lProdCan := map(lProd, map, SGA::term @ canonical);
    print(lProdCan);
    lElem    := map(lProdCan, TemperleyLiebInduct @ op);
    print(lElem);
    lVect    := map(lElem, SGA::toVect, lCan):
    print(lVect);
    matr     := sparsematrix(nops(lVect), catalan(n), lVect):
end_proc:
    
