/*****************************************************************************
  $Id: TensorAlgebra.mu,v 1.1.1.4 2005/01/26 13:42:23 nthiery Exp $
  File:		lib/EXPERIMENTAL/TensorAlgebra.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Created:	01/05/2003
  License:	LGPL
*****************************************************************************/

alias(TA  = experimental::TensorAlgebra):
alias(TAT = experimental::TensorAlgebraTools):
alias(declareConversion = operators::overloaded::declareConversion):
alias(declareSignature  = operators::overloaded::declareSignature):

//////////////////////////////////////////////////////////////////////////////
domain TAT
    info_str := "Helper tools for the domain experimental::SteenrodAlgebra";
end_domain:

//////////////////////////////////////////////////////////////////////////////
// The combinatorial class of the lists of elements in a combinatorial class
domain TAT::listsOf(class: DOM_DOMAIN)
    inherits Dom::BaseDomain;
    category Cat::CombinatorialClass;
    axiom    Ax::systemRep;

    domtype := DOM_LIST;
//    type := Type::ListOf(class);
    isA := (z) -> testtype(z, Type::ListOf(class));
end_domain:

domain TA
    (A: Type::Predicate(A->_lazy_and(domtype(A)=DOM_DOMAIN,
				     A::hasProp(Cat::ModuleWithBasis))))
    inherits Dom::FreeModule(A::coeffRing, TAT::listsOf(A::basisIndices));
    category Cat::AlgebraWithBasis(A::coeffRing);

    tensorConstructor := operators::makeBilinear
		       (prog::bless(proc(t1: A::basisIndices, t2: A::basisIndices)
				    begin
					dom::term([t1,t2])
				    end_proc,
				    dom), 
			Source=A, ImageSet=dom);
    tensor2 := operators::makeBilinear
	     (prog::bless(proc(t1: dom::basisIndices, t2: dom::basisIndices)
			  begin
			      dom::term(t1.t2)
			  end_proc,
			  dom), 
	      Source=dom, ImageSet=dom);
    initDomainAdd :=
    proc()
	option escape;
        local DOM;
    begin
	DOM := dom;
	declareSignature // Tensor product from A x A to dom
	(operators::tensor2, [A, A], dom::tensorConstructor);
	declareSignature // Tensor product from dom x dom to dom
	(operators::tensor2, [dom, dom], dom::tensor2);
	declareConversion(A, dom, 
			  operators::makeLinear(z -> DOM::term([z]), Source=A, ImageSet=dom));
    end_proc;

    // Basis elements are printed as W([a,b,a,a])
    basisName     := hold(T).A::basisName;
    // I prefer [..., ...] 
    printTerm := z -> map(z, A::printTerm);

    // The unit of the algebra: A::one tensor A::one tensor ...
    // Note that it does not have a unique representation!
    // If the unit of A is a monomial A(bla), then we can easily define
    // the product of two terms that do not have the same length,
    // and simply set A::one to the empty tensor product; otherwise,
    // only products of terms of same length are allowed so far.
    one :=
    if A::oneBasis<>FAIL or nterms(A::one) = 1 then
	dom::term([]):
    end_if;

    // The product in the algebra
    mult2Basis    :=
    if nterms(A::one) = 1 then
	prog::bless(
	proc()
	    local oneBasis;
	    option escape;
	begin
	    oneBasis := A::lsupport(A::one);
	    proc(t1: dom::basisIndices, t2: dom::basisIndices)
	    begin
		dom::tensorConstructor(op(zip(t1, t2, A::mult2Basis, oneBasis)));
	    end_proc;
	end_proc,
		    dom)();
    else
	proc(t1: dom::basisIndices, t2: dom::basisIndices)
	begin
	    dom::tensorConstructor(op(zip(t1, t2, A::mult2Basis)));
	end_proc;
    end_if;

end_domain:
