/*****************************************************************************
  $Id: Virasoro.mu,v 1.1.1.2 2005/01/26 13:42:23 nthiery Exp $
  File:		experimental/Virasoro.mu
  Authors:	francois descouens <francois.descouens@wanadoo.fr>
  Created:	28/11/2003
  Purpose:	Research tools around the Virasoro Algebra
  License:	LGPL
*****************************************************************************/


alias(Vira=experimental::VirasoroAlgebra):
alias(ViraT=experimental::VirasoroAlgebraTools):

//////////////////////////////////////////////////////////////////////////////
domain ViraT
    info_str := "Helper tools for the domain VirasoroAlgebra";
end_domain:

//////////////////////////////////////////////////////////////////////////////
domain ViraT::powersum(R: DOM_DOMAIN, C)
    inherits Dom::FreeModule(R, combinat::partitions);
    category Cat::HopfAlgebraWithBasis(R);

    info_str := "Domain for the q-Virasoro algebra on the powersum basis";

    interfaceAdd := {hold(enumerateBasis)};

    oneBasis := [];
    degreeBasis := _plus@op;

    basisName := hold(LL);
    exprTerm  := dom::exprTermIndex;

    straightenBasis :=
    proc(l: combinat::compositions) 
	local i;
    begin
	for i from 1 to nops(l)-1 do
	    if l[i] <= l[i+1] then 
		next;
	    end_if;
	    if l[i] = -l[i+1] then 
              return( dom::straightenBasis([op(l,1..i-1),l[i+1],l[i],op(l,i+2..nops(l))]) + 
                  dom::multcoeffs(dom::straightenBasis([op(l,1..i-1), l[i]+l[i+1],op(l,i+2..nops(l))]),(2*l[i]))
                + dom::multcoeffs(dom::straightenBasis([op(l,1..i-1), op(l,i+2..nops(l))]), C*(l[i]^3-l[i])/12)
                 );
	   else
               return( dom::straightenBasis([op(l,1..i-1),l[i+1],l[i],op(l,i+2..nops(l))]) + 
                  dom::multcoeffs(dom::straightenBasis([op(l,1..i-1), l[i]+l[i+1],op(l,i+2..nops(l))]),(l[i]-l[i+1]))
                 );
           end_if;          


	end_for;
	dom::term(l);
    end_proc;

    mult2Basis :=
    proc(l1: dom::basis, l2: dom::basis)
    begin
	dom::straightenBasis(l1.l2);
    end_proc;
	
end_domain:

//////////////////////////////////////////////////////////////////////////////
domain Vira(R=Dom::ExpressionField(): DOM_DOMAIN, C=hold(C))
    inherits Dom::BaseDomain;
    category Cat::HopfAlgebraWithBasis(R);

    info_str := "Library for the rational Virasoro algebra";

    interfaceAdd := {hold(LL), hold(powersum)};

    powersum       := ViraT::powersum(R, C);
    LL             := dom::powersum;
        
    bases := {dom::LL};
    testtype :=
    proc(e, T)
    begin
	if T = dom and contains(dom::bases, domtype(e)) then TRUE else FALSE end_if;
    end_proc;

    initDomain :=
    proc()
	option remember;
    begin
	declareConversion
	(dom::LL,
	 operators::makeLinear
	 (prog::bless
	  (proc(part: dom::LL::basis)
	   begin
	       if part=[] then
		   dom::LL::one;
	       else
		   FAIL;
	       end_if;
	   end_proc, dom),
	  ImageSet=dom::LL)):
     end_proc;

end_domain:

//Some computations into Verma modules which are a family of representations 
//indexed by two complex numbers c an h.

//Declaration of the Virasoro Algebra (in the package experimental)
//V:=experimental::VirasoroAlgebra();

//function for computing product operator into verma module representation
//the argument is a list of integers


domain experimental::Virasoro
    
    interface:={hold(enumerateBasis), hold(onSymetric)};
    
    // function for enumerating some basis operator between the integer n and p
    enumerateBasis:=
    proc(n,p,Vir)
       local k;
    begin
         [ Vir::LL([k]) $ k = n..p ];
    end_proc;

   //Virasoro operators action on schur functions	
    onSymetric :=
    proc(n, part)
	local k,res,tmp,S;
    begin
	   S:=examples::SymmetricFunctions();
	if n < 0 then
	   n:=-n;   
	   k:=1;
	   res:=0;
	   for k from 1 to nops(part)+n do
	       tmp:=combinat::ribbons::addRibbonsPartition(part, n, k);
	       //print("rubans de longueur",n ,"en position" , k , "donne",  tmp);
		if k <= nops(part) then
	           if tmp <> FAIL then res:=res + (part[k]-k+1)*S::s(tmp): end_if;
	       else
	       if tmp <> FAIL then res:=res + (-k+1)*S::s(tmp): end_if;
	   end_if;	 
	  end_for;    
	 return(res);
	end_if;

	if n > 0 then   
	   k:=1;
	   res:=0;
	   for k from 1 to nops(part) do
	       tmp:=combinat::ribbons::removeRibbonsPartition(part, n, k);
	       	if tmp <> FAIL then res:=res + (part[k]-k+1)*S::s(tmp): end_if;
	    end_for;    
	 return(res);
	end_if;

	if n=0 then return(S::s(part)); end_if;
	
    end_proc;
end_domain:

domain experimental::VermaModule
    
    interface :={hold(vermaProduct),
		 hold(scalarProduct),
		 hold(vermaGram),
		 hold(kacDet), hold(kacFormula)
		};
    
// function for computing linear combinaisons of Virasoro operator product
    vermaProduct:=proc(L)
		      local h,k,res;
		  begin
		      res:=0;
		      for k from 1 to nops(L) do
			  if(nops(L[k][2]) <>0 ) then
			      if(L[k][2][nops(L[k][2])]) = 0 then
				  res:=res+L[k][1]*h^(nops(L[k][2])):
			      end_if;
			  else
			      res:=res+L[k][1]:
			  end_if;
		      end_for;
		      return(res);
		  end_proc;

//function for computing  scalar product onto verma module
//the arguments are two list of integer and the virasoro algebra
// Two different partitions gives us 0 because Schur Function scalar product is used    

	scalarProduct:=proc(L1,L2,Vir)
		       local k,res;
		   begin
		       res:=Vir::LL([]);
		       for k from 0 to nops(L2)-1 do
			   res:=res*Vir::LL([L2[nops(L2)-k]]);
		       end_for;
		       for k from 1 to nops(L1) do
			   res:=res*Vir::LL([-L1[k]]);
		       end_for;
		     factor(dom::vermaProduct(poly2list(res)));
		   end_proc;
    
//function for computing the Gram determinant for the verma module scalar product
//the arguments are an integer (order of the determinant) and the virasoro algebra
    
	vermaGram:=proc(n,Vir)
		   local k,i,mat,temp,S;
	       begin
		   S:=combinat::partitions::list(n);
		   temp:=combinat::partitions::count(n);
		   mat:=matrix(temp,temp);
		   for k from 1 to temp do
		       for i from 1 to temp do
			   mat[k,i]:=dom::scalarProduct(S[k],S[i],Vir);
		       end_for;
		   end_for;
		   return(factor(linalg::det(mat)));
	       end_proc;

//Kac determinant formula computed with determinanal methods

    kacDet:=proc(n,Vir,m)
		local P,C;
	    begin
		P:=dom::vermaGram(n,Vir);
		P:=subs(P,C=1-6/( (m+2)*(m+3)));
		factor(P);
	    end_proc;
    
//Kac determinant formula implemented with Kac formula
    
    kacFormula:=proc(n,m)
		    local h,k,i,j,res,temp;
		begin
		    res:=1;
		    for k from 1 to n do
			temp:=1;
			for i from 1 to k do
			    for j from 1 to k do
				if i*j=k then
				    temp:=temp*(h - (((m+3)*i-(m+2)*j)^2 - 1)/( 4*(m+2)*(m+3) ) );
				end_if;
			    end_for;
			end_for;
			res:=res*temp^(combinat::partitions::count(n-k));
		    end_for;
		end_proc;
end_domain:

