/*****************************************************************************
  $Id: YangBaxterGraph.mu,v 1.1.1.2 2005/01/26 13:42:23 nthiery Exp $
  File:		lib/EXPERIMENTAL/YangBaxter.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Purpose:	Source code for the domain experimental::YangBaxter
  Created:	Tue Jul  6 10:38:35 2004
  License:	LGPL
  Project:	MuPAD-Combinat, http://mupad-combinat.sf.net
*****************************************************************************/

export(combinat, Alias, generators, permutations, compositions, integerVectors):

alias(YBG  = experimental::YangBaxterGraph):
alias(YBGT = experimental::YangBaxterGraphTools):

domain YBGT
    info_str := "Helper tools for the domain experimental::YangBaxterGraph";
end_domain:

domain YBGT::reducedWordsLexMin(N: Type::Union(Type::NonNegInt, integerVectors))
    inherits Dom::BaseDomain;
    category Cat::CombinatorialClass;
    axiom Ax::systemRep;


    domtype := DOM_DOMAIN;

    isA :=
    proc(w: Type::AnyType)
	local N2;
    begin
	if testtype(w, Type::ListOf(Type::PosInt)) and
	   w = dom::straighten(w) then
	    for i in w do
		if domtype(N) = DOM_INT then
		    if i > N then
			return(FALSE)
		    end_if;
		else
		    N2 := N;
		    if i > nops(N2)-1 or N2[i] <= N2[i+1] then
			return(FALSE);
			tmp := N2[i]; N2[i] := N2[i+1]; N2[i+1] := tmp;
		    end_if;
		end_if;
	    end_for;
	else
	    FALSE;
	end_if;
    end_proc;

    maximalPermutation :=
    if domtype(N)=DOM_INT then
	revert([$1..N]);
    else
	permutations::inverse(permutations::standard(N))
    end_if;
    
    count :=
    if domtype(N)=DOM_INT then
	N!;
    end_if;

    list :=
    if domtype(N)=DOM_LIST then
	proc()
	begin
	    sort(map(permutations::smallerPermutohedron(dom::maximalPermutation),
		     permutations::reducedWordLexMin));
	end_proc;
    end_if;
    
    generator :=
    if domtype(N)=DOM_INT then
	proc()
	begin
	    generators::map(permutations::generator(N), permutations::reducedWordLexMin);
	end_proc;
    end_if;
    
    straighten :=
    proc(w: compositions) // actually list of integers in 1..N-1
    begin
	permutations::reducedWordLexMin(permutations::fromReducedWord(w));
    end_proc;
end_domain:

// Morally, the default value for spectral vectors is so that 1/(xi-xj) is always zero

domain YBG(Top: Type::AnyType, Operators : DOM_LIST) //Compose = Mult)
    inherits Dom::BaseDomain;
    // category Cat::ModuleWithBasis(Top::dom::coeffRing);
    // could be a Cat::ModuleWithBasis if Top lives in some
    // module which is stabilized by the operators

    interfaceAdd := {
		     hold(basis), hold(basisIndices),
		     hold(top),
		     hold(term), hold(straightenBasis)
		    };

    info_str := "experimental::YangBaxterGraph -- domain for YangBaxter graphs";

    testtype := p->testtype(p,Type::AnyType);

    typeLetter := Type::PosInt;
    typeWord := Type::ListOf(dom::typeLetter);
    
    top := Top;
    spectralParameters :=
    if contains(Options, SpectralParameters) then Options[SpectralParameters]; end_if;
    
    basisIndices := YBGT::reducedWordsLexMin(if contains(Options, Values) then
						 Options[Values]
					     else
						 nops(Operators)+1
					     end_if);

    straightenBasis := dom::term @ dom::basisIndices::straighten;

    // The permutation associated to a node of the graph
    // That's equivalent to permutations::fromReducedWord(w),
    // but we want to remember all of them.
    permutation :=
    proc(w: dom::typeWord)
	option remember;
	local i;
    begin
	if nops(w) = 0 then
	    [ $ 1..nops(Operators)+1 ];
	else
	    i := w[nops(w)];
	    w := [op(w,1..nops(w)-1)];
	    permutations::fromReducedWord([i], dom::permutation(w));
	end_if;
    end_proc;

    term :=
    proc(w: dom::typeWord) : dom
	option remember;
	local X, p, x, i;
    begin
	if w = [] then
	    dom::top;
	else
	    i := w[nops(w)];
	    x := dom::term([op(w,1..nops(w)-1)]);
	    if (X := dom::spectralParameters) <> FAIL then
		p := dom::permutation(w);
		x := Operators[i](x, 1/(X[p[i]]-X[p[i+1]]));
	    else
		x := Operators[i](x);
	    end_if;
	end_if;
    end_proc;

    termIterative :=
    proc(p: dom::typeWord) : dom
	local x,w;
    begin
	x := dom::top;
	X := dom::spectralParameters;
	for i in p do
	    if X <> FAIL then
		tmp := X[i]; X[i] := X[i+1]; X[i+1] := tmp;
		x := Operators[i](x, 1/(X[i]-X[i+1]));
	    else
		x := Operators[i](x);
	    end_if;
	end_for;
	x;
    end_proc;

    basis := combinat::imageClass(dom::basisIndices, dom::term);

begin
    Options := combinat::getOptions([SpectralParameters, Values],
				    [SpectralParameters = DOM_LIST, Values=DOM_LIST], 2, Error);
    
    if testargs() then
	if contains(Options, SpectralParameters)
	   and nops(Operators) +1 <> nops(Options[SpectralParameters]) then
	    error("Incompatible number of operators and spectral vectors:".
		  expr2text(Operators).
		  expr2text(Options[SpectralParameters]));
	end_if;
	if contains(Options, Values)
	   and nops(Operators) +1 <> nops(Options[Values]) then
	    error("Incompatible number of operators and values:".
		  expr2text(Operators).
		  expr2text(Options[Values]));
	end_if;
    end_if;
end_domain:
