/*******************************************************************
  $Id: functors.mu,v 1.1.1.2 2005/01/26 13:42:23 nthiery Exp $
  File:         lib/EXPERIMENTAL/functor.mu
  Authors:      Florent Hivert <florent.hivert@univ-mlv.fr>
  Purpose:      Source code for the domain functors
  Created:      Tue Apr 13 12:06:02 2004
  License:      LGPL
  Project:      MuPAD-Combinat, http://mupad-combinat.sf.net
*******************************************************************/


//******************************************************
// Inverse of a functor by solving the triangular system
inverseFunctor := 
proc(funct, G, t, n)
    save a;
    local res, i, fRes, sol, sys;
begin
    delete(a);
    res  := 1;
    for i from 1 to n do
	res  := res + a[i]*t^i;
	fRes := funct(res, t, i);
        sys  := coeff(fRes, t, i) = coeff(G, t, i);
        sol  := solve(sys, a[i]);
        res  := subs(res, a[i]=op(sol));
        fRes := subs(fRes, a[i]=op(sol));
    end_for;
    res;
end_proc;


//***************************************************
// Functor T: Tensor algebra of a graded vector space
TFunctor :=
proc (H, t, n)
begin
    series(1/(1-H), t, n+1);
end:
TFunctorInv :=
proc(G, t, n)
begin
    series(1-1/G, t, n+1);
end:


//******************************************************
// Functor S: Symmetric algebra of a graded vector space
SFunctor := 
proc(H, t, n)
    local res, i;
begin
    res := 1;
    for i from 1 to n do
        res := res / series((1-t^i), t, n+1)^(coeff(H, t, i)) ;
    end_for;
end_proc:
SFunctorInv := 
proc(G, t, n)
begin
    inverseFunctor(SFunctor, G, t, n)
end_proc;


//*****************************************************
// Functor S: Free Lie algebra of a graded vector space
LFunctor    := (G, t, n) -> SFunctorInv(TFunctor(G, t, n), t, n):
LFunctorInv := (G, t, n) -> TFunctorInv(SFunctor(G, t, n), t, n):
