/*****************************************************************************
  $Id: IPC.mu,v 1.1.1.3 2004/10/04 17:25:13 nthiery Exp $
  File:		lib/LIBFILES/IPC.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Purpose:	Library for interprocess communications
  Created:	07/08/2002
  License:	LGPL
*****************************************************************************/

if module::which("IPC") = FAIL then
    // If the module is simply not available, fail silently
    userinfo(1, "Dynamic module IPC not available"):
    IPC := FAIL;
else
    // Avoids an internal error on exit of MuPAD 2.0.0
    %if version() = [2,0,0] then module::displace("stdmod"); end_if:
    if traperror(module("IPC")) <> 0 then
	warning("Error loading the dynamic module IPC:");
	lasterror();
    end_if;

IPC::close :=
proc(f)
begin
    f::dom::close(args());
end_proc:

IPC::getline :=
proc(f)
begin
    f::dom::getline(args());
end_proc:

IPC::putline :=
proc(f)
begin
    f::dom::putline(args());
end_proc:

IPC::interface :=
   {
    hold(doc), 
    hold(slave), hold(istream), hold(ostream),
    hold(popen), hold(popen2), hold(popen3),
    hold(getline), hold(putline),
    hold(close)
   }:

IPC::popen :=
proc(command: DOM_STRING, type: DOM_STRING)
begin
    if type = "r" then
	IPC::slave(command, FALSE, TRUE, FALSE);
    elif type = "w" then
	IPC::slave(command, TRUE, FALSE, FALSE);
    else
	error("Argument should be \"r\" or \"w\"");
    end_if;
end_proc:

IPC::popen2 :=
proc(command: DOM_STRING)
begin
    IPC::slave(command, TRUE, TRUE, FALSE);
end_proc:

IPC::popen3 :=
proc(command: DOM_STRING)
begin
    IPC::slave(command, TRUE, TRUE, TRUE);
end_proc:

domain IPC::istream
    getline := IPC::getline_istream;
    close := IPC::close_istream;
begin
end_domain:

domain IPC::ostream
    putline := IPC::putline_ostream;
    close := IPC::close_ostream;
begin
end_domain:

domain IPC::slave

    /**
	Domain for slave processes possibly connected by pipes

        The internal representation consists of 4 elements:
	the pid of the process: a DOM_INT
	the standard input  of the process: an IPC::istream or FAIL
	the standard output of the process: an IPC::ostream or FAIL
	the standard error  of the process: an IPC::ostream or FAIL
							
    **/

    new := IPC::pipe_open;
    
    pid :=
    proc(s: dom) : DOM_INT
    begin
	extop(s, 1);
    end_proc;

    stdin :=
    proc(s: dom) : IPC::ostream
    begin
	extop(s, 2);
    end_proc;

    stdout :=
    proc(s: dom) : IPC::istream
    begin
	extop(s, 3);
    end_proc;

    stderr :=
    proc(s: dom) : IPC::istream
    begin
	extop(s, 4);
    end_proc;
    
    getline :=
    proc(s: dom) : Type::Union(DOM_STRING, DOM_NULL)
	local stdout;
    begin
	if (stdout := dom::stdout(s)) = FAIL then
	    error("Not connected to the standard output of this slave");
	end_if;
	IPC::getline(stdout);
    end_proc;
    
    putline :=
    proc(s: dom, line: DOM_STRING)
	local stdin;
    begin
	if (stdin := dom::stdin(s)) = FAIL then
	    error("Not connected to the standard output of this slave");
	end_if;
	IPC::putline(stdin, line);
    end_proc;

    close :=
    proc(s: dom)
	local f;
    begin
	for f in [extop(s, 2..4)] do
	    if f <> FAIL then
		IPC::close(f);
	    end_if;
	end_for;
	IPC::kill(dom::pid(s), 1);
	IPC::waitpid(dom::pid(s));
    end_proc;
begin
end_domain:

end_if:
