/*****************************************************************************
  $Id: combinat.mu,v 1.1.1.7 2005/01/26 13:42:23 nthiery Exp $
  File:		lib/LIBFILES/combinat.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
                Florent Hivert <florent.hivert@univ-mlv.fr>
		Authors of the combinat package of MuPAD 2.0.0
  Taken from:	MuPAD 2.0.0, 16/02/2001
  License:	LGPL
*****************************************************************************/

/**
 * Test if the package Combinat is loaded, or is this the combinat file
 * from the official MuPAD library.
 **/
%if traperror(packages::Combinat) = 0 then
    alias(CombinatIsLoaded = TRUE)
else
    alias(CombinatIsLoaded = FALSE)
end:


//traperror(
    // The traperror avoids an error if the combinat package had
    // already been defined (say from MuPAD's library).
combinat:=newDomain("combinat"):
//          ):

combinat::info_str := "Library 'combinat': combinatorial classes and functions
 This library is a part of the open-source algebraic
 combinatorics contribution package MuPAD-Combinat
 http://mupad-combinat.sourceforge.net.":

combinat::interface :=
   {
    hold(bell), hold(catalan), hold(binomialRecursive), hold(multinomial),
    hold(stirling1), hold(stirling2), hold(modStirling),
    hold(generators),
    (if CombinatIsLoaded then hold(rankers) else null() end_if) ,
    hold(words), hold(subwords), hold(dyckWords), hold(permutations),
    (if CombinatIsLoaded then hold(splitNK), hold(chooseNK), hold(multisetsNK) else null() end_if),
    hold(integerListsLexTools), 
    hold(LattE),
    hold(countingFunctions),
    hold(singleton),
    hold(parkingFunctions),
    hold(independentSetsOfVectorSpace),
    (if CombinatIsLoaded then // interfaceAddIfExists
	 hold(MachineIntegerListsLex), hold(lrcalc), hold(GLIP), hold(Nauty), hold(graphs)
     else null() end_if),
    hold(partitions), hold(compositions), hold(integerVectors),
    hold(integerMatrices),
    hold(integerVectorsWeighted),
    (if CombinatIsLoaded then hold(integerVectorsModPermutationGroup) else null() end_if),
    hold(skewPartitions),
    (if CombinatIsLoaded then hold(setPartitions) else null() end_if),
    (if CombinatIsLoaded then hold(setPartitionsOrdered) else null() end_if),
    hold(tableaux), hold(skewTableaux), hold(ribbons),
    hold(subsets),
    hold(nonCrossingPartitions),
    hold(cartesianProduct),
    hold(decomposableObjects),
    hold(trees),
    hold(binaryTrees),
    (if CombinatIsLoaded then hold(labelledTrees) else null() end_if),
    hold(labelledBinaryTrees),
    hold(linearExtensions),
    (if CombinatIsLoaded then hold(imageClass) else null() end_if),
    (if CombinatIsLoaded then hold(finiteClass) else null() end_if),
    (if CombinatIsLoaded then hold(subClass) else null() end_if),
    hold(warnDeprecated),
    hold(treesGeneric),
    hold(freeTreesLabelled),
    hold(necklaces),
    hold(lyndonWords),
    hold(ribbonsTableaux)
   }:

alias(loaddomain=prog::loaddomain):
alias(path = pathname("COMBINAT")):
combinat::independentSetsOfVectorSpace :=
    loaddomain(combinat::independentSetsOfVectorSpace, path, "independentSetsOfVectorSpace"):
combinat::parkingFunctions :=
    loaddomain(combinat::parkingFunctions, path, "parkingFunctions"):
combinat::lrcalc :=
    loaddomain(combinat::lrcalc, path, "lrcalc"):
combinat::singleton :=
    loaddomain(combinat::singleton, path, "singleton"):
combinat::countingFunctions :=
    loaddomain(combinat::countingFunctions, path, "countingFunctions"):
combinat::LattE :=
    loaddomain(combinat::LattE, path, "LattE"):
combinat::graphs :=
    loaddomain(combinat::graphs, path, "graphs"):

/*****************************************************************************
   Utilities that should move into the MuPAD library
*****************************************************************************/

// This is prog::getOptions from MuPAD 3.0.0. It's provided under the
// name combinat::getOptions to avoid breaking former versions of
// MuPAD

combinat::getOptions :=
//%if version()[1]>=3 then
//    prog::getOptions
//else
    loadproc(combinat::getOptions, path, "getOptions"):
//end_if:

// Where to put this ?
combinat::typeDirection := 
    Type::Union(Type::Singleton(Left),Type::Singleton(Right)):

/*****************************************************************************
   Classical integer sequences
*****************************************************************************/

autoload(combinat::bell):
autoload(combinat::stirling1):
autoload(combinat::stirling2):
autoload(combinat::modStirling):
autoload(combinat::catalan):
autoload(combinat::binomialRecursive):

combinat::multinomial :=
    proc(c: Type::ListOf(Type::NonNegInt))
	local i;
    begin
	_plus(op(c))! / _mult(i! $ i in c);
    end_proc:

/*****************************************************************************
   Sub packages for classical combinatorial objects
*****************************************************************************/

combinat::words :=
    loaddomain(combinat::words, path, "words"):
combinat::subwords :=
    loaddomain(combinat::subwords, path, "subwords"):
combinat::permutations :=
    loaddomain(combinat::permutations, path, "permutations"):
combinat::splitNK :=
    loaddomain(combinat::splitNK, path, "splitNK"):
combinat::chooseNK :=
    loaddomain(combinat::chooseNK, path, "chooseNK"):
combinat::multisetsNK :=
    loaddomain(combinat::multisetsNK, path, "multisetsNK"):

combinat::necklaces :=
    loaddomain(combinat::necklaces, path, "necklaces"):
combinat::lyndonWords :=
    loaddomain(combinat::lyndonWords, path, "lyndonWords"):

combinat::dyckWords :=
    loaddomain(combinat::dyckWords, path, "dyckWords"):
combinat::nonCrossingPartitions :=
    loaddomain(combinat::nonCrossingPartitions, path, "nonCrossingPartitions"):

combinat::integerListsLexTools :=
    loaddomain(combinat::integerListsLexTools, path, "integerListsLexTools"):
combinat::partitions :=
    loaddomain(combinat::partitions, path, "partitions"):
combinat::compositions :=
    loaddomain(combinat::compositions, path, "compositions"):
combinat::integerMatrices :=
    loaddomain(combinat::integerMatrices, path, "integerMatrices"):
combinat::integerVectors :=
    loaddomain(combinat::integerVectors, path, "integerVectors"):
combinat::integerVectorsWeighted :=
    loaddomain(combinat::integerVectorsWeighted, path, "integerVectorsWeighted"):
combinat::integerVectorsModPermutationGroup :=
    loaddomain(combinat::integerVectorsModPermutationGroup, path, "integerVectorsModPermutationGroup"):
combinat::skewPartitions :=
    loaddomain(combinat::skewPartitions, path, "skewPartitions"):
combinat::setPartitions :=
    loaddomain(combinat::setPartitions, path, "setPartitions"):
combinat::setPartitionsOrdered :=
    loaddomain(combinat::setPartitionsOrdered, path, "setPartitionsOrdered"):

combinat::tableaux :=
    loaddomain(combinat::tableaux, path, "tableaux"):
combinat::skewTableaux :=
    loaddomain(combinat::skewTableaux, path, "skewTableaux"):
combinat::ribbons :=
    loaddomain(combinat::ribbons, path, "ribbons"):
combinat::ribbonsTableaux :=
    loaddomain(combinat::ribbonsTableaux, path, "ribbonsTableaux"):

combinat::subsets :=
    loaddomain(combinat::subsets, path, "subsets"):
combinat::cartesianProduct :=
    loaddomain(combinat::cartesianProduct, path, "cartesianProduct"):

combinat::decomposableObjects :=
    loaddomain(combinat::decomposableObjects, path, "decomposableObjects"):
combinat::recursiveObjects := () -> combinat::decomposableObjects(args()):

combinat::trees :=
    loaddomain(combinat::trees, path, "trees"):
combinat::labelledTrees :=
    loaddomain(combinat::labelledTrees, path, "labelledTrees"):
combinat::binaryTrees :=
    loaddomain(combinat::binaryTrees, path, "binaryTrees"):
combinat::labelledBinaryTrees :=
    loaddomain(combinat::labelledBinaryTrees, path, "labelledBinaryTrees"):
combinat::treesGeneric :=
    loaddomain(combinat::treesGeneric, path, "treesGeneric"):
combinat::freeTreesLabelled :=
    loaddomain(combinat::freeTreesLabelled, path, "freeTreesLabelled"):

combinat::linearExtensions :=
    loaddomain(combinat::linearExtensions, path, "linearExtensions"):

/*****************************************************************************
   Sub package for generators, rankers and tools
*****************************************************************************/

combinat::generators :=
    loaddomain(combinat::generators, path, "generators"):
combinat::rankers :=
    loaddomain(combinat::rankers, path, "rankers"):
combinat::transitiveIdeal :=
    loadproc(combinat::transitiveIdeal, path, "tools"):
//autoload(combinat::transitiveIdeal):
combinat::subClass :=
    loaddomain(combinat::subClass, path, "subClass"):
combinat::finiteClass :=
    loaddomain(combinat::finiteClass, path, "finiteClass"):
combinat::imageClass :=
    loaddomain(combinat::imageClass, path, "imageClass"):

/*****************************************************************************
   Dynamic Modules
*****************************************************************************/
combinat::MachineIntegerListsLex := loaddomain(combinat::MachineIntegerListsLex, 
					     path, "MachineIntegerListsLex"):
combinat::GLIP := loaddomain(combinat::GLIP, path, "GLIP"):
combinat::Nauty := loaddomain(combinat::Nauty, path, "Nauty"):

/*****************************************************************************
   Aliases for backward compatibility
*****************************************************************************/

combinat::warnDeprecated :=
    proc()
        local oldvalue;
    begin
        if args(0) = 0 then
            return(Pref::warnChanges()):
        else
            if testargs() and not testtype(args(1), DOM_BOOL) then
                error("expecting TRUE or FALSE");
            end_if;
            oldvalue := combinat::warnDeprecated();
            sysassign(combinat::warnDeprecated(), args(1));
        end_if;
        return(oldvalue);
    end_proc:

/*combinat::integerListsLexBuilder :=
    hold((if combinat::warnDeprecated() then
	      warning("combinat::integerListsLexBuilder is obsolete.
Please use 'combinat::integerListsLexTools' instead.");
	  end_if;
	  combinat::integerListsLexTools)):

combinat::integerListsLexBuilding :=
    hold((if combinat::warnDeprecated() then
	      warning("combinat::integerListsLexBuilding is obsolete.
Please use 'combinat::integerListsLexStructure' instead.");
	  end_if;
	  combinat::integerListsLexTools;)):
*/

combinat::permute :=
    proc(list: Type::Union(Type::NonNegInt, DOM_LIST))
    begin
	warning("combinat::permute(l) is obsolete.
Please use 'combinat::permutations::list(l)' instead.");
        combinat::permutations::list(args());
    end_proc:

combinat::choose := 
    proc(set: Type::Union(Type::NonNegInt, DOM_SET), k: Type::NonNegInt)
    begin
	warning("combinat::choose(s,k) is obsolete.
Please use 'combinat::subsets::list(s,k)' instead.");
        op(combinat::subsets::list(args()))
    end_proc:

combinat::powerset	:= 
    proc(set: Type::Union(Type::NonNegInt, DOM_SET, DOM_LIST))
        /*
	   If set is a list, it's interpreted as a multiset
           The algorithm is suboptimal in this case!!!
        */
    begin
        if testargs() then
            if args(0) <> 1 then
                error("Wrong number of arguments") ;
            end_if ;
        end_if;
	warning("combinat::powerset is obsolete.
Please use 'combinat::subsets' and 'combinat::subwords' instead.");
        if domtype(set)=DOM_LIST then
            return({op(combinat::subwords::list(sort(set)))})
        end_if;
        return({op(combinat::subsets::list(set))});
    end_proc:

combinat::composition	:=
    proc(n: Type::NonNegInt, k: Type::NonNegInt)
    begin
	warning("combinat::composition(n,k) is obsolete.
Please use 'combinat::integerVector::list(n,k, MinPart=1)' instead.");
        revert(combinat::integerVectors::list(n,k,hold(MinPart)=1));
    end_proc:

combinat::cartesian := 
     proc()
     begin
        warning("combinat::cartesian(s1,s2,...) is obsolete.
Please use 'combinat::cartesianProduct::list(s1,s2,...)' instead.");
         {op(combinat::cartesianProduct::list(args()))}
     end_proc:

// See also combinat::partitions::new!

/*****************************************************************************
   Stuff not yet reorganized!!!
*****************************************************************************/

combinat::numberOfSetPartitionColorings :=
loadproc(combinat::numberOfSetPartitionColorings, pathname("COMBINAT"), "nbspcols"):

null():
