/*****************************************************************************
  $Id: experimental.mu,v 1.1.1.8 2005/01/26 13:42:23 nthiery Exp $
  File:		lib/LIBFILES/combinat.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Created:	25/11/2002
  License:	LGPL
*****************************************************************************/

experimental:=newDomain("experimental"):

experimental::info_str := "Library 'experimental': experimental code for MuPAD-Combinat":

experimental::interface :=
   {
    hold(PBT),
    hold(AlgebraWithBasis),
    hold(CliffordAlgebra), hold(HeckeCliffordAlgebra),
    hold(HeckeSymmetricGroupAlgebra),
    hold(HeckeSymmetricGroupAlgebraTools),
    hold(SteenrodAlgebra), hold(Steenrod),
    hold(VirasoroAlgebra), hold(Virasoro), hold(VermaModule),
    hold(nonCommutativeMult),
    hold(makeMatrixFromPermutationAction),
    hold(time), hold(computationHeader),
    hold(FreeOperad),
    hold(LandoHopfAlgebra),
    hold(QSymSym),
    hold(RQuasiSymmetricFunctions),
    hold(algTools),
    hold(SubAlgebra),
    hold(QuotientAlgebra),
    hold(SubLeftAModule),
    hold(QuotientLeftAModule),
    hold(Schur),
    hold(YangBaxterGraph),
    hold(ArikiKoikeAlgebra),
    hold(ArikiKoikeLagrangeAlgebra),
    hold(ribbonsTableaux)
   }:

alias(loaddomain=prog::loaddomain):
alias(path = pathname("EXPERIMENTAL")):

experimental::ribbonsTableaux :=
    loaddomain(experimental::ribbonsTableaux, path, "ribbonsTableaux"):

experimental::YangBaxterGraph :=
    loaddomain(experimental::YangBaxterGraph, path, "YangBaxterGraph"):
experimental::Schur :=
    loaddomain(experimental::Schur, path, "Schur"):
experimental::RQuasiSymmetricFunctions :=
    loaddomain(experimental::RQuasiSymmetricFunctions, path, "RQuasiSymmetricFunctions"):
experimental::QSymSym :=
    loaddomain(experimental::QSymSym, path, "QSymSym"):
experimental::LandoHopfAlgebra :=
    loaddomain(experimental::LandoHopfAlgebra, path, "LandoHopfAlgebra"):
experimental::FreeOperad :=
    loaddomain(experimental::FreeOperad, path, "FreeOperad"):

experimental::algTools :=
    loaddomain(experimental::algTools, path, "AlgTools"):
experimental::SubAlgebra :=
    loaddomain(experimental::SubAlgebra, path, "SubAlgebra"):
experimental::QuotientAlgebra :=
    loaddomain(experimental::QuotientAlgebra, path, "QuotientAlgebra"):
experimental::SubLeftAModule :=
    loaddomain(experimental::SubLeftAModule, path, "SubLeftAModule"):
experimental::QuotientLeftAModule :=
    loaddomain(experimental::QuotientLeftAModule, path, "QuotientLeftAModule"):

experimental::TensorAlgebra :=
    loaddomain(experimental::TensorAlgebra, path, "TensorAlgebra"):
experimental::AlgebraWithBasis :=
    loaddomain(experimental::AlgebraWithBasis, path, "AlgebraWithBasis"):

experimental::PBT :=
    loaddomain(experimental::PBT, path, "PBT"):
experimental::Steenrod :=
    loaddomain(experimental::Steenrod, path, "Steenrod"):
experimental::SteenrodAlgebra :=
    loaddomain(experimental::SteenrodAlgebra, path, "Steenrod"):
experimental::HeckeSymmetricGroupAlgebra :=
    loaddomain(experimental::HeckeSymmetricGroupAlgebra, path, "HeckeSymmetricGroupAlgebra"):
experimental::HeckeSymmetricGroupAlgebraTools :=
    loaddomain(experimental::HeckeSymmetricGroupAlgebraTools, path, "HeckeSymmetricGroupAlgebra"):
experimental::Virasoro :=
    loaddomain(experimental::Virasoro, path, "Virasoro"):
experimental::VermaModule :=
    loaddomain(experimental::VermaModule, path, "VermaModule"):
experimental::VirasoroAlgebra :=
    loaddomain(experimental::VirasoroAlgebra, path, "Virasoro"):

unalias(path):

alias(path = pathname("EXPERIMENTAL", "HeckeClifford")):
experimental::CliffordAlgebra :=
    loaddomain(experimental::CliffordAlgebra, path, "HeckeClifford"):
experimental::HeckeCliffordAlgebra :=
    loaddomain(experimental::HeckeCliffordAlgebra, path, "HeckeClifford"):
experimental::HeckeAlgebra :=
    loaddomain(experimental::HeckeAlgebra, path, "Hecke0"):
unalias(path):

alias(path = pathname("EXPERIMENTAL", "ArikiKoike")):
experimental::ArikiKoikeAlgebra :=
    loaddomain(experimental::ArikiKoikeAlgebra, path, "ArikiKoike"):
experimental::FiniteQuotientOfPolynomials :=
    loaddomain(experimental::FiniteQuotientOfPolynomials, path, "FiniteQuotientOfPolynomials"):
experimental::ArikiKoikeLagrangeAlgebra :=
    loaddomain(experimental::ArikiKoikeLagrangeAlgebra, path, "ArikiKoikeLagrange"):
unalias(path):

//////////////////////////////////////////////////////////////////////////////
// Miscellaneous utilities
//////////////////////////////////////////////////////////////////////////////

// Returns a function that computes the representation matrix of an
// element of the algebra thedom, with respect to an action of the basis
// of thedom on a finite set.
// set is given as a list
// actionBasis: (x: dom::basisIndices, a: set) -> set is a procedure that computes
// the action of the basis element x on an element a of the set
experimental::makeMatrixFromPermutationAction :=
    proc(thedom, set, actionBasis) : DOM_PROC
	local rank, unrank, i;
	option escape;
    begin
	[rank, unrank] := combinat::rankers::fromList(set);
	operators::makeLinear
	(prog::bless
	 (
	  proc(x: thedom::basisIndices) : Dom::Matrix(thedom::coeffRing)
	  begin
	      Dom::Matrix(thedom::coeffRing)
	      (nops(set), nops(set),
	       [(rank(actionBasis(x, unrank(i))), i) = 1 $ i=1..nops(set)]):
	  end_proc,
	  thedom),
	 Source=thedom, ImageSet=Dom::Matrix(thedom::coeffRing));
    end_proc:

/* 
   gaussElimPID -- row-echelon form for PID's

Summary:
 - gaussElimPID -> Cat::Matrix::gaussElimPID
   implementation by column elimination + transposition if not option
   ColumnElimination

   This procedure can be used for matrices whose components
   are elements of an effective principal ideal domain R
   (that is with a method gcdex that computes extended gcds)

   It performs a Hermite-like fraction free elimination which
   preserves the R-module spanned by the rows.


mat := Dom::Matrix(Dom::Integer):
gaussElimPID := experimental::gaussElimPID:

prog::test(gaussElimPID(mat([[0, 2], [0, 1]])),
           mat([[0, 1], [0, 0]])):
prog::test(gaussElimPID(mat([[3], [2]])),
           mat([[1], [0]])):
prog::test(gaussElimPID(mat([[0, 6], [0, 4]])),
           mat([[0, 2], [0, 0]])):
prog::test(gaussElimPID(mat([[0, 0], [0, 2], [0, 1]])),
           mat([[0, 1], [0, 0], [0, 0]])):
prog::test(gaussElimPID(mat([[1,0,0,0],[1,3,3,9],[1,4,4,16],[1,5,5,25],[1,2,2,4]])),
           mat([[1, 0, 0, 0], [0, 1, 1, 7], [0, 0, 0, 2], [0, 0, 0, 0], [0, 0, 0, 0]])):

>> R:= Dom::DistributedPolynomial([x], Dom::Rational):
>> A:= Dom::Matrix(R)([[2*x+2,1,1],[4*x+4,1,1],[6*x+6,1,1]]);

                            +-               -+
                            |  2 x + 2, 1, 1  |
                            |                 |
                            |  4 x + 4, 1, 1  |
                            |                 |
                            |  6 x + 6, 1, 1  |
                            +-               -+

>> gaussElimPID(A);



                          +-                   -+
                          |  6 x + 6,  1,   1   |
                          |                     |
                          |     0,    1/3, 1/3  |
                          |                     |
                          |     0,     0,   0   |
                          +-                   -+

>> C:= Dom::Matrix(R)([[0,0,0],[4*x^3+4*x^2,0,1],[x+1,0,1]]);

                          +-                   -+
                          |       0,      0, 0  |
                          |                     |
                          |     3      2        |
                          |  4 x  + 4 x , 0, 1  |
                          |                     |
                          |     x + 1,    0, 1  |
                          +-                   -+

>> gaussElimPID(C);

                        +-                      -+
                        |  x + 1, 0,      1      |
                        |                        |
                        |    0,   0,      0      |
                        |                        |
                        |                 2      |
                        |    0,   0, - 4 x  + 1  |
                        +-                      -+

*/

experimental::gaussElimPID:= proc(A)
    local i, j, m, n, i0, R, Mat, g, s, t, c, rank;
begin
    if args(0) <> 1 then 
	error("expecting one argument")
    end_if;
    Mat:= A::dom;
    R:=Mat::coeffRing;
    if testargs() then
	if A::dom::hasProp( Cat::Matrix ) <> TRUE then
	    error("expecting matrix of 'Cat::Matrix'")
	end_if;
	if Mat::constructor <> Dom::Matrix then 
	    error("expecting a matrix of type 'Dom::Matrix(...)'")
	end_if;
	if not R::hasProp( Cat::EuclideanDomain ) then
	    error("expecting matrix over 'Cat::EuclideanDomain'")
	end_if;
    end_if;

    [m,n] := Mat::matdim(A);
    rank := 0;
    for j from 1 to n do // columns
	i0 := rank+1;
	c := A[i0,j];
	for i from i0+1 to m do // rows
	    if not iszero(A[i,j]) then
		if iszero(c) then
		    A := A::dom::swapRow(A, i0, i);
		    c := A[i0,j];
		else
		    // g := s * A[i0,j] + t * A[i,j]
		    [g, s, t] := [R::gcdex(c, A[i,j])];
		    if g <> c then
			// Reduce the main row, so that A[i0,j]=g
			c := g;
			if iszero(s) then
			    A := A::dom::swapRow(A,i,i0);
			else
			    A := A::dom::addRow(A, i, i0, t, s);
			end_if;
		    end_if;
		    A := A::dom::addRow(A, i0, i, -A[i,j]/c);
		end_if;
	    end_if;
	end_for;
	if not iszero(c) then
	    rank := rank+1;
	end_if;
    end_for;
    return(A);
end_proc:


/**
 * Table utilities
 **/

domain experimental::tables
    interface := { hold(keys), hold(values) };
    
    keys :=
    proc(t: DOM_TABLE)
    begin
	map([op(t)], op, 1);
    end_proc;

    values :=
    proc(t: DOM_TABLE)
    begin
	map([op(t)], op, 2);
    end_proc;
end_domain:
/**
 * Time utilities
 **/
domain experimental::time
    interface := { hold(format) };

    /**
     * Converts a time in milliseconds (as given by e.g. time()) into
     * a human readable string in weeks/days/hours/...
     **/
    format:=
    proc(ms: Type::NonNegInt) : DOM_STRING
	local result, s, m, h, d, w;
    begin
	result:="";
	if ms>=1000 then
	    s:=trunc(ms/1000);
	    ms:=ms mod 1000;
	    if s>=60 then
		m:=trunc(s/60);
		s:=s mod 60;
		if m>=60 then
		    h:=trunc(m/60);
		    m:=m mod 60;
		    if h>=24 then
			d:=trunc(h/24);
			h:=h mod 24;
			if d>=7 then
			    w:=trunc(d/7);
			    d:=d mod 7;
			    result:=result.w." week".
				   (if w=1 then " " else "s "; end_if);
			end_if;
			result:=result.d." day".
			       (if d=1 then " " else "s "; end_if);
		    end_if;
		    result:=result.h." hour".
			   (if h=1 then " " else "s "; end_if);
		end_if;
		result:=result.m." min "
	    end_if;
	    result:=result.s." s "
	end_if;
	result.ms. " ms";
    end_proc;
end_domain:

/**
 * Prints a standardized computation header
 **/
experimental::computationHeader :=
proc()
    local s;
begin
    userinfo(3, _concat("=" $ 78));
    for s in [args()] do
        userinfo(1, s);
    end_for;
    userinfo(3, "Version: $Id: experimental.mu,v 1.1.1.8 2005/01/26 13:42:23 nthiery Exp $");
    userinfo(hold(NoNL), 3, (system("echo Date: `date`");
                             system("echo Host: `hostname`");
                             ""));
    userinfo(3,_concat("=" $ 78));
end_proc:

// Utilities for pretty printing non commutative expressions
// experimental::nonCommutativeMult(a,b,1,2,c) is pretty printed
// as _mult(a,b,1,2,c), except that the terms will be left alone
// by the kernel simplifier (no reordering, ...)
experimental::nonCommutativeMult := proc(x) begin hold(slot)(experimental,"nonCommutativeMult")(args()) end_proc:
experimental::nonCommutativeMult := funcenv(experimental::nonCommutativeMult, extop(_mult,2)):

experimental::FreeQuasiSymmetricFunctions :=
    ()->error("experimental::FreeQuasiSymmetricFunctions has been obsoleted.
Please use 'examples::FreeQuasiSymmetricFunctions' instead"):

alias(P=polylib::Poly([hold(x)])):

domain experimental::intTableSort
    inherits Dom::BaseDomain;
    category Cat::Set;
    
    new :=
    proc()
	local eq;
    begin
	new(dom, P([[op(eq,2) = op(eq,1)] $ eq in [args()]]));
    end_proc;
    convert := new;
    
    set_index :=
    proc(container, index, value)
    begin
	container := extop(container);
	new(dom, container + P([[value-coeff(container, index), index]]));
    end_proc;

    _index :=
    proc(container, index)
    begin
	coeff(extop(container), index);
    end_proc;
    
    contains :=
    proc(container, index)
    begin
	bool(coeff(extop(container), index)<>0);
    end_proc;
    
    expr :=
    proc(container)
	local t;
    begin
	table((extop(t,2)=extop(t,1)) $ t in poly2list(extop(container)));
    end_proc;

    nops := nterms @ extop;
end:

domain experimental::intSetSort
    inherits experimental::intTableSort;
    category Cat::Set;
    
    new :=
    proc()
	local i;
    begin
	new(dom, P([[1,i] $ i in [args()]]));
    end_proc;
    
    min :=
    proc(container: dom) : Type::NonNegInt
    begin
	container := extop(container);
	op(poly2list(nthterm(container, nterms(container))),
	   [1,2]);
    end_proc;

    max := degree @ extop;

    expr :=
    proc(container: dom) : DOM_SET
    begin
	{op(map(poly2list(extop(container)), extop,2))}
    end_proc;

    op :=
    proc(container: dom)
    begin
	op(revert(map(poly2list(extop(container)), extop,2)));
    end_proc;

    _minus :=
    proc(container: dom, list: Type::AnyType)
	local x;
    begin
	for x in list do
	    container[x] := 0;
	end_for;
	container;
    end_proc;

    _union :=
    proc(container: dom, list: Type::AnyType)
	local x;
    begin
	for x in list do
	    container[x] := 1;
	end_for;
	container;
    end_proc;
end:

/*
>> time(proc(n)
>> begin
>>     p := experimental::intSetSort(1,2,5):
>>     for i from 1 to n do
>> 	contains(p,i)
>>     end
>> end(100000)) ;
                                              1253
>> time(proc(n)
>> begin
>>     p := experimental::intSetSort(1,2,5):
>>     for i from 1 to n do
>> 	min(p)
>>     end
>> end(100000)) ;
                                              1789
>> time(proc(n)
>> begin
>>     p := experimental::intSetSort(1,2,5):
>>     for i from 1 to n do
>> 	max(p)
>>     end
>> end(100000)) ;
                                              877

prog::test(contains(experimental::intSetSort(1, 2, 5), 5),
           TRUE):
prog::test(not contains(experimental::intSetSort(1, 2), 5),
           TRUE):
prog::test(nops(experimental::intSetSort(1, 2, 7)),
           3):
prog::test(expr(experimental::intSetSort(1, 2, 7)),
           {1, 2, 7}):
prog::test([op(experimental::intSetSort(1, 2, 7))],
           [1, 2, 7]):
s := experimental::intSetSort(1, 2, 7):
prog::test(s[3], 0):
s[3] := 0:
prog::test(s[3], 0):
s[3] := 1:
prog::test(s[3], 1):
s[3] := 1:
prog::test(s[3], 1):
s[3] := 0:
prog::test(s[3], 0):
prog::test(min(s), 1):
prog::test(max(s), 7):

     * Some timings for polynomial access:

>> time(proc(n)
>> begin
>>     for i from 1 to n do
>> 	poly(x^i)
>>     end
>> end(100000)) ;
                                   1230
>> time(proc(n)
>> begin
>>     for i from 1 to n do
>> 	poly([[1,i]],[x])
>>     end
>> end(100000)) ;

                                    761
                                                             Time: 762 msec
>> time(proc(n)
>> begin
>>     x := poly(x);
>>     for i from 1 to n do
>> 	x^i;
>>     end
>> end(100000)) ;

                                    695
                                                             Time: 695 msec
>> time(proc(n)
>> begin
>>     x := poly(x);
>>     for i from 1 to n do
>>         coeff(x,i)
>>     end
>> end(100000)) ;

                                    441
                                                             Time: 441 msec
>> time(proc(n)
begin
    x := poly(x);
    for i from 1 to n do
    end
end(100000))

                                    128
                                                             Time: 128 msec

// Hmm, this way of doing a setcoeff is linear in the size of p!
bla := proc(n)
begin
    p := poly([[1,i] $i=1..n], [hold(x)]);
    X := poly(hold(x));
    r := random(n);
    time(p + X^r());
end:
for n from 2 to 200000 step 2000 do
   fprint(Unquoted, 0, n, "\t", bla(n)/(n*1.0));
end
*/


null():
