/*
|  File ListComp (COMP PACKAGE) Mardi 20 mai 1997 16:01:09
|
|  ListCompo(n) returns the list of all compositions of n.
|
|  ListCompo(n, options) computes the list of all compositions of n constrained
|                        by the following allowed options:
|            * lg <= l       : length of compositions shall not exceed l,
|            * lg = l        : length of compositions is set to l,
|            * maxouter = m  : outermost possible shape,
|            * allowzeros    : compositions may include one or more zeros
|                              (length must be set by option lg = l).
|            * nb            : only returns the number of compositions;
| 
*/

/*
|  We may compute separatly the classical case...
|  VP200597
*/
muEC::COMP::ListCompo_rec_elem := proc(n)
  option remember;
local
  i;  /* var for seq... */
begin
  if (n = 0) then
    [[]];
  else
   [op(map(muEC::COMP::ListCompo_rec_elem(n+i), proc(x,y) begin [y, op(x)] end_proc,
                     -i)
          ) $ i=-n..-1];
  end_if;
end_proc:

/*
|  exactl is TRUE if l is constant; zerone either equals 0 or one.
|  VP220597
*/
muEC::COMP::ListCompo_rec := proc(n, l, exactl, zerone, m)
  option remember;
local
  i,  /* var for seq...                      */
  j,  /* var for seq...                      */
  gp; /* greatest part of outermost shape... */
begin
  if (l = 0) then
    if (n > 0) then   /* impossible case...       */
      [];
    else
      [[]];
    end_if;
  elif (n = 0) then
    if (l > 0) then   /* we may put some zeros... */
      if (exactl) then
        if (zerone = 0) then
          [[0 $ i=1..l]];
        else
          [];
        end_if;
      else
        if (zerone = 0) then
          [[], [0 $ i=1..j] $ j=1..l];
        else
          [[]];
        end_if;
      end_if;
    end_if;
  else
   if (m = []) then
     gp := 0;
   else
     gp := m[1];
   end_if;
   [op(map(
          muEC::COMP::ListCompo_rec(n+i, l-1, exactl,zerone,
                         [min(1+i+min(n,op(m, 1..min(1,nops(m)))),
                          op(m, 2..min(2,nops(m)))), op(m, 3..nops(m))]),
          proc(x,y) begin [y, op(x)] end_proc,
          -i)) $ i=-min(n, gp)..-zerone];
  end_if;
end_proc:

/*
|  Counting functions
|  VP200597
*/
muEC::COMP::NbCompo_elem := proc(n)
begin
  2^(n-1);
end_proc:

/*
|  VP220597
*/
muEC::COMP::NbCompo_rec := proc(n,l,exactl,zerone,m)
  option remember;
local
  i,  /* var for seq...                      */
  gp; /* greatest part of outermost shape... */
begin
  if (l = 0) then
    if (n > 0) then   /* impossible case...       */
      0;
    else
      1;
    end_if;
  elif (n = 0) then
    if (l > 0) then   /* we may put some zeros... */
      if (exactl) then
        if (zerone = 0) then
          1;
        else
          0;
        end_if;
      else
        if (zerone = 0) then
          1+l;
        else
          1;
        end_if;
      end_if;
    end_if;
  else
    if (m = []) then
      gp := 0;
    else
      gp := m[1];
    end_if;
    _plus(muEC::COMP::NbCompo_rec(n+i, l-1, exactl,zerone,
                        [min(1+i+min(n,op(m, 1..min(1,nops(m)))),
                         op(m, 2..min(2,nops(m)))), op(m, 3..nops(m))])
         $ i=-min(n, gp)..-zerone)
  end_if;
end_proc:
/*
| End of counting functions...
*/

/*
| VP220597 VP101298
| Treating options...
*/
muEC::COMP::ListCompo_core := proc(n)
local
  i,           /* var for seq...                            */
  j,           /* var for loop...                           */
  max_lg,      /* maximal length of compositions...         */
  set_lg,      /* length is in fact set...                  */
  max_out,     /* outermost possible shape...               */
  zero_one,    /* inserting zeros...                        */
  nb_count,    /* only counting...                          */
  dont_forget, /* we may not forget results...              */
  res;         /* result (needed for forget instruction)... */
begin
  max_lg   := n;       /* default value... */
  set_lg   := FALSE;   /* default value... */
  max_out  := [n$n];   /* default value... */
  zero_one := 1;       /* default value... */
  nb_count := FALSE;   /* default value... */
  dont_forget := FALSE;/* I remember when I was young... */
  if (args(0) >= 2) then
    for j from 2 to args(0) do
      if (type(args(j)) = "_equal") then
        if (op(args(j), 1) = hold(lg)) then
          max_lg  := op(args(j), 2);
          set_lg  := TRUE;
        elif (op(args(j), 1) = hold(maxouter)) then
          max_out := [op(op(args(j), 2)),/*max_out needs to be completed by 0's.*/
                      n $ i=1..n-nops(op(args(j), 2))];
        end_if;
      elif (type(args(j)) = "_leequal") then
        if (op(args(j), 1) = hold(lg)) then
          max_lg := op(args(j), 2);
        end_if;
      elif (args(j) = hold(allowzeros)) then
        zero_one := 0;
      elif (args(j) = hold(nb)) then
        nb_count := TRUE;
      elif (args(j) = hold(dontforget)) then
        dont_forget := TRUE;
      end_if;
    end_for;
  end_if;
  if (nb_count) then   /* only counting... */
    if (args(0)=2) then
      muEC::COMP::NbCompo_elem(n);
    else
      res := muEC::COMP::NbCompo_rec(n, max_lg, set_lg, zero_one, max_out);
      if (not dont_forget) then
          sysassign(muEC::COMP::NbCompo_rec,
                    muEC::clearRememberTable(muEC::COMP::NbCompo_rec));
      end_if;
      res;
    end_if;
  elif (args(0)=1) then 
    res := muEC::COMP::ListCompo_rec_elem(n);
    sysassign(muEC::COMP::ListCompo_rec_elem,
              muEC::clearRememberTable(muEC::COMP::ListCompo_rec_elem));
    res;
  elif (args(0)=2 and dont_forget) then /* we shall stay efficient... */
    muEC::COMP::ListCompo_rec_elem(n);
  else
    res := muEC::COMP::ListCompo_rec(n, max_lg, set_lg, zero_one, max_out);
      if (not dont_forget) then
        sysassign(muEC::COMP::ListCompo_rec,
                  muEC::clearRememberTable(muEC::COMP::ListCompo_rec));
      end_if;
    res;
  end_if;
end_proc:

/*
|  VP220597
*/
muEC::COMP::ListCompo_check := proc(n)
local
  i; /* var for loop... */
begin
  if (not(type(n)=DOM_INT)) then
    muEC::ERR::COMPPerror(1,procname);          /* Wrong type for first argument... */
  elif (args(0) >= 2) then
    for i from 2 to args(0) do
 /* Checking all possibilities... */
      if (type(args(i)) <> "_equal" and not(type(args(i))="_leequal")
          and args(i)<>hold(nb) and args(i)<>hold(allowzeros)
          and args(i)<>hold(dontforget)) then
          muEC::ERR::COMPPerror(2*i-1,procname);  /* Wrong type for ith argument... */
      end_if;
 /* Checking structured arguments... */
      if (type(args(i))="_equal") then
        if (op(args(i), 1) = hold(lg)) then
          if (not (type(op(args(i), 2))=DOM_INT and op(args(i), 2)>=0)) then
            muEC::ERR::COMPPerror(2*i-1,procname);/* Wrong type for ith argument... */
          end_if;
        elif (op(args(i), 1) = hold(maxouter)) then
          if (not(muEC::TYP::IsCompo(op(args(i), 2)))) then
            muEC::ERR::COMPPerror(2*i-1,procname);/* Wrong type for ith argument... */
          end_if;
       end_if;
      elif (type(args(i)) ="_leequal") then
        if (op(args(i), 1) = hold(lg)) then
          if (not (type(op(args(i), 2))=DOM_INT and op(args(i), 2)>=0)) then
            muEC::ERR::COMPPerror(2*i-1,procname);/* Wrong type for ith argument... */
          end_if;
        else
          muEC::ERR::COMPPerror(2*i-1,procname);  /* Wrong type for ith argument... */
        end_if;
      end_if;
    end_for;
  end_if;
end_proc:

/*
|  VP220597
*/
muEC::COMP::ListCompo := proc(compo)
begin
  if testargs() then
    muEC::COMP::ListCompo_check(args());
  end_if;
  muEC::COMP::ListCompo_core(args());
end_proc:
