/**
|  File PART.mu (muEC::PART PACKAGE) Vendredi 27 juin 1997 15:26:57
|
|            The 'muEC::PART' library for manipulating partitions.
|
| VP270697 VP020498
**/

/**
User Functions:

muEC::PART::Border2Part    -  Border2Part(border) returns the partition (tableau)
                        corresponding to the given border.
                        See also available options.

muEC::PART::CompPart       -  compares its two arguments.

muEC::PART::Diagonal2Part  -  Diagonal2Part(l) with l=[dl,o] returns the partition
                        associated to the diagonal list dl, with o, origin
                        of the tableau.

muEC::PART::Exp2Part       -  partition from its exponential notation.

muEC::PART::Frob2Part      -  converts Frobenius notation to standard notation for
                        partitions.

muEC::PART::ListPartIn     -  returns the list of all partitions contained in the
                        diagram of a given partition.
                        See also available options.

muEC::PART::ListPart       -  returns the list of all partitions of the
                        argument.
                        See also available options.

muEC::PART::ListSkewDiag   -  returns a list of all skew partitions of n.
                        See also available options.

muEC::PART::PCore2Part     -  partition from its P-Core/Quotient list.

muEC::PART::Part2Border    -  returns a list of 0 and 1, 0 denoting an horizontal
                        chunk of the border of lambda, 1 a vertical one.
                        See also available options.

muEC::PART::Part2Conjugate -  returns the conjugate partition.

muEC::PART::Part2Exp       -  exponential notation of a partition.

muEC::PART::Part2Diagonal  -  returns the list of diagonals of the partition
                        and the position of tableau's origin.

muEC::PART::Part2Frob      -  gives the Frobenius notation of the partition.

muEC::PART::Part2ListHook  -  returns the list of hook-lengths of the partition.
                        See also available options.

muEC::PART::Part2Mat       -  returns a `visual' matrix of the partition.
                        See also available options.

muEC::PART::Part2PCore     -  P-Core of a partition (p is the 2nd arg)

muEC::PART::PartOrderMat   -  matrix order between partitions.

muEC::PART::SkewPart2Mat   -  returns a `visual' matrix of the skew partition.

muEC::PART::SplitPart      -  set of ways of splitting a partition into n
                        components.


Partitions are encoded by integer lists in decreasing order.
**/

/**
Algorithms for PART are mainly from my own.
**/

muEC::PART := newDomain("PART"):
muEC::PART::info := "Library PART: manipulating PARTitions encoded as lists of decreasing integers":
muEC::PART::interface := { Border2Part, CompPart, Diagonal2Part, Exp2Part,
                       Frob2Part, ListPartIn, ListPart, ListSkewDiag,
                       PCore2Part, Part2Border, Part2Conjugate, Part2Diagonal,
                       Part2Exp, Part2Frob, Part2ListHook, Part2Mat,
                       Part2PCore, PartOrderMat, SkewPart2Mat, SplitPart}:

alias(path = pathname("MUEC", "PART")):

muEC::PART::Border2Part    := loadproc(muEC::PART::Border2Part,    path, "Bdr2Part"):
muEC::PART::CompPart       := loadproc(muEC::PART::CompPart,       path, "PartOrdr"):
muEC::PART::Diagonal2Part  := loadproc(muEC::PART::Diagonal2Part,  path, "Dgl2Part"):
muEC::PART::Exp2Part       := loadproc(muEC::PART::Exp2Part,       path, "Exp2Part"):
muEC::PART::Frob2Part      := loadproc(muEC::PART::Frob2Part,      path, "Frb2Part"):
muEC::PART::ListPartIn     := loadproc(muEC::PART::ListPartIn,     path, "ListPaIn"):
muEC::PART::ListPart       := loadproc(muEC::PART::ListPart,       path, "ListPart"):
muEC::PART::ListSkewDiag   := loadproc(muEC::PART::ListSkewDiag,   path, "ListSkDi"):
muEC::PART::PCore2Part     := loadproc(muEC::PART::PCore2Part,     path, "PCore2Pa"):
muEC::PART::Part2Border    := loadproc(muEC::PART::Part2Border,    path, "Part2Bdr"):
muEC::PART::Part2Conjugate := loadproc(muEC::PART::Part2Conjugate, path, "Part2Cjg"):
muEC::PART::Part2Diagonal  := loadproc(muEC::PART::Part2Diagonal,  path, "Part2Dgl"):
muEC::PART::Part2Exp       := loadproc(muEC::PART::Part2Exp,       path, "Part2Exp"):
muEC::PART::Part2Frob      := loadproc(muEC::PART::Part2Frob,      path, "Part2Frb"):
muEC::PART::Part2ListHook  := loadproc(muEC::PART::Part2ListHook,  path, "Part2LHk"):
muEC::PART::Part2Mat       := loadproc(muEC::PART::Part2Mat,       path, "Part2Mat"):
muEC::PART::Part2PCore     := loadproc(muEC::PART::Part2PCore,     path, "PCoreQuo"):
muEC::PART::PartOrderMat   := loadproc(muEC::PART::PartOrderMat,   path, "PartOrdr"):
muEC::PART::SkewPart2Mat   := loadproc(muEC::PART::SkewPart2Mat,   path, "SkPa2Mat"):
muEC::PART::SplitPart      := loadproc(muEC::PART::SplitPart,      path, "SpltPart"):

unalias(path):
