/*
|  File Bdr2Part (PART PACKAGE) Vendredi 30 mai 1997 08:34:52
| 
|  Border2Part(border) returns the partition (tableau) corresponding to the
|                      given border.
|                      The border is a list of 0 and 1, 0 denoting an
|                      horizontal chunk, 1 a vertical one.
|                      Extremity segments shall not be included (i.e. 1 on the
|                      left top, 0 on the right bottom).
|                      The returned partition is in decreasing order.
| 
| 
|  Border2Part(border, m) returns the same, m[1] denoting an horizontal chunk
|                         of the border, m[2] a vertical one.
| 
| 
*/

/*
|  VP300597 VP070797
*/
muEC::PART::Border2Part_int := proc(border)
local
  i,      /* var for loop...               */
  h,      /* horizontal encoding...        */
  n,      /* number of horizontal segments */
  lambda; /* the result                    */
begin
  lambda := null();
  n      := 0;
  if (args(0) = 2) then
    h := op(args(2), 1);  
  else
    h := 0;
  end_if;
  for i from 1 to nops(border) do
    if (border[i] = h) then
      n := n+1;
    else
      lambda := n, lambda;
    end_if;
  end_for;
  [lambda]
end_proc:

/*
|  Interfacing options...
|  VP300597
*/
muEC::PART::Border2Part_core := proc(border)
local
  i,      /* var for loop... */
  alphbt; /* the alphabet... */
begin
  alphbt := null(); /* default value... */
  if (args(0) >= 2) then
    for i from 2 to args(0) do
      if (op(args(i), 1) = hold(alphabet)) then
        alphbt := op(args(i), 2);
      end_if;
    end_for;
  end_if;
  muEC::PART::Border2Part_int(border,alphbt);
end_proc:

/*
|  VP300597
*/
muEC::PART::Border2Part_check := proc(border)
local
  i; /* car for seq...*/
begin
  if (args(0) >= 2) then
    if testtype(args(2), "_equal") then
      if (op(args(2), 1) = hold(alphabet)) then
        if not (testtype(op(args(2), 2), DOM_LIST)) then
          muEC::ERR::PARTPerror(4,procname);
        elif (nops(op(args(2), 2)) <> 2) then 
          muEC::ERR::PARTPerror(4,procname);
        end_if;
      else
        muEC::ERR::PARTPerror(4,procname);
      end_if;
    else
      muEC::ERR::PARTPerror(3,procname);
    end_if;
    if not (muEC::TYP::IsBorder(args(1), (args(i) $i=2..args(0)))) then
      muEC::ERR::PARTPerror(1,procname);
    end_if;
  elif not (muEC::TYP::IsBorder(args(1))) then
    muEC::ERR::PARTPerror(1,procname);
  end_if;
end_proc:

/*
|  VP300597
*/
muEC::PART::Border2Part := proc(Border)
begin
  if testargs() then
    muEC::PART::Border2Part_check(args());
  end_if;
  muEC::PART::Border2Part_core(args());
end_proc:
