/*
|  File ListPaIn (PART PACKAGE) Vendredi 30 mai 1997 10:58:43
|
|  ListPartIn(part) returns the list of all partitions contained in the diagram
|                   of a given partition part.
|
|
|  ListPartIn(part, partin) returns the list of all partitions contained
|                           between the two partitions. The diagram of partin
|                           in the smallest diagram in the ouput.
|
|  ListPartIn(part, options) computes the list of all partitions contained in
|                            the diagram of part, constrained by the following
|                            allowed options:
|            * weight = w   : weight of partitions must be w,
|            * nb           : only returns the number of partitions.
|
|  ListPartIn(part, partin, options) computes the list of all partitions
|                            contained between part and partin, constrained by
|                            the following allowed options:
|            * weight = w   : weight of partitions must be w,
|            * nb           : only returns the number of partitions.
*/

/*
|  VP300597
*/
muEC::PART::ListPartIn_rec := proc(n, part)
local
  i; /* var for seq... */
begin
  if (n = 1) then
    [[i] $ i=1..part[n]];
  else
    map( muEC::PART::ListPartIn_rec(n-1, part),
         proc(p, maxi) local i; begin
           [op(p), i] $ i=1..min(p[nops(p)], maxi);
         end_proc,
         part[n]);
  end_if;
end_proc:

/*
|  VP230498
*/
muEC::PART::ListPartIn_wght := proc(n, wght, p)
local
  m, i; /* var...         */
begin
  m := nops(p);
  if (wght=0 or n=0 or m=0 or n>m) then
    [];
  elif (n=1) then
    if (wght <= p[1]) then
      [[wght]];
    else
      [];
    end_if;
  else
   [op(map(muEC::PART::ListPartIn_wght
           (n-1, wght+i,
            [min(-i, op(p,2..min(2,m))), op(p,3..m)]),
	   proc(x,y) begin [y, op(x)] end_proc ,
           -i))
    $ i=-min(p[1], wght)..-iquo(wght+n-1, n)];
  end_if;
end_proc:

   
/*
|  VP300597
*/
muEC::PART::ListPartBetween_rec:=proc(n, part, partin)
local				     
      i; /* var for seq... */
begin
   if (n = 1) then
    if (partin=[]) then
      [[], [i] $ i=1..part[n]];
    else
      [[i] $ i=partin[n]..part[n]];
    end_if;
  else
  map( muEC::PART::ListPartBetween_rec(n-1, part, partin),
       proc(p, mini, maxi) local i; begin
         [op(p), i] $ i=mini..min(p[nops(p)], maxi);
        end_proc,
       partin[n], part[n]);
  end_if;
end_proc:

/*
|  VP230498
*/
muEC::PART::ListPartBetween_wght := proc(n, wght, mp, Mp)
local
  m,    /* var...         */
  M,    /* var...         */
  minp,i ; /* var...         */
begin
  m := nops(mp);
  M := nops(Mp);
  minp := [op(mp),1$(n-m)];
  if (wght=0 or n=0 or M=0 or n>M or n<m) then
    [];
  elif (n=1) then
    if (minp[1]<=wght and wght<=Mp[1]) then
      [[wght]];
    else
      [];
    end_if;
  else
   [op(map(muEC::PART::ListPartBetween_wght
           (n-1, wght+i,
            [min(wght+i, op(minp,2..min(2,n))), op(minp,3..n)],
            [min(-i, op(Mp,2..min(2,M))), op(Mp,3..M)]),
	   proc(x,y) begin [y, op(x)] end_proc,
           -i))
    $ i=-min(Mp[1], wght)..-max(iquo(wght+n-1, n),minp[1])];
  end_if;
end_proc:


/*
|  VP300597
|  computes the list of [m,lst], where m is the number of partitions ending by
|  part lst.
*/
muEC::PART::CountLastPartIn := proc(n, part)
local
  i,   /* var for loop...        */
  j,   /* var for loop... */
  k,   /* var for loop...        */
  rj,  /* counter...             */
  p,   /* a couple...            */
  L,   /* recursive call...      */
  res; /* the result...          */
begin
  if (n=1) then
    res := [1, j] $ j=1..part[n];
  else
    res := null();
    L   := muEC::PART::CountLastPartIn(n-1, part);
    for i from 1 to op(L[nops(L)], 2) do
      for j from i to min(i, part[n]) do
        rj := 0;
        for k from 1 to nops(L) do
          p := L[k];
          if (j <= p[2]) then
            rj := rj + p[1];
          end_if;
        end_for;
        res := res, [rj, j];
      end_for;
    end_for;
  end_if;
  [res];
end_proc:

/*
|  VP300597
*/
muEC::PART::CountLastPartBetween := proc(n, part, partin)
local
  i,   /* var for loop...        */
  j,   /* var for loop or seq... */
  k,   /* var for loop...        */
  rj,  /* counter...             */
  p,   /* a couple...            */
  L,   /* recursive call...      */
  res; /* the result...          */
begin
  if (n=1) then
    if (partin = []) then
      res := [], ([1, j] $ j=1..part[n]);
    else
      res := [1, j] $ j=partin[n]..part[n];
    end_if;
  else
    res := null();
    L   := muEC::PART::CountLastPartBetween(n-1, part, partin);
    for i from partin[n] to op(L[nops(L)], 2) do
      for j from i to min(i, part[n]) do
        rj := 0;
        for k from 1 to nops(L) do
        p := L[k];
          if (j <= p[2]) then
            rj := rj + p[1];
          end_if;
        end_for;
        res := res, [rj, j];
      end_for;
    end_for;
  end_if;
  [res];
end_proc:

/*
|  Should be optimized...
|  VP230498
*/
muEC::PART::CountListPartIn_wght := proc(n, wght, p)
local
  m, /* var...         */
  i; /* var for seq... */
begin
  m := nops(p);
  if (wght=0 or n=0 or m=0 or n>m) then
    0;
  elif (n=1) then
    if (wght <= p[1]) then
      1;
    else
      0;
    end_if;
  else
   _plus(
         muEC::PART::CountListPartIn_wght
         (n-1, wght+i,
          [min(-i, op(p,2..min(2,m))), op(p,3..m)])
         $ i=-min(p[1], wght)..-iquo(wght+n-1, n));
  end_if;
end_proc:

/*
|  Should be optimized...
|  VP230498
*/
muEC::PART::CountListPartBetween_wght := proc(n, wght, mp, Mp)
local
  m,    /* var...         */
  M,    /* var...         */
  minp, /* var...         */
  i;    /* var for seq... */
begin
  m := nops(mp);
  M := nops(Mp);
  minp := [op(mp),1$(n-m)];
  if (wght=0 or n=0 or M=0 or n>M or n<m) then
    0;
  elif (n=1) then
    if (minp[1]<=wght and wght<=Mp[1]) then
      1;
    else
      0;
    end_if;
  else
   _plus(muEC::PART::CountListPartBetween_wght
         (n-1,
          wght+i,
          [min(wght+i, op(minp,2..min(2,n))), op(minp,3..n)],
          [min(-i, op(Mp,2..min(2,M))), op(Mp,3..M)]
         )
         $ i=-min(Mp[1], wght)..-max(iquo(wght+n-1, n),minp[1]));
  end_if;
end_proc:


/*
|  VP300597 VP230498
*/
muEC::PART::NbPartIn := proc(part, max_lg, wght) 
local
  i; /* var for seq... */
begin
  if (args(0) = 3) or (args(0) = 4 and args(4)=[])  then
    if (wght < 0) then
      if (part = []) then
        1
      else
      /* the partition is not empty... */
        1 + _plus(op(map([(0,0),
                          op(muEC::PART::CountLastPartIn(i, part))
                          $ i=1..max(max_lg, nops(part))],
                          proc( x) begin op(x, 1) end_proc )));
      end_if;
    elif (wght=0) then
      1;
    else
      _plus(muEC::PART::CountListPartIn_wght(i,wght,part) $ i=1..max_lg);
    end_if;
  else /* args(0) = 4... */
    if ((part = []) and (args(4) = [])) then
      1
    elif (nops(part) < nops(args(4))) then
      0
    elif (wght < 0) then
    /* args(4) is a partition that is contained into part... */
      _plus(op(map(muEC::PART::CountLastPartBetween(max(max_lg, nops(part)), part,
                   [op(args(4)), 0$(nops(part)-nops(args(4)))]),
                   proc( x) begin op(x, 1) end_proc )));
    else
      _plus(muEC::PART::CountListPartBetween_wght(i,wght,args(4),part)
            $ i=1..max_lg);
    end_if;
  end_if;
end_proc:

/*
|  VP300597 VP230498
*/
muEC::PART::ListPartIn_core := proc(part)
local   
  i,        /* var for loop and seq... */
  j,        /* var for seq...          */
  part_in,  /* minimal partition...    */
  wght,     /* weight of partitions... */
  nb_count; /* only counting...        */
begin
  part_in  := [] ;        /* default value... */
  wght     := -1;         /* default value... */
  nb_count := FALSE;      /* default value... */
  if (args(0) >= 2) then
    for i from 2 to args(0) do
      if (op(args(i), 1) = hold(weight)) then
        wght := op(args(i), 2);
      elif (args(i) = hold(nb)) then
        nb_count := TRUE;
      else
        part_in  := args(i);
      end_if;         
    end_for; 
  end_if;
  if (nb_count) then
     muEC::PART::NbPartIn(part, nops(part), wght, part_in);
  else
    if (part = []) then
      if (nops(part_in)>0) then
        [];
      else
        [[]];
      end_if;
    elif (part_in = []) then
      if (wght<0) then
        [[], op(muEC::PART::ListPartIn_rec(j, part)) $ j=1..nops(part)];
      elif (wght = 0) then
        [[]];
      else
        [op(muEC::PART::ListPartIn_wght(j, wght, part)) $ j=1..nops(part)];
      end_if;
    elif (wght<0) then
      subs(muEC::PART::ListPartBetween_rec(nops(part), part,
              [op(part_in), 0$(nops(part) - nops(part_in))]), 0=null());
    else
       [op(muEC::PART::ListPartBetween_wght(j, wght, part_in, part))
        $ j = 1..nops(part)];
    end_if;
  end_if;
end_proc:

/*
|  VP300597 VP230498
*/
muEC::PART::ListPartIn_check := proc(part)
local
  i; /* variable for loop... */
begin
 if not (muEC::TYP::IsPart(part)) then
   muEC::ERR::PARTPerror(1,procname);           /* Wrong type for first argument... */
 elif (args(0) >= 2) then    
   for i from 2 to args(0) do
     if (not testtype(args(i),"_equal") 
        and (not muEC::TYP::IsPart(args(i))) and (args(i) <> hold(nb)))
     then
         muEC::ERR::PARTPerror(2*i-1,procname); /* Wrong type for ith argument... */
     end_if;
     if (testtype(args(i),"_equal")) then
       if (op(args(i), 1) = hold(weight)) then
         if (not testtype(op(args(i), 2), Type::NonNegInt)) then
           muEC::ERR::PARTPerror(2*i-1,procname); /* Wrong type for ith argument... */
         end_if;  
       else
         muEC::ERR::PARTPerror(2*i,procname); /* Wrong range for ith argument... */
       end_if;
     end_if;
   end_for; 
 end_if;    
end_proc:  

/*
|  VP270697
*/
muEC::PART::ListPartIn := proc(part)
begin
  if testargs() then
    muEC::PART::ListPartIn_check(args());
  end_if;
  muEC::PART::ListPartIn_core(args());
end_proc:
