/*
|  File ListPart (PART PACKAGE) Lundi 30 juin 1997 07:47:54
*/

/*
|  ListPart(n) returns the list of all partitions of n.
| 
|  ListPart(n, options) computes the list of all partitions of n constrained by
|                       the following allowed options:
|            * maxlg = l    : length of partitions shall not exceed l,
|            * maxpart = m  : maximal partition, i.e. outermost possible shape,
|            * regular = m  : only m-regular partitions,
|            * nb           : only returns the number of partitions;
|       where maxpart is a list whose components majorate the first ones of
|       the resulting partitions parts.
*/

/* 
|  VP306697
*/
muEC::PART::ListPart_rec := proc(n,l,m)
local
  i,  /* var for seq...   */
  gp; /* greatest part... */
begin
  if (n=0 or l=0) then
    [[]];
  else
    if (m = []) then
      gp := n;
    else
      gp := m[1];
    end_if;
   [_seqgen(op(map( muEC::PART::ListPart_rec(n+i, l-1,
                        [min(-i, op(m,2..min(2,nops(m)))), op(m,3..nops(m))]),
                    proc( x, y) begin [y, op(x)] end_proc ,
     -i)), i,-min(gp, n)..-iquo(n+l-1, l))];
  end_if;
end_proc:

/*
|  Counting functions
*/

/*
|  VP300697
*/
muEC::PART::NbPart_rec_elem := proc(n)
local
  np, /* number of partitions... = the result... */
  m,  /* parameter...                            */
  k;  /* var for loop...                         */
option remember;
save MAXDEPTH;
begin
  prog::setMaximalMAXDEPTH(); /* To avoid error on infinite loop detection */
  if (n=1 or n=0) then
    np := 1;
  else
    /* we know that n >= 2... */
    np := 0;
    m := n - 1;
    k := 2;
    while (m >= 0) do
      if ((k mod 2) = 0) then
        np := np + muEC::PART::NbPart_rec_elem(m);
      else
        np := np - muEC::PART::NbPart_rec_elem(m);
      end_if;
      m := n - k*(3*k - 1)/2;
      k := k + 1;
    end_while;
    m := n - 2;
    k := 2;
    while (m >= 0) do
      if ((k mod 2) = 0) then
        np := np + muEC::PART::NbPart_rec_elem(m);
      else
        np := np - muEC::PART::NbPart_rec_elem(m);
      end_if;
      m := n - k*(3*k + 1)/2;
      k := k + 1;
    end_while;
  end_if;
  np;
end_proc:

/*
|  VP300697 VP101298
*/
muEC::PART::NbPart_rec := proc(n,l,m)
local
  i,  /* var for seq...   */
  gp; /* greatest part... */
begin
  if (n=0 or l=0) then
    1;
  else
    if (m = []) then
      gp := n;
    else
      gp := m[1];
    end_if;
   _plus(
     _seqgen(muEC::PART::NbPart_rec(n+i, l-1,
                              [min(-i, op(m,2..min(2,nops(m)))),
                              op(m,3..nops(m))]),
           i,-min(gp, n)..-iquo(n+l-1, l)));
  end_if;
end_proc:

/*
|  End of counting functions...
*/

/*
|  VP300697 VP220498
*/
muEC::PART::ListPart_core := proc(n)
local 
  i,        /* var for loop and seq...         */
  max_lg,   /* maximal length of partitions... */
  max_pa,   /* maximal partition...            */
  n_reg,    /* regular options specified...    */
  nb_count; /* only counting...                */
begin
 max_lg := n;                    /* default value... */
 max_pa := [_seqgen(i, i,n..1)]; /* default value... */
 nb_count := FALSE;              /* default value... */
 n_reg := NIL;
 if (args(0) >= 2) then
   for i from 2 to args(0) do
     if (op(args(i), 1) = hold(maxlg)) then
       max_lg := op(args(i), 2);
     elif (op(args(i), 1) = hold(maxpart)) then
       max_pa := op(args(i), 2);
     elif (op(args(i), 1) = hold(regular)) then
       n_reg := op(args(i), 2);
     elif (args(i) = hold(nb)) then
       nb_count := TRUE;
     end_if;
   end_for
 end_if;
 if (nb_count) then
   if (args(0) = 2) then
     muEC::PART::NbPart_rec_elem(n);
   elif n_reg<>NIL then
       nops(select(muEC::PART::ListPart_rec(n, max_lg, max_pa),
                   p->muEC::TYP::IsRegPart(n_reg,p)));
   else
     muEC::PART::NbPart_rec(n, max_lg, max_pa);
   end_if;
 elif n_reg<>NIL then
     select(muEC::PART::ListPart_rec(n, max_lg, max_pa),
            p->muEC::TYP::IsRegPart(n_reg,p));
 else
   muEC::PART::ListPart_rec(n, max_lg, max_pa);
 end_if;
end_proc:

/*
|  VP300697 VP220498
*/
muEC::PART::ListPart_check := proc(n)
local
  i; /* var for loop... */
begin
  if (not testtype(n, DOM_INT)) then
    muEC::ERR::PARTPerror(1,procname);       /* Wrong type for first argument...  */
  elif (args(0) >= 2) then
    for i from 2 to args(0) do
      if (not testtype(args(i),"_equal") and args(i) <> hold(nb)) then
        muEC::ERR::PARTPerror(2*i-1,procname);  /* Wrong type for ith argument... */
      end_if;
      if (testtype(args(i), "_equal")) then
        if (op(args(i), 1) = hold(maxlg)) then
          if not (testtype(op(args(i), 2), DOM_INT) and op(args(i), 2)>=0) then
            muEC::ERR::PARTPerror(2*i-1,procname);/* Wrong type for ith argument... */
          end_if;
        elif (op(args(i), 1) = hold(maxpart)) then
          if (not muEC::TYP::IsPart(op(args(i), 2))) then
            muEC::ERR::PARTPerror(2*i-1,procname);/* Wrong type for ith argument... */
          end_if;
        elif (op(args(i), 1) = hold(regular)) then
          if (not testtype(op(args(i), 2), Type::PosInt)) then
            muEC::ERR::PARTPerror(2*i-1,procname);/* Wrong type for ith argument... */
          end_if;
        else
          muEC::ERR::PARTPerror(2*i,procname);  /* Wrong range for ith argument... */
        end_if;
      end_if;
    end_for
  end_if;
end_proc:

/*
|  VP300697
*/
muEC::PART::ListPart := proc(n)
begin
   if testargs() then
    muEC::PART::ListPart_check(args());
   end_if;
  muEC::PART::ListPart_core(args());
end_proc:
