/*
|  File ListSkDi (PART PACKAGE) Lundi 30 juin 1997 11:12:00
|
|  ListSkewDiag(n) returns a list of all skew partitions of n.
| 
|  ListSkewDiag(n, options) returns the skew partitions of n constrained by the
|                           following allowed options:
|                 *  maxlg = l    : length of partitions should not exceed l,
|                 *  mininner = p : minimal inner partition,
|                 *  maxinner = p : maximal inner partition,
|                 *  minouter = p : minimal outer partition,
|                 *  maxouter = p : maximal outer partition;
|             where the outer partition corresponds to the outer shape of the
|             skew partition and the inner partition corresponds to the inner
|             shape of the skew partition.
*/

/*
|  VP300697
|  Remark: shall be interfaced with muEC::PART::ListSkewDiag_rec, in order to
|          complete the 4 partitions arguments.
*/
muEC::PART::ListSkewDiagBetweenMultipleDiag_rec :=
  proc(n, p_lgth, m_inner, M_inner, m_outer, M_outer)
local
  i, /* var for seq... */
  j; /* var for seq... */
option remember;
begin
  if (n > 0 and m_inner[1] >= n) then
    [];
  elif (p_lgth = 0) then
    if (n > 0) then
      [];
    else
      [[[], []]];
    end_if;
  elif (n = 0) then
    if (m_outer[1] > 0) then
      [];
    else
      [[[], []]];
    end_if;
  else
    [_seqgen(_seqgen(op(map(
         muEC::PART::ListSkewDiagBetweenMultipleDiag_rec(n+i+j, p_lgth-1,
            [max(0, op(m_inner, 2..min(2, nops(m_inner)))), 
             op(m_inner, 3..nops(m_inner))],
            [min(j, op(M_inner, 2..min(2, nops(M_inner)))),  
             op(M_inner, 3..nops(M_inner))],
            [max(j, op(m_outer, 2..min(2, nops(m_outer)))),
              op(m_outer, 3..nops(m_outer))],
            [min(-i, op(M_outer, 2..min(2, nops(M_outer)))),
             op(M_outer, 3..nops(M_outer))]
            ),
         proc(skew,sp,p) begin
           [[sp, op(op(skew, 1))], [p, op(op(skew, 2))]];
         end_proc,
         -i,j)),
     j , m_inner[1]..min(M_inner[1],n-1,-i-1)),
     i , -min(n,M_outer[1])..-max(m_outer[1],iquo(n+p_lgth-1,p_lgth)))];
  end_if;
end_proc:

/*
| VP300697
*/
muEC::PART::ListSkewDiag_rec :=
     proc(n,p_lgth,m_inner,M_inner,m_outer,M_outer)
local i,  /* var for seq...             */
  new_mi, /* minimal inner partition... */
  new_Mi, /* maximal inner partition... */
  new_mo, /* minimal outer partition... */
  new_Mo, /* maximal inner partition... */
  lst_out,/* misc...                    */
  res;    /* the result...              */
begin
  if (M_outer=[]) then
    lst_out := n;
  else
    lst_out := M_outer[nops(M_outer)];
  end_if;
  new_mi := [op(m_inner),_seqgen(0,i,nops(m_inner)+1..n)];
  new_mo := [op(m_outer),_seqgen(0,i,nops(m_outer)+1..n)];
  new_Mo := [op(M_outer),_seqgen(lst_out, i,nops(M_outer)+1..n)];
  new_Mi := [op(M_inner),_seqgen(max(0,new_Mo[i]-1),i,nops(M_inner)+1..n)];
  res := subs(muEC::PART::ListSkewDiagBetweenMultipleDiag_rec(n,p_lgth,
                                  new_mi, new_Mi, new_mo, new_Mo), 0=null());
  sysassign(muEC::PART::ListSkewDiagBetweenMultipleDiag_rec,
            muEC::clearRememberTable
            (muEC::PART::ListSkewDiagBetweenMultipleDiag_rec));
  res;
end_proc:

/*
|  Counting functions
*/ 

/*
|  VP300697
|  Remark: shall be interfaced with muEC::PART::NbSkewDiag_rec, in order to
|          complete the 4 partitions arguments.
*/
muEC::PART::NbSkewDiagBetweenMultipleDiag_rec:=
  proc(n, p_lgth, m_inner, M_inner, m_outer, M_outer)
local
  i, /* var for seq... */
  j; /* var for seq... */
option remember;
begin
  if (n > 0 and m_inner[1] >= n) then
    0;
  elif (p_lgth = 0) then
    if (n > 0) then
      0;
    else
      1;
    end_if;
  elif (n = 0) then
    if (m_outer[1] > 0) then
      0;
    else
      1;
    end_if;
  else
    _plus(
      _seqgen(_seqgen(muEC::PART::NbSkewDiagBetweenMultipleDiag_rec(n+i+j, p_lgth-1,
              [max(0, op(m_inner, 2..min(2, nops(m_inner)))), 
                op(m_inner, 3..nops(m_inner))],
               [min(j, op(M_inner, 2..min(2, nops(M_inner)))),  
                op(M_inner, 3..nops(M_inner))],
               [max(j, op(m_outer, 2..min(2, nops(m_outer)))),
                op(m_outer, 3..nops(m_outer))],
              [min(-i, op(M_outer, 2..min(2, nops(M_outer)))),
               op(M_outer, 3..nops(M_outer))]
              ),
       j , m_inner[1]..min(M_inner[1],n-1,-i-1)),
       i , -min(n,M_outer[1])..-max(m_outer[1],iquo(n+p_lgth-1,p_lgth)))
    );
  end_if;
end_proc:

/*
|  VP300697
*/
muEC::PART::NbSkewDiag_rec :=
     proc(n,p_lgth,m_inner,M_inner,m_outer,M_outer)
local i,  /* var for seq...             */
  new_mi, /* minimal inner partition... */
  new_Mi, /* maximal inner partition... */
  new_mo, /* minimal outer partition... */
  new_Mo, /* maximal inner partition... */
  lst_out,/* misc...                    */
  res;    /* the result...              */
begin
  if (M_outer=[]) then
    lst_out := n;
  else
    lst_out := M_outer[nops(M_outer)];
  end_if;
  new_mi := [op(m_inner),_seqgen(0,i,nops(m_inner)+1..n)];
  new_mo := [op(m_outer),_seqgen(0,i,nops(m_outer)+1..n)];
  new_Mo := [op(M_outer),_seqgen(lst_out, i,nops(M_outer)+1..n)];
  new_Mi := [op(M_inner),_seqgen(max(0,new_Mo[i]-1),i,nops(M_inner)+1..n)];
  res := muEC::PART::NbSkewDiagBetweenMultipleDiag_rec(n,p_lgth,
                                             new_mi, new_Mi, new_mo, new_Mo);
  sysassign(muEC::PART::NbSkewDiagBetweenMultipleDiag_rec,
            muEC::clearRememberTable
            (muEC::PART::NbSkewDiagBetweenMultipleDiag_rec));
  res;
end_proc:
 
/*
|  End of counting functions
*/ 

/*
|  VP300697
*/
muEC::PART::ListSkewDiag_core := proc(n)
local
  i,         /* var for loop...                      */
  max_lg,    /* maximal length of skew partitions... */
  min_inner, /* minimal inner partition...           */
  max_inner, /* maximal inner partition...           */
  min_outer, /* minimal outer partition...           */
  max_outer, /* maximal outer partition...           */
  nb_count;  /* counting flag...                     */
begin
  max_lg    := n;     /* default value... */
  min_inner := [];    /* default value... */
  max_inner := [];    /* default value... */
  min_outer := [];    /* default value... */
  max_outer := [];    /* default value... */
  nb_count  := FALSE; /* default value... */
  if (args(0) >= 2) then
    for i from 2 to args(0) do
      if (args(i) = hold(nb)) then
        nb_count := TRUE;
      elif (op(args(i), 1) = hold(maxlg)) then
        max_lg  := op(args(i), 2);
      elif (op(args(i), 1) = hold(mininner)) then
        min_inner := op(args(i), 2);
      elif (op(args(i), 1) = hold(maxinner)) then
        max_inner := op(args(i), 2);
      elif (op(args(i), 1) = hold(minouter)) then
        min_outer := op(args(i), 2);
      elif (op(args(i), 1) = hold(maxouter)) then
        max_outer := op(args(i), 2);
      end_if;
    end_for;
  end_if;
  if (nb_count) then
    muEC::PART::NbSkewDiag_rec(n, max_lg, min_inner, max_inner,
                          min_outer, max_outer);
  else
    muEC::PART::ListSkewDiag_rec(n, max_lg, min_inner, max_inner,
                            min_outer, max_outer);
  end_if;
end_proc:

/*
|  VP300697
|  Type Checking...
*/
muEC::PART::ListSkewDiag_check := proc(n)
local
  i;        /* var for loop... */
begin
  if not (testtype(n,DOM_INT) and (n >=0)) then
    muEC::ERR::PARTPerror(1,procname);           /* Wrong type for first argument... */
  elif (args(0) >= 2) then
    for i from 2 to args(0) do
      if (not testtype(args(i),"_equal") and args(i) <> hold(nb)) then
        muEC::ERR::PARTPerror(2*i-1,procname);     /* Wrong type for ith argument... */
      end_if;
      if (args(i)<>hold(nb)) then
        if (op(args(i), 1) = hold(maxlg)) then
          if not (testtype(op(args(i), 2), DOM_INT) and (op(args(i), 2) >=0))
          then
            muEC::ERR::PARTPerror(2*i-1,procname); /* Wrong type for ith argument... */
          end_if;
        elif (op(args(i), 1) = hold(mininner)) then
          if (not muEC::TYP::IsPart(op(args(i), 2))) then
            muEC::ERR::PARTPerror(2*i-1,procname); /* Wrong type for ith argument... */
          end_if;
        elif (op(args(i), 1) = hold(maxinner)) then
          if (not muEC::TYP::IsPart(op(args(i), 2))) then
            muEC::ERR::PARTPerror(2*i-1,procname); /* Wrong type for ith argument... */
          end_if;
        elif (op(args(i), 1) = hold(minouter)) then
          if (not muEC::TYP::IsPart(op(args(i), 2))) then
            muEC::ERR::PARTPerror(2*i-1,procname); /* Wrong type for ith argument... */
          end_if;
        elif (op(args(i), 1) = hold(maxouter)) then
          if (not muEC::TYP::IsPart(op(args(i), 2))) then
            muEC::ERR::PARTPerror(2*i-1,procname); /* Wrong type for ith argument... */
          end_if;
        else
          muEC::ERR::PARTPerror(2*i,procname);    /* Wrong range for ith argument... */
        end_if;
      end_if;
    end_for;
  end_if;
end_proc:

/*
|  VP300697
*/
muEC::PART::ListSkewDiag := proc(n)
begin
  if testargs() then
    muEC::PART::ListSkewDiag_check(args());
  end_if;
  muEC::PART::ListSkewDiag_core(args());
end_proc:
