/*
|  File Part2Bdr (PART PACKAGE) Lundi 30 juin 1997 18:03:05
|
|  Part2Border(lambda) returns a list of 0 and 1, 0 denoting an horizontal
|                      chunk of the border of lambda, 1 a vertical one.
|                      Extremity segments are not included (i.e. 1 on the left
|                      top, 0 on the right bottom).
|                      Lambda is in decreasing order.
|
|  Part2Border(lambda,m) returns the same, m[1] denoting an horizontal chunk of
|                        the border of lambda, m[2] a vertical one.
*/

/*
|  VP300697
*/
muEC::PART::Part2Border_int := proc(lambda)
local
  i,   /* var for loop...        */
  h,   /* horizontal encoding... */
  v,   /* vertical encoding...   */  
  n,   /* length of lambda...    */
  lam, /* misc...                */
  res; /* the result...          */
begin
  if (args(0) = 2) then
    h :=op(args(2), 1);
    v :=op(args(2), 2);
  else
    h := 0;
    v := 1;
  end_if;
  n := nops(lambda);
  if (n <> 0) then
    lam := [op(lambda, 1), op(lambda), 0]; 
  else
    lam := [];
  end_if;
  res := null();
  for i from n+1 downto 2 step 1 do
    res := res, h $ j=1..lam[i]-lam[i+1], v;
  end_for;
  [res];
end_proc:

/*
|  VP300697
*/
muEC::PART::Part2Border_core := proc(lambda)
local
  i,      /* var for loop... */
  alphbt; /* the alphabet... */
begin
  alphbt := null(); /* default value... */
  if (args(0) >= 2) then
    for i from 2 to args(0) do
      if (op(args(i), 1) = hold(alphabet)) then
        alphbt := op(args(i), 2);
      end_if;
    end_for;
  end_if;
  muEC::PART::Part2Border_int(lambda,alphbt);
end_proc:

/*
|  VP300697
|  Type Checking.
*/
muEC::PART::Part2Border_check := proc(lambda)
begin
  if (not muEC::TYP::IsPart(lambda)) then
      muEC::ERR::PARTPerror(1,procname);          /* Wrong type 1st argument... */
  elif (args(0) >= 2) then
    if (testtype(args(2),"_equal")) then
      if (op(args(2), 1) = hold(alphabet)) then
        if (not testtype(op(args(2), 2), DOM_LIST)) then
          muEC::ERR::PARTPerror(4,procname);
        elif (nops(op(args(2), 2))<>2) then
          muEC::ERR::PARTPerror(4,procname);
        end_if;
      else
        muEC::ERR::PARTPerror(4,procname);
      end_if;
    else
      muEC::ERR::PARTPerror(3,procname);
    end_if;
  end_if;
end_proc:

/*
|  VP300697
*/
muEC::PART::Part2Border := proc(lambda)
begin
  if testargs() then
    muEC::PART::Part2Border_check(args());
  end_if;
  muEC::PART::Part2Border_core(args());
end_proc:
