/*
|  File PartOrdr (PART PACKAGE) Wed Apr 22 09:25:22 MET DST 1998
*/
/*
|  CompPart(part1, part2, kind) tests whether part1<=part2 for the given order.
|                               kind=hold(natural), hold(lexic), hold(cixel),
|                               Without a third argument, kind is assumed to be
|                               hold(natural).
|
|  PartOrderMat(n, kind) returns the order matrix on all partitions of
|                        ListPart(n) for the given order. 
|                        kind=hold(natural), hold(lexic), hold(cixel),
|                        Without a third argument, kind is assumed to be
|                       hold(natural).
|  VP220498
*/
muEC::PART::CompPart_natural := proc(part1, part2)
local   
    sum1, /* cumulated sum... */
    sum2, /* cumulated sum... */
    i,    /* var for seq and loop... */
    n;    /* nops(part1)... */
begin
  n := max(nops(part1),nops(part2));
  sum1 := [_seqgen(_plus(op(part1,1..i)),i,1..nops(part1))];
  if sum1=[] then
    sum1 := [0$n];
  else
    sum1 := [op(sum1),sum1[nops(part1)]$n-nops(part1)];
  end_if;
  sum2 := [_seqgen(_plus(op(part2,1..i)),i,1..nops(part2))];
  if sum2=[] then
    sum2 := [0$n];
  else
    sum2 := [op(sum2),sum2[nops(part2)]$n-nops(part2)];
  end_if;
  for i from 1 to n do
    if (sum1[i]>sum2[i]) then
      return(FALSE);
    end_if;
  end_for;
  TRUE;
end_proc:
/*
|  VP220498
*/
muEC::PART::CompPart_interface := proc(part1, part2, kind)
local
    i,  /* var for loop... */
    n,  /* var... */
    n1, /* nops(part1)... */
    n2; /* nops(part2)... */
begin
  n1 := nops(part1);
  n2 := nops(part2);
  if (kind=hold(lexic)) then
    n := min(n1,n2);
    for i from 1 to n do
      if (part1[i]>part2[i]) then
        return(TRUE);
      elif (part2[i]>part1[i]) then
        return(FALSE);
      end_if;
    end_for;
    bool(n1>=n2);
  elif (kind=hold(cixel)) then
    n:=min(n1,n2);
    for i from 0 to n-1 do
      if (part1[n1-i]>part2[n2-i]) then
        return(TRUE);
      elif (part2[n2-i]>part1[n1-i]) then
        return(FALSE);
      end_if;
    end_for;
    bool(n1>=n2);
  elif (kind=hold(natural)) then
    muEC::PART::CompPart_natural(part1, part2);
  end_if;
end_proc:
/*
|  VP220498
*/
muEC::PART::CompPart_core := proc(part1, part2)
begin
  if args(0)>2 then
    muEC::PART::CompPart_interface(args());
  else
    muEC::PART::CompPart_interface(args(), hold(natural));
  end_if;
end_proc:
/*
|  type checking
|  VP220498
*/
muEC::PART::CompPart_check := proc(part1, part2)
begin
  if not muEC::TYP::IsPart(part1) then
    muEC::ERR::PARTPerror(1,procname);
  elif not muEC::TYP::IsPart(part2) then
    muEC::ERR::PARTPerror(3,procname);
  elif args(0)>2 then
    if not contains({hold(natural), hold(lexic), hold(cixel)}, args(3)) then
      muEC::ERR::PARTPerror(6,procname);
    end_if;
  end_if;
end_proc:
/*
|  VP220498
*/
muEC::PART::CompPart := proc(part1,part2)
begin
  if testargs() then
    muEC::PART::CompPart_check(args());
  end_if;
  muEC::PART::CompPart_core(args());
end_proc:
/*
|  VP220498
*/
muEC::PART::PartOrderMat_interface := proc(n, kind)
local
    l,  /* ListPart(n)... */
    nb, /* ListPart(n,nb)... */
    i,  /* var for seq... */
    j;  /* var for seq... */
begin
  l := muEC::PART::ListPart(n);
  nb := nops(l);
  subs(
  array(1..nb, 1..nb,
    [_seqgen([_seqgen(muEC::PART::CompPart(l[i],l[j],kind),j,1..nb)],i,1..nb)]),
  TRUE=1, FALSE=0);
end_proc:
/*
|  VP220498
*/
muEC::PART::PartOrderMat_core := proc(n)
begin
  if args(0)>1 then
    muEC::PART::PartOrderMat_interface(args());
  else
    muEC::PART::PartOrderMat_interface(n, hold(natural));
  end_if;
end_proc:
/*
|  type checking
|  VP220498
*/
muEC::PART::PartOrderMat_check := proc(n)
begin
 if testtype(n,Type::NegInt) then
   muEC::ERR::PARTPerror(1,procname); /* Wrong type for first argument... */
 end_if;    
 if args(0)>1 then
   if not contains({hold(natural), hold(lexic), hold(cixel)},args(2)) then
     muEC::ERR::PARTPerror(3,procname); /* Wrong type for second argument... */
   end_if;    
 end_if;
end_proc:  
/*
|  VP220498
*/
muEC::PART::PartOrderMat := proc(n)
begin
  if testargs() then
    muEC::PART::PartOrderMat_check(args());
  end_if;
  muEC::PART::PartOrderMat_core(args());
end_proc:
