/*
|  File Monomial (SFA PACKAGE) Sun Apr 26 15:36:51 MET DST 1998
*/

/*
|  MonomialA(monom, bases) returns a table whose entries describe the sets of
|                       variables from bases that occur in monom. 
|                       bases may be a list or set of string names.
|                       If the second argument is a string --name of a basis--,
|                       then the "function" assumes that monom is expressed on
|                       this basis.
|                       An entry hold(coeff) is reserved for the rest of the
|                       monomial in the table.
|
|  MonomialA(monom, bases, alist) returns the same, but only functions on
|                                 alphabets in alist are considered: the others
|                                 are taken as scalars and will appear in the
|                                 hold(coeff) entry.
|                                 MonomialA(monom, bases, ["*"]) is
|                                 equivalent to MonomialA(monom,bases).
| VP260698
*/

/*
|  Inserts a dependent alphabet expression into tbases
|  To be used in muEC::SFA::Monom_Almost and muEC::SFA::MonomialA...
|  VP260698
*/
muEC::SFA::InsertAlphab := proc(t, alphab, monom)
begin
  if t=1  or not contains(t, alphab) then
     monom;
  else
     t[alphab] * monom;
  end_if;
end_proc:

/*
|  Tests whether the monom is indexed by a non-partition.
|  To be used in muEC::SFA::MonomialA...
|  VP260498
*/
muEC::SFA::Monom_Almost := proc(monom, base, tbases)
local
  part,        /* a partition...                  */
  IsAlmostFun, /* IsAlmost?A to be called...      */
  verymonom,   /* a monom...                      */
  monombis,    /* another one...                  */
  nmonom,      /* another one...                  */
  alphab,      /* a formal alphabet expression... */
  pow,         /* a power...                      */
  sig,         /* a signature...                  */
  IsAlmstFlg;  /* op(result,1) of main call...    */
begin
  sig := 1;              /* default value... */
  nmonom := 1;           /* default value... */
  IsAlmstFlg := TRUE;    /* default value... */
  if has(muEC::SFA::SFABases,base) then
    IsAlmostFun := muEC::cat("muEC::TYP::IsAlmost", base, "A");
    if testtype(monom, "_power") then
      monombis := op(monom,1);
      pow := op(monom,2);
    else
      monombis := monom;
      pow := 1;
    end_if;
    if testtype(monombis,"function") and testtype(op(monombis,0),"_index") then
      verymonom := op(monombis,0);
      alphab := op(monombis);
    end_if;
    if level(eval(IsAlmostFun(monombis)),2) then
      if base=hold(s) then
        part := muEC::SYMF::RedressVect([op(verymonom,2..nops(verymonom))]);
        sig := op(part,2); part := op(part,1);
        if (sig <> 0) then
          sig := sig^pow;
        end_if;
      else
        part := sort(subs([op(verymonom,2..nops(verymonom))],0=null()),
                     proc(x,y) begin x>y end_proc );
      end_if;
      nmonom := muEC::SFA::InsertAlphab(tbases, alphab, base[op(part)]^pow);
    else /* return default values... */
      IsAlmstFlg := FALSE;
    end_if;
  /* else */  /* another base: impossible... */
  end_if;
  [IsAlmstFlg, sig, nmonom];
end_proc:

/*
|  VP260498
*/
muEC::SFA::MonomialA := proc(monom, bases)
local
  alist, /* list of alphabets to be treated...      */
  testA, /* a test "function" on alphabets...       */
  test,  /* the test "function"... IshA, IssA, ...  */
  getv,  /* the current element has been treated... */
  v,v1,  /* current elements...                     */
  i,j,   /* variables for loops...                  */
  k,     /* variable for seq...                     */
  Alst,  /* result of muEC::SFA::Monom_Almost call...     */
  Mnul,  /* test wether monom is null...            */
  t;     /* the table...                            */
begin
  if (args(0) >= 3) then /* this is the list of valid alphabets... */
    alist := args(3);
  else
    alist := ["*"]; /* default is all alphabets... */
  end_if;
  testA := proc(alph, alist) begin
           if nops(alph) <> 1 then FALSE;
           else has(alist,op(alph)) or bool(alist=["*"]);
           end_if;
           end_proc:
  if testtype(bases, DOM_IDENT) then
    /* we init the table... */
    t := table(hold(coeff) = 1, bases = 1);
    test := muEC::cat("muEC::TYP::Is", bases, "A");
    /* we test the structure of the monomial... */
    if testtype(monom,"_mult") then
      for i from 1 to nops(monom) do
        v := op(monom,i);
        if testtype(v, "_power") then
          if not testA([op(op(v,1))], alist) then
            t[hold(coeff)] := t[hold(coeff)] * v;
          elif level(eval(test(op(v,1))),2) then
            t[bases][op(op(v,1))] := muEC::SFA::InsertAlphab(t[bases], op(op(v,1)),
                                                   op(op(v,1),0)^(op(v,2)));
          else
            Alst := muEC::SFA::Monom_Almost(v, bases, t[bases]);
            Mnul := op(Alst,2); /* sign */
            if Mnul=0 then return(table(hold(coeff) = 0)); end_if;
            if not op(Alst,1) then
              t[hold(coeff)] := t[hold(coeff)] * v;
            else
              t[hold(coeff)] := Mnul*t[hold(coeff)];
              t[bases][op(op(v,1))] := op(Alst,3);
            end_if;
          end_if;
        else
          if not testA([op(v)], alist) then
            t[hold(coeff)] := t[hold(coeff)] * v;
          elif level(eval(test(v)),2) then
            t[bases][op(v)] := muEC::SFA::InsertAlphab(t[bases], op(v), op(v,0));
          else
            Alst := muEC::SFA::Monom_Almost(v, bases, t[bases]);
            Mnul := op(Alst,2); /* sign */
            if Mnul=0 then return(table(hold(coeff) = 0)); end_if;
            if not op(Alst,1) then
              t[hold(coeff)] := t[hold(coeff)] * v;
            else
              t[hold(coeff)] := Mnul*t[hold(coeff)];
              t[bases][op(v)] := op(Alst,3);
            end_if;
          end_if;
        end_if;
      end_for;
    elif (testtype(monom,"_power")) then
      if not testA([op(op(monom,1))], alist) then
        t[hold(coeff)] := t[hold(coeff)] * monom;
      elif level(eval(test(op(monom,1))),2) then
        t[bases][op(op(monom,1))] := muEC::SFA::InsertAlphab(t[bases],
                                             op(op(monom,1)),
                                             op(op(monom,1),0)^(op(monom,2)));
      else
        Alst := muEC::SFA::Monom_Almost(monom, bases, t[bases]);
        Mnul := op(Alst,2); /* sign */
        if Mnul=0 then return(table(hold(coeff) = 0)); end_if;
        if not op(Alst,1) then
          t[hold(coeff)] := t[hold(coeff)] * monom;
        else
          t[hold(coeff)] := Mnul*t[hold(coeff)];
          t[bases][op(op(monom,1))] := op(Alst,3);
        end_if;
      end_if;
    else
      /* the monomial is made of one term... */
      if not testA([op(monom)], alist) then
        t[hold(coeff)] := t[hold(coeff)] * monom;
      elif level(eval(test(monom)),2) then
        t[bases][op(monom)] := muEC::SFA::InsertAlphab(t[bases],
                                                 op(monom), op(monom,0));
      else
        Alst := muEC::SFA::Monom_Almost(monom, bases, t[bases]);
        Mnul := op(Alst,2); /* sign */
        if Mnul=0 then return(table(hold(coeff) = 0)); end_if;
        if not op(Alst,1) then
          t[hold(coeff)] := t[hold(coeff)] * monom;
        else
          t[hold(coeff)] := Mnul*t[hold(coeff)];
          t[bases][op(monom)] := op(Alst,3);
        end_if;
      end_if;
    end_if;
  else
    /* not only one basis specified...          */
    /* we test the structure of the monomial... */
    /* we init the table...                     */
    t := table(hold(coeff)=1, op(map(bases,proc(b) begin b=1 end_proc )));
    if testtype(monom, "_mult") then
      for i from 1 to nops(monom) do
        v := op(monom,i);
        if testtype(v,"_power") then v1 := op(v,1); else v1 := v; end_if;
        getv := FALSE;
        for j in bases do
          test := muEC::cat("muEC::TYP::Is", j, "A");
          if not testA([op(v1)], alist) then
            t[hold(coeff)] := t[hold(coeff)] * v;
            getv := TRUE;
            break;
          elif level(eval(test(v1)),2) then
            getv := TRUE;
            if (testtype(v, "_power")) then
              t[j][op(v1)]:=muEC::SFA::InsertAlphab(t[j], op(v1), op(v1,0)^op(v,2));
            else
              t[j][op(v1)]:=muEC::SFA::InsertAlphab(t[j], op(v1), op(v1,0));
            end_if;
            break;
          end_if;
        end_for;
        if (not getv) then
          Alst := [muEC::SFA::Monom_Almost(v, op(bases,k),t[op(bases,k)])
                   $ k=1..nops(bases)];
          /* splitting Alst into 3 lists... */
          Alst := [map(Alst,op,1),map(Alst,op,2),map(Alst,op,3)];
          if not _or(op(Alst[1])) then
            t[hold(coeff)] := t[hold(coeff)] * v;
          elif _or(op(map(Alst[2],proc(x) begin bool(x=0) end_proc ))) then
            return(table(hold(coeff) = 0));
          else
            j := contains(Alst[1],TRUE);
            t[bases[j]][op(v1)] := Alst[3][j];
            t[hold(coeff)] := Alst[2][j]*t[hold(coeff)];
          end_if;
        end_if;
      end_for;
    elif (testtype(monom,"_power")) then
      v1 := op(monom,1);
      getv := FALSE;
      for j in bases do
        test := muEC::cat("muEC::TYP::Is", j, "A");
        if not testA([op(v1)], alist) then
          t[hold(coeff)] := t[hold(coeff)] * monom;
          getv := TRUE;
          break;
        elif level(eval(test(v1)),2) then
          getv := TRUE;
          t[j][op(v1)]:=muEC::SFA::InsertAlphab(t[j], op(v1), op(v1,0)^op(monom,2));
          break;
        end_if;
      end_for;
      if (not getv) then
        Alst := [muEC::SFA::Monom_Almost(monom, op(bases,k),t[op(bases,k)])
		 $ k=1..nops(bases)];
        /* splitting Alst into 3 lists... */
        Alst := [map(Alst,op,1),map(Alst,op,2),map(Alst,op,3)];
        if not _or(op(Alst[1])) then
          t[hold(coeff)] := t[hold(coeff)] * monom;
        elif _or(op(map(Alst[2],proc(x) begin bool(x=0) end_proc ))) then
          return(table(hold(coeff) = 0));
        else
          j := contains(Alst[1],TRUE);
          t[bases[j]][op(v1)] := Alst[3][j];
          t[hold(coeff)] := Alst[2][j]*t[hold(coeff)];
        end_if;
      end_if;
    else
      /* the monomial is made of one term... */
      getv := FALSE;
      for j in bases do
        test := muEC::cat("muEC::TYP::Is", j, "A");
        if not testA([op(monom)], alist) then
          t[hold(coeff)] := t[hold(coeff)] * monom;
          getv := TRUE;
          break;
        elif level(eval(test(monom)),2) then
          getv := TRUE;
          t[j][op(monom)]:=muEC::SFA::InsertAlphab(t[j], op(monom), op(monom,0));
          break;
        end_if;
      end_for;
      if (not getv) then
        Alst := [muEC::SFA::Monom_Almost(monom, op(bases,k),t[op(bases,k)])
		 $ k=1..nops(bases)];
        /* splitting Alst into 3 lists... */ 
        Alst := [map(Alst,op,1),map(Alst,op,2),map(Alst,op,3)];
        if not _or(op(Alst[1])) then
          t[hold(coeff)] := t[hold(coeff)] * monom;
        elif _or(op(map(Alst[2],proc(x) begin bool(x=0) end_proc ))) then
          return(table(hold(coeff) = 0));
        else
          j := contains(Alst[1],TRUE);
          t[bases[j]][op(monom)] := Alst[3][j];
          t[hold(coeff)] := Alst[2][j]*t[hold(coeff)];
        end_if;
      end_if;
    end_if;
  end_if;
  for j in bases do
    if (t[j] = 1) then _delete(t[j]) end_if;
  end_for;
  t;
end_proc:
