/*
|  File SfAColct (SFA PACKAGE) Mon Apr 27 12:18:09 MET DST 1998
*/

/*
|  SfACollect(sfa) groups all submonoms of a multiplicative symmetric function
|                  over a same alphabet into a single one.
|                  i.e. e[1](A1)^2 * e[2](A1) * K --> e[2,1,1](A1) * K
|
|  SfACollect(sfa, bases) does the same for the list of bases given in the
|                         second argument.
|
|  VP270498
*/
muEC::SFA::SfACollect_core := proc(sfa)
local
  i, j, l, /* vars for loop...  */
  t,       /* monomial table... */
  bases,
  res;     /* the result...     */
begin
  if args(0)<2 then
    bases := muEC::SFA::SFABases;
  else
    bases := args(2);
  end_if;
  if testtype(sfa, "_plus") then
    res := map(sfa, muEC::SFA::SfACollect_core, bases);
  else
    res := 1;
    t := muEC::SFA::MonomialA(sfa, bases);
    res := _mult(t[hold(coeff)], res,
                 _seqin(_seqin(muEC::SYMF::SfCollect(t[i][j],i)(j),
                         j,[op(op(t[i],l),1)$l=1..nops(t[i])]),
                     i,{op(op(t,l),1)$l=1..nops(t)} minus {hold(coeff)}));
  end_if;
  res;
end_proc:

/*
|  VP270498
*/
muEC::SFA::SfACollect_check := proc(sfa)
local
  i; /* var for seq... */
begin
  if args(0)>1 then
    if (not testtype(args(2),DOM_LIST)) and
       (not testtype(args(2),DOM_IDENT)) then
      muEC::ERR::SFAPerror(3,procname); /* wrong type for 2nd argument... */
    elif not _and(_seqin(has(muEC::SFA::SFABases,i),i,args(2))) then
      muEC::ERR::SFAPerror(3,procname); /* wrong type for 2nd argument... */
    end_if;
  end_if;
end_proc:

/*
|  VP270498
*/
muEC::SFA::SfACollect := proc(sfa)
begin
  if testargs() then
    muEC::SFA::SfACollect_check(args());
  end_if;
  muEC::SFA::SfACollect_core(args());
end_proc:
