/*
|  File SfAExpn (SFA PACKAGE) Tue Apr 28 22:22:32 MET DST 1998
*/

/*
|  SfAExpand(sfa) converts subterms in sfa that are on polynomial alphabet
|                 expressions into terms on one single alphabet.
|
|  The following formulae are used (some of them should be optimized later):
|
|  p[I](-A) = (-1)^size(I) p[I](A)
|  e[I](-A) = (-1)^|I|     h[I](A)
|  h[I](-A) = (-1)^|I|     e[I](A)
|  s[I](-A) = (-1)^|I|     s[I~](A)
|  m[I](-A) = TopA(m[I](A))
|
|  p[I](AB) = p[I](A) p[I](B)
|  e[I](AB) = (-1)^|I| product{i=1..size(I)}
|                        sum{|J|=I.i} (-1)^|J| ToeA(m[J](A)) e[J](B)
|  h[I](AB) = product{i=1..size(I)}
|               sum{|J|=I.i} TohA(m[J](A)) h[J](B)
|  s[I](AB) = sum{|J|=|I|} s[I*J](A) s[J](B) where * = SfInternal
|  m[I](AB) = TopA(m[I](AB))
|
|  For  any k <> 0, 1 :
|  p[I](kA) = k^(size(I)) p[I](A)
|  e[I](kA) = (-1)^|I| product{i=1..size(I)}
|                        sum{|J|=I.i} (-1)^|J| m[J](k) h[J](A)
|  h[I](kA) = product{i=1..size(I)}
|               sum{|J|=I.i} m[J](k) h[J](A)
|  s[I](kA) = sum{|J|=|I|} s[I*J](k) s[J](A) where * = SfInternal
|  m[I](kA) = TopA(m[I](kA))
|
|  p[I](A+B) = product{i=1..size(I)} p[I.i](A) + p[I.i](B)
|  e[I](A+B) = product{i=1..size(I)}
|                sum{j=0..I.i} e[I.i-j](A) e[j](B)
|  h[I](A+B) = product{i=1..size(I)}
|                sum{j=0..I.i} h[I.i-j](A) h[j](B)
|  s[I](A+B) = sum{J in I} s[J](B) SfDiff(s[J], s[I])(A)
|  m[I](A+B) = sum{I' U I"=I} m[I'](A) m[I"](B)
|
|  For  any k <> 0, 1 :
|  p[I](k) = k^size(I)
|  e[I](k) = product{i=1..size}
|              k(k-1)(k-2)..(k-I.i+1)_I.i!
|  h[I](k) = product{i=1..size}
|              k(k+1)(k+2)..(k+I.i-1)_I.i!
|  m[I](k) = product{i=1..size}
|              k(k-1)(k-2)..(k-n+1)_(a1!a2!..an!) with I=1^a1..1^an
|  s[I](k) = product{i=1..size} ld[i]*(i-dp+k) /
|            product(hooks(I))    with (dp, ld) = Part2Diagonal(I)
|
|  VP280498 VP080798 VP291098
*/
muEC::SFA::SfAExpand_core := proc(sfa)
local
  part,   /* indexing partition...   */
  alphab, /* alphabet...             */
  base,   /* a basis...              */
  lp,     /* a list of partitions... */
  l,      /* a list of lists...      */
  i,      /* var for loop and seq... */
  j,      /* var for loop...         */
  k,      /* var for seq...          */
  f,      /* a function...           */
  v,      /* var...                  */
  ra,     /* remaining alphabet...   */
  t;      /* a table...              */
//option remember;
begin
  if testtype(sfa, "_plus") then
    map(sfa, muEC::SFA::SfAExpand_core);
  elif testtype(sfa, "_mult") then
    t := muEC::SFA::MonomialA(sfa, muEC::SFA::SFABases);
    for i in {op(op(t,l),1)$l=1..nops(t)} minus {hold(coeff)} do
      if (i <> hold(coeff)) then
        for j in [op(op(t[i],l),1)$l=1..nops(t[i])] do
          if testtype(t[i][j], "_plus") then
            f :=  muEC::cat("muEC::SFA::To",i,"A");
            t[hold(coeff)] := t[hold(coeff)] *
               level(eval(f(expand(map(muEC::SFA::SfAExpand_core, t[i][j](j))))),2);
          else
            t[hold(coeff)] := t[hold(coeff)] * muEC::SFA::SfAExpand_core(t[i][j](j));
          end_if;
        end_for;
      end_if;
    end_for;
    expand(t[hold(coeff)]);
  elif testtype(sfa, "_power") then
    base := op(op(op(sfa,1),0),1);
    if (base=hold(p) or base =hold(e) or base=hold(h)) then
      /* To_A is much faster than SfACollect... */
      f := muEC::cat("muEC::SFA::To",base,"A");
      muEC::SFA::SfAExpand_core(level(eval(f(sfa)),2));
    elif (testtype(op(sfa,2), Type::PosInt)
         and (base=hold(s) or base=hold(m))) then
      f := muEC::cat("muEC::SYMF::To",base);
      muEC::SFA::SfAExpand_core(level(eval(f(op(op(sfa,1),0)^op(sfa,2))),2)(op(op(sfa,1))));
    else
      sfa;
    end_if;
  elif (testtype(sfa, "function") and (muEC::TYP::IspA(sfa) or muEC::TYP::IseA(sfa) or
       muEC::TYP::IshA(sfa) or muEC::TYP::IssA(sfa) or muEC::TYP::IsmA(sfa))) then
    part   := [op(op(sfa,0),2..nops(op(sfa,0)))];
    base   := op(op(sfa,0),1);
    alphab := op(sfa);
    if (part=[]) then
      if (base=hold(p)) then /* can not handle such a case... */
        print(Unquoted, "Warning: p[](...) can not be expanded!");
        sfa;
      else
       1;
      end_if;
    elif testtype(alphab, "_power") then
      if testtype(op(alphab,2), Type::PosInt) then
        if (base=hold(p)) then
          muEC::SFA::SfAExpand_core(hold(p)[op(part)](op(alphab,1))) *
          muEC::SFA::SfAExpand_core(hold(p)[op(part)](op(alphab,1)^(op(alphab,2)-1)));
        elif (base=hold(e)) then
          v := null();
          for i from 1 to nops(part) do
            lp := muEC::PART::ListPart(part[i]);
            v := v,
                _plus((-1)^(_plus(op(lp[j])))*
                muEC::SFA::SfAExpand_core(muEC::SFA::ToeA(hold(m)[op(lp[j])](op(alphab,1)))) *
                muEC::SFA::SfAExpand_core(hold(e)[op(lp[j])](op(alphab,1)^(op(alphab,2)-1)))
                $ j = 1..nops(lp));
          end_for;
          (-1)^(_plus(op(part)))*muEC::SFA::ToeA(expand(_mult(v)));
        elif (base=hold(h)) then
          v := null();
          for i from 1 to nops(part) do
            lp := muEC::PART::ListPart(part[i]);
            v := v,
                _plus(
                muEC::SFA::SfAExpand_core(muEC::SFA::TohA(hold(m)[op(lp[j])](op(alphab,1)))) *
                muEC::SFA::SfAExpand_core(hold(h)[op(lp[j])](op(alphab,1)^(op(alphab,2)-1)))
                $ j = 1..nops(lp));
          end_for;
          muEC::SFA::TohA(expand(_mult(v)));
        elif (base=hold(s)) then
          lp := muEC::PART::ListPart(_plus(op(part)));
          expand(_plus(
           muEC::SFA::Sf2SfA(muEC::SYMF::SfInternal(hold(s)[op(part)], hold(s)[op(lp[i])], hold(s)),
                           op(alphab,1))
            * muEC::SFA::SfAExpand_core(hold(s)[op(lp[i])](op(alphab,1)^(op(alphab,2)-1)))
                       $ i=1..nops(lp)));
        elif (base=hold(m)) then
          muEC::SFA::SfAExpand_core(muEC::SFA::TopA(sfa));
        end_if;
      elif testtype(op(alphab,2),Type::NegInt)
           or (testtype(op(alphab,2),"_mult") and op(sign(op(alphab,2)),2)=-1)
           then
/*  This is a fraction: we put the result on the p-basis... */
        if (base=hold(p)) then
          muEC::SFA::SfAExpand(hold(p)[op(part)](numer(alphab)))/
              _mult(subs(denom(alphab),
                                 map(muEC::SFA::SFAVars, proc(x) begin x=x^part[i] end_proc ))
                    $ i=1..nops(part));
        else
          muEC::SFA::SfAExpand(muEC::SFA::TopA(sfa, [alphab]));
        end_if;
      elif muEC::TYP::IsConstant(alphab) then
/*  The power is a constant... */
        if (base=hold(p)) then
          alphab^nops(part);
        elif (base=hold(e)) then
          _mult(_mult(alphab-j $ j=0..part[i]-1)/fact(part[i])
                $ i=1..nops(part));
        elif (base=hold(h)) then
          _mult(_mult(alphab+j $ j=0..part[i]-1)/fact(part[i])
                $ i=1..nops(part));
        elif (base=hold(s)) then
          _mult(alphab-i+j $ j=1..part[i] $ i=1..nops(part))/
          _mult(op(muEC::PART::Part2ListHook(part)));
        elif (base=hold(m)) then
          v := muEC::PART::Part2Exp(part);
          _mult(alphab-i $ i=0..nops(v)-1)/_mult(op(map(v,fact)));
        end_if;
      else
        sfa;
      end_if;
    elif testtype(alphab, "_mult") then
      if testtype(op(alphab,1), Type::NegInt) then
/*  Negative alphabet... */
        if (base=hold(p)) then
          (-1)^(nops(part)) * muEC::SFA::SfAExpand_core(hold(p)[op(part)](-alphab));
        elif (base=hold(e)) then
          (-1)^(_plus(op(part))) * muEC::SFA::SfAExpand_core(hold(h)[op(part)](-alphab));
        elif (base=hold(h)) then
          (-1)^(_plus(op(part))) * muEC::SFA::SfAExpand_core(hold(e)[op(part)](-alphab));
        elif (base=hold(s)) then
          (-1)^(_plus(op(part))) *
          muEC::SFA::SfAExpand_core(hold(s)[op(muEC::PART::Part2Conjugate(part))](-alphab));
        elif (base=hold(m)) then
          muEC::SFA::SfAExpand_core(muEC::SFA::TopA(sfa));
        end_if;
      elif testtype(op(alphab,1), "_power") then
/*  Product with a power as first term... */
        ra := _mult(op(alphab,2..nops(alphab)));
        if testtype(op(op(alphab,1),2), Type::PosInt) then
          if (base=hold(p)) then
            muEC::SFA::SfAExpand_core(hold(p)[op(part)](op(op(alphab,1),1))) *
            muEC::SFA::SfAExpand_core(hold(p)[op(part)](
                           op(op(alphab,1),1)^(op(op(alphab,1),2)-1)*ra));
          elif (base=hold(e)) then
            v := null();
            for i from 1 to nops(part) do
              lp := muEC::PART::ListPart(part[i]);
              v := v,
                  _plus((-1)^(_plus(op(lp[j])))*
                  muEC::SFA::SfAExpand_core(muEC::SFA::ToeA(
                     hold(m)[op(lp[j])](op(op(alphab,1),1)))) *
                  muEC::SFA::SfAExpand_core(hold(e)[op(lp[j])](
                            op(op(alphab,1),1)^(op(op(alphab,1),2)-1)*ra))
                        $ j = 1..nops(lp));
            end_for;
            (-1)^(_plus(op(part)))*muEC::SFA::ToeA(expand(_mult(v)));
          elif (base=hold(h)) then
            v := null();
            for i from 1 to nops(part) do
              lp := muEC::PART::ListPart(part[i]);
              v := v,
                  _plus(
                        muEC::SFA::SfAExpand_core(muEC::SFA::TohA(hold(m)[op(lp[j])](
                                                    op(op(alphab,1),1)))) *
                        muEC::SFA::SfAExpand_core
                        (hold(h)[op(lp[j])](op(op(alphab,1),1)^(op(op(alphab,1),2)-1)*ra))
                        $ j = 1..nops(lp));
            end_for;
            muEC::SFA::TohA(expand(_mult(v)));
          elif (base=hold(s)) then
            lp := muEC::PART::ListPart(_plus(op(part)));
            expand(_plus(
              muEC::SFA::Sf2SfA(muEC::SYMF::SfInternal(hold(s)[op(part)], hold(s)[op(lp[i])], hold(s)),
                             op(op(alphab,1),1))
              * muEC::SFA::SfAExpand_core(hold(s)[op(lp[i])](op(op(alphab,1),1)^
                                                  (op(op(alphab,1),2)-1)*ra))
                         $ i=1..nops(lp)));
          elif (base=hold(m)) then
            muEC::SFA::SfAExpand_core(muEC::SFA::TopA(sfa));
          end_if;
        elif testtype(op(alphab,[1,2]),Type::NegInt)
             or (testtype(op(alphab,[1,2]),"_mult") and
                 op(sign(op(alphab,[1,2])),2)=-1)
             then
/*  This is a fraction: we put the result on the p-basis... */
          if (base=hold(p)) then
            muEC::SFA::SfAExpand(hold(p)[op(part)](numer(op(alphab,1))))/
                _mult(subs(denom(op(alphab,1)),
                           map(muEC::SFA::SFAVars, proc(x) begin x=x^part[i] end_proc ))
                      $ i=1..nops(part))*
            muEC::SFA::SfAExpand(hold(p)[op(part)](op(alphab,2..nops(alphab))));
          else
            muEC::SFA::SfAExpand(muEC::SFA::TopA(sfa, [alphab]));
          end_if;
        else
          sfa;
        end_if;
      elif muEC::TYP::IsAlphabet_simple(op(alphab,1)) or
           muEC::TYP::IsVariable(op(alphab,1)) then
/*  Product of two alphabets... */
        if (base=hold(p)) then
          muEC::SFA::SfAExpand_core(hold(p)[op(part)](op(alphab,1))) *
          muEC::SFA::SfAExpand_core(hold(p)[op(part)]
                               (_mult(op(alphab,2..nops(alphab)))));
        elif (base=hold(e)) then
          v := null();
          for i from 1 to nops(part) do
            lp := muEC::PART::ListPart(part[i]);
            v := v,
                _plus(
                      (-1)^(_plus(op(lp[j])))*
                      muEC::SFA::SfAExpand_core(muEC::SFA::ToeA(hold(m)[op(lp[j])](op(alphab,1)))) *
                      muEC::SFA::SfAExpand_core(hold(e)[op(lp[j])]
                                                (_mult(op(alphab, 2..nops(alphab)))))
                      $ j = 1..nops(lp));
          end_for;
          (-1)^(_plus(op(part)))*muEC::SFA::ToeA(expand(_mult(v)));
        elif (base=hold(h)) then
          v := null();
          for i from 1 to nops(part) do
            lp := muEC::PART::ListPart(part[i]);
            v := v,
                _plus(
                      muEC::SFA::SfAExpand_core(muEC::SFA::TohA(hold(m)[op(lp[j])](op(alphab,1)))) *
                      muEC::SFA::SfAExpand_core(hold(h)[op(lp[j])]
                                                (_mult(op(alphab,2..nops(alphab)))))
                      $ j = 1..nops(lp));
          end_for;
          muEC::SFA::TohA(expand(_mult(v)));
        elif (base=hold(s)) then
          lp := muEC::PART::ListPart(_plus(op(part)));
          expand(_plus(
            muEC::SFA::Sf2SfA(muEC::SYMF::Tos(
                        muEC::SYMF::SfInternal(hold(s)[op(part)], hold(s)[op(lp[i])], hold(s))),
                           op(alphab,1))
            * muEC::SFA::SfAExpand_core(hold(s)[op(lp[i])](_mult(op(alphab,2..nops(alphab)))))
                       $ i=1..nops(lp)));
        elif (base=hold(m)) then
          muEC::SFA::SfAExpand_core(muEC::SFA::TopA(sfa));
        end_if;
      elif muEC::TYP::IsConstant(op(alphab,1)) then
/*  Product of an alphabet by a numeric/formal constant */
        if (base=hold(p)) then
          op(alphab,1)^(nops(part)) *
          muEC::SFA::SfAExpand_core(hold(p)[op(part)](_mult(op(alphab,2..nops(alphab)))));
        elif (base=hold(e)) then
          v := null();
          for i from 1 to nops(part) do
            lp := muEC::PART::ListPart(part[i]);
            l  := [muEC::PART::Part2Exp(lp[j]) $ j=1..nops(lp)];
            v  := v,
            _plus((-1)^(part[i])*
                  _mult(op(alphab,1)-k $ k=0.._plus(op(l[j]))-1)/
                  _mult(op(map(l[j],fact)))
                  * muEC::SFA::SfAExpand_core(hold(e)[op(lp[j])](_mult(op(alphab,2..nops(alphab)))))
                  $ j = 1..nops(lp));
          end_for;
          (-1)^(_plus(op(part)))*muEC::SFA::ToeA(expand(_mult(v)));
        elif (base=hold(h)) then
          v := null();
          for i from 1 to nops(part) do
            lp := muEC::PART::ListPart(part[i]);
            l  := [muEC::PART::Part2Exp(lp[j]) $ j=1..nops(lp)];
            v  := v,
            _plus( /* of multinomials... */
               _mult(op(alphab,1)-k $ k=0.._plus(op(l[j]))-1) /
               _mult(op(map(l[j],fact)))
              * muEC::SFA::SfAExpand_core(hold(h)[op(lp[j])](_mult(op(alphab,2..nops(alphab)))))
                  $ j=1..nops(lp));
          end_for;
          muEC::SFA::TohA(expand(_mult(v)));
        elif (base=hold(s)) then
          v := _plus(op(part));
          lp := muEC::PART::ListPart(v);
          expand(
          _plus(subs(muEC::SYMF::SfInternal(hold(s)[op(part)], hold(s)[op(lp[i])], hold(s)),
                     hold(p)[op(j)]=op(alphab,1)^nops(j) $ j in lp)) *
                 muEC::SFA::SfAExpand_core(hold(s)[op(lp[i])]
                                          (_mult(op(alphab,2..nops(alphab)))))
                 $ i=1..nops(lp));
        elif (base=hold(m)) then
        /* Nothing better for the moment... */
          muEC::SFA::SfAExpand_core(muEC::SFA::TopA(sfa));
        end_if;
      elif (testtype(op(alphab,1), "function") and
           (muEC::TYP::IspA(op(alphab,1)) or
            muEC::TYP::IseA(op(alphab,1)) or muEC::TYP::IshA(op(alphab,1)) or
            muEC::TYP::IssA(op(alphab,1)) or muEC::TYP::IsmA(op(alphab,1)))) then
/*  Alphabet may be a product of symmetric function... */
        if (muEC::TYP::IspA(op(alphab,1)) and base=hold(p)
        and muEC::TYP::IsAlphabet_simple(op(op(alphab,1)))) then
          hold(p)[op(sort([
                     op(map([op(op(op(alphab,1),0),2..nops(op(op(alphab,1),0)))],
                            _mult,part[i]))
                     $ i=1..nops(part)],
                  proc(x,y) begin x>y end_proc ))](op(op(alphab,1)))*
          muEC::SFA::SfAExpand_core(hold(p)[op(part)]
                   (_mult(op(alphab,2..nops(alphab)))));
  
        else
          muEC::SFA::SfAExpand_core(muEC::SFA::TopA(base[op(part)]
                                     (muEC::SFA::TopA(muEC::SFA::SfAExpand_core(alphab)))));
        end_if;
      else /* donna what to do with this... */
        sfa;
      end_if;
    elif testtype(alphab, "_plus") then
/*  Sum of two alphabets... */
      if (base=hold(p)) then
       muEC::SFA::SfACollect(
       expand(_mult(
                    muEC::SFA::SfAExpand_core(hold(p)[part[i]](op(alphab,1))) +
                    muEC::SFA::SfAExpand_core(hold(p)[part[i]](_plus(op(alphab,2..nops(alphab)))))
                    $ i=1..nops(part))),
       hold(p));
      elif (base=hold(e)) then
        muEC::SFA::ToeA(expand(_mult(
	   _plus(
                muEC::SFA::SfAExpand_core(hold(e)[subs(part[i]-j,0=null())](op(alphab,1))) *
                muEC::SFA::SfAExpand_core(hold(e)[subs(j,0=null())]
                               (_plus(op(alphab,2..nops(alphab)))))
                $ j=0..part[i]) $ i=1..nops(part))
        ));
      elif (base=hold(h)) then
        muEC::SFA::TohA(expand(_mult(
        _plus(
              muEC::SFA::SfAExpand_core(hold(h)[subs(part[i]-j,0=null())](op(alphab,1))) *
              muEC::SFA::SfAExpand_core(hold(h)[subs(j,0=null())]
                                        (_plus(op(alphab,2..nops(alphab))))),
              $ j=0..part[i])
                                     $ i=1..nops(part)
        )));
      elif (base=hold(s)) then
        lp := muEC::PART::ListPartIn(part);
        expand(_plus(
                     muEC::SFA::SfAExpand_core
                     (hold(s)[op(lp[i])](_plus(op(alphab,1..nops(alphab)-1)))) *
                     muEC::SFA::SfAExpand_core((muEC::SYMF::SfDiff(hold(s)[op(lp[i])], hold(s)[op(part)]))
                                               (op(alphab, nops(alphab))))
                     $ i=1..nops(lp)));
      elif (base=hold(m)) then
         lp := muEC::PART::SplitPart(part, 2);
         _plus(
               expand(muEC::SFA::SfAExpand_core(hold(m)[op(op(lp,i)[1])](op(alphab,1))) *
                      muEC::SFA::SfAExpand_core(hold(m)[op(op(lp,i)[2])]
                                                (_plus(op(alphab,2..nops(alphab))))))
               $ i=1..nops(lp));
      end_if;
    elif muEC::TYP::IsAlphabet_simple(alphab) then
/*  Alphabet is a single alphabet Ai... */
      sfa;
    elif muEC::TYP::IsVariable(alphab) then
/*  Alphabet is a variable... */
      if (base=hold(p)) or (base=hold(h)) then
        alphab^(_plus(op(part)));
      elif (base=hold(e)) then
        if part=[] then
          1;
        elif part=[1] then
          alphab;
        else
          0;
        end_if;
      elif (base=hold(s)) or (base=hold(m)) then
        if part=[] then
          1;
        elif nops(part)=1 then
          alphab^(part[1]);
        else
          0;
        end_if;
      end_if;
    elif (testtype(alphab, "function") and
         (muEC::TYP::IspA(alphab) or muEC::TYP::IseA(alphab)
          or muEC::TYP::IshA(alphab) or muEC::TYP::IssA(alphab) or muEC::TYP::IsmA(alphab)))
    then
/*  Alphabet may be a symmetric function... */
      if (muEC::TYP::IspA(alphab) and base=hold(p)
      and muEC::TYP::IsAlphabet_simple(op(alphab,1))) then
        hold(p)[op(sort([
                   op(map([op(op(alphab,0),2..nops(op(alphab,0)))],_mult,
                          part[i]))
                   $ i=1..nops(part)],
                proc(x,y) begin x>y end_proc ))](op(alphab,1));
      else
        muEC::SFA::SfAExpand_core(muEC::SFA::TopA(base[op(part)]
                                   (muEC::SFA::TopA(muEC::SFA::SfAExpand_core(alphab)))));
      end_if;
    else
/*  Alphabet is a numerical_formal constant... */
      if (base=hold(p)) then
        alphab^nops(part);
      elif (base=hold(e)) then
        _mult(_mult(alphab-j $ j=0..part[i]-1)/fact(part[i])
              $ i=1..nops(part));
      elif (base=hold(h)) then
        _mult(_mult(alphab+j $ j=0..part[i]-1)/fact(part[i])
              $ i=1..nops(part));
      elif (base=hold(s)) then
        _mult(alphab-i+j $ j=1..part[i] $ i=1..nops(part))/
        _mult(op(muEC::PART::Part2ListHook(part)));
      elif (base=hold(m)) then
        v := muEC::PART::Part2Exp(part);
        _mult(alphab-i $ i=0..nops(part)-1)/
        _mult(op(map(v,fact)));
      end_if;
    end_if;
   else
    sfa;
  end_if;
end_proc:
/*
|  No special check is made, since sfa must be first decomposed
|  VP280498
*/ 
muEC::SFA::SfAExpand_check := proc(sfa)
begin
end_proc:

/*
|  VP280498
*/
muEC::SFA::SfAExpand := proc(sfa)
begin
  if testargs() then
    muEC::SFA::SfAExpand_check(args());
  end_if;
  muEC::SFA::SfAExpand_core(args());
end_proc:
