/**
|  File SG.mu (SG PACKAGE) Fri Oct 24 11:56:21 MET DST 1997
|
|            The 'muEC::SG' library for manipulating symmetric functions.
|
| VP241097 VP020498
**/

/**
User Functions:

muEC::SG::Code2Perm      -  Transforms a Lehmer code into the permutation
muEC::SG::CompPerm       -  Compares the two permutations
muEC::SG::Cycle2Perm     -  Transforms a cycle notation into the permutation
muEC::SG::Desc2ListPerm  -  set of permutations with  given descent set
muEC::SG::FactorPerm     -  Tests wether the permutation factorizes or not
muEC::SG::GenPerm        -  Gives the first, lastm next or previous permutation
muEC::SG::ListPerm       -  Gives all permutations of S_<arg>
muEC::SG::Mat2Perm       -  Transforms a matrix into a permutation
muEC::SG::MutlPerm       -  Composes the given permutations
muEC::SG::Perm2Betti     -  Gives the Betti polynomial of the permutation
muEC::SG::Perm2Code      -  Transforms the permutation into its Lehmer code
muEC::SG::Perm2Cycle     -  Transforms the permutation into cycle notation
muEC::SG::Perm2CycleType -  Gives the cycle type of the permutation
muEC::SG::Perm2Desc      -  Gives the list of descents of the permutation
muEC::SG::Perm2Inv       -  Gives the inverse mapping of the permutation
muEC::SG::Perm2Interval  -  Gives the inferior interval (lehmer codes are returned)
muEC::SG::Perm2Length    -  Gives the length of the permutation
muEC::SG::Perm2ListInv   -  Gives the list of all inversions of the permutation
                      (pairs of positions)
muEC::SG::Perm2ListRd    -  Reduced decompositions of the permutation
muEC::SG::Perm2Mat       -  Transforms the permutation into its matrix
muEC::SG::Perm2RRep      -  Transforms the permutation into its matrix in the
                      regular representation of the symmetric group
muEC::SG::Perm2Rd        -  Transforms the permutation into its reduced word
muEC::SG::Perm2Rise      -  Gives all rises of the permutation
muEC::SG::Perm2Rothe     -  Gives the Rothe diagram of the permutation
muEC::SG::Perm2Part      -  Gives the shape of the permutation
muEC::SG::SgCharTable    -  Gives the character table of S_n
muEC::SG::SgDimRep       -  Gives the dimension of the irreducible representation
                      of the partition given as argument
**/

/**
Algorithms for SG are mainly from my own.
**/

muEC::SG := newDomain("muEC::SG"):
muEC::SG::info := "Library SG: Calculations over Symmetric Groups.":
muEC::SG::interface := { Code2Perm, CompPerm, Cycle2Perm, Desc2ListPerm,
                     FactorPerm, GenPerm, ListPerm, Mat2Perm, MultPerm,
                     Part2SgCC, Perm2Betti, Perm2Code, Perm2Cycle,
                     Perm2CycleType, Perm2Desc, Perm2Interval, Perm2Inv,
                     Perm2Length, Perm2ListInv, Perm2ListRd, Perm2Mat,
                     Perm2RRep, Perm2Rd, Perm2Rise, Perm2Rothe, Perm2Sign,
                     // Undocumented: Perm2Part,
		     RRep2Perm, RandPerm, Rd2Perm, SgCharTable,
                     SgDimRep, SgTranspo, Vect2Perm }:

alias(path = pathname("MUEC", "SG")):

muEC::SG::Code2Perm      := loadproc(muEC::SG::Code2Perm,      path, "Cod2Perm"):
muEC::SG::CompPerm       := loadproc(muEC::SG::CompPerm,       path, "Order"   ):
muEC::SG::Cycle2Perm     := loadproc(muEC::SG::Cycle2Perm,     path, "Dc2Perm" ):
muEC::SG::Desc2ListPerm  := loadproc(muEC::SG::Desc2ListPerm,  path, "Des2LPer"):
muEC::SG::FactorPerm     := loadproc(muEC::SG::FactorPerm,     path, "Factoriz"):
muEC::SG::GenPerm        := loadproc(muEC::SG::GenPerm,        path, "GenPerm" ):
muEC::SG::ListPerm       := loadproc(muEC::SG::ListPerm,       path, "ListPerm"):
muEC::SG::Mat2Perm       := loadproc(muEC::SG::Mat2Perm,       path, "Mat2Perm"):
muEC::SG::MultPerm       := loadproc(muEC::SG::MultPerm,       path, "Apply"   ):
muEC::SG::Part2SgCC      := loadproc(muEC::SG::Part2SgCC,      path, "Part2CC" ):
muEC::SG::Perm2Betti     := loadproc(muEC::SG::Perm2Betti,     path, "Betti"   ):
muEC::SG::Perm2Code      := loadproc(muEC::SG::Perm2Code,      path, "Perm2Cod"):
muEC::SG::Perm2Cycle     := loadproc(muEC::SG::Perm2Cycle,     path, "Perm2Dc" ):
muEC::SG::Perm2CycleType := loadproc(muEC::SG::Perm2CycleType, path, "CycleTyp"):
muEC::SG::Perm2Desc      := loadproc(muEC::SG::Perm2Desc,      path, "Perm2Des"):
muEC::SG::Perm2Inv       := loadproc(muEC::SG::Perm2Inv,       path, "Invert"  ):
muEC::SG::Perm2Interval  := loadproc(muEC::SG::Perm2Interval,  path, "Interval"):
muEC::SG::PiPerm         := loadproc(muEC::SG::PiPerm,         path, "Interval"):
muEC::SG::Perm2Length    := loadproc(muEC::SG::Perm2Length,    path, "PermLgth"):
muEC::SG::Perm2ListInv   := loadproc(muEC::SG::Perm2ListInv,   path, "Inv"     ):
muEC::SG::Perm2ListRd    := loadproc(muEC::SG::Perm2ListRd,    path, "ListRd"  ):
muEC::SG::Perm2Mat       := loadproc(muEC::SG::Perm2Mat,       path, "Perm2Mat"):
muEC::SG::Perm2RRep      := loadproc(muEC::SG::Perm2RRep,      path, "Perm2RR" ):
muEC::SG::Perm2Rd        := loadproc(muEC::SG::Perm2Rd,        path, "Perm2Rd" ):
muEC::SG::Perm2Rise      := loadproc(muEC::SG::Perm2Rise,      path, "Perm2Ris"):
muEC::SG::Perm2Rothe     := loadproc(muEC::SG::Perm2Rothe,     path, "Perm2Rot"):
muEC::SG::Perm2Sign      := loadproc(muEC::SG::Perm2Sign,      path, "Sign"    ):
muEC::SG::Perm2Part      := loadproc(muEC::SG::Perm2Part,      path, "Perm2Sha"):
muEC::SG::RRep2Perm      := loadproc(muEC::SG::RRep2Perm,      path, "RR2Perm" ):
muEC::SG::RandPerm       := loadproc(muEC::SG::RandPerm,       path, "RandPerm"):
muEC::SG::Rd2Perm        := loadproc(muEC::SG::Rd2Perm,        path, "Rd2Perm" ):
muEC::SG::SgCharTable    := loadproc(muEC::SG::SgCharTable,    path, "Charactr"):
muEC::SG::SgDimRep       := loadproc(muEC::SG::SgDimRep,       path, "DimRep"  ):
muEC::SG::SgTranspo      := loadproc(muEC::SG::SgTranspo,      path, "Transpo" ):
muEC::SG::Vect2Perm      := loadproc(muEC::SG::Vect2Perm,      path, "Vec2Perm"):

unalias(path):
