/*
|  File CharTabl (SG PACKAGE) Sat Oct 25 19:34:11 MET DST 1997
*/

/*
|  SgCharTable(n) returns the characters table for the irreducible
|                 representations of the symmetric group of degree n.
|                 Rows are indexed by irreducible representations.
|                 Lines are indexed by conjugacy classes.
|
|  SgCharTable(n,hold(index)) returns the index of such a table, that is
|                             the list of all partitions of n.
|
|  SgCharTable(n,Lambda,Rho) returns the corresponding entry of the table.
|                            It is the value of the constant character on the
|                            conjugacy class associated to the partition of
|                            n, Rho, in the representation associated to the
|                            partition of n, Lambda.
|                            This is done without calculating all the table.
|
|  Chi(I,J) computes the entry in the characters table corresponding
|           to the irreducible representation indexed by I, and the
|           conjugacy class indexed by J.
*/

/*
|  VP271197 VP080198
|  We call the SYMMETRICA function muEC::SYM::SgCharTable (chartafel).
*/
muEC::SG::SgCharTable_orig := proc(n)
begin
  if (n=0) then
    array(1..1,1..1,[[1]]);
  else
    muEC::SYM::SgCharTable(n);
  end_if;
end_proc:

/*
|  This "function" computes the scalar product between the Schur "function"
|  indexed by the partition I, and the product of power sum functions
|  "function" indexed by the partition J.
|  We assume that I and J are partitions of the same list since
|  this "function" is used to calculate entries of the characters table.
|  VP031297
*/

// NT: 05/11/2003 use directly muEC::SYM::Chi below to avoid error
// when Symmetrica is not available
// muEC::SG::Chi := muEC::SYM::Chi:

/*
|  VP251097 VP031297
*/
muEC::SG::SgCharTable_core := proc(n)
begin
  if (args(0)=1) then
    muEC::SG::SgCharTable_orig(args());
  elif (args(0)=2) then
    /* we compute all partitions of n... */
    muEC::PART::ListPart(n);
  else 
    /* we guess that args(0)>=3... */
    muEC::SYM::Chi(args(2),args(3));
  end_if;
end_proc:

/*
|  Type Checking
|  VP251097
*/
muEC::SG::SgCharTable_check := proc(n)
begin
  if (not testtype(n,DOM_INT)) then
    muEC::ERR::SGPerror(1,procname);
  end_if;
  if (n<0) then
    muEC::ERR::SGPerror(2,procname);
  end_if;
  if (args(0)=2) then
    if (not testtype(args(2),DOM_IDENT)) then
      muEC::ERR::SGPerror(3,procname);
    elif (args(2)<>hold(index)) then
      muEC::ERR::SGPerror(10,procname);
    end_if;
  elif (args(0)>=3) then
    if (not muEC::TYP::IsPart(args(2))) then
      muEC::ERR::SGPerror(3,procname);
    elif (_plus(op(args(2)))<>n) then
      muEC::ERR::SGPerror(4,procname);
    elif (not muEC::TYP::IsPart(args(3))) then
      muEC::ERR::SGPerror(5,procname);
    elif (_plus(op(args(3)))<>n) then
      muEC::ERR::SGPerror(6,procname);
    end_if;
  end_if;
end_proc:

/*
|  VP251097
*/
muEC::SG::SgCharTable := proc(n)
begin
  if testargs() then
    muEC::SG::SgCharTable_check(args());
  end_if;
  muEC::SG::SgCharTable_core(args());
end_proc:
