/*
|  File Cod2Perm (SG PACKAGE) Sat Oct 25 15:40:16 MET DST 1997
*/

/*
|  Code2Perm(code) calculates from the lehmer code, the corresponding
|                  permutation.
|                  A second argument n may be passed to specify the degree of
|                  the symmetric group.
*/

/*
|  VP251097 VP160698
*/
muEC::SG::Code2Perm_core := proc(code)
local
  perm, /* the result...                                               */
  nc,   /* new code...                                                 */
  i,    /* var for loop...                                             */
  j,    /* var for seq...                                              */
  l,    /* the list of all elements 1..n ...                           */
  aac,  /* adjusted code : length is set to degree(symmetric group)... */
  lc,   /* the length of the code...                                   */
  n;    /* degree of the symmetric group...                            */
begin
  lc := nops(code);
  if lc=0 then
    if args(0)=1 then
     return([1]);
    else
      return([$1..args(2)]);
    end_if;
  end_if;
  /*nc := [op(code),0$(max(_seqgen(code[j]+j,j,1..lc))-lc)]; print(nc); CC 29 06 01*/
  nc := [op(code),0$(max(code[j]+j $j=1..lc)-lc)]; //print(nc);
  /* adjusting the length of the code to the degree of the symmetric group... */
  if (args(0)>=2) then
    n  := args(2);
    lc := nops(nc);
    if (n<lc) then
      muEC::ERR::SGPerror(4,procname);
    end_if;
    /* and adding 1 to all elements of the code... */
    aac := map([op(nc),0$(n-lc)], proc(i) begin i+1 end_proc);
  else
    n := nops(nc);
    /* and adding 1 to all elements of the code... */
    aac:=map(nc, proc(i) begin i+1 end_proc);
  end_if;
  /* we init a list l to 1..n ... */
  l := [$1..n];
  /* step i: the ith element of the permutation is the aac[i]th elt of l...  */
  /*         extract from l the aac[i]th... */
  perm := null();
  for i from 1 to n do
    perm := perm, l[aac[i]];
    l := subs(l, l[aac[i]]=null());
  end_for;
  /* we return a list... */
  [perm];
end_proc:

/*
|  Type Checking
|  VP251097
*/
muEC::SG::Code2Perm_check:=proc(code)
local
  i; /* variable for loop... */
begin
  if (not testtype(code,DOM_LIST)) then
    muEC::ERR::SGPerror(1,procname);
  end_if;
  for i from 1 to nops(code) do
    if (not testtype(code[i],DOM_INT)) then
      muEC::ERR::SPPerror(2,procname);
    elif (code[i]<0) then
      muEC::ERR::SPPerror(2,procname);
    end_if;
  end_for;
  if (args(0)>=2) then
    if (not testtype(args(2),DOM_INT)) then
      muEC::ERR::SGPerror(3,procname);
    end_if;
    if (args(2)<nops(code)) then
      muEC::ERR::SGPerror(4,procname);
    end_if;
  end_if;
end_proc:

/*
|  VP251097
*/
muEC::SG::Code2Perm := proc(code)
begin
  if testargs() then
    muEC::SG::Code2Perm_check(args());
  end_if;
  muEC::SG::Code2Perm_core(args());
end_proc:

muEC::SG::tst:=proc(n)
begin
	n
end_proc;
