/*
|  File CycleTyp (SG PACKAGE) Sun Oct 26 12:38:32 MET 1997
*/

/*
|  Perm2CycleType(perm) computes the cycle type of the permutation perm.
|                       This is a partition written in decreasing order.
*/

/*
|  VP261097
*/
muEC::SG::Perm2CycleType_core := proc(perm)
local
  p,      /* the current position...                      */
  s,      /* starting position...                         */
  unused, /* unused positions by yet calculated cycles... */
  cl,     /* a cycle length...                            */
  ct,     /* the result...                                */
  n;      /* degree of the symmetric group...             */
begin
  ct := null();
  n := nops(perm);
  unused := [$1..n];
  while (unused<>[]) do
    /* we start at the first unused position in the permutation... */
    s := unused[1];
    unused := subs(unused, s=null());
    /* we init the next position to be perm[s]... */
    p := perm[s];
    /* we init a new cycle... length=1 by default... */
    cl := 1;
    /* while position p is different from starting position s... */
    while (p<>s) do
      cl := cl+1;
      unused := subs(unused, p=null());
      p := perm[p];
    end_while;
    ct := ct, cl;
  end_while;
  /* we return the partition that is the parts into decreasing order... */
  sort([ct],proc(x,y) begin bool(x>=y) end_proc);
end_proc:

/*
|  Type Checking
|  VP262097
*/
muEC::SG::Perm2CycleType_check:=proc(perm)
begin
  if (not muEC::TYP::IsPerm(perm)) then
    muEC::ERR::SGPerror(1,procname);
  end_if;
end_proc:

/*
|  VP261097
*/
muEC::SG::Perm2CycleType := proc(perm)
begin
  if testargs() then
    muEC::SG::Perm2CycleType_check(args());
  end_if;
  muEC::SG::Perm2CycleType_core(args());
end_proc:
