/*
|  File Dc2Perm (SG PACKAGE) Sun Oct 26 12:48:22 MET 1997
*/

/*
|  Cycle2Perm(dc) from disjoint cycles notation to permutation (list).
|                 Disjoint cycles notation is denoted by a list of the cycles
|                 that are also lists.
*/

/*
|  VP261097
*/
muEC::SG::Cycle2Perm_core := proc(dc)
local
  perm, /* the result...                        */
  c,    /* first step to list notation...       */
  n,    /* the degree of the symmetric group... */
  i,j;  /* vars for loops...                    */
begin
  /* we evaluate the degree of the symmetric group (minimal one !!)... */
  n := max(op(map(dc,op)));
  /* identity permutation... */
  if (n=0) then
    return([1]);
  end_if;
  /* we init the permutation to identity... */
  perm := [$1..n];
  for i from 1 to nops(dc) do
    /* we do not consider cycles of length 0 or 1... */
    if (nops(dc[i])<2) then
      next;
    end_if;
    c := ((dc[i][j]=dc[i][j+1])$j=1..(nops(dc[i])-1)),
         dc[i][nops(dc[i])]=dc[i][1];
    perm := subs(perm,{c})
  end_for;
  perm
end_proc:

/*
|  Type Checking
|  VP261097
*/
muEC::SG::Cycle2Perm_check:=proc(dc)
local
  used, /* yet used positions... */
  i,j;  /* vars for loop...      */
begin
  if (not testtype(dc,DOM_LIST)) then
    muEC::ERR::SGPerror(1,procname);
  end_if;
  used := {};
  for i from 1 to nops(dc) do
    if (not testtype(dc[i],DOM_LIST)) then
      muEC::ERR::SGPerror(1,procname);
    end_if;
    for j from 1 to nops(dc[i]) do
      if (not testtype(dc[i][j],Type::PosInt)) then
        muEC::ERR::SGPerror(1,procname);
      elif (contains(used,dc[i][j])) then
        /* these are not disjoint cycles... */
        muEC::ERR::SGPerror(1,procname);
      else
        used := used union {dc[i][j]};
      end_if;
    end_for;
  end_for;
end_proc:

/*
|  VP261097
*/
muEC::SG::Cycle2Perm := proc(dc)
begin
  if testargs() then
    muEC::SG::Cycle2Perm_check(args());
  end_if;
  muEC::SG::Cycle2Perm_core(args());
end_proc:
