/*
|  File DimRep (SG PACKAGE) Sat Oct 25 16:21:23 MET DST 1997
*/

/*
|  SgDimRep(part) computes the dimension of the irreducible representation
|                 corresponding to the partition part.
|                 When called with a skew partition, it computes the dimension
|                 of the representation indexed by the skew partition.
|                 With the extra parameter hold(q), it computes the 
|                 q-dimension.
*/

/*
|  VP251097 VP250498
*/
muEC::SG::SgDimRep_core := proc(part)
local
  i,j; /* vars for seq... */
begin
  if (args(0)=1) then
    if muEC::TYP::IsPart(part) then
      fact(_plus(op(part)))/_mult(op(muEC::PART::Part2ListHook(part)));
    else /* skew partition... */
      linalg::det(sparsematrix
		  (
		   subs(muEC::SYMF::SfJtMat(part, hold(b)),
			hold(b)[i]=1/fact(i)
			$ i= 1..op(op(part,1),1)+(nops(op(part,1))-1))))*
      fact(_plus(op(op(part,1)))-_plus(op(op(part,2))));
    end_if;
  else
    if muEC::TYP::IsPart(part) then
      /* q-dimension of an irreducible representation... */
      factor(_mult(1-args(2)^j $ j=1.._plus(op(part))) *
             linalg::det(sparsematrix
               (subs(muEC::SYMF::SfJtMat(part),
                     {hold(h)[i]=1/_mult(1-args(2)^j $ j=1..i)
                      $ i = 1..part[1]+nops(part)-1}))));
    else
      /* q-dimension of a representation indexed by a skew partition... */
      factor(_mult(1-args(2)^j
                   $ j=1..(_plus(op(op(part,1)))-_plus(op(op(part,2))))) *
             linalg::det(sparsematrix
               (subs(muEC::SYMF::SfJtMat(part),
                     {hold(h)[i]=1/_mult(1-args(2)^j $ j=1..i)
                      $ i = 1..op(op(part,1),1)+nops(op(part,1))-1}))));
    end_if;
  end_if;
end_proc:

/*
|  Type Checking
|  VP251097 VP250498
*/
muEC::SG::SgDimRep_check := proc(part)
begin
  if (not (muEC::TYP::IsPart(part) or muEC::TYP::IsSkewPart(part))) then
    muEC::ERR::SGPerror(1,procname);
  elif (part=[] or part=[[],[]]) then
    muEC::ERR::SGPerror(2,procname);
  end_if;
  if (args(0)=2) then
    if (not testtype(args(2), DOM_IDENT)) then
      muEC::ERR::SGPerror(3,procname);
    end_if;
  end_if;
end_proc:

/*
|  VP251097
*/
muEC::SG::SgDimRep := proc(part)
begin
  if testargs() then
    muEC::SG::SgDimRep_check(args());
  end_if;
  muEC::SG::SgDimRep_core(args());
end_proc:
