/*
|  File GenPerm (SG PACKAGE) Sun Oct 26 15:17:42 MET 1997
*/

/*
|  GenPerm(n, First) calculates the first permutation.
|
|  GenPerm(n, Last) calculates the last permutation.
|
|  GenPerm(perm, Next) calculates the next permutation after perm
|                      in lexicographic order. If perm is already the last 
|                      permutation, null() is returned.
|
|  GenPerm(perm, Previous) calculates the previous permutation before perm
|                          in the lexicographic order. If perm is already
|                          the first permutation, null() is returned.
*/

/*
|  VP261097
*/
muEC::SG::GenPerm_core := proc()
local
  i,      /* var for loop...                                            */
  j,      /* var for seq...                                             */
  se,     /* a seq...                                                   */
  n,      /* degree of the symmetric group...                           */
  lst,    /* the permutation args(1)...                                 */
  down,   /* the last descent of the permutation...                     */
  gsml,   /* greatest element smaller than down to the right of down... */
  up,     /* the last climb of the permutation...                       */
  rsup;   /* smallest element greater than up to the right of up...     */
begin
  if (testtype(args(1), DOM_INT)) then
    if (args(2)=hold(First)) then
      [_seqgen(j, j,1..args(1))];
    else
      [_seqgen(args(1)+1-j, j,1..args(1))];
    end_if;
  else
    if (args(2)=hold(Previous)) then
      n := nops(args(1));
      lst := args(1);
      down := -1;
      gsml := -1;
      /* looking for the values of down and gsml... */
      for i from 2 to n do
        if (lst[i-1]>lst[i]) then
          /* the descent is at position i-1... */
          down := i-1;
          /* gsml is supposed at position i... */
          gsml := i;
        else
          if gsml > -1 then
            /* by definition of gsml... */
            if ((lst[i]>lst[gsml]) and (lst[i]<lst[down])) then
              gsml := i;
            end_if;
          end_if;
        end_if;
      end_for;
      /* it is finished if down=-1 since there are no descent so that... */
      /* we have the first permutation : 1..n. ... */
      if (down=-1) then
        return(null());
      end_if;
      /* we swap the elements lst[down] and lst[gsml]... */
      if (down=1) then
        se := lst[gsml],op(lst,(down+1)..(gsml-1)),lst[down],
              op(lst,(gsml+1)..n);
      else
        se := op(lst,1..(down-1)),lst[gsml],op(lst,(down+1)..(gsml-1)),
            lst[down],op(lst,(gsml+1)..n);
      end_if;
      /* we sort the elements to the right of the position up... */
      /* Since they are in increasing order, we have to reverse the list... */
      /* from position down+1 to position n... */
      [op(se,1..down),_seqgen(se[(n+1)-j],j,1..(n-down))];
    else
      n := nops(args(1));
      lst := args(1);
      up := -1;
      rsup := -1;
      /* looking for the values of up and rsup... */
      for i from 2 to n do
        if (lst[i]>lst[i-1]) then
          /* the climb is at position i-1... */
          up := i-1;
          /* rsup is supposed at position i... */
          rsup := i;
        else
          if (rsup > -1) then
            /* by definition of rsup... */
            if ((lst[i]<lst[rsup]) and (lst[i]>lst[up])) then
              rsup := i;
            end_if;
          end_if;
        end_if;
      end_for;
      /* it is finished if up=-1 since there are only descents so that... */
      /* we have the longest permutation : n..1. ... */
      if (up=-1) then
        return(null());
      end_if;
      /* we swap the elements lst[up] and lst[rsup]... */
      if (up=1) then
        se := lst[rsup],op(lst,(up+1)..(rsup-1)),lst[up],op(lst,(rsup+1)..n);
      else
        se := op(lst,1..(up-1)),lst[rsup],op(lst,(up+1)..(rsup-1)),lst[up],
              op(lst,(rsup+1)..n);
      end_if;
      /* we sort the elements to the right of the position up... */
      /* Since they are in decreasing order, we have to reverse the list... */
      /* from position up+1 to position n... */
      [op(se,1..up),_seqgen(se[(n+1)-j],j,1..(n-up))];
    end_if;
  end_if;
end_proc:

/*
|  Type Checking
|  VP261097
*/
muEC::SG::GenPerm_check := proc()
begin
  if (args(0)<>2) then
    muEC::ERR::SGPerror(9,procname);
  end_if;
  if (testtype(args(1), DOM_INT)) then
    if (args(1)<=0) then
      muEC::ERR::SGPerror(2,procname);
    elif (not testtype(args(2), DOM_IDENT)) then
      muEC::ERR::SGPerror(3,procname);
    elif (not contains({hold(First), hold(Last)}, args(2))) then
      muEC::ERR::SGPerror(4,procname);
    end_if;
  elif (muEC::TYP::IsPerm(args(1))) then
    if (not testtype(args(2), DOM_IDENT)) then
      muEC::ERR::SGPerror(3,procname);
    elif (not contains({hold(Previous), hold(Next)}, args(2))) then
      muEC::ERR::SGPerror(4,procname);
    end_if;
  else
    muEC::ERR::SGPerror(1,procname);
  end_if;
end_proc:

/*
|  VP261097
*/
muEC::SG::GenPerm := proc(perm, flag)
begin
  if testargs() then
    muEC::SG::GenPerm_check(args());
  end_if;
  muEC::SG::GenPerm_core(args());
end_proc:
