/*
|  File Interval (SG PACKAGE) Sun Oct 26 17:04:59 MET 1997
*/

/*
|  Perm2Interval(perm) computes the inferior interval (for the Bruhat order).
|                      The result is a list of lehmer codes.
*/

/*
|  VP261097
*/
muEC::SG::PiPerm := proc(t,f,perm)
local
  l,   /* the length of perm... */
  r,   /* the result...         */
  i,j; /* vars for loops...     */
begin
  l := nops(perm);
  r := perm;
  for i from f to t do
    /* left multiplication by Pi_i... */
    j := 1;
    while(r[j]<>i and r[j]<>i+1 and j<=l) do j:=j+1; end_while;
    if (r[j]=i) then
      r := subsop(r,j=i+1);
      j := j+1;
      while(r[j]<>i+1) do j:=j+1; end_while;
      r := subsop(r,j=i);
    end_if;
  end_for;
  r;
end_proc:

/*
|  VP261097
*/
muEC::SG::Perm2IntervalRec := proc(perm, l)
local
  p,   /* a permutation...    */
  r,   /* the result...       */
  c,   /* the code of perm... */
  i;   /* vars for loops...   */
begin
  if (perm=[$1..l]) then
    return([[0$l]]);
  end_if;
  c := muEC::SG::Perm2Code(perm);
  p := map(subsop(perm,1=null()),
           proc(x,v) begin if (x>v) then x-1 else x end_if end_proc, perm[1]);
  if (c[1]=0) then
    return(map(muEC::SG::Perm2IntervalRec(p,l-1),
               proc(l,x) begin subsop(l,1=(x,l[1])) end_proc, 0));
  end_if;
  r := op(map(muEC::SG::Perm2IntervalRec(p,l-1),
              proc(l,x) begin subsop(l,1=(x,l[1])) end_proc, c[1]));
  r := r, op(map(muEC::SG::Perm2IntervalRec(p,l-1),
                 proc(l,x) begin subsop(l,1=(x,l[1])) end_proc, c[1]-1));
  for i from c[1]-1 downto 1 step 1 do
    r := r, op(map(muEC::SG::Perm2IntervalRec(muEC::SG::PiPerm(c[1]-1,i,p),l-1),
                   proc(l,x) begin subsop(l,1=(x,l[1])) end_proc, i-1));
  end_for;
  [r];
end_proc:

/*
|  VP261097
*/
muEC::SG::Perm2Interval_core := proc(perm)
begin
  muEC::SG::Perm2IntervalRec(perm,nops(perm));
end_proc:

/*
|  Type Checking
|  VP261097
*/
muEC::SG::Perm2Interval_check:=proc(perm)
begin
  if (not muEC::TYP::IsPerm(perm)) then
    muEC::ERR::SGPerror(1,procname);
  end_if;
end_proc:

/*
|  VP261097
*/
muEC::SG::Perm2Interval := proc(perm)
begin
  if testargs() then
    muEC::SG::Perm2Interval_check(args());
  end_if;
  muEC::SG::Perm2Interval_core(args());
end_proc:
