/*
|  File ListPerm (SG PACKAGE) Fri Oct 24 12:00:52 MET DST 1997
*/

/*
|  ListPerm(n) returns a list of all permutations (n!), elements of the
|              symmetric group of degree n. Very speed version !!!!!!!!
|  ListPerm(l) returns a list of all distinct permutations of the elements
|              contained in the list l. Is is the same if l is a set.
|  If a second argument str is added, then
|  str=hold(lexic)    -> the permutations are in lexicographic order.
|                    Available when first argument is an integer.
|  str=hold(cixel)    -> the permutations are in reverse lexicographic order.
|                    Available when first argument is an integer.
|  str=hold(level)    -> to list all permutations according to their length. 
|                    Available when first argument is an integer.
|  str=hold(dominant) -> to list the dominant permutations.
|                    Available when first argument is an integer.
|  str=hold(grassmannian)
|                -> to list the grassmannian permutations.
|                    Available when first argument is an integer.
|  str=hold(vexillary) -> to list the vexillary permutations.
|                    Available when first argument is an integer.
|  If a third argument hold(code) is added, then it returns codes instead
|  of permutation in the case of str=`dominant`, `grassmannian` or `vexillary`.
|  Adding another argument hold(nb), it returns the number of permutations.
|  InsertAll(l,v) inserts the value v at each place in l.
*/

/*
|  VP241097
*/
muEC::SG::FloorPerm_rec:=proc(n)
  option remember;
begin
  if (n=0) then
    [[0, []]]
  elif (n=1) then
    [[0, [1]]]
  else
    map(muEC::SG::FloorPerm_rec(n-1),
        proc(l, v)local i, v1, v2, nv2; begin
          v1 := op(l,1);
          v2 := op(l,2);
          nv2 := nops(v2);
          [v1, [op(v2),v]],
          _seqgen([v1+nv2-i, [op(v2,1..i),v,op(v2,i+1..nv2)]], i,1..nv2-1),
          [v1+nv2, [v,op(v2)]];
        end_proc,
        n);
  end_if;
end_proc:

/*
|  VP241097
*/
muEC::SG::FloorPerm:=proc(n)
local
    i, /* var for loop... */
    j, /* var for seq...  */
    r, /* a result...     */
    t; /* a table...      */
begin
  r:=muEC::SG::FloorPerm_rec(n);
  sysassign(muEC::SG::FloorPerm_rec,
            muEC::clearRememberTable(muEC::SG::FloorPerm_rec));
  t := table(_seqgen((j=null()), j,0..n*(n-1)/2));
  for i from 1 to nops(r) do
    t[op(op(r,i),1)] := t[op(op(r,i),1)], op(op(r,i),2);
  end_for;
  [_seqgen([t[j]], j,0..n*(n-1)/2)];
end_proc:

/*
|  VP241097
*/
muEC::SG::InsertAll:=proc(l,v)
local
  i; /* var for seq... */
begin
  [op(l),v],_seqgen([op(l,1..i),v,op(l,i+1..nops(l))],
 i,1..(nops(l)-1)),[v,op(l)];
end_proc:

/*
|  VP241097 VP080198
*/
muEC::SG::ListPerm_core:=proc(n)
local
    r,  /* the result...    */
    t,  /* temporary var... */
    pp, /* a var...         */
    i,  /* var for loop...  */
    o,  /* option set...    */
    oo, /* other option...  */
    j;  /* var for seq...   */
begin
  if (args(0)=1) then
    if (testtype(n,DOM_INT)) then
      if (n=0) then /* do not remove because of certain algorithms... */
        [[]];
      elif (n=1) then
        [[1]];
      else
        map(muEC::SG::ListPerm_core(n-1), muEC::SG::InsertAll, n);
      end_if;
    elif (testtype(n,DOM_LIST)) then
      if (nops(n)=0) then
        [[]];
      elif (nops(n)=1) then
        [n];
      else
        r:=null();
        for i from 1 to nops(n) do
          pp := contains(n, op(n,i));
          if (pp<i) then next end_if;
          r:=r,op(map(muEC::SG::ListPerm_core(subsop(n,i=null())),
                      proc(l,e) begin [e,op(l)] end_proc ,
                      op(n,i)));
        end_for;
        [r]
      end_if;
    elif (testtype(n,DOM_SET)) then
      muEC::SG::ListPerm_core([op(n)]);
    end_if;
  /* there is an option... */
  elif (args(0)>=2) then
    o[1] := contains([args(2..args(0))], hold(nb));    /* option nb...    */
    o[2] := contains([args(2..args(0))], hold(code));  /* option code...  */
    o[3] := contains([args(2..args(0))], hold(level)); /* option level... */
    pp := subs([args(2..args(0))],
               hold(code)=null(), hold(nb)=null(), hold(level)=null());
    if (pp<>[]) then
      pp := op(pp, 1);
    end_if;
    if (pp=hold(lexic) or pp=hold(cixel)) then
      if (n=0) then
        if (o[1]<>0) then
          if (o[3]<>0) then [[1]]; else [1]; end_if;
        elif (o[3]<>0) then
          [[[]]];
        else
          [[]];
        end_if;
      else
        if (o[1]<>0) then
          if (o[3]<>0) then
            muEC::SG::NbPerm_level(n);
          else
            fact(n);
          end_if;
        else
          if (o[3]<>0) then
            muEC::SG::NbPerm_level(n);
          else
            if (pp=hold(lexic)) then
              t := map(muEC::SG::ListPerm_core(n-1,hold(lexic)),
                       proc(m) begin map(m,_plus,1) end_proc );
              /* we put 1 at the beginning of each element of the list... */
              r := op(map(t, proc(l) begin [1,op(l)] end_proc ));
              /* we calculate all permutations... */
              for i from 2 to n do
                t := map(t, proc(l,i) begin subs(l, i=i-1) end_proc ,i);
                r := r, op(map(t, proc(m,k) begin [k,op(m)] end_proc ,i));
              end_for;
            else
              t := map(muEC::SG::ListPerm_core(n-1,hold(cixel)),
                       proc(m) begin map(m,_plus,1) end_proc );
              /* we put 1 at the end of each element of the list... */
              r := op(map(t, proc(l) begin [op(l),1] end_proc ));
              /* we calculate all permutations... */
              for i from 2 to n do
                t := map(t, proc(l,i) begin subs(l, i=i-1) end_proc ,i);
                r := r, op(map(t, proc(m,k) begin [op(m),k] end_proc ,i));
              end_for;
            end_if;
            if (o[2]<>0) then
              map([r], muEC::SG::Perm2Code);
            else
              [r];
            end_if;
          end_if;
        end_if;
      end_if;
    elif (pp=hold(dominant)) then
      if (o[1]<>0) then
        if (o[3]<>0) then
          r := muEC::SG::ListDominant(n, hold(code));
          /*t := table(sparse);*/
          t := table(_seqgen(j=0,j,0..n*(n-1)/2));
          for i from 1 to nops(r) do
            oo := _plus(op(r[i]));
            t[oo] := t[oo]+1;
          end_for;
          [_seqgen(subs(t[j],muEC::cat("t[",j,"]")=0), j,0..n*(n-1)/2)];
        else
          muEC::SG::NbPerm(n, pp);
        end_if;
      else
        if (o[2]<>0) then
          if (o[3]<>0) then
            r := muEC::SG::ListDominant(n, hold(code));
            t := table();
            for i from 1 to nops(r) do
              oo := _plus(op(r[i]));
              /*if (anames(t[oo])) then*/
              if (bool(t[oo]=hold(t[oo]))) then
                t[oo] := t[oo], r[i];
              else
                t[oo] := r[i];
              end_if;
            end_for;
            [_seqgen([subs(t[j],muEC::cat("t[",j,"]")=0)], j,0..n*(n-1)/2)];
          else
            muEC::SG::ListDominant(n, hold(code));
          end_if;
        else
          if (o[3]<>0) then
            r:=muEC::SG::ListDominant(n, hold(code));
            t:=table();
            for i from 1 to nops(r) do
              oo := _plus(op(r[i]));
              /*if (anames(t[oo])) then*/
              if (bool(t[oo]=hold(t[oo]))) then
                t[oo] := t[oo],muEC::SG::Code2Perm(r[i]);
              else
                t[oo] := muEC::SG::Code2Perm(r[i]);
              end_if;
            end_for;
            [_seqgen([subs(t[j],muEC::cat("t[",j,"]")=0)], j,0..n*(n-1)/2)];
          else
            muEC::SG::ListDominant(n);
          end_if;
        end_if;
      end_if;
    elif (pp=hold(grassmannian)) then
      if (o[1]<>0) then
        if (o[3]<>0) then
          r := muEC::SG::ListGrassmannian(n);
          /*t := table(sparse);*/
          t := table(_seqgen(j=0,j,0..n*(n-1)/2));
          for i from 1 to nops(r) do
            oo := muEC::SG::Perm2Length(r[i]);
            t[oo] := t[oo]+1;
          end_for;
          [_seqgen(subs(t[j],muEC::cat("t[",j,"]")=0), j,0..n*(n-1)/2)];
        else
          muEC::SG::NbPerm(n, pp);
        end_if;
      else
        if (o[2]<>0) then
          if (o[3]<>0) then
            r := muEC::SG::ListGrassmannian(n, hold(code));
            t := table();
            for i from 1 to nops(r) do
              oo := _plus(op(r[i]));
              /*if (anames(t[oo])) then*/
              if (bool(t[oo]=hold(t[oo]))) then
                t[oo] := t[oo],r[i];
              else
                t[oo] := r[i];
              end_if;
            end_for;
            [_seqgen([subs(t[j],muEC::cat("t[",j,"]")=0)], j,0..n*(n-1)/2)];
          else
            muEC::SG::ListGrassmannian(n, hold(code));
          end_if;
        else
          if (o[3]<>0) then
            r:=muEC::SG::ListGrassmannian(n);
            t:=table();
            for i from 1 to nops(r) do
              oo:=muEC::SG::Perm2Length(r[i]);
              /*if (anames(t[oo])) then*/
              if (bool(t[oo]=hold(t[oo]))) then
                t[oo] := t[oo],r[i];
              else
                t[oo] := r[i];
              end_if;
            end_for;
            [_seqgen([subs(t[j],muEC::cat("t[",j,"]")=0)], j,0..n*(n-1)/2)];
          else
            muEC::SG::ListGrassmannian(n);
          end_if;
        end_if;
      end_if;
    elif (pp=hold(vexillary)) then
      if (o[1]<>0) then
        if (o[3]<>0) then
          r := muEC::SG::ListVexillar(n);
          /*t := table(sparse);*/
          t := table(_seqgen(j=0,j,0..n*(n-1)/2));
          for i from 1 to nops(r) do
            oo := muEC::SG::Perm2Length(r[i]);
            t[oo]:=t[oo]+1;
          end_for;
          [_seqgen(subs(t[j],muEC::cat("t[",j,"]")=0), j,0..n*(n-1)/2)];
        else
          muEC::SG::NbPerm(n, pp);
        end_if;
      else
        if (o[2]<>0) then
          if (o[3]<>0) then
            r := muEC::SG::ListVexillar(n, hold(code));
            t := table();
            for i from 1 to nops(r) do
              oo := _plus(op(r[i]));
              /*if (anames(t[oo])) then*/
              if (bool(t[oo]=hold(t[oo]))) then
                t[oo] := t[oo],r[i];
              else
                t[oo] := r[i];
              end_if;
            end_for;
            [_seqgen([subs(t[j],muEC::cat("t[",j,"]")=0)], j,0..n*(n-1)/2)];
          else
            muEC::SG::ListVexillar(n, hold(code));
          end_if;
        else
          if (o[3]<>0) then
            r := muEC::SG::ListVexillar(n);
            t := table();
            for i from 1 to nops(r) do
              oo := muEC::SG::Perm2Length(r[i]);
              /*if (anames(t[oo])) then*/
              if (bool(t[oo]=hold(t[oo]))) then
                t[oo] := t[oo],r[i];
              else
                t[oo]:=r[i];
              end_if;
            end_for;
            [_seqgen([subs(t[j],muEC::cat("t[",j,"]")=0)], j,0..n*(n-1)/2)];
          else
            muEC::SG::ListVexillar(n);
          end_if;
        end_if;
      end_if;
    else
      if (o[1]<>0) then
        if (o[3]<>0) then
          muEC::SG::NbPerm_level(n);
        else
          fact(n);
        end_if;
      else
        if (o[2]<>0) then
          if (o[3]<>0) then
            map(muEC::SG::FloorPerm(n), proc(l) begin map(l, muEC::SG::Perm2Code) end_proc );
          else
            map(muEC::SG::ListPerm_core(n), muEC::SG::Perm2Code);
          end_if;
        else
          if (o[3]<>0) then
            muEC::SG::FloorPerm(n);
          else
            muEC::SG::ListPerm_core(n);
          end_if;
        end_if;
      end_if;
    end_if;
  end_if;
end_proc:

/*
|  VP241097
*/ 
muEC::SG::NbPerm:=proc(n)
begin
  if (args(0)=1) then
    /* all permutations... */
    fact(n);
  else
    if (args(2)=hold(dominant)) then
      fact(2*n)/(fact(n)*fact(n+1));
    elif (args(2)=hold(grassmannian)) then
      2^n-n;
    elif (args(2)=hold(vexillary)) then
      _plus(op(map(muEC::PART::ListPart(n,hold(maxlg)=3),
                   proc(x) begin muEC::SG::SgDimRep(x)^2 end_proc )));
    elif (args(2)=hold(level)) then
      muEC::SG::NbPerm_level(n);
    end_if;
  end_if;
end_proc:
/*
|  The next "function" counts permutations step number of inversions.
|  VP241097
*/
muEC::SG::NbPerm_level:=proc(n)
local
    rc,  /* recursive call...         */
    t,   /* total number of levels... */
    r,   /* the result...             */
    i,   /* var for loop...           */
    j,   /* var for seq...            */
    m;   /* middle...                 */
begin
  if (n=0) then
    [1];
  elif (n=1) then
    [1];
  elif (n=2) then
    [1,1];
  elif (n=3) then
    [1,2,2,1];
  else
    t := n*(n-1)/2+1;
    if ((t mod 2)=0) then
      m := t/2;
    else
      m := (t+1)/2;
    end_if;
    rc := muEC::SG::NbPerm_level(n-1); r := 1, n-1;
    for i from 3 to n do r := r, r[i-1]+rc[i]; end_for;
    for i from n+1 to m do r := r, r[i-1]+rc[i]-rc[i-n]; end_for;
    r := [r];
    if ((t mod 2)=0) then
      [op(r), _seqgen(r[-j], j,-nops(r)..-1)];
    else
      [op(r), _seqgen(r[-j], j,-nops(r)+1..-1)];
    end_if;
  end_if;
end_proc:

/*
|  ListDominant(n,hold(code)) returns the list of all codes of the dominant
|                         permutations of the symmetric group of degree n.
|                         When called with one argument, it returns the list
|                         of the corresponding permutations.
|  VP241097
*/
muEC::SG::ListDominant_rec:=proc(n)
begin
  if (n=1) then
    [[0]];
  else
    map(muEC::SG::ListDominant_rec(n-1),
        proc(pp,n) local i; begin _seqgen([-i,op(pp)],i,-n+1..-pp[1]) end_proc ,n);
  end_if;
end_proc:

/*
|  VP241097
*/
muEC::SG::ListDominant:=proc(n)
begin
  if (args(0)>=2) then
    muEC::SG::ListDominant_rec(n);
  else
    map(muEC::SG::ListDominant_rec(n), muEC::SG::Code2Perm);
  end_if;
end_proc:

/*
|  ListGrassmannian(n,hold(code)) returns the list of all codes of the
|                             grassmannian permutations of the symmetric group
|                             of degree n.
|                             When called with one argument, it returns the
|                             list of the corresponding permutations.
|  VP241097
*/
muEC::SG::ListGrassmannian_rec:=proc(n,maxi)
local
    i; /* variable for sequence... */
begin
  if (n=1) then
    [_seqgen([i],i,1..maxi)];
  else
    map(muEC::SG::ListGrassmannian_rec(n-1,maxi),
        proc(pp,maxi) local i; begin _seqgen([i,op(pp)],i,0..min(pp[1],maxi)) end_proc,
        maxi);
  end_if;
end_proc:
/*

|  We compute the list of all codes corresponding to partitions in increasing
|  order, contained in the rectangles n-1 x 1, n-2 x 2, ..., 1 x n-1.
|  VP241097
*/
muEC::SG::ListGrassmannian:=proc(n)
local
    i; /* variable for sequence... */
begin
  if (args(0)>=2) then
    map([[_seqgen(0, i,1..n)],
          _seqgen(op(muEC::SG::ListGrassmannian_rec(i,n-i) ),i,1..n-1)],
        proc(x,l) begin [op(x),0$(l-nops(x))] end_proc ,
        n);
  else
    map([[_seqgen(0, i,1..n)],
          _seqgen(op(muEC::SG::ListGrassmannian_rec(i,n-i)),i,1..n-1)],
        muEC::SG::Code2Perm,
        n);
  end_if;
end_proc: 

/*
|  ListVexillar(n,hold(code)) returns the list of all codes of the vexillar
|                         permutations of the symmetric group of degree n.
|                         When called with one argument, it returns the list of
|                         the corresponding permutations (J. West).
|  VP241097
*/
muEC::SG::Path2Vexi_rec:=proc(chemin)
local
    perm,  /* the result...                    */
    i,     /* variable for loop...             */
    j,     /* variable for sequence...         */
    dif,   /* difference between two values... */
    inspt, /* a value...                       */
    as,    /* a permutation...                 */
    oldas; /* a temporary permutation...       */
begin
  perm := [1];
  as := [1,2];
  for i from 2 to nops(chemin) do
    if (op(op(chemin,i),2)>op(op(chemin,i-1),2)) then
      perm := [op(perm,1..op(op(chemin,i),1)-2),i,
               op(perm,op(op(chemin,i),1)-1..nops(perm))];
      oldas := as;
      as := [_seqgen(j, j,1..op(op(chemin,i),1)),
             _seqgen(oldas[j]+1, j,op(op(chemin,i),1)..nops(oldas))];
    else
      dif := op(op(chemin,i),2)-op(op(chemin,i),1);
      inspt := op(as,nops(as)+1-dif);
      perm := [op(perm,1..inspt-1),i,op(perm,inspt..nops(perm))];
      oldas := as;
      as := [_seqgen(oldas[j], j,1..op(op(chemin,i),1)),
             _seqgen(oldas[j]+1,j,nops(oldas)+1-dif..nops(oldas))];
    end_if;
  end_for;
  perm
end_proc:  

/*
|  VP241097
*/
muEC::SG::ListVexillar_rec:=proc(n)
  option remember;
begin
  if (n=1) then
    [[[2,2]]];
  else
    map(muEC::SG::ListVexillar_rec(n-1),
        proc(chemin) local i, couple; begin
          couple := chemin[nops(chemin)];
         _seqgen([op(chemin), [i, couple[2]+1]], i,2..couple[1]+1),
         _seqgen([op(chemin), [couple[1], -i]], i,-couple[2]..-(couple[1]+1));
        end_proc);
  end_if;
end_proc:

/*
|  VP251097
*/ 
muEC::SG::ListVexillar:=proc(n)
begin
  if (args(0)>=2) then
    map(map(muEC::SG::ListVexillar_rec(n), muEC::SG::Path2Vexi_rec), muEC::SG::Perm2Code);
  else
    map(muEC::SG::ListVexillar_rec(n), muEC::SG::Path2Vexi_rec);
  end_if;
end_proc:

/*
|  Type Checking
|  VP251097
*/ 
muEC::SG::ListPerm_check:=proc(n)
local
  i; /* var for loop... */
begin
  if (args(0)=1) then
    if ((not testtype(n,DOM_INT)) and (not testtype(n,DOM_LIST)) and
        (not testtype(n,DOM_SET))) then
      muEC::ERR::SGPerror(1,procname);
    elif (testtype(n,DOM_INT)) then
      if (n<0) then
        muEC::ERR::SGPerror(2,procname);
      end_if;
    elif (nops(n)<0) then
      muEC::ERR::SGPerror(2,procname);
    end_if;
  /* there is an option... */
  elif (args(0)>=2 and args(0)<=5) then
    for i from 2 to args(0) do
      if (not testtype(args(i), DOM_IDENT)) then
        if (i<>5) then
          muEC::ERR::SGPerror(2*i-1, procname);
        else
          muEC::ERR::SGPerror(14, procname);
        end_if;
      end_if;
    end_for;
    if (testtype(n,DOM_INT)) then
      if (n<=0) then
        muEC::ERR::SGPerror(2,procname);
      end_if;
      if (bool(nops({args(2..args(0))})<>nops([args(2..args(0))]))) then
        /* parameters are repeated... */
        muEC::ERR::SGPerror(10,procname);
      end_if;
      for i from 2 to args(0) do
        if (not contains({hold(code), hold(nb), hold(lexic), hold(cixel),
            hold(dominant), hold(grassmannian), hold(vexillary), hold(level)},
            args(i))) then
          if (i<>5) then
            muEC::ERR::SGPerror(2*i,procname);
          else
            muEC::ERR::SGPerror(15,procname);
          end_if;
        end_if;
      end_for;
    elif (testtype(n,DOM_LIST) or testtype(n,DOM_SET)) then
      muEC::ERR::SGPerror(4,procname);
    else
      muEC::ERR::SGPerror(1,procname);
    end_if;
  else
    muEC::ERR::SGPerror(10,procname);
  end_if;
end_proc:

/*
|  VP251097
*/
muEC::SG::ListPerm := proc(nb)
begin
  if testargs() then
    muEC::SG::ListPerm_check(args());
  end_if;
  muEC::SG::ListPerm_core(args());
end_proc:
