/*
|  File Mat2Perm (SG PACKAGE) Mon Oct 27 08:47:31 MET 1997
*/

/*
|  Mat2Perm(graph) calculates the permutation from its matrix of permutation.
|                  This matrix associates to each position(row), the
|                  value in the permutation which lies at that position.
|  VP271097
*/
muEC::SG::Mat2Perm_core := proc(graph)
local
  i,j,  /* vars for loops...                */
  n,    /* degree of the symmetric group... */
  perm; /* the permutation...               */
begin
  n := op(op(op(graph, 0),2),2);
  perm := null();
  for i from 1 to n do
    for j from 1 to n do
      /* if we find a 1, the column j is the value that lies at position i... */
      if (graph[i,j]=1) then
        perm := perm, j;
        break;
      end_if;
    end_for;
  end_for; 
  /* we return a list... */
  [perm];
end_proc:

/*
|  Type Checking
|  VP270197
*/
muEC::SG::Mat2Perm_check:=proc(graph)
begin
  if (not testtype(graph,DOM_ARRAY)) then
    muEC::ERR::SGPerror(1,procname);
  end_if;
  /* graph should be a square table... */
  if (op(op(graph,0),2)<>op(op(graph,0),3)) then
    muEC::ERR::SGPerror(11,procname);
  end_if;
end_proc:

/*
|  VP271097
*/
muEC::SG::Mat2Perm := proc(graph)
begin
  if testargs() then
    muEC::SG::Mat2Perm_check(args());
  end_if;
  muEC::SG::Mat2Perm_core(args());
end_proc:
