/*
|  File Order (SG PACKAGE) Sun Oct 26 17:18:09 MET 1997
*/

/*
|  CompPerm(perm1,perm2,kind) compares the two permutations perm1 and perm2
|                             according to the kind ordering.
|                             kind=hold(lexic)   : lexicographic ordering
|                             kind=hold(cixel)   : reverse lexicographic order
|                             Tests whether perm1 >= perm2.
|
|  Bruhat(perm1,perm2) compares the two permutations perm1 and perm2
|                      according to the Bruhat ordering.
|                     (It tests whether perm1 >= perm2)
*/

/*
|  VP261097
*/
muEC::SG::Bruhat:=proc(perm1,perm2)
local
  p1, p2, /* permutations...                               */
  i,j,    /* vars for loops...                             */
  k,      /* var for seq...                                */
  c,      /* a counter...                                  */
  n1,     /* degree of the symmetric group... for perm1... */
  n2;     /* degree of the symmetric group... for perm2... */
begin
  n1 := nops(perm1);
  n2 := nops(perm2);
  if (n1<>n2) then
    p1 := [op(perm1), _seqgen(k, k,n1+1..max(n1, n2))];
    p2 := [op(perm2), _seqgen(k, k,n2+1..max(n1, n2))];
  else
    p1:=perm1;
    p2:=perm2;
  end_if;
  /* we are sure that n1=n2... */
  if ((p2[1]>p1[1]) or (p2[n1]<p1[n1])) then
    return(FALSE);
  end_if;
  for i from 1 to n1 do
    c := 0;
    for j from 1 to n1 do
      if (p1[j]>i) then
        c := c+1;
      end_if;
      if (p2[j]>i) then
        c := c-1;
      end_if;
      if (c<0) then
        return(FALSE);
      end_if;
    end_for;
  end_for;
  TRUE;
end_proc:

/*
|  VP261097
*/
muEC::SG::CompPerm_core := proc(perm1,perm2,kind)
local
  i,  /* var for loops...                              */
  n,  /* a degree...                                   */
  n1, /* degree of the symmetric group... for perm1... */
  n2; /* degree of the symmetric group... for perm2... */
begin
  n1 := nops(perm1);
  n2 := nops(perm2);
  if (kind=hold(lexic)) then
    n := min(n1,n2);
    for i from 1 to n do
      if (perm1[i]>perm2[i]) then
        return(TRUE);
      elif (perm2[i]>perm1[i]) then
        return(FALSE);
      end_if;
    end_for;
  elif (kind=hold(cixel)) then
    n := min(n1,n2);
    for i from 0 to n-1 do
      if (perm1[n1-i]>perm2[n2-i]) then
        return(TRUE);
      elif (perm2[n2-i]>perm1[n1-i]) then
        return(FALSE);
      end_if;
    end_for;
  elif (kind=hold(bruhat)) then
    return(muEC::SG::Bruhat(perm1, perm2));
  end_if;
  bool(n1>=n2);
end_proc:

/*
|  Type Checking
|  VP261097
*/
muEC::SG::CompPerm_check := proc(perm1,perm2,kind)
begin
  if (not muEC::TYP::IsPerm(perm1)) then
    muEC::ERR::SGPerror(1,procname);
  elif (not muEC::TYP::IsPerm(perm2)) then
    muEC::ERR::SGPerror(3,procname);
  elif (not testtype(kind,DOM_IDENT)) then
    muEC::ERR::SGPerror(5,procname);
  elif (kind<>hold(lexic) and kind<>hold(cixel) and kind<>hold(bruhat)) then
    muEC::ERR::SGPerror(6,procname);
  end_if;
end_proc:

/*
|  VP261097
*/
muEC::SG::CompPerm := proc(perm1,pem2,kind)
begin
  if testargs() then
    muEC::SG::CompPerm_check(args());
  end_if;
  muEC::SG::CompPerm_core(args());
end_proc:
