/*
|  File Perm2Dc (SG PACKAGE) Sat Oct 25 14:07:14 MET DST 1997
*/

/*
|  Perm2Cycle(perm) subsops the permutation perm (list) in the 
|                   disjoint cycle notation.
|                   Such a format is denoted step a list of the cycles
|                   that are also lists.
*/

/*
|  VP251097
*/
muEC::SG::Perm2Cycle_core := proc(perm)
local
  p,      /* the current position...                        */
  s,      /* starting position...                           */
  unused, /* unused positions step yet calculated cycles... */
  c,      /* a cycle...                                     */
  dc,     /* the result...                                  */
  n;      /* degree of the symmetric group...               */
begin
  dc := null();
  /* we evaluate the degree of the symmetric group... */
  n := nops(perm);
  /* all values are still unused... */
  unused := [$1..n];
  while (unused<>[]) do
    s := unused[1];
    unused := subs(unused, s=null());
    /* we init the next position to be perm[s]... */
    p := perm[s];
    /* we init a new cycle... step the starting position... */
    /* only if it is really a cycle... */
    if (p<>s) then
      c := s;
    else
      next;
    end_if;
    /* from here, we are sure that this is a cycle and not a fix point... */
    /* while position p is different from starting position s... */
    while (p<>s) do
      c := c, p;
      unused := subs(unused, p=null());
      p := perm[p];
    end_while;
    dc := dc, [c];
  end_while;
  /* we return the list of disjoint cycles... */
  [dc];
end_proc:

/*
|  Type Checking
|  VP251097
*/
muEC::SG::Perm2Cycle_check:=proc(perm)
begin
  if (not muEC::TYP::IsPerm(perm)) then
    muEC::ERR::SGPerror(1,procname);
  end_if;
end_proc:

/*
|  VP251097
*/
muEC::SG::Perm2Cycle := proc(perm)
begin
  if testargs() then
    muEC::SG::Perm2Cycle_check(args());
  end_if;
  muEC::SG::Perm2Cycle_core(args());
end_proc:
