/*
|  File Perm2Des (SG PACKAGE) Sat Oct 25 15:07:53 MET DST 1997
*/

/*
|  Perm2Rise(perm) returns the list of rises of perm.
|                  When called with the second argument hold(nb), it returns
|                  the number of rises.
|                  When called with the second argument hold(rin), it returns
|                  the sum of the rises positions.
|                  When called with the second argument hold(corin), it
|                  returns the sum of the (n - rise position) where n is the
|                  degree of the symmetric group.
*/

/*
|  VP261097
*/
muEC::SG::Perm2Rise_core := proc(perm)
local
  n, /* degree of the symmetric group... */
  r, /* the result...                    */
  j; /* var for loop...                  */
begin
  n := nops(perm);
  if (args(0)=1) then
    r := null();
    for j from 1 to n-1 do
      if (perm[j]<perm[j+1]) then
        r := r,j
      end_if;
    end_for;
    return([r]);
  end_if;
  if (args(2)=hold(nb)) then
    r := 0;
    for j from 1 to n-1 do
      if (perm[j]<perm[j+1]) then
        r := r+1;
      end_if;
    end_for;
    return(r);
  elif (args(2)=hold(rin)) then
    r := 0;
    for j from 1 to n-1 do
      if (perm[j]>perm[j+1]) then
        r := r+j;
      end_if;
    end_for;
    return(r);
  elif (args(2)=hold(corin)) then
    r := 0;
    for j from 1 to n-1 do
      if (perm[j]<perm[j+1]) then
        r := r+(n-j);
      end_if;
    end_for;
    return(r);
  end_if;
  muEC::ERR::SGPerror(10,procname);
end_proc:

/*
|  Type Checking
|  VP251097
*/
muEC::SG::Perm2Rise_check := proc(perm)
begin
  if (not muEC::TYP::IsPerm(perm)) then
    muEC::ERR::SGPerror(1,procname);
  end_if;
  if (args(0)=2) then
    if (not testtype(args(2),DOM_IDENT)) then
      muEC::ERR::SGPerror(3,procname);
    elif (args(2)<>hold(nb) and args(2)<>hold(rin) and args(2)<>hold(corin)) 
    then
      muEC::ERR::SGPerror(4,procname);
    end_if;
  end_if;
end_proc:

/*
|  VP251097
*/
muEC::SG::Perm2Rise := proc(perm)
begin
  if testargs() then
    muEC::SG::Perm2Rise_check(args());
  end_if;
  muEC::SG::Perm2Rise_core(args());
end_proc:
