/*
|  File RandPerm (SG PACKAGE) Thu Jan  8 08:15:59 MET 1998
*/

/*
|  RandPerm(n) computes a new random permutation of the symmetric
|              group of degree n.
|
|  RandPerm(n, hold(dominant)) computes a new random dominant permutation
|                              of the symmetric group of degree n.
|
|  RandPerm(n, hold(grassmannian)) computes a new random grassmannian
|                                  permutation of the symmetric group of
|                                  degree n.
|  VP080198
*/
muEC::SG::RandPerm_core := proc(n)
local
  j,       /* var for loop...             */
  unused,  /* the set of unused values... */
  used,    /* the set of used values...   */
  Last,    /* last value generated...     */
  descent, /* where the descent is...     */
  perm,    /* the result...               */
  ij,mj;   /* variables...                */
begin
  if (args(0)=1) then
    perm := [$1..n];
    for mj from n downto 1 step 1 do
      ij := random(1..mj)();
      perm := subsop(perm, ij=perm[mj],mj=perm[ij])
    end_for;
    perm;
  elif (args(2)=hold(dominant)) then
    muEC::SG::Code2Perm(sort([random(0..(n-j))() $ j=1..n-1],
                        proc(x,y) begin bool(x>=y) end_proc))
  elif (args(2)=hold(grassmannian)) then
    used := null();
    unused := [$ 1..n];
    descent := random(1..n)();
    for j from 1 to descent do
      Last := op(unused, random(1..nops(unused))());
      used := used, Last;
      unused:=subs(unused, Last=null())
    end_for;
    [op(sort([used])), op(unused)]
  end_if;
end_proc:

/*
|  Type Checking.
|  VP080198
*/
muEC::SG::RandPerm_check:=proc(n)
begin
  if (args(0)=1) then
    if not (testtype(n,DOM_INT)) then
      muEC::ERR::SGPerror(1,procname);
    elif (n<=0) then
      muEC::ERR::SGPerror(2,procname);
    end_if;
  /* there is an option... */   
  elif (args(0)=2) then       
    if (not testtype(args(2), DOM_IDENT)) then
      muEC::ERR::SGPerror(3, procname);
    end_if;
    if testtype(n,DOM_INT) then
      if contains({hold(dominant), hold(grassmannian)},args(2)) then
        if (n<=0) then   
          muEC::ERR::SGPerror(2,procname);
        end_if;
      else
        muEC::ERR::SGPerror(4,procname);
      end_if;
    else
      muEC::ERR::SGPerror(1,procname);
    end_if;
  end_if;
end_proc:

/*
|  VP080198
*/
muEC::SG::RandPerm := proc(n)
begin
  if testargs() then
    muEC::SG::RandPerm_check(args());
  end_if;
  muEC::SG::RandPerm_core(args());
end_proc:
