/*
|  File Transpo (SG PACKAGE) Thu Jan  8 08:42:51 MET 1998
*/

/*
|  SgTranspo(i,n) creates the (elementary) transposition that interchanges the
|                 the positions i and i+1 on the symmetric group of degree n.
|                 When n is omitted, it is taken to be i+1.
|
|  SgTranspo(i,j,n) creates the transposition that interchanges the
|                   the positions i and j on the symmetric group of degree n.
|  VP080198
*/
muEC::SG::SgTranspo_core := proc(i)
local
  k; /* variable for sequence... */
begin
  if (args(0)=1) then
    [_seqgen(k, k,1..i-1), i+1, i];
  elif (args(0)=2) then
    subs([_seqgen(k, k,1..args(2))],{i=i+1,i+1=i});
  else
    subs([_seqgen(k, k,1..args(3))],{i=args(2),args(2)=i});
  end_if;
end_proc:

/*
|  Type Checking.
|  VP080198
*/
muEC::SG::SgTranspo_check := proc(i)
begin
  if (not testtype(i,DOM_INT)) then
    muEC::ERR::SGPerror(1,procname);
  elif (i<=0) then
    muEC::ERR::SGPerror(2,procname);
  end_if;
  if (args(0)>=2) then
    if (not testtype(args(2),DOM_INT)) then
      muEC::ERR::SGPerror(3,procname);
    elif (args(2)<=0) then
      muEC::ERR::SGPerror(4,procname);
    end_if;
    if (args(0)=2) then
      if (i>args(2)-1) then
        muEC::ERR::SGPerror(2,procname);
      end_if;
    end_if;
  elif (args(0)=3) then
    if (not testtype(args(3),DOM_INT)) then
      muEC::ERR::SGPerror(5,procname);
    elif (args(3)<=0) then
      muEC::ERR::SGPerror(6,procname);
    elif (i>args(3)) then
      muEC::ERR::SGPerror(2,procname);
    elif (args(2)>args(3)) then
      muEC::ERR::SGPerror(4,procname);
    end_if;
  end_if;
end_proc:

/*
|  VP080198
*/
muEC::SG::SgTranspo := proc(i)
begin
  if testargs() then
    muEC::SG::SgTranspo_check(args());
  end_if;
  muEC::SG::SgTranspo_core(args());
end_proc:
