/*
|  File Vec2Perm (SG PACKAGE) Thu Jan  8 08:58:36 MET 1998
*/

/*
|  Vect2Perm(v) computes a generic permutation so that the cycle type
|               of the permutation is the reordered vector v into a
|               partition written in decreasing order.
|               [3, 2, 4] --> [3,1,2, 5,4, 9,6,7,8]
|  VP080198
*/
muEC::SG::Vect2Perm_core := proc(v)
local
  p, /* a permutation... */
  b, /* previous available value... */
  n, /* next available value... */
  i, /* variable for loop... */
  j; /* variable for sequence... */
begin
  p := null();
  b := 1;
  n := 0;
  for i from 1 to nops(v) do
    b := n+1;
    n := n+v[i];
    p := p,n,_seqgen(j, j,b..(n-1));
  end_for;
  [p];
end_proc:

/*
|  Type Checking.
|  VP080198
*/
muEC::SG::Vect2Perm_check:=proc(v)
local
  i; /* variable for loop... */
begin
  if (not testtype(v,DOM_LIST)) then
    muEC::ERR::SGPerror(1,procname);
  end_if;
  if (v=[]) then
    muEC::ERR::SGPerror(2,procname);
  end_if;
  for i from 1 to nops(v) do
    if (not testtype(v[i],Type::PosInt)) then
      muEC::ERR::SGPerror(2,procname);
    end_if;
  end_for;
end_proc:

/*
|  VP080198
*/
muEC::SG::Vect2Perm := proc(v)
begin
  if testargs() then
    muEC::SG::Vect2Perm_check(args());
  end_if;
  muEC::SG::Vect2Perm_core(args());
end_proc:
