/*
|  File MaxTr (SP PACKAGE) Mon May 18 18:56:58 MET DST 1998
*/

/*
|  MaxTr_short(perm) is a short release of the function MaxTr. The result does
|                    not contain the permutation perm. Moreover, the result is
|                    a list.
|  VP180598
*/ 
muEC::SP::MaxTr_short:=proc(perm)
local
  n,   /* degree of the symmetric group...    */
  nu,  /* a permutation...                    */
  Phi, /* a list of permutations...           */ 
  Min, /* explicit name...                    */
  r,   /* the position of the last descent... */
  i;   /* variable for loop...                */
option remember;
begin
  n:=nops(perm);
  r:=0;
  for i from n downto 2 do
    if perm[i]<perm[i-1] then
      /* we have just found the last descent of perm... */
      r := i-1;
      break;
    end_if;
  end_for;
  /* we are now looking for the permutation nu such that
  | perm = nu * t       where perm(k) is the greatest element to the right of
  |              rk     position r, lesser than perm(r).
  | For instance, perm = 52186347, ---> r = last descent = 5 --> perm(5) = 6
  | and k=7 since perm(k)=perm(7)=4 < perm(r)=6.
  | note that k>r
  */
  for i from n downto r+1 do
    if perm[i]<perm[r] then
      nu := subsop(perm,  r=perm[i],i=perm[r]);
      break;
    end_if;
  end_for;
  /* the aim now is to generate the DOM_SET Phi of permutations.
  | candidates are all transpositions t   such that j<r
  |                                    jr
  | the aim is that l(nu * t  ) = l(nu) + 1 so that the transposition
  |                         jr
  | adds only one inversion !!!
  */
  Min := 0;
  Phi := null();
  for i from r-1 downto 1 do
    if nu[i]<nu[r] and nu[i]>Min then
      Min := nu[i];
      Phi := Phi, subsop(nu, i=nu[r],r=Min);
    end_if;
  end_for;
  [r,nu,[Phi]];
end_proc:
/*
|  MaxTr(perm) calculates the maximal transition for the schubert polynomial
|              X[perm]. The result is Transition([perm],r,[u],[[v1],[v2],..])
|              so that
|                                    ---
|                                    \
|              X[perm] = x_r * X[u] + ) X[v]
|                                    /
|                                    ---
|
|  VP180598
*/
muEC::SP::MaxTr := proc(perm)
local
  n,   /* degree of the symmetric group...    */
  nu,  /* a permutation...                    */
  Phi, /* a list of permutations...           */ 
  Min, /* explicit name...                    */
  r,   /* the position of the last descent... */
  i;   /* variable for loop...                */
begin
  n := nops(perm);
  r := 0;
  for i from n downto 2 do
    if perm[i]<perm[i-1] then
      /* we have just found the last descent of perm... */
      r := i-1;
      break;
    end_if;
  end_for;
  /* we are now looking for the permutation nu such that
  | perm = nu * t       where perm(k) is the greatest element to the right of
  |              rk     position r, lesser than perm(r).
  | For instance, perm = 52186347, ---> r = last descent = 5 --> perm(5) = 6
  | and k=7 since perm(k)=perm(7)=4 < perm(r)=6.
  | note that k>r
  */
  for i from n downto r+1 do
    if perm[i]<perm[r] then
      nu := subsop(perm, r=perm[i],i=perm[r]);
      break;
    end_if;
  end_for;
  /* the aim now is to generate the DOM_SET Phi of permutations.
  | candidates are all transpositions t   such that j<r
  |                                    jr
  | the aim is that l(nu * t  ) = l(nu) + 1 so that the transposition
  |                         jr
  | adds only one inversion !!!
  */
  Min := 0;
  Phi := null();
  for i from r-1 downto 1 do
    if nu[i]<nu[r] and nu[i]>Min then
      Min := nu[i];
      Phi := Phi, subsop(nu, i=nu[r],r=Min);
    end_if;
  end_for;
  hold(Transition)(perm,r,nu,[Phi]);
end_proc:
