/*
|  File Monk (SP PACKAGE) Sun May 17 15:40:37 MET DST 1998
*/

/*
| SV160698
*/
muEC::SP::ReduceP := proc(perm)
local
  p, /* new permutation... */
  i; /* var for loop... */
begin
  if testtype(perm, DOM_LIST) then
    p := perm;
  else
    p := [op(perm, 2..nops(perm))];
  end_if;
  for i from nops(p) downto 1 do
    /*if p[i]=i then p[i]:=NIL; else break; end_if;*/
    if p[i]=i then delete(p[i]); else break; end_if;
  end_for;
  if nops(p)=0 then
    p := [1];
  end_if;
  if testtype(perm, DOM_LIST) then
    p;
  else
    op(perm,1)[op(p)];
  end_if;
end_proc:

/*
|  MonkP(k,p) multiplies the Schubert polynomial corresponding to the
|             permutation p by the k th variable...
|             Permutations are reduced to the smallest symmetric group
|             containing them.
|             The result is a list of two lists of permutations : the first one
|             contains the negative part of the result while the second
|             one contains the positive part.
|  VP170598 SV160698
*/
muEC::SP::MonkP:=proc(k,p)
local
  n,    /* degree of the symmetric group...                         */
  i,    /* var for loop...                                          */
  j,    /* var for seq...                                           */
  fl,   /* a flag=TRUE when there is only the extra term in PhiP... */
  PhiP, /* positive terms...                                        */
  PhiN, /* negative terms...                                        */
  Min,  /* an explicit name for a variable !!!...                   */
  perm; /* a permutation...                                         */
begin
  n := nops(p);
  fl := FALSE;
  /* we adjust the length of the permutation, adding sometimes a fix point... */
  if k>n then
    perm := [op(p), _seqgen(j, j,n+1..k+1)];
    fl := TRUE;
    n := k+1;
  elif k=n then
    perm := [op(p), n+1];
    fl := TRUE;
    n := n+1;
  else
    if p[n]=n then
      perm := p;
    else
      perm := [op(p), n+1];
      n := n+1;
      fl := TRUE;
      for i from k+1 to n-1 do
        if p[i]>p[k] then
          /* we do not need to look for permutations in the symmetric group */
          /* of degree n+1, since a value on the right side of the          */
          /* value p[k] is greater than it...                               */
          perm := p;
          n := n-1;
          fl := FALSE;
          break;
        end_if;
      end_for;
    end_if;
  end_if;
  /* the aim now is to generate a set Phi of permutations.              */
  /* The candidates are all transpositions t   .                        */
  /*                                        ik                          */
  /* The aim is that l(nu * t  ) = l(nu) + 1 so that the transposition  */
  /*                         ik                                         */
  /* adds only one inversion !!!                                        */
  Min := 0;
  PhiN := null();
  for i from k-1 downto 1 do
    if perm[i]<perm[k] and perm[i]>Min then
      Min := perm[i];
      PhiN := PhiN, muEC::SP::ReduceP(subsop(perm, i=perm[k], k=Min));
    end_if;
  end_for;
  if fl=FALSE then
    /* in fact, from now, Min is a maximal value !!! */
    Min := n+1;
    PhiP := null();
    for i from k+1 to n do
      if perm[i]>perm[k] and perm[i]<Min then
        Min := perm[i];
        PhiP := PhiP, muEC::SP::ReduceP(subsop(perm, i=perm[k], k=Min));
      end_if;
    end_for;
  else
    /* the positive part is composed of the single extra term so that */
    /* we do not need any loop...                                     */
    PhiP := muEC::SP::ReduceP(subsop(perm, n=perm[k], k=n));
  end_if;
  [[PhiN], [PhiP]]
end_proc:
/*
|  VP170598
*/
muEC::SP::Monk := proc(k,p)
local
  r;   /* the result... */
begin
  r := muEC::SP::MonkP(k, [op(p, 2..nops(p))]);
  if op(p,1)=hold(X) then
    _plus(op(map(r[1], proc(x) begin -hold(X)[op(x)] end_proc )), op(map(r[2], proc(x) begin hold(X)[op(x)] end_proc )));
  else      /* p is a double Schubert polynomial, so we add the extra term... */
    if k < nops(p) then
      _plus(op(map(r[1], proc(x) begin -hold(XX)[op(x)] end_proc )), op(map(r[2], proc(x) begin hold(XX)[op(x)] end_proc )),
            hold(y).(op(p,k+1))*muEC::SP::ReduceP(p));
    else
      _plus(op(map(r[1], proc(x) begin -hold(XX)[op(x)] end_proc )), op(map(r[2], proc(x) begin hold(XX)[op(x)] end_proc )),
            hold(y).(k)*muEC::SP::ReduceP(p));
    end_if;
  end_if;
end_proc:
