/*
|  File NewtonIn (SP PACKAGE) Tue Jun 16 09:53:11 MET DST 1998
*/

/*
|  NewtonInterp(f,n) expresses f in terms of XX Schubert polynomials.
|                    f is expressed on the monomial basis. Coefficients must
|                    not involve the y alphabet.
|                    The Newton interpolation formula is used and the sum is
|                    over the symmetric group of degree n.
|  SV160698 VP101198
*/
muEC::SP::NewtonInterp_core := proc(f, n)
local
  i,  /* var for seq...                   */
  nf; /* new function in terms...         */
begin
  nf := subs(f, map(indets(f),
                 proc(vr) local v; begin if muEC::TYP::Isx(vr)
                                 then v := muEC::SP::EValue(vr,hold(x)); vr=hold(y).v;
                                 else null() end_if end_proc
                 ));
  _plus(_seqin(muEC::SP::NewtonDd(nf,muEC::SG::Perm2Rd(i))*hold(XX)[op(i)], i,muEC::SG::ListPerm(n)))
end_proc:

/*
|  SV160698 VP290399
*/
muEC::SP::NewtonDd := proc(f,rd)
local
  l; /* a recursive call... */
  option remember;
begin
  if rd=[] then
    f;
  else
    l := muEC::SP::NewtonDd(f,[op(rd, 2..nops(rd))]);
    normal((l-(subs(l,
      {muEC::cat(hold(y),rd[1])=muEC::cat(hold(y),rd[1]+1), 
       muEC::cat(hold(y),rd[1]+1)=muEC::cat(hold(y),rd[1])})))/
      (muEC::cat(hold(y),rd[1])-muEC::cat(hold(y),rd[1]+1)));
  end_if;
end_proc:
/*
|  Type Checking
|  SV160698
*/
muEC::SP::NewtonInterp_check := proc(f,n)
begin
  if not testtype(n,DOM_INT) then
    muEC::ERR::SPPerror(1,procname);
  elif n<=0 then
    muEC::ERR::SPPerror(2,procname);
  end_if;
end_proc:

/*
|  VP300697
*/
muEC::SP::NewtonInterp := proc(f, n)
begin
  if testargs() then
    muEC::SP::NewtonInterp_check(args());
  end_if;
  muEC::SP::NewtonInterp_core(args());
end_proc:
