/*
|  File CollctSp (SP PACKAGE) Tue Jun 16 18:59:18 MET DST 1998
*/

/*
|  SpCollect(sp) takes a Schubert polynomial and collects it
|                in order to be treatable by other functions of SP.
|  SpCollect(sp,b)     does the same only for the specified base b.
|  SpCollect(sp,blist) does the same only for the specified bases blist.
|  SV160698
*/

muEC::SP::SpCollect := proc(sp, bases)
local
  i,    /* var for loop... */
  k,    /* var for loop... */
  j,    /* var for seq...  */
  ptmp, /* a var...        */
  tmp,  /* a var...        */
  pow,  /* a power...      */
  mn,   /* a monom...      */
  mn1,  /* a monom...      */
  t;    /* a table...      */
begin
  if testtype(sp,"_plus") then
    map(sp, muEC::SP::SpCollect, bases);
  else
    t := muEC::SP::Monomial(sp, bases);
    for i in bases do
      if (t[i]<>1) then
        if testtype(t[i], "_mult") then
          if i=hold(X) then
            t[i] := muEC::SP::XProduct(t[i]);
          else
            t[i] ; /* for the moment... */
          end_if;
        elif testtype(t[i], "_power") then
          pow := op(t[i], 2);
          mn := op(t[i],1);
          if has(muEC::SP::SPAlgBases,i) then
            if testtype(pow, DOM_INT) then
              t[i] := i[_seqgen(op(mn,j)$op(t[i],2),j,2..nops(mn))];
            end_if;
          elif i=hold(m) then
            t[i] := muEC::SP::Tom(muEC::SP::m2p(mn)^pow); /* for the moment... */
          else
            t[i] := muEC::SP::SchurProduct(t[i]);
          end_if;
        end_if;
      end_if;
    end_for;
    _mult(t[hold(coeff)],_seqin(t[i], i,bases));
  end_if;
end_proc:

/*
|  SpCollect(sp) applies SpCollect for all bases in muEC::SP::SPBases.
|  SpCollect(sp, blist) applies SpCollect for all bases in blist.
|  VP270498
*/
muEC::SP::SpCollect_core := proc(sp)
local
  blist; /* new blist...    */
begin
  if args(0)<2 then
    blist := muEC::SP::SPBases;
  else
    blist := args(2);
  end_if;
  muEC::SP::SpCollect(sp,blist);
end_proc:

/*
|
|  VP270498
*/
muEC::SP::SpCollect_check:=proc(sp)
local
  i; /* var for seq... */
begin
  if args(0)>1 then
    if (not testtype(args(2),DOM_LIST)) and
       (not testtype(args(2),DOM_IDENT)) then
      muEC::ERR::SPPerror(3,procname); /* wrong type for 2nd argument... */
    elif not _and(_seqin(has(muEC::SP::SPBases,i),i,args(2))) then
      muEC::ERR::SPPerror(3,procname); /* wrong type for 2nd argument... */
    end_if;
  end_if;
end_proc:

/*
|  VP270498
*/
muEC::SP::SpCollect := proc(sp)
begin
  if testargs() then
    muEC::SP::SpCollect_check(args());
  end_if;
  muEC::SP::SpCollect_core(args());
end_proc:

