/*
|  File TableX (SP PACKAGE) Fri May 22 18:20:05 MET DST 1998
*/

/*
|  TableX(n) returns the table of all Schubert polynomials lying in
|            the symmetric group of degree n (the table being indexed
|            by permutations). When used with the second parameter hold(Pe)
|            the result is expressed in the basis of product of elementary
|            symmetric function (dual basis of the monomial basis).
|  SV220598 VP290399
*/
muEC::SP::TableX_core := proc(n)
local
  t; /* the result... */
begin
  if args(0)=1 then
    t := table(op(map(muEC::SG::ListPerm(n), proc( p) begin p=muEC::SP::X2x_remember(p) end_proc )));
    sysassign(muEC::SP::X2x_remember,
              muEC::clearRememberTable(muEC::SP::X2x_remember));
    t;
  else
    t := table(op(map(muEC::SG::ListPerm(n),
                   proc(p) local res;
                   begin
                   res := muEC::SP::X2Pe_remember(p);
                   p=subs(res,
                          map([op(select(indets(res, RatExpr), muEC::TYP::IsPe))],
			      proc(prode) local i, pe; 
                              begin
                                  pe := [op(prode, 2..nops(prode))];
                                  for i from 1 to nops(pe) do
                                      if op(pe, i)<>0 then
                                          break;
                                      end_if;
                                  end_for;
                                  if i<=nops(pe) then
                                      prode=hold(Pe)[op(pe, i..nops(pe))];
                                  else
                                      prode=hold(Pe)[0];
                                  end_if;
                              end_proc));
                   end_proc)));
    sysassign(muEC::SP::X2Pe_remember,
              muEC::clearRememberTable(muEC::SP::X2Pe_remember));
    t;
  end_if;
end_proc:

/*
|  SV220598 VP290399
*/
muEC::SP::DdOnPe := proc(i, elemP)
begin
  subs(elemP, map([op(select(indets(elemP, RatExpr), muEC::TYP::IsPe))],
           proc(prode, i) local a, b, j, n, pe; begin
              pe := [op(prode,2..nops(prode))];
              n := nops(pe);
              a := op(pe,i);
              b := op(pe,i+1);
              hold(Pe)[op(pe)]=
                _plus(_seqgen(hold(Pe)[op(pe, 1..i-1), a-1-j, b+j,
                               op(pe, i+2..n)],
                             j,0..min(a-1, n-i-1-b)))
                -
                _plus(_seqgen(hold(Pe)[op(pe, 1..i-1), b+j, a-1-j,
                               op(pe, i+2..n)],
                             j,1..min(a-1, n-i-b)));
            end_proc,
            i));
end_proc:

/*
|  SV220598
*/
muEC::SP::X2Pe_remember := proc(perm)
local
  i, /* var for seq...                   */
  j, /* var for loop...                  */
  n; /* degree of the symmetric group... */
option remember;
begin
  n := nops(perm);
  if perm=[_seqgen(n+1-i, i,1..n)] then
    hold(Pe)[_seqgen(n-i, i,1..n)];
  else
    j := 1;
    while perm[j]>perm[j+1] and j<n do j:=j+1; end_while;
    muEC::SP::DdOnPe(n-j,
        muEC::SP::X2Pe_remember(subsop(perm, j=perm[j+1], j+1=perm[j])));
  end_if;
end_proc:
/*
|  SV220598
*/
muEC::SP::X2x_remember:=proc(p)
local
  maxtr; /* maximal transition for p... */
option remember;
begin
  if p=[$1..nops(p)] then
    1;
  else
    maxtr := muEC::SP::MaxTr_short(p);
    hold(x).(op(maxtr,1))*muEC::SP::X2x_remember(op(maxtr,2)) +
    _plus(op(map(op(maxtr,3), muEC::SP::X2x_remember)));
  end_if;
end_proc:

/*
|  Type Checking.
|  SV220598 VP290399
*/
muEC::SP::TableX_check := proc(n)
begin
  if not testtype(n,DOM_INT) then
    muEC::ERR::SPPerror(1,procname);
  elif n<=0 then
    muEC::ERR::SPPerror(2,procname);
  end_if;
  if args(0)=2 then
    if not testtype(args(2),DOM_IDENT) then
      muEC::ERR::SPPerror(3,procname);
    elif not contains({hold(Pe)}, args(2)) then
      muEC::ERR::SPPerror(4,procname);
    end_if;
  end_if;
end_proc:

/*
|  SV220598
*/
muEC::SP::TableX := proc(n)
begin
  if testargs() then
    muEC::SP::TableX_check(args());
  end_if;
  muEC::SP::TableX_core(args());
end_proc:
