/*
|  File TableXX (SP PACKAGE) Mon May 25 18:39:52 MET DST 1998
*/

/*
|  TableXX(n) returns the table of all double Schubert polynomials lying in
|             the symmetric group of degree n (the table being indexed
|             by permutations).
|             If a second argument perm is specified, then the second
|             alphabet is specialized as a permutation (perm) of the first
|             one.
|  SV250598 VP290399
*/
muEC::SP::TableXX_core := proc(n)
begin
  if args(0)=1 then
    table(op(map(muEC::SG::ListPerm(n), proc( p) begin p=muEC::SP::XX2x_remember(p) end_proc )));
  else
    table(op(map(muEC::SG::ListPerm(n), proc(p,s) begin p=muEC::SP::XX2x_remember(p,s) end_proc,
           args(2))));
  end_if;
end_proc:

/*
|  SV250598
*/
muEC::SP::XX2x_remember := proc(p)
local
  nx,    /* x.nx... */
  ny,    /* y.ny... */
  maxtr; /* maximal transition for p... */
option remember;
begin
  if p=[$1..nops(p)] then
    return(1);
  end_if;
  /* We compute the maximal transition for the permutation p... */
  maxtr := muEC::SP::MaxTr_short(p);
  /* we compute the maximal transition for the permutation p... */
  maxtr := muEC::SP::MaxTr_short(p);
  if args(0)=1 then
    return((hold(x).(op(maxtr,1))-hold(y).(op(op(maxtr,2),op(maxtr,1))))*
            muEC::SP::XX2x_remember(op(maxtr,2))
           + _plus(op(map(op(maxtr,3), muEC::SP::XX2x_remember))));
  end_if;
  /* a permutation is specified to specialize the second alphabet : the yi's... */
  nx := op(maxtr,1);
  ny := op(op(maxtr,2), op(maxtr,1));
  if nops(args(2))<ny then
    if nx<>ny then
      return((hold(x).(nx)-hold(x).(ny))*muEC::SP::XX2x_remember(op(maxtr,2),args(2..args(0)))
             + _plus(op(map(op(maxtr,3), muEC::SP::XX2x_remember,args(2..args(0))))));
    else
      return(_plus(op(map(op(maxtr,3),muEC::SP::XX2x_remember,args(2..args(0))))));
    end_if;
  else
    if nx<>op(args(2),ny) then
      return((hold(x).(nx)-hold(x).(op(args(2),ny)))*muEC::SP::XX2x_remember(op(maxtr,2),args(2..args(0)))
             + _plus(op(map(op(maxtr,3),muEC::SP::XX2x_remember,args(2..args(0))))));
    else
      return(_plus(op(map(op(maxtr,3),muEC::SP::XX2x_remember,args(2..args(0))))));
    end_if;
  end_if;
end_proc:

/*
|  Type Checking.
|  SV250598 VP290399
*/
muEC::SP::TableXX_check := proc(n)
begin
  if not testtype(n,DOM_INT) then
    muEC::ERR::SPPerror(1,procname);
  elif n<=0 then
    muEC::ERR::SPPerror(2,procname);
  end_if;
  if (args(0)>=2) then
    if not muEC::TYP::IsPerm(args(2)) then
      muEC::ERR::SPPerror(3,procname);
    end_if;
  end_if;
end_proc:

/*
|  VP300697
*/
muEC::SP::TableXX := proc(n)
begin
  if testargs() then
    muEC::SP::TableXX_check(args());
  end_if;
  muEC::SP::TableXX_core(args());
end_proc:
