/*
|  File x2XX (SP PACKAGE) Tue Jun 16 15:26:28 MET DST 1998
*/

/*
|  x2XX(f) from monomial basis to XX Schubert basis.
|  SV160698 VP290399
*/
muEC::SP::x2XX_heart := proc(f)
local
  i,  /* var for loop...                                                    */
  j,  /* var for seq...                                                     */
  l,  /* var for loop...                                                    */
  u,  /* temporary result...                                                */
  r,  /* temporary result...                                                */
  m,  /* the biggest monomial for lexicographic order from right to left... */
  d,  /* maximal i such that x.i appears in f...                            */
  g,  /* f without constant term...                                         */
  c,  /* constant term of f...                                              */
  lc, /* list of coefficients...                                            */
  k,  /* var for seq...                                                     */
  t;  /* a table representing a monomial...                                 */
begin
  if testtype(f,"_plus") then
    d := muEC::SP::Sp2TableVar(f,hold(x));
    if d={} then
      return(f*hold(XX)[1]);
    end_if;
    /* d is not empty... */
    d := max(op(d));
    c := subs(f, {_seqgen(hold(x).j=0, j,1..d)});
    /* g cannot be 0 because d is not 0, so that it exists at least one x.i... */
    g := f-c;
    if testtype(g, "_mult") then
      return(c*hold(XX)[1] + muEC::SP::x2XX_heart(g));
    end_if;
    m := op(g,1);
    u := [_seqgen(degree(m,hold(x).i),i,1..d)];
    lc := subs(m, {_seqgen(hold(x).k=1, k,1..d)});
    for i from 2 to nops(g) do
      m := op(g,i);
      if [_seqgen(degree(m,hold(x).k),k,1..d)]=u then
        lc := lc+subs(m, {_seqgen(hold(x).k=1, k,1..d)});
      else
        for l from d downto 1 do
          if degree(m, hold(x).l) > u[l] then
            u := [_seqgen(degree(m,hold(x).k), k,1..d)];
            lc := subs(m, {_seqgen(hold(x).k=1, k,1..d)});
            break;
          elif degree(m, hold(x).l) < u[l] then
            break;
          end_if;
        end_for;
      end_if;
    end_for;
    /* m is the biggest monomial for lexicographic order from right to left... */
    r := muEC::SG::Code2Perm(u);
    return(c*hold(XX)[1] + lc*hold(XX)[op(r)] + muEC::SP::x2XX_heart(expand(g-lc*muEC::SP::XX2x_vect(r))));
  end_if;
  /* this is a monomial... */
  /* and we treat all other bases as scalars... */
  t := muEC::SP::Monomial(f, hold(x));
  /* we treat the table... */
  if bool(t[hold(x)]=hold(t[x])) then
    t[hold(coeff)]*hold(XX)[1];
  else
    /* this is a monomial containing some xi's... */
    t[hold(coeff)]*muEC::SP::x2XXmonom(t[hold(x)],hold(XX)[1]);
  end_if;
end_proc:

/*
|  x2XXmonom(mon,res) converts the monomial in the xi's variables on the XX
|                     Schubert basis.
|                     The result is set to res at the beginning but it is
|                     assumed that res is a Schubert polynomial or an
|                     expression involving only Schubert polynomials and
|                     integers.
|  SV160698
*/
muEC::SP::x2XXmonom := proc(mon,res)
local
  i, /* var for seq...  */
  j, /* var for loop... */
  k, /* var for loop... */
  l, /* a list...       */
  r; /* the result...   */
begin
  if res=0 then
    return(0);
  end_if;
  r := res;
  l := degreevec(mon, [_seqgen(hold(x).i,i,1..max(op(muEC::SP::Sp2TableVar(mon,hold(x)))))]);
  for j from 1 to nops(l) do
    for k from 1 to l[j] do
      r := subs(r, map(muEC::SP::Sp2TableVar(r,hold(XX)),
                     proc( perm) begin hold(XX)[op(perm)] = muEC::SP::Monk(j,hold(XX)[op(perm)]) end_proc ));
    end_for;
  end_for;
  r;
end_proc:

/*
|  SV160698 VP290399
*/
muEC::SP::x2XX := proc(f)
local
  r; /* the result... */
begin
  if args(0)=1 then
    return(muEC::SP::x2XX_heart(expand(f)));
  elif args(0)=2 then
    if args(2)=hold(NoExpand) then
      return(muEC::SP::x2XX_heart(f));
    elif args(2)=hold(Collect) then
      r := muEC::SP::x2XX_heart(expand(f));
      return(collect(r,[op(map(muEC::SYMF::AllIndexed(r,hold(XX)), proc(
                                                              mu) begin hold(XX)[op(mu)] end_proc ))], normal));
    else
      muEC::ERR::SPPerror(4,procname);
    end_if;
  end_if;
  if args(2)=hold(NoExpand) then
    if args(3)=hold(Collect) then
      r:=muEC::SP::x2XX_heart(f);
      return(collect(r,[op(map(muEC::SYMF::AllIndexed(r,hold(XX)),
                        proc(mu) begin hold(XX)[op(mu)] end_proc ))], normal));
    else
      muEC::ERR::SPPerror(6,procname);
    end_if;
  else
    muEC::ERR::SPPerror(4,procname);
  end_if;
end_proc:
