/*
|  File CollctSf (SYMF PACKAGE) Tue Dec  2 11:38:16 MET 1997
*/

/*
|  SfCollect(sf) takes a symmetric function and collects symmetric
|                functions to be treatable by other functions of SYMF.
|                For instance e[2]^2*s[2,1]*e[1] will be transformed
|                into e[2,2,1]*s[2,1], products of Schur functions will be
|                expanded, and so on.
|  SfCollect(sf,b)     does the same only for the specified base b.
|  SfCollect(sf,blist) does the same only for the specified bases blist.
|  VP270498 VP280498
*/

muEC::SYMF::CollectSf := proc(sf, bases)
local
  i,    /* var for loop... */
  k,    /* var for loop... */
  j,    /* var for seq...  */
  ptmp, /* a var...        */
  tmp,  /* a var...        */
  pow,  /* a power...      */
  mn,   /* a monom...      */
  mn1,  /* a monom...      */
  t;    /* a table...      */
begin
  if testtype(sf,"_plus") then
    map(sf, muEC::SYMF::CollectSf, bases);
  else
    t := muEC::SYMF::Monomial(sf, bases);
    for i in bases do
      if (t[i]<>1) then
        if testtype(t[i], "_mult") then
          if has(muEC::SYMF::SYMFAlgBases,i) then
            tmp := null();
            ptmp := 1;
            for k from 1 to nops(t[i]) do
              mn1 := op(t[i],k);
              if testtype(mn1, "_power") then
                pow := op(mn1, 2);
                mn  := op(mn1, 1);
                if testtype(pow, DOM_INT) then
                  tmp := tmp, _seqgen(op(mn,j)$pow,j,2..nops(mn));
                else
                  ptmp := ptmp * mn1;
                end_if;
              else
                tmp := tmp, op(mn1,2..nops(mn1));
              end_if;
            end_for;
            t[i] := ptmp * i[op(sort([tmp], proc(a,b) begin a>b end_proc ))];
          elif i=hold(m) then
            t[i] := muEC::SYMF::Tom(map(t[i],muEC::SYMF::m2p)); /* for the moment... */
          else
            t[i] := muEC::SYMF::SchurProduct(t[i]);
          end_if;
        elif testtype(t[i], "_power") then
          pow := op(t[i], 2);
          mn := op(t[i],1);
          if has(muEC::SYMF::SYMFAlgBases,i) then
            if testtype(pow, DOM_INT) then
              t[i] := i[_seqgen(op(mn,j)$op(t[i],2),j,2..nops(mn))];
            end_if;
          elif i=hold(m) then
            t[i] := muEC::SYMF::Tom(muEC::SYMF::m2p(mn)^pow); /* for the moment... */
          else
            t[i] := muEC::SYMF::SchurProduct(t[i]);
          end_if;
        end_if;
      end_if;
    end_for;
    _mult(t[hold(coeff)],_seqin(t[i], i,bases));
  end_if;
end_proc:

/*
|  SfCollect(sf) applies CollectSf for all bases in muEC::SYMF::SYMFBases.
|  SfCollect(sf, blist) applies CollectSf for all bases in blist.
|  VP270498
*/
muEC::SYMF::SfCollect_core := proc(sf)
local
  blist; /* new blist...    */
begin
  if args(0)<2 then
    blist := muEC::SYMF::SYMFBases;
  else
    blist := args(2);
  end_if;
  muEC::SYMF::CollectSf(sf,blist);
end_proc:

/*
|
|  VP270498
*/
muEC::SYMF::SfCollect_check:=proc(sf)
local
  i; /* var for seq... */
begin
  if args(0)>1 then
    if (not testtype(args(2),DOM_LIST)) and
       (not testtype(args(2),DOM_IDENT)) then
      muEC::ERR::SYMFPerror(3,procname); /* wrong type for 2nd argument... */
    elif not _and(_seqin(has(muEC::SYMF::SYMFBases,i),i,args(2))) then
      muEC::ERR::SYMFPerror(3,procname); /* wrong type for 2nd argument... */
    end_if;
  end_if;
end_proc:

/*
|  VP270498
*/
muEC::SYMF::SfCollect := proc(sf)
begin
  if testargs() then
    muEC::SYMF::SfCollect_check(args());
  end_if;
  muEC::SYMF::SfCollect_core(args());
end_proc:

