/*
|  File Monomial (SYMF PACKAGE) Wed Oct 22 15:30:47 MET DST 1997
*/

/*
|  Monomial(monom,bases) returns a table whose entries describe the sets of
|                        variables from bases that occur in monom. 
|                        bases may be a list or list of string names.
|                        If the second argument is a string --name of a basis--,
|                        then the function assumes that monom is expressed on
|                        this basis.
|                        An entry hold(coeff) is reserved for the rest of the
|                        monomial in the table.
|  Monomial(monom)       does the same, putting bases:=SYMFBases.
*/

/*
|  Test wether the monom is indexed by a non-partition.
|  To be used in muEC::SYMF::Monomial...
|  (Result is: isalmost_flag, null_flag, new_monom, new_coeff)
|  VP221097
*/
muEC::SYMF::Monom_Almost := proc(monom, base)
local
    part,        /* a partition...            */
    IsAlmostFun, /* IsAlmost_ to be called... */
    pow,         /* a power...                */
    sig,         /* a signature...            */
    IsAlmstFlg,  /* result of main call...    */
    NullFlg,     /* is the result null?...    */
    monombis,    /* a monom...                */
    newcoeff;    /* a coeff...                */
begin
  sig := 1;               /* default value... */
  NullFlg := FALSE;       /* default value... */
  IsAlmstFlg := TRUE;     /* default value... */
  monombis := 0;          /* default value... */
  newcoeff := 1;          /* default value... */
  if (base=hold(s)) or (base=hold(m)) or (base=hold(e))
                    or (base=hold(h)) or (base=hold(p)) then
    IsAlmostFun := slot(muEC::TYP, "IsAlmost". base);
    if (testtype(monom, "_power")) then
      monombis := op(monom, 1);
      pow := op(monom, 2);
    else
      monombis := monom;
      pow := 1;
    end_if;
    if IsAlmostFun(monombis) then
      if (base=hold(s)) then
        part := muEC::SYMF::RedressVect([op(monombis,2..nops(monombis))]);
        sig := op(part, 2);
        part := op(part, 1);
        if (sig = 0) then
          NullFlg := TRUE; /* return(table([hold(coeff) = 0])) in main fun. */
        else
          newcoeff := sig^pow ;
        end_if;
      else
        part := sort(subs([op(monombis, 2..nops(monombis))], 0=null()),
                     proc(x,y) begin bool(x >= y) end_proc);
      end_if;
      monombis := base[op(part)]^pow;
    else /* return default values */
      IsAlmstFlg := FALSE;
    end_if;
  /* else   another base: impossible */
  end_if;
  IsAlmstFlg, NullFlg, monombis, newcoeff;
end_proc:

/*
|  VP221097 VP021297
*/
muEC::SYMF::Monomial := proc(monom)
local
  bases, /* base to be considered...                */
  test,  /* the test function... Ish, Iss, ...      */
  getv,  /* the current element has been treated... */
  v,v1,  /* current elements...                     */
  i,j,   /* vars for loop...                        */
  k,     /* var for seq...                          */
  Alst,  /* result of muEC::SYMF::Monom_Almost call...    */
  t;     /* the table...                            */
begin
  if (args(0)<2) then
    bases := muEC::SYMF::SYMFBases;
  else
    bases := args(2);
  end_if;
  if (testtype(bases, DOM_IDENT)) then
    /* we init the table... */
    t := table(hold(coeff) = 1, _seqin(k=1,k,muEC::SYMF::SYMFBases));
    test := muEC::cat("muEC::TYP::Is", bases);
    /* we test the structure of the monomial... */
    if (testtype(monom,"_mult")) then
      for i from 1 to nops(monom) do
        v := op(monom, i);
        if (testtype(v,"_power") and level(eval(test(op(v,1))),2))
        or  level(eval(test(v)),2) then
          t[bases] := t[bases] * v;
        else
          Alst := muEC::SYMF::Monom_Almost(v, bases);
          if op(Alst,2) then return(table(hold(coeff) = 0)); end_if;
          if not op(Alst,1) then
            t[hold(coeff)] := t[hold(coeff)] * v;
          else
            t[hold(coeff)] := t[hold(coeff)] * op(Alst,4);
            t[bases] := t[bases] * op(Alst,3);
          end_if;
        end_if;
      end_for;
    else
      v := monom;
      if (testtype(v,"_power") and level(eval(test(op(v,1))),2))
      or  level(eval(test(v)),2) then
          t[bases] := t[bases] * v ;
      else
        Alst := muEC::SYMF::Monom_Almost(v, bases);
        if op(Alst,2) then return(table(hold(coeff) = 0)); end_if;
        if not op(Alst,1) then
          t[hold(coeff)] := t[hold(coeff)] * v;
        else
          t[hold(coeff)] := t[hold(coeff)] * op(Alst,4);
          t[bases] := t[bases] * op(Alst,3);
        end_if;
      end_if;
    end_if;
  else
    /* not only one basis specified...          */
    /* we test the structure of the monomial... */
    /* we init the table...                     */
    t := table(hold(coeff)=1, op(map(bases,b -> b=1)));
    if testtype(monom, "_mult") then
      for i from 1 to nops(monom) do
        v := op(monom,i);
        if testtype(v,"_power") then v1 := op(v,1); else v1 := v; end_if;
        getv := FALSE;
        for j in bases do
          test := muEC::cat("muEC::TYP::Is", j);
          if (level(eval(test(v1)),2)) then
            getv := TRUE; t[j] := t[j] * v; break;
          end_if;
        end_for;
        if (not getv) then
          Alst := [_seqin([muEC::SYMF::Monom_Almost(v, j)], j,bases)];
          if not (_or(_seqin(op(j,1), j,Alst))) then
            t[hold(coeff)] := t[hold(coeff)] * v;
          else for j from 1 to nops(bases) do
            if (op(op(Alst,j),2)) then /* A null monom... */
              return(table(hold(coeff) = 0));
            end_if;
            if (op(op(Alst,j),1)) then /* We found a bases[j] function... */
              t[bases[j]] := t[bases[j]] * op(op(Alst,j),3);
              t[hold(coeff)] := t[hold(coeff)] * op(op(Alst,j),4);
            end_if;
          end_for;
          end_if;
        end_if;
      end_for;
    else
      v := monom;
      if testtype(v,"_power") then v1:=v; else v1:=op(v,1); end_if;
      getv := FALSE;
      for j in bases do
        test := slot(muEC::TYP, "Is".j);
        if test(v1) then
          t[j] := t[j] * v;
          getv := TRUE;
          break;
        end_if;
      end_for;
      if (not getv) then
        Alst := [[muEC::SYMF::Monom_Almost(v, j)] $ j in bases];
        if not (_or(op(j,1)$ j in Alst)) then
          t[hold(coeff)] := t[hold(coeff)] * v;
        else for j from 1 to nops(bases) do
          if (op(op(Alst,j),2)) then /* A null monom... */
            return(table(hold(coeff) = 0));
          end_if;
          if (op(op(Alst,j),1)) then /* We found a bases[j] function... */
            t[bases[j]] := t[bases[j]] * op(op(Alst,j),3);
            t[hold(coeff)] := t[hold(coeff)] * op(op(Alst,j),4);
          end_if;
        end_for;
        end_if;
      end_if;
    end_if;
  end_if;
/* VP021297
  for j in bases do
    if (t[j] = 1) then t[j]:=NIL ; end_if;
  end_for;
*/
  t;
end_proc:
