/*
|  File Pleths (SYMF PACKAGE) Fri Jan  9 19:21:42 MET 1998
*/

/*
|  Pleths(sf1, sf2) expresses the result of the plethysm sf1[sf2] on the
|                   Schur functions basis. sf1 is on the h-basis and sf2 on
|                   the s-basis.
|
|  Pleths(sf1, sf2, lg<=k) is the same function, but the Schur functions in the
|                        result have a length <= k.
|
|  Pleths(sf1, sf2, part<=k) is the same function, but the Schur functions in
|                            the result have their parts <= k.
|  VP090198 VP240498
*/
muEC::SYMF::Pleths := proc(sf1, sf2)
local
  opts, i,
  t,     /* a table...                  */
  r,     /* the result...               */
  mx,    /* maximal part in result...   */
  l;     /* maximal length in result... */
begin
  l := NIL; mx := NIL;
  if args(0)=3 then
    opts:=args(3);
    if testargs() and not testtype(opts, "_leequal") then
        error("Optional argument should be of the form option=value")
    end_if;
    if (op(opts,1)=hold(lg)) then
      l := op(opts,2);
    else
      mx := op(opts,2);
    end_if;
  end_if;
  if muEC::TYP::Iss(sf2) then
    if muEC::TYP::Ish(sf1) then
      if bool(sf2=hold(s)[]) then
        if nops(sf1)>2 then
          r := 0;
        else
          r := 1;
        end_if;
      else
        if (nops(sf1)>2) then /* a product */
          r := muEC::SYMF::Tos(normal(_mult(
              _seqgen(muEC::SYMF::Pleths_elem(hold(h)[op(sf1,i)],sf2,l,mx),
                                       i,2..nops(sf1)))));
        else                           
          r := muEC::SYMF::Pleths_elem(sf1,sf2,l,mx);
        end_if;
      end_if;
    elif testtype(sf1,"_plus") then
        r := map(sf1, muEC::SYMF::Pleths, sf2, args(3..args(0)));
    elif (sf1=hold(h)[]) then
      r := 1;
    else
      t := muEC::SYMF::Monomial(sf1, hold(h));
      r :=  t[hold(coeff)]
          * muEC::SYMF::Pleths(t[hold(h)],sf2, args(3..args(0)));
    end_if;
    sysassign(muEC::SYMF::Pleths_elem,
              muEC::clearRememberTable(muEC::SYMF::Pleths_elem));
    r;
  else /* a temporary solution... */
/*
ATTENTION, LES OPTIONS NE PEUVENT ETRE PASSEES QUE SOUS CERTAINES
CONDITIONS. -> A VOIR.
    muEC::SYMF::Tos(muEC::SYMF::Pletppp(muEC::SYMF::Top(sf1),muEC::SYMF::Top(sf2), opts ));
*/
    muEC::SYMF::Tos(muEC::SYMF::Pletppp(muEC::SYMF::Top(sf1),muEC::SYMF::Top(sf2)));
  end_if;
end_proc:

/*
| VP090198
*/
muEC::SYMF::Pleths_elem := proc(p1,p2,l,mx)
option remember;
begin
  if testtype(l, DOM_INT) then
    muEC::SYM::Pleths_elem_l(l,p1,p2):
  elif testtype(mx, DOM_INT) then
    muEC::SYM::Pleths_elem_mx(mx,p1,p2):
  else
    muEC::SYM::Pleths_elem(p1,p2):
  end_if;
end_proc:
