/*
|  File Plethysm (SYMF PACKAGE) Fri Jan  9 18:16:21 MET 1998
*/

/*
|  SfPlethysm(sf1,sf2,b1,b2) computes the plethysm sf1[sf2] of symmetric
|                            functions sf1, sf2 assuming that sf1 is expressed
|                            on basis b1 and sf2 is expressed on basis b2.
|                            (b1 and b2 may be forgotten...)
|  VP090198 VP240498
*/ 
muEC::SYMF::SfPlethysm_core := proc(sf1,sf2)
local
  b1,b2; /* bases... */
begin
  if (args(0)>=3) then
    b1 := args(3);
  else
    b1 := null();
  end_if;
  if (args(0)>=4) then
    b2 := args(4);
  else
    b2 := null();
  end_if;
/* e e */
  if ((b1=hold(e)) and (b2=hold(e))) then
    muEC::SYMF::Pletes(sf1, muEC::SYMF::Tos(sf2));
/* e h */
  elif ((b1=hold(e)) and (b2=hold(h))) then
    muEC::SYMF::Pletes(sf1, muEC::SYMF::Tos(sf2));
/* e s */
  elif ((b1=hold(e)) and (b2=hold(s))) then
    muEC::SYMF::Pletes(sf1, sf2);
/* h e */
  elif ((b1=hold(h)) and (b2=hold(e))) then
    muEC::SYMF::Pleths(sf1, muEC::SYMF::Tos(sf2));
/* h h */
  elif ((b1=hold(h)) and (b2=hold(h))) then
    muEC::SYMF::Pleths(sf1, muEC::SYMF::Tos(sf2));
/* h s */
  elif ((b1=hold(h)) and (b2=hold(s))) then
    muEC::SYMF::Pleths(sf1, sf2);
/* s e */
  elif ((b1=hold(s)) and (b2=hold(e))) then
    muEC::SYMF::Pletss(sf1, muEC::SYMF::Tos(sf2));
/* s h */
  elif ((b1=hold(s)) and (b2=hold(h))) then
    muEC::SYMF::Pletss(sf1, muEC::SYMF::Tos(sf2));
/* s s */
  elif ((b1=hold(s)) and (b2=hold(s))) then
    muEC::SYMF::Pletss(sf1, sf2);
/* p e */
  elif ((b1=hold(p)) and (b2=hold(e))) then
    muEC::SYMF::Pletps(sf1, muEC::SYMF::Tos(sf2));
/* p h */
  elif ((b1=hold(p)) and (b2=hold(h))) then
    muEC::SYMF::Pletps(sf1, muEC::SYMF::Tos(sf2));
/* p s */
  elif ((b1=hold(p)) and (b2=hold(s))) then
    muEC::SYMF::Pletps(sf1, sf2);
/* p p */
  elif ((b1=hold(p)) and (b2=hold(p))) then
    muEC::SYMF::Pletpp(sf1,sf2);
/* p m */
  elif ((b1=hold(p)) and (b2=hold(m))) then
    muEC::SYMF::Pletpp(sf1, muEC::SYMF::Top(sf2));
/* m e */
  elif ((b1=hold(m)) and (b2=hold(e))) then
    muEC::SYMF::Pletps(muEC::SYMF::Top(sf1), muEC::SYMF::Tos(sf2));
/* m h */
  elif ((b1=hold(m)) and (b2=hold(h))) then
    muEC::SYMF::Pletps(muEC::SYMF::Top(sf1), muEC::SYMF::Tos(sf2));
/* m p */
  elif ((b1=hold(m)) and (b2=hold(p))) then
    muEC::SYMF::Pletpp(muEC::SYMF::Top(sf1), sf2);
/* m s */
  elif ((b1=hold(m)) and (b2=hold(s))) then
    muEC::SYMF::Pletps(muEC::SYMF::Top(sf1), sf2);
/* m m */
  elif ((b1=hold(m)) and (b2=hold(m))) then
    muEC::SYMF::Pletpp(muEC::SYMF::Top(sf1), muEC::SYMF::Top(sf2));
/* ? ? */
  else
    muEC::SYMF::Pletppp(muEC::SYMF::Top(sf1), muEC::SYMF::Top(sf2));
  end_if;
end_proc:

/*
|  Type Checking.
|  VP090198
*/
muEC::SYMF::SfPlethysm_check := proc(sf1,sf2)
begin
  if (args(0)>=3) then
    if not (has(muEC::SYMF::SYMFBases,args(3))) then
      muEC::ERR::SYMFPerror(11,procname);
    end_if;
  end_if;
  if (args(0)>=4) then
    if not (has(muEC::SYMF::SYMFBases,args(4))) then
      muEC::ERR::SYMFPerror(12,procname);
    end_if;
  end_if;
end_proc:

/*
|  VP090198
*/
muEC::SYMF::SfPlethysm := proc(sf1,sf2)
begin
  if testargs() then
    muEC::SYMF::SfPlethysm_check(args());
  end_if;
  muEC::SYMF::SfPlethysm_core(args());
end_proc:
