/*
|  File SfCCProd (SYMF PACKAGE) Fri Jan  9 11:34:49 MET 1998
*/

/*
|  SfCCProd(sf1,sf2) computes the internal product of symmetric
|                    functions sf1 and sf2.
|                    This is called the inner tensor product.
|                    The result is expressed on p basis.
|  VP091098 VP240498
*/
muEC::SYMF::SfCCProd_core := proc(sf1,sf2)
local
  i,     /* var for loop...                           */
  j,     /* var...                                    */
  r,     /* the result...                             */
  allv1, /* all values appearing in ons1...           */
  allv2, /* all values appearing in ons2...           */
  ons1,  /* sf1 on s basis...                         */
  ons2,  /* sf2 on s basis...                         */
  c1,c2, /* coefficients in ons1 and ons2...          */
  mn,    /* a monom in allv1...                       */
  pa;    /* partition...                              */
begin
  ons1 := expand(muEC::SYMF::Tos(sf1));
  ons2 := expand(muEC::SYMF::Tos(sf2));
  allv1 := muEC::SYMF::AllIndexed(ons1,hold(s));
  allv2 := muEC::SYMF::AllIndexed(ons2,hold(s));
  allv1 := [op(map(allv1,proc(x) begin hold(s)[op(x)] end_proc ))]; /* need a list... */
  allv2 := [op(map(allv2,proc(x) begin hold(s)[op(x)] end_proc ))]; /* need a list... */
  c1 := [coeff(ons1,allv1)];
  c2 := [coeff(ons2,allv2)];
  r := 0;
  for i from 1 to nops(c1) do
    mn := allv1[i];
    pa := op(mn,2..nops(mn));
    j := contains(allv2,mn);
    if j<>0 then
      r := r+_mult(op(muEC::PART::Part2ListHook([pa])))/fact(_plus(pa))
            *normal(c1[i]*c2[j])*mn;
    end_if;
  end_for;
  r;
end_proc:

/*
|  Type Checking.
|  VP090198 VP240498
*/
muEC::SYMF::SfCCProd_check := proc(sf1,sf2)
begin
  if (args(0)>2) then
    muEC::ERR::SYMFPerror(11,procname);
  end_if;
end_proc:

/*
|  VP090198
*/
muEC::SYMF::SfCCProd := proc(sf1,sf2)
begin
  if testargs() then
    muEC::SYMF::SfCCProd_check(args());
  end_if;
  muEC::SYMF::SfCCProd_core(args());
end_proc:
