/*
|  File SfJtMat (SYMF PACKAGE) Fri Jan  9 10:24:40 MET 1998
*/

/*
|  SfJtMat(lambda) produces the Jacobi-Trudi matrix for the
|                  partition lambda. It does the same for a skew partition.
|  VP080198 VP240498
*/
muEC::SYMF::SfJtMat_core := proc()
local
  b,     /* the used basis... */
  n,     /* an integer...     */
  i,j,   /* vars for seq...   */
  mu,nu; /* two partitions... */
begin
  if (muEC::TYP::IsPart(args(1))) then
    mu := args(1);
    nu := [];
  else
    mu := op(args(1),1);
    nu := op(args(1),2)
  end_if;
  /* we init the second basis... */
  if (args(0)>1) then
    b := args(2);
  else
    b := hold(h);
  end_if;
  /* we adjust the lengths of the partitions by adding some zeros... */
  n  := max(1,nops(mu),nops(nu));
  mu := [op(mu),_seqgen(0, i,1..n)];
  nu := [op(nu),_seqgen(0, i,1..n)];
  /* we build the matrix... */
  array(1..n,1..n,[_seqgen(map([_seqgen(mu[i]-i+j-nu[j],j,1..n)],
                     proc(x,b) begin if (x>0) then b[x] elif (x=0)
                     then 1 else 0 end_if end_proc,
                     b),i,1..n)])
end_proc:

/*
|  Type Checking.
|  VP080198 VP240498
*/
muEC::SYMF::SfJtMat_check := proc(part)
begin
  if (not (muEC::TYP::IsPart(part) or muEC::TYP::IsSkewPart(part))) then
    muEC::ERR::SYMFPerror(1,procname);
  end_if;
  if (args(0)=2) and not testtype(args(2),DOM_IDENT) then 
    muEC::ERR::SYMFPerror(1,procname);
  end_if;
end_proc:

/*
|  VP08198
*/
muEC::SYMF::SfJtMat := proc(part)
begin
  if testargs() then
    muEC::SYMF::SfJtMat_check(args());
  end_if;
  muEC::SYMF::SfJtMat_core(args());
end_proc:
