/*
|  File SfScalar (SYMF PACKAGE) Thu Jan  8 10:44:10 MET 1998
*/

/*
|  SfScalar(sf1,sf2) computes the scalar product of the symmetric functions sf1
|                    sf2 assuming that the power sums are orthogonal and
|                   <p[mu],p[mu]>=SfZee(mu).
|
|  SfScalar(sf1,sf2,spf) the same but spf is a procedure so that
|                       <p[mu],p[mu]>=spf(mu).
|  VP081098
*/
muEC::SYMF::SfScalar_core := proc(sf1,sf2)
local
  i,     /* var for loop...                           */
  j,     /* var...                                    */
  k,     /* var for seq...                            */
  r,     /* the result...                             */
  mp,    /* max i (p.i) appearing in onp1 and onp2... */
  allv1, /* all values appearing in onp1...           */
  allv2, /* all values appearing in onp2...           */
  onp1,  /* sf1 on p basis...                         */
  onp2,  /* sf2 on p basis...                         */
  spf,   /* <p[mu],p[mu]>=spf([mu])...                */
  c1,c2, /* coefficients in onp1 and onp2...          */
  mn;    /* monom in c1...                            */
begin
  if (args(0)=2) then
    spf := muEC::SYMF::SfZee;
  elif (args(0)=3) then
    spf := args(3);
  end_if;
  onp1 := expand(muEC::SYMF::Top(sf1));
  onp2 := expand(muEC::SYMF::Top(sf2));
  allv1 := muEC::SYMF::AllIndexed(onp1,hold(p));
  allv2 := muEC::SYMF::AllIndexed(onp2,hold(p));
  mp := max(0,op({_seqin(op(k),k,allv1)} intersect {_seqin(op(k),k,allv2)}));
  allv1 := [op(map(allv1,proc(x) begin hold(p)[op(x)] end_proc))]; /* need a list... */
  allv2 := [op(map(allv2,proc(x) begin hold(p)[op(x)] end_proc))]; /* need a list... */
  c1 := [coeff(onp1,allv1)];
  c2 := [coeff(onp2,allv2)];
  r := 0;
  for i from 1 to nops(c1) do
    mn := op(allv1,i);
    j := contains(allv2,mn);
    if j<>0 then
      r := r+spf([op(mn,2..nops(mn))])*normal(c1[i]*c2[j]);
    end_if;
  end_for;
  r;
end_proc:

/*
|  Type Checking.
|  VP080198
*/
muEC::SYMF::SfScalar_check := proc(sf1,sf2)
begin
  if (args(0)>3) then
    muEC::ERR::SYMFPerror(11,procname);
  end_if;
end_proc:

/*
|  VP080198
*/
muEC::SYMF::SfScalar := proc(sf1,sf2)
begin
  if testargs() then
    muEC::SYMF::SfScalar_check(args());
  end_if;
  muEC::SYMF::SfScalar_core(args());
end_proc:
