/*
|  File Table (SYMF PACKAGE) Thu Apr 23 16:13:46 MET DST 1998
*/

/*
|  Sf2Table(sf, b) builds a table indexed by partitions, representing the
|                  element of the symmetric "function" algebra in basis b.
*/
/*
|  Table2Sf(t, b) builds an element of the symmetric "function" algebra
|                 from a table. All partitions are interpreted as b[part].
|  VP230498
*/
muEC::SYMF::Sf2Table_core := proc(sf, b)
local
  tmp, /* a temporary result... */
  v,   /* variables... */
  i,   /* variable for sequence... */
  t;   /* the result... */
begin
  v   := muEC::SYMF::AllIndexed(sf, b);
  t   := table(_seqin(i=coeff(sf, b[op(i)],1), i,v));
  tmp := subs(sf, {_seqin(b[op(i)]=0, i,v)});
  if (tmp<>0) then
    if (contains(v,[])) then
      t[[]]:=t[[]] + tmp;
    else
      t[[]]:=tmp;
    end_if;
  end_if;
  t;
end_proc:
/*
|  VP230498
*/
muEC::SYMF::Sf2Table_check := proc(sf, b)
begin
  if (not testtype(b,DOM_IDENT)) then
    muEC::ERR::SYMFPerror(3,procname);
  end_if;
  if (not has(muEC::SYMF::SYMFBases,b)) then
    muEC::ERR::SYMFPerror(4,procname);
  end_if;
end_proc:
/*
|  VP230498
*/
muEC::SYMF::Sf2Table := proc(sf, b)
begin
  if testargs() then
    muEC::SYMF::Sf2Table_check(args());
  end_if;
  muEC::SYMF::Sf2Table_core(args());
end_proc:
/*
|  VP230498
*/
muEC::SYMF::Table2Sf_core := proc(t, b)
local
    i; /* var for seq... */
begin
    _plus(_seqin(t[i]*b[op(i)], i,map([op(t)], op, 1)));
end_proc:
/* 
|  VP230498
*/ 
muEC::SYMF::Table2Sf_check:=proc(t, b)
begin
  if (not testtype(t,DOM_TABLE)) then
    muEC::ERR::SYMFPerror(1,procname);
  end_if;
  if (not testtype(b,DOM_IDENT)) then
    muEC::ERR::SYMFPerror(3,procname);
  end_if;
  if (not has(muEC::SYMF::SYMFBases,b)) then
    muEC::ERR::SYMFPerror(4,procname);
  end_if;
end_proc:
/*
|  VP230498
*/
muEC::SYMF::Table2Sf := proc(t, b)
begin
  if testargs() then
    muEC::SYMF::Table2Sf_check(args());
  end_if;
  muEC::SYMF::Table2Sf_core(args());
end_proc:
