/*
|  File ToSf (SYMF PACKAGE) Tue Dec  2 15:19:41 MET 1997
*/

/*
|  ToSf(sf,b) takes a symmetric function and converts it on the b basis.
|             The result may be expanded before or after conversion.
|  VP021297 VP080198 VP240498 VP270498
*/
muEC::SYMF::ToSf := proc(sf,b)
local
  i,      /* var for loop... */
  j,      /* var for seq...  */
  c_flg,  /* collect flag... */
  e_flg,  /* expand flag...  */
  ae_flg, /* already expanded flag... */
  bases,  /* bases...        */
  nsf,    /* new sf...       */
  f,      /* a function...   */
  g,      /* a function...   */
  rr,     /* a result...     */
  r,      /* the result...   */
  t;      /* a table...      */
/*
option remember;
*/
begin
  e_flg := TRUE;    /* expansion will be done... */
  c_flg := FALSE;   /* not collecting...         */
  ae_flg := FALSE;  /* expansion was already done on input... */
  bases := muEC::SYMF::SYMFBases;
  if (args(0)>2) then
    for i from 3 to args(0) do
      if (args(i)=hold(NoExpand)) then
        e_flg := FALSE;
      elif (args(i)=hold(Collect)) then
        c_flg := TRUE;
      elif (args(i)=hold(Already)) then
        ae_flg := TRUE;
      elif testtype(args(i),DOM_IDENT) or testtype(args(i),DOM_LIST) then
        bases := args(i);
      end_if;
    end_for;
  end_if;
  if e_flg and not ae_flg then /* expansion required... */
    nsf := expand(sf); 
  else
    nsf := sf;
  end_if;
  if testtype(nsf, "_plus") then
    r := map(nsf, muEC::SYMF::ToSf, subs(args(2..args(0)),hold(Already)=null()), hold(Already));
    if c_flg and (e_flg or ae_flg) then /* collection is required */
      rr := [op(map(muEC::SYMF::AllIndexed(r,b), proc(mu) begin b[op(mu)] end_proc ))];
      if nops(rr)>0 then
        r := collect(r, rr);
      end_if;
    end_if;
  elif testtype(nsf, "_power") then
    if e_flg then /* this is the result of an expansion */
    /* x^k or b[..]^k, k integer or expr^k, k formal */
      if testtype(op(nsf,2),DOM_INT) then
        if testtype(op(nsf,1),"_index") then
          for i in {op(bases)} do
            g := muEC::cat("muEC::TYP::IsAlmost",i);
            if level(eval(g(op(nsf,1))),2) then /* this is a symmetric function */
              if bool(b=i) then
                return(muEC::SYMF::SfCollect(nsf,i));
              else
                f := muEC::cat("muEC::SYMF::",i,2,b);
		// Tentative bug fix by NT: it was eval(level(...), 2)
                return(level(eval(f(muEC::SYMF::SfCollect(nsf,i))),2));
              end_if;
              break;
            end_if;
          end_for;
          r := nsf;
        else
          r := nsf;
        end_if;
      else /* expr^k */
        r := muEC::SYMF::ToSf(op(nsf,1), subs(args(2..args(0)),hold(Already)=null()));
        if c_flg then /* collection is required */
          rr := [op(map(muEC::SYMF::AllIndexed(r,b), proc(mu) begin b[op(mu)] end_proc ))];
          if nops(rr)>0 then
            r := collect(r, rr);
          end_if;
        end_if;
        r := r^op(nsf,2);
      end_if;
    else /* expansion not required */      
    /* expr^k, k integer or formal */
      r := muEC::SYMF::ToSf(op(nsf,1), subs(args(2..args(0)),hold(Already)=null()));
      if c_flg then /* collection is required */
        rr := [op(map(muEC::SYMF::AllIndexed(r,b), proc(mu) begin b[op(mu)] end_proc ))];
        if nops(rr)>0 then
          r := collect(r, rr);
        end_if;
      end_if;
      r := r^op(nsf,2);
    end_if;
  elif testtype(nsf, "_mult") then
    if e_flg then /* this is the result of an expansion */
      t := muEC::SYMF::Monomial(nsf, bases);
      if (t[hold(coeff)]=0) then
        return(0);
      end_if;
      r := 1;
      for i in {op(bases)} do
        if (t[i]<>1) then /* t[i] is i[..] or a product or a power */
          g := muEC::cat("muEC::TYP::Is",i);
          if not level(eval(g(t[i])),2) then
          /* not a symmetric function: a product or a power */
            t[i] := muEC::SYMF::SfCollect(t[i],i); 
          end_if;
          if bool(b=i) then
            r := r * t[i];
          else
            g := muEC::cat("muEC::SYMF::",i,2,b);
            if testtype(t[i],"_mult") then /* s[..] s[..]^k, k formal */
              r := r * level(eval(map(t[i],g)),2);
            else
              r := r * level(eval(g(t[i])),2);
            end_if;
          end_if;
        end_if;
      end_for;
      /* expansion was required */
      r := expand(r * map(t[hold(coeff)],muEC::SYMF::ToSf,
                  subs(args(2..args(0)),hold(Already)=null())));
      if nops(r)>1 and
         ((not testtype(r,"_plus")) or 
          (testtype(r,"_plus") and
           _or(_seqin(testtype(j,"_power") or testtype(j,"_mult"),j,r))))
      then
        r := muEC::SYMF::SfCollect(r);
      end_if;
      if c_flg then /* collection is required */
        rr := [op(map(muEC::SYMF::AllIndexed(r,b), proc(mu) begin b[op(mu)] end_proc ))];
        if nops(rr)>0 then
          r := collect(r, rr);
        end_if;
      end_if;
    else /* expansion not required */
      r := map(nsf,muEC::SYMF::ToSf,subs(args(2..args(0)),hold(Already)=null()),hold(Already)); 
    end_if;
  else /* a scalar or a symmetric function */
       /* same as "_mult", except it isn't recursive */
    if e_flg then /* this is the result of an expansion */
      t := muEC::SYMF::Monomial(nsf, bases);
      if (t[hold(coeff)]=0) then
        return(0);
      end_if;
      r := 1;
      for i in {op(bases)} do
        if (t[i]<>1) then /* t[i] is i[..] or a product or a power */
          g := muEC::cat("muEC::TYP::Is",i);
          if not level(eval(g(t[i])),2) then
          /* not a symmetric function: a product or a power */
            t[i] := muEC::SYMF::SfCollect(t[i],i); 
          end_if;
          if bool(b=i) then
            r := r * t[i];
          else
            g := muEC::cat("muEC::SYMF::",i,2,b);
            r := r * level(eval(g(t[i])),2);
          end_if;
        end_if;
      end_for;
      if nops(r)>1 and not testtype(r,"_plus") then /* a product or a power */
        r := muEC::SYMF::SfCollect(r);
      end_if;
      /* expansion was required */
      r := expand(r * t[hold(coeff)]); /* <> "_mult" */
      if c_flg then /* collection is required */
        rr := [op(map(muEC::SYMF::AllIndexed(r,b), proc(mu) begin b[op(mu)] end_proc ))];
        if nops(rr)>0 then
          r := collect(r, rr);
        end_if;
      end_if;
    else /* expansion not required */ /* <> "_mult" */
      if testtype(nsf,"_index") then
        for i in {op(bases)} do
          g := muEC::cat("muEC::TYP::IsAlmost",i);
          if level(eval(g(nsf)),2) then /* this is a symmetric function */
            if bool(b=i) then
              return(muEC::SYMF::SfCollect(nsf,i));
            else
              f := muEC::cat("muEC::SYMF::",i,2,b);
              return(level(eval(f(muEC::SYMF::SfCollect(nsf,i))),2));
            end_if;
            break;
          end_if;
        end_for;
        r := nsf;
      else
        r := nsf;
      end_if;
    end_if;
  end_if;
  r;
end_proc:

/*
|  Type Checking...
|  VP021297 VP240498 VP270498
*/
muEC::SYMF::To_check := proc(sf)
local
  i; /* var for loop... */
begin
  if (args(0)>1) then
    for i from 2 to args(0) do
      if bool(args(i)<>hold(NoExpand))
      and bool(args(i)<>hold(Collect))
      and not has(muEC::SYMF::SYMFBases,args(i))
      then
        muEC::ERR::SYMFPerror(2*i-1,procname);/* Wrong type for ith argument... */
      end_if;
    end_for;
  end_if;
end_proc:

/*
|  VP021297 VP240498
*/
muEC::SYMF::Toe := proc(sf)
begin
  if testargs() then
    muEC::SYMF::To_check(args());
  end_if;
  muEC::SYMF::ToSf(sf,hold(e),args(2..args(0)));
end_proc:

/*
|  VP021297 VP240498
*/
muEC::SYMF::Toh := proc(sf)
begin
  if testargs() then
    muEC::SYMF::To_check(args());
  end_if;
  muEC::SYMF::ToSf(sf,hold(h),args(2..args(0)));
end_proc:

/*
|  VP021297 VP240498
*/
muEC::SYMF::Top := proc(sf)
begin
  if testargs() then
    muEC::SYMF::To_check(args());
  end_if;
  muEC::SYMF::ToSf(sf,hold(p),args(2..args(0)));
end_proc:

/*
|  VP021297 VP240498
*/
muEC::SYMF::Tom := proc(sf)
begin
  if testargs() then
    muEC::SYMF::To_check(args());
  end_if;
  muEC::SYMF::ToSf(sf,hold(m),args(2..args(0)));
end_proc:

/*
|  VP021297 VP240498
*/
muEC::SYMF::Tos := proc(sf)
begin
  if testargs() then
    muEC::SYMF::To_check(args());
  end_if;
  muEC::SYMF::ToSf(sf,hold(s),args(2..args(0)));
end_proc:
