/*
|  File IsSf (TYP PACKAGE) Wed Oct 22 11:09:51 MET DST 1997
*/

/*
|  IsSfIndexed(mn,b) tests whether mn is like b[partition]
|  VP221097
*/
muEC::TYP::IsSfIndexed:=proc(mn,b)
begin
  if (not testtype(mn,"_index")) then
    FALSE;
  else
    if (op(mn,1)<>b) then
      FALSE;
    else
      muEC::TYP::IsPart([op(mn, 2..nops(mn))]);
    end_if;
  end_if;
end_proc:

/*
|  Ish(mn) tests whether mn is a product of complete symmetric functions.
|  VP221097
*/
muEC::TYP::Ish:=proc(mn)
begin
  muEC::TYP::IsSfIndexed(mn,hold(h));
end_proc:

/*
|  Ise(mn) tests whether mn is a product of elementary symmetric functions.
|  VP221097
*/
muEC::TYP::Ise:=proc(mn)
begin
  muEC::TYP::IsSfIndexed(mn,hold(e));
end_proc:

/*
|  Isp(mn) tests whether mn is a product of power sum symmetric functions.
|  VP221097
*/
muEC::TYP::Isp:=proc(mn)
begin
  muEC::TYP::IsSfIndexed(mn,hold(p));
end_proc:

/*
|  Iss(mn) tests whether mn is a Schur function.
|  VP221097
*/
muEC::TYP::Iss:=proc(mn)
begin
  muEC::TYP::IsSfIndexed(mn,hold(s));
end_proc:

/*
|  Ism(mn) tests whether mn is a monomial function.
|  VP221097
*/
muEC::TYP::Ism:=proc(mn)
begin
  muEC::TYP::IsSfIndexed(mn,hold(m));
end_proc:

/*
|  IsAlmost(mn) tests whether mn is an indexed function where
|               the index is not necessary a partition.
|  VP221097
*/
muEC::TYP::IsAlmost:=proc(mn,b)
local
   p, /* the partition?... */
   i; /* var for loop...   */
begin
  if (not testtype(mn,"_index")) then
    return(FALSE);
  end_if;
  if (op(mn,1)<>b) then
    return(FALSE);
  end_if;
  /* we just check that the index is made of integers(s)... */
  p:=[op(mn,2..nops(mn))];
  for i from 1 to nops(p) do
    if (not testtype(p[i],DOM_INT)) then
      return(FALSE);
    end_if;
  end_for;
  TRUE;
end_proc:

/*
|  IsAlmostWithZero(mn) tests whether mn is an indexed function where
|                       the index is not necessary a partition and may
|                       contain some zeros.
|  VP221097
*/
muEC::TYP::IsAlmostWithZero:=proc(mn,b)
local
   p, /* the partition?... */
   i; /* var for loop...   */
begin
  if (not testtype(mn,"_index")) then
    return(FALSE);
  end_if;
  if (op(mn,1)<>b) then
    return(FALSE);
  end_if;
  /* we just check that the index is made of integers(s)... */
  p:=[op(mn,2..nops(mn))];
  for i from 1 to nops(p) do
    if (not testtype(p[i],Type::NonNegInt)) then
      return(FALSE);
    end_if;
  end_for;
  TRUE;
end_proc:

/*
|  IsAlmostp(mn) tests whether mn is a product of power sum functions where
|                the index is not necessary a partition.
|  VP221097
*/
muEC::TYP::IsAlmostp:=proc(mn)
begin
  muEC::TYP::IsAlmost(mn,hold(p));
end_proc:

/*
|  IsAlmoste(mn) tests whether mn is a product of elementary functions where
|                the index is not necessary a partition.
|  VP221097
*/
muEC::TYP::IsAlmoste:=proc(mn)
begin
  muEC::TYP::IsAlmost(mn,hold(e));
end_proc:

/*
|  IsAlmosth(mn) tests whether mn is a product of complete  functions where
|                the index is not necessary a partition.
|  VP221097
*/
muEC::TYP::IsAlmosth:=proc(mn)
begin
  muEC::TYP::IsAlmost(mn,hold(h));
end_proc:

/*
|  IsAlmosts(mn) tests whether mn is a Schur function where
|                the index is not necessary a partition.
|  VP221097
*/
muEC::TYP::IsAlmosts:=proc(mn)
begin
  muEC::TYP::IsAlmost(mn,hold(s));
end_proc:

/*
|  IsAlmostm(mn) tests whether mn is a monomial function that is m[x] where x
|                a vector with extra 0's.
*/
muEC::TYP::IsAlmostm:=proc(mn)
begin
  muEC::TYP::IsAlmostWithZero(mn,hold(m));
end_proc:
