/*
|  File IsSfA (TYP PACKAGE) Thu Apr 23 13:54:11 MET DST 1998
*/
/*
|  Type Checking related to SFA package
*/

/*
|  Tests whether argument is a valid alphabet (empty or k*Ai, k DOM_INT).
|  VP230498
*/
muEC::TYP::IsAlphabet_simple := proc(alph)
begin
  muEC::TYP::IsName(alph, hold(A));
end_proc:
/*
|  Tests whether argument is a valid expression on alphabets.
|  VP230498
*/
muEC::TYP::IsAlphabet := proc(alph)
begin
  bool(alph<>null());
end_proc:
/*
|  Tests whether the argument is a variable as described in SFAVars.
|  VP130198
*/
muEC::TYP::IsVariable := proc(k)
local
  i; /* var for loop... */
begin
  if contains(muEC::SFA::SFAVars,k) then
    return(TRUE);
  elif testtype(k,DOM_IDENT) then
   for i in muEC::SFA::SFAVars do
     if (testtype(i, DOM_SET) and muEC::TYP::IsName(k,op(i))) then
       return(TRUE);  
     end_if;
   end_for;
  end_if;
  FALSE;
end_proc:
/*
|  Tests whether the argument is a constant in the sense of alphabets.
|  Products are not considered here.
|  VP230498
*/
muEC::TYP::IsConstant := proc(k)
local
  f, /* a function...  */
  i; /* var for seq... */
begin
  if testtype(k, _power) then
    muEC::TYP::IsConstant(op(k,1));
  elif testtype(k, Type::Fraction) then
    muEC::TYP::IsConstant(op(k,1)) and muEC::TYP::IsConstant(op(k,2));
  elif muEC::TYP::IsAlphabet_simple(k) or muEC::TYP::IsVariable(k) then
    FALSE;
  else
    f := [_seqin(muEC::cat("muEC::TYP::Is",i,"A"), i,muEC::SFA::SFABases)];
    not(_or(_seqin(level(eval(i(k)),2), i,f)));
  end_if; 
end_proc:
/*
|  Tests whether argument is a valid symmetric "function" (fn = e, h, m, p, s).
|  VP230498
*/
muEC::TYP::IsSfA := proc(mn, fn)
begin
  if not testtype(mn, "function") then
    FALSE;
  elif (fn=hold(p)) and (bool(nops(op(mn,0))=0)) then /* p[](..) not valid! */
    FALSE;  
  else
    muEC::TYP::IsAlphabet(op(mn)) and muEC::TYP::IsSfIndexed(op(mn,0), fn);
  end_if;
end_proc:
/*
|  Tests whether argument is a complete symmetric "function"
|  VP230498
*/
muEC::TYP::IshA := proc(mn)
begin
  muEC::TYP::IsSfA(mn, hold(h));
end_proc:
/*
|  Tests whether mn is a product of complete functions that is h[x](alphabet)
|  where x is not necessary a partition.
|  VP230498
*/ 
muEC::TYP::IsAlmosthA:=proc(mn)
local
  p, /* the partition?... */
  i; /* var for loop... */
begin
  if (not testtype(op(mn,0),"_index")) then
    FALSE;
  elif (op(op(mn,0),1) <> hold(h)) then
    FALSE;
  else;
    /* we just check that the index is made of integers(s)... */
    p := [op(op(mn,0),2..nops(op(mn,0)))];
    for i from 1 to nops(p) do
      if not testtype(p[i], Type::NonNegInt) then
        return(FALSE);
      end_if;
    end_for;
    muEC::TYP::IsAlphabet(op(mn));
  end_if;
end_proc:
/*
|  Tests whether argument is an elementary symmetric "function"
|  VP230498
*/
muEC::TYP::IseA := proc(mn)
begin
  muEC::TYP::IsSfA(mn, hold(e));
end_proc:
/*
|  Tests whether mn is a product of elementary functions that is e[x](alphabet)
|  where x is not necessary a partition.
|  VP230498
*/ 
muEC::TYP::IsAlmosteA:=proc(mn)
local
  p, /* the partition?... */
  i; /* var for loop... */
begin
  if (not testtype(op(mn,0),"_index")) then
    FALSE;
  elif (op(op(mn,0),1) <> hold(e)) then
    FALSE;
  else;
    /* we just check that the index is made of integers(s)... */
    p := [op(op(mn,0),2..nops(op(mn,0)))];
    for i from 1 to nops(p) do
      if not testtype(p[i], Type::NonNegInt) then
        return(FALSE);
      end_if;
    end_for;
    muEC::TYP::IsAlphabet(op(mn));
  end_if;
end_proc:
/*
|  Tests whether argument is a power symmetric "function"
|  VP230498
*/
muEC::TYP::IspA := proc(mn)
begin
  muEC::TYP::IsSfA(mn, hold(p));
end_proc:
/*
|  Tests whether mn is a product of power sums that is p[x](alphabet) where x is
|  not necessary a partition.
|  VP230498
*/
muEC::TYP::IsAlmostpA:=proc(mn)
local
  p, /* the partition?... */
  i; /* var for loop... */
begin
  if (not testtype(op(mn,0),"_index")) then
    FALSE;
  elif (op(op(mn,0),1) <> hold(p)) then
    FALSE;
  else;
    /* we just check that the index is made of integers(s)... */
    p := [op(op(mn,0),2..nops(op(mn,0)))];
    for i from 1 to nops(p) do
      if not testtype(p[i], Type::NonNegInt) then
        return(FALSE);
      end_if;
    end_for;
    muEC::TYP::IsAlphabet(op(mn));
  end_if;
end_proc:
/*
|  Tests whether argument is a Schur "function"
|  VP230498
*/
muEC::TYP::IssA := proc(mn)
begin
  muEC::TYP::IsSfA(mn, hold(s));
end_proc:
/*
|  Tests whether mn is a Schur "function" that is s[x](alphabet) where x is not
|  necessary a partition.
|  VP230498
*/
muEC::TYP::IsAlmostsA:=proc(mn)
local
  p, /* the partition?... */
  i; /* var for loop... */
begin
  if (not testtype(op(mn,0),"_index")) then
    FALSE;
  elif (op(op(mn,0),1) <> hold(s)) then
    FALSE;
  else;
    /* we just check that the index is made of integers(s)... */
    p := [op(op(mn,0),2..nops(op(mn,0)))];
    for i from 1 to nops(p) do
      if not testtype(p[i], Type::NonNegInt) then
        return(FALSE);
      end_if;
    end_for;
    muEC::TYP::IsAlphabet(op(mn));
  end_if;
end_proc:
/*
|  Tests whether argument is a monomial symmetric "function"
|  VP230498
*/
muEC::TYP::IsmA := proc(mn)
begin
  muEC::TYP::IsSfA(mn, hold(m));
end_proc:
/*
|  Tests whether mn is a monomial "function" that is m[x](alphabet) where x is a
|  vector with extra 0's.
|  VP230498
*/
muEC::TYP::IsAlmostmA:=proc(mn)
local
  p, /* a vector... */
  i; /* variable for loop... */
begin
  if (not testtype(op(mn,0),"_index")) then
    FALSE;
  elif (op(op(mn,0),1) <> hold(m)) then
    FALSE;
  else
    p := [op(op(mn,0),2..nops(op(mn,0)))];
    for i from 1 to nops(p) do
      if not testtype(p[i], Type::NonNegInt) then
        return(FALSE);
      end_if;
    end_for;
    muEC::TYP::IsAlphabet(op(mn));
  end_if;
end_proc:
