/**
  * DomainConstructor::getEntryFromSuperDomain(DOM, SuperDomain, "bla")
  *
  * Look for the definitions of the entry "bla" in the constructor of
  * the domain SuperDomain, evaluate it in the context of
  * SuperDomain, and, for a procedure, bless it as methods of A;
  * returns NIL if the entry was not found.
  *
  * Largely inspired from DomainConstructor::getSuperDomainEntry.
  **/

DomainConstructor::getEntryFromSuperDomain :=
proc(DOM: DOM_DOMAIN, SuperDomain: DOM_DOMAIN, e: DOM_STRING)
    option noDebug;
    local r;
begin
    // search for entry in super-domain constructor
    r:= extop(SuperDomain::constructor,4);
    if not contains(r, e) then
	return(NIL);
    end_if;
    // evaluate entry in closure of super-domain
    r:= _eval_entry(indexval(r, e), DOM, SuperDomain::closure);
    if r <> NIL then
	r := prog::blessAny(r, DOM);
    end_if;
    r;
end_proc:
