prog::testinit("Graph",  1.9, All):
prog::testfunc("Graph"):

// Test if the graph can be created (and the result is a graph) (01 - 04)
prog::test( domtype( (G := Graph([1,2,a],[[1,2],[2,a],[1,a]])) ), Graph, "Creating of graph (undirected)"):
prog::test( domtype( (G2 := Graph([a,b,c,d],[[a,b],[b,c],[a,c],[c,d]], EdgeWeights = [2,-3,1,2], Directed)) ), Graph, "Creating of graph (Directed)"):
prog::test( domtype( (G3 := Graph([1,2,a],[], Undirected)) ), Graph, "Creating of graph with no edges(Undirected)"):
prog::test( domtype( (TG := Graph([a,b,3,4,5,f],[[a,b],[b,3],[b,4],[3,4],[4,5],[4,f],[5,f]], VertexWeights = [10,20,None,0,infinity,-10], EdgeWeights = [50,10,60,20,30,70,40], EdgeCosts = [30,70,20,40,50,10,40], EdgeDescriptions = ["First Way", "Slow Route", None, None, None, "Express Way", None], Directed)) ), Graph, "Creating of graph (Directed)"):

// G must be different from G2 (Nr. 05)
prog::test(bool(G<>G2), TRUE):

// Check for directed/undirected (Nr. 06 - 07)
prog::test(Graph::isDirected(G), FALSE):
prog::test(Graph::isDirected(TG), TRUE):

// Now check for predefined values with generated values for graph G (Nr. 08 - 11)
prog::test( map(Graph::getEdgesLeaving(G),sort),
            map(table(a=[1, 2],2=[1, a],1=[2, a]), sort) ):
prog::test( map(Graph::getEdgesEntering(G), sort),
            map(table(a=[1, 2],2=[1, a],1=[2, a]),sort) ):
prog::test( sort(Graph::getEdges(G)),
            sort([[1, 2], [1, a], [2, 1], [2, a], [a, 1], [a, 2]]) ): 
prog::test( sort(Graph::getVertices(G)), sort([1, 2, a]) ):

// Some tests if the number of vertices and edges for the created graphs is correct. (Nr. 12 - 20)
prog::test(nops(Graph::getVertices(G2)),4):
prog::test(sort(Graph::getVertices(G)), sort([1, 2, a])):
prog::test(nops(Graph::getVertices(TG)),6):
prog::test(sort(Graph::getVertices(TG)), sort([3, 4, 5, a, b, f])):
prog::test(nops(Graph::getEdges(G)),6):
prog::test(nops(Graph::getEdges(G2)),4):
prog::test(nops(Graph::getEdges(G3)),0):
prog::test(nops(Graph::getEdges(TG)),7):
prog::test(sort(Graph::getEdges(TG)), sort([[3, 4], [4, 5], [4, f], [5, f], [a, b], [b, 3], [b, 4]])):

// Test for vertices and edges (Nr. 21 - 31)
prog::test(Graph::isVertex(G,[2]), TRUE):
prog::test(Graph::isVertex(G,[a]), TRUE):
prog::test(Graph::isVertex(G,[5]), FALSE):
prog::test(Graph::isVertex(TG,[5]), TRUE):
prog::test(Graph::isVertex(TG,[c]), FALSE):
prog::test(Graph::isEdge(G,[[2,a]]), TRUE):
prog::test(Graph::isEdge(G,[[a,2]]), TRUE):
prog::test(Graph::isEdge(G,[[a,4]]), FALSE):
prog::test(Graph::isEdge(TG,[[a,b]]), TRUE):
prog::test(Graph::isEdge(TG,[[b,a]]), FALSE):
prog::test(Graph::isEdge(TG,[[g,h]]), FALSE):

// Test for edge/vertex weights (Nr. 32 - 33)
prog::test(Graph::getVertexWeights(G), FAIL):
prog::test(Graph::getVertexWeights(TG), table(a=10,b=20,4=0,5=infinity,f=-10)):

// Test for outdegree (Nr. 34)
prog::test(Graph::outDegree(G,[2]), table(2=2)):
prog::test(Graph::inDegree(G,[a]), table(a=2)):

// Test for shortest path (sssp and apsp) (Nr. 36 - 38)
prog::test( traperror( Graph::sssp_Dijkstra(G,[2]) ), 0, "Dijkstra"):
prog::test( traperror( Graph::sssp_Bellman(G,[2]) ), 0, "Bellmann"):
prog::test( traperror( Graph::apsp_Floyd(G,[2]) ), 0, "Ford"):

// Check if topSort runs through (Nr. 39)
prog::test( traperror( Graph::topSort(G2) ), 0, "topSort"):

// Check if the indegree for graph G in vertex a is 2 (Nr. 40)
prog::test( Graph::inDegree(G, [a]), table(a=2) ):

// Check if plotCircleGraph runs through (Nr. 41 - 42)
prog::test( traperror( Graph::plotCircleGraph(G) ), 0,
	    if version()[1]=2 then Status=Analyzed
	    else "plotCircleGraph empty" end):
prog::test( traperror( Graph::plotCircleGraph(G,PointSize=2) ), 0,
	    if version()[1]=2 then Status=Analyzed
	    else "plotCircleGraph with Parameter" end):

// Check if remove/add edge works (Nr. 43 - 46)
prog::test( traperror( (TG := Graph::removeEdge(TG,[[a,b]])) ), 0, "removeEdge"):
prog::test(bool(Graph::isEdge(TG,[[a,b]])), FALSE):
prog::test( traperror( (TG := Graph::addEdges(TG,[[a,b]], EdgeWeights = [50], EdgeCosts = [30], EdgeDescriptions = ["First Way"])) ), 0, "addEdges"):
prog::test(bool(Graph::isEdge(TG,[[a,b]])), TRUE):

// Test for edgeWeight (Nr. 47)
prog::test( Graph::getEdgeWeights(TG), table([5, f]=40,[4, f]=70,[4, 5]=30,[3, 4]=20,[b, 4]=60,[b, 3]=10,[a, b]=50)):

// Check if remove/add vertex works (Nr. 48 - 53)
prog::test( traperror( (TG := Graph::removeVertex(TG,[a])) ), 0, "removeVertex"):
prog::test(bool(Graph::isVertex(TG,[a])), FALSE):
prog::test(bool(Graph::isEdge(TG,[[a,b]])), FALSE):
prog::test( traperror( (TG := Graph::addVertices(TG,[a], VertexWeights = [10])) ), 0, "addVertex"):
prog::test( traperror( (TG := Graph::addEdges(TG,[[a,b]], EdgeWeights = [50], EdgeCosts = [30], EdgeDescriptions = ["First Way"])) ), 0, "addEdges"):
prog::test(bool(Graph::isEdge(TG,[[a,b]])), TRUE):

// Test for edgeCosts (Nr. 54)
prog::test( Graph::getEdgeCosts(TG), table([a,b]=30,[b,3]=70,[b,4]=20,[3,4]=40,[4,5]=50,[4,f]=10,[5,f]=40)):

// Check if setEdgeCosts works (Nr. 55- 62)
prog::test( traperror( (TG := Graph::setEdgeCosts(TG, [[a,b]], [None])) ), 0, "setEdgeCosts 1"):
prog::test( Graph::getEdgeCosts(TG), table([b,3]=70,[b,4]=20,[3,4]=40,[4,5]=50,[4,f]=10,[5,f]=40)):

prog::test( traperror( (TG := Graph::setEdgeCosts(TG, [[a,b]], [infinity])) ), 0, "setEdgeCosts 2"):
prog::test( Graph::getEdgeCosts(TG), table([a,b]=infinity,[b,3]=70,[b,4]=20,[3,4]=40,[4,5]=50,[4,f]=10,[5,f]=40)):

prog::test( traperror( (TG := Graph::setEdgeCosts(TG, [[a,b]], [-30])) ), 0, "setEdgeCosts 3"):
prog::test( Graph::getEdgeCosts(TG), table([a,b]=-30,[b,3]=70,[b,4]=20,[3,4]=40,[4,5]=50,[4,f]=10,[5,f]=40)):

prog::test( traperror( (TG := Graph::setEdgeCosts(TG, [[a,b]], [30])) ), 0, "setEdgeCosts 4"):
prog::test( Graph::getEdgeCosts(TG), table([a,b]=30,[b,3]=70,[b,4]=20,[3,4]=40,[4,5]=50,[4,f]=10,[5,f]=40)):

// Now check for apsp_Floyd: (Nr. 63 - 64)
prog::test( Graph::apsp_Floyd(G), [table((a, a)=0,(a, 2)=1,(a, 1)=1,(2, a)=1,(2, 2)=0,(2, 1)=1,(1, a)=1,(1, 2)=1,(1, 1)=0), table((a, 2)=a,(a, 1)=a,(2, a)=2,(2, 1)=2,(1, a)=1,(1, 2)=1)]):
prog::test( Graph::apsp_Floyd(TG),
           [table((f, 3)=infinity,(3, f)=90,(5, 5)=0,(5, b)=infinity,(b, 5)=60,(b, b)=0,(5, 4)=infinity,(4, 5)=30,(4, b)=infinity,(b, 4)=30,(5, 3)=infinity,(4, 4)=0,(3, 5)=50,(3, b)=infinity,(b, 3)=10,(4, 3)=infinity,(3, 4)=20,(3, 3)=0,(f, a)=infinity,(a, f)=150,(5, a)=infinity,(b, a)=infinity,(a, 5)=110,(a, b)=50,(4, a)=infinity,(a, 4)=80,(3, a)=infinity,(a, 3)=60,(a, a)=0,(f, f)=0,(f, 5)=infinity,(f, b)=infinity,(5, f)=40,(b, f)=100,(f, 4)=infinity,(4, f)=70),
            table((f, 3)=infinity,(3, f)=4,(5, b)=infinity,(b, 5)=4,(5, 4)=infinity,(4, 5)=4,(4, b)=infinity,(b, 4)=3,(5, 3)=infinity,(3, 5)=4,(3, b)=infinity,(b, 3)=b,(4, 3)=infinity,(3, 4)=3,(f, a)=infinity,(a, f)=4,(5, a)=infinity,(b, a)=infinity,(a, 5)=4,(a, b)=a,(4, a)=infinity,(a, 4)=3,(3, a)=infinity,(a, 3)=b,(f, 5)=infinity,(f, b)=infinity,(5, f)=5,(b, f)=4,(f, 4)=infinity,(4, f)=4)]):

// Now check for sssp_Deijkstra: (Nr. 65- 69)
prog::test( Graph::sssp_Dijkstra(G, [1]), [table(a=1,2=1,1=0), table(a=1,2=1)]):
prog::test( Graph::sssp_Dijkstra(G, [a]), [table(a=0,2=1,1=1), table(1=a,2=a)]):
prog::test( Graph::sssp_Dijkstra(TG, [a]), [table(f=150,5=110,4=80,3=60,b=50,a=0), table(f=4,5=4,4=3,3=b,b=a)]):
prog::test( Graph::sssp_Dijkstra(TG, [f]), [table(f=0,5=infinity,4=infinity,3=infinity,b=infinity,a=infinity), table()]):
// Typical example for dijkstra to make a wrong calculation: (compare to Bellmann)
prog::test( Graph::sssp_Dijkstra(G2, [a]), [table(d=3,c=-1,b=2,a=0), table(d=c,c=a,b=a)]):

// Now check for sssp_Bellman (Nr. 70 - 74)
prog::test( Graph::sssp_Bellman(G, [1]), [table(a=1,2=1,1=0), table(a=1,2=1)]):
prog::test( Graph::sssp_Bellman(G, [a]), [table(a=0,2=1,1=1), table(1=a,2=a)]):
prog::test( Graph::sssp_Bellman(TG, [a]), [table(f=150,5=110,4=80,3=60,b=50,a=0), table(f=4,5=4,4=3,3=b,b=a)]):
prog::test( Graph::sssp_Bellman(TG, [f]), [table(f=0,5=infinity,4=infinity,3=infinity,b=infinity,a=infinity), table()]):
// Here it works correctly !
prog::test( Graph::sssp_Bellman(G2, [a]), [table(d=1,c=-1,b=2,a=0), table(d=c,c=b,b=a)]):

// Check if setEdgeWeights works (Nr. 75- 82)
prog::test( traperror( (TG := Graph::setEdgeWeights(TG, [[a,b]], [None])) ), 0, "setEdgeWeights 1"):
prog::test( Graph::getEdgeWeights(TG), table([5, f]=40,[4, f]=70,[4, 5]=30,[3, 4]=20,[b, 4]=60,[b, 3]=10) ):

prog::test( traperror( (TG := Graph::setEdgeWeights(TG, [[a,b]], [infinity])) ), 0, "setEdgeWeights 2"):
prog::test( Graph::getEdgeWeights(TG), table([5, f]=40,[4, f]=70,[4, 5]=30,[3, 4]=20,[b, 4]=60,[b, 3]=10, [a, b]=infinity)):

prog::test( traperror( (TG := Graph::setEdgeWeights(TG, [[a,b]], [-50])) ), 0, "setEdgeWeights 3"):
prog::test( Graph::getEdgeWeights(TG), table([5, f]=40,[4, f]=70,[4, 5]=30,[3, 4]=20,[b, 4]=60,[b, 3]=10,[a, b]=-50)):

prog::test( traperror( (TG := Graph::setEdgeWeights(TG, [[a,b]], [50])) ), 0, "setEdgeWeights 4"):
prog::test( Graph::getEdgeWeights(TG), table([5, f]=40,[4, f]=70,[4, 5]=30,[3, 4]=20,[b, 4]=60,[b, 3]=10,[a, b]=50)):

// Check for vertice/edge nubmers: (Nr. 83 - 86)
prog::test(Graph::getVertexNumber(G2), 4):
prog::test(Graph::getEdgeNumber(G2), 4):
prog::test(Graph::getVertexNumber(TG), 6):
prog::test(Graph::getEdgeNumber(TG), 7):

// Check for edgeDescriptions (Nr. 87 - 88)
prog::test( Graph::getEdgeDescriptions(TG), table([4, f]="Express Way",[b, 3]="Slow Route",[a, b] = "First Way") ):
prog::test( Graph::getEdgeDescriptions(G), FAIL ):

// Check for revert (Nr. 89 - 92)
prog::test( traperror( (TestG := Graph::revert(TG)) ), 0, "revert"):
prog::test( Graph::getEdgesNumber(TG), Graph::getEdgesNumber(TestG) ):
prog::test( Graph::getVertices(TG), Graph::getVertices(TestG) ):

// Check for selectEdge (Nr. 92 - 95)
prog::test( Graph::selectEdge(TG, [[a,b],[a,f]], TRUE, Show=FALSE) , [[a,b]]):
prog::test( Graph::selectEdge(TG, [[a,b],[a,f]], FALSE, Show=FALSE) , [[a,f]]):
prog::test( Graph::selectEdge(TestG, [[b,a],[f,a]], TRUE, Show=FALSE) , [[b,a]]):
prog::test( Graph::selectEdge(TestG, [[b,a],[f,a]], FALSE, Show=FALSE) , [[f,a]]):

// Check for setVertexWeights (Nr. 96 - 100)
prog::test( nops(Graph::getVertexWeights(TG)) , 5): 
prog::test( traperror( (TG := Graph::setVertexWeights(TG, [a], [None])) ), 0, "setVertexWeights 1"):
prog::test( nops(Graph::getVertexWeights(TG)) , 4): 
prog::test( traperror( (TG := Graph::setVertexWeights(TG, [a], [10])) ), 0, "setVertexWeights 2"):
prog::test( nops(Graph::getVertexWeights(TG)) , 5): 

// Check for checkForVertices (Nr. 101 - 104)
// Empty because even if the edges do not exist, the vertices do ! 
prog::test( Graph::checkForVertices([[a,b],[a,f],[a,c],[a,a]],Graph::getVertices(TG)) , [c]): 
prog::test( Graph::checkForVertices([[a,b],[1,3]],Graph::getVertices(TG)) , [1]): 
prog::test( sort(Graph::checkForVertices([[a,b],[a,3],[1,2],[5,6]],Graph::getVertices(TG))) , ([1, 2, 6])): 
prog::test( Graph::checkForVertices( [[]], [] ) , [] ): 

// Check for create complete Graph (Nr. 105 - 113)
prog::test( domtype( (CG := Graph::createCompleteGraph(3) )), Graph, "Creating of complete graph (undirected)"):
prog::test( Graph::getVertexNumber(CG), 3):
prog::test( Graph::getEdgeNumber(CG), 6):

prog::test( domtype( (CG := Graph::createCompleteGraph(50) )), Graph, "Creating of complete graph (undirected)"):
prog::test( Graph::getVertexNumber(CG), 50):
prog::test( Graph::getEdgeNumber(CG), 2450):

prog::test( domtype( (SG := Graph::getSubGraph(CG, [1,2,3]) )), Graph, "Creating of subgraph of complete graph (undirected)"):
prog::test( Graph::getVertexNumber(SG), 3):
prog::test( Graph::getEdgeNumber(SG), 6):

//Check for chromatic polynomial (Nr. 114)
prog::test( subs(op(Graph::chromaticPolynomial(SG, x),1), x = 5), 60):

//Check for depth first search (Nr. 115)
// This test fails with MuPAD 2.*. Is the result dependent on MuPAD internal
// ordering ?
prog::test( Graph::depthFirstSearch(TG, StartVertex = [a]), [table(f=6,5=5,4=4,3=3,b=2,a=1), table(a=12,b=11,3=10,4=9,5=8,f=7), table(f=5,b=a,a=a,5=4,4=3,3=b)],
	    if version()[1]=2 then Status=Registered else "" end):

//Check for createRandomGraph (Nr. 116 - 118)
prog::test( domtype( (CG := Graph::createRandomGraph(200, 100, Undirected)) ), Graph, "Creating of random graph (Undirected)"):
prog::test( Graph::getVertexNumber(CG), 200):
prog::test( Graph::getEdgeNumber(CG), 200): // because it is undirected !!!

// Check for MST (Kruskal) (Nr. 119 - 123)
prog::test( domtype( (MST_Graph := Graph([a,b,c,d,e,f,g,h,i],[[a,b],[a,h],[b,c],[b,h],[c,d],[c,f],[c,i],[d,e],[d,f],[e,f],[f,g],[g,i],[g,h],[h,i]], EdgeWeights = [4,8,8,11,7,4,2,9,14,10,2,6,1,14], EdgeCosts = [4,4,8,11,7,4,2,9,14,10,2,6,1,14])) ), Graph, "Creating of MST-Graph (undirected)"):
prog::test( op(Graph::mst_Kruskal(MST_Graph, ReturnAsTable), 2),  37 ):
prog::test( op(Graph::mst_Kruskal(MST_Graph, ReturnAsTable, Weights), 2), 37 ):
// This test fails with MuPAD 2.*. Is the result dependent on MuPAD internal
// ordering ?
prog::test( Graph::mst_Kruskal(MST_Graph, ReturnAsTable, Weights), [table([e, d]=9,[d, e]=9,[a, h]=8,[h, a]=8,[d, c]=7,[c, d]=7,[f, c]=4,[c, f]=4,[b, a]=4,[a, b]=4,[f, g]=2,[g, f]=2,[i, c]=2,[c, i]=2,[h, g]=1,[g, h]=1), 37],
	    if version()[1]=2 then Status=Registered else "" end):
prog::test( Graph::mst_Kruskal(MST_Graph, ReturnAsTable, Costs), [table([d, e]=9,[e, d]=9,[d, c]=7,[c, d]=7,[c, f]=4,[f, c]=4,[a, h]=4,[h, a]=4,[b, a]=4,[a, b]=4,[f, g]=2,[g, f]=2,[i, c]=2,[c, i]=2,[h, g]=1,[g, h]=1), 33] ):

//Check for breadth first search (Nr. 124 - 125)
prog::test( domtype( (ST := Graph([a,b,c,d,e,f,g],[[a,b],[a,c],[b,d],[c,e],[c,f],[d,g]], EdgeWeights = [10,10,10,10,10,10], EdgeCosts = [10,10,10,10,10,10], Directed)) ), Graph, "Creating of Searchtree (directed)"):
prog::test( Graph::breadthFirstSearch(ST, StartVertex = [a]), [table(g=7,f=6,e=5,d=4,c=3,b=2,a=1), table(g=3,f=2,e=2,d=2,c=1,b=1,a=0), table(g=d,f=c,e=c,d=b,c=a,b=a,a=a)] ):

// Check for shortest path with single source (Nr. 126 - 131)
prog::test( Graph::shortestPathSingleSource(TG, StartVertex = [a], SearchWith=Dijkstra), [table(f=150,5=110,4=80,3=60,b=50,a=0), table(f=4,5=4,4=3,3=b,b=a)]):
prog::test( Graph::shortestPathSingleSource(TG, StartVertex = [a], SearchWith=Dijkstra, SearchFor=Weights), [table(f=150,5=110,4=80,3=60,b=50,a=0), table(f=4,5=4,4=3,3=b,b=a)]):
prog::test( Graph::shortestPathSingleSource(TG, StartVertex = [a], SearchWith=Dijkstra, SearchFor=Costs), [table(f=60,b=30,a=0,5=100,4=50,3=100), table(f=4,5=4,4=b,3=b,b=a)]):
prog::test( Graph::shortestPathSingleSource(TG, StartVertex = [a], SearchWith=Bellman), [table(f=150,5=110,4=80,3=60,b=50,a=0), table(f=4,5=4,4=3,3=b,b=a)]):
prog::test( Graph::shortestPathSingleSource(TG, StartVertex = [a], SearchWith=Bellman, SearchFor=Weights), [table(f=150,5=110,4=80,3=60,b=50,a=0), table(f=4,5=4,4=3,3=b,b=a)]):
prog::test( Graph::shortestPathSingleSource(TG, StartVertex = [a], SearchWith=Bellman, SearchFor=Costs), [table(f=60,b=30,a=0,5=100,4=50,3=100), table(f=4,5=4,4=b,3=b,b=a)]):

// Check for shortest path with all pairs (Nr. 132)
prog::test( Graph::shortestPathAllPairs(G), [table((a, a)=0,(a, 2)=1,(a, 1)=1,(2, a)=1,(2, 2)=0,(2, 1)=1,(1, a)=1,(1, 2)=1,(1, 1)=0), table((a, 2)=a,(a, 1)=a,(2, a)=2,(2, 1)=2,(1, a)=1,(1, 2)=1)]):

// Check for strongly connected components (Nr. 133 - 135)
prog::test( domtype( (ST := Graph([1,2,3,4,5,6],[[1,2],[2,6],[6,1],[2,3],[3,4],[4,5],[5,3]])) ), Graph):
prog::test( domtype( op(Graph::stronglyConnectedComponents(ST)) ), Graph ):
prog::test( Graph::getEdges(ST) , [[1, 2], [1, 6], [2, 1], [2, 3], [2, 6], [3, 2], [3, 4], [3, 5], [4, 3], [4, 5], [5, 3], [5, 4], [6, 1], [6, 2]]):

// Check for circle graph (and getEdge/VertexNumber) (Nr. 136 - 141)
prog::test( domtype( (ST := Graph::createCircleGraph(100)) ), Graph ):
prog::test( Graph::getEdgeNumber(ST), 100 ):
prog::test( Graph::getVertexNumber(ST), 100 ):
prog::test( domtype( (ST := Graph::createCircleGraph(100, Undirected)) ), Graph ):
prog::test( Graph::getEdgeNumber(ST), 200 ):
prog::test( Graph::getVertexNumber(ST), 100 ):

// Check for createMinSort (Nr. 142 - 143)
prog::test( Graph::createMinSort(table(1=5,2=4,3=3)) , [3, 2, 1]):
prog::test( Graph::createMinSort(table(a=1,2=-5,b=20)) , [2, a, b]):

// Check for setRandomEdgeWeights/Costs (Nr. 144 - 147)
prog::test( nops(Graph::getEdgeWeights(Graph::createRandomEdgeWeights(CG, -100..100, Dom::Real))) , 200):
prog::test( nops(Graph::getEdgeWeights(Graph::createRandomEdgeWeights(CG, -100..100, Dom::Integer))) , 200):
prog::test( nops(Graph::getEdgeCosts(Graph::createRandomEdgeCosts(CG, -50..50, Dom::Integer))) , 200):
prog::test( nops(Graph::getEdgeCosts(Graph::createRandomEdgeCosts(CG, -50..50, Dom::Real))) , 200):

// Check for getBestAdjacentEdge (Nr. 148 - 152)
prog::test( Graph::getBestAdjacentEdge(G2, [a] ,[a,b,c,d]) , c):
prog::test( Graph::getBestAdjacentEdge(G2, [a] ,[a,b,d]) , b):
prog::test( Graph::getBestAdjacentEdge(G2, [b] ,[a,b,c,d]) , c):
prog::test( Graph::getBestAdjacentEdge(G2, [c] ,[a,b,c,d]) , d):
prog::test( Graph::getBestAdjacentEdge(G2, [d] ,[a,b,c,d]) , FAIL):

// Check for isConnected  (153 - 154)
prog::test( Graph::isConnected(G2) , TRUE):
prog::test( Graph::isConnected(G3) , FALSE):

// Check for everything that has to do with bipartite (Nr.155 - 165)
prog::test( domtype( (ST := Graph([a,b,c,d,e,f,g,h,i],[[a,b],[a,c],[b,d],[c,e],[c,f],[d,g]], EdgeWeights = [10,10,10,10,10,10], EdgeCosts = [10,10,10,10,10,10])) ), Graph):
prog::test( Graph::bipartite(G2) , FALSE):
prog::test( Graph::bipartite(G3) , TRUE):
prog::test( Graph::bipartite(ST) , TRUE):

prog::test( traperror( Graph::plotBipartiteGraph(ST) ), 0,
	    if version()[1]=2 then Status=Analyzed 
            else "plotBipartiteGraph empty" end):
prog::test( traperror( Graph::plotBipartiteGraph(ST, PointSize=2) ), 0,
	    if version()[1]=2 then Status=Analyzed else "" end):
prog::test( traperror( Graph::plotBipartiteGraph(ST, SpecialEdges = [[a,b],[a,c],[b,d]], SpecialEdgeColor=RGB::Blue) ), 0,
	    if version()[1]=2 then Status=Analyzed else "" end):
prog::test( traperror( Graph::plotBipartiteGraph(ST, SpecialEdges = [[a,b],[a,c],[b,d]], VertexColor = RGB::Yellow) ), 0,
	    if version()[1]=2 then Status=Analyzed else "" end):
prog::test( traperror( Graph::plotBipartiteGraph(ST, SpecialEdges = [[a,b],[a,c],[b,d]], Vertex1Color= RGB::Green) ), 0,
	    if version()[1]=2 then Status=Analyzed else "" end):
prog::test( traperror( Graph::plotBipartiteGraph(ST, SpecialEdges = [[a,b],[a,c],[b,d]], Vertex2Color= RGB::Green) ), 0,
	    if version()[1]=2 then Status=Analyzed else "" end):
prog::test( traperror( Graph::plotBipartiteGraph(ST, SpecialEdges = [[a,b],[a,c],[b,d]], SpecialEdgeColor=RGB::Blue, VertexColor = RGB::Yellow, Vertex1Color = RGB::Black, Vertex2Color= RGB::Green) ), 0,
	    if version()[1]=2 then Status=Analyzed else "" end):

// Check for minimumSpanningTree (Nr.166 - 168)
// This test fails with MuPAD 2.*. Is the result dependent on MuPAD internal
// ordering ?
prog::test( Graph::minimumSpanningTree(MST_Graph, ReturnAsTable, SearchFor=Weights), [table([e, d]=9,[d, e]=9,[a, h]=8,[h, a]=8,[d, c]=7,[c, d]=7,[f, c]=4,[c, f]=4,[b, a]=4,[a, b]=4,[f, g]=2,[g, f]=2,[i, c]=2,[c, i]=2,[h, g]=1,[g, h]=1), 37],
            if version()[1]=2 then Status=Registered else "" end):
prog::test( Graph::minimumSpanningTree(MST_Graph, ReturnAsTable, SearchFor=Costs), [table([d, e]=9,[e, d]=9,[d, c]=7,[c, d]=7,[c, f]=4,[f, c]=4,[a, h]=4,[h, a]=4,[b, a]=4,[a, b]=4,[f, g]=2,[g, f]=2,[i, c]=2,[c, i]=2,[h, g]=1,[g, h]=1), 33] ):
prog::test( domtype( Graph::minimumSpanningTree(MST_Graph, SearchFor=Costs) ) , Graph ):

// Check for contract (Nr. 169 - 172)
prog::test( domtype( (ST := Graph([a,b,c,d,e,f],[[a,c],[d,a],[f,c],[d,f],[e,b]], Directed)) ), Graph):
prog::test( domtype( (CG := Graph::contract(ST, table(A = [a,b,c], B = [d,f]) )) ) , Graph):
prog::test( Graph::getEdges(CG), [[A, A], [B, A], [B, B], [e, A]] ):
prog::test( Graph::getVertices(CG), [A, B, e] ):

// now delete all used variables everything
delete G:
delete G2:
delete G3:
delete MSP_Graph:
delete ST:
delete TG:
delete TestG:
delete CG:
delete ST:
delete SG:

/*****************************************************************************
  $Id: Graph.tst,v 1.1.1.2 2003/09/22 18:05:14 nthiery Exp $
  File:         lib/DOMAINS/DOMAIN/TEST/GraphAddon.mu
  Authors:      Florent Hivert <florent.hivert@univ-mlv.fr>
  Purpose:      Test file for the add-on to the library Graph
  Created:      30/09/2002
  License:      LGPL
*****************************************************************************/
prog::testfunc(Graph::levelDecomposition):

gr := Graph::new([a,b,c,d], [], Directed):
prog::test(Graph::levelDecomposition(gr),
           [[a, b, c, d]]):
gr := Graph::new([a,b,c], [[a,b],[a,c]], Directed):
prog::test(Graph::levelDecomposition(gr),
           [[a], [b, c]]):
gr := Graph::new([a,b,c], [[a,b],[a,c],[b,a]], Directed):
prog::test(Graph::levelDecomposition(gr),
           FAIL):
gr := Graph::new([a,b,c], [[a,b],[a,c],[b,c]], Directed):
prog::test(Graph::levelDecomposition(gr),
           [[a], [b], [c]]):
gr := Graph::new([a,b,c,d], [[a,b],[a,c],[b,d]], Directed):
prog::test(Graph::levelDecomposition(gr),
           [[a], [b, c], [d]]):
gr := Graph::new([a,b,c,d,e,f], [[a,b],[a,c],[b,d],[b,e],[c,f],[d,f]], 
		      Directed):
prog::test(Graph::levelDecomposition(gr),
           [[a], [b, c], [d, e], [f]]):

// isAcyclic is implemented on levelDecomposition so it need not so much tests.
prog::testfunc(Graph::isAcyclic):
gr := Graph::new([a,b,c,d], [], Directed):
prog::test(Graph::isAcyclic(gr),
           TRUE):
gr := Graph::new([a,b,c,d], [[a,b],[b,c]], Directed):
prog::test(Graph::isAcyclic(gr),
           TRUE):

// delete gr 
delete gr:
prog::testexit():
