/*****************************************************************************
  Authors:      Nicolas M. Thiery <nthiery@users.sourceforge.net>
                Florent Hivert <florent.hivert@univ-mlv.fr>
  Created:      30/09/2002
  License:      LGPL
*****************************************************************************/

Graph::levelDecomposition := 
/* Compute the level decomposition of a directed acyclic graph (DAG).
   
*/
proc(G : Graph) : Type::Union(Type::ListOf(DOM_LIST), DOM_FAIL)
    local _vertices, _inDegree, _edgesLeaving, res,
	  minimums,   // vertices that have no parents
	  newMins,    // new minimums
	  m, c; // remainings vertices
begin  
    if testargs() then
	if not Graph::isDirected(G) then
	    error("The graph must be directed.");
	end_if;
    end_if;
    _vertices := Graph::getVertices(G);
    _inDegree := Graph::inDegree(G);
    _edgesLeaving := Graph::getEdgesLeaving(G);
    minimums := select(_vertices, z -> (_inDegree[z]=0));
    res := [];
    while (minimums <> []) do
	newMins := [];
	for m in minimums do
	    delete(_inDegree[m]);
	    for c in _edgesLeaving[m] do
		_inDegree[c] := _inDegree[c] - 1;
		if _inDegree[c]= 0 then 
		    newMins := append(newMins, c);
		end_if;
	    end_for;
	end_for;
	res := append(res, minimums);
	minimums := newMins;
    end_while;
    if nops(_inDegree) = 0 then res else FAIL; end_if;
end_proc:


Graph::isAcyclic := 
// Test whether a directed graph is acyclic or not.
proc(G : Graph) : DOM_BOOL
begin
    if testargs() then
	if not Graph::isDirected(G) then
	    error("The graph must be directed.");
	end_if;
    end_if;
    if Graph::levelDecomposition(G) <> FAIL then TRUE else FALSE end_if;
end_proc:



// End of file
null():

