/*****************************************************************************
  $Id: tensorProductOfMaps.tst,v 1.1.1.2 2004/10/16 15:16:58 nthiery Exp $
  File:		lib/DOMAINS/DOMAIN/TEST/TensorProductOfMaps.tst
  Authors:	Florent Hivert <florent.hivert@univ-mlv.fr>
  Purpose:	Test suite for operators::tensorProductOfMaps
  Created:	Thu Feb  15 20:04:58 2004
  License:	LGPL
  Project:	MuPAD-Combinat, http://mupad-combinat.sf.net
*****************************************************************************/


alias(TPFM = Dom::TensorProductOfFreeModules):
alias(tens = operators::tensor):
prog::testinit("tensorProductOfMaps"):
prog::testfunc(operators::TensorProductOfMaps):
//////////////////////////////////////////////////////////////////////////////
// Let's build some free modules with different ranking methods
SGA := (N, R) -> Dom::FreeModulePoly(R, combinat::subClass(combinat::permutations, Parameters = N),
				     Variables = N,
				     MultirankerBasis = [id, id]):

///////////////////////////////////////////////////////////////////////
// The polynomial optimisation is possible => test over different rings
for ring in [Dom::IntegerMod(7), 
	     Dom::UnivariatePolynomial(X, Dom::Rational), 
	     Dom::ExpressionField()] do
///////////////////////////////////////////////////////////////////////

SGA2   := SGA(2, ring):
SGA3   := SGA(3, ring):
SGA22  := TPFM([SGA2,SGA2]):
SGA23  := TPFM([SGA2,SGA3]):
SGA222 := TPFM([SGA2,SGA2,SGA2]):
prog::test((f0 := operators::tensorProductOfMaps([id, id], SGA22, SGA22);
	    f0(SGA22::zero)), 
	   SGA22::zero):
prog::test(iszero(f0(tens(ring(2)*SGA2([2,1]), SGA2([1,2]))) -
		  tens(ring(2)*SGA2([2,1]), SGA2([1,2])))):
prog::test((f0 := operators::tensorProductOfMaps([z->SGA2::zero, id], SGA22, SGA22);
	    f0(SGA22::zero)), 
	   SGA22::zero):
prog::test(f0(tens(ring(2)*SGA2([2,1]), SGA2([1,2]))),
	   SGA22::zero):
prog::test((f0 := operators::tensorProductOfMaps([z->ring(2)*z, z->ring(3)*z], SGA22, SGA22);
	    f0(SGA22::zero)), 
	   SGA22::zero):
prog::test(iszero(f0(tens(ring(2)*SGA2([2,1]), SGA2([1,2]))) - 
		  ring(6)*tens(ring(2)*SGA2([2,1]), SGA2([1,2])))):

cat3 := operators::makeLinear(p2 -> SGA3::term(p2.[3]), Source = SGA2, ImageSet = SGA3):
prog::test((fcat3 := operators::tensorProductOfMaps([id, cat3], SGA22, SGA23);
	    fcat3(SGA22::zero)), 
	   SGA23::zero):
prog::test(iszero(fcat3(tens(ring(2)*SGA2([2,1]), SGA2([1,2]))) - 
		  tens(ring(2)*SGA2([2,1]), SGA3([1,2,3])))):

    
///////////////////////////////////////////////////////////////////////
end_for: // for ring in [...] do 
///////////////////////////////////////////////////////////////////////



///////////////////////////////////////////////////////////////////////
// The polynomial optimisation is impossible
SGAL := Dom::FreeModuleTable(Dom::Rational, combinat::permutations):
SGALL  := Dom::TensorProductOfFreeModules([SGAL, SGAL]):
SGALLL := Dom::TensorProductOfFreeModules([SGAL, SGAL, SGAL]):
//////////////////////////////////////////////////////////////////////////////
prog::test((fL0 := operators::tensorProductOfMaps([id, id], SGALL, SGALL);
	    fL0(SGALL::zero)), 
	   SGALL::zero):
prog::test(iszero(fL0(tens(2*SGAL([2,1]), SGAL([1,2]))) -
		  tens(2*SGAL([2,1]), SGAL([1,2])))):
prog::test((fL0 := operators::tensorProductOfMaps([z->SGAL::zero, id], SGALL, SGALL);
	    fL0(SGALL::zero)), 
	   SGALL::zero):
prog::test(fL0(tens(2*SGAL([2,1]), SGAL([1,2]))),
	   SGALL::zero):
prog::test((fL0 := operators::tensorProductOfMaps([z->2*z, z->3*z], SGALL, SGALL);
	    fL0(SGALL::zero)), 
	   SGALL::zero):
prog::test(iszero(fL0(tens(2*SGAL([2,1]), SGAL([1,2]))) - 
		    6*tens(2*SGAL([2,1]), SGAL([1,2])))):

cat3 := operators::makeLinear(p2 -> SGAL::term(p2.[3]), Source = SGAL, ImageSet = SGAL):
prog::test((fcat3 := operators::tensorProductOfMaps([id, cat3], SGALL, SGALL);
	    fcat3(SGALL::zero)), 
	   SGALL::zero):
prog::test(iszero(fcat3(tens(2*SGAL([2,1]), SGAL([1,2]))) - 
		  tens(2*SGAL([2,1]), SGAL([1,2,3])))):

prog::testexit():
//////////////////////////////////////////////////////////////////////////////

prog::check(operators::TensorProductOfMaps, 3):


// Local Variables:
// mode: mupad
// End:
