/*****************************************************************************
  $Id: persistentRemember.mu,v 1.1.1.3 2004/10/16 15:16:56 nthiery Exp $
  File:		lib/OPERATORS/persistentRemember.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Created:	28/10/2002
  License:	LGPL
*****************************************************************************/


/**
   Domain for functions with a remember table that persists from
   one MuPAD session to another.
   
   1st operand: the function used to do the calculation

   2nd operand: the directory name in the database where the results
   are stored
 
 **/


alias(GETFUNCTION(F)   = extop(F,1)):
alias(GETDIRECTORY(F)  = extop(F,2)):

if shell=FAIL then
    warning("operators::persistentRemember requires the dynamic module shell");
end_if:

domain operators::persistentRemember

    inherits Dom::BaseDomain;

    interfaceAdd := {hold(directory) // name of the root directory where the
				  // information are stored
		 };

    typeProcedure := Type::AnyType;

    directory := ".persistentRememberDataBase";

    setDirectory :=
    proc(dir: DOM_STRING)
    begin
	dom::directory := dir;
    end_proc;
    
    dirname :=
    proc(directory: DOM_STRING)
    begin
	// Can't use pathname, because dom::directory might already
	// contains / or others. Not portable ?
	dom::directory."/".directory."/";
    end_proc;

    filename :=
    proc(directory: DOM_STRING)
    begin
	_concat
	(
	 dom::dirname(directory),
	 substring(shell::system("md5sum", expr2text(args(2..args(0)))),
		   %if "12"[1]="1" then 1..32 else 0..31 end),
	 ".mu");
    end_proc;
    
    makeDirIfMissing :=
    proc(directory)
    begin
	shell::system("if test -d ".directory."; then echo ; else mkdir ".directory."; fi")
    end_proc;
    
    new :=
    proc(f: dom::typeProcedure,
	 directory = FAIL: Type::Union(DOM_FAIL, DOM_STRING))
	option escape;
    begin
	if directory = FAIL then
	    if domtype(f) = DOM_PROC then
		directory := expr2text(extop(f,6));
	    else
		directory := expr2text(f);
	    end_if;
	end_if;

	if traperror(dom::makeDirIfMissing(dom::directory)) <> 0 or
	   traperror(dom::makeDirIfMissing(dom::dirname(directory))) <> 0 then
	    error("Could not make database directory");
	end_if;
	
	new(dom, f, directory);
    end_proc;

    reset :=
    proc(x: dom)
	local directory;
    begin
	directory := dom::dirname(GETDIRECTORY(x));
	shell::system("if test -d ".directory."; then rm ".directory."/????????????????????????????????.mu 2>/dev/null; fi; test 1");
    end_proc;

    resetDirectory :=
    proc()
    begin
	shell::system("if test -d ".dom::directory."; then rm ".dom::directory."/*/????????????????????????????????.mu 2>/dev/null; rmdir ".dom::directory."/* 2>/dev/null; rmdir ".dom::directory." 2>/dev/null; fi; test 1");
    end_proc;


    func_call :=
    proc(x: dom)
	local f, directory, result, fd, arguments;
    begin
	f := GETFUNCTION(x);
	directory := GETDIRECTORY(x);
	arguments := context([args(2..args(0))]);
	if arguments[1] <> hold(Recalculate) then
	    // Lookup the result in the database
	    fd := fopen(dom::filename(directory, op(arguments)));
	    if fd <> FAIL then
		userinfo(5, "Fetching the result from the database");
		result := fread(fd, Quiet);
		fclose(fd);
		return(result);
	    end_if;
	    userinfo(5, "Could not read result from the database; recalculating");
	else
	    delete(arguments[1]);
	end_if;
	result := f(op(arguments));
	// Store the result
	fprint(Unquoted, Text, dom::filename(directory, op(arguments)),
	       expr2text(result));
	result;
    end_proc;

end_domain:
