/*****************************************************************************
  $Id: asciiArt.mu,v 1.1.1.5 2004/10/16 15:20:47 nthiery Exp $
  File:		lib/OUTPUT/asciiArt.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Created:	02/07/2002
  License:	LGPL
*****************************************************************************/

domain output::asciiArt
    inherits Dom::BaseDomain;
    category Cat::BaseCategory;

    // See the doc for the internal data structure

    interfaceAdd :=
    {hold(fromArray),
     hold(height), hold(width),
     hold(alignment),
     hold(setAlignment),
     hold(flipx), hold(flipy)//, hold(transpose)
    };

    //////////////////////////////////////////////////////////////////////////
    // Constructors
    //////////////////////////////////////////////////////////////////////////

    typeHorizontalAlignment := Type::Union(Type::Singleton(Left),
					   Type::Singleton(Center),
					   Type::Singleton(Right));
    typeVerticalAlignment   := Type::Union(Type::Singleton(Top),
					   Type::Singleton(Center),
					   Type::Singleton(Bottom),
					   Type::PosInt);

    new :=
    proc(list = [] : Type::ListOf(DOM_STRING),
	 hAlignment=Left: dom::typeHorizontalAlignment,
	 vAlignment=Center: dom::typeVerticalAlignment
	 )
	local s;
    begin
	dom::setAlignment
	    (new(dom,
		 map(list, stringlib::format, max(0, length(s) $ s in list), hAlignment),
		 NIL),
	     vAlignment);
    end_proc;

    // to please Cat::BaseCategory
    convert := dom::new;
    expr := drawing -> extop(drawing,1);
  
    fromArray :=
    funcenv
    (prog::bless
     (
    proc(M: DOM_ARRAY, separator = FAIL: Type::Union(DOM_FAIL, DOM_ARRAY, Type::Option) ) : dom
	local N,options, nrows, ncols, row, col, widths, heights, sep,
	      width, height, totalWidth,
	      x, y, drawing, alignment;
    begin
        if domtype(separator) = DOM_ARRAY then
	    sep := TRUE;
	    options := combinat::getOptions(dom::fromArray, 2, Error):
	else
	    sep := FALSE;
	    options := combinat::getOptions(dom::fromArray, 1, Error):
	end_if;
        nrows := 1+op(M, [0,2,2])- op(M, [0,2,1]);
        ncols := 1+op(M, [0,3,2])- op(M, [0,3,1]);
	// Returns the width of the formated content of a cell
	width  := x -> length(x);
	// Returns the height of the formated content of a cell
	height := x -> if x="" then 0 else 1 end_if;
	// convert everything that is not NIL or a string to a string,
	// and compute the heights and widths of the rows and columns
	widths  := [ options[MinWidth ] $ ncols ];
        heights := [ options[MinHeight] $ nrows ];
	//save M
	for row from 1 to nrows do
	    x := 1;
	    for col from 1 to ncols do
		y := 1;
		if contains(M, row, col) then
		    if domtype(M[row,col]) <> DOM_STRING then 
			    M[row,col] := expr2text(M[row,col]);  
		    end_if;
		    widths [col] := max(widths [col], width (M[row,col]));
		    heights[row] := max(heights[row], height(M[row,col]));
		end_if;
	    end_for;
	end_for;
        if options[Compact] or options[Packed] then
	    if max(0,op(heights)) <= 1 then
		options[RowSeparator] := "";
		options[Cross]        := "";
	    end_if;
	    options[HorizontalPadding] := "";
	end_if;
	if options[Packed] then
	    if max(0,op(widths)) <= 1 then
		options[ColumnSeparator] := "";
		options[Cross]           := "";
	    end_if;
	end_if;
	alignment := options[VerticalAlignment];
	if domtype(alignment) = DOM_INT then
	    if alignment < 0 then
		alignment := nrows+1+alignment;
	    end_if;
	    alignment := _plus(1, (nrows+1-alignment) * height(options[RowSeparator]),
			       heights[i] $ i = alignment+1..nrows);
	end_if;
	drawing := dom([], Left, alignment);
        x:=1;
	for col from 1 to ncols do
	    // Compute the y coordinates of the upper left corner of the ascii art
	    y := _plus(op(heights), (nrows+1)*height(options[RowSeparator]));
	    totalWidth := widths[col] + 2 * width(options[HorizontalPadding]);
	    for row from 1 to nrows do   
		if contains(M, row, col) and height(options[RowSeparator]) > 0 then
		    if sep and row<>1 and separator[row-1,col]=separator[row,col]
			then
			drawing[x,y] :=
			_concat(options[Cross],
				options[HorizontalPadding] $ totalWidth,
				options[Cross]);
		    else
			    drawing[x,y] :=
			    _concat(options[Cross],
				    options[RowSeparator] $ totalWidth,
				    options[Cross]);
		    end_if;
		end_if;
		y := y - height(options[RowSeparator]);
		if contains(M, row,col) then
		    // [x,y] are the coordinates of the upper left character
		    // of the cell we are going to fill
		    if sep then
		      if ncols=1 then
		          drawing[x,y] :=
			  _concat(options[ColumnSeparator],
				  options[HorizontalPadding],
				  stringlib::format(M[row,col], widths[col], Center),
				  options[HorizontalPadding],
		                  options[ColumnSeparator]);
		      elif col=1 and separator[row,col+1]=separator[row,col] then
			  drawing[x,y] :=
			  _concat(options[ColumnSeparator],
				options[HorizontalPadding],
				stringlib::format(M[row,col], widths[col], Center),
				options[HorizontalPadding],
				options[HorizontalPadding]);
		      elif col=ncols and separator[row,col]=separator[row,col-1] then
			drawing[x,y] :=
			_concat(options[HorizontalPadding],
				options[HorizontalPadding],
				stringlib::format(M[row,col], widths[col], Center),
				options[HorizontalPadding],
				options[ColumnSeparator]);
		      elif col <> ncols and col<>1 and separator[row,col+1]=separator[row,col] 
									    and separator[row,col]<>separator[row,col-1] 
			then
			drawing[x,y] :=
			_concat(options[ColumnSeparator],
				options[HorizontalPadding],
				stringlib::format(M[row,col], widths[col], Center),
				options[HorizontalPadding],
				options[HorizontalPadding]);
		      elif col <> ncols and col<> 1 and separator[row,col]=separator[row,col-1] 
					and separator[row,col]<>separator[row,col+1]
			then
			drawing[x,y] :=
			_concat(options[HorizontalPadding],
				options[HorizontalPadding],
				stringlib::format(M[row,col], widths[col], Center),
				options[HorizontalPadding],
				options[ColumnSeparator]);
		      elif col<> ncols and col<>1 and separator[row,col-1]=separator[row,col] 
				       and separator[row,col+1]=separator[row,col]
			then
			drawing[x,y] :=
			_concat(options[HorizontalPadding],
				options[HorizontalPadding],
				stringlib::format(M[row,col], widths[col], Center),
				options[HorizontalPadding],
				options[HorizontalPadding]);
		      else
			  drawing[x,y] :=
			  _concat(options[ColumnSeparator],
				options[HorizontalPadding],
				stringlib::format(M[row,col], widths[col], Center),
				options[HorizontalPadding],
				options[ColumnSeparator]);
		      end_if;
		    else
		      drawing[x,y] := 
		      _concat(options[ColumnSeparator],
				options[HorizontalPadding],
				stringlib::format(M[row,col], widths[col], Center),
				options[HorizontalPadding],
			      options[ColumnSeparator]);
		    end_if;  
		      
		end_if;
		y := y - heights[row];
		if contains(M, row, col) and height(options[RowSeparator]) > 0 then
		    drawing[x,y] :=
		    _concat(options[Cross],
			    options[RowSeparator] $ totalWidth,
			    options[Cross]);
		end_if;
	    end_for;
	    x := x + width(options[ColumnSeparator]) + totalWidth;
	end_for;
	drawing;
    end_proc,
      dom),
     FAIL,
     table("options" = [MinHeight = 1,
			MinWidth  = 1,
			Compact = FALSE,
			Packed = FALSE,
			ColumnSeparator = "|",
			RowSeparator = "-",
			HorizontalPadding = " ",
			Cross = "+",
			VerticalAlignment=Center // Top, Center, Bottom, or
			// a row number:  1,  2, ...: rows from top to bottom;
			//               -1, -2, ...: rows from bottom to top
			]));
  
  
    
    //////////////////////////////////////////////////////////////////////////
    // Display
    //////////////////////////////////////////////////////////////////////////
    // avoid typesetting
    Content := () -> error("not yet implemented");
  
    print := 
    proc(drawing: dom) : DOM_STRING
	local s, i, baseline;
    begin
	baseline := extop(drawing,2);
	drawing  := extop(drawing,1);
	if nops(drawing) = 0 then
	    return("");
	end_if;
	%if version() = [3,0,0] then
	    // Work around kernel crash with MuPAD 3.0.0 when pretty printing "\n"
	    // See MuPAD Bug#: 1395
	    // I don't see a way to make a feature test instead
	    if drawing[1] = "" then
		drawing := [" " $ nops(drawing) ];
	    end_if;
	end_if;
	s := "";
	for i from nops(drawing) downto 1 do
	    s := s.drawing[i]."\n";
	end_for;
	if op(version(),1)>=3 and baseline <> Center then
	    case baseline
	      of Top    do baseline := nops(drawing); break
	      of Bottom do baseline := 1; break;
	    end_case;
	    %if "12"[1]="1" then // work around a change in strings indexed 
			         // access from MuPAD 3.0.0 on
		s[(nops(drawing)+1-baseline)*(length(drawing[1])+1)] := 
		text2expr("\"\\b\"");
	    else
		s[(nops(drawing)+1-baseline)*(length(drawing[1])+1)-1] := 
		text2expr("\"\\b\"");
	        // this is equivalent to "\b", but avoids a warning
	        // with MuPAD <= 2.5.1
	    end_if;
	end_if;
	// removes the trailing \n if there is one
	%if "12"[1]="1" then
	    if s[length(s)]="\n" then
		s:=substring(s, 1..length(s));
	    end_if;
	else
	    if s[length(s)-1]="\n" then
		s:=substring(s, 0..(length(s)-1));
	    end_if;
	end_if;
	s;
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // Accessors
    //////////////////////////////////////////////////////////////////////////
    
    width :=
    proc(drawing: dom) : Type::NonNegInt
    begin
	drawing := extop(drawing,1);
	if nops(drawing)=0 then 
	    0
	else
	    length(drawing[1]);
	end_if;
    end_proc;

    height :=
    proc(drawing: dom) : Type::NonNegInt
    begin
	nops(extop(drawing,1));
    end_proc;
    
    //////////////////////////////////////////////////////////////////////////
    // Indexed access
    //////////////////////////////////////////////////////////////////////////

    _index :=
    proc(drawing: dom, x: Type::PosInt, y: Type::PosInt) : DOM_STRING
    begin
	if testargs() then
	    if y>dom::height(drawing) or x>dom::width(drawing) then
		error("Out of range");
	    end_if;
	end_if;
	%if "12"[1]="1" then
	    // Indexed access for strings changed with MuPAD 3.0.0
	    extop(drawing,1)[y][x]
	else
	    extop(drawing,1)[y][x-1]
	end_if;
    end_proc;

    set_index :=
    proc(drawing: dom, x: Type::PosInt, y: Type::PosInt, v: DOM_STRING) : dom
	local lines, width, height, xmax, s, i;
    begin
	width  := dom::width(drawing);
	height := dom::height(drawing);
	lines := extop(drawing,1);
	xmax := x+length(v)-1;
	if xmax>width then
	    lines := map(lines, s -> _concat(s, " " $(xmax-width)));
	    width := xmax;
	end_if;
	if y>height then
	    lines := append(lines, _concat("", " " $ width) $ y-height);
	end_if;
	s := lines[y];
	%if "12"[1]="1" then // work around a change in strings indexed 
	                   // access from MuPAD 3.0.0 on
	    for i from 1 to length(v) do
		s[x+i-1] := v[i];
	    end_for;
	else
	    for i from 0 to length(v)-1 do
		s[x+i-1] := v[i];
	    end_for;
	end_if;
	lines[y] := s;
	subsop(drawing, 1=lines);
    end_proc;

    alignment :=
    proc(drawing: dom)
    begin
	extop(drawing, 2);
    end_proc;
    
    setAlignment :=
    proc(drawing: dom, vAlignment: dom::typeVerticalAlignment)
    begin
	if _lazy_and(domtype(vAlignment)=DOM_INT, 
		     nops(extop(drawing,1)) < vAlignment) then
	    // Ensure that drawing has at least vAlignment rows
	    drawing[1,vAlignment]:=""; 
	end_if;
	subsop(drawing, 2=vAlignment);
    end_proc;
    
    //////////////////////////////////////////////////////////////////////////
    // Methods
    //////////////////////////////////////////////////////////////////////////

    expr2text :=
    proc(drawing: dom)
    begin
    	"new(".expr2text(dom).",".expr2text(extop(drawing)).")";
    end_proc;
    
    revert := dom::flipy;

    flipy :=
    proc(drawing: dom) : dom
    begin
	subsop(drawing, 1=revert(extop(drawing,1)));
    end_proc;
    
    flipx :=
    proc(drawing: dom) : dom
    begin
	subsop(drawing, 1=map(extop(drawing,1), revert));
    end_proc;

    
    // halfturn

    transpose :=
    proc(drawing: dom) : dom
    begin
	subsop(drawing, 1=map(revert(extop(drawing,1)), revert));
    end_proc;


    // rotate, ...

    concat :=
    funcenv(prog::bless(
    proc()
	local n, // number of drawings to concatenate
	      drawings, bottoms, tops, height, heights, widths, contents,
	      leftSpaces, rightSpaces, smooth, options, remove, spaces,
	      theLeftSpaces, theRightSpaces,
	      h, i, rLeft, rRight, y;
    begin
	for n from 1 to args(0) do
	    if domtype(args(n)) <> dom then
		break;
	    end;
	end_for;
	n := n-1;
	options := combinat::getOptions(dom::concat, n, Error);
	assert(n>=1);
	drawings := [args(1..n)];
	heights  := map(drawings, dom::height);
	widths   := map(drawings, dom::width);
	height   := max(op(heights));
	if options[Align]=Top then
	    bottoms := [ height-h $ h in heights];
	    tops    := [ height $ n];
	elif options[Align]=Bottom then
	    bottoms := [ 0 $ n ];
	    tops    := heights;
	end_if;
	contents := map(drawings, extop, 1);
	// Complete the contents so that they all have the same vertical size
	contents := [[_concat("", " " $ widths[i]) $ bottoms[i],
		      op(contents[i]),
		      _concat("", " " $ widths[i]) $ height-tops[i]
		     ]
		     $ i = 1..nops(contents)
		    ];
	if options[Packed] then
	// Counts the number of spaces on the left of the string
	leftSpaces :=
	proc(s: DOM_STRING)
	    local i;
	begin
	    for i from 1 to length(s) do
		if s[%if "12"[1]="1" then i else i-1 end] <> " " then break; end_if;
	    end_for;
	    i := i - 1;
	    // TODO
	    // if i=length(s) then infinity else i end_if;
	end_proc;
	// Counts the number of spaces on the right of the string
	rightSpaces :=
	proc(s: DOM_STRING)
	    local i;
	begin
	    for i from length(s) downto 1 do
		if s[%if "12"[1]="1" then i else i-1 end] <> " " then break; end_if;
	    end_for;
	    i := length(s) - i;
	    // TODO
	    // if i = length(s) then infinity else i end_if;
	end_proc;
	// Takes a list, and replaces each element by the minimum of itself,
	// the previous and the next element.
	smooth := l -> [min(if i>1 then l[i-1] else null() end, l[i], if i<nops(l) then l[i+1] else null(); end)
		   $ i = 1..nops(l)];
	// The number of spaces on the left and right of each column separation
	theRightSpaces := [smooth(map(contents[i], rightSpaces)) $ i = 1..n-1 ];
	theLeftSpaces  := [map(contents[i], leftSpaces ) $ i = 2..n  ];
	spaces         := zip(theRightSpaces, theLeftSpaces, (x,y)->zip(x,y, _plus));
	// The number of spaces we want to remove for each column separation
	remove         := map(spaces,
			      l -> if nops(l)=0 then
				       0
				   else
				       min(options[MaxRemove], op(l));
				   end_if);
	for i from 1 to n-1 do
	    if remove[i] > 0 then
		for y from 1 to height do
		    // number of spaces to remove on the right of the column
		    rRight := min(theRightSpaces[i][y], remove[i]);
		    // number of spaces to remove on the left of the next column
		    rLeft  := remove[i] - rRight;
		    // TODO: so far, this should never happen
		    // if r > widths[i] then // this cell is necessarily empty
		    contents[i][y]   := substring(contents[i][y],
						  %if "12"[1]="1" then
						      1..widths[i]-rRight
						  else
						      0..widths[i]-rRight-1
						  end_if);
		    contents[i+1][y] := substring(contents[i+1][y],
						  %if "12"[1]="1" then
						      rLeft+1..widths[i+1]
						  else
						      rLeft..widths[i+1]-1
						  end_if);
		end_for;
	    end_if;
	end_for;
	end_if;
	new(dom,
	    [
	     _concat((contents[i][y], " " $ options[Margin]) $ i = 1..n-1,
		     contents[n][y])
	     $ y=1..height
	    ],
	    extop(drawings[1],2));
    end_proc,
		dom),
	    FAIL,
	    table("options" = [Margin = 0,
			       Packed = FALSE,
			       MaxRemove = infinity,
			       Align  = Bottom
			      ]));


end_domain:
