/*****************************************************************************
  $Id: loaddomain.mu,v 1.1.1.3 2005/01/26 13:42:22 nthiery Exp $
  File:         lib/PROG/loaddomain.mu
  Authors:      Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Created:      03/11/2003
  License:      LGPL
*****************************************************************************/

/**
 * Problem:
 * If:
 *  - The domain blah is already defined before loading the package
 *    Combinat and has been used.
 *  - The package Combinat is loaded, and blah is redefined using loadproc
 *  - blah is used again
 * Then, the file blah.mu will be reread a second time
 * For things like functions, this is not an issue since one is allowed
 * to define them twice. However, for domains, categories, domain
 * constructors and such, this will raise an error.
 *
 * The purpose of the loaddomain function below is to work around
 * this problem.
 * Caveat: If the domain has already been used before loading the package, the
 * (potentially better) definition in the package will be silently ignored
 *
 * The call:
 *	blah := loaddomain(blah, "...", "blah")
 * is essentially equivalent to:
 *	blah := loadproc  (blah, "...", "blah")
 *
 * however, if blah already exists and has been loaded
 * (that is if blah is neither an identifier nor a loadproc object),
 * then blah is returned as is, without recreating a new loadproc.
*/

// FIXME: not tested!!!!

// The feature that we need might in fact be prog::isRedefinable(object),
// which tests if object is already a domain, category, ...

prog::isDefined :=
proc(object: Type::Union(DOM_EXPR, DOM_IDENT, DOM_VAR))
    option hold;
    local parent, entry, value;
begin
    if domtype(object) = DOM_VAR then
        // Recursive call (hopefully!)
	object := context(hold(val)(object));
    end_if;
    if domtype(object)=DOM_IDENT then
	// Trick to get the value of 'object', without triggering
	// a possible loadproc
	value := context(hold(val)(object));
	// return TRUE if object is not an identifier and not
	// a non evaluated loadproc
	return(_lazy_and(object <> value,
			 domtype(val(value)) <> stdlib::LoadProc))
    else
	// object has the form parent::entry
	parent    := op(object,1);
	if not prog::isDefined(parent) then return(FALSE); end_if;
	// parent is defined; we evaluate it fully
	parent    := context(parent);
	entry     := op(object,2);
	// Trick to get the value of 'object', without triggering
	// a possible loadproc of parent::entry
	// If entry was not yet inserted from parent's constructor,
	// the line below will set value to FAIL.
	value := extop(select(level(extop(parent), 1), x->op(x,1)=entry), 2);
	// return TRUE if object is FAIL or a non evaluated loadproc
	return(_lazy_and(value <> FAIL,
			 domtype(val(value)) <> stdlib::LoadProc));
    end;
end:

prog::loaddomain :=
proc(object: Type::Union(DOM_EXPR, DOM_IDENT),
     path, // : DOM_STRING beware of the option hold!
     filename: DOM_STRING)
    option hold;
    local parent, entry, value, override;
begin
    if domtype(object)=DOM_IDENT then
	// Trick to get the value of 'object', without triggering
	// a possible loadproc
	value := context(hold(val)(object));
	// set override to TRUE if object is an identifier or a
	// non evaluated loadproc
	override := _lazy_or(domtype(val(value)) = stdlib::LoadProc,
			     object=value);
    else
	// object has the form parent::entry
	// Note: the following line may trigger the loadproc of parent!
	parent    := context(op(object,1));
	entry     := op(object,2);
	// Trick to get the value of 'object', without triggering
	// a possible loadproc of parent::entry
	// If entry was not yet inserted from parent's constructor,
	// the line below will set value to FAIL.
	value := extop(select(level(extop(parent), 1), x->op(x,1)=entry), 2);
	// set override to TRUE if object is FAIL or a non evaluated loadproc
	override := _lazy_or(value=FAIL,
			     domtype(val(value)) = stdlib::LoadProc);
    end;
    if override then
	context(hold(loadproc)(object, op([args(2..args(0))])));
    else
	warning("Cannot override the old version of ".expr2text(object)." because it has already been loaded; some features may be non functional.");
	context(object);
    end;
end:
