/*****************************************************************************
  $Id: testUnit.mu,v 1.1.1.3 2005/01/26 13:42:22 nthiery Exp $
  File:		lib/PROG/testUnit.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Purpose:	Source code for the domain prog::testUnit
  Created:	Thu Apr 29 09:31:47 2004
  License:	LGPL
  Project:	MuPAD-Combinat, http://mupad-combinat.sf.net
*****************************************************************************/

domain prog::testUnit

    interface := { hold(sourceDir), hold(runTests) };
    info_str := "prog::testUnit -- extensions a la xUnit to the MuPAD test framework";

    // Try to guess the name of a MuPAD object
    nameOf :=
    proc(x: Type::AnyType) : DOM_STRING
    begin
	if   domtype(x) = DOM_STRING then x
	elif domtype(x) = DOM_PROC   then expr2text(extop(x, 6))
	else
	    expr2text(x);
	end_if;
    end_proc;
    
    split :=
    if stringlib::split <> FAIL then
	s -> stringlib::split(s, "::");
    else // MuPAD 2.5.*
	proc(s: DOM_STRING) : Type::ListOf(DOM_STRING)
	    local pos, newpos, result;
	begin
	    if s="" then return([""]); end_if; // stringlib::pos requires a non empty string ...
	    pos := 0;
	    result := [];
	    while (newpos := stringlib::pos(s, "::", pos)) <> FAIL do
		result := append(result, substring(s, pos..newpos-1));
		pos := newpos+2;
	    end_while;
	    result := append(result, substring(s, pos..length(s)-1));
	end_proc:
    end_if;
    
    top :=
    proc(x: DOM_STRING) //: Type::Sequence(DOM_STRING $ 3)
    begin
	dom::split(x, "::")[1];
    end_proc;

    subTop :=
    proc(x: DOM_STRING) //: Type::Sequence(DOM_STRING $ 3)
	local l;
    begin
	l := dom::split(x, "::");
	if nops(l)>=2 then l[2] else ""; end_if;
    end_proc;

    sourceDir :=
    proc(x: DOM_STRING) : DOM_STRING
	local top;
    begin
	top := dom::top(x);
	if top = "Dom" then
	    "DOMAINS/DOMAIN/"
	elif top = "Cat" then
	    "DOMAINS/CATEGORY/"
	elif top = "Ax" then
	    "DOMAINS/AXIOM/"
	else
	    stringlib::upper(top)."/";
	end_if;
    end_proc;

    testFile :=
    proc(x: Type::AnyType) : DOM_STRING
	local subTop;
    begin
	x := dom::nameOf(x);
	subTop := dom::subTop(x);
	dom::sourceDir(x)."TEST/".if subTop<>"" then subTop else dom::top(x) end_if.".tst";
    end_proc;

    runTests :=
    proc(x: Type::AnyType)
	local oldwarn, oldcheck, oldtest;
    begin
	x := dom::nameOf(x);
	oldwarn  := Pref::warnDeadProcEnv(TRUE):
	oldcheck := Pref::typeCheck(Always):
	oldtest  := testargs(TRUE):
	sysassign(prog::testOrig,     prog::test);
	sysassign(prog::testinitOrig, prog::testinit);
	sysassign(prog::testexitOrig, prog::testexit);
	sysassign(prog::testfuncOrig, prog::testfunc);
	sysassign(prog::test, dom::testNoTrap);
	sysassign(prog::testinit, id);
	sysassign(prog::testexit, id);
	sysassign(prog::testfunc, proc(s) begin fprint(Unquoted, 0, "Testing: ", s); end_proc);

	// Run the tests
	read(dom::testFile(x), Plain);

	// Reset the original value
	Pref::warnDeadProcEnv(oldwarn);
	Pref::typeCheck(oldcheck):
	testargs(oldtest):
	sysassign(prog::test, dom::testOrig);
	sysassign(prog::testinit, dom::testinitOrig);
	sysassign(prog::testexit, dom::testexitOrig);
	sysassign(prog::testfunc, dom::testfuncOrig);
	null():
    end_proc;


    testNoTrap :=
    proc()
	local res1, res2;
	option hold;
    begin
	if args(0)=1 then
	    res1:=context(args(1));
	    if res1<>TRUE then
		fprint(Unquoted, 0, "error: ", args(1), " should yield TRUE and not ", res1);
	    end_if;
	elif args(0)=2 then
	    if op(args(2),1)=hold(TrapError) then
		fprint(Unquoted, 0, "test for ", args(2), " ignored"):
	    else
		res1:=context(args(1));
		res2:=context(args(2));
		if res1<>res2 then
		    fprint(Unquoted, 0, "error: ", args(1), " should yield ", res2, "and not ", res1);
		end_if;
	    end_if;
	end_if;
    end_proc;

end_domain:
