/*****************************************************************************
  $Id: mupad.tst,v 1.1.1.4 2005/01/26 13:42:23 nthiery Exp $
  File:		lib/mupad.tst
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Purpose:	Test file for the library of MuPAD
  Created:	03/03/2003
  License:	LGPL
*****************************************************************************/

// We test here some features of the MuPAD library, and in particular
// our bug fixes to the former versions of MuPAD.

prog::testinit("mupad"):

/**
   Testing that (Dom::ExpressionField())::equal works properly
   (buggy in plain MuPAD < 3.0.0)
**/
prog::testfunc((Dom::ExpressionField())::equal):

prog::test(bool((Dom::ExpressionField())::equal(f(1,x),f(1,y))<>TRUE)):
prog::test(bool((Dom::ExpressionField())::equal(a[1],  a[2]  )<>TRUE)):

prog::testfunc(Dom::ArithmeticalExpression::convert):
prog::test(testtype(a([1]), Dom::ArithmeticalExpression)):

prog::testfunc((Dom::ExpressionField())::convert):
prog::test(Dom::ExpressionField()(hold(f)(poly(x))), hold(f)(poly(x))):

// Add some tests for Graph

/**
  Testing how to hack an entry of an existing domain constructor
  Used in operators::overloaded to add an initDomain entry to
  Dom::IntegerMod, and in some other places as well
**/
prog::testfunc("constructor hacking"):

domain A()
    x:=2;
begin
end_domain:

slot(extop(A,4),"y",1):

B:=A():
prog::test(B::y, 1):
prog::test(B::x, 2):

//////////////////////////////////////////////////////////////////////////////
// Testing that the initDomain method of a domain is systematically
// called upon creation of this domain
//////////////////////////////////////////////////////////////////////////////
prog::testfunc("initDomain feature"):

domain B()
    initDomain := () -> (called :=TRUE);
end_domain:
called := FALSE:
B():
prog::test(called):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("Inheritance does not bless expressions bla@ble"):
//////////////////////////////////////////////////////////////////////////////

domain bla1
   bli := proc() begin dom end;
end:
domain bla2
   bli := bla1::bli @ id;
end:
prog::test(bla2::bli(), bla1):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("Type::PolyExpr accepts a(1) as variable"):
//////////////////////////////////////////////////////////////////////////////
prog::test(solvelib::solve_islinear([a(1)+3*a(3)], [a(1), a(3)]),
	   [a(1)+3*a(3)]):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("Type::Function"):
//////////////////////////////////////////////////////////////////////////////
// test that overloaded operators are considered as functions
prog::test(testtype(operators::plus2, Type::Function)):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("multcoeffs and mapcoeffs work for all kinds of matrices"):
//////////////////////////////////////////////////////////////////////////////

for matrixdomain in [matrix, sparsematrix, densematrix] do
    prog::test(multcoeffs(matrixdomain(2,2,[[a,b],[c,d]]),2),
	       matrixdomain(2,2,[[2*a,2*b],[2*c,2*d]]));
end_for:

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("Dom::SquareMatrix is a working free module"):
//////////////////////////////////////////////////////////////////////////////
m:=(Dom::SquareMatrix(3))::one:
prog::test(poly2list(m),
           [[1, [1, 1]], [1, [2, 2]], [1, [3, 3]]]):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("poly2list works and can be overloaded"):
//////////////////////////////////////////////////////////////////////////////

prog::test(poly2list(poly(x + y, [x, y])),
	   [[1, [1, 0]], [1, [0, 1]]]):
prog::test(poly2list(Dom::DistributedPolynomial([x, y], Dom::Rational)(x + y)),
           [[1, [1, 0]], [1, [0, 1]]]):
// see also test above

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("linalg::matlinsolveLU accepts the permutation as argument"):
//////////////////////////////////////////////////////////////////////////////
M := sparsematrix(3,3,[[0,1,0],[1,0,0],[0,0,1]]):
b := sparsematrix([a1,a2,a3]):
[L,U,p]:=linalg::factorLU(M):
prog::test(M*linalg::matlinsolveLU(L,U,p,b), b):

delete(b):
delete(M):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("Dom::SparseMatrix works with semi-rings"):
//////////////////////////////////////////////////////////////////////////////

B := Dom::BooleanSemiRing:
M := Dom::Matrix(B):
m := M(2,2):
prog::test((m[1,1] := B(1)), B(1)):
prog::test(m[1,1], B(1)):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("Substitutions of function environments and generators"):
//////////////////////////////////////////////////////////////////////////////

makeGenerator :=
proc()
    option escape;
    local i;
begin
    i := 0;
    proc() begin i := i+1 end;
end:
g1 := makeGenerator():
g2 := makeGenerator():
prog::test([g1() $ i = 1..3],
           [1, 2, 3]):
prog::test([g1() $ i = 1..1],
           [4]):
g1 := makeGenerator():
g2 := makeGenerator():
prog::test([g1() $ i = 1..5],
           [1, 2, 3, 4, 5]):
prog::test([g2() $ i = 1..2],
           [1, 2]):
g2 := extsubsop(g2, 12=extop(g1,12)):
prog::test([g2() $ i = 1..2],
           [6, 7]):
prog::test([g1() $ i = 1..2],
           [8, 9]):

makePairOfGenerator :=
proc()
    option escape;
    local i;
begin
    i := 0;
    [proc() begin i := i+1 end, proc() begin i := i-1 end]
end:
[f,g] := makePairOfGenerator():
prog::test([f() $ i = 1..5, g() $ i = 1..3],
           [1, 2, 3, 4, 5, 4, 3, 2]):
[f,g] := makePairOfGenerator():
prog::test([f() $ i = 1..5],
           [1, 2, 3, 4, 5]):
prog::test([g() $ i = 1..2],
           [4, 3]):
prog::test([f() $ i = 1..5],
           [4, 5, 6, 7, 8]):

// Temptative hack to copy both procedures at the same time
// while keeping the same environment
f1 := copyClosure(f):
g1 := extsubsop(g, 12=extop(f1,12)):

// Now, the two pairs f,g and f1,g1 should be independent
prog::test([f() $ i = 1..5],
           [9, 10, 11, 12, 13]):
prog::test([g() $ i = 1..2],
           [12, 11]):
prog::test([f1() $ i = 1..5],
           [9, 10, 11, 12, 13]):
prog::test([g1() $ i = 1..2],
           [12, 11]):

prog::test([f() $ i = 1..10],
           [12, 13, 14, 15, 16, 17, 18, 19, 20, 21]):
prog::test([g() $ i = 1..3],
           [20, 19, 18]):

prog::test([f1() $ i = 1..2],
           [12, 13]):
prog::test([g1() $ i = 1..2],
           [12, 11]):

prog::test([f() $ i = 1..10],
           [19, 20, 21, 22, 23, 24, 25, 26, 27, 28]):
prog::test([g() $ i = 1..3],
           [27, 26, 25]):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("text2expr does not evaluate nor simplify the resulting expression"):
//////////////////////////////////////////////////////////////////////////////

proc()
    local string, x;
begin
    string := "a+a+2*a^s + (b+b) + s(1/2) + a*4 + a * (a + 2 * (b+c) * 3) ":
    x := text2expr(string);
    prog::test([op(op(x, 6))],
	       [a, 4]):
    prog::test([op(op(x, 3))],
	       [2, a^s]):
    prog::test([op(op(x, [3, 2]))],
	       [a, s]):
    prog::test([op(op(x, 4))],
	       [b, b]):
end_proc():

prog::testexit():

// Local Variables:
// mode: mupad
// End:
