//////////////////////////////////////////////////////////////////////////////
// $Id: MAPITL.tst,v 1.1.1.2 2004/10/16 15:11:32 nthiery Exp $
// Name:	MAPITL_test.cc
// Purpose:	Template Library for the
//		MuPAD Application Programming Interface
//		(Test suite, MuPAD part)
// Author:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
// Created:	2000/09
// Copyright:	LGPL
//////////////////////////////////////////////////////////////////////////////

TestPrecision:=14:	// Precision for floating point tests
fprint(Unquoted,0,"Warning: making floating point tests with only ", TestPrecision, " decimals."):
fequal :=
proc(x,y) begin
   return(bool(abs(x-y)<float(10^(-TestPrecision))));
end_proc:

export(prog, test):
prog::testinit("MAPITL"):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("Module loading"):
//////////////////////////////////////////////////////////////////////////////

module(MAPITL_test):
test(bool(MAPITL_test::doc<>FAIL)):
alias(M=MAPITL_test):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("Cell"):
//////////////////////////////////////////////////////////////////////////////

test(M::test_Cell_cell      (1), 2):
test(M::test_Cell_bool      (FALSE), TRUE):
test(M::test_Cell_int       (1), 2):
test(M::test_Cell_short_int (1), 2):
test(M::test_Cell_long      (1), 2):
test(M::test_Cell_string("bla"), "bla1"):
test(M::test_Cell_pair(1,2), [2,3]):
test(M::test_Cell_index([a,b,c],2), b):
test(M::test_Cell_at([a,b,c],2),    b):
test(M::test_Cell_atTyped([1,2,3],2), 2.0):
test(M::test_Cell_atAssign     ([1,2,3], 2, 1.5), [1,1.5,3]):
test(M::test_Cell_atAssignTyped([1,2,3], 2, 1.5), [1,1.5,3]):
test(M::test_Cell_indexAssign  ([1,2,3], 2, 1.5), [1,1.5,3]):
test(M::test_Cell_toArray([7,8,9],2), 8):
test(M::test_Cell_iterator([5,3,1,7]), [5,3,1,7]):
test(M::test_Cell_iteratorWrite([5, 3, 1, 7], [a, b, c, d, e]),
     [5, 3, 1, 7, e]):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("LongCell"):
//////////////////////////////////////////////////////////////////////////////

test(MAPITL_test::LongCellNew(1), TrapError=1028):

domain LongCell
    inherits Dom::BaseDomain;
    category Cat::AbelianGroup;

    zero    := dom::new(0);

    new     := MAPITL_test::LongCellNew;
    expr    := MAPITL_test::LongCellExpr;
    _plus   := MAPITL_test::LongCell_plus;
    _negate := MAPITL_test::LongCell_negate;
end_domain:

bla := LongCell(1):
test(domtype(bla), LongCell):
test(expr(bla), 1):
test(expr(bla+bla), 2):
if version()[1]<3 or (version()[1]=3 and version()[2]<1) then
    // Known to fail with MuPAD < 3.1.0
    fprint(Unquoted,0,"Warning: skipping inequality tests for ObjectInCells"):
else
    test(bool(LongCell(1) =  LongCell(1))):
    test(bool(LongCell(1) <> LongCell(2))):
end_if:

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("ByteArrayInCell"):
//////////////////////////////////////////////////////////////////////////////

domain ByteArrayInCell
    inherits Dom::BaseDomain;
    category Cat::AbelianGroup;

    zero    := dom::new(0);

    new     := MAPITL_test::ByteArrayInCellNew;
    expr    := MAPITL_test::ByteArrayInCellExpr;
    op      := MAPITL_test::ByteArrayInCellOp;
    _index  := MAPITL_test::ByteArrayInCellOp;
    nops    := MAPITL_test::ByteArrayInCellNops;
end_domain:
bla := ByteArrayInCell([4,3,5,1,2]):
test(domtype(bla), ByteArrayInCell):
test(nops(bla), 5):
test(expr(bla), [4,3,5,1,2]):
test(bla[1],4):
test(bla[2],3):
if version()[1]<3 or (version()[1]=3 and version()[2]<1) then
    // Known to fail with MuPAD < 3.1.0
    fprint(Unquoted,0,"Warning: skipping inequality tests for ObjectInCells"):
else
    test(bool(ByteArrayInCell([1,2]) =  ByteArrayInCell([1,2]))):
    test(bool(ByteArrayInCell([2,1]) <> ByteArrayInCell([1,2]))):
end_if:

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("Conversions of boolean types"):
//////////////////////////////////////////////////////////////////////////////

test(M::test_bool(FALSE), FALSE):
test(M::test_bool(TRUE ), TRUE ):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("Conversions of integer types"):
//////////////////////////////////////////////////////////////////////////////

test(M::test_int(1),2):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("Conversions of float types"):
//////////////////////////////////////////////////////////////////////////////

test(fequal(M::test_float (4  ),5  )):
test(fequal(M::test_float (4.5),5.5)):
test(fequal(M::test_double(4  ),5  )):
test(fequal(M::test_double(4.5),5.5)):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("Conversions of pointers"):
//////////////////////////////////////////////////////////////////////////////

// test(M::test_pointer(18),???):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("Conversions of strings"):
//////////////////////////////////////////////////////////////////////////////

test(M::test_string		   ("Coucou"),		"uocuoC"):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("Conversions of containers"):
//////////////////////////////////////////////////////////////////////////////

test(M::test_iterator		   (),			[0,1,2,3,4]):
test(M::test_pair		   ([1,"bla"]),		[2, "alb"]):
test(M::test_list		   ([1,2,3,4]),		[4,3,2,1]):
test(M::test_list2		   ([1,2,3,4]),		[4,3,2,1]):
test(M::test_listlist		   ([[1,2],[3,4]]),	[[3,4],[1,2]]):

//////////////////////////////////////////////////////////////////////////////
prog::testfunc("Container"):
//////////////////////////////////////////////////////////////////////////////

test(M::test_Container_size	   	([1,2,3,4]),	4):
test(M::test_Container_reference   	([1,2,7,4],3),	7):
test(M::test_Container_subscript_const	([1,2,7,4],3),	7):
test(M::test_Container_subscript	([1,2,7,4],3,5),[1,2,5,4]):
test(M::test_Container_iterator_const	([1,2,7,4]),	[4,7,2,1]):
test(M::test_Container_push_back	([1,2,7,4],3),	[1,2,7,4,3]):
test(M::test_Container_iterator		([1,2,7,4]),	[4,7,2,1]):

// This tests causes an internal failure. Why ?
// M::test_Container_iterator	   ([1]),1:

prog::testexit():

// Local Variables: 
// mode: mupad
// End:
