/////////////////////////////////////////////////////////////////////////////
// $Id: Nauty.tst,v 1.1.1.1 2004/10/04 17:25:09 nthiery Exp $
// Name:        Nauty.tst
// Purpose:     MuPAD dynamic module for the nauty C library
//              (test suite)
// Author:      Florent Hivert <florent.hivert@univ-mlv.fr>
//              Nicolas M. Thiry <nthiery@users.sourceforge.net>
// Created:     2004/04/12
// Copyright:   LGPL
//////////////////////////////////////////////////////////////////////////////

alias(test=prog::test):
//alias(test=print):

prog::testinit("Nauty"):

prog::testfunc("Module creation"):
unprotect(combinat):
domain combinat::graphs
    category Cat::BaseCategory; // pour le printing;
    expr := Nauty::expr;
begin
end_domain:
module(Nauty):


prog::testfunc("Object creation"):
bla := Nauty::new([[],[]]):
test(Nauty::expr(bla), [[],[]]):
for g in [ [], [[]], [[2],[1]], 
	  [[],[],[]], [[],[3],[2]], [[3],[3],[1,2]], [[2,3],[1,3],[1,2]],
	 [[2,3,5], [1,4,5], [1,4,5], [2,3,5], [1,2,3,4]]] do
    test(Nauty::expr(Nauty::new(g)), g):
end_for:

prog::testfunc("Args checking"):
test(Nauty::addEdge(Nauty::new([[],[]]), [1,3]), TrapError =1028):
test(Nauty::removeEdge(Nauty::new([[],[]]), [1,3]), TrapError =1028):
test(Nauty::outDegree(Nauty::new([[3], [3,4], [2,4],[2,3]]), 5), TrapError =1028):

prog::testfunc("Object Listing"):
list := n -> Nauty::list(["geng", expr2text(n), "-l", "-q", "-u"]):

test(map(list(1), Nauty::expr),
           [[[]]]):
test(map(list(2), Nauty::expr),
           [[[], []], [[2], [1]]]):
test({op(map(list(3), Nauty::expr))},
     {[[], [], []], [[], [3], [2]], [[3], [3], [1, 2]], [[2, 3], [1, 3], [1, 2]]}):
test({op(map(list(4), Nauty::expr))},
     {[[], [], [], []], [[], [], [4], [3]], [[2], [1], [4], [3]], 
      [[], [4], [4], [2, 3]], [[3], [4], [1, 4], [2, 3]], 
      [[], [3, 4], [2, 4], [2, 3]], [[4], [4], [4], [1, 2, 3]], 
      [[2, 3], [1, 4], [1, 4], [2, 3]], [[4], [3, 4], [2, 4], [1, 2, 3]], 
      [[3, 4], [3, 4], [1, 2, 4], [1, 2, 3]], [[2, 3, 4], [1, 3, 4], [1, 2, 4], [1, 2, 3]]}):
test({op(map(list(5), Nauty::expr))},
     {[[],[],[],[],[]],
      [[],[],[],[5],[4]],
      [[],[3],[2],[5],[4]],
      [[],[],[5],[5],[3,4]],
      [[5],[5],[4],[3],[1,2]],
      [[],[4],[5],[2,5],[3,4]],
      [[],[],[4,5],[3,5],[3,4]],
      [[],[5],[5],[5],[2,3,4]],
      [[2],[1],[4,5],[3,5],[3,4]],
      [[4],[5],[4,5],[1,3],[2,3]],
      [[],[3,4],[2,5],[2,5],[3,4]],
      [[5],[5],[4],[3,5],[1,2,4]],
      [[5],[5],[5],[5],[1,2,3,4]],
      [[2,3],[1,4],[1,5],[2,5],[3,4]],
      [[],[5],[4,5],[3,5],[2,3,4]],
      [[2],[1,5],[4,5],[3,5],[2,3,4]],
      [[5],[4,5],[4,5],[2,3],[1,2,3]],
      [[4],[5],[4,5],[1,3,5],[2,3,4]],
      [[],[4,5],[4,5],[2,3,5],[2,3,4]],
      [[5],[5],[4,5],[3,5],[1,2,3,4]],
      [[2,4],[1,5],[4,5],[1,3,5],[2,3,4]],
      [[4,5],[4,5],[4,5],[1,2,3],[1,2,3]],
      [[2,5],[1,5],[4,5],[3,5],[1,2,3,4]],
      [[5],[3,4],[2,4,5],[2,3,5],[1,3,4]],
      [[],[3,4,5],[2,4,5],[2,3,5],[2,3,4]],
      [[5],[4,5],[4,5],[2,3,5],[1,2,3,4]],
      [[2,3],[1,4,5],[1,4,5],[2,3,5],[2,3,4]],
      [[3,5],[4,5],[1,4,5],[2,3,5],[1,2,3,4]],
      [[5],[3,4,5],[2,4,5],[2,3,5],[1,2,3,4]],
      [[4,5],[4,5],[4,5],[1,2,3,5],[1,2,3,4]],
      [[2,3,5],[1,4,5],[1,4,5],[2,3,5],[1,2,3,4]],
      [[4,5],[3,4,5],[2,4,5],[1,2,3,5],[1,2,3,4]],
      [[3,4,5],[3,4,5],[1,2,4,5],[1,2,3,5],[1,2,3,4]],
      [[2,3,4,5],[1,3,4,5],[1,2,4,5],[1,2,3,5],[1,2,3,4]]}):

prog::testfunc("Canonise"):
test(Nauty::canonise(Nauty::new([])), Nauty::new([])):
test(Nauty::canonise(Nauty::new([[]])), Nauty::new([[]])):
test(Nauty::canonise(Nauty::new([[1],[2],[]])), Nauty::new([[], [2], [3]])):
test(Nauty::canonise(Nauty::new([[4], [3], [2, 4], [3, 1, 5], [4]])), 
     Nauty::new([[5], [5], [4], [3, 5], [1, 2, 4]])):

prog::testfunc("addEdge"):
test(Nauty::addEdge(Nauty::new([[],[]]), [1,2]), Nauty::new([[2],[1]])):
test(Nauty::addEdge(Nauty::new([[],[],[]]), [1,2]), Nauty::new([[2],[1],[]])):
test(Nauty::addEdge(Nauty::new([[3],[3],[3]]), [1,2]), Nauty::new([[2,3],[1,3],[3]])):

prog::testfunc("removeEdge"):
test(Nauty::removeEdge(Nauty::new([[2],[1]]), [1,2]), Nauty::new([[],[]])):
test(Nauty::removeEdge(Nauty::new([[2],[1],[]]), [1,2]), Nauty::new([[],[],[]])):
test(Nauty::removeEdge(Nauty::new([[2,3],[1,3],[3]]), [1,2]), Nauty::new([[3],[3],[3]])):

prog::testfunc("size"):
test(Nauty::size(Nauty::new([])), 0):
test(Nauty::size(Nauty::new([[2],[1]])), 2):
test(Nauty::size(Nauty::new([[2],[1],[]])), 3):
test(Nauty::size(Nauty::new([[2,3],[1,3],[3]])), 3):

prog::testfunc("getEdgesNumber"):
test(Nauty::getEdgeNumber(Nauty::new([])), 0):
test(Nauty::getEdgeNumber(Nauty::new([[2],[1]])), 2):
test(Nauty::getEdgeNumber(Nauty::new([[2],[1],[]])), 2):
test(Nauty::getEdgeNumber(Nauty::new([[2,3],[1,3],[3]])), 5):

prog::testfunc("outDegree"):
test(Nauty::outDegree(Nauty::new([[2],[1]]),1), 1):
test([Nauty::outDegree(Nauty::new([[2],[1],[]]),i) $ i=1..3], [1,1,0]):
test([Nauty::outDegree(Nauty::new([[2,3],[1,3],[3]]), i) $ i=1..3], [2,2,1]):

prog::testfunc("complement"):
test(Nauty::complement(Nauty::new([])), Nauty::new([])):
test(Nauty::complement(Nauty::new([[2],[1]])), Nauty::new([[],[]])):
test(Nauty::complement(Nauty::new([[2],[1],[]])), Nauty::new([[3], [3], [1, 2]])):
test(Nauty::complement(Nauty::new([[2,3],[1,3],[3]])), Nauty::new([[1], [2], [1, 2]])):

prog::testfunc("converse"):
test(Nauty::converse(Nauty::new([])), Nauty::new([])):
test(Nauty::converse(Nauty::new([[2],[1]])), Nauty::new([[2],[1]])):
test(Nauty::converse(Nauty::new([[2],[1],[]])), Nauty::new([[2], [1], []])):
test(Nauty::converse(Nauty::new([[2,3],[1,3],[3]])), Nauty::new([[2], [1], [1, 2, 3]])):

prog::testfunc("relabel"):
test(Nauty::relabel(Nauty::new([]), []), Nauty::new([])):
test(Nauty::relabel(Nauty::new([[2],[1]]), [1,2]), Nauty::new([[2],[1]])):
test(Nauty::relabel(Nauty::new([[2],[1]]), [2,1]), Nauty::new([[2],[1]])):
test(Nauty::relabel(Nauty::new([[2],[1],[]]), [2,1,3]), Nauty::new([[2],[1],[]])):
test(Nauty::relabel(Nauty::new([[2],[1],[]]), [3,2,1]), Nauty::new([[],[3],[2]])):
test(Nauty::relabel(Nauty::new([[2],[1],[]]), [3,1,2]), Nauty::new([[],[3],[2]])):
test(Nauty::relabel(Nauty::new([[2,3],[1,3],[3],[5],[3,6,4],[1,3,4]]),[3,2,6,4,5,1]),
     Nauty::new([[1], [1, 6], [1, 4, 6], [5], [1, 3, 4], [1, 2]])):

prog::testfunc("sublabel"):
test(Nauty::sublabel(Nauty::new([[2,3],[1,3],[3],[5],[3,6,4],[1,3,4]]),[3,2,6,4]),
     Nauty::new([[1], [1], [1, 4], []])):

prog::testfunc("directSum"):
test(Nauty::directSum(Nauty::new([[2],[1]]),
		      Nauty::new([[]]),
		      Nauty::new([]),
		      Nauty::new([[1]])),
     Nauty::new([[2], [1], [], [4]])):
test(Nauty::directSum(), Nauty::new([])):

prog::testfunc("LandoTilde"):
prog::test(Nauty::LandoTilde(Nauty::new([[3], [3, 4], [1, 2, 4], [2, 3]]), [1, 2]),
           Nauty::new([[4], [3, 4], [2, 4], [1, 2, 3]])):
prog::test(Nauty::LandoTilde(Nauty::new([[3, 5], [3, 4], [1, 2, 4], [2, 3], [1]]), [1, 2]),
           Nauty::new([[4, 5], [3, 4], [2, 4], [1, 2, 3], [1]])):

prog::testexit():
null():

