
// This file "tests" the Scilab package
// Tests means:
//  - Is the module loadable
//  - Is it possible to execute a module function
//  - Call some scilab function() to test the conersion of 
//    the datatypes
//
// It is used by the target test inside the Makefile

prog::testinit("ScilabModule") :
prog::testfunc(Scilab):

// numerical test of relative precision, "stolen" from the 
// stats testfiles
mytest:= proc(ist, soll)
begin
   macheps:= 10^(-DIGITS):
   DIGITS:= 2*DIGITS:
   soll:= float(soll);
   DIGITS:= DIGITS/2:
   if abs(float(ist-soll)) <= macheps*(1 + abs(float(soll))) then
       TRUE
   else print("DIGITS" = DIGITS,
              "[ist, soll]" = [ist, soll],
              "Relative Error" =  abs(ist-soll)/abs(soll)/macheps);
       FALSE
   end_if:
end_proc:

scilab::sqrt(45);

prog::test( mytest(scilab::sqrt(345), 18.57417562) ) ;
prog::test( mytest(scilab::sqrt(3.45), 1.857417562) ) ;
a:=scilab::sqrt(-1) ;
prog::test(a-1.0*I, 0.0) ;
prog::test( mytest(scilab::sqrt(a), 0.7071067812 + 0.7071067812*I) ) ;
prog::test( mytest(scilab::cos(3.4345+3*I), - 9.638864814 + 2.892530748*I) ) ;

// We can also see if Scilab or MuPAD is faster with
// the computation, but with such an easy call like sqrt() the
// overhead of the data conversion is too big
rtime( (for i from 1 to 1000 do scilab::sqrt(i) end_for;) ) ;
rtime( (for i from 1 to 1000 do sqrt(i) end_for;) ) ;


a:="This is" ; b:=" a string" ;
prog::test( scilab::_plus(a,b), "This is a string") ;
// Here one can see that the method scilab::_plus implements the
// Scilab operator '+'
// Ok now an easy example with a list
l:=[1,2,45,"lala",[3,"la"]] ;
prog::test( scilab::size(l), 5.0 ) ;

P:=Dom::UnivariatePolynomial(x,Dom::Float,LexOrder):
p:=P(-10*x+x^2+3*x^5) ;
Mp := Dom::Matrix()(1,6,[0.0, -10.0, 1.0, 0.0, 0.0, 3.0]) ;
Sp := scilab::coeff(p) ;
prog::test( Sp, Mp ) ; 
Sp := scilab::_mult(p,p,p) ;
Mp := Dom::UnivariatePolynomial(x, Dom::ExpressionField())(\
       27.0*x^15 + 27.0*x^12 - 270.0*x^11 + 9.0*x^9 - 180.0*x^8 + 900.0*x^7 +\
        1.0*x^6 - 30.0*x^5 + 300.0*x^4 - 1000.0*x^3) ;
prog::test( Sp, Mp ) ;

// But we can also define a polynomial with the help of scilab
s:=scilab::poly(0,"x") ;
pp:=-10*s+s^2+3*s^5 ;
Sp := scilab::_mult(pp,pp,pp) ;
Mp := Dom::UnivariatePolynomial(x, Dom::ExpressionField())(\
       27.0*x^15 + 27.0*x^12 - 270.0*x^11 + 9.0*x^9 - 180.0*x^8 + 900.0*x^7 +\
        1.0*x^6 - 30.0*x^5 + 300.0*x^4 - 1000.0*x^3) ;
prog::test( Sp, Mp ) ;

A:=Dom::Matrix(Dom::Float)([[23.0, 24.34, 54.0],\
                           [12.0, 23.0, 45455.333],\
                           [212.0,34323.0,342.123]]) ;

prog::test( mytest(scilab::det(A), linalg::det(A)) ) ;
prog::test( mytest(norm(scilab::sqrt(A)-map(A,sqrt)),0.0) ) ;

A:=Dom::DenseMatrix(Dom::Float)([[23.0, 24.34, 54.0],\
                           [12.0, 23.0, 45455.333],\
                           [212.0,34323.0,342.123]]) ;

prog::test( mytest(scilab::det(A), linalg::det(A)) ) ;
prog::test( mytest(norm(scilab::sqrt(A)-map(A,sqrt)),0.0) ) ;

// Creating some other larger matrices
M:=Dom::SquareMatrix(20,Dom::Float):
A:=M::random():
B:=M::random():

// And another random matrix which is created by scilab
// one can see that this is much more faster
C:=scilab::rand(20,20):
rtime(A*C);
rtime(scilab::_mult(A,C)) ;
rtime(A*C) ;
rtime(scilab::det(B)) ;
rtime(linalg::det(B)) ;
prog::test( mytest(scilab::det(B), linalg::det(B)) ) ;
// For bigger matrices Scilab is much more faster than MuPAD
// We use scilab co create the random matrix because it faster
A:=scilab::rand(50,50):
B:=scilab::rand(50,50):
domtype(A) ;
domtype(B) ;
rtime(scilab::_mult(A,B)) ;
rtime(A*B) ;
rtime(scilab::det(B)) ;
rtime(linalg::det(B)) ;
prog::test( mytest(scilab::det(B), linalg::det(B)) ) ;

// Some more matrix examples with boolean and sparse boolean matrices
A:=array(1..2,1..2,[[TRUE,FALSE],[FALSE,TRUE]]) ;
B:=array(1..2,1..2,[[FALSE,TRUE],[FALSE,TRUE]]) ;
prog::test( scilab::_and(A,B), array(1..2, 1..2,(1,1) = FALSE, (1,2) = FALSE,\
                                                (2,1) = FALSE, (2,2) = TRUE) );
prog::test( scilab::_or(A,B), array(1..2, 1..2, (1,1) = TRUE, (1,2) = TRUE,\
                                                (2,1) = FALSE, (2,2) = TRUE) );

// Now using sparse boolean matrices in Scilab
M:=Dom::SparseMatrixF2 ;
A:=M(2,2,[{1},{2}]) ;
B:=M(2,2,[{2},{2}]) ;
prog::test( scilab::_and(A,B), M(2,2, [{}, {2}]) ) ;
prog::test( scilab::_or(A,B),  M(2,2, [{1, 2}, {2}]) ) ;

// Here comes the first examples
f:=scilab::func( ["i1","i2"], ["r1", "r2", "r3"], ["r1=i1^2","r2=i2^2","r3=i1+i2"] ) :
tmp:=[f(3,5)];
prog::test( tmp, [9.0, 25.0, 8.0] ) ;
g:=scilab::func(["a","b","c"],["r1","r2"],["r1=a+b+c ", "r2=a*b*c"]):
tmp:=[g(34,68,2)];
prog::test( tmp, [104.0, 4624.0] ) ;
h:=scilab::func(["a","b","c"],["r1","r2"],["r1=a+b+c ", "r2=a*b*xx"]) :
prog::test( h(1,2,3), [FAIL, "Scilab Error", 4.0] ) ;
  // Because xx isn't defined inside Scilab !

s:=scilab::poly(0,"x") ;
p:=-10*s+s^2+3*s^5 ;
q:=2*p ;
// The Scilab function simp(), [n1,d1]=simp(n,d) calculates two
// polynomials n1 and d1 such that n1/d1 = n/d.
prog::test( scilab::simp(q,p), [FAIL, "Scilab Error", 41.0] ) ;


// But ...
h:=scilab::func(["p1", "p2"], ["n","d"], ["[n,d]=simp(p1,p2)"]) :
tmp:= [h(q,p)] ;
prog::test( tmp, [Dom::UnivariatePolynomial(x, Dom::ExpressionField())(2.0),\
                   Dom::UnivariatePolynomial(x, Dom::ExpressionField())(1.0)] ) ;

// The last example will solve the ODE 
// dy/dt = f(t,y) = y^2 - y sin(t) + cos(t)

// With Scilab
h:=scilab::func([],["y"],["deff('[ydot]=f(t,y)','ydot=y^2-y*sin(t)+cos(t)')",\
                        "y0=0;t0=0;t=0:0.1:%pi", \
                        "y=ode(y0,t0,t,f)"]) :
Se := h() ;

// And with MuPAD
f:=proc(t,y) begin [ y[1]^2-y[1]*sin(t)+cos(t) ] end_proc:
tmp := numeric::odesolve( 0..3.1, f, [0], Stepsize=0.1, Alldata=1 ) ;
Me := Dom::Matrix(Dom::Float)(1,32,[tmp[i][2][1] $i=1..32]) ;
prog::test( mytest(norm(Se-Me), 0.000005366172714) ) ;

// Just some test with complex matrices, complex polynomials and 
// matrices of polynomials

// Complex matrices
B:=Dom::Matrix(Dom::Numerical)([[23.0+I, 24.34+I, 54.0+I],\
                           [12.0+I, 23.0+I, 45455.333+I],\
                           [212.0+I,34323.0+I,342.123+I]]) ;

CA:=Dom::Matrix(Dom::Numerical)([[23.0, 24.34, 54.0],\
                           [12.0, 23.0, 45455.333],\
                           [212.0,34323.0,342.123]]) ;

BB:=scilab::_plus(B,CA) ; 
prog::test( mytest(norm(BB - Dom::Matrix(Dom::Numerical)(3,3,\
         [[46.0 + 1.0*I,  48.68 + 1.0*I,    108.0 + 1.0*I],\
	      [24.0 + 1.0*I,   46.0 + 1.0*I,  90910.666 + 1.0*I],\
	      [424.0 + 1.0*I, 68646.0 + 1.0*I,  684.246 + 1.0*I]])),0.0) ) ;

prog::test( mytest(scilab::det(BB), - 2.850171669e11 - 6195993123.0*I) ) ;


CC:=scilab::_plus(CA,CA) ;
domtype(CC) ;
prog::test( mytest(norm(CC - Dom::Matrix(Dom::Numerical)(3,3,\
              [[46.0,  48.68,    108.0],\
	       [24.0,   46.0,  90910.666],\
	       [424.0, 68646.0,  684.246]])),0.0) ) ;

// Complex polynomials

P:=Dom::UnivariatePolynomial(x, Dom::ExpressionField());
p:=P((10+I)*x+(1+2*I)*x^2+3*x^5)
pp:=2*p
ppp:=3*p

scilab::coeff(p) ;
pa:=scilab::_plus(p,pp) ;
domtype(pa);

prog::test(ppp-pa, Dom::UnivariatePolynomial(x, Dom::ExpressionField())(0)) ;


// Matrices of real polynomials
Q:=Dom::UnivariatePolynomial(x,Dom::Float,LexOrder)
q:=Q(10*x+x^2+3*x^6)
qq:=2*q;
qqq:=3*q;

MQ:=Dom::Matrix(Q) ;
AQ:=MQ([[q,qq,qqq],[qq,qqq,q]])

scilab::_plus(AQ,AQ);
domtype(%);   

// Matrices of complex polynomials
MP:=Dom::Matrix(P)
AP:=MP([[p,pp,ppp],[pp,ppp,p]])

scilab::_plus(AP,AP);
domtype(%);

prog::testexit():


