// $Id: PPC.h,v 1.4 2004/01/16 12:10:47 ccr Exp $

// Inline assembly for the PowerPC
// Adapted from PPC.s to Gnu CC syntax by ccr

// All of these functions have their C versions listed, too.
// You can switch off each assembly version separately by simply
// changing the corresponding '#if 0' to '#if 1'.

// Notice: While it is possible to simply use registers in the
// assembly code and tell gcc that they are clobbered, I advise
// against it.  Instead, the code below uses register variables and
// refers to them as output variables so that the compiler may choose
// whatever registers suit the surroundings best.

// Notice: Do not use "=m" around here -- the optimisations of gcc do
// not take into account that you changed a global variable, so a
// check on, say, overflow may be moved to *before* it is actually
// set!

#ifndef PPC_INLINE_ASSEMBLY_H
#define PPC_INLINE_ASSEMBLY_H

#if 0
inline long
addll(ulong x, ulong y)
{
  ulong z;

  z=x+y;overflow=(z<x)?1:0; return (long)z;
}
#else
inline ulong addll(ulong x, ulong y) {
  register ulong z, local_overflow;
  __asm__("addc  %0,%2,%3\n" /* %0 = z, %2 = x, %2 = y */
	  "subfe %1,%1,%1\n" /* %1 = local_overflow */
	  "addi  %1,%1,1\n"
 
	  : "=r" (z), "=r" (local_overflow)  /* output */
	  : "r" (x), "r" (y) /* input */
	  // workaround: stop the compiler from using r0
	  : "r0", "cc"
	  );
  overflow = local_overflow;

  return(z);
}
#endif

#if 0
inline long
addllx(ulong x, ulong y)
{
  ulong z;

  z=x+y+overflow;overflow=(z<x)||((z==x)&&overflow)?1:0; return (long)z;
}
#else
inline ulong
addllx(ulong x, ulong y)
{
  register ulong z, dummy1 __asm__("r5"), dummy2;

  __asm__("addc  %0,%4,%5\n" /* %0 = z, %4 = x, %5=y */
	  "subfe %2,%2,%2\n" /* %2 = dummy1 */
	  "lwz   %3,%1\n"    /* %1 = overflow, %3 = dummy2 */
	  "addc	 %0,%3,%0\n" /* %0 = z */
	  "subfe %3,%3,%3\n"
	  "addc	 %2,%3,%2\n"
	  "addi	 %2,%2,2\n"
	  "stw   %2,%1\n"    /* %1 = overflow */
	  : "=r" (z), "=m" (overflow), /* output */
	    "=r" (dummy1), "=r" (dummy2)
	  : "r" (x), "r" (y) /* input */
	  : "r0", "cc");

  return(z);
}
#endif

#if 0
inline long
subll(ulong x, ulong y)
{
  ulong z;

  z=x-y;overflow=(z>x)?1:0; return (long)z;
}
#else
inline ulong
subll(ulong x, ulong y)
{
  register ulong z, local_overflow;

  __asm__("subfc %0,%3,%2\n" /* %0=z, %2=x, %3=y */
	  "subfe %1,%1,%1\n"
	  "neg   %1,%1\n"
	  : "=r" (z), "=r" (local_overflow)
	  : "r" (x), "r" (y)
	  // workaround: stop the compiler from using r0
	  : "r0", "cc");
  overflow = local_overflow;
  return(z);
}
#endif

#if 0
inline long
subllx(ulong x, ulong y)
{
  ulong z;

  z=x-y-overflow;overflow=(z>x)||((z==x)&&overflow)?1:0; return (long)z;
}
#else
inline long
subllx(ulong x, ulong y)
{
  register ulong z, dummy1, dummy2;

  // Notice: This code uses one more register than strictly necessary,
  // to make better use of the superscalar processor architecture.

  register ulong local_overflow = overflow;
  __asm__("subfc %0,%5,%4\n" /* %0 = z, %4 = x, %5 = y */
	  "subfe %2,%2,%2\n" /* %2 = dummy1 */
	  "subfc %0,%1,%0\n" /* %1 = local_overflow */
	  "subfe %3,%3,%3\n" /* %3 = dummy2 */
	  "addc  %2,%3,%2\n"
	  "neg   %1,%2\n"
	  : "=r" (z), "=r" (local_overflow),
	  "=r" (dummy1), "=r" (dummy2)
	  : "r" (x), "r" (y), "1" (local_overflow)
	  : "r0", "cc");

  overflow = local_overflow;

  return(z);
}
#endif

// TODO
inline ulong
shiftl(ulong x, ulong y)
{
  hiremainder=x>>(BITS_IN_LONG-y);return (x<<y);
}

// TODO
inline long
shiftlr(ulong x, ulong y)
{
  hiremainder=x<<(BITS_IN_LONG-y);return (x>>y);
}

#if 0
inline long
mulll(ulong x, ulong y)
{
  ulong xlo,xhi,ylo,yhi;
  ulong xylo,xymid,xyhi;
  ulong xymidhi, xymidlo;
  xlo = LOWWORD(x); xhi = HIGHWORD(x);
  ylo = LOWWORD(y); yhi = HIGHWORD(y);

  xylo = xlo*ylo; xyhi = xhi*yhi;
  xymid = (xhi + xlo)*(yhi + ylo) - (xyhi + xylo);

  xymidhi = HIGHWORD(xymid);
  xymidlo = xymid << BITS_IN_HALFULONG;

  xylo += xymidlo;
  hiremainder = xyhi + xymidhi + (xylo < xymidlo)
     + (((((xhi + xlo) + (yhi + ylo)) >> 1) - xymidhi) & HIGHMASK);

  return xylo;
}
#else
// TODO: PPC.s says this is "plus lente" -- slower
// check, including what "dialect POWER" means
inline ulong
mulll(ulong x, ulong y)
{
  register ulong z, local_hirem;
  __asm__("mulhwu %1,%2,%3\n"
	  "mullw  %0,%2,%3\n"
	  : "=&r" (z), "=&r" (local_hirem)
	  : "r" (x), "r" (y)
	  : "cc");
  hiremainder = local_hirem;
  return(z);
}
#endif

#if 0
inline long
addmul(ulong x, ulong y)
{
  ulong xlo,xhi,ylo,yhi;
  ulong xylo,xymid,xyhi;
  ulong xymidhi, xymidlo;

  xlo = LOWWORD(x); xhi = HIGHWORD(x);
  ylo = LOWWORD(y); yhi = HIGHWORD(y);

  xylo = xlo*ylo; xyhi = xhi*yhi;
  xymid = (xhi + xlo)*(yhi + ylo) - (xyhi + xylo);

  xylo += hiremainder; xyhi += (xylo < hiremainder);

  xymidhi = HIGHWORD(xymid);
  xymidlo = xymid << BITS_IN_HALFULONG;

  xylo += xymidlo;
  hiremainder = xyhi + xymidhi + (xylo < xymidlo)
     + (((((xhi + xlo) + (yhi + ylo)) >> 1) - xymidhi) & HIGHMASK);

  return xylo;
}
#else
// TODO: As for mulll, PPC.s claims this to be the slower variant
inline ulong
addmul(ulong x, ulong y)
{
  register ulong z, local_hirem;
  local_hirem = hiremainder;
  __asm__("mulhwu %1,%2,%3\n"
	  "mullw  %0,%2,%3\n"
	  "addc   %0,%4,%0\n"
	  "addze  %1,%1\n"
	  // The & sign prevents gcc from
	  // using the same registers as for x and y.
	  : "=&r" (z), "=&r" (local_hirem)
	  : "r" (x), "r" (y), "r" (local_hirem));
  hiremainder = local_hirem;
  return(z);
}
#endif

#if 0
inline int
bfffo(ulong x)
{
  int sc;
  static int tabshi[16]={4,3,2,2,1,1,1,1,0,0,0,0,0,0,0,0};

  sc = BITS_IN_LONG - 4;
#ifdef LONG_IS_64BIT
  if(x & 0xffffffff00000000) {sc -= 32; x >>= 32;}
#endif
  if(x > 0xffffUL) {sc -= 16; x >>= 16;}
  if(x > 0x00ffUL) {sc -= 8; x >>= 8;}
  if(x > 0x000fUL) {sc -= 4; x >>= 4;}
  return sc + tabshi[x];
}
#else
// Count leading zeroes
inline int
bfffo(ulong x)
{
  register unsigned int z;
  __asm__("cntlzw %0,%1"
	  : "=r"(z) : "r" (x));
  return(z);
}
#endif

// Not ready yet
#if 1
inline long
divll(ulong x, ulong y)
{
  // The code in PPC.s was very hard to read and had 75 lines,
  // compared with 5 lines for the intel version.
  // Not converted to gcc-ppc-assembly therefore.

  // See pari-users-guide for the definition of divll
#define GLUE(hi, lo) (((hi) << BITS_IN_HALFULONG) + (lo))
#define SPLIT(a, b, c) b = HIGHWORD(a); c = LOWWORD(a)

  ulong v1, v2, u3, u4, q1, q2, aux, aux1, aux2;
  int k;
  if (hiremainder >= y) err(talker, "Invalid arguments to divll");
  else if (hiremainder == 0)
  {    /* Only one division needed */
    hiremainder = x % y;
    return x/y;
  }
  else if (y <= LOWMASK)
  {    /* Use two half-word divisions */
    hiremainder = GLUE(hiremainder, HIGHWORD(x));
    q1 = hiremainder / y;
    hiremainder = GLUE(hiremainder % y, LOWWORD(x));
    q2 = hiremainder / y;
    hiremainder = hiremainder % y;
    return GLUE(q1, q2);
  }
  // Shift x, y, and hiremainder such that the leading bit of y is 1.
  k = bfffo(y);
  if(k) {
    hiremainder = (hiremainder << k) + (x >> (BITS_IN_LONG - k));
    x <<= k;
    y <<= k;
  }
  SPLIT(y, v1, v2);
  SPLIT(x, u3, u4);
  q1 = hiremainder / v1;
  if (q1 > LOWMASK) q1 = LOWMASK;
  hiremainder -= q1 * v1;
  aux = v2 * q1;
 again:
  SPLIT(aux, aux1, aux2);
  if (aux2 > u3) aux1++;
  if (aux1 > hiremainder) {
    q1--;
    hiremainder += v1;
    aux -= v2;
    goto again;
  }
  
  hiremainder = GLUE(hiremainder - aux1, LOWWORD(u3 - aux2));
  q2 = hiremainder / v1;
  if (q2 > LOWMASK) q2 = LOWMASK;
  hiremainder -= q2 * v1;
  aux = v2 * q2;
 again2:
  SPLIT(aux, aux1, aux2);
  if (aux2 > u4) aux1++;
  if (aux1 > hiremainder) {
    q2--;
    hiremainder += v1;
    aux -= v2;
    goto again2;
  }
  hiremainder = GLUE(hiremainder - aux1, LOWWORD(u4 - aux2)) >> k;
  return GLUE(q1, q2);
}
#else
// this code is almost directly from the
// PowerPC Compiler Writer's Guide, section
// 3.2.3.7
inline long
divll(ulong x, ulong y) {
  if(hiremainder >=y) err(talker, "Invalid arguments to divll");
  else if (0==hiremainder) { // 32-bit division is sufficient
    hiremainder = x%y;
    return x/y;
  }
  // dividend: R3:R4
  // divisor:  R5:R6
  // ToDo: In our situation, R5 is always 0, make use of this!
  __asm__(/// 64-bit bfffo (count leading zeroes --> R0
	  "cmpwi cr0,r3,0\n" // dvd.msw == 0?
	  "cntlzw r0,r3\n"   // R0 = dvd.msw.LZ
	  "cntlzw r9,r4\n"   // R9 = dvd.lsw.LZ
	  "bne cr0,1f\n"     // if(dvd.msw == 0) dvd.LZ = dvd.msw.LZ
	  "addi r0,r9,32\n"  // dvd.LZ = dvd.lsw.LZ + 32
	  "1:\n"
	  /// same for divisor --> R9
	  "cmpwi cr0,r5,0\n" // dvd.msw == 0?
	  "cntlzw r9,r5\n"   // R9 = dvs.msw.LZ
	  "cntlzw r10,r6\n"  // R10 = dvs.lsw.LZ
	  "bne cr0,1f\n"     // if(dvs.msw == 0) dvs.LZ = dvs.msw.LZ /// TODO: can't happen, right?  if(0==hiremainder)... above
	  "addi r9,r10,32\n" // dvs.LZ = dvs.lsw.LZ + 32
	  "1:\n"
	  // determine shift amounts to minimize the number of iterations
	  "cmpw cr0,r0,r9\n" // compare dvd.LZ to dvs.LZ
	  "subfic r10,r0,64\n" // R10 = dvd.SD /// r10 = 64-r0 // TODO: Why carry?
	  "bgt cr0,9f\n"     // if(dvs > dvd) quotient = 0 /// TODO: Can this happen in our situation?  We already checked r3>0 before the asm call.
	  "addi r9,r9,1\n"   // ++dvs.LZ (or --dvs.SD)
	  "subfic r9,r9,64\n" // R9 = dvs.SD /// r9 = 64-r9 // TODO: Why carry?
	  "add r0,r0,r9\n"   // (dvd.LZ + dvs.SD) = left shift of dvd for
	                     // initial dvd
	  "subf r9,r9,r10\n" // (dvd.SD - dvs.SD) = right shift of dvd for
	                     // initial tmp
	  "mtctr r9\n"       // number of iterations = dvd.SD - dvs.SD
	  /// R7:R8 = R3:R4 >> R9
	  "cmpwi cr0,r9,32\n" // compare R9 to 32
	  "addi r7,r9,-32\n"
	  "blt cr0,3f\n"     // if(R9 < 32) jump to 3
	  "srw r8,r3,r7\n"   // tmp.lsw = dvd.msw >> (R9 - 32)
	  "li r7,0\n"        // tmp.msw = 0
	  "b 4f\n"
	  "3:\n"
	  "srw r8,r4,r9\n"   // R8 = dvd.lsw >> R9
	  "subfic r7,r9,32\n"
	  "slw r7,r3,r7\n"   // R7 = dvd.msw << 32 - R9
	  "or r8,r8,r7\n"    // tmp.lsw = R8 | R7
	  "srw r7,r3,r9\n"   // tmp.msw = dvd.msw >> R9
	  "4:\n"
	  // R3:R4 = R3:R4 << R0
	  "cmpwi cr0,r0,32\n" // compare R0 to 32
	  "addic r9,r0,-32\n"
	  "blt cr0,5f\n"      // if(R0 < 32) jump to 5
	  "slw r3,r4,r9\n"    // dvd.msw = dvd.lsw << R9
	  "li r4,0\n"         // dvd.lsw = 0
	  "b 6f\n"
	  "5:\n"
	  "slw r3,r3,r0\n"    // R3 = dvd.msw << R0
	  "subfic r9,r0,32\n"
	  "srw r9,r4,r9\n"    // R9 = dvd.lsw >> 32 - R0
	  "or r3,r3,r9\n"     // dvd.msw = R3 | R9
	  "slw r4,r4,r0\n"    // dvd.lsw = dvd.lsw << R0
	  "6:\n"
	  // restoring division shift and subtract loop
	  "li r10,-1\n"       // R10 = -1
	  "addic r7,r7,0\n"   // clear carry bit before loop starts
	  "7:\n"
	  // tmp:dvd is considered one large register
	  // each portion is shifted left 1 bit by adding it to itself
	  // adde sums the carry from the previous and creates a new carry
	  "adde r4,r4,r4\n"   // shift dvd.lsw left 1 bit
	  "adde r3,r3,r3\n"   // shift dvd.msw to left 1 bit
	  "adde r8,r8,r8\n"   // shift tmp.lsw to left 1 bit
	  "adde r7,r7,r7\n"   // shift tmp.msw to left 1 bit
	  "subfc r0,r6,r8\n"  // tmp.lsw - dvs.lsw
	  "subfe. r9,r5,r7\n" // tmp.msw - dvs.msw
	  "blt cr0,8f\n"      // if(result < 0) clear carry bit
	  "mr r8,r0\n"        // move lsw
	  "mr r7,r9\n"        // move msw
	  "addic r0,r10,1\n"  // set carry bit
	  "8:\n"
	  "bdnz 7b\n"
	  // write quotient and remainder
	  "adde r4,r4,r4\n"   // quo.lsw (lsb = CA)
	  "adde r3,r3,r3\n"   // quo.msw (lsb from lsw)
	  "mr r6,r8\n"        // rem.lsw
	  "mr r5,r7\n"        // rem.msw
	  "b 1f\n"           // return
	  /// Quotient is 0 (dvs > dvd)
	  "9:\n"
	  "mr r6,r4\n"        // rmd.lsw = dvd.lsw
	  "mr r5,r3\n"        // rmd.msw = dvd.msw
	  "li r4,0\n"         // dvd.lsw = 0
	  "li r3,0\n"
	  // FINI
	  "1:\n"
	  : /* output */ 
	  : /* input */
	  : "cr0", "r3", "r0", "r9", "r4", 
	  "r5", "r10", "r7", "r8");
}
#endif

#endif // PPC_INLINE_ASSEMBLY_H
