// $Id: pari-notation.h,v 1.3 2003/05/08 11:19:44 ccr Exp $

#include "genpari.h"
#include "pari-arithmetic.h"

// Utility class wrapping PARI GENs.  This class
// is designed not to leave any traces in the compiled
// code (if compiled for release versions), but to render
// code more readable.

class PariGEN {
 public:
  GEN val_;
  PariGEN(GEN val) : val_(val) {};
  PariGEN(): val_(0) {};
  PariGEN(double d) { val_ = dbltor(d); } // increases stack usage
  PariGEN(long int n) { val_ = stoi(n); } // increases stack usage

  operator GEN() { return val_; }
};

inline PariGEN operator+(PariGEN a, PariGEN b) { return gadd(a, b);}
inline PariGEN operator+(PariGEN a, long b)    { return gaddgs(a, b); }
inline PariGEN operator+(PariGEN a, int  b)    { return gaddgs(a, b); }
inline PariGEN operator+(PariGEN a, double b)  { return gadd(a, dbltor(b)); }
inline PariGEN operator+(long a, PariGEN b)    { return gaddsg(a, b); }
inline PariGEN operator+(int  a, PariGEN b)    { return gaddsg(a, b); }
inline PariGEN operator+(double a, PariGEN b)  { return gadd(dbltor(a), b); }

inline PariGEN operator-(PariGEN a, PariGEN b) { return gsub(a, b); }
inline PariGEN operator-(PariGEN a, long b)    { return gsubgs(a, b); }
inline PariGEN operator-(PariGEN a, int  b)    { return gsubgs(a, b); }
inline PariGEN operator-(PariGEN a, double b)  { return gsub(a, dbltor(b)); }
inline PariGEN operator-(long a, PariGEN b)    { return gsubsg(a, b); }
inline PariGEN operator-(int  a, PariGEN b)    { return gsubsg(a, b); }
inline PariGEN operator-(double a, PariGEN b)  { return gsub(dbltor(a), b); }

inline PariGEN operator*(PariGEN a, PariGEN b) { return gmul(a, b); }
inline PariGEN operator*(PariGEN a, long b)    { return gmulgs(a, b); }
inline PariGEN operator*(PariGEN a, int  b)    { return gmulgs(a, b); }
inline PariGEN operator*(PariGEN a, double b)  { return gmul(a, dbltor(b)); }
inline PariGEN operator*(long a, PariGEN b)    { return gmulsg(a, b); }
inline PariGEN operator*(int  a, PariGEN b)    { return gmulsg(a, b); }
inline PariGEN operator*(double a, PariGEN b)  { return gmul(dbltor(a), b); }

inline PariGEN operator/(PariGEN a, PariGEN b) { return gdiv(a, b);}
inline PariGEN operator/(PariGEN a, long b)    { return gdivgs(a, b); }
inline PariGEN operator/(PariGEN a, int  b)    { return gdivgs(a, b); }
inline PariGEN operator/(PariGEN a, double b)  { return gdiv(a, dbltor(b)); }
inline PariGEN operator/(long a, PariGEN b)    { return gdivsg(a, b); }
inline PariGEN operator/(int  a, PariGEN b)    { return gdivsg(a, b); }
inline PariGEN operator/(double a, PariGEN b)  { return gdiv(dbltor(a), b); }

