/**
   adt::Stack -- the abstract data type stack


   Calling Sequence:

     adt::Stack([obj1, obj2, ... objn])


   Methods:

     push   -- push new elements on the stack
     pop    -- pop elements from stack
     top    -- top of the stack
               (optional) the top - n-th e of the stack
     empty  -- stack empty ?
     depth  -- depth of the stack
     reverse  -- reverts the order of the stack


   Examples:

>> A := adt::Stack(1,2,3,4)

                                  Stack1

>> A::top()

                                     4

>> A::top(1)

                                     3

>> A::top(2)

                                     2

>> A::pop()

                                     4

>> A::top()

                                     3

>> A::push(42)

                                    42

>> A::depth()  

                                     4

>> A::empty()

                                   FALSE



**/

adt::Stack :=  proc() option escape; local DOM;
begin
  DOM := newDomain(genident("Stack")):
  DOM::STACK := [args()]:
		     
  DOM::pop :=
    proc()
      local popVal;
    begin
      if DOM::empty() then return(FAIL) end_if;
      popVal := DOM::STACK[nops(DOM::STACK)];
      delete DOM::STACK[nops(DOM::STACK)];
      popVal
    end_proc:

  DOM::push := value -> (DOM::STACK := append(DOM::STACK, value); value);
  DOM::empty := () -> bool(nops(DOM::STACK) = 0):
  DOM::top :=  proc(k = 0)
	       begin
		 if nops(DOM::STACK) <= k then FAIL
		 else DOM::STACK[nops(DOM::STACK) - k]: end_if
	       end_proc:
  DOM::depth := () -> nops(DOM::STACK):
  DOM::reverse := () -> (DOM::STACK := revert(DOM::STACK); null()):

  DOM::print := () -> revert(DOM::STACK):

  DOM
end_proc:

