/*****************************************************************************
  $Id: binaryTrees.mu,v 1.33 2004/09/29 22:30:08 nthiery Exp $
  File:		lib/COMBINAT/binaryTrees.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
                Florent Hivert <florent.hivert@univ-mlv.fr>
  Created:	28/06/2002
  License:	LGPL
*****************************************************************************/

domain combinat::binaryTrees
    inherits combinat::trees;
    category Cat::TreesClass(Binary);
    axiom    Ax::normalRep, Ax::canonicalRep;

    info := "Library 'combinat::binaryTrees': utilities for binary trees";

    interfaceAdd := combinat::trees::interfaceAdd union // CHECKME
       {
	hold(unrank),
	hold(print),
	hold(over), hold(under), hold(compose),
	hold(recursiveRevertRightBranch),
//		 hold(rightLevels),
	hold(toDyckWord),
	hold(branchSlice)
       };
    interfaceAddIfExists :=
       {
	hold(fromSaillance),
	hold(shuffle)
       };

    //////////////////////////////////////////////////////////////////////////
    // Description of the grammar
    //////////////////////////////////////////////////////////////////////////
    
    typeLabelled := combinat::labelledBinaryTrees;

    one := dom([dom::dummyLabel]);

    convert :=
    proc(x)
	local i;
    begin
	case domtype(x)
	of dom do 
	    return(x);
	of DOM_LIST do
	    if nops(x)>3 then
		error("Too many childs");
	    elif nops(x)=0 then
		return(dom::zero);
	    else
		return(new(dom, 
			   x[1], 
			   if nops(x)>=i then
			       dom::convert(x[i])
			   else 
			       dom::zero
			   end_if $i=2..3))
	    end_if;
	end_case;
	FAIL;
    end_proc;

    // Add constructor with terse syntax for unlabelled trees

    //////////////////////////////////////////////////////////////////////////
    // Operations
    //////////////////////////////////////////////////////////////////////////

    /*
    // Assumption: t[1] and t[2] returns dom::zero when there are no
    // left/right tree
    
    index :=
    proc(t: dom, i: Type::NonNegInt)
    begin
	if i > extnops(t)+1 and i<=2 then
	    return(dom::zero);
	else
	    return(extop(t,i+1));
	end_if;
    end_proc;
    */

    over :=
    proc(t: dom, s: dom) : dom
    begin
	if s=dom::zero then
	    t;
	else
	    subsop(s, 1=dom::over(t,s[1]));
	end_if;
    end_proc;

    under :=
    proc(t: dom, s:dom) : dom
    begin
	if t=dom::zero then
	    s;
	else
	    subsop(t, 2=dom::under(t[2],s));
	end_if;
    end_proc;

    compose :=
    proc(t: dom, s:dom) : dom
    begin
	if t=dom::zero then
	    dom::zero;
	else
	    dom::under(dom::over(dom::compose(t[1], s), s),
		       dom::compose(t[2], s));
	end_if;
    end_proc;

    recursiveRevertRightBranch := 
    // involution which reverse recursively all the right branches
    proc(t: dom) : dom
	local result;
    begin
	if t=dom::zero then
	    return(t);
	else
	    result := dom::zero;
	    repeat
	        result := new(dom, 
			      t[0], 
			      dom::recursiveRevertRightBranch(t[1]),
			      result);
	    until (t:=t[2])=dom::zero end_repeat;
	end_if;
        result;
    end_proc;

    // Not tested!!!
    rightLevels := 
    proc(t: dom)
	local rightBranch, node, res, i;
    begin
	if t = dom::one or t = dom::zero then
	    [[0]];
	else
	    rightBranch := [t[1]];
	    node := t;
	    while node[2] <> dom::atom and node[2] <> dom::zero do
		node := node[2];
		rightBranch := append(rightBranch, node[1]);
	    end_while;
	    res := [];
	    for i in rightBranch do
		res := zip(res, dom::rightLevels(i), _concat, []);
	    end_for;
	    [[nops(rightBranch)]] . res;
	end_if;
    end_proc;

    toDyckWord :=
    proc(t : dom) : combinat::dyckWords
    begin
	if t=dom::zero then
	    [];
	else
	    [1, op(dom::toDyckWord(t[1])), 0, op(dom::toDyckWord(t[2]))]
	end_if;
    end_proc;

    branchSlice :=
    proc(t: dom, direction=hold(Right): combinat::typeDirection) : Type::ListOf(dom)
	local result;
    begin
	direction := if direction=Left then 1 else 2; end_if;
	result := [];
	while t<>dom::zero do
	    result := append(result, subsop(t, direction=dom::zero));
	    t:=t[direction];
	end_while;
        result;
    end_proc;

begin
end_domain:

// Use combinat::trees::generic instead ?
domain combinat::binaryTrees::fromSaillance
    inherits Dom::BaseDomain;
    category Cat::CombinatorialClass;
    axiom    Ax::systemRep; // not quite; Ax::facadeDomain ?

    grammar :=
    proc(k)
	local tree;
    begin
	tree := LeafNil;
	(tree := Prod(Node, CompleteBinaryTree, tree)) $ i=1..k;
	combinat::decomposableObjects
	([Tree               = tree,
	  CompleteBinaryTree = Union(LeafNil, Prod(Node, CompleteBinaryTree,
						    CompleteBinaryTree)),
	  LeafNil           = Epsilon,
	  Node		     = Atom
	 ]);
    end_proc;

    count :=
    proc(n: Type::PosInt, k: Type::PosInt)
    begin
	(dom::grammar(k))::count(n);
    end_proc;
    
    unrank :=
    proc(n: Type::PosInt, k: Type::PosInt, r: Type::PosInt)
    begin
	dom::fromGrammarOld((dom::grammar(k))::unrank(n,r));
    end_proc;

    generator :=
    proc(n: Type::PosInt, k: Type::PosInt)
    begin
	combinat::generators::map((dom::grammar(k))::generator(n),
				  combinat::binaryTrees::fromGrammarOld
				 );
    end_proc;

    random :=
    proc(n: Type::PosInt, k: Type::PosInt)
    begin
	dom::fromGrammarOld((dom::grammar(k))::random(n));
    end_proc;

end_domain:

domain combinat::binaryTrees::shuffle
    inherits Dom::BaseDomain;
    category Cat::CombinatorialClass;
    axiom    Ax::systemRep; // not quite; Ax::facadeDomain ?

    Name      := "combinat::binaryTrees::shuffle"; // Required?
    info      := "'combinat::binaryTrees::shuffle': the combinatorial class of the shuffle product of two binary trees";
    typeTree := Type::Predicate(t->_lazy_and(t::dom::hasProp<>FAIL,
					     t::dom::hasProp(combinat::trees)));

    generator :=
    proc(tree1: dom::typeTree,
	 tree2: dom::typeTree) : combinat::generators
	option escape;
	local len1, len2, subtrees1, subtrees2, T;
    begin
	T := tree1::dom; // implements "virtual behaviour"
	assert(T=tree2::dom);
	subtrees1 := T::branchSlice(tree1, Right);
	subtrees2 := T::branchSlice(tree2, Left);
	len1 := nops(subtrees1);
	len2 := nops(subtrees2);
	combinat::generators::map
	(combinat::integerVectors::generator(len1, len1+len2, MaxPart=1),
         proc(vect: combinat::integerVectors)
	     local i1, i2, x, result;
	 begin
	     i1 := len1; i2 := len2;
	     result := T::zero;
	     for x in vect do
		 if x=1 then
		     result := subsop(subtrees1[i1], 2=result);
		     i1:=i1-1;
		 else
		     result := subsop(subtrees2[i2], 1=result);
		     i2:=i2-1;
		 end_if
	     end_for;
	     result;
	 end_proc)
    end_proc;

end_domain:

