/*****************************************************************************
  $Id: cartesianProduct.mu,v 1.13 2004/03/02 16:49:59 nthiery Exp $
  File:		lib/COMBINAT/cartesianProduct.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Created:	21/02/2001
  License:	LGPL
*****************************************************************************/

domain combinat::cartesianProduct
    inherits Dom::BaseDomain;
    category Cat::CombinatorialClass;
    axiom    Ax::systemRep;

    info := "Library 'combinat::cartesianProduct': utilities for cartesianProducts";

    domtype := DOM_LIST;
    isA     :=  
	proc(l : Type::AnyType) 
	    local parameters, i;
	begin
	    if args(0) <> 1 then 
		parameters := [args(2..args(0))];
		for i from 1 to nops(parameters) do
		    if testargs() and not 
				      testtype(parameters[i], 
					       Type::Union(DOM_SET, DOM_LIST, Type::NonNegInt))
			then
			error ("Wrong type of argument: ".expr2text(parameters[i]));
		    end_if;
		    if domtype(parameters[i])=DOM_INT then
			parameters[i]:={$1..parameters[i]};
		    elif domtype(parameters[i])=DOM_LIST then
			parameters[i]:={op(parameters[i])};
		    end_if;
		end_for;
		
		if domtype(l) <> DOM_LIST then return(FALSE) end_if;
		
		if nops(l) <> nops(parameters) then return(FALSE) end_if;
		for i from 1 to nops(l) do
		    if not contains(parameters[i], l[i]) then return(FALSE) end_if;
		end_for;
	    else
		if domtype(l) <> DOM_LIST then return(FALSE) end_if;
	    end_if;
	    TRUE;
	end_proc;

    //////////////////////////////////////////////////////////////////////////
    // count
    //////////////////////////////////////////////////////////////////////////

    count :=
    proc()
	local s;
    begin
        _mult((if domtype(s)=DOM_INT then s else nops(s) end_if)
              $ s in [args()]);
    end_proc;

    //////////////////////////////////////////////////////////////////////////
    // list
    //////////////////////////////////////////////////////////////////////////

    generator :=
    proc()
        : combinat::generators
    /* 
       generator(set1,...,setN)	generator for all elements of set1 x ... x setN

       Each call of the generator yields the following element
       lexicographically or FAIL if there is no more. There is
       absolutely no waranty if the generator is called again after
       FAIL was returned.

       Is the result reasonable when a set contains twice an element ? */

        local word, word_int, A, i, first_time, s;
        option escape;
    begin
        if args(0)=0 then
            return(combinat::generators::singleton([]));
        end_if;
	A:=[args()]:
        for i from 1 to nops(A) do
            if domtype(A[i])=DOM_INT then
                A[i]:={$1..A[i]};
	    elif testargs() and not (testtype(A[i], DOM_LIST) or testtype(A[i], DOM_SET)) then
		error("Expecting an integer, list, or set; got: ".expr2text(A[i]));
            end_if;
            if nops(A[i])=0 then
                return(combinat::generators::empty);
            end_if;
        end_for;
        word_int := [1 $ args(0)];
        word     := [op(s,1) $ s in A];
        first_time:=TRUE;
        // Invariants:
        //  - word_int is a sequence of k integers with
        //		1 <= word_int[i] <= nops(A[i])
        //  - word is the corresponding element of A[1] x ... x A[k]
        //  - first_time is true if and only if it's the first run
        return(proc(): Type::Union(dom, DOM_FAIL)
                   local index;
               begin
                   if first_time then
                       first_time:=FALSE;
                       return(word);
                   end_if;
                   for index from nops(A) downto 1 do
                       if word_int[index] < nops(A[index]) then
                           word_int[index] := word_int[index]+1;
                           word[index] := op(A[index], word_int[index]);
                           return(word);
                       else 
                           word_int[index] := 1;
                           word[index] := op(A[index],1);
                       end_if;
                   end_for;
                   return(FAIL);
               end_proc)
    end_proc;

begin
end_domain:

