/*****************************************************************************
  $Id: integerListsLexStructure.mu,v 1.2 2003/02/05 10:28:31 nthiery Exp $
  File:		lib/COMBINAT/integerListsLexStructure.mu
  Authors:	Florent Hivert <Florent.Hivert@univ-mlv.fr
		Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Created:	16/06/2001
  License:	LGPL
*****************************************************************************/

category combinat::integerListsLexStructure

    category combinat::structure;

    info := "Category 'combinat::integerListsLexStructure': wrapper around combinat::integerListsLexTools";

    interface_IntegerListLexBuilding := dom::interface_Structure union 
        {
         hold(first), hold(Next), hold(_less)
         };
    
    interface := dom::interface_IntegerListLexBuilding; // to allow for reuse

    /// Returns constraints for integerListsLexTools from user-friendly options
    parseOptions := toBeDefined;

    /**	Helper for constructing the option parser dom::parseOptions

        This generic parser makes it easy for a domain of the category
        combinat::IntegerListLexBuilding to build his own option parser.

	Example:

        domain ivectors
	    category combinat::integerListsLexStructure;

	    parseOptions :=
	    proc(k)
	    begin
		parseOptions_IntegerListLexBuilding
		   (table(
                          Length	= k,
                          MinPart	= 0,
               		  MaxPart	= RD_INF,
               		  Inner	= NIL,
               		  Outer	= NIL,
	       		  MinSlope= RD_NINF,
	       		  MaxSlope= RD_INF
			 ),
		    args());
	    end_proc;
	end_domain;
	ivectors::list(MinLength=4, Outer=[1,3,4], MaxSlope=4);

        It's up to the user (or the domain!) to ensure that Inner and
        Outer are compatible with MinPart, MaxPart, MinSlope and MaxSlope.
    **/
    parseOptions_IntegerListLexBuilding :=
    proc(defaultOptions: Type::OptionsTable)
        : Type::SequenceOf(Type::AnyType,6,6)
         // sequence of 6 constraints
        local options, inner, outer, minpart;
        option escape;
    begin
        options := combinat::getOptions(defaultOptions,
					dom::parseOptions_OptionTypes,
					1);
        if options[Length] <> NIL then
            options[MinLength] := options[Length];
            options[MaxLength] := options[Length];
        end_if;
        minpart := options[MinPart];
        if options[Inner] <> NIL then
            if domtype(options[Inner]) = DOM_LIST then
                inner := options[Inner];	// slight optimization ?
                options[Inner] :=
                    i->if i>nops(inner) then minpart else inner[i]; end_if;
                options[MinLength] := max(options[MinLength],
                                                nops(inner));
            end_if;
        else
	    // Note: a constant can be considered as a function.
            options[Inner] := minpart;
        end_if;
        if options[Outer] <> NIL then
            if domtype(options[Outer]) = DOM_LIST then
                outer := options[Outer];	// slight optimization ?
                options[Outer] := i-> outer[i];
                options[MaxLength] := min(options[MaxLength],
                                                nops(outer));
            end_if;
        else
	    // Note: a constant can be considered as a function.
            options[Outer] := options[MaxPart];
        end_if;
        
        options[MinLength], options[MaxLength],
        options[Inner], options[Outer],
        options[MinSlope], options[MaxSlope];
    end_proc;

    parseOptions_OptionTypes :=
    table(
          MinLength = Type::NonNegInt,
          MaxLength = combinat::integerListsLexTools::typeIntBound,
	  Length    = Type::Union(Type::NonNegInt, DOM_NIL),
          MinPart   = Type::NonNegInt,
          MaxPart   = combinat::integerListsLexTools::typeIntBound,
	  Inner     = Type::Union(combinat::integerListsLexTools,
                                        DOM_NIL),
//	  Inner     = combinat::integerListsLexTools,
          Outer     = Type::Union(combinat::integerListsLexTools::typeCompositionBound,
                                        DOM_NIL),
	  MinSlope  = combinat::integerListsLexTools::typeSlopeBound,
	  MaxSlope  = combinat::integerListsLexTools::typeSlopeBound
         );

    //////////////////////////////////////////////////////////////////////////

    type := Type::ListOf(Type::NonNegInt);

    size := l -> nops(l);

    // Not very useful yet, as domains of this category could as well
    // call dom::count_FromGenerator right away. But if some time in
    // the future we can improve integerListsLexTools::count, we will
    // just have to make a modification here to let all domains of
    // this category get the benefit.

    count_integerListsLexStructure := dom::count_FromGenerator;
    count := count_integerListsLexStructure;

    //////////////////////////////////////////////////////////////////////////
    // wrapper methods around combinat::integerListsLexTools
    // first, Next, ...
    //////////////////////////////////////////////////////////////////////////

    first :=
    proc(n: Type::Integer)
        : Type::Union(dom, DOM_FAIL)
    begin
	combinat::integerListsLexTools::first(n, dom::parseOptions(args(2..args(0))));
    end_proc;

    last :=
    proc(n: Type::Integer) 
        : Type::Union(dom, DOM_FAIL)
    begin
	combinat::integerListsLexTools::last(l, dom::parseOptions(args(2..args(0))));
    end_proc;

    Next :=
    proc(l: dom) : Type::Union(dom, DOM_FAIL)
    begin
	combinat::integerListsLexTools::Next(l, dom::parseOptions(args(2..args(0))));
    end_proc;

    previous :=
    proc(l: dom) 
        : Type::Union(dom, DOM_FAIL)
    begin
        combinat::integerListsLexTools::previous(l, dom::parseOptions(args(2..args(0))));
    end_proc;

    generator :=
    proc(n: Type::Integer)
        : combinat::generators
        local constraints;
    begin
        constraints := dom::parseOptions(args(2..args(0)));
        combinat::generators::fromNext
        (combinat::integerListsLexTools::first(n, constraints),
         combinat::integerListsLexTools::Next, constraints);
    end_proc;

    _less :=
    proc(l1: dom, l2: dom)
	local i;
    begin
	for i from 1 to min(nops(l1), nops(l2)) do
	    if l1[i] <> l2[i] then
		return(l1[i] < l2[i]);
	    end_if;
	end_for;
	nops(l1) < nops(l2);
    end_proc;

begin
end_category:

stdlib::LoadlibState["combinat::integerListsLexStructure"] := TRUE:
