/*****************************************************************************
  $Id: labelledBinaryTrees.mu,v 1.23 2004/07/19 20:44:19 nthiery Exp $
  File:		lib/COMBINAT/labelledBinaryTrees.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
                Florent Hivert <florent.hivert@univ-mlv.fr>
  Created:	28/06/2002
  License:	LGPL
*****************************************************************************/

domain combinat::labelledBinaryTrees
    inherits combinat::binaryTrees;
    category Cat::TreesClass(Labelled,Binary);
    axiom    Ax::normalRep, Ax::canonicalRep;
    
    interfaceAdd :=
       {
	hold(zero),
	hold(size),
	hold(nops),
	hold(subsop),
	hold(op),
	hold(convert),
	hold(expr),
	hold(print),
	hold(over),
	hold(under),
	hold(branchSlice),
	hold(compose),
	hold(revert),
	hold(fromShapeAndWord),
	hold(recursiveRevertRightBranch),
	hold(shuffle),
	hold(toPoset),
	hold(toWord),
	hold(decreasingTree),
	hold(searchTreeInsertLetter),
	hold(searchTreeInsertWord),
	hold(toBinaryTree)
       };

    typeUnlabelled := combinat::binaryTrees;

    //////////////////////////////////////////////////////////////////////////
    //
    //////////////////////////////////////////////////////////////////////////

    decreasingTree :=
    proc(word: combinat::words) : dom
	local m, i;
    begin
	if nops(word)=0 then
	    dom::zero
	else
	    // find the greatest index i such that m[i] is maximal
	    m:=1;
	    if word[i]>=word[m] then m :=i; end_if $ i=1..nops(word);
	    new(dom, 
		word[m],
		dom::decreasingTree([op(word,1..m-1)]),
		dom::decreasingTree([op(word,m+1..nops(word))]))
	end_if;
    end_proc;

    searchTreeInsertLetter :=
    proc(t: dom, letter)
    begin
	if t=dom::zero then
	    dom([letter]);
	else
	    if letter>t[0] then
		subsop(t, 2=dom::searchTreeInsertLetter(t[2], letter));
	    else
		subsop(t, 1=dom::searchTreeInsertLetter(t[1], letter));
	    end_if;
	end_if;
    end_proc;

    searchTreeInsertWord :=
    proc(t: dom, word: combinat::words)
	local l;
    begin
	(t := dom::searchTreeInsertLetter(t, l)) $ l in word;
	t;
    end_proc;

    toBinaryTree := dom::shape; 
    // Temporary alias for backward compatibility with some recents
    // experimental files. To be removed soon.
  
begin
end_domain:

domain combinat::labelledBinaryTrees::shuffle
    inherits combinat::binaryTrees::shuffle;
    category Cat::CombinatorialClass;
    axiom    Ax::systemRep; // not quite; Ax::facadeDomain ?

    Name      := "combinat::labelledBinaryTrees::shuffle";
    info      := "Library 'combinat::labelledBinaryTrees::shuffle': utilities for the shuffle product of two binary Trees";
    typeTrees      := combinat::labelledBinaryTrees;
begin
end_domain:
