/*****************************************************************************
  $Id: AlgebraWithBasis.mu,v 1.15 2004/09/22 15:45:02 hivert Exp $
  File:		lib/DOMAINS/CATEGORY/AlgebraWithBasis.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Created:	28/10/2002
  License:	LGPL
*****************************************************************************/

/**
    Cat::AlgebraWithBasis(R)

    R: ring

    For now, R needs to be commutative.
**/


alias(plus2		= operators::plus2):
alias(mult2		= operators::mult2):
alias(declareConversion = operators::overloaded::declareConversion):
alias(declareSignature  = operators::overloaded::declareSignature);

category Cat::AlgebraWithBasis(R: DOM_DOMAIN)

    category Cat::Algebra(R), Cat::ModuleWithBasis(R);

    info := "Cat::AlgebraWithBasis: category of algebras with a distinguished basis";

    interfaceAdd :=
		{
		 hold(one), hold(_power), //hold(_divide),
		 hold(_invert),
		 hold(characteristic)
		};

    // To be moved to Cat::Algebra
    characteristic :=
    // We inherit the characteristic of R if it's prime, or if R or
    // one of its coeffring ancestors is a field
    if isprime(R::characteristic) or
       proc(x: DOM_DOMAIN)
       begin
	   repeat
		  if x::hasProp(Cat::Field()) then
		      return(TRUE);
		  end_if;
	   until (x := x::coeffRing) = FAIL end_repeat;
	   return(FALSE);
       end_proc(R) then
	R::characteristic;
    end_if;

    //////////////////////////////////////////////////////////////////////////
    // Conversions
    //////////////////////////////////////////////////////////////////////////

    // Should go in Cat::SemiRing
    fromCoeffRing :=
    proc(r: dom::coeffRing)
    begin
	dom::multcoeffs(dom::one, r)
    end_proc;

    // Default initialization using the overloaded and conversion
    // mechanism from operators::overloaded.
    //
    // A domain in this category can set its own initialization by
    // overriding initDomainAlgebraWithBasis

    initDomainAdd := dom::initDomainAlgebraWithBasis;
    initDomainAlgebraWithBasis :=
    proc()
    begin
	declareConversion(dom::coeffRing, dom, dom::fromCoeffRing);
	//if dom::_mult = operators::_mult and dom::mult2<>FAIL then
	if dom::mult2 <> dom::mult2Default then
	    declareSignature (operators::mult2, [dom, dom], dom::mult2);
	    declareSignature
	    (operators::_power, [dom, Dom::Integer],
	     operators::repeatedSquaring
	     (dom::mult2, Source=dom,
	      Zero=dom::one, Opposite = dom::_invert));
	end_if;
    end_proc;

    one :=
    if dom::oneBasis <> FAIL then
	dom::term(dom::oneBasis)
    end_if;

    ground :=
    if dom::oneBasis <> FAIL then
	proc(x: dom)
	begin
	    dom::coeff(x,dom::oneBasis);
	end_proc;
    end_if;

    // for backward compatibility
    const := ground;

    //////////////////////////////////////////////////////////////////////////
    // Arithmetic operations
    //////////////////////////////////////////////////////////////////////////

    /// product of two elements of dom::basisIndices, as an element of dom
    mult2Basis :=
    if dom::multBasis <> FAIL then
	dom::multBasis
    end_if;

    /// product of n>=0 elements of dom::basisIndices, as an element of dom
    // multBasis

    /// product of two elements of dom
    mult2Default := dom::convert@operators::_mult;
    mult2 :=
    if dom::mult2Basis<>FAIL then
	operators::makeBilinear(dom::mult2Basis, Source=dom, ImageSet=dom);
    else
	dom::mult2Default;
    end_if;

    /// product of n>=0 elements of dom
    // Should go in Cat::Algebra
    // should use dom::multBasis if it's defined
    mult :=
    // if dom::multBasis<>FAIL then
    //     operators::makeMultilinear(dom::mult2Basis, Source=dom, ImageSet=dom);
    // end_if;
    if dom::mult2 <> dom::mult2Default then
	operators::makeAssociative
  	    (dom::mult2, Zero=dom::one, Source=dom, ImageSet=dom);
    else // try to do it by overloading. 
	() -> dom::convert(operators::_mult(args()))
    end_if;


    // Should disapear as soon as all our domains use operators::_mult
    _mult :=
    operators::makeAssociativeAlgebraProduct
    (dom, dom::coeffRing,
     dom::one, dom::multcoeffs, dom::mult2
    );

    // stub implementation for _invert.
    // Following the specifications of Cat::Monoid, _invert should try
    // to compute an inverse for x, and return FAIL otherwise.

    _invert := x->FAIL;

    _power := operators::_power;

    // Should go in Cat::Algebra
    // Could be renamed as bracket ?
    commutator :=
    proc(x,y)
    begin
	x*y-y*x;
    end_proc;

end_category:
