/*****************************************************************************
  $Id: SemiRing.mu,v 1.3 2004/03/26 15:56:49 nthiery Exp $
  File:		lib/DOMAINS/CATEGORY/SemiRing.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Created:	02/09/2003
  License:	LGPL
*****************************************************************************/

/*++
Cat::SemiRing -- the category of semi-rings

A Cat::SemiRing is essentially a Cat::Ring, except that + need not
have an opposite operation. So it's a Cat::AbelianMonoid and a
Cat::Monoid with the distributive laws a*(b+c) = a*b + a*c and (a+b)*c
= a*c + b*c for each a, b, c.

It's also almost a Cat::LeftModule(dom), except that the latter asks
for an abelian group.

Entries:-
characteristic - the characteristic of the ring ?
++*/

category Cat::SemiRing
    category Cat::SemiRng, Cat::Monoid; // Cat::LeftModule(dom) ;

    star :=
    if dom::hasProp(Cat::Field) then
	// To be moved to Cat::Field at some point
	proc(x: dom) : Type::Union(dom, DOM_FAIL)
	begin
	    x := dom::one-x;
	    if iszero(x) then
		FAIL
	    else
		x^-1
	    end_if;
	end_proc;
    else
	proc(x: dom) : Type::Union(dom, DOM_FAIL)
	begin
	    if iszero(x) then
		dom::one;
	    else
		FAIL
	    end_if;
	end_proc;
    end_if;
end_category:
