/* ++
Dom::HomogenisedWeylAlgebra

Author: Marcus Hausdorf, Uni Karlsruhe
        (hausdorf@ira.uka.de)
Created: 	Wed Oct 17 10:31:49 2001
Last update: 	Mon Apr 28 15:41:21 2003
++ */


alias(REP(x) = extop(x, 1),
    A_ERR = error("wrong type of arguments"),
    N_ERR = error("wrong number of arguments")):

domain Dom::HomogenisedWeylAlgebra(W, hv:DOM_IDENT)
local HVar_l:DOM_LIST, HR:DOM_DOMAIN, HOrder, i;
inherits Dom::WeylAlgebra(HVar_l, HR, HOrder, hv);
category Cat::SolvableAlgebra(HR);
axiom (if HR::hasProp(Ax::normalRep) then Ax::normalRep end_if),
      (if HR::hasProp(Ax::canonicalRep) then Ax::canonicalRep end_if);
 
//================
// body of domain 
//================

homogenise := (x -> new(dom, poly(map(W::poly2list(x), l -> ([l[1], 
    [W::degree(x) - _plus(op(l[2]))] . l[2]])), dom::dummyList, HR)));
    
dehomogenise := proc(x)
    local DOM;
    begin
	DOM := dom;
	new(W, poly(map(dom::poly2list(x), l -> ([l[1],
	    [op(l[2], 2..DOM::numVars)]])), W::dummyList, HR));
    end_proc;

janetComplete := proc(F:Type::ListOf(dom), L)
    local DOM:DOM_DOMAIN, T:DOM_LIST, Q:DOM_LIST, sepm_t:DOM_TABLE, sepnm_t:DOM_TABLE,  
	criterion:DOM_PROC, g, h, hr_f:DOM_BOOL, tmp, j, k:DOM_INT, noMore:DOM_BOOL, i, p_f,
	nss;
    begin
	// argument checks 
	if args(0) > 4 or args(0) < 2 then
	    N_ERR;
	end_if;
	hr_f := FALSE;
	p_f := FALSE;
	if args(0) > 2 then
	    if contains({args(3..args(0))}, hold(Head)) then
		hr_f := TRUE;
	    end_if;
	    if contains({args(3..args(0))}, hold(Output)) then
		p_f := TRUE;
	    end_if;
	end_if;
	if (domtype(L) <> DOM_DOMAIN
	    or L::hasProp = FAIL
	    or not L::hasProp(Cat::InvolutiveDivision))
	then
	    A_ERR;
	end_if;
	DOM := dom;
	// Buchberger's chain (resp. second) criterion
	criterion := proc(g)
	    local f;
	    begin
		for f in T do
		    if (not iszero(L::isInvDivisor(f["lmon"][2], g["lmon"][2], 
			sepm_t[f["lmon"][2]]))
			and DOM::exponentLess(zip(g["crit"], f["crit"], 
			    (l1, l2) -> max(l1, l2)), g["lmon"][2]))
		    then
			if p_f then
			    print(Unquoted, "The C R I T E R I O N has been invoked");
			end_if;
			return(TRUE);
		    end_if;
		end_for;
		FALSE;
	    end_proc;
	// autoreduce F. The result is an ordered list.
	F := dom::autoreduce(F);
	// set up T and Q; doneNMP_t lists all nonmultiplicative prolongations
	// carried out so far.
	// Data structures: row - "elem": element represented by the row
	//			      "mons": all monomials with respect to which the row
	//			              may be reduced
	// 			      "lmon": leading monomial
	//			      "crit": multi index used for Buchberger criterion
	//			      
	//		        (monomial = [coefficient, multi index])
	//			
	// T - [row], Q - [row], doneNMP - (row["elem"] - {int})						
	// sepm_t contains the multiplicative variables for the elements in T.
	F := map(F, l -> table("elem" = l, "lmon" = [DOM::lcoeff(l), DOM::lexponent(l)],
	    "crit" = DOM::lexponent(l), "nmp" = {}));
	T := [F[1]];
	Q := [F[i] $ i = 2..nops(F)];
	sepm_t := L::separationTableMult([T[1]["lmon"][2]]);
	repeat
	    if p_f then
		print(Unquoted, "=======================");
		print(Unquoted, "recomputing T and Q ...");
		print(Unquoted, "=======================");
	    end_if;
	    h := table("elem" = dom::zero);
	    while Q <> [] and dom::iszero(h["elem"]) do
		g := Q[1];
		delete Q[1];
		if not criterion(g) then
		    if not contains(g, "mons") then
			if hr_f then
			    g["mons"] := [g["lmon"]];
			else
			    g["mons"] := dom::poly2list(g["elem"]);
			end_if;
		    end_if;
		    h := dom::internalInvolutiveNormalForm(g, T, sepm_t, L, hr_f);
		end_if    
	    end_while;
	    if not dom::iszero(h["elem"]) then
		if g["elem"] <> h["elem"] then 
		    h["crit"] := h["lmon"][2];
		    h["nmp"] := {};
		    tmp := [];
		    for k from nops(T) downto 1 do
			if dom::exponentLess(h["lmon"][2], T[k]["lmon"][2]) then
			    tmp := append(tmp, T[k]);
			    delete T[k];
			else
			    break;
			end_if;
		    end_for;
		    if tmp <> [] then
			Q := listlib::singleMerge(tmp, Q, 
			    (l1, l2) -> (DOM::exponentLess(l1["lmon"][2], l2["lmon"][2])));
		    end_if;
		    T := append(T, h);
		else
		    T := append(T, g);
		end_if;	
	    end_if;
	    if p_f then
		print(Unquoted, "T: " , map(T, l -> l["elem"]));
		print(Unquoted, "Q: " , map(Q, l -> l["elem"]));			
		print(Unquoted, "=======================");
	    end_if;
	    noMore := FALSE;
	    repeat 
		if p_f then
		    print(Unquoted, "T: " , map(T, l -> l["elem"]));
		    print(Unquoted, "-----------------------");
		end_if;			
		sepm_t := L::separationTableMult([T[i]["lmon"][2] $ i = 1..nops(T)]);
		sepnm_t := map(sepm_t, l -> ({i $ i = 1..DOM::numVars} minus l));
		if p_f then
		    print(Unquoted, ".......................");
		    print(Unquoted, "non-multiplicative variables: " , sepnm_t);
		    print(Unquoted, ".......................");
		end_if;
		tmp := sort([[T[i], subsop(T[i]["lmon"][2], j = T[i]["lmon"][2][j] + 1), j, 
		    i] $ j in (sepnm_t[T[i]["lmon"][2]] minus T[i]["nmp"])
		    $ i = 1..nops(T)], (l1, l2) -> (DOM::exponentLess(l1[2], l2[2])));
		if (tmp = [] or 
		    (Q <> [] and not dom::exponentLess(tmp[1][2], Q[1]["lmon"][2]))) 
		then
		    noMore := TRUE;
		else
		    if p_f then
			print(Unquoted, "prolonging " , tmp[1][1]["elem"] , " w.r.t " 
			, tmp[1][3]);
		    end_if;         
		    T[tmp[1][4]]["nmp"] := T[tmp[1][4]]["nmp"] union {tmp[1][3]};
		    g := dom::exponent2monomial(subsop([0 $ dom::numVars], tmp[1][3] = 1)) 
			* tmp[1][1]["elem"];
		    g := table("elem" = g, "lmon" = [dom::lcoeff(g), tmp[1][2]], 
			"crit" = tmp[1][1]["lmon"][2], "nmp" = {});
		    nss := NIL;
		    if tmp[1][3] = 1 then
			for k from 1 to nops(T) do
			    if T[k]["lmon"][2] = subsop(tmp[1][1]["lmon"][2], 1 = 
				tmp[1][1]["lmon"][2][1] + 1)
			    then
				nss := T[k];
				break;
			    end_if;
			end_for;
		    end_if;
		    if nss <> NIL then
			delete T[k];
			T := listlib::insert(T, g,  
			    (l1, l2) -> (DOM::exponentLess(l1["lmon"][2], l2["lmon"][2])));
			g := nss;
		    end_if;    
		    if p_f then
			print(Unquoted, "yielding " , g); 
		    end_if;
		    if criterion(g) then
			h := table("elem" = dom::zero);
		    else
			if hr_f then
			    g["mons"] := [g["lmon"]];
			else
			    g["mons"] := dom::poly2list(g["elem"]);
			end_if;
			h := dom::internalInvolutiveNormalForm(g, T, sepm_t, L, hr_f);
		    end_if;
		    if p_f then
			print(Unquoted, "reducing to " , h["elem"]);
			print(Unquoted, "-----------------------"); 
		    end_if;
		    if not dom::iszero(h["elem"]) then
			h["nmp"] := {};
			if h["elem"] <> g["elem"] then 
			    h["crit"] := h["lmon"][2];				
			    tmp := [];
			    for k from nops(T) downto 1 do
				if dom::exponentLess(h["lmon"][2], T[k]["lmon"][2]) then
				    tmp := append(tmp, T[k]);
				    delete T[k];
				else
				    break;
				end_if;
			    end_for;
			    T := append(T, h);
			    if tmp <> [] then
				Q := listlib::singleMerge(tmp, Q, 
				(l1, l2) -> (DOM::exponentLess(l1["lmon"][2], l2["lmon"][2])));
			    end_if;
			else  // no reduction
			    h["crit"] := g["crit"];
			    T := append(T, h);
			end_if;
		    end_if; 
		end_if;
	    until noMore end_repeat;
	until Q = [] end_repeat;
	map(T, l -> DOM::multcoeffs(l["elem"], 1 / l["lmon"][1]));
    end_proc;
    
invMoraNormalForm := proc(f:dom, F:Type::ListOf(dom), L:DOM_DOMAIN)
    local DOM:DOM_DOMAIN, i;
    begin
	if args(0) < 2 or args(0) > 3 then
	    N_ERR;
	end_if;
	DOM := dom;;
	(dom::intInvMoraNForm(table("elem" = f, "lmon" = [dom::lcoeff(f), dom::lexponent(f)]), 
	    map(F, l -> table("elem" = l, "lmon" = [DOM::lcoeff(l), DOM::lexponent(l)])), 
		L::separationTableMult([args(2)[i]["lmon"][2] $ i = 1..nops(args(2))]), L))["elem"];
    end_proc;
    
intInvMoraNForm := proc(f, G, sep_t, L)  
    local hmindiff, hg, g, hdiff, d, tmp, hcf, hd, ev; 
    begin
	repeat
	    hmindiff := infinity;
	    hg := NIL;
	    for g in G do
		hdiff := min(max(g["lmon"][2][1] - f["lmon"][2][1], 0), 0);
		if not iszero((d := L::isInvDivisor(g["lmon"][2], subsop(f["lmon"][2], 
		    1 = f["lmon"][2][1] + hdiff), sep_t[g["lmon"][2]]))) 
		then
		    if hmindiff > hdiff then
			hmindiff := hdiff;
			hg := g;
			hd := d;
		    end_if;
		end_if;
	    end_for;
	    if hmindiff <> infinity then
		if not iszero(hmindiff) then    
		    G := append(G, f);
		    sep_t := L::separationTableMult([g["lmon"][2] $ g in G]); 
		end_if;
		tmp := dom::convert(hv^hdiff) * f["elem"];
		f := dom::poly2list(dom::reduce(tmp, hg["elem"], hg["lmon"], 
		    [f["lmon"][1], subsop(f["lmon"][2], 1 = f["lmon"][2][1] + hdiff)], hd));
		hcf := min(ev[2][1] $ ev in f);
		if not iszero(hcf) then
		    f := map(f, l -> [l[1], subsop(l[2], 1 = l[2][1] - hcf)]);
		end_if;
		f := dom::convert(f);
		f := table("elem" = f, 	
		    "lmon" = [dom::lcoeff(f), dom::lexponent(f)]);
	    end_if;
	until (hmindiff = infinity) end_repeat;
	f;
    end_proc;
    
    _tmp_invnform := dom::involutiveNormalForm;
    
    _tmp_invnform := dom::internalInvolutiveNormalForm;

setMoraNormalForm := proc()
    begin
	dom::involutiveNormalForm := dom::invMoraNormalForm;
	dom::internalInvolutiveNormalForm := dom::intInvMoraNForm;
    end_proc;
    
resetNormalForm := proc()
    begin
	dom::involutiveNormalForm := dom::_tmp_invnform;
	dom::internalInvolutiveNormalForm := dom::_tmp_invnform;   
    end_proc;
	        
begin
    if args(0) < 1 or args(0) > 2 then
	error("wrong number of arguments");
    end_if;
    if (domtype(W) <> DOM_DOMAIN
	or not W::constructor = Dom::WeylAlgebra)
    then
	error("wrong type of arguments");
    end_if;
    if not iszero(contains(W::variables, hv)) then
	error("incorrect homogenisation variable");
    end_if;
    HVar_l := [W::variables[i] $ i = 1..(W::numVars / 2)];
    HR := W::coeffRing;
    HOrder := Dom::MonomOrdering(
        Matrix(
          linalg::stackMatrix(/*dense*/matrix([[1 $ (W::numVars + 1)]]),
                              linalg::concatMatrix(/*dense*/matrix([[0] $ linalg::nrows(W::orderMatrix)]),
                                                   W::orderMatrix)
                             )
              )                 );
end_domain:

unalias(REP(x), N_ERR, A_ERR):
