/* Dom::InvolutiveBasis(gens, ID) - Domain representing the involutive basis of the 
    module generated by the elements gens w.r.t. the involutive division ID 

Author: Marcus Hausdorf, Uni Heidelberg
        (marcus.hausdorf@iwr.uni-heidelberg.de)
Created: 	Mon Aug 25 17:26:57 2003
Last update: 	Fri Sep 19 11:50:39 2003
*/

alias(A_ERR = error("wrong type of arguments"),
      N_ERR = error("wrong number of arguments"));
      
domain Dom::InvolutiveBasis(gens:DOM_LIST, ID)
local tmp, gtype, mdim, b_l, le_l, le_t, bl, mv_l, k, mvt_l, le2_l, nV, i, lt_l,
    _list2table;
inherits Dom::BaseDomain;
category Cat::BaseCategory;

expr := (() -> FAIL);

convert := (() -> FAIL);

basisElements := b_l;

lexps := le_l;

lterms := lt_l;

multVars := mv_l;

basisInfo := zip(b_l, zip(lt_l, mv_l, [id]), (l1, l2) -> [l1, op(l2)]); 

involutiveNForm := proc(x:gtype)
    local hr_f, a1, a2;
    begin
	if not iszero(contains([args(2..args(0))], hold(Head))) then
	    hr_f := TRUE;
	    a1 := table("elem" = x, "mons" = [gtype::leader(x)]);
	else
	    hr_f := FALSE;
	    a1 := table("elem" = x, "mons" = gtype::allMonomials(x));
	end_if;
	a2 := map(b_l, l -> table("elem" = l, "lmon" = gtype::leader(l)));
	if mdim = 1 then
	    return((gtype::internalInvolutiveNForm(a1, a2, mvt_l, ID, hr_f, FALSE))["elem"]);
	else
	    return((gtype::internalInvolutiveNForm(a1, a2, mvt_l, ID, hr_f, FALSE))[1]["elem"]);
	end_if;
    end_proc;
    
involutiveSRep := proc(x:gtype)
    local hr_f, a1, a2, res;
    begin
	if not iszero(contains([args(2..args(0))], hold(Head))) then
	    hr_f := TRUE;
	    a1 := table("elem" = x, "mons" = [gtype::leader(x)]);
	else
	    hr_f := FALSE;
	    a1 := table("elem" = x, "mons" = gtype::allMonomials(x));
	end_if;
	a2 := map(b_l, l -> table("elem" = l, "lmon" = gtype::leader(l)));	
	res := gtype::internalInvolutiveNForm(a1, a2, mvt_l, ID, hr_f, TRUE);
	if mdim = 1 then
	    return([res[2], res[1]["elem"]]);
	else
	    return([res[3], res[1]["elem"]]);
	end_if;
    end_proc;
    
syzygies := proc()
    local res_l, allvars, z, nmp, tt, tmp, tmp2, i;
    begin
	res_l := [];
	allvars := {i $ i = 1..nV};
	if mdim = 1 then
	    tt := gtype;
	else
	    tt := gtype::coeffRing;
	end_if;
	z := [0 $ nV];
	for k from 1 to bl do
	    for nmp in (allvars minus mv_l[k]) do
		tmp := tt::exponent2monomial(subsop(z, nmp = 1));
		tmp2 := dom::involutiveSRep(gtype::_mult(tmp, b_l[k]))[1];
		res_l := append(res_l, subsop(tmp2, k = tmp2[k] - tmp));
	    end_for;
	end_for;
	res_l;
    end_proc;
    
standardMonomials := proc()
    local res_l, k, tmp, lem_l;
    begin
	if mdim = 1 then
	    return(map(ID::combinatorialDecomposition(zip(le_l, mv_l, 
	    (l1, l2) -> [l1, l2])), l -> [gtype::exponent2monomial(l[1]), 
		    map(l[2], l3 -> gtype::variables[l3])]));
	else
	    res_l := [];
	    lem_l := zip(le_l, mv_l, (l1, l2) -> [l1, l2]);
	    for k from 1 to mdim do
		tmp := map(select(lem_l, l -> bool(l[1][2] = k)), l2 -> [l2[1][1], l2[2]]);
		if tmp <> [] then
		    res_l := append(res_l, map(ID::combinatorialDecomposition(tmp), 
			l -> [gtype::exponent2monomial([l[1], k]), 
			    map(l[2], l2 -> gtype::coeffRing::variables[l2])]));
		end_if;
	    end_for;
	    return(map(res_l, op));
	end_if; 
    end_proc;
    
hilbertSeries := proc(var:DOM_IDENT)
    option remember;
    local i;
    begin
	normal(mdim / (1 - var)^nV - _plus((var^(_plus(op(le2_l[i])))) 
	    / ((1 - var)^nops(mv_l[i])) $ i = 1..bl));
    end_proc;
    
hilbertPolynomial := proc(var:DOM_IDENT)
    local hs, d, hn, i, _dummy, binomial2, tmp;
    begin
	binomial2 := proc(n, s, r)
	    local i;
	    begin
		if s = 1 then return(1) end_if;
		_mult((s - 1 + n - r) $ i = 1..(s - 1)) / fact(s - 1);
	    end_proc;
	
	_dummy := genident();
	hs := dom::hilbertSeries(_dummy);
	d := degree(denom(hs));
	if iszero(d) then return(poly(0, [var], Dom::Rational)) end_if;
	if testtype(d, Type::Even) then
	    hn := poly(numer(hs), [_dummy], Dom::Integer);
	else
	    hn := poly(-numer(hs), [_dummy], Dom::Integer);
	end_if;
	tmp := _plus(poly(coeff(hn, _dummy, i) * binomial2(var, d, i), [var], Dom::Rational) 
	    $ i = 0..(degree(hn, _dummy)));
	delete _dummy;	 
	tmp;
    end_proc;
    
hilbertFunction := proc(d:Type::NonNegInt)
    option remember;
    local _dummy, tmp;
    begin
	_dummy := genident();
	tmp := op(series(dom::hilbertSeries(_dummy), _dummy, d + 1), 5);
	delete _dummy;
	if (d + 1) > nops(tmp) then 
	    return(0);
	else
	    return(tmp[d + 1]);
	end_if;
    end_proc;

hilbertRegularity := proc()
    option remember;
    local _dummy, tmp;
    begin
	_dummy := genident();
	tmp := dom::hilbertSeries(_dummy);
	if iszero(degree(denom(tmp), _dummy)) then
	    tmp := degree(numer(tmp), _dummy) + 1;
	else
	    tmp := degree(numer(tmp), _dummy);
	end_if;
	delete _dummy;
	return(tmp);
    end_proc;

begin
    //check arguments
    if args(0) < 2 then
	N_ERR;
    end_if;
    tmp := {op(map(gens, domtype))};
    if nops(tmp) = 1 then
	tmp := op(tmp, 1);
	if domtype(tmp) = DOM_DOMAIN then
	    if tmp::hasProp(Cat::SolvableAlgebra) then
		gtype := tmp;
		mdim := 1;
	    elif bool(tmp::constructor = Dom::SolvableModule) then
		gtype := tmp;
		mdim := gtype::dimension;
	    else
		A_ERR;
	    end_if;
	else
	    A_ERR;
	end_if;
    else
	A_ERR;
    end_if;
    if domtype(ID) <> DOM_DOMAIN
	or ID::hasProp = FAIL
	or not ID::hasProp(Cat::InvolutiveDivision)
    then
	A_ERR;
    end_if;
    //compute involutive basis
    nV := ID::numVars;
    _list2table := (l -> table(op(map(l, l1 -> l1[1] = l1[2]))));    
    b_l := gtype::involutiveComplete(gens, ID, args(3..args(0)));
    le_l := map(b_l, l -> gtype::leader(l)[2]);
    lt_l := map(le_l, gtype::exponent2monomial);
    bl := nops(le_l);
    le_t := table(zip(le_l, [i $ i = 1..bl], (l1, l2) -> l1 = l2));
    if mdim = 1 then
	tmp := ID::sepListMult(le_l);
       	mvt_l := _list2table(tmp);
	mv_l := [mvt_l[i] $ i in le_l];
	le2_l := le_l;
    else
	mv_l := [0 $ bl];
	mvt_l := [table() $ mdim];
	le2_l := map(le_l, op, 1);
	for k from 1 to mdim do
	    tmp := ID::sepListMult(map(select(le_l, l -> bool(l[2] = k)), op, 1));
	    mvt_l[k] := _list2table(tmp);
	end_for;
	mv_l := [mvt_l[i[2]][i[1]] $ i in le_l];
    end_if;
end_domain;

unalias(A_ERR, N_ERR):
