/*
Dom::SolvablePolynomial -- Domain of Polynomials in Cat::SolvableAlgebra

Author: Marcus Hausdorf, Uni Mannheim
        (hausdorf@math.uni-mannheim.de)
Created: 	Fri Oct 18 10:28:29 2002
Last update: 	Tue Sep  9 11:53:31 2003
*/

/* This domain directly inherits from Dom::MultivariatePolynomial; the
only difference is that it is a member of Cat::SolvableAlgebra, so the
computation of involutive bases is possible. */

alias(REP(x) = extop(x, 1)):

domain Dom::SolvablePolynomial(Vars=[x,y,z]:Type::ListOf(DOM_IDENT), 
                                   R=Dom::ExpressionField(normal), 
                                   Order=DegInvLexOrder)
local Rzero, Rone, nV, MO;
inherits Dom::MultivariatePolynomial(Vars, R, Order);
category Cat::SolvableAlgebra(R),
    (if nops(Vars) = 1 then
         Cat::UnivariatePoynomial(R)
     else
         Cat::Polynomial(R)
     end_if);
axiom (if R::hasProp(Ax::normalRep) then Ax::normalRep end_if),
      (if R::hasProp(Ax::canonicalRep) then Ax::canonicalRep end_if);
 
coeffRing := R;

variables := Vars;

allMonomials := dom::poly2list; 

poly2list := (p -> (if dom::iszero(p) then return([])
    elif nV > 1 then return(poly2list(REP(p)))
    else return(map(poly2list(REP(p)), (l -> [l[1], [l[2]]]))) end_if));

exponent2monomial := (mu -> new(dom, poly([[Rone, mu]], Vars, R)));

monomial2exponent := (p -> dom::poly2list(p)[1][2]);

coeff := proc(x:dom)
    local tmp;
    begin
	if args(0) = 1 then
	    return(coeff(REP(x)));
	else
	    tmp := coeff(REP(x), args(2..args(0)));
	    if domtype(tmp) = DOM_POLY then
		return(new(dom, tmp));
	    else
		return(tmp);
	    end_if;
	end_if;
    end_proc;

leader := (x -> op(dom::allMonomials(dom::lmonomial(x))));

lmonomial := proc(x:dom)
    local DOM; 
    begin
	DOM := dom;
	if args(0) = 1 then
	    return(new(dom, lmonomial(REP(x), Order)));
	elif args(2) = hold(Rem) then
	    return(map(lmonomial(REP(x), Order, hold(Rem)), 
		l -> (new(DOM, l))));
	elif args(0) = 2 then
	    return(new(dom, lmonomial(REP(x), args(2))));
	else 
	    return(map(lmonomial(REP(x), args(2), args(3)), 
		l -> (new(DOM, l))));
	end_if;
    end_proc;

// reduce := proc(f:dom, g:dom, mu:DOM_LIST, t:DOM_LIST)
//     local d, tmp;
//     begin
// 	if args(0) = 5 then
// 	    d := args(5);
// 	else
// 	    d := zip(t[2], mu[2], _subtract);
// 	    if select(d, l -> bool(l < 0)) <> [] then
// 		error("cannot reduce");
// 	    end_if;
// 	end_if;
// 	tmp := dom::exponent2monomial(d) * g;
// 	dom::multcoeffs(f, dom::lcoeff(tmp)) - dom::multcoeffs(tmp, t[1]);
//     end_proc;

exponentOrder := Order;

numVars := nV;

begin
    if args(0) < 1 or args(0) > 3 then
	error("wrong number of arguments");
    end_if;
    if (domtype(R) <> DOM_DOMAIN
	or R::hasProp = FAIL
	or not R::hasProp(Cat::Ring))
    then
	error("second argument must be in category Cat::Ring");
    end_if;
    if domtype(Vars) <> DOM_LIST then
	error("first argument must be a list");
    end_if;
    if (not contains({LexOrder, DegreeOrder, DegInvLexOrder}, Order)
	and domtype(Order) <> Dom::MonomOrdering)
    then
	error("unknown monomial ordering");
    end_if;
    nV := nops(Vars);
    MO := Dom::MonomOrdering;
    case Order 
    of LexOrder do
	Order := MO(Lex(nV));
	break;
    of DegreeOrder do
	Order := MO(DegLex(nV));
	break;
    of DegInvLexOrder do
	Order := MO(DegInvLex(nV));
	break;
    end_case;
    Rzero := R::zero;
    Rone := R::one;
end_domain:

unalias(REP(x)):
