/*-
 Output of dom
-*/
Graph::print := proc()
begin 
   "Graph(...)"
end_proc:

/*-
 printGraphInformation - prints some information of a Graph
 Parameters:
   G - Graph
-*/
Graph::printGraphInformation := proc(G : Graph)
save PRETTYPRINT;
begin

  PRETTYPRINT := FALSE;
  if Graph::getVertices(G) = FAIL then
    print(Unquoted, "Vertices: no vertices.");
  else
    print(Unquoted, "Vertices: ".expr2text(Graph::getVertices(G)));
  end_if;
 
  if Graph::getEdges(G) = FAIL then
     print(Unquoted, "Edges: no edges.");
  else
     print(Unquoted, "Edges: ".expr2text(Graph::getEdges(G)));
  end_if;

  if Graph::getVertexWeights(G) = FAIL then
    if nops(Graph::getVertices(G)) > 0 then
      print(Unquoted, "Vertex weights: no vertex weights.");     
    end_if; 
  else
     print(Unquoted, "Vertex weights: ".expr2text(op(Graph::getVertexWeights(G)))." (other existing vertices have no weight)");  
  end_if;

  if Graph::getEdgeDescriptions(G) = FAIL then
    if nops(Graph::getEdges(G)) > 0 then
      print(Unquoted, "Edge descriptions: no edge descriptions.");
    end_if;
  else
     print(Unquoted, "Edge descriptions: ".expr2text(op(Graph::getEdgeDescriptions(G))));  
  end_if;

  if Graph::getEdgeWeights(G) = FAIL then
    if nops(Graph::getEdges(G)) > 0 then
       print(Unquoted, "Edge weights: no edge weights.");
    end_if;
  else
     print(Unquoted, "Edge weights: ".expr2text(op(Graph::getEdgeWeights(G)))." (other existing edges have no weight)"); 
  end_if;

  if Graph::getEdgeCosts(G) = FAIL then
    if nops(Graph::getEdges(G)) > 0 then
      print(Unquoted, "Edge costs: no edge costs.");
    end_if;
  else
    print(Unquoted, "Edge costs: " . expr2text(op(Graph::getEdgeCosts(G))) . " (other existing edges have costs zero)"); 
  end_if;

  if Graph::getEdgesLeaving(G) = FAIL then
    if nops(Graph::getEdges(G)) > 0 then
      print(Unquoted, "Adjacency list (out): no edges.");
    end_if;
  else
     print(Unquoted, "Adjacency list (out): ".expr2text(op(Graph::getEdgesLeaving(G)))); 
  end_if;

  if Graph::getEdgesEntering(G) = FAIL then
    if nops(Graph::getEdges(G)) > 0 then
      print(Unquoted, "Adjacency list (in): no edges.");
    end_if;
  else
     print(Unquoted, "Adjacency list (in): ".expr2text(op(Graph::getEdgesEntering(G))));
  end_if;

  if Graph::isDirected(G) then
    print(Unquoted, "Graph is directed.");
  else
    print(Unquoted, "Graph is undirected.");
  end_if;

  null();

end_proc:

/*-
 printVertexInformation - prints detailed information about the vertices of a Graph
 Parameters:
   G - Graph
-*/
Graph::printVertexInformation := proc(G : Graph)
save PRETTYPRINT;
local v, _vertices, _vertexWeights;
begin

  PRETTYPRINT := FALSE;
  if Graph::getVertices(G) = FAIL then
    print(Unquoted, "No vertices.");
  else
    print(Unquoted, "Vertices existing in the graph:");
    print(Unquoted, "-------------------------------");
    _vertices := Graph::getVertices(G);
    _vertexWeights := Graph::getVertexWeights(G);
    for v in _vertices do
      print(NoNL, "Vertex " . expr2text(v) . " has weight ");
      if (_vertexWeights = FAIL) or (contains(_vertexWeights,v) = FALSE)then
        print(Unquoted, "None");
      else
        print(Unquoted, expr2text(_vertexWeights[v]));
      end_if;
    end_for;
  end_if;
  null();
end_proc:

/*-
 printEdgeInformation - prints detailed information about the edges of a Graph
 Parameters:
   G - Graph
-*/
Graph::printEdgeInformation := proc(G)
save PRETTYPRINT;
begin
  if testargs() then
    Graph::graphCheck(G);
  end_if;

  PRETTYPRINT := FALSE;
  if Graph::getEdges(G) = FAIL then
    print(Unquoted, "No edges.");
  else
    print(Unquoted, "Edges existing in the graph:");
    print(Unquoted, "----------------------------");
    print(Unquoted, expr2text(op(Graph::getEdges(G))));
  end_if;
  null();
end_proc:

/*-
 printEdgeDescriptionInformation - prints detailed information about the edge descriptions of a Graph
 Parameters:
   G - Graph
-*/
Graph::printEdgeDescInformation := proc(G)
save PRETTYPRINT;
local _edge, _edges, _edgeDescriptions;
begin
  if testargs() then
    Graph::graphCheck(G);
  end_if;

  PRETTYPRINT := FALSE;
  _edges := Graph::getEdges(G);
  _edgeDescriptions := Graph::getEdgeDescriptions(G);
  if _edgeDescriptions = FAIL then
    print(Unquoted, "No edge descriptions defined.");
  else
    print(Unquoted, "Edge descriptions existing in the graph:");
    print(Unquoted, "----------------------------------------");

    for _edge in _edges do
      print(NoNL, "Edge " . expr2text(_edge) . " = ");
      if (_edgeDescriptions = FAIL) or (contains(_edgeDescriptions, _edge) = FALSE)then
        print(Unquoted, "no description.");
      else
        print(Unquoted, expr2text(_edgeDescriptions[_edge]) );
      end_if;
      // print(Unquoted, "Edge " . expr2text(_edge) . " has cost " . expr2text(_edgeCosts[_edge])); 
    end_for;

  end_if;
 null();
end_proc:

/*-
 printEdgeCostInformation - prints detailed information about the edge costs of a Graph
 Parameters:
   G - Graph
-*/
Graph::printEdgeCostInformation := proc(G)
save PRETTYPRINT;
local _edge, _edges, _edgeCosts;
begin
  if testargs() then
    Graph::graphCheck(G);
  end_if;

  PRETTYPRINT := FALSE;
  _edges := Graph::getEdges(G);
  _edgeCosts := Graph::getEdgeCosts(G);
  if _edgeCosts = FAIL then
    print(Unquoted, "No edge costs defined.");
  else
    print(Unquoted, "Edge costs existing in the graph:");
    print(Unquoted, "----------------------------------------");
    for _edge in _edges do
      print(NoNL, "Edge " . expr2text(_edge) . " has cost ");
      if (_edgeCosts = FAIL) or (contains(_edgeCosts, _edge) = FALSE)then
        print(Unquoted, "0");
      else
        print(Unquoted, expr2text(_edgeCosts[_edge]));
      end_if;
      // print(Unquoted, "Edge " . expr2text(_edge) . " has cost " . expr2text(_edgeCosts[_edge])); 
    end_for;
  end_if;
  null();
end_proc:

/*-
 printEdgeWeightInformation - prints detailed information about the edge weights of a Graph
 Parameters:
   G - Graph
-*/
Graph::printEdgeWeightInformation := proc(G)
save PRETTYPRINT;
local _edge, _edges, _edgeWeights;
begin
  if testargs() then
    Graph::graphCheck(G);
  end_if;

  PRETTYPRINT := FALSE;
  _edges := Graph::getEdges(G);
  _edgeWeights := Graph::getEdgeWeights(G);

  if _edgeWeights = FAIL then
    print(Unquoted, "No edge weights defined.");
  else
    print(Unquoted, "Edge weights existing in the graph:");
    print(Unquoted, "----------------------------------------");
    for _edge in _edges do
      print(NoNL, "Edge " . expr2text(_edge) . " has weight ");
      if (_edgeWeights = FAIL) or (contains(_edgeWeights, _edge) = FALSE)then
        print(Unquoted, "0");
      else
        print(Unquoted, expr2text(_edgeWeights[_edge]));
      end_if;
      //print(Unquoted, "Edge " . expr2text(_edge) . " has weight " . expr2text(_edgeWeight[_edge])); 
    end_for;
  end_if;
  null();
end_proc:

// End of file
null():
