// $Id: ceil.mu,v 1.9 2002/05/29 12:14:21 ccr Exp $

DOM_INTERVAL::ceil := proc(a)
begin
  a := hull(a); // handle unions etc.

  if op(a,2) = RD_INF then infinity;
  // ceil(RD_NINF) cannot be -infinity, since there
  // are finite numbers represented as RD_NINF.
  // We return the smallest integer pari can handle on 32 bit.
  // This takes some while, but it's the best correct answer
  // we can give.
  elif op(a,2) = RD_NINF then /* too slow - _plus(2^i$i=0..2097023) */
        -2^2097023;
  else ceil(op(Re(a),2)+I*op(Im(a),2)); end_if;
end_proc:

DOM_INTERVAL::rhs :=
proc(iv)
begin
  iv := hull(iv);

  if iszero(Im(iv)) then
    op(iv, 2);
  else
    op(iv, [1,2]) + I*op(iv,[2,2]);
  end_if;
end_proc:


DOM_INTERVAL::floor := proc(a)
begin
  a := hull(a); // handle unions etc.

  if op(a,1) = RD_NINF then -infinity;
  // floor(RD_INF) cannot be infinity, since there
  // are finite numbers represented as RD_INF.
  // We return the largest integer pari can handle on 32 bit.
  // This takes some while, but it's the best correct answer
  // we can give.
  elif op(a,1) = RD_INF then  /* too slow _plus(2^i$i=0..2097023) */
        2^2097023;
  else floor(op(Re(a),1)+I*op(Im(a),1)); end_if;
end_proc:

DOM_INTERVAL::lhs :=
proc(iv)
begin
  iv := hull(iv);

  if iszero(Im(iv)) then
    op(iv, 1);
  else
    op(iv, [1,1]) + I*op(iv,[2,1]);
  end_if;
end_proc:

DOM_INTERVAL::trunc := proc(a)
begin
  a := hull(a);

  if 0 in a then 0
  elif op(a, 0) = hold(hull) then
    if a > 0 then DOM_INTERVAL::floor(a)
    else DOM_INTERVAL::ceil(a);
    end_if:
  else DOM_INTERVAL::trunc(Re(a)) + I*DOM_INTERVAL::trunc(Im(a));
  end_if;
end_proc:

DOM_INTERVAL::round := proc(a)
begin
  a := hull(a);

  if lhs(a) in {-infinity, RD_NINF}
    or rhs(a) in {infinity, RD_INF} then
    if lhs(a) in {-infinity, RD_NINF} then
      if rhs(a) in {infinity, RD_INF}
        then 0
      else lhs(a)
      end_if;
    else
      rhs(a)
    end_if;
  else
    round(op(Re(a),1)/2 + op(Re(a),2)/2
          + I*op(Im(a),1)/2 + I*op(Im(a),2)/2);
  end_if;
end_proc:
