// $Id: intTaylor.mu,v 1.4 2002/12/18 10:33:41 ccr Exp $

// interval::intTaylor -- integrate using Taylor series bounds
// as provided by interval::Taylor.

/* TODO

 Adaptive order?  The current value of 12 has been chosen based on the
 example sin(x^2), x=0..1, where it yields the best balance between
 the time required for subdivisions and the time spent calculating the
 taylor series.  Other examples may (and do) differ.

 The precision goal should in some way be a relative error.  This is
 conceptually slightly problematic.

 bail out when 
Info: linear part: 4.264487423e2525222 ... RD_INF

 Alternatively, check first if RD_NINF is also attained, otherwise
 return 4.264487423e2525222 ... RD_INF as the result.  Case in question:
 interval::intTaylor(exp(x^(-1/x^2)), x=0..1)

*/

// anon proc to get file scope static vars

proc()
  option escape;
  local wideTaylor, narrowTaylor,
  getWideT, getNarrowT;
begin
  wideTaylor := table();
  narrowTaylor := table();
  
  getWideT :=
  proc(order)
  begin
    if not contains(wideTaylor, order) then
      wideTaylor[order] := interval::Taylor(order, `# #x`, 0...1);
    end_if;
    wideTaylor[order];
  end_proc:

  getNarrowT :=
  proc(order)
  begin
    if not contains(narrowTaylor, order) then
      narrowTaylor[order] := interval::Taylor(order, `# #x`, hull(1/2));
    end_if;
    narrowTaylor[order];
  end_proc:

  interval::intTaylor :=
  proc(f_in, xr, order=12)
    local x, f01, f, rl, rr, todo, sum, eps,// order,
          T, subdiv, Tfp, i, h, j, good, Tn, Tnfp, DIGITS_in,
          partsum, left, right;
    save DIGITS;
  begin
    DIGITS_in := DIGITS;
    if type(xr) <> "_equal" then
      error("wrong type of second arg");
    end_if;
    x := lhs(xr);
    [rl, rr] := [op(rhs(xr))];
    // scale the integrand to 0...1
    f01 := (rr-rl)*subs(f_in, x = rl+`# #x`*(rr-rl));
    eps := 0.25*10.0^(3-DIGITS); // TODO: is this reasonable?
    todo := {[]}; // [] denotes no subdivisions, hence 0...1.
    sum := 0:
    T := getWideT(order);
    Tn := getNarrowT(order);
    while todo <> {} do
      subdiv := op(todo, 1);
      todo := todo minus {subdiv};
      good := FALSE;
      DIGITS := max(DIGITS_in, 5+floor(nops(subdiv)/3));
      left := _plus(subdiv[i]*2^-i$i=1..nops(subdiv));
      right := left + 2^-nops(subdiv);
      h := hull(2^(-nops(subdiv)));
// scale f, so we always integrate over 0...1
// TODO: this method is wasteful.
      f := h*subs(f01, `# #x` = eval(left+`# #x`*h));
      Tfp := T::taylorPoly(T(f));
//      h := h/2;
      for i from order downto 0 step 2 do
	if DOM_INTERVAL::width(coeff(Tfp, i))*h^i < eps*(i+1) then
	  Tnfp := Tn::taylorPoly(Tn(f));
          partsum := (_plus((coeff(Tnfp, 2*j)
                             /(2*j+1)/2^(2*j))$j=0..i/2-1))
                     + coeff(Tfp,i)/(i+1)/2^i;
          sum := sum + partsum;
          good := TRUE;
          break;
	end_if;
      end_for;
      if not good then
        // add subdivisions to the todo list
        userinfo(2, "subdividing ".expr2text(subdiv));
	userinfo(5, "Taylor poly: ".expr2text(Tfp));
        userinfo(3, "linear part: ".expr2text(coeff(Tfp, 0)));
        todo := todo union {subdiv.[0], subdiv.[1]};
      else
        userinfo(1, "value over ".expr2text(left)."..".expr2text(right).
                 ": ". expr2text(partsum));
      end_if;
    end_while;
    sum;
  end_proc:
end_proc(): // the anonymous one

null():

