// $Id: intTaylorDiff.mu,v 1.4 2003/08/12 13:31:56 ccr Exp $

// interval::intTaylorDiff -- integrate using Taylor series bounds
// using symbolic differentiation

/* TODO

 Adaptive order?  The current value of 6 has been chosen based on the
 example arctan(sqrt(2+t^2))/(1+t^2)/sqrt(2+t^2), where it yields the
 best balance between the time required for subdivisions and the time
 spent calculating the taylor series.  Other examples may (and do)
 differ.

 The precision goal should in some way be a relative error.  This is
 conceptually slightly problematic.

 bail out when 
Info: linear part: 4.264487423e2525222 ... RD_INF

 Alternatively, check first if RD_NINF is also attained, otherwise
 return 4.264487423e2525222 ... RD_INF as the result.  Case in question:
 interval::intTaylor(exp(x^(-1/x^2)), x=0..1)

*/

interval::_evalListOfDiffs := 
  proc(Lf, x, iv)
    local i, res;
  begin
    res := [FAIL$nops(Lf)];
    res[-1] := interval::evalSimple(Lf[-1], x=iv);
    for i from nops(Lf)-1 downto 1 do
      if iszero(Im(res[i+1])) and not 0 in res[i+1] then
	// next lower derivative is monotonous
	res[i] := [interval::evalSimple(Lf[i], x = subsop(iv, 2=lhs(iv))),
		   interval::evalSimple(Lf[i], x = subsop(iv, 1=rhs(iv)))];
	if res[i+1] < 0 then
	  res[i] := RD_NINF ... res[i][1]
		   intersect res[i][2] ... RD_INF;
	else
	  res[i] := RD_NINF ... res[i][2]
		   intersect res[i][1] ... RD_INF;
	end_if;
      else
	res[i] := interval::evalSlope(Lf[i], x=iv);
      end_if
    end_for;
    res;
  end_proc:

interval::intTaylorDiff :=
  proc(f, xr, order=6)
    local x, df, rl, rr, todo, sum, eps, range, Tw, Tn, cent;
    save DIGITS;
  begin
    DIGITS_in := DIGITS;
    if type(xr) <> "_equal" then
      error("wrong type of second arg");
    end_if;
    x := lhs(xr);
    [rl, rr] := [op(rhs(xr))];
    %if interval::genplot=TRUE then
      found_encl := table();
    end_if;
    df := interval::autodiff(f, order);
//    eps := 0.25*10.0^(3-DIGITS); // TODO: is this reasonable?
    eps := 0.01;
    todo := {[rl, rr]};
    sum := 0;
    while todo <> {} do
      range := op(todo, 1);
      todo := subsop(todo, 1=null());
      good := FALSE;
      h := hull(range[2]-range[1]);
      // TODO: adaptively increase DIGITS
//      DIGITS := max(DIGITS_in, 5+floor(nops(subdiv)/3));
      Tw := df(hull(range));
      
      %if interval::genplot=TRUE then
	min_encl := [RD_NINF ... RD_INF];
      end_if;
      
      for i from order downto 0 step 2 do
	%if interval::genplot=TRUE then
	  if DOM_INTERVAL::width(h*Tw[i+1]*(h/2)^i/(i+1))
	     < DOM_INTERVAL::width(min_encl[1]) then
	    Tn := df(hull((range[2]+range[1])/2));
	    min_encl := [h*Tw[i+1]*(h/2)^i/i+1].Tn;
	  end_if;
	end_if;
	
	// TODO: relative error
	if DOM_INTERVAL::width(Tw[i+1])*h^(i) < eps*(i+1) then
	  Tn := df(hull((range[2]+range[1])/2));
	  partsum := h*_plus((Tn[2*j+1]*(h/2)^(2*j)
			      /(2*j+1))$j=0..i/2-1) +
		    h*Tw[i+1]*(h/2)^(i)/(i+1);
          sum := sum + partsum;
          good := TRUE;
          break;
	end_if;
      end_for;
      
      %if interval::genplot=TRUE then
	if DOM_INTERVAL::width(min_encl[1]) < 10 then
	  found_encl[hull(cent)] := min_encl;
	end_if;
      end_if;

      if not good then
        // add subdivisions to the todo list
        userinfo(2, "subdividing ".expr2text(range));
	userinfo(5, "Taylor poly: ".expr2text(Tw));
        userinfo(3..4, "linear part: ".expr2text(Tw[1]));
	cent := (range[2]+range[1])/2;
	assert(cent in hull(range));
        todo := todo union {[range[1], cent], [cent, range[2]]};
      else
        userinfo(2, "value over ".expr2text(range[1])."..".expr2text(range[2]).
                 ": ". expr2text(partsum));
	userinfo(1, expr2text(nops(todo))." intervals to go");
      end_if;
    end_while;
    sum;
  end_proc:

null():

