// $Id: psi.mu,v 1.4 2004/06/18 12:49:41 ccr Exp $

// psi, the logarithmic derivative of gamma.

// these series for |z| < 1 compute psi(1+z).
// the first one is not used currently, since it seems to perform badly:
DOM_INTERVAL::_psi_small_z_nonzero :=
proc(z)
  local n, z2, z2n, sum1, sum2, prec, zeta2n;
begin
  assert(abs(z) < 1);
  sum1 := 1/(2*z)-interval(PI/2*cot(PI*z))-1/(1-z^2) +
	1 - hull(EULER);
  sum2 := hull(1/4);
  z2 := z^2;
  n := 0;
  z2n := 1; // z^(2*n)
  prec := 10.0^(-DIGITS);
  while lhs(abs(z2n*sum2)) > prec do
    n := n+1;
//    z2n := z2n*z2;
    z2n := z^(2*n);
    zeta2n := DOM_INTERVAL::zeta(2*n+1)-1;
    sum1 := sum1 - zeta2n * z2n;
    sum2 := sum2 - zeta2n;
  end_while;
  if iszero(Im(z)) then
    sum1 + (-1...1)*abs(z2*z2n)*sum2;
  else
    sum1 + (-1-I...1+I)*abs(z2*z2n)*sum2;
  end_if;
end_proc:

// this one should be expected to be worse, but actually is better:
DOM_INTERVAL::_psi_small_z :=
proc(z)
  local n, zn, sum1, sum2, prec, zetan;
begin
  assert(abs(z) < 1);
  sum1 := - hull(EULER);
  sum2 := hull(1);
  n := 1;
  zn := 1; // (-z)^(n-1)
  prec := 10.0^(2-DIGITS);
  while lhs(abs(zn*sum2)) > prec do
    n := n+1;
    zn := -zn*z;
    zetan := DOM_INTERVAL::zeta(n);
    sum1 := sum1 - zetan * zn;
    sum2 := sum2 - zetan;
  end_while;
  if iszero(Im(z)) then
    sum1 + (-1...1)*abs(zn)*sum2;
  else
    sum1 + (-1-I...1+I)*abs(zn)*sum2;
  end_if;
end_proc:

// use the duplication formula to achieve abs(z) < 1.
// The use of A&S 6.3.8, unfortunately, leads to
// exponentially many calls to the series above
// for increasing Im(z).
DOM_INTERVAL::_psi_doubling :=
proc(iv)
  local k, n, z, pow2;
begin
  if op(abs(iv-1), 2) < 1/2 then
    return(DOM_INTERVAL::_psi_small_z(iv-1));
  end_if;
  n := max(ceil(log(2, op(abs(iv-1), 2))),0)+2;
  pow2 := hull(1/2)^n;
  z := iv*pow2;
  while op(abs(z-1),2) >= 1/2 do
    n := n+1;
    pow2 := hull(1/2)^n;
    z := iv*pow2;
  end_while;
  _plus(DOM_INTERVAL::_psi_small_z(z+k*pow2-1)$k=0..2^n-1) * pow2 +
  interval((2-2^(-n-1))*ln(2));
end_proc:

// formulae based on Spouge, "Computation of the Gamma, Digamma,
// and Trigamma Functions", SIAM Journal of Numerical Analysis,
// Vol 31, No. 3 (June 1994), 931-944

DOM_INTERVAL::_psi_spouge_coeffs :=
proc(a, prec)
  option remember;
  local N, k, fac, c;
begin
  fac := hull(1/sqrt(2*PI));
  N := ceil(a)-1;
  c := [0$k=0..N];
  for k from 1 to N do
    c[k+1] := fac * hull(a-k)^(k-1/2)*DOM_INTERVAL::exp(a-k);
    fac := -fac/k;
  end_for;
  c;
end_proc:

DOM_INTERVAL::_psi_spouge :=
proc(z)
  local a, N, c, f1, f2, k, eps, errorfact;
begin
  a := 5.5; // TODO: compute good value?
  N := ceil(a)-1;
  c := DOM_INTERVAL::_psi_spouge_coeffs(a, DIGITS);
  eps := interval(1/((2*PI)^(a+1/2)*Re(z+a)));
  if iszero(Im(z)) then
    errorfact := -1...1;
  else
    errorfact := -1-I...1+I;
  end_if;
  f1 := c[1] + _plus(c[k+1]/(z+k)$k=1..N) +
      // error bound
      errorfact*interval(1.022*ln(2*a)/sqrt(a)*eps);
  f2 := -_plus(c[k+1]/(z+k)^2$k=1..N) +
      // error bound
      errorfact*interval(sqrt(a)*eps);
  
  interval(ln(z+a)-(a-1/2)/(z+a)+f2/f1);
end_proc:

// for real arguments, psi is monotoneous,
// with simple poles at nonpositive integers:

// for real intervals, use monotonicity
DOM_INTERVAL::_psi_real :=
proc(iv)
  local k, l, r, psi_neg, psi_pos;
begin
  psi_pos :=
  (s -> DOM_INTERVAL::_psi_spouge(s+5)-_plus(1/(s+k) $ k = 0..5));
  // psi(z) = psi(1-z)+PI*cot(PI*(1-z))
  psi_neg :=
  (s -> psi_pos(1-s) +
       interval(PI)*cot(interval(PI)*(1-s)));
  
  [l, r] := [op(iv)];
  if r <= 0 then
    if r > ceil(l) then
      // negative integer inside the interval
      if r >= l+1 then
	RD_NINF...RD_INF
      else
	psi_neg(subsop(iv, 2=l)) ... RD_INF
	union
	RD_NINF...psi_neg(subsop(iv, 1=r));
      end_if;
    elif iszero(r - ceil(l)) then
      psi_neg(subsop(iv, 2=l)) ... RD_INF
    elif iszero(l - floor(r)) then
      RD_NINF...psi_neg(1-subsop(iv, 1=r));
    else
      // r and l negative, but no integer between them
      psi_neg(subsop(iv, 2=l))
      ...
      psi_neg(subsop(iv, 1=r));
    end_if;
  else // r >0
    if l < 0 then
      psi_neg(subsop(iv, 2=l)) ... RD_INF
      union
      RD_NINF...psi_pos(subsop(iv, 1=r), DIGITS);
    elif iszero(l) then
      RD_NINF...psi_pos(subsop(iv, 1=r), DIGITS);
    else
      psi_pos(subsop(iv, 2=l), DIGITS)
      ...
      psi_pos(subsop(iv, 1=r), DIGITS);
    end_if
  end_if
end_proc:



//////////////////////////////////////////////////


DOM_INTERVAL::psi :=
proc(iv)
  local n, k;
begin
  iv := interval(iv);
  if iv::dom <> DOM_INTERVAL then
    if iv = {} then return(iv); end_if;
    return(FAIL);
  end_if;
  
  if op(iv,0) = hold(_union) then
    return(_union(map(op(iv), DOM_INTERVAL::psi)));
  elif op(iv, 0) = hold(hull) then // real interval
    return(DOM_INTERVAL::_psi_real(iv));
  end_if;
  
  if Re(iv) intersect RD_NINF...0 <> {} and
     not iszero(op(Re(iv), 1)) then
    return(DOM_INTERVAL::psi((1-iv) intersect ((RD_NINF*hull(I)) ... RD_INF+RD_INF*I))
	   + interval(PI)*cot(interval(PI*(1-iv)))
	   union
	   DOM_INTERVAL::psi(iv intersect ((RD_NINF*hull(I)) ... RD_INF+RD_INF*I)));
  end_if;
  
  if TRUE then
    // Spouge formulae
    // use recurrence formula to *increase* the argument
    // for better error bounds, esp. in [0,2]
    // Note that _psi_spouge gets z-1 to calculate psi(z).
    DOM_INTERVAL::_psi_spouge(iv+5)-_plus(1/(iv+k) $ k = 0..5);
  else
    // based on A&S formulae
    if op(abs(iv-1), 2) < 1 then
      DOM_INTERVAL::_psi_doubling(iv)
    else
      // how many times to apply psi(z+1)=psi(z)+1/z
      // so that lhs(Re(iv-1)) < 1
      n := ceil(op(Re(iv-1), 1))-1;
      DOM_INTERVAL::_psi_doubling(iv-n+1) +
      _plus(1/(iv-k)$k=1..n-1);
    end_if;
  end_if;
end_proc:
