// $Id: compile_patterns_indefinite.mu,v 1.6 2004/07/21 14:53:44 ccr Exp $

// Pref::warnDeadProcEnv(TRUE):
testargs(TRUE):

// patterns for indefinite integration

// Globale Annahmen:
// Integrationsvariable x
// a, b, c, ..., v, w, y, z  sind von x unabhngig
// m, n, m0, n0 positiv und ganzzahlig
// m2, n2 positiv, ganzzahlig, gerade
// m1, n1 positiv, ganzzahlig, ungerade
// m3, n3 ganzzahlig
// mp1 = m+1
// m_ = -m, n_ = -n, m0_ = -m0, n0_ = -n0
// fx, gx are unrestricted
// px, qx are polynomial expressions
// qx1 is the reciprocal of a polynomial expression

LEVEL := 1: // spart Zeit beim Einlesen

START := time():

// etliche Pattern sind hier aufgenommen, obwohl MuPAD
// mit der Berechnung keinerlei Probleme hat -- damit
// sie auch in der Form fuer die Substitutionsmethode
// erkannt werden.

// --------------------------------------------------
//                     ABKRZUNGEN:
// --------------------------------------------------
// Bx : Bronstein, Semendjajew, 1991, Teubner,
//       Taschaenbuch der Mathematik,  Integral Nr.x
// K : Kusch Rosenthal, Girardet,
//       Mathematik, Band 4, Integralrechnung
// M  : Integranden, die MuPAD zwar integrieren kann, 
//      aber eine unschne Ausgabe liefert
// L  : Integranden, die Mupad nur sehr langsam
//      integrieren kann
// GH: Wolgang Gr"obner, Nikolaus Hofreiter, Integraltafel
//      1. Teil: Unbestimmte Integrale.  Springer 1965
// F  : Fatemans Pattern-Beispiele,
//      vgl. www.cs.berkeley.edu/~fateman
// BD : Herbert Bristol Dwight, Tables of Itegrals
//      and other mathematical data, MacMillan, 1961
//      
//---------------------------------------------------
//---------------------------------------------------
//                        ToDo:
//---------------------------------------------------        
/* 
 Bronstein:    B329
 
 Integranden, die bei der Lsung 
 von DGLn bentigt werden:
 
 according to Abramowitz and Stegun 
 int(airyAi(x),x) and
 int(airyBi(x), x)
 could be expressed in terms of hypergeometric functions.
 Moreover, one has 
 int(x*airyAi(x), x)=airyAi(x,1) 
 from the definition of Airy and by integration by parts one has
 int(x^2*airyAi(x),x)=x*airyAi(x,1)-airyAi(x)
 and so on for 
 int(x^n*airyAi(x),x)...
 
 
 */
//---------------------------------------------------

// Notation aus GH
// triprod := proc(m, d, v) save i; begin delete i; product((m+i*d), i=0..v-1) end:

read("INTLIB/patterns_indefinite.mu"):

// int_patterns := []:
// int_patterns := int_patterns[150..150].int_patterns[277..277]:
//direct_patterns := []:
// direct_patterns := direct_patterns[-1..-1]:

// check for errors happening often:
for pat in int_patterns do
  if nops(pat) > 2 then
    if domtype(pat[3]) <> DOM_LIST then
      error("illegal conditions at ".expr2text(pat[1]));
    end_if;
  end_if;
end_for:

assumptions := map([a,b,c,d,e,f,g,h,i,j,k,l,o,p,q,r,s,t,u,v,w,y,z],
                   v -> subs(bl(not has(`#v`, _X)), `#v`=v)).
              map([m,n,m0,n0,m1,m2,n1,n2],
                  v -> subs(bl(testtype(`#v`, Type::PosInt)), `#v`=v)).
              map([m_, n_, m0_, n0_],
                  v -> subs(bl(testtype(`#v`, Type::NegInt)), `#v`=v)).
              [bl(testtype(m2, Type::Even)),
               bl(testtype(n2, Type::Even)),
               bl(testtype(m1, Type::Odd)),
               bl(testtype(n1, Type::Odd)),
               bl(testtype(n3, Type::Integer)),
               bl(testtype(m3, Type::Integer)),
               mp1 = m+1,
               m_ = -m, n_ = -n,
               m0_ = -m0, n0_ = -n0,
               bl(testtype(px, Type::PolyExpr([x], Type::IndepOf(x)))),
               bl(testtype(qx, Type::PolyExpr([x], Type::IndepOf(x)))),
               bl(testtype(1/qx1, Type::PolyExpr([x], Type::IndepOf(x)))),
               null()
               ]:

// now, this is an ugly hack, but essential for speed:
unprotect(matchlib):
matchlib::PrefixTree:
matchlib::PrefixTreeFixatedResults:
matchlib::PrefixTreeFixatedResults::find_conds :=
proc(result : matchlib::PrefixTreeResult)
begin
  matchlib::PrefixTreeResult::conditions(result).
  map([op(indets(matchlib::PrefixTreeResult::result(result)))],
      proc(ind)
        local s;
      begin
        s := expr2text(ind);
        if length(s) < 2 or s[1..2] <> "`#" then return(null()); end_if;
        case s[3]
          // note: f, g, p, and q must not appear here,
          // otherwise, fx, gx, px, and qx would be restricted, too.
          of "a" do of "b" do of "c" do of "d" do of "e" do
          of "h" do of "i" do of "j" do of "k" do of "l" do
          of "o" do of "r" do of "s" do of "t" do of "u" do
          of "v" do of "w" do of "y" do of "z" do
            return(subs(bl(not has(`#`, _X)),
                        `#`=ind));
            break;
          of "m" do of "n" do
          // we can get away with a lax test,
          // since the "real" one is performed at the end anyway
            return(subs(bl(testtype(`#`, Type::Integer)),
                        `#`=ind));
        end_case;
        return(null());
      end_proc);
end_proc:

select_assumptions :=
  proc(ex)
    local vars;
  begin
    vars := indets(ex) union stdlib::PROTECTED union {_X};
    select(val(assumptions), a->indets(a) minus vars = {});
  end_proc:

substs := [a=`#a`, b=`#b`, c=`#c`,
           d=`#d`, e=`#e`, f=`#f`,
           g=`#g`, h=`#h`, i=`#i`,
           j=`#j`, k=`#k`, l=`#l`,
           m=`#m`, n=`#n`, o=`#o`,
           p=`#p`, q=`#q`, r=`#r`,
           s=`#s`, t=`#t`, u=`#u`,
           v=`#v`, w=`#w`, y=`#y`,
           z=`#z`,
           m0=`#m0`, n0=`#n0`,
           m1=`#m1`, n1=`#n1`,
           m2=`#m2`, n1=`#n2`,
           m3=`#m3`, n3=`#n3`,
           m_ = `#-m`, n_ = `#-n`,
           m0_ = `#-m0`, n0_ = `#-n0`,
           mp1=`#m+1`,
           fx=`#fx`, gx=`#gx`,
           px=`#px`, qx=`#qx`, qx1=`#qx1`]:

assumptions :=
subs(val(assumptions), substs, Unsimplified):

// constant factors
int_patterns2 := map(int_patterns,
                     proc(pat)
                       local pat1, noX, dummy;
                     begin
                       if type(pat[1]) = "_mult" then
                         [pat1, noX, dummy] := split([op(pat[1])],
                                                     has, [x, fx, gx]);
                         [_mult(op(pat1)),
                          pat[2]/_mult(op(noX)),
                          if nops(pat)>2 then
                            pat[3]
                          else
                            null()
                          end_if];
                       else
                         pat
                       end_if;
                     end_proc):

// for userinfo
for i from 1 to nops(int_patterns2) do
  int_patterns2[i][2] := [int_patterns2[i][2],
                          "Matched ".expr2text(i).":".
                          expr2text(int_patterns2[i][1])];
end_for:

for i from 1 to nops(direct_patterns) do
  direct_patterns[i][2] := [direct_patterns[i][2],
                            "Matched ".expr2text(i)."(d):".
                            expr2text(direct_patterns[i][1])];
end_for:

print(Unquoted,
      _outputSequence(nops(int_patterns2) + nops(direct_patterns),
                      " patterns read in ",
                      (time()-START)/1000.0, " seconds")):

// for unblocking, these should be deactivated:
evalsubsts := [hold(sum)=`#sum`,
               hold(product)=`#product`,
               hold(fact)=`#fact`,
               hold(int)=`#int`,
               hold(binomial)=`#binom`,
               hold(intlib::tryChangeVar)=`#tryCV`,
               hold(hold)=`#hold`]:
// redo the change:
substsaftereval := map(evalsubsts, x -> op(x,2)=op(x,1)):

// for patterns containing a*something or b+something,
// also add the special cases a=1 and b=0.
// a=0 is not added; a<>0 is considered an implicit assumption.
// to think about: a*x^2+b*x+c: b=0?
additional := {}:
add_pat :=
proc(pat, vars)
  local pos, v, p, set0, set1, e, t;
begin
  if nops(vars) = 0 then return(); end;
  v := op(vars, 1);
  vars := vars minus {v};
  set0 := FALSE;
  set1 := FALSE;
  pos := [prog::find(pat[1], v)];
  for p in pos do
    if nops(p) = 0 then next; end;
    case op(pat[1], p[1..-2].[0])
      of hold(_mult) do set1 := TRUE; break;
      of hold(_plus) do set0 := TRUE; break;
      of hold(_power) do
        if p[-1] = 2 then set1 := TRUE; set0 := TRUE;
        elif nops(p) > 1 then
          case op(pat[1], p[1..-3].[0])
            of hold(_mult) do set1 := TRUE; break;
            of hold(_plus) do set0 := TRUE; break;
          end_case;
        end_if;
    end_case;
  end_for;
  if set0 then
    traperror((e := eval(subs(pat, v=0));
               // unblock to avoid "blockTransparent(1/0*x)" etc.
               // that leads us into the traperror(...) around here.
               if not has(eval(unblock(subs(e, evalsubsts)))[1],
                          x) then
                 error("stop");
               end_if;
               // simplification
               if hastype(e[1], bl) then
                 e[1] := subs(subs(bl(1), 1=eval(unblock(subs(e[1], evalsubsts)))),
                              substsaftereval);
               end_if;
               // simplification
               if hastype(e[2][1], bl) then
                 e[2][1] := subs(subs(bl(1),
                                 1=eval(unblock(subs(e[2][1], evalsubsts)))),
                              substsaftereval);
               end_if;
               e[2][2] := e[2][2].", ".expr2text(v=0);
               t := additional union {e};
               additional := t;
               add_pat(e, vars);
              ));
  end_if;
  if set1 then
    traperror((e := eval(subs(pat, v=1));
               // unblock to avoid "blockTransparent(1/0*x)" etc.
               // that leads us into the traperror(...) around here.
               if not has(eval(unblock(subs(e, evalsubsts)))[1],
                          x) then
                 error("stop");
               end_if;
               // simplification
               if hastype(e[1], bl) then
                 e[1] := subs(subs(bl(1), 1=eval(unblock(subs(e[1], evalsubsts)))),
                              substsaftereval);
               end_if;
               // simplification
               if hastype(e[2][1], bl) then
                 e[2][1] := subs(subs(bl(1),
                                 1=eval(unblock(subs(e[2][1], evalsubsts)))),
                              substsaftereval);
               end_if;
               e[2][2] := e[2][2].", ".expr2text(v=1);
               t := additional union {e};
               additional := t;
               add_pat(e, vars);
              ));
  end_if;
  add_pat(pat, vars);
  null();
end_proc:

for pat in int_patterns2 do
  add_pat(pat, indets(pat) minus {x});
end_for:

int_patterns3 := select([op({op(int_patterns2)} union additional)],
                        proc(pat)
                          local res;
                        begin
                          _lazy_or(nops(pat) = 2,
                                   has(pat[3], hold(testtype)),
                                   _lazy_and(traperror((res :=
                                                        is(_and(op(unblock(pat[3]))))))
                                             = 0,
                                             res <> FALSE));
                        end_proc):

// TODO: combine multiple matches, i.e., identical patterns.
//       Ideally, this would also take care of patterns
//       identical up to variable renaming
extract := pat -> if nops(pat) = 2 then [pat[1]] else [pat[1], pat[3]] end_if:
tableOfPatterns := table():
for pat in int_patterns3 do
  if contains(tableOfPatterns, extract(pat)) then
    print(pat[2][2], tableOfPatterns[extract(pat)][2][2]);
    print(pat[1]);
  end_if;
  tableOfPatterns[extract(pat)] := pat;
end_for:

// now, add patterns for integration by parts:
int_patterns3_parts :=  
map(int_patterns3,
    (pat) -> [`##`(`#f'(_X)*const`)
              *subs(pat[1], x=`##`(`#f(_X)`)),
              [subs(bl(rewrite(`#f'(_X)*const`/
                               diff(`#f(_X)`, x) * `#p`, D)),
                    `#p` = subs(pat[2][1], x=`#f(_X)`)),
               pat[2][2]],
              if nops(pat)>2 then
                subs(pat[3], x=`#f(_X)`)
              else
                []
              end_if .
              [bl(has(`#f(_X)`, x)),
               bl(not has(rewrite(`#f'(_X)*const`/
                                  diff(`#f(_X)`, x), D),
                          x))]]):
               // needed for f'(x) = const.:
int_patterns4 :=
map(int_patterns3,
    (pat) -> [subs(pat[1], x=`##`(`#f(_X)`)),
              [subs(bl(`#p`/diff(`#f(_X)`, x)),
                    `#p` = subs(pat[2][1], x=`#f(_X)`)),
               pat[2][2]],
              if nops(pat)>2 then
                subs(pat[3], x=`#f(_X)`)
              else
                []
              end_if .
              [bl(has(`#f(_X)`, x)),
               bl(not has(diff(`#f(_X)`, x), x))
               ]]):

print(Unquoted,
      _outputSequence("preprocessing done (now at ",
                      (time()-START)/1000.0, " seconds)")):


substs_with_ass := map(substs,
                       ex -> lhs(ex)=`##`(rhs(ex)
                                          /* , select_assumptions(rhs(ex)) */
                                               )) .
                 [x=_X]:

conv := proc(pat)
          local ex, ex2, res, cond;
        begin
          cond := [];
          case nops(pat)
            of 3 do
              cond := subs(pat[3],
                           substs.[x=_X]);
              if domtype(cond) <> DOM_LIST then
                error("illegal pattern ".expr2text(pat));
              end_if;
                     // fall through
            of 2 do
              ex := subs(pat[1],
                         substs_with_ass);
              res := subs(pat[2],
                          substs.[x=_X]);
              break;
            otherwise
              error("illegal pattern ".expr2text(pat));
          end_case;
                 // mark the results:
//               res := [res, expr2text(ex)];
// care for neg. exponents
          ex2 := subs(ex, [-`##`(`#n`)  = `##`(`#-n`),
                           -`##`(`#m`)  = `##`(`#-m`),
                           -`##`(`#n0`) = `##`(`#-n0`),
                           -`##`(`#m0`) = `##`(`#-m0`)]);
          if ex <> ex2 then
            ex := ex2;
            ex2 := [if not has(ex, `#n`) then
                      `#n`=-`#-n`
                    else null(); end,
                    if not has(ex, `#m`) then
                      `#m`=-`#-m`
                    else null(); end,
                    if not has(ex, `#n0`) then
                      `#n0`=-`#-n0`
                    else null(); end,
                    if not has(ex, `#m0`) then
                      `#m0`=-`#-m0`
                    else null(); end];
            res := subs(res, ex2);
            cond := subs(cond, ex2);
          end_if;
          [ex, res, select_assumptions(ex).cond];
        end_proc:
int_pat := map(int_patterns4.direct_patterns, conv):
int_pat_parts := map(int_patterns3_parts, conv):

// make sure no pattern contains `##`(a)+`##`(b)+...
// or  `##`(a)*`##`(b)*...
checkBadPatterns :=
proc(pat)
begin
  misc::maprec(pat[1],
               {"_plus", "_mult"} =
               (ex -> if nops(select([op(ex)],
                                     x -> op(x,0)=`##`)) > 1
                        then error("bad pattern ".expr2text(pat[1]));
                      else ex
                      end_if));
end_proc:

// map(int_pat, checkBadPatterns):
// map(int_pat_parts, checkBadPatterns):

int_pat2 := int_pat.int_pat_parts:

allpatterns := int_pat2:

int_pat := table():
[int_pat[RatExpr], int_pat2, dummy]:=
split(int_pat2,
      x -> testtype(subs(eval(subs(x[1], `##`=id)),
                         [`#m`=3,   `#n`=1,
                          `#-m`=-3, `#-n`=-1,
                          `#m1`=3,  `#n1`=3,
                          `#m2`=2,  `#n2`=2,
                          `#m0`=1,  `#n0`=2,
                          `#m3`=3,  `#n3`=3,
                          `#px`=`#fx`,
                          `#qx`=`#fx`,
                          `#qx1`=1/`#fx`,
                          `#f(_X)`=`#fx`,
                          `#f'(_X)*const`=`#fx`
                         ],
                         Unsimplified),
                    Type::RatExpr(`#fx`,
                                  Type::IndepOf(`#fx`)))):

for fn in ["sin", "cos", "ln", "tan",
           "arctan", "sinh", "cosh",
           "exp", "arcsin", "arccos",
           "arcsinh", "arccosh", "arccot"] do
  [int_pat[fn], int_pat2, dummy] := split(int_pat2,
                                          x -> hastype(x[1], fn)):
end_for:

int_pat[Default] := int_pat2:

print(Unquoted,
      _outputSequence("starting compilation proper (time: ",
                      (time()-START)/1000.0, " seconds,",
                      " no. of patterns:",
                      expr2text(op(map(int_pat, nops))), ")")):

int_pat := select(int_pat, x->nops(rhs(x))>0):

sysassign(intlib::patternFSA, map(int_pat, matchlib::compile::toFSA)):

print(Unquoted,
      _outputSequence("compilation finished, ", (time()-START)/1000.0, " seconds")):
               
bytes();
// share():
// bytes();

patternFSA := intlib::patternFSA:
write("/tmp/patternFSA.mb", patternFSA):

intlib::lookup((x^2 + 2*x + 11)^(3/2), x)
