// $Date: 2004/07/26 07:42:35 $ $Author: ccr $ $Revision: 1.92 $ 

/*--
intlib::int(f,x) - computes the antiderivative of the function f(x) with
                   respect to the variable x.
--*/

// the following (protected) identifier is used to detect the recursion level
// of intlib::int. This becomes necessary to initialize the set intlib::tries
// correctly. Only at top level intlib::tries is new initialized.
// Note: This is only secure for using intlib::int, if one calls
//       intlib::int_intern instead wrong counting will occure.
//sysassign(intlibDepthLevel,0):

intlib::int:=
  proc(f :Type::Arithmetical,x :DOM_IDENT)
    local prop,C,F;
    save _X; //necessary to make assumption on integration variable
  begin
    //if testargs() then f:=eval(f) end_if;
    C:=genident("__X");
    if property::hasprop(_X) then 
      //assume(C,prop1)  // does'nt work due to an error in assume
      zip([C],[getprop(_X)],assume);
    end_if;
    delete _X;
    if property::hasprop(x) then
      prop:=getprop(x);
      if prop=Type::Complex then
        //assume(_X,prop) // does'nt work due to an error in assume
        zip([_X],[Type::Complex],assume);        
      elif not(property::Fail((prop:=prop and Type::Real))) then
        if prop=Type::Zero then
          prop:=Type::Complex;
          if intlib::printWarningsFlag then
            warning("Cannot integrate when ".strprint(NoNL, x).
                    " has property ".strprint(NoNL, getprop(x)).".\n".
                    "While integrating, we will assume ".strprint(NoNL, x).
                    " has property ".strprint(NoNL, prop).".");
          end_if;
        elif is(x,Type::Rational)=TRUE then
          prop:=Type::Real;
          if intlib::printWarningsFlag then
            warning("Cannot integrate when ".strprint(NoNL, x).
                    " has property ".strprint(NoNL, getprop(x)).".\n".
                    "While integrating, we will assume ".strprint(NoNL, x).
                    " has property ".strprint(NoNL, prop).".");
          end_if;
        end_if;
        //assume(_X,prop) // does'nt work due to an error in assume
        zip([_X],[prop],assume);
      elif intlib::printWarningsFlag then
        warning("Cannot assume ".strprint(NoNL, x).
               " is a real variable,\n"."since that would be inconsistent to ".
                "user given properties")
      end_if
    else
      assume(_X,Type::Real)
    end_if;
    if x<>_X then f:=subs(f,[_X=C,x=_X]) end_if;
    F:=subs(intlib::int_intern(f,_X),[_X=x,C=_X]);
    unassume(C);
    F    
  end_proc:


intlib::int_intern:=
  proc(_F_,_x_)
  //option remember; // could lead to conflicts with user given properties
    local integral,int2,Float,_f_,F0;
    save intlibDepthLevel;//make counter secure for error or exception handling
  begin
    F0:=_F_;
    if intlibDepthLevel = hold(intlibDepthLevel) then 
       sysassign(intlib::tries,{});
       sysassign(intlibDepthLevel,0):
    end_if;
    sysassign(intlibDepthLevel,intlibDepthLevel+1);
    
    _F_ := misc::maprec(_F_, {DOM_POLY} = expr);
    
    // First, transform each floating point number to its
    // rational approximation 
    _f_:=misc::maprec(_F_,
         {DOM_FLOAT,DOM_COMPLEX}=(x -> numeric::rationalize(x,Minimize)) );
    Float:=(if _f_ <> _F_ then float else id end_if)@(x->combine(x, exp));
    
    if has(_f_,arg) then
      _f_:=eval(subs(_f_,hold(arg)=(z->-I*ln(z/abs(z)))));
    end_if;

    if has(_f_,log) then
      _f_:=eval(subs(_f_,hold(log)=((b,x) -> ln(x)/ln(b))));
    end_if;
    
    if testtype(_f_, Type::RatExpr(_x_, Type::IndepOf(_x_))) then
      // short circuit: rational functions are handled completely
      // by Risch, but the result returned there may not be in the
      // form desired.
      userinfo(2,"rational expression, trying Risch integration");
      integral:=intlib::rischInt(_f_,_x_);
      if integral = FAIL then
	// if Risch's algorithm can't integrate a rational,
        // the only chance is integration by terms:
        userinfo(2,"try to integrate term by term");
        integral:=intlib::termByTerm(_f_,_x_);
        sysassign(intlibDepthLevel,intlibDepthLevel-1);
        if integral<>FAIL then
          return(Float(intlib::expandExp(integral)))
        end_if;
        
	if stdlib::hasmsign(F0) then
	  return(-hold(int)(-F0, _x_));
	else
	  return(hold(int)(F0,_x_));
	end_if;
      else
	// solveable, try to get nicer form
	// as a mild example, compare
	// intlib::rischInt((2*x+3)^3/(3*x+4)^3, x)
	// intlib::lookup((2*x+3)^3/(3*x+4)^3, x)
	// or
	// intlib::rischInt((2*x+3)^3/(3*x+4)^6, x)
	// intlib::lookup((2*x+3)^3/(3*x+4)^6, x)
	integral := intlib::expandExp(integral);
	userinfo(2,"trying lookup method for nicer representation");
	int2 := intlib::lookup(_f_, _x_, [RatExpr]);
	if int2 <> FAIL then integral := int2; end_if;
	sysassign(intlibDepthLevel,intlibDepthLevel-1);
	return(Float(integral))
      end_if;
    end_if;
      
    // test if integrand contains special functions that can be treated
    integral:=intlib::specialFunctionInt(_f_,_x_);
    if integral<>FAIL then
      sysassign(intlibDepthLevel,intlibDepthLevel-1);
      return(Float(intlib::expandExp(integral)))
    end_if;
                        
    // lookup method 
    userinfo(2,"trying lookup method");
    integral:=intlib::lookup(_f_,_x_);
    if integral<>FAIL then
      sysassign(intlibDepthLevel,intlibDepthLevel-1);
      return(Float(integral))
    end_if;

    // trying Risch Integration 
    userinfo(2,"lookup failed, trying Risch integration");
    integral:=intlib::rischInt(_f_,_x_);
    if integral<>FAIL then
      sysassign(intlibDepthLevel,intlibDepthLevel-1);
      return(Float(intlib::expandExp(integral)))
    end_if;
                        
    // try to simplify the integrand or doing a last hack 
  //  userinfo(2,"Risch's algorithm failed, try to simplify integrand");
  //  if not contains(intlib::tries,_f_) then
  //    sysassign(intlib::tries,intlib::tries union {_f_});
  //  end_if;
  //  integral:=intlib::simplifyIntegrand(_f_,_x_);
  //  if integral<>FAIL then return(Float(intlib::expandExp(integral))) end_if;
  
    // using linearity of integration 
    userinfo(2,"try to integrate term by term");
    integral:=intlib::termByTerm(_f_,_x_);
    if integral<>FAIL then
      sysassign(intlibDepthLevel,intlibDepthLevel-1);
      return(Float(intlib::expandExp(integral)))
    end_if;
    
    if has(_f_, {hold(sin), hold(cos)}) then
      integral := intlib::ratSinCos(_f_, _x_);
    end_if;
    
    sysassign(intlibDepthLevel,intlibDepthLevel-1);
    if integral<>FAIL then return(Float(intlib::expandExp(integral))) end_if;
    if stdlib::hasmsign(F0) then
      -hold(int)(-F0, _x_)
    else
      hold(int)(F0,_x_)
    end_if;
  end_proc:

intlib::tries:={}:

intlib::expandExp:=
  proc(f)
  begin
    if has({f},exp) then
      misc::maprec(f,{"exp"}=proc() begin expand(exp(op(args()))) end_proc )
    else
      f
    end_if;
  end_proc:

intlib::simplifyIntegrand:=
  proc(f0,x)
    local f, F;
  begin
    f:=simplify(f0);   // for the following a normalform for f would be needed.
    if f<>f0 and not contains(intlib::tries,f) then
      // to avoid loops
      sysassign(intlib::tries,intlib::tries union {f});
      F:=intlib::int(f,x);
      if type(F)<>"int" then return(F) end_if;
    end_if;
    FAIL
  end_proc:
    
/* if both the lookup method and Risch algorithm failed,
  and if the function is a sum, it can be that some
  terms can be integrated by the lookup method and some
  other by Risch algorithm, like for example int(diff(g(x),x)+x+2,x) */
intlib::termByTerm := proc(f,x)
local f0, e, i, r, r2;
begin
   f0:=f;		
   if type(f)="_mult" and contains(map({op(f)},type),"_plus") then
//      f:=expand(f)
     f := [op(f)];
     // find the first sum in the operands, expand
     for i from 1 to nops(f) do
       if type(f[i]) = "_plus" then
	 e := f[i];
	 delete f[i];
	 f := `*`(op(f));
	 f := map(e, `*`, f);
	 break;
       end_if;
     end_for;
   end_if;
   if type(f)="_plus" then
     r:=0;
     r2 := 0;
     for e in [op(f)] do
       i:=intlib::int(e,x);
       if hastype(i,"int") then
	 r2 := r2+e;
//	 return(FAIL)
       else
         r:=r+i
       end_if
     end_for;
     if iszero(r2) then
       return(r);
     else
       return(r+hold(int)(r2, x));
     end_if;
   end_if;
   FAIL
end_proc:

/* for integrands involving sin and cos, as a last resort try using
   combine(..., sincos) and if that leads to no change, try using
   the standard tangent substitutions */

intlib::ratSinCos :=
proc(f, x)
  local f2, t, integral;
begin
  f2 := combine(f, sincos);
  if f2 <> f then
    userinfo(2, "trying to integrate after combining sin/cos terms");
    integral := intlib::int_intern(f2, x);
    if not hastype(integral, "int") then
      return(integral);
    end_if;
  end_if;
  
  if has(f, `#t`) then
    t := genident("tmp") = `#t`;
    f := subs(f, op(t, 2) = op(t, 1), Unsimplified);
  else
    t := null();
  end_if;
  
  f := expand(f);
  
  f2 := subsex(f, [sin(x)^2=`#t`^2/(`#t`^2+1),
		   cos(x)^2=1/(`#t`^2+1),
		   sin(x)*cos(x)=`#t`/(`#t`^2+1),
		   tan(x)=`#t`]);
  if not has(f2, x) then
    userinfo(2,"try to substitute t=tan(".expr2text(x).")");
    integral := intlib::int(f2/(`#t`^2+1), `#t`);
    if not hastype(integral, "int") then
      return(subs(integral, `#t`=tan(x), t));
    end_if;
  end_if;
  
  f2 := subsex(f/sin(x), [sin(x)^2=1-`#t`^2,
			  cos(x)=`#t`]);
  if not has(f2, x) then
    userinfo(2,"try to substitute t=cos(".expr2text(x).")");
    integral := intlib::int(f2, `#t`);
    if not hastype(integral, "int") then
      return(-subs(integral, `#t`=cos(x), t));
    end_if;
  end_if;
  
  f2 := subsex(f/cos(x), [sin(x)=`#t`,
			  cos(x)^2=1-`#t`^2]);
  if not has(f2, x) then
    integral := intlib::int(f2, `#t`);
    userinfo(2,"try to substitute t=sin(".expr2text(x).")");
    if not hastype(integral, "int") then
      return(subs(integral, `#t`=sin(x), t));
    end_if;
  end_if;
  
  f2 := subsex(f, [sin(x)=2*`#t`/(1+`#t`^2),
		   cos(x)=(1-`#t`^2)/(1+`#t`^2),
		   tan(x)=2*`#t`/(1-`#t`^2),
		   cot(x)=(1-`#t`^2)/2/`#t`,
		   tan(x/2)=`#t`]);
  if not has(f2, x) then
    userinfo(2,"try to substitute t=tan(".expr2text(x)."/2)");
    integral := intlib::int(2*f2/(1+`#t`^2), `#t`);
    if not hastype(integral, "int") then
      return(subs(integral, `#t`=tan(x/2), t));
    end_if;
  end_if;
  
  FAIL;
end_proc:
