// $Date: 2004/10/02 16:26:28 $ $Author: nthiery $ $Revision: 1.68 $ 
/*++
Dom -- a library of basic domain constructors
++*/

domains: // force loading 'domains' 

Dom:= newDomain("Dom"):
Dom::info:= "Library 'Dom': basic domain constructors":
Dom::interface:= {
    hold(BaseDomain),
    hold(Expression),
//  hold(NCExpressionField),
    hold(ArithmeticalExpression),
    hold(Numerical),
    hold(Float),
    hold(FloatIV),
    hold(Rational),
    hold(Integer),
    hold(IntegerMod),
    hold(ExpressionField),
    hold(Fraction),
    hold(Polynomial),
    hold(DistributedPolynomial),
    hold(Product),
    hold(/*Sparse*/Matrix),
    hold(DenseMatrix),
    hold(SquareMatrix),
    //hold(TruncatedPowerSeries), experimental and not documented
    hold(AlgebraicExtension),
    hold(Interval),
    hold(GaloisField),
    hold(ImageSet),
    hold(Multiset),
    hold(MatrixGroup),
    hold(PermutationGroup),
    hold(SymmetricGroup),
    hold(DihedralGroup),
    hold(FreeModule),
    hold(FreeModuleList),
    hold(FreeModulePoly),
    hold(FreeModuleTable),
    hold(Real),
    hold(Complex),
    hold(Quaternion),
    hold(Ideal),
    hold(MultivariatePolynomial),
//    hold(MultivariateSeries),
    hold(UnivariatePolynomial),
//    hold(UnivariateSkewPolynomial),
    hold(LinearOrdinaryDifferentialOperator),
//    hold(PowerProduct),
//    hold(MonoidAlgebra),
//    hold(ExteriorProduct),
//    hold(ExteriorAlgebra),
//    hold(MonoidOperatorAlgebra),
//    hold(LinearDifferentialOperator),
    hold(SparseMatrixF2),
//    hold(DifferentialVariable),
//    hold(RestrictedDifferentialVariable),
//    hold(DifferentialPolynomial),
//    hold(DifferentialExpression),
//    hold(LinearDifferentialFunction),
//    hold(TensorProduct),
//    hold(TensorAlgebra),
//    hold(WeylAlgebra),
//    hold(HomogenisedWeylAlgebra),
//    hold(PommaretDivision),
//    hold(JanetDivision),
//    hold(LiftedDivision),
//    hold(SolvableModule),
//    hold(SolvablePolynomial),
//    hold(InvolutiveBasis),
//    hold(VectorField),
//    hold(JetVectorField),
    hold(MonomOrdering)
    //hold(Utilities::IntModInverse) // utility, not to be documented
}:

alias(path = pathname("DOMAINS", "DOMAIN")):

Dom::BaseDomain:= loadproc(Dom::BaseDomain, path, "BaseDom"):
Dom::Expression:= loadproc(Dom::Expression, path, "Expr"):
Dom::ArithmeticalExpression:= loadproc(Dom::ArithmeticalExpression, path, "AritExpr"):
Dom::Numerical:= loadproc(Dom::Numerical, path, "Numeric"):
Dom::Float:= loadproc(Dom::Float, path, "Float"):
Dom::FloatIV := loadproc(Dom::FloatIV, path, "FloatIV"):
Dom::Rational:= loadproc(Dom::Rational, path, "Rational"):
Dom::Integer:= loadproc(Dom::Integer, path, "Integer"):
Dom::IntegerMod:= loadproc(Dom::IntegerMod, path, "IntMod"):
Dom::ExpressionField:= loadproc(Dom::ExpressionField, path, "ExprFld"):
// Dom::NCExpressionField:= loadproc(Dom::NCExpressionField, path, "NCExprFld"):
Dom::Fraction:= loadproc(Dom::Fraction, path, "Fraction"):
Dom::Polynomial:= loadproc(Dom::Polynomial, path, "Polynom"):
Dom::DistributedPolynomial:=
      loadproc(Dom::DistributedPolynomial, path, "DistPoly"):
Dom::Product:= loadproc(Dom::Product, path, "Product"):
Dom::DenseMatrix:= loadproc(Dom::DenseMatrix, path, "DenseMatrix"):
Dom::SquareMatrix:= loadproc(Dom::SquareMatrix, path, "SqMatrix"):
// experimental and not documented
Dom::TruncatedPowerSeries:= loadproc(Dom::TruncatedPowerSeries, path, "TrPowSer"):
Dom::AlgebraicExtension:= loadproc(Dom::AlgebraicExtension, path, "AlgebExt"):
Dom::Interval:= loadproc(Dom::Interval, path, "Interval"):
Dom::GaloisField:= loadproc(Dom::GaloisField, path, "GaloisF"):
Dom::SubSet:= loadproc(Dom::SubSet, path, "SubSet"):
Dom::ImageSet:= loadproc(Dom::ImageSet, path, "ImageSet"):
Dom::Multiset:= loadproc(Dom::Multiset, path, "Multiset"):
Dom::MatrixGroup:= loadproc(Dom::MatrixGroup, path, "Mat"):
//Dom::PermutationGroup:= loadproc(Dom::PermutationGroup, path, "PermGrp"):
Dom::PermutationGroup:= loadproc(Dom::PermutationGroup, path, "PermutationGroup"):
Dom::SymmetricGroup:= loadproc(Dom::SymmetricGroup, path, "SymmetricGroup"):
Dom::DihedralGroup:= loadproc(Dom::DihedralGroup, path, "Dihedral"):
Dom::FreeModuleList := loadproc(Dom::FreeModuleList, path, "FreeModuleList"):
Dom::FreeModulePoly := loadproc(Dom::FreeModulePoly, path, "FreeModulePoly"):
Dom::FreeModuleTable := loadproc(Dom::FreeModuleTable, path, "FreeModuleTable"):
// Default implementation of Dom::FreeModule is Dom::FreeModulePoly
Dom::FreeModule :=
proc(R, S)
begin
    if domtype(S) <> DOM_DOMAIN or S::hasProp=FAIL or not S::hasProp(Cat::CombinatorialClass) then
	error("Wrong type of 2. argument (Cat::Combinatorial Class expected)\nNote: Dom::FreeModule has changed; maybe you want to use the former implementation Dom::FreeModuleOld.");
    end_if;
    Dom::FreeModulePoly(args());
end_proc:
Dom::FreeModuleOld:= loadproc(Dom::FreeModuleOld, path, "FreeMod"):
autoload(Dom::TensorProductOfFreeModules):
Dom::Real:= loadproc(Dom::Real, path, "Real"):
Dom::Complex:= loadproc(Dom::Complex, path, "Complex"):
Dom::Quaternion:= loadproc(Dom::Quaternion, path, "Quatern"):
Dom::MultivariatePolynomial:= loadproc(Dom::MultivariatePolynomial,path,"MultPoly"):
Dom::MultivariateSeries:= loadproc(Dom::MultivariateSeries,path,"MultiSeries"):
Dom::UnivariatePolynomial:=loadproc(Dom::UnivariatePolynomial,path,"UniPoly"):
Dom::PolynomialIdeal:=loadproc(Dom::PolynomialIdeal,path,"PolIdeal"):
Dom::Ideal:=loadproc(Dom::Ideal, path, "Ideal"):
Dom::UnivariateSkewPolynomial:= loadproc(Dom::UnivariateSkewPolynomial,path,"OrePoly"):
Dom::LinearOrdinaryDifferentialOperator:= loadproc(Dom::LinearOrdinaryDifferentialOperator,path,"Lodo"):
Dom::PowerProduct:=loadproc(Dom::PowerProduct,path,"PowProd"):
Dom::MonoidAlgebra:=loadproc(Dom::MonoidAlgebra,path,"MonAlg"):
Dom::ExteriorProduct:=loadproc(Dom::ExteriorProduct,path,"ExtProd"):
Dom::ExteriorAlgebra:=loadproc(Dom::ExteriorAlgebra,path,"ExtAlg"):
Dom::MonoidOperatorAlgebra:=loadproc(Dom::MonoidOperatorAlgebra,path,"MonOpAlg"):
Dom::LinearDifferentialOperator:=loadproc(Dom::LinearDifferentialOperator,
		path,"LinDiffOp"):
Dom::/*Sparse*/Matrix:=loadproc(Dom::/*Sparse*/Matrix,path,"Matrix"):
Dom::SparseMatrixOld:=loadproc(Dom::SparseMatrixOld,path,"Sparse"):
Dom::SparseMatrixF2:=loadproc(Dom::SparseMatrixF2, path, "SparseF2"):
Dom::LinearDifferentialFunction:=loadproc(Dom::LinearDifferentialFunction,path,"LinFun"):
Dom::DifferentialPolynomial:=loadproc(Dom::DifferentialPolynomial,path,"DiffPoly"):
Dom::DifferentialExpression:=loadproc(Dom::DifferentialExpression,path,"DiffExpr"):
Dom::LinearDifferentialFunction:=loadproc(Dom::LinearDifferentialFunction,path,"LinFun"):
Dom::DifferentialVariable:=loadproc(Dom::DifferentialVariable,path,"DiffVar"):
Dom::RestrictedDifferentialVariable:=loadproc(Dom::RestrictedDifferentialVariable,path,"RestDVar"):
Dom::TensorProduct:=loadproc(Dom::TensorProduct,path,"TensProd"):
Dom::TensorAlgebra:=loadproc(Dom::TensorAlgebra,path,"TensAlg"):
Dom::WeylAlgebra:=loadproc(Dom::WeylAlgebra,path,"WeylAlg"):
Dom::HomogenisedWeylAlgebra:=loadproc(Dom::HomogenisedWeylAlgebra,path,"HWeylAlg"):
Dom::PommaretDivision:=loadproc(Dom::PommaretDivision,path,"PomDiv"):
Dom::JanetDivision:=loadproc(Dom::JanetDivision,path,"JanetDiv"):
Dom::LiftedDivision:=loadproc(Dom::LiftedDivision,path,"LiftDiv"):
Dom::SolvableModule:=loadproc(Dom::SolvableModule,path,"SolvMod"):
Dom::SolvablePolynomial:=loadproc(Dom::SolvablePolynomial,path,"SolvPol"):
Dom::InvolutiveBasis:=loadproc(Dom::InvolutiveBasis,path,"InvBas"):
Dom::VectorField:=loadproc(Dom::VectorField,path,"VecFld"):
Dom::JetVectorField:=loadproc(Dom::JetVectorField,path,"JetVecFld"):
Dom::MonomOrdering:=loadproc(Dom::MonomOrdering, path, "MonomOrd"):

// Utilities
Dom::Utilities:= loadproc(Dom::Utilities, path, "Utilities"):

null():

// end of file 
