// $Date: 2004/01/16 11:47:32 $ $Author: bij $ $Revision: 1.20 $

// rules for 'simplify'
// 
// bij 2000/2001
// 
// Rule (internal parameters)
// 
//        PATTERN      - pattern (DOM_EXPR)
//        RESULT       - return expression (DOM_EXPR)
//        COND         - conditions (DOM_SET)
//                       set containing procedures or expressions (see match)
//        STRATEGY     - a table of eq.
//                       strategyname = priority


// examples
// Rule(sin(X)^2 + cos(X)^2, 1)
//
//

Rule:= newDomain("Rule"):
Rule::create_dom:=hold(Rule):

alias(ARGS = hold(["pattern", "result", "cond", "strategy"])):

Rule::DEBUG := FALSE:

Rule::new:=
  proc(PATTERN, RESULT, COND = {}, // :DOM_SET|DOM_PROC
       STRATEGY = table())         //: DOM_TABLE
    local ret;
  begin
    sysassign(Rule::COUNT, Rule::COUNT + 1);
    if testtype(PATTERN, Type::Function) then
      // procedure
      if testargs() then
        // 
      end_if;
      ret := new(Rule, PATTERN, FAIL,
                       (if args(0) > 1 then RESULT else {} end_if),
                       (if args(0) > 2 then COND else table() end_if))
    else
      // match rule
      if testargs() then
        if args(0) < 2 then
          error("at least two expressions required")
        end_if
      end_if;
      ret := new(Rule, PATTERN, RESULT, COND, STRATEGY)
    end_if;
    %if Rule::DEBUG = TRUE then
       if contains(Rule::VAL, ret) and Rule::result(ret) <> FAIL then
         // only for matching rules
         warning("rule exists: ".expr2text(ret))
       end_if;
     end_if;//%end
    sysassign(Rule::VAL[ret], [1, 0, 0, 0]);
    ret
  end_proc:

Rule::COUNT:= 0:

proc()
  local k;
begin
// create 4 similar procedures
//   from alias 'ARGS' for each parameter of a rule 
for k from 1 to nops(ARGS) do
  slot(Rule, op(ARGS, k)):=
  subsop(proc(RULE, VALUE = null())
         begin
           if VALUE = null() then // get
             extop(RULE, FAIL)
           else // set
             extsubsop(RULE, FAIL = VALUE)
           end_if
         end_proc, [4, 2, 2] = k, [4, 3, 2, 1] = k, Unsimplified)
end_for:
end():

Rule::VAL := table():

// priority: liefert die erwartete Komplexitaet des Ergebnisses
Rule::priority:=
proc(RULE, strat, valuation)
  local m;
  //option remember;
begin
  if contains(Rule::strategy(RULE), strat) then
    Rule::strategy(RULE)[strat]
  elif contains(Rule::strategy(RULE), "Default") then
    Rule::strategy(RULE)["Default"]
  elif contains(Rule::VAL, RULE) then
    Rule::VAL[RULE][1]
  else
    m := max(valuation(Rule::result(RULE))/valuation(Rule::pattern(RULE)), 0.5);
    //Rule::strategy(RULE, table("Default" = m, Rule::strategy(RULE)));
    // initialization of Rule::VAL[RULE]
    sysassign(Rule::VAL[RULE], [float(m), 1]);
//fprint(0, ">>", RULE = m);
    m
  end_if
end_proc:

// apply rule
// ARGS: RULE, EXPRESSION
Rule::apply:=
  proc(RULE : Rule, EXPRESSION)
    local MATCH;
  begin
    // special case - strange procedure
    /*if not domtype(RULE) = Rule then
      return(RULE(EXPRESSION))
    el*/
    if Rule::result(RULE) = FAIL then
      // procedure
      if Rule::cond(RULE) = {} or
        Rule::cond(RULE)(EXPRESSION) = TRUE then
        return(Rule::pattern(RULE)(EXPRESSION))
      end_if
    elif domtype(Rule::pattern(RULE)) = DOM_IDENT then
      // no match, the pattern is an identifier
      if Rule::cond(RULE) = {} or
        Rule::cond(RULE)(EXPRESSION) = TRUE then
        return(eval(subs(Rule::result(RULE), Rule::pattern(RULE) = EXPRESSION)))
      end_if
    elif (MATCH:= match(EXPRESSION, Rule::pattern(RULE),
                        hold(Cond) = Rule::cond(RULE))) <> FAIL then
      %if Rule::DEBUG = TRUE then
        sysassign(Rule::VAL[RULE], zip(Rule::VAL[RULE], [0, 0, 1, 0], _plus))
      end_if;//%end
      return(eval(subs(Rule::result(RULE), MATCH)))
    end_if;
    %if Rule::DEBUG = TRUE then
      sysassign(Rule::VAL[RULE], zip(Rule::VAL[RULE], [0, 0, 0, 1], _plus))
    end_if;//%end
    FAIL
  end_proc:

Rule::print:=
  proc(RULE)
  begin
    (if Rule::result(RULE) = FAIL then
       //"X --> ".
       (if domtype(Rule::pattern(RULE)) = DOM_PROC then
          DOM_PROC::print(Rule::pattern(RULE))
        else
          expr2text(Rule::pattern(RULE))
        end_if).
       "(X)"
     else
       expr2text(Rule::pattern(RULE))." ==> ".expr2text(Rule::result(RULE))//."\n"
     end_if)
    .(if Rule::cond(RULE) <> {} then
        " if " . expr2text(Rule::cond(RULE))
      else
        ""
      end_if)
    //._if(Rule::strategy(RULE) <> table(), "   Strat: ".expr2text(Rule::strategy(RULE)), "")
  end_proc:

// rules moved to Simplify::MatchRules

null():
