// $Id: Set.mu,v 1.8 2003/08/19 13:59:22 tonner Exp $

Set := newDomain("Set"):
Set::create_dom := hold(Set):
Set::info := "Library 'Set': basic Set":

Set::interface:=
  {// internal domains
   hold(Rectform) , 
   hold(Polarform),
   hold(Reald)    ,
   hold(Finite)   ,
   hold(Interval) ,
   hold(Union)    ,
   hold(Intersect),
   hold(Minus)    ,
   hold(Bigunion) ,
   hold(Imageset) ,
   // predefined objects
   hold(Real)
   }:


// creates a new set
Set::new:=
  proc(TYPE)
  begin
    if has(Set::internalTypes, TYPE) then
      return(new(Set, TYPE, args(2..args(0))))
    end_if;
    
    // implicit - try conversion
    case type(TYPE)
      of DOM_SET do // finite set
      of DOM_INT do
      of DOM_RAT do
      of DOM_FLOAT do
        Set::Finite(TYPE); break
      //of DOM_COMPLEX do
      otherwise
        error("cannot determine set type")
    end_case
  end_proc:

Set::_new:=
  proc(TYPE)
  begin
    new(Set, TYPE, args(2..args(0)))
  end_proc:

// Set objects can be used as properties
Set::prop:= TRUE:

// all internal type names
Set::internalTypes:= ["Rectform",
                      "Polarform",
                      "Realdom",
                      "Finite",
                      "Interval",
                      "Union",
                      "Intersect",
                      "Minus",
                      "Bigunion",
                      "Imageset"]:

((slot(Set, k) := subs(proc() name Set::new;
                       begin
                         Set::new("SUBS", args())
                       end_proc, "SUBS" = k))
  $ k in Set::internalTypes):

Set::type:=
  proc(SET)
  begin
    extop(SET, 1)
  end_proc:

// overloaded methods
proc()
  option escape;
  local k;
begin
  for k in ["op", "subsop", "nops"] do
    slot(Set, k):= proc(SET)
                   begin
                     if contains(Set::internalTypes, Set::type(SET)) <> 0
                       and contains(slot(Set, Set::type(SET)), k) then
                       (slot(slot(Set, Set::type(SET)), k))(SET)
                     elif domtype(SET) = Set then // special objects
                       FAIL // must be implemented
                     else
                       FAIL
                     end_if
                   end_proc:
  end_for:
end_proc():

// output of sets
Set::print:=
  proc(SET)
  begin
    if contains(Set::internalTypes, Set::type(SET)) <> 0
       and contains(slot(Set, Set::type(SET)), "print") then
      (slot(Set, Set::type(SET)))::print(SET)
    //elif extop(SET) = "universe" then // the top set
    //  Set::Complex
    else // special objects
      extop(SET)
    end_if
  end_proc:

alias(path = pathname("SET")):

// internal data types
Set::Rectform   := loadproc(Set::Rectform,   path, "Rectform"  ):
Set::Polarform  := loadproc(Set::Polarform,  path, "Polarform" ):
Set::Realdom    := loadproc(Set::Realdom,    path, "Realdom"   ):
Set::Finite     := loadproc(Set::Finite,     path, "Finite"    ):
Set::Interval   := loadproc(Set::Interval,   path, "Interval"  ):
Set::Union      := loadproc(Set::Union,      path, "Union"     ):
Set::Intersect  := loadproc(Set::Intersect,  path, "Intersect" ):
Set::Minus      := loadproc(Set::Minus,      path, "Minus"     ):
Set::Bigunion   := loadproc(Set::Bigunion,   path, "Bigunion"  ):
Set::Imageset   := loadproc(Set::Imageset,   path, "Imageset"  ):

// generate n-ary and binary operators

Set::commassop  := loadproc(Set::commassop,  path, "commassop" ):
Set::binop      := loadproc(Set::binop,      path, "binop"     ):

// standard operations
Set::_union     := loadproc(Set::_union,     path, "commassop" ):
Set::_intersect := loadproc(Set::_intersect, path, "commassop" ):
Set::_minus     := loadproc(Set::_minus,     path, "binop"     ):

Set::_implies     := loadproc(Set::_implies, path, "binop"     ):

Set::_plus      := loadproc(Set::_plus,      path, "commassop" ):
Set::_subtract  := loadproc(Set::_subtract,  path, "binop"     ):
Set::_negate    := loadproc(Set::_negate,    path, "_negate"   ):
Set::_mult      := loadproc(Set::_mult,      path, "commassop" ):
Set::_divide    := loadproc(Set::_divide,    path, "binop"     ):
Set::_invert    := loadproc(Set::_invert,    path, "_invert"   ):
Set::_power     := loadproc(Set::_power,     path, "binop"     ):

// conversion
Set::coerce     := loadproc(Set::coerce,     path, "coerce"    ):

// conversion of standard data types
(eval((hold(slot)(t, k) := loadproc(hold(slot)(t, k), pathname("SET", "COERCE"), expr2text(t)))
        $ k in Set::internalTypes)
          $ t in [DOM_INT, DOM_RAT, DOM_FLOAT, DOM_COMPLEX,
                  DOM_INTERVAL/*, DOM_SET*/]):

// standard slots for generic conversable types
// Set::Realdom(..) _op_ Set::Rectform(..)
// => Set::Realdom::_op_Rectform:= createConversion(Set::Rectform, _op_, Reverse)
//    Set::Rectform::_op_Realdom:= createConversion(Set::Rectform, _op_)
Set::createConversion:=
  proc(convertTo, slotName, OPT = hold(None))
    option escape;
  begin
    if OPT = hold(None) then
      proc(X1, X2)
      begin
        slot(convertTo, slotName)(X1, Set::coerce(X2, convertTo))
      end_proc
    else // swap operands
      proc(X1, X2)
      begin
        slot(convertTo, slotName)(X2, Set::coerce(X1, convertTo))
      end_proc
    end_if
  end_proc:

Set::map:=
  proc(FUNC, SET, ADD = null())
    local ARGS;
  begin
    if contains(SET::dom, expr2text(FUNC)) then // method exists
      slot(SET::dom, expr2text(FUNC))(SET, ADD)
    elif ADD <> null() and contains(ADD::dom, expr2text(FUNC)) then // method exists
      slot(ADD::dom, expr2text(FUNC))(SET, ADD)
    else // no method
      Set::Imageset(FUNC(`#X`), `#X`, SET, ADD)
    end_if
  end_proc:


//
((slot(Set, k) := subs(() -> Set::map("SUBS", args()), "SUBS" = k))
  $ k in ["Ci", "D", "Ei",
          "Im", "Re", "Si",
          //"_mod",
          
          "abs", "arccos", "arccosh",
          "arccot", "arccoth",
          "arccsc", "arccsch", "arcsec", "arcsech", "arcsin", "arcsinh",
          "arctan", "arctanh", "arg",
          "bernoulli", "besselI", "besselJ", "besselK", "besselY",
          "beta",
          "binomial",
          "conjugate",
          "cos", "cosh",
          "cot", "coth", "csc", "csch",
          "dilog", "dirac",
          "erf", "erfc", "exp", "fact",
          "gamma", "heaviside",
          "lambertV", "lambertW",
          "ln", "log",
          //"max", "min",
          "psi",
          "sec", "sech", "sign", "signIm",
          "sin",
          "sinh", "sqrt",
          "tan", "tanh", "zeta"
          ]):

// special method for pretty output
Set::Z:=
  proc(SET)
  begin
    if contains(Set::internalTypes, Set::type(SET)) <> 0
       and contains(slot(Set, Set::type(SET)), "Z") then
      (slot(Set, Set::type(SET)))::Z
    else
      ""
    end_if
  end_proc:

////////////////////////////////////////////////////////////////////////////////////////
// predefined objects
////////////////////////////////////////////////////////////////////////////////////////
Set::Universe    := new(Set, "universe"):                         // all
// domain Set::Rectform
Set::Complex     := loadproc(Set::Complex,     path, "Rectform"): // complex numbers
// domain Set::Realdom
Set::Real        := loadproc(Set::Real       , path, "Realdom" ): // real numbers
Set::Negative    := loadproc(Set::Negative   , path, "Realdom" ): // negative numbers
Set::NonNegative := loadproc(Set::NonPositive, path, "Realdom" ): // nonnegative numbers
Set::NonZeroReal := loadproc(Set::NonZeroReal, path, "Realdom" ): // nonzero real numbers
Set::NonPositive := loadproc(Set::NonPositive, path, "Realdom" ): // nonpositive numbers
Set::Positive    := loadproc(Set::Positive   , path, "Realdom" ): // positive numbers
// domain Set::Realdom
Set::Rational    := loadproc(Set::Rational   , path, "Realdom" ): // real numbers
Set::NegRat      := loadproc(Set::NegRat     , path, "Realdom" ): // negative numbers
Set::NonNegRat   := loadproc(Set::NonNegRat  , path, "Realdom" ): // nonnegative numbers
Set::NonZeroRat  := loadproc(Set::NonZeroRat , path, "Realdom" ): // nonzero real numbers
Set::NonPosRat   := loadproc(Set::NonPosRat  , path, "Realdom" ): // nonpositive numbers
Set::PosRat      := loadproc(Set::PosRat     , path, "Realdom" ): // positive numbers
// domain Set::Realdom
Set::Integer     := loadproc(Set::Integer    , path, "Realdom" ): // real numbers
Set::NegInt      := loadproc(Set::NegInt     , path, "Realdom" ): // negative numbers
Set::NonNegInt   := loadproc(Set::NonNegInt  , path, "Realdom" ): // nonnegative numbers
Set::NonZeroInt  := loadproc(Set::NonZeroInt , path, "Realdom" ): // nonzero real numbers
Set::NonPosInt   := loadproc(Set::NonPosInt  , path, "Realdom" ): // nonpositive numbers
Set::PosInt      := loadproc(Set::PosInt     , path, "Realdom" ): // positive numbers
// domain Set::Finite
Set::Empty       := loadproc(Set::Empty,       path, "Finite"  ): // empty set
Set::Zero        := loadproc(Set::Zero,        path, "Finite"  ): // zero
Set::One         := loadproc(Set::One,         path, "Finite"  ): // one
// Set::Minus
Set::NonZero     := loadproc(Set::NonZero,     path, "Minus"   ): // non zero
////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////

null():
