/*****************************************************************************
  $Id: operators.mu,v 1.35 2004/10/03 08:17:38 nthiery Exp $
  File:		lib/LIBFILES/operators.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Created:	27/09/2002
  License:	LGPL
*****************************************************************************/
/**
 * Test if the package Combinat is loaded, or is this the combinat file
 * from the official MuPAD library.
 **/
%if traperror(packages::Combinat) = 0 then
    alias(CombinatIsLoaded = TRUE)
else
    alias(CombinatIsLoaded = FALSE)
end:

domain operators

    info := "Library 'operators': tools for constructing operators";

    interface :=
    {
     hold(overloaded),
     if CombinatIsLoaded then
     hold(makeAssociative),
     hold(makeLinear),
     hold(makeBilinear),
     hold(repeatedSquaring),
     hold(tensorProductOfMaps),
     hold(plus2),
     hold(mult2),
     hold(_power),
     hold(coproduct),
     hold(antipode),
     hold(mu),
     hold(tensor),
     hold(scalar),
     hold(persistentRemember)
     else
	null();
     end_if
    };

    typeFunction := Type::Union(Type::Function, operators::overloaded);
    
    //////////////////////////////////////////////////////////////////////////
    // Tools to build operators
    //////////////////////////////////////////////////////////////////////////

    makeAssociative_Inner :=
    proc(f: dom::typeFunction)
	option escape;
	local options, zero, source, imageSet;
    begin
	options := combinat::getOptions(dom::makeAssociative, 1, Error):
	source  := options[Source];
	imageSet:= options[ImageSet];
	if contains(options, Zero) then
	    zero := options[Zero];
	elif domtype(imageSet)=DOM_DOMAIN and imageSet::zero<>FAIL then
	    zero := imageSet::zero;
	else
	    zero := 0;
	end_if;
	proc() : imageSet
	    local p, i;
	begin
	    if args(0) = 0 then
		return(zero);
	    end_if;
	    if testargs() then
		for i from 1 to args(0) do
		    if not testtype(args(i), source) then
			error("Wrong type of ".i.". argument (type '".expr2text(source)."' expected,\ngot argument '".expr2text(args(i)).");");
		    end_if;
		end_for;
	    end_if;
	    p := args(1);
	    for i from 2 to args(0) do
		p := f(p,args(i));
	    end_for;
	    return(p);
	end_proc:
    end_proc;

    makeAssociative :=
    funcenv(dom::makeAssociative_Inner,
	    FAIL,
	    table("options"     =      [Zero,
					Source   = Type::AnyType,
					ImageSet = Type::AnyType]));


    /**
     * Common utility for analysing the options related to the imageset
     * that is a module
     **/
    parseImageSetOptions :=
    proc(options: Type::OptionsTable) : Type::ListProduct(Type::AnyType $3)
	local imageSet, plus, zero, myMultcoeffs;
	option escape;
    begin
	imageSet := options[ImageSet];
	if contains(options, Multcoeffs) then
	    myMultcoeffs := options[Multcoeffs];
	elif domtype(imageSet)=DOM_DOMAIN and imageSet::multcoeffs <> FAIL then
	    myMultcoeffs := imageSet::multcoeffs;
	else
	    myMultcoeffs := multcoeffs;
	end_if;
	if domtype(imageSet)=DOM_DOMAIN and imageSet::plus <> FAIL then
	    plus := imageSet::plus;
	else
	    if contains(options, Zero) then
		zero := options[Zero];
	    elif domtype(imageSet)=DOM_DOMAIN and imageSet::zero<>FAIL then
		zero := imageSet::zero;
	    else
		zero := 0;
	    end_if;
	    if zero <> 0 then
		plus := () -> if args(0)=0 then
				  zero;
			      else
				  _plus(args())
			      end_if;
	    else
		plus := _plus;
	    end_if;
	end_if;
	[imageSet, plus, myMultcoeffs]
    end_proc;

    /**
	Assumptions:
	E and F are free modules over the same coefficient ring.
        The basis elements of E and F commute with scalars (this
	does not imply that the coefficient ring is commutative!)
        f is a function from E::basisIndices to F
	
	Returns the linear extension of f to E as a function from E to F.

	Extra arguments of the constructed operator are passed down to f.
    **/
    makeLinear :=
    funcenv(prog::bless(
    proc(f: dom::typeFunction)
	option escape;
	local options, plus, source, imageSet, myMultcoeffs;
    begin
	options := combinat::getOptions(dom::makeLinear, 1, Error):
	source  := options[Source];
	[imageSet, plus, myMultcoeffs] := dom::parseImageSetOptions(options);
	proc(x: source) : imageSet
	    name makeLinear(f);
	    local s;
	begin
	    plus(myMultcoeffs(f(op(s,2), args(2..args(0))),
			      op(s,1))
		 $ s in poly2list(x));
	end_proc;
    end_proc,
			dom),
	    FAIL,
	    table("options"     =      [Zero, hold(Multcoeffs),
					Source   = Type::AnyType,
					ImageSet = Type::AnyType]));
    /* TODO: refine the type checking
    		  "optionTypes" = table(Zero     = Type::AnyType,
					ImageSet = DOM_DOMAIN))); */

    /*************************************************************************
        makeBilinear(f, Source1=E1, Source2=E2, ImageSet=F, Zero)

        E1, E2, F are free modules over the same coefficient ring.

        f is a function from s1::basisIndices x s2::basisIndices to s3
	
	Returns the linear extension of f to s1 x s2

	Extra arguments of the constructed operator are passed down to f.
    *************************************************************************/

    
    makeBilinear :=
    funcenv(prog::bless(
    proc(f: dom::typeFunction)
	option escape;
	local options, source1, source2, imageSet, plus, myMultcoeffs;
    begin
	options := combinat::getOptions(dom::makeBilinear, 1, Error):
	source1  := options[Source];
	source2  := options[Source];
	if contains(options, Source1) then
	    source1 := options[Source1];
	end_if;
	if contains(options, Source2) then
	    source2 := options[Source2];
	end_if;
	[imageSet, plus, myMultcoeffs] := dom::parseImageSetOptions(options);
	proc(x1: source1, x2: source2) : imageSet
	    name makeBilinear(f);
	    local s1, s2;
	begin
	    plus(myMultcoeffs(f(op(s1,2), op(s2,2), args(3..args(0))),
			      op(s1,1)*op(s2,1))
		 $ s1 in poly2list(x1)
		 $ s2 in poly2list(x2));
	end_proc;
    end_proc,
			dom),
	    FAIL,
	    table("options"     =      [Zero,
					Source   = Type::AnyType,
					Source1,
					Source2,
					ImageSet = Type::AnyType]));
    /* TODO: refine the type checking
    		  "optionTypes" = table(Zero     = Type::AnyType,
					ImageSet = DOM_DOMAIN))); */

    makeBilinearProduct_Table :=
    proc(product: dom::typeFunction)
        // product returns the product of two basis elements of F, as a table
        // table(term1 = coeff1, ...) or a list [term1=coeff1, ...]
	local DOM;
	option escape;
    begin
	DOM := dom;
	proc(x1: DOM, x2: DOM): DOM
	    local t, s1, s2, r, s;
	begin
	    t := table();
	    for s1 in REP(x1) do
		for s2 in REP(x2) do
		    r := op(s1,2)*op(s2,2);
		    for s in product(op(s1,1), op(s2,1)) do
			t:=DOM::addToRep(t,op(s,1), r*op(s,2)); // R ?
		    end_for;
		end_for
	    end_for;

	    return(new(dom, t));
	end_proc;
    end_proc;

    makeBilinearProduct := dom::makeBilinearFunction;

    makeAssociativeAlgebraProduct :=
    /**
       Given a binary product (DOM,DOM)->DOM, and an external product
       (coeffRing, DOM) -> DOM, build a product for the corresponding
       associative algebra over the coefficient ring.
       Assumptions:
	 - the base ring commutes with elements of dom
         - the binary product is associative
	 - DOM is not a facade domain
       Type coercion:
	 - at the toplevel (testargs()=TRUE):
	    - test if domtype(x) = DOM
	    - test if domtype(x) = coeffRing
	    - try to coerce x into DOM (too dangerous)
	    - try to coerce x into coeffRing
	    - otherwise, throw an error
	 - internally:
	    - test if domtype(x) = DOM
	    - otherwise it's a coefficient
    **/
    proc(DOM: DOM_DOMAIN, 
	 coeffRing: DOM_DOMAIN,
	 one,
	 externalProduct: dom::typeFunction,
	 binaryProduct: dom::typeFunction)
	option escape;
    begin
	proc()
	    name AssociativeAlgebraProduct;
	    local x, r, rs, xs, others, i;
	begin
	    if args(0)=0 then return(one); end_if;
	    if testargs() then
		xs:=[];
		rs:=[];
		for x in [args()] do
		    if domtype(x)=DOM then
			xs := append(xs,x);
		    elif domtype(x)=coeffRing then
			rs := append(rs,x);
		    elif (r := coerce(x, DOM)) <> FAIL then
			xs := append(xs,r);
		    elif (r := coerce(x, coeffRing)) <> FAIL then
			rs := append(rs,r);
		    else
			error("wrong type of argument");
		    end_if;
		end_for;
	    else
		([xs, rs, others]) := split([args()], 
					    x->domtype(x)=DOM);
	    end_if;
	    if nops(xs)=0 then 
		error("wrong type of argument");
	    end_if;
	    x:=xs[1];
	    for i from 2 to nops(xs) do
		x:=binaryProduct(x,xs[i])
	    end_for;
	    if nops(rs)>0 then
		x:=externalProduct(x, coeffRing::_mult(op(rs)))
	    end_if;
	    x;
	end_proc;
    end_proc;


begin
end_domain:

alias(path = pathname("OPERATORS")):
alias(loaddomain=prog::loaddomain):

operators::repeatedSquaring :=
    loadproc(operators::repeatedSquaring, path, "repeatedSquaring"):
operators::overloaded :=
    loaddomain(operators::overloaded, path, "overloaded"):
operators::_plus :=
    loadproc(operators::_plus, path, "overloaded"):
operators::plus2 :=
    loadproc(operators::plus2, path, "overloaded"):
operators::_mult :=
    loadproc(operators::_mult, path, "overloaded"):
operators::mult2 :=
    loadproc(operators::mult2, path, "overloaded"):
operators::_power :=
    loadproc(operators::_power, path, "overloaded"):
operators::tensor :=
    loadproc(operators::tensor, path, "overloaded"):
operators::tensor2 :=
    loadproc(operators::tensor2, path, "overloaded"):
operators::mu :=
    loadproc(operators::mu, path, "overloaded"):
operators::coproduct :=
    loadproc(operators::coproduct, path, "overloaded"):
operators::antipode :=
    loadproc(operators::antipode, path, "overloaded"):
operators::scalar :=
    loadproc(operators::scalar, path, "overloaded"):
operators::tensorProductOfMaps :=
    loadproc(operators::tensorProductOfMaps, path, "tensorProductOfMaps"):

operators::persistentRemember :=
    loaddomain(operators::persistentRemember, path, "persistentRemember"):

null():
