// $Date: 2003/05/23 12:56:28 $ $Author: tonner $ $Revision: 1.52 $ 

// kg, 13/01/94 

/*--
specfunc.mu -- define/load special functions
--*/

specfunc::interface := {}:

specfunc::exported := { 
hold(sin),hold(cos),hold(tan),hold(arcsin),hold(arccos),hold(arctan),
hold(arg),  
hold(sinh), hold(cosh),hold(tanh),
hold(arcsinh),hold(arccosh),hold(arctanh),
hold(exp),
hold(ln),hold(sqrt),hold(surd),
hold(gamma),hold(igamma),hold(zeta),hold(erfc),
hold(Ei),
hold(ceil),hold(floor),hold(frac),hold(trunc),hold(round),
hold(lambertW),hold(psi),
hold(Ci),hold(Si),hold(arccsc),
hold(cot),hold(csc),hold(sec),hold(arccot),hold(arcsec),hold(coth),
hold(csch),hold(arccoth),hold(arccsch),hold(arcsech),
hold(besselI), hold(besselJ), hold(besselY),hold(besselK),
hold(hypergeom),
// hold(meijerG), // mangels Dokumentation nicht offiziell
hold(erf),hold(fact),hold(fact2),hold(dirac),
hold(heaviside),hold(Si),hold(Ci),hold(abs),hold(beta),hold(dilog),
hold(polylog),hold(bernoulli),hold(binomial),
hold(sign), hold(signIm), hold(log),
hold(kummerU), hold(whittakerM), hold(whittakerW), hold(airyAi), hold(airyBi)
}:

alias(path = pathname("SPECFUNC")):

/* define special functions which are evaluated numerically by system
  functions */

sin:= loadproc(sin, path, "sin"):
cos:= loadproc(cos, path, "cos"):
tan:= loadproc(tan, path, "tan"):
arcsin:= loadproc(arcsin, path, "asin"):
arccos:= loadproc(arccos, path, "acos"):
arctan:= loadproc(arctan, path, "atan"):
arg:= loadproc(arg, path, "arg"):
sinh:= loadproc(sinh, path, "sinh"):
cosh:= loadproc(cosh, path, "cosh"):
tanh:= loadproc(tanh, path, "tanh"):
arcsinh:= loadproc(arcsinh, path, "asinh"):
arccosh:= loadproc(arccosh, path, "acosh"):
arctanh:= loadproc(arctanh, path, "atanh"):
exp:= loadproc(exp, path, "exp"):
ln:= loadproc(ln, path, "ln"):
log:= loadproc(log, path, "log"):
sqrt:= loadproc(sqrt, path, "sqrt"):
surd:= loadproc(surd, path, "surd"):
gamma:= loadproc(gamma, path, "gamma"):
zeta:= loadproc(zeta, path, "zeta"):
erfc:=loadproc(erfc, path, "erfc"):
Ei:= loadproc(Ei, path, "Ei"):
lambertW:= loadproc(lambertW, path, "lambertW"):
Ci:=loadproc(Ci, path, "Ci"):
Si:=loadproc(Si, path, "Si"):
psi:= loadproc(psi, path, "psi"):
// specfunc::ibeta = undocumented utility function without
// interface, used for statistical distributions
specfunc::ibeta:= loadproc(specfunc::ibeta, path, "ibeta"):
specfunc::ibeta2:= loadproc(specfunc::ibeta2, path, "ibeta2"):
// specfunc::inverf = undocumented utility function without
// interface, used for statistical distributions
specfunc::inverf:= loadproc(specfunc::inverf, path, "inverf"):
igamma:= loadproc(igamma, path, "igamma"):
floor:= loadproc(floor, path, "floor"):
ceil := loadproc(ceil, path, "ceil"):
frac := loadproc(frac, path, "frac"):
round:= loadproc(round, path, "round"):
trunc:= loadproc(trunc, path, "trunc"):
cot:= loadproc(cot, path, "cot"):
csc:= loadproc(csc, path, "csc"):
sec:= loadproc(sec, path, "sec"):
arccot:= loadproc(arccot, path, "acot"):
arcsec:= loadproc(arcsec, path, "asec"):
arccsc:= loadproc(arccsc, path, "acsc"):
coth:= loadproc(coth, path, "coth"):
csch:= loadproc(csch, path, "csch"):
sech:= loadproc(sech, path, "sech"):
arccoth:= loadproc(arccoth, path, "acoth"):
arccsch:= loadproc(arccsch, path, "acsch"):
arcsech:= loadproc(arcsech, path, "asech"):
besselI:= loadproc(besselI, path, "besselI"):
besselJ:= loadproc(besselJ, path, "besselJ"):
besselY:= loadproc(besselY, path, "besselY"):
besselK:= loadproc(besselK, path, "besselK"):
hypergeom:= loadproc(hypergeom, path, "hypergeom"):
meijerG:= loadproc(meijerG, path, "meijerG"):
specfunc::MeijerG := loadproc(specfunc::MeijerG, path, "meijerG"):
erf:= loadproc(erf, path, "erf"):
fact:= loadproc(fact, path, "fact"):
fact2:= loadproc(fact2, path, "fact2"):
dirac:= loadproc(dirac, path, "dirac"):
heaviside:= loadproc(heaviside, path, "heavisid"):
abs:= loadproc(abs, path, "abs"):
beta:= loadproc(beta, path, "beta"):
dilog:= loadproc(dilog, path, "dilog"):
polylog:= loadproc(polylog, path, "polylog"):
bernoulli:= loadproc(bernoulli, path, "bernoull"):
binomial:= loadproc(binomial, path, "binomial"):
sign:= loadproc(sign, pathname("SPECFUNC","SIGN"), "sign"):
signIm:= loadproc(signIm, path, "signIm"):
whittakerM:= loadproc(whittakerM, path, "whittakerM"):
whittakerW:= loadproc(whittakerW, path, "whittakerW"):
kummerU:= loadproc(kummerU, path, "kummerU"):
airyAi:= loadproc(airyAi, path, "airyAi"):
airyBi:= loadproc(airyBi, path, "airyBi"):

/* Auxilary functions for bessel{IJKY} */

specfunc::Bessel:= loadproc(specfunc::Bessel, path, "bess_aux"):
specfunc::Hypergeom:= loadproc(specfunc::Hypergeom, path, "hyper_aux"):
specfunc::besselDiffMain:= loadproc(specfunc::besselDiffMain,
                        path, "besselDiffMain"):

null():

// end of file 

