// $Date: 2005/01/17 11:41:20 $  $Author: cube $  $Revision: 1.385.2.1 $ #
// stdlib.mu -- lib-file for standard library STDLIB

stdlib::exported:=
  {hold(D),
   hold(Im),
   hold(O),
   hold(Re),
   hold(RootOf),
   hold(_implies),
   hold(_in),
   hold(_notsubset),
   hold(_subset),
   hold(_xor),
   hold(anames),
   hold(assert),
   hold(assign),
   hold(assume),
   hold(asympt),
   hold(bytes),
   hold(card),
   hold(coeff),
   hold(coerce),
   hold(collect),
   hold(combine),
   hold(complexInfinity),
   hold(conjugate),
   hold(contains),
   hold(content),
   hold(contfrac),
   hold(degreevec),
   hold(denom),
   hold(diff),
   hold(discont),
   hold(doprint),
   hold(evalAt),
   hold(expand),
   hold(export),
   hold(expose),
   hold(expr),
   hold(expr2text),
   hold(factor),
   hold(fclose),
   hold(fopen),
   hold(fprint),
   hold(freeze),
   hold(gcd),
   hold(gcdex),
   hold(genident),
   hold(getprop),
   hold(ground),
   hold(hastype),
   hold(icontent),
   hold(igcd),
   hold(igcdex),
   hold(ilcm),
   hold(infinity),
   hold(info),
   hold(int),
   hold(interpolate),
   hold(irreducible),
   hold(is),
   hold(isprime),
   hold(ithprime),
   hold(lcm),
   hold(lcoeff),
   hold(length),
   hold(lhs),
   hold(limit),
   hold(linsolve),
   hold(lmonomial),
   hold(lterm),
   hold(map),
   hold(maprat),
   hold(max),
   hold(min),
   hold(normal),
   hold(nthcoeff),
   hold(nthmonomial),
   hold(nthterm),
   hold(numer),
   hold(package),
   hold(pade),
   hold(partfrac),
   hold(piecewise),
   hold(plotfunc2d),
   hold(plotfunc3d),
   hold(powermod),
   hold(print),
   hold(product),
   hold(protocol),
   hold(radsimp),
   hold(random),
   hold(rationalize),
   hold(read),
   hold(readbytes),
   hold(rectform),
   hold(revert),
   hold(rewrite),
   hold(rhs),
   hold(rtime),
   hold(select),
   hold(series),
   hold(setuserinfo),
   hold(simplify),
   hold(solve),
   hold(split),
   hold(strmatch),
   hold(substring),
   hold(sum),
   hold(system),
   hold(taylor),
   hold(tcoeff),
   hold(testeq),
   hold(text2expr),
   hold(time),
   hold(unassume),
   hold(undefined),
   hold(unexport),
   hold(unfreeze),
   hold(universe),
   hold(version),
   hold(zip)
   }:
// Note: do not include 'densematrix' (see below) in order to be consistent
// with 'array', 'table', etc.

stdlib::interface:= {}:

/*--
  If a library routine often uses protected variables, these
  statements may be executed by syseval.
  This routine is used by loadlib and loadproc, so it has to
  be defined before the first execution of both.

  Usage:

     stdlib::syseval( expr )

  !!! stdlib::syseval is used inside of loadproc. Do not move !!!!
--*/
stdlib::syseval :=
proc()
  option hold, noDebug;
  local oldAssign, res;
begin
   oldAssign := ::_assign;
   sysassign(::_assign, sysassign);
   res := context(args());
   sysassign(::_assign, oldAssign);
  res
end_proc:

/*--
  The following sets are used for the protect mechanism of the
  MuPAD library. The variables in stdlib::ENVIRONMENT_VARIABLES
  and stdlib::SYSTEM_CONSTANTS are not protected, the variables
  in stdlib::LIBRARY_CONSTANTS are protected.
  The protection is done at the end of .mupadsysinit.
--*/
// use strings here in order to avoid subsequent evaluation of
// library package initialization files:
stdlib::LIBRARIES:= {
  hold(import), hold(Ax), hold(Cat), hold(Dom), hold(Graph), hold(RGB),
  hold(Series), hold(property), hold(Type), hold(adt), hold(intlib),
  hold(combinat), hold(fp), hold(generate), hold(groebner), hold(linalg),
  hold(listlib), hold(linopt), hold(matchlib), hold(module), hold(numeric),
  hold(numlib), hold(ode), hold(orthpoly), hold(output), hold(plot), hold(polylib),
  hold(prog), hold(solvelib), hold(stats), hold(stringlib), hold(student), hold(transform),
  hold(stdlib), hold(specfunc), hold(detools)
}:

stdlib::ENVIRONMENT_VARIABLES:= {
  hold(DIGITS), hold(HISTORY),
  hold(LEVEL), hold(LIBPATH), hold(PACKAGEPATH), hold(MAXDEPTH), hold(MAXLEVEL),
  hold(ORDER), hold(PRETTYPRINT), hold(READPATH),
  hold(ROUNDING), hold(SEED), hold(TEXTWIDTH), hold(WRITEPATH),
  hold(TESTPATH), hold(_mod)
}:


stdlib::SYSTEM_CONSTANTS := {
  hold(CATALAN),
  hold(EULER), hold(FALSE), hold(I), hold(PI), hold(TRUE),
  hold(UNKNOWN)
}:

stdlib::LIBRARY_CONSTANTS := {
  hold(infinity), hold(MSDOS), hold(toBeDefined), hold(UNBOUNDED),
  hold(undefined), hold(EMPTY), hold(OPTIMAL), hold(complexInfinity),
  hold(Global) // global property
}:

stdlib::OPTIONS := 
{
  hold(AbsoluteError), // (undocumented) option for numeric::quadrature,
                       // and (documented) for numeric::odesolve
//  hold(Above),
  hold(Adaptive),
  hold(Annotation),
  hold(All),
  hold(AllExponents),
  hold(AllRealRoots), // option of numeric::solve
  hold(Alldata),
  hold(AcceptSingularity),
  hold(Always), hold(Ansatz), hold(Any),
  hold(Append),
  hold(ApplyRule), // Simplify
  hold(Approx),
  hold(Arch),
  hold(Args),
  hold(Area), //option for plot::histogram2d/3d
//  hold(Arrows),
// hold(Attached),
  hold(Automatic),
  hold(Axes), hold(AxesOrigin),
  hold(AxesInFront),
  hold(Averaged), // option for stats::empiricalQuantile

  hold(BackGround),
  hold(BackSubstitution),
  hold(Banded),
  hold(BigEndian),
  hold(Bin),
  hold(Binary),
  hold(Bottom),
  hold(Boxed),
  hold(BravaisPearson), // option for stats::Correlation
  hold(BUTCHER6),
  hold(Byte),

//  hold(Capacity), // old Network stuff
  hold(CDF), // option for stats::csgoft
  hold(Cells), // for plot::histogram2d/3d
  hold(Center),
  hold(Circles),
  hold(CK45), hold(CK54),
  hold(Clear), hold(ClearPrevent), // for prog::remember
  hold(Closed),
  hold(Coeff),
  hold(Coeffs),
  hold(Color),
  hold(Colors),
//  hold(ColorPatches),
  hold(Columns), // Option fuer plotlib::dataplot
  hold(ColumnElimination), // Option fuer /*sparse*/matrix::gaussElim
  hold(Complete),
  hold(Connected),
  hold(Constrained),
  hold(Content),
  hold(Contfrac), // numeric::minimax
  hold(Continuous), // Option fuer int
  hold(Contours),
  hold(CovarianceMatrix), // option for stats::linReg and stats::reg
  hold(Crit),
  hold(Cyclic),

  hold(Default),  // used by numeric in scilab::func(..,ReturnType=[Default, ..])
  hold(DegInvLexOrder),
  hold(Degree),
  hold(DegreeOrder),
  hold(DegInvLex),
  hold(DegLex), hold(DegRevLex),
  hold(Delete), hold(Diagonal),
  hold(Diagonalization),
  hold(DiffVars),
  hold(Digits), // student::plotRiemann
  hold(Dimension),
  hold(Discard), // used by Simplify
  hold(Discont),
  hold(Domain),
  hold(DomainsOnly),
  hold(Double),
  hold(Directed),  // for Graph
  hold(DontRewriteBySystem),
  hold(DOPRI45),
  hold(DOPRI54),
  hold(DOPRI56),
  hold(DOPRI65),
  hold(DOPRI78),
  hold(DOPRI87),
  hold(DualPrices),
  hold(DrawMode), // Option fuer plot::Pointset
  hold(Duplicate),

  hold(Euclidean), // Option fuer norm auf Matrizen

  hold(Error),  // for prog::getOptions
  hold(EULER1),
//  hold(Eweight),
  hold(Exact),
  hold(ExpandFinite), // for sum
  hold(Expr),
  hold(Extended),

  hold(Factor),
  hold(Fechner), // option for stats::Correlation
  hold(Filled),
  hold(FilledCircles),
  hold(FilledSquares),
  hold(First),
  hold(FixedPrecision),
  hold(Flat),
  hold(Float),
  hold(FloatFormat),
  hold(FocalPoint),
  hold(Forced),
  hold(ForeGround),
  hold(Format),
  hold(Frames),
  hold(Frobenius),
  hold(Function),

  hold(Gauss),
  hold(GAUSS),
  hold(GaussChebyshev), hold(GaussLegendre),
  hold(GaussTschebyscheff),
  hold(Generic),
  hold(GenOrder),
  hold(Goal), // used by Simplify
  hold(GC), hold(GL), hold(GT),

  hold(Hard),           // shortcut for 'HardwareFloats'
  hold(HardwareFloats), // for the numeric routines: use scilab
  hold(Height), hold(Hlode), hold(HlodeOverRF),
  hold(Homogeneous),
  hold(Horizontal),

  hold(IgnoreProperties), hold(IgnoreSpecialCases),
  hold(Include), hold(Index), hold(IndexList),
  hold(Infinity), hold(InputOnly), hold(Interpolation), hold(IntMod),
  hold(InSet),
  hold(Interactive),
  hold(Integral), // used in plot::Integral
  hold(Intervals),
  hold(Irreducible),
  hold(InverseTransformation), // for linalg::ogCoordTab

  hold(KeepOrder),
  hold(Krylov),

  hold(Left), hold(Length), hold(LexOrder),
  hold(LieGroupAction),
  hold(Line), // Option fuer Turtleplot
  hold(Lines), // Option fuer plotlib::dataploat
  hold(LinesPoints), // Option fuer plotlib::dataplot
  hold(LineStyle), hold(LineWidth), hold(List),
  hold(LittleEndian),
  hold(Lode),
  hold(LodeOverRF), hold(Logic),
  hold(Lower), // student::riemann, student::plotRiemann

  hold(Maximum), // Option fr norm auf Matrizen

  hold(MaxDegree),
  hold(MaxOrder), // for sum
  hold(MaxInfoDepth), // Simplify
  hold(MaxRecLevel),
  hold(MaxCalls),
  hold(Mem),
  hold(Merge), // plot::boxplot
  hold(Mesh),
  hold(Middle), // student::riemann, student::plotRiemann
  hold(MinorExpansion),
  hold(ModuleTrace),
  hold(Monic),
  hold(Move), // Option fuer Turtleplot
  hold(Multiple),
  hold(MultiSolutions), // for numeric::fsolve
  hold(Minimize),

  hold(Name), hold(Nary), hold(Natural), hold(New),
  hold(NewtonCotes), hold(NoArgs), hold(NoCheck), hold(NoErrors),
  hold(NoLeftVectors), hold(NonNested), hold(NonNegative),
  hold(None), // changed to 'ProtectLevelNone'
  hold(Noop),
  hold(NoOperators),

  hold(NoNL), hold(NoRightVectors), hold(Normal), hold(NotAKnot), hold(NC),
  hold(Not),
  hold(Notched), // plot::boxplot
  hold(NoWarning),

  hold(OnlySpecifiedEdges),  // for Graph
  hold(Open), hold(Origin),
  hold(Order), hold(OutSet), hold(Output), hold(OutputDomain), hold(OutputOnly),
  hold(OldFromNew),
  hold(OutSet), hold(Output), hold(OutputDomain), hold(OutputType),

  hold(Path),
  hold(PDF),
  hold(Periodic),
  hold(PF),
  hold(Plain),
  hold(Preconditioning),
  hold(Prefix),
  hold(Presentation),
  hold(Postfix),
  hold(Points), // Option fuer plotlib::dataplot
  hold(PointStyle),
  hold(PolyExpr),
  hold(Pop),
  hold(Population), // Option for stats::stdev und stats::variance
  hold(PostMap),
  hold(PreMap),
  hold(Pretty),
  hold(PreventRecursion), // for prog::remember
  hold(PrimeLimit), // Option for ithprime and ifactor
  hold(Primitive),
  hold(PrincipalValue),
  hold(Print), // for prog::remember
  hold(Proc), hold(Properties), hold(Protected),
  hold(ProtectLevelError),
  hold(ProtectLevelNone),
  hold(ProtectLevelWarning),
  hold(Push),

  hold(Quiet),
  hold(Quo),
  hold(QDet),
  hold(QRD), // option for numeric::leastSquares (QR-decomposition)
  hold(Random), //replaces option Seed = s in numeric::fsolve
  hold(Ranges), // for linalg::ogCoordTab
  hold(RatExpr),
  hold(Read), // option for fopen
  hold(Real),
  hold(Recursive),
  hold(RelativeError),
  hold(Rem),
  hold(Remember), // Simplify, prog::remember
  hold(RememberLast), // for numeric::odesolve2
  hold(Reorder),
  hold(Rest),
  hold(RestrictedSearch),
  hold(Restore),
  hold(ReturnType), // for the numeric routines dealing with matrices
  hold(Right),
  hold(RK4),
  hold(RKF43),
  hold(RKF34),
  hold(RKF54a),
  hold(RKF45a),
  hold(RKF54b),
  hold(RKF45b),
  hold(RKF87),
  hold(RKF78),
  hold(Root),
  hold(RuleBase), // Simplify

  hold(Sample),
  hold(Scales),
  hold(Scaling),
  hold(SearchLevel),
  hold(Seconds), // used by Simplify
  hold(Seed), // for various stats::XXXrandom
  hold(SelectGlobalRules), // Simplify
  hold(SelectRules),  // Simplify
  hold(Shift), // for plot::boxplot
  hold(Show),  // for Graph
  hold(Soft),           // shortcut for 'SoftwareFloats'
  hold(SoftwareFloats), // for numeric:: 'HardwareFloats' versus 'SoftwareFloats'
  hold(Sparse),
  hold(Short),
  hold(ShowAssumptions),
  hold(ShowDependencies),
  hold(SignedByte),
  hold(SignedShort),
  hold(SignedWord),
  hold(Solved),
  hold(Sort),  // output::tableForm
  hold(Special),
  hold(SquareFree),
  hold(StartingValues), // Option fuer stats::reg
  hold(Steps),
  hold(Stepsize),
  hold(Strategy), // Simplify
  hold(Style),
  hold(Subset),
  hold(Superset),
  hold(SVD), // option for numeric::leastSquares (singular value decomposition)
  hold(Symbolic),
  hold(SymbolicAnalysis),
  hold(Symmetric), hold(Squares),
  hold(System),

  hold(TaylorExpansion),
  hold(TempFile),
  hold(Terms),
  hold(Test),
  hold(Text),
  hold(Title),

  hold(TitlePosition),
  hold(Titles),
  hold(ToNearest),
  hold(ToInfinity),

  hold(ToMinusInfinity),
  hold(ToZero),
  hold(Trace), // Simplify
  hold(TrailingZeroes),
  hold(Transform),
  hold(Transformation), // for linalg::ogCoordTab
  hold(Transposed),
  hold(Tree),
  hold(Types),
  hold(Typeset),

  hold(Undefined),
  hold(Undirected),
  hold(Unique),
  hold(Unquoted),
  hold(UnrestrictedSearch), // for numeric::fsolve
  hold(User),
  hold(Unsimplified),
  hold(Unsorted),
  hold(Up),
  hold(Upper), // student::riemann, student::plotRiemann
  hold(UsePrimeTab),
  hold(UnitVectors), // for linalg::ogCoordTab

  hold(Valuation), // Simplify
  hold(ViewingBox),
//  hold(Vweight), // old Network stuff
  hold(Vars),
  hold(Vertical),

  hold(Laurent),
  hold(Taylor),
  hold(Puiseux),

  hold(Warning),
  hold(Width), //option for plot::histogram2d/3d
  hold(Word),
  hold(Write),

//  hold(X11),
  hold(xCK54),
  hold(xCK45),
  hold(xDOPRI54),
  hold(xDOPRI45),
  hold(xDOPRI56),
  hold(xDOPRI65),
  hold(xDOPRI78),
  hold(xDOPRI87),
  hold(xRKF43),
  hold(xRKF34),
  hold(xRKF54a),
  hold(xRKF45a),
  hold(xRKF54b),
  hold(xRKF45b),
  hold(xRKF78),
  hold(xRKF87),
  hold(XMax),
  hold(XMin),

  hold(YMax),
  hold(YMin),

  hold(ZMax),
  hold(ZMin),

  // targets for rewrite: first letter is lower case

  hold(andor),
  hold(sincos),
  hold(sinhcosh),

  // new plot options
  hold(AffectViewingBox),
  hold(AnimationStyle),
  hold(AVIMode),
  hold(Absolute),
  hold(AdaptiveMesh),
  hold(Angle),
  hold(ArrowLength),
  hold(Automatic),
  hold(AutoPlay),
  hold(AxesInFront),
  hold(AxesLineColor),
  hold(AxesLineWidth),
  hold(AxesOriginX),
  hold(AxesOriginY),
  hold(AxesOriginZ),
  hold(AxesStyle),
  hold(AxesTips),
  hold(AxesTitleAlignment),
  hold(AxesTitleFont),
  hold(AxesTitles),
  hold(AxesVisible),
  hold(Axis),
  hold(AxisX),
  hold(AxisY),
  hold(AxisZ),
  hold(BackAndForth), 
  hold(BackgroundColor),
  hold(BackgroundColor2),
  hold(BackgroundStyle),
  hold(BackgroundTransparent),
  hold(Base),
  hold(BaseLine),
  hold(BaseRadius),
  hold(BaseRadius),
  hold(BaseX),
  hold(BaseY),
  hold(BaseZ),
  hold(Baseline),
  hold(Begin),
  hold(Billboarding),
  hold(Bold),
  hold(BorderColor),
  hold(BorderWidth),
  hold(Bottom),
  hold(BottomMargin),
  hold(CameraCoordinates),
  hold(CameraDirection),
  hold(CameraDirectionX),
  hold(CameraDirectionY),
  hold(CameraDirectionZ),
  hold(Center),
  hold(CenterX),
  hold(CenterY),
  hold(CenterZ),
  hold(Circles),
  hold(Closed),
  hold(Colors),
  hold(Constrained),
  hold(Contours),
  hold(CoordinateType),
  hold(CrossedLines),
  hold(Crosses),
  hold(CurrentTime),
  hold(Dashed),
  hold(Diagonal),
  hold(DiagonalLines),
  hold(Diamonds),
  hold(Dichromatic),
  hold(DiscontinuitySearch),
  hold(DotsPerInch),
  hold(Dotted),
  hold(EPSMode),
  hold(End),
  hold(EvenOdd),
  hold(Extension),
  hold(FDiagonalLines),
  hold(FillColor),
  hold(FillColorDirection),
  hold(FillColorDirectionX),
  hold(FillColorDirectionY),
  hold(FillColorDirectionZ),
  hold(FillColor2),
  hold(FillColorFunction),
  hold(FillColorType),
  hold(FillPattern),
  hold(FillStyle),
  hold(Filled),
  hold(FilledCircles),
  hold(FilledDiamonds),
  hold(FilledSquares),
  hold(Finite),
  hold(Fixed),
  hold(Flat),
  hold(FocalPointX),
  hold(FocalPointY),
  hold(FocalPointZ),
  hold(Footer),
  hold(FooterAlignment),
  hold(FooterFont),
  hold(Frame),
  hold(Frames),
  hold(FramesPerSecond),
  hold(From),
  hold(FromX),
  hold(FromY),
  hold(FromZ),
  hold(Function),
  hold(Functional),
  hold(Function1),
  hold(Function2),
  hold(GridInFront),
  hold(GridLineColor),
  hold(GridLineStyle),
  hold(GridLineWidth),
  hold(GridVisible),
  hold(Header),
  hold(HeaderAlignment),
  hold(HeaderFont),
  hold(Height),
  hold(High),
  hold(Horizontal),
  hold(HorizontalAlignment),
  hold(HorizontalLines),
  hold(Infinite),
  hold(InitialTime), 
  hold(InterpolationStyle), 
  hold(Italic),
  hold(JPEGMode),
  hold(KeepUpVector), 
  hold(Layout),
  hold(Left),
  hold(LeftMargin),
  hold(LeftRight),
  hold(Legend),
  hold(LegendEntry),
  hold(LegendAlignment),
  hold(LegendColor),
  hold(LegendFont),
  hold(LegendPlacement),
  hold(LegendText),
  hold(LegendVisible),
  hold(LightColor),
  hold(LightIntensity),
  hold(Lighting),
  hold(LinLin),
  hold(LinLog),
  hold(LineColor),
  hold(LineColorDirection),
  hold(LineColorDirectionX),
  hold(LineColorDirectionY),
  hold(LineColorDirectionZ),
  hold(LineColor2),
  hold(LineColorFunction),
  hold(LineColorType),
  hold(LineStyle),
  hold(LineWidth),
  hold(Linear),
  hold(Lines),
  hold(LinesVisible),
  hold(LogLin),
  hold(LogLog),
  hold(Logarithmic),
  hold(Loop), 
  hold(Low),
  hold(Margin),
  hold(Matrix2d),
  hold(Matrix3d),
  hold(Mesh),
  hold(MeshVisible),
  hold(MinimalExpectedCellFrequency),
  hold(Monochrome),
  hold(Name),
  hold(None),
  hold(NormalX),
  hold(NormalY),
  hold(NormalZ),
  hold(Open),
  hold(OrthogonalProjection),
  hold(OutputFile),
  hold(OutputOptions),
  hold(OutputUnits),
  hold(ParameterBegin),
  hold(ParameterEnd),
  hold(ParameterName),
  hold(ParameterRange),
  hold(ParameterValue),
  hold(PlotAt),
  hold(PointColor),
  hold(PointSize),
  hold(PointStyle),
  hold(Points),
  hold(Points2d),
  hold(Points3d),
  hold(PointsVisible),
  hold(Position),
  hold(PositionX),
  hold(PositionY),
  hold(PositionZ),
  hold(Proportional),
  hold(PValue), hold(PValue1), hold(PValue2), // for stats::xyzGOFT
  hold(Pyramid),
  hold(Quality),
  hold(Radius),
  hold(Rainbow),
  hold(ReduceTo256Colors),
  hold(Relative),
  hold(Right),
  hold(RightMargin),
  hold(RotationAngle),
  hold(Rows),
  hold(RunOnce), 
  hold(Scale),
  hold(ScaleX),
  hold(ScaleY),
  hold(ScaleZ),
  hold(Scaling),
  hold(SemiAxes),
  hold(SemiAxisX),
  hold(SemiAxisY),
  hold(SemiAxisZ),
  hold(SemiInfinite),
  hold(Shading),
  hold(ShiftX),
  hold(ShiftY),
  hold(ShiftZ),
  hold(Shifted),
  hold(Smooth),
  hold(Solid),
  hold(Splines),
  hold(Spacing),
  hold(SpotAngle),
  hold(Squares),
  hold(Stars),
  hold(StartingPoint),
  hold(StatValue), hold(StatValue1), hold(StatValue2), // for stats::xyzGOFT
  hold(SubgridLineColor),
  hold(SubgridLineStyle),
  hold(SubgridLineWidth),
  hold(SubgridVisible),
  hold(Submesh),
  hold(Tabular),
  hold(Target),
  hold(TargetX),
  hold(TargetY),
  hold(TargetZ),
  hold(TextFont),
  hold(TextOrientation),
  hold(TextRotation),
  hold(TicksAnchor),
  hold(TicksAt),
  hold(TicksBetween),
  hold(TicksDistance),
  hold(TicksLabelFont),
  hold(TicksLabelStyle),
  hold(TicksLabelsVisible),
  hold(TicksLength),
  hold(TicksNumber),
  hold(TicksVisible),
  hold(TimeBegin),
  hold(TimeEnd),
  hold(TimeRange),
  hold(TimeValue),
  hold(TipAngle),
  hold(TipLength),
  hold(TipStyle),
  hold(Title),
  hold(TitleAlignment),
  hold(TitleFont),
  hold(TitlePositionX),
  hold(TitlePositionY),
  hold(TitlePositionZ),
  hold(To),
  hold(ToX),
  hold(ToY),
  hold(ToZ),
  hold(Top),
  hold(TopBottom),
  hold(TopMargin),
  hold(TopRadius),
  hold(TopX),
  hold(TopY),
  hold(TopZ),
  hold(TubeDiameter),
  hold(Tubes),
  hold(Tubular),
  hold(ULinesVisible),
  hold(UMax),
  hold(UMesh),
  hold(UMin),
  hold(UName),
  hold(UpVector), 
  hold(UpVectorX), 
  hold(UpVectorY), 
  hold(UpVectorZ), 
  hold(URange),
  hold(USubmesh),
  hold(Unconstrained),
  hold(VLinesVisible),
  hold(VMax),
  hold(VMesh),
  hold(VMin),
  hold(VName),
  hold(VRange),
  hold(VSubmesh),
  hold(Vertical),
  hold(VerticalAlignment),
  hold(VerticalAsymptotesColor),
  hold(VerticalAsymptotesStyle),
  hold(VerticalAsymptotesVisible),
  hold(VerticalAsymptotesWidth),
  hold(VerticalLines),
  hold(ViewDirectional),
  hold(ViewFocalPointX),
  hold(ViewFocalPointY),
  hold(ViewFocalPointZ),
  hold(ViewKeepUpVector),
  hold(ViewOrthogonalProjection),
  hold(ViewPositionX),
  hold(ViewPositionY),
  hold(ViewPositionZ),
  hold(ViewTranslationX),
  hold(ViewTranslationY),
  hold(ViewTranslationZ),
  hold(ViewUpVectorX), 
  hold(ViewUpVectorY), 
  hold(ViewUpVectorZ), 
  hold(ViewViewingAngle),
  hold(ViewXMax),
  hold(ViewXMin),
  hold(ViewYMax),
  hold(ViewYMin),
  hold(ViewingAngle),
  hold(ViewingBoxXMax),
  hold(ViewingBoxXMin),
  hold(ViewingBoxXRange),
  hold(ViewingBoxYMax),
  hold(ViewingBoxYMin),
  hold(ViewingBoxYRange),
  hold(ViewingBoxZMax),
  hold(ViewingBoxZMin),
  hold(ViewingBoxZRange),
  hold(Visible),
  hold(VisibleAfter),
  hold(VisibleAfterEnd),
  hold(VisibleBefore),
  hold(VisibleBeforeBegin),
  hold(VisibleFromTo),
  hold(Width),
  hold(Winding),
  hold(XAxisTitle),
  hold(XAxisTitleAlignment),
  hold(XAxisVisible),
  hold(XContours),
  hold(XCrossedLines),
  hold(XCrosses),
  hold(XFunction),
  hold(XFunction1),
  hold(XFunction2),
  hold(XGridVisible),
  hold(XLinesVisible),
  hold(XMax),
  hold(XMesh),
  hold(XMin),
  hold(XName),
  hold(XRange),
  hold(XSubgridVisible),
  hold(XSubmesh),
  hold(XTicksAnchor),
  hold(XTicksAt),
  hold(XTicksAutoLabels),
  hold(XTicksBetween),
  hold(XTicksDistance),
  hold(XTicksLabelStyle),
  hold(XTicksLabelsVisible),
  hold(XTicksNumber),
  hold(XTicksVisible),
  hold(YAxisTitle),
  hold(YAxisTitleAlignment),
  hold(YAxisTitleOrientation),
  hold(YAxisVisible),
  hold(YContours),
  hold(YFunction),
  hold(YFunction1),
  hold(YFunction2),
  hold(YGridVisible),
  hold(YLinesVisible),
  hold(YMax),
  hold(YMesh),
  hold(YMin),
  hold(YName),
  hold(YRange),
  hold(YSubgridVisible),
  hold(YSubmesh),
  hold(YTicksAnchor),
  hold(YTicksAt),
  hold(YTicksAutoLabels),
  hold(YTicksBetween),
  hold(YTicksDistance),
  hold(YTicksLabelStyle),
  hold(YTicksLabelsVisible),
  hold(YTicksNumber),
  hold(YTicksVisible),
  hold(YXRatio),
  hold(ZAxisTitle),
  hold(ZAxisTitleAlignment),
  hold(ZAxisVisible),
  hold(ZContours),
  hold(ZFunction),
  hold(ZFunction1),
  hold(ZFunction2),
  hold(ZGridVisible),
  hold(ZMax),
  hold(ZMesh),
  hold(ZMin),
  hold(ZName),
  hold(ZRange),
  hold(ZSubgridVisible),
  hold(ZTicksAnchor),
  hold(ZTicksAt),
  hold(ZTicksBetween),
  hold(ZTicksDistance),
  hold(ZTicksLabelStyle),
  hold(ZTicksLabelsVisible),
  hold(ZTicksNumber),
  hold(ZTicksVisible),
  hold(ZXRatio),
  hold(Empty), hold(shell),
  hold(Raw), hold(AutoViewingBox),
  hold(Forward), hold(Down), hold(Push), hold(Pop), hold(Move),


                    
  //////////////////////////////////////////////////////////////////////////
  // options used in the combinat package
  // the following section is maintained by the combinat developers

  hold(MinLength),   // intListLexTools & inherited
  hold(MaxLength),   //  ...
  hold(Length),      //  ...
  hold(MinPart),     //  ...
  hold(MaxPart),     //  ...
  hold(Inner),       //  ...
  hold(Outer),       //  ...
  hold(MinSlope),    //  ...
  hold(MaxSlope),    //  ...
  hold(Repeat),      // subwords
  hold(FinalDescent),// permutations & compositions
  // binaryTrees & labelledBinaryTrees
  hold(Complete), hold(Binary), hold(CompleteBinary), 
  hold(BinaryTree), hold(CompleteBinaryTree),
  hold(Ordered),
  hold(Label), hold(Leaf), hold(LeafNil), hold(Node),
  hold(Left), hold(Right),
  hold(Prefix), hold(RightPrefix), hold(LeftPrefix),
  hold(Infix), hold(RightInfix), hold(LeftInfix),
  hold(Postfix), hold(RightPostfix), hold(LeftPostfix),
  // output::asciiArt
  hold(Top), hold(Center), hold(Bottom),
  hold(MinHeight), hold(MinWidth),
  hold(Compact), hold(Packed),
  hold(ColumnSeparator), hold(RowSeparator),
  hold(Cross), hold(HorizontalPadding),
  hold(VerticalAlignment), hold(Align), hold(MaxRemove),

  // combinat::decomposableObjects
  hold(NonTerminal),
  hold(Epsilon), hold(Atom),
  hold(Union), hold(Powerset), hold(Multiset), hold(Sequence),
  hold(Cycle), hold(Lyndon),
  hold(Predefined), hold(Domain),
  hold(Prod), hold(Int), hold(Theta), hold(Delta), hold(Alias),
  hold(Labelled), hold(Products), hold(File), hold(Float),
  hold(MainNonTerminal), hold(MainTerminal), hold(Target),
  hold(Lazy), hold(Holonomic), hold(Naive), hold(Auto),
  hold(Boustrophedonic),
  hold(Mark), hold(Unmark),
  hold(DivX), hold(MultX), // Temporary
  hold(Count), hold(Random), hold(Unrank),
  hold(Generator), hold(GeneratorFromRank),

  // Graph.pm from MuPAD 3.0.0
  hold(SearchWith), hold(Bellman),
  hold(StartVertex), hold(EndVertex),
  hold(EdgeWeights), hold(EdgeDescriptions), hold(EdgeCosts),
  hold(VertexWeights), hold(Backward), hold(Forward),
  // operators::overloaded
  hold(Explicit), hold(Facade), hold(Cost),
  // operators::make*
  hold(Source), hold(Source1), hold(Source2), hold(ImageSet),
  hold(Zero), hold(Opposite),

  // combinat::graphs,
  hold(C3Free), hold(C4Free), hold(Bipartite), hold(Biconnected),
  hold(Quiet), hold(Straighten), hold(MinDegree), hold(MaxDegree),
  hold(MinEdges), hold(MaxEdges), hold(GengOptions)

  // end of combinat section
  //////////////////////////////////////////////////////////
}:

stdlib::SYSTEM_TABLES := {
  hold(stdlib::Alias)
}:

// alias/unalias -- in .mupadsysinit definiert

alias(path = pathname("STDLIB")):

D:= loadproc(D, path, "D"):
Im:= loadproc(Im, pathname("STDLIB","IM"), "Im"):
O:= loadproc(O, path, "O"):
pade := loadproc(pade, path, "pade"):
Re:= loadproc(Re, pathname("STDLIB","RE"), "Re"):
taylor:= loadproc(taylor, path, "taylor"):
_fnest:= loadproc(_fnest, path, "fnest"):
_implies:= loadproc(_implies, path, "implies"):
_equiv:= loadproc(_equiv, path, "equiv"):
_in:= loadproc(_in, path, "in"):
_subset := loadproc(_subset, path, "subset"):
_notsubset := loadproc(_notsubset, path, "subset"):
card:= loadproc(card, path, "card"):
_print_userinfo:= loadproc(_print_userinfo, path, "userinfo"):
_xor:= loadproc(_xor, path, "xor"):
anames:= loadproc(anames, path, "anames"):
assert:= loadproc(assert, path, "assert"):
assign:= loadproc(assign, path, "assign"):
asympt:= loadproc(asympt, path, "asympt"):
autoload := loadproc(autoload, path, "autoload"):
stdlib::standardFilename := loadproc(stdlib::standardFilename, path, "autoload"):
collect:= loadproc(collect, path, "collect"):
combine:= loadproc(combine, path, "combine"):
complexInfinity:= loadproc(complexInfinity, path, "cinfty"):
stdlib::CInfinity:= loadproc(stdlib::CInfinity, path, "cinfty"):
conjugate:= loadproc(conjugate, pathname("STDLIB","CONJ"), "conj"):
content:= loadproc(content, path, "content"):
contfrac:= loadproc(contfrac, path, "contfrac"):
denom:= loadproc(denom, path, "denom"):
discont:= loadproc(discont, path, "discont"):
doprint:= loadproc(doprint, path, "doprint"):
evalassign:= loadproc(evalassign, path, "evalass"):
evalAt:= loadproc(evalAt, path, "evalAt"):
export:= loadproc(export, path, "export"):
expose:= loadproc(expose, path, "expose"):
factor:= loadproc(factor, path, "factor"):
freeze:= loadproc(freeze, path, "freeze"):
stdlib::frozen:= loadproc(stdlib::frozen, path, "freeze"):
fun:= loadproc(fun, path, "fun"):
func:= loadproc(func, path, "func"):
gcd:= loadproc(gcd, path, "gcd"):
gcdex:= loadproc(gcdex, path, "gcdex"):
ground:= loadproc(ground, path, "ground"):
stdlib::hasaccess:= loadproc(stdlib::hasaccess, path, "hasaccess"):
hastype:= loadproc(hastype, path, "hastype"):
stdlib::hasmsign:= loadproc(stdlib::hasmsign, path, "hasmsign"):
infinity:= loadproc(infinity, path, "infinity"):
stdlib::Infinity:= loadproc(stdlib::Infinity, path, "infinity"):
info:= loadproc(info, path, "info"):
int:= loadproc(int, path, "int"):
interpolate:= loadproc(interpolate, path, "interpolate"):
iquo:= loadproc(iquo, path, "iquo"):
irem:= loadproc(irem, path, "irem"):
irreducible:= loadproc(irreducible, path, "irreduc"):
ithprime:=loadproc(ithprime, path, "ithprime"):
lcm:= loadproc(lcm, path, "lcm"):
lhs := loadproc(lhs,path,"lhsrhs"):
linsolve:= loadproc(linsolve, path, "linsolve"):
maprat:= loadproc(maprat, path, "maprat"):
match:= loadproc(match, pathname("MATCHLIB"), "match"):
max:= loadproc(max, path, "minmax"):
min:= loadproc(min, path, "minmax"):
normal:= loadproc(normal, path, "normal"):
stdlib::normalizesign:= loadproc(stdlib::normalizesign, path, "normalizesign"):
stdlib::normal:= loadproc(stdlib::normal, path, "normal"):
numer:= loadproc(numer, path, "numer"):
operator:= loadproc(operator, path, "operator"):
package := loadproc(package, path, "package"):
partfrac:= loadproc(partfrac, path, "partfrac"):
plotfunc2d:= loadproc(plotfunc2d, path, "plotfunc2d"):
plotfunc3d:= loadproc(plotfunc3d, path, "plotfunc3d"):
powermod:= loadproc(powermod, path, "powermod"):
product:= loadproc(product, path, "product"):
radsimp:= loadproc(radsimp, path, "radsimp"):
stdlib::radnormal:= loadproc(stdlib::radnormal, path, "radsimp"):
random:= loadproc(random, path, "random"):
rationalize := loadproc(rationalize, path, "maprat"):
readbytes := loadproc(readbytes, path, "readbytes"):
rectform:= loadproc(rectform, pathname("STDLIB","RECTFORM"), "rectform"):
revert := loadproc(revert, path, "revert"):
rewrite:= loadproc(rewrite, path, "rewrite"):
rhs := loadproc(rhs,path,"lhsrhs"):
series:= loadproc(series, path, "series"):
setuserinfo:= loadproc(setuserinfo, path, "setuinfo"):
simplify:= loadproc(simplify, pathname("STDLIB","SIMPLIFY"), "simplify"):
sum:= loadproc(sum, path, "sum"):
Sum:= loadproc(Sum, path, "sum"):
sysevalassign:= loadproc(sysevalassign, path, "sysevass"):
undefined:= loadproc(undefined, path, "undef"):
unexport:= loadproc(unexport, path, "unexport"):
unfreeze:= loadproc(unfreeze, path, "freeze"):
universe:= loadproc(universe, path, "universe"):
version:= loadproc(version, path, "version"):
expr:= loadproc(expr, path, "expr"):
ifactor:= loadproc(ifactor, path, "ifactor"):
coerce:= loadproc(coerce, path, "coerce"):
densematrix:= loadproc(densematrix, path, "densematrix"):
sparsematrix:= loadproc(sparsematrix, path, "sparsematrix"):
matrix:= loadproc(matrix, path, "matrix"):
Factored:= loadproc(Factored, path, "Factored"):
frandom:= loadproc(frandom, path, "frandom"):

testeq := loadproc(testeq, path, "testeq"):

/* obsolete plot stuff */
plot2d := () -> error("plot2d no longer available"):
plot3d := () -> error("plot3d no longer available"):
point :=
() -> error("point no longer available; use plot::Point2d, plot::Point3d instead"):
polygon :=
() -> error("polygon no longer available; use plot::Polygon2d, plot::Polygon3d instead"):

/* -------- RootOf -------- */
RootOf := loadproc(RootOf, path, "RootOf"):
// the following methods are used in 'sum':
stdlib::RootOf_polysum:= loadproc(stdlib::RootOf_polysum, path, "RootOf" ):
stdlib::RootOf_evala:= loadproc(stdlib::RootOf_evala, path, "RootOf" ):
stdlib::RootOf_expand_power:= loadproc(stdlib::RootOf_expand_power, path, "RootOf" ):

/* -------- block/unblock --*/
block := loadproc(block, path, "block"):
blockIdents:= loadproc(blockIdents, path, "block"):
blockTransparent:= loadproc(blockTransparent, path, "block"):
unblock:= loadproc(unblock, path, "block"):


/* ------- limit --------- */
limit:= loadproc(limit, path, "limit"):
stdlib::limit:= loadproc(stdlib::limit, path, "limit"):

/* -------  solve --------- */
solve                  := loadproc(solve,                  path,"solve"):

/* ------- piecewise ------- */
piecewise:= loadproc(piecewise, path, "piecewis"):
stdlib::branch:= loadproc(stdlib::branch, path, "piecewis"):

/* ------- DOM_SET --------- */
DOM_SET::func_call:= DOM_ARRAY::func_call:
DOM_SET::new:= proc() name new; begin {args()} end_proc:
DOM_SET::new_extelement:= DOM_ARRAY::new_extelement:
DOM_SET::sort:= loadproc(DOM_SET::sort, path, "setops"):
DOM_SET::_index:= loadproc(DOM_SET::_index, path, "setops"):
DOM_SET::_plus:= loadproc(DOM_SET::_plus, path, "setops"):
DOM_SET::_mult:= loadproc(DOM_SET::_mult, path, "setops"):
DOM_SET::_power:= loadproc(DOM_SET::_power, path, "setops"):
DOM_SET::bin_power:= loadproc(DOM_SET::bin_power, path, "setops"):
DOM_SET::inhomogleft_power:= loadproc(DOM_SET::inhomogleft_power, path, "setops"):
DOM_SET::inhomogright_power:= loadproc(DOM_SET::inhomogright_power, path, "setops"):
DOM_SET::coerce:= loadproc(DOM_SET::coerce, path, "setops"):

/*-------- combine-attributes for _plus, _mult, _power -----------*/

_plus:= slot(_plus,"combine",
        loadproc(slot(_plus,"combine"),pathname("STDLIB","COMBINE"),
        "_plus")):


_mult:= slot(_mult,"combine",
        loadproc(slot(_mult,"combine"),pathname("STDLIB","COMBINE"),
        "_mult")):

_power:= slot(_power, "combine",
              loadproc(slot(_power, "combine"), pathname("STDLIB","COMBINE"),
        "_power")):

// the bitmap module, for import::readbitmap
stdlib::bitmap := loadproc(stdlib::bitmap, path, "bitmap"):

/*-------- export some of the functions of property ----- */
unalias(path):
alias(path = pathname("PROP")):

is              := loadproc(is,         path, "Propis" ):
getprop         := loadproc(getprop,    path, "Propget"):
unassume        := loadproc(unassume,   path, "Propdel"):
assume          := loadproc(assume,     path, "Propset"):
_assuming       := loadproc(_assuming,  path, "assuming"):

/*------ CONTINUE -----------*/
alias(ProtOldOptions = [hold(None), hold(Warning), hold(Error)]):
alias(ProtNewOptions = [hold(ProtectLevelNone), hold(ProtectLevelWarning),
                           hold(ProtectLevelError)]):
alias(old2new(OPT) = _if(contains(ProtOldOptions, OPT) > 0,
                         op(ProtNewOptions, contains(ProtOldOptions, OPT)),
                         OPT)):
alias(new2old(OPT) = _if(contains(ProtNewOptions, OPT) > 0,
                         op(ProtOldOptions, contains(ProtNewOptions, OPT)),
                         OPT)):
protect:=
  proc()
    name protect;
    option hold;
    local LEVEL;
  begin
    sysassign(stdlib::PROTECTED, stdlib::PROTECTED union {args(1)});
    LEVEL:= (if args(0) = 2 then
               protected(args(1), new2old(context(args(2))))
             else
               protected(args(1), hold(Warning))
             end_if);
    old2new(LEVEL)
  end_proc:

unprotect:=
  proc()
    name unprotect;
    option hold;
    local LEVEL;
  begin
    sysassign(stdlib::PROTECTED, stdlib::PROTECTED minus {args(1)});
    LEVEL:= protected(args(1), hold(None));
    old2new(LEVEL)
  end_proc:

unalias(ProtOldOptions):
unalias(ProtNewOptions):
unalias(old2new):
unalias(new2old):

// Domain fuer spez. Ausgabe
stdlib::Exposed:= newDomain("stdlib::Exposed"):
stdlib::Exposed::create_dom:=hold(stdlib::Exposed):
stdlib::Exposed::op:= proc(x, rg = null()) begin op(extop(x, 2..extnops(x)), rg) end_proc:
stdlib::Exposed::subsop:= proc(x) begin subsop(extop(x, 2..extnops(x)), args(2..args(0))) end_proc:
stdlib::Exposed::nops:= (x)-> extnops(x) - 1:
stdlib::Exposed::print:= proc(x) begin extop(x, 1)(extop(x, 2..extnops(x))) end_proc:
// erstes
stdlib::Exposed::new:=
  proc(x)
  begin
    if args(0) = 1 or not(contains({DOM_PROC, DOM_FUNC_ENV}, type(x))) then
      new(stdlib::Exposed, extop, args())
    else
      new(stdlib::Exposed, args())
    end_if
  end_proc:
stdlib::Exposed::Content := () -> error("not yet implemented"):

// function environment uses for Pretty outputs -- used in Dom::ImageSet
_outputSequence :=
funcenv(_outputSequence,
        builtin(1100, 25, "", "_outputSequence")):
_outputSequenceINFINITE :=
funcenv(hold(_outputSequenceINFINITE),
        builtin(1100, 1000000, "", "_outputSequenceINFINITE")):
_outputSequencePlus :=
funcenv(_outputSequencePlus,
        builtin(1100, 1000, "", "_outputSequencePlus")):
_outputSequenceMult :=
funcenv(_outputSequenceMult,
        builtin(1100, 1100, "", "_outputSequenceMult")):
_outputSequencePower :=
funcenv(_outputSequencePower,
        builtin(1100, 1200, "", "_outputSequencePower")):

// stdlib::interactive(): Has the calling function been called interactively?
stdlib::interactive := () -> bool(context(hold(context(hold(_act_proc_env())))) = null()):

// Aligns strings by adding spaces in front of short strings and
// concatenating them with newline inbetween; used for int, sum, product outputs
stdlib::align :=
proc()
  local len, strings, i ,j;
begin
  len := max(op(map([args()], length)));
  strings := [args()];
  for i from 1 to nops(strings) do
    strings[i] := _concat(" " $ j = 1..(len-length(strings[i])+1) div 2).strings[i];
    if i < nops(strings) then
      strings[i] := strings[i]."\n"
    end_if;
  end_for;
  _concat(op(strings))
end_proc:

_pref(hold(Alias)=[TRUE,
                   proc()
                     local config, trap;
                   begin
                     config := _parser_config();
                     if config = null() then
                       return(table())
                     else
                       config := op(config,2);
                       trap := traperror((config := map([op(config)],
                                     x -> (x, eval(op(x, 1))=op(x, 2)))));
                       if trap = 0 then
                         return(table(op(config)));
                       else
                       // if this line is reached, we had an error above
                         return(table());
                       end_if;
                     end_if;
                   end_proc]):

// initialisiere Completion
_pref(hold(Complete) =
proc(x)
    name Complete;
    option noDebug;
    local globals, libs, funcEnvs, interface, all, makeNames, i,
          num, domName, DOMAIN, KEYWORDS, verboseRead, try;
begin
  // disable verboseRead during Completion
  verboseRead := Pref::verboseRead(0);
  // KEYWORDS keywords - don't evaluate
  KEYWORDS := {"and", "begin", "break", "case", "delete", "div", "do", "domain",
                 "downto", "elif", "else", "end", "end_case", "end_domain", "end_for" ,
                 "end_if",
                 "end_proc", "end_repeat", "end_while", "for", "from", "if", "in",
                 "intersect", "local", "minus", "mod", "name", "next", "not", "of",
                 "option", "or", "otherwise", "proc", "quit",
                 "repeat", "save", "step", "then", "to", "union", "until", "while", "frame"};
  makeNames := proc(d, s)
    name makeNames;
    local i;
  begin
     d."::".op(s,i) $ i = 1..nops(s);
  end_proc:

  // if procedure gets the whole input line:
  // extract left word // !! end of loop: stringlib::FirstIndex !!
  try := x;
  for i from length(try) downto 1 do
    x := substring(try, i..length(try));
    if contains(stringlib::punctuation, substring(try, i)) <> 0
       and contains(["_", ":"], substring(try, i)) = 0 then
      break
    end_if
  end_for;
  
  if contains({"", ":", "::"}, x) or
    x[1]=":" and length(x) > 1 and x[2] <> ":" then
    return("")
  end_if;

  // select only matching globals to determine DOM_DOMAIN and DOM_FUNC_ENV
  globals := map(select(stdlib::anames(3), X -> strmatch(expr2text(X), x."\*")),
                 X -> context(hold(val)(X)));
  [libs,funcEnvs,globals] :=
      split(globals, X -> op([UNKNOWN, TRUE, FALSE],
                             contains([DOM_DOMAIN, DOM_FUNC_ENV],
                                      type(X)) + 1));
  // All
  all:= select(Pref::allComplete(), strmatch, x."\*");

  // if given a ':' or '::'
  num := op(strmatch(x, "\*:", Index), 2):

  if type(num) = DOM_INT then
    // possibly domain
    if x[num - 1] = ":" then
      // two ':'
      if num > 2 and x[num - 2] = ":" then
        // more than two ':'
        return("")
      else
        num:= num - 1
      end_if
    end_if;
    domName := substring(x, 1, num-1);

    if contains(KEYWORDS, domName) then
      return("")
    else
      if traperror((DOMAIN := eval(text2expr(domName)))) <> 0 then
        return("");
      end_if;
      interface := {};
      if domtype(DOMAIN) = DOM_DOMAIN or
        // completion for domain elements
        domtype(DOMAIN::dom::complete(DOMAIN)) = DOM_SET then
        if domtype(DOMAIN) = DOM_DOMAIN then
          if domtype(DOMAIN::interface) = DOM_SET then
            interface := map(DOMAIN::interface, expr2text);
          elif domtype(DOMAIN::allEntries()) = DOM_SET then
            interface := DOMAIN::allEntries();
          end_if
        else
          // completion for domain elements
          interface := map(DOMAIN::dom::complete(DOMAIN), expr2text);
        end_if;

        interface := {makeNames(domName, interface)};
      elif domtype(DOMAIN) = DOM_FUNC_ENV then
        // DOMAIN is a function environment
        if domtype(op(DOMAIN,3)) = DOM_TABLE then
          // there are slots defined
          if domtype(DOMAIN::interface) = DOM_SET then
            interface := {makeNames(domName, DOMAIN::interface)};
          else
            interface := {};
          end_if;
        else
          interface := {};
        end_if;
      else
        // DOMAIN is no domain
        interface := {};
      end_if
    end_if
  else

    // uniq completion; try loading loadproc
    if nops(all) = 1 and not contains(KEYWORDS, op(all,1)) then
        text2expr(op(all, 1));
        if traperror((DOMAIN := eval(%))) <> 0 then
          return("");
        end_if;
        if domtype(DOMAIN) = DOM_DOMAIN then
          // do not use expr2text; now densematrix shows ::
          if DOMAIN::interface = {} then
            libs := {};
          else
            libs := all;
          end_if;
        else
          libs := {}
        end_if;
    else
      // normal name completion, no ':' given
      libs:= select(libs, x->x::interface <> {});
      libs:= map(libs, expr2text);
    end_if;

    if nops(libs) = 1 and all = libs then
      // additionally the domain without '::'
      all:= all union map(libs minus Pref::noComplete(), _concat, "::")
    end_if;
    interface := {};
  end_if;

  all := op(sort(select([op((interface union all) minus
                            Pref::noComplete())], strmatch, x."\*"),
                 (s1, s2) -> stringlib::lower(s1) < stringlib::lower(s2)));
  // restore Pref::verboseRead
  Pref::verboseRead(verboseRead);
  if domtype(all) = DOM_STRING /* and x <> all */ then
      // try to complete domains in domains
    try :=  _pref(hold(Complete))(all."::");
    if try <> null() and try <> "" then
      return(all, all."::")
    end_if
  end_if;
  all;
end_proc):

// Old library implementation, in case someone wants to check for differences:

/// _seqin := proc() local i, _0_expr; option hold;
/// begin
///    if not contains({DOM_IDENT, DOM_VAR}, type(args(2))) then
///       error("illegal arguments") end_if;
///    _0_expr:= context(args(3));
///    // in context we have to prevent that _0_expr is evaluated a second
///    // time, therefore the hold(level)(..., 0)
///    context(subs(args(1),args(2)=hold(level)(op(_0_expr,i),0),Unsimplified)) $
///                            i=1..nops(_0_expr)
/// end_proc:
/// _seqin := funcenv(_seqin, builtin(1125, 300, " $ ", "_seqin"),
///                   table("type"="_seqin")):


// this set contains names ot temporary files which will be deleted
// by Pref::callOnExit, when the system quits
stdlib::TempFiles := {}:


// special environments used for integration
// _pattern -- defining patterns used in intlib's pattern matcher
_pattern:=funcenv(_pattern):
_pattern::type := "_pattern": // for pattern variable

// counter to detect recursion level of method int (used in INTLIB/int.mu
protected(intlibDepthLevel,Error):
protected(intlibLookupDepthLevel,Error):
// global flag used in definite integration to prevent some simplifications
protected(calledFromWithinDefInt,Error):

// global flag for ode solver, it needs to be global for using save which is
// necessary to be correct if errors/exceptions occurs
protected(calledFromWithinDSolve,Error):

null():
