
// author: Kai Gehrs
// linalg::SSS -- solve a system a linear equations with 
//                coeffcient matrix A und right-hand-side 
//                b 

linalg::SSS:= proc(A, b) 
        local m, n, Mat, T, Rone, charind, rank, det, kernelCols, col, 
              dets, detA, sig, R, RHS, Rzero, addRows, detMult, f0, f1, 
              i, ind, j, kernelColVectors, l;
      begin 
        if args(0) <> 2 then 
          error("expecting two arguments") 
        end_if;
        Mat:= A::dom;
        if Mat::hasProp(Cat::Matrix) <> TRUE then
          error("expecting a matrix of 'Cat::Matrix'");
        end_if;
        if Mat <> b::dom then 
          error("second argument must be of the same type as the first argument");
        end_if;
        if A::dom::matdim(A)[1] <> b::dom::matdim(b)[1] then 
          error("illegal operands");
        end_if;
        R:= A::dom::coeffRing;
        Rone:= R::one;
        Rzero:= R::zero;
        if Mat::constructor <> Dom::Matrix then 
          //==================================================
          // Convert the matrix into a matrix over Dom::Matrix
          //==================================================
          A:= Dom::Matrix(R)(A);
          b:= Dom::Matrix(R)(b);
        end_if;
        T:= linalg::ffG(A::dom::concatMatrix(A,b));
        rank:= T[2];
        charind:= T[4];
        sig:= T[5];
        T:= T[1];
        m:= T::dom::matdim(T)[1]:
        n:= T::dom::matdim(T)[2]:
        kernelCols:= {};
        //==============================================
        // Does there exist any solution at all? 
        // Check whether rank(A,B) = rank(A). 
        //==============================================
        if rank > 0 then
          for j from rank to n-1 do
            if not iszero( T[rank,j] ) then 
              break;
            end_if
          end_for;
          if j > n-1 then 
            return([]);
          end_if
        end_if;
        //==============================================
        // Positioning of the characteristic columns 
        // "along the main diagonal". Insert -1 on the 
        // main diagonal in zero rows. 
        //==============================================
        if m < n-1 then 
          addRows:= Dom::Matrix(R)(n-1-m, n);
          T:= T::dom::stackMatrix(T,addRows);
        end_if;
        for i from 1 to n-1 do
          if iszero(T[i,i]) then 
            for l from i+1 to n-1 do 
              if not iszero(T[i,l]) then 
                T:= T::dom::swapRow(T, i, l);
              end_if;
            end_for;
            T[i,i]:= Rone;
            kernelCols:= kernelCols union {i};
          end_if;
        end_for;
        //==============================================
        // Remove zero rows at the end of the matrix
        //==============================================
        for i from min(m,n-1) + 1 to m do  
          T:= T::dom::delRow(T, i);
        end_for;
/*
        if m < n-1 then 
          //==============================================
          // Insert zero-rows and -1 on the main diagonal 
          //==============================================
          addRows:= Dom::Matrix(R)(n-m-1, n);
          for i from 1 to n-m-1 do 
            addRows[i,m+i]:= Rone;
          end_for;
          T:= T::dom::stackMatrix(T,addRows);
          kernelCols:= kernelCols union {i $ i = m+1..n-1};
        end_if;
*/
        //=========================================
        // Compute the determinant of the matrix A
        //=========================================
        detA:= Rone; 
        for i from 1 to n-1 do
          detA:= detA * T[i,i];
        end_for;
        //================================================
        // Now we need a special solution of the system.
        // This solution is computed via Cramer's rule.
        //================================================
        b:= T::dom::col(T,n);
        T:= T::dom::delCol(T,n);
        n:= n-1;
        dets:= [Rzero $ n];
        det:= Rone;
        detMult:= Rone;
        //================================================
        // Note that in the case b = 0 we do not really 
        // have to compute all determinants. We simply
        // define them to be zero. 
        //================================================
        if not iszero(b) then 
          A:= T::dom::setCol(T, n, b);
          for i from 1 to n do 
            det:= det * A[i,i];          
          end_for;
          dets[n]:= det; 
          //================================================
          // else dets[n]:= Rzero; which is actually the 
          // the case already. 
          //================================================
        end_if;
        detMult:= Rone;
        if not iszero(b) then 
          for col from n-1 downto 1 do 
            A:= T::dom::setCol(T, col, b);
            det:= Rone;
            //============================
            // We are in the i-th row and 
            // the col-th column. 
            //============================
            if not iszero(A[col + 1,col]) then 
              f0:= A[col+1,col+1];
              f1:= -A[col+1,col];
              detMult:= detMult/f0;
              //==================================================
              // addCol(A,col,col+1,A[col+1,col+1],-A[col+1,col]);
              //==================================================
              for i from col downto 1 do
                A[i,col]:= f0 * A[i,col] + f1 * A[i,col+1];
              end_for;
              A[col+1,col]:= Rzero;
//            else 
//              detMult:= detMult * A[col+1,col+1];
            end_if;
            //==============================================
            // Now A should is in upper triangular form. 
            // We compute the determinant as the product 
            // of the main diagonal entries.
            //==============================================
            det:= _mult(A[i,i] $ i = 1..n) * detMult;
            //det:= _mult(A[i,i] $ i = 1..col) * det;
            b:= A::dom::col(A, col);
            dets[col]:= det;
          end_for;
          //================================================
          // else dets[col]:= Rzero; which is actually the 
          // the case already. 
          //================================================
        end_if;
        //==============================================
        // Compute a special solution of the system of 
        // linear equations. 
        //==============================================
        dets:= map(dets, ind -> ind/detA);
        b:= Dom::Matrix(R)(dets);
        //==============================================
        // Compute the kernel of the matrix A: 
        // kernelCols contains the indices of the  
        // columns, where we inserted 'Rone' on the 
        // main diagonal. 
        //================================================
        kernelColVectors:= [];
        for ind in kernelCols do 
          RHS:= Dom::Matrix(R)(n,1);
          RHS[ind,1]:= Rone; 
          det:= Rone;
          if ind = n then 
            A:= T::dom::setCol(T, n, RHS);
            det:= _mult(A[i,i] $ i = 1..n);
            dets[n]:= det; 
          else
            (dets[i]:= Rzero) $ i = ind + 1..n; 
          end_if;
          detMult:= Rone;
          for col from min(ind, n-1) downto 1 do 
            A:= T::dom::setCol(T, col, RHS);
            det:= Rone;
            //=========================================
            // We are in the i-th row and 
            // the col-th column. 
            //=========================================
            if not iszero(A[col + 1,col]) then 
              f0:= A[col+1,col+1];
              f1:= -A[col+1,col];
              detMult:= detMult/f0;
              //==================================================
              // addCol(A,col,col+1,A[col+1,col+1],-A[col+1,col]);
              //==================================================
              for i from col downto 1 do
                A[i,col]:= f0 * A[i,col] + f1 * A[i,col+1];
              end_for;
              A[col+1,col]:= Rzero;
//            else 
//              detMult:= detMult * A[col+1,col+1];
            end_if;
            //==============================================
            // Now A is in upper triangular form. 
            // We compute the determinant as the product 
            // of the main diagonal entries.
            //==============================================
            det:= _mult(A[i,i] $ i = 1..n) * detMult;
            //det:= _mult(A[i,i] $ i = 1..col) * detMult;
            RHS:= A::dom::col(A, col);
            dets[col]:= det;
          end_for;
          dets:= map(dets, ind -> ind/detA);
          kernelColVectors:= append(kernelColVectors, Mat(dets));
        end_for;
        return([b, kernelColVectors, kernelCols]);
     end_proc:
