// $Date: 2003/09/02 12:54:50 $  $Author: acrowley $   $Revision: 1.21 $
// frankp, 21/02/94

/*
        linalg::det( A )

        A: square matrix

        Computes the determinant of a matrix over a commutative ring.
*/

linalg::det := proc(A)
    local R, Mat, Rnormal, Rmult, Rnegate, heuristic, j;
begin
    if args(0) <> 1 then
        error("expecting one argument, a matrix of 'Cat::Matrix'")
    end_if;

    Mat:= A::dom;
    if testargs() then
        if Mat::hasProp( Cat::Matrix ) <> TRUE then
            error("expecting a matrix of 'Cat::Matrix'")
        elif not (Mat::coeffRing)::hasProp( Cat::CommutativeRing ) then
            error("expecting matrix over 'Cat::CommutativeRing'")
        end_if;
        R:= Mat::matdim(A);
        if R[1] <> R[2] then
            error("not a square matrix")
        end_if
    end_if;

    R:= Mat::coeffRing;
    case op(Mat::matdim(A),1)
    of 1 do 
        return( A[1,1] );
    of 2 do
        userinfo(1,"use formula for the determinant of 2x2 matrices");

        Rmult:= R::_mult;
        if (Rnormal:= R::normal) = FAIL then
            if R::hasProp(Ax::systemRep) then Rnormal:= normal else Rnormal:= id end_if
        end_if;
 
        return( Rnormal(R::_plus( 
           Rmult( A[1,1],A[2,2] ), R::_negate( Rmult( A[1,2],A[2,1] ) )
        )) )
    of 3 do
        userinfo(1,"use formula for the determinant of 3x3 matrices");

        Rmult:= R::_mult;
        if (Rnormal:= R::normal) = FAIL then
            if R::hasProp(Ax::systemRep) then Rnormal:= normal else Rnormal:= id end_if
        end_if;
 
        Rmult:= R::_mult;
        Rnegate:= R::_negate;
        return( Rnormal(R::_plus( 
            Rmult( A[1,1],A[2,2],A[3,3] ),
            Rnegate( Rmult( A[1,1],A[2,3],A[3,2] ) ),
            Rnegate( Rmult( A[2,1],A[1,2],A[3,3] ) ),
            Rmult( A[2,1],A[1,3],A[3,2] ),
            Rmult( A[3,1],A[1,2],A[2,3] ),
            Rnegate( Rmult( A[3,1],A[1,3],A[2,2] ) )
        )) )
    otherwise
        case R
        of Dom::Float do
            userinfo(1,"call 'numeric::det'");
            return(numeric::det(A))
        otherwise
            if R::hasProp(Cat::Field) and R::hasProp( Ax::systemRep ) then
              if nops(indets(A)) > 0 then 
                heuristic:= proc(A) 
                  local vars, nvars, pvars, nZeroes, comp, AA, maxLength, i;
                begin
                  vars:= indets(A);
                  nvars:= nops(vars); // number of variables in A
                  AA:= map(A, elem -> nops(indets(elem))); // AA gives a hint of 
                                                           // the allocation of 
                                                           // the variables in A
                  pvars:= AA::dom::nonZeroes(AA); // the number of components of 
                                                  // A, which do contain variables
                  nZeroes:= 0; // the number of non-zero matrix components below 
                               // the main diagonal 
                  for j from 1 to A::dom::matdim(A)[2] - 1 do
                    for i from j+1 to A::dom::matdim(A)[1] do
                      if not iszero(A[i,j]) then 
                        nZeroes:= nZeroes + 1;
                      end_if;
                    end_for;
                  end_for;

                  // Find out something about the complexity of
                  // of the matrix components. The variable 'comp'
                  // stores the list of the length of each matrix 
                  // compontent. 
                  AA:= map(A, elem -> length(elem));
                  comp:= [AA::dom::nonZeroOperands(AA)];
                  maxLength:= max(comp); // The maximal length of a
                                         // matrix component
  
                  if nZeroes in {0,1,2} then 
                    // A is nearly upper triangular. 
                    // Hence, use linalg::SSS since 
                    // there are only less elimination 
                    // steps to be done. The backsubstitution 
                    // is done by linalg::SSS due to 
                    // Cramer's rule avoiding expression 
                    // by using only n elimination for 
                    // the computation of each determinant.
                    return(1); // 1 indicates that linalg::SSS should 
                               // be used
                  end_if;

                  if pvars <= 13 and nvars <=13 then 
                    // Only less components of A contain
                    // variables. Since linalg::SSS uses 
                    // a pivot stratety which considers 
                    // the pivot size of elements, we 
                    // can be sure, that the algorithm
                    // will choose 'small' pivot elements
                    // within the elimination. Hence, it 
                    // should be faster than 'linalg::SSS2'.  
                    // Note: If R = Dom::ExpressionField()
                    // then R::pivotSive = length.
                    return(1); // 1 indicates that linalg::SSS should 
                               // be used
                  end_if;

                  if nvars >= 2 then 
                    //    if nZeroes < floor((A::dom::matdim(A)[1])/2) then 
                    if nZeroes < floor((A::dom::matdim(A)[1] + A::dom::matdim(A)[2])/2) then 
                      // i.e. A is nearly upper triangular 
                      return(1); // 1 indicates that linalg::SSS should 
                                 // be used
                    else 
                      return(2); // 2 indicates that linalg::SSS2 should 
                                 // be used
                    end_if;
                  else 
                    return(1); // 1 indicates that linalg::SSS should 
                               // be used
                  end_if;
                end_proc:

                if heuristic(A) = 1 then 
                  return(linalg::ffG(A)[3]);
                else 
                  return(linalg::symbolicDet(A));
                end_if;            

              end_if;
              // else: proceed the standard way and use numeric::det with 
              //       option 'Symbolic'. 
              userinfo(1,"call 'numeric::det' with option 'Symbolic'");
              return( numeric::det(A,hold(Symbolic) ) )
            elif R::hasProp( Cat::IntegralDomain ) then
                userinfo(1,"use Gaussian elimination");
                return( op(Mat::gaussElim(A),3) )
            else
                userinfo(1,"use Berkowitz algorithm");
                return( linalg::detBerkowitz(A) )
            end_if
        end_case
    end_case
end_proc:

linalg::det := funcenv(linalg::det):
linalg::det::Content := (Out, data) -> Out::Capply(Out::Cdeterminant, 
                                                   Out(op(data))):

/*--
K. F. Gehrs, 2000/09/26

    linalg::detBerkowitz -- returns the determinant of the matrix A

    linalg::detBerkowitz( A )

    A: square matrix over a commutative ring
 
    Computes the the determinant of the matrix A
    The coefficient domain of A has to be a commutative ring.

    The algorithm is based on the "Berkowitz-Algorithm" to compute the 
    characterisitic polynomial. This algorithm was modified and is now
    implemented to replace the method of Laplace for computing the 
    determinant of a matrix over an ordinary commutative ring. 
            
    Reference: The Berkowitz Algorithm, Maple and Computing the 
               Characteristic Polynomial in an Arbitrary Commutative Ring
               by Jounaidi Abdeljaoued, May 1996
--*/
linalg::detBerkowitz:=
proc(A)
  local Rzero, Rmult, Rplus, Rnegate, Rone, Vect, i, j, r, C, S, Q, R, n, k;
begin
    R:= A::dom::coeffRing;
    n:= A::dom::matdim(A)[1];

    Rone:= R::one;
    Rzero:= R::zero;
    Rmult:= R::_mult;
    Rplus:= R::_plus;
    Rnegate:= R::_negate;

    Vect:= (S:= (Q:= array( 1..n+1 )));
    Vect[1]:= Rnegate(Rone); 
    Vect[2]:= A[1,1];
    C[1]:= Vect[1];

    for r from 2 to n do
        S:= assignElements(S, [i] = A[i,r] $ i=1..r-1);
        C[2]:= A[r,r];
        for i from 1 to r-2 do
            C[i+2]:= Rzero;
            ( C[i+2]:= Rplus( C[i+2], Rmult( A[r,j], S[j])) ) $ j=1..r-1;
            for j from 1 to r-1 do
                Q[j]:= Rzero;
                ( Q[j]:= Rplus( Q[j], Rmult( A[j,k], S[k])) ) $ k=1..r-1
            end_for;
            S:= assignElements(S, [j] = Q[j] $ j=1..r-1);
        end_for;
        C[r+1]:= Rzero;
        ( C[r+1]:= Rplus( C[r+1], Rmult( A[r,j], S[j])) ) $ j=1..r-1;
        for i from 1 to r+1 do
            Q[i]:= Rzero;
            ( Q[i]:= Rplus( Q[i], Rmult( C[i+1-j], Vect[j])) ) $ j=1..min(r,i)
        end_for;
        Vect:= assignElements(Vect, [i] = Q[i] $ i=1..r+1);
    end_for;

    if Vect[1] = Rone then 
        return( Vect[n+1] )
    elif n mod 2 = 1 then // (-1)^(n+1) = 1
        return( Vect[n+1] )
    else 
        return(Rnegate( Vect[n+1] ))
    end_if
end_proc:

