// $Id: ACutils.mu,v 1.2 2004/07/12 11:49:05 ccr Exp $

// utilities for AC matching

// for speeding up int:
matchlib::_var_mayhave_X := TRUE:

matchlib::_var_mayhave_X_int :=
proc(ident)
  option remember;
  local s;
begin
  // all the vars allowed to have x either start with `#f...
  // or with the form `#?x`.
  s := expr2text(ident);
  return(bool(_lazy_and(length(s)> 3,
			s[3]="f" or s[4]="x")));
end_proc:
matchlib::_var_mayhave_X_int(`##123`) := TRUE:

matchlib::_remainders_to_vars :=
proc(terms, vars, fn)
  option escape;
  local terms_X, terms_noX,
	multiset_X, multiset_noX,
	list_X, list_noX,
	vars_X, vars_noX,
	select_generator_X, select_generator_noX,
	select_generator,
	nops_vars_X, nops_vars_noX,
	i, j, dummy;
begin
  [vars_X, vars_noX, dummy] := split(vars, matchlib::_var_mayhave_X);
  if vars_noX = [] then
    terms_X := terms;
    terms_noX := [];
    
    nops_vars_X := nops(vars_X);
    nops_vars_noX := 0;

    multiset_X := Dom::Multiset(op(terms));
    list_X := coerce(coerce(multiset_X, DOM_SET), DOM_LIST);
    list_noX := [];
    select_generator_X :=
    (matchlib::_orderedMultisetPartitionsGenerator
     ([Dom::Multiset::multiplicity(multiset_X, list_X[i])
       $ i=1..nops(list_X)],
      nops_vars_X));
    
    proc()
      local selection;
    begin
      selection := select_generator_X();
      if selection = FAIL then
	return(FAIL);
      end_if;
      
      eval([vars_X[i]
	    = fn((list_X[j]
		  $ selection[i][j])
		 $ j = 1..nops(selection[i]))
	     $ i = 1..nops_vars_X
	   ]);
    end_proc:
  else
    [terms_X, terms_noX, dummy] := split([op(terms)], has, hold(_X));

    if vars_X = [] and terms_X <> [] then
      return(FAIL);
    end_if;

    nops_vars_X := nops(vars_X);
    nops_vars_noX := nops(vars_noX);

    multiset_X := Dom::Multiset(op(terms_X));
    list_X := coerce(coerce(multiset_X, DOM_SET), DOM_LIST);
    multiset_noX := Dom::Multiset(op(terms_noX));
    list_noX := coerce(coerce(multiset_noX, DOM_SET), DOM_LIST);
    select_generator_X :=
    (matchlib::_orderedMultisetPartitionsGenerator
     ([Dom::Multiset::multiplicity(multiset_X, list_X[i])
       $ i=1..nops(list_X)],
      nops_vars_X));
    select_generator_noX :=
    (matchlib::_orderedMultisetPartitionsGenerator
     ([Dom::Multiset::multiplicity(multiset_noX, list_noX[i])
       $ i=1..nops(list_noX)],
      nops_vars_noX+nops_vars_X));

    select_generator :=
    combinat::generators::cartesian(select_generator_noX,
				    select_generator_X);
    
    proc()
      local selection, selection_X, selection_noX;
    begin
      selection := select_generator();
      if selection = FAIL then
	return(FAIL);
      end_if;
      
      [selection_noX, selection_X] := selection;
      
      eval([vars_noX[i]
	    = fn((list_noX[j]
		  $ selection_noX[i][j])
		 $ j = 1..nops(selection_noX[i]))
	     $ i = 1..nops_vars_noX,
	    vars_X[i]
	    = fn((list_noX[j]
		  $ selection_noX[i+nops_vars_noX][j])
		 $ j = 1..nops(selection_noX[i+nops_vars_noX]),
		 (list_X[j]
		  $ selection_X[i][j])
		 $ j = 1..nops(selection_X[i]))
	     $ i = 1..nops_vars_X
	   ]);
    end_proc:
  end_if;
end_proc:

matchlib::_construct_AC_solutions :=
proc(template, indets_ordered, plain_indets, this_sol, subject, fn, conds)
  local ret, fixed, vargen, vargen_val, not_yet_used, i, j, tmp;
begin
  fixed := [];
  for j from 1 to nops(indets_ordered) do
    tmp := select(this_sol, s->op(lhs(s),1)=j);
    if nops(tmp) = 1 then // same as nops(tmp)>0
      fixed := fixed . zip(indets_ordered[j],rhs(op(tmp)),`=`);
    end_if;
  end_for;

  // places of sub-subjects not appearing in this_sol
  not_yet_used := {$1..nops(subject)} minus {op(map(this_sol, op, [1,2]))};

  if not_yet_used = {} then
    ret := [fixed.
	    map(plain_indets, `=`, matchlib::AC[fn])];
  else
    // the actual subterms not used
    not_yet_used := map(not_yet_used, i->op(subject, i));
    vargen := matchlib::_remainders_to_vars(not_yet_used, plain_indets, fn);

    ret := [];
    while ((vargen_val := vargen())) <> FAIL do
      ret := ret . [fixed.vargen_val];
    end_while;
  end_if;
  // check conditions
  ret := select(ret,
                c -> _lazy_and(op(map(eval(subs(conds[i], c)), bool))
                                    $i=1..nops(conds)));
  
  return(map(ret, s->subs(template, s)));
end:

// Code by Nicholas Thiery:
matchlib::_orderedMultisetPartitionsGenerator :=
proc(m, k: Type::NonNegInt)
  option escape;
  local i,j;
begin
  // this special case is used very often:
  if nops(m)=1 and k=1 then
    return(combinat::generators::fromList([[m]]));
  end_if;
  if iszero(k) then
    FAIL;
  else
    combinat::generators::compose
    (l->[[l[j][i] $ j=1..nops(m)] $ i=1..k],
     combinat::generators::cartesian
   (combinat::integerListsLexTools::generator(m[i],k,k,0, RD_INF, RD_NINF, RD_INF) $ i=1..nops(m)));
//     (combinat::integerVectors::generator(m[i], k) $ i=1..nops(m)));
  end_if;
end_proc:
