
ode::specfunc :=
proc(eq,y,x)
  local unimod,v,nuinfty,dv,ddv,u,dvi,vi,wi,lQ,i,j,hi,Q,dQ,n,eqz,p,a,P,r,uu,lc,lc2,
        N,z,m,mu,nu,eps,valuation;

begin
  eq := ode::isLODE(eq, y(x), HlodeOverRF);
  if eq<>FALSE and eq[4]=2 then

    /*
The fonction valuation, let's call it nu_p(q), is such that :
--> nu_infty(0)=nu_p(0)=0 for p \in C[x]
--> nu_infty(q)=-degree(q) for q \in C[x]\{0}
--> nu_p(q)=max{n \in N | p^n|q} for p \in C[x] irreducible
--> nu_infty(a/b)=nu_infty(a)-nu_infty(b) and
    nu_p(a/b)=nu_p(a)-nu_p(b) for a and b polynomials
*/

valuation :=
proc(q,x,p)
  local dq,nq,ddq,dnq,n,m;

begin
  if p<>infinity and not testtype(p, Type::PolyExpr(x)) then
    return(error("Illegal third argument"))
  end_if;
  if q=0 then
      return(infinity);
  elif testtype(q, Type::RatExpr(x)) then
      q:=normal(q); dq:=denom(q); nq:=numer(q);
      ddq:=degree(dq,x); dnq:=degree(nq,x);
      if p=infinity then
        return(-dnq+ddq);
      elif irreducible(p) then
        n:=ddq+1; repeat n:=n-1; until divide(dq,p^n,[x],Rem)=0 end_repeat:
        m:=dnq+1; repeat m:=m-1; until divide(nq,p^m,[x],Rem)=0 end_repeat:
        return(m-n);
      else
        return(error("Illegal third argument : polynomial not irreducible"))
      end_if
  else
      return(error("Illegal first argument : not a rational function"))  
  end_if;
end_proc:

    unimod:=ode::mkUnimodular(eq,Transform);
    v:=normal(-unimod[3][1]);
    nuinfty:=valuation(v,x,infinity);
    dv:=denom(v); ddv:=diff(dv,x);
    u:=gcd(dv,ddv); vi:=divide(dv,u,[x],Quo); wi:=divide(ddv,u,[x],Quo);
    lQ:=[]; i:=1;
    repeat
      dvi:=diff(vi,x);
      hi:=gcd(vi,wi-dvi);
      vi:=divide(vi,hi,[x],Quo); wi:=divide(wi-dvi,hi,[x],Quo);
      i:=i+1;
      lQ:=append(lQ,hi);
    until vi=1 or vi=-1 end:
//    until vi=1 end:
    r:=genident(); a:=genident("a"); uu:=genident("u");
    mu:=genident("m"); nu:=genident("n"); eps:=genident("e");
    
//    Case Airy :
    Q:=_mult(lQ[3*i+2]^i $ i=1..floor((nops(lQ)-2)/3));
    dQ:=degree(Q,x);
    n:=max(dQ,dQ+floor((2-nuinfty)/3));
    eqz:=3*diff(y(x),x$2)^2-2*diff(y(x),x)*diff(y(x),x$3)+4*y(x)*diff(y(x),x)^4-4*v*diff(y(x),x)^2;
    p:=numer(eval(subs(eqz, y(x)=r(x)/Q))):
    P:=_plus(a.i*x^i $ i=0..n); p:=eval(subs(p, r(x)=P)):
    lc:={coeff(p,x,i) $ i=0..degree(p,x)}:
    N:=select([i $ i=0..n], ()->coeff(Q,x,args(1))<>0)[1];
    lc:=lc union
         {1-_plus((coeff(Q,x,i)*a.N-coeff(Q,x,N)*a.i)*a.(n+i+1) $ i=0..n)}:
    lc:=solvelib::discreteSolve(lc,{a.i $ i=0..2*n+1});
    if lc<>{} and lc<>FAIL then
      lc:=map(lc, ()->map(args(1),normal@expand));
      lc:=select(lc, ()->bool(map(args(1),
                         proc(eq) begin
                            if lhs(eq)=a.n and not has(rhs(eq),I) then
                              return(TRUE)
                            else
                              null()
                            end_if
                                  end_proc)<>[]));
      lc2:=select(lc, ()->testtype(factor(subs(normal(P/Q), args(1)))[1], Type::Negative)<>TRUE);
      if lc2<>{} then lc:=lc2 end_if:
      z:=subs(normal(P/Q), lc[1]);
      m:=normal(expand(1/diff(z,x)));
      m:=factor(simplify(expand(unimod[4]^2*m)));
      m:=_mult(m[2*i]^(m[2*i+1]/2) $ i in select({j $j=1..nops(m) div 2}, ()->has(m[2*args(1)],x)));

      if not has(m,x) then
        return({airyAi(z), airyBi(z)})
      else
        return({m*airyAi(z), m*airyBi(z)})
      end_if;
    end_if;
    
//    Case Bessel :
    Q:=_mult(lQ[2*i+2]^i $ i=1..floor((nops(lQ)-2)/2));
    dQ:=degree(Q,x);
    n:=max(dQ+1,dQ+floor(1-nuinfty/2));
    eqz:=3*diff(y(x),x$2)^2-2*diff(y(x),x)*diff(y(x),x$3)+(4*nu^2-1)*diff(y(x),x)^4/y(x)^2-4*eps*diff(y(x),x)^4-4*v*diff(y(x),x)^2;
    p:=numer(eval(subs(eqz, y(x)=r(x)/Q))):
    P:=_plus(a.i*x^i $ i=0..n); p:=eval(subs(p, r(x)=P)):
    lc:={coeff(p,x,i) $ i=0..degree(p,x)}:
    N:=select([i $ i=0..n], ()->coeff(Q,x,args(1))<>0)[1];
    lc:=lc union
         {1-_plus((coeff(Q,x,i)*a.N-coeff(Q,x,N)*a.i)*a.(n+i+1) $ i=0..n)}:
    lc:=solvelib::discreteSolve(lc union
                                {eps^2-1},{eps, nu, a.i $ i=0..2*n+1});
    if lc<>{} and lc<>FAIL then
      lc:=map(lc, ()->map(args(1),normal@expand));
      lc:=select(lc, ()->bool(map(args(1),
                         proc(eq) begin
                            if lhs(eq)=nu and factor(rhs(eq))[1]<>-1 then
                              return(TRUE)
                            else
                              null()
                            end_if
                                  end_proc)<>[]));
      lc2:=select(lc, ()->testtype(factor(subs(normal(P/Q), args(1)))[1], Type::Negative)<>TRUE);
      if lc2<>{} then lc:=lc2 end_if:
      z:=subs(normal(P/Q), lc[1]);
      m:=normal(expand(z/diff(z,x)));
      m:=factor(simplify(expand(unimod[4]^2*m)));
      m:=_mult(m[2*i]^(m[2*i+1]/2) $ i in select({j $j=1..nops(m) div 2}, ()->has(m[2*args(1)],x)));

      nu:=subs(nu, lc[1]); eps:=subs(eps, lc[1]);
      if eps=1 then
        if not has(m,x) then
          return({besselJ(nu,z), besselY(nu,z)})
        else
          return({m*besselJ(nu,z), m*besselY(nu,z)})
        end_if;
      else
        if not has(m,x) then
          return({besselI(nu,z), besselK(nu,z)})
        else
          return({m*besselI(nu,z), m*besselK(nu,z)})
        end_if;  
      end_if;
    end_if;


//    Case Whittaker :
    Q:=_mult(lQ[2*i+2]^i $ i=1..floor((nops(lQ)-2)/2));
    dQ:=degree(Q,x);
    n:=max(dQ+1,dQ+floor(1-nuinfty/2));
    eqz:=3*diff(y(x),x$2)^2-2*diff(y(x),x)*diff(y(x),x$3)+(1-4*mu/y(x)-(1-4*nu^2)/y(x)^2)*diff(y(x),x)^4-4*v*diff(y(x),x)^2;
    p:=numer(eval(subs(eqz, y(x)=r(x)/Q))):
    P:=_plus(a.i*x^i $ i=0..n); p:=eval(subs(p, r(x)=P)):
    lc:={coeff(p,x,i) $ i=0..degree(p,x)}:
    N:=select([i $ i=0..n], ()->coeff(Q,x,args(1))<>0)[1];
    lc:=lc union
         {1-_plus((coeff(Q,x,i)*a.N-coeff(Q,x,N)*a.i)*a.(n+i+1) $ i=0..n)}:
    lc:=solvelib::discreteSolve(lc,{mu, nu, a.i $ i=0..2*n+1});
    if lc<>{} and lc<>FAIL then
      lc:=map(lc, ()->map(args(1),normal@expand));
      lc:=select(lc, ()->bool(map(args(1),
                         proc(eq) begin
                            if lhs(eq)=nu and factor(rhs(eq))[1]<>-1 then
                              return(TRUE)
                            else
                              null()
                            end_if
                         end_proc)<>[]));
      lc2:=select(lc, ()->testtype(factor(subs(normal(P/Q), args(1)))[1], Type::Negative)<>TRUE);
      if lc2<>{} then lc:=lc2 end_if:
      z:=subs(normal(P/Q), lc[1]);
      m:=normal(expand(1/diff(z,x)));
      m:=factor(simplify(expand(unimod[4]^2*m)));
      m:=_mult(m[2*i]^(m[2*i+1]/2) $ i in select({j $j=1..nops(m) div 2}, ()->has(m[2*args(1)],x)));

      mu:=subs(mu, lc[1]); nu:=subs(nu, lc[1]);
      if not has(m,x) then
        return({whittakerM(mu,nu,z),whittakerW(mu,nu,z)})
      else
        return({m*whittakerM(mu,nu,z),m*whittakerW(mu,nu,z)})
      end_if;
    end_if;


// Case Bessel with algebraic transformation :
    Q:=_mult(lQ[i]^(i-2) $ i=3..nops(lQ));
    dQ:=degree(Q,x);
    n:=max(0, min(2*(dQ+2)-1, 2*dQ+2-nuinfty)); // should be max, but the algebraic solver is too slow
    eqz:=3*diff(y(x),x$2)^2-2*diff(y(x),x)*diff(y(x),x$3)+(4*nu^2-1)*diff(y(x),x)^4/y(x)^2-4*eps*diff(y(x),x)^4-4*v*diff(y(x),x)^2;
    p:=eval(subs(eqz, y(x)=r(x)/Q^(1/2))):
    P:=_plus(a.i*x^(i/2) $ i=0..n); p:=eval(subs(p, r(x)=P)):
    assume(uu, Type::Positive);
    p:=numer(simplify(subs(p, x=uu^2)));
    lc:={coeff(p,uu,i) $ i=0..degree(p,uu)}:
    N:=select([i $ i=0..n], ()->coeff(Q,x,args(1))<>0)[1];
    lc:=lc union
         {1-_plus((coeff(Q,x,i)*a.N-coeff(Q,x,N)*a.i)*a.(n+i+1) $ i=0..n)}:
    lc:=solvelib::discreteSolve(lc union
                                {eps^2-1},{eps, nu, a.i $ i=0..2*n+1});
    if lc<>{} and lc<>FAIL then
      lc:=map(lc, ()->map(args(1),normal@expand));
      lc:=select(lc, ()->bool(map(args(1),
                         proc(eq) begin
                            if lhs(eq)=nu and factor(rhs(eq))[1]<>-1 then
                              return(TRUE)
                            else
                              null()
                            end_if
                                  end_proc)<>[]));
      lc2:=select(lc, ()->testtype(factor(subs(normal(P/Q), args(1)))[1], Type::Negative)<>TRUE);
      if lc2<>{} then lc:=lc2 end_if:
      z:=subs(normal(P/Q^(1/2)), lc[1]);
      m:=normal(expand(z/diff(z,x)));
      m:=factor(simplify(expand(unimod[4]^2*m)));
      m:=_mult(m[2*i]^(m[2*i+1]/2) $ i in select({j $j=1..nops(m) div 2}, ()->has(m[2*args(1)],x)));

      nu:=subs(nu, lc[1]); eps:=subs(eps, lc[1]);
      if eps=1 then
        if not has(m,x) then
          return({besselJ(nu,z), besselY(nu,z)})
        else
          return({m*besselJ(nu,z), m*besselY(nu,z)})
        end_if;
      else
        if not has(m,x) then
          return({besselI(nu,z), besselK(nu,z)})
        else
          return({m*besselI(nu,z), m*besselK(nu,z)})
        end_if;  
      end_if;
    end_if;


//    Case Whittaker with algebraic transformation :
    Q:=_mult(lQ[i]^(i-2) $ i=3..nops(lQ));
    dQ:=degree(Q,x);
    n:=max(0, min(2*(dQ+2)-1, 2*dQ+2-nuinfty)); // should be max, but the algebraic solver is too slow
    eqz:=3*diff(y(x),x$2)^2-2*diff(y(x),x)*diff(y(x),x$3)+(1-4*mu/y(x)-(1-4*nu^2)/y(x)^2)*diff(y(x),x)^4-4*v*diff(y(x),x)^2;
    p:=eval(subs(eqz, y(x)=r(x)/Q^(1/2))):
    P:=_plus(a.i*x^(i/2) $ i=0..n); p:=eval(subs(p, r(x)=P)):
    assume(uu, Type::Positive);
    p:=numer(simplify(subs(p, x=uu^2)));
    lc:={coeff(p,uu,i) $ i=0..degree(p,uu)}:
    N:=select([i $ i=0..n], ()->coeff(Q,x,args(1))<>0)[1];
    lc:=lc union
         {1-_plus((coeff(Q,x,i)*a.N-coeff(Q,x,N)*a.i)*a.(n+i+1) $ i=0..n)}:
    lc:=solvelib::discreteSolve(lc,{mu, nu, a.i $ i=0..2*n+1});
    if lc<>{} and lc<>FAIL then
      lc:=map(lc, ()->map(args(1),normal@expand));
      lc:=select(lc, ()->bool(map(args(1),
                         proc(eq) begin
                            if lhs(eq)=nu and factor(rhs(eq))[1]<>-1 then
                              return(TRUE)
                            else
                              null()
                            end_if
                         end_proc)<>[]));
      lc2:=select(lc, ()->testtype(factor(subs(normal(P/Q), args(1)))[1], Type::Negative)<>TRUE);
      if lc2<>{} then lc:=lc2 end_if:
      z:=subs(normal(P/Q^(1/2)), lc[1]);
      m:=normal(expand(1/diff(z,x)));
      m:=factor(simplify(expand(unimod[4]^2*m)));
      m:=_mult(m[2*i]^(m[2*i+1]/2) $ i in select({j $j=1..nops(m) div 2}, ()->has(m[2*args(1)],x)));

      mu:=subs(mu, lc[1]); nu:=subs(nu, lc[1]);
      if not has(m,x) then
        return({whittakerM(mu,nu,z),whittakerW(mu,nu,z)})
      else
        return({m*whittakerM(mu,nu,z),m*whittakerW(mu,nu,z)})
      end_if;
    end_if;



// A la Cheb-Terrab :
/*    a:=genident("a"); b:=genident("b"); c:=genident("c");
    X:=genident("x");
    eqz:=3*diff(y(x),x$2)^2-2*diff(y(x),x)*diff(y(x),x$3)+(-2*c-y(x)^2+c^2+2*y(x)*c+y(x)^2*b^2+4*a*b*y(x)-2*y(x)^2*a*b-2*y(x)*b*c-2*y(x)*a*c+y(x)^2*a^2)/y(x)^2/(y(x)-1)^2*diff(y(x),x)^4-4*v*diff(y(x),x)^2;
    p:=numer(eval(subs(eqz, y(x)=(x^1+X.2)/(X.3*x^1+X.4)))):
    assume(n, Type::Rational);
    lc:={coeff(p,x,i) $ i=0..degree(p,x)}:
    lc:=solvelib::discreteSolve(lc union {X.5*(X.4-X.2*X.3)-1},
                                   {a, b, c, X.i $ i=1..5});

*/
  end_if;
  {}
end_proc:

/*

eq:=diff(y(x),x$2)-27*x*y(x)/(2*x+1)/(x-1)^4;
eq:=x^2*diff(y(x),x$2)+x*diff(y(x),x)-(n^2+4*x^2-x^4)*y(x);
eq:=x^2*diff(y(x),x$2)+x*diff(y(x),x)-(a^2+b^2*x^2+c^2*x^4)*y(x);
eq:=x^2*diff(y(x),x$2)+a*x*diff(y(x),x)+x^2*(a1+b1*x^2)*y(x);
eq:=x^2*diff(y(x),x$2)+(a+b*x)*diff(y(x),x)+c*y(x);


eq:=diff(y(x),x$2)-1/20*(405*x^6-5670*x^4+58604*x^2+13720)/x^2/(3*x^2-14)^3*y(x);
eq:=diff(y(x),x$2)-(x+1)*(-2+x)/(x-1)^6*y(x);
eq:=diff(y(x),x$2)-(v0+v1*x)^4*y(x);
eq:=diff(y(x),x$2)+(a*x^4+b*x)*y(x);

eq:=x*(x-1)*diff(y(x),x$2)+((1+1/2+1)*x-1/3)*diff(y(x),x)+1*1/2*y(x);

eq:=9*x*diff(y(x), x, x)*(x - 1)^2*(2*x - 1) - y(x)*(15*x - 22*x^2 + 4*x^3 - \
1) + 6*x*diff(y(x), x)*(x - 1)*(4*x^3 - 8*x^2 + 3);

eq:=diff(y(x),x$2)+(192*x^12-269*x^8+122*x^4-9)/12/(x-1)^2/x^2/(x+1)^2/(x^2+1)^2*y(x);

eq:=diff(y(x),x$2)+2/x*diff(y(x),x)+4*(Mu*x^4-(103+Mu)*x^2+13)/(x-1)^2/(x+1)^2*y(x);

eq:=diff(y(x),x$2)-x*(3*x+1)*(5*x+7)/4/(x+1)^5*y(x);
eq:=diff(y(x),x$2)-27*x*y(x)/(1+2*x)/(x-1)^4;

eq:=x*(1-x)*diff(y(x),x$2)+(C-(A+B+1)*x)*diff(y(x),x)-A*B*y(x);



lc:=[a11*a15 + a12*a16 + a13*a17 + 1, a10^2*a11^4 + 12*a10^2*a12^2 + 4*a11^4*\
n1^2 - a11^4 - 12*a10^2*a11*a13 - 4*a10*a11^4*m1, 2*a10*a11^5 - 8*a11^3*a1\
2 - 4*a11^5*m1 + 24*a10*a11*a12^2 - 24*a10*a11^2*a13 + 48*a10^2*a12*a13 + \
4*BB*a10^2*a11^2 + 8*a10^2*a11^3*a12 + 32*a11^3*a12*n1^2 - 32*a10*a11^3*a1\
2*m1, 24*a10*a12^3 - 24*a11^3*a13 + 72*a10^2*a13^2 - 12*a11^2*a12^2 + a11^\
6 + 24*a10^2*a11^2*a12^2 + 96*a11^2*a12^2*n1^2 + 8*BB*a10*a11^3 + 18*a10*a\
11^4*a12 - 36*a11^4*a12*m1 + 12*a10^2*a11^3*a13 + 48*a11^3*a13*n1^2 + 16*B\
B*a10^2*a11*a12 - 48*a10*a11^3*a13*m1 - 96*a10*a11^2*a12^2*m1 + 72*a10*a11\
*a12*a13, 4*BB*a11^4 - 8*a11*a12^3 + 10*a11^5*a12 + 120*a10*a11*a13^2 + 26\
*a10*a11^4*a13 + 120*a10*a12^2*a13 - 48*a11^2*a12*a13 - 52*a11^4*a13*m1 + \
16*BB*a10^2*a12^2 + 64*a10*a11^3*a12^2 + 32*a10^2*a11*a12^3 - 128*a11^3*a1\
2^2*m1 + 128*a11*a12^3*n1^2 + 40*BB*a10*a11^2*a12 + 24*BB*a10^2*a11*a13 - \
128*a10*a11*a12^3*m1 + 72*a10^2*a11^2*a12*a13 + 288*a11^2*a12*a13*n1^2 - 2\
88*a10*a11^2*a12*a13*m1, 14*a11^5*a13 + 16*a10^2*a12^4 - 6*a11^2*a13^2 + 4\
1*a11^4*a12^2 + 64*a12^4*n1^2 - 4*a12^4 + 54*a10^2*a11^2*a13^2 + 216*a11^2\
*a13^2*n1^2 + 24*BB*a11^3*a12 + 240*a10*a12*a13^2 - 36*a11*a12^2*a13 - 64*\
a10*a12^4*m1 + 4*AA*a10^2*a11^2 + 112*a10*a11^2*a12^3 - 224*a11^2*a12^3*m1\
 + 64*BB*a10*a11*a12^2 + 56*BB*a10*a11^2*a13 + 48*BB*a10^2*a12*a13 + 184*a\
10*a11^3*a12*a13 - 368*a11^3*a12*a13*m1 + 144*a10^2*a11*a12^2*a13 - 216*a1\
0*a11^2*a13^2*m1 + 576*a11*a12^2*a13*n1^2 - 576*a10*a11*a12^2*a13*m1, 144*\
a10*a13^3 - 24*a12^3*a13 + 88*a11^3*a12^3 + 8*AA*a10*a11^3 + 32*BB*a10*a12\
^3 + 32*BB*a11^3*a13 + 96*a10*a11*a12^4 + 114*a11^4*a12*a13 - 192*a11*a12^\
4*m1 + 36*BB*a10^2*a13^2 + 52*BB*a11^2*a12^2 + 132*a10*a11^3*a13^2 + 96*a1\
0^2*a12^3*a13 - 264*a11^3*a13^2*m1 + 384*a12^3*a13*n1^2 + 16*AA*a10^2*a11*\
a12 - 384*a10*a12^3*a13*m1 + 480*a10*a11^2*a12^2*a13 + 216*a10^2*a11*a12*a\
13^2 - 960*a11^2*a12^2*a13*m1 + 864*a11*a12*a13^2*n1^2 - 864*a10*a11*a12*a\
13^2*m1 + 176*BB*a10*a11*a12*a13, 4*AA*a11^4 + 32*a10*a12^5 + 24*a11*a13^3\
 - 64*a12^5*m1 + 104*a11^2*a12^4 + 79*a11^4*a13^2 - 36*a12^2*a13^2 + 216*a\
10^2*a12^2*a13^2 + 864*a12^2*a13^2*n1^2 + 48*BB*a11*a12^3 + 16*AA*a10^2*a1\
2^2 + 108*a10^2*a11*a13^3 + 364*a11^3*a12^2*a13 + 432*a11*a13^3*n1^2 + 40*\
AA*a10*a11^2*a12 + 24*AA*a10^2*a11*a13 + 120*BB*a10*a11*a13^2 + 128*BB*a10\
*a12^2*a13 + 136*BB*a11^2*a12*a13 + 544*a10*a11*a12^3*a13 - 432*a10*a11*a1\
3^3*m1 - 1088*a11*a12^3*a13*m1 + 684*a10*a11^2*a12*a13^2 - 864*a10*a12^2*a\
13^2*m1 - 1368*a11^2*a12*a13^2*m1, 16*BB*a12^4 + 64*a11*a12^5 - 24*a12*a13\
^3 + 24*AA*a11^3*a12 + 224*a10*a12^4*a13 - 448*a12^4*a13*m1 + 88*BB*a11^2*\
a13^2 + 324*a10*a11^2*a13^3 + 216*a10^2*a12*a13^3 + 568*a11^2*a12^3*a13 + \
500*a11^3*a12*a13^2 - 648*a11^2*a13^3*m1 + 864*a12*a13^3*n1^2 + 64*AA*a10*\
a11*a12^2 + 56*AA*a10*a11^2*a13 + 48*AA*a10^2*a12*a13 + 168*BB*a10*a12*a13\
^2 + 184*BB*a11*a12^2*a13 - 864*a10*a12*a13^3*m1 + 1152*a10*a11*a12^2*a13^\
2 - 2304*a11*a12^2*a13^2*m1, 81*a10^2*a13^4 + 228*a11^3*a13^3 + 324*a13^4*\
n1^2 + 16*a12^6 - 9*a13^4 + 1158*a11^2*a12^2*a13^2 + 32*AA*a10*a12^3 + 32*\
AA*a11^3*a13 + 72*BB*a10*a13^3 + 80*BB*a12^3*a13 + 432*a11*a12^4*a13 - 324\
*a10*a13^4*m1 + 36*AA*a10^2*a13^2 + 52*AA*a11^2*a12^2 + 624*a10*a12^3*a13^\
2 - 1248*a12^3*a13^2*m1 + 232*BB*a11*a12*a13^2 + 1080*a10*a11*a12*a13^3 - \
2160*a11*a12*a13^3*m1 + 176*AA*a10*a11*a12*a13, 128*a12^5*a13 + 48*AA*a11*\
a12^3 + 96*BB*a11*a13^3 + 378*a10*a11*a13^4 - 756*a11*a13^4*m1 + 148*BB*a1\
2^2*a13^2 + 864*a10*a12^2*a13^3 + 1160*a11*a12^3*a13^2 + 1044*a11^2*a12*a1\
3^3 - 1728*a12^2*a13^3*m1 + 120*AA*a10*a11*a13^2 + 128*AA*a10*a12^2*a13 + \
136*AA*a11^2*a12*a13, 16*AA*a12^4 + 351*a11^2*a13^4 + 424*a12^4*a13^2 + 12\
0*BB*a12*a13^3 + 594*a10*a12*a13^4 - 1188*a12*a13^4*m1 + 88*AA*a11^2*a13^2\
 + 1548*a11*a12^2*a13^3 + 168*AA*a10*a12*a13^2 + 184*AA*a11*a12^2*a13, 36*\
BB*a13^4 + 162*a10*a13^5 - 324*a13^5*m1 + 744*a12^3*a13^3 + 72*AA*a10*a13^\
3 + 80*AA*a12^3*a13 + 1026*a11*a12*a13^4 + 232*AA*a11*a12*a13^2, 270*a11*a\
13^5 + 729*a12^2*a13^4 + 96*AA*a11*a13^3 + 148*AA*a12^2*a13^2, 378*a12*a13\
^5 + 120*AA*a12*a13^3, 36*AA*a13^4 + 81*a13^6]:

sys:=FGbRS(lc, [AA,BB,m1,n1,a10,a11,a12,a13,a14,a15,a16,a17])
bg0 := FGbRS::Gbasis(sys)

*/
