// $Date: 2003/06/17 15:24:53 $ $Author: cormier $ $Revision: 1.19 $ 

/*  Input: eq is a first-order ODE in y(x).
   Output: a solution or FAIL.
   Example: ode::solve((x*diff(y(x),x)-y(x))^2-diff(y(x),x)^2-1,y(x));

           2     1/2     2     1/2                    1/2        1/2
{C2 x - (C2  + 1)   , (C2  + 1)    + C2 x, - I (x - 1)    (x + 1)   ,

            1/2        1/2
   I (x - 1)    (x + 1)   }

   Reference: Zwillinger chap. 38 page 158
*/

ode::clairaut :=
proc(eq,y,x) 
  local eq0, f, yp, sp, Z, Y, res;
// save Z, yp;
begin
   // delete yp;
  res:=FAIL;
  if type(eq)="_equal" then eq:=op(eq,1)-op(eq,2) end_if;
  yp:= genident();
  Z:= genident();
  eq0:=eq;
  eq:=expand(subs(eq,diff(y(x),x)=yp,y(x)=x*yp-Z));
  if not has(eq,{x,y}) and type(eq)="_plus" then
    f:=select(eq, has, Z);
//    The following test is too restrictive : for ex. yy'-xy'^2+1=0
//    is a Clairaut equation but fails the test
    if not has(f,yp) then
      userinfo(1,"Clairaut's equation");
      res:=ode::clairautEq(subs(f,Z=x),subs(f-eq,yp=x),x);
// else we make this new test :
    else
      if type(f)<>"_mult" then f:=hold(_mult)(f) end_if;
      sp:=split(f, has, Z);
      if not has(sp[1], yp) then
        userinfo(1,"Clairaut's equation");
        res:=ode::clairautEq(subs(f/sp[2],Z=x),
                               expand(subs(f/sp[2]-eq/sp[2],yp=x)),x);
      end_if
    end_if
  end_if;
  if res=FAIL and ode::order(eq0,{y})=1 then
    eq:=subs(eq0,diff(y(x),x)=yp,y(x)=Z);
    if testtype((Y:=solvelib::discreteSolve(eq,Z)),DOM_SET) then
      Y:=map(Y, () -> if not has(normal(-args(1)+x*yp), x) then
                     op(ode::clairautEq(x,subs(normal(-args(1)+x*yp),yp=x),x))
                      else null()
                      end_if);
      if Y<>{} then
        userinfo(1,"Clairaut's equation");
        res:=Y
      end_if;
    end_if;
  end_if;
  return(res);
end_proc:

/* solves f(x*y'-y) = g(y').
   A general solution is given by f(xC-y)=g(C) where C is a constant.
   Example: ode::clairautEq(x^2,x^2-1,x);
*/
ode::clairautEq :=
proc(_f_,_g_,x) // f and g are expressions in x 
  local C,gensol,eq,partsol,Partsol,p,y, yp;
// save y, yp;
begin
   // delete y,yp;
  y:= genident();
  yp:= genident();
  userinfo(3,"f=",_f_,"g=",_g_);
   // search for a general solution 
  C:=genident("C");
  gensol:=solvelib::discreteSolve(subs(_f_,x=x*C-y)=subs(_g_,x=C),y);
   //lev:=max(2-ode::depth,ode::maxdegree);
   //gensol:=solvelib::discreteSolve(subs(_f_,x=x*C-y)=subs(_g_,x=C),y,
   //                                MaxDegree=lev);
  if gensol=FAIL then
    userinfo(3,"finding general solution failed while solving ".
              expr2text(subs(_f_,x=x*C-y)=subs(_g_,x=C))." in ".expr2text(y));
    return(FAIL);
  else
    userinfo(3,"general solution is",gensol);
  end_if;
   // search for a singular solution 
  eq:=subs(diff(_f_,x),x=x*yp-y)*x-subs(diff(_g_,x),x=yp);
  partsol:=solvelib::discreteSolve(eq,yp);
   //partsol:=solvelib::discreteSolve(eq,yp,MaxDegree=ode::maxdegree);
  userinfo(3,"singular solution for y' is",partsol);
   //if type(partsol)="solve" then partsol:={} end_if;
   // ugly poor work around
  if type(partsol)="solve" or has(partsol, solve) or partsol=FAIL then
    partsol:={};
    userinfo(3,"particular solution cannot be found!");         
  end_if;
  Partsol:={};
  if partsol<>{} then
    for p in partsol do
      eq:=normal(expand(subs(_f_,x=x*p-y)-subs(_g_,x=p)));
      p:=solvelib::discreteSolve(eq,y);
      //partsol:=solvelib::discreteSolve(eq,y,MaxDegree=lev);
      if p<>FAIL then
        Partsol:=Partsol union p
      end_if; // ugly poor work around
    end_for;  
  end_if;
  userinfo(3,"particular solution is",Partsol);
  gensol union Partsol
end_proc:
