/* Input: eq is an ODE in y(x).
   Output: a solution or FAIL.
   Uses the differentiation method, cf Zwillinger chap. 44 p. 176.
   Example: 
    ode::solve(2*y(x)*diff(y(x),x,x)-diff(y(x),x)^2-1/3*(diff(y(x),x)
                     -x*diff(y(x),x,x))^2,y(x));

               {                 2  2       2             }
               {     3     C3  C7  x  + 3 C8  + 3 C7 C8 x }
               { C4 x , - ---, -------------------------- }
               {          4 x             3 C8            }

*/

ode::differentiation := proc(eq,_0_y,_0_x) local neweq,e,sol,s,l,c,_f,_fc,i;
begin
   userinfo(2,"trying differentiation method");
   neweq:=Factored::convert_to(factor(diff(eq,_0_x)),DOM_EXPR);
   if type(neweq)="_mult" then
	   neweq:=select(neweq, has,{_0_x,_0_y});
   end_if;
   if type(neweq)="_mult" then neweq:=[op(neweq)] else neweq:=[neweq] end_if;
   if max(op(map(neweq,length)))<length(eq) then // to avoid loops 
      sol:={};
      for e in neweq do
         sysassign(ode::depth,ode::depth+1);
         s:=ode::solve_eq(e,_0_y,_0_x);
         sysassign(ode::depth,ode::depth-1);
         if type(s)=DOM_SET and s<>{} and s<>{FAIL} then
            s:=op(s,1);
	    if type(s)="RootOf" then
		s:=op(solvelib::discreteSolve(op(s)),1)
	    end_if;
            if traperror((e:=simplify(eval(subs(eq,_0_y(_0_x)=s)))))=0 then
              if e=0 then sol:=sol union {s}
              else // check in the initial equation 
  	        if type(e)="_mult" then e:=[op(e)] else e:=[e] end_if;
  	         for _f in e do
		   l:=indets(_f) minus indets(eq);
		   if l={} then next end_if;
                   c:=op(l,1);
                   _fc:=solvelib::discreteSolve(_f,c);
		   sol:=sol union 
                        simplify({subs(s,c=op(_fc, i)) $ i=1..nops(_fc)})
                        //{subs(s,c=op(solvelib::discreteSolve(_f,c,
                        //                      MaxDegree=ode::maxdegree)))}
	         end_for
              end_if
            else return(FAIL)
            end_if
         end_if
      end_for;
      if sol<>{} then 
         userinfo(1,"differentiation method worked");
         return(sol)
      end_if;
   end_if;
//   userinfo(1,"differentiation method failed");
   FAIL
end_proc:

