// $Date: 2003/09/29 12:04:20 $ $Author: cormier $ $Revision: 1.22 $ 

/* ode::solvable_for_x(eq,y,x) tries to solve the first-order ODE eq=0 wrt y(x)
   using the method of "solvable in x" equations (cf Zwillinger pages 304-305).

   Output: either FAIL or a set of solutions.

   Example:
   ode::solvable_for_x(2*x*diff(y(x),x)+y(x)*diff(y(x),x)^2-y(x),y,x);

                           1/2                2 1/2
           {(C2 (C2 + 2 x))   , - (2 C2 x + C2 )   , -I x, I x}

*/

ode::solvable_for_x := proc(eq,y,x) local p,xx,sol,l,soll;
begin
   userinfo(2,"trying to recognize an equation solvable for x");
   p:=genident("p");
   eq:=subs(eq,diff(y(x),x)=p,y(x)=y);
   // Note, I use MaxDegree only, since otherwise solve needs too much time
   sol:=solvelib::discreteSolve(eq,x,MaxDegree=1); //,MaxDegree=1);
   if not ode::isResultOfSolveOK(sol,"NoRootOf") then return(FAIL) end_if;
   //if type(sol)="solve" or sol=FAIL then return(FAIL) end_if;
   //sol:=select(sol,not hastype,RootOf);
   if nops(sol)=0 or sol={0} or not has(sol, y) then return(FAIL) end_if;
   userinfo(1,"equation solvable for x");
   xx:=subs(op(sol,1),p=p(y));
   sol:=numer(diff(xx,y)-1/p(y));
   sysassign(ode::depth,ode::depth+1);
   sol:=ode::solve_eq(sol,p,y,{});
   sysassign(ode::depth,ode::depth-1);
   // don't allow nested implicit solutions 
   if type(sol)="solve" or has(sol,FAIL) then 
      userinfo(1,"solvable for x method failed");return(FAIL)
   elif type(sol)=DOM_SET  then
      if map(sol,type) intersect {"solve", RootOf} <> {} then
         userinfo(1,"solvable for x method failed");return(FAIL) 
      end_if
   elif type(sol)="_union" then
      if {map(op(sol),type)} intersect {"solve", RootOf} <> {} then
         userinfo(1,"solvable for x method failed");return(FAIL) 
      end_if
   else
      userinfo(1,"solvable for x method failed");return(FAIL)
   end_if;
   sol:=map(sol,proc() begin factor(numer(normal(subs(eq,p=args(1))))) end_proc );
   sol:=map(sol,solvelib::discreteSolve,y);
   if not(has(sol, FAIL)) then
     userinfo(1,"solvable for x method worked");
     soll:={};
     for l in sol do
       if type(l)="_union" then
         l:=map({op(l)}, ()-> (if type(args(1))="solve" then args(1)
                      else op(args(1))
                            end_if));
       end_if;
       soll:=soll union l;
     end_for;
     soll;
   else
     FAIL
   end_if;
end_proc:

