 // $Date: 2003/09/17 15:57:16 $ $Author: cormier $ $Revision: 1.47 $ 
 
 /* main function to solve a single differential equation eq=0 in y(z) 
   with initial conditions inits */
 
 
ode::solve_eq :=
 proc(eq,y,z,inits={}:DOM_SET)
   local _l,i,den,s;
 // global ode::depth; 
 begin
   if ode::depth>ode::maxDepth then
     return({FAIL})
   end_if;
   if type(eq)="_equal" then
     eq:=op(eq,1)-op(eq,2)
   end_if;
   if has(eq,exp) then
     eq:=combine(eq,exp)
   end_if;// simplify exp(t)*exp(-t)
   if has(eq, sin) and has(eq, cos) then
     eq:=combine(eq, sincos)
   end_if;  // to simplify sin(z)^2+cos(z)^2 
   if iszero(eq) then
     return({y(z)})
   end_if;
   den:=denom(eq); // must not vanish
   eq:=numer(eq);
   userinfo(2,"trying to factor");
    _l:=maprat(eq,factor); // to avoid problems with gcd 
   userinfo(2,"factorization ".
            (if nops(_l)=3 then "failed" else "worked" end_if));
   s:=_union(ode::solve_eq_irred(_l[2*i],y,z,inits,den) $ i=1..nops(_l) div 2);
/*   if calledFromWithinDSolve = TRUE then
     if ode::isResultOfSolveOK(s,"AllowImplicit") then
       s
     else // try to extract partial solutions which are ok
       if type(s)="_union" and (s:=select([op(s)],testtype, DOM_SET))<>[] then
         _union(op(s))
       else
         return({FAIL})
       end_if;
     end_if;
   else
     s
   end_if;*/
   s;
end_proc:
 
 
 // returns a list of solutions or [] if no solution found 
 
ode::solve_eq_irred :=
 proc(eq,y,z,inits,den)
   local n,s,_l0,_l,sol,Lhs,_a,i,csts,islin,yp,elim,r;
   // save yp;
 begin
   islin := FALSE;
    userinfo(2,"solving ODE",eq,"with respect to",y(z));
    // delete yp;
   yp:= genident();
   if not testtype(eq,Type::ODE(y,z)) then
     error("not an ordinary differential equation in the given variable")
   end_if;
   n:=ode::order(eq,{y});
   if n=-1 then
     return({})
   end_if; // no y(..)
   userinfo(2,"ordinary differential equation of order",n);
   if n=0 then
     s:=subs(eq,y(z)=yp);
     sol:=solvelib::discreteSolve(s,yp);
       //# solve up to quadratics here, don't use ode::maxdegree #
       //sol:=solvelib::discreteSolve(s,yp,MaxDegree=2);
   elif (sol:=ode::separate(eq,y,z,n))=FAIL then
       // first try to recognize linear equations 
     _l0:=[diff(y(z),z$i) $ i=0..n];
/*     I replace in the following line eq by eq0=expand(simplify(eq/den)),
     because if eq0 is linear and the homogeneous part of eq0 has
     rational coefficients and the inhomogeneous part is non-rational, then
     the coefficients of eq are no longer rational and solve_linear fails. */
     if den <> 1 then _l:=Type::Linear(expand(simplify(eq/den)),_l0) end_if:
     if den=1 or _l=FALSE then _l:=Type::Linear(eq,_l0) end_if:
     if _l<>FALSE then
       islin:=TRUE;
       sol:=ode::solve_linear(_l,z,n)
     else // non linear
       userinfo(2,"non linear ordinary differential equation");
       if n=2 then
         sol:=ode::exact_second_order(eq/den,y,z);
         if sol=FAIL then sol:=ode::int_factors2(eq,y,z) end_if:
       end_if:
       if sol=FAIL and n>=3 then
         sol:=ode::exact_nth_order_nonlinear(eq/den,y,z)
       end_if:
       if sol=FAIL then sol:=ode::autonomousEq(eq,y,z,n) end_if:
        // exact n-th order equations 
       if sol=FAIL then
         if n=1 then
           sol:=ode::exact_first_order(eq/den,y,z);
           if sol=FAIL then
             sol:=ode::homogeneous(eq,y,z)
           end_if;
           if sol=FAIL then
             sol:=ode::int_factors(eq,y,z)
           end_if;
/*           if sol=FAIL then
             sol:=ode::lagrange(eq,y,z)
           end_if;*/
           if sol=FAIL then
             sol:=ode::riccati(eq,y,z)
           end_if;
           if sol=FAIL then
             sol:=ode::homogeneousG(eq,y,z)
           end_if;
           if sol=FAIL then
             sol:=ode::homogeneousC(eq,y,z)
           end_if;
           if sol=FAIL then
             sol:=ode::homogeneousD(eq,y,z)
           end_if;
           if sol=FAIL then
             sol:=ode::lagrange(eq,y,z)
           end_if;
           if sol=FAIL then
             sol:=ode::bernoulli(eq,y,z)
           end_if;
           if sol=FAIL then
             sol:=ode::abel(eq,y,z)
           end_if;
           if sol=FAIL then
             sol:=ode::chini(eq,y,z)
           end_if;
                //if sol=FAIL then 
                //   sol:=ode::solvable_for_x(eq,y,z) end_if;
                //solvable_for_y currently not used because it can return only
                //parametric solutions, which are not yet dealt with 
                //if sol=FAIL then 
                //  sol:=ode::solvable_for_y(eq,y,z) end_if; 
//         elif n=2 then
//           sol:=ode::exact_second_order(eq/den,y,z)
//   We have to consider the denominator because here eq is the numerator
//   of the very first ODE in solve and this latter one may be exact whereas
//   eq may not be!      
         end_if
       end_if;
//       if sol=FAIL then sol:=ode::bernoulli(eq,y,z) end_if;
       if sol=FAIL then sol:=ode::clairaut(eq,y,z) end_if;
//      if sol=FAIL and n=2 then sol:=ode::int_factors2(eq,y,z) end_if;
//       if sol=FAIL then sol:=ode::differentiation(eq,y,z) end_if;
       if sol=FAIL and n=1 then sol:=ode::contact(eq,y,z) end_if;
       if sol=FAIL and n=1 then sol:=ode::solvable_for_x(eq,y,z) end_if;
       if sol=FAIL and n=1 then sol:=ode::solvable_for_y(eq,y,z) end_if;
     end_if;
       // currently factoring of diff. operator only implemented for linear eqs 
     if sol=FAIL and islin=TRUE then sol:=ode::factor(eq,y,z,n) end_if;
     if sol=FAIL then sol:=ode::ymissing(eq,y,z,n) end_if;
     if sol=FAIL then sol:=ode::differentiation(eq,y,z) end_if;
     if sol=FAIL then sol:=ode::scale_invariant(eq,y,z,n) end_if;
     if sol=FAIL then sol:=ode::equidimensional_in_y(eq,y,z,n) end_if;
     if sol=FAIL then sol:=ode::equidimensional_in_x(eq,y,z,n) end_if;
       // methods that call other methods and/or call solve_eq recursively 
     if sol=FAIL then sol:=ode::interchange(eq,y,z,n) end_if;
     // scale_invariant calls equidimensional_in_x
//     if sol=FAIL then sol:=ode::scale_invariant(eq,y,z,n) end_if;
//     if sol=FAIL then sol:=ode::differentiation(eq,y,z) end_if;
   end_if;
   if sol=FAIL then
     userinfo(2,"solution is",sol);
     return({FAIL}) // do not return FAIL because of _concat
   else

     // if sol contains some solve or RootOf then replace the argument by the initial
     // dependent variable y :
     sol:=ode::mapsol(sol, s -> if domtype(s)=RootOf then
                                  subs(s, op(s,2)=y)
                                elif type(s)="solve" then
                                  hold(solve)(subs(op(s,1), op(s,2)=y), y)
                                else s
                                end_if );
     
/* Some procedure (such as ode::autonomousEq) may introduce potential constant
     solution that are fact not solution of the original equation. The
     following checks if constant are indeed solutions. As we solve the
     numerator of the very first ODE, we have also to check if explicit
     solutions are not zero of the denominator. Finally, if a RootOf appears
     in a set then it is deleted and added as a union. */
     elim:=proc(l)
     begin
       if type(l)=DOM_SET then
         l:=select(l,()->expand(eval(subs(den, y(z)=args(1))))<>0);
         l:=select(l, ()->
                  if testtype(args(1), Type::Constant) and
                     simplify(expand(eval(subs(eq, y(z)=args(1)))))<>0 then
                     FALSE
                  else
                     TRUE
                  end_if);
         r:=select(l, testtype, RootOf);
         return(l minus r union _union(op(r)));
       else
         return(l)
       end_if
     end_proc:

     if type(sol)=DOM_SET then
       sol:=elim(sol)
     else
       sol:=map(sol, elim)
     end_if;

     userinfo(2,"solution is",sol);
                  
   end_if;
   
   if inits={} then
     return(sol)
   end_if;
   userinfo(1,"trying to fulfill initial conditions");
   userinfo(2,"which are",inits);
    // do not deal with implicit solutions for BVPs 
/*   if type(sol)="solve" or domtype(sol)=RootOf then
     /* nothing !? */
   elif type(sol)<>DOM_SET then
     return({FAIL})
   end_if;*/
   

   s:={};
//   csts:=indets(sol) minus (indets(eq) union {y,z});
   csts:=indets(sol) minus (indets(eq) union {y,z}) minus
          {PI, CATALAN, EULER, IgnoreProperties, IgnoreSpecialCases};
   for eq in inits do
     if has(op(eq,2),y) then
       error("dependent variable not allowed in rhs of initial conditions")
     end_if;
     Lhs:=op(eq,1); i:=0; _a:=op(Lhs);
     Lhs:=op(Lhs,0); // should be y or D(...D(y)...)
     case type(Lhs)
       of DOM_IDENT do
         break
       of "D" do
         repeat
           Lhs:=op(Lhs);
           i:=i+1
         until type(Lhs)<>"D" end_repeat;
         break
       otherwise
         error("invalid initial conditions")
     end_case;
     if Lhs<>y then
       error("invalid initial conditions")
     end_if;
     s := s union {[i,_a,op(eq,2)]}; // [i,a,b] means (D@@i)(f)(a)=b
//     Following lines are commented because they produced no solutions
//     for the equation : y'=1/sqrt(x), y(0)=1 
/*     if eval(subs(den,z=_a,op(inits)))=0 then
       userinfo(1,"denominator ".expr2text(den).
                " vanishes for given initial conditions");
       return({})
     end_if; */
   end_for;

   sol:=ode::mapsol(sol,ode::solvinit,s,csts,z);
   if sol<>{FAIL} and has(sol, FAIL) then
     sol:= sol minus {FAIL}
   end_if;
   sol;
end_proc:
 
 // inits is a set of triples [i,a,b] 
ode::solvinit :=
 proc(sol,inits,csts,z)
   local y,eq,ini,_a,l,r,s,t,i,a,definiteInt,c,newcsts,serR,serL,bord,res,ldeg;
 begin
   userinfo(3,"general solution is",sol);
   userinfo(3,"initial conditions are",inits);

   if type(sol)=DOM_LIST then
     // output for parametric solutions (some examples of Lagrange's equations)
     // don't know how to solve the ICs problem in this case.
     return({FAIL})
   end_if:

   if type(sol)="solve" or domtype(sol)=RootOf then
     y:=op(sol,2);
//     if nops(inits)=1 and op(inits,[1,1])=0 then
       eq:=op(sol,1);
       if type(eq)="_equal" then
         eq:=subs(op(eq,1)-op(eq,2), y=y(z))
       else
         eq:=subs(eq, y=y(z))
       end_if:
//       inits:=op(inits,1);
       a:={subs(diff(y(z),z$op(ini,1))=op(ini,3), z=op(ini,2)) $ ini in inits};
       if traperror((l:=map( subs( {subs(diff(eq,z$op(ini,1)),z=op(ini,2)) $ ini in inits}, a), eval)))=0 then
         if l={0} then // every constant satisfies the conditions
           _a:={[a=a] $ a in csts}
         else
           _a:=solvelib::discreteSolve(l,csts)
         end_if:
         if type(_a)=DOM_SET then
           if _a={} then
             return({}) // no solution
           else
             l:=_union(solvelib::discreteSolve(radsimp(eval(subs(eq,a))),y(z),MaxDegree=4) $ a in _a);
             if type(l)="solve" then
               return(hold(solve)(subs(op(l,1),op(l,2)=y), y))
             elif traperror((l:=select(l, ()->bool(
                                              {radsimp(eval(subs(diff(args(1),z$op(ini,1)),
                                                       z=op(ini,2))))-op(ini,3) $ ini in inits} = {0})
                             )))=0 then
               if l<>{} then
                 r:=select(l, testtype, RootOf);
                 return(l minus r union _union(op(r)));
               end_if:
             end_if:
           end_if:
         end_if:
       end_if:
 //    end_if;
     return({FAIL})
   end_if:
   
   s:={};

   for t in inits do
     if traperror((
                   if type(t[2])=stdlib::Infinity then
                     a:=limit(diff(sol,z$t[1]), z=t[2]);
                     if (a=infinity and t[3]=infinity) or (a=-infinity and t[3]=-infinity) then
                       a:=0
                     else
                       a:=a-t[3]
                     end_if:
                   else
                     a:=eval(subs(diff(sol,z$t[1]),
                     hold(int)=proc() begin 0 end_proc ,z=t[2]))-t[3]
                   end_if:
                   ))=0 then
       s:=s union {a}
     elif sol<>0 then
       serR:=series(sol, z=t[2], Right);
       serL:=series(sol, z=t[2], Left);
       if type(serR)=Series::Puiseux and type(serL)=Series::Puiseux and
         (iszero(coeff(serR,0)-coeff(serL,0)) or has(coeff(serR,0)-coeff(serL,0),csts)) then
           c:=coeff(serR, t[1])*fact(t[1])-t[3];
           s:=s union {c};
           if (ldeg:=ldegree(serR))<0 then
             s:=s union {coeff(serR, ldeg)}
           elif (bord:=op(serR, 2))>1 then
             s:=s union {coeff(serR, i/bord) $ i=1..bord-1}
           end_if:

         // The coefficient c may contain some logarithmic terms containing z :
           if type(c)="_mult" then
             s:=s union split({c}, ()->has(args(),ln) and has(args(),z))[1]
           else
             s:=s union {split(c, ()->has(args(),ln) and has(args(),z))[1]}
           end_if:
         
       else
         return(FAIL)
       end_if:
     else
         s:=s union {t[3]}
     end_if:
   end_for;

   
   if hastype(s,RootOf) then return(FAIL) end_if;

//   Warning : new constants may appear when we have boundary problem
//   such as : y(0)=y(1). This may produce no solution!
//   In this case a new variable _vi is introduced and boundary problem
//   become : y(0)=_vi, y(1)=_vi.
   newcsts:={};
   for c in indets(inits) minus {z, PI, CATALAN, EULER} do
     if stringlib::contains(expr2text(c), "_v") then
       newcsts:=newcsts union {c}
     end_if;
   end_for;
   userinfo(3,"system to solve is",s,"with respect to",csts union newcsts);
   if s={0} then
     // the solution is a constant equal to the desired value
     return(sol)
   else
     s:=solvelib::discreteSolve(s, csts union newcsts, MaxDegree=4);
     s:={select(op(s), has, csts)};
     userinfo(3,"solution for initial conditions is",s);
     s:=select(s, ()->contains({DOM_LIST,DOM_SET},type(args(1))) );
      //userinfo(3,"solution for initial conditions is",s);
     if type(s)<>DOM_SET then
       FAIL
      //elif nops(s)=0 then null() # no solution #  // solve changed once more
     elif s={} or s={[]} then
       null() // no solution
     else // s is a set of solutions
       a:=select(inits, e->e[1]=0);
       if a={} then a:=op(inits,1)[2] else a:=op(a,1)[2] end_if;
       t:=genident("t");
       definiteInt:=proc()
                      local l;
                    begin
                      l:=subs([op(args())],z=t);
                      l[2]:= t=a..z;
                      int(op(l))
                    end_proc;
       sol:=misc::maprec(sol,{"int"}=definiteInt,PostMap);
//       sol:=map([subs(sol,op(s,i)) $ i=1..nops(s)], eval);
       res:=[];
       for i from 1 to nops(s) do
         if traperror((a:=eval(subs(sol, op(s,i)))))=0 then
           res:=append(res, a)
         end_if:
       end_for:
       if has(res,{sin,cos}) then
         res:=map(map(res, expand), combine, sincos)
       end_if;
       op(res)
     end_if
   end_if:
end_proc:
