/* $Date: 2004/08/02 12:42:15 $ $Author: tonner $ $Revision: 1.54 $ */
/* tonner, 23/06/98 */

/** output::CatUtils -- Utilies for the output generation
 */


/**  Utilies for the output generation
 *
 *   This domain provides utilities for Content based outputs.
 */
category output::CatUtils
  
/** Name : DOM_STRING
 *
 *  Every domain derived from output::CatUtils has to provide a Name
 *  entry which is used as slot name to store output methods for this
 *  output type.  E.g. if a domain output::MMLcontent derived from
 *  output::CatContent has the Name entry "MMLContent" the output 
 *  method for lists for the output type output::MMLcontent is stored
 *  and searched for in DOM_LIST::MMLContent.
 */
  Name;
  
/** outputType : DOM_STRING
 *
 *  outputType may be one of the strings "Content" or "Presentation"
 *  and defines which kind of output should be generated.
 */
  outputType;
  

/** create and print
 *
 *  These methods define how to create the internal representation
 *  and how to print it on the screen 
 */
  create;  print;

/** setAliasTab
 *
 *  set a table for alias replacements
 */
  setAliasTab := x -> sysassign(dom::aliasTab, x);
  aliasTab := table();

  currentObject := FAIL;
/** Method new - converts MuPAD data into its output representation
  *
  * hier ist das Auffinden von Methoden implementiert.  
  * Sollte in den Kern !!!!
  */
  new :=
  proc()
    local argType, method, result;
    save _domainOutputMode;
  begin
    // look for alias replacements
    if contains(dom::aliasTab, args()) then
      return(dom::new(dom::aliasTab[args()]))
    end_if;
    
    // do not assign args() to a local variable.  If
    // args() is an unnamed procedure, it will receive the name
    // of the local variable as a side effect !!!
    argType := (args())::dom;

    if (method := slot(argType, dom::Name)) <> FAIL then
      /*
       * a special output function is defined; use it
       */
      result := method(dom, args());
    elif (method := slot(argType, dom::outputType)) <> FAIL then
      /*
       * 
       */

      /* avoid assignment below; generate::MathML(matrix([[1 ,2], [3, 4]]))
         crashes  */
//      eval(hold(sysassign)( hold(slot)(text2expr(expr2text(argType)),
//                                       dom::Name), method));
      result := method(dom, args());
    elif (method := argType::print) <> FAIL then
      /*
       *  und nun alte "print"-Methode Aufrufen und deren Ergebnis ausgeben  
       */
      _domainOutputMode := TRUE;
      result := dom(method(args()));
      if domtype(result) = DOM_STRING then
      /* we want to see the output unquoted */
        result := hold(``).result
      end_if;
    else
      result := dom::unknownType(args());
    end_if;
    
    return(result)
  end_proc;
  
  testtype := FALSE;

  stdFunc;  // Domains of this type have to define a method 
            // stdFunc for the output of generic functions

  statement;  // defines the output of statements
  
  // used for elements of domains without output methods
  unknownType :=
  () -> dom::stdFunc(hold(new)(domtype(args()), extop(args())));
  

  // called once per output -- e.g.  set LaTeX output in \[ ... \] 
  enclose := id;


 //++++++++++++++++ a collection of utility routines  ++++++++++++++

  // decide whether x is an operator
  isOperator :=
  proc(x)
  begin
    if domtype(x) = DOM_IDENT then
      x := eval(x);
    end_if;
    if domtype(x) = DOM_FUNC_ENV then
      if domtype(op(x,[2,3])) = DOM_STRING then
        TRUE else FALSE
      end_if;
    else
      FALSE
    end_if;
  end_proc;
  // decide whether x is an infix operator
  isInfixOperator :=
  proc(x)
  begin
    if domtype(x) = DOM_IDENT then
      x := eval(x);
    end_if;
    if domtype(x) = DOM_FUNC_ENV then
      if domtype(op(x,[2,3])) = DOM_STRING
        and op(x, [2,1]) = 1100 then
        TRUE else FALSE
      end_if;
    else
      FALSE
    end_if;
  end_proc;
  // decide whether x is a prefix operator
  isPrefixOperator :=
  proc(x)
  begin
    if domtype(x) = DOM_IDENT then
      x := eval(x);
    end_if;
    if domtype(x) = DOM_FUNC_ENV then
      if domtype(op(x,[2,3])) = DOM_STRING
        and op(x, [2,1]) = 1098 then
        TRUE else FALSE
      end_if;
    else
      FALSE
    end_if;
  end_proc;
  // decide whether x is a postfix operator
  isPostfixOperator :=
  proc(x)
  begin
    if domtype(x) = DOM_IDENT then
      x := eval(x);
    end_if;
    if domtype(x) = DOM_FUNC_ENV then
      if domtype(op(x,[2,3])) = DOM_STRING
        and op(x, [2,1]) = 1099 then
        TRUE else FALSE
      end_if;
    else
      FALSE
    end_if;
  end_proc;

//++++++++++++++++ a collection of utility routines  ++++++++++++++
 
end_category:

// end of file 
