domain output::MMLContent
  category output::CatContent;
    inherits Content;
    
    Name := "MMLContent";

    // Counter for IDs of subexpressions
    refCount := 0;

// no own representation!
// elements of adt::XML are created    
    create := proc(ID, arg2)
                local nextRef;
              begin
                if not contains({"math", "semantics", "annotation",
                                 "annotation-xml", "bvar", "degree",
                                 "lowlimit", "uplimit", "condition"}, ID) and
                  args(0) > 1 and args(args(0)) <> DOM_SET then
                  dom::refCount := dom::refCount+1;
                  nextRef := "id"="No".dom::refCount;
                else
                  nextRef := null();
                end_if;

                // make sure that the id is always the 1st attribute
                if args(0) > 1 and domtype(arg2) = DOM_LIST then
                  adt::XML::Element(ID,
                      [nextRef, op(args(2))],
                      args(3..args(0)))
                else
                  adt::XML::Element(ID, [nextRef], args(2..args(0)))
                end_if;
              end_proc;
    
    // access element entries
    typString := proc(c)
                 begin
                   if contains({DOM_INT, DOM_STRING, DOM_FLOAT, DOM_IDENT},
                               domtype(c)) then
                     expr2text(domtype(c))
                   else
                     extop(c, 1)
                   end
                 end;

    attributString := proc(x)
                        local tab, str, tabNops, i;
                      begin
                        assert(domtype(extop(x, 2)) = DOM_SET);
                        
                        tab := table(op(extop(x, 2)));
                        tabNops := nops(tab);
                        if contains(tab, hold(id)) then
                          str := "id='".tab[hold(id)]."'";
                          delete tab[hold(id)];
                          tabNops := nops(tab);
                          if tabNops > 0 then
                            str := str." "
                          end_if;
                        else
                          str := "";
                        end_if;
                        for i from 1 to tabNops do
                          assert(domtype(op(tab, [i, 1]))=DOM_IDENT);
                          assert(domtype(op(tab, [i, 2]))=DOM_STRING);
                          str := str.expr2text(op(tab, [i, 1]))."='".
                                 op(tab, [i, 2]);
                          if i < tabNops then
                            str := str."' ";
                          else
                            str := str."'";
                          end_if;
                        end_for;
                        str;
                      end_proc;

    attributes := x -> extop(x, 2);

    str2cdata := s -> 
                 (s := stringlib::subs(s, "&"="&amp;");
                  s := stringlib::subs(s, "<"="&lt;");
                  s := stringlib::subs(s, ">"="&gt;");
                  stringlib::collapseWhitespace(s)
                  );
    expr :=
    proc(c)
      local cType, attributes;
    begin
      cType := domtype(c);

    // here we need a special conversion from backquoted identifiers
    // representing MML-constants to MuPAD constants
      if cType = DOM_IDENT and contains(dom::MML2ident, c) then
        return(dom::MML2ident[c]);
      end_if;

      if not contains({DOM_STRING, DOM_INT, DOM_IDENT, DOM_FLOAT, DOM_SET},
                      cType) and
        domtype(c) <> dom then
        error("wrong type")
      end_if;

      if contains({DOM_STRING, DOM_INT, DOM_IDENT, DOM_FLOAT}, cType) then
        return(c);
      end_if;

      if cType = DOM_SET then
        return(null())
      end_if;
      if nops(dom::ContentTable[dom::typString(c)]) > 1 then
        attributes := table(text2expr(dom::attributString(c)));
        return(dom::ContentTable[dom::typString(c)][2](c, attributes,
                                                       dom::infos(c)));
    // --- frankp: otherwise we would enter into an infinite loop, e.g., if c 
    // ---         is of domain type DOM_INT, and the constructor is returns
    // ---         an object of a library domain.

    // inserted  'and domtype(c) <> dom' because Franks patch broke
    // expr(Content(x!))         tonner 
/*      elif nops(c) = 1 and domtype(c) <> dom then
        subs(hold(XxX)(c),
             hold(XxX)=op(dom::ContentTable[dom::typString(c)], 1),
             Unsimplified)
      else
        subs(hold(XxX)(dom::expr(op(c,i)) $ i = 1 .. nops(c)),
             hold(XxX)=op(dom::ContentTable[dom::typString(c)], 1),
             Unsimplified) */
      else
        op(dom::ContentTable[dom::typString(c)], 1)
      end_if;
    end_proc;
  
    infos := () -> "";
    
    op := proc(c,i=-1)
            local res;
          begin
            if i = -1 then
              res := extop(c, 3..extnops(c));
              if res = FAIL then
                null()
              else
                res
              end_if;
            elif testtype(i, "_range") then
              extop(c, 2+op(i,1)..2+op(i,2))
            else
              extop(c, 2+i)
            end_if
          end_proc;

    subsop := proc(c,eq=-1)
              begin
                if type(eq) = "_equal" then
                  if domtype(op(eq, 1)) = DOM_INT then
                    extsubsop(c, 2+op(eq, 1) = op(eq, 2))
                  elif domtype(op(eq, 1)) = DOM_LIST then
                    if nops(op(eq, 1)) > 1 then
                      extsubsop(c, 2+op(eq, [1, 1]) =
                                dom::subsop(dom::op(c, op(eq, [1, 1])),
                                            subsop(eq, [1, 1] = null())))
                    elif nops(op(eq, 1)) = 1 then
                      extsubsop(c, 2+op(eq, [1, 1]) = op(eq, 2))
                    else // empty list - replace whole expression
                      op(eq, 2)
                    end_if
                  end_if
                else
                  c
                end_if
              end_proc;

    nops := c -> extnops(c) - 2;

    enclose :=
    () -> dom::Cmath([hold(xmlns)="http://www.w3.org/1998/Math/MathML"],
             dom::Csemantics(args(),
                dom::Cannotation([hold(encoding)="MuPAD"],
                                 dom::str2cdata(
                                    expr2text(dom::currentObject)
                                                )
                                 )));

    map := (x, y) -> dom::create(extop(x, 1),
                                 map(extop(x, 3..extnops(x)), y));
  
    /* output in XML notation */
    print := proc(arg1) begin dom::do_print_xml(arg1) end_proc;

    // used below in stdFunc
    useObsoleteFn := proc()
                       option escape;
                       local value;
                     begin
                       value := FALSE;  // initial setting
                       proc(x)
                         local oldVal;
                       begin
                         if args(0) = 1 then
                           if contains({TRUE, FALSE}, x) then
                             oldVal := value;
                             value := x;
                             oldVal
                           else
                             error("TRUE or FALSE expected")
                           end_if;
                         else
                           // return value
                           value
                         end_if;
                       end_proc;
                     end_proc();
    
    // print a standard function f(a,b)
    stdFunc := () -> if dom::useObsoleteFn() = TRUE then
                       dom::create("apply",
                                   dom::create("fn", dom(op(args(), 0))),
                                   op(map([op(args())], dom)));
                     else
                       dom::create("apply", dom(op(args(), 0)),
                                 op(map([op(args())], dom)));
                     end_if;

    Cmath:= () -> dom::create("math", args());
    Ccn:= () -> dom::create("cn", args());

    Cci:= () -> if contains(dom::ident2MML, args()) then
                  dom::ident2MML[args()];
                else
                  dom::create("ci", args());      // Content Identifier
                end_if;

    Capply:= () -> dom::create("apply", args(1)(), args(2..args(0)));

    // OBSOLETE
    Cfn := () -> dom::create("fn", args());  // user-defined function
    Cinterval := () -> dom::create("interval", args());
    
    Cinverse :=  () -> dom::create("inverse", args());
    Ccondition // properties of an identifier
	 := () -> dom::create("condition", args());
//  Cdeclare;   // declaration
  
   Csep :=  () -> dom::create("sep", args());       // separator
   Ccompose :=   // composition operator
           () -> dom::create("compose", args());
   Cident  :=    () -> dom::create("ident", args());// identity function
 
   // Arithmetic, Algebra and Logic
   Cquotient :=  () -> dom::create("quotient", args());
   Cexp :=       () -> dom::create("exp", args());   
   Cfactorial := () -> dom::create("factorial", args());
   Cdivide :=    () -> dom::create("divide", args());
   Cmax :=  () -> dom::create("max", args());
   Cmin :=  () -> dom::create("min", args());
   Cminus :=     () -> dom::create("minus", args());
   //kd: Cminus :=     () -> dom::create("negate", "", "", args());
   Cplus := () -> dom::create("plus", args());
   Cpower :=     () -> dom::create("power", args());
   Crem   :=     () -> dom::create("rem", args());
   Ctimes :=     () -> dom::create("times", args());
   Croot :=      () -> dom::create("root", args());
 //  Cgcd;
   Cand :=       () -> dom::create("and", args());
   Cor :=        () -> dom::create("or", args());
   Cxor :=       () -> dom::create("xor", args());
   Cnot :=       () -> dom::create("not", args());
   Cimplies :=       () -> dom::create("implies", args());
   Cequiv :=       () -> dom::create("equiv", args());
     // Cforall;  Cexists;
   Cabs :=       () -> dom::create("abs", args());
   Cconjugate :=       () -> dom::create("conjugate", args());
   Creal := () -> dom::create("real", args());
   Cimaginary := () -> dom::create("imaginary", args());
 
   // Relations
   Ceq  :=     () -> dom::create("eq", args());
   Cneq :=    () -> dom::create("neq", args());
   Cgt  :=    () -> dom::create("gt", args());
   Clt  := () -> dom::create("lt", args());
   Cgeq := () -> dom::create("geq", args());
   Cleq := () -> dom::create("leq", args());
 
   // Calculus
   Cln := () -> dom::create("ln", args());
   Clog := () -> dom::create("log", args());
   Cint := () -> dom::create("int", args());
   Cdiff := () -> dom::create("diff", args());  
   Cpartialdiff := () -> dom::create("partialdiff", args());
   Clowlimit := () -> dom::create("lowlimit", args());
   Cuplimit  := () -> dom::create("uplimit", args());
   Cbvar := () -> dom::create("bvar", args());
   Cdegree := () -> dom::create("degree", args());
   Cgrad :=  () -> dom::create("grad", args());
   Cdivergence := () -> dom::create("divergence", args());
   Ccurl :=  () -> dom::create("curl", args());
   Claplacian :=  () -> dom::create("laplacian", args());
 
   // Constants
   Cintegers := () -> dom::create("integers", args());
   Creals    := () -> dom::create("reals", args());
   Crationals := () -> dom::create("rationals", args());
   Cnaturalnumbers := () -> dom::create("naturalnumbers", args());
   Ccomplexes := () -> dom::create("complexes", args());
   Cprimes := () -> dom::create("primes", args());
   Cexponentiale := () -> dom::create("exponentiale", args());
   Cimaginaryi := () -> dom::create("imaginaryi", args());
   Cnotanumber := () -> dom::create("notanumber", args());
   Ctrue := () -> dom::create("true", args());
   Cfalse := () -> dom::create("false", args());
   Cunknown := () -> dom::create("unknown", args());
   Cemptyset := () -> dom::create("emptyset", args());
   Ceulergamma := () -> dom::create("eulergamma", args());
   Cinfinity := () -> dom::create("infinity", args());
   Cpi := () -> dom::create("pi", args());
 
     
   // Theory of sets
   Cset := () -> if args(0) = 0 then dom::Cemptyset() else 
               dom::create("set", args()) end_if;
   Clist := () -> dom::create("list", args());
   Cunion := () -> dom::create("union", args());
   CUnion := () -> dom::create("Union", args());
   Cintersect := () -> dom::create("intersect", args());
   Cin := () -> dom::create("in", args()); 
   Cnotin := () -> dom::create("notin", args());
   Csubset := () -> dom::create("subset", args());
   Cprsubset := () -> dom::create("prsubset", args());
   Cnotsubset := () -> dom::create("notsubset", args());
   Cnotprsubset := () -> dom::create("notprsubset", args());
   Csetdiff := () -> dom::create("setdiff", args());
   Ccard := () -> dom::create("card", args());
 
   // Sequences and Series
   Csum := () -> dom::create("sum", args());
                   
   Cproduct := () -> dom::create("product", args());
 
   Climit := () -> dom::create("limit", args());
 
   Ctendsto := () -> dom::create("tendsto", args());
 
   // Trigonometry
   Csin     := () -> dom::create("sin",     args());
   Ccos     := () -> dom::create("cos",     args());
   Ctan     := () -> dom::create("tan",     args());
   Csec     := () -> dom::create("sec",     args());
   Ccsc     := () -> dom::create("csc",     args());
   Ccot     := () -> dom::create("cot",     args());
   Csinh    := () -> dom::create("sinh",    args());
   Ccosh    := () -> dom::create("cosh",    args());
   Ctanh    := () -> dom::create("tanh",    args());
   Csech    := () -> dom::create("sech",    args());
   Ccsch    := () -> dom::create("csch",    args());
   Ccoth    := () -> dom::create("coth",    args());
   Carcsin  := () -> dom::create("arcsin",  args());
   Carccos  := () -> dom::create("arccos",  args());  
   Carctan  := () -> dom::create("arctan",  args());
   Carccosh := () -> dom::create("arccosh", args());
   Carccot  := () -> dom::create("arccot",  args());  
   Carccoth := () -> dom::create("arccoth", args());
   Carccsc  := () -> dom::create("arccsc",  args());
   Carccsch := () -> dom::create("arccsch", args());  
   Carcsec  := () -> dom::create("arcsec",  args());
   Carcsech := () -> dom::create("arcsech", args());
   Carcsinh := () -> dom::create("arcsinh", args());  
   Carctanh := () -> dom::create("arctanh", args());
 
   // Statistics
   Cmean := () -> dom::create("mean", args());
   CquadraticMean := () -> dom::create("quadraticMean", args());
   Csdev := () -> dom::create("sdev", args());
   Cvariance := () -> dom::create("variance", args());
   Cmedian := () -> dom::create("median", args());
   Cmode := () -> dom::create("mode", args());
   Cmoment := () -> dom::create("moment", args());
   Cmomentabout := () -> dom::create("momentabout", args());
 
   // Linear Algebra
 //  Cvector;
   Cmatrix := () -> dom::create("matrix", args());
   Cmatrixrow := () -> dom::create("matrixrow", args());
   Cdeterminant := () -> dom::create("determinant", args());  
   Ctranspose := () -> dom::create("transpose", args());
 //  Cselect;
   Cscalarproduct := () -> dom::create("scalarproduct", args()); 
   Cvectorproduct := () -> dom::create("vectorproduct", args()); 
 // Couterproduct;
 
   // Semantic Mapping Elements
   Cannotation := () -> dom::create("annotation", args()); 
   Csemantics := () -> dom::create("semantics", args());
   CannotationXml := () -> dom::create("annotationXml", args());
 
   Cexprseq := () -> dom::create("seq", args());
   Clambda := () -> dom::create("lambda", args());
   Cfnest := () -> dom::create("fnest", args());
   Cceil := () -> dom::create("ceil", args());
   CbesselI := () -> dom::create("besselI", args());
   CbesselJ := () -> dom::create("besselJ", args());
   CbesselK := () -> dom::create("besselK", args());
   CbesselY := () -> dom::create("besselY", args());
   ClambertW := () -> dom::create("lambertW", args());
   CD := () -> error("not supported");
   Cfloor := () -> dom::create("floor", args());
   Cslot :=  () -> dom::create("slot", args());
   Cdilog := () -> dom::create("dilog", args());
   Cpolylog := () -> dom::create("polylog", args());
   Cnorm := () -> if args(0) = 1 then
                    dom::create("infinitynorm", args());
                  elif args(2) = hold(Infinity) then
                    dom::create("infinitynorm", args(1));
                  elif args(2) = 1 then
                    dom::create("onenorm", args(1));
                  elif args(2) = hold(Frobenius) then
                    dom::create("norm", args(1));
                  else
                    dom::create("othernorm", args());
                  end_if;
   Cindex := () -> dom::create("index", args());
   Cbinomial := () -> dom::create("binomial", args()); 
   Crange := () -> dom::create("range", args()); 
   Cpiecewise :=  () -> dom::create("piecewise",  args());
   Cpiece :=  () -> dom::create("piece",  args());
   Ctable := () -> dom::create("table", args());
 
   // equation system;  will be translated into a set of equations
   CEQS := () -> dom::create("EQS", args());
   Ctableau := () -> dom::create("tableau", args());
   Cassign :=     () -> dom::create("assign", args());
   Crow :=        () -> dom::create("row", args());
   Cconcat :=     () -> dom::create("concat", args());
   Cfenced :=     () -> dom::create("fenced", args());
   Cgamma :=      () -> dom::create("gamma", args());
   Cigamma :=      () -> dom::create("igamma", args());

  end_domain:

// initialize identifier translation tables
map([  hold(`&NIL;`) = NIL, hold(`&FAIL;`) = FAIL, 
       hold(``) = null(), hold(`&NULL;`) = null(),
       `&pi;` = PI, output::MMLContent::Cpi() = PI,
       output::MMLContent::Cinfinity() = hold(infinity),
       output::MMLContent::Ceulergamma() = EULER,  
       output::MMLContent::Cintegers() = hold(Z_),
       output::MMLContent::Crationals() = hold(Q_),  
       output::MMLContent::Creals() = hold(R_),
       output::MMLContent::Ccomplexes() = hold(C_), 
       output::MMLContent::Cnaturalnumbers() = hold(N_),
       output::MMLContent::Cprimes() = hold(P_)], 
     output::MMLContent::translateIdents):


alias(MathML=output::MMLContent):
MathML::set("ci", FAIL, bla, x-> op(x)):  /* ! keine Simplifizierung */
MathML::set("subtract", FAIL, _subtract):  /* ! keine Simplifizierung */
MathML::set("apply", FAIL, error,
            x -> MathML::expr(op(x, 1))(map(extop(x, 3..nops(x)), MathML::expr))):
MathML::set("plus", FAIL, _plus):  /* ! keine Simplifizierung */
