// $Id: Bars2d.mu,v 1.8 2004/04/20 09:06:03 walter Exp $

/*
 plot::Bars2d -- plot statistical data, comparing groups

*/

plot::createPlotDomain("Bars2d",
                       "graphical primitive for bar plots",
                       2,
             	       [LinesVisible, LineStyle, LineWidth, LineColor, Color,
                        PointSize, PointStyle, Filled,
                        Colors,
                        [FillPatterns, ["Optional", NIL],
                          [["Style", "Surface"], "ExprSeq", FAIL,
                            "Fill patterns for the bars.", FALSE]],
                        [Data, ["Mandatory", NIL],
                         ["Definition", "ExprSeq", FAIL,
                         "Data to plot.", TRUE]],
                        [DrawMode, ["Optional", Vertical],
                          ["Style", "Prop", "TitleOrientation",
                           "Orientation of bars/boxes", TRUE]],
                        [BarStyle, ["Optional", Boxes],
                          [["Style", "Surface"], "Prop", 
                          {Boxes, Lines, Points, LinesPoints},
                          "Type of bars.", TRUE]],
                        [BarWidths, ["Optional", NIL],
                          [["Style", "Surface"], "ExprSeq", FAIL,
                          "List of widths for the bars", TRUE]],
                        [BarCenters, ["Optional", NIL],
                          [["Style", "Surface"], "ExprSeq", FAIL,
                          "List of centers for the bars", TRUE]],
                        [Shadows, ["Optional", FALSE],
                          [["Style", "Surface"], "Prop", "Bool",
                          "Shadows behind bars?", TRUE]]
                       ]):
//--------------------------------------------------------
plot::Bars2d::styleSheet := table(LinesVisible = TRUE,
                                  LineColor = RGB::Black,
                                  FillPatterns = [Solid],
                                  Colors = RGB::ColorList[1..20],
                                  BarWidths = [[1.0]]
                              //  BarCenters = [[0.0]] // Do not(!) set a default
                                 ):
//--------------------------------------------------------
plot::Bars2d::hints := {AxesInFront = TRUE, Axes = Frame}:
//--------------------------------------------------------
plot::Bars2d::new :=
  proc()
    local object, other, check;
  begin
    object := dom::checkArgs([], args());
    
    other := object::other;
    if nops(other) > 0 then
      if testtype(other[1], Type::ListOf(Type::Arithmetical)) = TRUE then
         other[1] := [other[1]];
         while nops(other) > 1 and
               testtype(other[2], Type::ListOf(Type::Arithmetical)) = TRUE do
           other[1] := other[1].[other[2]];
           delete other[2];
         end_while;
      end_if;
      if nops(other) > 1 then
         error("unexpected argument ".expr2text(other[2]));
      end_if; 
      check := dom::changeNotifier(object, "Data" = other[1]);
      if domtype(check) = DOM_STRING then
         error(check);
      end_if;
    end_if;
    
    if object::FillPatterns <> FAIL then
      check := dom::changeNotifier(object, "FillPatterns"=object::FillPatterns);
      if domtype(check) = DOM_STRING then
         error(check);
      end_if;
    end_if;
    if object::BarCenters <> FAIL then
      check := dom::changeNotifier(object, "BarCenters"=object::BarCenters);
      if domtype(check) = DOM_STRING then
         error(check);
      end_if;
    end_if;
    if object::BarWidths <> FAIL then
      check := dom::changeNotifier(object, "BarWidths"=object::BarWidths);
      if domtype(check) = DOM_STRING then
         error(check);
      end_if;
    end_if;
    if object::Data <> FAIL then
      check := dom::changeNotifier(object, "Data"=object::Data);
      if domtype(check) = DOM_STRING then
         error(check);
      end_if;
    end_if;
    
    dom::checkObject(object);
  end_proc:
//--------------------------------------------------------
plot::Bars2d::print := b -> hold(plot::Bars2d)(b::Data):
//--------------------------------------------------------
plot::Bars2d::changeNotifier :=
  proc(b: plot::Bars2d, newval)
    local slotname, value, v, c, i, j;
  begin
    [slotname, value] := [op(newval)];
    if value <> NIL then
      case slotname
         of "FillPatterns" do
           if domtype(value) <> DOM_LIST or nops(value)<1 then
             return("attribute FillPatterns requires a non-empty list");
           end_if;
           if not {op(value)} subset plot::attributes[FillPattern][3] then
             return("invalid FillPatterns: ".expr2text(op({op(value)} minus plot::attributes[FillPattern][3])));
           end_if;
           break;
         of "BarCenters" do
           if domtype(float(value)) = DOM_FLOAT then
              value := [[value]];
           end_if;
           if domtype(value) <> DOM_LIST then
              return("attribute BarCenters requires a list"):
           end_if;
           if domtype(value[1]) <> DOM_LIST then
              value:= [value]:
           end_if;
           dom::extslot(b, "BarCenters", value);
           return(FALSE);
           break;
         of "BarWidths" do
           if domtype(float(value)) = DOM_FLOAT then
              value := [[value]];
           end_if;
           if domtype(value) <> DOM_LIST then
              return("attribute BarWidths requires a list"):
           end_if;
           if domtype(value[1]) <> DOM_LIST then
              value:= [value]:
           end_if;
           dom::extslot(b, "BarWidths", value);
           return(FALSE);
           break;
         of "Data" do
           if value::dom::hasProp(Cat::Matrix) = TRUE then
             value := coerce(value, DOM_ARRAY);
           end_if;
           if domtype(value) = DOM_ARRAY then
             case op(value, [0,1])
               of 1 do
                  value := [coerce(value, DOM_LIST)];
                  break;
               of 2 do
                  value := [[value[i,j]$j=op(value, [0,3])]$i=op(value, [0,2])];
                  break;
               otherwise
                  return("bad array dimension for Data");
             end_case;
           end_if;
           if testtype(value, Type::ListOf(Type::Arithmetical))=TRUE then
             value := [value];
           end_if;
           if domtype(value) <> DOM_LIST then
             return("wrong type of value for Data");
           end_if;
           if nops(map({op(value)}, nops)) > 1 then
             return("Data must be list of lists of same length");
           end_if;
           for v in value do
             c := plot::checkList(dom, "Value", v);
             if c::dom = DOM_STRING then
               return("Invalid entry in Data: ".expr2text(v));
             end_if;
           end_for;
           dom::extslot(b, "Data", value);
           // hint
           return(FALSE);
           break;
      end_case;
    end_if;
    return(TRUE);
  end_proc:
//--------------------------------------------------------
plot::Bars2d::MuPlotML :=
  proc(b: plot::Bars2d, attributes, inheritedAttributes)
    local groups, colors, patterns,
           i, j, pattern, color, values, swap, 
           doDraw, prP2, barwidth,
           shadowwidth, shadowheight, allvalues,
           barsep, groupsep, shadowcolor,
           bbs, centers, widths, lastcenters, xmin, xmax;
  begin
    groups   := attributes[Data];
    patterns := attributes[FillPatterns];
    colors   := attributes[Colors];
    shadowcolor := RGB::LightGrey;
    barwidth  := attributes[BarWidths][1][1];

    if attributes[Shadows] = TRUE and 
       attributes[BarStyle] = Boxes then
         allvalues := {op(map(groups, op))};
         shadowwidth  := 0.10 * barwidth;
         shadowheight := 0.10 * (max(0, op(allvalues)) - min(0, op(allvalues))) / 
                         (nops(groups) * nops(groups[1]));
         barsep      := 0.05 * barwidth;
         // we usually have an aspect ratio of approx 2/3
         if attributes[DrawMode] = Horizontal then
            shadowwidth := 1.5*shadowwidth;
            shadowheight := 0.66*shadowheight;
         end_if;
    else
         shadowwidth  := 0.0;
         shadowheight := 0.0;
         barsep       := 0.0;
    end_if;
    if nops(groups) = 1 then
         groupsep:= 0.0:
    else groupsep:= 0.3*barwidth;
    end_if:
    
    if attributes[DrawMode] = Horizontal then
         swap := (x, y) -> (y, x);
    else swap:= id;
    end_if;

    bbs := barwidth + barsep + shadowwidth:
    //---------------------------------
    //---------------------------------
    // Widths
    //---------------------------------
    //---------------------------------
    if contains(attributes, BarWidths) then
         widths:= float(attributes[BarWidths]);
    else widths:= [[]]; // default values are inserted below
    end_if;
    //--------------------------------
    // set default widths if necessary
    //--------------------------------
    for i from 1 to nops(groups) do
        if i > nops(widths) then
           widths:= widths.[[barwidth $ nops(groups[i])]];
        end_if;
        if nops(widths[i]) < nops(groups[i]) then
           widths[i]:= widths[i].[barwidth $ nops(groups[i]) - nops(widths[i])];
        end_if;
    end_for:
    //---------------------------------
    //---------------------------------
    // Centers
    //---------------------------------
    //---------------------------------
    if contains(attributes, BarCenters) then
         centers:= float(attributes[BarCenters]);
    else centers:= [[]]; // default values are inserted below
    end_if:
    //---------------------------------
    // set default centers if necessary
    //---------------------------------
    if nops(centers) = 1 and nops(centers[1]) = 0 then
       centers := [ [(j-1)*(nops(groups)*bbs + groupsep)
                    +(i-1)*bbs 
                    $ j = 1..nops(groups[i])
                  ] $ i = 1..nops(groups)];
    end_if;
    for i from 1 to nops(groups) do
        if i > nops(centers) then
           if i = 1 then
              lastcenters:= [(j-1)*(nops(groups)*bbs + groupsep) $ $ j = 1..nops(groups[i])]:
           else
              lastcenters:= centers[i - 1]
           end_if;
           centers:= centers.[map(lastcenters, _plus, bbs)];
        end_if;
        if nops(centers[i]) < nops(groups[i]) then
           centers[i]:= centers[i].[centers[i][nops(centers[i])] + j*(nops(groups)*bbs + groupsep)
                         $ j = 1 .. nops(groups[i]) - nops(centers[i])];
        end_if;
    end_for:
    //---------------------------------
    // find ViewingBox
    //---------------------------------
    xmin:= centers[1][1]:
    xmax:= centers[1][1]:
    for i from 1 to nops(groups) do
     for j from 1 to nops(groups[i]) do
         xmin:= min(xmin, centers[i][j] - widths[i][j]/2):
         xmax:= max(xmax, centers[i][j] + widths[i][j]/2 + shadowwidth):
     end_for;
    end_for:
    //---------------------------------
    // prepare plotting
    //---------------------------------
    prP2 := (x,y) -> plot::MuPlotML::prP2(swap(float(x),float(y)));
    
    // doDraw(<nr of group>, <nr in group>, <height>, <pattern>, <color>)
    doDraw := proc(i, j, v, pattern, color)
                  local c, r, l;
               begin
                  c := centers[i][j];
                  l := c - widths[i][j]/2;
                  r := c + widths[i][j]/2;
                  if attributes[Shadows] then
                    plot::MuPlotML::beginElem("Poly2d",
                                              "Closed" = TRUE,
                                              "FillPattern" = Solid,
	                                      "FillColor" = shadowcolor,
                                              "LinesVisible" = FALSE);
                    if v > 0 then
                      prP2(r,0);
                      prP2(r,v);
                      prP2(l+shadowwidth, v);
                      prP2(l+shadowwidth, v+shadowheight);
                      prP2(r+shadowwidth, v+shadowheight);
                      prP2(r+shadowwidth, 0);
                    else
                      prP2(l+shadowwidth, shadowheight);
                      prP2(r+shadowwidth, shadowheight);
                      prP2(r+shadowwidth, min(0, v+shadowheight));
                      prP2(r, min(0, v+shadowheight));
                      prP2(r, 0);
                      prP2(l+shadowwidth, 0);
                    end_if;
                    plot::MuPlotML::endElem("Poly2d");
                  end_if;
                  plot::MuPlotML::beginElem("Poly2d",
                                            "Closed"=TRUE,
                                            "FillPattern"=pattern,
                                            "FillColor"=color);
                  prP2(l, 0);
                  prP2(l, v);
                  prP2(r, v);
                  prP2(r, 0);
                  plot::MuPlotML::endElem("Poly2d");
               end_proc:
    
    if attributes[BarStyle] = Lines then
      doDraw := proc(i, j, v, pattern, color)
                    local c;
                  begin
                    c := centers[i][j];
                    plot::MuPlotML::beginElem("Poly2d",
                                              "Closed"=FALSE,
                                              "LinesVisible"=TRUE,
                                              "LineColor"=color);
                    prP2(c, 0);
                    prP2(c, v);
                    plot::MuPlotML::endElem("Poly2d");
                  end_proc:
    elif attributes[BarStyle] = Points then
      doDraw := proc(i, j, v, pattern, color)
                    local c;
                  begin
                    c := centers[i][j];
                    plot::MuPlotML::beginElem("Pt2d",
                                              "PointsVisible"=TRUE,
                                              "PointColor"=color,
                                              "Position"=(swap(float(c),
                                                               float(v))),
                                              Empty);
                  end_proc:
    elif attributes[BarStyle] = LinesPoints then
      doDraw := proc(i, j, v, pattern, color)
                    local c;
                  begin
                    c := centers[i][j];
                    plot::MuPlotML::beginElem("Poly2d",
                                              "Closed"=FALSE,
                                              "LinesVisible"=TRUE,
                                              "LineColor"=color);
                    prP2(c, 0);
                    prP2(c, v);
                    plot::MuPlotML::endElem("Poly2d");
                    plot::MuPlotML::beginElem("Pt2d",
                                              "PointColor"=color,
                                              "PointsVisible"=TRUE,
                                              "Position"=(swap(float(c),
                                                               float(v))),
                                              Empty);
                  end_proc:
    end_if;
    
    //--------------------------------
    // doDraw
    //--------------------------------
    for i from 1 to nops(groups) do
      pattern := patterns[modp(i-1, nops(patterns))+1];
      color := float(colors[modp(i-1, nops(colors))+1]);
      values := float(groups[i]);
      for j from 1 to nops(values) do
        doDraw(i, j, values[j], pattern, color);
      end_for;
    end_for;
    //--------------------------------
    // return ViewingBox
    //--------------------------------
    [swap(xmin..xmax,
          float(min(0, op(map(map(groups, op), `+`, -shadowheight))))
       .. float(max(0, op(map(map(groups, op), `+`,  shadowheight)))))];
  end_proc:
