// $Id: Conformal.mu,v 1.5 2004/09/29 09:18:14 tonner Exp $

plot::createPlotDomain("Conformal",
		       "graphical primitive for conformal plots",
		       2,
		       [LineStyle, LineWidth,
			ZName, ZMin, ZMax, ZRange,
			XMesh, YMesh, Mesh, 
			XSubmesh, YSubmesh, Submesh,
			PointsVisible, AdaptiveMesh,
			LineColor, Color, LineColor2, LineColorType,
			Function,
         LineColorDirection, LineColorDirectionX, LineColorDirectionY]):

plot::Conformal::styleSheet := table(LegendEntry = TRUE,
                                     XMesh = 11, YMesh = 11):

plot::Conformal::new :=
  proc()
    local object, other;
  begin
    object := dom::checkArgs(["Z"], args());
    
    other := object::other;
    
    if nops(other) > 1 then
      error("unexpected argument ".expr2text(other[2]));
    end_if;
    if nops(other) > 0 then
      object::Function := other[1];
    end_if;
    
    dom::checkObject(object);
  end_proc:
  
plot::Conformal::print :=
  proc(c)
  begin
    hold(plot::Conformal)(c::Function, c::ZName = c::ZRange)
  end_proc:

plot::Conformal::MuPlotML :=
  proc(c: plot::Conformal, attributes, inheritedAttributes)
    local ziv, z, i, xmin, xmax, ymin, ymax, cleaned_attribs,
	  f, umesh, vmesh, vbox;
  begin
    cleaned_attribs := 
    plot::MuPlotML::fixAttributes(attributes, plot::Curve2d);
    
    z := attributes[ZName];
    ziv := hull(attributes[ZMin], attributes[ZMax]);
    [xmin, xmax] := [op(Re(ziv))];
    [ymin, ymax] := [op(Im(ziv))];
    f := subs(c::Function, attributes[ParameterName]=attributes[ParameterValue], Unsimplified);
    f := fp::unapply(f, z);
    umesh := attributes[XMesh] - 1;
    vmesh := attributes[YMesh] - 1;
    
    vbox := {};
    
//    if attributes[XLinesVisible] <> FALSE then
    for i from 0 to umesh do
      vbox := vbox union
	     {plot::MuPlotML(plot::Curve2d([Re::lazy, Im::lazy]
					   (f((umesh-i)/umesh*xmin+
					      i/umesh*xmax+
					      z*I)),
					   z=ymin..ymax,
					   op(cleaned_attribs),
					   UMesh=attributes[YMesh],
					   USubmesh=attributes[YSubmesh]
					  ),
			     inheritedAttributes, Raw)};
    end_for;
//    end_if;
    
//    if attributes[VLinesVisible] <> FALSE then
    for i from 0 to vmesh do
      vbox := vbox union
	     {plot::MuPlotML(plot::Curve2d([Re::lazy, Im::lazy]
					   (f(z + 
					      ((vmesh-i)/vmesh*ymin+
					       i/vmesh*ymax)*I)),
					   z=xmin..xmax,
					   op(cleaned_attribs),
					   UMesh=attributes[XMesh],
				      Submesh=attributes[XSubmesh]
					  ),
			     inheritedAttributes, Raw)};
    end_for;
//    end_if;

//    [min(op(map(vbox, op, [1,1])))..max(op(map(vbox, op, [1,2]))),
//     min(op(map(vbox, op, [2,1])))..max(op(map(vbox, op, [2,2])))]
    
    plot::MuPlotML::commonViewingBox(vbox);
    
  end_proc:
  
