// plot::Cylindrical -- surfaces in cylindrical coordinates

plot::createPlotDomain("Cylindrical",
                       "graphical primitive for surfaces in cylindrical coordinates",
                       3,
                       [XFunction, YFunction, ZFunction, 
                        UName, UMin, UMax, URange, VName, VMin, VMax, VRange,
                        UMesh, VMesh, Mesh, USubmesh, VSubmesh, Submesh, AdaptiveMesh,
                        Filled, FillColor, Color, FillColor2, FillColorType, FillColorFunction,
                        LineWidth, LineStyle, MeshVisible,
                        LineColor, LineColor2,
                        LineColorType, LineColorFunction,
                        PointSize, PointStyle, PointsVisible, 
			ULinesVisible, VLinesVisible,
			XContours, YContours, ZContours,
    LineColorDirectionX, LineColorDirectionY, LineColorDirectionZ,
    LineColorDirection, FillColorDirection,
    FillColorDirectionX, FillColorDirectionY, FillColorDirectionZ
                       ]):

//------------------------------------------------------------------------
// Hints for parents:
plot::Cylindrical::hints := {Scaling = Constrained}:

plot::Cylindrical::styleSheet := table(LegendEntry = TRUE,
                                       LineColor = RGB::Black.[0.25],
                                       MeshVisible = FALSE,
                                       LineColorDirectionY=0):
//------------------------------------------------------------------------

plot::Cylindrical::new :=
  proc()
    local object, other;
  begin
    object := dom::checkArgs(["U", "V"], args());    
    other := object::other;
    
    if nops(other) > 0 then
      if nops(other) > 1 then
        error("unexpected argument ".expr2text(other[2]));
      end_if;
      if domtype(other[1]) <> DOM_LIST then 
        if nops(other[1]) <> 3 then
          error("expecting a list of 3 expressions or a matrix");
        elif (other[1])::dom::hasProp(Cat::Matrix) = TRUE then 
          other[1]:= [op(other[1])];
        end_if;          
      end_if;
      object::XFunction := other[1][1];
      object::YFunction := other[1][2];
      object::ZFunction := other[1][3];
    end_if;
    
    dom::checkObject(object);
  end_proc:

//--------------------------------------------------------
plot::Cylindrical::print := p -> hold(plot::Cylindrical)(
        [p::XFunction, p::YFunction, p::ZFunction],
        p::UName = p::URange, p::VName = p::VRange):
//--------------------------------------------------------
plot::Cylindrical::MuPlotML :=
  proc(p, attributes, inheritedAttributes)
    local x, y, z, u, v, cleaned_att;
  begin
    u := attributes[UName];
    v := attributes[VName];
    x := attributes[XFunction];
    y := attributes[YFunction];
    z := attributes[ZFunction]; 
    cleaned_att := plot::MuPlotML::fixAttributes(attributes, plot::Surface);
    cleaned_att[XFunction] := x * (cos@y);
    cleaned_att[YFunction] := x * (sin@y);
    cleaned_att[ZFunction] := z;
    if contains(cleaned_att, LineColorFunction) then
      cleaned_att[LineColorFunction] := 
      subs(((u, v, x, y, z) -> `#f`(u, v,
                                    `#r`(u, v),
                                    `#phi`(u, v),
                                    z,
                                    args(3..args(0)))),
           [`#f` = cleaned_att[LineColorFunction],
            `#r` = attributes[XFunction],
            `#phi` = attributes[YFunction]]);
    end_if;
    if contains(cleaned_att, FillColorFunction) then
      cleaned_att[FillColorFunction] := 
      subs(((u, v, x, y, z) -> `#f`(u, v,
                                    `#r`(u, v),
                                    `#phi`(u, v),
                                    z,
                                    args(3..args(0)))),
           [`#f` = cleaned_att[FillColorFunction],
            `#r` = attributes[XFunction],
            `#phi` = attributes[YFunction]]);
    end_if;
    plot::MuPlotML(plot::Surface(op(cleaned_att)), inheritedAttributes, Raw):
  end_proc:
//--------------------------------------------------------
