// $Date: 2004/09/23 13:00:51 $  $Author: tonner $   $Revision: 1.8 $

/* -----------------------------------------------------------
    Plane -- The graphical primitive for 3D planes

    Syntax:
    plot::Plane(Point, Normal <, op1, op2, ...>)

    Point, Normal         : lists of 3 expressions
    op1, op2, ...         : options of the form 'option = value'

    Example:
    >> plot::Plane([1, 2, 3], [4, 5, 6])

    >> plot::Plane(matrix([[1,4],[2, 5],[3,6]]))

    >> plot::Plane([1, 2, 3], [4, 5, 6], FillColor = RGB::Pink)
----------------------------------------------------------------*/ 

plot::createPlotDomain("Plane",
                       "graphical primitive for 3D planes",
                       3,
                       [PositionX, PositionY, PositionZ, Position,
                        NormalX, NormalY, NormalZ, Normal, 
                        UMesh, VMesh, Mesh, 
                        LinesVisible, LineColor,
                        FillColor
                       ]):

//-----------------------------------------------------------------------------
plot::Plane::styleSheet:= table(
      PositionX = 0,
      PositionY = 0,
      PositionZ = 0,
      NormalX = 0,
      NormalY = 0,
      NormalZ = 1,
      UMesh = 15, 
      VMesh = 15, 
      LinesVisible = TRUE,
      LineColor = RGB::Black.[0.25], // konsistent with Surface
      FillColor = RGB::LightBlue    // konsistent with Sphere, Parallelogram etc.
):
//-----------------------------------------------------------------------------
// methods yet to be implemented:  convert, convert_to, expr
//-----------------------------------------------------------------------------
// here we have to do something
plot::Plane::new:=
  proc()
    local object, other;
  begin
    // check all known options from the argument list
    object := dom::checkArgs([], args());
    
    // get arguments which are not yet processed
    other := object::other;
    
    if testargs() and nops(other) > 0 then
      if not testtype(other[1],
                      Type::ListOf(Type::Arithmetical, 3, 3)) and
        not (other[1])::dom::hasProp(Cat::Matrix)=TRUE then
        error("expecting a list of 3 expressions as 1st argument")
      end_if;
      
      if nops(other) > 1 and
        not testtype(other[2],
                     Type::ListOf(Type::Arithmetical, 3, 3)) and
        not (other[2])::dom::hasProp(Cat::Matrix)=TRUE then
        error("expecting a list of 3 expressions as 2nd argument")
      end_if;

      if nops(other) > 2 then
       error("unexpected argument: ".expr2text(other[3]));
      end_if;
    end_if;
    
    if nops(other) > 0 then
      if (other[1])::dom::hasProp(Cat::Matrix)=TRUE then
         other[1]:= [op(other[1])];
      end_if;
      if nops(other[1]) = 6 then
         if nops(other) > 1 then
            error("1st and 2nd argument: conflicting input for attribute 'Normal'"):
         else
            other:= [ [other[1][1],other[1][3],other[1][5]],
                      [other[1][2],other[1][4],other[1][6]] ]:
         end_if;
      end_if;
      object::Position:= other[1];
      if nops(other) > 1 then
         if (other[2])::dom::hasProp(Cat::Matrix)=TRUE then
            other[2]:= [op(other[2])];
         end_if;
        object::Normal  := other[2];
      end_if;
    end_if;
    
    // semantically check for validity
    dom::checkObject(object);
  end_proc:

plot::Plane::print :=
  obj -> hold(plot::Plane)(obj::Position, obj::Normal):

plot::Plane::MuPlotML:=
  proc(obj, attrib)
  local fx, fy, fz, tx, ty, tz;
  begin
    [fx,fy,fz]:= float([attrib[PositionX],attrib[PositionY],attrib[PositionZ]]);
    [tx,ty,tz]:= float([attrib[NormalX],attrib[NormalY],attrib[NormalZ]]);

   plot::MuPlotML::beginElem("Plane",
                             "Position" = (fx, fy, fz),
                             "Normal" =   (tx, ty, tz),
                             Empty);
   return([fx..fx,fy..fy,fz..fz]);
  end_proc:
