// $Date: 2004/06/30 11:15:51 $  $Author: walter $   $Revision: 1.4 $

/* -----------------------------------------------------------
    PointList2d -- an opaque list of 2D points

    Syntax:
    PointList2d(f, var=a..b, <, op1, op2, ...>)

    f, a, b               : arithmetical expressions
    op1, op2, ...         : options of the form 'option = value'

    Example:
    >> plot::PointList2d([[1, 2], [2, 3]])

    >> plot::PointList2d([[1, 2, [0.5, 0.5, 0.5]], [2, 3, [1.0, 0.0, 0.1]]],
                         PointStyle = XCrosses)
----------------------------------------------------------------*/ 

plot::createPlotDomain("PointList2d",
                       "an opaque list of 2D points",
                       2,  // Dimension
   [Points2d, PointStyle, PointSize, PointColor, Color]):

//-------------------------------------------
plot::PointList2d::new :=
  proc()
    local object, other;
  begin
    object := dom::checkArgs([], args());
    
    other := object::other;
  
    if nops(other) = 1 then
      // object::Points2d := other[1];
      dom::changeNotifier(object, "Points2d" = other[1]);
    elif nops(other) <> 0 then
      error("unexpected argument: ".expr2text(other[2]));
    end_if;

    dom::checkObject(object);
end_proc:
//-------------------------------------------
plot::PointList2d::changeNotifier :=
  proc(object, eq)
    local slotName, newval, l, i, j;
  begin
    [slotName, newval] := [op(eq)];
    case slotName
      of "Points2d" do
        l := newval;
        if l::dom::hasProp(Cat::Matrix) = TRUE then
           l:= expr(l);
        end_if;
        if domtype(l) = DOM_ARRAY then
           if op(l, [0, 1]) <> 2 then 
              error("expecting an array/matrix with 2 columns");
           end_if;
           if op(l, [0, 3, 2]) = op(l, [0, 3, 1]) then
              error("expecting an array/matrix with 2 columns");
           end_if;
           l:=[ [l[i,j] $ j = op(l, [0,3,1])..op(l, [0,3,1]) + 1]
                        $ i = op(l, [0,2,1])..op(l, [0,2,2])];
        end_if;
        dom::extslot(object, "Points2d", l);

        return(FALSE); // we've done all there is to do
        break;
    end_case;
    return(TRUE);
end_proc:

//-------------------------------------------
plot::PointList2d::print :=
  obj -> if nops(obj::Points2d) < 10 then
           hold(plot::PointList2d)(obj::Points2d):
         else
           hold(plot::PointList2d)():
         end_if:
//-------------------------------------------
plot::PointList2d::MuPlotML:=
  proc(object, attributes)
    local coords, i, xmin, xmax, ymin, ymax, points;
  begin
    points := float(attributes[Points2d]);
   
    plot::MuPlotML::beginElem("Pts2d", "PointsVisible"=TRUE);
  
    for i in points do
      plot::MuPlotML::prP2(op(i));
    end_for;
    
    plot::MuPlotML::endElem("Pts2d"):
    
    coords := map(points, op, 1);
    xmin := min(op(coords)):
    xmax := max(op(coords)):
    coords := map(points, op, 2);
    ymin := min(op(coords)):
    ymax := max(op(coords)):
  
    [xmin..xmax, ymin..ymax]
  end_proc:
