// $Id: Riemann.mu,v 1.2 2004/06/14 11:16:44 bij Exp $

/*
 plot::Riemann -- plot riemann quadrature

*/

plot::createPlotDomain("Riemann",
                       "graphical primitive for riemann quadrature",
                       2,
             	       [// function ==================================
                        Function, XName, XMin, XMax, XRange, XMesh, XSubmesh, Mesh, Submesh,
                        LineColor, LineColor2, LineWidth, LineStyle, LinesVisible,
                        LineColorType, LineColorFunction, PointSize, PointStyle, PointsVisible,
                        Color,
                        [Nodes, ["Optional", NIL],
                         ["Definition", "ExprSeq", FAIL,
                         "Number of rectangles or list of nodes.", TRUE]],
                        [LeftY, ["Optional", FALSE],
                         ["Definition", "Prop", "Bool",
                         "Height of each rectangle to the functions leftpoint in interval.",
                          TRUE]],
                        [RightY, ["Optional", FALSE],
                         ["Definition", "Prop", "Bool",
                         "Height of each rectangle to the functions rightpoint in interval.",
                         TRUE]],
                        [MiddleY, ["Optional", FALSE],
                         ["Definition", "Prop", "Bool",
                         "Height of each rectangle to the functions middlepoint in interval.",
                         TRUE]],
                        [Upper, ["Optional", FALSE],
                         ["Definition", "Prop", "Bool",
                         "Height of each rectangle to the functions maximum in interval.",
                         TRUE]],
                        [Lower, ["Optional", FALSE],
                         ["Definition", "Prop", "Bool",
                         "Height of each rectangle to the functions minimum in interval.",
                         TRUE]],
                        // rectangles ================================
                        Colors, FillPattern, FillColor, Filled,
                        // text ======================================
                        [TextPosition, ["Optional", NIL],
                         [["Definition", "Text"], "ExprSeq", FAIL,
                         "Position of the text.", TRUE]],
                        //Text, // the Text of the object
                        [TextVisible, ["Optional", TRUE],
                         [["Definition", "Text"], "Prop", "Bool",
                         "Display area information.",
                         TRUE]],
                        [Digits, ["Optional", NIL],
                         [["Definition", "Text"], "Expr", FAIL,
                         "Digits of values.", TRUE]],
                        [Steps, ["Optional", FALSE],
                         [["Definition", "Text"], "Prop", "Bool",
                         "Show number of steps?", TRUE]],
                        TextFont,
                        TextRotation,
                        HorizontalAlignment, VerticalAlignment
                       ]):
//--------------------------------------------------------
plot::Riemann::styleSheet := table(LinesVisible = TRUE,
                                   LineColor = RGB::Black,
                                   FillPattern = Solid,
                                   Colors = [[0.85, 0.85, 1.0], [1.0, 0.95, 0.4],
                                             [1.0, 0.45, 0.45], [1.0, 0.7, 0.65],
                                             [1.0, 0.85, 0.8]],
                                   Digits = 2,
                                   hold(Nodes) = [10],
                                   XMesh = plot::getDefault(plot::Function2d::XMesh),
                                   Mesh = plot::getDefault(plot::Function2d::Mesh),
                                   // text
                                   HorizontalAlignment = Left,
                                   VerticalAlignment = Bottom,
                                   TextPosition = [infinity, infinity] // tag for not set
                                   //Text = "" // tag for not set
                                   ):
//--------------------------------------------------------
plot::Riemann::hints := {AxesInFront = TRUE, Axes = Origin}:
//--------------------------------------------------------
plot::Riemann::new :=
  proc()
    local object, other;
  begin
    object := dom::checkArgs(["X"], args());
    
    other := object::other;
    if nops(other) > 0 then
      if testtype(other[1], Type::Arithmetical) or
         testtype(other[1], Type::Function) then
        object::Function := other[1]
      else
        error("1st argument: arithmetical expression expected")
      end_if
    end_if;
    
    dom::checkObject(object);
  end_proc:
//--------------------------------------------------------
plot::Riemann::print := b -> hold(plot::Riemann)(b::Function, b::XName = b::XRange):
//--------------------------------------------------------
plot::Riemann::MuPlotML :=
  proc(obj: plot::Riemann, attributes, inheritedAttributes)
    local f, fterm, z, a, b, N, x, tmp, i, n, k, pos, area, text, addtext,
          color, linecolor, pvalues, values;
  begin
    // 
    f := attributes[hold(Function)];
    z := attributes[hold(XName)];
    a := attributes[hold(XMin)];
    b := attributes[hold(XMax)];
    
    // 
    if domtype(f) <> DOM_PROC then
      fterm := f;
      //f := X -> eval(subs(fterm, z = X));
      f := fp::unapply(f)
    end_if;
    
    N := attributes[hold(Nodes)];
    if nops(N) = 1 then // only one node given
      N := op(N)
    end_if;
    x:= table();
    
    // x-ranges for rectangles
    if domtype(N) = DOM_LIST then
      N := sort(float(N));
      //if float(a) > N[1] or float(b) < op(N, nops(N)) then
      //  error("range must contain all elements of the node set");
      //end_if;
      
      tmp := [a, op(N), b];
      N := nops(tmp) - 1;
      (x[i-1] := tmp[i]) $ i = 1..N+1
    else
      N := ceil(N); // if N is animati
      (x[i] := a + i * (b-a)/N) $ i=0..N
    end_if:

    // text position
    if attributes[hold(TextPosition)] = [infinity, infinity] then
      tmp := eval(f(a...b) union 0...0); // rectangles
      // y
      if max(tmp) <= 0 then // prevent collision with x axis
        tmp := (11*max(tmp)-min(tmp))/10;
      else
        tmp := (22*max(tmp)-min(tmp))/20;
      end_if;
      // x
      if float(a) <= 0 and float(b) > 0 and float(-a/b) < 0.55 then // prevent collision with y axis
        tmp := [(b - a)/20, tmp];
      else
        tmp := [a, tmp] //[(19*a+b)/20, tmp];
      end_if;
      pos := tmp
    else
      pos := attributes[hold(TextPosition)]
    end_if;

    // area of rectanlges
    area := values -> _plus((x[i]-x[i-1])*_plus(op(values[i])) $ i = 1..N);

    text := "";
    addtext := (s, v) -> (text := text."\n".prog::getname(s).": ".v);

    pvalues :=
      proc(opt)
        local i;
      begin
        case opt
          of hold(Upper) do
            map([eval(f(x[i]...x[i+1]))$i=0..N-1],
                n -> if sign(n) = 1...1 then // positive interval
                       [0, max(n)]
                     elif sign(n) = -1...-1 then // negative interval
                       [0, min(n)]
                     else // both
                       [op(n)]
                     end_if);
            break
          of hold(MiddleY) do
            [[0, f((x[i]+x[i+1])/2)]$i=0..N-1];
            break
          of hold(RightY) do
            [[0, f(x[i+1])]$i=0..N-1];
            break
          of hold(LeftY) do
            [[0, f(x[i])]$i=0..N-1];
            break
          of hold(Lower) do
            map([eval(f(x[i]...x[i+1]))$i=0..N-1],
                n -> if sign(n) = 1...1 then // positive interval
                       [0, min(n)]
                     elif sign(n) = -1...-1 then // negative interval
                       [0, max(n)]
                     else // zero
                       [0, 0]
                     end_if);
            break
        end_case
      end_proc;
                 

    
    // DRAW and return common viewing box
    plot::MuPlotML::commonViewingBox
    ({
      // rectangles
      //////////////////////////////////////////////////////////////////////
      (n := 0; // index for color from Colors
       ( // one group of rectangles
        if attributes[k] = TRUE then
          color := attributes[hold(Colors)][(n mod (nops(attributes[hold(Colors)])))+1];
          //linecolor := attributes[hold(LineColor)][(n mod (nops(attributes[hold(Colors)])))+1];
          n := n + 1;
          linecolor := [0.4, 0.4, 0.4]; // attributes[hold(LineColor2)];
          values := pvalues(k);
          addtext(k, stringlib::formatf(float(area(values)), attributes[hold(Digits)]));

          // rectangles by hand - options problems
          (plot::MuPlotML::beginElem("Poly2d",
                                     "Closed"=TRUE,
                                     "FillColor" = color,
                                     //"LineColor" = linecolor,
                                     //"FillPattern" = "Solid",
                                     "LineWidth" = 0.1);
           plot::MuPlotML::prP2(float(x[i-1]), float(min(values[i])));
           plot::MuPlotML::prP2(float(x[i-1]), float(max(values[i])));
           plot::MuPlotML::prP2(float(x[i]), float(max(values[i])));
           plot::MuPlotML::prP2(float(x[i]), float(min(values[i])));
           plot::MuPlotML::endElem("Poly2d"))
          $ i = 1..N;
          null()
        else
          null()
        end_if) $ k in [hold(Upper), hold(MiddleY), hold(LeftY), hold(RightY), hold(Lower)]),

      // function
      plot::MuPlotML(plot::Function2d(f, z = a..b,
                                      op(plot::MuPlotML::fixAttributes(attributes, plot::Function2d))),
                     inheritedAttributes,
                     Raw),
      
      // text
      (if attributes[hold(TextVisible)] = TRUE then
         plot::MuPlotML(plot::Text2d((// text definition
                                      "Intergral: ".
                                      stringlib::formatf(numeric::int(f(z), z = a..b),
                                                         attributes[hold(Digits)]).
                                      text.
                                      (if attributes[hold(Steps)] = TRUE then
                                         "\nSteps: ".expr2text(N)
                                       else
                                         ""
                                       end_if)
                                      ),
                                     // default position
                                     pos,
                                     op(plot::MuPlotML::fixAttributes(attributes, plot::Text2d))
                                     ),
                        inheritedAttributes, Raw)
       else
         null()
       end_if
       )})
    
  end_proc:
