
/*

refineFactorization(l)

takes a list [a1, n1, .., ak, nk]
where the ai are arithmetical expressions and the ni are integers
denoting an object a = a1^n1 * ... * ak^nk


returns a list of the same format, denoting the same a,
but satisfying in addition gcd(ai, aj) for i \neq j 

*/



polylib::refineFactorization:=
proc(l: DOM_LIST, gcdfunc): DOM_LIST
  local i: DOM_INT,
        j: DOM_INT,
        g;
  
  
begin
  if args(0) = 1 then
    gcdfunc:= (x,y) -> gcd(numer(x), numer(y))/ gcd(denom(x), denom(y))
  end_if;
  
  i:=1;
  while i <= nops(l) - 2 do
    // take care that every j > i satisfies gcd(ai, aj) = 1
    // divide off the gcd if necessary
    j:= i + 2;
    while j < nops(l) and l[i]<>1 do
      g:= gcdfunc(l[i], l[j]);
      if g<>1 then
        l[i]:= normal(l[i]/g);
        l[j]:= normal(l[j]/g);
        l:= l.[g, l[i+1] + l[j+1]];
      end_if;
      j:= j+2    
    end_while;
    i:= i+2
  end_while;

  i:=1;
  while i <= nops(l) do
    if l[i] = 1 then
      // delete l[i] and its multiplicity
      delete l[i]; delete l[i];
      i:= i-2;
    end_if;
    i:= i+2;
  end_while;
  
  l

end_proc:

