/*****************************************************************************
  $Id: bless.mu,v 1.6 2004/09/26 00:10:28 nthiery Exp $
  File:		lib/PROG/bless.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Created:	16/02/2001
  License:	LGPL
*****************************************************************************/

/**
prog::bless -- utility for preparing test cases

prog::bless(f, A)

Bless f as a method of the domain A; that is set dom inside f to A.

This has the same effect as assigning f to A for procs.

**/

prog::bless :=
// Modif : Florent to allows for blessing f@g...
//         kd enabled blessing of func-envs...
proc(f: Type::Union(DOM_PROC, DOM_EXPR, DOM_FUNC_ENV), DOM: DOM_DOMAIN)
    option noDebug;
begin
    case domtype(f)
    of DOM_PROC do
        subsop(f, 13 = DOM);
        break;
    of DOM_FUNC_ENV do
        subsop(f, 1 = prog::bless(op(f, 1), DOM));
        break;
/*	Disabled because unsafe.
	See email-discussion, mupad-combinat-devel 16/02/2004
      of DOM_EXPR do
        if op(f, 0) = hold(_fconcat) then
            map(f, prog::bless, DOM);
            break;
        end_if;
        // fall through*/
    otherwise
	print(domtype(f), op(f, 0), bool(domtype(f) = DOM_EXPR), bool(op(f, 0) = _fconcat));
	error("Unable to Bless ".expr2text(f));
    end_case
end_proc:

/**
prog::blessAny -- bless any object

prog::blessAny(f, A)

Same as prog::bless, but do not throw an error if the argument
is no proc, func-env or fconcat-expression; instead return f
unchanged.
**/

prog::blessAny :=
proc(f, DOM: DOM_DOMAIN)
    option noDebug;
begin
    // A cute idea: in debug mode, running prog::check on all
    // procedures when they are assigned to a domain.
    // This requires some more work, as the following crude
    // implementation triggers an infinite recursion
    // (probably in the instanciation of some routines used by
    // prog::check):
    //
    // if testargs() then prog::check(f,3) end_if;

    case domtype(f)
    of DOM_PROC do
        subsop(f, 13 = DOM);
        break;
    of DOM_FUNC_ENV do
        subsop(f, 1 = prog::blessAny(op(f, 1), DOM));
        break;
/*    of DOM_EXPR do
        if op(f, 0) = hold(_fconcat) then
            map(f, prog::blessAny, DOM);
            break;
        end_if;*/
    otherwise
        f
    end_case
end_proc:
