/*****************************************************************************
  $Id: domainWrapper.mu,v 1.4 2004/07/21 15:40:19 hivert Exp $
  File:         lib/PROG/domainWrapper.mu
  Authors:      Nicolas M. Thiry <nthiery@users.sourceforge.net>
                Florent Hivert <florent.hivert@univ-mlv.fr>
  Created:      20/09/2003
  License:      LGPL
*****************************************************************************/

alias(REP(x)=extop(x,1)):


domain prog::domainWrapper
/**
    w := prog::domainWrapper(bla):

    Wrap the domain bla into an object such that:
    w(a,b) calls bla::new(a,b)
    w[a,b] calls bla::_indexDomain(a,b)
    testtype(x,w) calls testtype(x, bla)
**/

    inherits Dom::BaseDomain;
    
    new :=
    proc(T : DOM_DOMAIN)
    begin
        new(dom, T);
    end_proc;

    _index :=
    proc(T: dom)
    begin
        (REP(T))::_indexDomain(args(2..args(0)));
    end_proc;

    func_call :=
    proc(T: dom)
    begin
        (REP(T))::new(context(args(2..args(0))));
    end_proc;

    testtype :=
    proc(x, T)
    begin
	if T = dom then
	    return(domtype(x) = dom)
	elif domtype(T) = dom then
	    testtype(x, REP(T));
	else
	    FAIL;
	end_if;
    end_proc;

    slot :=
    proc(T: dom, s: DOM_STRING)
        option noDebug;
    begin
        if args(0)=2 then
            slot(REP(T), s);
        else
            slot(REP(T), s, args(3..args(0)));
            T;
        end_if;
    end_proc;
end_domain:

domain prog::indexedProcedure
/**
    indexedProcedure:

    Domain to wrap a function into an object using bracket notations.
    If A is an object of the domain indexedProcedure created with
    A := indexedProcedure( f ), then A[1,2] will be evaluated as f(1,2);
**/

    inherits Dom::BaseDomain;

    new :=
    proc(p : Type::AnyType)
    begin
	new(dom, p);
    end_proc;

    _index :=
    proc(o: dom)
    begin
	REP(o)(args(2..args(0)));
    end_proc;

begin
end_domain:
