// $Date: 2004/08/02 12:42:41 $ $Author: bij $ $Revision: 1.6 $

// prog::getOptions -- extract and verify options within a procedure
// 
// 

prog::getOptions:=
  proc(allOptions = [], optionTypes = table(), skipArgs = 0)
    //option noDebug;
    local ARGS, NAMES, OPTS, ERRWARN, k, n;
  begin
    // Error or Warning
    ERRWARN:= contains([hold(Warning), hold(Error)], args(args(0)));

    // missing optionTypes
    if domtype(optionTypes) = DOM_INT then
      skipArgs:= optionTypes;
      optionTypes:= table()
    elif domtype(optionTypes) = DOM_LIST then
      skipArgs:= optionTypes;
      optionTypes:= table()
    elif optionTypes = hold(Error) then
      optionTypes := table()
    end_if;

    if domtype(skipArgs) <> DOM_INT and domtype(skipArgs) <> DOM_LIST then
      skipArgs:= 0
    end_if;
    
    // function environment
    if domtype(allOptions) = DOM_FUNC_ENV then
      if (n:= slot(allOptions, "options")) = FAIL then
        op([id, warning, error], ERRWARN + 1)
        ("no options defined in '".prog::getname(allOptions)."'");
        return(table())
      end_if;
      if (k:= slot(allOptions, "optionTypes")) <> FAIL then
        optionTypes:= k
      else
        optionTypes:= []
      end_if;
      allOptions:= n
    end_if;
    
    if domtype(skipArgs) = DOM_INT and // lazy_and!!
       context(hold(args)(0)) < skipArgs + 1 then
      // return default valued options
      return(table(op(select([op(allOptions)], testtype, "_equal"))))
    end_if;
    
    if domtype(skipArgs) = DOM_LIST then
      ARGS:= skipArgs; // list of explicitely given arguments
      skipArgs := 0 // for later
    else
      // get arguments of calling procedure
      ARGS:= [context(hold(context)(hold(args)(skipArgs+1..hold(args)(0))))];
    end_if;
    
    NAMES:= map([op(allOptions)], X -> if type(X) = "_equal" then op(X, 1) else X end);
    // select all options
    OPTS:= select(ARGS, has, NAMES);

    // explicit error or warning
    if ERRWARN > 0 then
      if nops(OPTS) < nops(ARGS) then
        // first argument not an option
        n:= contains(ARGS, op(select(ARGS, _not@has, NAMES), 1));
        context(op([hold(warning), hold(error)], ERRWARN)
                ("wrong ".output::ordinal(skipArgs + n)." argument"))
      end_if
    end_if;

    // return value
    OPTS:= table(op(select(allOptions, testtype, "_equal")), // default values
                 op(map(OPTS, X -> if type(X) = "_equal" then X else X = TRUE end)),
                 (if args(args(0)) = hold(Explicit) then
                    hold(__USERDEFINED__) = {op(map(OPTS, X -> if type(X) = "_equal" then
                                                                 op(X, 1)
                                                               else
                                                                 X
                                                               end))}
                  else
                    null()
                  end_if));

    optionTypes:= [op(optionTypes)];
     // do type check
    if optionTypes <> [] then
      // check type of options
      for k in optionTypes do
        if type(k) <> "_equal" then
          error("option types must be given as equation")
        elif contains(OPTS, op(k, 1)) then
          if type(op(k, 2)) = DOM_PROC and bool(op(k, 2)(OPTS[op(k, 1)])) <> TRUE
             or type(op(k, 2)) <> DOM_PROC and not testtype(OPTS[op(k, 1)], op(k, 2)) then
            context(op([hold(error), hold(warning)], (ERRWARN mod 2) + 1)
                    ("wrong type of ".
                     output::ordinal(skipArgs + contains(ARGS, op(k, 1) = OPTS[op(k, 1)])
                                     + contains(ARGS, op(k, 1)))." argument"))
          end_if
        end_if;
      end_for
    end_if;
    OPTS
  end_proc: