/*****************************************************************************
  $Id: makeExample.mu,v 1.3 2003/09/20 18:55:28 nthiery Exp $
  File:		lib/PROG/makeExample.mu
  Authors:	Nicolas M. Thiry <nthiery@users.sourceforge.net>
  Created:	16/02/2001
  License:	LGPL
*****************************************************************************/

/**
prog::makeExample -- utility for preparing an example in the documentation

prog::makeExample(expression1, expression2, ...):

Prints out an example which runs each expression in turn, ready to be
copy-pasted into a MuPAD LaTeX documentation file.

Returns null():

Example:

>> prog::makeExample(combinat::partitions::count(4), combinat::partitions::list(3)):
  \begin{Mexin}
combinat::partitions::count(4);
combinat::partitions::list(3)
  \end{Mexin}
  \begin{Mexout}
                               5

                   [[3], [2, 1], [1, 1, 1]]
  \end{Mexout}

**/

prog::makeExample :=
proc()
    option hold;
    local expression, i;
    save TEXTWIDTH;
begin
    TEXTWIDTH := 63;
    fprint(Unquoted, 0, "  \\begin{Mexin}");
    for i from 1 to args(0)-1 do
	fprint(Unquoted, 0, expr2text(args(i)),";");
    end_for;
    fprint(Unquoted, 0, expr2text(args(args(0))));
    fprint(NoNL, 0, "  \\end{Mexin}\n  \\begin{Mexout}");
    for expression in [args()] do
	print(context(expression));
    end_for;
    fprint(Unquoted, 0, "  \\end{Mexout}");
    null():
end_proc:
