// $Date: 2003/08/25 08:37:07 $ $Author: bij $ $Revision: 1.4 $

// prog::nTime
//   ()          - returns a factor to normalize timings
//                 evaluation time must be divided by this factor
// 
//   (statement) - returns evaluation time of 'statement' normalized
//                 to 1000 MHz processor
//

prog::ntime:=
  proc(TIME)
    local FACTOR, k, T, ntimeTestProc, x, i;
    option hold;
  begin
    // dummy proc as time measure for procedure calls
    ntimeTestProc:= proc(x)
                      local y;
                    begin
                      y:= x;
                      if x > y then
                        return(x)
                      else
                        return(y)
                      end_if
                    end_proc;
    
    // return normalize factor
    if args(0) = 0 then
      // calibrate processor time
      T:= time();
      FACTOR:= 0;
      while time() - T < 200 do // 200 milli seconds
        FACTOR:= FACTOR + 1;
      end_while;
      
      // perform several MuPAD operations
      T:= time((T:= [];
                for k from 0 to FACTOR/7 do
                  // sometimes procedure call and memory-allocation operations
                  if specfunc::trunc(k/20) mod 20 = 0 then
                    T:= append(T, ntimeTestProc(i) $ i = 1..50)
                  else
                    x:= 42;
                    x*(x + 1) $ i = 1..20;
                    x:= 1.235;
                    x*(x + 1) $ i = 1..20;
                  end_if
                end_for));

      return(T/(FACTOR/25.0)/1.1) // difference to version 2.5
    else
      FACTOR:= prog::ntime();
      T:= time(context(TIME));
      if T <= 10 and domtype((k:= context(TIME))) = DOM_INT then
        specfunc::trunc(k/FACTOR)
      elif T <= 10 and domtype(k) = DOM_FLOAT then
        k/FACTOR
      else
        specfunc::trunc(T/FACTOR)
      end_if
    end_if
  end_proc:
