// $Date: 2003/11/08 16:37:52 $ $Author: bij $ $Revision: 1.16 $

// prog::remember(FUNC, <DEP, <DEF, >> <, OPTION>)
//   returns a procedure, that calls FUNC and keeps results for later using
//   
// FUNC      - any MuPAD procedure or function environment
// DEP       - procedure, that is called with the input to FUNC and
//             must return a info to determine, whether two inputs are
//             equal or not, e.g., get out all properties of involved identifiers
// DEF       - procedure (or constant value), that is called with the
//             input, when a recursion is detected, and that returns the
//             value to return
// OPTION    - PreventRecursion, Trace, All
// 
//   PreventRecursion - verhindert endlos-Rekursionen
//   Debug            - Option noDebug wird nicht gesetzt
//   All              - merkt sich alle Paare input, depends(input)
// 

//!!!! insert names of arguments and default values

prog::remember:=
  proc(FKT, DEP = FAIL, PRE = null(), DEF = null())
    option escape, noDebug;
    local rememberTable, FN;
  begin
    if contains({DOM_PROC, DOM_FUNC_ENV}, domtype(FKT)) then
      if domtype(FKT) = DOM_FUNC_ENV then
        subsop(FKT, 1 = prog::remember(op(FKT, 1), DEP, args(args(0))))
      else
        rememberTable:= table();
        FN:= prog::getname(FKT);

        //if args(args(0)) = hold(All) then
        //  subsop(proc()
        //           local IND;
        //           option noDebug;
        //         begin
        //           IND:= [[args()], [DEP(args())]];
        //           if contains(rememberTable, IND) then
        //             userinfo(10, "I remember me...[".FN."]");
        //             return(rememberTable[IND])
        //           end_if;
        //           rememberTable[IND]:= FKT(args())
        //         end_proc,
        //         6 = text2expr("remember::".FN),
        //         1 = op(FKT, 1))
        // wie option remember
        if args(0) = 1 then
          subsop(proc()
                   local RET;
                   option noDebug;
                 begin
                   if args(0) > 0 and args(1) = hold(Remember) then
                     //userinfo(10, "Only remember actions ... [".FN."]");
                     if args(2) = hold(ClearPrevent) then
                       for RET in rememberTable do
                         if op(RET, 2) = hold(PreventRecursion) then
                           delete rememberTable[op(RET, 1)]
                         end_if
                       end_for
                     elif args(2) = hold(Clear) then
                       rememberTable:= table()
                     elif args(2) = hold(Print) then
                       return(rememberTable)
                     end_if;
                     return()
                   end_if;
                   if contains(rememberTable, [args()]) then
                     userinfo(10, "I remember me... [".FN."]");
                     RET:= rememberTable[[args()]];
                     return(op(RET))
                   end_if;
                   RET:= FKT(args());
                   rememberTable[[args()]]:= [RET];
                   RET
                 end_proc,
                 6 = text2expr("remember::".FN),
                 1 = op(FKT, 1))
        // keine Endlosrekursion
        elif DEP = hold(PreventRecursion) then
          if args(args(0)) <> hold(PreventRecursion) then
            DEF := args(args(0))
          else
            DEF := id
          end_if;
          subsop(proc()
                 local RET;
                 option noDebug;
                 begin
                   if args(0) > 0 and args(1) = hold(Remember) then
                     //userinfo(10, "Only remember actions ... [".FN."]");
                     if args(2) = hold(ClearPrevent) then
                       for RET in rememberTable do
                         if op(RET, 2) = hold(PreventRecursion) then
                           delete rememberTable[op(RET, 1)]
                         end_if
                       end_for
                     elif args(2) = hold(Clear) then
                       rememberTable:= table()
                     elif args(2) = hold(Print) then
                       return(rememberTable)
                     end_if;
                     return()
                   end_if;
                   if contains(rememberTable, [args()]) then
                      userinfo(10, "Recursion detected ... [".FN."]");
                      return(DEF(args()))
                   end_if;
                   rememberTable[[args()]]:= hold(PreventRecursion);
                   RET:= FKT(args());
                   delete rememberTable[[args()]];
                   RET
                 end_proc,
                 6 = text2expr("remember::".FN),
                 1 = op(FKT, 1))
        elif PRE = hold(PreventRecursion) then // DEP, Prevent
          if args(args(0)) <> hold(PreventRecursion) then
            DEF := args(args(0))
          else
            DEF := id
          end_if;
          if DEP = FAIL then // Prevent mit remember, ohne DEP
            subsop(proc()
                     local RET;
                     option noDebug;
                   begin
                     if args(0) > 0 and args(1) = hold(Remember) then
                       //userinfo(10, "Only remember actions ... [".FN."]");
                       if args(2) = hold(ClearPrevent) then
                         for RET in rememberTable do
                           if op(RET, 2) = hold(PreventRecursion) then
                             delete rememberTable[op(RET, 1)]
                           end_if
                         end_for
                       elif args(2) = hold(Clear) then
                         rememberTable:= table()
                       elif args(2) = hold(Print) then
                         return(rememberTable)
                       end_if;
                       return()
                     end_if;
                     if contains(rememberTable, [args()]) then
                       RET:= rememberTable[[args()]];
                       if RET = hold(PreventRecursion) then
                          userinfo(10, "Recursion detected ... [".FN."]");
                          //delete rememberTable[[args()]];
                          return(DEF(args()))
                       else
                         userinfo(10, "I remember me... [".FN."]");
                         return(op(RET, 1))
                       end_if
                     end_if;
                     rememberTable[[args()]]:= hold(PreventRecursion);
                     RET:= FKT(args());
                     rememberTable[[args()]]:= [RET];
                     RET
                   end_proc,
                   6 = text2expr("remember::".FN),
                   1 = op(FKT, 1))
        else // Prevent mit remember, mit DEP
          subsop(proc()
                   local RET;
                   option noDebug;
                 begin
                   if args(0) > 0 and args(1) = hold(Remember) then
                     //userinfo(10, "Only remember actions ... [".FN."]");
                     if args(2) = hold(ClearPrevent) then
                       for RET in rememberTable do
                         if op(RET, 2) = hold(PreventRecursion) then
                           delete rememberTable[op(RET, 1)]
                         end_if
                       end_for
                     elif args(2) = hold(Clear) then
                       rememberTable:= table()
                     elif args(2) = hold(Print) then
                       return(rememberTable)
                     end_if;
                     return()
                   end_if;
                   if contains(rememberTable, [args()]) then
                     RET:= rememberTable[[args()]];
                     if RET = hold(PreventRecursion) then
                        userinfo(10, "Recursion detected ... [".FN."]");
                        //delete rememberTable[[args()]];
                        return(DEF(args()))
                     elif DEP(args()) = op(op(RET, 2)) then
                       userinfo(10, "I remember me... [".FN."]");
                       return(op(op(RET, 1)))
                     else
                       userinfo(10, "Remember value invalid... [".FN."]");
                     end_if
                   end_if;
                   rememberTable[[args()]]:= hold(PreventRecursion);
                   RET:= FKT(args());
                   rememberTable[[args()]]:= [[RET], [DEP(args())]];
                   RET
                 end_proc,
                 6 = text2expr("remember::".FN),
                 1 = op(FKT, 1))
        end_if
      else // DEP, kein Prevent
        subsop(proc()
                 local RET;
                 option noDebug;
               begin
                 if args(0) > 0 and args(1) = hold(Remember) then
                   //userinfo(10, "Only remember actions ... [".FN."]");
                   if args(2) = hold(ClearPrevent) then
                     for RET in rememberTable do
                       if op(RET, 2) = hold(PreventRecursion) then
                         delete rememberTable[op(RET, 1)]
                       end_if
                     end_for
                   elif args(2) = hold(Clear) then
                     rememberTable:= table()
                   elif args(2) = hold(Print) then
                     return(rememberTable)
                   end_if;
                   return()
                 end_if;
                 if contains(rememberTable, [args()]) then
                   RET:= rememberTable[[args()]];
                   if DEP(args()) = op(op(RET, 2)) then
                     userinfo(10, "I remember me... [".FN."]");
                     return(op(op(RET, 1)))
                   else
                     userinfo(10, "Remember value invalid... [".FN."]");
                   end_if
                 end_if;
                 RET:= FKT(args());
                 rememberTable[[args()]]:= [[RET], [DEP(args())]];
                 RET
               end_proc,
               6 = text2expr("remember::".FN),
               1 = op(FKT, 1))
        end_if
      end_if
    else
      error("wrong type of 1st argument (DOM_PROC or DOM_FUNC_ENV expected)")
    end_if
  end_proc:
