// $Date: 2004/10/05 07:25:38 $ $Author: bij $ $Revision: 1.18 $

/*

Das Datenfile hat die Form:

<int>:<int>:
  ~     ~
  ~     ~
<int>:<int>:
-1:-1:
<str>
  ~
<str>

<int> ganze Zahl >= 0
<str> Zeichenkette in " " eingeschlossen.

Der erste Eintrag ist der Index des zugehoerigen Dateinamens, der
zweite Eintrag die Zeilennummer. Nach der Zeile -1:-1: folgen die
Dateinamen. 

Bsp.:

0:2:
1:4:
-1:-1:
"test1":
"test2":

Zeile 2 der Datei test1, die den Index 0 hat, und Zeile 4 der
Datei test2, die den Index 1 hat, wurden durchlaufen.

*/

// 

prog::tcov :=
proc(STMT)
  local OPT, fname, fd, chName, file, line, l, max_file, cnt, filename, OFILE, result;
  option hold;
begin
  if args(0) = 0 then
     return();
  end_if;

  if debug() <> TRUE then
    error("MuPAD kernel must be started in debug modus (see '?debug')")
  end_if;

  // get options with error, skip first argument
  OPT:= prog::getOptions([hold(Path), hold(File), hold(Stat), hold(Append), hold(Extern)],
                         table(hold(Path)   = DOM_STRING,
                               hold(Append) = DOM_BOOL,
                               hold(Stat)   = DOM_BOOL,
                               hold(File)   = DOM_BOOL,
                               hold(Extern) = DOM_BOOL),   1, hold(Error));

  // change given filename
  chName:=
    proc(X, ch)
      local Y;
    begin
      if (Y:= stringlib::subs(X, ch)) <> X then
        (if contains(OPT, hold(Path)) then
           OPT[hold(Path)]
         else
           ""
         end_if).
        stringlib::subs(X, ch)
      else
        FAIL
      end_if
    end_proc;

  // session depending name of temporary file
  fname := "/tmp/tcov".getpid().".dat";

  // result of the command, start kernel tcov process
  result := stdlib::gcov(context(STMT), fname);

  if contains(OPT, hold(Extern)) then
    fprint(Unquoted, 0,  "Info: Skip interpretation and keep temporary file '".fname."'.");
    return(result)
  end_if;
  
  // 
  //fprint(Unquoted, 0, "Info: Evaluation of '", 
  //                    if length(expr2text(STMT)) > 35 then
  //                      stringlib::format(expr2text(STMT), 35), "..."
  //                    else
  //                      stringlib::format(expr2text(STMT)), "' finished."
  //                    end_if);
  fprint(Unquoted, 0,  "Info: Evaluate coverage information.");

  if (fd := fopen(fname)) = FAIL then
    error("cannot open temporary file '".fname."' for reading")
  end_if;

  // 
  cnt:= table();       // contains a table for each file
                       // each table contains on entry line = passes for each passed line
              
  // read data file
  while finput(fd, filename, l) <> null() and filename > -1 do
    [file, line] := [filename, l]; // file = passed line
    if not contains(cnt, file) then
      cnt[file] := table()
    end_if;
    if contains(cnt[file], line) then
      cnt[file][line] := cnt[file][line] + 1
    else
      cnt[file][line] := 1;
    end_if
  end_while;

  if filename <> -1 then
    error("illegal format of temporary file '".fname."'")
  end_if;
  
  filename := table(); // contains entry number = filename for each file

  max_file := -1; // indexed from 0
  while (finput(fd, l); l) <> null() do
     max_file := max_file + 1;
     filename[max_file] := l
  end_while;

  fclose(fd);

  // reads in existing status files
  if contains(OPT, hold(Append)) then // extend existing files
    for file from 0 to max_file do
      if traperror((fd := fopen(chName(filename[file], ".mu" = ".tcov")))) = 0 and fd <> FAIL
         and contains(cnt, file) and cnt[file] <> table() then
        // file exists
        while (finput(fd, line, l)) <> null() do
          if contains(cnt[file], line) then
            cnt[file][line]:= cnt[file][line] + l
          else
            cnt[file][line]:= l
          end_if
        end_while;
        fclose(fd)
      else
        traperror(fclose(fd)) // close if could be opened
      end_if
    end_for
  end_if;

  if not contains(OPT, hold(File)) and not contains(OPT, hold(Stat)) then // screen output
    for file from 0 to max_file do
      if contains(cnt, file) and cnt[file] <> table() then
        fprint(Unquoted, 0, "File: ", filename[file]);
        for line in sort([map(op(cnt[file]), op, 1)]) do
          fprint(Unquoted, 0,
                 "      Line", stringlib::format(expr2text(line), 6, hold(Right)),
                          ":", stringlib::format(expr2text(cnt[file][line]), 6, hold(Right)),
                          " time", _if(cnt[file][line] <> 1, "s", ""))
        end_for
      end_if
    end_for
  // 
  else // write status files
    for file from 0 to max_file do
      // open status file for writing
      if contains(cnt, file) and cnt[file] <> table() then // any lines used
        if traperror((fd:= fopen(hold(Text),
                                 chName(filename[file], ".mu" = ".tcov"),
                                 hold(Write)))) <> 0
          or fd = FAIL then
          if chName(filename[file], ".mu" = ".tcov") <> FAIL then
            warning("cannot open status file '".chName(filename[file], ".mu" = ".tcov")
                    ."' for writing")
          else
            // tmp file
          end_if;
          next
        end_if;
        for line in sort([map(op(cnt[file]), op, 1)]) do
          fprint(fd, line, cnt[file][line]) // format: line:passes:
        end_for;
        fclose(fd)
      end_if;
    end_for;
    
    fprint(Unquoted, 0, "Info: Writing '.tcov' status files done.");
    
    if contains(OPT, hold(File)) then // write file with commented lines
      // open source files and files for output
      for file from 0 to max_file do
        // any line used??
        if contains(cnt, file) and cnt[file] <> table() then // any lines used
          if traperror((fd:= fopen(filename[file]))) <> 0
            or fd = FAIL then
            warning("cannot open source file '".filename[file]."'");
            next
          end_if;
          if traperror((OFILE := fopen(hold(Text),
                                       chName(filename[file], ".mu" = "_tcov.mu"),
                                       hold(Write)))) <> 0
             or OFILE = FAIL then
            if chName(filename[file], ".mu" = "_tcov.mu") <> FAIL then
              warning("cannot open file '".chName(filename[file], ".mu" = "_tcov.mu")
                      ."' for writing");
              next
            else
              // tmp file
            end_if;
            next
          end_if;
          l:= 0;
          fprint(Unquoted, OFILE, "// !!! Three lines are inserted in front of this file !!! -*-tcov-*- ");
          fprint(Unquoted, OFILE, "// Generated by ",
                 if contains(OPT, hold(Append))
                 then "*appending* "
                 else "" end_if,
                 "'prog::tcov' call: ", stringlib::format(expr2text(STMT), 35), "\n");
          while (l:= l + 1; line:= ftextinput(fd)) <> null() do
            if contains(cnt, file) and contains(cnt[file], l)
              and domtype(cnt[file][l]) = DOM_INT then
              fprint(Unquoted, OFILE, "/*", 
                     stringlib::format(expr2text(min(cnt[file][l], 999999)), 6, hold(Right)),
                     " */", line)
            else
              fprint(Unquoted, OFILE, "/* ----- */", line)
            end_if
          end_while;
          fclose(fd);
          fclose(OFILE)
        end_if
      end_for;
      fprint(Unquoted, 0, "Info: Writing '_tcov.mu' files done.")
    end_if
  end_if;

  // UNCOMMENT THIS
  stdlib::gcov(NIL, fname); // remove tcov file 

  fprint(Unquoted, 0, ""); // newline
  result

end_proc:
