// $Date: 2004/06/04 12:40:01 $ $Author: bij $ $Revision: 1.15 $

/*
prog::testexit -- exit testing

prog::testexit()

Close test results file.
*/

prog::testexit:=
  proc()
    local LogFile, TIME, ntime, ntimearg, ntimelog, k, NAME, F;
  begin
    // cancel memory limit
    if prog::UserMaxMem = -1 then // untouched by testinit: save users value here
      sysassign(prog::UserMaxMem, Pref::maxMem(0))
    else // already set in testinit
      Pref::maxMem(0)
    end_if;
    
    // move new table with error states to the default place
    sysassign(prog::TestErrorTable, prog::TestErrorTableNew);
    sysassign(prog::TestErrorTableNew, table());

    TIME:= time() - prog::TestInitTime; // gesamte Test-Zeit
    F:= X -> stringlib::formatf(float(X), 1);
    if domtype(prog::TestFile) = DOM_INT then
      fclose(prog::TestFile)
    end_if;

    // normalized time factor
    if op(prog::TestNTime, 1) = 1 then // compute only when not given
      ntime:= prog::ntime()
    else
      ntime:= op(prog::TestNTime, 1)/1000
    end_if;
    
    if debug() = TRUE then // kernel is running in debug mode
      ntime := ntime*1.1
    end_if;

    // difference between whole test time and testtime inside of 'prog::test'
    if (TIME + prog::TestTime) > 0 and TIME/(TIME + prog::TestTime) > 0.66 then // 2 times long
      fprint(Unquoted, 0, "Info: time used outside of 'prog::test' takes ".
                          expr2text(trunc(TIME/(TIME + prog::TestTime)*100)) ."%")
    end_if;
    // take whole test time instead of netto time using by all 'prog::test' calls
    // if option 'All' is given
    if op(prog::TestNTime, 2) = TRUE then
      sysassign(prog::TestTime, TIME) // whole test time
    end_if;
    
    ntimearg:= prog::TestExpectedTime; // expected time given to 'prog::testinit'

    // warning for too short test files
    if ntimearg <> 0 and trunc(1/(ntimearg/1000*ntime)*100) > 149 then
      fprint(Unquoted, 0, "Info: expected test time is very short; only changes above ".
                          expr2text(trunc(1/(ntimearg/1000*ntime)*10)*10)."% are reported")
    end_if;

    // interaktiv: Meldung ausgeben
    if not prog::TestQuiet /*and prog::TestLogFile = FAIL*/ then
      //Info: property::assume - 26 examples,  2 errors, 12 warnings, time: 18.10 s (expected: 25.00 s)
      fprint(Unquoted, 0, "Info: ",
             expr2text(prog::TestCount),
             " test"._if(prog::TestCount = 1, "", "s").", ",
             expr2text(prog::TestErrorCount),
             " error"._if(prog::TestErrorCount = 1, "", "s")
//             );
//      fprint(Unquoted, 0, "Info: "
             ,", runtime factor ", F(prog::TestTime/1000/ntime),
             _if(ntimearg <> 0,
                 ("  (expected ", F(ntimearg/1000), ")"),
                 "  (nothing expected)")
             //," (testing and loading overhead", F((TIME - prog::TestTime)/1000/ntime), ")"
             );
      // old or expected timings exists and > 1000 ms
      if ntimearg <> 0 and specfunc::abs(ntimearg - prog::TestTime/ntime) >= 1000 then 
        if prog::TestTime/ntimearg/ntime >= 1.55 then
          warning("expected time and test time differ "
                  .F(float(((prog::TestTime/ntimearg/ntime) - 1)*100))." %")
        elif prog::TestTime/ntimearg/ntime < 0.6 then
          warning("expected time and test time differ "
                  .F(float(((prog::TestTime/ntimearg/ntime) - 1)*100))." %")
        end_if
      end_if;

      if prog::TestTimings <> FAIL then
        fprint(Unquoted, 0, "Info: '",
               expr2text(prog::TestTimingsFunc),
               "' takes ", F(prog::TestTimings/1000), " s")
      end_if;
    end_if;
    fprint(Unquoted, 0, "Info: All tests ", expr2text(prog::TestCount), " [prog::testexit]");
    // Aufruf von Test: Log-Datei schreiben
    // Achtung: Meldungen nicht ndern! werden von Test eingelesen
    if prog::TestLogFile <> FAIL then
      ntimelog:= prog::TestOldErrors["Time", 0]; // expected time stored in '.err' file
      // format:
      //  expect time  yesterdays time  yesterdays difference 
      //  14490,       14650,           320
      //   time in (unnormalized) ms

      // Meldung wird von Test ausgewertet
      NAME:= prog::TestOldErrors["Name"]; // Rechnername bergeben von Test
      if traperror((LogFile:= fopen(Text, prog::TestLogFile, Write))) = 0
         and LogFile <> FAIL then
        for k in prog::TestErrors do // alle aktuellen Fehler in Logdatei merken
          if op(k, 2) = TRUE then
            fprint(LogFile, op(k, 1))
          end_if
        end_for;
        for k in prog::TestOldErrors do // old state information
          if op(op(k, 1), 1) = "Time" and op(k, [1, 2]) <> 0 then
            fprint(LogFile, [op(op(k, 1)), op(k, 2)])
          end_if
        end_for;
        // neu setzen einer Erwartungs-Zeit
        if prog::TestOldErrors[hold(RESETTIMINGS)] = TRUE then
          fprint(LogFile, ["Time", NAME, prog::TestTime, prog::TestTime, op(ntimelog, 2) - op(ntimelog, 1)])
        else // keep old timing
          //                             old timing       akt timing      yesterdays diff
          fprint(LogFile, ["Time", NAME, op(ntimelog, 1), prog::TestTime, op(ntimelog, 2) - op(ntimelog, 1)])
        end_if;
        fclose(LogFile)
      else
        warning("could not open log file '".prog::TestLogFile."'")
      end_if;
    // Time check
      if ntimearg <> 0 then
        ntime:= ntimearg*ntime;
      else
        ntime:= op(ntimelog, 1); // old timing from '.err'-Datei
      end_if;
      //fprint(Unquoted, 0, "Info: time -- ntime: ", ntime);
      if ntime >= 1000 then
        // old or expected timings exists and are measurable
        F:= proc(x) // x in [-1.0...-0.4[ union ]0.5...infinity[
              local x2;
            begin
              if not testtype(x, Type::Arithmetical)
                 or specfunc::abs(x) <= 10 then
                return("0")
              end_if;
              x2:= 10^specfunc::trunc(log(10, specfunc::abs(x)));
              op(["-", "", "+"], sign(x) + 2).expr2text(specfunc::round(specfunc::abs(x)/x2)*x2)
            end;
        if prog::TestTime/ntime > 1.5 or prog::TestTime/ntime < 0.6 then
          warning("Time ".F(((prog::TestTime/ntime) - 1)*100)._if(ntimearg = 0, " X ", " Y ").
                  F((prog::TestTime/op(ntimelog, 2) - 1)*100))
        end_if;
      else
        fprint(0, "Info: time not measurable [prog::testexit]")
      end_if
    end_if;
    
    // reset all global values for next test
    sysassign(prog::TestFile, FAIL);
    sysassign(prog::TestLogFile, FAIL);
    sysassign(prog::TestNo, 1);
    sysassign(prog::TestOldErrors, FAIL);
    sysassign(prog::TestFunc, FAIL);
    sysassign(prog::TestErrorCount, 0);
    sysassign(prog::TestCount, 0);
    sysassign(prog::TestExpectedTime, 0);
    sysassign(prog::TestNTime, 1);
    // re-set users memory limit
    Pref::maxMem(prog::UserMaxMem);
    sysassign(prog::UserMaxMem, -1);
    null()
  end_proc:

