// $Date: 2004/09/23 07:51:33 $ $Author: bij $ $Revision: 1.12 $

/*++
prog::testinit -- init testing

prog::testinit(f, <expected_ntime>, <Option>)

f                - file name (default is "test")
expected_ntime   - normalized!! Time (with 'prog::ntime') in
                   milli seconds (DOM_INT) or seconds (DOM_FLOAT)
Option           - _in(Quiet, Timer = t)
                   Quiet = suppress 'prog::testexit' messages
                   Timer = use 't' instead of 'prog::ntime' to
                           compute the normalization factor
                   All   = measure time of whole test

Opens the test result file 'f.res'. If the variable 'TESTPATH' has a
value, it is used as a prefix for the path name.
++*/

alias(AllOptions = [hold(Quiet), // no messages
                    hold(Timer), // = func/value instead of 'prog::ntime()'
                    hold(All),   //count difference between 'testinit' and 'testexit'
                                 // otherwise: only time for evaluating of first
                                 // arguments inside of 'prog::test' calls is counted
                    // architectures: times
                    "UNIX", "MSDOS", "MACOS", "linux", "solaris",
                    hold(NoTestArgs) // switch off automatic testargs usage
                    ]):

// global variables
prog::TestFile:= FAIL:        // descriptor to test file
prog::TestLogFile:= FAIL:     // desriptor to log file
prog::TestOldErrors:= FAIL:   // table of old errors for logging
prog::TestErrors:= FAIL:      // table of errors for logging
prog::TestErrorCount:= 0:     // number of errors
prog::TestExpectedTime:= 0:   // expected time factor
prog::TestNTime:= 1:          // list: [normalize factor, measure whole test time : DOM_BOOL]
prog::TestQuiet:= FALSE:      // 
prog::TestNo:= 1:             // number of test call
prog::UserMaxMem:= -1:        // save users memory limit - init value: '-1' !
prog::TestLogExample:= FAIL:  // log each example
prog::TestTimeTable := FAIL:

prog::testinit:=
  proc(f = "test", TIME = 0, TIMEUNIT = 0)
    local UOPT, LogFile, inp, S, NM, OPTS, TestNTime;
  begin
    if testargs() then
      if args(0) = 0 then
        // ...
      elif domtype(f) <> DOM_STRING then
          error("first argument must be a string")
      end_if
    end_if;
    
    if TESTPATH <> hold(TESTPATH) then
      f:= TESTPATH.f
    end_if;
    sysassign(prog::TestFile, fopen(Text, f.".res", Write));

    OPTS:= prog::getOptions(AllOptions, 1);
    if contains(OPTS, hold(Arch)) then
      OPTS:= table(op(OPTS), op(OPTS[hold(Arch)]))
    end_if;

    if OPTS[hold(NoTestArgs)] = TRUE then
      userinfo(10, "switch OFF setting testargs/typecheck in prog::test in testinit");
      sysassign(prog::TestNoTestargs, TRUE)
    end_if;

    // table containing older results
    // select valid user option entries
    if strmatch(Pref::userOptions(), "TESTPATH", Index) <> FALSE
       or strmatch(Pref::userOptions(), "NAME", Index) <> FALSE 
       or strmatch(Pref::userOptions(), "LOG", Index) <> FALSE then
      UOPT:= [text2expr(Pref::userOptions())];
      UOPT:= table(select(UOPT, testtype, Type::Equation()));
      // MuPAD Option: -U "TESTPATH=\"PROP/TEST/property.err\",NAME=\"galois\"
      //                                                      ,TIMING=is,NEWTIMINGS=TRUE"
      // gesetzte 'Useroption'
      if nops(UOPT) > 1 then
        if contains(UOPT, hold(TESTPATH)) then
          // Dateiname relativ zum LIBPATH
          if contains(UOPT, hold(NAME)) then
            NM:= UOPT[hold(NAME)]
          else
            NM:= "noname"
          end_if;
          S:= table("Name" = NM);
          // Name der Log-Datei
          if traperror((LogFile:= fopen(UOPT[TESTPATH]))) = 0 and LogFile <> FAIL then
            if traperror((
                          while (inp:= finput(LogFile)) <> null() do
                            if op(inp, 1) = "Time" then
                              // save global timing information
                              // Line format:
                              //    const   comp-name  expect time  yesterdays time  yesterdays difference
                              //   ["Time", "local",   14490,       14650,           320]
                              //   time in (unnormalized) ms
                              if op(inp, 2) = NM then // this computer
                                S["Time", op(inp, 2)]:= (op(inp, 3..nops(inp)), 0, 0) // fill up with dummies
                                // for first set of empty times
                              else // uncomment this to keep all information
                                //S["Time", op(inp, 2)]:= (op(inp, 3..nops(inp)))
                              end_if
                          else
                          S[inp]:= FALSE
                          end_if
                          end_while)) <> 0 then
              warning("serious problem while reading the log file '".UOPT[TESTPATH]."'");
              
            end_if;
            fclose(LogFile);
            sysassign(prog::TestLogFile, UOPT[hold(TESTPATH)])
          else
            userinfo(1, "Yesterdays log-file not found");
            // set default values
            S["Time", 0]:= [0, 0];
            UOPT[hold(RESETTIMINGS)]:= TRUE; // set initial timing
            sysassign(prog::TestLogFile, FAIL)
          end_if;
        end_if;
        // perform timings of one function
        if contains(UOPT, hold(TIMING)) then
          // perform timings
          f:= genident("_INT");
          evalassign(f, eval(UOPT[hold(TIMING)]), 2);
          protected(f, hold(Error));
          inp:= UOPT[hold(TIMING)];
          prog::init(eval(UOPT[hold(TIMING)]));
          sysevalassign(inp,
                        subsop(proc()
                                 local RES;
                               begin
                                 sysassign(prog::TestTimings,
                                           prog::TestTimings + time((RES:= NIL(args()))));
                                 RES
                               end_proc, [4, 1, 2, 2, 1, 2, 0] = eval(f)), 2,    // f
                                         6 = text2expr(prog::getname(eval(f)))); // name of f
          sysassign(prog::TestTimingsFunc, UOPT[hold(TIMING)]);
          sysassign(prog::TestTimings, 0)
        end_if;
        // reset old timings inside of logfiles
        if contains(UOPT, hold(RESETTIMINGS)) then
          S[hold(RESETTIMINGS)]:= TRUE
        end_if;
        // don't set testargs(TRUE) and Pref::typeCheck(Always) in 'prog::test'
        if contains(UOPT, hold(NOTESTARGS)) and UOPT[hold(NOTESTARGS)] = TRUE then
          userinfo(10, "switch OFF setting testargs/typecheck in prog::test by userpref");
          sysassign(prog::TestNoTestargs, TRUE)
        end_if;
        // memory limit
        if contains(UOPT, hold(MEMORYLIMIT)) then
          sysassign(prog::UserMaxMem, Pref::maxMem()); // save users default
          Pref::maxMem(specfunc::trunc(UOPT[hold(MEMORYLIMIT)]*1000))   // set to Test scripts value
        end_if;
        // log current example to identify kernel crashes
        if contains(UOPT, hold(LOG)) then
          sysassign(prog::TestLogExample, UOPT[hold(LOG)])
        end_if
      else
        S:= table();
        NM:= FAIL
      end_if;
      // write older results
      if contains(S, ("Time", NM)) then
        S["Time", 0]:= S["Time", NM]; // expected Timings from logfile: unnormalized!
        delete S["Time", NM]
      else
        S["Time", 0]:= 0
      end_if;
      sysassign(prog::TestOldErrors, S);
      sysassign(prog::TestErrors, table());
    end_if;
    
    // evaluate time unit -> inside of 'prog::testexit'
    // expected time for whole test
    if domtype(TIME) = DOM_INT then // integer number as second argument
      sysassign(prog::TestExpectedTime, TIME)  // expected cumputing time in milli seconds
                                               // normalized with prog::normalizeTime
    elif domtype(TIME) = DOM_FLOAT then // real number as second argument
      // expected cumputing time in seconds
      sysassign(prog::TestExpectedTime, TIME)
    elif contains(OPTS, sysname(hold(Arch))) then
      sysassign(prog::TestExpectedTime, OPTS[sysname(hold(Arch))])
    elif contains(OPTS, sysname()) then
      sysassign(prog::TestExpectedTime, OPTS[sysname()])
    end_if;
    if domtype(prog::TestExpectedTime) = DOM_FLOAT then
      // keep time in seconds
      sysassign(prog::TestExpectedTime, specfunc::trunc(prog::TestExpectedTime*1000))
    end_if;
    
    if prog::TestFile = FAIL then
      error("could not open results file")
    end_if;
    sysassign(prog::TestNo, 1);
    sysassign(prog::TestFunc, "anonymous");
    sysassign(prog::TestInitTime, time()); // gesamte Zeit
    sysassign(prog::TestTime, 0); // Rechenzeit
    sysassign(prog::TestTimeTable, table()); // alle Zeiten pro Test speichern
    sysassign(prog::TestErrorCount, 0);
    sysassign(prog::TestCount, 0);
    if contains(OPTS, hold(Timer)) then
      if domtype(OPTS[hold(Timer)]) = DOM_FLOAT then
        TestNTime := specfunc::trunc(OPTS[hold(Timer)]*1000)
      else
        TestNTime := OPTS[hold(Timer)]
      end_if
    else
      TestNTime := 1
    end_if;
    sysassign(prog::TestQuiet, contains(OPTS, hold(Quiet)));
    sysassign(prog::TestNTime, [TestNTime, contains(OPTS, hold(All))]);
    // for prog::testerrors
    sysassign(prog::TestErrorTable, table());
    sysassign(prog::TestErrorTableNew, table());
    // for prog::testcall
    sysassign(prog::TestErrorCalls, table());
    null()
  end_proc:
