// $Date: 2003/11/25 09:19:34 $ $Author: bij $ $Revision: 1.87 $

/*(
   IV     - Domain of numerical property 'Intervall'

)*/

alias(IV = property::IV):
alias(IVnat = property::IVnat):
alias(Arg(x) = op(extop(x, 5), 1)):
alias(RG(x) = op(extop(x, 5), 2)):
alias(Ar = [5, 1]): alias(Rg = [5, 2]):

IV:= newDomain("property::IV"):
IV::create_dom:=hold(property::IV):
IV::info:= "Domain of numerical property 'Interval of a number'":
IV::domtype:= (()->(Type)):
IV::type:= "IV":
IV::prop:= TRUE:
IV::propdom:= slot(property, "IV"):
//IV::autoload:= TRUE:
IV::testtype:= Type::testtype:

IV::noTrace:= {hold(print), hold(testtype), hold(domtype)}:

IV::ax:= []:

IV::new:= 
  proc()
    local iv, rg;
  begin
    if type(args()) = "id" then
      slot(Type, "Interval")(args())
    else
      if contains({"NRange", "Residue", "Rel"}, type(args(args(0))))then
        rg:= args(args(0));
        iv:= 1
      else
        rg:= Type::Real;
        iv:= 0
      end_if;
      iv:= IVnat(args(1..(args(0) - iv)));

      IV::check(iv, rg)
    end_if
  end_proc:

// 
IV::check:=
  proc(iv, rg)
  begin
    if iv = property::Null then return(property::Null) end_if;
    //if iv = Dom::Interval(-infinity, infinity) then return(rg) end_if;
    if property::implies(rg, Type::Integer) = TRUE then
      iv:= IVnat::Integer(iv);
      if property::Fail(iv) then return(iv) end_if;
    end_if;
    if type(rg) <> "NRange" and type(rg) <> "Residue" then
      rg:= property::convert_to(rg, property::NRange, hold(Sloppy))
    end_if;
    // rg can be NRange or Residue!
    //if rg::dom::sign(rg) <> {-1, 0, 1} then
    //  iv := IVnat::_and(iv, IV::sign2IV(rg::dom::sign(rg)))
    //end_if;
    Type::Interval(hold(id)([iv, rg]))
  end_proc:

IV::_check:=
  proc(iv, rg = Type::Real)
    local k;
  begin
    if type(iv) = "_or" and nops(iv) > 1 then // union
      subsop(hold(id)(IV::check(op(iv, k), rg) $ k = 1..nops(iv)), 0 = op(iv, 0), Unsimplified)
      //map(iv, IV::check, rg) // wird evaluiert!
    elif type(iv) = "IVnat" then
      IV::check(iv, rg)
    else // error
      iv
    end_if
  end_proc:

IV::sign2IV:=
  proc(s)
  begin
    if domtype(s) = DOM_INT then
      s := {s}
    end_if;
    if s union {-1, 0, 1} <> {-1, 0, 1} then
      return(IV::new(-infinity, infinity))
    end_if;
    table({-1}       = IV::new(-infinity, 0),
          {-1, 0}    = IV::new(-infinity, [0]),
          {-1, 1}    = (IV::new(-infinity, 0) or IV::new(0, infinity)),
          {-1, 0, 1} = IV::new(-infinity, infinity),
          {0}        = IV::new([0], [0]),
          {0, 1}     = IV::new([0], infinity),
          {1}        = IV::new(0, infinity)
          )[s]
  end_proc:

// TRUE if only one point
IV::isPoint:= iv->IVnat::isPoint(Arg(iv)):

IV::op:=
  proc(iv, opr = 1..3)
  begin
    op((IV, IVnat::leftB(Arg(iv)), IVnat::rightB(Arg(iv)), RG(iv)), map(opr, _plus, 1))
  end_proc:

// _in
IV::testtype_in:=
  proc(x, iv, rg)
  begin
    if testtype(x, Type::Real) then
      if testtype(x, rg) then
        bool(op([hold(_less), hold(_leequal)], IVnat::_lopen(iv) + 1)(IVnat::left(iv), x))
        and bool(op([hold(_less), hold(_leequal)], IVnat::_ropen(iv) + 1)(x, IVnat::right(iv)))
      else
        FALSE
      end_if
    else
      FAIL
    end_if
  end_proc:

// point or FAIL
IV::unequalPoint:=
  proc(iv1, iv2)
  begin
    if IVnat::leftB(Arg(iv1)) = -infinity
       and IVnat::rightB(Arg(iv2)) = infinity
       and IVnat::ropen(Arg(iv1))
       and IVnat::lopen(Arg(iv2))
       and is(IVnat::rightB(Arg(iv1)) = IVnat::leftB(Arg(iv2))) = TRUE then
      return(IVnat::rightB(Arg(iv1)))
    elif IVnat::leftB(Arg(iv2)) = -infinity
       and IVnat::rightB(Arg(iv1)) = infinity
       and IVnat::ropen(Arg(iv2))
       and IVnat::lopen(Arg(iv1))
       and is(IVnat::rightB(Arg(iv2)) = IVnat::leftB(Arg(iv1))) = TRUE then
      return(IVnat::leftB(Arg(iv1)))
    end_if;
    FAIL
  end_proc:

IV::simplify:=
  proc(ivs, rg = FAIL)
    local l, lo, r, ro, iv;
  begin
    // one interval
    //if rg <> FAIL then
    //  iv := ivs
    //else
      iv:= Arg(ivs);
      if type(iv) <> "IVnat" then
        return(iv)
      end_if;
      rg:= RG(ivs);
    //end_if;
    
    l:= IVnat::left(iv);
    lo:= IVnat::_lopen(iv);
    r:= IVnat::right(iv);
    ro:= IVnat::_ropen(iv);
    //if property::RESET = 0 then // nur interaktiv??
      if rg = Type::Real
         or (property::NRange::_or(rg, Type::Integer) = Type::Real
             and property::NRange::sign(rg) = IVnat::sign(iv)) then // real interval
	if r = infinity then
          if l = -infinity then
            return(rg)
          else
            return(property::Rel(-op([hold(_less, _leequal)], lo + 1), l))
          end_if
	elif l = -infinity then
	  return(property::Rel(op([hold(_less, _leequal)], ro + 1), r))
        end_if
      elif type(rg) = "Residue" and property::Residue::constant(rg) then
        // constant value
        return(rg)
      elif type(rg) = "Residue" and property::Residue::modulus(rg) = 1
           // implicit assignment because of lazy and, break
           and (rg:= property::Residue::NRange(rg)) = FAIL then
        // never reached
      else // ranges - forget borders +/-infinity and 0
	if r = infinity then
          if l = -infinity then
            return(rg)
	  elif l = 0 then // > 0
	    if rg::dom::sign(rg) = {-1} then
	      return(property::Null)
	    else
	      return(rg::dom::_and(op([Type::Positive, Type::NonNegative], lo + 1), rg))
            end_if
	  end_if
	elif l = -infinity then
	  if r = 0 then // < 0 or <= 0!
	    if rg::dom::sign(rg) = {1} then
	      return(property::Null)
            elif ro = 1 then
              // !!!! should be Type::NonPositive [non existing]
	      return(rg::dom::_and(Type::Real, rg))
            else
	      return(rg::dom::_and(Type::Negative, rg))
	    end_if
          end_if
        // change ranges or intervals
        elif (r:= rg::dom::sign(rg)) intersect (l:= IVnat::sign(iv)) <> {} then
          if l minus r <> {} then // rg kleiner
            case r
              of {-1} do // negative
                iv:= iv and IVnat(-infinity, 0); break
              of {0} do // zero
                return(Type::Zero)
              of {-1, 0} do // negative or zero
                iv:= iv and IVnat(-infinity, [0]); break
              of {0, 1} do // non negative
                iv:= iv and IVnat([0], infinity); break
              of {1} do // positive
                iv:= iv and IVnat(0, infinity); break
            end_case
          elif r minus l <> {} then // iv kleiner
            case l
              of {-1} do // negative
                rg:= rg and Type::Negative; break
              of {0} do // zero
                return(Type::Zero)
              of {-1, 0} do // negative or zero
                // 
                //rg:= rg and Type::Negative or Type::Zero; break
              of {0, 1} do // non negative
                rg:= rg and Type::NonNegative; break
              of {1} do // positive
                rg:= rg and Type::Positive; break
            end_case
          end_if
	end_if
      end_if;
//    end_if;
    if type(iv) <> "IVnat" then
      // could be an intersection
      ivs
    else
      IV::check(iv, rg)
    end_if
  end_proc:

IV::IVnat:=
  proc(iv)
  begin
    Arg(iv)
  end_proc:

IV::is:=
  proc(iv1, iv2)
  begin
    if type(iv1) <> "IV" then
      return(UNKNOWN)
    end_if;

    // NRange
    if type(iv2) = "NRange" then
      if property::implies(RG(iv1), iv2) = TRUE then
        return(TRUE)
      else
        return(property::implies(IV::NRange(iv1), iv2))
      end_if
    elif type(iv2) = "Rel" then
      return(IVnat::is(Arg(iv1), property::Rel::IVnat(iv2)))
    elif type(iv2) <> "IV" then
      return(UNKNOWN)
    end_if;

    return(IVnat::is(Arg(iv1), Arg(iv2)) and property::NRange::is(RG(iv1), RG(iv2)))
  end_proc:

// Type::Constant
// IV::constant:=
// 	proc(iv)
// 	begin
//           IVnat::constant(Arg(iv))
// 	end_proc:

IV::_intersect:= hold(_and):

// Schnitt von Mengen
IV::_and:=
  proc()
    local i, t, k;
  begin
    // one argument is a union of intervals
    t:= [args()];
    if (i:= contains(map(t, type), "_or")) > 0 then
      return(IV::_or(IV::_and(op(subsop(t, i = null())), op(t, [i, k])) $ k = 1..nops(t)))
    end_if;
    
    property::_Op(IV, hold(_and),
      proc(iv1, iv2)
      begin
	IV::_check(IVnat::_and(Arg(iv1), Arg(iv2)), _and(RG(iv1), RG(iv2)))
      end_proc, args())
  end_proc:

IV::_union:= hold(_or):

// Vereinigung von Mengen
IV::_or:=
  proc()
    local RET, RES, INT, i, n, k;
  begin
    // collect sets equal to one integer
    INT:= split([args()],
                X -> _lazy_or(_lazy_and(type(X) = "IV",
                                        type(IVnat::left(Arg(X))) = DOM_INT,
                                        type(IVnat::right(Arg(X))) = DOM_INT,
                                        _lazy_or(property::implies(RG(X), Type::Integer) = TRUE)),
                              _lazy_and(type(X) = "Rel",
                                        property::Rel::op(X, 0) = _equal,
                                        domtype(property::Rel::op(X)) = DOM_INT)));
    if nops(op(INT, 1)) > 1 then // at least two points/intervals
      // select/unify/sort integers
      RET:= sort(map(op(INT, 1),
                     proc(X)
                      
                       begin
                         if type(X) = "IV" then
                           // we may do this, since we have only selected
                           // intervals with integer borders
                             k $ k=IVnat::left(Arg(X))..IVnat::right(Arg(X))
                         else
                           property::Rel::op(X)
                         end_if
                       end_proc    
                           ));
      RES:= [];
      // extract corners
      n:= op(RET, 1); // left bound
      for i from 1 to nops(RET) - 1 do
        if op(RET, i) + 1 < op(RET, i + 1) then // break!
          RES:= append(RES, [n, op(RET, i)]);
          n:= op(RET, i + 1);
        end_if
      end_for;
      RES:= append(RES, [n, op(RET, i)]);
      // create intervals directly
      RES:= map(RES, X -> Type::Interval(hold(id)([IVnat::_new(op(X, 1), 1, op(X, 2), 1, TRUE),
                                                   property::NRange::_or(DOM_INT::NRange(op(X, 1)),
                                                                         DOM_INT::NRange(op(X, 2)))
                                                   ])));
      if nops(op(INT, 2)) > 0 then
        return(IV::_or(op(op(INT, 2)), op(RES)))
      else // ready
        if nops(RES) > 1 then
          return(hold(_or)(op(RES)))
        else
          return(op(RES))
        end_if
      end_if
    end_if;

    property::_Op
    (IV, hold(_or),
     proc(iv1, iv2)
     begin
       IV::_check(IVnat::_or(Arg(iv1), Arg(iv2)), _or(RG(iv1), RG(iv2)));
     end_proc, args())
  end_proc:

// Differenz-Menge
//IV::_not:=
//  proc()
//  begin
//    
//  end_proc:

IV::sign:=
  proc(iv)
  begin
    property::IVnat::sign(Arg(iv))
  end_proc:

IV::_plus:=
  proc()
    local sp;
  begin
    if hastype([args()], "_or") then
      sp:= split([args()], X -> (type(X) = "_or" and map({op(X)}, type) = {"IV"}));
      if op(sp, 1) <> [] and op(sp, 2) <> [] then
        return(map(op(op(sp, 1)), IV::_plus, op(op(sp, 2))))
      end_if
    end_if;
    property::_Op(IV, hold(_plus),
	          proc(iv1, iv2)
                  begin
                    IV::_check(Arg(iv1) + Arg(iv2), RG(iv1) + RG(iv2))
	          end_proc, args())
  end_proc:

IV::_subtract:=
  proc()
  begin
    property::_Op(IV, hold(_subtract),
	          proc(iv1, iv2)
	          begin
                    IV::_check(Arg(iv1) - Arg(iv2), RG(iv1) - RG(iv2))
	          end_proc, args())
  end_proc:

IV::_negate:=
  proc(iv)
  begin
    IV::_check(-Arg(iv), -RG(iv))
  end_proc:

IV::_mult:=
  proc()
    local sp;
  begin
    if contains(map([args()], type), "_or") > 0 then
      sp:= split([args()], X -> (type(X) = "_or" and map({op(X)}, type) = {"IV"}));
      if op(sp, 1) <> [] and op(sp, 2) <> [] then
        return(map(op(op(sp, 1)), IV::_mult, op(op(sp, 2))))
      end_if
    end_if;
    property::_Op(IV, hold(_mult),
	          proc(iv1, iv2)
                  begin
                    IV::_check(Arg(iv1)*Arg(iv2), RG(iv1)*RG(iv2))
	          end_proc, args())
  end_proc:

IV::_divide:=
  proc()
  begin
    property::_Op(IV, hold(_divide),
      proc(iv1, iv2)
      begin
        IV::_check(Arg(iv1)/Arg(iv2), RG(iv1)/RG(iv2))
      end_proc, args())
  end_proc:

IV::_invert:=
  proc(iv)
  begin
    IV::_check(1/Arg(iv), 1/RG(iv))
  end_proc:

IV::_power:=
  proc(iv, pow)
    local c;
  begin
    if type(pow) <> "IV" then
      if property::isrel(pow = 0) = TRUE or property::isrel(pow = 1) = TRUE then
        IV::_check(Arg(iv)^pow/*, RG(iv)^pow*/) // RG^pow -> errors for (0, 1)^0 !
      else
        IV::_check(Arg(iv)^pow, RG(iv)^pow)
      end_if
    elif type(iv) <> "IV" then
      if type((c:= property::convert_to(iv, IV))) = "IV" then
        return(IV::_power(c, pow))
      else
        return(hold(_power)(iv, pow))
      end_if
    else
      if contains(IV::sign(iv), -1) // can be complex
         and (RG(pow) or Type::Integer) = Type::Integer then // can be real
        // basis is integer
        if (RG(iv) or Type::Integer) = Type::Integer then // can be real
          
          // split
          //IVnat::_or()
          Type::Complex
          
        else // basis is not integer
          IV::_check(Arg(iv)^Arg(pow), RG(iv)^RG(pow))
        end_if
      else
        IV::_check(Arg(iv)^Arg(pow), RG(iv)^RG(pow))
      end_if
    end_if
  end_proc:

 IV::ln:=
   proc(iv)
     local rg;
   begin
     rg:= Arg(iv);
     if is(rg::dom::left(rg) > 0) = TRUE then
       rg:= property::IVnat::subsleft(rg, ln(rg::dom::left(rg)));
       rg:= property::IVnat::subsright(rg, ln(rg::dom::right(rg)));
       IV::_check(rg, Type::Real)
     elif iszero(rg::dom::left(rg)) then
       if rg::dom::ropen(rg) then
         property::Rel(_less, ln(rg::dom::right(rg)))
       else
         property::Rel(_leequal, ln(rg::dom::right(rg)))
       end_if
     else
       Type::Complex
     end_if     
   end_proc:

IV::abs:=
  proc(iv)
  begin
    Type::Interval(hold(id)([IVnat::abs(Arg(iv)), property::NRange::abs(RG(iv))]))
  end_proc:

IV::sqrt:=
  proc(iv)
  begin
    IV::_power(iv, 1/2)
  end_proc:

IV::NRange:=
  proc(iv, OPT = hold(Approx))
    local Ex;
  begin
    // nothing to do (should not appear)
    if type(iv) = "NRange" then
      return(iv)
    end_if;
    // first: simplification method
    if type((Ex:= IV::simplify(iv))) = "NRange" then
      return(Ex)
    // if result is a relation, try its conversion method first
    elif type(Ex) = "Rel" and type((Ex:= property::Rel::NRange(Ex, OPT))) = "NRange" then
      return(Ex)
    end_if;
    Ex:= // 
      proc(RET, NULL, INF)
      begin
        if OPT = hold(Exact) and (is(NULL = 0) <> TRUE or type(INF) <> stdlib::Infinity) then
          FAIL
        else
          RET
        end_if
      end_proc;
    case IVnat::sign(Arg(iv))
      of {1} do
        Ex(property::NRange::_and(property::NRange("Positive"), RG(iv)),
           IVnat::left(Arg(iv)), IVnat::right(Arg(iv))); break
      of {-1} do
        Ex(property::NRange::_and(property::NRange("Negative"), RG(iv)),
           IVnat::right(Arg(iv)), IVnat::left(Arg(iv))); break
      of {-1, 0} do
        // could return "IV"!
        if type((Ex:= _or(property::NRange::_and(property::NRange("Negative"),
                                                 RG(iv)),
                          Type::Zero))) <> "NRange" then
          if OPT = hold(Exact) then
            FAIL
          else
            RG(iv)
          end_if
        else
          Ex
        end_if; break
      of {0, 1} do
        property::NRange::_and(property::NRange("NonNegative"), RG(iv)); break
      otherwise
        Ex(property::NRange::_and(property::NRange("Real"), RG(iv)), 1, 1)
    end_case
  end_proc:

IV::Rel:=
  proc(iv, Opt = hold(Exact), fi = FAIL)
    local BOR, BORo, bo;
  begin
    //if IV::constant(iv) then
    //  property::Rel(_equal, IVnat::left(Arg(iv), 1))
    //end_if;
    if Opt = hold(Exact) and property::implies(RG(iv), Type::Rational) = TRUE then
      if RG(iv) = Type::Zero then
        return(property::Rel(_equal, 0))
      else
        // cannot simplify subset of Real
        return(iv)
      end_if
    end_if;
    BOR:= IVnat::left(Arg(iv)), IVnat::right(Arg(iv));
    BORo:= IVnat::_lopen(Arg(iv)) + 1, IVnat::_ropen(Arg(iv)) + 1;
    // one point
    if _equal(BOR) then
      return(property::Rel(_equal, op(BOR, 1)))
    end_if;
    fi:= contains(["<", ">", hold(Left), hold(Right)], fi); // (can 1, 2), (must 3, 4)
    if fi = 0 then
      if op(BOR, 1) = -infinity then
        fi:= 4
      elif op(BOR, 2) = infinity then
        fi:= 3
      end_if
    end_if;
    if fi <> 0 then
      bo:= op(BOR, fi);
      if fi > 2 then // nur eine Grenze!
        return(property::Rel(op([-1, 1], fi - 2)*op([hold(_less), hold(_leequal)],
                                                    op(BORo, fi - 2)),
                             op(BOR, fi - 2)))
      else
        if property::_is(bo = (op([-1, 1], fi))*infinity) = TRUE then
          return(property::Rel(op([1, -1], fi)*op([_less, _leequal], op(BORo, 3 - fi)),
                               op(BOR, 3 - fi)))
        end_if;
        bo:= op(BOR, 3 - fi);
        if property::_is(bo = (op([1, -1], fi))*infinity) = TRUE then
          return(property::Rel(op([-1, 1], fi)*op([_less, _leequal], op(BORo, fi)),
                               op(BOR, fi)))
        end_if;
        if property::_is(_equal(BOR)) = TRUE and _mult(BORo) = 4 then
          return(property::Rel(_equal, op(BOR, 1)))
        end_if
      end_if
    end_if;
    hold(_and)(property::Rel(-op([_less, _leequal], op(BORo, 1)),
                             op(BOR, 1)),
               property::Rel(op([_less, _leequal], op(BORo, 2)),
                             op(BOR, 2)))
  end_proc:

IV::Residue:=
  proc(iv, Opt = hold(Exact))
  begin
    if type(RG(iv)) = "Residue" then
      RG(iv)
    elif Opt <> hold(Exact) then
      RG(iv)
    else
      FAIL
    end_if
  end_proc:

IV::expr:=
  proc(iv, ident = FAIL)
  begin
    if ident = FAIL then
      ident:= genident("ID")
    end_if;
    iv:= IV::Rel(iv);
    if domtype(iv) = DOM_EXPR then
      map(iv, property::Rel::expr, ident)
    else
      property::Rel::expr(iv, ident)
    end_if
  end_proc:

IV::Complex:=
  proc(iv)
  begin
    property::Complex(iv, Type::Zero)
  end_proc:

IV::hull:=
  proc(iv)
  begin
    IVnat::hull(Arg(iv))
  end_proc:

IV::prop2set:=
  proc(iv)
  begin
    if type(iv) = "_or" then
      hold(_union)(map(op(iv), IV::prop2set))
    elif property::implies(RG(iv), Type::Integer) = TRUE then
      // prevent displaying as DOM_SET with all numbers as arguments
      Dom::Interval::convert(iv) intersect Z_
    elif property::implies(RG(iv), Type::Rational) = TRUE then
      Dom::Interval::convert(iv) intersect Q_
    else
      Dom::Interval::convert(iv)
    end_if
  end_proc:

unalias(IV):

// Zeichen fur 'property::Func'
property::IV::Z:= "in ":

property::IV::expr2text:=
  proc(XX)
  begin
    if Arg(XX) = property::Null then
      "property::Null"
    else
      "Type::Interval(".
      expr2text(IVnat::leftB(Arg(XX))).
      ", ".
      expr2text(IVnat::rightB(Arg(XX))).
      (if RG(XX) <> Type::Real then
         ", ".expr2text(RG(XX))
       else
         ""
       end_if).
      ")"
    end_if
  end_proc:

// property::IV::print:=
//   proc(iv)
//     //local x, i;
//   begin
//     if type(RG(iv)) = "Residue" and property::Residue::constant(RG(iv)) then
//       // numeric constant
//       property::Residue::print(RG(iv))
//     else
//       expr2text(Arg(iv))
//       ._if(RG(iv) <> Type::Real, " " . (RG(iv))::dom::Z(RG(iv)) . expr2text(RG(iv)), "")
//     end_if
//   end_proc:

property::IV::print:=
  proc(iv)
  begin
    if type(iv) = "_or" then
      hold(_union)(map(op(iv), property::IV::print))
    elif type(RG(iv)) <> "NRange" then
      hold(_intersect)(Arg(iv), (RG(iv))::dom::print(RG(iv)))
    elif property::implies(RG(iv), Type::Integer) = TRUE then
      hold(_intersect)(Arg(iv), Z_)
    elif property::implies(RG(iv), Type::Rational) = TRUE then
      hold(_intersect)(Arg(iv), Q_)
    else
      Arg(iv)
    end_if
  end_proc:

// property::IV::Content:=
//   proc(Out, iv)
//     //local x, i;
//   begin
//     Content(Out, property::IV::prop2set(iv))
//   end_proc:

unalias(RG):
unalias(Rg):
unalias(Ar):
unalias(Arg):
