// $Date: 2002/08/16 09:41:49 $ $Author: bij $ $Revision: 1.5 $

/*
   Real     - Domain of real numbers

   op(Real) - basic number domain: [real = 0, rational = 1, integer = 2]
              signs: set of 0, -1, 1 

*/

alias(Real = Set::Real):
alias(TYPE(x) = extop(x, 1)): // type
alias(Ndom(x) = extop(x, 2)): // basic number domain
alias(Sign(x) = extop(x, 3)): // set of signs

alias(REAL = 0):
alias(RATIONAL = 1):
alias(INTEGER = 2):

Set::Real:= newDomain("Set::Real"):
Real::create_dom:=hold(Set::Real):
Real::info:= "Domain of real number domains":
Real::domtype:= ()->Set:
Real::type:= "Real":
Real::prop:= TRUE: // 

Real::new:=
  proc(nd = 0, sgn = {-1, 0, 1})
  begin
    if domtype(nd) = DOM_STRING then
      nd:= contains(["rational", "integer"], nd)
           + contains(["rat", "int"], nd) + contains(["r", "i"], nd);
    elif domtype(nd) = DOM_INT then
      nd:= nd mod 3
    end_if;
    if domtype(sgn) = DOM_INT then
      sgn:= {sgn}
    elif sgn = {} then
      return(Set::Empty)
    elif sgn = {0} then
      nd:= INTEGER
    end_if;
    Set::_new("Real", nd, sgn)
  end_proc:

// implies
Real::contains:=
  proc(b1, b2)
  begin
    // Real, Real
    if Ndom(b1) >= Ndom(b2) and Sign(b1) union Sign(b2) = Sign(b2) then
      return(TRUE)
    elif Real::_and(b1, b2) = property::Null then
      return(FALSE)
    else
      UNKNOWN
    end_if
  end_proc:

// implies
Real::_implies:=
  proc(b1, b2)
  begin
    // Real, Real
    if Ndom(b1) >= Ndom(b2) and Sign(b1) union Sign(b2) = Sign(b2) then
      return(TRUE)
    elif Real::_and(b1, b2) = property::Null then
      return(FALSE)
    else
      UNKNOWN
    end_if
  end_proc:

proc()
  local k;
begin
  // convert Real object to Rectform
  for k in ["_union", "_intersect", "_minus", "_plus", "_subtract",
            "_mult", "_divide", "_power"] do
    slot(Real, k."Rectform"):=
      Set::createConversion(Set::Rectform, k, hold(Reverse)):
  end_for
end_proc():

Real::_union:=
  proc(b1, b2)
  begin
    Real::new(stdlib::min(Ndom(b1), Ndom(b2)), Sign(b1) union Sign(b2))
  end_proc:

Real::_intersect:=
  proc(b1, b2)
  begin
    Real::new(stdlib::max(Ndom(b1), Ndom(b2)), Sign(b1) intersect Sign(b2))
  end_proc:

Real::_minus:=
  proc(b1, b2)
  begin
    if Ndom(b2) > Ndom(b1) then
      Set::Minus(b1, Real::_intersectReal(b1, b2))
    elif Ndom(b1) = Ndom(b2) then
      Real::new(Ndom(b1), Sign(b1) minus Sign(b2))
    else
      b1
    end_if
  end_proc:

Real::_plus:=
  proc(b1, b2)
  begin
    if Sign(b1) union Sign(b2) = {-1, 1} then
      Real::new(stdlib::min(Ndom(b1), Ndom(b2)), {-1, 0, 1})
    else
      Real::new(stdlib::min(Ndom(b1), Ndom(b2)), Sign(b1) union Sign(b2))
    end_if
  end_proc:

Real::_subtract:=
  proc(b1, b2)
  begin
    Real::_plus(b1, -b2)
  end_proc:

Real::_negate:=
  proc(b)
  begin
    Real::new(Ndom(b), map(Sign(b), _negate))
  end_proc:

Real::_mult:=
  proc(b1, b2)
  begin
    Real::new(stdlib::min(Ndom(b1), Ndom(b2)), Sign(b1)*Sign(b2))
  end_proc:

Real::_divide:=
  proc(b1, b2)
  begin
    Real::_mult(b1, 1/b2)
  end_proc:

Real::_invert:=
  proc(b)
  begin
    if Sign(b) = {0} then
      Real::Empty
    else
      Real::new(stdlib::min(Ndom(b), 1), Sign(b) minus {0})
    end_if
  end_proc:

Real::_power:=
  proc(b, pow)
    local ret, k;
  begin
    ret:= null();
    if is(pow, Type::Even) = TRUE then
      if is(pow >= 0) = TRUE then
        return(Real::abs(b))
      else
        return(Real::_invert(Real::abs(b)))
      end_if
    end_if;
    for k in Sign(pow) do
      if k = 1 then
        if Ndom(pow) = INTEGER then
          ret:= ret, Real::_or(b, Real::abs(b))
        else
          ret:= ret, Real(REAL, Sign(b) union abs(Sign(b)))
        end_if
      elif k = -1 then
        ret:= ret, 1/Real::_power(b, Real::new(Ndom(pow), {1}))
      else // zero
        ret:= ret, Real("int", 1)
      end_if;
    end_for;
    Real::_or(ret)
  end_proc:

Real::sign:=
  proc(b)
  begin
    Sign(b)
  end_proc:

Real::abs:=
  proc(b)
  begin
    Real::new(Ndom(b), abs(Sign(b)))
  end_proc:

Real::expr2text:=
  proc(bnd)
  begin
    "Set::Real(".
    op(["\"real\"", "\"rat\"", "\"int\""], Ndom(bnd) + 1).
    ", ".
    expr2text(Sign(bnd)).")"
  end_proc:

// conversion methods
Real::Interval:=
  proc(b)
    local IV;
  begin
    if Sign(b) = {-1, 1} then
      return(Set::Minus(Real::Interval(Real::new(Ndom(b))), Set::Zero))
    else
      IV:= Set::Interval((case Sign(b)
                            of {-1} do
                              -infinity, 0; break
                            of {-1, 0} do
                              -infinity, [0]; break
                            of {0} do
                              [0]; break
                            of {0, 1} do
                              [0], infinity; break
                            of {1} do
                              0, infinity; break
                          end_case));
    end_if;
    if Ndom(b) = REAL then
      IV
    else
      Set::Intersect(IV, b)
    end_if
  end_proc:

// obsolete
Real::NRange:=
  proc(bnd)
  begin
    if Sign(bnd) = {0} then
      Type::Zero
    else
      property::NRange::_and(op([Type::Real, Type::Rational, Type::Integer], Ndom(bnd) + 1),
                             (case Sign(bnd)
                                of {1} do
                                  Type::Positive; break
                                of {0, 1} do
                                  Type::NonNegative; break
                                of {-1} do
                                  Type::Negative; break
                                of {-1, 1} do
                                  not Type::Zero; break
                                  //of {-1, 0} do
                                  //  "nonpositive "; break
                                otherwise
                                  Type::Real
                              end_case))
    end_if
  end_proc:

Real::Z:= "is ":

Real::print:=
  proc(bnd)
  begin
    if Sign(bnd) = {0} then
      "zero"
    else
      (case Sign(bnd)
         of {1} do
           "positive "; break
         of {0, 1} do
           "nonnegative "; break
         of {-1} do
           "negative "; break
         of {-1, 0} do
           "nonpositive "; break
         of {-1, 1} do
           "nonzero "; break
         otherwise
           ""
       end_case).op(["real", "rational", "integer"], Ndom(bnd) + 1)
    end_if
  end_proc:

