// $Date: 2002/08/16 09:41:49 $  $Author: bij $  $Revision: 1.2 $

//
//  Rectform - Domain of 
//            'Rectform identifiers'
//

alias(Rectform = Set::Rectform):
alias(RE(x) = extop(x, 2)):
alias(IM(x) = extop(x, 3)):
alias(_RE = 2):
alias(_IM = 3):

Rectform:= newDomain("Set::Rectform"):
Rectform::create_dom:=property::hold(Rectform):
Rectform::info:= "Properties of complex identifiers":
Rectform::domtype:= ()->Set:
Rectform::type:= "Rectform":
Rectform::prop:= TRUE:
Rectform::testtype:= Type::testtype:

Rectform::noTrace:= {hold(print), hold(testtype), hold(domtype)}:

Rectform::new:= 
  proc(re = Set::Real(), im = Set::Real())
  begin
//     if contains({Type::Imaginary, Type::Complex}, re) then
//       error("the real part can only be real")
//     elif contains({Type::Imaginary, Type::Complex}, im) then
//       error("the imaginary part can only be real")
//     end_if;
    new(Set, "Rectform", re, im)
  end_proc:

Rectform::_new:=
  proc(re = Set::Real(), im = Set::Real())
  begin
//     if contains({Type::Imaginary, Type::Complex}, re) then
//       re:= re and Type::Real
//     end_if;
//     if contains({Type::Imaginary, Type::Complex}, im) then
//       im:= im and Type::Real
//     end_if;
    new(Set, "Rectform", re, im)
  end_proc:

Rectform::_implies:= 
  proc(FU1, FU2)
    local i;
  begin
    if type(FU1) = "Real" then
      FU1:= Set::Real::Rectform(FU1)
    end_if;

    if type(FU2) = "Real" then
      FU2:= Set::Real::Rectform(FU2):
      return(property::implies(RE(FU1), RE(FU2)) and property::implies(IM(FU1), IM(FU2)))
    end_if;

    if type(FU2) <> "Rectform" then
      return(UNKNOWN)
    end_if;
    if (i:= contains([RE(FU2), IM(FU2)], FAIL)) > 0 then
      if i = 2 then
        return(property::implies(RE(FU1), RE(FU2)))
      else
        return(property::implies(IM(FU1), IM(FU2)))
      end_if
    end_if;
    
    property::implies(RE(FU1), RE(FU2)) and property::implies(IM(FU1), IM(FU2))
  end_proc:

// Uberladung
Rectform::contains:=
        proc(CP, nm)
        begin
          if type(CP) <> "Rectform" then
            FALSE
          else
            bool(property::_is(nm, CP) = TRUE)
          end_if
        end_proc:

proc()
  local k;
begin
  for k in ["_union", "_intersect", "_minus", "_plus", "_subtract",
            "_mult", "_divide", "_power"] do
    // Set::Real
    slot(Rectform, k."Real"):=
      Set::createConversion(Rectform, k):
    // Set::Interval
    slot(Rectform, k."Interval"):=
      Set::createConversion(Rectform, k):
end_for:
end_proc():

Rectform::_intersect:=
  proc(FU1, FU2)
    local re, im, ret;
  begin
    ret:= _if(RE(FU1) = Set::Real(), null(), RE(FU1)),
          _if(RE(FU2) = Set::Real(), null(), RE(FU2));
    re:= eval(_if((re:= _intersect(ret)) = universe, Set::Real(), re));
    ret:= _if(IM(FU1) = Set::Real(), null(), IM(FU1)),
          _if(IM(FU2) = Set::Real(), null(), IM(FU2));
    im:= eval(_if((im:= _intersect(ret)) = universe, Set::Real(), im));
    if property::implies(im, Type::Zero) = TRUE then return(re) end_if;
    if property::Fail(re, im) then
      return(Set::Empty)
    else
      return(Rectform::_new(re, im))
    end_if
  end_proc:

Rectform::_union:=
  proc(FU1, FU2)
    local re, im, ret;
  begin
    ret:= _if(RE(FU1) = Set::Real(), null(), RE(FU1)),
          _if(RE(FU2) = Set::Real(), null(), RE(FU2));
    re:= eval(_if((re:= _union(ret)) = {}, Set::Real(), re));
    ret:= _if(IM(FU1) = Set::Real(), null(), IM(FU1)),
          _if(IM(FU2) = Set::Real(), null(), IM(FU2));
    im:= eval(_if((im:= _union(ret)) = {}, Set::Real(), im));
    //if property::implies(im, Type::Zero) then return(re) end_if;
    return(Rectform::_new(re, im))
  end_proc:

Rectform::_plus:=
  proc(FU1, FU2)
//    local sp, const;
  begin
//     if op((sp:= split([args()], testtype, Set::Complex)), 1) <> [] then
//       const:= _plus(op(op(sp, 1)));
//       if nops(op(sp, 2)) = 1 // 
//          and type((sp:= Rectform::_plus(op(op(sp, 2))))) = "Rectform" then
//         return(property::Rectform(RE(sp)+Re(const), IM(sp) + Im(const)))
//       end_if
//     end_if;
    Rectform::_new(RE(FU1) + RE(FU2), IM(FU1) + IM(FU2))
  end_proc:

Rectform::_subtract:=
  proc(FU1, FU2)
  begin
//     if testtype(FU2, Type::Real) then
//       return(Rectform::_plus(FU1, -FU2))
//     end_if;

    Rectform::_plus(FU1, Rectform::_negate(FU2))
  end_proc:

Rectform::_negate:=
  proc(FU)
  begin
    subsop(FU, _RE = _negate(RE(FU)), _IM = _negate(IM(FU)))
  end_proc:

Rectform::_mult:=
  proc(FU1, FU2)
//    local sp, const;
  begin
//     if op((sp:= split([args()], X -> testtype(X, Type::Constant)
//                                      and testtype(X, Type::Arithmetical))), 1) <> [] then
//       const:= _mult(op(op(sp, 1)));
//       sp:= op(op(sp, 2));
//       if type((sp:= property::convert_to(Rectform::_mult(sp),
//                                             Rectform))) = "Rectform" then
//         // split [complex, non real, real]
//         if type(const) = "_mult" then
//           const:= split(const, X -> _lazy_or(domtype(X) = DOM_COMPLEX,
//                                              _lazy_and(testtype(X, Type::Real),
//                                                        UNKNOWN)))
//         else
//           const:= subsop([1, 1, 1],
//                          contains([TRUE, UNKNOWN, FALSE],
//                                   _lazy_or(domtype(const) = DOM_COMPLEX,
//                                            _lazy_and(testtype(const, Type::Real),
//                                                      UNKNOWN))) = const)
//         end_if;
//         if op(const, 3) <> 1 then // real number
//           sp:= property::Rectform(RE(sp)*op(const, 3), IM(sp)*op(const, 3))
//         end_if;
//         if op(const, 2) <> 1 then // symbolic constant or expression
//           sp:= property::Rectform(RE(sp)*op(const, 2), IM(sp)*op(const, 2))
//         end_if;
//         if (const:= op(const, 1)) <> 1 and const <> null() then // complex number
//           if op(const, 1) = 0 then
//             sp:= property::Rectform(-IM(sp)*op(const, 2), RE(sp)*op(const, 2))
//           else
//             sp:= property::Rectform(RE(sp)*op(const, 1) - IM(sp)*op(const, 2),
//                                    RE(sp)*op(const, 2) + IM(sp)*op(const, 1))
//           end_if
//         end_if;
//         return(sp)
//       end_if
//     end_if;

       Rectform::_new(RE(FU1)*RE(FU2) - IM(FU1)*IM(FU2),
                      RE(FU1)*IM(FU2) + IM(FU1)*RE(FU2))
  end_proc:

Rectform::_divide:=
  proc(FU1, FU2)
  begin
//     if testtype(FU2, Type::Real) then
//       return(Rectform::_mult(FU1, 1/FU2))
//     end_if;

    Rectform::_mult(FU1, Rectform::_invert(FU2))
  end_proc:

Rectform::_invert:=
  proc(FU)
    local x;
  begin
    x:= RE(FU)^2 + IM(FU)^2;
    if property::implies(RE(FU), Set::NonZero) = TRUE then
      Rectform::_new(RE(FU)/x intersect Set::NonZero, IM(FU)/x)
    elif property::implies(IM(FU), Set::NonZero) = TRUE then
      Rectform::_new(RE(FU)/x, IM(FU)/x intersect Set::NonZero)
    else
      Rectform::_new(RE(FU)/x, IM(FU)/x)
    end_if
  end_proc:

Rectform::_power:=
  proc(FU1, POW)
  begin // 
    if type(FU1) <> "Rectform"
       and type((FU1:= property::convert_to(FU1, Rectform))) <> "Rectform" then
      return(hold(_power)(FU1, POW))
    end_if;
    if property::_is(POW, Type::Even) = TRUE then
      if property::_is(RE(FU1), Set::Zero) = TRUE then
        if property::_is(POW, Type::Residue(0, 4)) = TRUE then
          return(IM(FU1)^POW)
        elif property::_is(POW, Type::Residue(2, 4)) = TRUE then
          return(-IM(FU1)^POW)
        end_if;
        return(hold(_or)(IM(FU1)^2, -IM(FU1)^2))
      else
        return(Set::Complex)
      end_if
    elif property::_is(POW, Set::Real()) = TRUE then
      // ???
      return(Set::Complex)
    end_if;
    hold(_power)(FU1, POW)
  end_proc:

// Rectform::simplify:=
//   proc(CP)
//   begin
//     
//   end_proc:

Rectform::Real:= 
  proc(CP)
  begin
    if property::implies(IM(CP), Set::Zero) = TRUE then
      return(coerce(RE(CP), Set::Real))
    elif property::implies(RE(CP), Set::Zero) = TRUE
         and property::implies(IM(CP), Set::Zero) = TRUE then
      return(Set::Zero)
    end_if;
    FAIL
  end_proc:

// alias(SB = solvelib::BasicSet):
// alias(DI = Dom::ImageSet):
// Rectform::prop2set:=
//   proc(CP)
//     local ID1, ID2;
//   begin
//     if IM(CP) = Type::Zero then
//       property::prop2set(RE(CP))
//     elif RE(CP) = Type::Real and IM(CP) = Type::Real then
//       SB(Dom::Rectform)
//     else
//       ID1:= genident("X");
//       ID2:= genident("Y");
//       DI(ID1 + ID2*I, [ID1, ID2], [property::prop2set(RE(CP)), property::prop2set(IM(CP))])
//     end_if
//   end_proc:
// unalias(DI):
// unalias(SB):

Rectform::Re:=
  proc(CP)
  begin
    RE(CP)
  end_proc:

Rectform::Im:=
  proc(CP)
  begin
    IM(CP)
  end_proc:


Rectform::abs:=
  proc(CP)
  begin
    sqrt(RE(CP)^2 + IM(CP)^2)
  end_proc:

Rectform::sign:=
  proc(CP)
  begin
    Rectform::_divide(CP, Rectform::abs(CP))
  end_proc:

Rectform::hull:=
  proc(CP)
  begin
    hull(RE(CP)) + I*hull(IM(CP))
  end_proc:

Rectform::print:=
  proc(XX)
    local REAL, ZERO;
  begin
    // 
    REAL:= (if RE(XX) = Set::Real() then 0 else 1 end_if) +
           (if IM(XX) = Set::Real() then 0 else 2 end_if);
    ZERO:= (if RE(XX) = Set::Zero then 0 else 1 end_if) +
           (if IM(XX) = Set::Zero then 0 else 2 end_if);
    if REAL = 0 then
      "Set::Complex"
    elif ZERO = 0 then 
      "Set::Zero"
    elif ZERO = 2 and REAL = 2 then
      "Set::Imaginary"
    elif ZERO = 1 then
      RE(XX)
    else
      (_if(REAL mod 2 = 1, ("Re(.) " .
                            (_if(Set::Z(RE(XX)) <> FAIL, Set::Z(RE(XX)), ""))
                            . expr2text(RE(XX))), "")) .
      (_if(REAL = 3, " and ", "")).
      (_if(REAL - 1 > 0, ("Im(.) " .
                          (_if(Set::Z(IM(XX)) <> FAIL, Set::Z(IM(XX)), ""))
                          . expr2text(IM(XX))), ""))
    end_if
  end_proc:

unalias(Rectform):
