

/*++


commassop - return a commutative and associative operator               

commassop(operatorname)

operatorname must be a string such that text2expr
       is an associative and commutative operator



The associative operators are realized as follows:

first, the arguments are sorted by their type. Types which do not
belong to Set are handled separately; the others are processed
in a round-robin manner.
....
                              
Pairing of two operands of different types is done as follows:
....
                       
++*/


Set::commassop:=
proc(operatorname)
  option escape;
  local Operator;
begin
  Operator:=text2expr(operatorname);
      
  // return value:
  proc()
    local sets, others, i,j, results, res, types, operands, a, b, sl;
  begin
    if args(0)=1 then
      return(args(1))
    end_if;

    sets:=split([args()], x -> domtype(x) = Set);
    others:=op(sets,2);
    sets:=op(sets,1);

    // handle arguments of other types separately

    case nops(others)
      of 0 do
      of 1 do
        break;
      otherwise
        others:=eval(Operator)(op(others));
        if type(others) = operatorname then
          others:= [op(others)]
        else
          others:=[others]
        end_if;
    end_case;
    
    // sort arguments by type
    sets:=sort(sets, (x,y) -> sysorder(type(x), type(y)));
    types:= map(sets, type);
                           

    others:= map(others, Set::coerce);
    if contains(others, FAIL) > 0 then
      error("Could not convert operand into a Set")
    end_if;
    
    operands:= sets.others;    
    results:=[];

    // round - robin
    // ?? better strategy - not all combinations

    i:=1;
    while i<nops(operands) do
      j:=i+1;
      while j<=nops(operands) do
        a:=operands[i];
        b:=operands[j];
            userinfo(20, "Trying binary operation for operands ".
                     expr2text(a)." and ". expr2text(b));           
        // entry exists ?
        if type(a) = type(b)
           and (sl:= slot(slot(Set, type(a)), operatorname)) <> FAIL then
          res:= sl(a, b)
        elif (sl:= slot(slot(Set, type(a)), operatorname.type(b))) <> FAIL then
          res:= sl(a, b)
        elif (sl:= slot(slot(Set, type(b)), operatorname.type(a))) <> FAIL then
          res:= sl(b, a)
        else
          res:= FAIL
        end_if;
     
        if res = FAIL /* or type(res)=operatorname */ then
          userinfo(10, "Simplification failed for ".
                expr2text(a)." and ".expr2text(b));
          j:=j+1
        else
          results:=append(results, res);
          delete operands[j];
          delete operands[i];
          /* by delete, the former operands[i+1]
                   has become operands[i] now, so the next
                   iteration of the outer while-loop is started
                   implicitly */
          j:=i+1
        end_if;
   
      end_while; /* inner while loop over j */
      i:=i+1
    end_while; /* outer while loop */

        // try to combine operands until no new results appear 
    if results=[] then
      results:=operands
    elif nops(results) + nops(operands) = 1 then // assert(nops(result) > 0)
      // one operand, return it
      return(op(results))
    else
      // simplification occurred: recursive call
      results:=eval(Operator)(op(results), op(operands));
      return(results)
    end_if;

    if nops(results)=0 then
      Operator()
    elif nops(results)=1 then
      op(results, 1)
    else
      // force sorting by sysorder
      subsop(hold(id)(op(sort(results))), 0 = Operator, Unsimplified)
    end_if
  end_proc

      /* end of return value */
            
end_proc;


  
proc()
  local i;
begin
for i in {"_union", "_intersect",
          "_plus", "_mult"} do
  slot(Set, i):= Set::commassop(i)
end_for
end_proc():