// $Date: 2004/06/30 12:45:20 $ $Author: stefanw $ $Revision: 1.33 $

// 06/2003, bij, stefanw
// 
// simplify::Simplifier
//   returns a simplifier procedure
// 
// local (in this file) defined help procedures for
// 
//
//
//

alias(ExpectedSimplificationPerStep = 0.98):

alias(queueinsert(QUEUE, VAL, TYPE, CONTENT) =
        (
         assert(contains({DOM_FLOAT, DOM_RAT, DOM_INT}, domtype(VAL)));
         while contains(QUEUE, VAL) do
           VAL := VAL + 0.001
         end_while;
         QUEUE[VAL] := [TYPE, CONTENT]
         )):

alias(getQueueMinimalVal(QUEUE, VAL) =
        (if nops(QUEUE) > 1 then
           VAL := stdlib::min(map(op(QUEUE), op, 1))
         elif nops(QUEUE) = 0 then
           VAL:= 10^10
         else
           VAL := op(QUEUE, [1, 1])
         end_if)
         ):

alias(LOCALPENALTY(ex, glob, depth) = float((glob/ex)^(1/(1 + depth)))):




// returns a procedure, a simplifier reacting to given options
Simplify::Simplifier:=
proc()
  option escape;
  local OPTIONS, OPTIONT, PROC;
begin

  // set default options for simplifier created without options
  // !! however, each option can be set at any call to the created simplifier
  
  OPTIONS:= table(hold(Discard)     = Simplify::exclude,
                  hold(Goal)        = 0,  // default goal
                  hold(Seconds)     = infinity,
                  hold(Steps)       = 50,
                  hold(OutputType)  = "Best",
                  hold(SelectRules) = Simplify::selectRules,
                  hold(RuleBase)    = Simplify,
                  hold(ApplyRule)   = Rule::apply,
                  hold(Strategy)    = "Default",
                  hold(Valuation)   = Simplify::defaultValuation,
                  hold(All)         = FALSE,
                  hold(AllValuation)= FALSE,
                  //hold(Debug)       = FALSE,
                  hold(Trace)       = {},
                  hold(Remember)    = TRUE,
                  hold(MaxInfoDepth) = 0,
                  hold(Equiv)       = {});

  OPTIONT:= table(hold(Discard)     = Type::Union(Type::Function, DOM_BOOL),
                  hold(Goal)        = Type::AnyType,
                  hold(Seconds)     = Type::Union(Type::PosInt, stdlib::Infinity),
                  hold(Steps)       = Type::Union(Type::PosInt, stdlib::Infinity),
                  hold(OutputType)  = DOM_STRING, // 
                  hold(SelectRules) = Type::Function,
                  hold(RuleBase)    = DOM_DOMAIN, //Type::Union(DOM_LIST, DOM_DOMAIN),
                  hold(ApplyRule)   = Type::Function,
                  hold(Strategy)    = DOM_STRING,
                  hold(Valuation)   = Type::Function,
                  hold(All)         = DOM_BOOL,
                  hold(AllValuation)= DOM_BOOL,
                  //hold(Debug)       = DOM_BOOL,
                  hold(Trace)       = DOM_SET,
                  hold(Remember)    = DOM_BOOL,
                  hold(MaxInfoDepth) = Type::Union(Type::NonNegInt, stdlib::Infinity),
                  hold(Equiv)       = DOM_SET);

  // combine default options and user defined options
  OPTIONS:= prog::getOptions(OPTIONS, OPTIONT, Error);

  ///////////////////////////////////////////////////////
  //  begin of  S I M P L I F I E R  ////////////////////
  ///////////////////////////////////////////////////////
  PROC := 
   proc(EXPR)                // expression to simplify
     name Simplify;
     local SFUNC,
     valuation,
     paireval,
     //queueinsert,
     //getQueueMinimalVal,
     proofout,
     improveRules: DOM_PROC,
     exclude,
     selectRules,      // procedure, that select rules for given pattern
     ruleBase,         // collection of rules, used as first argument of select rules
     apply,            // procedure, that applies given rule to given expression
     strategy,
     expressions: DOM_TABLE, // expressions[EXPR]: list of all expressions
                            // equivalent to EXPR
     proofs: DOM_TABLE,      // proofs[EXPR][i]: a proof that
                            // expressions[EXPR][i] is equivalent to EXPR
                            // such proof can be recognized by its proof type 
                            // (first list entry).
                            // There are the following kinds of proofs:
                            // a: ["axiom"] (i=1)
                            // b: ["global", k, rule]
                            //    rule has been applied 
                            //    to  the k-th equivalent expression as a whole
                            // c: ["subexpr", k, [i1, ..,im ]]
                            //    (where m = nops(EXPR[k])) 
                            //    in the k-th equivalent exprssion to EXPR, 
                            //    the j-th operand has been replaced by the
                            //    ij-th expression equivalent to it
                            // d: ["subsum", k, subexpr, rule]
                            //    apply rule to subexpr, where subexpr
                            //    is a subsum(or -product) of the k-th
                            //    equivalent expression to EXPR
  
     RESULTS: DOM_TABLE,     // the same, but with valuation attached to each
                            // expression
     simplifier,
     result,
     i,
     evalSubexpression,
     createSimplifier: DOM_PROC,
     NEXTVAL,
     SIMPLIFIERS: DOM_TABLE, // SIMPLIFIERS[EXPR]: simplification method for EXPR
     OPTS:        DOM_TABLE,
     maxSteps, steps, maxTime, TIME, failed, matches;
  
     option escape;
  
   begin
     // set default options by global default options and options given to top call
     // get given options
     OPTS:= prog::getOptions(OPTIONS, OPTIONT, 1, Error);
     
     if OPTS[hold(Trace)] <> {} then
       if OPTS[hold(Trace)] = TRUE then
         OPTS[hold(Trace)] := {hold(ApplyRule), hold(SelectRules), hold(Valuation)}
       end_if;
       // trace some functions
       for i in OPTS[hold(Trace)] do
         OPTS[i] := prog::trace(OPTS[i], hold(Proc), hold(Name) = expr2text(i))
       end_for
     end_if;
     
     // temporary equivalent expressions
     if OPTS[hold(Equiv)] <> {} then
       if map(OPTS[hold(Equiv)], type) <> {"_equal"} then
         error("illegal argument given for option 'Equiv'")
       end_if;
       
         sysassign(Simplify::tempRules,
                   subs(proc(t)
                          local k;
                          option escape;
                        begin
                          [
                           (if t = type(op(k, 1)) then
                              Rule(subs(proc() // replace left side by right side
                                          name Simplify::tempRule1;
                                        begin
                                          if args() = `#LEFT` then
                                            `#RIGHT`
                                          else
                                            FAIL
                                          end
                                        end,
                                        `#LEFT` = op(k, 1), `#RIGHT` = op(k, 2)))
                            else
                              null()
                            end_if,
                            if t = type(op(k, 2)) then
                              Rule(subs(proc() // replace right side by left side
                                          name Simplify::tempRule2;
                                        begin
                                          if args() = `#LEFT` then
                                            `#RIGHT`
                                          else
                                            FAIL
                                          end
                                        end,
                                        `#LEFT` = op(k, 2), `#RIGHT` = op(k, 1)))
                            else
                              null()
                            end_if) $ k in `#EQU`]
                        end_proc, `#EQU` = OPTS[hold(Equiv)]))
     end_if;
     
     valuation:=   OPTS[hold(Valuation)];
     selectRules:= OPTS[hold(SelectRules)];
     ruleBase:=    OPTS[hold(RuleBase)];
     apply:=       OPTS[hold(ApplyRule)];
     strategy:=    OPTS[hold(Strategy)];
     maxTime:=     OPTS[hold(Seconds)];
     if maxTime <> infinity then
       maxSteps:=  infinity
     else
       maxSteps:=  OPTS[hold(Steps)]
     end_if;
     exclude:=     OPTS[hold(Discard)];
     
     paireval:= Simplify::paireval;
     
     SFUNC:= (X, Y) -> op(X, 2) < op(Y, 2) or
             op(X, 2) = op(Y, 2) and sysorder(op(X, 1), op(Y, 1));
     
     RESULTS:= expressions:= proofs:= NEXTVAL:= SIMPLIFIERS:= table();
    
    // local methods
                                    
    //                                
    // proofout(out, EXPR) - use the table proof to generate 
    //                       a proof for out from EXPR
    // out is the result, EXPR the simplified input expression
                                    
    proofout:=                                
    proc(out, EXPR)
      name Simplify::proofout;
      local i, j, proofout_recursive, makeLemma;
    begin
      i:= contains(expressions[EXPR], out);
      if i=0 then
        return("No proof available \n")
      end_if;
          
      proofout_recursive:=
      proc(i, EXPR)
        local laststep;
      begin
      
        laststep:= proofs[EXPR][i];
        
        case laststep[1]      
          of "axiom" do
            assert(i=1);
            assert(expressions[EXPR][1] = EXPR);
            "Input was ".expr2text(EXPR).". \n";
            break;
          of "global" do
            proofout_recursive(laststep[2], EXPR).
            "Applying the rule ".expr2text(laststep[3])." to ".
            expr2text(expressions[EXPR][laststep[2]])." gives ".
            expr2text(expressions[EXPR][i])." \n";
            break;
          of "subexpr" do
            proofout_recursive(laststep[2], EXPR).
            _concat(makeLemma(laststep[3][j], 
                                                 op(expressions[EXPR]
                                                    [laststep[2]], j))
                    $j=1..nops(laststep[3])). 
            "substituting gives ".expr2text(expressions[EXPR][i])." \n";
            break
          of "subsum" do
            proofout_recursive(laststep[2], EXPR).
            "Applying rule ".expr2text(laststep[4])." to subexpression ".
            expr2text(laststep[3])." gives ".
            expr2text(expressions[EXPR][i])." \n";
            break;
          otherwise
            error("Unknown type of proof step")
        end_case;
      end_proc;
      
      // creates a lemma of the form:
      // subexpression is equivalent to expressions[subexpression][newindex]
      makeLemma:=
      proc(newindex, subexpression)
      begin
        if newindex = 1 then
          // no lemma is necessary for the rivial statement
          // that an expression is equal to itself
          return("")
        end_if;
        
        "Lemma: ".
        expr2text(subexpression = expressions[subexpression][newindex]).
        "\n".
        proofout_recursive(newindex, subexpression).
        "End of lemma \n"
      end_proc;
      
      stdlib::Exposed(proofout_recursive(i, EXPR)."END OF PROOF \n")
      
    end_proc;
                                    
    // improve the valuation/priority of all rules, that
    // leads to good simplification
    // the path of each simplification is stored in proofs[EXPR]
    improveRules:=
      proc(RESULT, EXPR, VAL = 1.0)
        local i, j, improveRules_REC;
      begin
        i:= contains(expressions[EXPR], RESULT);
        if i=0 then
          fprint(0, ">> no proof for", EXPR);
          return()
        end_if;
            
        improveRules_REC:=
          proc(i, EXPR, depth = 0)
            local laststep;
          begin
            // maximal depth
            if depth > 4 then return() end_if;
    
            laststep:= proofs[EXPR][i];
            
            case laststep[1]      
              of "axiom" do
                // do nothing
                break;
              of "global" do
                improveRules_REC(laststep[2], EXPR, depth + 1);
                assert(contains(Rule::VAL, laststep[3]));
                if not contains(Rule::VAL, laststep[3]) then
                  warning("rule ".expr2text(laststep[3])." failed")
                end_if;
                sysassign(Rule::VAL[laststep[3]], [(Rule::VAL[laststep[3]][1]*VAL)^0.5,
                                                   Rule::VAL[laststep[3]][2] + 1,
                                                   // keep statistics
                                                   Rule::VAL[laststep[3]][3], Rule::VAL[laststep[3]][4]]);
                break;
              of "subexpr" do
                improveRules_REC(laststep[2], EXPR, depth + 1);
                break
              of "subsum" do
                improveRules_REC(laststep[2], EXPR, depth + 1);
                assert(contains(Rule::VAL, laststep[4]));
                if not contains(Rule::VAL, laststep[4]) then
                  warning("rule ".expr2text(laststep[4])." failed")
                end_if;
                sysassign(Rule::VAL[laststep[4]],
                          [(Rule::VAL[laststep[4]][1]*VAL)^0.5,
                           Rule::VAL[laststep[4]][2] + 1,
                           // keep statistics
                           Rule::VAL[laststep[4]][3],
                           Rule::VAL[laststep[4]][4]]);
                break;
              otherwise
                error("Unknown type of proof step")
            end_case;
          end_proc;
  
        improveRules_REC(i, EXPR)
        
      end_proc;
  
       
  
    // local method evalSubexpression
    // evalSubexpression(ex, i, totalval)
    // 
    // determine the priority of doing the first simplification step
    // on the i-th subexpression of ex
    // totalval = valuation(ex)
  
    
    evalSubexpression:=
      proc(ex, i, totalval)
        local valuat;
      begin
        valuat:= valuation(op(ex, i));
        // upper and lower bound
        if valuat < 0.1*totalval then
           valuat:= 0.1*totalval
        elif valuat > totalval then
           // should not happen
           valuat:= totalval
        end_if;
        totalval - (1 - ExpectedSimplificationPerStep) * valuat;
      end_proc;


    
    ///////////////////////////////////////////////////////
    // internal utility function
    ///////////////////////////////////////////////////////
    // createSimplifier(EXPR)
    //
    // returns a procedure that
    //
    // appends result to RESULTS[EXPR]
    // sets              NEXTVAL[EXPR] to next valuation in queue
    //
    // returns
    // - null() if some internal step has been carried out
    //          that has not produced any new equivalent result
    // - FAIL   if the internal queue is empty and no further
    //          equivalent results can be produced in the future
    // - TRUE   if a new equivalent result has been produced
    //
      
    createSimplifier:=
    proc(EXPR)
      name Simplify::createSimplifier;
      option escape;
      local QUEUE, // queue of tasks to do; see below
            MINVAL, // minimum complexity of expression equivalent to EXPR
            RET,  //
            RULES, //
            SUBPTR: DOM_TABLE, // SUBPTR[expression] is a list of integers
                               // SUBPTR[expression][i] = j means:
                               // replacing the 1st to jth equivalent expression
                               // for the ith operand has already been tried
            SUBEXPRS, //
            currentval, beenthere, overloading;
    begin
      userinfo(20, "Creating simplifier for ".expr2text(EXPR));
  

      // initializations
      MINVAL:= valuation(EXPR);
      
      RESULTS[EXPR]:= [[EXPR, MINVAL]];
      expressions[EXPR]:= [EXPR];
      proofs[EXPR]:= [["axiom"]];
      //QUEUE:= [[0, EXPR, ExpectedSimplificationPerStep*MINVAL]];
      /*%*/ QUEUE := table(ExpectedSimplificationPerStep*MINVAL = [0, EXPR]);
      //# QUEUE := adt::Heap();
      //# QUEUE::insert(ExpectedSimplificationPerStep*MINVAL, [0, EXPR]);
      
      beenthere:=[];
            
      // QUEUE is a table of tasks that we have to do
      // Types of tasks:
      // 0 - the queue element is an expression, and
      //     we have to find all applicable rules
      //
      // 1 - the queue element is a pair [expression, rule]
      //     we have to apply the rule to the expression
      //
      // 2 - the queue element is a list [i, expression]
      //     Let subexpression:= op(expression, i)
      //     we have to produce another expression equivalent
      //     to subexpression ---
      //     either by calling the simplifier or by getting the next
      //     unused result ---
      //     and replace the i-th operand by that expression
      //     and all other operands by expressions equivalent
      //     to them, in all possible ways
      //     SUBPTR[i] is the current position in RESULTS[expression]
      //    
      // 3 - the queue element is a list [expression, rule, addargs]
      //     where addargs is a list
      //     we have to apply rule to the expression; if this succeeds
      //     and produces result, 
      //     we obtain a new expression op(expression, 0)(result, op(addargs))


      /************************************************************
        return value if overloaded:
        a procedure that creates only the task to run
        overloading and always accept its result
      *************************************************************/

      if (overloading:= EXPR::dom::Simplify) = FAIL then
        overloading:= EXPR::dom::simplify
      end_if;

      if overloading = FAIL and domtype(eval(op(EXPR, 0))) = DOM_FUNC_ENV then
        overloading:= slot(eval(op(EXPR, 0)), "Simplify")
      end_if;
      
      if overloading <> FAIL and
        domtype(overloading) <> DOM_LIST then
        
        return
        (
         proc(depth = 0: DOM_INT)
           name Simplify::SimplifyOneStepOverloaded;
           local uinfo, VAL, TYPE, CONTENT;
         begin
           assert(overloading <> FAIL);

           if depth <= OPTIONS[hold(MaxInfoDepth)] then
             uinfo:= userinfo
           else
             uinfo:= null()
           end_if;

           
           if nops(QUEUE) = 0 then
             //# if QUEUE::nops() = 0 then
             // nothing to do
             uinfo(Text, 10, "Queue for ", EXPR, " is empty");
             return(FAIL)
           end_if;
           assert(nops(QUEUE) = 1);
           getQueueMinimalVal(QUEUE, VAL);
           [TYPE, CONTENT]:= QUEUE[VAL];
           delete QUEUE[VAL]; // remove "minimal" element
    
     
   
           if exclude(VAL, MINVAL, depth) then
             return(FAIL)
           end_if;

         if depth > 0 then
           uinfo(Text, 10, "Depth is ", depth)
         end_if;
         uinfo(Text, 10, "Working on rule ", [TYPE, CONTENT], " ", [VAL]);
         uinfo(NoNL, 5, Simplify::queueprint(QUEUE), "");
  
         uinfo(Text, 5, "Queue element is: ", VAL = [TYPE, CONTENT]);

         case TYPE
  
            of 0 do
                
           
             beenthere:= append(beenthere, CONTENT);
              // overloadimng has highest priority
             queueinsert(QUEUE, 0.0, 1,
                         [Rule(X -> overloading(X)), EXPR]);
             NEXTVAL[EXPR] := 0.0;
             uinfo(Text, 10, "Internal step finished for ", EXPR);
             return();
    

           of 1 do
             RET:= overloading(EXPR);
             if RET <> FAIL and contains(expressions[EXPR], RET) = 0 then
               
               expressions[EXPR]:= append(expressions[EXPR], RET);
               proofs[EXPR] := append(proofs[EXPR],
                                      ["global",
                                       1,
                                       Rule(X -> overloading(X))
                                       ]
                                      ):

               RESULTS[EXPR]:= append(RESULTS[EXPR], [RET, 0]);
  
                // no queueinsert: accept the result as optimal
          
               NEXTVAL[EXPR] := 10^10;
        
              
                uinfo(Text, 10, "Obtained equivalent expression ", RET,
                      " for ", EXPR);
               return(TRUE)
                
              else
             
          
                NEXTVAL[EXPR] := 10^10;
          
              
                uinfo(10,
                       if RET = FAIL then
                           "Overloading method returned FAIL"
                       else
                           "Overloading produced already known result"
                       end_if
                      );
                return()
              end_if;

              // NOT REACHED
              assert(FALSE)   
           end_case
         end_proc
         )

      end_if;



      /*************************************************************
        return value:
        procedure simplifyOneStep
        arguments:
        depth  - nonnegative integer: number of calls to simplifyOneStep that
                 lie below the current one in the dynamical execution stack.


       ************************************************************/
      proc(depth = 0: DOM_INT)
        name Simplify::simplifyOneStep;
        local ptr, L, xpr, ex, chooseSubsets, k, v, c, t, uinfo,
              VAL, TYPE, CONTENT; // first element in queue, to do next
      begin
        if depth <= OPTIONS[hold(MaxInfoDepth)] then
          uinfo:= userinfo
        else
          uinfo:= null()
        end_if;

         
        if EXPR::dom::simplify = FAIL and
        not contains({DOM_EXPR, DOM_SET, DOM_LIST}, domtype(EXPR))
        then
        //if domtype(EXPR) <> DOM_EXPR then
          uinfo(Text, 10, EXPR, " is not an expression, do not simplify");
          return(FAIL)
        end_if;
                
        /*%*/ if nops(QUEUE) = 0 then
        //# if QUEUE::nops() = 0 then
          // nothing to do
          uinfo(Text, 10, "Queue for ", EXPR, " is empty");
          return(FAIL)
        end_if;
  
        /*%*/ getQueueMinimalVal(QUEUE, VAL);
        /*%*/ [TYPE, CONTENT]:= QUEUE[VAL];
        /*%*/ delete QUEUE[VAL]; // remove "minimal"" element
        //# [VAL, TYPE] := QUEUE::min_pair();
        //# [TYPE, CONTENT] := TYPE;
        //# QUEUE::delete_min();
        
        if exclude(VAL, MINVAL, depth) then
          // since the queue is sorted, all elements
          // following QUELEM may also be discarded
          return(FAIL)
        end_if;

        if depth > 0 then
          uinfo(Text, 10, "Depth is ", depth)
        end_if;
        uinfo(Text, 10, "Working on rule ", [TYPE, CONTENT], " ", [VAL]);
        uinfo(NoNL, 5, Simplify::queueprint(QUEUE), "");
  
        uinfo(Text, 5, "Queue element is: ", VAL = [TYPE, CONTENT]);
        
        case TYPE
  
          of 0 do
                
            //assert(contains(beenthere, CONTENT) = 0);
            beenthere:= append(beenthere, CONTENT);
            // select rule base
  
            RULES:= selectRules(ruleBase, CONTENT, bool(depth = 0), strategy);

            // since we have to apply many rules, use a remember
            // mechanism for combinat::choose
  
            chooseSubsets:=
            proc(k)
              option remember;
            begin
              {op(combinat::subsets::list({op(CONTENT)}, k))}
            end_proc;
            
            RULES:= map(RULES,
                        proc(rule)
                          local k, n, oper;
                        begin
                          if Rule::result(rule) <> FAIL and // procedures not!
                             (k:= nops(Rule::pattern(rule))) < (n:= nops(CONTENT)) and
                            // !!!! not??
                            contains(Simplify::comm intersect Simplify::ass,
                                     op(Rule::pattern(rule), 0)) then
                            // partial sums/products/.. that have
                            // exactly as many operands as the pattern
                            if k > n then
                              // do nothing
                              return(null())
                            else
                              oper:= op(CONTENT, 0);
                              // form all k-element subsets of the set of
                              // operands of the expression
                              // for each such subset S, generate the
                              // task to simplify the partial expression
                              // consisting of the elements of S
                              // note that we assume that no operand can
                              // appear twice in the expression!
                              
                              op(map
                                 (
                                  chooseSubsets(k),
                                  proc(S)
                                  begin
                                    [paireval(Rule::priority
                                              (rule, strategy, valuation),
                                              VAL),
                                     3, [rule, oper(op(S)),
                                         [op({op(CONTENT)} minus S) ]]
                                     // this should be improved:
                                     ]
                                  end_proc
                                  ))
                            end_if
                          else
                            [paireval(Rule::priority(rule, strategy, valuation),
                                      VAL),
                             1,
                             [rule, CONTENT]]
  
                          end_if
                        end_proc);
  
            if contains({DOM_EXPR, DOM_SET, DOM_LIST}, domtype(CONTENT)) then
              
            // Konfigurierbar machen !!!!!
              SUBEXPRS:= map([$1..nops(CONTENT)],
                             proc(i: DOM_INT)
                             begin
                               [evalSubexpression(CONTENT, i, VAL),
                                2,
                                [i, CONTENT]]
                             end_proc);
  
            // SUBPTR[i] = j means: the 1st to jth equivalent expression
            // to op(QUELEM[2], i) have already been used
              SUBPTR[CONTENT]:= [1 $ nops(CONTENT)];
  
            else
  
              SUBEXPRS:= []
  
            end_if;

            uinfo(Text, 10, "Appending ", nops(RULES),
                  " rule-application tasks and ", nops(SUBEXPRS),
                  " subexpression-tasks");
            
            //QUEUE:= sort(QUEUE.RULES.SUBEXPRS, (x, y) -> x[3] <= y[3] );
            /*%*/ ([v, t, c] := k; queueinsert(QUEUE, v, t, c)) $ k in RULES;
            /*%*/ ([v, t, c] := k; queueinsert(QUEUE, v, t, c)) $ k in SUBEXPRS;
            //# QUEUE::insert(op(k, 1), [op(k, 2..3)]) $ k in RULES;
            //# QUEUE::insert(op(k, 1), [op(k, 2..3)]) $ k in SUBEXPRS;
            //if nops(QUEUE) > 0 then
            //  NEXTVAL[EXPR] := QUEUE[1][3];
            //else
            //  NEXTVAL[EXPR] := infinity
            //end_if;
            /*%*/ getQueueMinimalVal(QUEUE, VAL);
            //# [VAL, TYPE] := QUEUE::min_pair();
            NEXTVAL[EXPR] := VAL;
                  
            // only internal step
            uinfo(Text, 10, "Internal step finished for ", EXPR);
            return()
  
          of 1 do
            // TYP = 1
            // apply the rule CONTENT[1] to the expression CONTENT[2]
  
            RET:= apply(op(CONTENT));         
              
            if RET <> FAIL and contains(expressions[EXPR], RET) = 0 then
  
              matches:= matches + 1;
              
              expressions[EXPR]:= append(expressions[EXPR], RET);
              assert(contains(expressions[EXPR], CONTENT[2]) > 0);
              proofs[EXPR] := append(proofs[EXPR],
                                     ["global",
                                      contains(expressions[EXPR], CONTENT[2]),
                                      CONTENT[1]
                                      ]
                                    ):
              currentval:= valuation(RET);
  
              //
              // if currentval/VAL < 1.0 then
              //   improveRules(RET, EXPR, currentval/VAL)
              // else
              //   improveRules(RET, EXPR, 1.0);
              //   //warning("rule application with val ".expr2text(currentval/VAL))
              // end_if;
  
              if currentval < MINVAL then
                MINVAL:= currentval
              end_if;
  
              RESULTS[EXPR]:= append(RESULTS[EXPR], [RET, currentval]);
  
              if not exclude(currentval, MINVAL, depth) then
                // QUEUE insert
                /*%*/ queueinsert(QUEUE, currentval, 0, RET)
                //# QUEUE::insert(currentval, [0, RET])
              end_if;
  
              /*%*/ if nops(QUEUE) > 0 then    
                /*%*/ getQueueMinimalVal(QUEUE, VAL);
              //# if QUEUE::nops() > 0 then
              //#   [VAL, TYPE] := QUEUE::min_pair();
                NEXTVAL[EXPR] := VAL
              else
                NEXTVAL[EXPR] := 10^10
              end_if;
              
              assert(RET <> FAIL);
              uinfo(Text, 10, "Obtained equivalent expression ", RET,
                       " for ", EXPR);
              return(TRUE)
            else
             
              /*%*/ if nops(QUEUE) > 0 then    
                /*%*/ getQueueMinimalVal(QUEUE, VAL);
              //# if QUEUE::nops() > 0 then
              //#   [VAL, TYPE] := QUEUE::min_pair();
                NEXTVAL[EXPR] := VAL
              else
                NEXTVAL[EXPR] := 10^10
              end_if;
              
              failed:= failed + 1;
              uinfo(10,
                       if RET = FAIL then
                           "Rule did not match"
                       else
                           "Rule produced already known result"
                       end_if
                       );
              return()
            end_if;
  
                  // NOT REACHED
            assert(FALSE)
  
          of 2 do
            
            ptr:= CONTENT[1];
            xpr:= CONTENT[2];
            ex:= op(xpr, ptr);
                 
                  
            if not contains(SIMPLIFIERS, ex) then
              SIMPLIFIERS[ex]:= createSimplifier(ex)
            end_if;
  
            if SUBPTR[CONTENT[2]][ptr] >= nops(RESULTS[ex]) then
              // get another expression equivalent to ex
  
                    
              RET:=SIMPLIFIERS[ex](depth + 1);
              // if RET=FAIL, do nothing: do not call
              // the simplifier again
              if RET = null() then
                // nothing has been produced; call the
                // simplifier again later
  
                assert(type(CONTENT[1]) = DOM_INT);

                v :=
max(0.1* VAL, VAL - valuation(ex)) +
                NEXTVAL[ex] *
LOCALPENALTY(valuation(ex), valuation(xpr), depth);
                
                /*%*/ queueinsert(QUEUE, v, 2, CONTENT);
                //# QUEUE::insert(v, [2, CONTENT])
                                
              elif RET <> FAIL then
                assert(RET = TRUE);
                SUBPTR[CONTENT[2]][ptr]:= SUBPTR[CONTENT[2]][ptr] + 1;
              end_if
            else
              // use known result
              SUBPTR[CONTENT[2]][ptr] := SUBPTR[CONTENT[2]][ptr] + 1;
              
              RET:= TRUE; // dummy value
            end_if;
                          
            if RET <> FAIL and RET <> null() then
              assert(RET = TRUE);
              // we have to replace each subexpression by
              // an equivalent expression, in all possible ways
              // therefore, we create a set L of lists
              // [n1, ..., nk] denoting the task to replace, for each i,
              // the i-th operand by the n_i-th expression equivalent to
              // it
              // ptr is number of the operand for which we just
              // created another equivalent expression, we always
              // use that newly created expression
                    
              L:= combinat::cartesianProduct
              ({$1..SUBPTR[CONTENT[2]][i]} $i=1..ptr-1,
               {SUBPTR[CONTENT[2]][ptr]},
               {$1..SUBPTR[CONTENT[2]][i]} $i=ptr+1..nops(CONTENT[2]));
                                
  
              L:= map
                  (L,
                   proc(list)
                     local result, valuat, indexlist;
                           
                   begin
                     indexlist:= select([$1..nops(list)],
                                        i -> contains(expressions,
                                                      op(xpr, i)));
                     if domtype(xpr) = DOM_SET then
                       // avoid subsop for sets !!
                       result:={op(eval(subsop
                                        ([op(xpr)],
                                         i = expressions[op(xpr, i)][list[i]]
                                         $i in indexlist)))}
                     else
                       result:=eval(subsop(xpr,
                                           i = expressions[op(xpr, i)][list[i]]
                                           $i in indexlist))
                     end_if;
                     assert(contains(expressions, EXPR));
                     assert(contains(RESULTS, EXPR));
                     if contains(expressions[EXPR], result) = 0 then
                       expressions[EXPR]:= append(expressions[EXPR],
                                                  result);
                       proofs[EXPR]:= append(proofs[EXPR], 
                                             ["subexpr",
                                              contains(expressions[EXPR], 
                                                       CONTENT[2]),
                                              list
                                              ]); 
                       valuat:= valuation(result);
                       RESULTS[EXPR]:= append(RESULTS[EXPR],
                                              [result, valuat]);
  
                       [ExpectedSimplificationPerStep * valuat, 0, result] // 
                     else
                       null()
                     end_if
                   end_proc
                   );
  
              assert(not hastype(L, DOM_TABLE));
  
              // now L contains everything we want to do
              // with the newly generated expressions
  
              // we also want to try simplifying
              // the i-th subexpression another time
  
            /*  QUEUE:= queueinsert(QUEUE,
                                  [2, QUELEM[2],
                                   QUELEM[3] - valuation(ex)+
                                   NEXTVAL[ex]]); */
  
              assert(type(CONTENT[1]) = DOM_INT);
  
              v :=
max(0.1*VAL, VAL - valuation(ex)) + NEXTVAL[ex]*LOCALPENALTY(valuation(ex), valuation(xpr), depth) ;

              // new formula -> error in some cases
              //v:= VAL - (valuation(ex) - NEXTVAL[ex])*valuation(subsex(xpr, ex = 1 ));
              /*%*/ queueinsert(QUEUE, v, 2, CONTENT);
              //# QUEUE::insert(v, [2, CONTENT]);
              
               //QUEUE:= queueinsert(QUEUE,
               //                     [2, QUELEM[2],
               //                      max(0.1* QUELEM[3],
               //                      QUELEM[3]/ExpectedSimplificationPerStep
               //                          - valuation(ex)) +
               //                      NEXTVAL[ex]]);

              uinfo(Text, 10, "Obtained ", nops(L),
                    "new results in subexpression step"); 
              // insert all new ... from L
              /*%*/ ([v, t, c] := k; queueinsert(QUEUE, v, t, c)) $ k in L;
              //# QUEUE::insert(op(k, 1), [op(k, 2..3)]) $ k in L;
              
              //QUEUE:= sort(QUEUE.L, (x, y) -> x[3] <= y[3]);
              if nops(L) = 0 then
                RET:= null()
              else
                assert(RET=TRUE);
              end_if;
            end_if;
  
            /*%*/ if nops(QUEUE) > 0 then
              /*%*/ getQueueMinimalVal(QUEUE, VAL);
            //# if QUEUE::nops() > 0 then
            //#   [VAL, TYPE] := QUEUE::min_pair();
              NEXTVAL[EXPR] := VAL
            else
              NEXTVAL[EXPR] := 10^10
            end_if;
  
            if RET= FAIL then
              return()
            end_if;
            
            uinfo(Text, 10, "Finished subexpression-step for ", EXPR);
            return(RET)
  
          of 3 do
  
            RET:= apply(CONTENT[1], CONTENT[2]);
            
            if RET <> FAIL and
              // add the additional operands
              (RET:= eval(op(CONTENT[2], 0)(RET, op(CONTENT[3])));
               contains(expressions[EXPR], RET) = 0) then
              
              matches:= matches + 1;
  
              expressions[EXPR]:= append(expressions[EXPR], RET);
              assert(contains(expressions[EXPR], 
                              eval(op(CONTENT[2], 0)(op(CONTENT[2]), 
                                                  op(CONTENT[3])))) 
                     > 0);
  
              proofs[EXPR]:=append(proofs[EXPR], 
                                   ["subsum",
                                    contains(expressions[EXPR], 
                              eval(op(CONTENT[2], 0)(op(CONTENT[2]), 
                                                     op(CONTENT[3])))),
                                    CONTENT[2],
                                    CONTENT[1]
                                    ] 
                                   );
              currentval:= valuation(RET);
              
              // 
              // if currentval/VAL < 1.0 then
              //   improveRules(RET, EXPR, currentval/VAL)
              // else
              //   improveRules(RET, EXPR, 1.0, -1);
              //   //warning("rule application with val ".expr2text(currentval/VAL))
              // end_if;
  
              if currentval < MINVAL then
                MINVAL:= currentval
              end_if;
  
              RESULTS[EXPR]:= RESULTS[EXPR]. [[RET, currentval]];
              
              if not exclude(currentval, MINVAL, depth) then
                /*%*/ queueinsert(QUEUE, currentval, 0, RET)
                //# QUEUE::insert(currentval, [0, RET])
              end_if;
              
              /*%*/ getQueueMinimalVal(QUEUE, VAL);
              //# [VAL, TYPE] := QUEUE::min_pair();
              NEXTVAL[EXPR] := VAL;
  
              assert(RET <> FAIL);
              uinfo(Text, 10, "Obtained equivalent expression ", RET,
                       " for ", EXPR);
              return(TRUE)
  
            else
              /*%*/ if nops(QUEUE) > 0 then
                /*%*/ getQueueMinimalVal(QUEUE, VAL);
              //# if QUEUE::nops() > 0 then
              //#   [VAL, TYPE] := QUEUE::min_pair();
                NEXTVAL[EXPR] := VAL;
              else
                NEXTVAL[EXPR] := 10^10
              end_if;
              
              failed:= failed + 1;
              uinfo(10, "Rule did not match");
              return()
  
            end_if;
            
        end_case;
      end_proc    // returned procedure simplifyOneStep
                          
    end_proc;   // createSimplifier
  
    ////////////////////////////////////////////////////////////////////////
    // main program
  
    steps:= 0;
    if maxTime <> infinity then
      TIME:= time();
      maxTime:= 1000*maxTime + TIME
    else
      TIME:= 0
    end_if;
    failed:= 0;
    matches := 0;
    
    simplifier:= createSimplifier(EXPR);
  
    SIMPLIFIERS[EXPR]:= simplifier;
  
    while steps < maxSteps and TIME <= maxTime do
      steps:= steps + 1;
      TIME := time();
      userinfo(8, "**************************************************\nStarting step ".expr2text(steps));
      result:= simplifier();
      if result = FAIL then
        userinfo(2, "Simplification finished because of empty queue after ".
                    expr2text(steps)." steps");
        break
        // elif result <> null() then
  
      end_if;
  
      if contains(expressions[EXPR], OPTS[hold(Goal)]) <> 0 then
        userinfo(1, "Goal reached after ".expr2text(steps)." steps");
        break
      end_if;

    end_while;

    // clean up temporary rules
    //sysassign(Simplify::tempRules, []);

    userinfo(5, "Failed :".expr2text(failed)."; Matches: ".expr2text(matches));
  
    result := sort(RESULTS[EXPR], SFUNC);
    
    if OPTS[hold(All)] = TRUE then
      return(map(result, op, 1))
    elif OPTS[hold(AllValuation)] = TRUE then
      return(result)
    elif OPTS[OutputType] = "Best" then // default
      return(op(result, [1, 1]))
    elif OPTS[OutputType] = "Proof" then
      return(proofout(op(result, [1, 1]), EXPR))
    elif OPTS[OutputType] = "NumberOfSteps" then
      return(steps)
    else
      error("Unknown output type")
    end_if
  end_proc;

  // call of REM to use prog::remember depending on options
  if OPTIONS[hold(Remember)] = TRUE then
     PROC := prog::remember(PROC, property::depends, hold(PreventRecursion), id)
     // dont catch errors !!
     // Because e.g. if in Simplify(1/X), X simplifies to zero, we
     // want the error to be thrown over and over again
  end_if;
  
  ///////////////////////////////////////////////////////
  //  end of created  S I M P L I F I E R  //////////////
  ///////////////////////////////////////////////////////

end_proc:

// 
Simplify::queueprint:=
proc(queue)
  //local i, result;
  local qulist;
begin
  //result:="";
  //for i from 1 to nops(queue) do
  //  result:= result."*****".expr2text(i)."****".expr2text(queue[i])."\n"
  //end_for
  /*
  fprint(Unquoted, 0, "\n***** QUEUE:");
  output::tableForm(queue, hold(Sort) = ((X,Y) -> op(X, 1) < op(Y, 1)));
  fprint(Unquoted, 0, "*****")
  */

  qulist:= sort([op(queue)],(X,Y) -> op(X, 1) < op(Y, 1) );
  fprint(Unquoted, 0, "\n***** QUEUE:");
  output::tableForm(table(op(qulist[1..min(8, nops(qulist))])),
                    Sort = ((X, Y) -> op(X, 1) < op(Y, 1))  );
  fprint(Unquoted, 0, "Total: ".expr2text(nops(qulist))." elements in queue");
  fprint(Unquoted, 0, "*****")
end_proc:

// must return a list of procedures/rules
Simplify::selectRules:=
proc(Ruleset : DOM_DOMAIN, expression, global = FALSE, strat = "")
  local ret;
begin
  //fprint(0, ">>", select(Simplify::tempRules, X -> type(X) = type(expression)), type(expression));
  // user rules stored in predefined domain Simplify::userRules
  (Simplify::tempRules(type(expression)))
  .
  (if contains(Simplify::userRules, "Global") then
     ret:= slot(Simplify::userRules, "Global");
     userinfo(1, "user rules GLOBAL");
     if domtype(ret) = DOM_LIST then
       ret
     else
       assert(domtype(ret) = DOM_PROC);
       ret(expression)
     end_if
   else
     null()
   end_if)
  .
  (if contains(Simplify::userRules, type(expression)) then
     ret:= slot(Simplify::userRules, type(expression));
     userinfo(1, "user rules ".type(expression));
     if domtype(ret) = DOM_LIST then
       ret
     else
       assert(domtype(ret) = DOM_PROC);
       ret(expression)
     end_if
   else
     null()
   end_if)
  .
  (if global = TRUE and contains(Ruleset, "Global") then
     ret:= slot(Ruleset, "Global");
     if domtype(ret) = DOM_LIST then
       ret
     else
       assert(domtype(ret) = DOM_PROC);
       ret(expression)
     end_if
   else
     null()
   end_if)
  .
  (if contains(Ruleset, type(expression)) then
     ret:= slot(Ruleset, type(expression));
     if domtype(ret) = DOM_LIST then
       ret
     else
       assert(domtype(ret) = DOM_PROC);
       ret(expression)
     end_if
   else
     []
   end_if)
  .
  (if contains(Ruleset, "All") then
     ret:= slot(Ruleset, "All");
     if domtype(ret) = DOM_LIST then
       ret
     else
       assert(domtype(ret) = DOM_PROC);
       ret(expression)
     end_if
   else
     null()
   end_if)
end_proc:

Simplify::defaultValuation:=
proc(EXPR)
begin
  //fprint(Unquoted, 0, "val '", EXPR, "':",
  //       float(Simplify::indetComplexity(EXPR) + Simplify::complexity(EXPR)/2
  //             + Simplify::pairPenalty(EXPR)), " (",
  //       Simplify::indetComplexity(EXPR), " | ", Simplify::complexity(EXPR)/2, " | ",
  //       Simplify::pairPenalty(EXPR), ")");
  float(Simplify::indetComplexity(EXPR) + Simplify::complexity(EXPR)/2
        + Simplify::pairPenalty(EXPR)  + Simplify::outputSize(EXPR)/10)
end_proc:

Simplify::paireval:=
proc(rulePriority, valuation)
begin
  rulePriority * valuation
end_proc:



// simplify::exclude(currentval, MINVAL)
// currentval: complexity of current expression
// MINVAL: minimum complexity among equivalent expressions
// depth: depth in expression tree

Simplify::exclude:=
proc(currentval, MINVAL, depth=0)
begin
  currentval >= 10 + MINVAL * (2 + 12 * 1.5^(-depth))
end_proc:


// operators regarded as associative and commutative
Simplify::ass:= {hold(_mult), hold(_plus), hold(_and), hold(_or),
                     hold(_union), hold(_intersect)}:

Simplify::comm:= {hold(_mult), hold(_plus), hold(_and), hold(_or),
                     hold(_union), hold(_intersect)}:

//unalias(queueinsert):
//unalias(getQueueMinimalVal):

