// $Date: 2004/06/02 12:53:47 $ $Author: stefanw $ $Revision: 1.16 $

Simplify::defaultOperatorTable:=
table(
      "Default"   = 1000, // = unknown
            
      DOM_FAIL    = 0, // nothing
      "Atomic"    = 1, // non -expressions
      DOM_IDENT   = 50,
      DOM_INT     = (q -> specfunc::ln(1 + specfunc::abs(q)/5 + (2 - specfunc::sign(q))/10)),
      DOM_RAT     = (q -> 1 + (2 - specfunc::sign(q))/10
                            + .8*specfunc::ln(specfunc::abs(op(q, 1)))
                            + 1.2*specfunc::ln(specfunc::abs(op(q, 2)))),
      DOM_FLOAT   = 2,

      piecewise   = proc(pw)
                      local cond, exprs, ops;
                      begin
                        ops:= [op(pw)];
                        cond:= map(ops, op, 1);
                        exprs:= map(ops, op, 2);
                        50 + 2* Simplify::complexity(cond) +
                                Simplify::complexity(exprs)
                    end_proc,
      
      
      "_index"    = 2,
      
      "_plus"     = //3,
                    proc(EXPR)
                    begin
                      2 +
                      specfunc::ln(1 + nops(EXPR))
                        *_plus(Simplify::complexity(op(EXPR, k)) $ k=1..nops(EXPR))
                    end_proc,
      "_mult"     = proc(EXPR)
                    begin
                      //fprint(0, "mult", EXPR);
                      if op(EXPR, nops(EXPR)) = -1 then
                        // negate
                        1.25*Simplify::complexity(-EXPR)
                      else
                        3 + _plus(1.1*Simplify::complexity(op(EXPR, k)) $ k=1..nops(EXPR))
                      end_if
                    end_proc,
      "_power"    = proc(EXPR)
                    begin
                      if domtype(op(EXPR, 2)) = DOM_INT then
                        if op(EXPR, 2) < 0 then
                          // fraction
                          2.4*Simplify::complexity(1/EXPR)
                        else
                          // no penalty for base, slight penalty for exponent
                          10 + Simplify::complexity(op(EXPR, 1)) +
                          2*Simplify::complexity(op(EXPR, 2))
                        end_if
                      elif domtype(op(EXPR, 2)) = DOM_RAT then
                        10 + 2.5*Simplify::complexity(op(EXPR, 1)) +
                        3.5*Simplify::complexity(op(EXPR, 2)) 
                      else
                        15 + 3*Simplify::complexity(op(EXPR, 1)) +
                        4*Simplify::complexity(op(EXPR, 2))
                      end_if
                    end_proc,
      "I"         = 100, // special case
      "exp"       = (X -> 50 + 2*Simplify::complexity(op(X, 1))),
      "ln"        = (X -> 55 + 2*Simplify::complexity(op(X, 1))),    
      "log"       = (X -> 70 + 2*Simplify::complexity(op(X, 1))
                             + 2*Simplify::complexity(op(X, 2))),
      "sin"       = (X -> 60 + 2*Simplify::complexity(op(X, 1))),
      "cos"       = (X -> 60 + 2*Simplify::complexity(op(X, 1))),
      "tan"       = (X -> 70 + 2*Simplify::complexity(op(X, 1))),
      "cot"       = (X -> 75 + 2*Simplify::complexity(op(X, 1))),
      "Re"        = 100,
      "Im"        = 100,
      "arcsin"    = (X -> 100 + 2*Simplify::complexity(op(X, 1))),
      "arccos"    = (X -> 100 + 2*Simplify::complexity(op(X, 1))),
      "arctan"    = (X -> 150 + 2*Simplify::complexity(op(X, 1))),
      "arccot"    = (X -> 180 + 2*Simplify::complexity(op(X, 1))),
      "sinh"      = (X -> 80 + 2*Simplify::complexity(op(X, 1))),
      "cosh"      = (X -> 80 + 2*Simplify::complexity(op(X, 1))),
      "tanh"      = (X -> 200 + 2*Simplify::complexity(op(X, 1))),
      "coth"      = (X -> 220 + 2*Simplify::complexity(op(X, 1))),
      "arcsinh"   = (X -> 200 + 2*Simplify::complexity(op(X, 1))),
      "arccosh"   = (X -> 200 + 2*Simplify::complexity(op(X, 1))),
      "arctanh"   = (X -> 300 + 2*Simplify::complexity(op(X, 1))),
      "arccoth"   = (X -> 350 + 2*Simplify::complexity(op(X, 1))),

      "whittakerM" = (X -> 2000 + 4*Simplify::complexity(op(X, 1))),

      "int"       = (X -> 100 + 3*Simplify::complexity(op(X, 1))),

      "abs"       = (X -> 20 + 1.2*Simplify::complexity(op(X, 1))),
      "sign"      = (X -> 30 + 1.2*Simplify::complexity(op(X, 1))),

      // operators used for forming boolean expressions

      "_equal"    = 3,
      "_unequal"  = 4,
      "_leequal"  = 5,
      "_less"     = 5,
      "_not"      = 10,
      "_and"      = 15,
      "_or"       = 15,
      "_in"       = 20
      ):

// overloading domains
rectform::complexity:=
proc(rf: rectform)
begin
  Simplify::complexity(expr(rf))
end_proc:

undefined::dom::complexity:= 1:

Simplify::createValuation:=
  proc(T = Simplify::defaultOperatorTable/*, DepthPenalty = 1*/)
    option escape;
    local f: DOM_PROC;
  begin
  
    f :=
    // return value:
    proc(EXPR = FAIL, depth = 0)
      option remember;
      local i:DOM_INT;
      name Simplify::complexity;
    begin
      // special case for complex numbers: regard them as expressions
      case domtype(EXPR)
        of DOM_COMPLEX do
          if EXPR = I then
            if contains(T, "I") then
              T["I"]
            else
              T["Atomic"]
            end_if;
          elif iszero(op(EXPR, 1)) then
            T["_mult"]([op(EXPR, 2), I])
          else
            T["_plus"](op(EXPR, 1)) + f(op(EXPR, 2) * I, depth + 1)
          end_if;
          break
        of DOM_SET do
        of DOM_LIST do
          1 + _plus(f(op(EXPR, i), depth + 1) $i=1..nops(EXPR));
          break
      
        of DOM_EXPR do
          if contains(T, type(EXPR)) then
            if contains({DOM_PROC, DOM_FUNC_ENV}, domtype(T[type(EXPR)])) then
              T[type(EXPR)](EXPR)
            else
              T[type(EXPR)] + _plus(f(op(EXPR, i), depth + 1) $i=1..nops(EXPR))
            end_if
          else
            T["Default"] + _plus(f(op(EXPR, i), depth + 1) $i=1..nops(EXPR))
          end_if;
          break
  
        otherwise
          if contains(T, type(EXPR)) then
            T[type(EXPR)](EXPR)
          elif EXPR::dom::complexity <> FAIL then
            EXPR::dom::complexity(EXPR)
          else
            T["Atomic"]
          end_if
      end_case
      
      //* DepthPenalty^depth
    end_proc;

  end_proc:

// default
Simplify::complexity := Simplify::createValuation():
