// $Id: userRules.mu,v 1.1 2004/05/27 10:26:10 bij Exp $
// 2004 - bij
//

// 
// Simplify::userRules -- domain and functions for user defined rules
// 

// user defined rules
Simplify::userRules := newDomain("Simplify::userRules"):
Simplify::userRules::info := "user defined rules for Simplify":

// function to manage user rule base
// (rule, <Type>, <Add>)
// (rule, <Type>, Delete)
// (rule, <Type>, Show)
// rule - a single rule or a list of rules or a rule base (domain of type sorted rules)
Simplify::userRules::new :=
  proc(rule, TYPE = "Global", OPT = hold(Add))
    local t;
  begin
    if testargs() then
      // check arguments
      
    end_if;
    
    if has([hold(Add), hold(Replace), hold(Delete), hold(Show)], TYPE) or args(0) = 1 then
      // no type given
      OPT := TYPE;
      if domtype(rule) = Rule then
        // type of the pattern
        TYPE := type(Rule::pattern(rule))
      else
        // list of rules or rule base
        TYPE := hold(All)
      end_if;
      if args(0) = 1 then
        OPT := hold(Add)
      end_if
    else
      // check type
      if TYPE <> "Global" or (domtype(rule) = Rule and type(Rule::pattern(rule)) <> TYPE) then
        warning("given type and rule pattern doesn't match")
      end_if
    end_if;

    if args(0) = 1 and rule = hold(Clear) then
      Simplify::userRules(All, Delete);
      return()
    end_if;

    case OPT
      of hold(Add) do
      of hold(Replace) do
        if not contains(Simplify::userRules, TYPE) then
          // create empty slot for single rule or a given type
          sysassign(slot(Simplify::userRules, TYPE), [])
        elif domtype(slot(Simplify::userRules, TYPE)) = DOM_PROC and OPT = hold(Add) then
          error("rule for this type is a procedure, can only be replaced")
        end_if;
        if domtype(rule) = DOM_PROC then
          sysassign(slot(Simplify::userRules, TYPE), rule)
        elif domtype(rule) = Rule then
          sysassign(slot(Simplify::userRules, TYPE), slot(Simplify::userRules, TYPE).[rule])
        elif domtype(rule) = DOM_DOMAIN then
          // whole rule base given
          if OPT = hold(Replace) then
            // clear? Simplify::userRules(All, Delete);
            for t in {map(op(rule), op, 1)} minus {"info", "key", "new"} do
              sysassign(slot(Simplify::userRules, t), slot(rule, t))
            end_for
          else // add
            for t in {map(op(rule), op, 1)} minus {"info", "key", "new"} do
              // add all rules, not for procedures!
              Simplify::userRules(slot(rule, t), t, hold(Add))
            end_for
          end_if
        elif domtype(rule) = DOM_LIST then
          // add all rules
          Simplify::userRules(t, (if TYPE = "Global" then "Global" else null() end), hold(OPT))
            $ t in rule
        end_if;
        break
      of hold(Delete) do
        if rule = hold(All) then
          if TYPE = hold(All) or TYPE = type(rule) then // no type given
            //delete whole user rule base
            for t in {map(op(Simplify::userRules), op, 1)} minus {"info", "key", "new"} do
              sysdelete(slot(Simplify::userRules, t))
            end_for;
          else
            // specific type given
            if contains(Simplify::userRules, TYPE) then
              sysdelete(slot(Simplify::userRules, TYPE))
            else
              warning("no rules defined for this type")
            end_if
          end_if
        elif domtype(rule) = Rule then
          // one rule given
          if contains(Simplify::userRules, TYPE) then
            // valid type given
            if (t := contains(slot(Simplify::userRules, TYPE), rule)) > 0 then
              // remove rule from list
              sysassign(slot(Simplify::userRules, TYPE), subsop(slot(Simplify::userRules, TYPE), t = null()))
            else
              warning("this rule is not defined")
            end_if
          else //if TYPE = hold(All)
            // search all types for given rule
            for t in {map(op(Simplify::userRules), op, 1)} minus {"info", "key", "new"} do
              if has(slot(Simplify::userRules, t), rule) then
                sysassign(slot(Simplify::userRules, TYPE),
                          subsop(slot(Simplify::userRules, TYPE),
                                 contains(slot(Simplify::userRules, t), rule) = null()))
              end_if;
            end_for
          end_if
        else
          error("illegal first argument")
        end_if;
        break
      of hold(Show) do
        error("not implemented yet");
        break
      otherwise
        error("unknown option '".OPT."'")
    end_case;
    null()
  end_proc:

