/* */
/* $Date: 2003/10/20 12:35:07 $ $Author: stefanw $ $Revision: 1.39 $ */
/* */
/* zimmerma, 14/12/95 */


// Methods for solving algebraic systems

// the function solvelib::algebraic (former interface function in 1.4)
// has been removed




/*--
	solvelib::solve_algebraic(sys, unk, dm, options) solves the algebraic
	system 'sys' with respect to the unknowns 'unk' over the
	coefficient field 'dm'.

	If 'unk' is a list, then it gives the elimination order.
--*/

solvelib::solve_algebraic:=
proc(syss, unkk, domm) 
  local g, gb, reord, sys, unk, dm, options, storeLeadingCoeffs,
  lcoefflist: DOM_LIST, linear, lc, i, inds,
  indepSets, 
  monic, cond, dimen;
  
begin

  lcoefflist:= [];
  
  // procedure storeLeadingCoeffs
  storeLeadingCoeffs:=
  proc(f, o)
    local l;
  begin
    l:= lcoeff(f, o);
    if testtype(expr(l), Type::Constant) <> TRUE then
      lcoefflist:= append(lcoefflist, l)
    end_if;
    multcoeffs(f, 1/l)
  end_proc;
    
  sys:= syss;
  unk:= unkk;
  if args(0) >= 3 then
    dm:= domm
  else
    dm:= NIL
  end_if;
  options:= solvelib::getoptions(args(4..args(0)));
  if type(unk) = DOM_SET then
    unk:= [op(unk)];
    reord:= Reorder
  else
    reord:= null()
  end_if;
  userinfo(3, "Entering solver for algebraic systems");
  if dm= NIL then
    if map(sys, testtype, Type::PolyExpr(unk, Type::Rational)) = {TRUE}
      then
      dm:=Expr;
      monic:= groebner::primpart
    else
      dm:=Dom::ExpressionField(normal, iszero@normal);
      monic:= storeLeadingCoeffs
    end_if
  else
    monic:= f -> multcoeffs(f, _invert(lcoeff(f)))
  end_if;
  
  sys:= map([op(sys)], poly, unk, dm);
  /* first compute a basis wrt DegInvLexOrder */
  gb:=groebner::gbasis(sys, DegInvLexOrder, reord, hold(Monic) = monic);
  /* then transform it in a LexOrder basis (the Grobner walk) */
  g:=groebner::gbasis(gb, LexOrder, reord, hold(Monic)= monic );

  if nops(g) = 1 and degree(g[1])=0 then
    if iszero(g[1]) then
      userinfo(20, "Zero equation: everything is a solution");
      g:= {[]}
    else
      g:= {} /* no solution */
    end_if;
    indepSets:= {};
    
  else  

  // it may be that the order of unknowns has to be exchanged
    indepSets:= groebner::stronglyIndependentSets(g);
    userinfo(10, "Independent sets are :".expr2text(indepSets[3]));
    dimen:= indepSets[1];
    indepSets:= indepSets[2];
  // the number of indep. var. should equal the dimension of the variety
    assert(dimen = nops(indepSets));
  // now indepSets is a set x1, x2, ..., xn of variables,
  // such that the solutions to the system can be expressed as
  // [x1 = ..., x2 = ..., ..., xn = ...]

    unk:= split(unk, v -> not contains(indepSets, v));
    assert(unk[3] = []);

    if nops(unk[2])= 0 then
      userinfo(10, "Zero-dimensional system")
    else
      dm:=Dom::ExpressionField(normal, iszero@normal);
      monic:= storeLeadingCoeffs;
      gb:= map(gb, poly, unk[1], dm);
    // the grobner basis has to be re-computed, and the order of variables
    // must not be changed
      reord:= null();
      lcoefflist:= [];
      g:=groebner::gbasis(gb, LexOrder, hold(Monic)= monic );
    end_if;
      
  
    userinfo(2,"Groebner basis is",map(g, expr));
    userinfo(2,"Groebner basis is valid if the following leading coefficients ".
    "are all nonzero:".expr2text(lcoefflist));
  
    g:=groebner::factor(g, reord);
    g:=map(g, op@groebner::triangular_set);
    g:=_union(op(map(g, solvelib::allvals, {op(unk[1])}, options)));
    unk:= unk[1]
  end_if;

  assert(testtype(g, Type::Set));
  
  if nops(lcoefflist) = 0 or options["IgnoreSpecialCases"] then
    return(solvelib::free2Bound(g, indepSets))
  else
    lcoefflist:= map(lcoefflist, expr);
    cond:= _and(op(map(lcoefflist, _unequal, 0)));
    // search for degree 1 leading coefficient:
    // if it exists, we also solve the special case
    linear:= FAIL;
    for lc in lcoefflist do
      inds:= indets(lc);
      for i in inds do
        if testtype(lc, Type::PolyExpr([i])) and
          degree(lc, [i]) = 1 then
          linear:= [lc, i];
          break
        end_if
      end_for;
      if linear <> FAIL then break; end_if
    end_for;
    
    
    if linear <> FAIL then
      poly(linear[1], [linear[2]]);
      return(solvelib::free2Bound
             (
              piecewise([cond, g],
                [linear[1] = 0,
                 solve(subs(syss,
                            linear[2] = -coeff(%,0) / coeff(%, 1)),
                       unk)] ),
             indepSets))
    end_if;
    
    // if nothing helps, accept also higher degree polynomials

    for lc in lcoefflist do
      inds:= indets(lc);
      for i in inds do
        if testtype(lc, Type::PolyExpr([i])) then
          linear:= [lc, i];
          break
        end_if
      end_for;
      if linear <> FAIL then break; end_if
    end_for;

    // attach the condition ,,leading coeff = 0'' as
    // additional equation to the polynomial system, and start over
    if linear <> FAIL then
      return(solvelib::free2Bound
             (
              piecewise([cond, g],
                        [not cond, solvelib::bound2Free
                         (solvelib::solve_algebraic
                          (syss union {linear[1]}, unk.[i]), {i})]),
             indepSets))
    else  
      // only non-polynomial leading coefficients
      solvelib::free2Bound(piecewise([cond, g]), indepSets)
    end_if
  end_if 
end_proc:


/*--
solvelib::allvals(g, unk, options)

g - a list of polynomials, in triangular form
unk - set of unknowns
options - table of options, as returned by solvelib::getoptions()

returns a set of solutions, each being a list; the list entries
may be equations, inequalities, and expressions of type "_in"



--*/

solvelib::allvals :=
proc(g: Type::ListOf(DOM_POLY),
     unk: Type::SetOf(DOM_IDENT),
     options: DOM_TABLE)
  local x, sol,
  i, j, r, 
  addToSolution: DOM_PROC,
  addToSolutions: DOM_PROC;
  
begin
  sol:={[]};
 
  // return solutions, with "x in S" added to all solutions
  addToSolutions:=
  proc(solutions, x, S): Type::Set

  begin
    if type(solutions) = piecewise then
      return(piecewise::extmap(solutions, addToSolutions, x, S))
    end_if;
    
    assert(type(solutions) = DOM_SET);

    map(solutions, addToSolution, x, S);
    op(%);
    _union(%)
  end_proc;


  // addToSolution: take one solution, and returns the set
  // of all solutions obtained by adding "x in S" in front
  addToSolution:=
  proc(solution: DOM_LIST, x, S):Type::Set
    local equations;
  begin
    equations:= select(solution, s -> type(s) = "_equal");
    case type(S)
      of DOM_SET do
        if property::hasprop(x) then
          // select only those solutions that do not contradict
          // the properties of x

          // case is(..) = UNKNOWN has to be improved !!
          S:=select(S, s -> is(s, getprop(x)) <> FALSE )
        end_if;

        if options["BackSubstitution"] = TRUE then
          return({[x=subs(op(S,i), equations)].solution $i=1..nops(S)})
        else
          return({[x=op(S, i)].solution  $i=1..nops(S)})
        end_if;
        
      of piecewise do
        S:= eval(subs(S, equations));
        if type(S) <> piecewise then
          return(addToSolution(solution, x, S))
        end_if;
        return(piecewise::extmap(S, s -> addToSolution(solution, x, s)))
      of "_union" do
      of "_intersect" do  
        return(map(S, s-> addToSolution(solution, x, s)))
      of solvelib::BasicSet do
        if S=C_ then
          // there is no additional constraint
          return({solution})
        else
          break
        end_if
  
    end_case;
    
    // default     
    return({[x in S].solution})
    
  end_proc;

  
  
  // if the ideal is <1>, there is no solution
  if nops(g) = 1 and degree(g[1]) = 0 then
    return(null())
  end_if;

  
  for i from nops(g) downto 1 do

    // add the constraints imposed by equation g[i] = 0
    
    r:= g[i];
    x:= indets(op(r,1)) intersect unk;
    if nops(x) = 0 then
      // add the constraint g[i] to the solutions !?
      next
    end_if;

    
    /* choose the largest unknown */
    for j from 1 to nops(op(r,2)) do
      if contains(x, op(r,[2, j])) then
        x:= op(r, [2, j]);
        break
      end_if
    end_for;

    // unk: remaining unknowns to solve for
    unk:= unk minus {x}; 

    j:= solvelib::solve_poly(poly(op(r,1),[x]), x, options);
    userinfo(5, "Solving polynomial ".expr2text(op(r,1))." gives ".
             expr2text(j));
    
    // we have to add "x in j" in front of each solution

    sol:= addToSolutions(sol, x, j);

    userinfo(5, "Solutions so far are ".expr2text(sol));


  end_for; // i from nops(g) downto 1

  sol
end_proc:

/* end of file */

