/* */
/* $Date: 2003/12/08 16:22:05 $ $Author: stefanw $ $Revision: 1.46 $ */
/* */
/* zimmerma, 14/12/95 */

/*--
	solvelib::solve_eq
--*/

solvelib::solve_eq:=
proc(eq, x)
  local options, l, r, dummy;
begin
  userinfo(3, "solve_eq called with equation ".expr2text(eq));

  if eq::dom::solve <> FAIL then
    return(eq::dom::solve(args()))
  end_if;
  
  options:=solvelib::getoptions(args(3..args(0)));


  // special case
  //
  // expressions containing RootOf's cannot be handled

  // the code below does not really work
  
/*
  l:= misc::subExpressions(eq, RootOf);
  if nops(l) > 0 then

    F:= Dom::Rational;
    l:= [op(l)]; // list of equations
    r:= [FAIL $nops(l)]; // substitutions
    for j from 1 to nops(l) do
      // we have l[j] = RootOf(f(z), z) for some z
      // we replace this z by a new variable X
      // add a new equation f(X) = 0
      // and a new substitution RootOf(f(z), z) -> X
      assert(type(l[j]) = RootOf);
      X:= genident();
      r[j]:= l[j] = X; 
      l[j]:= subs(op(l[j], 1), op(l[j], 2) = X);
    end_for;
    // carry out all substitutions in eq
    eq:= subs(eq, r);
    // solve the system containing eq and the newly created equations
    // f(X) = 0, and back-substitute
    r:= map(r, equ -> op(equ, 2) = op(equ, 1));
    return(subs(solve([eq].l, x), r))    
  end_if; // end of special case
*/ 

  if hastype(eq, RootOf) then
    userinfo(2, "Cannot solve equations containing RootOf's");
    return(hold(solve)(eq, x))
  end_if;
  
  case domtype(eq)
    of DOM_INT do
    of DOM_FLOAT do
    of DOM_COMPLEX do  
      return((if iszero(eq) then
                if options["Multiple"]=TRUE then
                  error("Infinitely many solutions, cannot create multiset")
                else
                  solvelib::BaseSet
                end_if
              else
                {}
              end_if))
    of DOM_RAT do
      // there is no DOM_RAT representing zero
      return({})
    of DOM_IDENT do /* solve(x,x) or maybe solve(y,x) */
      return(solvelib::solve_poly_irred(poly(eq,[x]), x, options))
    of DOM_EXPR do
      if contains( Type::ConstantIdents,x ) then
        error("Invalid variable (mathematical constant was given)")
      elif type(x)=DOM_FUNC_ENV then
        error("cannot solve for ".expr2text(x))
      end_if;

      case type(eq)
        of "_mult" do
        // do *not* immediately call the polynomial solver
        // because expanding a product would be the worst thing
        // one could do
          userinfo(5, "Solving factors ".expr2text(op(eq))." separately");
          return(solvelib::solve_minus
                 (_union(map(op(eq), solvelib::solve_eq, x, options)),
                  discont(eq, x, undefined), options))
        of "_power" do
          if is(op(eq,2) > 0)= FALSE then
            return({})
          end_if;
          if not has(op(eq,2), x) then
            userinfo(5, "Equation ".expr2text(eq)." reduces to ".
                     expr2text(op(eq,1)));
            l:= solvelib::solve_eq(op(eq,1), x, options);
            if options["Multiple"]=TRUE then
              if type(op(eq, 2)) = DOM_INT then 
                return(_union(l $op(eq,2)))
              else
                error("Option multiple is only allowed for polynomial equations")
              end_if
            else
              return(piecewise([op(eq, 2) > 0, l],
                               [not op(eq, 2) > 0, {}]))
            end_if
          end_if
        of "_plus" do
          if testtype(eq,Type::PolyExpr(x)) then
            // test for special case f(x)^n = C
            // in this case, return the union of the sets
            // of solutions for f(x)=C^(1/n) + exp(2*PI*I*j/n)
            // for 0 <= j <= n-1


              [l, r, dummy]:= split(eq, has, x);
              if type(l)= "_power" then
                // polynomial of special form x^n + r for some integer n
                return(solvelib::purePolynomial(l, r, x, options))
              end_if;
           
            // polynomial has not special form
            return(solvelib::solve_poly(poly(eq,[x]),x,options))
          end_if;
          break
        of "function" do
          return(hold(solve)(eq, x))
      end_case;
        
     
      // type(eq) is none of _mult, _plus, or _power, or eq is a sum
      // but not polynomial in x
      
      if options["Multiple"]=TRUE then
        error("Option multiple is only allowed for polynomial equations")
      end_if;


      // handle floating-point input
  
      if stdlib::hasfloat(eq) then
        eq:=numeric::rationalize(eq);
        solvelib::solve_eq(eq, x, options);
        // first replace the RootOf's to avoid floats in the exponents
        misc::maprec(%, {RootOf}=float);
        return(misc::maprec(%, {Dom::ImageSet, DOM_INT, DOM_RAT, DOM_COMPLEX,
                                DOM_IDENT}= float))
      end_if;


      
      // do *not* normalize here, because this would destroy
      // sums where some summands do not contain x
      
      return(solvelib::isolate(eq, 0, x, options))
      
    of DOM_POLY do
      return(solvelib::solve_poly(eq, x, options))

      // some strange cases

    of stdlib::Infinity do
    of stdlib::CInfinity do
      return({})
      
  end_case;
  // NOT REACHED

  assert(FALSE);
  return(hold(solve)(eq, x))

end_proc:



/* end of file */

