
// solvelib::solve_intersect

// compute the intersection of sets in the *mathematical* sense
// i.e., {a} intersect {b} equals {} only if a<>b can be proven



solvelib::solve_intersect:=
proc(S, T)
  local tbl: DOM_TABLE, res1, res2;

  
begin


// main program

  case args(0)
    of 0 do
      return(C_)
    of 1 do
      return(args(1))
    of 2 do
      break
    otherwise
      res1:= solvelib::solve_intersect(args(1..args(0) div 2));
      res2:= solvelib::solve_intersect(args((args(0) div 2) + 1..args(0)));
      // if one of these intersection returns unevaluated, flatten out
      // and return unevaluated
      if type(res1) = "_intersect" then
        if type(res2) = "_intersect" then
          return(hold(_intersect)(op(res1), op(res2)))
        else
          return(hold(_intersect)(op(res1), res2))
        end_if
      elif type(res2) = "_intersect" then
        return(hold(_intersect)(res1, op(res2)))
      end_if;
          
      return(solvelib::solve_intersect(res1, res2));            
  end_case;

  assert(args(0) = 2);


  if S = T then
    return(S)
  end_if;


  if type(S) = "_union" then
    return(map(S, solvelib::solve_intersect, T))
  elif type(T) = "_union" then
    return(map(T, solvelib::solve_intersect, S))
  end_if;
  
  if S::dom::_intersect <> FAIL or T::dom::_intersect <> FAIL then
    return(S intersect T)
  end_if; 
  
  tbl:= solvelib::intersectTable;
  
  if contains(tbl, type(S), type(T)) then
    tbl[type(S), type(T)](S, T)
  elif contains(tbl, type(T), type(S)) then
    tbl[type(T), type(S)](T, S)
  else
    hold(_intersect)(S, T)
  end_if
  
end_proc:


solvelib::intersectTable:=
table(
      (DOM_SET, DOM_SET) =
      proc(S, T)
        local res, s, t, possible, pw;
      begin
        res:= S intersect T;
        pw:= {};
        S:= S minus res;
        T:= T minus res;
        for s in S do
          possible:= {};
          for t in T do
            case is(s=t)
              of TRUE do
                if length(s) <= length(t) then
                  res:= res union {s}
                else
                  res:= res union {t}
                end_if;
                S:= S minus {s};
                T:= T minus {t}
              of UNKNOWN do
                possible:= possible union {t}
              // of FALSE: do nothing
            end_case;
         end_for;
         pw:= pw union
              piecewise([s in possible, {s}], [not s in possible, {}]);
        end_for;
        res union pw
      end_proc, 


      (DOM_SET, "solve") =
      proc(S, T)
        local x, eq, a, eq2, operands;
      begin
        if type((x:= op(T, 2))) = DOM_IDENT then
          eq:= op(T, 1);
          if type(eq) = "_equal" then
            eq:= op(eq, 1) - op(eq, 2)
          end_if;
          operands:= [];
          for a in S do
            // include a in the result iff it is a solution
            if traperror((eq2:= eval(subs(eq, x = a)))) <> 0 then
              // a is not a solution
              next
            end_if;
            operands:= operands.[piecewise([eq2 = 0, {a}], [eq2 <> 0, {}])];
          end_for;
          operands:= select(operands, _unequal, {});
          return(_union(op(operands)))  
        end_if;
        hold(_intersect)(S, T)
      end_proc,

      (DOM_SET, "_minus") =
      proc(S, T)
        local A, B, res;
      begin
        A:= op(T, 1);
        B:= op(T, 2);
        // S intersect (A minus B) = (S intersect A) minus B 
        if type((res:= solvelib::solve_intersect(S, A))) <> "_intersect" then
          solvelib::solve_minus(res, B)
        else
          hold(_intersect)(S, T)
        end_if 
      end_proc,
      
      ("_minus", "_minus") =
      proc(S, T)
      begin
        if op(S, 1) = op(T, 1) then
          return(op(S,1) minus (op(S, 2) union op(T, 2)))
        end_if;
        hold(_intersect)(S, T)
      end_proc


      ):

