/* */
/* $Date: 2004/11/17 10:31:47 $ $Author: stefanw $ $Revision: 1.129.2.1 $ */
/* */
/* zimmerma, 14/12/95 */

/*--
        solvelib::isolate  --  isolate an identifier to the lhs
--*/


solvelib::isolate :=
proc(lhside, Rhs, x, options: DOM_TABLE, reclevel = 1 : DOM_INT, known={})
  local Lhs,
  maxreclevel: DOM_INT,
  l,i,a, b, s,
  ar : DOM_IDENT,
  lev,
  newvars,
  newlhs,
  newrhs,
  pw,
  u : DOM_IDENT,
  v : DOM_IDENT,
  p, q,
  roots: DOM_BOOL,
  getfractpow : DOM_PROC,
  applyHeuristic: DOM_PROC,
  checkSols:DOM_PROC,
  R2toC: DOM_PROC,
  CC1, CC2, CC3, n, k;

begin


// l o c a l   m e t h o d s
  
  

// local method getfractpow
// checks whether the expresion a is a fractional power or a product
// in which exactly one fractional power occurs

// if no, FAIL is returned
// if yes, the denominators of the fractional exponent
// is returned, such that raising a to that power would remove it
// also, the cofactor is returned

getfractpow :=
proc(a)
  local
    frc,  // factors that are fractional powers
  other,  // other factors
  dummy;
begin
  case type(a)
    of "_power" do
      if type(op(a,2))=DOM_RAT then
        return([op(a, [2, 2]), 1])
      else
        return(FAIL)
      end_if;
    of "_mult" do
      [frc, other, dummy]:= split([op(a)],
                                   fac -> type(fac)="_power" and
                                   type(op(fac ,2)) = DOM_RAT and
                                   has(fac, x)
                                  );
      assert(dummy = []);
      if nops(frc)<>1 then
        return(FAIL)
      end_if;
      // return the denominator (sec. op)
      // of the exponent (sec.op)
      // of the only element of frc
      return([op(frc, [1, 2, 2]), _mult(op(other))])
  end_case;
  FAIL
end_proc;


// applyHeuristic(Lhs, Rhs)

// solves Lhs(x) = Rhs(x) heuristically by applying the same
// function to both sides of the equation, or returns FAIL
// if the heuristic did not succeed


applyHeuristic:=
proc(Lhs, Rhs)
  local X, result, expo1, expo2, g;
begin
  if type(Lhs) = type(Rhs) then
    case type(Lhs)
      of "_power" do
        // a^(1/2) = b^(1/2) -> a=b  and similar cases
        if type((expo1:= op(Lhs, 2))) = DOM_RAT and
          type((expo2:= op(Rhs, 2))) = DOM_RAT and
          op(expo1, 1) = 1 and op(expo2, 1) = 1 and
          (g:= igcd(op(expo1, 2), op(expo2, 2))) > 1 then
          return(solvelib::isolate(Lhs^g - Rhs^g, 0, x, options, reclevel+1))
        else  
          break
        end_if;
      of "exp" do
       X:= genident("k"); 
       return(solvelib::Union
              (solvelib::isolate
               (op(Lhs) - op(Rhs), 2*I*PI*X, x, options, reclevel+1),
               X, Z_))
      of "sin" do
        X:= genident("k");
        return(solvelib::Union
               (solvelib::isolate
                (op(Lhs) - op(Rhs), 2*PI*X, x, options, reclevel+1)
                union
                solvelib::isolate(op(Lhs) + op(Rhs), PI + 2*PI*X, x, options,
                                  reclevel+1),
                X, Z_)
               )
      of "cos" do
        X:= genident("k");
        return(solvelib::Union
               (solvelib::isolate
                (op(Lhs) - op(Rhs), 2*PI*X, x, options, reclevel+1)
               union
               solvelib::isolate(op(Lhs) + op(Rhs), 2*PI*X, x, options,
                                 reclevel+1),
                X, Z_)
               )
      of "tan" do
        // tan(f(x)) = tan(g(x)) iff f(x) = g(x) + k*PI
        X:= genident("k");
        return(solvelib::Union
               (solvelib::isolate
               (op(Lhs) - op(Rhs), PI*X, x, options, reclevel+1),
               X, Z_))
      // injective functions where f(x) = f(y) iff x=y  
      of "ln" do
      of "arcsin" do
      of "arccos" do  
        return(solvelib::isolate(op(Lhs) - op(Rhs), 0, x, options, reclevel+1))
    end_case;
  end_if;

  if (type(Lhs) = "_mult" or type(Rhs) = "_mult")
    and has(Lhs, x) and has(Rhs, x) and is(Rhs <> 0) = TRUE then
    // division heuristic
    result:= solvelib::isolate(Lhs/Rhs, 1, x, options, reclevel+1);
    if not has(result, hold(solve)) then
      // care for Rhs = 0 
      return(result)
    end_if
  end_if;
  
  FAIL
end_proc;




//////////////////////////////////////////////////////////////////
// b e g i n   o f   m a i n   p r o g r a m
//
/////////////////////////////////////////////////////////////////

  Lhs:=lhside;  
  assert(not has(Rhs, x));

  lev:=options["MaxDegree"];

  pw:=if options["IgnoreSpecialCases"] then
        solvelib::ignoreSpecialCases
      else
        piecewise::new
      end_if;
 

  maxreclevel:= options["MaxRecLevel"];
  if reclevel>= maxreclevel then
    userinfo(2, "Maximum recursion level exceeded in solvelib::isolate");
    return(hold(solve)(Lhs=Rhs,x))
  end_if;

  // isolate has been callee recursively, and the current arguments have
  // already been there
  if contains(known, [Lhs, Rhs, x]) then
    // determine simplest result in call stack
    sort([op(known)], (l1, l2) -> length(l1[1]) <= length(l2[1])) [1];
    return(hold(solve)(%[1] = %[2], %[3]))
  else
    known:= known union {[Lhs, Rhs, x]}
  end_if;

  userinfo(3,"trying to isolate ".expr2text(x)." in ".expr2text(Lhs=Rhs));
  userinfo(5,"Recursion level is ".expr2text(reclevel));
  if Lhs=x then
    return({Rhs})
  elif not has(Lhs,x) then
    return(pw([Lhs=Rhs, solvelib::BaseSet],
                      [Lhs<>Rhs, {}]))
  end_if;

  // case analysis, depending on the type of left hand side
  // solve f(x) = C for x, for different types of f    


  // first try overloading

  if domtype(Lhs) = DOM_EXPR and
    type((a := eval(op(Lhs, 0)))) = DOM_FUNC_ENV then
    if a::isolate <> FAIL then
      return(a::isolate(Lhs, Rhs, x, options, reclevel, known))
    end_if
  end_if;

  case type(Lhs)
    of "_plus" do

      if iszero(Rhs) and nops(Lhs) = 2 then
        // handle f(x)+g(x)=0 by trying to solve f(x)=-g(x) heuristically
        
        if type(op(Lhs,1))="_mult" then
          a:= applyHeuristic(op(Lhs, 2), -op(Lhs, 1))
        else
          a:= applyHeuristic(op(Lhs, 1), -op(Lhs, 2))
        end_if;

        if a <> FAIL then
          return(a)
        end_if;
       
      end_if; // nops(Lhs)=2
     

      
      a := split(Lhs, has, x);
      if not iszero(a[2]) then
        userinfo(5, "Subtracting ".expr2text(a[2])." on both sides gives ".
                 expr2text(a[1]). " = ". expr2text(Rhs-a[2]));
        return(solvelib::isolate(a[1], Rhs-a[2], x, options, reclevel+1,
                                 known)) 
      end_if;



      // heuristic for arcsin, arccos, arctan

      // we sort the summands by "priority": Summands of the form
      // +/- arcsin/cos/tan(..) get priority 1, k-folds of such
      // get priority k
      // we then apply the inverse function of the "best" summand
 
      u:= proc(y)
            local prio;
            begin
              if type(y) = "_mult" and type(op(y, nops(y))) = DOM_INT then
                prio:= abs(op(y, nops(y)));
                y:= y* 1/ op(y, nops(y))
              else
                prio:= 1
              end_if;
              if not contains({"arcsin", "arccos", "arctan"}, type(y)) then
                null()
              else  
                [type(y), prio]
              end_if;
          end_proc;
      
      a:= map([op(Lhs)], u);

   

      if nops(a) >0 then
      
        a:= sort(a, (pair1, pair2) -> pair1[1] <= pair2[1] );

     
        
        case a[1][1]
      
          of "arcsin" do
            // we try to solve Lhs - Rhs = 0
            // by solving the equivalent
            // sin(Lhs - Rhs) = 0 and - PI < Lhs - Rhs < PI

            u:= solvelib::isolate(expand(sin(Lhs - Rhs)), 0, x, options,
                                  reclevel +1, known );
            break              
          of "arccos" do
            u:= solvelib::isolate(expand(cos(Lhs - Rhs)), 1, x, options,
                                  reclevel +1, known );
            break
          of "arctan" do
       
            u:= solvelib::isolate(expand(tan(Lhs - Rhs)), 0, x, options,
                                  reclevel +1, known );
            
        end_case;

        if options["IgnoreSpecialCases"] = FALSE then
          if type(u) = DOM_SET then
          // at the moment, we can only check this type of solutions


          
            return(
                   select(u, y -> is(subs(Lhs - Rhs, x = y) = 0) <> FALSE )
                   )
          end_if
        else
          if type(u) <> "solve" then
            return(u)
          end_if;
        end_if; 
      end_if;
      

      
      // find out whether exactly one summand is a fractional power
      // with exponent p/q
      // or a product of fractional powers
      // if yes, raise that summand and all others to the power of q, such
      // that no fractional power occurs any more
      // i.e. s1+s2+..+si^(p/a[i])+..+sn = Rhs becomes
      // si^p = (Rhs -(s1+s2+..+s(i-1)+s(i+1)+...+sn))^a[i]
      // note that this increases the number of solutions !!
      
      a := map([op(Lhs)], getfractpow);
      if nops(select(a, _unequal, FAIL)) = 1 then
        // get the index of the uniquely determined non-FAIL entry
        i := contains(map(a, type), DOM_LIST);
        [s, l] := a[i]; // denom of exponent, cofactor
        newrhs := - _plus(op(Lhs, 1..i-1), op(Lhs, i+1..nops(Lhs))) + Rhs;
        a := op(Lhs, i)^s 
        - newrhs^s;
        // from the solutions of
        // si^p = (Rhs -(s1+s2+..+s(i-1)+s(i+1)+...+sn))^a[i] ,
        // select those for which si has small polar angle

        solvelib::isolate(a, 0, x, options, reclevel+1, known);
        if type(%) = "solve" then
          return(hold(solve)(Lhs=Rhs, x, solvelib::nonDefaultOptions(options)))
        else
          return(solvelib::checkAngle(%,
                 l*newrhs, x, PI/s, options, maxreclevel-reclevel))
        end_if
      end_if;


      // handle C1*f(x)^n + C2 * ln(C3*f(x)^k) = C4  (LambertW case)  
      // unfortunately, match is unable to handle this
      // in what follows, write a = f(x)

      [u, v, p] := split(Lhs, hastype, "ln");
      assert(p = 0);
      if type(u) = "_mult" then
        [p, CC2, q] := split(u, has, x);
        assert(q = 1);
      else
        CC2:= 1;
         p:= u
      end_if;
      if type(p) = "ln" then
        p:= op(p);
        if type(p) = "_mult" then
          [p, CC3, q]:= split(p, has, x);
          assert(q=1);
        else
          CC3:=1
        end_if;
        
        if type(p) = "_power" and not has(op(p, 2), x) then
          a:= op(p, 1);
          k:= op(p, 2)
        else
          a:= p;
          k:= 1
        end_if;

        if type(v) = "_mult" then
          [p, CC1, q] := split(v, has, x);
          assert(q = 1)
        else  
          CC1 := 1;
          p  := v
        end_if;

        if type(p) = "_power" and not has(op(p, 2), x) then
          b:= op(p, 1);
          n:= op(p, 2)
        else
          b:= p;
          n:= 1
        end_if;

        if a = b then
          genident();
          return(solvelib::preImage
                 (a, x,
                  Dom::ImageSet
                  (exp(- (lambertW(%, n*CC1*exp((Rhs -CC2* ln(CC3))*n/CC2/k)/
                                   CC2/k) *CC2*k
                          + n*CC2*ln(CC3) - n*Rhs) / CC2/k/n ), %, Z_)));


      // else: heuristic failed
        end_if; // a = b
      end_if; // type(p) = "ln"


      // handle C1*f(x) + C2* exp(f(x)) = Rhs  (second LambertW case)
      // write a = f(x)

      
      [u, v, p] := split(Lhs, hastype, "exp");
      assert(p = 0);
      if type(u) = "_mult" then
        [p, CC2, q] := split(u, has, x);
        assert(q = 1);
      else
        CC2:= 1;
         p:= u
      end_if;
      if type(p) = "exp" then
        a:= op(p);
        if type(v) = "_mult" then
          [b, CC1, q] := split(v, has, x);
          assert(q = 1)
        else
          CC1 := 1;
          b  := v
        end_if;

        if not has(b/a, x) then
          // write C1*f(x) as (C1*b/a) b =: CC1 * b
          // such that the argument of exp and the factor b become equal
          CC1:= CC1 * (b/a);
          genident();
          return(solvelib::preImage
                 (a, x, Dom::ImageSet
                  (- (CC1*lambertW(%, CC2*exp(Rhs/CC1)/CC1) - Rhs)/CC1,
                   %, Z_)))
        end_if;
      end_if;


                
    
        
      if iszero(Rhs) then
        
          /* try to factor the left hand side, but only over Q */
        l:=maprat(numer(Lhs), expr@factor);
        if type(l)="_mult" then
          return(_union(solvelib::isolate(op(l,i),0,x,options, reclevel+1,
                                          known)
                        $i=1..nops(l))
                 minus
                _union(discont(op(l,i), x, hold(Undefined)) $i=1..nops(l))
                 )
        end_if
      end_if; // iszero(Rhs)

      /* try to recognize a polynomial */
      if testtype(Lhs,Type::PolyExpr(x)) then
        return(solvelib::solve_poly(poly(Lhs-Rhs,[x]),x,options))
      end_if;
      if has(Lhs,ln) then
        combine(Lhs,ln);
        if %<>Lhs then
          return(solvelib::isolate(%,Rhs,x,options, reclevel+1, known))
        end_if
      end_if;
      break;
    of "_mult" do
      // divide off factors that are independent of x
      a:= split(Lhs, has, x);
      if a[2]<>1 then
        userinfo(5, "Dividing both sides by ".expr2text(a[2])." gives ".
                 expr2text(a[1]). " = ". expr2text(Rhs/a[2]));
        return(pw
               ([a[2]<>0,
                 solvelib::isolate(a[1], Rhs/a[2], x, options, reclevel+1,
                                   known)],
                [a[2]=0, solve(Rhs,x)]))
      end_if;

      
      if iszero(Rhs) then /* one factor must be zero */
        userinfo(10, "Solving each factor separately");
          return(solvelib::solve_minus
                 (_union(solvelib::isolate(op(Lhs,i),0,x,options,
                                           reclevel+1, known)
                        $ i=1..nops(Lhs)), discont(Lhs, x, undefined),
                  options))
      end_if;


      // handle products with negative powers by multiplying both sides
      // of the equation:
      // f(x) * g(x)^(-k) = C is equivalent to
      // f(x) = C* g(x)^k and g(x) <> 0, which in turn is equivalent to
      // f(x) - C*g(x)^k = 0
      
      a:= split(Lhs, fctor ->
                type(fctor) ="_power" and
                testtype(op(fctor, 2), Type::Negative) = TRUE);
      if a[1] <> 1 then
        return(solvelib::solve_minus
               (solvelib::isolate(a[2] - Rhs/a[1], 0, x, options,
                                  reclevel+1, known ),
                solvelib::isolate(op(a[1], 1), 0, x, options,
                                  reclevel+1, known), options
                ))
      end_if;

      // handle special case f(x)^C1 * exp(C2*f(x)^k) = Rhs
      // (lambertW case)
      [u, v, p]:= split(Lhs, fctor-> type(fctor) = "exp");
      assert(p=1);
      u:= combine(u, exp);
      if type(u) = "exp" then
        p:= op(u);
        if type(p)="_mult" then
          [p, CC2, q] := split(p, has, x);
          assert(q=1);
        else
          CC2:=1
        end_if;
        if type(p)="_power" and not has(op(p,2), x) then
          k:= op(p, 2);
          p:= op(p, 1)
        else
          k:= 1
        end_if;

        if type(v)="_power" and not has(op(v,2), x) then
          CC1:= op(v, 2);
           v:= op(v, 1)
        else
          CC1:= 1
        end_if;

        if p = v then
          s:= genident();
          return(piecewise
                 ([Rhs <> 0 and CC1 <>0,
                   solvelib::preImage
                   (v, x, Dom::ImageSet
                    (
                    exp((-lambertW(s, k*CC2*exp(k*ln(Rhs)/CC1)/CC1) * CC1 + k*ln(Rhs))
                        / k / CC1), s, Z_))],
                  // since we have handled Rhs = 0 before, we are just
                  // left with tyhe case that Rhs is a symbolic expression
                  // that represents 0
                  [Rhs = 0, solvelib::isolate(Lhs, 0, x, options,
                                              reclevel+1, known)],
                  // CC1 = 0: the equation is really exp(CC2*y^k) = Rhs
                  [CC1 = 0, solvelib::isolate(exp(CC2*x^k), Rhs, x, options,
                                             reclevel+1, known)]))
        end_if;
      end_if;
      break
    of "_power" do
      s:=op(Lhs, 2);
      
      if not has(s,x) then
        // f(x)^C = 0 iff f(x)=0 for positive C; this is not solvable
        // for negative C or C=0
        // in general (complex C) one has f(x)^C = exp(C *ln(f(x)))
        // for nonzero f(x), which cannot be zero
        if iszero(Rhs) then
          return(piecewise([s>0,
                            solvelib::isolate(op(Lhs,1), 0, x, options,
                                              reclevel+1, known)],
                           [not s>0, {}]))
        end_if;
      
        case type(s)
          of DOM_INT do
            if s>0 then
              // solving u^s = a for u
              a:= solvelib::solve_eq(x^s-Rhs, x, options);
              if op(Lhs,1)=x then
                return(a)
              else
                // it remains to solve op(Lhs,1)=u for x
                return(solvelib::preImage(op(Lhs,1), x, a, options))
              end_if
            else // s<0 -> invert both sides iff Rhs <> 0
              return(pw
                     ([Rhs=0, {}],
                      [Rhs<>0,
                       solvelib::isolate(op(Lhs,1)^(-s), 1/Rhs, x, options,
                                         reclevel+1, known)]))
            end_if
            // NOT REACHED
          of DOM_RAT do

            p:= op(s,1);
            q:= op(s,2);
            
            // note that base^(p/q) is the same as (base^(1/q))^p
            // and not the same as (base^p)^(1/q)

            // let u be the solutions of v^p=Rhs for v
            v:=genident();
            u:=solvelib::solve_eq(v^p-Rhs, v, options );
            
            /* u:=_union(solvelib::solve_eq(op(Lhs,1)-(Rhs)^(1/p)*
                                         exp(2*PI*I*j/p),
                                      x, options)
                      $ j=0..p-1); */

            // base^(1/q) is in u iff base is some u^q and u has small
            // argument
       
           
            u:= solvelib::checkAngle(u, x, x, PI/q, options,
                                     maxreclevel-reclevel);
           
            return(
                   solvelib::preImage(op(Lhs,1), x, u^q,
                                      options))
            
          otherwise
            /* for complex numbers and symbolic expressions,
             write the Lhs as exp(...*ln(..))
             to solve: exp(s*ln(base)) = Rhs
             (where base <>0)
             this becomes (Rhs<>0) s*ln(base)=ln(Rhs)+2*k*PI*I */
            return(pw
                  ([Rhs=0 and s>0,
                   solvelib::isolate(op(Lhs,1), 0, x, options,
                                     reclevel+1, known)],
                   [Rhs=0 and not s>0, {}],
                   [Rhs = 1 and s=0,
                    solvelib::isolate(op(Lhs,1),0, x,
                                       options, reclevel+1, known)],
                   [Rhs<>0 and (Rhs <> 1 or s<>0),
                   solvelib::preImage(s*ln(op(Lhs,1)),x, ln(Rhs)+2*PI*I*Z_,
                   options)]
                   ))
        end_case;

      
      else // x occurs in the exponent s
        if iszero(Rhs) then
          // solving a(x)^b(x) for x
          // this is zero iff a(x)=0 and b(x) > 0
          return(solvelib::solve_intersect
                  (solvelib::isolate(op(Lhs,1),0, x, options, reclevel+1,
                                     known),
                   solve(s>0, x, options)))
        elif contains({DOM_INT, DOM_RAT}, type((u:= op(Lhs, 1)))) and u>0 then
          assert(u<>1);
          // try to express the solution in terms of log if possible
          // we have that u^(s(x)) = Rhs is equivalent to
          // \exists k: s(x) = log(u, Rhs) + 2*PI*k*I/ln(u)
          return(solvelib::preImage(s, x, log(u, Rhs) + 2*PI*I*Z_/ln(u),
                                    options) )
        else
          return(solvelib::isolate(exp(s*ln(op(Lhs,1))), Rhs,
                                   x,options, reclevel+1, known))
        end_if
      end_if;
      break;
    of "arctan" do
      // new: can only have one argument 
        return(solvelib::isolate(op(Lhs),tan(Rhs),x,options, reclevel+1, known))
    of "arcsin" do
      return(pw([Re(Rhs) > PI/2 or Re(Rhs) < - PI/2, {}],
                       [Re(Rhs) <= PI/2 and Re(Rhs)>=-PI/2,
                        solvelib::isolate(op(Lhs), sin(Rhs), x,
                                          options, reclevel+1, known)]))
    of "arccos" do
      return(pw([Re(Rhs) < 0 or Re(Rhs) > PI, {}],
                       [Re(Rhs)>=0 and Re(Rhs)<=PI,
                        solvelib::isolate(op(Lhs), cos(Rhs), x, options,
                                          reclevel+1, known)]))
    of "log" do
      return(solvelib::isolate(rewrite(Lhs, ln), Rhs, x, options,
             reclevel + 1, known)) 
    of "ln" do // the imaginary part of a log must be  > -PI and <= PI
      // Rhs := expr(rectform(Rhs));
      return(pw([Im(Rhs) <= -PI or Im(Rhs)>PI, {}],
                       [Im(Rhs)>-PI and Im(Rhs)<=PI,
                        solvelib::isolate(op(Lhs),exp(Rhs),x,options,
                                          reclevel+1, known)]))
    of "exp" do /* exp(a)=b <==> a = ln(b) + 2*I*k*PI */
      return(pw
             ([Rhs=0, {}],
              [Rhs<>0, solvelib::preImage(op(Lhs), x, ln(Rhs) +2*I*PI*Z_,
               options)]))
    of "lambertW" do
      assert(nops(Lhs) = 2);
      // only one special case implemented
      if op(Lhs, 1) = 0 and is(Rhs >=0 ) = TRUE then
        return(solvelib::isolate(op(Lhs, 2), Rhs * exp(Rhs), x, options,
                                 reclevel + 1, known))
      end_if;
      // default:
      return(hold(solve)(Lhs=Rhs, x))      
    of "sinh" do
      return(solvelib::preImage(op(Lhs),x ,arcsinh(Rhs)+ I*PI*Z_, options))
    of "cosh" do
      l:=arccosh(Rhs);
      if l=0 then 
         return(solvelib::preImage(op(Lhs),x, 2*I*PI*Z_, options))
      else 
         return(solvelib::preImage(op(Lhs),x, l+2*I*PI*Z_,options) union
                solvelib::preImage(op(Lhs),x, -l+2*I*PI*Z_, options))
      end_if
    of "coth" do
    of "tanh" do
      return(solvelib::isolate(rewrite(Lhs, sincos), Rhs, x, options,
             reclevel+1, known))
    of "arcsinh" do
      return(solvelib::isolate(op(Lhs),sinh(Rhs),x,options, reclevel+1, known))
    of "arccosh" do
      return(solvelib::isolate(op(Lhs),cosh(Rhs),x,options, reclevel+1, known))
    of "cos" do /* cos(a)=b <==> a = +-arccos(b) + 2*k*PI */
      l:=arccos(Rhs);
      if l=0 then 
         return(solvelib::preImage(op(Lhs),x,2*PI*Z_, options))
      elif l=PI/2 then 
         return(solvelib::preImage(op(Lhs),x, l+PI*Z_, options))
      elif l=PI then 
         return(solvelib::preImage(op(Lhs),x, l+2*PI*Z_, options))
      else 
         return(solvelib::preImage(op(Lhs),x, l+2*PI*Z_, options) union
                solvelib::preImage(op(Lhs),x,-l+2*PI*Z_, options))
      end_if
    of "sin" do /* sin(a)=b <==> a = {arcsin(b),PI-arcsin(b)} + 2*k*PI */
      if Rhs=0 then 
         return(solvelib::preImage(op(Lhs),x,PI*Z_, options))
      elif Rhs=1 then 
         return(solvelib::preImage(op(Lhs),x, arcsin(Rhs)+2*PI*Z_, options))
      elif Rhs=-1 then 
         return(solvelib::preImage(op(Lhs),x,-PI/2+2*PI*Z_, options))
      else 
         return(solvelib::preImage(op(Lhs),x,arcsin(Rhs)+2*PI*Z_, options) union
               solvelib::preImage(op(Lhs),x, PI-arcsin(Rhs)+2*PI*Z_, options))
      end_if
    of "tan" do /* tan(a)=b <==> a=arctan(b)+k*PI */
      ar:=genident();
      return(
      piecewise([Rhs<>I and Rhs<>-I,
      solvelib::Union(
        solvelib::isolate(op(Lhs), ar, x, options, reclevel+1, known),
        ar, arctan(Rhs)+PI*Z_)], [Rhs=I or Rhs=-I, {}])
      )
    of "cot" do
      ar:= genident();
      return
      (piecewise
       ([Rhs <> I and Rhs <> -I,
         solvelib::Union
         (solvelib::isolate(op(Lhs), ar, x, options, reclevel+1, known),
          ar, arccot(Rhs)+PI*Z_)],         
        [Rhs = I or Rhs = -I, {}]))
    of "arg" do
      assert(nops(Lhs) = 1);
      a:= op(Lhs);
      return(piecewise(
             [Rhs = 0, solve(a >=0, x, options)],
             [Rhs = PI, solve(a < 0 , x, options)],
             [Rhs > 0 and Rhs < PI or Rhs>-PI and Rhs<0,
              solvelib::preImage(a, x,
              exp(I*Rhs) * Dom::Interval(0, infinity) , options) ],
             [not (Rhs> -PI  and Rhs<= PI), {}]))
    of "abs" do // abs(a)=b <=> (a=b or -a=b) and b>=0 if a is real
      a:=op(Lhs);
      if is(a, Type::Real)=TRUE then
        return(
        piecewise([Rhs>=0,
            solvelib::isolate(a,Rhs,x, options, reclevel+1, known)
                   union solvelib::isolate(-a, Rhs, x, options, reclevel+1,
                                           known)],
          [not Rhs>=0, {}]))

      else
        // do *not* fall through
        // (solve(abs(x) =17, x) must work)
                  
        // abs(x) = y is equivalent to
        // x \in { y exp(2*PI*t*I), t \in [0, 1]}
        
        ar := genident("alpha");
        return
        (piecewise
         ([Rhs >= 0, solvelib::preImage
           (op(Lhs), x, Dom::ImageSet(Rhs * exp(2*PI*I*ar), ar,
                                      Dom::Interval([0, 1])))],
          [not Rhs>=0, {}]
          ))
      end_if
    of "binomial" do
      // handle below by rewriting
      break
    of "sign" do
      // to solve: sign(f(x)) = C
      // there are three cases:
      // if C=0, the equation is equivalent to f(x) = 0
      // if |C| = 1, the equation is equivalent to f(x) in
      // {lambda*C; lambda in (0, infinity) }
      // otherwise, there is no solution
      a:= op(Lhs);
      return(piecewise([Rhs = 0, solvelib::isolate(a, Rhs, x, options,
                                                   reclevel + 1, known)],
                       [abs(Rhs) = 1,
                        solvelib::preImage
                        (a, x, Rhs*Dom::Interval(0, infinity))],
                       [Rhs <> 0 and abs(Rhs) <> 1, {}]))
    of "ceil" do
      // ceil(x) = C iff C is an integer and x in "]C-1, C]"
      // do this for real x or C only
      a:= op(Lhs);
      if is(a, Type::Real) = TRUE or is(Rhs, Type::Real) = TRUE then
        return(piecewise([Rhs in Z_,
                          solvelib::preImage(a, x,
                                             Dom::Interval(Rhs-1, [Rhs]))],
                         [not Rhs in Z_, {}]))
      else
        return(hold(solve)(Lhs=Rhs,x))
      end_if;
      // NOT REACHED
      assert(FALSE)
    of "floor" do
      a:= op(Lhs);
      if is(a, Type::Real) = TRUE or is(Rhs, Type::Real) = TRUE then
        return(piecewise([Rhs in Z_,
                          solvelib::preImage(a, x,
                                             Dom::Interval([Rhs], Rhs+1))],
                         [not Rhs in Z_, {}]))
      else
        return(hold(solve)(Lhs=Rhs,x))
      end_if;
      // NOT REACHED
      assert(FALSE)
    of "round" do
      a:= op(Lhs);
      if is(a, Type::Real) = TRUE or is(Rhs, Type::Real) = TRUE then
        return(piecewise([Rhs in Z_,
                          solvelib::preImage(a, x,
                                             Dom::Interval([Rhs - 1/2],
                                                           Rhs + 1/2))],
                         [not Rhs in Z_, {}]))
      else
        return(hold(solve)(Lhs=Rhs,x))
      end_if;
      // NOT REACHED
      assert(FALSE)
    of "trunc" do
      a:= op(Lhs);
      if is(a, Type::Real) = TRUE or is(Rhs, Type::Real) = TRUE then
        return(piecewise([Rhs = 0, Dom::Interval(-1, 1)],
                         [Rhs in Z_ and Rhs > 0,
                          solvelib::preImage(a, x,
                                             Dom::Interval([Rhs],
                                                           Rhs + 1))],
                         [Rhs in Z_ and Rhs < 0,
                          solvelib::preImage(a, x,
                                             Dom::Interval(Rhs - 1,
                                                           [Rhs]))
                          ],
                         [not Rhs in Z_, {}]))
      else
        return(hold(solve)(Lhs=Rhs,x))
      end_if;
      // NOT REACHED
      assert(FALSE)
    of "Re" do
      a := rectform(op(Lhs, 1));
      if op(a, 3) = 0 and type(op(a, 1)) <> "Re" then
        return(solvelib::isolate(op(a, 1), Rhs, x, options, reclevel+1, known))
      end_if;
      // else handle below by rewriting
      break
    of "Im" do
      a := rectform(op(Lhs, 1));
       if op(a, 3) = 0 and type(op(a, 2)) <> "Im" then
        return(solvelib::isolate(op(a, 2), Rhs, x, options, reclevel+1, known))
      end_if;
      // handle below by rewriting
      break
    of "heaviside" do
      a:=op(Lhs);
      if Rhs=heaviside(0) then
        return(solvelib::isolate(a, 0, x, options, reclevel+1, known))
      end_if;
      return(piecewise
            ([Rhs=0, solve(a<0, x, options)],
             [Rhs=1, solve(a>0, x, options)],
             [Rhs<>0 and Rhs<>1, {}]))
    of "fact" do
    of "gamma" do
      if iszero(Rhs) then
        return({})
      else
        // we have no inverse gamma function
        return(hold(solve)(Lhs=Rhs, x))
      end_if
    of piecewise do
      return(piecewise::extmap(Lhs, solvelib::isolate, args(2..args(0))))      
    // of "function" do
    otherwise // cannot solve f(x)=C if f is unknown
      return(hold(solve)(Lhs=Rhs,x))
  end_case;


// remove calls to abs and sign
  Lhs:= eval(misc::maprec(Lhs,
                     {"abs"} = proc(t)
                                 name rewriteAbs;
                                 local a;
                               begin
                                 a:= op(t);
                                 if has(a, x) then
                                   if is(a, Type::Real) = TRUE then
                                     piecewise([a>=0, a], [a<=0, -a])
                                   else
                                     sqrt(Re(a)^2 + Im(a)^2 )
                                   end_if
                                 else
                                   t
                                 end_if
                               end_proc,
                     {"sign"} = proc(t)
                                  name rewriteSign;
                                  local a;
                                begin
                                  a:= op(t);
                                  if has(a, x) then
                                    if is(a, Type::Real) = TRUE then
                                      piecewise([a=0, 0], [a<0, -1],
                                                [a>0, 1])
                                    else
                                      piecewise(
                                                [a=0, 0],
                                                [a<>0,
                                                 a/ sqrt(Re(a)^2 + Im(a)^2 )])
                                    end_if
                                  else
                                    t
                                  end_if
                                end_proc,
                     {"heaviside"} = proc(t)
                                       local a;
                                     begin
                                       a:= op(t);
                                       if has(a, x) then
                                         piecewise([a<0, 0], [a>0, 1])
                                       else
                                         t
                                       end_if
                                     end_proc,
                     {"min"} = proc(t)
                                 local argv,
                                 j: DOM_INT;
                               begin
                                 // do no case analysis for arguments that
                                 // do not contain x
                                 argv:= split([op(t)], has, x);
                                 if argv[1] =[] then
                                   return(t)
                                 elif argv[2] =[] then
                                   argv:= argv[1]
                                 else         
                                   argv:= argv[1].[min(op(argv[2]))]
                                 end_if;
                                 piecewise(
                      [_and(argv[i] <= argv[j] $j=1..nops(argv)), argv[i]]
                      $i=1..nops(argv))
                               end_proc,
                     {"max"} = proc(t)
                                 local argv,
                                 j: DOM_INT;
                               begin
                                  argv:= split([op(t)], has, x);
                                 if argv[1] =[] then
                                   return(t)
                                 elif argv[2] =[] then
                                   argv:= argv[1]
                                 else         
                                   argv:= argv[1].[max(op(argv[2]))]
                                 end_if;
                                 piecewise(
                      [_and(argv[i] >= argv[j] $j=1..nops(argv)), argv[i]]
                      $i=1..nops(argv))
                               end_proc
                                 ));




  if type(Lhs)=piecewise then
    return(piecewise::solve(piecewise::extmap(Lhs, _subtract, Rhs),
                            x, options))
  end_if;


 // some changes on the left hand side
  // expand expressions like Re(x^2) using rectform
  // replace binomials by gamma

  Lhs:= misc::maprec(Lhs,
                    {"Re", "Im", "conjugate"}=
                    proc(t)
                      name applyRectform;
                    begin
                      if has(t, x) then
                        expr(rectform(t))
                      else
                        t
                      end_if
                    end_proc,
                    {"binomial"} =
                    proc(t)
                      name applyRewriteGamma;
                    begin
                      if has(t, x) then
                        rewrite(t, gamma)
                      else
                        t
                      end_if
                    end_proc
                    ); 
  
  userinfo(10, "Applying rectform to certain special functions gives ".
           expr2text(Lhs)." = ". expr2text(Rhs));




  ////////////////////////////////////////////////////////////////////
  ///  handle equations that contain Re or Im
  ///////////////////////////////////////////////////////////////////

  if has(Lhs, hold(Re)(x)) or has(Lhs, hold(Im)(x)) then
    // substitute both Re(x) and Im(x) by new variables u and v, x=u+Iv
    u:=genident("U");
    v:=genident("V");
    // assume does not evaluate its arguments, we need a trick
    eval(hold(assume)(u, Type::Real));
    eval(hold(assume)(v, Type::Real));
    // immediately substitute Re(x) and Im(x) in order
    // to avoid the computation of Re(u+I*v) etc.
    Lhs:= subs(Lhs, hold(Re)(x)=u, hold(Im)(x)=v);
    Lhs:= eval(subs(Lhs, x=u+I*v));
    userinfo(10, "Substituting ".expr2text(hold(Re)(x))." by ".expr2text(u).
             " and ".expr2text(hold(Im)(x))." by ".expr2text(v)." gives ".
             expr2text(Lhs=Rhs));
    l:=solve(Lhs-Rhs, [u,v], Domain=Dom::Real );
    userinfo(10, "Solving the system gives ".expr2text(l));
    if hastype(l, "solve") then
      return(hold(solve)(lhside=Rhs, x))
    end_if;
    // now l is either an "_in" or a set of lists of equations
    // l contains solutions (u_0,v_0) for u and v; for each such solution,
    // u_0 + I*v_0 is a solution of the orig. equation for x
    l := solvelib::vectorForm(l, [u,v]);
    // now l consists of tuples (u, v), that have to be interpreted as u+I*v

    R2toC:=
    proc(set)
    begin
      case type(set)
        of piecewise do
          return(piecewise::extmap(set, R2toC))
        of DOM_SET do
        of Dom::ImageSet do
          return(map(set, t -> t[1] + I*t[2]))
        of "_union" do
        of "_intersect" do
        of "_minus" do 
          return(map(set, R2toC))
      end_case;
      // NOT REACHED
      assert(FALSE)
    end_proc;

    l:= R2toC(l);
    
    userinfo(10, "Back substituting gives ".expr2text(l));
    // indeterminates in the solution must be understood as
    // ranging over all reals
    if (a:=indets(l) intersect {u,v})={} then
      return(l)
    elif nops(a)=2 then
      // should not happen !
      return(solvelib::Union(solvelib::Union(l, u, R_), v, R_))
    else
      return(solvelib::Union(l, op(a), R_))
    end_if;
  end_if;


  //////////////////////////////////////////////////////////////////////
  ///  Rewrite by system
  //
  // try to see the expression Lhs as an element of a field
  // extension of Q(x)
  //////////////////////////////////////////////////////////////////////

  if options["DontRewriteBySystem"] = FALSE then
    l:= solvelib::makeSystem({Lhs - Rhs}, [x]);
    if nops(op(l,3))>1 then
    // there was at least one variable created by a substitution
      newvars:= select(op(l, 3), _unequal, x);
      userinfo(10, "Newly created variables are :".expr2text(newvars));
//      a:=op(l,4);
//      userinfo(10, "Conditions are ".expr2text(a));
      u:=subs(op(l,2), `#I` = I);
      userinfo(10, "Nonalgebraic part is ".expr2text(u));
      l:=op(l,1), op(l,3);
      userinfo(10, "Algebraic system is  ".expr2text(l));

      // did we have to rewrite radicals as extra equations ?
      roots:= bool(nops(op(l, 1)) > 1); 

      
      // choose variable to solve for
      case nops(u)
        of 0 do
          v:=x;
          newlhs:= Lhs;
          break
        of 1 do
          // only one non-alg. condition
          v:=lhs(op(u,1));         
          newlhs:= subs(Lhs, rhs(op(u,1)) = v)
      end_case;

      if nops(u)<= 1 then
        userinfo(10, "Solving the system for ".expr2text(v))
      else
        userinfo(10, "Cannot solve system with more than one non-algebraic".
                 " equation")
      end_if;
    
      if nops(u)=0
        // purely algebraic case
        or nops(u)=1 and not has(op(l,1),x) then
      // not really an eq. in x, but in f(x) (with f(x)=rhs(op(u,1)) )
        l:=solve(l, options);
        userinfo(10, "Solving gives ".expr2text(l));   

      // now l is the set of all tuples [v, var1, var2, ...]
      // that solve the system created by makeSystem
      // (but possibly, there are too many solutions)  
      // Here, var1,  var2, etc. are variables newly created by makeSystem
      // We apply a projection to the variable v:

        l:= solvelib::project(l, v,
                              indets({Lhs, Rhs}) minus Type::ConstantIdents
                              minus {v});

        
        if l=FAIL then
          return(hold(solve)(lhside=Rhs, x))
        end_if;

       // now we have to throw away the additional solutions by attaching
       // an extra condition "valid if satisfying Lhs=Rhs" to every solution 
       // provided that there were roots in the equations

        checkSols:=
        proc(l)
          local conds: DOM_LIST;
        begin
          if type(l) <> DOM_SET then
            // gives additional solutions, improve !!
            l
          elif options["IgnoreSpecialCases"]=TRUE then
            // we do not construct a piecewise, but just check
            // whether something *can* be a solution
            select(l,
                   proc(sol)
                     name checkSolution;
                     local cond;
                   begin
                      if traperror(
                            (cond:= subs(newlhs, v = sol) = Rhs )) <> 0 then
                        FALSE
                      else
                        bool(is(cond) <> FALSE)
                      end_if
                   end_proc
                     )
          else
            l:= [op(l)];
            conds:= map(l,
                        proc(sol)
                          local result;
                        begin
                          if traperror(
                            (result:= subs(newlhs, v = sol) = Rhs )) = 0 then
                            result
                          else
                            FALSE
                          end_if
                        end_proc
                        );
            _union(piecewise([conds[i], {l[i]}],
                           [not conds[i], {}])
                 $i=1..nops(l))
          end_if
        end_proc;

        if roots then
          // checking solutions
          if type(l) = piecewise then
            l:= piecewise::extmap(l, checkSols)
          else
            l:= checkSols(l)
          end_if;
        end_if;
        
        if nops(u)=0 then
          return(l)
        else
          // we have solved for v= f(x); return the set of all x
          // such that f(x) is in that set
          if domtype(l) = DOM_SET and nops(l) = 1 then
            return(solvelib::isolate(rhs(op(u, 1)), op(l, 1), x, options,
                                     reclevel+1, known  ))
          else
            return(solvelib::preImage(rhs(op(u,1)), x, l, options))
          end_if
        end_if;
      end_if;
    end_if;
  end_if; // option["DontRewriteBySystem""] = FALSE

    
   /* try to expand the left hand side if it contains tan, cot, ... */
  l:=expand(Lhs);
  if indets(l,RatExpr) <> indets(Lhs,RatExpr) then
     userinfo(5, "Expanding the left hand side gives ".
              expr2text(l). " = ".expr2text(Rhs));
     return(solvelib::isolate(l, Rhs, x, options, reclevel+1, known))
   end_if;
   if testtype(Lhs, Type::RatExpr(x)) then
     return(solvelib::solve_minus
            (solvelib::solve_poly(poly(numer(Lhs-Rhs),[x]), x, options),
             solvelib::solve_poly(poly(denom(Lhs-Rhs), [x]), x, options),
             options))
   end_if;
   /* try to decompose */
  l:=rationalize(Lhs, NIL, NIL, {"sin","exp"});
  userinfo(3,"after rationalize, equation is",l[1]=Rhs,"with",l[2]);
  l[1]:=l[1]-Rhs;

   // special case: only one substitution and equations remains univariate
  if nops(l[2])=1 and not has(l[1],x) then
    i:=op(l[2],[1,1]);
     a:=solvelib::solve_eq(l[1],i,options);
     userinfo(3,"Set of solutions of ",l[1]," for ",i," is ",a);
     if not has(a, "solve")
       and a<>{i-l[1]} then
       /* to avoid inf. loops, i.e. in solve(exp(x)^(1/4)=C,x) */
       return(solvelib::preImage(op(l[2],[1,2]), x, a, options))
     end_if
  end_if;
  // end of special case

  // try to rewrite

  l:= rewrite(Lhs, exp);
  // unify expressions like exp(a) and exp(-a)
  l:= misc::maprec(l, {"exp"} =
                   proc(subexpr)
                     local a;

                   begin
                     assert(type(subexpr) = "exp");
                     a:= op(subexpr);
                     if stdlib::hasmsign(a) then
                       1/exp(-a)
                     else
                       subexpr
                     end_if
                   end_proc
                  );
  if l <> Lhs then
    solvelib::isolate(l, Rhs, x, options, reclevel + 1, known);
    if not hastype(%, "solve") then
      return(%)
    end_if
  end_if;

  // try simplification: but only once, and increase the recursion level
  // such that few furher calls are possible

  if reclevel <= 19 then
    simplify(Lhs);
    if %<>Lhs then
      // solve the simplified version
      // note that the singularities may be different now!
      return(solvelib::isolate(%, Rhs, x, options, max(reclevel+1, 20), known))
    end_if;
  end_if;


  // give up
  hold(solve)(Lhs=Rhs, x, solvelib::nonDefaultOptions(options))

end_proc:


// checkAngle
// return those elements el of set such that
// the argument of subs(formula, var=el) is > -angle and <= angle

solvelib::checkAngle:=
proc(S, formula, var, angle, options, maxreclevel = 10)
  local yes, no, dontknow, y, correct, testel, selectFromSET, i;
begin
  // correct - formula that is equivalent to "x has an arg. > -angle and
                                                // <= angle "
  correct:=
  proc(x)
    local argx, cond, xpr;
  begin
    argx:= arg(x);
    if type(argx)=piecewise then
      // flatten out
      cond:= [extop(argx)];
      [cond, xpr]:= [map(cond, op, 1), map(cond, op, 2)];
      _or((cond[i] and xpr[i] <= angle and xpr[i]> -angle)
          $i=1..extnops(argx))
    else
      arg(x) <= angle and arg(x) > -angle
    end_if
  end_proc;


  // testel -> tests whether el has to be an element of the set
  // to be returned
  testel:=
  proc(el)
    local result;
  begin
    if traperror((result:=correct(subs(formula, var=el)))) <> 0 then
      return(FALSE)
    else
      return(result)
    end_if
  end_proc;

  selectFromSET :=
  proc(S)
  begin
    case type(S)
      of DOM_SET do
        // yes: elements that must be returned
        // no : eleemnts that must not be returned
        // dontknow: (parametrized) elements where it depends on the parameters
        //           whether they are solutions or not
        [yes, no, dontknow] := split(S, is@testel);
        if options["IgnoreSpecialCases"] = FALSE then 
          yes union _union(op(map(dontknow,
                                  el -> piecewise([testel(el), {el}],
                                                  [not testel(el), {}]))))
        else
          yes union dontknow
        end_if;
        break
      of piecewise do
        piecewise::extmap(S, selectFromSET); break
      of "_union" do
      of "_intersect" do  
        map(S, selectFromSET); break
      of "_minus" do
        selectFromSET(op(S, 1)) minus op(S, 2); break
      otherwise
        if options["IgnoreSpecialCases"] = TRUE then
          S
        else
          // too complicated ?
          y:= genident();
          S intersect solve(testel(y), y, MaxRecLevel=maxreclevel)
        end_if
    end_case;    
  end_proc;
     
 
  selectFromSET(S)
 
end_proc:
    


/* end of file */

