/* $Date: 2004/09/30 16:47:48 $ $Author: stefanw $ $Revision: 1.31 $ */


/* solvelib::preImage(eq, x, S)

   determine the set of all C such that subs(eq, x=C) is an
   element of S

*/


solvelib::preImage:=
proc(eq, x: DOM_IDENT, S)
local i, opt, v;

begin
  if args(0)<3 then
    error("Wrong number of arguments")
  end_if;

  opt:=solvelib::getoptions(args(4..args(0)));
  
  if S::dom::preImage<>FAIL then
    return(S::dom::preImage(args()))
  end_if;
  
  if S=solvelib::BaseSet then
    return(solvelib::solve_minus(solvelib::BaseSet, discont(eq, x, undefined),
                                 opt))
  end_if;
  if eq=x then
    return(S)
  end_if;
  case type(S)
    of DOM_SET do
      return(_union(solve(eq=op(S,i), x, opt) $i=1..nops(S)))
    of "_union" do  
    of "_intersect" do
    of "_minus" do  
      return(map(S, u-> solvelib::preImage(eq, x, u, opt)))
    of Dom::Interval do
      return(solve(_in::expand(eq, S), x, opt))
    of Dom::ImageSet do
      // S = { f(y); y \in A }
      // then eq(x) \in S iff eq(x)=f(y)
      // but first take care of the alias problem x=y
      if S::dom::nvars(S)=1 then
        v:= S::dom::variables(S)[1];
        if v = x or
          has(eq, v) then
          v:= genident();
          S:= S::dom::setvar(S, v)
        end_if;
        save v;
        assume(v, solvelib::set2prop(S::dom::sets(S)[1], Superset));
        return(solvelib::Union(solve(eq=expr(S), x, opt),
                                       op(S::dom::variables(S), 1),
                                       op(S::dom::sets(S), 1)))
      else
        break
      end_if
    of solvelib::BasicSet do
      case S
        of solvelib::BasicSet(Dom::Complex) do
          return(S minus discont(eq, x, undefined))
        of solvelib::BasicSet(Dom::Real) do
          return(solvelib::preImageReal(eq, x, opt))
        of solvelib::BasicSet(Dom::Integer) do
          // we can do very little here
          if type(eq) = "_plus" then
            eq:= select(eq, summand -> is(summand in Z_) <> TRUE)
          end_if;
          break // continue by solving below          
          // otherwise
          // fall through 
      end_case;
  end_case;
  
  i:=genident();
  solve(eq=i, x, opt);
  solvelib::Union(%, i, S)
end_proc:
    

// special case S = R_: determine all x such that f(x) is real

solvelib::preImageReal:=
proc(eq, x, options)
  local zeroes, fac;
begin
  // simple test
  if is(eq, Type::Real) = TRUE then
    return(C_)
  end_if;

  case type(eq)
    of "_plus" do
      eq:= select(eq, s -> is(s in R_) <> TRUE);
      if type(eq) <> "_plus" then
        return(solvelib::preImageReal(eq, x, options))
      else
        break
      end_if
    of "_mult" do
      zeroes:= split([op(eq)], s -> is(s in R_) <> TRUE);
      assert(nops(zeroes[3]) = 0);
      if nops(zeroes[2]) = 0 then
        // there are no real factors to divide off
        break
      end_if;
      // there are real factors; eq is zero iff the product of the other
      // factors is real or if one of the real factors is zero 
      eq:= _mult(op(zeroes[1]));
      return(solvelib::preImageReal(eq, x, options) union
             _union(solve(fac, x, options) $fac in zeroes[2]))
    of "_power" do
      if type(op(eq, 2)) = DOM_RAT then
        return(solve(op(eq, 1) >= 0, x) )
      end_if;
      break
  end_case;
  
  return(solve(expr(Im(rectform(eq))), x, options))

end_proc:
