/* */
/* $Date: 2003/05/13 09:17:26 $ $Author: stefanw $ $Revision: 1.52 $ */
/* */
/* zimmerma, 14/12/95 */

/*--
	solvelib::solve_sys
--*/

// Sys - set or list of equations
// Unk - set or list of unknowns

solvelib::solve_sys :=
proc(Sys, Unk:Type::Union(DOM_SET, DOM_LIST))
  local
  sys:DOM_LIST,
  newsys,eq,i,R, pw,
  unk: DOM_LIST,
  newunk: DOM_LIST,
  options: DOM_TABLE,
  eqtypes, ring,
  simplifyCond,
  removeIdents: DOM_PROC,
  check: DOM_PROC,
//  substitutions: DOM_SET,
//  params: DOM_SET,
  ineqs: DOM_LIST, dummy, solutions, sol, f,
  cond1, cond2, inds, lc, lcoefflist, linear, branches;
  
begin
  case type(Sys)
    of DOM_SET do
      sys:=[op(Sys)];
      break
    of DOM_LIST do
      sys:=Sys;
      break
    otherwise
      error("First argument must be list or set")
  end_case;

  options:=solvelib::getoptions(args(3..args(0)));


  if options["IgnoreSpecialCases"] then
    pw:= solvelib::ignoreSpecialCases
  else
    pw:= piecewise
  end_if;

  
  if nops(sys)=1 and nops(Unk)=1 then
    // really only one equation in one unknown
    R:=solve(op(sys, 1), op(Unk, 1), options);
    case type(R)
      of "solve" do
        return(hold(solve)(Sys, Unk, solvelib::nonDefaultOptions(options)))
      of DOM_SET do
        return({[op(Unk, 1)= op(R, i)] $i=1..nops(R)})
      otherwise
        return(_in(op(Unk, 1), R))
    end_case
  end_if;
  
  R:= options["Domain"];
  
  // convert all equations and polys to expressions

  
  sys:=map(sys,
           proc(equation)
             name makeExpression;
             local ring;
           begin
             case type(equation)
               of "_equal" do
                 op(equation,1)-op(equation,2);
                 break
               of DOM_POLY do
                 ring:= op(equation, 3);
                 if has(ring, IntMod) then
                   ring:= Dom::IntegerMod(op(ring,1))
                 end_if;
           if R=hold(Expr) then
             R:=ring
           else
             // try to understand equation
             // as a polynomial over the previously chosen ring
             equation:= poly(equation, R);
             if equation = FAIL then
               error("Coefficient ring of polynomial does not ".
                     "match previously chosen domain for solving".
                               " or other coefficient ring");
             end_if;
           end_if;
           expr(equation);
           break
             otherwise
           equation
           end_case
           end_proc );

  eqtypes:= map(sys, domtype);
  eqtypes:= select(eqtypes, c -> (slot(c, "solve_sys")<>FAIL));
  if nops(eqtypes) > 0 then
    userinfo(50, "System solver has been overloaded");
    ring:= eqtypes[1];
    sys:= map(sys, ring::convert);
    if has(sys, FAIL) then
      error("Could not convert all of the equations")
    end_if;
    options["Domain"]:=R;
    return(ring::solve_sys(sys, Unk, options))
  end_if;


  // toDo: single objects overload solve - intersect solutions
  // with solution to the rest of the system
  
  userinfo(3, "System solver called for ".expr2text(sys)." in unknowns ".
	expr2text(Unk));
  userinfo(10, "Options are ".expr2text(options));

  if stdlib::hasfloat(sys) then
    sys:=numeric::rationalize(sys);
    solvelib::solve_sys(sys, Unk, options);
    return(misc::maprec(%, {Dom::ImageSet, DOM_INT, DOM_RAT, DOM_COMPLEX,
                            DOM_IDENT}= float))
  end_if;
    
  
  if options["Multiple"]=TRUE then
    error("Option multiple is not allowed in the system solver")
  end_if;
  
  unk:=[op(Unk)];


  [sys, ineqs, dummy]:= split(sys, x -> type(x) <> "_unequal");

  if nops(ineqs) > 0 then
    // we can only handle a few cases of mixed systems, unfortunately
    solutions := solvelib::solve_sys(sys, unk, options);
    if type(solutions) <> DOM_SET then
      // cannot handle _in - format
      return(hold(solve)(Sys, unk))
    end_if;
    if nops(solutions) = 0 then
      // empty set
      return({})
    end_if;
    assert(map(solutions, type) = {DOM_LIST});
    // use dummy as a flag:
    // TRUE means that producing a result is still possible
    dummy := TRUE;
    solutions:= map
                (solutions,
                 proc(l: DOM_LIST)
                   name checkSolution;
                   local newineqs, l1, l2, l3;
                 begin
                   if not dummy then
                     FALSE
                   elif {op(map(l, type))} <> {"_equal"} then
                     dummy:= FALSE
                      // return FALSE; the return value
                      // isn't used anyway
                   else
                     newineqs:=map(ineqs, subs, l);
                     [l1, l2, l3]:= split(newineqs,
                                          ineq ->
                                          indets(ineq) intersect {op(unk)} = {});
               // _equal should never produce UNKNOWN:
                     assert(nops(l3) = 0);
               
               // now l1 contains the conditions on the free
               // parameters, and l2 contains a system of inequalities
               // we would still have to solve
               // since we cannot solve systems of inequalities,
               // we give up unless the property mechanism can help:
                     if map({op(l2)}, is) minus {TRUE} <> {} then
                       dummy:= FALSE
                     else
                 // return the solution plus a condition
                       _and(op(l1));
                       pw([%, {l}], [not %, {}])
                     end_if
                 end_if
                 end_proc
                 );
    
    if not dummy then
      return(hold(solve)(Sys, unk))
    else  
      // solutions is a set of piecewises [condition, {solution}].
      // piecewise::_union will expand this
      return(_union(op(solutions)))
    end_if    
  end_if;

  // handle inequalities
  if map({op(sys)}, type) intersect
    {"_less", "_leequal"} <> {} then
    return(solvelib::ineqsys(sys, {op(Unk)}))
  end_if;


  // remove zero equations

  sys:= select(sys, _not@iszero);

  
  if {op(Unk)} intersect Type::ConstantIdents <> {} then
    error("Invalid variable (mathematical constant was given)")
  elif nops(unk) = 0 then
    error("Need variable(s) to solve for")
  end_if;

  // no equations left?

  if nops(sys)=0 then
    return({[]})
  end_if;
  
   /* remove unknowns not appearing in equations */
  unk:= select(unk,
               proc()
               begin
                 has(sys, args(1))
               end_proc );

  // no unknowns remaining after selection ?
  
  if unk=[] then
    return((if {op(sys)}={0} then
              {[]}
            else
              {}
            end_if))
  end_if;
  
  // substitute unknowns that are not identifiers
  // substitute all indexed identifiers, including free parameters
  eq:= select({op(unk)}, not testtype, DOM_IDENT)
  union misc::subExpressions(sys, {"_index"});
  eq:= [op(eq)];
  if eq<>[] then
    eq:= [(op(eq,i) = genident()) $i=1..nops(eq)];
    newsys:= solvelib::solve_sys(subs(sys, eq), subs(Unk, eq), options);
    subs(newsys, (op(eq,[i,2])=op(eq,[i,1])) $i=1..nops(eq));
    return(%)
  end_if;


  // split products into separate equations
  // giving several systems
  // still to do ...
  

  
  // remove powers

  sys:= map(sys, proc(eq)
                 begin
                   if type(eq)<>"_power" then
                     eq
                   elif testtype(op(eq,2), Type::Constant)<>TRUE then
                     eq
                   elif is(op(eq, 2)>0) then
                     op(eq, 1)
                   else
                     // negative exponent: unsolvable
                     FAIL
                   end_if
                 end_proc
                   );
  if contains(sys, FAIL)>0 then
    return({})
  end_if;
  

  
  
  // handle linear systems
  // parameters are handled via ShowAssumptions, over Expr only


  if (newsys:= solvelib::solve_islinear(sys, unk, options)) <> FALSE then
    /* solve returns a set of solutions
       whereas linsolve always returns one solution */
    if R = hold(Expr) then
      i:= numeric::linsolve(newsys, unk, hold(Symbolic),
                            hold(ShowAssumptions));
      if op(i, 1) = FAIL then
        solutions:= {}
      else
        solutions:= {op(i, 1)}
      end_if;


      
  


      lcoefflist:= op(i, 3);
      cond1:=_and(op(op(i,2)));
      cond2:=_and(op(op(i, 3)));

      if nops(lcoefflist) = 0 or options["IgnoreSpecialCases"] then
        return(pw([cond1 and cond2,
                   solutions],
                  [not cond1, {}]))
      end_if;

      assert(map({op(lcoefflist)}, type) = {"_unequal"});

      // replace a <> b by a-b
      lcoefflist:= map(lcoefflist, ineq -> op(ineq, 1) - op(ineq, 2));
      
      branches:=[[cond1 and cond2, solutions],
                    [not cond1, {}]];
      // try to find out what happens if lc = 0 for as many elements
      // of lcoefflist as possible (currently, only linear polys are handled)
      
      for lc in lcoefflist do
        inds:= indets(lc);
        for i in inds do
          if testtype(lc, Type::PolyExpr([i])) and
            degree(lc, [i]) = 1 then
            linear:= [lc, i];
            f:= poly(linear[1], [linear[2]]);
            if traperror((dummy:= eval
                          (subs(newsys,
                                linear[2] = -coeff(f,0) / coeff(f, 1)))))
              = 0 then
              branches:= branches.[[linear[1] = 0, solve(dummy, unk)]]
            end_if              
          end_if
        end_for;
      end_for;
    
          
      return(pw(op(branches)))
         
    
      
    else // solve over a domain R
      i:= linsolve(newsys, unk, hold(Domain) = R);
      if i = FAIL then
        return({}) /* no solution */
      else
        return({i})
      end_if
    end_if;

  end_if;
  
   // handle algebraic systems (*do* handle parameters in a special way)

  if (newsys:= solvelib::solve_isalgebraic({op(sys)}, unk)) <> FALSE then


    solutions:=  solvelib::solve_algebraic(op(newsys, 1), unk, NIL, options);
    
    check:=
    proc(solution)
      local i, substituted;
    begin
      // assume that a solution is correct if it does not consist
      // of equations
      if map({op(solution)}, type) minus {"_equal"} <> {} then
        return(TRUE)
      end_if;
      if traperror((substituted:=eval(subs(op(newsys, 2), solution)))) <> 0 then
        FALSE
      else  
        _and((op(substituted, i) <> 0) $i=1..nops(substituted))
      end_if
    end_proc;

    
    // select correct solutions, depending on op(newsys, 4)

    // comment the following line to produce a kernel bug !!



    proc(solutions)
      begin
    
        solutions:=[op(solutions)];

   
    
        solutions:= (pw([check(op(solutions, i)), {op(solutions, i)}],
                        [not check(op(solutions, i)), {}])
                     $i=1..nops(solutions));
    
        return(_union(solutions))
    end_proc;

    if type(solutions) = piecewise then
      piecewise::extmap(solutions, %)
    else
      %(solutions)
    end_if;
    return(%)

   
  end_if;
  

  // try to make the system into a polynomial system
  // by additional substitutions

  
  newsys:= map(sys, numer);
  // toDo: check whether solutions are zeroes of the denominator
  newsys:= solvelib::makeSystem({op(newsys)}, unk);
  // now newsys is a sequence equations, non-algebraic, vars, conditions
  if nops(op(newsys, 2)) = 0 then
    // no non-algebraic equations
    sol:= solve(op(newsys, 1),
   // select(op(newsys, 3), var -> not has(unk, var)).unk,
               op(newsys, 3),
               options);


      
    removeIdents:=
    proc(S: DOM_SET): Type::SetOf(DOM_LIST)
      local expandOr;
    begin
      S:=
      map(S,
          proc(sollist): DOM_LIST
            name removeIdentsFromSolution;
            local i, inversions, conditions;
          begin
          // look for inversions  

            conditions:= op(newsys, 4);
            
            repeat
              inversions:= FALSE;
              for i from 1 to nops(sollist) do
            // look for inversions of the variable ordering:
                if type(sollist[i]) = "_equal" and
                  contains(unk, op(sollist[i], 1)) = 0 and
                  indets(op(sollist[i], 2)) intersect {op(unk)} <> {} then
                  conditions:= subs(conditions, sollist[i]);
                  sollist:= subs(sollist, sollist[i]);
                  delete sollist[i];
                  inversions:= TRUE;
                  break
                end_if;
              end_for
             until not inversions end_repeat;

          sollist:= select(sollist, equ ->
                           indets(equ) minus
                           ({op(unk)} union Type::ConstantIdents) = {});
          conditions:= select([op(conditions)],
                              cond -> indets(cond) minus
                              ({op(unk)} union Type::ConstantIdents) = {});


          for i from 1 to nops(sollist) do
            if type(sollist[i]) = "_equal" then
              conditions:= subs(conditions, sollist[i])
            end_if
          end_for;

          conditions:= map(conditions, property::simpex);
          if contains(conditions, FALSE) > 0 then
            return(null())
          else  
            conditions:=select(conditions, _unequal, TRUE);
            sollist.conditions
          end_if;
          end_proc
          );
      // now S may still not be well-formed: the solutions could
      // contain things like _or


      // expandOr: write a solution [...., cond1 or cond2, ...]
      // as a set {[...., cond1, ..], [..., cond2, ...]}
      
      expandOr:=
      proc(l: DOM_LIST): DOM_SET
        local i, j;
      begin
        if not has(l, _or ) then
          return({l})          
        end_if;

        for i from 1 to nops(l) do
          if type(l[i])= "_or" then
            return(_union(expandOr([op(l, 1..i-1), op(l[i], j),
                              op(l, i+1..nops(l))]) $j=1..nops(l[i])))
          end_if;
        end_for;
      end_proc;

      _union(op(map(S, expandOr)))
      
    end_proc;

    
    
    if type(sol) <> piecewise then
      solutions:= removeIdents(sol);
    else
      solutions:={};
      // consider conditions on the additional variables as satisfiable
      // (this is questionable !)
      simplifyCond:=
      proc(cond: Type::Boolean): Type::Boolean
      begin
        case type(cond)
          of "_and" do
          of "_or" do
            map(cond, simplifyCond);
            break
          otherwise
            if indets(cond) minus ({op(unk)} union Type::ConstantIdents)
              <> {} then
              TRUE
            else
              cond
            end_if
        end_case
      end_proc;

   
      
      for i from 1 to nops(sol) do
        simplifyCond(piecewise::condition(sol, i));
        solutions:= solutions union
                    pw([%, removeIdents(piecewise::expression(sol, i))],
                              [not %, {}])
      end_for;
    end_if;


    check:=
    proc(solution)
      local i, substituted;
    begin
      // assume that a solution is correct if it does not consist
      // of equations
      if map({op(solution)}, type) minus {"_equal"} <> {} then
        return(TRUE)
      end_if;
      if traperror((substituted:=eval(subs(sys, solution)))) <> 0 then
        FALSE
      else  
        _and((op(substituted, i) = 0) $i=1..nops(%))
      end_if
    end_proc;

    
    // select correct solutions, depending on op(newsys, 4)

    // comment the following line to produce a kernel bug !!
    solutions:=[op(solutions)];
      
    solutions:= (pw([check(op(solutions, i)), {op(solutions, i)}],
                     [not check(op(solutions, i)), {}])
                  $i=1..nops(solutions));
    
    return(_union(solutions))

  end_if;


  // test whether the system is polynomial in at least some of the
  // variables; if yes, solve at least for them
  newunk:= select(unk,
           proc(x)
           begin
             not has(indets(sys, PolyExpr) minus {x}, x)
           end_proc
             );
  // now newunk is a list consisting of all "good" unknowns
  // select those equations that contain only "good" unknowns,
  // and solve for them
if nops(newunk) > 0 then
  newsys:= select(sys, equ -> indets(equ) intersect {op(unk)}
    minus {op(newunk)} = {});
  solutions:= solve(newsys, newunk);
  if type(solutions) = DOM_SET then
      // solver returned set - of - lists format
    if solutions = {} then
      return({})
    end_if;
    assert(map(solutions, type) = {DOM_LIST} );
        // for each solution (given as a list), append the
        // solutions for the non-polynomial vars
    if not contains(map(solutions, nops), 0) then
      // to avoid infinite recursions; otherwise localsys =
      // the original system
      return(map(solutions,
              proc(liste)
                local localsys, eq, sol, localunk, tail, subsfailed: DOM_BOOL;
              begin
                localsys:= sys;
                localunk:= unk;
                subsfailed := FALSE;
                for eq in liste do
                  if type(eq) = "_equal" then
                    assert(type(lhs(eq)) = DOM_IDENT);
                    localsys:= subs(localsys, lhs(eq) = rhs(eq))
                  else
                    // cannot handle this at the moment
                    subsfailed:= TRUE
                  end_if;
                  localunk:= select(localunk, _unequal, op(eq, 1));
                end_for;
                // there must be unknowns remaining; for, otherwise
                // the original system is algebraic, and we would have
                // noticed this before
                assert(nops(localunk) > 0);
                if subsfailed then
                  // this is arguably a bad format, but ...
                  // we just append an unevaluated solve-call to the list
                  return(liste.[hold(solve)(localsys, localunk)])
                else
                  // all substitutions could be performed; do
                  // another solve-call
                  sol:= solve(localsys, localunk);
                  if type(sol) = DOM_SET then
                    // set - of - lists format
                    // replace liste by sequence of lists
                    return(liste.tail $tail in sol)
                  else
                    // sol is some ... in ..., or hold(solve)(..)
                    return(append(liste, sol))
                  end_if
                end_if;
                  // NOT REACHED
                assert(FALSE)
              end_proc
                ))
      // else fall through
    end_if
    // else fall through
  end_if;
else // nops(newunk) = 0
  userinfo(50, "System is not polynomial in any of the unknowns")
end_if;



  if nops(sys)=1 then /* only one equation */
    eq:=op(sys);
      /* look for the optimal unknown */
    if nops(unk)=1 then
      // this  can happen if some equations (e.g. 0=0) have
      // been removed since the head of solve_system was processed
      i:=1
    else
      /*
      for i from 1 to nops(unk) do
        if Type::Linear(eq,[op(unk,i)])<>FALSE then
          break
        end_if
      end_for;
      if i>nops(unk) then
        i:=1
      end_if */
      return(hold(solve)(sys, Unk))
    end_if;
    newsys:=solvelib::solve_eq(eq, op(unk,i), options); 
    if type(newsys)<>"solve" then
      return(op(unk, i) in  newsys)
    end_if
  end_if;
  userinfo(1,"Could not solve the non-algebraic system");
  hold(solve)(sys,Unk)
 
end_proc:

/* end of file */

