// $Date: 2003/10/20 14:10:05 $ $Author: stefanw $ $Revision: 1.7 $

// solvelib::vectorForm - writes the output of the system solver
// as a set of vectors
// a solution var1=value1,..., varn=valuen
// corresponds to the vector [value1, ...,valuen]
// if varlist was [var1,...,varn]


solvelib::vectorForm:=
proc(solution, varlist)
  local x, Rhs, i;
begin
case type(solution)
  of piecewise do
    return(piecewise::extmap(solution, solvelib::vectorForm, varlist))
  of "_in" do
    x:=op(solution,1);
    Rhs:=op(solution,2);
    case type(Rhs)
      of DOM_SET do
      // expand
        return({subs(varlist, x=op(Rhs, i)) $i=1..nops(Rhs)})
      of piecewise do
        return(map(Rhs, S-> solvelib::vectorForm(hold(_in)(x,S), varlist)))
      otherwise
        return(Dom::ImageSet(varlist, x, Rhs))
    end_case
  of DOM_SET do
    // solution consists of lists of equations or
    // inequalities or in-expressions
    
    return(_union
           (op
            (map
             (solution,
              proc(eqlist)
                local equ, result, cond, var;
              begin
                cond:= table();
                result:= varlist;
                for equ in eqlist do
                  if type(equ) = "_equal" then
                    result:= subs(result, equ)
                  else
                    // create an image set
                    assert(type(equ) = "_leequal" or type(equ) = "_less"
                           or type(equ) = "_in");
                    if contains(varlist, op(equ, 1)) > 0  then
                      var:= op(equ, 1)
                    elif contains(varlist, op(equ, 2)) > 0 then
                      var:= op(equ, 2)
                    else
                      error("cannot find either side of ".expr2text(equ).
                            "in variable list".expr2text(varlist))
                    end_if;
                    cond[var]:= solve(equ, var)  
                   end_if;
                 end_for;
               // missing: handle unsubstituted vars ?!
              if nops(cond) = 0 then
                {result}
              else
                [op(cond)];
                Dom::ImageSet(result, map(%, op, 1), map(%, op, 2))
              end_if
              end_proc
              ))))
end_case;

// NOT REACHED

  error("Unknown type of solution")

end_proc:
  