// $Date: 2002/11/21 13:35:05 $ $Author: ccr $ $Revision: 1.21 $ 

/*++
        Ci -- the cosine integral

	Ci(x):= EULER + ln(x) + int((cos(t)-1)/t, t=0..x)
++*/

Ci:=
proc(x)
    option noDebug;
begin
  if args(0) = 0 then error("no arguments given")
  elif x::dom::Ci <> FAIL then return(x::dom::Ci(args())) 
  elif args(0) <> 1 then error("wrong no of args")
  end_if;

  case type(x)
    of DOM_SET do
    of "_union" do
      return(map(x, Ci))
  end_case;
  
  if iszero(x) then error("singularity") end_if;
      
  case domtype(x)
    of DOM_INT do
    of DOM_RAT do
      if x < 0 then return(Ci(-x)+I*PI) end_if; break
    of DOM_FLOAT do
      if x < 0 // workaround for bug in Ci::float
         then return(Ci::float(-x)+I*float(PI))
         else return(Ci::float(x))
      end_if;
      break;
    of DOM_COMPLEX do
      if domtype(op(x,1)) = DOM_FLOAT or
        domtype(op(x,2)) = DOM_FLOAT then
        return(Ci::float(x))
      end_if;
  end_case;

  if not testtype(x,Type::Arithmetical) then
      error("argument must be of 'Type::Arithmetical'")
  end_if;

  procname(x)
end_proc:

Ci(infinity):=0:
Ci(-infinity):= I*PI: // = limit(Ci(x), x=-infinity, Right)
                      // along the negative real axis
Ci:= funcenv(Ci, op(specfunc::Ci, 2)):
Ci::print:="Ci":
Ci::type:="Ci":
Ci::info:=
   "Ci(x) -- the Cosine integral EULER + ln(x) + int((cos(t)-1)/t, t=0..x)":
Ci::float:= specfunc::Ci:

Ci::diff:=  proc(x, y)
              local op1;
            begin  // e=Ci(f)
                op1 := op(x, 1);
                cos(op1)/op1 * diff(op1, y)
              end_proc:

Ci::undefined:= {0}:
Ci::realDiscont:= {0}:
Ci::complexDiscont :=
    loadproc(Ci::complexDiscont, pathname("STDLIB","DISCONT"), "Ci"):

Ci::series:= loadproc(Ci::series, pathname("SERIES"), "Ci"):

// end of file 
