// $Date: 2003/10/09 14:43:54 $ $Author: stefanw $ $Revision: 1.22 $ 

// frankp, 4.05.1995 

/*++
   Ei.mu

	Ei  --  The Exponential Integral
        Ei(x) = int(exp(-t)/t,t=x..infinity) [ denoted Ei(1,x) in Maple ]
++*/

Ei:=
proc(x)
  option noDebug;
  name Ei;
begin
  if args(0) = 0 then
    error("no arguments given")
  elif x::dom::Ei <> FAIL then
    return(x::dom::Ei(args()))
  elif args(0) <> 1 then
    error("wrong no of args")
  end_if;

  case type(x)
    of DOM_SET do
    of "_union" do
      return(map(x, Ei))
  end_case;
 
  if iszero(x) then error("singularity") end_if;

  case domtype(x)
    of DOM_FLOAT do
      return(Ei::float(x));
    of DOM_COMPLEX do
      if domtype(op(x, 1)) = DOM_FLOAT or domtype(op(x, 2)) = DOM_FLOAT then
        return(Ei::float(x));
      end_if
  end_case;
  
  if not testtype(x,Type::Arithmetical) then
    error("argument must be of 'Type::Arithmetical'")
  end_if;

  procname(x)
end_proc:


Ei(infinity):= 0:
Ei(-infinity):= -infinity:
Ei:= funcenv(Ei, op(specfunc::Ei, 2)):
Ei::info:=
    "Ei -- the exponential integral [try ?Ei for details]":
Ei::print:= "Ei":
Ei::type:= "Ei":
Ei::float:= specfunc::Ei:

Ei::diff:= loadproc(Ei::diff, pathname("STDLIB", "DIFF"), "Ei"):

Ei::series:= loadproc(Ei::series, pathname("SERIES"), "Ei"):

Ei::rectform:= loadproc(Ei::rectform, pathname("STDLIB", "RECTFORM"), "Ei"):

Ei::complexDiscont :=
    loadproc(Ei::complexDiscont, pathname("STDLIB","DISCONT"), "Ei"):
Ei::realDiscont:= {0}:
Ei::undefined:= {0}:

Ei::conjugate:=
  proc(x)
  begin
    if is(x > 0) = TRUE then
      return(Ei(x))
    end_if:
    if is(x < 0) = TRUE then // Ei(x) = ..real..-I*PI
      return(Ei(x) + 2*PI*I)
    end_if:
    hold(conjugate)(hold(Ei)(x))
  end_proc:

Ei::laplace:= loadproc(Ei::laplace, pathname("TRANS", "LAPLACE"), "L_Ei"):

// end of file 
