// 
// $Date: 2002/11/21 13:35:06 $ $Author: ccr $ $Revision: 1.24 $ 

// zimmerma, 10/08/95 

/*++
        Si -- the Sine integral

	Si(x):= int(sin(t)/t,t=0..x)

	Reference: Abramowitz & Stegun, pages 231-233
++*/

Si:=
proc(x)
  option noDebug;
  local f;
begin
  if args(0) < 1 then error("no arguments given")
  elif x::dom::Si <> FAIL then return(x::dom::Si(args()))
  elif args(0) <> 1 then error("wrong no of args")
  end_if;

  case type(x)
    of DOM_SET do
    of "_union" do
      return(map(x, Si))
    of DOM_INT do
    of DOM_RAT do
      if x < 0 then return(-Si(-x)) end_if; break
    of DOM_FLOAT do return(Si::float(x))
    of DOM_COMPLEX do
      if domtype(op(x,1)) = DOM_FLOAT or
         domtype(op(x,2)) = DOM_FLOAT
      then return(Si::float(x))
      end_if;
    //also normalize complex numbers ?
    //if op(x,1)<0 or (op(x,1)=0 and op(x,2)<0) then
    //   return(-Si(-x))
    //end_if;
  end_case;

  if not testtype(x,Type::Arithmetical) then
    error("argument must be of 'Type::Arithmetical'")
  end_if;

// in analogy to sin, cos:
  if type(x)= "_mult" then
      f:= op(x, nops(x));
      if testtype(f, Type::Real) and f < 0
         then return(-Si(-x))
      end_if;

    //also normalize complex factors ?
    //if testtype(f, Type::Complex) and 
    //   (Re(f) < 0 or (Re(f)=0 and Im(f)<0))
    //   then return(-Si(-x))
    //end_if;
  end_if;

  procname(x)
end_proc:

Si(0):=0:
Si(infinity):=PI/2:
Si(-infinity):=-PI/2:

Si:= funcenv(Si, op(specfunc::Si, 2)):
Si::print:= "Si":
Si::type:= "Si":
Si::info:= "Si(x) -- the sine integral int(sin(t)/t,t=0..x)":
Si::float:= specfunc::Si:

Si::diff:= proc(x, y)
             local op1;
           begin  // e=Si(f)
             op1 := op(x,1);
             sin(op1)/op1*diff(op1, y)
           end_proc:

Si::complexDiscont:= {}:
Si::realDiscont:= {}:
Si::undefined:= {}:

Si::series:= loadproc(Si::series, pathname("SERIES"), "Si"):

// end of file 
