
// ok, 27/01/94 

/*++
arccoth -- the inverse hyperbolic cotangens

arccoth(x)

x - expression
++*/

arccoth:= proc(x)
  option noDebug;
  name arccoth;
begin
  if args(0) < 1 then error("no arguments given")
  elif x::dom::arccoth <> FAIL then return(x::dom::arccoth(args()))
  elif args(0) <> 1 then error("wrong no of args") 
  end_if;

  case type(x)
    of DOM_SET do
    of "_union" do
      return(map(x, arccoth))

    of DOM_INT do
    of DOM_RAT do
	if x = 1 then
           error("singularity")
	elif x < 0 then
           return(-arccoth(-x))
        end_if;
	break;
    
    of DOM_FLOAT do
	return(arccoth::float(x));

    of DOM_COMPLEX do
	if domtype(op(x,1)) = DOM_FLOAT or
	   domtype(op(x,2)) = DOM_FLOAT then
	    return(arccoth::float(x))
//      elif op(x,1) = 0 then
//          return(-I*arccot(-I*x))
//      elif op(x,1) < 0 and op(x,2) < 0 then
//          return(-arccoth(-x))
	end_if;
	break;
  end_case;

  if not testtype(x,Type::Arithmetical) then
      error("argument must be of 'Type::Arithmetical'")
  end_if;

  procname(x)
end_proc:

arccoth(0) := PI/2*I:

arccoth:= funcenv(arccoth):
arccoth::type := "arccoth":
arccoth::print := "arccoth":
arccoth::info := "arccoth -- the inverse hyperbolic cotangent":
arccoth::inverse := "coth":

arccoth::undefined := {-1, 1}:
arccoth::realDiscont := {-1, 1}:
arccoth::complexDiscont :=
    loadproc(arccoth::complexDiscont, pathname("STDLIB","DISCONT"), "arccoth"):

arccoth::diff :=
  proc(f)
    local op1;
  begin
    op1 := op(f, 1);
    diff(op1, args(2..args(0))) * (-1/(op1^2-1))
  end_proc:

arccoth::float := e -> // arccoth(0) = PI/2*I              
                       // arccoth(1) = singularity
                       // arccoth(e) = arctanh(1/e) , e <> 0 
                  if e = 1 or e = float(1) then
                    error("singularity");
                  elif iszero(e)
                    then float(PI/2*I)
                  elif indets(float(e)) <> {} then
                    // e is symbolic
                    hold(arccoth)(float(e))
                  else arctanh::float(1/e)
                  end_if:

arccoth::hull := DOM_INTERVAL::arccoth@hull:

arccoth::series := loadproc(arccoth::series, pathname("SERIES"), "arccoth"):

arccoth::Content := (Out, data) -> Out::Capply(Out::Carccoth, Out(op(data))):

// end of file 
