// $Date: 2002/11/21 13:35:06 $ $Author: ccr $ $Revision: 1.26 $ 

// kg, 13/01/94 

/*++
arcsinh -- the inverse hyperbolic sine

arcsinh(x)

x - expression
++*/

arcsinh:= proc(x)
  option noDebug;
  local f;
  name arcsinh;
begin
  if args(0) = 0 then error("no arguments given")
  elif x::dom::arcsinh <> FAIL then return(x::dom::arcsinh(args()))
  elif args(0) <> 1 then error("wrong no of args") 
  end_if;

  case type(x)
    of DOM_SET do
    of "_union" do
      return(map(x,arcsinh))
    of DOM_INT do
    of DOM_RAT do
      if x < 0 then return(-arcsinh(-x)) end_if;
      break;
    of DOM_FLOAT do
      return(arcsinh::float(x));
    of DOM_COMPLEX do
      if domtype(op(x,1)) = DOM_FLOAT or
        domtype(op(x,2)) = DOM_FLOAT then
        return(arcsinh::float(x))
      end_if;
  end_case; 

  if not testtype(x,Type::Arithmetical) then
      error("argument must be of 'Type::Arithmetical'")
  end_if;

  case type(x)
    of "_mult" do
      f:= op(x, nops(x));
      if testtype(f, Type::Real) then
        if f < 0 then return(-arcsinh(-x)) end_if;
      end_if;
      break;
    of "sinh" do // arcsinh(sinh(x))=x when -PI/2 < Im x <= PI/2
      f:=round(-Im(op(x))/PI);
      if type(f)=DOM_INT then return((-1)^f*(op(x)+f*I*PI)) end_if;
  end_case;

  procname(x)
end_proc:

arcsinh(0):= 0:

arcsinh:= funcenv(arcsinh, op(specfunc::arcsinh, 2)):
arcsinh::print := "arcsinh":
arcsinh::info := "arcsinh -- the inverse hyperbolic sine":
arcsinh::type := "arcsinh":
arcsinh::float := specfunc::arcsinh:
// potential library fix for PARI's arcsinh:
// proc(x) begin
//    x:= float(x);
//    ln(x +sqrt(1+x^2));
//  end_proc):

arcsinh::hull := DOM_INTERVAL::arcsinh@hull:

arcsinh::inverse := "sinh":

arcsinh::undefined := {}:
arcsinh::realDiscont := {}:
arcsinh::complexDiscont :=
  loadproc(arcsinh::complexDiscont, pathname("STDLIB","DISCONT"), "arcsinh"):

arcsinh::diff :=
  proc(f)
    local op1;
  begin
    op1 := op(f, 1);
    diff(op1, args(2..args(0))) / sqrt(op1^2+1)
  end_proc:

arcsinh::series := loadproc(arcsinh::series, pathname("SERIES"), "arcsinh"):

arcsinh::Content := (Out, data) -> Out::Capply(Out::Carcsinh, Out(op(data))):

// end of file 
