// $Date: 2002/11/21 13:35:06 $ $Author: ccr $ $Revision: 1.29 $ 

// kg, 13/01/94 

/*++
arctanh -- the inverse hyperbolic tangens

arctanh(x)

x - expression
++*/

arctanh:= proc(x)
  option noDebug;
  local f;
  name arctanh;
begin
  if args(0) = 0 then error("no arguments given")
  elif x::dom::arctanh <> FAIL then return(x::dom::arctanh(args()))
  elif args(0) <> 1 then error("wrong no of args") 
  end_if;

  case type(x)
    of DOM_SET do
    of "_union" do
      return(map(x, arctanh))
    of DOM_INT do
    of DOM_RAT do
        if x = 1 then
           error("singularity");
	elif x < 0 then return(-arctanh(-x)) end_if;
	break;
    
    of DOM_FLOAT do
	return(arctanh::float(x));

    of DOM_COMPLEX do
	if domtype(op(x,1)) = DOM_FLOAT or
	   domtype(op(x,2)) = DOM_FLOAT then
	    return(arctanh::float(x))
	end_if;
  end_case;

  if not testtype(x,Type::Arithmetical) then
     error("argument must be of 'Type::Arithmetical'")
  end_if;

  case type(x)
    of "_mult" do
	f:= op(x, nops(x));
	if testtype(f, Type::Real) then
	    if f < 0 then return(-arctanh(-x)) end_if;
	end_if;
	break;

    of "tanh" do // arctanh(tanh(x))=x iff -PI/2 < Im(x) < PI/2 
        f:=round(Im(op(x))/PI);
        if type(f)=DOM_INT then return(op(x)-f*I*PI) end_if;
        break;
  end_case;

  procname(x)
end_proc:

arctanh(0):= 0:
arctanh:= funcenv(arctanh, op(specfunc::arctanh, 2)):
arctanh::info := "arctanh - the inverse hyperbolic tangens":
arctanh::type := "arctanh":
arctanh::print := "arctanh":
arctanh::float := specfunc::arctanh:
// potential library fix for PARI's arctanh:
// proc(x) begin
//   x:= float(x):
//   ln( (1+x)^(1/2) ) - ln( (1-x)^(1/2) ) ;
// end_proc):  

arctanh::hull := DOM_INTERVAL::arctanh@hull:

arctanh::inverse := "tanh":

arctanh::undefined := {-1, 1}:
arctanh::realDiscont := {-1, 1}:
arctanh::complexDiscont :=
    loadproc(arctanh::complexDiscont, pathname("STDLIB","DISCONT"), "arctanh"):


arctanh::diff :=
  proc(f)
    local op1;
  begin 
    op1 := op(f, 1);
    diff(op1, args(2..args(0))) / (1-op1^2)
  end_proc:

arctanh::series := loadproc(arctanh::series, pathname("SERIES"), "arctanh"):

arctanh::Content := (Out, data) -> Out::Capply(Out::Carctanh, Out(op(data))):

// end of file 
