
// ok, 21/01/94 

/*++
Bernoulli numbers and polynomials

bernoulli(n)
bernoulli(n, x)

n - a nonnegative integer
x - an expression
++*/

bernoulli :=
proc(n, x)
  local t, k, s, k0, x2;
begin
  case args(0)
    of 1 do
       if domtype(n) = DOM_INT and n >= 0 then
          return(specfunc::bernoulliNumber(n));
       end_if;
       if testtype(float(n), Type::Numeric) then
          error("first argument must be symbolic or a nonnegative integer")
       end_if;
       if not testtype(n,Type::Arithmetical) then
          error("first argument must be of type 'Type::Arithmetical'")
       end_if;
       return(procname(n));
    of 2 do
        // most probable case first: n is a nonnegative integer
        if domtype(n) = DOM_INT and n>=0
        then if n = 0 then return(1)
             elif n = 1 then return(x-1/2)
             // special cases, no need to evaluate the polynomial:
             elif x = 0  then return(specfunc::bernoulliNumber(n))
             elif x = 1 then return((-1)^n*specfunc::bernoulliNumber(n))
             elif x = 1/2 then return((2^(1-n)-1)*specfunc::bernoulliNumber(n)) 
             elif x = 1/3 and domtype(n/2)=DOM_INT then
                  return((3^(1-n)-1)/2*specfunc::bernoulliNumber(n)) 
             elif x = 2/3 and domtype(n/2)=DOM_INT then
                  return((3^(1-n)-1)/2*specfunc::bernoulliNumber(n)) 
             elif x = 1/4 and domtype(n/2)=DOM_INT then
                  return((2^(1-n)-1)/2^n*specfunc::bernoulliNumber(n)) 
             elif x = 3/4 and domtype(n/2)=DOM_INT then
                  return((2^(1-n)-1)/2^n*specfunc::bernoulliNumber(n)) 
             elif x = 1/6 and domtype(n/2)=DOM_INT then
                  return((2^(1-n)-1)*(3^(1-n)-1)/2*specfunc::bernoulliNumber(n)) 
             elif x = 5/6 and domtype(n/2)=DOM_INT then
                  return((2^(1-n)-1)*(3^(1-n)-1)/2*specfunc::bernoulliNumber(n)) 
             else // evaluate the polynomial, use
                  // B(n,x) = sum(binomial(n,k)*x^k*B(n-k), k=0..n)
                  if n mod 2 = 0
                  then  t:= 1;
                        s:= specfunc::bernoulliNumber(n);
                       k0:= 2;
                  else  t:= n*x;
                        s:= t*specfunc::bernoulliNumber(n-1):
                       k0:= 3;
                  end_if;
                  //compute  sum(binomial(n,k)*x^k*B(n-k), k=0..n-2)
                  x2:= x^2:
                  for k from k0 to n-2 step 2 do
                    t:= t*(n-k+2)*(n-k+1)/k/(k-1)*x2;
                    s:= s + t*specfunc::bernoulliNumber(n - k);
                  end_for:
                  // add the last 2 terms:
                  t:= t*x/(n-1);
                  s:= s - t;
                  t:= 2*x*t/n;
                  return(s+t);
             end_if;
        // Various simplifications, if n is not a nonnegative integer.
        // The call bernoulli(n) checks, if n is valid:
        elif x = 0 then return(bernoulli(n))
        elif x = 1 then return((-1)^n*bernoulli(n))
        elif x =-1 then return(bernoulli(n) + n*(-1)^n)
        elif x = 1/2 then return((2^(1-n)-1)*bernoulli(n)) 
        elif x = 1/3 and domtype(n/2)=DOM_INT then
             return((3^(1-n)-1)/2*bernoulli(n)) 
        elif x = 2/3 and domtype(n/2)=DOM_INT then
             return((3^(1-n)-1)/2*bernoulli(n)) 
        elif x = 1/4 and domtype(n/2)=DOM_INT then
             return((2^(1-n)-1)/2^n*bernoulli(n)) 
        elif x = 3/4 and domtype(n/2)=DOM_INT then
             return((2^(1-n)-1)/2^n*bernoulli(n)) 
        elif x = 1/6 and domtype(n/2)=DOM_INT then
             return((2^(1-n)-1)*(3^(1-n)-1)/2*bernoulli(n)) 
        elif x = 5/6 and domtype(n/2)=DOM_INT then
             return((2^(1-n)-1)*(3^(1-n)-1)/2*bernoulli(n)) 
        elif testtype(x, Type::Real) and x > 1/2 and x < 1 then
             return((-1)^n*bernoulli(n,1-x))
        elif testtype(x, Type::Real) and x < 0 then
             return((-1)^n*bernoulli(n,-x) + n*(-1)^n*(-x)^(n-1))
        // errors: 
        elif testtype(float(n), Type::Numeric) then
             error("first argument must be symbolic or a nonnegative integer")
        elif not testtype(n,Type::Arithmetical) then
             error("first argument must be of type 'Type::Arithmetical'")
        elif not testtype(x,Type::Arithmetical) then
             error("second argument must be of type 'Type::Arithmetical'")
        // symbolic return
        else return(procname(n, x))
        end_if;
        break;
    otherwise
        error("1 or 2 arguments expected.")
  end_case
end_proc:

bernoulli:= funcenv(bernoulli):
bernoulli::type:= "bernoulli":
bernoulli::print:= "bernoulli":
bernoulli::conjugate:=
  proc(n, x)
  begin
    if args(0) = 1
      then bernoulli(n)
    else bernoulli(n, conjugate(x))
    end_if;
  end_proc:

// auxiliary routine for the Bernoulli numbers
// uses ideas from
// Hans Riesel: An exact formula for the 2n-th Bernoulli number
// Acta arithmetica, volume 26 (1975), pages 273-277
specfunc::bernoulliNumber:=
proc(n: Type::PosInt)
  local Q, prefact, bound, zetaval, k,
  bernoullDenom: DOM_PROC;
  save DIGITS;
  option remember;
begin
  if n mod 2 <> 0 then
    return(0)
  else
    n:= n div 2
  end_if;
  
  // local method
  // returns the denominator of the  2n - th Bernoulli number
  // it is known that this one equals the product of all primes p
  // for which p-1 divides 2n
  bernoullDenom:=
  proc(n: DOM_INT): DOM_INT
  begin
    _mult(op(select(map(numlib::divisors(2*n), _plus, 1), isprime)))
  end_proc:

  
  DIGITS:= 8;
  Q:= bernoullDenom(n);

  // looking for the absolute value of the numerator
  // Q*bernoulli(2*n) =( Q* (2*n)! / 2^(2*n-1)/PI^(2*n) ) * zeta(2*n)
  // first, we compute the first factor with low precision
  prefact:= gamma(float(2*n+1)) * Q/2.0^(2*n-1)/float(PI)^2*n;
  // we know that our numerator must be an integer; hence it suffices
  // to sum the zeta series up to a bound such that
  // sum(k^(-2*n), k=bound..infinity) * prefact  < 1
  // the left hand side is < 2*bound^(-2*n) * prefact
  // hence we need bound^(2*n) > 2*prefact
  bound:= ceil((2*prefact)^(1/2/n));
  userinfo(20, "Bound:".expr2text(bound));
  // since zeta equals 1 (rather precisely), we need as many digits as
  // prefact has in front of the decimal point
  // add some safety digits
  DIGITS:= max(8, ceil(ln(prefact) / ln(10.0)) + 5);
  userinfo(20, "Setting DIGITS to ".expr2text(DIGITS));
  prefact:= fact(2*n) * Q/2.0^(2*n-1)/float(PI)^(2*n);
  zetaval:= _plus(float(k^(-2*n)) $k=1..bound);
  (-1)^(n-1)*ceil(prefact*zetaval) / Q
end_proc:


specfunc::bernoulliNumber(0):= 1:
specfunc::bernoulliNumber(1):= -1/2:

/* end of file */
