
// ok, 27/01/94 

/*++
coth -- the hyperbolic cotangens

coth(x)

x - expression
++*/

coth:=
proc(x)
  option noDebug;
  local f;
  name coth;
begin
  if args(0) = 0 then error("no arguments given")
  elif x::dom::coth <> FAIL then return(x::dom::coth(args()))
  elif args(0) <> 1 then error("wrong no of args") 
  end_if;

  case domtype(x)
    of DOM_SET do
     return(map(x,coth));
      
    of DOM_INT do
    of DOM_RAT do
        if x = 0 then
           error("singularity")
        elif x < 0 then
          return(-coth(-x))
        end_if;
      break;
    
    of DOM_FLOAT do
	return(coth::float(x));

    of DOM_COMPLEX do
	if domtype(op(x,1)) = DOM_FLOAT or
	   domtype(op(x,2)) = DOM_FLOAT then
	    return(coth::float(x))
//      elif op(x, 1) = 0 then
//           return(-I*cot(-I*x))
//      elif op(x, 1) < 0 and op(x, 2) < 0 then
//          return(-coth(-x))
        end_if;
	break;
  end_case;

  if not testtype(x,Type::Arithmetical) then
     error("argument must be of 'Type::Arithmetical'")
  end_if;

  case type(x)    
    of "_mult" do
	f:= op(x, nops(x));
	if testtype(f, Type::Real) then
             if f < 0 then return(-coth(-x)) end_if;
        elif type((f:=x/I/PI))=DOM_INT then
             error("singularity");
        elif type(2*f)=DOM_INT then
             return(0);
        end_if;
	break;

    of "arccoth" do return(op(x));
    of "arcsinh" do return(sqrt(op(x)^2+1)/op(x));
    of "arccsch" do return(sqrt(op(x)^2+1));
    of "arccosh" do return(op(x)/sqrt(op(x)^2-1));
    of "arcsech" do return(1/sqrt(1-op(x)^2));
    of "arctanh" do return(1/op(x));

    end_case;

    procname(x)
end_proc:

coth(  I*PI/2):= 0:
coth(  I*PI/3):= I*(3^(1/2))/3:
coth(  I*PI/4):= I:
coth(  I*PI/5):= I*1/5*5^(1/2)*(5 + 2*5^(1/2))^(1/2):
coth(2*I*PI/5):= I*1/5*5^(1/2)*(5 - 2*5^(1/2))^(1/2):
coth(  I*PI/6):= I*(3^(1/2)):
coth(  I*PI/8):= I*(2^(1/2)+1):
coth(3*I*PI/8):= I*(2^(1/2)-1):
coth(  I*PI/10):= I*(5+2*5^(1/2))^(1/2):
coth(3*I*PI/10):= I*(5-2*5^(1/2))^(1/2):
coth(  I*PI/12):= I*(2+3^(1/2)):
coth(5*I*PI/12):= I*(2-3^(1/2)):

coth( infinity):= 1:
coth(-infinity):=-1:

coth:= funcenv(coth):
coth::type := "coth":
coth::print := "coth":
coth::info := "coth -- the hyperbolic cotangent":

coth::inverse := "arccoth":

coth::realDiscont := {0}:
coth::complexDiscont :=
    loadproc(coth::complexDiscont, pathname("STDLIB","DISCONT"), "coth"):
coth::undefined := 
    loadproc(coth::undefined, pathname("STDLIB","DISCONT"), "coth"):

coth::diff :=
  proc(f)
    local op1;
  begin
    op1 := op(f, 1);
    diff(op1, args(2..args(0))) * (1-coth(op1)^2)
  end_proc:

coth::float := e ->  // coth(e) = 1/tanh(e) 
               if e = 0 or iszero(e) then
                 error("singularity")
               elif indets(float(e)) <> {} then
                 // e is symbolic
                 hold(coth)(float(e))
               else
                 1/tanh::float(e)
               end_if:

coth::hull := DOM_INTERVAL::coth@hull:

coth::expand := x -> expand(cosh(x)/sinh(x)):

coth::conjugate :=
    loadproc(coth::conjugate, pathname("STDLIB","CONJ"), "coth"):

coth::rectform :=
    loadproc(coth::rectform, pathname("STDLIB","RECTFORM"), "coth"):

coth::series :=
  proc(f,x,n,dir)
  begin
    Series::series(cosh(f),x,n,dir)/Series::series(sinh(f),x,n,dir)
  end_proc:

coth::simplify :=
    loadproc(coth::simplify, pathname("STDLIB","SIMPLIFY"), "coth"):

coth::Re := loadproc(coth::Re, pathname("STDLIB","RE"), "coth"):

coth::Im := loadproc(coth::Im, pathname("STDLIB","IM"), "coth"):

coth::Content := (Out, data) -> Out::Capply(Out::Ccoth, Out(op(data))):

// end of file 
