// ok, 25/01/94 

/*++
The complementary error function

erfc(x) = 1 - erf(x)

x - an expression
++*/

erfc :=
proc(x) 
  local f;
begin
  if args(0) = 0 then error("no arguments given")
  elif x::dom::erfc <> FAIL then return(x::dom::erfc(args()))
  elif args(0) <> 1 then error("1 argument expected");
  end_if;

  case type(x)
    of DOM_FLOAT do
        return(erfc::float(x));
    of DOM_COMPLEX do
        if type(op(x,1)) = DOM_FLOAT or type(op(x,2)) = DOM_FLOAT then
           return(erfc::float(x));
        end_if;
        break;

    of DOM_INT do
    of DOM_RAT do return((if x<0 then 2-procname(-x) else procname(x) end_if))

    of DOM_SET do
    of "_union" do
      return(map(x, erfc))
  end_case;
  
  if not testtype(x,Type::Arithmetical) then
     error("argument must be of 'Type::Arithmetical'")
  end_if;
  
  case type(x)
    of "_mult" do
        f:= op(x, nops(x));
        if testtype(f, Type::Real) then
            if f < 0 then return(2-erfc(-x)) end_if;
            if domtype(f) = DOM_FLOAT then break end_if;
        else
            break
        end_if;

  end_case;
  procname(x) 
end_proc:

erfc(0) := 1:
erfc(infinity) := 0:
erfc(-infinity) := 2:

erfc:= funcenv(erfc, op(specfunc::erfc, 2)):
erfc::print:= "erfc":
erfc::info:="erfc -- the complementary error function":
erfc::type:= "erfc":

erfc::rectform:=
proc(xx)
  local a, x;
begin
  x := rectform::new(xx);
  if extop(x, 3) <> 0 or not iszero(extop(x, 2)) then
    new(rectform, 0, 0, hold(erfc)(xx))
  else
    a := extop(x, 1);
    if domtype(a) = DOM_FLOAT then
      rectform::new(erfc(a))
    else
      new(rectform, erfc(a), 0, 0)
    end_if
  end_if
end_proc:


erfc::float:=
            proc(x)
              name erfc::float;
              local result, Rex, Imx, fx, Imx2;
              save DIGITS;
            begin
               case traperror( (result:= specfunc::erfc(x)) ) 
               of  0 do return(result)
               of 20 do // overflow/underflow
                  x:= float(x);
                  if domtype(x) = DOM_FLOAT 
                  then if x <= -1000 // PARI underflow for Re(x)<-2411.3;
                       then return(float(2))
                       end_if;
                       if x >= 1000 // PARI underflow for Re(x)>2411.3;
                       then return(float(0))
                       end_if;
                  end_if;
                  if domtype(x) = DOM_COMPLEX
                  then Rex:= op(x,1): // the real part of x^2
                       Imx:= op(x,2): // the imaginary part of x^2
                       if Rex <=-10^3 and specfunc::abs(Imx)<=-Rex/10 then
                          return(float(2))
                       end_if;
                       if Rex >= 10^3 and specfunc::abs(Imx)<=Rex/10 then
                          return(float(0))
                       end_if;
                  end_if:
                  lasterror();
               of 431 do // loss of precision
                  fx:= float(x);
                  if domtype(fx) = DOM_COMPLEX
                  then Imx2:= op(fx^2,2): // the imaginary part of x^2
                       DIGITS:= DIGITS + max(10, ceil(ln(Imx2)/2.302585093)):
                       x:= float(x);
                       return(specfunc::erfc(x));
                  end_if:
               otherwise lasterror(); // report any other errors
               end_case;
            end_proc:

erfc::conjugate:= 
    loadproc(erfc::conjugate, pathname("STDLIB", "CONJ"), "erfc"):

erfc::diff:=
  proc()
    local op1;
  begin 
    op1 := op(args(1),1);
    -2/PI^(1/2) * exp(-op1^2) * diff(op1, args(2..args(0)))
  end_proc:

erfc::series:= loadproc(erfc::series, pathname("SERIES"), "erfc"):

// end of file 
