// $Date: 2004/08/30 12:01:42 $ $Author: stefanw $ $Revision: 1.43 $ 

// kg, 13/01/94 

/*++
exp -- the exponential function

exp(x)

x - expression
++*/

exp:= proc(x)
  name exp;
  local y, lny, c;
begin
  if args(0) = 0 then error("expecting one argument")
  elif x::dom::exp <> FAIL then return(x::dom::exp(args()))
  elif args(0) <> 1 then error("expecting one argument")
  end_if;

  case type(x)
    of DOM_SET do
    of "_union" do
      return(map(x, exp))
  end_case;

  if not testtype(x,Type::Arithmetical) then
     error("argument must be of 'Type::Arithmetical'")
  end_if;

  case type(x)
    of DOM_FLOAT do
      return(exp::float(x));

    of DOM_COMPLEX do
      if domtype(op(x,1)) = DOM_FLOAT or
         domtype(op(x,2)) = DOM_FLOAT 
      then return(exp::float(x))
      end_if;
      break;

    of DOM_INTERVAL do
      return(exp::hull(x));

    of "_mult" do
      // handle case exp(y*PI*I) for rational y  
      y:=x/PI/I;
      if testtype(y, DOM_INT) then
        return((-1)^y)
      end_if;
      if testtype(y, DOM_RAT) then
        if y>=1 or y<0 then
          // reduce to case exp(y*PI*I) with 0<=y<1
          /* recursive call is necessary to use the
            predefined values below */
          y:=2*frac(y/2); // this is >=0 and < 2, and its difference to the
                         // previous y is a multiple of 2
          if %<1 then
            return(exp(PI*I*%))
          else
            // exp(y*PI*I) = exp(-PI*I)*exp((y-1)*PI*I)
            //             =   (-1)    *exp((y-1)*PI*I)
            return(-exp(PI*I*(%-1)))
          end_if;
        else
          // do not determine the function value except for the
          // predefined cases below
          break
        end_if
      end_if;
     
      // exp(ln(y)*Constant) = y^Constant 
      if has(x, hold(ln)) then
         // split x = ln(y)*Constant
         lny:= select(x, testtype, "ln");
         // select can return 1 --> test again 
         if testtype(lny, "ln") then
            // x = ln(y)* Constant ??
            c:= x/lny;
            if testtype(c, Type::Constant) then
               return(op(lny,1)^c)
            end_if;
         end_if;
      end_if:
      break;

    of "_plus" do
      // handle exp(y + integer*PI*I)
      if has(x, PI) and has(x, I) then
         for y in x do
             if testtype(y/2/PI/I, DOM_INT) then
                return(exp(x-y))
             end_if;
             if testtype(y/PI/I, DOM_INT) then
                return(-exp(x-y))
             end_if;
         end_for:
      end_if:
      break;

    of "ln" do
      return(op(x,1));
    of "lambertW" do
      if nops(x) = 1 then
        return(op(x,1)/x)
      else
        return(op(x,2)/x)
      end_if;
  end_case;

  procname(x)
end_proc:

// standard simplifications 

exp(infinity):= infinity:
exp(-infinity):= 0:
exp(I*infinity):= undefined:
exp(-I*infinity):= undefined:
exp(infinity + I*infinity):= undefined: // oscillates between -inf and inf
exp(infinity - I*infinity):= undefined:
exp(0):= 1:
exp(I*PI):= -1:

exp(I*PI  /2):=  I:

exp(I*PI  /3):= 1/2+ 1/2*I*3^(1/2):
exp(I*PI*2/3):= -1/2 + 1/2*I*3^(1/2):

exp(I*PI  /4):= (1+I)/2^(1/2):
exp(I*PI*3/4):= -(1-I)/2^(1/2):

exp(I*PI  /5):=  (5^(1/2)+1)/4 + (5-5^(1/2))^(1/2)*2^(1/2)/4*I:
exp(I*PI*2/5):=  (5^(1/2)-1)/4 + (5+5^(1/2))^(1/2)*2^(1/2)/4*I:
exp(I*PI*3/5):= (-5^(1/2)+1)/4 + (5+5^(1/2))^(1/2)*2^(1/2)/4*I:
exp(I*PI*4/5):= -(5^(1/2)+1)/4 + (5-5^(1/2))^(1/2)*2^(1/2)/4*I:

exp(I*PI   /6):=  3^(1/2)/2 + I/2:
exp(I*PI* 5/6):= -3^(1/2)/2 + I/2:

exp(I*PI   /8):= (2+2^(1/2))^(1/2)/2 + (2-2^(1/2))^(1/2)/2*I:
exp(I*PI* 3/8):= (2-2^(1/2))^(1/2)/2 + (2+2^(1/2))^(1/2)/2*I:
exp(I*PI* 5/8):=-(2-2^(1/2))^(1/2)/2 + (2+2^(1/2))^(1/2)/2*I:
exp(I*PI* 7/8):=-(2+2^(1/2))^(1/2)/2 + (2-2^(1/2))^(1/2)/2*I:

exp(I*PI   /10):=  (5+5^(1/2))^(1/2)*2^(1/2)/4 + (5^(1/2)-1)/4*I:
exp(I*PI* 3/10):=  (5-5^(1/2))^(1/2)*2^(1/2)/4 + (5^(1/2)+1)/4*I:
exp(I*PI* 7/10):= -(5-5^(1/2))^(1/2)*2^(1/2)/4 + (5^(1/2)+1)/4*I:
exp(I*PI* 9/10):= -(5+5^(1/2))^(1/2)*2^(1/2)/4 + (5^(1/2)-1)/4*I:

exp(I*PI   /12):=  (1-I)/4*2^(1/2) + (1+I)/4*6^(1/2):
exp(I*PI* 5/12):= -(1-I)/4*2^(1/2) + (1+I)/4*6^(1/2):
exp(I*PI* 7/12):=  (1+I)/4*2^(1/2) - (1-I)/4*6^(1/2):
exp(I*PI*11/12):= -(1+I)/4*2^(1/2) - (1-I)/4*6^(1/2):

// function attributes 

exp:= funcenv(exp):
exp:= subsop(exp, 2=op(specfunc::exp, 2)):
exp::print:= "exp":
exp::info:= "exp -- the exponential function":
exp::type:= "exp":
exp::float:=
           proc(x) local err, tmp;
           begin
            err:= traperror( (tmp:= specfunc::exp(x)) );
            if err = 0
            then return(tmp)
            elif err = 20 // Error: Overflow/underflow
            then x:= float(x);
                 if domtype(op(x,1)) = DOM_FLOAT and
                    op(x,1) < -5.81*10^(6) // PARI underflow for Re(x)<-5.81*10^6;
                 then return(float(0))
                 end_if
            end_if;
            // error erneut ausloesen
            lasterror();
          end_proc:

exp::hull:= DOM_INTERVAL::exp@hull:
exp::inverse:= "ln":
exp::abs:= x -> exp(Re(x)):

exp::sign:=     loadproc(exp::sign,      pathname("SPECFUNC","SIGN"),   "exp"):
exp::expand:=   loadproc(exp::expand,    pathname("STDLIB","EXPAND"),   "exp"):
exp::conjugate:=
                loadproc(exp::conjugate, pathname("STDLIB","CONJ"),     "exp"):
exp::rectform:= loadproc(exp::rectform,  pathname("STDLIB","RECTFORM"), "exp"):
exp::simplify:= loadproc(exp::simplify,  pathname("STDLIB","SIMPLIFY"), "exp"):
exp::series:=   loadproc(exp::series,    pathname("SERIES"),            "exp"):
exp::Re:=       loadproc(exp::Re,        pathname("STDLIB","RE"),       "exp"):
exp::Im:=       loadproc(exp::Im,        pathname("STDLIB","IM"),       "exp"):
exp::diff:=     loadproc(exp::diff,      pathname("STDLIB","DIFF"),     "exp"):

exp::laplace:=
    loadproc( exp::laplace,
        pathname("TRANS","LAPLACE"), "L_exp"):

// typesetting and MathML generation
exp::Content:= (Out, arg1) ->
               if op(arg1) = 1/2 then
                 Out::Capply(Out::Croot, Out(exp(1)))
               elif op(arg1) = 1 then
                 Out::Cexponentiale()
               else
                 Out::Capply(Out::Cexp, Out(op(arg1)))
               end_if:

exp::TeX :=
  proc(e, data, priority)
    local expo;
  begin
    expo := op(data, 1);
    if expo = 1/2 then
      "\\sqrt{\\mathrm{e}}"
    elif expo = 1 then
      "\\mathrm{e}"
    else
      generate::TeXquote("\\mathrm{e}^{".
			 generate::tex(expo, output::Priority::Noop)."}",
			 priority, output::Priority::Power);
    end_if;
  end_proc:
// end of file 
