// $Date: 2004/06/23 11:59:08 $ $Author: stefanw $ $Revision: 1.19 $ 

// zimmerma, 01/08/95 

/*++
	heaviside -- the Heaviside function

	heaviside(x)

	x - expression

	heaviside(x) :=      1            if x>0
                        0            if x<0
                        1/2          if x=0
                        undefined    if x is a non-real complex number
                        heaviside(x) otherwise.
++*/

heaviside :=
proc(x)
  option noDebug;
begin
  if args(0) = 0 then
    error("no arguments given")
  elif x::dom::heaviside <> FAIL then
    return(x::dom::heaviside(args()))
  elif args(0) <> 1 then
    error("expecting one argument")
  end_if;

  case type(x)
    of DOM_SET do // fall through
    of "_union" do
      return(map(x, heaviside))
    of "_intersect" do
    of "_minus" do
      return(procname(x))
  end_case;

  if domtype(x) = DOM_FLOAT then
    if x > 0 then
      return(float(1));
    end_if;
    if x < 0 then
      return(float(0));
    end_if;
    // this only leaves x = 0.0
    // return(procname(x));
    return(0.5)
  end_if;

  if not testtype(x,Type::Arithmetical) then
    error("argument must be of 'Type::Arithmetical'")
  elif is(x>0)=TRUE then
    1
  elif is(x<0)=TRUE then
    0
  elif domtype(x)=DOM_COMPLEX then
    undefined
  else
    procname(x)
  end_if
end_proc:

// the best choice for the sake of integration and Fourier analysis 
heaviside(0):= 1/2:

heaviside:= funcenv(heaviside):
heaviside::type:= "heaviside":
heaviside::info:= "heaviside -- the Heaviside step function":
heaviside::print:= "heaviside":
heaviside::float:= x -> heaviside(float(x)):

heaviside::series:=
	loadproc(heaviside::series, pathname("SERIES"), "heaviside"):

heaviside::diff:=
  proc(f,x)
  begin
    eval(diff(subsop(f,0=dirac)*diff(op(f),x),args(3..args(0))))
  end_proc:

heaviside::laplace:=
    loadproc(heaviside::laplace, pathname("TRANS","LAPLACE"), "L_heavis"):


// end of file 
