// $Date: 2002/11/20 09:20:23 $ $Author: tonner $ $Revision: 1.15 $ 

// stefanw, 14/08/98 

/*++
lambertV - the lower branch of the functional inverse of
           x -> x*exp(x)

lambertV(x)

x - expression
++*/

lambertV:=
proc(x)
  option noDebug;
  local fx,i,f;
begin
  if args(0) <> 1 then
    error("wrong no of args")
  elif x::dom::lambertV <> FAIL then
    return(x::dom::lambertV(args())) 
  end_if;

case type(x)
  of DOM_SET do
  of "_union" do
    return(map(x, lambertV))
end_case;
  
  if not testtype(x,Type::Arithmetical) then
    error("argument must be of 'Type::Arithmetical'")
  end_if;

  fx:= float(x):
  if domtype(fx) <> DOM_FLOAT or
     fx < float(-exp(-1)) or
     fx > 0
  then return(procname(args()))
  end_if:
  
  case type(x)
    of DOM_FLOAT do
       if iszero(x) then 
          error("singularity")
       end_if:
       return(lambertV::float(x));
    of "_mult" do
      for i from 1 to nops(x) do
        if op(x,i)=exp((f:=_mult(op(x,1..i-1),op(x,i+1..nops(x))))) then
          return(f)
        end_if;
      end_for;
      break
  end_case;
	
  procname(x)
end_proc:

lambertV(0):= -infinity:
lambertV(-exp(-1)):=-1:

lambertV:= funcenv(lambertV):
lambertV:= subsop(lambertV, 2=op(specfunc::lambertV, 2)):
lambertV::print:= "lambertV":
lambertV::info:= "lambertV -- lower branch of the inverse of x*exp(x)":
lambertV::type:= "lambertV":
lambertV::float:= specfunc::lambertV:

lambertV::inverse:= "id*exp":


lambertV::diff:= (f,x) -> f*diff(op(f,1),x)/(op(f,1)*(1+f)):

lambertV::series:=
    loadproc(lambertV::series, pathname("SERIES"), "lambertV"):

// end of file 
