// $Date: 2004/02/19 13:51:47 $ $Author: ccr $ $Revision: 1.23 $ 

/*++
lambertW - the main branch of the functional inverse of
           x -> x*exp(x)

lambertW(x)

x - expression
++*/

lambertW:=
proc(k, x)
  option noDebug;
  local i,f;
begin
  if args(0) = 1 then
    x := k;
    k := 0;
  end_if;
  
  if x::dom::lambertW <> FAIL then
     return(x::dom::lambertW(args()))
  elif args(0) <> 1 and args(0) <> 2 then 
     error("wrong no of args")
  end_if;

  case type(x)
    of DOM_SET do
    of "_union" do
      return(map(x, a -> lambertW(k, a)))
  end_case;
  
  if not (testtype(x, Type::Arithmetical) and
          testtype(k, Type::Arithmetical)) then
    error("arguments must be of 'Type::Arithmetical'")
  end_if;

  case type(x)
    of DOM_FLOAT do
      return(lambertW::float(k, x));
    of DOM_COMPLEX do
      if domtype(Re(x)) = DOM_FLOAT or domtype(Im(x)) = DOM_FLOAT then
        return(lambertW::float(k, x));
      end_if;
      break;
    of "_mult" do
      if type(k) = DOM_INT then
        // if k is symbolic, no floating point evaluation is possible!
        for i from 1 to nops(x) do
          if op(x,i)=exp((f:=_mult(op(x,1..i-1),op(x,i+1..nops(x))))) then
            if domtype(float(f)) in {DOM_FLOAT, DOM_COMPLEX} and
              abs(float(f)-lambertW::float(k, x))
              < abs(float(f)*10.0^(-DIGITS)) + 10.0^(-DIGITS)
              then
              return(f);
            end_if;
          end_if;
        end_for;
      end_if;
      break
  end_case;
	
  procname(k, x)
end_proc:

lambertW(0):=0:
lambertW(exp(1)):=1:
lambertW(-exp(-1)):=-1:
lambertW(infinity):= infinity:

lambertW(0, 0):=0:
lambertW(0, exp(1)):=1:
lambertW(0, -exp(-1)):=-1:
lambertW(0, infinity):= infinity:

lambertW(-1, 0):= -infinity:
lambertW(-1, -exp(-1)) := -1:

lambertW:= funcenv(lambertW, op(specfunc::lambertW, 2)):
lambertW::print := "lambertW":
lambertW::info  := "lambertW -- the inverse of x*exp(x)":
lambertW::type  := "lambertW":
lambertW::float := specfunc::lambertW:

lambertW::inverse := id*exp:

lambertW::diff :=
proc(f,x)
begin
  if nops(f) = 1 then
    f*diff(op(f,1),x)/(op(f,1)*(1+f))
  else
    f*diff(op(f,2),x)/(op(f,2)*(1+f))
  end_if
end_proc:

lambertW::series :=
loadproc(lambertW::series, pathname("SERIES"), "lambertW"):

lambertW::TeX :=
proc(fn, ex, prio)
  local s;
begin
  if nops(ex) = 1 then
    s := generate::tex(op(ex,1), output::Priority::Fconcat);
    if length(s) < 6 or s[1..6] <> "\\left(" then
      s := "\\left(".s."\\right)";
    end_if;
    "W".s;
  else
    s := generate::tex(op(ex,2), output::Priority::Fconcat);
    if length(s) < 6 or s[1..6] <> "\\left(" then
      s := "\\left(".s."\\right)";
    end_if;
    _concat("W_{",
            generate::TeX(op(ex,1)),
            "}",
            s);

  end_if;
end_proc:

lambertW::Content := (Out, data) -> Out::Capply(Out::ClambertW,
                                               map(op(data), Out)):
// end of file 
