// $Date: 2004/03/23 15:32:50 $ $Author: stefanw $ $Revision: 1.36 $ 

// kg, 13/01/94 

/*++
ln -- the natural logarithm

ln(x)

x - expression
++*/

ln:= proc(x)
  option noDebug;
  name ln;
  local a, k, res;
begin
  if args(0) = 0 then error("no arguments given")
  elif x::dom::ln <> FAIL then return(x::dom::ln(args()))
  elif args(0) <> 1 then error("wrong no of args") 
  end_if;

  if iszero(x) then error("singularity") end_if;

  case type(x)

    of DOM_FLOAT do
	return(ln::float(x))

    of DOM_COMPLEX do
	if domtype(op(x,1)) = DOM_FLOAT or
	   domtype(op(x,2)) = DOM_FLOAT 
	then
	    return(ln::float(x))
	end_if;
	break

    of DOM_INTERVAL do
	return(ln::hull(x))

    of DOM_INT do
        if x < 0 then return(ln(-x) + I*PI) end_if;
	break

    of DOM_RAT do
        if x < 0 then return(ln(-x) + I*PI) end_if;
        if op(x,1)=1 then return(-ln(op(x,2))) end_if;
	break

    of DOM_SET do
    of "_union" do
      return(map(x, ln))
  end_case;

  if not testtype(x,Type::Arithmetical) then
    error("argument must be of 'Type::Arithmetical'")
  end_if;

  case type(x)
    of "exp" do
	if testtype(op(x), Type::Numeric) then
          k:=round(-Im(op(x))/2/PI);
          return(op(x)+2*k*PI*I)
        end_if;
	break

    of "_power" do
        if type(op(x, 1)) = "exp" then
          // ln(exp(a)^b) = ln(exp(b*ln(exp(a))))
          //              = ln(exp(b*(a + 2*k*PI*I)))
          // with k = round(-Im(a)/2/PI);
          // If a is real then
          //    ln(exp(a)^b) = ln(exp(a*b));
          // a = op(op(x, 1)), b = op(x, 2)
          a:= op(op(x, 1));
          if testtype(a, Type::Numeric) then
            // recursive call to ln that will
            // end up in the branch above
            return(ln(exp(a * op(x,2))));
          end_if;
        end_if;
        break

    of "lambertW" do
      if nops(x) = 1 then
	res := ln(op(x,1))-x
      else
        res := ln(op(x,2))-x+2*PI*I*op(x,1)
      end_if;
      // To avoid problems with branch cuts, check the result numerically
      // example for problems: ln(lambertW(0, I*3^(1/2) + 1))
      if domtype(float(res)) in {DOM_COMPLEX, DOM_FLOAT}
        and abs(float(res) - ln(float(x))) < 10.0^(2-DIGITS)
        then return(res);
      end_if;
      break;

  end_case;

  procname(x)
end_proc:

ln(1):= 0:
ln(-1):= I*PI:
ln(I):= I*PI/2:
ln(-I):=-I*PI/2:
ln(infinity):= infinity:
ln(-infinity):= I*PI + infinity:

ln:= funcenv(ln, op(specfunc::ln, 2)):
ln::print := "ln":
ln::type := "ln":
ln::info := "ln -- the natural logarithm":
ln::float := specfunc::ln:
ln::hull := DOM_INTERVAL::ln@hull:

ln::inverse := "exp":

ln::undefined := {0}:
ln::realDiscont := {0}:
ln::complexDiscont :=
    loadproc(ln::complexDiscont, pathname("STDLIB","DISCONT"), "ln"):

ln::expand :=
    loadproc(ln::expand, pathname("STDLIB","EXPAND"), "ln"):

ln::simplify :=
    loadproc(ln::simplify, pathname("STDLIB","SIMPLIFY"), "ln"):
ln::Simplify:= [Rule(ln::simplify, {}, table("Default" = 0.5))]:

ln::sign :=
    loadproc(ln::sign, pathname("SPECFUNC","SIGN"), "ln"):

ln::rectform :=
    loadproc(ln::rectform, pathname("STDLIB","RECTFORM"), "ln"):

ln::series := loadproc(ln::series, pathname("SERIES"), "ln"):

ln::Re := loadproc(ln::Re, pathname("STDLIB","RE"), "ln"):
ln::Im := loadproc(ln::Im, pathname("STDLIB","IM"), "ln"):

ln::diff := loadproc(ln::diff, pathname("STDLIB","DIFF"), "ln"):

ln::laplace:=
    loadproc( ln::laplace,
        pathname("TRANS","LAPLACE"), "L_ln"):

ln::invlaplace:=
    loadproc( ln::invlaplace,
        pathname("TRANS","LAPLACE"), "IL_ln"):

ln::Content := (Out, data) -> Out::Capply(Out::Cln, Out(op(data))):

// end of file 
