// $Date: 2002/11/21 13:35:06 $ $Author: ccr $ $Revision: 1.23 $ 

// kg, 13/01/94 

/*++
sinh -- the hyperbolic sine

sinh(x)

x - expression
++*/

sinh:= proc(x)
  option noDebug;
  local f;
  name sinh;
begin
  if args(0) = 0 then error("no arguments given")
  elif x::dom::sinh <> FAIL then return(x::dom::sinh(args()))
  elif args(0) <> 1 then error("wrong no of args") 
  end_if;
  
  case type(x)
    of DOM_INT do
    of DOM_RAT do
	if x < 0 then return(-sinh(-x)) end_if;
	break;
    
    of DOM_FLOAT do
	return(sinh::float(x));

    of DOM_COMPLEX do
	if domtype(op(x,1)) = DOM_FLOAT or
	   domtype(op(x,2)) = DOM_FLOAT then
	    return(sinh::float(x))
	end_if;
	break;

    of DOM_SET do
    of "_union" do
      return(map(x, sinh))
  end_case;

  if not testtype(x,Type::Arithmetical) then
    error("argument must be of 'Type::Arithmetical'")
  end_if;

  case type(x)
    of "_mult" do
	f:= op(x, nops(x));
	if testtype(f, Type::Real) then
	    if f < 0 then return(-sinh(-x)) end_if;
        elif type((f:=x/I/PI))=DOM_INT then return(0) // sinh(n*I*PI)=0 
        elif type(2*f)=DOM_INT then return((-1)^(f-1/2)*I)
	end_if;
	break;
    
    // cf Table 4.5.60 p. 85 of Abramowitz and Stegun 
    of "arcsinh" do return(op(x))
    of "arccosh" do return(sqrt(op(x)^2-1))
    of "arctanh" do return(op(x)/sqrt(1-op(x))/sqrt(1+op(x)))
  //of "arccoth" do return(sqrt(1/(op(x)^2-1)))
    of "arccoth" do 
           x:= 1/op(x): // note that op(x) <> 0, because arccoth(0) is
                        // simplified to I*PI/2
           return(x/sqrt(1-x)/sqrt(1+x))

    end_case;

    procname(x)
end_proc:

sinh(0):= 0:
sinh(I*PI):= 0:
sinh(I*PI/2):= I:
sinh(I*PI/3):= I*(3^(1/2))/2:
sinh(I*PI/4):= I*(2^(1/2))/2:
sinh(I*PI/  5):= I*1/4*2^(1/2)*(5-5^(1/2))^(1/2):
sinh(I*PI*2/5):= I*1/4*2^(1/2)*(5+5^(1/2))^(1/2):
sinh(I*PI/6):= I*1/2:
sinh(I*PI/  8):= I*((2-2^(1/2))^(1/2))/2:
sinh(I*PI*3/8):= I*((2+2^(1/2))^(1/2))/2:
sinh(I*PI/  10):= I*1/4*(5^(1/2)-1):
sinh(I*PI*3/10):= I*1/4*(5^(1/2)+1):
sinh(I*PI/  12):= I*(6^(1/2)-2^(1/2))/4: // simpler th. ((2-3^(1/2))^(1/2))/2: 
sinh(I*PI*5/12):= I*(2^(1/2)+6^(1/2))/4: // = ((2+3^(1/2))^(1/2))/2:

sinh( infinity):= infinity:
sinh(-infinity):=-infinity:

sinh:= funcenv(sinh, op(specfunc::sinh, 2)):
sinh::info := "sinh -- the hyperbolic sine":
sinh::print := "sinh":
sinh::type := "sinh":
sinh::float := specfunc::sinh:

sinh::hull := DOM_INTERVAL::sinh@hull:

sinh::inverse := "arcsinh":

sinh::diff :=
  proc(f)
    local op1;
  begin
    op1 := op(f, 1);
    diff(op1, args(2..args(0))) * cosh(op1)
  end_proc:

sinh::expand :=
    loadproc(sinh::expand, pathname("STDLIB","EXPAND"),"sinh"):

sinh::conjugate := 
    loadproc(sinh::conjugate, pathname("STDLIB","CONJ"),"sinh"):

sinh::rectform :=
    loadproc(sinh::rectform, pathname("STDLIB","RECTFORM"),"sinh"):

sinh::series :=
    loadproc(sinh::series, pathname("SERIES"),"sinh"):

sinh::simplify :=
    loadproc(sinh::simplify, pathname("STDLIB","SIMPLIFY"),"sinh"):

sinh::Re := loadproc(sinh::Re, pathname("STDLIB","RE"),"sinh"):

sinh::Im := loadproc(sinh::Im, pathname("STDLIB","IM"),"sinh"):

sinh::laplace:=
    loadproc( sinh::laplace,
        pathname("TRANS","LAPLACE"), "L_sinh"):

sinh::Content := (Out, data) -> Out::Capply(Out::Csinh, Out(op(data))):

// end of file 
