// $Date: 2004/06/21 12:22:25 $ $Author: ccr $ $Revision: 1.6 $

/*

surd -  n-th root

surd(x, n)

returns that solution y of the equation y^n = x the argument of which
is closest to x

n is understood to be an integer


does not map to sets yet

is not overloadable yet

*/


surd:=
proc(x: Type::Arithmetical, n: Type::Arithmetical)
  local k;
begin
  if testargs() then
    if args(0) > 2 then
      error("Too many arguments")
    end_if;

    if testtype(n, Type::Numeric) and type(n) <> DOM_INT then
      error("Second argument must be integer")
    end_if;

    if n=0 then
      error("Second argument must be non-zero")
    end_if;
  end_if;


  // special case
  if n=2 then
    return(sqrt(x))
  end_if;
  
  if contains({DOM_INT, DOM_RAT, DOM_FLOAT}, domtype(x)) and x >= 0 then
    return(x^(1/n))
  end_if;
    
  if domtype(n) = DOM_INT then
    case type(x)
      of DOM_INT do
      of DOM_RAT do
        // we know that x < 0
        return((-1)^(1 - modp(n + 1, 2)/n)*(-x)^(1/n))
      of DOM_FLOAT do  
        // we know that x < 0
        return(float((-1)^(1 - modp(n + 1, 2)/n)*(-x)^(1/n)))
      of DOM_COMPLEX do
        if domtype(op(x, 2)) = DOM_FLOAT then
           return(surd::float(x, n));
        end_if;
        if iszero(op(x,1)) and n mod 4 = 1 then
          return(I*sign(op(x, 2))*specfunc::abs(x)^(1/n))
        end_if;
        k := round(1/2*arg(x)*(n - 1)/PI);
        if type(k) = DOM_INT then
          return(exp(2*I*k*PI/n)*x^(1/n))
        end_if;
    end_case
  end_if;

  // symbolic x or symbolic n

  procname(args())
    
end_proc:


surd:= funcenv(surd):

surd::print:= "surd":
surd::type:= "surd":
surd::info:= "surd -- nth root":

//-----------------------------------------------
surd::float:= proc(x, n)
local k, fx;
begin
  fx:= float(x):
  if domtype(fx) = DOM_FLOAT then
     if fx >= 0 then
        return(fx^(1/n));
     else // fx < 0 
        if n mod 2 = 0 then
             return(exp(I*(1 - 1/n)*float(PI))*(-fx)^(1/n));
        else return(-(-fx)^(1/n));
        end_if;
     end_if;
  end_if;
  if domtype(fx) = DOM_COMPLEX then
     // Copy the symbolic code above
     // special case: Re(x) = 0
     if iszero(op(fx,1)) and n mod 4 = 1 then
        return(I*sign(op(fx, 2))*specfunc::abs(fx)^(1/n))
     end_if;
     k := round(1/2*arg(fx)*(n - 1)/PI);
     if type(k) = DOM_INT then
        return(exp(float(2*I*k*PI/n))*fx^(1/n))
     end_if;
  end_if;
  hold(surd)(fx, n);
end_proc:
//-----------------------------------------------


surd::diff:= (f, x) -> f/op(f,1)/op(f,2) * diff(op(f,1), x):

surd::TeX := (f, arg, prio) -> "\\sqrt[".generate::TeX(op(arg,2)).
			      "]{".generate::TeX(op(arg,1))."}":

/* some attributes still missing :

hull
expand
conjugate
rectform
simplify
series
Re
Im
laplace
Content


*/



// end of file
