// $Date: 2002/11/21 13:35:06 $ $Author: ccr $ $Revision: 1.29 $ 

// kg, 13/01/94 

/*++
tanh -- the hyperbolic tangens

tanh(x)

x - expression
++*/

tanh:= proc(x)
  option noDebug;
  local f;
  name tanh;
begin
  if args(0) = 0 then error("no arguments given")
  elif x::dom::tanh <> FAIL then return(x::dom::tanh(args()))
  elif args(0) <> 1 then error("wrong no of args") 
  end_if;
  
  case type(x)
    of DOM_INT do
    of DOM_RAT do
	if x < 0 then return(-tanh(-x)) end_if;
	break;
    
    of DOM_FLOAT do
	return(tanh::float(x));

    of DOM_COMPLEX do
	if domtype(op(x,1)) = DOM_FLOAT or
	   domtype(op(x,2)) = DOM_FLOAT then
	    return(tanh::float(x))
	end_if;
	break;

    of DOM_SET do
    of "_union" do
      return(map(x, tanh))
  end_case;

  if not testtype(x,Type::Arithmetical) then
    error("argument must be of 'Type::Arithmetical'")
  end_if;

  case type(x)
    of "_mult" do
	f:= op(x, nops(x));
	if testtype(f, Type::Real) then
	    if f < 0 then return(-tanh(-x)) end_if;
        elif type((f:=x/I/PI))=DOM_INT then return(0) // tanh(k*I*PI)=0 
        elif type(2*f)=DOM_INT then
           //return((-1)^(f-1/2)*I*infinity)
             error("singularity");
	end_if;
	break;

    // Table 4.5.60 p. 85 of Abramowitz and Stegun 
    of "arctanh" do return(op(x))
    of "arccoth" do return(1/op(x))
    of "arcsinh" do return(op(x)/sqrt(1+op(x)^2))
    of "arccosh" do return(sqrt(op(x)^2-1)/op(x))

    end_case;

    procname(x)
end_proc:

tanh(0):= 0:
tanh(I*PI):= 0:
tanh(  I*PI/3 ):= I*(3^(1/2)):
tanh(  I*PI/4 ):= I:
tanh(  I*PI/5 ):= I*(5 - 2*5^(1/2))^(1/2):
tanh(2*I*PI/5 ):= I*(1 + 5^(1/2)) * (2*5^(1/2) + 10)^(1/2)/4:
tanh(  I*PI/6 ):= I*(3^(1/2))/3:
tanh(  I*PI/8 ):= I*(2^(1/2)-1):
tanh(3*I*PI/8 ):= I*(2^(1/2)+1):
tanh(  I*PI/10):= I*1/5*5^(1/2)*(5 - 2*5^(1/2))^(1/2):
tanh(3*I*PI/10):= I*1/5*5^(1/2)*(5 + 2*5^(1/2))^(1/2):
tanh(  I*PI/12):= I*(2-3^(1/2)):
tanh(5*I*PI/12):= I*(2+3^(1/2)):

tanh(infinity):=1:
tanh(-infinity):=-1:

tanh:= funcenv(tanh, op(specfunc::tanh, 2)):
tanh::info := "tanh -- the hyperbolic tangent":
tanh::print := "tanh":
tanh::type := "tanh":
tanh::float :=
           proc(x) local err, tmp;
           begin
            err:= traperror( (tmp:= specfunc::tanh(x)) );
            if err = 0 then
                 return(tmp)
            elif err = 20 then // Error: Overflow/underflow
                 x:= float(x);
                 if domtype(op(x,1)) = DOM_FLOAT then
                    // PARI underflow for |Re(x)|>-5.81*10^6;
                    if op(x,1)<-10^(6) then return(float(-1)) end_if:
                    if op(x,1)> 10^(6) then return(float( 1)) end_if:
                 end_if
            end_if;
            // error erneut ausloesen
            lasterror();
          end_proc:

tanh::hull := DOM_INTERVAL::tanh@hull:

tanh::inverse := "arctanh":

tanh::undefined :=
    loadproc(tanh::undefined, pathname("STDLIB","DISCONT"), "tanh"):
tanh::realDiscont := {}:
tanh::complexDiscont :=
    loadproc(tanh::complexDiscont, pathname("STDLIB","DISCONT"), "tanh"):

tanh::diff :=
  proc(f)
    local op1;
  begin
    op1 := op(f, 1);
    diff(op1, args(2..args(0))) * (1 - tanh(op1)^2)
  end_proc:

tanh::expand := f -> expand(sinh(f)/cosh(f)):

tanh::conjugate :=
    loadproc(tanh::conjugate, pathname("STDLIB","CONJ"), "tanh"):

tanh::rectform :=
    loadproc(tanh::rectform, pathname("STDLIB","RECTFORM"), "tanh"):

tanh::series :=
  proc(f,x,n,dir)
  begin
    Series::series(sinh(f),x,n,dir)/Series::series(cosh(f),x,n,dir)
  end_proc:

tanh::simplify :=
    loadproc(tanh::simplify, pathname("STDLIB","SIMPLIFY"), "tanh"):

tanh::Re := loadproc(tanh::Re, pathname("STDLIB","RE"), "tanh"):

tanh::Im := loadproc(tanh::Im, pathname("STDLIB","IM"), "tanh"):

tanh::Content := (Out, data) -> Out::Capply(Out::Ctanh, Out(op(data))):

// end of file 
