

// block - Domain of objects protected against evaluation



// basic shape
/*
Blocks are atomic: each one is the only operand of itself, even
if the block contains a sequence
subs, eval, float do nothing
expr does nothing, too
arithmetical operations return unevaluated

*/


domain block
  inherits Dom::BaseDomain;
  category  Cat::BaseCategory;

    new:=
    proc()
      option hold;
    begin
      new(dom,args())
    end;

    convert:= proc(a) begin if a::dom = dom then a else FAIL end_if end;

    nops:= 1;
    
    op:=
    proc(x : dom, n)
    begin
      case args(0)
        of 1 do
          return(x)
        of 2 do
          if domtype(n) <> DOM_INT then
            error("Illegal argument")
          end_if;
          if n<>1 then
            FAIL
          else
            x
          end_if;
          break
      otherwise
          error("Wrong number of arguments")
      end_case
    end_proc;

      
    expr:= id;
    
    print := x-> extop(x);
    
    expr2text := x -> expr2text(dom)."(".expr2text(extop(x)).")";

    func_call:= x -> subs(hold(a)(hold(b)), hold(a) = x, hold(b) = args(2..args(0)));

    flatten:= bl -> eval(hold(block)(unblock(bl)));
    
  end_domain:



// blockTransparent
/*
A transparent block is similar to an object enclosed in hold(...)
It is not evaluated (even not using eval)
but it has operands, and op, subs work as usual
It can be converted into an expression using expr
Arithmetical operations return unevaluated
*/


  
domain blockTransparent

  inherits block;
  category  Cat::BaseCategory; 


    nops:= bl -> nops(extop(bl));

    op:= bl -> op(extop(bl), args(2..args(0)));
    
    subs:= bl -> if args(args(0)) = Unsimplified then
		   new(dom, subs(extop(bl), args(2..args(0))));
		 else
		   new(dom, subs(extop(bl), args(2..args(0)), Unsimplified));
		 end_if;
    
    expr:= bl -> extop(bl);
    
    has := bl -> has([extop(bl)], args(2..args(0)));
    
    // testtype ?! subsop ?! has, hastype, indets, ...
  end_domain:
  

  

  // blockIdents
  // blocks that evaluate and simplify their input, preventing
  // the substitution of certain identifiers

domain blockIdents(idents: DOM_SET)
  local internalSubstitutions, ind, equ;
  inherits blockTransparent;
  category  Cat::BaseCategory;

    new:=
    proc()
      option hold;
    begin
      new(dom, args());
      eval(%)
    end_proc;
      
    subs:=
    proc(bl)
    begin
      bl:= subs(extop(bl), internalSubstitutions);
      bl:= subs(bl, args(2..args(0)));
      new(dom, subs(bl, internalBackSubstitutions))
    end_proc;

    evaluate:=
    proc(bl)
    begin
      bl:= subs(extop(bl), internalSubstitutions);
      bl:= eval(bl);
      new(dom, subs(bl, internalBackSubstitutions))
    end_proc;


    
  begin
    internalSubstitutions:= [ind = genident("blockInternal")
                             $ind in idents];
    internalBackSubstitutions:= [op(equ, 2) = op(equ, 1)
                                 $equ in internalSubstitutions];
      
  end_domain:







  
  
unblock:=
proc(x, blockdomain = block: DOM_DOMAIN, Recurse = TRUE: DOM_BOOL)
begin
  if x::dom::hasProp(blockdomain) = TRUE then
    eval(extop(x))
  elif Recurse and
    contains({DOM_EXPR, DOM_LIST, DOM_TABLE, DOM_ARRAY, DOM_SET},
             domtype(x)) then
    map(x, unblock, blockdomain, Recurse)
  else
    x
  end_if
end_proc:
    
