// $Date: 2004/09/14 14:45:43 $ $Author: ccr $ $Revision: 1.44 $ 

/*++
	int -- integration

	int(f,x)
	int(p <, ... >)
	int(f,x=a..b)
	int(f,x=a..b, options)

	f - expression or domain
	x - identifier
	p - piecewise defined function or domain element

        currently as options are allowed:
           - PrincipalValue
           - Continuous

	This is merely a front end for indefinite integration via intlib::int,
	and definite via intlib::defInt.
++*/

int:= funcenv(
prog::remember(
proc(f, x=null())
  name int;
begin
  if args(0)=0 then error("No argument given") end_if;

  if testargs() and has([args()],FAIL) then
    error("illegal argument(s)")
  end_if;

  if f::dom::int <> FAIL then 
    return( f::dom::int(args()) )
  elif testtype(f,Type::Arithmetical)=FALSE then
    error("Illegal integrand")
  end_if;

  // now f is of type DOM_EXPR
  if args(0)=1 then 
      error("Second argument is missing")
  end_if;

  if domtype(x)=DOM_IDENT then 
     // indefinite integration 
     if args(0)<>2 then 
         error("Expecting two arguments")
     else
         return(intlib::int(f,x))
     end_if
  else
     if type(x)= "_equal" then        
       // definite integration 
       if testargs() then intlib::xrngTest(x) end_if;
       intlib::defInt(f,x,intlib::getOptions(args(3..args(0))))
     else
       error("Second argument must be either of form x or x=a..b")
     end_if;   
  end_if
end_proc, () -> [Pref::experimentalInt(),
		 property::depends([args()])]),
proc(a)
  local eq, integralSign, PVintegralSign;
begin
  if PRETTYPRINT = TRUE then
    integralSign := "  /",
                    " | ",
                    " | ",
                    "/  ":
    PVintegralSign := "  /",
                      "_|_",
                      " | ",
                      "/  ":
    if contains({op(a, 3..nops(a))}, PrincipalValue) then
      integralSign := PVintegralSign;
    end_if;
    if type(op(a, 2)) = "_equal" then
      // definite integral
      eq := op(a, [2,2]);
      _outputSequencePower(stdlib::Exposed(stdlib::align(expr2text(op(eq, 2)),
                                                  integralSign,
                                                  expr2text(op(eq, 1)))),
                    _outputSequenceINFINITE(stdlib::Exposed(" "), op(a, 1),
                    stdlib::Exposed(" d"), op(a, [2,1])));
     else
      // indefinite integral
      _outputSequencePower(stdlib::Exposed(stdlib::align(integralSign)),
                      _outputSequenceINFINITE(stdlib::Exposed(" "), op(a, 1),
                      stdlib::Exposed(" d"), op(a, 2)));
     end_if;
  else
    FAIL
  end:
end_proc):
int:= slot(int, "type", "int" ):

int:= slot( int,"info",
           "definite and indefinite integration [try ?int for details]" ):

int::type:= "int":
int::print:= "int":

// diff-attribute of int 

int::diff :=
    proc(f)
	local i, a, u, t, x,chainmult,evaluate;
    begin
	if args(0) = 1 then return(f) end_if;

	a:= []; // variables that appear in the integration variable or the bounds 
	u:= []; // other variables 
	for i from 2 to args(0) do
	    if has(op(f,2), args(i)) then
		a:= append(a, args(i))
	    else
		u:= append(u, args(i))
	    end_if
	end_for;

	// exchange diff and int 
	if nops(u) <> 0 then
	    return(diff(int(diff(op(f,1), op(u)), op(f,2)), op(a)))
	end_if;

	i:= contains(a, op(f,2));
	if i = 0 then
	    // diff(int(g,t=a..b),y) 
	    if contains(a, (t:=op(f,[2,1]))) = 0 then
		// diff(int(g,t=a(x)..b(x)),x) 
		// hold(diff)(args()) 
                x:=a[1]; delete a[1];
                // chainmult is necessary to prevent {+/-}infinity*0
                chainmult:=(f,d)->if d=0 then 0 else d*f end_if;
                // evaluate takes care of limitation process e.g. sin(t)/t,t=0
                evaluate:=
                  proc(g,x,dir)
                    local s;
                  begin
                    s:=NIL;
                    traperror((s:=subs(g,x)));
                    if s=NIL then
                      s:=intlib::limit(g,x,dir);
                    else
                      s
                    end_if;
                  end_proc;
                diff(chainmult(evaluate(op(f,1),t=op(f,[2,2,2]),Left),
                               diff(op(f,[2,2,2]),x))
                     -chainmult(evaluate(op(f,1),t=op(f,[2,2,1]),Right),
                                diff(op(f,[2,2,1]),x))
                     +int(diff(op(f,1),x),op(f,2)),
                     op(a))
	    else // diff(int(g,x=a..b),x) 
		0
	    end_if
	else // diff(int(g,x),x) 
	    delete a[i];
	    diff(op(f,1), op(a))
	end_if
    end_proc:

// float-attribute of int (numerical integration) 

// old float attribute
//int:= slot(int, "float", proc(f, x) local r; begin
//   r := intlib::numint(f,x);
//   if r = FAIL then hold(int)(f, x) else r end_if
//end_proc):
// changed to new float attribute (W. Oevel, 31.8.99)

int::float := numeric::int:

int::Simplify:=
proc(f)
  local S;
begin
  S:= Simplify(op(f, 1));
  if S <> op(f, 1) then
    eval(hold(int)(S, op(f, 2..nops(f))))
  else
    f
  end_if
end_proc:

int::laplace:=
    loadproc( int::laplace,
        pathname("TRANS","LAPLACE"), "L_int"):

int::Content      := proc(Out, data)
                     begin
                       if type(op(data, 2)) = "_equal" then
                         // definite integral
                         Out::Capply(Out::Cint,
                                     Out::Cbvar(Out(op(data, [2,1]))),
                                     Out::Clowlimit(Out(op(data, [2,2,1]))),
                                     Out::Cuplimit( Out(op(data, [2,2,2]))),
                                     Out(op(data,1)));
                       else
                         // indefinite integral
                         Out::Capply(Out::Cint,
                                     Out::Cbvar(Out(op(data, 2))),
                                     Out(op(data,1)));
                       end_if;
                     end_proc:

// end of file 
