// $Id: subs.mu,v 1.11 2003/09/18 11:15:42 bij Exp $

/*- 

stringlib::subs -- substitute strings

Calling sequence:

stringlib::subs(s, t = r)
stringlib::subs(s, t = r, First)

Parameter:

s     - non-empty string
t     - non-empty string
r     - string
First - an optional name

Summary:

stringlib::subs(s, t = r, <t1 = r1, ...>) replaces all substrings t in the string s by
the string r (t1 by r1...). This new string is returned.
stringlib::subs(s, t = r, ..., First) replaces only the first occurrence of t in s by r
(t1 by r1...).

Examples:

>> stringlib::subs("abcdabe", "ab" = "X");

           "XcdXe"
		   
>> stringlib::subs("abcdabe", "ab" = "X", First);

           "Xcdabe"
		   
>> stringlib::subs("abcdabe", "ab" = "");

           "cde"
		   
>> stringlib::subs("abcdabe", "abd" = "X");

           "abcdabe"
		   
See also:

subs, stringlib::contains

-*/

stringlib::subs :=
  proc(s = "")
    local m, n, i, k, l, r, sn, t, SUBS, OPT, X;
  begin
    if s = "" then
      return(s)
    end_if;
    
    // examine arguments
    if args(args(0)) = hold(First) then
      if args(0) < 3 then
        return(s)
      end_if;
      OPT := hold(Index);
      SUBS := [args(2..args(0) - 1)]
    else
      OPT := hold(IndexList);
      SUBS := [args(2..args(0))]
    end_if;
    
    if testargs() then
      // check substitution equations
      X := select(SUBS, testtype, Type::Equation(DOM_STRING, DOM_STRING));
      if nops(X) <> nops(SUBS) then
        if not testtype(args(args(0)), Type::Equation(DOM_STRING, DOM_STRING)) and
           args(args(0)) <> hold(First) then
          error("illegal last argument")
        else
          error("wrong type of substitution argument")
        end_if
      end_if
    end_if;

    // old code, now indexed assignment to strings is available
    for X in SUBS do
      k := stringlib::FirstIndex;
      t := op(X, 1);
      r := op(X, 2);
      
      l := stringlib::contains(s, t, OPT); // Index or IndexList
      if l = FALSE or l = [] then
        next
      end_if;
      n := length(s); m := length(t);
      sn := "";
      for i in l do
        if i > k then
          sn := sn, substring(s, k, i-k)
        end_if;
        sn := sn, r; k := i + m;
      end_for;
      sn := _concat(sn);
      if n >= k then
        sn := sn.s[k..-1]
      end_if;
      s := sn
    end_for;

    //// var 1
    //for X in SUBS do
    //  k := stringlib::FirstIndex;
    //  t := op(X, 1);
    //  r := op(X, 2);
    //  
    //  l := stringlib::contains(s, t, OPT); // Index or IndexList
    //  if l = FALSE or l = [] then
    //    next
    //  end_if;
    //  m := length(t) - 1; // replacement
    //  for i from 1 to nops(l) do
    //    n := l[-i]; // start index
    //    s[n..n+m] := r
    //  end_for
    //end_for;

    // var 2
    //for X in SUBS do
    //  l := [];
    //  k := length(op(X, 1)); // pattern
    //  m := 0;
    //  if OPT <> hold(Index) then
    //    while (n := strmatch(s[m+1..-1], op(X, 1), hold(Index))) <> FALSE do
    //      l := append(l, map(n, _plus, m));
    //      m := m + k
    //    end_while
    //  else
    //    if (n := strmatch(s[m..-1], op(X, 1), hold(Index))) <> FALSE then
    //      l := append(l, n)
    //    end_if
    //  end_if;
    //  if l = [] then
    //    next
    //  end_if;
    //  r := op(X, 2);
    //  n := length(r) - 1;
    //  for i from 1 to nops(l) do
    //    k := l[-i];
    //    s[k[1]..k[2]] := r
    //  end_for
    //end_for;

    // alternative, is slower!
    //if args(args(0)) = hold(First) then
    //  _lazy_and((k := strmatch(s, op(r, 1), hold(Index))) <> FALSE,
    //            (s[op(k, 1)..op(k, 2)] := op(r, 2); TRUE)) $ r in SUBS
    //else
    //  (l := 1; while _lazy_and((k := strmatch(s[l..-1], op(r, 1), hold(Index))) <> FALSE,
    //                           (s[(l - 1) + op(k, 1)..(l - 1) + op(k, 2)] := op(r, 2);
    //                            l <= length(s))) do
    //             l := l - 1 + length(op(r, 2)) + op(k, 1)
    //           end_while) $ r in SUBS
    //end_if;
    s
  end_proc:

