// Dirk Mueller, olorien, 28/10/96, v1.0 

alias( IntTrans = transform::IntTrans ):
alias( laplace = transform::laplace ):

laplace:=newDomain("laplace"):
laplace::create_dom:=hold(transform::laplace):
laplace::Name:= "transform::laplace":

laplace::has:=proc() local T, v;
begin
  T:=args(1); v:=args(2);
  bool( has(extop(T,1), v) or
        has(extop(T,2), v) or
        has(extop(T,3), v) or
        v=hold(laplace))
end_proc:

laplace::numer:= id:
laplace::denom:= 1:

// ----- we sure need that (S.H. Aug 2002):
// this will be called when _f itself is a
// laplace object
laplace::laplace:=proc(_f, var1, var2)
begin
	return(new(laplace, _f, var1, var2));
end_proc:

laplace::normal:=id:

laplace::new:=
proc(_f, var1, var2)
   local back, dummy;
begin
   if args(0) = 0 then error("no arguments given") end_if;

   // allow overloading 
   if _f::dom::laplace<>FAIL then
     back := _f::dom::laplace(args());
     if back <> FAIL then
       return(back)
     else
       return(new(laplace, args()))
     end_if
   end_if;

   if testargs() then
    if args(0)<>3 then error("wrong number of arguments") end_if;
 
    if type(_f) = "_equal" then
        if not testtype(_f,Type::Arithmetical=Type::Arithmetical) then
            error("invalid 1st argument")
        end_if
    elif not testtype(_f,Type::Arithmetical) then
        error("1st argument must be of 'Type::Arithmetical'")
    end_if;

    if domtype(var1)<>DOM_IDENT then
      error("second argument must be of type DOM_IDENT");
    end_if;
   end_if;

   // --------------- S.H. August 2002 ------------------
   // var1 must not occur within var2, 
   if has(var2, var1) then
	error("third argument may not contain second argument");
   end_if;  

   // laplace::compute may use differentiation w.r.t. var2,
   // so make sure, that we can differentiate symbolically:
   if domtype(var2)<> DOM_IDENT 
     then dummy:= genident():
     else dummy:= var2;
   end_if;
   
   // -------------------------------------------------------
   // by Stephan Huckemann, May 2002: it might be wise to try
   // to compute explicitly before going for symbolic 
   // differentation:

   back:=laplace::compute(_f, var1, var2);
   if back<> FAIL then
     return(back);
   end_if;
   // --------------------------------------------------------
   
   back:=laplace::compute(_f, var1, dummy);
 
   if back=FAIL then
	// ----- S.H. Aug. 2002 -------------
	// if we messed up variables by internal
	// calls etc. untangle variables
	if has(var2, var1) then
		dummy := genident();
		_f := subs(_f, var1=dummy);
		var1 := dummy;
	end_if;
	// -----------------------------------
     return(new(laplace, _f, var1, var2))
   else
     if dummy <> var2 then
        back:= eval(subs(back, dummy = var2));
     end_if:
     return(back)
   end_if;
end_proc:


laplace::print:=proc(T)
begin
  hold(laplace)(extop(T,1), extop(T,2), extop(T,3));
end_proc:

//this part computes the laplacian transform
laplace::compute:=proc(_g, var1, var2) local _f, back, typ, i, tmp;
begin 

  _f:=_g;
  if not has(_f, var1) then
    return(_f/var2);
  elif _f=var1 then
    return(1/var2^2);
  end_if;

  typ:=type(_f);

  if typ="_equal" then
    return(_equal(laplace(op(_f,1), var1, var2),
            laplace(op(_f,2), var1, var2)))
  end_if;

	// S.H. Sept 2002 first try built-ins
	// if that fails then we can still 
	// decompose into summands etc what's done
	// in IntTrans::linear
/*
  if typ="function" and type(op(_f,0))="D" then
    _f:=IntTrans::D2diff(_f);
  end_if;

  back:=IntTrans::builtin(new(laplace, _f, var1, var2));
  if not (back=FAIL) then
    return(back);
  end_if;
*/

  if typ="_mult" or typ="_plus" then

    back:=IntTrans::linear(new(laplace, _f, var1, var2));

    // -------------------------------------------------
    // Walter, 4.5.2000
    // If no success, then try again with normalization 
    // inside IntTrans::linear. This is triggered by the
    // second (dummy) argument 0:
    if back = FAIL or hastype(back, "transform::laplace") then
       back:=IntTrans::linear(new(laplace, _f, var1, var2), 0);
    end_if:
    // -------------------------------------------------

    if not (back=FAIL) then
      return(back);
    end_if;
  end_if;

/*
  if typ="_mult" and type(op(_f,1))="_power" and type(op(_f,2))="_plus" then
    if op(op(_f,1),2)<0 then
      back:=_plus(
          laplace( op(op(_f,2), i)*op(_f, 1), var1, var2)
             $ i=1..nops(op(_f,2)));
    end_if;
    if back<>FAIL then return(back) end_if;
  end_if;
*/

/* ------------ S.H. Sep. 2002 it appears to be wiser to do this earlier  */
  if typ="function" and type(op(_f,0))="D" then
    _f:=IntTrans::D2diff(_f);
  end_if;

  back:=IntTrans::builtin(new(laplace, _f, var1, var2));
  if not (back=FAIL) then
    return(back);
  end_if;
/*-------------------------*/

//---------------- making a product of sums a sum of products ----------------------
// Stephan.H. June 2002: I don't understand why restriction to _power in first
// and _plus in 2nd operand. So I changed the code slightly:

/*  if typ="_mult" and type(op(_f,1))="_power" and type(op(_f,2))="_plus" then
    tmp:= op(op(_f,1),2);
      if testtype(tmp, Type::Real) then
        if op(op(_f,1),2)<0 then
          back:=_plus(
              laplace( op(op(_f,2), i)*op(_f, 1), var1, var2)
                 $ i=1..nops(op(_f,2)));
        end_if;
        if back<>FAIL then
          return(back)
        end_if;
      end_if;
    end_if; */
 
    if typ="_mult" and type(op(_f,2))="_plus" then
          back:=_plus(
              laplace( op(op(_f,2), i)*op(_f, 1), var1, var2)
                 $ i=1..nops(op(_f,2)));
        if back<>FAIL then
          return(back)
        end_if;
    elif typ="_mult" and type(op(_f,1))="_plus" then
          back:=_plus(
              laplace( op(op(_f,1), i)*op(_f, 2), var1, var2)
                 $ i=1..nops(op(_f,1)));
        if back<>FAIL then
          return(back)
        end_if;
    end_if; 

   //---------------------------------------------
   // Stephan, Jan. 2 .2002
   // the code above will produce a FAIL with
   // powers like f(x) = (x - 1)^2. 
   // expanding might help. Expanding should not be
   // done to early, though,  as e.g.
   // expand((sin 2t)^2) = 4 (cos t)^2 (sin t)^2

   if typ="_power" then
	tmp := op(_f,2);
	if testtype(tmp, DOM_INT) and tmp <> -1 then
      tmp := expand(_f);
      // only try with expanded argument, when expand
      // changed something.  Otherwise we get infinite
      // recursions
      if tmp <> _f then
        back := laplace::new(tmp, var1, var2);
        return(back);
      end_if;
   end_if;
   end_if;
 
  // --------------------------------------------

 return(FAIL);
end_proc:


laplace::diff:=proc() local f, e, x, S, vars, var, dummy;
begin
 
  // diff(anything) = anything
  if args(0) = 1 then return(args(1)) end_if;

  // analyze the arguments
  f:=args(1); // f = transform::laplace(e, x, S)
  e:= extop(f, 1); // this can be a multivariate expression 
  x:= extop(f, 2); // this is an identifier
  S:= extop(f, 3); // this can be some expression S(some variables)!
                 
  vars:= [args(2..args(0))]; // we are differentiating w.r.t. these vars 

  //----- S.H. Aug 2002: what if the variable we're integrating
  // with (and which should be gone once the integral is done)
  // carries by odd chance the same name as the variable
  // we are differentiating with. At this point we do not know
  // in advance the name of the integrating variable, so staying
  // on the safe side ....

	dummy := genident();
        e := subs(e, x = dummy);


  // diff(transform::laplace(expr, x, S), x) = 0
/*  if has(vars, x) then return(0) end_if;*/

  // vars cannot have x any more


  for var in vars do
//      e:= -x*diff(S,var)*e + diff(e, var);
      e:= -dummy*diff(S,var)*e + diff(e, var);
  // ---- end changes -----------------------------
  end_for;

//  return(laplace(e, x, S)); 
  return(laplace(e, dummy, S)); 
end_proc:

// ------------------- S.H. Aug. 2002 ----------------------
laplace::equal:=
proc(o1,o2)
	local f1, x1, t1, f2, x2, t2, dummy, l1, l2;
begin	// undocumented, user is responsible for his errors
	// this will only work if
	// o1 = sum_i a_i*laplace(f_i, x_i, t)
	// o2 = sum_j b_i*laplace(g_j, y_j, s)
	// then we check
	// whether s = t and
	// sum_i a_i*f_i = sum_j b_j*g_j (making all x_i to x,
	//					all y_j to x)
	// we're trying to find laplacians in o1 and o2

	//create common variable for integrals
/*	x := genident();	
	// initialize sums
	f1 := 0;
	f2 := 0;

	o1:=expand(o1);
	o2:=expand(o2);
	
	typ1:= type(o1);
	typ2:= type(o2);

	if typ1 = "transform::laplace" then //fine
	elif typ1 = "_plus" then
		for i from 1 to nops(o1) do
			typ1 := type(extop(o1,i));
			if typ1 = "transform::laplace" then // fine
			elif typ1 = "_mult then
				c := 1;
				temp := 0;
				for j from t to nops(extop(o1,i))
					temp

				f1 := f1 + 		
	else	// worse than expected
		return(UNKNOWN);
	end_if;

	if typ1 = "_mult" then
		if testtype(extop(o1,1), transform::laplace) = TRUE and
		   testtype(extop(o1,1), transform::laplace) = FALSE and


	if testtype(o1, transform::laplace) = FALSE 
		if testtype(o1
or
	   testtype(l2, transform::laplace) = FALSE 
		then return(UNKNOWN);
*/	l1:=o1;
	l2:=o2;	
	// each l1, l2 should be an unevaluated laplace object
	// l1 = laplace(f1, x1, t1)
	// l2 = laplace(f2, x2, t2)
	if testtype(l1, transform::laplace) = FALSE or
	   testtype(l2, transform::laplace) = FALSE 
		then return(UNKNOWN);
	end_if;
	
	t1:=extop(l1, 3);
	t2:=extop(l2, 3);
	if t1 <> t2 then 
		return(FALSE); 
	else
		f1:=extop(l1, 1);
		x1:=extop(l1, 2);
		f2:=extop(l2, 1);
		x2:=extop(l2, 2);
		// make common range for the integral
		dummy := genident();
		f1:=subs(f1, x1 = dummy);
		f2:=subs(f2, x2 = dummy);
		if f1 <> f2 then
			return(FALSE);
		end_if;
	end_if;
	return(TRUE);
end_proc:

laplace::int:=
proc()
  local _F, _f, _a, _b, _t, var1, var2;
begin
  _F:=args(1); _t:=op(args(2),1); _a:=op(op(args(2),2),1); _b:=op(op(args(2),2),2);
  _f:=extop(_F,1); var1:=extop(_F,2); var2:=extop(_F,3);
  if _t=var2 and _a=0 and _b=infinity then
    return(_F/var1)
  end_if;
  hold(int)(_F, _t=_a.._b);
end_proc:


laplace::invlaplace:=
proc(_f: transform::laplace, var1: DOM_IDENT, var2)
  local s0;
begin
 if extop(_f, 3) = var1 then
        return(eval(subs(extop(_f,1), extop(_f,2)=var2)))
     end_if;
     //   invlaplace( laplace(f(t), t, s + s0), s, x)
     // = invlaplace( laplace(exp(-s0*t)*f(t), t, s), s, x)
     // =  exp(-s0*x)*f(x)
     s0 := extop(_f, 3) - var1;
     if not has(s0, var1) then
        return(exp(-s0*var2)*eval(subs(extop(_f,1), extop(_f,2)=var2)));
     end_if;
end_proc:

 // S.H.9/2002: computes laplace transform of _fu = (_a*var1)^(_n)*exp(_c/var1)
laplace::compute_x_n_exp_c_over_x:=loadproc(laplace::compute_x_n_exp_c_over_x,
	        pathname("TRANS","LAPLACE"), "L_specials"):

 // S.H.10/2002: computes laplace transform of _fu = (_a*var1)^(_n)*sin(_c*var1)
laplace::compute_x_n_sin_x:=loadproc(laplace::compute_x_n_sin_x,
	        pathname("TRANS","LAPLACE"), "L_specials"):


/* obsolete

arctan:= slot( arctan, "transform::invlaplace",
    loadproc(slot( arctan, "transform::invlaplace"),
        pathname("TRANS","LAPLACE"), "IL_atan"
    )
):


sinh:= slot( sinh, "transform::laplace",
    loadproc(slot( sinh, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_sinh"
    )
):

cos:= slot( cos, "transform::laplace",
    loadproc(slot( cos, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_cos"
    )
):

cosh:= slot( cosh, "transform::laplace",
    loadproc(slot( cosh, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_cosh"
    )
):

diff:= slot( diff, "transform::laplace",
    loadproc(slot( diff, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_diff"
    )
):

ln:= slot( ln, "transform::laplace",
    loadproc(slot( ln, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_ln"
    )
):

heaviside:= slot( heaviside, "transform::laplace",
    loadproc(slot( heaviside, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_heavis"
    )
):

exp:= slot( exp, "transform::laplace",
    loadproc(slot( exp, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_exp"
    )
):

int:= slot( int, "transform::laplace",
    loadproc(slot( int, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_int"
    )
):

dirac:= slot( dirac, "transform::laplace",
    loadproc(slot( dirac, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_dirac"
    )
):

besselJ:= slot( besselJ, "transform::laplace",
    loadproc(slot( besselJ, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_bessJ"
    )
):

abs:= slot( abs, "transform::laplace",
    loadproc(slot( abs, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_abs"
    )
):

Ei:= slot( Ei, "transform::laplace",
    loadproc(slot( Ei, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_Ei"
    )
):
*/

unalias( laplace ):
unalias( IntTrans ):

// end of file 
